#line 1 "/home/iurt/rpmbuild/BUILD/clisp-2.49.93-build/clisp-master/modules/gdbm/gdbm.c"
/*
 * GDBM - The GNU database manager
 * <http://www.gnu.org/software/gdbm/>
 * Copyright (C) 2007  Masayuki Onjo <onjo@lispuser.net>
 * Copyright (C) 2007-2008, 2010-2012, 2018  Sam Steingold <sds@gnu.org>
 * This is Free Software, distributed under the GNU GPL v2+
 */

#include "clisp.h"
#include "config.h"

#include <string.h>             /* memcpy */
#include <unistd.h>             /* for lseek */

#if defined(HAVE_GDBM_H)
# include <gdbm.h>
#elif defined(HAVE_GDBM_GDBM_H)
# include <gdbm/gdbm.h>
#else
# error No GDBM headers!
#endif

/* gotta do it before DEFMODULE */
typedef enum {
  GDBM_DATA_STRING,
  GDBM_DATA_VECTOR,
  /* No support for sub-byte bit vectors because if we do, we have to
   * keep the vector length in the datum too (to avoid having #*10100000
   * retrieved when #*101 was stored), and the length would take a full
   * word which would nix any potential saving from using bits instead
   * of full bytes (this is similar to sub-byte :ELEMENT-TYPE for streams,
   * see <http://clisp.org/impnotes/stream-dict.html#eltype>).
   * Also, Atype_16Bit is not exported from lispbibl.
   *     GDBM_DATA_BIT_VECTOR,
   *     GDBM_DATA_2BIT_VECTOR,
   *     GDBM_DATA_4BIT_VECTOR, */
  GDBM_DATA_8BIT_VECTOR,
  /*     GDBM_DATA_16BIT_VECTOR, */
  GDBM_DATA_32BIT_VECTOR,
  GDBM_DATA_INTEGER,
  GDBM_DATA_SINGLE_FLOAT,
  GDBM_DATA_DOUBLE_FLOAT,
  GDBM_DATA_NOTYPE              /* raise an error on conversion */
} gdbm_data_t;

/* must be distinct from all gdbm_setopt options */
#define GDBM_GETDEFAULT_VALUE_TYPE -1
#define GDBM_SETDEFAULT_VALUE_TYPE GDBM_GETDEFAULT_VALUE_TYPE
#define GDBM_GETDEFAULT_KEY_TYPE   -2
#define GDBM_SETDEFAULT_KEY_TYPE   GDBM_GETDEFAULT_KEY_TYPE

DEFMODULE(gdbm,"GDBM");

void module__gdbm__init_function_1 (module_t* module);
void module__gdbm__init_function_2 (module_t* module);

#define O(varname) module__gdbm__object_tab._##varname
#define F(varname) subr_tab_ptr_as_object(&this_module_stab->_##varname)

struct module__gdbm__object_tab_t {
  gcv_object_t _object_cl__double_float;
  gcv_object_t _object_cl__integer;
  gcv_object_t _object_cl__single_float;
  gcv_object_t _object_cl__string;
  gcv_object_t _object_cl__vector;
  gcv_object_t _object_ext__32bit_vector;
  gcv_object_t _object_ext__8bit_vector;
  gcv_object_t _object_gdbm__gdbm;
  gcv_object_t _object_gdbm__gdbm_error;
#if defined(HAVE_GDBM_OPEN)
  gcv_object_t _object_gdbm__make_gdbm;
#endif
#if defined(GDBM_BACKUP_FAILED)
  gcv_object_t _object_Kbackup_failed;
#endif
#if defined(GDBM_BAD_AVAIL)
  gcv_object_t _object_Kbad_avail;
#endif
#if defined(GDBM_BAD_BUCKET)
  gcv_object_t _object_Kbad_bucket;
#endif
#if defined(GDBM_BAD_DIR_ENTRY)
  gcv_object_t _object_Kbad_dir_entry;
#endif
#if defined(GDBM_BAD_FILE_OFFSET)
  gcv_object_t _object_Kbad_file_offset;
#endif
#if defined(GDBM_BAD_HASH_TABLE)
  gcv_object_t _object_Kbad_hash_table;
#endif
#if defined(GDBM_BAD_HEADER)
  gcv_object_t _object_Kbad_header;
#endif
#if defined(GDBM_BAD_MAGIC_NUMBER)
  gcv_object_t _object_Kbad_magic_number;
#endif
#if defined(GDBM_BAD_OPEN_FLAGS)
  gcv_object_t _object_Kbad_open_flags;
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETBLOCKSIZE))
  gcv_object_t _object_Kblocksize;
#endif
#if defined(GDBM_BLOCK_SIZE_ERROR)
  gcv_object_t _object_Kblock_size_error;
#endif
#if defined(GDBM_BSEXACT)
  gcv_object_t _object_Kbsexact;
#endif
#if defined(GDBM_BYTE_SWAPPED)
  gcv_object_t _object_Kbyte_swapped;
#endif
#if (defined(GDBM_GETCACHESIZE)) || (defined(GDBM_SETCACHESIZE))
  gcv_object_t _object_Kcachesize;
#endif
#if defined(GDBM_CANNOT_REPLACE)
  gcv_object_t _object_Kcannot_replace;
#endif
#if defined(GDBM_CANT_BE_READER)
  gcv_object_t _object_Kcant_be_reader;
#endif
#if defined(GDBM_CANT_BE_WRITER)
  gcv_object_t _object_Kcant_be_writer;
#endif
#if (defined(GDBM_GETCENTFREE)) || (defined(GDBM_SETCENTFREE))
  gcv_object_t _object_Kcentfree;
#endif
#if defined(GDBM_CLOEXEC)
  gcv_object_t _object_Kcloexec;
#endif
  gcv_object_t _object_Kclosed_file;
#if (defined(GDBM_GETCOALESCEBLKS)) || (defined(GDBM_SETCOALESCEBLKS))
  gcv_object_t _object_Kcoalesceblks;
#endif
  gcv_object_t _object_Kdatum_type;
#if defined(GDBM_GETDBNAME)
  gcv_object_t _object_Kdbname;
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETDEFAULT_KEY_TYPE)) || (defined(GDBM_SETDEFAULT_KEY_TYPE))
  gcv_object_t _object_Kdefault_key_type;
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETDEFAULT_VALUE_TYPE)) || (defined(GDBM_SETDEFAULT_VALUE_TYPE))
  gcv_object_t _object_Kdefault_value_type;
#endif
#if defined(GDBM_DIR_OVERFLOW)
  gcv_object_t _object_Kdir_overflow;
#endif
#if defined(GDBM_EMPTY_DATABASE)
  gcv_object_t _object_Kempty_database;
#endif
#if defined(GDBM_ERR_FILE_MODE)
  gcv_object_t _object_Kerr_file_mode;
#endif
#if defined(GDBM_ERR_FILE_OWNER)
  gcv_object_t _object_Kerr_file_owner;
#endif
#if defined(GDBM_FAST)
  gcv_object_t _object_Kfast;
#endif
  gcv_object_t _object_Kfatal;
#if defined(GDBM_FILE_EOF)
  gcv_object_t _object_Kfile_eof;
#endif
#if defined(GDBM_FILE_OPEN_ERROR)
  gcv_object_t _object_Kfile_open_error;
#endif
#if defined(GDBM_FILE_READ_ERROR)
  gcv_object_t _object_Kfile_read_error;
#endif
#if defined(GDBM_FILE_SEEK_ERROR)
  gcv_object_t _object_Kfile_seek_error;
#endif
#if defined(GDBM_FILE_STAT_ERROR)
  gcv_object_t _object_Kfile_stat_error;
#endif
#if defined(GDBM_FILE_WRITE_ERROR)
  gcv_object_t _object_Kfile_write_error;
#endif
#if defined(HAVE_GDBM_STORE)
  gcv_object_t _object_Kflag;
#endif
#if defined(GDBM_GETFLAGS)
  gcv_object_t _object_Kflags;
#endif
#if defined(GDBM_ILLEGAL_DATA)
  gcv_object_t _object_Killegal_data;
#endif
#if defined(GDBM_INSERT)
  gcv_object_t _object_Kinsert;
#endif
#if defined(GDBM_ITEM_NOT_FOUND)
  gcv_object_t _object_Kitem_not_found;
#endif
  gcv_object_t _object_Klisp_type;
#if defined(GDBM_MALLOC_ERROR)
  gcv_object_t _object_Kmalloc_error;
#endif
#if (defined(GDBM_GETMAXMAPSIZE)) || (defined(GDBM_SETMAXMAPSIZE))
  gcv_object_t _object_Kmaxmapsize;
#endif
  gcv_object_t _object_Kmessage;
#if (defined(GDBM_GETMMAP)) || (defined(GDBM_SETMMAP))
  gcv_object_t _object_Kmmap;
#endif
#if defined(HAVE_GDBM_OPEN)
  gcv_object_t _object_Kmode;
#endif
#if defined(GDBM_NEED_RECOVERY)
  gcv_object_t _object_Kneed_recovery;
#endif
#if defined(GDBM_NEWDB)
  gcv_object_t _object_Knewdb;
#endif
#if defined(GDBM_NOLOCK)
  gcv_object_t _object_Knolock;
#endif
#if defined(GDBM_NOMMAP)
  gcv_object_t _object_Knommap;
#endif
#if defined(GDBM_NO_DBNAME)
  gcv_object_t _object_Kno_dbname;
#endif
#if defined(GDBM_NO_ERROR)
  gcv_object_t _object_Kno_error;
#endif
#if defined(HAVE_GDBM_OPEN)
  gcv_object_t _object_Koptions;
#endif
#if defined(GDBM_OPT_ALREADY_SET)
  gcv_object_t _object_Kopt_already_set;
#endif
#if defined(GDBM_OPT_ILLEGAL)
  gcv_object_t _object_Kopt_illegal;
#endif
#if defined(GDBM_READER)
  gcv_object_t _object_Kreader;
#endif
#if defined(GDBM_READER_CANT_DELETE)
  gcv_object_t _object_Kreader_cant_delete;
#endif
#if defined(GDBM_READER_CANT_REORGANIZE)
  gcv_object_t _object_Kreader_cant_reorganize;
#endif
#if defined(GDBM_READER_CANT_STORE)
  gcv_object_t _object_Kreader_cant_store;
#endif
#if defined(HAVE_GDBM_OPEN)
  gcv_object_t _object_Kread_write;
#endif
#if defined(GDBM_REORGANIZE_FAILED)
  gcv_object_t _object_Kreorganize_failed;
#endif
#if defined(GDBM_REPLACE)
  gcv_object_t _object_Kreplace;
#endif
#if defined(GDBM_SYNC)
  gcv_object_t _object_Ksync;
#endif
#if (defined(GDBM_GETSYNCMODE)) || (defined(GDBM_SETSYNCMODE))
  gcv_object_t _object_Ksyncmode;
#endif
#if (defined(HAVE_GDBM_FETCH)) || (defined(HAVE_GDBM_FIRSTKEY)) || (defined(HAVE_GDBM_NEXTKEY))
  gcv_object_t _object_Ktype;
#endif
#if defined(GDBM_UNKNOWN_ERROR)
  gcv_object_t _object_Kunknown_error;
#endif
#if defined(GDBM_WRCREAT)
  gcv_object_t _object_Kwrcreat;
#endif
#if defined(GDBM_WRITER)
  gcv_object_t _object_Kwriter;
#endif
  gcv_object_t _object__2232bit_vector_20conversion_20requires_20a_20datum_20length_20divisible_20by_204_22;
  gcv_object_t _object__22desired_20lisp_20type_20not_20specified_22;
  gcv_object_t _object__22invalid_20lisp_20object_20type__20_22;
  gcv_object_t _object__22open_20gdbm_20file_20required_22;
  gcv_object_t _object__22_7Es__20_7Ea_22;
  gcv_object_t _object__28or_20null_20integer_20_28member_20cl__string_20cl__vector_20ext__8bit_vector_20ext__32bit_vector_20cl__integer_20cl__single_float_20cl__double_float_29_29;
  gcv_object_t _object___28or_20integer_20_28member_defined_28gdbm_fast_29_Kfast_defined_28gdbm_sync_29_Ksync_defined_28gdbm_nolock_29_Knolock_defined_28gdbm_nommap_29_Knommap_defined_28gdbm_cloexec_29_Kcloexec_defined_28gdbm_bsexact_29_Kbsexact__29_29;
  gcv_object_t _object___28or_20integer_20_28member_defined_28gdbm_getflags_29_Kflags_defined_28gdbm_getmmap_29_Kmmap_defined_28gdbm_getcachesize_29_Kcachesize_defined_28gdbm_getsyncmode_29_Ksyncmode_defined_28gdbm_getcentfree_29_Kcentfree_defined_28gdbm_getcoalesceblks_29_Kcoalesceblks_defined_28gdbm_getmaxmapsize_29_Kmaxmapsize_defined_28gdbm_getdbname_29_Kdbname_defined_28gdbm_getblocksize_29_Kblocksize_defined_28gdbm_getdefault_value_type_29_Kdefault_value_type_defined_28gdbm_getdefault_key_type_29_Kdefault_key_type__29_29;
  gcv_object_t _object___28or_20integer_20_28member_defined_28gdbm_no_error_29_Kno_error_defined_28gdbm_malloc_error_29_Kmalloc_error_defined_28gdbm_block_size_error_29_Kblock_size_error_defined_28gdbm_file_open_error_29_Kfile_open_error_defined_28gdbm_file_write_error_29_Kfile_write_error_defined_28gdbm_file_seek_error_29_Kfile_seek_error_defined_28gdbm_file_read_error_29_Kfile_read_error_defined_28gdbm_bad_magic_number_29_Kbad_magic_number_defined_28gdbm_empty_database_29_Kempty_database_defined_28gdbm_cant_be_reader_29_Kcant_be_reader_defined_28gdbm_cant_be_writer_29_Kcant_be_writer_defined_28gdbm_reader_cant_delete_29_Kreader_cant_delete_defined_28gdbm_reader_cant_store_29_Kreader_cant_store_defined_28gdbm_reader_cant_reorganize_29_Kreader_cant_reorganize_defined_28gdbm_unknown_error_29_Kunknown_error_defined_28gdbm_item_not_found_29_Kitem_not_found_defined_28gdbm_reorganize_failed_29_Kreorganize_failed_defined_28gdbm_cannot_replace_29_Kcannot_replace_defined_28gdbm_illegal_data_29_Killegal_data_defined_28gdbm_opt_already_set_29_Kopt_already_set_defined_28gdbm_opt_illegal_29_Kopt_illegal_defined_28gdbm_byte_swapped_29_Kbyte_swapped_defined_28gdbm_bad_file_offset_29_Kbad_file_offset_defined_28gdbm_bad_open_flags_29_Kbad_open_flags_defined_28gdbm_file_stat_error_29_Kfile_stat_error_defined_28gdbm_file_eof_29_Kfile_eof_defined_28gdbm_no_dbname_29_Kno_dbname_defined_28gdbm_err_file_owner_29_Kerr_file_owner_defined_28gdbm_err_file_mode_29_Kerr_file_mode_defined_28gdbm_need_recovery_29_Kneed_recovery_defined_28gdbm_backup_failed_29_Kbackup_failed_defined_28gdbm_dir_overflow_29_Kdir_overflow_defined_28gdbm_bad_bucket_29_Kbad_bucket_defined_28gdbm_bad_header_29_Kbad_header_defined_28gdbm_bad_avail_29_Kbad_avail_defined_28gdbm_bad_hash_table_29_Kbad_hash_table_defined_28gdbm_bad_dir_entry_29_Kbad_dir_entry__29_29;
  gcv_object_t _object___28or_20integer_20_28member_defined_28gdbm_setcachesize_29_Kcachesize_defined_28gdbm_setsyncmode_29_Ksyncmode_defined_28gdbm_setcentfree_29_Kcentfree_defined_28gdbm_setcoalesceblks_29_Kcoalesceblks_defined_28gdbm_setmaxmapsize_29_Kmaxmapsize_defined_28gdbm_setmmap_29_Kmmap_defined_28gdbm_setdefault_value_type_29_Kdefault_value_type_defined_28gdbm_setdefault_key_type_29_Kdefault_key_type__29_29;
  gcv_object_t _object___28or_20null_20integer_20_28member_defined_28gdbm_reader_29_Kreader_defined_28gdbm_writer_29_Kwriter_defined_28gdbm_wrcreat_29_Kwrcreat_defined_28gdbm_newdb_29_Knewdb__29_29;
  gcv_object_t _object___28or_20null_20integer_20_28member_defined_28gdbm_replace_29_Kreplace_defined_28gdbm_insert_29_Kinsert__29_29;
  gcv_object_t _gdbm_version_cache;
} module__gdbm__object_tab;
uintC module__gdbm__object_tab_size = sizeof(module__gdbm__object_tab)/sizeof(gcv_object_t);

struct module__gdbm__object_tab_initdata_t {
  object_initdata_t _object_cl__double_float;
  object_initdata_t _object_cl__integer;
  object_initdata_t _object_cl__single_float;
  object_initdata_t _object_cl__string;
  object_initdata_t _object_cl__vector;
  object_initdata_t _object_ext__32bit_vector;
  object_initdata_t _object_ext__8bit_vector;
  object_initdata_t _object_gdbm__gdbm;
  object_initdata_t _object_gdbm__gdbm_error;
#if defined(HAVE_GDBM_OPEN)
  object_initdata_t _object_gdbm__make_gdbm;
#endif
#if defined(GDBM_BACKUP_FAILED)
  object_initdata_t _object_Kbackup_failed;
#endif
#if defined(GDBM_BAD_AVAIL)
  object_initdata_t _object_Kbad_avail;
#endif
#if defined(GDBM_BAD_BUCKET)
  object_initdata_t _object_Kbad_bucket;
#endif
#if defined(GDBM_BAD_DIR_ENTRY)
  object_initdata_t _object_Kbad_dir_entry;
#endif
#if defined(GDBM_BAD_FILE_OFFSET)
  object_initdata_t _object_Kbad_file_offset;
#endif
#if defined(GDBM_BAD_HASH_TABLE)
  object_initdata_t _object_Kbad_hash_table;
#endif
#if defined(GDBM_BAD_HEADER)
  object_initdata_t _object_Kbad_header;
#endif
#if defined(GDBM_BAD_MAGIC_NUMBER)
  object_initdata_t _object_Kbad_magic_number;
#endif
#if defined(GDBM_BAD_OPEN_FLAGS)
  object_initdata_t _object_Kbad_open_flags;
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETBLOCKSIZE))
  object_initdata_t _object_Kblocksize;
#endif
#if defined(GDBM_BLOCK_SIZE_ERROR)
  object_initdata_t _object_Kblock_size_error;
#endif
#if defined(GDBM_BSEXACT)
  object_initdata_t _object_Kbsexact;
#endif
#if defined(GDBM_BYTE_SWAPPED)
  object_initdata_t _object_Kbyte_swapped;
#endif
#if (defined(GDBM_GETCACHESIZE)) || (defined(GDBM_SETCACHESIZE))
  object_initdata_t _object_Kcachesize;
#endif
#if defined(GDBM_CANNOT_REPLACE)
  object_initdata_t _object_Kcannot_replace;
#endif
#if defined(GDBM_CANT_BE_READER)
  object_initdata_t _object_Kcant_be_reader;
#endif
#if defined(GDBM_CANT_BE_WRITER)
  object_initdata_t _object_Kcant_be_writer;
#endif
#if (defined(GDBM_GETCENTFREE)) || (defined(GDBM_SETCENTFREE))
  object_initdata_t _object_Kcentfree;
#endif
#if defined(GDBM_CLOEXEC)
  object_initdata_t _object_Kcloexec;
#endif
  object_initdata_t _object_Kclosed_file;
#if (defined(GDBM_GETCOALESCEBLKS)) || (defined(GDBM_SETCOALESCEBLKS))
  object_initdata_t _object_Kcoalesceblks;
#endif
  object_initdata_t _object_Kdatum_type;
#if defined(GDBM_GETDBNAME)
  object_initdata_t _object_Kdbname;
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETDEFAULT_KEY_TYPE)) || (defined(GDBM_SETDEFAULT_KEY_TYPE))
  object_initdata_t _object_Kdefault_key_type;
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETDEFAULT_VALUE_TYPE)) || (defined(GDBM_SETDEFAULT_VALUE_TYPE))
  object_initdata_t _object_Kdefault_value_type;
#endif
#if defined(GDBM_DIR_OVERFLOW)
  object_initdata_t _object_Kdir_overflow;
#endif
#if defined(GDBM_EMPTY_DATABASE)
  object_initdata_t _object_Kempty_database;
#endif
#if defined(GDBM_ERR_FILE_MODE)
  object_initdata_t _object_Kerr_file_mode;
#endif
#if defined(GDBM_ERR_FILE_OWNER)
  object_initdata_t _object_Kerr_file_owner;
#endif
#if defined(GDBM_FAST)
  object_initdata_t _object_Kfast;
#endif
  object_initdata_t _object_Kfatal;
#if defined(GDBM_FILE_EOF)
  object_initdata_t _object_Kfile_eof;
#endif
#if defined(GDBM_FILE_OPEN_ERROR)
  object_initdata_t _object_Kfile_open_error;
#endif
#if defined(GDBM_FILE_READ_ERROR)
  object_initdata_t _object_Kfile_read_error;
#endif
#if defined(GDBM_FILE_SEEK_ERROR)
  object_initdata_t _object_Kfile_seek_error;
#endif
#if defined(GDBM_FILE_STAT_ERROR)
  object_initdata_t _object_Kfile_stat_error;
#endif
#if defined(GDBM_FILE_WRITE_ERROR)
  object_initdata_t _object_Kfile_write_error;
#endif
#if defined(HAVE_GDBM_STORE)
  object_initdata_t _object_Kflag;
#endif
#if defined(GDBM_GETFLAGS)
  object_initdata_t _object_Kflags;
#endif
#if defined(GDBM_ILLEGAL_DATA)
  object_initdata_t _object_Killegal_data;
#endif
#if defined(GDBM_INSERT)
  object_initdata_t _object_Kinsert;
#endif
#if defined(GDBM_ITEM_NOT_FOUND)
  object_initdata_t _object_Kitem_not_found;
#endif
  object_initdata_t _object_Klisp_type;
#if defined(GDBM_MALLOC_ERROR)
  object_initdata_t _object_Kmalloc_error;
#endif
#if (defined(GDBM_GETMAXMAPSIZE)) || (defined(GDBM_SETMAXMAPSIZE))
  object_initdata_t _object_Kmaxmapsize;
#endif
  object_initdata_t _object_Kmessage;
#if (defined(GDBM_GETMMAP)) || (defined(GDBM_SETMMAP))
  object_initdata_t _object_Kmmap;
#endif
#if defined(HAVE_GDBM_OPEN)
  object_initdata_t _object_Kmode;
#endif
#if defined(GDBM_NEED_RECOVERY)
  object_initdata_t _object_Kneed_recovery;
#endif
#if defined(GDBM_NEWDB)
  object_initdata_t _object_Knewdb;
#endif
#if defined(GDBM_NOLOCK)
  object_initdata_t _object_Knolock;
#endif
#if defined(GDBM_NOMMAP)
  object_initdata_t _object_Knommap;
#endif
#if defined(GDBM_NO_DBNAME)
  object_initdata_t _object_Kno_dbname;
#endif
#if defined(GDBM_NO_ERROR)
  object_initdata_t _object_Kno_error;
#endif
#if defined(HAVE_GDBM_OPEN)
  object_initdata_t _object_Koptions;
#endif
#if defined(GDBM_OPT_ALREADY_SET)
  object_initdata_t _object_Kopt_already_set;
#endif
#if defined(GDBM_OPT_ILLEGAL)
  object_initdata_t _object_Kopt_illegal;
#endif
#if defined(GDBM_READER)
  object_initdata_t _object_Kreader;
#endif
#if defined(GDBM_READER_CANT_DELETE)
  object_initdata_t _object_Kreader_cant_delete;
#endif
#if defined(GDBM_READER_CANT_REORGANIZE)
  object_initdata_t _object_Kreader_cant_reorganize;
#endif
#if defined(GDBM_READER_CANT_STORE)
  object_initdata_t _object_Kreader_cant_store;
#endif
#if defined(HAVE_GDBM_OPEN)
  object_initdata_t _object_Kread_write;
#endif
#if defined(GDBM_REORGANIZE_FAILED)
  object_initdata_t _object_Kreorganize_failed;
#endif
#if defined(GDBM_REPLACE)
  object_initdata_t _object_Kreplace;
#endif
#if defined(GDBM_SYNC)
  object_initdata_t _object_Ksync;
#endif
#if (defined(GDBM_GETSYNCMODE)) || (defined(GDBM_SETSYNCMODE))
  object_initdata_t _object_Ksyncmode;
#endif
#if (defined(HAVE_GDBM_FETCH)) || (defined(HAVE_GDBM_FIRSTKEY)) || (defined(HAVE_GDBM_NEXTKEY))
  object_initdata_t _object_Ktype;
#endif
#if defined(GDBM_UNKNOWN_ERROR)
  object_initdata_t _object_Kunknown_error;
#endif
#if defined(GDBM_WRCREAT)
  object_initdata_t _object_Kwrcreat;
#endif
#if defined(GDBM_WRITER)
  object_initdata_t _object_Kwriter;
#endif
  object_initdata_t _object__2232bit_vector_20conversion_20requires_20a_20datum_20length_20divisible_20by_204_22;
  object_initdata_t _object__22desired_20lisp_20type_20not_20specified_22;
  object_initdata_t _object__22invalid_20lisp_20object_20type__20_22;
  object_initdata_t _object__22open_20gdbm_20file_20required_22;
  object_initdata_t _object__22_7Es__20_7Ea_22;
  object_initdata_t _object__28or_20null_20integer_20_28member_20cl__string_20cl__vector_20ext__8bit_vector_20ext__32bit_vector_20cl__integer_20cl__single_float_20cl__double_float_29_29;
  object_initdata_t _object___28or_20integer_20_28member_defined_28gdbm_fast_29_Kfast_defined_28gdbm_sync_29_Ksync_defined_28gdbm_nolock_29_Knolock_defined_28gdbm_nommap_29_Knommap_defined_28gdbm_cloexec_29_Kcloexec_defined_28gdbm_bsexact_29_Kbsexact__29_29;
  object_initdata_t _object___28or_20integer_20_28member_defined_28gdbm_getflags_29_Kflags_defined_28gdbm_getmmap_29_Kmmap_defined_28gdbm_getcachesize_29_Kcachesize_defined_28gdbm_getsyncmode_29_Ksyncmode_defined_28gdbm_getcentfree_29_Kcentfree_defined_28gdbm_getcoalesceblks_29_Kcoalesceblks_defined_28gdbm_getmaxmapsize_29_Kmaxmapsize_defined_28gdbm_getdbname_29_Kdbname_defined_28gdbm_getblocksize_29_Kblocksize_defined_28gdbm_getdefault_value_type_29_Kdefault_value_type_defined_28gdbm_getdefault_key_type_29_Kdefault_key_type__29_29;
  object_initdata_t _object___28or_20integer_20_28member_defined_28gdbm_no_error_29_Kno_error_defined_28gdbm_malloc_error_29_Kmalloc_error_defined_28gdbm_block_size_error_29_Kblock_size_error_defined_28gdbm_file_open_error_29_Kfile_open_error_defined_28gdbm_file_write_error_29_Kfile_write_error_defined_28gdbm_file_seek_error_29_Kfile_seek_error_defined_28gdbm_file_read_error_29_Kfile_read_error_defined_28gdbm_bad_magic_number_29_Kbad_magic_number_defined_28gdbm_empty_database_29_Kempty_database_defined_28gdbm_cant_be_reader_29_Kcant_be_reader_defined_28gdbm_cant_be_writer_29_Kcant_be_writer_defined_28gdbm_reader_cant_delete_29_Kreader_cant_delete_defined_28gdbm_reader_cant_store_29_Kreader_cant_store_defined_28gdbm_reader_cant_reorganize_29_Kreader_cant_reorganize_defined_28gdbm_unknown_error_29_Kunknown_error_defined_28gdbm_item_not_found_29_Kitem_not_found_defined_28gdbm_reorganize_failed_29_Kreorganize_failed_defined_28gdbm_cannot_replace_29_Kcannot_replace_defined_28gdbm_illegal_data_29_Killegal_data_defined_28gdbm_opt_already_set_29_Kopt_already_set_defined_28gdbm_opt_illegal_29_Kopt_illegal_defined_28gdbm_byte_swapped_29_Kbyte_swapped_defined_28gdbm_bad_file_offset_29_Kbad_file_offset_defined_28gdbm_bad_open_flags_29_Kbad_open_flags_defined_28gdbm_file_stat_error_29_Kfile_stat_error_defined_28gdbm_file_eof_29_Kfile_eof_defined_28gdbm_no_dbname_29_Kno_dbname_defined_28gdbm_err_file_owner_29_Kerr_file_owner_defined_28gdbm_err_file_mode_29_Kerr_file_mode_defined_28gdbm_need_recovery_29_Kneed_recovery_defined_28gdbm_backup_failed_29_Kbackup_failed_defined_28gdbm_dir_overflow_29_Kdir_overflow_defined_28gdbm_bad_bucket_29_Kbad_bucket_defined_28gdbm_bad_header_29_Kbad_header_defined_28gdbm_bad_avail_29_Kbad_avail_defined_28gdbm_bad_hash_table_29_Kbad_hash_table_defined_28gdbm_bad_dir_entry_29_Kbad_dir_entry__29_29;
  object_initdata_t _object___28or_20integer_20_28member_defined_28gdbm_setcachesize_29_Kcachesize_defined_28gdbm_setsyncmode_29_Ksyncmode_defined_28gdbm_setcentfree_29_Kcentfree_defined_28gdbm_setcoalesceblks_29_Kcoalesceblks_defined_28gdbm_setmaxmapsize_29_Kmaxmapsize_defined_28gdbm_setmmap_29_Kmmap_defined_28gdbm_setdefault_value_type_29_Kdefault_value_type_defined_28gdbm_setdefault_key_type_29_Kdefault_key_type__29_29;
  object_initdata_t _object___28or_20null_20integer_20_28member_defined_28gdbm_reader_29_Kreader_defined_28gdbm_writer_29_Kwriter_defined_28gdbm_wrcreat_29_Kwrcreat_defined_28gdbm_newdb_29_Knewdb__29_29;
  object_initdata_t _object___28or_20null_20integer_20_28member_defined_28gdbm_replace_29_Kreplace_defined_28gdbm_insert_29_Kinsert__29_29;
  object_initdata_t _gdbm_version_cache;
  int _dummy_to_avoid_trailing_comma_in_initializer;
} module__gdbm__object_tab_initdata = {
  { "CL::DOUBLE-FLOAT" },
  { "CL::INTEGER" },
  { "CL::SINGLE-FLOAT" },
  { "CL::STRING" },
  { "CL::VECTOR" },
  { "EXT::32BIT-VECTOR" },
  { "EXT::8BIT-VECTOR" },
  { "GDBM::GDBM" },
  { "GDBM::GDBM-ERROR" },
#if defined(HAVE_GDBM_OPEN)
  { "GDBM::MAKE-GDBM" },
#endif
#if defined(GDBM_BACKUP_FAILED)
  { ":BACKUP-FAILED" },
#endif
#if defined(GDBM_BAD_AVAIL)
  { ":BAD-AVAIL" },
#endif
#if defined(GDBM_BAD_BUCKET)
  { ":BAD-BUCKET" },
#endif
#if defined(GDBM_BAD_DIR_ENTRY)
  { ":BAD-DIR-ENTRY" },
#endif
#if defined(GDBM_BAD_FILE_OFFSET)
  { ":BAD-FILE-OFFSET" },
#endif
#if defined(GDBM_BAD_HASH_TABLE)
  { ":BAD-HASH-TABLE" },
#endif
#if defined(GDBM_BAD_HEADER)
  { ":BAD-HEADER" },
#endif
#if defined(GDBM_BAD_MAGIC_NUMBER)
  { ":BAD-MAGIC-NUMBER" },
#endif
#if defined(GDBM_BAD_OPEN_FLAGS)
  { ":BAD-OPEN-FLAGS" },
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETBLOCKSIZE))
  { ":BLOCKSIZE" },
#endif
#if defined(GDBM_BLOCK_SIZE_ERROR)
  { ":BLOCK-SIZE-ERROR" },
#endif
#if defined(GDBM_BSEXACT)
  { ":BSEXACT" },
#endif
#if defined(GDBM_BYTE_SWAPPED)
  { ":BYTE-SWAPPED" },
#endif
#if (defined(GDBM_GETCACHESIZE)) || (defined(GDBM_SETCACHESIZE))
  { ":CACHESIZE" },
#endif
#if defined(GDBM_CANNOT_REPLACE)
  { ":CANNOT-REPLACE" },
#endif
#if defined(GDBM_CANT_BE_READER)
  { ":CANT-BE-READER" },
#endif
#if defined(GDBM_CANT_BE_WRITER)
  { ":CANT-BE-WRITER" },
#endif
#if (defined(GDBM_GETCENTFREE)) || (defined(GDBM_SETCENTFREE))
  { ":CENTFREE" },
#endif
#if defined(GDBM_CLOEXEC)
  { ":CLOEXEC" },
#endif
  { ":CLOSED-FILE" },
#if (defined(GDBM_GETCOALESCEBLKS)) || (defined(GDBM_SETCOALESCEBLKS))
  { ":COALESCEBLKS" },
#endif
  { ":DATUM-TYPE" },
#if defined(GDBM_GETDBNAME)
  { ":DBNAME" },
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETDEFAULT_KEY_TYPE)) || (defined(GDBM_SETDEFAULT_KEY_TYPE))
  { ":DEFAULT-KEY-TYPE" },
#endif
#if (defined(HAVE_GDBM_OPEN)) || (defined(GDBM_GETDEFAULT_VALUE_TYPE)) || (defined(GDBM_SETDEFAULT_VALUE_TYPE))
  { ":DEFAULT-VALUE-TYPE" },
#endif
#if defined(GDBM_DIR_OVERFLOW)
  { ":DIR-OVERFLOW" },
#endif
#if defined(GDBM_EMPTY_DATABASE)
  { ":EMPTY-DATABASE" },
#endif
#if defined(GDBM_ERR_FILE_MODE)
  { ":ERR-FILE-MODE" },
#endif
#if defined(GDBM_ERR_FILE_OWNER)
  { ":ERR-FILE-OWNER" },
#endif
#if defined(GDBM_FAST)
  { ":FAST" },
#endif
  { ":FATAL" },
#if defined(GDBM_FILE_EOF)
  { ":FILE-EOF" },
#endif
#if defined(GDBM_FILE_OPEN_ERROR)
  { ":FILE-OPEN-ERROR" },
#endif
#if defined(GDBM_FILE_READ_ERROR)
  { ":FILE-READ-ERROR" },
#endif
#if defined(GDBM_FILE_SEEK_ERROR)
  { ":FILE-SEEK-ERROR" },
#endif
#if defined(GDBM_FILE_STAT_ERROR)
  { ":FILE-STAT-ERROR" },
#endif
#if defined(GDBM_FILE_WRITE_ERROR)
  { ":FILE-WRITE-ERROR" },
#endif
#if defined(HAVE_GDBM_STORE)
  { ":FLAG" },
#endif
#if defined(GDBM_GETFLAGS)
  { ":FLAGS" },
#endif
#if defined(GDBM_ILLEGAL_DATA)
  { ":ILLEGAL-DATA" },
#endif
#if defined(GDBM_INSERT)
  { ":INSERT" },
#endif
#if defined(GDBM_ITEM_NOT_FOUND)
  { ":ITEM-NOT-FOUND" },
#endif
  { ":LISP-TYPE" },
#if defined(GDBM_MALLOC_ERROR)
  { ":MALLOC-ERROR" },
#endif
#if (defined(GDBM_GETMAXMAPSIZE)) || (defined(GDBM_SETMAXMAPSIZE))
  { ":MAXMAPSIZE" },
#endif
  { ":MESSAGE" },
#if (defined(GDBM_GETMMAP)) || (defined(GDBM_SETMMAP))
  { ":MMAP" },
#endif
#if defined(HAVE_GDBM_OPEN)
  { ":MODE" },
#endif
#if defined(GDBM_NEED_RECOVERY)
  { ":NEED-RECOVERY" },
#endif
#if defined(GDBM_NEWDB)
  { ":NEWDB" },
#endif
#if defined(GDBM_NOLOCK)
  { ":NOLOCK" },
#endif
#if defined(GDBM_NOMMAP)
  { ":NOMMAP" },
#endif
#if defined(GDBM_NO_DBNAME)
  { ":NO-DBNAME" },
#endif
#if defined(GDBM_NO_ERROR)
  { ":NO-ERROR" },
#endif
#if defined(HAVE_GDBM_OPEN)
  { ":OPTIONS" },
#endif
#if defined(GDBM_OPT_ALREADY_SET)
  { ":OPT-ALREADY-SET" },
#endif
#if defined(GDBM_OPT_ILLEGAL)
  { ":OPT-ILLEGAL" },
#endif
#if defined(GDBM_READER)
  { ":READER" },
#endif
#if defined(GDBM_READER_CANT_DELETE)
  { ":READER-CANT-DELETE" },
#endif
#if defined(GDBM_READER_CANT_REORGANIZE)
  { ":READER-CANT-REORGANIZE" },
#endif
#if defined(GDBM_READER_CANT_STORE)
  { ":READER-CANT-STORE" },
#endif
#if defined(HAVE_GDBM_OPEN)
  { ":READ-WRITE" },
#endif
#if defined(GDBM_REORGANIZE_FAILED)
  { ":REORGANIZE-FAILED" },
#endif
#if defined(GDBM_REPLACE)
  { ":REPLACE" },
#endif
#if defined(GDBM_SYNC)
  { ":SYNC" },
#endif
#if (defined(GDBM_GETSYNCMODE)) || (defined(GDBM_SETSYNCMODE))
  { ":SYNCMODE" },
#endif
#if (defined(HAVE_GDBM_FETCH)) || (defined(HAVE_GDBM_FIRSTKEY)) || (defined(HAVE_GDBM_NEXTKEY))
  { ":TYPE" },
#endif
#if defined(GDBM_UNKNOWN_ERROR)
  { ":UNKNOWN-ERROR" },
#endif
#if defined(GDBM_WRCREAT)
  { ":WRCREAT" },
#endif
#if defined(GDBM_WRITER)
  { ":WRITER" },
#endif
  { "\"32BIT-VECTOR conversion requires a datum length divisible by 4\"" },
  { "\"desired lisp type not specified\"" },
  { "\"invalid lisp object type: \"" },
  { "\"open GDBM file required\"" },
  { "\"~S: ~A\"" },
  { "(OR NULL INTEGER (MEMBER CL::STRING CL::VECTOR EXT::8BIT-VECTOR EXT::32BIT-VECTOR CL::INTEGER CL::SINGLE-FLOAT CL::DOUBLE-FLOAT))" },
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(GDBM_FAST)
    " " ":FAST"
#  endif
#  if defined(GDBM_SYNC)
    " " ":SYNC"
#  endif
#  if defined(GDBM_NOLOCK)
    " " ":NOLOCK"
#  endif
#  if defined(GDBM_NOMMAP)
    " " ":NOMMAP"
#  endif
#  if defined(GDBM_CLOEXEC)
    " " ":CLOEXEC"
#  endif
#  if defined(GDBM_BSEXACT)
    " " ":BSEXACT"
#  endif
    " " "))"
  },
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(GDBM_GETFLAGS)
    " " ":FLAGS"
#  endif
#  if defined(GDBM_GETMMAP)
    " " ":MMAP"
#  endif
#  if defined(GDBM_GETCACHESIZE)
    " " ":CACHESIZE"
#  endif
#  if defined(GDBM_GETSYNCMODE)
    " " ":SYNCMODE"
#  endif
#  if defined(GDBM_GETCENTFREE)
    " " ":CENTFREE"
#  endif
#  if defined(GDBM_GETCOALESCEBLKS)
    " " ":COALESCEBLKS"
#  endif
#  if defined(GDBM_GETMAXMAPSIZE)
    " " ":MAXMAPSIZE"
#  endif
#  if defined(GDBM_GETDBNAME)
    " " ":DBNAME"
#  endif
#  if defined(GDBM_GETBLOCKSIZE)
    " " ":BLOCKSIZE"
#  endif
#  if defined(GDBM_GETDEFAULT_VALUE_TYPE)
    " " ":DEFAULT-VALUE-TYPE"
#  endif
#  if defined(GDBM_GETDEFAULT_KEY_TYPE)
    " " ":DEFAULT-KEY-TYPE"
#  endif
    " " "))"
  },
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(GDBM_NO_ERROR)
    " " ":NO-ERROR"
#  endif
#  if defined(GDBM_MALLOC_ERROR)
    " " ":MALLOC-ERROR"
#  endif
#  if defined(GDBM_BLOCK_SIZE_ERROR)
    " " ":BLOCK-SIZE-ERROR"
#  endif
#  if defined(GDBM_FILE_OPEN_ERROR)
    " " ":FILE-OPEN-ERROR"
#  endif
#  if defined(GDBM_FILE_WRITE_ERROR)
    " " ":FILE-WRITE-ERROR"
#  endif
#  if defined(GDBM_FILE_SEEK_ERROR)
    " " ":FILE-SEEK-ERROR"
#  endif
#  if defined(GDBM_FILE_READ_ERROR)
    " " ":FILE-READ-ERROR"
#  endif
#  if defined(GDBM_BAD_MAGIC_NUMBER)
    " " ":BAD-MAGIC-NUMBER"
#  endif
#  if defined(GDBM_EMPTY_DATABASE)
    " " ":EMPTY-DATABASE"
#  endif
#  if defined(GDBM_CANT_BE_READER)
    " " ":CANT-BE-READER"
#  endif
#  if defined(GDBM_CANT_BE_WRITER)
    " " ":CANT-BE-WRITER"
#  endif
#  if defined(GDBM_READER_CANT_DELETE)
    " " ":READER-CANT-DELETE"
#  endif
#  if defined(GDBM_READER_CANT_STORE)
    " " ":READER-CANT-STORE"
#  endif
#  if defined(GDBM_READER_CANT_REORGANIZE)
    " " ":READER-CANT-REORGANIZE"
#  endif
#  if defined(GDBM_UNKNOWN_ERROR)
    " " ":UNKNOWN-ERROR"
#  endif
#  if defined(GDBM_ITEM_NOT_FOUND)
    " " ":ITEM-NOT-FOUND"
#  endif
#  if defined(GDBM_REORGANIZE_FAILED)
    " " ":REORGANIZE-FAILED"
#  endif
#  if defined(GDBM_CANNOT_REPLACE)
    " " ":CANNOT-REPLACE"
#  endif
#  if defined(GDBM_ILLEGAL_DATA)
    " " ":ILLEGAL-DATA"
#  endif
#  if defined(GDBM_OPT_ALREADY_SET)
    " " ":OPT-ALREADY-SET"
#  endif
#  if defined(GDBM_OPT_ILLEGAL)
    " " ":OPT-ILLEGAL"
#  endif
#  if defined(GDBM_BYTE_SWAPPED)
    " " ":BYTE-SWAPPED"
#  endif
#  if defined(GDBM_BAD_FILE_OFFSET)
    " " ":BAD-FILE-OFFSET"
#  endif
#  if defined(GDBM_BAD_OPEN_FLAGS)
    " " ":BAD-OPEN-FLAGS"
#  endif
#  if defined(GDBM_FILE_STAT_ERROR)
    " " ":FILE-STAT-ERROR"
#  endif
#  if defined(GDBM_FILE_EOF)
    " " ":FILE-EOF"
#  endif
#  if defined(GDBM_NO_DBNAME)
    " " ":NO-DBNAME"
#  endif
#  if defined(GDBM_ERR_FILE_OWNER)
    " " ":ERR-FILE-OWNER"
#  endif
#  if defined(GDBM_ERR_FILE_MODE)
    " " ":ERR-FILE-MODE"
#  endif
#  if defined(GDBM_NEED_RECOVERY)
    " " ":NEED-RECOVERY"
#  endif
#  if defined(GDBM_BACKUP_FAILED)
    " " ":BACKUP-FAILED"
#  endif
#  if defined(GDBM_DIR_OVERFLOW)
    " " ":DIR-OVERFLOW"
#  endif
#  if defined(GDBM_BAD_BUCKET)
    " " ":BAD-BUCKET"
#  endif
#  if defined(GDBM_BAD_HEADER)
    " " ":BAD-HEADER"
#  endif
#  if defined(GDBM_BAD_AVAIL)
    " " ":BAD-AVAIL"
#  endif
#  if defined(GDBM_BAD_HASH_TABLE)
    " " ":BAD-HASH-TABLE"
#  endif
#  if defined(GDBM_BAD_DIR_ENTRY)
    " " ":BAD-DIR-ENTRY"
#  endif
    " " "))"
  },
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(GDBM_SETCACHESIZE)
    " " ":CACHESIZE"
#  endif
#  if defined(GDBM_SETSYNCMODE)
    " " ":SYNCMODE"
#  endif
#  if defined(GDBM_SETCENTFREE)
    " " ":CENTFREE"
#  endif
#  if defined(GDBM_SETCOALESCEBLKS)
    " " ":COALESCEBLKS"
#  endif
#  if defined(GDBM_SETMAXMAPSIZE)
    " " ":MAXMAPSIZE"
#  endif
#  if defined(GDBM_SETMMAP)
    " " ":MMAP"
#  endif
#  if defined(GDBM_SETDEFAULT_VALUE_TYPE)
    " " ":DEFAULT-VALUE-TYPE"
#  endif
#  if defined(GDBM_SETDEFAULT_KEY_TYPE)
    " " ":DEFAULT-KEY-TYPE"
#  endif
    " " "))"
  },
  {
    " " "(OR NULL INTEGER (MEMBER"
#  if defined(GDBM_READER)
    " " ":READER"
#  endif
#  if defined(GDBM_WRITER)
    " " ":WRITER"
#  endif
#  if defined(GDBM_WRCREAT)
    " " ":WRCREAT"
#  endif
#  if defined(GDBM_NEWDB)
    " " ":NEWDB"
#  endif
    " " "))"
  },
  {
    " " "(OR NULL INTEGER (MEMBER"
#  if defined(GDBM_REPLACE)
    " " ":REPLACE"
#  endif
#  if defined(GDBM_INSERT)
    " " ":INSERT"
#  endif
    " " "))"
  },
  { "NIL" },
  0
};

struct module__gdbm__subr_tab_t {
  VAROBJECTS_ALIGNMENT_DUMMY_DECL
#if (defined(HAVE_GDBM_OPEN)) || (defined(HAVE_GDBM_CLOSE))
  subr_t _subr_gdbm_gdbm_close;
#endif
#if defined(HAVE_GDBM_COUNT)
  subr_t _subr_gdbm_gdbm_count;
#endif
#if defined(HAVE_GDBM_DELETE)
  subr_t _subr_gdbm_gdbm_delete;
#endif
#if defined(HAVE_GDBM_EXISTS)
  subr_t _subr_gdbm_gdbm_exists;
#endif
#if defined(HAVE_GDBM_FETCH)
  subr_t _subr_gdbm_gdbm_fetch;
#endif
#if defined(HAVE_GDBM_FDESC)
  subr_t _subr_gdbm_gdbm_file_size;
#endif
#if defined(HAVE_GDBM_FIRSTKEY)
  subr_t _subr_gdbm_gdbm_firstkey;
#endif
#if defined(HAVE_GDBM_NEXTKEY)
  subr_t _subr_gdbm_gdbm_nextkey;
#endif
#if defined(HAVE_GDBM_OPEN)
  subr_t _subr_gdbm_gdbm_open;
#endif
  subr_t _subr_gdbm_gdbm_opt;
#if defined(HAVE_GDBM_REORGANIZE)
  subr_t _subr_gdbm_gdbm_reorganize;
#endif
#if defined(HAVE_GDBM_STORE)
  subr_t _subr_gdbm_gdbm_store;
#endif
#if defined(HAVE_GDBM_SYNC)
  subr_t _subr_gdbm_gdbm_sync;
#endif
  subr_t _subr_gdbm_gdbm_version;
  subr_t _subr_gdbm__25set_gdbm_opt;
  int _dummy_to_avoid_trailing_comma_in_initializer;
};
extern struct module__gdbm__subr_tab_t module__gdbm__subr_tab;

struct module__gdbm__stab_t {
#if (defined(HAVE_GDBM_OPEN)) || (defined(HAVE_GDBM_CLOSE))
  subr_t _subr_gdbm_gdbm_close;
#endif
#if defined(HAVE_GDBM_COUNT)
  subr_t _subr_gdbm_gdbm_count;
#endif
#if defined(HAVE_GDBM_DELETE)
  subr_t _subr_gdbm_gdbm_delete;
#endif
#if defined(HAVE_GDBM_EXISTS)
  subr_t _subr_gdbm_gdbm_exists;
#endif
#if defined(HAVE_GDBM_FETCH)
  subr_t _subr_gdbm_gdbm_fetch;
#endif
#if defined(HAVE_GDBM_FDESC)
  subr_t _subr_gdbm_gdbm_file_size;
#endif
#if defined(HAVE_GDBM_FIRSTKEY)
  subr_t _subr_gdbm_gdbm_firstkey;
#endif
#if defined(HAVE_GDBM_NEXTKEY)
  subr_t _subr_gdbm_gdbm_nextkey;
#endif
#if defined(HAVE_GDBM_OPEN)
  subr_t _subr_gdbm_gdbm_open;
#endif
  subr_t _subr_gdbm_gdbm_opt;
#if defined(HAVE_GDBM_REORGANIZE)
  subr_t _subr_gdbm_gdbm_reorganize;
#endif
#if defined(HAVE_GDBM_STORE)
  subr_t _subr_gdbm_gdbm_store;
#endif
#if defined(HAVE_GDBM_SYNC)
  subr_t _subr_gdbm_gdbm_sync;
#endif
  subr_t _subr_gdbm_gdbm_version;
  subr_t _subr_gdbm__25set_gdbm_opt;
  int _dummy_to_avoid_trailing_comma_in_initializer;
};
static struct module__gdbm__stab_t* this_module_stab;


static const c_lisp_pair_t check_data_type_table[] = {
  { GDBM_DATA_STRING, &(O(object_cl__string)) },
  { GDBM_DATA_VECTOR, &(O(object_cl__vector)) },
  { GDBM_DATA_8BIT_VECTOR, &(O(object_ext__8bit_vector)) },
  { GDBM_DATA_32BIT_VECTOR, &(O(object_ext__32bit_vector)) },
  { GDBM_DATA_INTEGER, &(O(object_cl__integer)) },
  { GDBM_DATA_SINGLE_FLOAT, &(O(object_cl__single_float)) },
  { GDBM_DATA_DOUBLE_FLOAT, &(O(object_cl__double_float)) },
  { 0, NULL }
};
static const c_lisp_map_t check_data_type_map = {
  check_data_type_table,
  (sizeof(check_data_type_table)/sizeof(c_lisp_pair_t))-1,
  GDBM_DATA_NOTYPE,true,
  "check_data_type"
};
#define check_data_type(a) (gdbm_data_t)map_lisp_to_c(a,&check_data_type_map)
#define check_data_type_reverse(a) map_c_to_lisp(a,&check_data_type_map)

static const c_lisp_pair_t check_gdbm_errno_table[] = {
 #ifdef GDBM_NO_ERROR
  { GDBM_NO_ERROR, &(O(object_Kno_error)) },
 #endif
 #ifdef GDBM_MALLOC_ERROR
  { GDBM_MALLOC_ERROR, &(O(object_Kmalloc_error)) },
 #endif
 #ifdef GDBM_BLOCK_SIZE_ERROR
  { GDBM_BLOCK_SIZE_ERROR, &(O(object_Kblock_size_error)) },
 #endif
 #ifdef GDBM_FILE_OPEN_ERROR
  { GDBM_FILE_OPEN_ERROR, &(O(object_Kfile_open_error)) },
 #endif
 #ifdef GDBM_FILE_WRITE_ERROR
  { GDBM_FILE_WRITE_ERROR, &(O(object_Kfile_write_error)) },
 #endif
 #ifdef GDBM_FILE_SEEK_ERROR
  { GDBM_FILE_SEEK_ERROR, &(O(object_Kfile_seek_error)) },
 #endif
 #ifdef GDBM_FILE_READ_ERROR
  { GDBM_FILE_READ_ERROR, &(O(object_Kfile_read_error)) },
 #endif
 #ifdef GDBM_BAD_MAGIC_NUMBER
  { GDBM_BAD_MAGIC_NUMBER, &(O(object_Kbad_magic_number)) },
 #endif
 #ifdef GDBM_EMPTY_DATABASE
  { GDBM_EMPTY_DATABASE, &(O(object_Kempty_database)) },
 #endif
 #ifdef GDBM_CANT_BE_READER
  { GDBM_CANT_BE_READER, &(O(object_Kcant_be_reader)) },
 #endif
 #ifdef GDBM_CANT_BE_WRITER
  { GDBM_CANT_BE_WRITER, &(O(object_Kcant_be_writer)) },
 #endif
 #ifdef GDBM_READER_CANT_DELETE
  { GDBM_READER_CANT_DELETE, &(O(object_Kreader_cant_delete)) },
 #endif
 #ifdef GDBM_READER_CANT_STORE
  { GDBM_READER_CANT_STORE, &(O(object_Kreader_cant_store)) },
 #endif
 #ifdef GDBM_READER_CANT_REORGANIZE
  { GDBM_READER_CANT_REORGANIZE, &(O(object_Kreader_cant_reorganize)) },
 #endif
 #ifdef GDBM_UNKNOWN_ERROR
  { GDBM_UNKNOWN_ERROR, &(O(object_Kunknown_error)) },
 #endif
 #ifdef GDBM_ITEM_NOT_FOUND
  { GDBM_ITEM_NOT_FOUND, &(O(object_Kitem_not_found)) },
 #endif
 #ifdef GDBM_REORGANIZE_FAILED
  { GDBM_REORGANIZE_FAILED, &(O(object_Kreorganize_failed)) },
 #endif
 #ifdef GDBM_CANNOT_REPLACE
  { GDBM_CANNOT_REPLACE, &(O(object_Kcannot_replace)) },
 #endif
 #ifdef GDBM_ILLEGAL_DATA
  { GDBM_ILLEGAL_DATA, &(O(object_Killegal_data)) },
 #endif
 #ifdef GDBM_OPT_ALREADY_SET
  { GDBM_OPT_ALREADY_SET, &(O(object_Kopt_already_set)) },
 #endif
 #ifdef GDBM_OPT_ILLEGAL
  { GDBM_OPT_ILLEGAL, &(O(object_Kopt_illegal)) },
 #endif
 #ifdef GDBM_BYTE_SWAPPED
  { GDBM_BYTE_SWAPPED, &(O(object_Kbyte_swapped)) },
 #endif
 #ifdef GDBM_BAD_FILE_OFFSET
  { GDBM_BAD_FILE_OFFSET, &(O(object_Kbad_file_offset)) },
 #endif
 #ifdef GDBM_BAD_OPEN_FLAGS
  { GDBM_BAD_OPEN_FLAGS, &(O(object_Kbad_open_flags)) },
 #endif
 #ifdef GDBM_FILE_STAT_ERROR
  { GDBM_FILE_STAT_ERROR, &(O(object_Kfile_stat_error)) },
 #endif
 #ifdef GDBM_FILE_EOF
  { GDBM_FILE_EOF, &(O(object_Kfile_eof)) },
 #endif
 #ifdef GDBM_NO_DBNAME
  { GDBM_NO_DBNAME, &(O(object_Kno_dbname)) },
 #endif
 #ifdef GDBM_ERR_FILE_OWNER
  { GDBM_ERR_FILE_OWNER, &(O(object_Kerr_file_owner)) },
 #endif
 #ifdef GDBM_ERR_FILE_MODE
  { GDBM_ERR_FILE_MODE, &(O(object_Kerr_file_mode)) },
 #endif
 #ifdef GDBM_NEED_RECOVERY
  { GDBM_NEED_RECOVERY, &(O(object_Kneed_recovery)) },
 #endif
 #ifdef GDBM_BACKUP_FAILED
  { GDBM_BACKUP_FAILED, &(O(object_Kbackup_failed)) },
 #endif
 #ifdef GDBM_DIR_OVERFLOW
  { GDBM_DIR_OVERFLOW, &(O(object_Kdir_overflow)) },
 #endif
 #ifdef GDBM_BAD_BUCKET
  { GDBM_BAD_BUCKET, &(O(object_Kbad_bucket)) },
 #endif
 #ifdef GDBM_BAD_HEADER
  { GDBM_BAD_HEADER, &(O(object_Kbad_header)) },
 #endif
 #ifdef GDBM_BAD_AVAIL
  { GDBM_BAD_AVAIL, &(O(object_Kbad_avail)) },
 #endif
 #ifdef GDBM_BAD_HASH_TABLE
  { GDBM_BAD_HASH_TABLE, &(O(object_Kbad_hash_table)) },
 #endif
 #ifdef GDBM_BAD_DIR_ENTRY
  { GDBM_BAD_DIR_ENTRY, &(O(object_Kbad_dir_entry)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_gdbm_errno_map = {
  check_gdbm_errno_table,
  (sizeof(check_gdbm_errno_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_gdbm_errno"
};
#define check_gdbm_errno(a) (int)map_lisp_to_c(a,&check_gdbm_errno_map)
#define check_gdbm_errno_reverse(a) map_c_to_lisp(a,&check_gdbm_errno_map)

static const c_lisp_pair_t gdbm_open_read_write_table[] = {
 #ifdef GDBM_READER
  { GDBM_READER, &(O(object_Kreader)) },
 #endif
 #ifdef GDBM_WRITER
  { GDBM_WRITER, &(O(object_Kwriter)) },
 #endif
 #ifdef GDBM_WRCREAT
  { GDBM_WRCREAT, &(O(object_Kwrcreat)) },
 #endif
 #ifdef GDBM_NEWDB
  { GDBM_NEWDB, &(O(object_Knewdb)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t gdbm_open_read_write_map = {
  gdbm_open_read_write_table,
  (sizeof(gdbm_open_read_write_table)/sizeof(c_lisp_pair_t))-1,
# if defined(GDBM_WRCREAT)
  GDBM_WRCREAT,true,
# else
  0,false,
# endif
  "gdbm_open_read_write"
};
#define gdbm_open_read_write(a) (int)map_lisp_to_c(a,&gdbm_open_read_write_map)
#define gdbm_open_read_write_reverse(a) map_c_to_lisp(a,&gdbm_open_read_write_map)

static const c_lisp_pair_t gdbm_open_flags_table[] = {
 #ifdef GDBM_FAST
  { GDBM_FAST, &(O(object_Kfast)) },
 #endif
 #ifdef GDBM_SYNC
  { GDBM_SYNC, &(O(object_Ksync)) },
 #endif
 #ifdef GDBM_NOLOCK
  { GDBM_NOLOCK, &(O(object_Knolock)) },
 #endif
 #ifdef GDBM_NOMMAP
  { GDBM_NOMMAP, &(O(object_Knommap)) },
 #endif
 #ifdef GDBM_CLOEXEC
  { GDBM_CLOEXEC, &(O(object_Kcloexec)) },
 #endif
 #ifdef GDBM_BSEXACT
  { GDBM_BSEXACT, &(O(object_Kbsexact)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t gdbm_open_flags_map = {
  gdbm_open_flags_table,
  (sizeof(gdbm_open_flags_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "gdbm_open_flags"
};
#define gdbm_open_flags(a) (int)map_lisp_to_c(a,&gdbm_open_flags_map)
#define gdbm_open_flags_reverse(a) map_c_to_lisp(a,&gdbm_open_flags_map)
#define gdbm_open_flags_to_list(a) map_c_to_list(a,&gdbm_open_flags_map)
#define gdbm_open_flags_of_list(a) map_list_to_c(a,&gdbm_open_flags_map)

static const c_lisp_pair_t gdbm_store_flag_table[] = {
 #ifdef GDBM_REPLACE
  { GDBM_REPLACE, &(O(object_Kreplace)) },
 #endif
 #ifdef GDBM_INSERT
  { GDBM_INSERT, &(O(object_Kinsert)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t gdbm_store_flag_map = {
  gdbm_store_flag_table,
  (sizeof(gdbm_store_flag_table)/sizeof(c_lisp_pair_t))-1,
# if defined(GDBM_REPLACE)
  GDBM_REPLACE,true,
# else
  0,false,
# endif
  "gdbm_store_flag"
};
#define gdbm_store_flag(a) (int)map_lisp_to_c(a,&gdbm_store_flag_map)
#define gdbm_store_flag_reverse(a) map_c_to_lisp(a,&gdbm_store_flag_map)

static const c_lisp_pair_t gdbm_getopt_option_table[] = {
 #ifdef GDBM_GETFLAGS
  { GDBM_GETFLAGS, &(O(object_Kflags)) },
 #endif
 #ifdef GDBM_GETMMAP
  { GDBM_GETMMAP, &(O(object_Kmmap)) },
 #endif
 #ifdef GDBM_GETCACHESIZE
  { GDBM_GETCACHESIZE, &(O(object_Kcachesize)) },
 #endif
 #ifdef GDBM_GETSYNCMODE
  { GDBM_GETSYNCMODE, &(O(object_Ksyncmode)) },
 #endif
 #ifdef GDBM_GETCENTFREE
  { GDBM_GETCENTFREE, &(O(object_Kcentfree)) },
 #endif
 #ifdef GDBM_GETCOALESCEBLKS
  { GDBM_GETCOALESCEBLKS, &(O(object_Kcoalesceblks)) },
 #endif
 #ifdef GDBM_GETMAXMAPSIZE
  { GDBM_GETMAXMAPSIZE, &(O(object_Kmaxmapsize)) },
 #endif
 #ifdef GDBM_GETDBNAME
  { GDBM_GETDBNAME, &(O(object_Kdbname)) },
 #endif
 #ifdef GDBM_GETBLOCKSIZE
  { GDBM_GETBLOCKSIZE, &(O(object_Kblocksize)) },
 #endif
 #ifdef GDBM_GETDEFAULT_VALUE_TYPE
  { GDBM_GETDEFAULT_VALUE_TYPE, &(O(object_Kdefault_value_type)) },
 #endif
 #ifdef GDBM_GETDEFAULT_KEY_TYPE
  { GDBM_GETDEFAULT_KEY_TYPE, &(O(object_Kdefault_key_type)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t gdbm_getopt_option_map = {
  gdbm_getopt_option_table,
  (sizeof(gdbm_getopt_option_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "gdbm_getopt_option"
};
#define gdbm_getopt_option(a) (int)map_lisp_to_c(a,&gdbm_getopt_option_map)
#define gdbm_getopt_option_reverse(a) map_c_to_lisp(a,&gdbm_getopt_option_map)

static const c_lisp_pair_t gdbm_setopt_option_table[] = {
 #ifdef GDBM_SETCACHESIZE
  { GDBM_SETCACHESIZE, &(O(object_Kcachesize)) },
 #endif
 #ifdef GDBM_SETSYNCMODE
  { GDBM_SETSYNCMODE, &(O(object_Ksyncmode)) },
 #endif
 #ifdef GDBM_SETCENTFREE
  { GDBM_SETCENTFREE, &(O(object_Kcentfree)) },
 #endif
 #ifdef GDBM_SETCOALESCEBLKS
  { GDBM_SETCOALESCEBLKS, &(O(object_Kcoalesceblks)) },
 #endif
 #ifdef GDBM_SETMAXMAPSIZE
  { GDBM_SETMAXMAPSIZE, &(O(object_Kmaxmapsize)) },
 #endif
 #ifdef GDBM_SETMMAP
  { GDBM_SETMMAP, &(O(object_Kmmap)) },
 #endif
 #ifdef GDBM_SETDEFAULT_VALUE_TYPE
  { GDBM_SETDEFAULT_VALUE_TYPE, &(O(object_Kdefault_value_type)) },
 #endif
 #ifdef GDBM_SETDEFAULT_KEY_TYPE
  { GDBM_SETDEFAULT_KEY_TYPE, &(O(object_Kdefault_key_type)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t gdbm_setopt_option_map = {
  gdbm_setopt_option_table,
  (sizeof(gdbm_setopt_option_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "gdbm_setopt_option"
};
#define gdbm_setopt_option(a) (int)map_lisp_to_c(a,&gdbm_setopt_option_map)
#define gdbm_setopt_option_reverse(a) map_c_to_lisp(a,&gdbm_setopt_option_map)


#line 53

/* CL::BIT-VECTOR EXT::2BIT-VECTOR EXT::4BIT-VECTOR EXT::16BIT-VECTOR */
#line 58

#line 68

static _Noreturn void error_gdbm (const char *fatal_message) {
  end_blocking_system_call(); /* in case we are called from _gdbm_fatal() */
  pushSTACK(O(object_gdbm__gdbm_error));
  pushSTACK(O(object_Kmessage));
  if (fatal_message) {
    pushSTACK(asciz_to_string(fatal_message, GLO(misc_encoding)));
    pushSTACK(S(Kcode)); pushSTACK(O(object_Kfatal));
  } else {
    pushSTACK(safe_to_string(gdbm_strerror(gdbm_errno)));
    pushSTACK(S(Kcode)); pushSTACK(check_gdbm_errno_reverse(gdbm_errno));
  }
  pushSTACK(O(object__22_7Es__20_7Ea_22));
  pushSTACK(TheSubr(subr_self)->name);
  pushSTACK(STACK_4); /* message */
  funcall(L(error_of_type), 8);
  NOTREACHED;
}

DEFVAR(gdbm_version_cache)
DEFUN(GDBM::GDBM-VERSION,,(subr_gdbm_gdbm_version,seclass_default,0,0,norest,nokey,0,NIL))
{ VALUES1(!nullp(O(gdbm_version_cache)) ? O(gdbm_version_cache)
          : (O(gdbm_version_cache) = safe_to_string(gdbm_version))); }

/* can trigger GC */
#define GDBM_SLOT_FILE  1
#define GDBM_SLOT_PATH  2
#define GDBM_SLOT_KEY   3
#define GDBM_SLOT_VAL   4
static GDBM_FILE check_gdbm (gcv_object_t *gdbm, gdbm_data_t *key,
                             gdbm_data_t *val, bool require_valid_handle)
{ /* gdbm is modified, so it has to be a pointer */
  *gdbm = check_classname(*gdbm,O(object_gdbm__gdbm));
  if (key && *key == GDBM_DATA_NOTYPE)
    *key = (gdbm_data_t)
      posfixnum_to_V(TheStructure(*gdbm)->recdata[GDBM_SLOT_KEY]);
  if (val && *val == GDBM_DATA_NOTYPE)
    *val = (gdbm_data_t)
      posfixnum_to_V(TheStructure(*gdbm)->recdata[GDBM_SLOT_VAL]);
  { object fp = TheStructure(*gdbm)->recdata[GDBM_SLOT_FILE];
    if (fpointerp(fp)) return (GDBM_FILE)TheFpointer(fp)->fp_pointer;
    else if (require_valid_handle) {
      pushSTACK(O(object_gdbm__gdbm_error));
      pushSTACK(O(object_Kmessage));
      pushSTACK(O(object__22open_20gdbm_20file_20required_22));
      pushSTACK(S(Kcode)); pushSTACK(O(object_Kclosed_file));
      pushSTACK(O(object__22_7Es__20_7Ea_22));
      pushSTACK(TheSubr(subr_self)->name);
      pushSTACK(STACK_4); /* message */
      funcall(L(error_of_type), 8);
      NOTREACHED;
    } else return NULL;
  }
}

#define SYSCALL(statement) begin_blocking_system_call(); statement; end_blocking_system_call()

static object open_gdbm (object path, int bsize, int rw, int mode) {
  GDBM_FILE gdbm;
  with_string_0(path, GLO(pathname_encoding), name, {
      SYSCALL(gdbm = gdbm_open(name, bsize, rw, mode, error_gdbm));
    });
  if (gdbm == NULL) error_gdbm(NULL);
  return allocate_fpointer(gdbm);
}

#line 135

#line 137

#if defined(HAVE_GDBM_OPEN)
#line 140
DEFUN(GDBM::GDBM-OPEN, name &key BLOCKSIZE READ-WRITE        OPTIONS MODE DEFAULT-KEY-TYPE DEFAULT-VALUE-TYPE,(subr_gdbm_gdbm_open,seclass_default,1,0,norest,key,6,NIL))
{
  gdbm_data_t default_value_type = check_data_type(popSTACK());
  gdbm_data_t default_key_type = check_data_type(popSTACK());
  int mode = check_uint_defaulted(popSTACK(), 0644);
  int rw_opt1 = gdbm_open_flags_of_list(popSTACK());
  int rw_opt2 = gdbm_open_read_write(popSTACK());
  int rw = rw_opt1 | rw_opt2;
  int bsize = check_uint_defaulted(popSTACK(), 512);
  if (typep_classname(STACK_0,O(object_gdbm__gdbm))) { /* reuse */
    if (!check_gdbm(&STACK_0,&default_key_type,&default_value_type,false)) {
      value1 = open_gdbm(TheStructure(STACK_0)->recdata[GDBM_SLOT_PATH],
                         bsize, rw, mode); /* reopen */
      TheStructure(STACK_0)->recdata[GDBM_SLOT_FILE] = value1;
    }
    TheStructure(STACK_0)->recdata[GDBM_SLOT_KEY]=fixnum(default_key_type);
    TheStructure(STACK_0)->recdata[GDBM_SLOT_VAL]=fixnum(default_value_type);
    VALUES1(popSTACK());        /* return the argument */
    return;
  }
  pushSTACK(open_gdbm(physical_namestring(STACK_0), bsize, rw, mode));
  pushSTACK(STACK_1);         /* path */
  pushSTACK(fixnum(default_key_type));
  pushSTACK(fixnum(default_value_type));
  funcall(O(object_gdbm__make_gdbm),4);
  STACK_0 = value1;        /* save GDBM object, drop path */
  pushSTACK(STACK_0); pushSTACK(F(subr_gdbm_gdbm_close));
  funcall(L(finalize),2);
  VALUES1(popSTACK());      /* restore */
}
#endif  /* HAVE_GDBM_OPEN */

#if defined(HAVE_GDBM_CLOSE)
DEFUN(GDBM:GDBM-CLOSE, dbf,(subr_gdbm_gdbm_close,seclass_default,1,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_0,NULL,NULL,false);
  if (dbf) {
    SYSCALL(gdbm_close(dbf));
    TheStructure(STACK_0)->recdata[GDBM_SLOT_FILE] = NIL;
    VALUES1(T);
  } else
    VALUES1(NIL);
  skipSTACK(1);
}
#endif  /* HAVE_GDBM_CLOSE */

#if defined(HAVE_GDBM_FDESC)
DEFUN(GDBM:GDBM-FILE-SIZE, dbf,(subr_gdbm_gdbm_file_size,seclass_default,1,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_0,NULL,NULL,true);
  off_t ret;
  SYSCALL(ret = handle_length(NULL,gdbm_fdesc(dbf)));
  VALUES1(off_to_I(ret)); skipSTACK(1);
}
#endif  /* HAVE_GDBM_FDESC */

static object coerce_bitvector (object arg) {
  if (bit_vector_p(Atype_8Bit,arg)) return arg;
  else {
    pushSTACK(arg); pushSTACK(GLO(type_uint8_vector));
    funcall(L(coerce),2);
    if (!bit_vector_p(Atype_8Bit,value1)) { NOTREACHED; }
    return value1;
  }
}

static _Noreturn void error_bad_type (object lisp_obj) {
  pushSTACK(O(object_gdbm__gdbm_error));
  pushSTACK(O(object_Kmessage));
  pushSTACK(O(object__22invalid_20lisp_20object_20type__20_22));
  pushSTACK(lisp_obj); funcall(L(prin1_to_string),1);
  pushSTACK(value1); value1 = string_concat(2); pushSTACK(value1);
  pushSTACK(S(Kcode)); pushSTACK(O(object_Klisp_type));
  pushSTACK(O(object__22_7Es__20_7Ea_22));
  pushSTACK(TheSubr(subr_self)->name);
  pushSTACK(STACK_4); /* message */
  funcall(L(error_of_type), 8);
  NOTREACHED;
}

#line 258
#define with_datum(lisp_obj, datum_var, statement)  do {                   datum datum_var;                                                         if (stringp(lisp_obj)) {                                                   with_string_0(lisp_obj, GLO(misc_encoding), datum_var##string, {           datum_var.dptr = datum_var##string;                                      datum_var.dsize = datum_var##string_len;                                 SYSCALL(statement);                                                    });                                                                    } else if (bit_vector_p(Atype_32Bit,lisp_obj)) {                           datum_var.dsize = 4 * vector_length(lisp_obj);                           datum_var.dptr = (char*)TheSbvector(lisp_obj)->data;                     SYSCALL(statement);                                                    } else if (vectorp(lisp_obj)) { /* assume Atype_8Bit */                    lisp_obj = coerce_bitvector(lisp_obj);                                   datum_var.dsize = vector_length(lisp_obj);                               datum_var.dptr = (char*)TheSbvector(lisp_obj)->data;                     SYSCALL(statement);                                                    } else if (integerp(lisp_obj)) {                                           unsigned long datum_var##bitsize =                                         1 + I_integer_length(lisp_obj); /* an extra bit for the sign */        datum_var.dsize = ceiling(datum_var##bitsize,8);                         datum_var.dptr = (char*)alloca(datum_var.dsize);                         if (I_to_LEbytes(lisp_obj,8*datum_var.dsize,(uintB*)datum_var.dptr))        NOTREACHED; /* there must not be an overflow! */                       SYSCALL(statement);                                                    } else if (single_float_p(lisp_obj)) {                                     ffloatjanus datum_var##ffloat;                                           FF_to_c_float(lisp_obj, &datum_var##ffloat);                             datum_var.dptr = (char*)&datum_var##ffloat;                              datum_var.dsize = sizeof(ffloat);                                        SYSCALL(statement);                                                    } else if (double_float_p(lisp_obj)) {                                     dfloatjanus datum_var##dfloat;                                           DF_to_c_double(lisp_obj, &datum_var##dfloat);                            datum_var.dptr = (char*)&datum_var##dfloat;                              datum_var.dsize = sizeof(dfloat);                                        SYSCALL(statement);                                                    } else error_bad_type(lisp_obj);                                        } while(0)



#if defined(HAVE_GDBM_STORE)
DEFUN(GDBM:GDBM-STORE, dbf key content &key FLAG,(subr_gdbm_gdbm_store,seclass_default,3,0,norest,key,1,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_3,NULL,NULL,true);
  int flag = gdbm_store_flag(STACK_0), status;
  with_datum(STACK_2, key,
             with_datum(STACK_1, content,
                        status = gdbm_store(dbf, key, content, flag)));
  if (status) error_gdbm(NULL); /* reader call */
  VALUES0; skipSTACK(4);        /* cleanup */
}
#endif  /* HAVE_GDBM_STORE */

/* convert datum to Lisp string and release memory in datum
 can trigger GC */
static object datum_to_object (datum d, gdbm_data_t data_type) {
  if (d.dptr == NULL) return NIL;
  switch (data_type) {
    case GDBM_DATA_STRING: {
      object o = n_char_to_string(d.dptr, d.dsize, GLO(misc_encoding));
      free(d.dptr);
      return o;
    }
    case GDBM_DATA_VECTOR: case GDBM_DATA_8BIT_VECTOR: {
      object o = data_to_sb8vector(d.dptr,d.dsize);
      free(d.dptr);
      return o;
    }
    case GDBM_DATA_32BIT_VECTOR:
      if (d.dsize % 4) {
        pushSTACK(O(object_gdbm__gdbm_error));
        pushSTACK(O(object_Kmessage));
        pushSTACK(O(object__2232bit_vector_20conversion_20requires_20a_20datum_20length_20divisible_20by_204_22));
        pushSTACK(S(Kcode)); pushSTACK(O(object_Kdatum_type));
        pushSTACK(O(object__22_7Es__20_7Ea_22));
        pushSTACK(TheSubr(subr_self)->name);
        pushSTACK(STACK_4); /* message */
        funcall(L(error_of_type), 8);
        NOTREACHED;
      } else {
        object o = data_to_sbvector(Atype_32Bit,d.dsize/4,d.dptr,d.dsize);
        free(d.dptr);
        return o;
      }
    case GDBM_DATA_INTEGER: {
      object o = LEbytes_to_I(d.dsize,(uintB*)d.dptr);
      free(d.dptr);
      return o;
    }
    case GDBM_DATA_SINGLE_FLOAT: {
      object o = c_float_to_FF((ffloatjanus*)d.dptr);
      free(d.dptr);
      return o;
    }
    case GDBM_DATA_DOUBLE_FLOAT: {
      object o = c_double_to_DF((dfloatjanus*)d.dptr);
      free(d.dptr);
      return o;
    }
    case GDBM_DATA_NOTYPE:
      pushSTACK(O(object_gdbm__gdbm_error));
      pushSTACK(O(object_Kmessage));
      pushSTACK(O(object__22desired_20lisp_20type_20not_20specified_22));
      pushSTACK(S(Kcode)); pushSTACK(O(object_Kdatum_type));
      pushSTACK(O(object__22_7Es__20_7Ea_22));
      pushSTACK(TheSubr(subr_self)->name);
      pushSTACK(STACK_4); /* message */
      funcall(L(error_of_type), 8);
      NOTREACHED;
    default: NOTREACHED;        /* pacify the compiler */
  }
}

#if defined(HAVE_GDBM_FETCH)
DEFUN(GDBM:GDBM-FETCH, dbf key &key :TYPE,(subr_gdbm_gdbm_fetch,seclass_default,2,0,norest,key,1,NIL))
{
  gdbm_data_t data_type = check_data_type(popSTACK());
  GDBM_FILE dbf = check_gdbm(&STACK_1,NULL,&data_type,true);
  datum res;
  with_datum(STACK_0, key, res = gdbm_fetch(dbf,key));
  VALUES1(datum_to_object(res,data_type));
  skipSTACK(2);                 /* cleanup */
}
#endif  /* HAVE_GDBM_FETCH */

#if defined(HAVE_GDBM_DELETE)
DEFUN(GDBM:GDBM-DELETE, dbf key,(subr_gdbm_gdbm_delete,seclass_default,2,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_1,NULL,NULL,true);
  int status;
  with_datum(STACK_0, key, status = gdbm_delete(dbf,key));
  if (status) error_gdbm(NULL); /* reader call */
  VALUES0; skipSTACK(2);        /* cleanup */
}
#endif  /* HAVE_GDBM_DELETE */

#if defined(HAVE_GDBM_FIRSTKEY)
DEFUN(GDBM:GDBM-FIRSTKEY, dbf &key :TYPE,(subr_gdbm_gdbm_firstkey,seclass_default,1,0,norest,key,1,NIL))
{
  gdbm_data_t data_type = check_data_type(popSTACK());
  GDBM_FILE dbf = check_gdbm(&STACK_0,&data_type,NULL,true);
  datum res;
  SYSCALL(res = gdbm_firstkey(dbf));
  VALUES1(datum_to_object(res,data_type)); skipSTACK(1);
}
#endif  /* HAVE_GDBM_FIRSTKEY */

#if defined(HAVE_GDBM_NEXTKEY)
DEFUN(GDBM:GDBM-NEXTKEY, dbf key &key :TYPE,(subr_gdbm_gdbm_nextkey,seclass_default,2,0,norest,key,1,NIL))
{
  gdbm_data_t data_type = check_data_type(STACK_0);
  GDBM_FILE dbf = check_gdbm(&STACK_2,&data_type,NULL,true);
  datum res;
  with_datum(STACK_1, key, res = gdbm_nextkey(dbf,key));
  VALUES1(datum_to_object(res,data_type));
  skipSTACK(3);                 /* cleanup */
}
#endif  /* HAVE_GDBM_NEXTKEY */

#line 386
#define CHECK_RUN(statement)  do {                   int status;                                      SYSCALL(status = statement);                     if (status) error_gdbm(NULL);                    else VALUES0;                                  } while(0)

#if defined(HAVE_GDBM_REORGANIZE)
DEFUN(GDBM:GDBM-REORGANIZE, dbf,(subr_gdbm_gdbm_reorganize,seclass_default,1,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_0,NULL,NULL,true);
  CHECK_RUN(gdbm_reorganize(dbf)); skipSTACK(1);
}
#endif  /* HAVE_GDBM_REORGANIZE */

#if defined(HAVE_GDBM_SYNC)
DEFUN(GDBM:GDBM-SYNC, dbf,(subr_gdbm_gdbm_sync,seclass_default,1,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_0,NULL,NULL,true);
  SYSCALL(gdbm_sync(dbf));
  VALUES0; skipSTACK(1);
}
#endif  /* HAVE_GDBM_SYNC */

#if defined(HAVE_GDBM_EXISTS)
DEFUN(GDBM:GDBM-EXISTS, dbf key,(subr_gdbm_gdbm_exists,seclass_default,2,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_1,NULL,NULL,true);
  int status;
  with_datum(STACK_0, key, status = gdbm_exists(dbf, key));
  VALUES_IF(status);
  skipSTACK(2);                 /* cleanup */
}
#endif  /* HAVE_GDBM_EXISTS */

#if defined(HAVE_GDBM_COUNT)
DEFUN(GDBM:GDBM-COUNT, dbf,(subr_gdbm_gdbm_count,seclass_default,1,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_0,NULL,NULL,true);
  unsigned long long int count; /* gdbm_count_t */
  int status;
  SYSCALL(status = gdbm_count(dbf, &count));
  if (status) error_gdbm(NULL);
  VALUES1(uint64_to_I(count));
  skipSTACK(1);                 /* cleanup */
}
#endif  /* HAVE_GDBM_COUNT */

#line 432

DEFUN(GDBM:GDBM-OPT, dbf option,(subr_gdbm_gdbm_opt,seclass_default,2,0,norest,nokey,0,NIL))
{
  gdbm_data_t key = GDBM_DATA_NOTYPE;
  gdbm_data_t val = GDBM_DATA_NOTYPE;
  GDBM_FILE dbf = check_gdbm(&STACK_1,&key,&val,true);
  int option = gdbm_getopt_option(STACK_0);
  switch (option) {
    case GDBM_GETDEFAULT_KEY_TYPE:
      VALUES1(check_data_type_reverse(key));
      break;
    case GDBM_GETDEFAULT_VALUE_TYPE:
      VALUES1(check_data_type_reverse(val));
      break;
    default:
#    if defined(HAVE_GDBM_SETOPT)
      switch (option) {
#      ifdef GDBM_GETFLAGS
        case GDBM_GETFLAGS: {
          int v;
          CHECK_RUN(gdbm_setopt(dbf, option, &v, sizeof(v)));
          STACK_0 = gdbm_open_read_write_reverse(v & GDBM_OPENMASK);
          value1 = gdbm_open_flags_to_list(v & ~GDBM_OPENMASK);
          STACK_1 = value1;
          value1 = allocate_cons();
          Car(value1) = STACK_0;
          Cdr(value1) = STACK_1;
          mv_count = 1;
        } break;
#      endif
#      if defined(GDBM_GETCACHESIZE) && defined(GDBM_GETMAXMAPSIZE)
        case GDBM_GETCACHESIZE: case GDBM_GETMAXMAPSIZE: {
          size_t v;
          CHECK_RUN(gdbm_setopt(dbf, option, &v, sizeof(v)));
          VALUES1(size_to_I(v));
        } break;
#      endif
#      ifdef GDBM_GETBLOCKSIZE
        case GDBM_GETBLOCKSIZE: {
          int v;
          CHECK_RUN(gdbm_setopt(dbf, option, &v, sizeof(v)));
          VALUES1(sint_to_I(v));
        } break;
#      endif
#      if defined(GDBM_GETMMAP) && defined(GDBM_GETCENTFREE) && defined(GDBM_GETCENTFREE) && defined(GDBM_GETCOALESCEBLKS)
        case GDBM_GETMMAP: case GDBM_GETSYNCMODE:
        case GDBM_GETCENTFREE: case GDBM_GETCOALESCEBLKS: {
          int v;
          CHECK_RUN(gdbm_setopt(dbf, option, &v, sizeof(v)));
          VALUES_IF(v);
        } break;
#      endif
#      ifdef GDBM_GETDBNAME
        case GDBM_GETDBNAME: {
          char *v;
          CHECK_RUN(gdbm_setopt(dbf, option, &v, sizeof(v)));
          VALUES1(asciz_to_string(v,GLO(pathname_encoding)));
          begin_system_call(); free(v); end_system_call();
        } break;
#      endif
        default: NOTREACHED;
      }
#    else
      NOTREACHED;
#    endif  /* HAVE_GDBM_SETOPT */
  }
  skipSTACK(2);
}

#line 503

DEFUN(GDBM::%SET-GDBM-OPT, dbf option value,(subr_gdbm__25set_gdbm_opt,seclass_default,3,0,norest,nokey,0,NIL))
{
  GDBM_FILE dbf = check_gdbm(&STACK_2,NULL,NULL,true);
  int option = gdbm_setopt_option(STACK_1);
  int v;
  switch (option) {
    case GDBM_SETDEFAULT_VALUE_TYPE: v = GDBM_SLOT_VAL; goto gdbm_setopt_slot;
    case GDBM_SETDEFAULT_KEY_TYPE: v = GDBM_SLOT_KEY; gdbm_setopt_slot:
      value1 = fixnum(check_data_type(STACK_0));
      TheStructure(STACK_2)->recdata[v] = value1;
      break;
# if defined(HAVE_GDBM_SETOPT)
#  ifdef GDBM_SETCACHESIZE
    case GDBM_SETCACHESIZE:
#  endif
#  ifdef GDBM_SETMAXMAPSIZE
    case GDBM_SETMAXMAPSIZE:
#  endif
      v = I_to_sint(check_sint(STACK_0));
      goto gdbm_setopt_common;
#  ifdef GDBM_SETSYNCMODE
    case GDBM_SETSYNCMODE:
#  endif
#  ifdef GDBM_SETMMAP
    case GDBM_SETMMAP:
#  endif
#  ifdef GDBM_SETCENTFREE
    case GDBM_SETCENTFREE:
#  endif
#  ifdef GDBM_SETCOALESCEBLKS
    case GDBM_SETCOALESCEBLKS:
#  endif
      v = !nullp(STACK_0);
    gdbm_setopt_common:
      CHECK_RUN(gdbm_setopt(dbf, option, &v, sizeof(v)));
      break;
# endif
    default: NOTREACHED;
  }
  VALUES1(STACK_0);
  skipSTACK(3);
}


struct module__gdbm__subr_tab_t module__gdbm__subr_tab
  #if defined(HEAPCODES) && (alignment_long < varobject_alignment) && defined(__GNUC__)
    __attribute__ ((aligned (varobject_alignment)))
  #endif
  = {
  #if varobjects_misaligned
  { 0 },
  #endif
#if defined(HAVE_GDBM_CLOSE)
  LISPFUN_F(subr_gdbm_gdbm_close,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GDBM_COUNT)
  LISPFUN_F(subr_gdbm_gdbm_count,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GDBM_DELETE)
  LISPFUN_F(subr_gdbm_gdbm_delete,seclass_default,2,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GDBM_EXISTS)
  LISPFUN_F(subr_gdbm_gdbm_exists,seclass_default,2,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GDBM_FETCH)
  LISPFUN_F(subr_gdbm_gdbm_fetch,seclass_default,2,0,norest,key,1,NIL)
#endif
#if defined(HAVE_GDBM_FDESC)
  LISPFUN_F(subr_gdbm_gdbm_file_size,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GDBM_FIRSTKEY)
  LISPFUN_F(subr_gdbm_gdbm_firstkey,seclass_default,1,0,norest,key,1,NIL)
#endif
#if defined(HAVE_GDBM_NEXTKEY)
  LISPFUN_F(subr_gdbm_gdbm_nextkey,seclass_default,2,0,norest,key,1,NIL)
#endif
#if defined(HAVE_GDBM_OPEN)
  LISPFUN_F(subr_gdbm_gdbm_open,seclass_default,1,0,norest,key,6,NIL)
#endif
  LISPFUN_F(subr_gdbm_gdbm_opt,seclass_default,2,0,norest,nokey,0,NIL)
#if defined(HAVE_GDBM_REORGANIZE)
  LISPFUN_F(subr_gdbm_gdbm_reorganize,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GDBM_STORE)
  LISPFUN_F(subr_gdbm_gdbm_store,seclass_default,3,0,norest,key,1,NIL)
#endif
#if defined(HAVE_GDBM_SYNC)
  LISPFUN_F(subr_gdbm_gdbm_sync,seclass_default,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_gdbm_gdbm_version,seclass_default,0,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_gdbm__25set_gdbm_opt,seclass_default,3,0,norest,nokey,0,NIL)
  0
};
uintC module__gdbm__subr_tab_size = (sizeof(struct module__gdbm__subr_tab_t)-varobjects_misaligned-sizeof(int))/sizeof(subr_t);

struct module__gdbm__subr_tab_initdata_t {
#if (defined(HAVE_GDBM_OPEN)) || (defined(HAVE_GDBM_CLOSE))
  subr_initdata_t _subr_gdbm_gdbm_close;
#endif
#if defined(HAVE_GDBM_COUNT)
  subr_initdata_t _subr_gdbm_gdbm_count;
#endif
#if defined(HAVE_GDBM_DELETE)
  subr_initdata_t _subr_gdbm_gdbm_delete;
#endif
#if defined(HAVE_GDBM_EXISTS)
  subr_initdata_t _subr_gdbm_gdbm_exists;
#endif
#if defined(HAVE_GDBM_FETCH)
  subr_initdata_t _subr_gdbm_gdbm_fetch;
#endif
#if defined(HAVE_GDBM_FDESC)
  subr_initdata_t _subr_gdbm_gdbm_file_size;
#endif
#if defined(HAVE_GDBM_FIRSTKEY)
  subr_initdata_t _subr_gdbm_gdbm_firstkey;
#endif
#if defined(HAVE_GDBM_NEXTKEY)
  subr_initdata_t _subr_gdbm_gdbm_nextkey;
#endif
#if defined(HAVE_GDBM_OPEN)
  subr_initdata_t _subr_gdbm_gdbm_open;
#endif
  subr_initdata_t _subr_gdbm_gdbm_opt;
#if defined(HAVE_GDBM_REORGANIZE)
  subr_initdata_t _subr_gdbm_gdbm_reorganize;
#endif
#if defined(HAVE_GDBM_STORE)
  subr_initdata_t _subr_gdbm_gdbm_store;
#endif
#if defined(HAVE_GDBM_SYNC)
  subr_initdata_t _subr_gdbm_gdbm_sync;
#endif
  subr_initdata_t _subr_gdbm_gdbm_version;
  subr_initdata_t _subr_gdbm__25set_gdbm_opt;
  int _dummy_to_avoid_trailing_comma_in_initializer;
} module__gdbm__subr_tab_initdata = {
#if (defined(HAVE_GDBM_OPEN)) || (defined(HAVE_GDBM_CLOSE))
  { "GDBM", "GDBM-CLOSE" },
#endif
#if defined(HAVE_GDBM_COUNT)
  { "GDBM", "GDBM-COUNT" },
#endif
#if defined(HAVE_GDBM_DELETE)
  { "GDBM", "GDBM-DELETE" },
#endif
#if defined(HAVE_GDBM_EXISTS)
  { "GDBM", "GDBM-EXISTS" },
#endif
#if defined(HAVE_GDBM_FETCH)
  { "GDBM", "GDBM-FETCH" },
#endif
#if defined(HAVE_GDBM_FDESC)
  { "GDBM", "GDBM-FILE-SIZE" },
#endif
#if defined(HAVE_GDBM_FIRSTKEY)
  { "GDBM", "GDBM-FIRSTKEY" },
#endif
#if defined(HAVE_GDBM_NEXTKEY)
  { "GDBM", "GDBM-NEXTKEY" },
#endif
#if defined(HAVE_GDBM_OPEN)
  { "GDBM", "GDBM-OPEN" },
#endif
  { "GDBM", "GDBM-OPT" },
#if defined(HAVE_GDBM_REORGANIZE)
  { "GDBM", "GDBM-REORGANIZE" },
#endif
#if defined(HAVE_GDBM_STORE)
  { "GDBM", "GDBM-STORE" },
#endif
#if defined(HAVE_GDBM_SYNC)
  { "GDBM", "GDBM-SYNC" },
#endif
  { "GDBM", "GDBM-VERSION" },
  { "GDBM", "%SET-GDBM-OPT" },
  0
};

void module__gdbm__init_function_1 (module_t* module)
{
  (void)module; /* avoid -Wunused-parameter */
  this_module_stab = (struct module__gdbm__stab_t*)module->stab;
#if defined(HAVE_GDBM_FETCH)
  pushSTACK(O(object_Ktype));
  ((struct module__gdbm__stab_t*)module->stab)->_subr_gdbm_gdbm_fetch.keywords = vectorof(1);
#endif
#if defined(HAVE_GDBM_FIRSTKEY)
  pushSTACK(O(object_Ktype));
  ((struct module__gdbm__stab_t*)module->stab)->_subr_gdbm_gdbm_firstkey.keywords = vectorof(1);
#endif
#if defined(HAVE_GDBM_NEXTKEY)
  pushSTACK(O(object_Ktype));
  ((struct module__gdbm__stab_t*)module->stab)->_subr_gdbm_gdbm_nextkey.keywords = vectorof(1);
#endif
#if defined(HAVE_GDBM_OPEN)
  pushSTACK(O(object_Kblocksize));
  pushSTACK(O(object_Kread_write));
  pushSTACK(O(object_Koptions));
  pushSTACK(O(object_Kmode));
  pushSTACK(O(object_Kdefault_key_type));
  pushSTACK(O(object_Kdefault_value_type));
  ((struct module__gdbm__stab_t*)module->stab)->_subr_gdbm_gdbm_open.keywords = vectorof(6);
#endif
#if defined(HAVE_GDBM_STORE)
  pushSTACK(O(object_Kflag));
  ((struct module__gdbm__stab_t*)module->stab)->_subr_gdbm_gdbm_store.keywords = vectorof(1);
#endif
  O(gdbm_version_cache) = (NIL);
}

void module__gdbm__init_function_2 (module_t* module)
{
  (void)module; /* avoid -Wunused-parameter */
  this_module_stab = (struct module__gdbm__stab_t*)module->stab;
}

void module__gdbm__fini_function (module_t* module);
void module__gdbm__fini_function (module_t* module)
{
  (void)module; /* avoid -Wunused-parameter */
}
