#line 1 "/home/iurt/rpmbuild/BUILD/clisp-2.49.93-build/clisp-master/modules/syscalls/calls.c"
/*
 * system calls
 * Copyright (C) 2003-2012,2016-2017 Sam Steingold
 * Copyright (C) 2005,2008,2017-2024 Bruno Haible
 * Copyright (C) 2005,2010 Arseny Slobodyuk
 * This is Free Software, distributed under the GNU GPL v2+
 */

#if defined(_WIN32)
/* need this for CreateHardLink to work */
# define WINVER 0x0500
#endif
#if defined(__CYGWIN__)
# define UNIX_CYGWIN
#endif

/* clisp.h includes system headers among other stuff
   (windows.h on windows) */

#include "clisp.h"
#include "config.h"

# include <sys/time.h>
# include <time.h>
# include <unistd.h>
#if defined(HAVE_SYS_UNISTD_H)
# include <sys/unistd.h>
#endif
#include <errno.h>              /* from gnulib */
#include <sys/types.h>
#if defined(HAVE_SYS_STAT_H)
# include <sys/stat.h>
#endif
#if defined(HAVE_SYS_RESOURCE_H)
# include <sys/resource.h>
#endif
#include <sys/wait.h>    /* always present on unix, imported from gnulib elsewhere */
#if defined(HAVE_SYS_STATVFS_H)
# include <sys/statvfs.h>
#endif
#if defined(HAVE_UTIME_H)
# include <utime.h>
#endif
#include <wchar.h>
#include <limits.h>
#if !defined(NZERO)             /* should be defined in <limits.h> */
# define NZERO 20
#endif
#if defined(HAVE_SYSLOG_H)
# include <syslog.h>
#endif
#if defined(HAVE_UTMPX_H)
# include <utmpx.h>
#endif
#if defined(HAVE_SIGNAL_H)
# include <signal.h>
#endif
#if defined(HAVE_FCNTL_H)
# include <fcntl.h>
#endif
#if defined(HAVE_SYS_PARAM_H)   /* might not be present on woe32 */
# include <sys/param.h>
#endif
#if defined(HAVE_FTW_H)
# include <ftw.h>
#endif
#include <fnmatch.h>

#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
#include <initguid.h>
DEFINE_GUID(FMTID_SummaryInformation, 0xF29F85E0, 0x4FF9, 0x1068,
            0xAB, 0x91, 0x08, 0x00, 0x2B, 0x27, 0xB3, 0xD9);
DEFINE_GUID(FMTID_UserDefinedProperties, 0xD5CDD505, 0x2E9C, 0x101B,
            0x93, 0x97, 0x08, 0x00, 0x2B, 0x2C, 0xF9, 0xAE);
#endif

#include <stdio.h>              /* for BUFSIZ */
#include <stdlib.h>
#include <string.h>             /* for strcpy(), strcat() */

/* #define DEBUG */
#if defined(DEBUG)
extern object nobject_out (FILE* stream, object obj);
#line 86
# define XOUT(obj,label)                                                   (printf("[%s:%d] %s: %s:\n",__FILE__,__LINE__,STRING(obj),label),         obj=nobject_out(stdout,obj), printf("\n"))
#else
# undef OBJECT_OUT
# define OBJECT_OUT(o,l)
# define XOUT(o,l)
#endif

/* http://opengroup.org/onlinepubs/9699919799/basedefs/sys/types.h.html
   specifies tha pid_t is signed, assume that uid_t & gid_t are signed too */
#if SIZEOF_PID_T == 8
# define pid_to_I(g)  sint64_to_I(g)
# define I_to_pid(g)  I_to_sint64(g=check_sint64(g))
#else
# define pid_to_I(g)  sint32_to_I(g)
# define I_to_pid(g)  I_to_sint32(g=check_sint32(g))
#endif
#if SIZEOF_UID_T == 8
# define uid_to_I(g)  sint64_to_I(g)
# define I_to_uid(g)  I_to_sint64(g=check_sint64(g))
#else
# define uid_to_I(g)  sint32_to_I(g)
# define I_to_uid(g)  I_to_sint32(g=check_sint32(g))
#endif
#if SIZEOF_GID_T == 8
# define gid_to_I(g)  sint64_to_I(g)
# define I_to_gid(g)  I_to_sint64(g=check_sint64(g))
#else
# define gid_to_I(g)  sint32_to_I(g)
# define I_to_gid(g)  I_to_sint32(g=check_sint32(g))
#endif

/* general convenience macros */
/* when the portability is ensured by gnulib, use ANSIC_error;
   when we use WIN32_NATIVE functions, use OS_error */
#line 123
#define GETTER(type,conv,call)                                     type id;                                                         begin_system_call(); id = call(); end_system_call();             VALUES1(conv##_to_I(id))
#define GETTER0(type,call)  GETTER(type##_t,type,call)
#line 130
#define GETTER1(type,call)                                 type##_t id = I_to_##type(STACK_0);                      type##_t ret;                                            begin_system_call(); ret=call(id); end_system_call();    if (ret==(type##_t)-1) ANSIC_error();                    VALUES1(type##_to_I(ret)); skipSTACK(1)
#line 136
#define SETTER(type,conv,call)                                     type val = conv(STACK_0);                                        int status;                                                      begin_system_call(); status = call(val); end_system_call();      if (status) ANSIC_error();                                       VALUES1(popSTACK())
#define SETTER1(type,call)  SETTER(type##_t,I_to_##type,call)
#line 144
#define SETTER2(type,call)                                                 type##_t eid = I_to_##type(STACK_0);                                     type##_t rid = I_to_##type(STACK_1);                                     int status;                                                              begin_system_call(); status = call(rid,eid); end_system_call();          if (status) ANSIC_error();                                               VALUES0; skipSTACK(2)

/* for COPY-FILE, must come before DEFMODULE for DEFCHECKER to work */
typedef enum {
  COPY_METHOD_COPY,
  COPY_METHOD_SYMLINK,
  COPY_METHOD_HARDLINK,
  COPY_METHOD_HARDLINK_OR_COPY, /* EXDEV=>COPY */
  COPY_METHOD_RENAME
} copy_method_t;

DEFMODULE(syscalls,"POSIX")

void module__syscalls__init_function_1 (module_t* module);
void module__syscalls__init_function_2__modprep (module_t* module);

#define O(varname) module__syscalls__object_tab._##varname
#define F(varname) subr_tab_ptr_as_object(&this_module_stab->_##varname)

struct module__syscalls__object_tab_t {
  gcv_object_t _object_cl_decode_universal_time;
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_CHAR_TERM))
  gcv_object_t _object_K2_char_term;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_C_BIND))
  gcv_object_t _object_K2_c_bind;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_C_DEV))
  gcv_object_t _object_K2_c_dev;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_FORT_DEV))
  gcv_object_t _object_K2_fort_dev;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_FORT_RUN))
  gcv_object_t _object_K2_fort_run;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_LOCALEDEF))
  gcv_object_t _object_K2_localedef;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS))
  gcv_object_t _object_K2_pbs;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_ACCOUNTING))
  gcv_object_t _object_K2_pbs_accounting;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_CHECKPOINT))
  gcv_object_t _object_K2_pbs_checkpoint;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_LOCATE))
  gcv_object_t _object_K2_pbs_locate;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_MESSAGE))
  gcv_object_t _object_K2_pbs_message;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_TRACK))
  gcv_object_t _object_K2_pbs_track;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_SW_DEV))
  gcv_object_t _object_K2_sw_dev;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_2_SYMLINKS))
  gcv_object_t _object_K2_symlinks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_UPE))
  gcv_object_t _object_K2_upe;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_VERSION))
  gcv_object_t _object_K2_version;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(ABOVE_NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  gcv_object_t _object_Kabove_normal;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(ACCOUNTING))
  gcv_object_t _object_Kaccounting;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ADVISORY_INFO))
  gcv_object_t _object_Kadvisory_info;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_LISTIO_MAX))
  gcv_object_t _object_Kaio_listio_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_MAX))
  gcv_object_t _object_Kaio_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_PRIO_DELTA_MAX))
  gcv_object_t _object_Kaio_prio_delta_max;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_ALERT))
  gcv_object_t _object_Kalert;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_ALLOC_SIZE_MIN))
  gcv_object_t _object_Kalloc_size_min;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ALPHA))
  gcv_object_t _object_Kalpha;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ALPHA64))
  gcv_object_t _object_Kalpha64;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kalt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_AMD64))
  gcv_object_t _object_Kamd64;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_APPEND))
  gcv_object_t _object_Kappend;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kappname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_ARCHIVE))
  gcv_object_t _object_Karchive;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Karguments;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ARG_MAX))
  gcv_object_t _object_Karg_max;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ARM))
  gcv_object_t _object_Karm;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_AS))
  gcv_object_t _object_Kas;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ASYNCHRONOUS_IO))
  gcv_object_t _object_Kasynchronous_io;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_ASYNC_IO))
  gcv_object_t _object_Kasync_io;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ATEXIT_MAX))
  gcv_object_t _object_Katexit_max;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  gcv_object_t _object_Katime;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_AUTH))
  gcv_object_t _object_Kauth;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kauthor;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_AUTHPRIV))
  gcv_object_t _object_Kauthpriv;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AVPHYS_PAGES))
  gcv_object_t _object_Kavphys_pages;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kbackoffice;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BARRIERS))
  gcv_object_t _object_Kbarriers;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_BASE_MAX))
  gcv_object_t _object_Kbc_base_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_DIM_MAX))
  gcv_object_t _object_Kbc_dim_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_SCALE_MAX))
  gcv_object_t _object_Kbc_scale_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_STRING_MAX))
  gcv_object_t _object_Kbc_string_max;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(BELOW_NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  gcv_object_t _object_Kbelow_normal;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_BINARY))
  gcv_object_t _object_Kbinary;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kblob;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  gcv_object_t _object_Kblock;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kbool;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(BOOT_TIME))
  gcv_object_t _object_Kboot_time;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kbstr;
#endif
  gcv_object_t _object_Kbuffered;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kbuilt_in;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kbyref;
#endif
  gcv_object_t _object_Kcase_sensitive;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kcharcount;
#endif
#if defined(HAVE_NFTW)
  gcv_object_t _object_Kchdir;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_CHILDREN))
  gcv_object_t _object_Kchildren;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CHILD_MAX))
  gcv_object_t _object_Kchild_max;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_CHOWN_RESTRICTED))
  gcv_object_t _object_Kchown_restricted;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kclipboard_format;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CLK_TCK))
  gcv_object_t _object_Kclk_tck;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CLOCK_SELECTION))
  gcv_object_t _object_Kclock_selection;
#endif
#if (defined(HAVE_FCNTL)) && (defined(FD_CLOEXEC))
  gcv_object_t _object_Kcloexec;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kcodepage;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_COLL_WEIGHTS_MAX))
  gcv_object_t _object_Kcoll_weights_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kcomments;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_COMPRESSED))
  gcv_object_t _object_Kcompressed;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  gcv_object_t _object_Kcons;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_Kcontinued;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kcontrol;
#endif
  gcv_object_t _object_Kcopy;
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_CORE))
  gcv_object_t _object_Kcore;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_CPU))
  gcv_object_t _object_Kcpu;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CPUTIME))
  gcv_object_t _object_Kcputime;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_CREAT))
  gcv_object_t _object_Kcreat;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kcreate_dtm;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_CRIT))
  gcv_object_t _object_Kcrit;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_CRON))
  gcv_object_t _object_Kcron;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kcy;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_D))
  gcv_object_t _object_Kd;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_DAEMON))
  gcv_object_t _object_Kdaemon;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_DATA))
  gcv_object_t _object_Kdata;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kdatacenter;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kdate;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(DEAD_PROCESS))
  gcv_object_t _object_Kdead_process;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_DEBUG))
  gcv_object_t _object_Kdebug;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_DELAYTIMER_MAX))
  gcv_object_t _object_Kdelaytimer_max;
#endif
#if defined(HAVE_NFTW)
  gcv_object_t _object_Kdepth;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kdescription;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_DEVICE))
  gcv_object_t _object_Kdevice;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_DIRECT))
  gcv_object_t _object_Kdirect;
#endif
  gcv_object_t _object_Kdirection;
#if ((defined(HAVE_FCNTL)) && (defined(O_DIRECTORY))) || ((defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_DIRECTORY)))
  gcv_object_t _object_Kdirectory;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_DNR))
  gcv_object_t _object_Kdnr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_AUTOZONE_ALREADY_EXISTS))
  gcv_object_t _object_Kdns_error_autozone_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_AXFR))
  gcv_object_t _object_Kdns_error_axfr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_BAD_PACKET))
  gcv_object_t _object_Kdns_error_bad_packet;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CANNOT_FIND_ROOT_HINTS))
  gcv_object_t _object_Kdns_error_cannot_find_root_hints;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CNAME_COLLISION))
  gcv_object_t _object_Kdns_error_cname_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CNAME_LOOP))
  gcv_object_t _object_Kdns_error_cname_loop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DATAFILE_OPEN_FAILURE))
  gcv_object_t _object_Kdns_error_datafile_open_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DATAFILE_PARSING))
  gcv_object_t _object_Kdns_error_datafile_parsing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_ALREADY_ENLISTED))
  gcv_object_t _object_Kdns_error_dp_already_enlisted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_ALREADY_EXISTS))
  gcv_object_t _object_Kdns_error_dp_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_DOES_NOT_EXIST))
  gcv_object_t _object_Kdns_error_dp_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_NOT_AVAILABLE))
  gcv_object_t _object_Kdns_error_dp_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_NOT_ENLISTED))
  gcv_object_t _object_Kdns_error_dp_not_enlisted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DS_UNAVAILABLE))
  gcv_object_t _object_Kdns_error_ds_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DS_ZONE_ALREADY_EXISTS))
  gcv_object_t _object_Kdns_error_ds_zone_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_FILE_WRITEBACK_FAILED))
  gcv_object_t _object_Kdns_error_file_writeback_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_FORWARDER_ALREADY_EXISTS))
  gcv_object_t _object_Kdns_error_forwarder_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INCONSISTENT_ROOT_HINTS))
  gcv_object_t _object_Kdns_error_inconsistent_root_hints;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_DATAFILE_NAME))
  gcv_object_t _object_Kdns_error_invalid_datafile_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_IP_ADDRESS))
  gcv_object_t _object_Kdns_error_invalid_ip_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_NAME_CHAR))
  gcv_object_t _object_Kdns_error_invalid_name_char;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_PROPERTY))
  gcv_object_t _object_Kdns_error_invalid_property;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_TYPE))
  gcv_object_t _object_Kdns_error_invalid_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_ZONE_OPERATION))
  gcv_object_t _object_Kdns_error_invalid_zone_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_ZONE_TYPE))
  gcv_object_t _object_Kdns_error_invalid_zone_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NAME_DOES_NOT_EXIST))
  gcv_object_t _object_Kdns_error_name_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NAME_NOT_IN_ZONE))
  gcv_object_t _object_Kdns_error_name_not_in_zone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NBSTAT_INIT_FAILED))
  gcv_object_t _object_Kdns_error_nbstat_init_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NEED_SECONDARY_ADDRESSES))
  gcv_object_t _object_Kdns_error_need_secondary_addresses;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NEED_WINS_SERVERS))
  gcv_object_t _object_Kdns_error_need_wins_servers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NODE_CREATION_FAILED))
  gcv_object_t _object_Kdns_error_node_creation_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NODE_IS_CNAME))
  gcv_object_t _object_Kdns_error_node_is_cname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NON_RFC_NAME))
  gcv_object_t _object_Kdns_error_non_rfc_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER))
  gcv_object_t _object_Kdns_error_not_allowed_on_root_server;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION))
  gcv_object_t _object_Kdns_error_not_allowed_under_delegation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_UNIQUE))
  gcv_object_t _object_Kdns_error_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE))
  gcv_object_t _object_Kdns_error_no_bootfile_if_ds_zone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_CREATE_CACHE_DATA))
  gcv_object_t _object_Kdns_error_no_create_cache_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_DNS_SERVERS))
  gcv_object_t _object_Kdns_error_no_dns_servers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_PACKET))
  gcv_object_t _object_Kdns_error_no_packet;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_TCPIP))
  gcv_object_t _object_Kdns_error_no_tcpip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_ZONE_INFO))
  gcv_object_t _object_Kdns_error_no_zone_info;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NUMERIC_NAME))
  gcv_object_t _object_Kdns_error_numeric_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_PRIMARY_REQUIRES_DATAFILE))
  gcv_object_t _object_Kdns_error_primary_requires_datafile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE))
  gcv_object_t _object_Kdns_error_rcode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADKEY))
  gcv_object_t _object_Kdns_error_rcode_badkey;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADSIG))
  gcv_object_t _object_Kdns_error_rcode_badsig;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADTIME))
  gcv_object_t _object_Kdns_error_rcode_badtime;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_FORMAT_ERROR))
  gcv_object_t _object_Kdns_error_rcode_format_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NAME_ERROR))
  gcv_object_t _object_Kdns_error_rcode_name_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOTAUTH))
  gcv_object_t _object_Kdns_error_rcode_notauth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOTZONE))
  gcv_object_t _object_Kdns_error_rcode_notzone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOT_IMPLEMENTED))
  gcv_object_t _object_Kdns_error_rcode_not_implemented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NXRRSET))
  gcv_object_t _object_Kdns_error_rcode_nxrrset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_REFUSED))
  gcv_object_t _object_Kdns_error_rcode_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_SERVER_FAILURE))
  gcv_object_t _object_Kdns_error_rcode_server_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_YXDOMAIN))
  gcv_object_t _object_Kdns_error_rcode_yxdomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_YXRRSET))
  gcv_object_t _object_Kdns_error_rcode_yxrrset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_ALREADY_EXISTS))
  gcv_object_t _object_Kdns_error_record_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_DOES_NOT_EXIST))
  gcv_object_t _object_Kdns_error_record_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_FORMAT))
  gcv_object_t _object_Kdns_error_record_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT))
  gcv_object_t _object_Kdns_error_record_only_at_zone_root;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_TIMED_OUT))
  gcv_object_t _object_Kdns_error_record_timed_out;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SECONDARY_DATA))
  gcv_object_t _object_Kdns_error_secondary_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP))
  gcv_object_t _object_Kdns_error_secondary_requires_master_ip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SOA_DELETE_INVALID))
  gcv_object_t _object_Kdns_error_soa_delete_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_TRY_AGAIN_LATER))
  gcv_object_t _object_Kdns_error_try_again_later;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_UNKNOWN_RECORD_TYPE))
  gcv_object_t _object_Kdns_error_unknown_record_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_UNSECURE_PACKET))
  gcv_object_t _object_Kdns_error_unsecure_packet;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_WINS_INIT_FAILED))
  gcv_object_t _object_Kdns_error_wins_init_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_ALREADY_EXISTS))
  gcv_object_t _object_Kdns_error_zone_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_CONFIGURATION_ERROR))
  gcv_object_t _object_Kdns_error_zone_configuration_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_CREATION_FAILED))
  gcv_object_t _object_Kdns_error_zone_creation_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_DOES_NOT_EXIST))
  gcv_object_t _object_Kdns_error_zone_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_HAS_NO_NS_RECORDS))
  gcv_object_t _object_Kdns_error_zone_has_no_ns_records;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_HAS_NO_SOA_RECORD))
  gcv_object_t _object_Kdns_error_zone_has_no_soa_record;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_IS_SHUTDOWN))
  gcv_object_t _object_Kdns_error_zone_is_shutdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_LOCKED))
  gcv_object_t _object_Kdns_error_zone_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_NOT_SECONDARY))
  gcv_object_t _object_Kdns_error_zone_not_secondary;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_REQUIRES_MASTER_IP))
  gcv_object_t _object_Kdns_error_zone_requires_master_ip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_ADDED_LOCAL_WINS))
  gcv_object_t _object_Kdns_info_added_local_wins;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_AXFR_COMPLETE))
  gcv_object_t _object_Kdns_info_axfr_complete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_NO_RECORDS))
  gcv_object_t _object_Kdns_info_no_records;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_CONTINUE_NEEDED))
  gcv_object_t _object_Kdns_status_continue_needed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_DOTTED_NAME))
  gcv_object_t _object_Kdns_status_dotted_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_FQDN))
  gcv_object_t _object_Kdns_status_fqdn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_SINGLE_PART_NAME))
  gcv_object_t _object_Kdns_status_single_part_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_WARNING_DOMAIN_UNDELETED))
  gcv_object_t _object_Kdns_warning_domain_undeleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_WARNING_PTR_CREATE_FAILED))
  gcv_object_t _object_Kdns_warning_ptr_create_failed;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kdoc_security;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kdomain_controller;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_DP))
  gcv_object_t _object_Kdp;
#endif
#if defined(E2BIG)
  gcv_object_t _object_Ke2big;
#endif
#if defined(EACCES)
  gcv_object_t _object_Keacces;
#endif
#if defined(EADDRINUSE)
  gcv_object_t _object_Keaddrinuse;
#endif
#if defined(EADDRNOTAVAIL)
  gcv_object_t _object_Keaddrnotavail;
#endif
#if defined(EADV)
  gcv_object_t _object_Keadv;
#endif
#if defined(EAFNOSUPPORT)
  gcv_object_t _object_Keafnosupport;
#endif
#if defined(EAGAIN)
  gcv_object_t _object_Keagain;
#endif
#if defined(EAIO)
  gcv_object_t _object_Keaio;
#endif
#if defined(EALIGN)
  gcv_object_t _object_Kealign;
#endif
#if defined(EALREADY)
  gcv_object_t _object_Kealready;
#endif
#if defined(EAUTH)
  gcv_object_t _object_Keauth;
#endif
#if defined(EBADE)
  gcv_object_t _object_Kebade;
#endif
#if defined(EBADF)
  gcv_object_t _object_Kebadf;
#endif
#if defined(EBADFD)
  gcv_object_t _object_Kebadfd;
#endif
#if defined(EBADMSG)
  gcv_object_t _object_Kebadmsg;
#endif
#if defined(EBADR)
  gcv_object_t _object_Kebadr;
#endif
#if defined(EBADRPC)
  gcv_object_t _object_Kebadrpc;
#endif
#if defined(EBADRQC)
  gcv_object_t _object_Kebadrqc;
#endif
#if defined(EBADSLT)
  gcv_object_t _object_Kebadslt;
#endif
#if defined(EBFONT)
  gcv_object_t _object_Kebfont;
#endif
#if defined(EBUSY)
  gcv_object_t _object_Kebusy;
#endif
#if defined(ECANCELED)
  gcv_object_t _object_Kecanceled;
#endif
#if defined(ECASECLASH)
  gcv_object_t _object_Kecaseclash;
#endif
#if defined(ECHILD)
  gcv_object_t _object_Kechild;
#endif
#if defined(ECHRNG)
  gcv_object_t _object_Kechrng;
#endif
#if defined(ECLONEME)
  gcv_object_t _object_Kecloneme;
#endif
#if defined(ECOMM)
  gcv_object_t _object_Kecomm;
#endif
#if defined(ECONNABORTED)
  gcv_object_t _object_Keconnaborted;
#endif
#if defined(ECONNREFUSED)
  gcv_object_t _object_Keconnrefused;
#endif
#if defined(ECONNRESET)
  gcv_object_t _object_Keconnreset;
#endif
#if defined(EDEADLK)
  gcv_object_t _object_Kedeadlk;
#endif
#if defined(EDEADLOCK)
  gcv_object_t _object_Kedeadlock;
#endif
#if defined(EDESTADDRREQ)
  gcv_object_t _object_Kedestaddrreq;
#endif
#if defined(EDIRIOCTL)
  gcv_object_t _object_Kedirioctl;
#endif
#if defined(EDIRTY)
  gcv_object_t _object_Kedirty;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kedittime;
#endif
#if defined(EDOM)
  gcv_object_t _object_Kedom;
#endif
#if defined(EDOOFUS)
  gcv_object_t _object_Kedoofus;
#endif
#if defined(EDOTDOT)
  gcv_object_t _object_Kedotdot;
#endif
#if defined(EDQUOT)
  gcv_object_t _object_Kedquot;
#endif
#if defined(EDUPPKG)
  gcv_object_t _object_Keduppkg;
#endif
#if defined(EEXIST)
  gcv_object_t _object_Keexist;
#endif
#if defined(EFAIL)
  gcv_object_t _object_Kefail;
#endif
#if defined(EFAULT)
  gcv_object_t _object_Kefault;
#endif
#if defined(EFBIG)
  gcv_object_t _object_Kefbig;
#endif
#if defined(EFTYPE)
  gcv_object_t _object_Keftype;
#endif
#if defined(EHOSTDOWN)
  gcv_object_t _object_Kehostdown;
#endif
#if defined(EHOSTUNREACH)
  gcv_object_t _object_Kehostunreach;
#endif
#if defined(EHWPOISON)
  gcv_object_t _object_Kehwpoison;
#endif
#if defined(EIDRM)
  gcv_object_t _object_Keidrm;
#endif
#if defined(EILSEQ)
  gcv_object_t _object_Keilseq;
#endif
#if defined(EINIT)
  gcv_object_t _object_Keinit;
#endif
#if defined(EINPROG)
  gcv_object_t _object_Keinprog;
#endif
#if defined(EINPROGRESS)
  gcv_object_t _object_Keinprogress;
#endif
#if defined(EINTR)
  gcv_object_t _object_Keintr;
#endif
#if defined(EINVAL)
  gcv_object_t _object_Keinval;
#endif
#if defined(EIO)
  gcv_object_t _object_Keio;
#endif
#if defined(EISCONN)
  gcv_object_t _object_Keisconn;
#endif
#if defined(EISDIR)
  gcv_object_t _object_Keisdir;
#endif
#if defined(EISNAM)
  gcv_object_t _object_Keisnam;
#endif
#if defined(EISNAME)
  gcv_object_t _object_Keisname;
#endif
#if defined(EJUSTRETURN)
  gcv_object_t _object_Kejustreturn;
#endif
#if defined(EKEYEXPIRED)
  gcv_object_t _object_Kekeyexpired;
#endif
#if defined(EKEYREJECTED)
  gcv_object_t _object_Kekeyrejected;
#endif
#if defined(EKEYREVOKED)
  gcv_object_t _object_Kekeyrevoked;
#endif
#if defined(EL2HLT)
  gcv_object_t _object_Kel2hlt;
#endif
#if defined(EL2NSYNC)
  gcv_object_t _object_Kel2nsync;
#endif
#if defined(EL3HLT)
  gcv_object_t _object_Kel3hlt;
#endif
#if defined(EL3RST)
  gcv_object_t _object_Kel3rst;
#endif
#if defined(ELAST)
  gcv_object_t _object_Kelast;
#endif
#if defined(ELBIN)
  gcv_object_t _object_Kelbin;
#endif
  gcv_object_t _object_Kelement_type;
#if defined(ELIBACC)
  gcv_object_t _object_Kelibacc;
#endif
#if defined(ELIBBAD)
  gcv_object_t _object_Kelibbad;
#endif
#if defined(ELIBEXEC)
  gcv_object_t _object_Kelibexec;
#endif
#if defined(ELIBMAX)
  gcv_object_t _object_Kelibmax;
#endif
#if defined(ELIBSCN)
  gcv_object_t _object_Kelibscn;
#endif
#if defined(ELNRNG)
  gcv_object_t _object_Kelnrng;
#endif
#if defined(ELOCKUNMAPPED)
  gcv_object_t _object_Kelockunmapped;
#endif
#if defined(ELOOP)
  gcv_object_t _object_Keloop;
#endif
#if defined(EMEDIUMTYPE)
  gcv_object_t _object_Kemediumtype;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_EMERG))
  gcv_object_t _object_Kemerg;
#endif
#if defined(EMFILE)
  gcv_object_t _object_Kemfile;
#endif
#if defined(EMLINK)
  gcv_object_t _object_Kemlink;
#endif
#if ((defined(HAVE_UTMPX_H)) && (defined(EMPTY))) || (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN))
  gcv_object_t _object_Kempty;
#endif
#if defined(EMSGSIZE)
  gcv_object_t _object_Kemsgsize;
#endif
#if defined(EMTIMERS)
  gcv_object_t _object_Kemtimers;
#endif
#if defined(EMULTIHOP)
  gcv_object_t _object_Kemultihop;
#endif
#if defined(ENAMETOOLONG)
  gcv_object_t _object_Kenametoolong;
#endif
#if defined(ENAVAIL)
  gcv_object_t _object_Kenavail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_ENCRYPTED))
  gcv_object_t _object_Kencrypted;
#endif
#if defined(ENEEDAUTH)
  gcv_object_t _object_Keneedauth;
#endif
#if defined(ENET)
  gcv_object_t _object_Kenet;
#endif
#if defined(ENETDOWN)
  gcv_object_t _object_Kenetdown;
#endif
#if defined(ENETRESET)
  gcv_object_t _object_Kenetreset;
#endif
#if defined(ENETUNREACH)
  gcv_object_t _object_Kenetunreach;
#endif
#if defined(ENFILE)
  gcv_object_t _object_Kenfile;
#endif
#if defined(ENMFILE)
  gcv_object_t _object_Kenmfile;
#endif
#if defined(ENOANO)
  gcv_object_t _object_Kenoano;
#endif
#if defined(ENOATTR)
  gcv_object_t _object_Kenoattr;
#endif
#if defined(ENOBUFS)
  gcv_object_t _object_Kenobufs;
#endif
#if defined(ENOCSI)
  gcv_object_t _object_Kenocsi;
#endif
#if defined(ENODATA)
  gcv_object_t _object_Kenodata;
#endif
#if defined(ENODEV)
  gcv_object_t _object_Kenodev;
#endif
#if defined(ENOENT)
  gcv_object_t _object_Kenoent;
#endif
#if defined(ENOEXEC)
  gcv_object_t _object_Kenoexec;
#endif
#if defined(ENOFILE)
  gcv_object_t _object_Kenofile;
#endif
#if defined(ENOIOCTL)
  gcv_object_t _object_Kenoioctl;
#endif
#if defined(ENOKEY)
  gcv_object_t _object_Kenokey;
#endif
#if defined(ENOLCK)
  gcv_object_t _object_Kenolck;
#endif
#if defined(ENOLINK)
  gcv_object_t _object_Kenolink;
#endif
#if defined(ENOMEDIUM)
  gcv_object_t _object_Kenomedium;
#endif
#if defined(ENOMEM)
  gcv_object_t _object_Kenomem;
#endif
#if defined(ENOMSG)
  gcv_object_t _object_Kenomsg;
#endif
#if defined(ENONET)
  gcv_object_t _object_Kenonet;
#endif
#if defined(ENOPKG)
  gcv_object_t _object_Kenopkg;
#endif
#if defined(ENOPROTOOPT)
  gcv_object_t _object_Kenoprotoopt;
#endif
#if defined(ENOSHARE)
  gcv_object_t _object_Kenoshare;
#endif
#if defined(ENOSPC)
  gcv_object_t _object_Kenospc;
#endif
#if defined(ENOSR)
  gcv_object_t _object_Kenosr;
#endif
#if defined(ENOSTR)
  gcv_object_t _object_Kenostr;
#endif
#if defined(ENOSYM)
  gcv_object_t _object_Kenosym;
#endif
#if defined(ENOSYS)
  gcv_object_t _object_Kenosys;
#endif
#if defined(ENOTACTIVE)
  gcv_object_t _object_Kenotactive;
#endif
#if defined(ENOTBLK)
  gcv_object_t _object_Kenotblk;
#endif
#if defined(ENOTCONN)
  gcv_object_t _object_Kenotconn;
#endif
#if defined(ENOTDIR)
  gcv_object_t _object_Kenotdir;
#endif
#if defined(ENOTEMPTY)
  gcv_object_t _object_Kenotempty;
#endif
#if defined(ENOTNAM)
  gcv_object_t _object_Kenotnam;
#endif
#if defined(ENOTRECOVERABLE)
  gcv_object_t _object_Kenotrecoverable;
#endif
#if defined(ENOTSOCK)
  gcv_object_t _object_Kenotsock;
#endif
#if defined(ENOTSUP)
  gcv_object_t _object_Kenotsup;
#endif
#if defined(ENOTTY)
  gcv_object_t _object_Kenotty;
#endif
#if defined(ENOTUNIQ)
  gcv_object_t _object_Kenotuniq;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kenterprise;
#endif
#if defined(ENXIO)
  gcv_object_t _object_Kenxio;
#endif
#if defined(EOPNOTSUPP)
  gcv_object_t _object_Keopnotsupp;
#endif
#if defined(EOVERFLOW)
  gcv_object_t _object_Keoverflow;
#endif
#if defined(EOWNERDEAD)
  gcv_object_t _object_Keownerdead;
#endif
#if defined(EPERM)
  gcv_object_t _object_Keperm;
#endif
#if defined(EPFNOSUPPORT)
  gcv_object_t _object_Kepfnosupport;
#endif
#if defined(EPIPE)
  gcv_object_t _object_Kepipe;
#endif
#if defined(EPROCLIM)
  gcv_object_t _object_Keproclim;
#endif
#if defined(EPROCUNAVAIL)
  gcv_object_t _object_Keprocunavail;
#endif
#if defined(EPROGMISMATCH)
  gcv_object_t _object_Keprogmismatch;
#endif
#if defined(EPROGUNAVAIL)
  gcv_object_t _object_Keprogunavail;
#endif
#if defined(EPROTO)
  gcv_object_t _object_Keproto;
#endif
#if defined(EPROTONOSUPPORT)
  gcv_object_t _object_Keprotonosupport;
#endif
#if defined(EPROTOTYPE)
  gcv_object_t _object_Keprototype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_CANT_CREATE))
  gcv_object_t _object_Kept_s_cant_create;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_CANT_PERFORM_OP))
  gcv_object_t _object_Kept_s_cant_perform_op;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_INVALID_ENTRY))
  gcv_object_t _object_Kept_s_invalid_entry;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_NOT_REGISTERED))
  gcv_object_t _object_Kept_s_not_registered;
#endif
#if defined(ERANGE)
  gcv_object_t _object_Kerange;
#endif
#if defined(EREMCHG)
  gcv_object_t _object_Keremchg;
#endif
#if defined(EREMDEV)
  gcv_object_t _object_Keremdev;
#endif
#if defined(EREMOTE)
  gcv_object_t _object_Keremote;
#endif
#if defined(EREMOTEIO)
  gcv_object_t _object_Keremoteio;
#endif
#if defined(EREMOTERELEASE)
  gcv_object_t _object_Keremoterelease;
#endif
#if defined(ERESTART)
  gcv_object_t _object_Kerestart;
#endif
#if defined(ERESTARTNOHAND)
  gcv_object_t _object_Kerestartnohand;
#endif
#if defined(ERESTARTNOINTR)
  gcv_object_t _object_Kerestartnointr;
#endif
#if defined(ERESTARTSYS)
  gcv_object_t _object_Kerestartsys;
#endif
#if defined(ERFKILL)
  gcv_object_t _object_Kerfkill;
#endif
#if defined(EROFS)
  gcv_object_t _object_Kerofs;
#endif
#if defined(ERPCMISMATCH)
  gcv_object_t _object_Kerpcmismatch;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_ERR))
  gcv_object_t _object_Kerr;
#endif
#if defined(ERREMOTE)
  gcv_object_t _object_Kerremote;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kerror;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DENIED))
  gcv_object_t _object_Kerror_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_BY_POLICY))
  gcv_object_t _object_Kerror_access_disabled_by_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_WEBBLADE))
  gcv_object_t _object_Kerror_access_disabled_webblade;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER))
  gcv_object_t _object_Kerror_access_disabled_webblade_tamper;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_DISABLED))
  gcv_object_t _object_Kerror_account_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_EXPIRED))
  gcv_object_t _object_Kerror_account_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_LOCKED_OUT))
  gcv_object_t _object_Kerror_account_locked_out;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_RESTRICTION))
  gcv_object_t _object_Kerror_account_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACTIVATION_COUNT_EXCEEDED))
  gcv_object_t _object_Kerror_activation_count_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACTIVE_CONNECTIONS))
  gcv_object_t _object_Kerror_active_connections;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADAP_HDW_ERR))
  gcv_object_t _object_Kerror_adap_hdw_err;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADDRESS_ALREADY_ASSOCIATED))
  gcv_object_t _object_Kerror_address_already_associated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADDRESS_NOT_ASSOCIATED))
  gcv_object_t _object_Kerror_address_not_associated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALIAS_EXISTS))
  gcv_object_t _object_Kerror_alias_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALLOTTED_SPACE_EXCEEDED))
  gcv_object_t _object_Kerror_allotted_space_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALL_NODES_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_all_nodes_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED))
  gcv_object_t _object_Kerror_all_user_trust_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_ASSIGNED))
  gcv_object_t _object_Kerror_already_assigned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_FIBER))
  gcv_object_t _object_Kerror_already_fiber;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_INITIALIZED))
  gcv_object_t _object_Kerror_already_initialized;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_REGISTERED))
  gcv_object_t _object_Kerror_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_RUNNING_LKG))
  gcv_object_t _object_Kerror_already_running_lkg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_THREAD))
  gcv_object_t _object_Kerror_already_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_WAITING))
  gcv_object_t _object_Kerror_already_waiting;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_APPHELP_BLOCK))
  gcv_object_t _object_Kerror_apphelp_block;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_APP_WRONG_OS))
  gcv_object_t _object_Kerror_app_wrong_os;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ARENA_TRASHED))
  gcv_object_t _object_Kerror_arena_trashed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ARITHMETIC_OVERFLOW))
  gcv_object_t _object_Kerror_arithmetic_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ATOMIC_LOCKS_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_atomic_locks_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_AUTODATASEG_EXCEEDS_64k))
  gcv_object_t _object_Kerror_autodataseg_exceeds_64k;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BADDB))
  gcv_object_t _object_Kerror_baddb;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BADKEY))
  gcv_object_t _object_Kerror_badkey;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_ARGUMENTS))
  gcv_object_t _object_Kerror_bad_arguments;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_COMMAND))
  gcv_object_t _object_Kerror_bad_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_CONFIGURATION))
  gcv_object_t _object_Kerror_bad_configuration;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DESCRIPTOR_FORMAT))
  gcv_object_t _object_Kerror_bad_descriptor_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DEVICE))
  gcv_object_t _object_Kerror_bad_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DEV_TYPE))
  gcv_object_t _object_Kerror_bad_dev_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DRIVER))
  gcv_object_t _object_Kerror_bad_driver;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DRIVER_LEVEL))
  gcv_object_t _object_Kerror_bad_driver_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_ENVIRONMENT))
  gcv_object_t _object_Kerror_bad_environment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_EXE_FORMAT))
  gcv_object_t _object_Kerror_bad_exe_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_FORMAT))
  gcv_object_t _object_Kerror_bad_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_IMPERSONATION_LEVEL))
  gcv_object_t _object_Kerror_bad_impersonation_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_INHERITANCE_ACL))
  gcv_object_t _object_Kerror_bad_inheritance_acl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_LENGTH))
  gcv_object_t _object_Kerror_bad_length;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_LOGON_SESSION_STATE))
  gcv_object_t _object_Kerror_bad_logon_session_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NETPATH))
  gcv_object_t _object_Kerror_bad_netpath;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NET_NAME))
  gcv_object_t _object_Kerror_bad_net_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NET_RESP))
  gcv_object_t _object_Kerror_bad_net_resp;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PATHNAME))
  gcv_object_t _object_Kerror_bad_pathname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PIPE))
  gcv_object_t _object_Kerror_bad_pipe;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PROFILE))
  gcv_object_t _object_Kerror_bad_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PROVIDER))
  gcv_object_t _object_Kerror_bad_provider;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_QUERY_SYNTAX))
  gcv_object_t _object_Kerror_bad_query_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_RECOVERY_POLICY))
  gcv_object_t _object_Kerror_bad_recovery_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_REM_ADAP))
  gcv_object_t _object_Kerror_bad_rem_adap;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_THREADID_ADDR))
  gcv_object_t _object_Kerror_bad_threadid_addr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_TOKEN_TYPE))
  gcv_object_t _object_Kerror_bad_token_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_UNIT))
  gcv_object_t _object_Kerror_bad_unit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_USERNAME))
  gcv_object_t _object_Kerror_bad_username;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_USER_PROFILE))
  gcv_object_t _object_Kerror_bad_user_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_VALIDATION_CLASS))
  gcv_object_t _object_Kerror_bad_validation_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BEGINNING_OF_MEDIA))
  gcv_object_t _object_Kerror_beginning_of_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BOOT_ALREADY_ACCEPTED))
  gcv_object_t _object_Kerror_boot_already_accepted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BROKEN_PIPE))
  gcv_object_t _object_Kerror_broken_pipe;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUFFER_OVERFLOW))
  gcv_object_t _object_Kerror_buffer_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUSY))
  gcv_object_t _object_Kerror_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUSY_DRIVE))
  gcv_object_t _object_Kerror_busy_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUS_RESET))
  gcv_object_t _object_Kerror_bus_reset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CALLBACK_SUPPLIED_INVALID_DATA))
  gcv_object_t _object_Kerror_callback_supplied_invalid_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CALL_NOT_IMPLEMENTED))
  gcv_object_t _object_Kerror_call_not_implemented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANCELLED))
  gcv_object_t _object_Kerror_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANCEL_VIOLATION))
  gcv_object_t _object_Kerror_cancel_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_COPY))
  gcv_object_t _object_Kerror_cannot_copy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_DETECT_DRIVER_FAILURE))
  gcv_object_t _object_Kerror_cannot_detect_driver_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_DETECT_PROCESS_ABORT))
  gcv_object_t _object_Kerror_cannot_detect_process_abort;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_FIND_WND_CLASS))
  gcv_object_t _object_Kerror_cannot_find_wnd_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_IMPERSONATE))
  gcv_object_t _object_Kerror_cannot_impersonate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_MAKE))
  gcv_object_t _object_Kerror_cannot_make;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_OPEN_PROFILE))
  gcv_object_t _object_Kerror_cannot_open_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTOPEN))
  gcv_object_t _object_Kerror_cantopen;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTREAD))
  gcv_object_t _object_Kerror_cantread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTWRITE))
  gcv_object_t _object_Kerror_cantwrite;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_ACCESS_DOMAIN_INFO))
  gcv_object_t _object_Kerror_cant_access_domain_info;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_ACCESS_FILE))
  gcv_object_t _object_Kerror_cant_access_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_DELETE_LAST_ITEM))
  gcv_object_t _object_Kerror_cant_delete_last_item;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_DISABLE_MANDATORY))
  gcv_object_t _object_Kerror_cant_disable_mandatory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_EVICT_ACTIVE_NODE))
  gcv_object_t _object_Kerror_cant_evict_active_node;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_OPEN_ANONYMOUS))
  gcv_object_t _object_Kerror_cant_open_anonymous;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_RESOLVE_FILENAME))
  gcv_object_t _object_Kerror_cant_resolve_filename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CAN_NOT_COMPLETE))
  gcv_object_t _object_Kerror_can_not_complete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CAN_NOT_DEL_LOCAL_WINS))
  gcv_object_t _object_Kerror_can_not_del_local_wins;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_MUST_BE_VOLATILE))
  gcv_object_t _object_Kerror_child_must_be_volatile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_NOT_COMPLETE))
  gcv_object_t _object_Kerror_child_not_complete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_WINDOW_MENU))
  gcv_object_t _object_Kerror_child_window_menu;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CIRCULAR_DEPENDENCY))
  gcv_object_t _object_Kerror_circular_dependency;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_class_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_DOES_NOT_EXIST))
  gcv_object_t _object_Kerror_class_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_HAS_WINDOWS))
  gcv_object_t _object_Kerror_class_has_windows;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_CARTRIDGE_INSTALLED))
  gcv_object_t _object_Kerror_cleaner_cartridge_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_CARTRIDGE_SPENT))
  gcv_object_t _object_Kerror_cleaner_cartridge_spent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_SLOT_NOT_SET))
  gcv_object_t _object_Kerror_cleaner_slot_not_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_SLOT_SET))
  gcv_object_t _object_Kerror_cleaner_slot_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLIPBOARD_NOT_OPEN))
  gcv_object_t _object_Kerror_clipboard_not_open;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLIPPING_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_clipping_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_ALREADY_COMMITTED))
  gcv_object_t _object_Kerror_cluscfg_already_committed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_ROLLBACK_FAILED))
  gcv_object_t _object_Kerror_cluscfg_rollback_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT))
  gcv_object_t _object_Kerror_cluscfg_system_disk_drive_letter_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND))
  gcv_object_t _object_Kerror_clusterlog_chkpoint_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_CORRUPT))
  gcv_object_t _object_Kerror_clusterlog_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE))
  gcv_object_t _object_Kerror_clusterlog_exceeds_maxsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE))
  gcv_object_t _object_Kerror_clusterlog_not_enough_space;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE))
  gcv_object_t _object_Kerror_clusterlog_record_exceeds_maxsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME))
  gcv_object_t _object_Kerror_cluster_cant_create_dup_cluster_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_DATABASE_SEQMISMATCH))
  gcv_object_t _object_Kerror_cluster_database_seqmismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP))
  gcv_object_t _object_Kerror_cluster_evict_without_cleanup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_GUM_NOT_LOCKER))
  gcv_object_t _object_Kerror_cluster_gum_not_locker;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INCOMPATIBLE_VERSIONS))
  gcv_object_t _object_Kerror_cluster_incompatible_versions;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INSTANCE_ID_MISMATCH))
  gcv_object_t _object_Kerror_cluster_instance_id_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NETWORK))
  gcv_object_t _object_Kerror_cluster_invalid_network;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NETWORK_PROVIDER))
  gcv_object_t _object_Kerror_cluster_invalid_network_provider;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NODE))
  gcv_object_t _object_Kerror_cluster_invalid_node;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_REQUEST))
  gcv_object_t _object_Kerror_cluster_invalid_request;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_IPADDR_IN_USE))
  gcv_object_t _object_Kerror_cluster_ipaddr_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_ABORTED))
  gcv_object_t _object_Kerror_cluster_join_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_IN_PROGRESS))
  gcv_object_t _object_Kerror_cluster_join_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS))
  gcv_object_t _object_Kerror_cluster_join_not_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_LAST_INTERNAL_NETWORK))
  gcv_object_t _object_Kerror_cluster_last_internal_network;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND))
  gcv_object_t _object_Kerror_cluster_local_node_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED))
  gcv_object_t _object_Kerror_cluster_maxnum_of_resources_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MEMBERSHIP_HALT))
  gcv_object_t _object_Kerror_cluster_membership_halt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE))
  gcv_object_t _object_Kerror_cluster_membership_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME))
  gcv_object_t _object_Kerror_cluster_mismatched_computer_acct_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETINTERFACE_EXISTS))
  gcv_object_t _object_Kerror_cluster_netinterface_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETINTERFACE_NOT_FOUND))
  gcv_object_t _object_Kerror_cluster_netinterface_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE))
  gcv_object_t _object_Kerror_cluster_network_already_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_ALREADY_ONLINE))
  gcv_object_t _object_Kerror_cluster_network_already_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_EXISTS))
  gcv_object_t _object_Kerror_cluster_network_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS))
  gcv_object_t _object_Kerror_cluster_network_has_dependents;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_FOUND))
  gcv_object_t _object_Kerror_cluster_network_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP))
  gcv_object_t _object_Kerror_cluster_network_not_found_for_ip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_INTERNAL))
  gcv_object_t _object_Kerror_cluster_network_not_internal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_DOWN))
  gcv_object_t _object_Kerror_cluster_node_already_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT))
  gcv_object_t _object_Kerror_cluster_node_already_has_dfs_root;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_MEMBER))
  gcv_object_t _object_Kerror_cluster_node_already_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_UP))
  gcv_object_t _object_Kerror_cluster_node_already_up;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_DOWN))
  gcv_object_t _object_Kerror_cluster_node_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_EXISTS))
  gcv_object_t _object_Kerror_cluster_node_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_FOUND))
  gcv_object_t _object_Kerror_cluster_node_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_MEMBER))
  gcv_object_t _object_Kerror_cluster_node_not_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_PAUSED))
  gcv_object_t _object_Kerror_cluster_node_not_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_READY))
  gcv_object_t _object_Kerror_cluster_node_not_ready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_PAUSED))
  gcv_object_t _object_Kerror_cluster_node_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_SHUTTING_DOWN))
  gcv_object_t _object_Kerror_cluster_node_shutting_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_UNREACHABLE))
  gcv_object_t _object_Kerror_cluster_node_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_UP))
  gcv_object_t _object_Kerror_cluster_node_up;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED))
  gcv_object_t _object_Kerror_cluster_no_rpc_packages_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NO_SECURITY_CONTEXT))
  gcv_object_t _object_Kerror_cluster_no_security_context;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_OLD_VERSION))
  gcv_object_t _object_Kerror_cluster_old_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST))
  gcv_object_t _object_Kerror_cluster_owner_not_in_preflist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_PARAMETER_MISMATCH))
  gcv_object_t _object_Kerror_cluster_parameter_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH))
  gcv_object_t _object_Kerror_cluster_property_data_type_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_QUORUMLOG_NOT_FOUND))
  gcv_object_t _object_Kerror_cluster_quorumlog_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESNAME_NOT_FOUND))
  gcv_object_t _object_Kerror_cluster_resname_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND))
  gcv_object_t _object_Kerror_cluster_resource_type_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_cluster_restype_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_SHUTTING_DOWN))
  gcv_object_t _object_Kerror_cluster_shutting_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED))
  gcv_object_t _object_Kerror_cluster_system_config_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_WRONG_OS_VERSION))
  gcv_object_t _object_Kerror_cluster_wrong_os_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COLORSPACE_MISMATCH))
  gcv_object_t _object_Kerror_colorspace_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COMMITMENT_LIMIT))
  gcv_object_t _object_Kerror_commitment_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTED_OTHER_PASSWORD))
  gcv_object_t _object_Kerror_connected_other_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT))
  gcv_object_t _object_Kerror_connected_other_password_default;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_ABORTED))
  gcv_object_t _object_Kerror_connection_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_ACTIVE))
  gcv_object_t _object_Kerror_connection_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_COUNT_LIMIT))
  gcv_object_t _object_Kerror_connection_count_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_INVALID))
  gcv_object_t _object_Kerror_connection_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_REFUSED))
  gcv_object_t _object_Kerror_connection_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_UNAVAIL))
  gcv_object_t _object_Kerror_connection_unavail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTEXT_EXPIRED))
  gcv_object_t _object_Kerror_context_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTINUE))
  gcv_object_t _object_Kerror_continue;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTROLLING_IEPORT))
  gcv_object_t _object_Kerror_controlling_ieport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTROL_ID_NOT_FOUND))
  gcv_object_t _object_Kerror_control_id_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CORE_RESOURCE))
  gcv_object_t _object_Kerror_core_resource;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COUNTER_TIMEOUT))
  gcv_object_t _object_Kerror_counter_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CRC))
  gcv_object_t _object_Kerror_crc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CREATE_FAILED))
  gcv_object_t _object_Kerror_create_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CSCSHARE_OFFLINE))
  gcv_object_t _object_Kerror_cscshare_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_BAD_VIDEO_MODE))
  gcv_object_t _object_Kerror_ctx_bad_video_mode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY))
  gcv_object_t _object_Kerror_ctx_cannot_make_eventlog_entry;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_LICENSE_IN_USE))
  gcv_object_t _object_Kerror_ctx_client_license_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_LICENSE_NOT_SET))
  gcv_object_t _object_Kerror_ctx_client_license_not_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_QUERY_TIMEOUT))
  gcv_object_t _object_Kerror_ctx_client_query_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLOSE_PENDING))
  gcv_object_t _object_Kerror_ctx_close_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CONSOLE_CONNECT))
  gcv_object_t _object_Kerror_ctx_console_connect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CONSOLE_DISCONNECT))
  gcv_object_t _object_Kerror_ctx_console_disconnect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_GRAPHICS_INVALID))
  gcv_object_t _object_Kerror_ctx_graphics_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_MODEMNAME))
  gcv_object_t _object_Kerror_ctx_invalid_modemname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_PD))
  gcv_object_t _object_Kerror_ctx_invalid_pd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_WD))
  gcv_object_t _object_Kerror_ctx_invalid_wd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_CLIENT_INVALID))
  gcv_object_t _object_Kerror_ctx_license_client_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_EXPIRED))
  gcv_object_t _object_Kerror_ctx_license_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_ctx_license_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LOGON_DISABLED))
  gcv_object_t _object_Kerror_ctx_logon_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_INF_NOT_FOUND))
  gcv_object_t _object_Kerror_ctx_modem_inf_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_BUSY))
  gcv_object_t _object_Kerror_ctx_modem_response_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_ERROR))
  gcv_object_t _object_Kerror_ctx_modem_response_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_NO_CARRIER))
  gcv_object_t _object_Kerror_ctx_modem_response_no_carrier;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE))
  gcv_object_t _object_Kerror_ctx_modem_response_no_dialtone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_TIMEOUT))
  gcv_object_t _object_Kerror_ctx_modem_response_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_VOICE))
  gcv_object_t _object_Kerror_ctx_modem_response_voice;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_NOT_CONSOLE))
  gcv_object_t _object_Kerror_ctx_not_console;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_NO_OUTBUF))
  gcv_object_t _object_Kerror_ctx_no_outbuf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_PD_NOT_FOUND))
  gcv_object_t _object_Kerror_ctx_pd_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SERVICE_NAME_COLLISION))
  gcv_object_t _object_Kerror_ctx_service_name_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_DENIED))
  gcv_object_t _object_Kerror_ctx_shadow_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_DISABLED))
  gcv_object_t _object_Kerror_ctx_shadow_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE))
  gcv_object_t _object_Kerror_ctx_shadow_ended_by_mode_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_INVALID))
  gcv_object_t _object_Kerror_ctx_shadow_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_NOT_RUNNING))
  gcv_object_t _object_Kerror_ctx_shadow_not_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_TD_ERROR))
  gcv_object_t _object_Kerror_ctx_td_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WD_NOT_FOUND))
  gcv_object_t _object_Kerror_ctx_wd_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_ACCESS_DENIED))
  gcv_object_t _object_Kerror_ctx_winstation_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_ctx_winstation_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_BUSY))
  gcv_object_t _object_Kerror_ctx_winstation_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_NAME_INVALID))
  gcv_object_t _object_Kerror_ctx_winstation_name_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_NOT_FOUND))
  gcv_object_t _object_Kerror_ctx_winstation_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CURRENT_DIRECTORY))
  gcv_object_t _object_Kerror_current_directory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CURRENT_DOMAIN_NOT_ALLOWED))
  gcv_object_t _object_Kerror_current_domain_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_BACKUP_CORRUPT))
  gcv_object_t _object_Kerror_database_backup_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_DOES_NOT_EXIST))
  gcv_object_t _object_Kerror_database_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_FAILURE))
  gcv_object_t _object_Kerror_database_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_FULL))
  gcv_object_t _object_Kerror_database_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATATYPE_MISMATCH))
  gcv_object_t _object_Kerror_datatype_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DC_NOT_FOUND))
  gcv_object_t _object_Kerror_dc_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DDE_FAIL))
  gcv_object_t _object_Kerror_dde_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEBUGGER_INACTIVE))
  gcv_object_t _object_Kerror_debugger_inactive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DECRYPTION_FAILED))
  gcv_object_t _object_Kerror_decryption_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DELETE_PENDING))
  gcv_object_t _object_Kerror_delete_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DELETING_ICM_XFORM))
  gcv_object_t _object_Kerror_deleting_icm_xform;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_dependency_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_NOT_ALLOWED))
  gcv_object_t _object_Kerror_dependency_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_NOT_FOUND))
  gcv_object_t _object_Kerror_dependency_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENT_RESOURCE_EXISTS))
  gcv_object_t _object_Kerror_dependent_resource_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENT_SERVICES_RUNNING))
  gcv_object_t _object_Kerror_dependent_services_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DESTINATION_ELEMENT_FULL))
  gcv_object_t _object_Kerror_destination_element_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DESTROY_OBJECT_OF_OTHER_THREAD))
  gcv_object_t _object_Kerror_destroy_object_of_other_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_ALREADY_REMEMBERED))
  gcv_object_t _object_Kerror_device_already_remembered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_DOOR_OPEN))
  gcv_object_t _object_Kerror_device_door_open;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_IN_USE))
  gcv_object_t _object_Kerror_device_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_device_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_CONNECTED))
  gcv_object_t _object_Kerror_device_not_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_PARTITIONED))
  gcv_object_t _object_Kerror_device_not_partitioned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REINITIALIZATION_NEEDED))
  gcv_object_t _object_Kerror_device_reinitialization_needed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REMOVED))
  gcv_object_t _object_Kerror_device_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REQUIRES_CLEANING))
  gcv_object_t _object_Kerror_device_requires_cleaning;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEV_NOT_EXIST))
  gcv_object_t _object_Kerror_dev_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DHCP_ADDRESS_CONFLICT))
  gcv_object_t _object_Kerror_dhcp_address_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIFFERENT_SERVICE_ACCOUNT))
  gcv_object_t _object_Kerror_different_service_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIRECTORY))
  gcv_object_t _object_Kerror_directory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIRECT_ACCESS_HANDLE))
  gcv_object_t _object_Kerror_direct_access_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_EFS_DISALLOWED))
  gcv_object_t _object_Kerror_dir_efs_disallowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_NOT_EMPTY))
  gcv_object_t _object_Kerror_dir_not_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_NOT_ROOT))
  gcv_object_t _object_Kerror_dir_not_root;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISCARDED))
  gcv_object_t _object_Kerror_discarded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_CHANGE))
  gcv_object_t _object_Kerror_disk_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_CORRUPT))
  gcv_object_t _object_Kerror_disk_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_FULL))
  gcv_object_t _object_Kerror_disk_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_OPERATION_FAILED))
  gcv_object_t _object_Kerror_disk_operation_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_RECALIBRATE_FAILED))
  gcv_object_t _object_Kerror_disk_recalibrate_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_RESET_FAILED))
  gcv_object_t _object_Kerror_disk_reset_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_TOO_FRAGMENTED))
  gcv_object_t _object_Kerror_disk_too_fragmented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DLL_INIT_FAILED))
  gcv_object_t _object_Kerror_dll_init_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DLL_NOT_FOUND))
  gcv_object_t _object_Kerror_dll_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_CONTROLLER_EXISTS))
  gcv_object_t _object_Kerror_domain_controller_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_CONTROLLER_NOT_FOUND))
  gcv_object_t _object_Kerror_domain_controller_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_EXISTS))
  gcv_object_t _object_Kerror_domain_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_LIMIT_EXCEEDED))
  gcv_object_t _object_Kerror_domain_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_TRUST_INCONSISTENT))
  gcv_object_t _object_Kerror_domain_trust_inconsistent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOWNGRADE_DETECTED))
  gcv_object_t _object_Kerror_downgrade_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVER_BLOCKED))
  gcv_object_t _object_Kerror_driver_blocked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVE_LOCKED))
  gcv_object_t _object_Kerror_drive_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVE_MEDIA_MISMATCH))
  gcv_object_t _object_Kerror_drive_media_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ADD_REPLICA_INHIBITED))
  gcv_object_t _object_Kerror_ds_add_replica_inhibited;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ADMIN_LIMIT_EXCEEDED))
  gcv_object_t _object_Kerror_ds_admin_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AFFECTS_MULTIPLE_DSAS))
  gcv_object_t _object_Kerror_ds_affects_multiple_dsas;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER))
  gcv_object_t _object_Kerror_ds_ag_cant_have_universal_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIASED_OBJ_MISSING))
  gcv_object_t _object_Kerror_ds_aliased_obj_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_DEREF_PROBLEM))
  gcv_object_t _object_Kerror_ds_alias_deref_problem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_POINTS_TO_ALIAS))
  gcv_object_t _object_Kerror_ds_alias_points_to_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_PROBLEM))
  gcv_object_t _object_Kerror_ds_alias_problem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS))
  gcv_object_t _object_Kerror_ds_attribute_or_value_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_OWNED_BY_SAM))
  gcv_object_t _object_Kerror_ds_attribute_owned_by_sam;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED))
  gcv_object_t _object_Kerror_ds_attribute_type_undefined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_ds_att_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_IS_NOT_ON_OBJ))
  gcv_object_t _object_Kerror_ds_att_is_not_on_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_NOT_DEF_FOR_CLASS))
  gcv_object_t _object_Kerror_ds_att_not_def_for_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_NOT_DEF_IN_SCHEMA))
  gcv_object_t _object_Kerror_ds_att_not_def_in_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_SCHEMA_REQ_ID))
  gcv_object_t _object_Kerror_ds_att_schema_req_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_SCHEMA_REQ_SYNTAX))
  gcv_object_t _object_Kerror_ds_att_schema_req_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_VAL_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_ds_att_val_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTHORIZATION_FAILED))
  gcv_object_t _object_Kerror_ds_authorization_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTH_METHOD_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_ds_auth_method_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTH_UNKNOWN))
  gcv_object_t _object_Kerror_ds_auth_unknown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUX_CLS_TEST_FAIL))
  gcv_object_t _object_Kerror_ds_aux_cls_test_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BACKLINK_WITHOUT_LINK))
  gcv_object_t _object_Kerror_ds_backlink_without_link;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_ATT_SCHEMA_SYNTAX))
  gcv_object_t _object_Kerror_ds_bad_att_schema_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_HIERARCHY_FILE))
  gcv_object_t _object_Kerror_ds_bad_hierarchy_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_INSTANCE_TYPE))
  gcv_object_t _object_Kerror_ds_bad_instance_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_NAME_SYNTAX))
  gcv_object_t _object_Kerror_ds_bad_name_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_RDN_ATT_ID_SYNTAX))
  gcv_object_t _object_Kerror_ds_bad_rdn_att_id_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED))
  gcv_object_t _object_Kerror_ds_build_hierarchy_table_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BUSY))
  gcv_object_t _object_Kerror_ds_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD))
  gcv_object_t _object_Kerror_ds_cant_access_remote_part_of_ad;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_ATT_VALUES))
  gcv_object_t _object_Kerror_ds_cant_add_att_values;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_SYSTEM_ONLY))
  gcv_object_t _object_Kerror_ds_cant_add_system_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_TO_GC))
  gcv_object_t _object_Kerror_ds_cant_add_to_gc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CACHE_ATT))
  gcv_object_t _object_Kerror_ds_cant_cache_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CACHE_CLASS))
  gcv_object_t _object_Kerror_ds_cant_cache_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC))
  gcv_object_t _object_Kerror_ds_cant_create_in_nondomain_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CREATE_UNDER_SCHEMA))
  gcv_object_t _object_Kerror_ds_cant_create_under_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DELETE))
  gcv_object_t _object_Kerror_ds_cant_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DELETE_DSA_OBJ))
  gcv_object_t _object_Kerror_ds_cant_delete_dsa_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEL_MASTER_CROSSREF))
  gcv_object_t _object_Kerror_ds_cant_del_master_crossref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC))
  gcv_object_t _object_Kerror_ds_cant_demote_with_writeable_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEREF_ALIAS))
  gcv_object_t _object_Kerror_ds_cant_deref_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN))
  gcv_object_t _object_Kerror_ds_cant_derive_spn_for_deleted_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF))
  gcv_object_t _object_Kerror_ds_cant_derive_spn_without_server_ref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN))
  gcv_object_t _object_Kerror_ds_cant_find_dc_for_src_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_DSA_OBJ))
  gcv_object_t _object_Kerror_ds_cant_find_dsa_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_EXPECTED_NC))
  gcv_object_t _object_Kerror_ds_cant_find_expected_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_NC_IN_CACHE))
  gcv_object_t _object_Kerror_ds_cant_find_nc_in_cache;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MIX_MASTER_AND_REPS))
  gcv_object_t _object_Kerror_ds_cant_mix_master_and_reps;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_OBJ_CLASS))
  gcv_object_t _object_Kerror_ds_cant_mod_obj_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_PRIMARYGROUPID))
  gcv_object_t _object_Kerror_ds_cant_mod_primarygroupid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_SYSTEM_ONLY))
  gcv_object_t _object_Kerror_ds_cant_mod_system_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_ACCOUNT_GROUP))
  gcv_object_t _object_Kerror_ds_cant_move_account_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_APP_BASIC_GROUP))
  gcv_object_t _object_Kerror_ds_cant_move_app_basic_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_APP_QUERY_GROUP))
  gcv_object_t _object_Kerror_ds_cant_move_app_query_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_DELETED_OBJECT))
  gcv_object_t _object_Kerror_ds_cant_move_deleted_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_RESOURCE_GROUP))
  gcv_object_t _object_Kerror_ds_cant_move_resource_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ON_NON_LEAF))
  gcv_object_t _object_Kerror_ds_cant_on_non_leaf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ON_RDN))
  gcv_object_t _object_Kerror_ds_cant_on_rdn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REMOVE_ATT_CACHE))
  gcv_object_t _object_Kerror_ds_cant_remove_att_cache;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REMOVE_CLASS_CACHE))
  gcv_object_t _object_Kerror_ds_cant_remove_class_cache;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REM_MISSING_ATT))
  gcv_object_t _object_Kerror_ds_cant_rem_missing_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REM_MISSING_ATT_VAL))
  gcv_object_t _object_Kerror_ds_cant_rem_missing_att_val;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REPLACE_HIDDEN_REC))
  gcv_object_t _object_Kerror_ds_cant_replace_hidden_rec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_ATTS))
  gcv_object_t _object_Kerror_ds_cant_retrieve_atts;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_CHILD))
  gcv_object_t _object_Kerror_ds_cant_retrieve_child;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_DN))
  gcv_object_t _object_Kerror_ds_cant_retrieve_dn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_INSTANCE))
  gcv_object_t _object_Kerror_ds_cant_retrieve_instance;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_SD))
  gcv_object_t _object_Kerror_ds_cant_retrieve_sd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_START))
  gcv_object_t _object_Kerror_ds_cant_start;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ))
  gcv_object_t _object_Kerror_ds_cant_tree_delete_critical_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS))
  gcv_object_t _object_Kerror_ds_cant_with_acct_group_membershps;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CHILDREN_EXIST))
  gcv_object_t _object_Kerror_ds_children_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLASS_MUST_BE_CONCRETE))
  gcv_object_t _object_Kerror_ds_class_must_be_concrete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLASS_NOT_DSA))
  gcv_object_t _object_Kerror_ds_class_not_dsa;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLIENT_LOOP))
  gcv_object_t _object_Kerror_ds_client_loop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CODE_INCONSISTENCY))
  gcv_object_t _object_Kerror_ds_code_inconsistency;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COMPARE_FALSE))
  gcv_object_t _object_Kerror_ds_compare_false;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COMPARE_TRUE))
  gcv_object_t _object_Kerror_ds_compare_true;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONFIDENTIALITY_REQUIRED))
  gcv_object_t _object_Kerror_ds_confidentiality_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONFIG_PARAM_MISSING))
  gcv_object_t _object_Kerror_ds_config_param_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONSTRAINT_VIOLATION))
  gcv_object_t _object_Kerror_ds_constraint_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONSTRUCTED_ATT_MOD))
  gcv_object_t _object_Kerror_ds_constructed_att_mod;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONTROL_NOT_FOUND))
  gcv_object_t _object_Kerror_ds_control_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_CONTACT_FSMO))
  gcv_object_t _object_Kerror_ds_couldnt_contact_fsmo;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE))
  gcv_object_t _object_Kerror_ds_couldnt_identify_objects_for_tree_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE))
  gcv_object_t _object_Kerror_ds_couldnt_lock_tree_for_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_UPDATE_SPNS))
  gcv_object_t _object_Kerror_ds_couldnt_update_spns;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COUNTING_AB_INDICES_FAILED))
  gcv_object_t _object_Kerror_ds_counting_ab_indices_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD))
  gcv_object_t _object_Kerror_ds_cross_domain_cleanup_reqd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_DOM_MOVE_ERROR))
  gcv_object_t _object_Kerror_ds_cross_dom_move_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_NC_DN_RENAME))
  gcv_object_t _object_Kerror_ds_cross_nc_dn_rename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_REF_BUSY))
  gcv_object_t _object_Kerror_ds_cross_ref_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_REF_EXISTS))
  gcv_object_t _object_Kerror_ds_cross_ref_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE))
  gcv_object_t _object_Kerror_ds_cr_impossible_to_validate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2))
  gcv_object_t _object_Kerror_ds_cr_impossible_to_validate_v2;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DATABASE_ERROR))
  gcv_object_t _object_Kerror_ds_database_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DECODING_ERROR))
  gcv_object_t _object_Kerror_ds_decoding_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED))
  gcv_object_t _object_Kerror_ds_destination_auditing_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST))
  gcv_object_t _object_Kerror_ds_destination_domain_not_in_forest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DIFFERENT_REPL_EPOCHS))
  gcv_object_t _object_Kerror_ds_different_repl_epochs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_RENAME_IN_PROGRESS))
  gcv_object_t _object_Kerror_ds_domain_rename_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_VERSION_TOO_HIGH))
  gcv_object_t _object_Kerror_ds_domain_version_too_high;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_VERSION_TOO_LOW))
  gcv_object_t _object_Kerror_ds_domain_version_too_low;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_ABANDON_SYNC))
  gcv_object_t _object_Kerror_ds_dra_abandon_sync;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_ACCESS_DENIED))
  gcv_object_t _object_Kerror_ds_dra_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_DN))
  gcv_object_t _object_Kerror_ds_dra_bad_dn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_INSTANCE_TYPE))
  gcv_object_t _object_Kerror_ds_dra_bad_instance_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_NC))
  gcv_object_t _object_Kerror_ds_dra_bad_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BUSY))
  gcv_object_t _object_Kerror_ds_dra_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_CONNECTION_FAILED))
  gcv_object_t _object_Kerror_ds_dra_connection_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_DB_ERROR))
  gcv_object_t _object_Kerror_ds_dra_db_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_DN_EXISTS))
  gcv_object_t _object_Kerror_ds_dra_dn_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT))
  gcv_object_t _object_Kerror_ds_dra_earlier_schema_conlict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_EXTN_CONNECTION_FAILED))
  gcv_object_t _object_Kerror_ds_dra_extn_connection_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_GENERIC))
  gcv_object_t _object_Kerror_ds_dra_generic;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET))
  gcv_object_t _object_Kerror_ds_dra_incompatible_partial_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INCONSISTENT_DIT))
  gcv_object_t _object_Kerror_ds_dra_inconsistent_dit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INTERNAL_ERROR))
  gcv_object_t _object_Kerror_ds_dra_internal_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INVALID_PARAMETER))
  gcv_object_t _object_Kerror_ds_dra_invalid_parameter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_MAIL_PROBLEM))
  gcv_object_t _object_Kerror_ds_dra_mail_problem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_MISSING_PARENT))
  gcv_object_t _object_Kerror_ds_dra_missing_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NAME_COLLISION))
  gcv_object_t _object_Kerror_ds_dra_name_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_ds_dra_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NO_REPLICA))
  gcv_object_t _object_Kerror_ds_dra_no_replica;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OBJ_IS_REP_SOURCE))
  gcv_object_t _object_Kerror_ds_dra_obj_is_rep_source;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OBJ_NC_MISMATCH))
  gcv_object_t _object_Kerror_ds_dra_obj_nc_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OUT_OF_MEM))
  gcv_object_t _object_Kerror_ds_dra_out_of_mem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_PREEMPTED))
  gcv_object_t _object_Kerror_ds_dra_preempted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REF_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_ds_dra_ref_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REF_NOT_FOUND))
  gcv_object_t _object_Kerror_ds_dra_ref_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REPL_PENDING))
  gcv_object_t _object_Kerror_ds_dra_repl_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_RPC_CANCELLED))
  gcv_object_t _object_Kerror_ds_dra_rpc_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_CONFLICT))
  gcv_object_t _object_Kerror_ds_dra_schema_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_INFO_SHIP))
  gcv_object_t _object_Kerror_ds_dra_schema_info_ship;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_MISMATCH))
  gcv_object_t _object_Kerror_ds_dra_schema_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SHUTDOWN))
  gcv_object_t _object_Kerror_ds_dra_shutdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SINK_DISABLED))
  gcv_object_t _object_Kerror_ds_dra_sink_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_DISABLED))
  gcv_object_t _object_Kerror_ds_dra_source_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA))
  gcv_object_t _object_Kerror_ds_dra_source_is_partial_replica;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_REINSTALLED))
  gcv_object_t _object_Kerror_ds_dra_source_reinstalled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRS_EXTENSIONS_CHANGED))
  gcv_object_t _object_Kerror_ds_drs_extensions_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DSA_MUST_BE_INT_MASTER))
  gcv_object_t _object_Kerror_ds_dsa_must_be_int_master;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DST_DOMAIN_NOT_NATIVE))
  gcv_object_t _object_Kerror_ds_dst_domain_not_native;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DST_NC_MISMATCH))
  gcv_object_t _object_Kerror_ds_dst_nc_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DS_REQUIRED))
  gcv_object_t _object_Kerror_ds_ds_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUPLICATE_ID_FOUND))
  gcv_object_t _object_Kerror_ds_duplicate_id_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_LDAP_DISPLAY_NAME))
  gcv_object_t _object_Kerror_ds_dup_ldap_display_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_LINK_ID))
  gcv_object_t _object_Kerror_ds_dup_link_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_MAPI_ID))
  gcv_object_t _object_Kerror_ds_dup_mapi_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_MSDS_INTID))
  gcv_object_t _object_Kerror_ds_dup_msds_intid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_OID))
  gcv_object_t _object_Kerror_ds_dup_oid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_RDN))
  gcv_object_t _object_Kerror_ds_dup_rdn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_SCHEMA_ID_GUID))
  gcv_object_t _object_Kerror_ds_dup_schema_id_guid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ENCODING_ERROR))
  gcv_object_t _object_Kerror_ds_encoding_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EPOCH_MISMATCH))
  gcv_object_t _object_Kerror_ds_epoch_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTING_AD_CHILD_NC))
  gcv_object_t _object_Kerror_ds_existing_ad_child_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_AUX_CLS))
  gcv_object_t _object_Kerror_ds_exists_in_aux_cls;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_MAY_HAVE))
  gcv_object_t _object_Kerror_ds_exists_in_may_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_MUST_HAVE))
  gcv_object_t _object_Kerror_ds_exists_in_must_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_POSS_SUP))
  gcv_object_t _object_Kerror_ds_exists_in_poss_sup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_RDNATTID))
  gcv_object_t _object_Kerror_ds_exists_in_rdnattid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_SUB_CLS))
  gcv_object_t _object_Kerror_ds_exists_in_sub_cls;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FILTER_UNKNOWN))
  gcv_object_t _object_Kerror_ds_filter_unknown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS))
  gcv_object_t _object_Kerror_ds_filter_uses_contructed_attrs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FOREST_VERSION_TOO_HIGH))
  gcv_object_t _object_Kerror_ds_forest_version_too_high;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FOREST_VERSION_TOO_LOW))
  gcv_object_t _object_Kerror_ds_forest_version_too_low;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GCVERIFY_ERROR))
  gcv_object_t _object_Kerror_ds_gcverify_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GC_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_ds_gc_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GC_REQUIRED))
  gcv_object_t _object_Kerror_ds_gc_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GENERIC_ERROR))
  gcv_object_t _object_Kerror_ds_generic_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER))
  gcv_object_t _object_Kerror_ds_global_cant_have_crossdomain_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER))
  gcv_object_t _object_Kerror_ds_global_cant_have_local_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER))
  gcv_object_t _object_Kerror_ds_global_cant_have_universal_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GOVERNSID_MISSING))
  gcv_object_t _object_Kerror_ds_governsid_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GROUP_CONVERSION_ERROR))
  gcv_object_t _object_Kerror_ds_group_conversion_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_HAVE_PRIMARY_MEMBERS))
  gcv_object_t _object_Kerror_ds_have_primary_members;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED))
  gcv_object_t _object_Kerror_ds_hierarchy_table_malloc_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD))
  gcv_object_t _object_Kerror_ds_illegal_base_schema_mod;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_MOD_OPERATION))
  gcv_object_t _object_Kerror_ds_illegal_mod_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_SUPERIOR))
  gcv_object_t _object_Kerror_ds_illegal_superior;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION))
  gcv_object_t _object_Kerror_ds_illegal_xdom_move_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INAPPROPRIATE_AUTH))
  gcv_object_t _object_Kerror_ds_inappropriate_auth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INAPPROPRIATE_MATCHING))
  gcv_object_t _object_Kerror_ds_inappropriate_matching;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCOMPATIBLE_CONTROLS_USED))
  gcv_object_t _object_Kerror_ds_incompatible_controls_used;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCOMPATIBLE_VERSION))
  gcv_object_t _object_Kerror_ds_incompatible_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCORRECT_ROLE_OWNER))
  gcv_object_t _object_Kerror_ds_incorrect_role_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INIT_FAILURE))
  gcv_object_t _object_Kerror_ds_init_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INIT_FAILURE_CONSOLE))
  gcv_object_t _object_Kerror_ds_init_failure_console;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE))
  gcv_object_t _object_Kerror_ds_install_no_sch_version_in_inifile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_NO_SRC_SCH_VERSION))
  gcv_object_t _object_Kerror_ds_install_no_src_sch_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_SCHEMA_MISMATCH))
  gcv_object_t _object_Kerror_ds_install_schema_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT))
  gcv_object_t _object_Kerror_ds_insufficient_attr_to_create_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSUFF_ACCESS_RIGHTS))
  gcv_object_t _object_Kerror_ds_insuff_access_rights;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INTERNAL_FAILURE))
  gcv_object_t _object_Kerror_ds_internal_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_ATTRIBUTE_SYNTAX))
  gcv_object_t _object_Kerror_ds_invalid_attribute_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_DMD))
  gcv_object_t _object_Kerror_ds_invalid_dmd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_DN_SYNTAX))
  gcv_object_t _object_Kerror_ds_invalid_dn_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_GROUP_TYPE))
  gcv_object_t _object_Kerror_ds_invalid_group_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_LDAP_DISPLAY_NAME))
  gcv_object_t _object_Kerror_ds_invalid_ldap_display_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_NAME_FOR_SPN))
  gcv_object_t _object_Kerror_ds_invalid_name_for_spn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_ROLE_OWNER))
  gcv_object_t _object_Kerror_ds_invalid_role_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_SCRIPT))
  gcv_object_t _object_Kerror_ds_invalid_script;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_SEARCH_FLAG))
  gcv_object_t _object_Kerror_ds_invalid_search_flag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_IS_LEAF))
  gcv_object_t _object_Kerror_ds_is_leaf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_KEY_NOT_UNIQUE))
  gcv_object_t _object_Kerror_ds_key_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LINK_ID_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_ds_link_id_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER))
  gcv_object_t _object_Kerror_ds_local_cant_have_crossdomain_local_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_ERROR))
  gcv_object_t _object_Kerror_ds_local_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY))
  gcv_object_t _object_Kerror_ds_local_member_of_local_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOOKUP_FAILURE))
  gcv_object_t _object_Kerror_ds_lookup_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOOP_DETECT))
  gcv_object_t _object_Kerror_ds_loop_detect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOW_DSA_VERSION))
  gcv_object_t _object_Kerror_ds_low_dsa_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4))
  gcv_object_t _object_Kerror_ds_machine_account_created_prent4;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED))
  gcv_object_t _object_Kerror_ds_machine_account_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MASTERDSA_REQUIRED))
  gcv_object_t _object_Kerror_ds_masterdsa_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MAX_OBJ_SIZE_EXCEEDED))
  gcv_object_t _object_Kerror_ds_max_obj_size_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY))
  gcv_object_t _object_Kerror_ds_membership_evaluated_locally;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_EXPECTED_ATT))
  gcv_object_t _object_Kerror_ds_missing_expected_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_FSMO_SETTINGS))
  gcv_object_t _object_Kerror_ds_missing_fsmo_settings;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER))
  gcv_object_t _object_Kerror_ds_missing_infrastructure_container;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_REQUIRED_ATT))
  gcv_object_t _object_Kerror_ds_missing_required_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_SUPREF))
  gcv_object_t _object_Kerror_ds_missing_supref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG))
  gcv_object_t _object_Kerror_ds_modifydn_disallowed_by_flag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE))
  gcv_object_t _object_Kerror_ds_modifydn_disallowed_by_instance_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_WRONG_GRANDPARENT))
  gcv_object_t _object_Kerror_ds_modifydn_wrong_grandparent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MUST_BE_RUN_ON_DST_DC))
  gcv_object_t _object_Kerror_ds_must_be_run_on_dst_dc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_DOMAIN_ONLY))
  gcv_object_t _object_Kerror_ds_name_error_domain_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NOT_FOUND))
  gcv_object_t _object_Kerror_ds_name_error_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NOT_UNIQUE))
  gcv_object_t _object_Kerror_ds_name_error_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NO_MAPPING))
  gcv_object_t _object_Kerror_ds_name_error_no_mapping;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING))
  gcv_object_t _object_Kerror_ds_name_error_no_syntactical_mapping;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_RESOLVING))
  gcv_object_t _object_Kerror_ds_name_error_resolving;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_TRUST_REFERRAL))
  gcv_object_t _object_Kerror_ds_name_error_trust_referral;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_NOT_UNIQUE))
  gcv_object_t _object_Kerror_ds_name_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_REFERENCE_INVALID))
  gcv_object_t _object_Kerror_ds_name_reference_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TOO_LONG))
  gcv_object_t _object_Kerror_ds_name_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TOO_MANY_PARTS))
  gcv_object_t _object_Kerror_ds_name_too_many_parts;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TYPE_UNKNOWN))
  gcv_object_t _object_Kerror_ds_name_type_unknown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_UNPARSEABLE))
  gcv_object_t _object_Kerror_ds_name_unparseable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_VALUE_TOO_LONG))
  gcv_object_t _object_Kerror_ds_name_value_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAMING_MASTER_GC))
  gcv_object_t _object_Kerror_ds_naming_master_gc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAMING_VIOLATION))
  gcv_object_t _object_Kerror_ds_naming_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NCNAME_MISSING_CR_REF))
  gcv_object_t _object_Kerror_ds_ncname_missing_cr_ref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NCNAME_MUST_BE_NC))
  gcv_object_t _object_Kerror_ds_ncname_must_be_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NC_MUST_HAVE_NC_PARENT))
  gcv_object_t _object_Kerror_ds_nc_must_have_nc_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NC_STILL_HAS_DSAS))
  gcv_object_t _object_Kerror_ds_nc_still_has_dsas;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_MAY_HAVE))
  gcv_object_t _object_Kerror_ds_nonexistent_may_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_MUST_HAVE))
  gcv_object_t _object_Kerror_ds_nonexistent_must_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_POSS_SUP))
  gcv_object_t _object_Kerror_ds_nonexistent_poss_sup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONSAFE_SCHEMA_CHANGE))
  gcv_object_t _object_Kerror_ds_nonsafe_schema_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NON_BASE_SEARCH))
  gcv_object_t _object_Kerror_ds_non_base_search;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX))
  gcv_object_t _object_Kerror_ds_notify_filter_too_complex;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_AN_OBJECT))
  gcv_object_t _object_Kerror_ds_not_an_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC))
  gcv_object_t _object_Kerror_ds_not_authoritive_for_dst_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_CLOSEST))
  gcv_object_t _object_Kerror_ds_not_closest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_INSTALLED))
  gcv_object_t _object_Kerror_ds_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_ON_BACKLINK))
  gcv_object_t _object_Kerror_ds_not_on_backlink;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_ds_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_SUPPORTED_SORT_ORDER))
  gcv_object_t _object_Kerror_ds_not_supported_sort_order;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_ATTRIBUTE_OR_VALUE))
  gcv_object_t _object_Kerror_ds_no_attribute_or_value;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN))
  gcv_object_t _object_Kerror_ds_no_behavior_version_in_mixeddomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHAINED_EVAL))
  gcv_object_t _object_Kerror_ds_no_chained_eval;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHAINING))
  gcv_object_t _object_Kerror_ds_no_chaining;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHECKPOINT_WITH_PDC))
  gcv_object_t _object_Kerror_ds_no_checkpoint_with_pdc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CROSSREF_FOR_NC))
  gcv_object_t _object_Kerror_ds_no_crossref_for_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_DELETED_NAME))
  gcv_object_t _object_Kerror_ds_no_deleted_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS))
  gcv_object_t _object_Kerror_ds_no_fpo_in_universal_groups;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_MORE_RIDS))
  gcv_object_t _object_Kerror_ds_no_more_rids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_MSDS_INTID))
  gcv_object_t _object_Kerror_ds_no_msds_intid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN))
  gcv_object_t _object_Kerror_ds_no_nest_globalgroup_in_mixeddomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN))
  gcv_object_t _object_Kerror_ds_no_nest_localgroup_in_mixeddomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC))
  gcv_object_t _object_Kerror_ds_no_object_move_in_schema_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_PARENT_OBJECT))
  gcv_object_t _object_Kerror_ds_no_parent_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION))
  gcv_object_t _object_Kerror_ds_no_pkt_privacy_on_connection;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA))
  gcv_object_t _object_Kerror_ds_no_rdn_defined_in_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_REF_DOMAIN))
  gcv_object_t _object_Kerror_ds_no_ref_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_REQUESTED_ATTS_FOUND))
  gcv_object_t _object_Kerror_ds_no_requested_atts_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RESULTS_RETURNED))
  gcv_object_t _object_Kerror_ds_no_results_returned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RIDS_ALLOCATED))
  gcv_object_t _object_Kerror_ds_no_rids_allocated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_SUCH_OBJECT))
  gcv_object_t _object_Kerror_ds_no_such_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_TREE_DELETE_ABOVE_NC))
  gcv_object_t _object_Kerror_ds_no_tree_delete_above_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NTDSCRIPT_PROCESS_ERROR))
  gcv_object_t _object_Kerror_ds_ntdscript_process_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NTDSCRIPT_SYNTAX_ERROR))
  gcv_object_t _object_Kerror_ds_ntdscript_syntax_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_BEING_REMOVED))
  gcv_object_t _object_Kerror_ds_object_being_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_CLASS_REQUIRED))
  gcv_object_t _object_Kerror_ds_object_class_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_RESULTS_TOO_LARGE))
  gcv_object_t _object_Kerror_ds_object_results_too_large;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_NOT_DEFINED))
  gcv_object_t _object_Kerror_ds_obj_class_not_defined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_NOT_SUBCLASS))
  gcv_object_t _object_Kerror_ds_obj_class_not_subclass;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_VIOLATION))
  gcv_object_t _object_Kerror_ds_obj_class_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_GUID_EXISTS))
  gcv_object_t _object_Kerror_ds_obj_guid_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_NOT_FOUND))
  gcv_object_t _object_Kerror_ds_obj_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_STRING_NAME_EXISTS))
  gcv_object_t _object_Kerror_ds_obj_string_name_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_TOO_LARGE))
  gcv_object_t _object_Kerror_ds_obj_too_large;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OFFSET_RANGE_ERROR))
  gcv_object_t _object_Kerror_ds_offset_range_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OPERATIONS_ERROR))
  gcv_object_t _object_Kerror_ds_operations_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OUT_OF_SCOPE))
  gcv_object_t _object_Kerror_ds_out_of_scope;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OUT_OF_VERSION_STORE))
  gcv_object_t _object_Kerror_ds_out_of_version_store;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PARAM_ERROR))
  gcv_object_t _object_Kerror_ds_param_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PARENT_IS_AN_ALIAS))
  gcv_object_t _object_Kerror_ds_parent_is_an_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PDC_OPERATION_IN_PROGRESS))
  gcv_object_t _object_Kerror_ds_pdc_operation_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PROTOCOL_ERROR))
  gcv_object_t _object_Kerror_ds_protocol_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RANGE_CONSTRAINT))
  gcv_object_t _object_Kerror_ds_range_constraint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RDN_DOESNT_MATCH_SCHEMA))
  gcv_object_t _object_Kerror_ds_rdn_doesnt_match_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RECALCSCHEMA_FAILED))
  gcv_object_t _object_Kerror_ds_recalcschema_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFERRAL))
  gcv_object_t _object_Kerror_ds_referral;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFERRAL_LIMIT_EXCEEDED))
  gcv_object_t _object_Kerror_ds_referral_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFUSING_FSMO_ROLES))
  gcv_object_t _object_Kerror_ds_refusing_fsmo_roles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REMOTE_CROSSREF_OP_FAILED))
  gcv_object_t _object_Kerror_ds_remote_crossref_op_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REPLICATOR_ONLY))
  gcv_object_t _object_Kerror_ds_replicator_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR))
  gcv_object_t _object_Kerror_ds_replica_set_change_not_allowed_on_disabled_cr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RESERVED_LINK_ID))
  gcv_object_t _object_Kerror_ds_reserved_link_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RIDMGR_INIT_ERROR))
  gcv_object_t _object_Kerror_ds_ridmgr_init_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROLE_NOT_VERIFIED))
  gcv_object_t _object_Kerror_ds_role_not_verified;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_CANT_BE_SUBREF))
  gcv_object_t _object_Kerror_ds_root_cant_be_subref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_MUST_BE_NC))
  gcv_object_t _object_Kerror_ds_root_must_be_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_REQUIRES_CLASS_TOP))
  gcv_object_t _object_Kerror_ds_root_requires_class_top;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_INIT_FAILURE))
  gcv_object_t _object_Kerror_ds_sam_init_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_INIT_FAILURE_CONSOLE))
  gcv_object_t _object_Kerror_ds_sam_init_failure_console;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY))
  gcv_object_t _object_Kerror_ds_sam_need_bootkey_floppy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD))
  gcv_object_t _object_Kerror_ds_sam_need_bootkey_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_ALLOC_FAILED))
  gcv_object_t _object_Kerror_ds_schema_alloc_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_NOT_LOADED))
  gcv_object_t _object_Kerror_ds_schema_not_loaded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_UPDATE_DISALLOWED))
  gcv_object_t _object_Kerror_ds_schema_update_disallowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SECURITY_CHECKING_ERROR))
  gcv_object_t _object_Kerror_ds_security_checking_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SECURITY_ILLEGAL_MODIFY))
  gcv_object_t _object_Kerror_ds_security_illegal_modify;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEC_DESC_INVALID))
  gcv_object_t _object_Kerror_ds_sec_desc_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEC_DESC_TOO_SHORT))
  gcv_object_t _object_Kerror_ds_sec_desc_too_short;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEMANTIC_ATT_TEST))
  gcv_object_t _object_Kerror_ds_semantic_att_test;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SENSITIVE_GROUP_VIOLATION))
  gcv_object_t _object_Kerror_ds_sensitive_group_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SERVER_DOWN))
  gcv_object_t _object_Kerror_ds_server_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SHUTTING_DOWN))
  gcv_object_t _object_Kerror_ds_shutting_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SINGLE_USER_MODE_FAILED))
  gcv_object_t _object_Kerror_ds_single_user_mode_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SINGLE_VALUE_CONSTRAINT))
  gcv_object_t _object_Kerror_ds_single_value_constraint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SIZELIMIT_EXCEEDED))
  gcv_object_t _object_Kerror_ds_sizelimit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SORT_CONTROL_MISSING))
  gcv_object_t _object_Kerror_ds_sort_control_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SOURCE_AUDITING_NOT_ENABLED))
  gcv_object_t _object_Kerror_ds_source_auditing_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SOURCE_DOMAIN_IN_FOREST))
  gcv_object_t _object_Kerror_ds_source_domain_in_forest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_AND_DST_NC_IDENTICAL))
  gcv_object_t _object_Kerror_ds_src_and_dst_nc_identical;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH))
  gcv_object_t _object_Kerror_ds_src_and_dst_object_class_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER))
  gcv_object_t _object_Kerror_ds_src_dc_must_be_sp4_or_greater;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_GUID_MISMATCH))
  gcv_object_t _object_Kerror_ds_src_guid_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_NAME_MISMATCH))
  gcv_object_t _object_Kerror_ds_src_name_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER))
  gcv_object_t _object_Kerror_ds_src_obj_not_group_or_user;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_SID_EXISTS_IN_FOREST))
  gcv_object_t _object_Kerror_ds_src_sid_exists_in_forest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_STRING_SD_CONVERSION_FAILED))
  gcv_object_t _object_Kerror_ds_string_sd_conversion_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_STRONG_AUTH_REQUIRED))
  gcv_object_t _object_Kerror_ds_strong_auth_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUBREF_MUST_HAVE_PARENT))
  gcv_object_t _object_Kerror_ds_subref_must_have_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD))
  gcv_object_t _object_Kerror_ds_subtree_notify_not_nc_head;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUB_CLS_TEST_FAIL))
  gcv_object_t _object_Kerror_ds_sub_cls_test_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SYNTAX_MISMATCH))
  gcv_object_t _object_Kerror_ds_syntax_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_THREAD_LIMIT_EXCEEDED))
  gcv_object_t _object_Kerror_ds_thread_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_TIMELIMIT_EXCEEDED))
  gcv_object_t _object_Kerror_ds_timelimit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_TREE_DELETE_NOT_FINISHED))
  gcv_object_t _object_Kerror_ds_tree_delete_not_finished;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNABLE_TO_SURRENDER_ROLES))
  gcv_object_t _object_Kerror_ds_unable_to_surrender_roles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNAVAILABLE))
  gcv_object_t _object_Kerror_ds_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNAVAILABLE_CRIT_EXTENSION))
  gcv_object_t _object_Kerror_ds_unavailable_crit_extension;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNICODEPWD_NOT_IN_QUOTES))
  gcv_object_t _object_Kerror_ds_unicodepwd_not_in_quotes;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER))
  gcv_object_t _object_Kerror_ds_universal_cant_have_local_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNKNOWN_ERROR))
  gcv_object_t _object_Kerror_ds_unknown_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNKNOWN_OPERATION))
  gcv_object_t _object_Kerror_ds_unknown_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNWILLING_TO_PERFORM))
  gcv_object_t _object_Kerror_ds_unwilling_to_perform;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_USER_BUFFER_TO_SMALL))
  gcv_object_t _object_Kerror_ds_user_buffer_to_small;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL))
  gcv_object_t _object_Kerror_ds_wko_container_cannot_be_special;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WRONG_LINKED_ATT_SYNTAX))
  gcv_object_t _object_Kerror_ds_wrong_linked_att_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WRONG_OM_OBJ_CLASS))
  gcv_object_t _object_Kerror_ds_wrong_om_obj_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUPLICATE_SERVICE_NAME))
  gcv_object_t _object_Kerror_duplicate_service_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUPLICATE_TAG))
  gcv_object_t _object_Kerror_duplicate_tag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUP_DOMAINNAME))
  gcv_object_t _object_Kerror_dup_domainname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUP_NAME))
  gcv_object_t _object_Kerror_dup_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DYNLINK_FROM_INVALID_RING))
  gcv_object_t _object_Kerror_dynlink_from_invalid_ring;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EAS_DIDNT_FIT))
  gcv_object_t _object_Kerror_eas_didnt_fit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EAS_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_eas_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_ACCESS_DENIED))
  gcv_object_t _object_Kerror_ea_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_FILE_CORRUPT))
  gcv_object_t _object_Kerror_ea_file_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_LIST_INCONSISTENT))
  gcv_object_t _object_Kerror_ea_list_inconsistent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_TABLE_FULL))
  gcv_object_t _object_Kerror_ea_table_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_ALG_BLOB_TOO_BIG))
  gcv_object_t _object_Kerror_efs_alg_blob_too_big;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_DISABLED))
  gcv_object_t _object_Kerror_efs_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_SERVER_NOT_TRUSTED))
  gcv_object_t _object_Kerror_efs_server_not_trusted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_VERSION_NOT_SUPPORT))
  gcv_object_t _object_Kerror_efs_version_not_support;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EMPTY))
  gcv_object_t _object_Kerror_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ENCRYPTION_FAILED))
  gcv_object_t _object_Kerror_encryption_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_END_OF_MEDIA))
  gcv_object_t _object_Kerror_end_of_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ENVVAR_NOT_FOUND))
  gcv_object_t _object_Kerror_envvar_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EOM_OVERFLOW))
  gcv_object_t _object_Kerror_eom_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_CANT_START))
  gcv_object_t _object_Kerror_eventlog_cant_start;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_FILE_CHANGED))
  gcv_object_t _object_Kerror_eventlog_file_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_FILE_CORRUPT))
  gcv_object_t _object_Kerror_eventlog_file_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXCEPTION_IN_SERVICE))
  gcv_object_t _object_Kerror_exception_in_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXCL_SEM_ALREADY_OWNED))
  gcv_object_t _object_Kerror_excl_sem_already_owned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY))
  gcv_object_t _object_Kerror_exe_cannot_modify_signed_binary;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY))
  gcv_object_t _object_Kerror_exe_cannot_modify_strong_signed_binary;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_MACHINE_TYPE_MISMATCH))
  gcv_object_t _object_Kerror_exe_machine_type_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_MARKED_INVALID))
  gcv_object_t _object_Kerror_exe_marked_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXTENDED_ERROR))
  gcv_object_t _object_Kerror_extended_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FAILED_SERVICE_CONTROLLER_CONNECT))
  gcv_object_t _object_Kerror_failed_service_controller_connect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FAIL_I24))
  gcv_object_t _object_Kerror_fail_i24;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILEMARK_DETECTED))
  gcv_object_t _object_Kerror_filemark_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILENAME_EXCED_RANGE))
  gcv_object_t _object_Kerror_filename_exced_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_CORRUPT))
  gcv_object_t _object_Kerror_file_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_ENCRYPTED))
  gcv_object_t _object_Kerror_file_encrypted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_EXISTS))
  gcv_object_t _object_Kerror_file_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_INVALID))
  gcv_object_t _object_Kerror_file_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_NOT_ENCRYPTED))
  gcv_object_t _object_Kerror_file_not_encrypted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_NOT_FOUND))
  gcv_object_t _object_Kerror_file_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_OFFLINE))
  gcv_object_t _object_Kerror_file_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_READ_ONLY))
  gcv_object_t _object_Kerror_file_read_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_BAD_REGISTERS))
  gcv_object_t _object_Kerror_floppy_bad_registers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_ID_MARK_NOT_FOUND))
  gcv_object_t _object_Kerror_floppy_id_mark_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_UNKNOWN_ERROR))
  gcv_object_t _object_Kerror_floppy_unknown_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_WRONG_CYLINDER))
  gcv_object_t _object_Kerror_floppy_wrong_cylinder;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FULLSCREEN_MODE))
  gcv_object_t _object_Kerror_fullscreen_mode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FULL_BACKUP))
  gcv_object_t _object_Kerror_full_backup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FUNCTION_FAILED))
  gcv_object_t _object_Kerror_function_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FUNCTION_NOT_CALLED))
  gcv_object_t _object_Kerror_function_not_called;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GENERIC_NOT_MAPPED))
  gcv_object_t _object_Kerror_generic_not_mapped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GEN_FAILURE))
  gcv_object_t _object_Kerror_gen_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GLOBAL_ONLY_HOOK))
  gcv_object_t _object_Kerror_global_only_hook;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GRACEFUL_DISCONNECT))
  gcv_object_t _object_Kerror_graceful_disconnect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_EXISTS))
  gcv_object_t _object_Kerror_group_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_group_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_FOUND))
  gcv_object_t _object_Kerror_group_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_ONLINE))
  gcv_object_t _object_Kerror_group_not_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HANDLE_DISK_FULL))
  gcv_object_t _object_Kerror_handle_disk_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HANDLE_EOF))
  gcv_object_t _object_Kerror_handle_eof;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_NEEDS_HMOD))
  gcv_object_t _object_Kerror_hook_needs_hmod;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_NOT_INSTALLED))
  gcv_object_t _object_Kerror_hook_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_TYPE_NOT_ALLOWED))
  gcv_object_t _object_Kerror_hook_type_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_DOWN))
  gcv_object_t _object_Kerror_host_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_host_node_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_GROUP_OWNER))
  gcv_object_t _object_Kerror_host_node_not_group_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_RESOURCE_OWNER))
  gcv_object_t _object_Kerror_host_node_not_resource_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_UNREACHABLE))
  gcv_object_t _object_Kerror_host_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOTKEY_ALREADY_REGISTERED))
  gcv_object_t _object_Kerror_hotkey_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOTKEY_NOT_REGISTERED))
  gcv_object_t _object_Kerror_hotkey_not_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HWNDS_HAVE_DIFF_PARENT))
  gcv_object_t _object_Kerror_hwnds_have_diff_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ICM_NOT_ENABLED))
  gcv_object_t _object_Kerror_icm_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ILLEGAL_ELEMENT_ADDRESS))
  gcv_object_t _object_Kerror_illegal_element_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ILL_FORMED_PASSWORD))
  gcv_object_t _object_Kerror_ill_formed_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INCORRECT_ADDRESS))
  gcv_object_t _object_Kerror_incorrect_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INC_BACKUP))
  gcv_object_t _object_Kerror_inc_backup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INDEX_ABSENT))
  gcv_object_t _object_Kerror_index_absent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INDIGENOUS_TYPE))
  gcv_object_t _object_Kerror_indigenous_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INFLOOP_IN_RELOC_CHAIN))
  gcv_object_t _object_Kerror_infloop_in_reloc_chain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_ALREADY_RUNNING))
  gcv_object_t _object_Kerror_install_already_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_FAILURE))
  gcv_object_t _object_Kerror_install_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_LANGUAGE_UNSUPPORTED))
  gcv_object_t _object_Kerror_install_language_unsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_LOG_FAILURE))
  gcv_object_t _object_Kerror_install_log_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_NOTUSED))
  gcv_object_t _object_Kerror_install_notused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_INVALID))
  gcv_object_t _object_Kerror_install_package_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_OPEN_FAILED))
  gcv_object_t _object_Kerror_install_package_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_REJECTED))
  gcv_object_t _object_Kerror_install_package_rejected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_VERSION))
  gcv_object_t _object_Kerror_install_package_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PLATFORM_UNSUPPORTED))
  gcv_object_t _object_Kerror_install_platform_unsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_REMOTE_DISALLOWED))
  gcv_object_t _object_Kerror_install_remote_disallowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_REMOTE_PROHIBITED))
  gcv_object_t _object_Kerror_install_remote_prohibited;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SERVICE_FAILURE))
  gcv_object_t _object_Kerror_install_service_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SOURCE_ABSENT))
  gcv_object_t _object_Kerror_install_source_absent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SUSPEND))
  gcv_object_t _object_Kerror_install_suspend;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TEMP_UNWRITABLE))
  gcv_object_t _object_Kerror_install_temp_unwritable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TRANSFORM_FAILURE))
  gcv_object_t _object_Kerror_install_transform_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TRANSFORM_REJECTED))
  gcv_object_t _object_Kerror_install_transform_rejected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_UI_FAILURE))
  gcv_object_t _object_Kerror_install_ui_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_USEREXIT))
  gcv_object_t _object_Kerror_install_userexit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSUFFICIENT_BUFFER))
  gcv_object_t _object_Kerror_insufficient_buffer;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_DB_CORRUPTION))
  gcv_object_t _object_Kerror_internal_db_corruption;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_DB_ERROR))
  gcv_object_t _object_Kerror_internal_db_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_ERROR))
  gcv_object_t _object_Kerror_internal_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCEL_HANDLE))
  gcv_object_t _object_Kerror_invalid_accel_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCESS))
  gcv_object_t _object_Kerror_invalid_access;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCOUNT_NAME))
  gcv_object_t _object_Kerror_invalid_account_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACL))
  gcv_object_t _object_Kerror_invalid_acl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ADDRESS))
  gcv_object_t _object_Kerror_invalid_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_AT_INTERRUPT_TIME))
  gcv_object_t _object_Kerror_invalid_at_interrupt_time;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_BLOCK))
  gcv_object_t _object_Kerror_invalid_block;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_BLOCK_LENGTH))
  gcv_object_t _object_Kerror_invalid_block_length;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CATEGORY))
  gcv_object_t _object_Kerror_invalid_category;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CLEANER))
  gcv_object_t _object_Kerror_invalid_cleaner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CMM))
  gcv_object_t _object_Kerror_invalid_cmm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COLORINDEX))
  gcv_object_t _object_Kerror_invalid_colorindex;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COLORSPACE))
  gcv_object_t _object_Kerror_invalid_colorspace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMBOBOX_MESSAGE))
  gcv_object_t _object_Kerror_invalid_combobox_message;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMMAND_LINE))
  gcv_object_t _object_Kerror_invalid_command_line;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMPUTERNAME))
  gcv_object_t _object_Kerror_invalid_computername;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CURSOR_HANDLE))
  gcv_object_t _object_Kerror_invalid_cursor_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DATA))
  gcv_object_t _object_Kerror_invalid_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DATATYPE))
  gcv_object_t _object_Kerror_invalid_datatype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DLL))
  gcv_object_t _object_Kerror_invalid_dll;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAINNAME))
  gcv_object_t _object_Kerror_invalid_domainname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAIN_ROLE))
  gcv_object_t _object_Kerror_invalid_domain_role;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAIN_STATE))
  gcv_object_t _object_Kerror_invalid_domain_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DRIVE))
  gcv_object_t _object_Kerror_invalid_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DRIVE_OBJECT))
  gcv_object_t _object_Kerror_invalid_drive_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DWP_HANDLE))
  gcv_object_t _object_Kerror_invalid_dwp_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EA_HANDLE))
  gcv_object_t _object_Kerror_invalid_ea_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EA_NAME))
  gcv_object_t _object_Kerror_invalid_ea_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EDIT_HEIGHT))
  gcv_object_t _object_Kerror_invalid_edit_height;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ENVIRONMENT))
  gcv_object_t _object_Kerror_invalid_environment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EVENTNAME))
  gcv_object_t _object_Kerror_invalid_eventname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EVENT_COUNT))
  gcv_object_t _object_Kerror_invalid_event_count;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EXE_SIGNATURE))
  gcv_object_t _object_Kerror_invalid_exe_signature;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FIELD))
  gcv_object_t _object_Kerror_invalid_field;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FILTER_PROC))
  gcv_object_t _object_Kerror_invalid_filter_proc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FLAGS))
  gcv_object_t _object_Kerror_invalid_flags;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FLAG_NUMBER))
  gcv_object_t _object_Kerror_invalid_flag_number;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FORM_NAME))
  gcv_object_t _object_Kerror_invalid_form_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FORM_SIZE))
  gcv_object_t _object_Kerror_invalid_form_size;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FUNCTION))
  gcv_object_t _object_Kerror_invalid_function;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GROUPNAME))
  gcv_object_t _object_Kerror_invalid_groupname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GROUP_ATTRIBUTES))
  gcv_object_t _object_Kerror_invalid_group_attributes;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GW_COMMAND))
  gcv_object_t _object_Kerror_invalid_gw_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HANDLE))
  gcv_object_t _object_Kerror_invalid_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HANDLE_STATE))
  gcv_object_t _object_Kerror_invalid_handle_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HOOK_FILTER))
  gcv_object_t _object_Kerror_invalid_hook_filter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HOOK_HANDLE))
  gcv_object_t _object_Kerror_invalid_hook_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ICON_HANDLE))
  gcv_object_t _object_Kerror_invalid_icon_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ID_AUTHORITY))
  gcv_object_t _object_Kerror_invalid_id_authority;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_IMPORT_OF_NON_DLL))
  gcv_object_t _object_Kerror_invalid_import_of_non_dll;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_INDEX))
  gcv_object_t _object_Kerror_invalid_index;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_KEYBOARD_HANDLE))
  gcv_object_t _object_Kerror_invalid_keyboard_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LB_MESSAGE))
  gcv_object_t _object_Kerror_invalid_lb_message;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LEVEL))
  gcv_object_t _object_Kerror_invalid_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LIBRARY))
  gcv_object_t _object_Kerror_invalid_library;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LIST_FORMAT))
  gcv_object_t _object_Kerror_invalid_list_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LOGON_HOURS))
  gcv_object_t _object_Kerror_invalid_logon_hours;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LOGON_TYPE))
  gcv_object_t _object_Kerror_invalid_logon_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEDIA))
  gcv_object_t _object_Kerror_invalid_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEDIA_POOL))
  gcv_object_t _object_Kerror_invalid_media_pool;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEMBER))
  gcv_object_t _object_Kerror_invalid_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MENU_HANDLE))
  gcv_object_t _object_Kerror_invalid_menu_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGE))
  gcv_object_t _object_Kerror_invalid_message;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGEDEST))
  gcv_object_t _object_Kerror_invalid_messagedest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGENAME))
  gcv_object_t _object_Kerror_invalid_messagename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MINALLOCSIZE))
  gcv_object_t _object_Kerror_invalid_minallocsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MODULETYPE))
  gcv_object_t _object_Kerror_invalid_moduletype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MONITOR_HANDLE))
  gcv_object_t _object_Kerror_invalid_monitor_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MSGBOX_STYLE))
  gcv_object_t _object_Kerror_invalid_msgbox_style;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_NAME))
  gcv_object_t _object_Kerror_invalid_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_NETNAME))
  gcv_object_t _object_Kerror_invalid_netname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPERATION))
  gcv_object_t _object_Kerror_invalid_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPERATION_ON_QUORUM))
  gcv_object_t _object_Kerror_invalid_operation_on_quorum;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPLOCK_PROTOCOL))
  gcv_object_t _object_Kerror_invalid_oplock_protocol;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ORDINAL))
  gcv_object_t _object_Kerror_invalid_ordinal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OWNER))
  gcv_object_t _object_Kerror_invalid_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PARAMETER))
  gcv_object_t _object_Kerror_invalid_parameter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PASSWORD))
  gcv_object_t _object_Kerror_invalid_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PASSWORDNAME))
  gcv_object_t _object_Kerror_invalid_passwordname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PIXEL_FORMAT))
  gcv_object_t _object_Kerror_invalid_pixel_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRIMARY_GROUP))
  gcv_object_t _object_Kerror_invalid_primary_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_COMMAND))
  gcv_object_t _object_Kerror_invalid_printer_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_NAME))
  gcv_object_t _object_Kerror_invalid_printer_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_STATE))
  gcv_object_t _object_Kerror_invalid_printer_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINT_MONITOR))
  gcv_object_t _object_Kerror_invalid_print_monitor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRIORITY))
  gcv_object_t _object_Kerror_invalid_priority;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PROFILE))
  gcv_object_t _object_Kerror_invalid_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_REPARSE_DATA))
  gcv_object_t _object_Kerror_invalid_reparse_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SCROLLBAR_RANGE))
  gcv_object_t _object_Kerror_invalid_scrollbar_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SECURITY_DESCR))
  gcv_object_t _object_Kerror_invalid_security_descr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEGDPL))
  gcv_object_t _object_Kerror_invalid_segdpl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEGMENT_NUMBER))
  gcv_object_t _object_Kerror_invalid_segment_number;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEPARATOR_FILE))
  gcv_object_t _object_Kerror_invalid_separator_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVER_STATE))
  gcv_object_t _object_Kerror_invalid_server_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICENAME))
  gcv_object_t _object_Kerror_invalid_servicename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_ACCOUNT))
  gcv_object_t _object_Kerror_invalid_service_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_CONTROL))
  gcv_object_t _object_Kerror_invalid_service_control;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_LOCK))
  gcv_object_t _object_Kerror_invalid_service_lock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SHARENAME))
  gcv_object_t _object_Kerror_invalid_sharename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SHOWWIN_COMMAND))
  gcv_object_t _object_Kerror_invalid_showwin_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SID))
  gcv_object_t _object_Kerror_invalid_sid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SIGNAL_NUMBER))
  gcv_object_t _object_Kerror_invalid_signal_number;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SPI_VALUE))
  gcv_object_t _object_Kerror_invalid_spi_value;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STACKSEG))
  gcv_object_t _object_Kerror_invalid_stackseg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STARTING_CODESEG))
  gcv_object_t _object_Kerror_invalid_starting_codeseg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STATE))
  gcv_object_t _object_Kerror_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SUB_AUTHORITY))
  gcv_object_t _object_Kerror_invalid_sub_authority;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TABLE))
  gcv_object_t _object_Kerror_invalid_table;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TARGET_HANDLE))
  gcv_object_t _object_Kerror_invalid_target_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_THREAD_ID))
  gcv_object_t _object_Kerror_invalid_thread_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TIME))
  gcv_object_t _object_Kerror_invalid_time;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TRANSFORM))
  gcv_object_t _object_Kerror_invalid_transform;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_USER_BUFFER))
  gcv_object_t _object_Kerror_invalid_user_buffer;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_VERIFY_SWITCH))
  gcv_object_t _object_Kerror_invalid_verify_switch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WINDOW_HANDLE))
  gcv_object_t _object_Kerror_invalid_window_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WINDOW_STYLE))
  gcv_object_t _object_Kerror_invalid_window_style;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WORKSTATION))
  gcv_object_t _object_Kerror_invalid_workstation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IOPL_NOT_ENABLED))
  gcv_object_t _object_Kerror_iopl_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_DEVICE))
  gcv_object_t _object_Kerror_io_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_INCOMPLETE))
  gcv_object_t _object_Kerror_io_incomplete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_PENDING))
  gcv_object_t _object_Kerror_io_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_default_mm_auth_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_default_mm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_default_qm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED))
  gcv_object_t _object_Kerror_ipsec_ike_add_update_key_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ATTRIB_FAIL))
  gcv_object_t _object_Kerror_ipsec_ike_attrib_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_AUTH_FAIL))
  gcv_object_t _object_Kerror_ipsec_ike_auth_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_BENIGN_REINIT))
  gcv_object_t _object_Kerror_ipsec_ike_benign_reinit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_CRL_FAILED))
  gcv_object_t _object_Kerror_ipsec_ike_crl_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DECRYPT))
  gcv_object_t _object_Kerror_ipsec_ike_decrypt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DH_FAIL))
  gcv_object_t _object_Kerror_ipsec_ike_dh_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DH_FAILURE))
  gcv_object_t _object_Kerror_ipsec_ike_dh_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DROP_NO_RESPONSE))
  gcv_object_t _object_Kerror_ipsec_ike_drop_no_response;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ENCRYPT))
  gcv_object_t _object_Kerror_ipsec_ike_encrypt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ERROR))
  gcv_object_t _object_Kerror_ipsec_ike_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_FAILQUERYSSP))
  gcv_object_t _object_Kerror_ipsec_ike_failqueryssp;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_FAILSSPINIT))
  gcv_object_t _object_Kerror_ipsec_ike_failsspinit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR))
  gcv_object_t _object_Kerror_ipsec_ike_general_processing_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_GETSPIFAIL))
  gcv_object_t _object_Kerror_ipsec_ike_getspifail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_AUTH_ALG))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_auth_alg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_cert_keylen;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_CERT_TYPE))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_cert_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_COOKIE))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_cookie;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_encrypt_alg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_FILTER))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_filter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_GROUP))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_hash;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH_ALG))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_hash_alg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH_SIZE))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_hash_size;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HEADER))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_header;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_KEY_USAGE))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_key_usage;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_PAYLOAD))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_payload;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_POLICY))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_responder_lifetime_notify;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SIG))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_sig;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SIGNATURE))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_signature;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SITUATION))
  gcv_object_t _object_Kerror_ipsec_ike_invalid_situation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_KERBEROS_ERROR))
  gcv_object_t _object_Kerror_ipsec_ike_kerberos_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_LOAD_FAILED))
  gcv_object_t _object_Kerror_ipsec_ike_load_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_LOAD_SOFT_SA))
  gcv_object_t _object_Kerror_ipsec_ike_load_soft_sa;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_ACQUIRE_DROP))
  gcv_object_t _object_Kerror_ipsec_ike_mm_acquire_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_DELAY_DROP))
  gcv_object_t _object_Kerror_ipsec_ike_mm_delay_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_LIMIT))
  gcv_object_t _object_Kerror_ipsec_ike_mm_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEGOTIATION_DISABLED))
  gcv_object_t _object_Kerror_ipsec_ike_negotiation_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEGOTIATION_PENDING))
  gcv_object_t _object_Kerror_ipsec_ike_negotiation_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEG_STATUS_END))
  gcv_object_t _object_Kerror_ipsec_ike_neg_status_end;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NOTCBPRIV))
  gcv_object_t _object_Kerror_ipsec_ike_notcbpriv;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_CERT))
  gcv_object_t _object_Kerror_ipsec_ike_no_cert;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_MM_POLICY))
  gcv_object_t _object_Kerror_ipsec_ike_no_mm_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PEER_CERT))
  gcv_object_t _object_Kerror_ipsec_ike_no_peer_cert;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_POLICY))
  gcv_object_t _object_Kerror_ipsec_ike_no_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PRIVATE_KEY))
  gcv_object_t _object_Kerror_ipsec_ike_no_private_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PUBLIC_KEY))
  gcv_object_t _object_Kerror_ipsec_ike_no_public_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_OUT_OF_MEMORY))
  gcv_object_t _object_Kerror_ipsec_ike_out_of_memory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PEER_CRL_FAILED))
  gcv_object_t _object_Kerror_ipsec_ike_peer_crl_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_POLICY_CHANGE))
  gcv_object_t _object_Kerror_ipsec_ike_policy_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_POLICY_MATCH))
  gcv_object_t _object_Kerror_ipsec_ike_policy_match;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR))
  gcv_object_t _object_Kerror_ipsec_ike_process_err;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_cert;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_cert_req;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_DELETE))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_HASH))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_hash;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_ID))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_KE))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_ke;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_NONCE))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_nonce;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_notify;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_PROP))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_prop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_SA))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_sa;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_SIG))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_sig;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_TRANS))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_trans;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR))
  gcv_object_t _object_Kerror_ipsec_ike_process_err_vendor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QM_ACQUIRE_DROP))
  gcv_object_t _object_Kerror_ipsec_ike_qm_acquire_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QM_DELAY_DROP))
  gcv_object_t _object_Kerror_ipsec_ike_qm_delay_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QUEUE_DROP_MM))
  gcv_object_t _object_Kerror_ipsec_ike_queue_drop_mm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM))
  gcv_object_t _object_Kerror_ipsec_ike_queue_drop_no_mm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_RPC_DELETE))
  gcv_object_t _object_Kerror_ipsec_ike_rpc_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SA_DELETED))
  gcv_object_t _object_Kerror_ipsec_ike_sa_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SA_REAPED))
  gcv_object_t _object_Kerror_ipsec_ike_sa_reaped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SECLOADFAIL))
  gcv_object_t _object_Kerror_ipsec_ike_secloadfail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN))
  gcv_object_t _object_Kerror_ipsec_ike_soft_sa_torn_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SRVACQFAIL))
  gcv_object_t _object_Kerror_ipsec_ike_srvacqfail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SRVQUERYCRED))
  gcv_object_t _object_Kerror_ipsec_ike_srvquerycred;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_TIMED_OUT))
  gcv_object_t _object_Kerror_ipsec_ike_timed_out;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_UNKNOWN_DOI))
  gcv_object_t _object_Kerror_ipsec_ike_unknown_doi;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_UNSUPPORTED_ID))
  gcv_object_t _object_Kerror_ipsec_ike_unsupported_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_EXISTS))
  gcv_object_t _object_Kerror_ipsec_mm_auth_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_IN_USE))
  gcv_object_t _object_Kerror_ipsec_mm_auth_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_mm_auth_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_PENDING_DELETION))
  gcv_object_t _object_Kerror_ipsec_mm_auth_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_EXISTS))
  gcv_object_t _object_Kerror_ipsec_mm_filter_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_mm_filter_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_PENDING_DELETION))
  gcv_object_t _object_Kerror_ipsec_mm_filter_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_EXISTS))
  gcv_object_t _object_Kerror_ipsec_mm_policy_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_IN_USE))
  gcv_object_t _object_Kerror_ipsec_mm_policy_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_mm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_PENDING_DELETION))
  gcv_object_t _object_Kerror_ipsec_mm_policy_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_EXISTS))
  gcv_object_t _object_Kerror_ipsec_qm_policy_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_IN_USE))
  gcv_object_t _object_Kerror_ipsec_qm_policy_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_qm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_PENDING_DELETION))
  gcv_object_t _object_Kerror_ipsec_qm_policy_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_EXISTS))
  gcv_object_t _object_Kerror_ipsec_transport_filter_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_transport_filter_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION))
  gcv_object_t _object_Kerror_ipsec_transport_filter_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_EXISTS))
  gcv_object_t _object_Kerror_ipsec_tunnel_filter_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND))
  gcv_object_t _object_Kerror_ipsec_tunnel_filter_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION))
  gcv_object_t _object_Kerror_ipsec_tunnel_filter_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IRQ_BUSY))
  gcv_object_t _object_Kerror_irq_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOINED))
  gcv_object_t _object_Kerror_is_joined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOIN_PATH))
  gcv_object_t _object_Kerror_is_join_path;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOIN_TARGET))
  gcv_object_t _object_Kerror_is_join_target;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBSTED))
  gcv_object_t _object_Kerror_is_substed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBST_PATH))
  gcv_object_t _object_Kerror_is_subst_path;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBST_TARGET))
  gcv_object_t _object_Kerror_is_subst_target;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ITERATED_DATA_EXCEEDS_64k))
  gcv_object_t _object_Kerror_iterated_data_exceeds_64k;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOIN_TO_JOIN))
  gcv_object_t _object_Kerror_join_to_join;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOIN_TO_SUBST))
  gcv_object_t _object_Kerror_join_to_subst;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_DELETE_IN_PROGRESS))
  gcv_object_t _object_Kerror_journal_delete_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_ENTRY_DELETED))
  gcv_object_t _object_Kerror_journal_entry_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_HOOK_SET))
  gcv_object_t _object_Kerror_journal_hook_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_NOT_ACTIVE))
  gcv_object_t _object_Kerror_journal_not_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KEY_DELETED))
  gcv_object_t _object_Kerror_key_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KEY_HAS_CHILDREN))
  gcv_object_t _object_Kerror_key_has_children;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KM_DRIVER_BLOCKED))
  gcv_object_t _object_Kerror_km_driver_blocked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LABEL_TOO_LONG))
  gcv_object_t _object_Kerror_label_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LAST_ADMIN))
  gcv_object_t _object_Kerror_last_admin;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LB_WITHOUT_TABSTOPS))
  gcv_object_t _object_Kerror_lb_without_tabstops;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LIBRARY_FULL))
  gcv_object_t _object_Kerror_library_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LIBRARY_OFFLINE))
  gcv_object_t _object_Kerror_library_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LICENSE_QUOTA_EXCEEDED))
  gcv_object_t _object_Kerror_license_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LISTBOX_ID_NOT_FOUND))
  gcv_object_t _object_Kerror_listbox_id_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LM_CROSS_ENCRYPTION_REQUIRED))
  gcv_object_t _object_Kerror_lm_cross_encryption_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCAL_USER_SESSION_KEY))
  gcv_object_t _object_Kerror_local_user_session_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCKED))
  gcv_object_t _object_Kerror_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCK_FAILED))
  gcv_object_t _object_Kerror_lock_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCK_VIOLATION))
  gcv_object_t _object_Kerror_lock_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGIN_TIME_RESTRICTION))
  gcv_object_t _object_Kerror_login_time_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGIN_WKSTA_RESTRICTION))
  gcv_object_t _object_Kerror_login_wksta_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_FAILURE))
  gcv_object_t _object_Kerror_logon_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_NOT_GRANTED))
  gcv_object_t _object_Kerror_logon_not_granted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_SESSION_COLLISION))
  gcv_object_t _object_Kerror_logon_session_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_SESSION_EXISTS))
  gcv_object_t _object_Kerror_logon_session_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_TYPE_NOT_GRANTED))
  gcv_object_t _object_Kerror_logon_type_not_granted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOG_FILE_FULL))
  gcv_object_t _object_Kerror_log_file_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LUIDS_EXHAUSTED))
  gcv_object_t _object_Kerror_luids_exhausted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MACHINE_LOCKED))
  gcv_object_t _object_Kerror_machine_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAGAZINE_NOT_PRESENT))
  gcv_object_t _object_Kerror_magazine_not_present;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAPPED_ALIGNMENT))
  gcv_object_t _object_Kerror_mapped_alignment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAX_THRDS_REACHED))
  gcv_object_t _object_Kerror_max_thrds_reached;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_CHANGED))
  gcv_object_t _object_Kerror_media_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_INCOMPATIBLE))
  gcv_object_t _object_Kerror_media_incompatible;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_media_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_OFFLINE))
  gcv_object_t _object_Kerror_media_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_UNAVAILABLE))
  gcv_object_t _object_Kerror_media_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIUM_NOT_ACCESSIBLE))
  gcv_object_t _object_Kerror_medium_not_accessible;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBERS_PRIMARY_GROUP))
  gcv_object_t _object_Kerror_members_primary_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_IN_ALIAS))
  gcv_object_t _object_Kerror_member_in_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_IN_GROUP))
  gcv_object_t _object_Kerror_member_in_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_NOT_IN_ALIAS))
  gcv_object_t _object_Kerror_member_not_in_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_NOT_IN_GROUP))
  gcv_object_t _object_Kerror_member_not_in_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MENU_ITEM_NOT_FOUND))
  gcv_object_t _object_Kerror_menu_item_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MESSAGE_EXCEEDS_MAX_SIZE))
  gcv_object_t _object_Kerror_message_exceeds_max_size;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MESSAGE_SYNC_ONLY))
  gcv_object_t _object_Kerror_message_sync_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_METAFILE_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_metafile_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_META_EXPANSION_TOO_LONG))
  gcv_object_t _object_Kerror_meta_expansion_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MOD_NOT_FOUND))
  gcv_object_t _object_Kerror_mod_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MORE_DATA))
  gcv_object_t _object_Kerror_more_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MORE_WRITES))
  gcv_object_t _object_Kerror_more_writes;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MR_MID_NOT_FOUND))
  gcv_object_t _object_Kerror_mr_mid_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MUTUAL_AUTH_FAILED))
  gcv_object_t _object_Kerror_mutual_auth_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NEGATIVE_SEEK))
  gcv_object_t _object_Kerror_negative_seek;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NESTING_NOT_ALLOWED))
  gcv_object_t _object_Kerror_nesting_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETLOGON_NOT_STARTED))
  gcv_object_t _object_Kerror_netlogon_not_started;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETNAME_DELETED))
  gcv_object_t _object_Kerror_netname_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_ACCESS_DENIED))
  gcv_object_t _object_Kerror_network_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_BUSY))
  gcv_object_t _object_Kerror_network_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_network_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_UNREACHABLE))
  gcv_object_t _object_Kerror_network_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NET_WRITE_FAULT))
  gcv_object_t _object_Kerror_net_write_fault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOACCESS))
  gcv_object_t _object_Kerror_noaccess;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_CANNOT_BE_CLUSTERED))
  gcv_object_t _object_Kerror_node_cannot_be_clustered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_CANT_HOST_RESOURCE))
  gcv_object_t _object_Kerror_node_cant_host_resource;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_node_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT))
  gcv_object_t _object_Kerror_nologon_interdomain_trust_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_SERVER_TRUST_ACCOUNT))
  gcv_object_t _object_Kerror_nologon_server_trust_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT))
  gcv_object_t _object_Kerror_nologon_workstation_trust_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NONE_MAPPED))
  gcv_object_t _object_Kerror_none_mapped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NONPAGED_SYSTEM_RESOURCES))
  gcv_object_t _object_Kerror_nonpaged_system_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_ACCOUNT_SID))
  gcv_object_t _object_Kerror_non_account_sid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_DOMAIN_SID))
  gcv_object_t _object_Kerror_non_domain_sid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_MDICHILD_WINDOW))
  gcv_object_t _object_Kerror_non_mdichild_window;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOTIFY_ENUM_DIR))
  gcv_object_t _object_Kerror_notify_enum_dir;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ALL_ASSIGNED))
  gcv_object_t _object_Kerror_not_all_assigned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_AUTHENTICATED))
  gcv_object_t _object_Kerror_not_authenticated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_A_REPARSE_POINT))
  gcv_object_t _object_Kerror_not_a_reparse_point;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CHILD_WINDOW))
  gcv_object_t _object_Kerror_not_child_window;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CONNECTED))
  gcv_object_t _object_Kerror_not_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CONTAINER))
  gcv_object_t _object_Kerror_not_container;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_DOS_DISK))
  gcv_object_t _object_Kerror_not_dos_disk;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_EMPTY))
  gcv_object_t _object_Kerror_not_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_MEMORY))
  gcv_object_t _object_Kerror_not_enough_memory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_QUOTA))
  gcv_object_t _object_Kerror_not_enough_quota;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_SERVER_MEMORY))
  gcv_object_t _object_Kerror_not_enough_server_memory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_EXPORT_FORMAT))
  gcv_object_t _object_Kerror_not_export_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_FOUND))
  gcv_object_t _object_Kerror_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_JOINED))
  gcv_object_t _object_Kerror_not_joined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOCKED))
  gcv_object_t _object_Kerror_not_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOGGED_ON))
  gcv_object_t _object_Kerror_not_logged_on;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOGON_PROCESS))
  gcv_object_t _object_Kerror_not_logon_process;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_OWNER))
  gcv_object_t _object_Kerror_not_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_QUORUM_CAPABLE))
  gcv_object_t _object_Kerror_not_quorum_capable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_QUORUM_CLASS))
  gcv_object_t _object_Kerror_not_quorum_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_READY))
  gcv_object_t _object_Kerror_not_ready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_REGISTRY_FILE))
  gcv_object_t _object_Kerror_not_registry_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SAFEBOOT_SERVICE))
  gcv_object_t _object_Kerror_not_safeboot_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SAME_DEVICE))
  gcv_object_t _object_Kerror_not_same_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUBSTED))
  gcv_object_t _object_Kerror_not_substed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED_ON_SBS))
  gcv_object_t _object_Kerror_not_supported_on_sbs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER))
  gcv_object_t _object_Kerror_not_supported_on_standard_server;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_ASSOCIATION))
  gcv_object_t _object_Kerror_no_association;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_BROWSER_SERVERS_FOUND))
  gcv_object_t _object_Kerror_no_browser_servers_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_DATA))
  gcv_object_t _object_Kerror_no_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_DATA_DETECTED))
  gcv_object_t _object_Kerror_no_data_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_EFS))
  gcv_object_t _object_Kerror_no_efs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_IMPERSONATION_TOKEN))
  gcv_object_t _object_Kerror_no_impersonation_token;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_INHERITANCE))
  gcv_object_t _object_Kerror_no_inheritance;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_LOGON_SERVERS))
  gcv_object_t _object_Kerror_no_logon_servers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_LOG_SPACE))
  gcv_object_t _object_Kerror_no_log_space;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MATCH))
  gcv_object_t _object_Kerror_no_match;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MEDIA_IN_DRIVE))
  gcv_object_t _object_Kerror_no_media_in_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_DEVICES))
  gcv_object_t _object_Kerror_no_more_devices;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_FILES))
  gcv_object_t _object_Kerror_no_more_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_ITEMS))
  gcv_object_t _object_Kerror_no_more_items;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_SEARCH_HANDLES))
  gcv_object_t _object_Kerror_no_more_search_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_USER_HANDLES))
  gcv_object_t _object_Kerror_no_more_user_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_NETWORK))
  gcv_object_t _object_Kerror_no_network;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_NET_OR_BAD_PATH))
  gcv_object_t _object_Kerror_no_net_or_bad_path;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_PROC_SLOTS))
  gcv_object_t _object_Kerror_no_proc_slots;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_PROMOTION_ACTIVE))
  gcv_object_t _object_Kerror_no_promotion_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_QUOTAS_FOR_ACCOUNT))
  gcv_object_t _object_Kerror_no_quotas_for_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_RECOVERY_POLICY))
  gcv_object_t _object_Kerror_no_recovery_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_RECOVERY_PROGRAM))
  gcv_object_t _object_Kerror_no_recovery_program;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SCROLLBARS))
  gcv_object_t _object_Kerror_no_scrollbars;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SECURITY_ON_OBJECT))
  gcv_object_t _object_Kerror_no_security_on_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SHUTDOWN_IN_PROGRESS))
  gcv_object_t _object_Kerror_no_shutdown_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SIGNAL_SENT))
  gcv_object_t _object_Kerror_no_signal_sent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SITENAME))
  gcv_object_t _object_Kerror_no_sitename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SPOOL_SPACE))
  gcv_object_t _object_Kerror_no_spool_space;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_ALIAS))
  gcv_object_t _object_Kerror_no_such_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_DOMAIN))
  gcv_object_t _object_Kerror_no_such_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_GROUP))
  gcv_object_t _object_Kerror_no_such_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_LOGON_SESSION))
  gcv_object_t _object_Kerror_no_such_logon_session;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_MEMBER))
  gcv_object_t _object_Kerror_no_such_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_PACKAGE))
  gcv_object_t _object_Kerror_no_such_package;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_PRIVILEGE))
  gcv_object_t _object_Kerror_no_such_privilege;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_SITE))
  gcv_object_t _object_Kerror_no_such_site;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_USER))
  gcv_object_t _object_Kerror_no_such_user;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUPPORTING_DRIVES))
  gcv_object_t _object_Kerror_no_supporting_drives;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SYSTEM_MENU))
  gcv_object_t _object_Kerror_no_system_menu;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SYSTEM_RESOURCES))
  gcv_object_t _object_Kerror_no_system_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TOKEN))
  gcv_object_t _object_Kerror_no_token;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRACKING_SERVICE))
  gcv_object_t _object_Kerror_no_tracking_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRUST_LSA_SECRET))
  gcv_object_t _object_Kerror_no_trust_lsa_secret;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRUST_SAM_ACCOUNT))
  gcv_object_t _object_Kerror_no_trust_sam_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_UNICODE_TRANSLATION))
  gcv_object_t _object_Kerror_no_unicode_translation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_USER_KEYS))
  gcv_object_t _object_Kerror_no_user_keys;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_USER_SESSION_KEY))
  gcv_object_t _object_Kerror_no_user_session_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_VOLUME_ID))
  gcv_object_t _object_Kerror_no_volume_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_VOLUME_LABEL))
  gcv_object_t _object_Kerror_no_volume_label;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_WILDCARD_CHARACTERS))
  gcv_object_t _object_Kerror_no_wildcard_characters;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NT_CROSS_ENCRYPTION_REQUIRED))
  gcv_object_t _object_Kerror_nt_cross_encryption_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NULL_LM_PASSWORD))
  gcv_object_t _object_Kerror_null_lm_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_object_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_IN_LIST))
  gcv_object_t _object_Kerror_object_in_list;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_NOT_FOUND))
  gcv_object_t _object_Kerror_object_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OLD_WIN_VERSION))
  gcv_object_t _object_Kerror_old_win_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ONLY_IF_CONNECTED))
  gcv_object_t _object_Kerror_only_if_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPEN_FAILED))
  gcv_object_t _object_Kerror_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPEN_FILES))
  gcv_object_t _object_Kerror_open_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPERATION_ABORTED))
  gcv_object_t _object_Kerror_operation_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPLOCK_NOT_GRANTED))
  gcv_object_t _object_Kerror_oplock_not_granted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUTOFMEMORY))
  gcv_object_t _object_Kerror_outofmemory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUT_OF_PAPER))
  gcv_object_t _object_Kerror_out_of_paper;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUT_OF_STRUCTURES))
  gcv_object_t _object_Kerror_out_of_structures;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OVERRIDE_NOCHANGES))
  gcv_object_t _object_Kerror_override_nochanges;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PAGED_SYSTEM_RESOURCES))
  gcv_object_t _object_Kerror_paged_system_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PAGEFILE_QUOTA))
  gcv_object_t _object_Kerror_pagefile_quota;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARAMETER_QUOTA_EXCEEDED))
  gcv_object_t _object_Kerror_parameter_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARTIAL_COPY))
  gcv_object_t _object_Kerror_partial_copy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARTITION_FAILURE))
  gcv_object_t _object_Kerror_partition_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_EXPIRED))
  gcv_object_t _object_Kerror_password_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_MUST_CHANGE))
  gcv_object_t _object_Kerror_password_must_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_RESTRICTION))
  gcv_object_t _object_Kerror_password_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_INVALID))
  gcv_object_t _object_Kerror_patch_package_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_OPEN_FAILED))
  gcv_object_t _object_Kerror_patch_package_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_REJECTED))
  gcv_object_t _object_Kerror_patch_package_rejected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_UNSUPPORTED))
  gcv_object_t _object_Kerror_patch_package_unsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_TARGET_NOT_FOUND))
  gcv_object_t _object_Kerror_patch_target_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATH_BUSY))
  gcv_object_t _object_Kerror_path_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATH_NOT_FOUND))
  gcv_object_t _object_Kerror_path_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PER_USER_TRUST_QUOTA_EXCEEDED))
  gcv_object_t _object_Kerror_per_user_trust_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_BUSY))
  gcv_object_t _object_Kerror_pipe_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_CONNECTED))
  gcv_object_t _object_Kerror_pipe_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_LISTENING))
  gcv_object_t _object_Kerror_pipe_listening;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_NOT_CONNECTED))
  gcv_object_t _object_Kerror_pipe_not_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PKINIT_FAILURE))
  gcv_object_t _object_Kerror_pkinit_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POINT_NOT_FOUND))
  gcv_object_t _object_Kerror_point_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POLICY_OBJECT_NOT_FOUND))
  gcv_object_t _object_Kerror_policy_object_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POLICY_ONLY_IN_DS))
  gcv_object_t _object_Kerror_policy_only_in_ds;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POPUP_ALREADY_ACTIVE))
  gcv_object_t _object_Kerror_popup_already_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PORT_UNREACHABLE))
  gcv_object_t _object_Kerror_port_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POSSIBLE_DEADLOCK))
  gcv_object_t _object_Kerror_possible_deadlock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POTENTIAL_FILE_FOUND))
  gcv_object_t _object_Kerror_potential_file_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_ALREADY_EXISTS))
  gcv_object_t _object_Kerror_printer_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DELETED))
  gcv_object_t _object_Kerror_printer_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_ALREADY_INSTALLED))
  gcv_object_t _object_Kerror_printer_driver_already_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_BLOCKED))
  gcv_object_t _object_Kerror_printer_driver_blocked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_IN_USE))
  gcv_object_t _object_Kerror_printer_driver_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_WARNED))
  gcv_object_t _object_Kerror_printer_driver_warned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_HAS_JOBS_QUEUED))
  gcv_object_t _object_Kerror_printer_has_jobs_queued;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_NOT_FOUND))
  gcv_object_t _object_Kerror_printer_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTQ_FULL))
  gcv_object_t _object_Kerror_printq_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_CANCELLED))
  gcv_object_t _object_Kerror_print_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_MONITOR_ALREADY_INSTALLED))
  gcv_object_t _object_Kerror_print_monitor_already_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_MONITOR_IN_USE))
  gcv_object_t _object_Kerror_print_monitor_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED))
  gcv_object_t _object_Kerror_print_processor_already_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRIVATE_DIALOG_INDEX))
  gcv_object_t _object_Kerror_private_dialog_index;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRIVILEGE_NOT_HELD))
  gcv_object_t _object_Kerror_privilege_not_held;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROCESS_ABORTED))
  gcv_object_t _object_Kerror_process_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROC_NOT_FOUND))
  gcv_object_t _object_Kerror_proc_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRODUCT_UNINSTALLED))
  gcv_object_t _object_Kerror_product_uninstalled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRODUCT_VERSION))
  gcv_object_t _object_Kerror_product_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE))
  gcv_object_t _object_Kerror_profile_not_associated_with_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROFILE_NOT_FOUND))
  gcv_object_t _object_Kerror_profile_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROMOTION_ACTIVE))
  gcv_object_t _object_Kerror_promotion_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROTOCOL_UNREACHABLE))
  gcv_object_t _object_Kerror_protocol_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUMLOG_OPEN_FAILED))
  gcv_object_t _object_Kerror_quorumlog_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_DISK_NOT_FOUND))
  gcv_object_t _object_Kerror_quorum_disk_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_OWNER_ALIVE))
  gcv_object_t _object_Kerror_quorum_owner_alive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_RESOURCE))
  gcv_object_t _object_Kerror_quorum_resource;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_RESOURCE_ONLINE_FAILED))
  gcv_object_t _object_Kerror_quorum_resource_online_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_READ_FAULT))
  gcv_object_t _object_Kerror_read_fault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RECOVERY_FAILURE))
  gcv_object_t _object_Kerror_recovery_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REC_NON_EXISTENT))
  gcv_object_t _object_Kerror_rec_non_existent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REDIRECTOR_HAS_OPEN_HANDLES))
  gcv_object_t _object_Kerror_redirector_has_open_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REDIR_PAUSED))
  gcv_object_t _object_Kerror_redir_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_CORRUPT))
  gcv_object_t _object_Kerror_registry_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_IO_FAILED))
  gcv_object_t _object_Kerror_registry_io_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_RECOVERED))
  gcv_object_t _object_Kerror_registry_recovered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REG_NAT_CONSUMPTION))
  gcv_object_t _object_Kerror_reg_nat_consumption;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RELOC_CHAIN_XEEDS_SEGLIM))
  gcv_object_t _object_Kerror_reloc_chain_xeeds_seglim;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_SESSION_LIMIT_EXCEEDED))
  gcv_object_t _object_Kerror_remote_session_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_STORAGE_MEDIA_ERROR))
  gcv_object_t _object_Kerror_remote_storage_media_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_STORAGE_NOT_ACTIVE))
  gcv_object_t _object_Kerror_remote_storage_not_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REM_NOT_LIST))
  gcv_object_t _object_Kerror_rem_not_list;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_ATTRIBUTE_CONFLICT))
  gcv_object_t _object_Kerror_reparse_attribute_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_TAG_INVALID))
  gcv_object_t _object_Kerror_reparse_tag_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_TAG_MISMATCH))
  gcv_object_t _object_Kerror_reparse_tag_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUEST_ABORTED))
  gcv_object_t _object_Kerror_request_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUEST_REFUSED))
  gcv_object_t _object_Kerror_request_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION))
  gcv_object_t _object_Kerror_requires_interactive_windowstation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQ_NOT_ACCEP))
  gcv_object_t _object_Kerror_req_not_accep;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_CREATE_FAILED))
  gcv_object_t _object_Kerror_resmon_create_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_INVALID_STATE))
  gcv_object_t _object_Kerror_resmon_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_ONLINE_FAILED))
  gcv_object_t _object_Kerror_resmon_online_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_DATA_NOT_FOUND))
  gcv_object_t _object_Kerror_resource_data_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_DISABLED))
  gcv_object_t _object_Kerror_resource_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_FAILED))
  gcv_object_t _object_Kerror_resource_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_LANG_NOT_FOUND))
  gcv_object_t _object_Kerror_resource_lang_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NAME_NOT_FOUND))
  gcv_object_t _object_Kerror_resource_name_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_AVAILABLE))
  gcv_object_t _object_Kerror_resource_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_FOUND))
  gcv_object_t _object_Kerror_resource_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_ONLINE))
  gcv_object_t _object_Kerror_resource_not_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_PRESENT))
  gcv_object_t _object_Kerror_resource_not_present;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_ONLINE))
  gcv_object_t _object_Kerror_resource_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_PROPERTIES_STORED))
  gcv_object_t _object_Kerror_resource_properties_stored;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_PROPERTY_UNCHANGEABLE))
  gcv_object_t _object_Kerror_resource_property_unchangeable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_TYPE_NOT_FOUND))
  gcv_object_t _object_Kerror_resource_type_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RETRY))
  gcv_object_t _object_Kerror_retry;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REVISION_MISMATCH))
  gcv_object_t _object_Kerror_revision_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RING2SEG_MUST_BE_MOVABLE))
  gcv_object_t _object_Kerror_ring2seg_must_be_movable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RING2_STACK_IN_USE))
  gcv_object_t _object_Kerror_ring2_stack_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RMODE_APP))
  gcv_object_t _object_Kerror_rmode_app;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RPL_NOT_ALLOWED))
  gcv_object_t _object_Kerror_rpl_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RXACT_COMMIT_FAILURE))
  gcv_object_t _object_Kerror_rxact_commit_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RXACT_INVALID_STATE))
  gcv_object_t _object_Kerror_rxact_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SAME_DRIVE))
  gcv_object_t _object_Kerror_same_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SAM_INIT_FAILURE))
  gcv_object_t _object_Kerror_sam_init_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SCOPE_NOT_FOUND))
  gcv_object_t _object_Kerror_scope_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SCREEN_ALREADY_LOCKED))
  gcv_object_t _object_Kerror_screen_already_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SECRET_TOO_LONG))
  gcv_object_t _object_Kerror_secret_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SECTOR_NOT_FOUND))
  gcv_object_t _object_Kerror_sector_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEEK))
  gcv_object_t _object_Kerror_seek;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEEK_ON_DEVICE))
  gcv_object_t _object_Kerror_seek_on_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_IS_SET))
  gcv_object_t _object_Kerror_sem_is_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_NOT_FOUND))
  gcv_object_t _object_Kerror_sem_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_OWNER_DIED))
  gcv_object_t _object_Kerror_sem_owner_died;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_TIMEOUT))
  gcv_object_t _object_Kerror_sem_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_USER_LIMIT))
  gcv_object_t _object_Kerror_sem_user_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERIAL_NO_DEVICE))
  gcv_object_t _object_Kerror_serial_no_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_DISABLED))
  gcv_object_t _object_Kerror_server_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_HAS_OPEN_HANDLES))
  gcv_object_t _object_Kerror_server_has_open_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_NOT_DISABLED))
  gcv_object_t _object_Kerror_server_not_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_SHUTDOWN_IN_PROGRESS))
  gcv_object_t _object_Kerror_server_shutdown_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_ALREADY_RUNNING))
  gcv_object_t _object_Kerror_service_already_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_CANNOT_ACCEPT_CTRL))
  gcv_object_t _object_Kerror_service_cannot_accept_ctrl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DATABASE_LOCKED))
  gcv_object_t _object_Kerror_service_database_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DEPENDENCY_DELETED))
  gcv_object_t _object_Kerror_service_dependency_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DEPENDENCY_FAIL))
  gcv_object_t _object_Kerror_service_dependency_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DISABLED))
  gcv_object_t _object_Kerror_service_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DOES_NOT_EXIST))
  gcv_object_t _object_Kerror_service_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_EXISTS))
  gcv_object_t _object_Kerror_service_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_LOGON_FAILED))
  gcv_object_t _object_Kerror_service_logon_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_MARKED_FOR_DELETE))
  gcv_object_t _object_Kerror_service_marked_for_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NEVER_STARTED))
  gcv_object_t _object_Kerror_service_never_started;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_ACTIVE))
  gcv_object_t _object_Kerror_service_not_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_FOUND))
  gcv_object_t _object_Kerror_service_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_IN_EXE))
  gcv_object_t _object_Kerror_service_not_in_exe;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NO_THREAD))
  gcv_object_t _object_Kerror_service_no_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_REQUEST_TIMEOUT))
  gcv_object_t _object_Kerror_service_request_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_SPECIFIC_ERROR))
  gcv_object_t _object_Kerror_service_specific_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_START_HANG))
  gcv_object_t _object_Kerror_service_start_hang;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SESSION_CREDENTIAL_CONFLICT))
  gcv_object_t _object_Kerror_session_credential_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SETCOUNT_ON_BAD_LB))
  gcv_object_t _object_Kerror_setcount_on_bad_lb;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SETMARK_DETECTED))
  gcv_object_t _object_Kerror_setmark_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_NOT_FOUND))
  gcv_object_t _object_Kerror_set_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_POWER_STATE_FAILED))
  gcv_object_t _object_Kerror_set_power_state_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_POWER_STATE_VETOED))
  gcv_object_t _object_Kerror_set_power_state_vetoed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARED_POLICY))
  gcv_object_t _object_Kerror_shared_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_BUFFER_EXCEEDED))
  gcv_object_t _object_Kerror_sharing_buffer_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_PAUSED))
  gcv_object_t _object_Kerror_sharing_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_VIOLATION))
  gcv_object_t _object_Kerror_sharing_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHUTDOWN_CLUSTER))
  gcv_object_t _object_Kerror_shutdown_cluster;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHUTDOWN_IN_PROGRESS))
  gcv_object_t _object_Kerror_shutdown_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SIGNAL_PENDING))
  gcv_object_t _object_Kerror_signal_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SIGNAL_REFUSED))
  gcv_object_t _object_Kerror_signal_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SINGLE_INSTANCE_APP))
  gcv_object_t _object_Kerror_single_instance_app;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SMARTCARD_SUBSYSTEM_FAILURE))
  gcv_object_t _object_Kerror_smartcard_subsystem_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SOME_NOT_MAPPED))
  gcv_object_t _object_Kerror_some_not_mapped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SOURCE_ELEMENT_EMPTY))
  gcv_object_t _object_Kerror_source_element_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_ACCOUNT))
  gcv_object_t _object_Kerror_special_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_GROUP))
  gcv_object_t _object_Kerror_special_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_USER))
  gcv_object_t _object_Kerror_special_user;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPL_NO_ADDJOB))
  gcv_object_t _object_Kerror_spl_no_addjob;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPL_NO_STARTDOC))
  gcv_object_t _object_Kerror_spl_no_startdoc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPOOL_FILE_NOT_FOUND))
  gcv_object_t _object_Kerror_spool_file_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STACK_BUFFER_OVERRUN))
  gcv_object_t _object_Kerror_stack_buffer_overrun;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STACK_OVERFLOW))
  gcv_object_t _object_Kerror_stack_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STATIC_INIT))
  gcv_object_t _object_Kerror_static_init;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUBST_TO_JOIN))
  gcv_object_t _object_Kerror_subst_to_join;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUBST_TO_SUBST))
  gcv_object_t _object_Kerror_subst_to_subst;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_REBOOT_INITIATED))
  gcv_object_t _object_Kerror_success_reboot_initiated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_REBOOT_REQUIRED))
  gcv_object_t _object_Kerror_success_reboot_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_RESTART_REQUIRED))
  gcv_object_t _object_Kerror_success_restart_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SWAPERROR))
  gcv_object_t _object_Kerror_swaperror;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ACTIVATION_CONTEXT_DISABLED))
  gcv_object_t _object_Kerror_sxs_activation_context_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ASSEMBLY_NOT_FOUND))
  gcv_object_t _object_Kerror_sxs_assembly_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_CANT_GEN_ACTCTX))
  gcv_object_t _object_Kerror_sxs_cant_gen_actctx;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_ASSEMBLY_NAME))
  gcv_object_t _object_Kerror_sxs_duplicate_assembly_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_CLSID))
  gcv_object_t _object_Kerror_sxs_duplicate_clsid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_DLL_NAME))
  gcv_object_t _object_Kerror_sxs_duplicate_dll_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_IID))
  gcv_object_t _object_Kerror_sxs_duplicate_iid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_PROGID))
  gcv_object_t _object_Kerror_sxs_duplicate_progid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_TLBID))
  gcv_object_t _object_Kerror_sxs_duplicate_tlbid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME))
  gcv_object_t _object_Kerror_sxs_duplicate_windowclass_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_FILE_HASH_MISMATCH))
  gcv_object_t _object_Kerror_sxs_file_hash_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ACTCTXDATA_FORMAT))
  gcv_object_t _object_Kerror_sxs_invalid_actctxdata_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE))
  gcv_object_t _object_Kerror_sxs_invalid_assembly_identity_attribute;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME))
  gcv_object_t _object_Kerror_sxs_invalid_assembly_identity_attribute_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_XML_NAMESPACE_URI))
  gcv_object_t _object_Kerror_sxs_invalid_xml_namespace_uri;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_KEY_NOT_FOUND))
  gcv_object_t _object_Kerror_sxs_key_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED))
  gcv_object_t _object_Kerror_sxs_leaf_manifest_dependency_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_FORMAT_ERROR))
  gcv_object_t _object_Kerror_sxs_manifest_format_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE))
  gcv_object_t _object_Kerror_sxs_manifest_invalid_required_default_namespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE))
  gcv_object_t _object_Kerror_sxs_manifest_missing_required_default_namespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_PARSE_ERROR))
  gcv_object_t _object_Kerror_sxs_manifest_parse_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE))
  gcv_object_t _object_Kerror_sxs_missing_assembly_identity_attribute;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_POLICY_PARSE_ERROR))
  gcv_object_t _object_Kerror_sxs_policy_parse_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT))
  gcv_object_t _object_Kerror_sxs_private_manifest_cross_path_with_reparse_point;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET))
  gcv_object_t _object_Kerror_sxs_process_default_already_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING))
  gcv_object_t _object_Kerror_sxs_protection_catalog_file_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_CATALOG_NOT_VALID))
  gcv_object_t _object_Kerror_sxs_protection_catalog_not_valid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT))
  gcv_object_t _object_Kerror_sxs_protection_public_key_too_short;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_RECOVERY_FAILED))
  gcv_object_t _object_Kerror_sxs_protection_recovery_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED))
  gcv_object_t _object_Kerror_sxs_root_manifest_dependency_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_SECTION_NOT_FOUND))
  gcv_object_t _object_Kerror_sxs_section_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_THREAD_QUERIES_DISABLED))
  gcv_object_t _object_Kerror_sxs_thread_queries_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNKNOWN_ENCODING))
  gcv_object_t _object_Kerror_sxs_unknown_encoding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNKNOWN_ENCODING_GROUP))
  gcv_object_t _object_Kerror_sxs_unknown_encoding_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNTRANSLATABLE_HRESULT))
  gcv_object_t _object_Kerror_sxs_untranslatable_hresult;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_VERSION_CONFLICT))
  gcv_object_t _object_Kerror_sxs_version_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_WRONG_SECTION_TYPE))
  gcv_object_t _object_Kerror_sxs_wrong_section_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADCHARDATA))
  gcv_object_t _object_Kerror_sxs_xml_e_badchardata;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADCHARINSTRING))
  gcv_object_t _object_Kerror_sxs_xml_e_badcharinstring;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADNAMECHAR))
  gcv_object_t _object_Kerror_sxs_xml_e_badnamechar;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADPEREFINSUBSET))
  gcv_object_t _object_Kerror_sxs_xml_e_badperefinsubset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADSTARTNAMECHAR))
  gcv_object_t _object_Kerror_sxs_xml_e_badstartnamechar;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADXMLCASE))
  gcv_object_t _object_Kerror_sxs_xml_e_badxmlcase;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADXMLDECL))
  gcv_object_t _object_Kerror_sxs_xml_e_badxmldecl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_COMMENTSYNTAX))
  gcv_object_t _object_Kerror_sxs_xml_e_commentsyntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_DUPLICATEATTRIBUTE))
  gcv_object_t _object_Kerror_sxs_xml_e_duplicateattribute;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE))
  gcv_object_t _object_Kerror_sxs_xml_e_expectingclosequote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_EXPECTINGTAGEND))
  gcv_object_t _object_Kerror_sxs_xml_e_expectingtagend;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INCOMPLETE_ENCODING))
  gcv_object_t _object_Kerror_sxs_xml_e_incomplete_encoding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INTERNALERROR))
  gcv_object_t _object_Kerror_sxs_xml_e_internalerror;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDATROOTLEVEL))
  gcv_object_t _object_Kerror_sxs_xml_e_invalidatrootlevel;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDENCODING))
  gcv_object_t _object_Kerror_sxs_xml_e_invalidencoding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDSWITCH))
  gcv_object_t _object_Kerror_sxs_xml_e_invalidswitch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_DECIMAL))
  gcv_object_t _object_Kerror_sxs_xml_e_invalid_decimal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_HEXIDECIMAL))
  gcv_object_t _object_Kerror_sxs_xml_e_invalid_hexidecimal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_STANDALONE))
  gcv_object_t _object_Kerror_sxs_xml_e_invalid_standalone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_UNICODE))
  gcv_object_t _object_Kerror_sxs_xml_e_invalid_unicode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_VERSION))
  gcv_object_t _object_Kerror_sxs_xml_e_invalid_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGEQUALS))
  gcv_object_t _object_Kerror_sxs_xml_e_missingequals;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGQUOTE))
  gcv_object_t _object_Kerror_sxs_xml_e_missingquote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGROOT))
  gcv_object_t _object_Kerror_sxs_xml_e_missingroot;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGSEMICOLON))
  gcv_object_t _object_Kerror_sxs_xml_e_missingsemicolon;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGWHITESPACE))
  gcv_object_t _object_Kerror_sxs_xml_e_missingwhitespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSING_PAREN))
  gcv_object_t _object_Kerror_sxs_xml_e_missing_paren;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MULTIPLEROOTS))
  gcv_object_t _object_Kerror_sxs_xml_e_multipleroots;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MULTIPLE_COLONS))
  gcv_object_t _object_Kerror_sxs_xml_e_multiple_colons;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_RESERVEDNAMESPACE))
  gcv_object_t _object_Kerror_sxs_xml_e_reservednamespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNBALANCEDPAREN))
  gcv_object_t _object_Kerror_sxs_xml_e_unbalancedparen;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDCDATA))
  gcv_object_t _object_Kerror_sxs_xml_e_unclosedcdata;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDCOMMENT))
  gcv_object_t _object_Kerror_sxs_xml_e_unclosedcomment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDDECL))
  gcv_object_t _object_Kerror_sxs_xml_e_uncloseddecl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDENDTAG))
  gcv_object_t _object_Kerror_sxs_xml_e_unclosedendtag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDSTARTTAG))
  gcv_object_t _object_Kerror_sxs_xml_e_unclosedstarttag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDSTRING))
  gcv_object_t _object_Kerror_sxs_xml_e_unclosedstring;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDTAG))
  gcv_object_t _object_Kerror_sxs_xml_e_unclosedtag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTEDENDTAG))
  gcv_object_t _object_Kerror_sxs_xml_e_unexpectedendtag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTEDEOF))
  gcv_object_t _object_Kerror_sxs_xml_e_unexpectedeof;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTED_STANDALONE))
  gcv_object_t _object_Kerror_sxs_xml_e_unexpected_standalone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE))
  gcv_object_t _object_Kerror_sxs_xml_e_unexpected_whitespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK))
  gcv_object_t _object_Kerror_sxs_xml_e_whitespaceorquestionmark;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_XMLDECLSYNTAX))
  gcv_object_t _object_Kerror_sxs_xml_e_xmldeclsyntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED))
  gcv_object_t _object_Kerror_sync_foreground_refresh_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SYSTEM_TRACE))
  gcv_object_t _object_Kerror_system_trace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TAG_NOT_FOUND))
  gcv_object_t _object_Kerror_tag_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TAG_NOT_PRESENT))
  gcv_object_t _object_Kerror_tag_not_present;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_THREAD_1_INACTIVE))
  gcv_object_t _object_Kerror_thread_1_inactive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TIMEOUT))
  gcv_object_t _object_Kerror_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TIME_SKEW))
  gcv_object_t _object_Kerror_time_skew;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TLW_WITH_WSCHILD))
  gcv_object_t _object_Kerror_tlw_with_wschild;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOKEN_ALREADY_IN_USE))
  gcv_object_t _object_Kerror_token_already_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_CMDS))
  gcv_object_t _object_Kerror_too_many_cmds;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_CONTEXT_IDS))
  gcv_object_t _object_Kerror_too_many_context_ids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_LINKS))
  gcv_object_t _object_Kerror_too_many_links;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_LUIDS_REQUESTED))
  gcv_object_t _object_Kerror_too_many_luids_requested;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_MODULES))
  gcv_object_t _object_Kerror_too_many_modules;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_MUXWAITERS))
  gcv_object_t _object_Kerror_too_many_muxwaiters;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_NAMES))
  gcv_object_t _object_Kerror_too_many_names;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_OPEN_FILES))
  gcv_object_t _object_Kerror_too_many_open_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_POSTS))
  gcv_object_t _object_Kerror_too_many_posts;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SECRETS))
  gcv_object_t _object_Kerror_too_many_secrets;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SEMAPHORES))
  gcv_object_t _object_Kerror_too_many_semaphores;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SEM_REQUESTS))
  gcv_object_t _object_Kerror_too_many_sem_requests;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SESS))
  gcv_object_t _object_Kerror_too_many_sess;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SIDS))
  gcv_object_t _object_Kerror_too_many_sids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_TCBS))
  gcv_object_t _object_Kerror_too_many_tcbs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRANSFORM_NOT_SUPPORTED))
  gcv_object_t _object_Kerror_transform_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRANSPORT_FULL))
  gcv_object_t _object_Kerror_transport_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUSTED_DOMAIN_FAILURE))
  gcv_object_t _object_Kerror_trusted_domain_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUSTED_RELATIONSHIP_FAILURE))
  gcv_object_t _object_Kerror_trusted_relationship_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUST_FAILURE))
  gcv_object_t _object_Kerror_trust_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_CLEAN))
  gcv_object_t _object_Kerror_unable_to_clean;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA))
  gcv_object_t _object_Kerror_unable_to_eject_mounted_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_DRIVE))
  gcv_object_t _object_Kerror_unable_to_inventory_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_SLOT))
  gcv_object_t _object_Kerror_unable_to_inventory_slot;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_TRANSPORT))
  gcv_object_t _object_Kerror_unable_to_inventory_transport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_LOAD_MEDIUM))
  gcv_object_t _object_Kerror_unable_to_load_medium;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_LOCK_MEDIA))
  gcv_object_t _object_Kerror_unable_to_lock_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT))
  gcv_object_t _object_Kerror_unable_to_move_replacement;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT_2))
  gcv_object_t _object_Kerror_unable_to_move_replacement_2;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_REMOVE_REPLACED))
  gcv_object_t _object_Kerror_unable_to_remove_replaced;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_UNLOAD_MEDIA))
  gcv_object_t _object_Kerror_unable_to_unload_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNEXPECTED_OMID))
  gcv_object_t _object_Kerror_unexpected_omid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNEXP_NET_ERR))
  gcv_object_t _object_Kerror_unexp_net_err;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_COMPONENT))
  gcv_object_t _object_Kerror_unknown_component;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_FEATURE))
  gcv_object_t _object_Kerror_unknown_feature;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PORT))
  gcv_object_t _object_Kerror_unknown_port;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINTER_DRIVER))
  gcv_object_t _object_Kerror_unknown_printer_driver;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINTPROCESSOR))
  gcv_object_t _object_Kerror_unknown_printprocessor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINT_MONITOR))
  gcv_object_t _object_Kerror_unknown_print_monitor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRODUCT))
  gcv_object_t _object_Kerror_unknown_product;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PROPERTY))
  gcv_object_t _object_Kerror_unknown_property;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_REVISION))
  gcv_object_t _object_Kerror_unknown_revision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNRECOGNIZED_MEDIA))
  gcv_object_t _object_Kerror_unrecognized_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNRECOGNIZED_VOLUME))
  gcv_object_t _object_Kerror_unrecognized_volume;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNSUPPORTED_TYPE))
  gcv_object_t _object_Kerror_unsupported_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED))
  gcv_object_t _object_Kerror_user_delete_trust_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_EXISTS))
  gcv_object_t _object_Kerror_user_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_MAPPED_FILE))
  gcv_object_t _object_Kerror_user_mapped_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VC_DISCONNECTED))
  gcv_object_t _object_Kerror_vc_disconnected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_CONTAINS_SYS_FILES))
  gcv_object_t _object_Kerror_volume_contains_sys_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_NOT_SIS_ENABLED))
  gcv_object_t _object_Kerror_volume_not_sis_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_NOT_SUPPORT_EFS))
  gcv_object_t _object_Kerror_volume_not_support_efs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WAIT_NO_CHILDREN))
  gcv_object_t _object_Kerror_wait_no_children;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_NOT_COMBOBOX))
  gcv_object_t _object_Kerror_window_not_combobox;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_NOT_DIALOG))
  gcv_object_t _object_Kerror_window_not_dialog;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_OF_OTHER_THREAD))
  gcv_object_t _object_Kerror_window_of_other_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINS_INTERNAL))
  gcv_object_t _object_Kerror_wins_internal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ALREADY_DISABLED))
  gcv_object_t _object_Kerror_wmi_already_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ALREADY_ENABLED))
  gcv_object_t _object_Kerror_wmi_already_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_DP_FAILED))
  gcv_object_t _object_Kerror_wmi_dp_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_DP_NOT_FOUND))
  gcv_object_t _object_Kerror_wmi_dp_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_GUID_DISCONNECTED))
  gcv_object_t _object_Kerror_wmi_guid_disconnected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_GUID_NOT_FOUND))
  gcv_object_t _object_Kerror_wmi_guid_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INSTANCE_NOT_FOUND))
  gcv_object_t _object_Kerror_wmi_instance_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INVALID_MOF))
  gcv_object_t _object_Kerror_wmi_invalid_mof;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INVALID_REGINFO))
  gcv_object_t _object_Kerror_wmi_invalid_reginfo;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ITEMID_NOT_FOUND))
  gcv_object_t _object_Kerror_wmi_itemid_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_READ_ONLY))
  gcv_object_t _object_Kerror_wmi_read_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_SERVER_UNAVAILABLE))
  gcv_object_t _object_Kerror_wmi_server_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_SET_FAILURE))
  gcv_object_t _object_Kerror_wmi_set_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_TRY_AGAIN))
  gcv_object_t _object_Kerror_wmi_try_again;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_UNRESOLVED_INSTANCE_REF))
  gcv_object_t _object_Kerror_wmi_unresolved_instance_ref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WORKING_SET_QUOTA))
  gcv_object_t _object_Kerror_working_set_quota;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRITE_FAULT))
  gcv_object_t _object_Kerror_write_fault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRITE_PROTECT))
  gcv_object_t _object_Kerror_write_protect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_DISK))
  gcv_object_t _object_Kerror_wrong_disk;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_EFS))
  gcv_object_t _object_Kerror_wrong_efs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_PASSWORD))
  gcv_object_t _object_Kerror_wrong_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_TARGET_NAME))
  gcv_object_t _object_Kerror_wrong_target_name;
#endif
#if defined(ESHUTDOWN)
  gcv_object_t _object_Keshutdown;
#endif
#if defined(ESOCKTNOSUPPORT)
  gcv_object_t _object_Kesocktnosupport;
#endif
#if defined(ESPIPE)
  gcv_object_t _object_Kespipe;
#endif
#if defined(ESRCH)
  gcv_object_t _object_Kesrch;
#endif
#if defined(ESRMNT)
  gcv_object_t _object_Kesrmnt;
#endif
#if defined(ESTALE)
  gcv_object_t _object_Kestale;
#endif
#if defined(ESTRPIPE)
  gcv_object_t _object_Kestrpipe;
#endif
#if defined(ETIME)
  gcv_object_t _object_Ketime;
#endif
#if defined(ETIMEDOUT)
  gcv_object_t _object_Ketimedout;
#endif
#if defined(ETOOMANYREFS)
  gcv_object_t _object_Ketoomanyrefs;
#endif
#if defined(ETXTBSY)
  gcv_object_t _object_Ketxtbsy;
#endif
#if defined(EUCLEAN)
  gcv_object_t _object_Keuclean;
#endif
#if defined(EUNATCH)
  gcv_object_t _object_Keunatch;
#endif
#if defined(EUSERS)
  gcv_object_t _object_Keusers;
#endif
#if defined(EVERSION)
  gcv_object_t _object_Keversion;
#endif
#if defined(EWOULDBLOCK)
  gcv_object_t _object_Kewouldblock;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_EXCL))
  gcv_object_t _object_Kexcl;
#endif
#if defined(EXDEV)
  gcv_object_t _object_Kexdev;
#endif
#if defined(EXFULL)
  gcv_object_t _object_Kexfull;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_Kexited;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_EXPR_NEST_MAX))
  gcv_object_t _object_Kexpr_nest_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kext;
#endif
  gcv_object_t _object_Kexternal_format;
#if (defined(HAVE_NFTW)) && (defined(FTW_F))
  gcv_object_t _object_Kf;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  gcv_object_t _object_Kfacility;
#endif
#if defined(S_IFBLK)
  gcv_object_t _object_Kfblk;
#endif
#if defined(S_IFCHR)
  gcv_object_t _object_Kfchr;
#endif
#if (defined(HAVE_FCNTL)) && (defined(F_GETFD))
  gcv_object_t _object_Kfd;
#endif
#if defined(S_IFDIR)
  gcv_object_t _object_Kfdir;
#endif
#if defined(HAVE_NFTW)
  gcv_object_t _object_Kfd_limit;
#endif
#if defined(S_IFIFO)
  gcv_object_t _object_Kfifo;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_FILESIZEBITS))
  gcv_object_t _object_Kfilesizebits;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kfiletime;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_NAMED_STREAMS))
  gcv_object_t _object_Kfile_named_streams;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_READ_ONLY_VOLUME))
  gcv_object_t _object_Kfile_read_only_volume;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_ENCRYPTION))
  gcv_object_t _object_Kfile_supports_encryption;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_OBJECT_IDS))
  gcv_object_t _object_Kfile_supports_object_ids;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_REPARSE_POINTS))
  gcv_object_t _object_Kfile_supports_reparse_points;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_SPARSE_FILES))
  gcv_object_t _object_Kfile_supports_sparse_files;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_VOLUME_QUOTAS))
  gcv_object_t _object_Kfile_volume_quotas;
#endif
#if (defined(HAVE_FCNTL)) && (defined(F_GETFL))
  gcv_object_t _object_Kfl;
#endif
#if defined(S_IFREG)
  gcv_object_t _object_Kfreg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_AUTHENTICATION))
  gcv_object_t _object_Kfrs_err_authentication;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_CHILD_TO_PARENT_COMM))
  gcv_object_t _object_Kfrs_err_child_to_parent_comm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INSUFFICIENT_PRIV))
  gcv_object_t _object_Kfrs_err_insufficient_priv;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INTERNAL))
  gcv_object_t _object_Kfrs_err_internal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INTERNAL_API))
  gcv_object_t _object_Kfrs_err_internal_api;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INVALID_API_SEQUENCE))
  gcv_object_t _object_Kfrs_err_invalid_api_sequence;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INVALID_SERVICE_PARAMETER))
  gcv_object_t _object_Kfrs_err_invalid_service_parameter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_AUTHENTICATION))
  gcv_object_t _object_Kfrs_err_parent_authentication;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_INSUFFICIENT_PRIV))
  gcv_object_t _object_Kfrs_err_parent_insufficient_priv;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_TO_CHILD_COMM))
  gcv_object_t _object_Kfrs_err_parent_to_child_comm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SERVICE_COMM))
  gcv_object_t _object_Kfrs_err_service_comm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_STARTING_SERVICE))
  gcv_object_t _object_Kfrs_err_starting_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_STOPPING_SERVICE))
  gcv_object_t _object_Kfrs_err_stopping_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_DEMOTE))
  gcv_object_t _object_Kfrs_err_sysvol_demote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_IS_BUSY))
  gcv_object_t _object_Kfrs_err_sysvol_is_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_POPULATE))
  gcv_object_t _object_Kfrs_err_sysvol_populate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_POPULATE_TIMEOUT))
  gcv_object_t _object_Kfrs_err_sysvol_populate_timeout;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_FSIZE))
  gcv_object_t _object_Kfsize;
#endif
#if defined(S_IFSOCK)
  gcv_object_t _object_Kfsock;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_FSYNC))
  gcv_object_t _object_Kfsync;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_CASE_IS_PRESERVED))
  gcv_object_t _object_Kfs_case_is_preserved;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_CASE_SENSITIVE))
  gcv_object_t _object_Kfs_case_sensitive;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_FILE_COMPRESSION))
  gcv_object_t _object_Kfs_file_compression;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_FILE_ENCRYPTION))
  gcv_object_t _object_Kfs_file_encryption;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_PERSISTENT_ACLS))
  gcv_object_t _object_Kfs_persistent_acls;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_UNICODE_STORED_ON_DISK))
  gcv_object_t _object_Kfs_unicode_stored_on_disk;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_VOL_IS_COMPRESSED))
  gcv_object_t _object_Kfs_vol_is_compressed;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_FTP))
  gcv_object_t _object_Kftp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_GETGR_R_SIZE_MAX))
  gcv_object_t _object_Kgetgr_r_size_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_GETPW_R_SIZE_MAX))
  gcv_object_t _object_Kgetpw_r_size_max;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  gcv_object_t _object_Kgid;
#endif
  gcv_object_t _object_Khardlink;
  gcv_object_t _object_Khardlink_or_copy;
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_HIDDEN))
  gcv_object_t _object_Khidden;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(HIGH_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  gcv_object_t _object_Khigh;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_HOST_NAME_MAX))
  gcv_object_t _object_Khost_name_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Khot_key;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ki1;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ki2;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ki4;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ki8;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_IA32_ON_WIN64))
  gcv_object_t _object_Kia32_on_win64;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_IA64))
  gcv_object_t _object_Kia64;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kicon;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(IDLE_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  gcv_object_t _object_Kidle;
#endif
  gcv_object_t _object_Kif_does_not_exist;
  gcv_object_t _object_Kif_exists;
#if (defined(HAVE_SYSLOG)) && (defined(LOG_INFO))
  gcv_object_t _object_Kinfo;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kinitid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(INIT_PROCESS))
  gcv_object_t _object_Kinit_process;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_INTEL))
  gcv_object_t _object_Kintel;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_IOV_MAX))
  gcv_object_t _object_Kiov_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_IPV6))
  gcv_object_t _object_Kipv6;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_JOB_CONTROL))
  gcv_object_t _object_Kjob_control;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_KERN))
  gcv_object_t _object_Kkern;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kkeywords;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_LARGEFILE))
  gcv_object_t _object_Klargefile;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Klastauthor;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Klastprinted;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Klastsave_dtm;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  gcv_object_t _object_Klength;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_LINE_MAX))
  gcv_object_t _object_Kline_max;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_LINK_MAX))
  gcv_object_t _object_Klink_max;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL0))
  gcv_object_t _object_Klocal0;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL1))
  gcv_object_t _object_Klocal1;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL2))
  gcv_object_t _object_Klocal2;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL3))
  gcv_object_t _object_Klocal3;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL4))
  gcv_object_t _object_Klocal4;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL5))
  gcv_object_t _object_Klocal5;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL6))
  gcv_object_t _object_Klocal6;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL7))
  gcv_object_t _object_Klocal7;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Klocale;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_LOCKS))
  gcv_object_t _object_Klocks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_LOGIN_NAME_MAX))
  gcv_object_t _object_Klogin_name_max;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(LOGIN_PROCESS))
  gcv_object_t _object_Klogin_process;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(LOW_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  gcv_object_t _object_Klow;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LPR))
  gcv_object_t _object_Klpr;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Klpstr;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Klpwstr;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_LWP))
  gcv_object_t _object_Klwp;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_MAIL))
  gcv_object_t _object_Kmail;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MAPPED_FILES))
  gcv_object_t _object_Kmapped_files;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kmax;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_MAX_CANON))
  gcv_object_t _object_Kmax_canon;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_MAX_INPUT))
  gcv_object_t _object_Kmax_input;
#endif
#if ((defined(HAVE_SYSCONF)) && (defined(_SC_MEMLOCK))) || ((defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_MEMLOCK)))
  gcv_object_t _object_Kmemlock;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MEMLOCK_RANGE))
  gcv_object_t _object_Kmemlock_range;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MEMORY_PROTECTION))
  gcv_object_t _object_Kmemory_protection;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MESSAGE_PASSING))
  gcv_object_t _object_Kmessage_passing;
#endif
  gcv_object_t _object_Kmethod;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kmin;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_MIPS))
  gcv_object_t _object_Kmips;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  gcv_object_t _object_Kmode;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MONOTONIC_CLOCK))
  gcv_object_t _object_Kmonotonic_clock;
#endif
#if defined(HAVE_NFTW)
  gcv_object_t _object_Kmount;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MQ_OPEN_MAX))
  gcv_object_t _object_Kmq_open_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MQ_PRIO_MAX))
  gcv_object_t _object_Kmq_prio_max;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_MSIL))
  gcv_object_t _object_Kmsil;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  gcv_object_t _object_Kmtime;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_NAME_MAX))
  gcv_object_t _object_Kname_max;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  gcv_object_t _object_Kndelay;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_NEWS))
  gcv_object_t _object_Knews;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(NEW_TIME))
  gcv_object_t _object_Knew_time;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NGROUPS_MAX))
  gcv_object_t _object_Kngroups_max;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOCTTY))
  gcv_object_t _object_Knoctty;
#endif
  gcv_object_t _object_Knoescape;
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_NOFILE))
  gcv_object_t _object_Knofile;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOFOLLOW))
  gcv_object_t _object_Knofollow;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_Knohang;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOINHERIT))
  gcv_object_t _object_Knoinherit;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NONBLOCK))
  gcv_object_t _object_Knonblock;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE))) || ((defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_NORMAL)))
  gcv_object_t _object_Knormal;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_NOTICE))
  gcv_object_t _object_Knotice;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Knotimplemented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_NOT_CONTENT_INDEXED))
  gcv_object_t _object_Knot_content_indexed;
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))) || (defined(HAVE_SYS_RESOURCE_H))
  gcv_object_t _object_Knowait;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_NO_TRUNC))
  gcv_object_t _object_Kno_trunc;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_NPROC))
  gcv_object_t _object_Knproc;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NPROCESSORS_CONF))
  gcv_object_t _object_Knprocessors_conf;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NPROCESSORS_ONLN))
  gcv_object_t _object_Knprocessors_onln;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_NS))
  gcv_object_t _object_Kns;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Knt;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Knull;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  gcv_object_t _object_Kodelay;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_OFFLINE))
  gcv_object_t _object_Koffline;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(OLD_TIME))
  gcv_object_t _object_Kold_time;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_OPEN_MAX))
  gcv_object_t _object_Kopen_max;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_OID))
  gcv_object_t _object_Kor_invalid_oid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_OXID))
  gcv_object_t _object_Kor_invalid_oxid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_SET))
  gcv_object_t _object_Kor_invalid_set;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kpagecount;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PAGESIZE))
  gcv_object_t _object_Kpagesize;
#endif
#if ((defined(HAVE_CONFSTR)) && (defined(_CS_PATH))) || (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN))
  gcv_object_t _object_Kpath;
#endif
  gcv_object_t _object_Kpathname;
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PATH_MAX))
  gcv_object_t _object_Kpath_max;
#endif
  gcv_object_t _object_Kperiod;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kpersonal;
#endif
#if defined(PRIO_PGRP)
  gcv_object_t _object_Kpgrp;
#endif
#if defined(HAVE_NFTW)
  gcv_object_t _object_Kphys;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PHYS_PAGES))
  gcv_object_t _object_Kphys_pages;
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))) || (defined(HAVE_SYS_RESOURCE_H))
  gcv_object_t _object_Kpid;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PIPE_BUF))
  gcv_object_t _object_Kpipe_buf;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_CFLAGS))
  gcv_object_t _object_Kposix_v6_ilp32_off32_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_LDFLAGS))
  gcv_object_t _object_Kposix_v6_ilp32_off32_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_LIBS))
  gcv_object_t _object_Kposix_v6_ilp32_off32_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_CFLAGS))
  gcv_object_t _object_Kposix_v6_ilp32_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS))
  gcv_object_t _object_Kposix_v6_ilp32_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_LIBS))
  gcv_object_t _object_Kposix_v6_ilp32_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_CFLAGS))
  gcv_object_t _object_Kposix_v6_lp64_off64_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_LDFLAGS))
  gcv_object_t _object_Kposix_v6_lp64_off64_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_LIBS))
  gcv_object_t _object_Kposix_v6_lp64_off64_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS))
  gcv_object_t _object_Kposix_v6_lpbig_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS))
  gcv_object_t _object_Kposix_v6_lpbig_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_LIBS))
  gcv_object_t _object_Kposix_v6_lpbig_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_WIDTH_RESTRICTED_ENVS))
  gcv_object_t _object_Kposix_v6_width_restricted_envs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_CFLAGS))
  gcv_object_t _object_Kposix_v7_ilp32_off32_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_LDFLAGS))
  gcv_object_t _object_Kposix_v7_ilp32_off32_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_LIBS))
  gcv_object_t _object_Kposix_v7_ilp32_off32_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_CFLAGS))
  gcv_object_t _object_Kposix_v7_ilp32_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS))
  gcv_object_t _object_Kposix_v7_ilp32_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_LIBS))
  gcv_object_t _object_Kposix_v7_ilp32_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_CFLAGS))
  gcv_object_t _object_Kposix_v7_lp64_off64_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_LDFLAGS))
  gcv_object_t _object_Kposix_v7_lp64_off64_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_LIBS))
  gcv_object_t _object_Kposix_v7_lp64_off64_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS))
  gcv_object_t _object_Kposix_v7_lpbig_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS))
  gcv_object_t _object_Kposix_v7_lpbig_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_LIBS))
  gcv_object_t _object_Kposix_v7_lpbig_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_THREADS_CFLAGS))
  gcv_object_t _object_Kposix_v7_threads_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_THREADS_LDFLAGS))
  gcv_object_t _object_Kposix_v7_threads_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_WIDTH_RESTRICTED_ENVS))
  gcv_object_t _object_Kposix_v7_width_restricted_envs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_PPC))
  gcv_object_t _object_Kppc;
#endif
  gcv_object_t _object_Kpreserve;
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PRIORITIZED_IO))
  gcv_object_t _object_Kprioritized_io;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PRIORITY_SCHEDULING))
  gcv_object_t _object_Kpriority_scheduling;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PRIO_IO))
  gcv_object_t _object_Kprio_io;
#endif
#if defined(PRIO_PROCESS)
  gcv_object_t _object_Kprocess;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kr4;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kr8;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RAW_SOCKETS))
  gcv_object_t _object_Kraw_sockets;
#endif
#if defined(HAVE_FCNTL)
  gcv_object_t _object_Krdonly;
#endif
#if defined(HAVE_FCNTL)
  gcv_object_t _object_Krdwr;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_READER_WRITER_LOCKS))
  gcv_object_t _object_Kreader_writer_locks;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_READONLY))
  gcv_object_t _object_Kreadonly;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(REALTIME_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  gcv_object_t _object_Krealtime;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_REALTIME_SIGNALS))
  gcv_object_t _object_Krealtime_signals;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_INCR_XFER_SIZE))
  gcv_object_t _object_Krec_incr_xfer_size;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_MAX_XFER_SIZE))
  gcv_object_t _object_Krec_max_xfer_size;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_MIN_XFER_SIZE))
  gcv_object_t _object_Krec_min_xfer_size;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_XFER_ALIGN))
  gcv_object_t _object_Krec_xfer_align;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_REGEXP))
  gcv_object_t _object_Kregexp;
#endif
  gcv_object_t _object_Krename;
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_REPARSE_POINT))
  gcv_object_t _object_Kreparse_point;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Krevnumber;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RE_DUP_MAX))
  gcv_object_t _object_Kre_dup_max;
#endif
#if defined(S_IRGRP)
  gcv_object_t _object_Krgrp;
#endif
#if defined(S_IROTH)
  gcv_object_t _object_Kroth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ADDRESS_ERROR))
  gcv_object_t _object_Krpc_s_address_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ALREADY_LISTENING))
  gcv_object_t _object_Krpc_s_already_listening;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ALREADY_REGISTERED))
  gcv_object_t _object_Krpc_s_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_BINDING_HAS_NO_AUTH))
  gcv_object_t _object_Krpc_s_binding_has_no_auth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_BINDING_INCOMPLETE))
  gcv_object_t _object_Krpc_s_binding_incomplete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_CANCELLED))
  gcv_object_t _object_Krpc_s_call_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_FAILED))
  gcv_object_t _object_Krpc_s_call_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_FAILED_DNE))
  gcv_object_t _object_Krpc_s_call_failed_dne;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_IN_PROGRESS))
  gcv_object_t _object_Krpc_s_call_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CANNOT_SUPPORT))
  gcv_object_t _object_Krpc_s_cannot_support;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CANT_CREATE_ENDPOINT))
  gcv_object_t _object_Krpc_s_cant_create_endpoint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_COMM_FAILURE))
  gcv_object_t _object_Krpc_s_comm_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_DUPLICATE_ENDPOINT))
  gcv_object_t _object_Krpc_s_duplicate_endpoint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_ALREADY_EXISTS))
  gcv_object_t _object_Krpc_s_entry_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_NOT_FOUND))
  gcv_object_t _object_Krpc_s_entry_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_TYPE_MISMATCH))
  gcv_object_t _object_Krpc_s_entry_type_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_DIV_ZERO))
  gcv_object_t _object_Krpc_s_fp_div_zero;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_OVERFLOW))
  gcv_object_t _object_Krpc_s_fp_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_UNDERFLOW))
  gcv_object_t _object_Krpc_s_fp_underflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GROUP_MEMBER_NOT_FOUND))
  gcv_object_t _object_Krpc_s_group_member_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GRP_ELT_NOT_ADDED))
  gcv_object_t _object_Krpc_s_grp_elt_not_added;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GRP_ELT_NOT_REMOVED))
  gcv_object_t _object_Krpc_s_grp_elt_not_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INCOMPLETE_NAME))
  gcv_object_t _object_Krpc_s_incomplete_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERFACE_NOT_EXPORTED))
  gcv_object_t _object_Krpc_s_interface_not_exported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERFACE_NOT_FOUND))
  gcv_object_t _object_Krpc_s_interface_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERNAL_ERROR))
  gcv_object_t _object_Krpc_s_internal_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ASYNC_CALL))
  gcv_object_t _object_Krpc_s_invalid_async_call;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ASYNC_HANDLE))
  gcv_object_t _object_Krpc_s_invalid_async_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_AUTH_IDENTITY))
  gcv_object_t _object_Krpc_s_invalid_auth_identity;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_BINDING))
  gcv_object_t _object_Krpc_s_invalid_binding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_BOUND))
  gcv_object_t _object_Krpc_s_invalid_bound;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ENDPOINT_FORMAT))
  gcv_object_t _object_Krpc_s_invalid_endpoint_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NAF_ID))
  gcv_object_t _object_Krpc_s_invalid_naf_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NAME_SYNTAX))
  gcv_object_t _object_Krpc_s_invalid_name_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NETWORK_OPTIONS))
  gcv_object_t _object_Krpc_s_invalid_network_options;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NET_ADDR))
  gcv_object_t _object_Krpc_s_invalid_net_addr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_OBJECT))
  gcv_object_t _object_Krpc_s_invalid_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_RPC_PROTSEQ))
  gcv_object_t _object_Krpc_s_invalid_rpc_protseq;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_STRING_BINDING))
  gcv_object_t _object_Krpc_s_invalid_string_binding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_STRING_UUID))
  gcv_object_t _object_Krpc_s_invalid_string_uuid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_TAG))
  gcv_object_t _object_Krpc_s_invalid_tag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_TIMEOUT))
  gcv_object_t _object_Krpc_s_invalid_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_VERS_OPTION))
  gcv_object_t _object_Krpc_s_invalid_vers_option;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_MAX_CALLS_TOO_SMALL))
  gcv_object_t _object_Krpc_s_max_calls_too_small;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NAME_SERVICE_UNAVAILABLE))
  gcv_object_t _object_Krpc_s_name_service_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOTHING_TO_EXPORT))
  gcv_object_t _object_Krpc_s_nothing_to_export;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_ALL_OBJS_EXPORTED))
  gcv_object_t _object_Krpc_s_not_all_objs_exported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_ALL_OBJS_UNEXPORTED))
  gcv_object_t _object_Krpc_s_not_all_objs_unexported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_CANCELLED))
  gcv_object_t _object_Krpc_s_not_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_LISTENING))
  gcv_object_t _object_Krpc_s_not_listening;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_RPC_ERROR))
  gcv_object_t _object_Krpc_s_not_rpc_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_BINDINGS))
  gcv_object_t _object_Krpc_s_no_bindings;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_CALL_ACTIVE))
  gcv_object_t _object_Krpc_s_no_call_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_CONTEXT_AVAILABLE))
  gcv_object_t _object_Krpc_s_no_context_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_ENDPOINT_FOUND))
  gcv_object_t _object_Krpc_s_no_endpoint_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_ENTRY_NAME))
  gcv_object_t _object_Krpc_s_no_entry_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_INTERFACES))
  gcv_object_t _object_Krpc_s_no_interfaces;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_MORE_BINDINGS))
  gcv_object_t _object_Krpc_s_no_more_bindings;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_MORE_MEMBERS))
  gcv_object_t _object_Krpc_s_no_more_members;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PRINC_NAME))
  gcv_object_t _object_Krpc_s_no_princ_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PROTSEQS))
  gcv_object_t _object_Krpc_s_no_protseqs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PROTSEQS_REGISTERED))
  gcv_object_t _object_Krpc_s_no_protseqs_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_OBJECT_NOT_FOUND))
  gcv_object_t _object_Krpc_s_object_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_OUT_OF_RESOURCES))
  gcv_object_t _object_Krpc_s_out_of_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PRF_ELT_NOT_ADDED))
  gcv_object_t _object_Krpc_s_prf_elt_not_added;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PRF_ELT_NOT_REMOVED))
  gcv_object_t _object_Krpc_s_prf_elt_not_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROCNUM_OUT_OF_RANGE))
  gcv_object_t _object_Krpc_s_procnum_out_of_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROFILE_NOT_ADDED))
  gcv_object_t _object_Krpc_s_profile_not_added;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTOCOL_ERROR))
  gcv_object_t _object_Krpc_s_protocol_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTSEQ_NOT_FOUND))
  gcv_object_t _object_Krpc_s_protseq_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTSEQ_NOT_SUPPORTED))
  gcv_object_t _object_Krpc_s_protseq_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SEC_PKG_ERROR))
  gcv_object_t _object_Krpc_s_sec_pkg_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SEND_INCOMPLETE))
  gcv_object_t _object_Krpc_s_send_incomplete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SERVER_TOO_BUSY))
  gcv_object_t _object_Krpc_s_server_too_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SERVER_UNAVAILABLE))
  gcv_object_t _object_Krpc_s_server_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_STRING_TOO_LONG))
  gcv_object_t _object_Krpc_s_string_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_TYPE_ALREADY_REGISTERED))
  gcv_object_t _object_Krpc_s_type_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_LEVEL))
  gcv_object_t _object_Krpc_s_unknown_authn_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_SERVICE))
  gcv_object_t _object_Krpc_s_unknown_authn_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_TYPE))
  gcv_object_t _object_Krpc_s_unknown_authn_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHZ_SERVICE))
  gcv_object_t _object_Krpc_s_unknown_authz_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_IF))
  gcv_object_t _object_Krpc_s_unknown_if;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_MGR_TYPE))
  gcv_object_t _object_Krpc_s_unknown_mgr_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_AUTHN_LEVEL))
  gcv_object_t _object_Krpc_s_unsupported_authn_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_NAME_SYNTAX))
  gcv_object_t _object_Krpc_s_unsupported_name_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_TRANS_SYN))
  gcv_object_t _object_Krpc_s_unsupported_trans_syn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_TYPE))
  gcv_object_t _object_Krpc_s_unsupported_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UUID_LOCAL_ONLY))
  gcv_object_t _object_Krpc_s_uuid_local_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UUID_NO_ADDRESS))
  gcv_object_t _object_Krpc_s_uuid_no_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_WRONG_KIND_OF_BINDING))
  gcv_object_t _object_Krpc_s_wrong_kind_of_binding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ZERO_DIVIDE))
  gcv_object_t _object_Krpc_s_zero_divide;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_BAD_STUB_DATA))
  gcv_object_t _object_Krpc_x_bad_stub_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_BYTE_COUNT_TOO_SMALL))
  gcv_object_t _object_Krpc_x_byte_count_too_small;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_ENUM_VALUE_OUT_OF_RANGE))
  gcv_object_t _object_Krpc_x_enum_value_out_of_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_INVALID_ES_ACTION))
  gcv_object_t _object_Krpc_x_invalid_es_action;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_INVALID_PIPE_OBJECT))
  gcv_object_t _object_Krpc_x_invalid_pipe_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_NO_MORE_ENTRIES))
  gcv_object_t _object_Krpc_x_no_more_entries;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_NULL_REF_POINTER))
  gcv_object_t _object_Krpc_x_null_ref_pointer;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_CLOSED))
  gcv_object_t _object_Krpc_x_pipe_closed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_DISCIPLINE_ERROR))
  gcv_object_t _object_Krpc_x_pipe_discipline_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_EMPTY))
  gcv_object_t _object_Krpc_x_pipe_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CANNOT_GET_CALL_HANDLE))
  gcv_object_t _object_Krpc_x_ss_cannot_get_call_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CHAR_TRANS_OPEN_FAIL))
  gcv_object_t _object_Krpc_x_ss_char_trans_open_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CHAR_TRANS_SHORT_FILE))
  gcv_object_t _object_Krpc_x_ss_char_trans_short_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CONTEXT_DAMAGED))
  gcv_object_t _object_Krpc_x_ss_context_damaged;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_HANDLES_MISMATCH))
  gcv_object_t _object_Krpc_x_ss_handles_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_IN_NULL_CONTEXT))
  gcv_object_t _object_Krpc_x_ss_in_null_context;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_ES_VERSION))
  gcv_object_t _object_Krpc_x_wrong_es_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_PIPE_ORDER))
  gcv_object_t _object_Krpc_x_wrong_pipe_order;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_PIPE_VERSION))
  gcv_object_t _object_Krpc_x_wrong_pipe_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_STUB_VERSION))
  gcv_object_t _object_Krpc_x_wrong_stub_version;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_RSS))
  gcv_object_t _object_Krss;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RTSIG_MAX))
  gcv_object_t _object_Krtsig_max;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(RUN_LVL))
  gcv_object_t _object_Krun_lvl;
#endif
#if defined(S_IRUSR)
  gcv_object_t _object_Krusr;
#endif
#if defined(S_IRWXG)
  gcv_object_t _object_Krwxg;
#endif
#if defined(S_IRWXO)
  gcv_object_t _object_Krwxo;
#endif
#if defined(S_IRWXU)
  gcv_object_t _object_Krwxu;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SAVED_IDS))
  gcv_object_t _object_Ksaved_ids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SCHED_E_SERVICE_NOT_LOCALSYSTEM))
  gcv_object_t _object_Ksched_e_service_not_localsystem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_ISSUING_CA_UNTRUSTED))
  gcv_object_t _object_Ksec_e_issuing_ca_untrusted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_PKINIT_CLIENT_FAILUR))
  gcv_object_t _object_Ksec_e_pkinit_client_failur;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_REVOCATION_OFFLINE_C))
  gcv_object_t _object_Ksec_e_revocation_offline_c;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_SMARTCARD_CERT_EXPIRED))
  gcv_object_t _object_Ksec_e_smartcard_cert_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_SMARTCARD_CERT_REVOKED))
  gcv_object_t _object_Ksec_e_smartcard_cert_revoked;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_SELF))
  gcv_object_t _object_Kself;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEMAPHORES))
  gcv_object_t _object_Ksemaphores;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEM_NSEMS_MAX))
  gcv_object_t _object_Ksem_nsems_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEM_VALUE_MAX))
  gcv_object_t _object_Ksem_value_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kserver;
#endif
#if defined(S_ISGID)
  gcv_object_t _object_Ksgid;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  gcv_object_t _object_Kshared;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SHARED_MEMORY_OBJECTS))
  gcv_object_t _object_Kshared_memory_objects;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SHELL))
  gcv_object_t _object_Kshell;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kshift;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kshow_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_SHX))
  gcv_object_t _object_Kshx;
#endif
#if defined(SIGABRT)
  gcv_object_t _object_Ksigabrt;
#endif
#if defined(SIGALRM)
  gcv_object_t _object_Ksigalrm;
#endif
#if defined(SIGBUS)
  gcv_object_t _object_Ksigbus;
#endif
#if defined(SIGCHLD)
  gcv_object_t _object_Ksigchld;
#endif
#if defined(SIGCONT)
  gcv_object_t _object_Ksigcont;
#endif
#if defined(SIGFPE)
  gcv_object_t _object_Ksigfpe;
#endif
#if defined(SIGHUP)
  gcv_object_t _object_Ksighup;
#endif
#if defined(SIGILL)
  gcv_object_t _object_Ksigill;
#endif
#if defined(SIGINT)
  gcv_object_t _object_Ksigint;
#endif
#if defined(SIGKILL)
  gcv_object_t _object_Ksigkill;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_Ksignaled;
#endif
#if defined(SIGPIPE)
  gcv_object_t _object_Ksigpipe;
#endif
#if defined(SIGPOLL)
  gcv_object_t _object_Ksigpoll;
#endif
#if defined(SIGPROF)
  gcv_object_t _object_Ksigprof;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SIGQUEUE_MAX))
  gcv_object_t _object_Ksigqueue_max;
#endif
#if defined(SIGQUIT)
  gcv_object_t _object_Ksigquit;
#endif
#if defined(SIGSEGV)
  gcv_object_t _object_Ksigsegv;
#endif
#if defined(SIGSTOP)
  gcv_object_t _object_Ksigstop;
#endif
#if defined(SIGSYS)
  gcv_object_t _object_Ksigsys;
#endif
#if defined(SIGTERM)
  gcv_object_t _object_Ksigterm;
#endif
#if defined(SIGTRAP)
  gcv_object_t _object_Ksigtrap;
#endif
#if defined(SIGTSTP)
  gcv_object_t _object_Ksigtstp;
#endif
#if defined(SIGTTIN)
  gcv_object_t _object_Ksigttin;
#endif
#if defined(SIGTTOU)
  gcv_object_t _object_Ksigttou;
#endif
#if defined(SIGURG)
  gcv_object_t _object_Ksigurg;
#endif
#if defined(SIGUSR1)
  gcv_object_t _object_Ksigusr1;
#endif
#if defined(SIGUSR2)
  gcv_object_t _object_Ksigusr2;
#endif
#if defined(SIGVTALRM)
  gcv_object_t _object_Ksigvtalrm;
#endif
#if defined(SIGXCPU)
  gcv_object_t _object_Ksigxcpu;
#endif
#if defined(SIGXFSZ)
  gcv_object_t _object_Ksigxfsz;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_SL))
  gcv_object_t _object_Ksl;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_SLN))
  gcv_object_t _object_Ksln;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ksmallbusiness;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ksmallbusiness_restricted;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SOCK_MAXBUF))
  gcv_object_t _object_Ksock_maxbuf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_SPARSE_FILE))
  gcv_object_t _object_Ksparse_file;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPAWN))
  gcv_object_t _object_Kspawn;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPIN_LOCKS))
  gcv_object_t _object_Kspin_locks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPORADIC_SERVER))
  gcv_object_t _object_Ksporadic_server;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SS_REPL_MAX))
  gcv_object_t _object_Kss_repl_max;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_STACK))
  gcv_object_t _object_Kstack;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  gcv_object_t _object_Kstart;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_Kstopped;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_STREAM_MAX))
  gcv_object_t _object_Kstream_max;
#endif
#if defined(STRUNCATE)
  gcv_object_t _object_Kstruncate;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_APPEND))
  gcv_object_t _object_Kst_append;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_IMMUTABLE))
  gcv_object_t _object_Kst_immutable;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_MANDLOCK))
  gcv_object_t _object_Kst_mandlock;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOATIME))
  gcv_object_t _object_Kst_noatime;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NODEV))
  gcv_object_t _object_Kst_nodev;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NODIRATIME))
  gcv_object_t _object_Kst_nodiratime;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOEXEC))
  gcv_object_t _object_Kst_noexec;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOSUID))
  gcv_object_t _object_Kst_nosuid;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOTRUNC))
  gcv_object_t _object_Kst_notrunc;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_RDONLY))
  gcv_object_t _object_Kst_rdonly;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_SYNCHRONOUS))
  gcv_object_t _object_Kst_synchronous;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_WRITE))
  gcv_object_t _object_Kst_write;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ksubject;
#endif
#if defined(S_ISUID)
  gcv_object_t _object_Ksuid;
#endif
#if defined(S_ISVTX)
  gcv_object_t _object_Ksvtx;
#endif
  gcv_object_t _object_Ksymlink;
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SYMLINK_MAX))
  gcv_object_t _object_Ksymlink_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SYMLOOP_MAX))
  gcv_object_t _object_Ksymloop_max;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_SYNC))
  gcv_object_t _object_Ksync;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SYNCHRONIZED_IO))
  gcv_object_t _object_Ksynchronized_io;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SYNC_IO))
  gcv_object_t _object_Ksync_io;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_SYSLOG))
  gcv_object_t _object_Ksyslog;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_SYSTEM))
  gcv_object_t _object_Ksystem;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ktemplate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_TEMPORARY))
  gcv_object_t _object_Ktemporary;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kterminal;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_TEXT))
  gcv_object_t _object_Ktext;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_THREAD))
  gcv_object_t _object_Kthread;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREADS))
  gcv_object_t _object_Kthreads;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ATTR_STACKADDR))
  gcv_object_t _object_Kthread_attr_stackaddr;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ATTR_STACKSIZE))
  gcv_object_t _object_Kthread_attr_stacksize;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_CPUTIME))
  gcv_object_t _object_Kthread_cputime;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_DESTRUCTOR_ITERATIONS))
  gcv_object_t _object_Kthread_destructor_iterations;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_KEYS_MAX))
  gcv_object_t _object_Kthread_keys_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIORITY_SCHEDULING))
  gcv_object_t _object_Kthread_priority_scheduling;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIO_INHERIT))
  gcv_object_t _object_Kthread_prio_inherit;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIO_PROTECT))
  gcv_object_t _object_Kthread_prio_protect;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PROCESS_SHARED))
  gcv_object_t _object_Kthread_process_shared;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ROBUST_PRIO_INHERIT))
  gcv_object_t _object_Kthread_robust_prio_inherit;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ROBUST_PRIO_PROTECT))
  gcv_object_t _object_Kthread_robust_prio_protect;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_SAFE_FUNCTIONS))
  gcv_object_t _object_Kthread_safe_functions;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_SPORADIC_SERVER))
  gcv_object_t _object_Kthread_sporadic_server;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_STACK_MIN))
  gcv_object_t _object_Kthread_stack_min;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_THREADS_MAX))
  gcv_object_t _object_Kthread_threads_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kthumbnail;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMEOUTS))
  gcv_object_t _object_Ktimeouts;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMERS))
  gcv_object_t _object_Ktimers;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMER_MAX))
  gcv_object_t _object_Ktimer_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Ktitle;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE))
  gcv_object_t _object_Ktrace;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_EVENT_FILTER))
  gcv_object_t _object_Ktrace_event_filter;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_EVENT_NAME_MAX))
  gcv_object_t _object_Ktrace_event_name_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_INHERIT))
  gcv_object_t _object_Ktrace_inherit;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_LOG))
  gcv_object_t _object_Ktrace_log;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_NAME_MAX))
  gcv_object_t _object_Ktrace_name_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_SYS_MAX))
  gcv_object_t _object_Ktrace_sys_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_USER_EVENT_MAX))
  gcv_object_t _object_Ktrace_user_event_max;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_TRUNC))
  gcv_object_t _object_Ktrunc;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TTY_NAME_MAX))
  gcv_object_t _object_Ktty_name_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TYPED_MEMORY_OBJECTS))
  gcv_object_t _object_Ktyped_memory_objects;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TZNAME_MAX))
  gcv_object_t _object_Ktzname_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kui1;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kui2;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kui4;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kui8;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  gcv_object_t _object_Kuid;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kuint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_UNKNOWN))
  gcv_object_t _object_Kunknown;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_Kuntraced;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_Kusage;
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(LOG_USER))) || (defined(PRIO_USER))
  gcv_object_t _object_Kuser;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kuser_defined;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(USER_PROCESS))
  gcv_object_t _object_Kuser_process;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_UUCP))
  gcv_object_t _object_Kuucp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_ILP32_OFF32))
  gcv_object_t _object_Kv6_ilp32_off32;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_ILP32_OFFBIG))
  gcv_object_t _object_Kv6_ilp32_offbig;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_LP64_OFF64))
  gcv_object_t _object_Kv6_lp64_off64;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_LPBIG_OFFBIG))
  gcv_object_t _object_Kv6_lpbig_offbig;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_V7_ENV))
  gcv_object_t _object_Kv7_env;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_ILP32_OFF32))
  gcv_object_t _object_Kv7_ilp32_off32;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_ILP32_OFFBIG))
  gcv_object_t _object_Kv7_ilp32_offbig;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_LP64_OFF64))
  gcv_object_t _object_Kv7_lp64_off64;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_LPBIG_OFFBIG))
  gcv_object_t _object_Kv7_lpbig_offbig;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_VDISABLE))
  gcv_object_t _object_Kvdisable;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_VERSION))
  gcv_object_t _object_Kversion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WAIT_TIMEOUT))
  gcv_object_t _object_Kwait_timeout;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_WARNING))
  gcv_object_t _object_Kwarning;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WARNING_IPSEC_MM_POLICY_PRUNED))
  gcv_object_t _object_Kwarning_ipsec_mm_policy_pruned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WARNING_IPSEC_QM_POLICY_PRUNED))
  gcv_object_t _object_Kwarning_ipsec_qm_policy_pruned;
#endif
#if defined(S_IWGRP)
  gcv_object_t _object_Kwgrp;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kwindows;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kwordcount;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kworking_directory;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_Kworkstation;
#endif
#if defined(S_IWOTH)
  gcv_object_t _object_Kwoth;
#endif
#if defined(HAVE_FCNTL)
  gcv_object_t _object_Kwronly;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSABASEERR))
  gcv_object_t _object_Kwsabaseerr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEACCES))
  gcv_object_t _object_Kwsaeacces;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEADDRINUSE))
  gcv_object_t _object_Kwsaeaddrinuse;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEADDRNOTAVAIL))
  gcv_object_t _object_Kwsaeaddrnotavail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEAFNOSUPPORT))
  gcv_object_t _object_Kwsaeafnosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEALREADY))
  gcv_object_t _object_Kwsaealready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEBADF))
  gcv_object_t _object_Kwsaebadf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECANCELLED))
  gcv_object_t _object_Kwsaecancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNABORTED))
  gcv_object_t _object_Kwsaeconnaborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNREFUSED))
  gcv_object_t _object_Kwsaeconnrefused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNRESET))
  gcv_object_t _object_Kwsaeconnreset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDESTADDRREQ))
  gcv_object_t _object_Kwsaedestaddrreq;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDISCON))
  gcv_object_t _object_Kwsaediscon;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDQUOT))
  gcv_object_t _object_Kwsaedquot;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEFAULT))
  gcv_object_t _object_Kwsaefault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEHOSTDOWN))
  gcv_object_t _object_Kwsaehostdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEHOSTUNREACH))
  gcv_object_t _object_Kwsaehostunreach;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINPROGRESS))
  gcv_object_t _object_Kwsaeinprogress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINTR))
  gcv_object_t _object_Kwsaeintr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVAL))
  gcv_object_t _object_Kwsaeinval;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVALIDPROCTABLE))
  gcv_object_t _object_Kwsaeinvalidproctable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVALIDPROVIDER))
  gcv_object_t _object_Kwsaeinvalidprovider;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEISCONN))
  gcv_object_t _object_Kwsaeisconn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAELOOP))
  gcv_object_t _object_Kwsaeloop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEMFILE))
  gcv_object_t _object_Kwsaemfile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEMSGSIZE))
  gcv_object_t _object_Kwsaemsgsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENAMETOOLONG))
  gcv_object_t _object_Kwsaenametoolong;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETDOWN))
  gcv_object_t _object_Kwsaenetdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETRESET))
  gcv_object_t _object_Kwsaenetreset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETUNREACH))
  gcv_object_t _object_Kwsaenetunreach;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOBUFS))
  gcv_object_t _object_Kwsaenobufs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOMORE))
  gcv_object_t _object_Kwsaenomore;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOPROTOOPT))
  gcv_object_t _object_Kwsaenoprotoopt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTCONN))
  gcv_object_t _object_Kwsaenotconn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTEMPTY))
  gcv_object_t _object_Kwsaenotempty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTSOCK))
  gcv_object_t _object_Kwsaenotsock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEOPNOTSUPP))
  gcv_object_t _object_Kwsaeopnotsupp;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPFNOSUPPORT))
  gcv_object_t _object_Kwsaepfnosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROCLIM))
  gcv_object_t _object_Kwsaeproclim;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROTONOSUPPORT))
  gcv_object_t _object_Kwsaeprotonosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROTOTYPE))
  gcv_object_t _object_Kwsaeprototype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROVIDERFAILEDINIT))
  gcv_object_t _object_Kwsaeproviderfailedinit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEREFUSED))
  gcv_object_t _object_Kwsaerefused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEREMOTE))
  gcv_object_t _object_Kwsaeremote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESHUTDOWN))
  gcv_object_t _object_Kwsaeshutdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESOCKTNOSUPPORT))
  gcv_object_t _object_Kwsaesocktnosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESTALE))
  gcv_object_t _object_Kwsaestale;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAETIMEDOUT))
  gcv_object_t _object_Kwsaetimedout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAETOOMANYREFS))
  gcv_object_t _object_Kwsaetoomanyrefs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEUSERS))
  gcv_object_t _object_Kwsaeusers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEWOULDBLOCK))
  gcv_object_t _object_Kwsaewouldblock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAHOST_NOT_FOUND))
  gcv_object_t _object_Kwsahost_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANOTINITIALISED))
  gcv_object_t _object_Kwsanotinitialised;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANO_DATA))
  gcv_object_t _object_Kwsano_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANO_RECOVERY))
  gcv_object_t _object_Kwsano_recovery;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASERVICE_NOT_FOUND))
  gcv_object_t _object_Kwsaservice_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASYSCALLFAILURE))
  gcv_object_t _object_Kwsasyscallfailure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASYSNOTREADY))
  gcv_object_t _object_Kwsasysnotready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSATRY_AGAIN))
  gcv_object_t _object_Kwsatry_again;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSATYPE_NOT_FOUND))
  gcv_object_t _object_Kwsatype_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAVERNOTSUPPORTED))
  gcv_object_t _object_Kwsavernotsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_E_CANCELLED))
  gcv_object_t _object_Kwsa_e_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_E_NO_MORE))
  gcv_object_t _object_Kwsa_e_no_more;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ADMISSION_FAILURE))
  gcv_object_t _object_Kwsa_qos_admission_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_BAD_OBJECT))
  gcv_object_t _object_Kwsa_qos_bad_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_BAD_STYLE))
  gcv_object_t _object_Kwsa_qos_bad_style;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERCOUNT))
  gcv_object_t _object_Kwsa_qos_efiltercount;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERSTYLE))
  gcv_object_t _object_Kwsa_qos_efilterstyle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERTYPE))
  gcv_object_t _object_Kwsa_qos_efiltertype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWCOUNT))
  gcv_object_t _object_Kwsa_qos_eflowcount;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWDESC))
  gcv_object_t _object_Kwsa_qos_eflowdesc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWSPEC))
  gcv_object_t _object_Kwsa_qos_eflowspec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EOBJLENGTH))
  gcv_object_t _object_Kwsa_qos_eobjlength;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPOLICYOBJ))
  gcv_object_t _object_Kwsa_qos_epolicyobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPROVSPECBUF))
  gcv_object_t _object_Kwsa_qos_eprovspecbuf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPSFILTERSPEC))
  gcv_object_t _object_Kwsa_qos_epsfilterspec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPSFLOWSPEC))
  gcv_object_t _object_Kwsa_qos_epsflowspec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESDMODEOBJ))
  gcv_object_t _object_Kwsa_qos_esdmodeobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESERVICETYPE))
  gcv_object_t _object_Kwsa_qos_eservicetype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESHAPERATEOBJ))
  gcv_object_t _object_Kwsa_qos_eshaperateobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EUNKNOWNPSOBJ))
  gcv_object_t _object_Kwsa_qos_eunknownpsobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_GENERIC_ERROR))
  gcv_object_t _object_Kwsa_qos_generic_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_NO_RECEIVERS))
  gcv_object_t _object_Kwsa_qos_no_receivers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_NO_SENDERS))
  gcv_object_t _object_Kwsa_qos_no_senders;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_POLICY_FAILURE))
  gcv_object_t _object_Kwsa_qos_policy_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_RECEIVERS))
  gcv_object_t _object_Kwsa_qos_receivers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_REQUEST_CONFIRMED))
  gcv_object_t _object_Kwsa_qos_request_confirmed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_RESERVED_PETYPE))
  gcv_object_t _object_Kwsa_qos_reserved_petype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_SENDERS))
  gcv_object_t _object_Kwsa_qos_senders;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_TRAFFIC_CTRL_ERROR))
  gcv_object_t _object_Kwsa_qos_traffic_ctrl_error;
#endif
#if defined(S_IWUSR)
  gcv_object_t _object_Kwusr;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_ILP32_OFF32))
  gcv_object_t _object_Kxbs5_ilp32_off32;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_CFLAGS))
  gcv_object_t _object_Kxbs5_ilp32_off32_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LDFLAGS))
  gcv_object_t _object_Kxbs5_ilp32_off32_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LIBS))
  gcv_object_t _object_Kxbs5_ilp32_off32_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LINTFLAGS))
  gcv_object_t _object_Kxbs5_ilp32_off32_lintflags;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_ILP32_OFFBIG))
  gcv_object_t _object_Kxbs5_ilp32_offbig;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_CFLAGS))
  gcv_object_t _object_Kxbs5_ilp32_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LDFLAGS))
  gcv_object_t _object_Kxbs5_ilp32_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LIBS))
  gcv_object_t _object_Kxbs5_ilp32_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LINTFLAGS))
  gcv_object_t _object_Kxbs5_ilp32_offbig_lintflags;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_LP64_OFF64))
  gcv_object_t _object_Kxbs5_lp64_off64;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_CFLAGS))
  gcv_object_t _object_Kxbs5_lp64_off64_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LDFLAGS))
  gcv_object_t _object_Kxbs5_lp64_off64_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LIBS))
  gcv_object_t _object_Kxbs5_lp64_off64_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LINTFLAGS))
  gcv_object_t _object_Kxbs5_lp64_off64_lintflags;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_LPBIG_OFFBIG))
  gcv_object_t _object_Kxbs5_lpbig_offbig;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_CFLAGS))
  gcv_object_t _object_Kxbs5_lpbig_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LDFLAGS))
  gcv_object_t _object_Kxbs5_lpbig_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LIBS))
  gcv_object_t _object_Kxbs5_lpbig_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LINTFLAGS))
  gcv_object_t _object_Kxbs5_lpbig_offbig_lintflags;
#endif
#if defined(S_IXGRP)
  gcv_object_t _object_Kxgrp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_CRYPT))
  gcv_object_t _object_Kxopen_crypt;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_ENH_I18N))
  gcv_object_t _object_Kxopen_enh_i18n;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_LEGACY))
  gcv_object_t _object_Kxopen_legacy;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_REALTIME))
  gcv_object_t _object_Kxopen_realtime;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_REALTIME_THREADS))
  gcv_object_t _object_Kxopen_realtime_threads;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_SHM))
  gcv_object_t _object_Kxopen_shm;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_STREAMS))
  gcv_object_t _object_Kxopen_streams;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_UNIX))
  gcv_object_t _object_Kxopen_unix;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_UUCP))
  gcv_object_t _object_Kxopen_uucp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_VERSION))
  gcv_object_t _object_Kxopen_version;
#endif
#if defined(S_IXOTH)
  gcv_object_t _object_Kxoth;
#endif
#if defined(S_IXUSR)
  gcv_object_t _object_Kxusr;
#endif
  gcv_object_t _object_posix__copy_file;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_posix__make_file_info;
#endif
#if (defined(HAVE_FSTAT) && defined(HAVE_STAT)) || (defined(HAVE_NFTW))
  gcv_object_t _object_posix__make_file_stat;
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  gcv_object_t _object_posix__make_group_info;
#endif
  gcv_object_t _object_posix__make_hostent;
#if defined(HAVE_GETRLIMIT)
  gcv_object_t _object_posix__make_rlimit;
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  gcv_object_t _object_posix__make_service;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_posix__make_shortcut_info;
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  gcv_object_t _object_posix__make_stat_vfs;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_posix__make_system_info;
#endif
  gcv_object_t _object_posix__make_uname;
#if defined(HAVE_SYS_RESOURCE_H)
  gcv_object_t _object_posix__make_usage;
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  gcv_object_t _object_posix__make_user_info;
#endif
#if defined(HAVE_UTMPX_H)
  gcv_object_t _object_posix__make_utmpx;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_posix__make_version;
#endif
#if defined(WIN32_NATIVE)
  gcv_object_t _object_posix__mkfilever;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object_posix__mkmemstat;
#endif
  gcv_object_t _object_posix__resolve_host_ipaddr;
#if defined(HAVE_SETRLIMIT)
  gcv_object_t _object_posix__rlimit;
#endif
#if defined(HAVE_UTMPX_H)
  gcv_object_t _object_posix__utmpx;
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!(defined(WIN32_NATIVE))) && (!(defined(HAVE_CHMOD)))
  gcv_object_t _object__22chmod_28_29_22;
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!(defined(HAVE_CHOWN)))
  gcv_object_t _object__22chown_28_29_22;
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!defined(WIN32_NATIVE)) && (!(defined(HAVE_UTIME)))
  gcv_object_t _object__22utime_28_29_22;
#endif
  gcv_object_t _object__28member_200_201_202_29;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object__28member_20_alt_20_control_20_ext_20_shift_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object__28member_20_normal_20_max_20_min_29;
#endif
  gcv_object_t _object__28or_20null_20integer_20_28member_20_copy_20_symlink_20_hardlink_20_hardlink_or_copy_20_rename_29_29;
  gcv_object_t _object___28or_20integer_20_28member_defined_28e2big_29_Ke2big_defined_28eacces_29_Keacces_defined_28eaddrinuse_29_Keaddrinuse_defined_28eaddrnotavail_29_Keaddrnotavail_defined_28eafnosupport_29_Keafnosupport_defined_28eagain_29_Keagain_defined_28ealready_29_Kealready_defined_28ebadf_29_Kebadf_defined_28ebadmsg_29_Kebadmsg_defined_28ebusy_29_Kebusy_defined_28ecanceled_29_Kecanceled_defined_28echild_29_Kechild_defined_28econnaborted_29_Keconnaborted_defined_28econnrefused_29_Keconnrefused_defined_28econnreset_29_Keconnreset_defined_28edeadlk_29_Kedeadlk_defined_28edestaddrreq_29_Kedestaddrreq_defined_28edom_29_Kedom_defined_28edquot_29_Kedquot_defined_28eexist_29_Keexist_defined_28efault_29_Kefault_defined_28efbig_29_Kefbig_defined_28ehostunreach_29_Kehostunreach_defined_28eidrm_29_Keidrm_defined_28eilseq_29_Keilseq_defined_28einprogress_29_Keinprogress_defined_28eintr_29_Keintr_defined_28einval_29_Keinval_defined_28eio_29_Keio_defined_28eisconn_29_Keisconn_defined_28eisdir_29_Keisdir_defined_28eloop_29_Keloop_defined_28emfile_29_Kemfile_defined_28emlink_29_Kemlink_defined_28emsgsize_29_Kemsgsize_defined_28emultihop_29_Kemultihop_defined_28enametoolong_29_Kenametoolong_defined_28enetdown_29_Kenetdown_defined_28enetreset_29_Kenetreset_defined_28enetunreach_29_Kenetunreach_defined_28enfile_29_Kenfile_defined_28enobufs_29_Kenobufs_defined_28enodata_29_Kenodata_defined_28enodev_29_Kenodev_defined_28enoent_29_Kenoent_defined_28enoexec_29_Kenoexec_defined_28enolck_29_Kenolck_defined_28enolink_29_Kenolink_defined_28enomem_29_Kenomem_defined_28enomsg_29_Kenomsg_defined_28enoprotoopt_29_Kenoprotoopt_defined_28enospc_29_Kenospc_defined_28enosr_29_Kenosr_defined_28enostr_29_Kenostr_defined_28enosys_29_Kenosys_defined_28enotconn_29_Kenotconn_defined_28enotdir_29_Kenotdir_defined_28enotempty_29_Kenotempty_defined_28enotsock_29_Kenotsock_defined_28enotsup_29_Kenotsup_defined_28enotty_29_Kenotty_defined_28enxio_29_Kenxio_defined_28eopnotsupp_29_Keopnotsupp_defined_28eoverflow_29_Keo;
#if defined(HAVE_UTMPX_H)
  gcv_object_t _object___28or_20integer_20_28member_defined_28empty_29_Kempty_defined_28run_lvl_29_Krun_lvl_defined_28boot_time_29_Kboot_time_defined_28old_time_29_Kold_time_defined_28new_time_29_Knew_time_defined_28user_process_29_Kuser_process_defined_28init_process_29_Kinit_process_defined_28login_process_29_Klogin_process_defined_28dead_process_29_Kdead_process_defined_28accounting_29_Kaccounting__29_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object___28or_20integer_20_28member_defined_28error_invalid_function_29_Kerror_invalid_function_defined_28error_file_not_found_29_Kerror_file_not_found_defined_28error_path_not_found_29_Kerror_path_not_found_defined_28error_too_many_open_files_29_Kerror_too_many_open_files_defined_28error_access_denied_29_Kerror_access_denied_defined_28error_invalid_handle_29_Kerror_invalid_handle_defined_28error_arena_trashed_29_Kerror_arena_trashed_defined_28error_not_enough_memory_29_Kerror_not_enough_memory_defined_28error_invalid_block_29_Kerror_invalid_block_defined_28error_bad_environment_29_Kerror_bad_environment_defined_28error_bad_format_29_Kerror_bad_format_defined_28error_invalid_access_29_Kerror_invalid_access_defined_28error_invalid_data_29_Kerror_invalid_data_defined_28error_outofmemory_29_Kerror_outofmemory_defined_28error_invalid_drive_29_Kerror_invalid_drive_defined_28error_current_directory_29_Kerror_current_directory_defined_28error_not_same_device_29_Kerror_not_same_device_defined_28error_no_more_files_29_Kerror_no_more_files_defined_28error_write_protect_29_Kerror_write_protect_defined_28error_bad_unit_29_Kerror_bad_unit_defined_28error_not_ready_29_Kerror_not_ready_defined_28error_bad_command_29_Kerror_bad_command_defined_28error_crc_29_Kerror_crc_defined_28error_bad_length_29_Kerror_bad_length_defined_28error_seek_29_Kerror_seek_defined_28error_not_dos_disk_29_Kerror_not_dos_disk_defined_28error_sector_not_found_29_Kerror_sector_not_found_defined_28error_out_of_paper_29_Kerror_out_of_paper_defined_28error_write_fault_29_Kerror_write_fault_defined_28error_read_fault_29_Kerror_read_fault_defined_28error_gen_failure_29_Kerror_gen_failure_defined_28error_sharing_violation_29_Kerror_sharing_violation_defined_28error_lock_violation_29_Kerror_lock_violation_defined_28error_wrong_disk_29_Kerror_wrong_disk_defined_28error_sharing_buffer_exceeded_29_Kerror_sharing_buffer_exceeded_defined_28error_handle_eof_29_Kerror_handle_eof_defined_28error_handle_disk_full_29_Kerror;
#endif
#if defined(HAVE_FCNTL)
  gcv_object_t _object___28or_20integer_20_28member_defined_28fd_cloexec_29_Kcloexec__29_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object___28or_20integer_20_28member_defined_28file_attribute_archive_29_Karchive_defined_28file_attribute_compressed_29_Kcompressed_defined_28file_attribute_device_29_Kdevice_defined_28file_attribute_directory_29_Kdirectory_defined_28file_attribute_encrypted_29_Kencrypted_defined_28file_attribute_hidden_29_Khidden_defined_28file_attribute_normal_29_Knormal_defined_28file_attribute_not_content_indexed_29_Knot_content_indexed_defined_28file_attribute_offline_29_Koffline_defined_28file_attribute_readonly_29_Kreadonly_defined_28file_attribute_reparse_point_29_Kreparse_point_defined_28file_attribute_sparse_file_29_Ksparse_file_defined_28file_attribute_system_29_Ksystem_defined_28file_attribute_temporary_29_Ktemporary__29_29;
#endif
#if defined(HAVE_NFTW)
  gcv_object_t _object___28or_20integer_20_28member_defined_28ftw_f_29_Kf_defined_28ftw_d_29_Kd_defined_28ftw_dp_29_Kdp_defined_28ftw_sl_29_Ksl_defined_28ftw_sln_29_Ksln_defined_28ftw_dnr_29_Kdnr_defined_28ftw_ns_29_Kns__29_29;
#endif
#if defined(HAVE_FCNTL)
  gcv_object_t _object___28or_20integer_20_28member_defined_28f_getfd_29_Kfd_defined_28f_getfl_29_Kfl__29_29;
#endif
#if defined(HAVE_SYSLOG)
  gcv_object_t _object___28or_20integer_20_28member_defined_28log_emerg_29_Kemerg_defined_28log_alert_29_Kalert_defined_28log_crit_29_Kcrit_defined_28log_err_29_Kerr_defined_28log_warning_29_Kwarning_defined_28log_notice_29_Knotice_defined_28log_info_29_Kinfo_defined_28log_debug_29_Kdebug__29_29;
#endif
#if defined(HAVE_FCNTL)
  gcv_object_t _object___28or_20integer_20_28member_defined_28o_rdonly_29_Krdonly_defined_28o_wronly_29_Kwronly_defined_28o_rdwr_29_Krdwr_defined_28o_append_29_Kappend_defined_28o_creat_29_Kcreat_defined_28o_trunc_29_Ktrunc_defined_28o_excl_29_Kexcl_defined_28o_noctty_29_Knoctty_defined_28o_sync_29_Ksync_defined_28o_nonblock_29_Knonblock_defined_28o_binary_29_Kbinary_defined_28o_text_29_Ktext_defined_28o_noinherit_29_Knoinherit_defined_28o_direct_29_Kdirect_defined_28o_largefile_29_Klargefile_defined_28o_directory_29_Kdirectory_defined_28o_nofollow_29_Knofollow__29_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  gcv_object_t _object___28or_20integer_20_28member_defined_28processor_architecture_intel_29_Kintel_defined_28processor_architecture_mips_29_Kmips_defined_28processor_architecture_alpha_29_Kalpha_defined_28processor_architecture_ppc_29_Kppc_defined_28processor_architecture_shx_29_Kshx_defined_28processor_architecture_arm_29_Karm_defined_28processor_architecture_ia64_29_Kia64_defined_28processor_architecture_alpha64_29_Kalpha64_defined_28processor_architecture_msil_29_Kmsil_defined_28processor_architecture_amd64_29_Kamd64_defined_28processor_architecture_ia32_on_win64_29_Kia32_on_win64_defined_28processor_architecture_unknown_29_Kunknown__29_29;
#endif
#if defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)
  gcv_object_t _object___28or_20integer_20_28member_defined_28rlimit_cpu_29_Kcpu_defined_28rlimit_fsize_29_Kfsize_defined_28rlimit_data_29_Kdata_defined_28rlimit_stack_29_Kstack_defined_28rlimit_core_29_Kcore_defined_28rlimit_rss_29_Krss_defined_28rlimit_nofile_29_Knofile_defined_28rlimit_as_29_Kas_defined_28rlimit_nproc_29_Knproc_defined_28rlimit_memlock_29_Kmemlock_defined_28rlimit_locks_29_Klocks__29_29;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  gcv_object_t _object___28or_20integer_20_28member_defined_28rusage_self_29_Kself_defined_28rusage_children_29_Kchildren_defined_28rusage_thread_29_Kthread_defined_28rusage_lwp_29_Klwp__29_29;
#endif
  gcv_object_t _object___28or_20integer_20_28member_defined_28sigabrt_29_Ksigabrt_defined_28sigalrm_29_Ksigalrm_defined_28sigbus_29_Ksigbus_defined_28sigchld_29_Ksigchld_defined_28sigcont_29_Ksigcont_defined_28sigfpe_29_Ksigfpe_defined_28sighup_29_Ksighup_defined_28sigill_29_Ksigill_defined_28sigint_29_Ksigint_defined_28sigkill_29_Ksigkill_defined_28sigpipe_29_Ksigpipe_defined_28sigquit_29_Ksigquit_defined_28sigsegv_29_Ksigsegv_defined_28sigstop_29_Ksigstop_defined_28sigterm_29_Ksigterm_defined_28sigtstp_29_Ksigtstp_defined_28sigttin_29_Ksigttin_defined_28sigttou_29_Ksigttou_defined_28sigusr1_29_Ksigusr1_defined_28sigusr2_29_Ksigusr2_defined_28sigpoll_29_Ksigpoll_defined_28sigprof_29_Ksigprof_defined_28sigsys_29_Ksigsys_defined_28sigtrap_29_Ksigtrap_defined_28sigurg_29_Ksigurg_defined_28sigvtalrm_29_Ksigvtalrm_defined_28sigxcpu_29_Ksigxcpu_defined_28sigxfsz_29_Ksigxfsz__29_29;
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  gcv_object_t _object___28or_20integer_20_28member_defined_28st_rdonly_29_Kst_rdonly_defined_28st_nosuid_29_Kst_nosuid_defined_28st_notrunc_29_Kst_notrunc_defined_28st_nodev_29_Kst_nodev_defined_28st_noexec_29_Kst_noexec_defined_28st_synchronous_29_Kst_synchronous_defined_28st_mandlock_29_Kst_mandlock_defined_28st_write_29_Kst_write_defined_28st_append_29_Kst_append_defined_28st_immutable_29_Kst_immutable_defined_28st_noatime_29_Kst_noatime_defined_28st_nodiratime_29_Kst_nodiratime_defined_28file_named_streams_29_Kfile_named_streams_defined_28file_read_only_volume_29_Kfile_read_only_volume_defined_28file_supports_object_ids_29_Kfile_supports_object_ids_defined_28file_supports_reparse_points_29_Kfile_supports_reparse_points_defined_28file_supports_sparse_files_29_Kfile_supports_sparse_files_defined_28file_volume_quotas_29_Kfile_volume_quotas_defined_28file_supports_encryption_29_Kfile_supports_encryption_defined_28fs_case_is_preserved_29_Kfs_case_is_preserved_defined_28fs_case_sensitive_29_Kfs_case_sensitive_defined_28fs_file_compression_29_Kfs_file_compression_defined_28fs_file_encryption_29_Kfs_file_encryption_defined_28fs_persistent_acls_29_Kfs_persistent_acls_defined_28fs_unicode_stored_on_disk_29_Kfs_unicode_stored_on_disk_defined_28fs_vol_is_compressed_29_Kfs_vol_is_compressed__29_29;
#endif
  gcv_object_t _object___28or_20integer_20_28member_defined_28s_ififo_29_Kfifo_defined_28s_ifsock_29_Kfsock_defined_28s_ifchr_29_Kfchr_defined_28s_ifdir_29_Kfdir_defined_28s_ifblk_29_Kfblk_defined_28s_ifreg_29_Kfreg__29_29;
  gcv_object_t _object___28or_20integer_20_28member_defined_28s_isuid_29_Ksuid_defined_28s_isgid_29_Ksgid_defined_28s_isvtx_29_Ksvtx_defined_28s_irwxu_29_Krwxu_defined_28s_irusr_29_Krusr_defined_28s_iwusr_29_Kwusr_defined_28s_ixusr_29_Kxusr_defined_28s_irwxg_29_Krwxg_defined_28s_irgrp_29_Krgrp_defined_28s_iwgrp_29_Kwgrp_defined_28s_ixgrp_29_Kxgrp_defined_28s_irwxo_29_Krwxo_defined_28s_iroth_29_Kroth_defined_28s_iwoth_29_Kwoth_defined_28s_ixoth_29_Kxoth__29_29;
#if defined(HAVE_CONFSTR)
  gcv_object_t _object___28or_20integer_20_28member_defined_28_cs_path_29_Kpath_defined_28_cs_posix_v6_ilp32_off32_cflags_29_Kposix_v6_ilp32_off32_cflags_defined_28_cs_posix_v6_ilp32_off32_ldflags_29_Kposix_v6_ilp32_off32_ldflags_defined_28_cs_posix_v6_ilp32_off32_libs_29_Kposix_v6_ilp32_off32_libs_defined_28_cs_posix_v6_ilp32_offbig_cflags_29_Kposix_v6_ilp32_offbig_cflags_defined_28_cs_posix_v6_ilp32_offbig_ldflags_29_Kposix_v6_ilp32_offbig_ldflags_defined_28_cs_posix_v6_ilp32_offbig_libs_29_Kposix_v6_ilp32_offbig_libs_defined_28_cs_posix_v6_lp64_off64_cflags_29_Kposix_v6_lp64_off64_cflags_defined_28_cs_posix_v6_lp64_off64_ldflags_29_Kposix_v6_lp64_off64_ldflags_defined_28_cs_posix_v6_lp64_off64_libs_29_Kposix_v6_lp64_off64_libs_defined_28_cs_posix_v6_lpbig_offbig_cflags_29_Kposix_v6_lpbig_offbig_cflags_defined_28_cs_posix_v6_lpbig_offbig_ldflags_29_Kposix_v6_lpbig_offbig_ldflags_defined_28_cs_posix_v6_lpbig_offbig_libs_29_Kposix_v6_lpbig_offbig_libs_defined_28_cs_posix_v6_width_restricted_envs_29_Kposix_v6_width_restricted_envs_defined_28_cs_v7_env_29_Kv7_env_defined_28_cs_posix_v7_ilp32_off32_cflags_29_Kposix_v7_ilp32_off32_cflags_defined_28_cs_posix_v7_ilp32_off32_ldflags_29_Kposix_v7_ilp32_off32_ldflags_defined_28_cs_posix_v7_ilp32_off32_libs_29_Kposix_v7_ilp32_off32_libs_defined_28_cs_posix_v7_ilp32_offbig_cflags_29_Kposix_v7_ilp32_offbig_cflags_defined_28_cs_posix_v7_ilp32_offbig_ldflags_29_Kposix_v7_ilp32_offbig_ldflags_defined_28_cs_posix_v7_ilp32_offbig_libs_29_Kposix_v7_ilp32_offbig_libs_defined_28_cs_posix_v7_lp64_off64_cflags_29_Kposix_v7_lp64_off64_cflags_defined_28_cs_posix_v7_lp64_off64_ldflags_29_Kposix_v7_lp64_off64_ldflags_defined_28_cs_posix_v7_lp64_off64_libs_29_Kposix_v7_lp64_off64_libs_defined_28_cs_posix_v7_lpbig_offbig_cflags_29_Kposix_v7_lpbig_offbig_cflags_defined_28_cs_posix_v7_lpbig_offbig_ldflags_29_Kposix_v7_lpbig_offbig_ldflags_defined_28_cs_posix_v7_lpbig_offbig_libs_29_Kposix_v7_lpbig_offbig_libs_defined_28_cs_posix_v7_threads_cflags_29_Kposix_v7_t;
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  gcv_object_t _object___28or_20integer_20_28member_defined_28_pc_filesizebits_29_Kfilesizebits_defined_28_pc_link_max_29_Klink_max_defined_28_pc_max_canon_29_Kmax_canon_defined_28_pc_max_input_29_Kmax_input_defined_28_pc_name_max_29_Kname_max_defined_28_pc_path_max_29_Kpath_max_defined_28_pc_pipe_buf_29_Kpipe_buf_defined_28_pc_2_symlinks_29_K2_symlinks_defined_28_pc_alloc_size_min_29_Kalloc_size_min_defined_28_pc_rec_incr_xfer_size_29_Krec_incr_xfer_size_defined_28_pc_rec_max_xfer_size_29_Krec_max_xfer_size_defined_28_pc_rec_min_xfer_size_29_Krec_min_xfer_size_defined_28_pc_rec_xfer_align_29_Krec_xfer_align_defined_28_pc_symlink_max_29_Ksymlink_max_defined_28_pc_chown_restricted_29_Kchown_restricted_defined_28_pc_no_trunc_29_Kno_trunc_defined_28_pc_vdisable_29_Kvdisable_defined_28_pc_async_io_29_Kasync_io_defined_28_pc_prio_io_29_Kprio_io_defined_28_pc_sync_io_29_Ksync_io_defined_28_pc_sock_maxbuf_29_Ksock_maxbuf__29_29;
#endif
#if defined(HAVE_SYSCONF)
  gcv_object_t _object___28or_20integer_20_28member_defined_28_sc_aio_listio_max_29_Kaio_listio_max_defined_28_sc_aio_max_29_Kaio_max_defined_28_sc_aio_prio_delta_max_29_Kaio_prio_delta_max_defined_28_sc_arg_max_29_Karg_max_defined_28_sc_atexit_max_29_Katexit_max_defined_28_sc_bc_base_max_29_Kbc_base_max_defined_28_sc_bc_dim_max_29_Kbc_dim_max_defined_28_sc_bc_scale_max_29_Kbc_scale_max_defined_28_sc_bc_string_max_29_Kbc_string_max_defined_28_sc_child_max_29_Kchild_max_defined_28_sc_clk_tck_29_Kclk_tck_defined_28_sc_coll_weights_max_29_Kcoll_weights_max_defined_28_sc_delaytimer_max_29_Kdelaytimer_max_defined_28_sc_expr_nest_max_29_Kexpr_nest_max_defined_28_sc_host_name_max_29_Khost_name_max_defined_28_sc_iov_max_29_Kiov_max_defined_28_sc_line_max_29_Kline_max_defined_28_sc_login_name_max_29_Klogin_name_max_defined_28_sc_ngroups_max_29_Kngroups_max_defined_28_sc_getgr_r_size_max_29_Kgetgr_r_size_max_defined_28_sc_getpw_r_size_max_29_Kgetpw_r_size_max_defined_28_sc_mq_open_max_29_Kmq_open_max_defined_28_sc_mq_prio_max_29_Kmq_prio_max_defined_28_sc_open_max_29_Kopen_max_defined_28_sc_advisory_info_29_Kadvisory_info_defined_28_sc_barriers_29_Kbarriers_defined_28_sc_asynchronous_io_29_Kasynchronous_io_defined_28_sc_clock_selection_29_Kclock_selection_defined_28_sc_cputime_29_Kcputime_defined_28_sc_fsync_29_Kfsync_defined_28_sc_ipv6_29_Kipv6_defined_28_sc_job_control_29_Kjob_control_defined_28_sc_mapped_files_29_Kmapped_files_defined_28_sc_memlock_29_Kmemlock_defined_28_sc_memlock_range_29_Kmemlock_range_defined_28_sc_memory_protection_29_Kmemory_protection_defined_28_sc_message_passing_29_Kmessage_passing_defined_28_sc_monotonic_clock_29_Kmonotonic_clock_defined_28_sc_prioritized_io_29_Kprioritized_io_defined_28_sc_priority_scheduling_29_Kpriority_scheduling_defined_28_sc_raw_sockets_29_Kraw_sockets_defined_28_sc_reader_writer_locks_29_Kreader_writer_locks_defined_28_sc_realtime_signals_29_Krealtime_signals_defined_28_sc_regexp_29_Kregexp_defined_28_sc_saved_ids_29_Ksaved_ids_defined_28;
#endif
#if defined(HAVE_SYSLOG)
  gcv_object_t _object___28or_20null_20integer_20_28member_defined_28log_kern_29_Kkern_defined_28log_user_29_Kuser_defined_28log_mail_29_Kmail_defined_28log_news_29_Knews_defined_28log_uucp_29_Kuucp_defined_28log_daemon_29_Kdaemon_defined_28log_auth_29_Kauth_defined_28log_cron_29_Kcron_defined_28log_lpr_29_Klpr_defined_28log_syslog_29_Ksyslog_defined_28log_authpriv_29_Kauthpriv_defined_28log_ftp_29_Kftp_defined_28log_local0_29_Klocal0_defined_28log_local1_29_Klocal1_defined_28log_local2_29_Klocal2_defined_28log_local3_29_Klocal3_defined_28log_local4_29_Klocal4_defined_28log_local5_29_Klocal5_defined_28log_local6_29_Klocal6_defined_28log_local7_29_Klocal7__29_29;
#endif
  gcv_object_t _object___28or_20null_20integer_20_28member_defined_28prio_process_29_Kprocess_defined_28prio_pgrp_29_Kpgrp_defined_28prio_user_29_Kuser__29_29;
#if defined(WIN32_NATIVE)
  gcv_object_t _object___28or_20null_20integer_20_28member_defined_28realtime_priority_class_29_Krealtime_defined_28high_priority_class_29_Khigh_defined_28above_normal_priority_class_29_Kabove_normal_defined_28normal_priority_class_29_Knormal_defined_28below_normal_priority_class_29_Kbelow_normal_defined_28low_priority_class_29_Klow_defined_28idle_priority_class_29_Kidle__29_29;
#endif
#if !(defined(WIN32_NATIVE))
  gcv_object_t _object___28or_20null_20integer_20_28member_Krealtime_Khigh_Kabove_normal_Knormal_Kbelow_normal_Klow_Kidle__29_29;
#endif
  gcv_object_t _my_stdin;
  gcv_object_t _my_stdout;
  gcv_object_t _my_stderr;
} module__syscalls__object_tab;
uintC module__syscalls__object_tab_size = sizeof(module__syscalls__object_tab)/sizeof(gcv_object_t);

struct module__syscalls__object_tab_initdata_t {
  object_initdata_t _object_cl_decode_universal_time;
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_CHAR_TERM))
  object_initdata_t _object_K2_char_term;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_C_BIND))
  object_initdata_t _object_K2_c_bind;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_C_DEV))
  object_initdata_t _object_K2_c_dev;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_FORT_DEV))
  object_initdata_t _object_K2_fort_dev;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_FORT_RUN))
  object_initdata_t _object_K2_fort_run;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_LOCALEDEF))
  object_initdata_t _object_K2_localedef;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS))
  object_initdata_t _object_K2_pbs;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_ACCOUNTING))
  object_initdata_t _object_K2_pbs_accounting;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_CHECKPOINT))
  object_initdata_t _object_K2_pbs_checkpoint;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_LOCATE))
  object_initdata_t _object_K2_pbs_locate;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_MESSAGE))
  object_initdata_t _object_K2_pbs_message;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_TRACK))
  object_initdata_t _object_K2_pbs_track;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_SW_DEV))
  object_initdata_t _object_K2_sw_dev;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_2_SYMLINKS))
  object_initdata_t _object_K2_symlinks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_UPE))
  object_initdata_t _object_K2_upe;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_VERSION))
  object_initdata_t _object_K2_version;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(ABOVE_NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  object_initdata_t _object_Kabove_normal;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(ACCOUNTING))
  object_initdata_t _object_Kaccounting;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ADVISORY_INFO))
  object_initdata_t _object_Kadvisory_info;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_LISTIO_MAX))
  object_initdata_t _object_Kaio_listio_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_MAX))
  object_initdata_t _object_Kaio_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_PRIO_DELTA_MAX))
  object_initdata_t _object_Kaio_prio_delta_max;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_ALERT))
  object_initdata_t _object_Kalert;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_ALLOC_SIZE_MIN))
  object_initdata_t _object_Kalloc_size_min;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ALPHA))
  object_initdata_t _object_Kalpha;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ALPHA64))
  object_initdata_t _object_Kalpha64;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kalt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_AMD64))
  object_initdata_t _object_Kamd64;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_APPEND))
  object_initdata_t _object_Kappend;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kappname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_ARCHIVE))
  object_initdata_t _object_Karchive;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Karguments;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ARG_MAX))
  object_initdata_t _object_Karg_max;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ARM))
  object_initdata_t _object_Karm;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_AS))
  object_initdata_t _object_Kas;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ASYNCHRONOUS_IO))
  object_initdata_t _object_Kasynchronous_io;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_ASYNC_IO))
  object_initdata_t _object_Kasync_io;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ATEXIT_MAX))
  object_initdata_t _object_Katexit_max;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  object_initdata_t _object_Katime;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_AUTH))
  object_initdata_t _object_Kauth;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kauthor;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_AUTHPRIV))
  object_initdata_t _object_Kauthpriv;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AVPHYS_PAGES))
  object_initdata_t _object_Kavphys_pages;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kbackoffice;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BARRIERS))
  object_initdata_t _object_Kbarriers;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_BASE_MAX))
  object_initdata_t _object_Kbc_base_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_DIM_MAX))
  object_initdata_t _object_Kbc_dim_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_SCALE_MAX))
  object_initdata_t _object_Kbc_scale_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_STRING_MAX))
  object_initdata_t _object_Kbc_string_max;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(BELOW_NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  object_initdata_t _object_Kbelow_normal;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_BINARY))
  object_initdata_t _object_Kbinary;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kblob;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  object_initdata_t _object_Kblock;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kbool;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(BOOT_TIME))
  object_initdata_t _object_Kboot_time;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kbstr;
#endif
  object_initdata_t _object_Kbuffered;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kbuilt_in;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kbyref;
#endif
  object_initdata_t _object_Kcase_sensitive;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kcharcount;
#endif
#if defined(HAVE_NFTW)
  object_initdata_t _object_Kchdir;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_CHILDREN))
  object_initdata_t _object_Kchildren;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CHILD_MAX))
  object_initdata_t _object_Kchild_max;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_CHOWN_RESTRICTED))
  object_initdata_t _object_Kchown_restricted;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kclipboard_format;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CLK_TCK))
  object_initdata_t _object_Kclk_tck;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CLOCK_SELECTION))
  object_initdata_t _object_Kclock_selection;
#endif
#if (defined(HAVE_FCNTL)) && (defined(FD_CLOEXEC))
  object_initdata_t _object_Kcloexec;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kcodepage;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_COLL_WEIGHTS_MAX))
  object_initdata_t _object_Kcoll_weights_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kcomments;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_COMPRESSED))
  object_initdata_t _object_Kcompressed;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  object_initdata_t _object_Kcons;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_Kcontinued;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kcontrol;
#endif
  object_initdata_t _object_Kcopy;
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_CORE))
  object_initdata_t _object_Kcore;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_CPU))
  object_initdata_t _object_Kcpu;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CPUTIME))
  object_initdata_t _object_Kcputime;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_CREAT))
  object_initdata_t _object_Kcreat;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kcreate_dtm;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_CRIT))
  object_initdata_t _object_Kcrit;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_CRON))
  object_initdata_t _object_Kcron;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kcy;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_D))
  object_initdata_t _object_Kd;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_DAEMON))
  object_initdata_t _object_Kdaemon;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_DATA))
  object_initdata_t _object_Kdata;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kdatacenter;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kdate;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(DEAD_PROCESS))
  object_initdata_t _object_Kdead_process;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_DEBUG))
  object_initdata_t _object_Kdebug;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_DELAYTIMER_MAX))
  object_initdata_t _object_Kdelaytimer_max;
#endif
#if defined(HAVE_NFTW)
  object_initdata_t _object_Kdepth;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kdescription;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_DEVICE))
  object_initdata_t _object_Kdevice;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_DIRECT))
  object_initdata_t _object_Kdirect;
#endif
  object_initdata_t _object_Kdirection;
#if ((defined(HAVE_FCNTL)) && (defined(O_DIRECTORY))) || ((defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_DIRECTORY)))
  object_initdata_t _object_Kdirectory;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_DNR))
  object_initdata_t _object_Kdnr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_AUTOZONE_ALREADY_EXISTS))
  object_initdata_t _object_Kdns_error_autozone_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_AXFR))
  object_initdata_t _object_Kdns_error_axfr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_BAD_PACKET))
  object_initdata_t _object_Kdns_error_bad_packet;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CANNOT_FIND_ROOT_HINTS))
  object_initdata_t _object_Kdns_error_cannot_find_root_hints;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CNAME_COLLISION))
  object_initdata_t _object_Kdns_error_cname_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CNAME_LOOP))
  object_initdata_t _object_Kdns_error_cname_loop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DATAFILE_OPEN_FAILURE))
  object_initdata_t _object_Kdns_error_datafile_open_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DATAFILE_PARSING))
  object_initdata_t _object_Kdns_error_datafile_parsing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_ALREADY_ENLISTED))
  object_initdata_t _object_Kdns_error_dp_already_enlisted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_ALREADY_EXISTS))
  object_initdata_t _object_Kdns_error_dp_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_DOES_NOT_EXIST))
  object_initdata_t _object_Kdns_error_dp_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_NOT_AVAILABLE))
  object_initdata_t _object_Kdns_error_dp_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_NOT_ENLISTED))
  object_initdata_t _object_Kdns_error_dp_not_enlisted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DS_UNAVAILABLE))
  object_initdata_t _object_Kdns_error_ds_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DS_ZONE_ALREADY_EXISTS))
  object_initdata_t _object_Kdns_error_ds_zone_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_FILE_WRITEBACK_FAILED))
  object_initdata_t _object_Kdns_error_file_writeback_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_FORWARDER_ALREADY_EXISTS))
  object_initdata_t _object_Kdns_error_forwarder_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INCONSISTENT_ROOT_HINTS))
  object_initdata_t _object_Kdns_error_inconsistent_root_hints;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_DATAFILE_NAME))
  object_initdata_t _object_Kdns_error_invalid_datafile_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_IP_ADDRESS))
  object_initdata_t _object_Kdns_error_invalid_ip_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_NAME_CHAR))
  object_initdata_t _object_Kdns_error_invalid_name_char;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_PROPERTY))
  object_initdata_t _object_Kdns_error_invalid_property;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_TYPE))
  object_initdata_t _object_Kdns_error_invalid_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_ZONE_OPERATION))
  object_initdata_t _object_Kdns_error_invalid_zone_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_ZONE_TYPE))
  object_initdata_t _object_Kdns_error_invalid_zone_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NAME_DOES_NOT_EXIST))
  object_initdata_t _object_Kdns_error_name_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NAME_NOT_IN_ZONE))
  object_initdata_t _object_Kdns_error_name_not_in_zone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NBSTAT_INIT_FAILED))
  object_initdata_t _object_Kdns_error_nbstat_init_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NEED_SECONDARY_ADDRESSES))
  object_initdata_t _object_Kdns_error_need_secondary_addresses;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NEED_WINS_SERVERS))
  object_initdata_t _object_Kdns_error_need_wins_servers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NODE_CREATION_FAILED))
  object_initdata_t _object_Kdns_error_node_creation_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NODE_IS_CNAME))
  object_initdata_t _object_Kdns_error_node_is_cname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NON_RFC_NAME))
  object_initdata_t _object_Kdns_error_non_rfc_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER))
  object_initdata_t _object_Kdns_error_not_allowed_on_root_server;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION))
  object_initdata_t _object_Kdns_error_not_allowed_under_delegation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_UNIQUE))
  object_initdata_t _object_Kdns_error_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE))
  object_initdata_t _object_Kdns_error_no_bootfile_if_ds_zone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_CREATE_CACHE_DATA))
  object_initdata_t _object_Kdns_error_no_create_cache_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_DNS_SERVERS))
  object_initdata_t _object_Kdns_error_no_dns_servers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_PACKET))
  object_initdata_t _object_Kdns_error_no_packet;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_TCPIP))
  object_initdata_t _object_Kdns_error_no_tcpip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_ZONE_INFO))
  object_initdata_t _object_Kdns_error_no_zone_info;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NUMERIC_NAME))
  object_initdata_t _object_Kdns_error_numeric_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_PRIMARY_REQUIRES_DATAFILE))
  object_initdata_t _object_Kdns_error_primary_requires_datafile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE))
  object_initdata_t _object_Kdns_error_rcode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADKEY))
  object_initdata_t _object_Kdns_error_rcode_badkey;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADSIG))
  object_initdata_t _object_Kdns_error_rcode_badsig;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADTIME))
  object_initdata_t _object_Kdns_error_rcode_badtime;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_FORMAT_ERROR))
  object_initdata_t _object_Kdns_error_rcode_format_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NAME_ERROR))
  object_initdata_t _object_Kdns_error_rcode_name_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOTAUTH))
  object_initdata_t _object_Kdns_error_rcode_notauth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOTZONE))
  object_initdata_t _object_Kdns_error_rcode_notzone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOT_IMPLEMENTED))
  object_initdata_t _object_Kdns_error_rcode_not_implemented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NXRRSET))
  object_initdata_t _object_Kdns_error_rcode_nxrrset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_REFUSED))
  object_initdata_t _object_Kdns_error_rcode_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_SERVER_FAILURE))
  object_initdata_t _object_Kdns_error_rcode_server_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_YXDOMAIN))
  object_initdata_t _object_Kdns_error_rcode_yxdomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_YXRRSET))
  object_initdata_t _object_Kdns_error_rcode_yxrrset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_ALREADY_EXISTS))
  object_initdata_t _object_Kdns_error_record_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_DOES_NOT_EXIST))
  object_initdata_t _object_Kdns_error_record_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_FORMAT))
  object_initdata_t _object_Kdns_error_record_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT))
  object_initdata_t _object_Kdns_error_record_only_at_zone_root;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_TIMED_OUT))
  object_initdata_t _object_Kdns_error_record_timed_out;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SECONDARY_DATA))
  object_initdata_t _object_Kdns_error_secondary_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP))
  object_initdata_t _object_Kdns_error_secondary_requires_master_ip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SOA_DELETE_INVALID))
  object_initdata_t _object_Kdns_error_soa_delete_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_TRY_AGAIN_LATER))
  object_initdata_t _object_Kdns_error_try_again_later;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_UNKNOWN_RECORD_TYPE))
  object_initdata_t _object_Kdns_error_unknown_record_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_UNSECURE_PACKET))
  object_initdata_t _object_Kdns_error_unsecure_packet;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_WINS_INIT_FAILED))
  object_initdata_t _object_Kdns_error_wins_init_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_ALREADY_EXISTS))
  object_initdata_t _object_Kdns_error_zone_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_CONFIGURATION_ERROR))
  object_initdata_t _object_Kdns_error_zone_configuration_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_CREATION_FAILED))
  object_initdata_t _object_Kdns_error_zone_creation_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_DOES_NOT_EXIST))
  object_initdata_t _object_Kdns_error_zone_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_HAS_NO_NS_RECORDS))
  object_initdata_t _object_Kdns_error_zone_has_no_ns_records;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_HAS_NO_SOA_RECORD))
  object_initdata_t _object_Kdns_error_zone_has_no_soa_record;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_IS_SHUTDOWN))
  object_initdata_t _object_Kdns_error_zone_is_shutdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_LOCKED))
  object_initdata_t _object_Kdns_error_zone_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_NOT_SECONDARY))
  object_initdata_t _object_Kdns_error_zone_not_secondary;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_REQUIRES_MASTER_IP))
  object_initdata_t _object_Kdns_error_zone_requires_master_ip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_ADDED_LOCAL_WINS))
  object_initdata_t _object_Kdns_info_added_local_wins;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_AXFR_COMPLETE))
  object_initdata_t _object_Kdns_info_axfr_complete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_NO_RECORDS))
  object_initdata_t _object_Kdns_info_no_records;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_CONTINUE_NEEDED))
  object_initdata_t _object_Kdns_status_continue_needed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_DOTTED_NAME))
  object_initdata_t _object_Kdns_status_dotted_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_FQDN))
  object_initdata_t _object_Kdns_status_fqdn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_SINGLE_PART_NAME))
  object_initdata_t _object_Kdns_status_single_part_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_WARNING_DOMAIN_UNDELETED))
  object_initdata_t _object_Kdns_warning_domain_undeleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_WARNING_PTR_CREATE_FAILED))
  object_initdata_t _object_Kdns_warning_ptr_create_failed;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kdoc_security;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kdomain_controller;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_DP))
  object_initdata_t _object_Kdp;
#endif
#if defined(E2BIG)
  object_initdata_t _object_Ke2big;
#endif
#if defined(EACCES)
  object_initdata_t _object_Keacces;
#endif
#if defined(EADDRINUSE)
  object_initdata_t _object_Keaddrinuse;
#endif
#if defined(EADDRNOTAVAIL)
  object_initdata_t _object_Keaddrnotavail;
#endif
#if defined(EADV)
  object_initdata_t _object_Keadv;
#endif
#if defined(EAFNOSUPPORT)
  object_initdata_t _object_Keafnosupport;
#endif
#if defined(EAGAIN)
  object_initdata_t _object_Keagain;
#endif
#if defined(EAIO)
  object_initdata_t _object_Keaio;
#endif
#if defined(EALIGN)
  object_initdata_t _object_Kealign;
#endif
#if defined(EALREADY)
  object_initdata_t _object_Kealready;
#endif
#if defined(EAUTH)
  object_initdata_t _object_Keauth;
#endif
#if defined(EBADE)
  object_initdata_t _object_Kebade;
#endif
#if defined(EBADF)
  object_initdata_t _object_Kebadf;
#endif
#if defined(EBADFD)
  object_initdata_t _object_Kebadfd;
#endif
#if defined(EBADMSG)
  object_initdata_t _object_Kebadmsg;
#endif
#if defined(EBADR)
  object_initdata_t _object_Kebadr;
#endif
#if defined(EBADRPC)
  object_initdata_t _object_Kebadrpc;
#endif
#if defined(EBADRQC)
  object_initdata_t _object_Kebadrqc;
#endif
#if defined(EBADSLT)
  object_initdata_t _object_Kebadslt;
#endif
#if defined(EBFONT)
  object_initdata_t _object_Kebfont;
#endif
#if defined(EBUSY)
  object_initdata_t _object_Kebusy;
#endif
#if defined(ECANCELED)
  object_initdata_t _object_Kecanceled;
#endif
#if defined(ECASECLASH)
  object_initdata_t _object_Kecaseclash;
#endif
#if defined(ECHILD)
  object_initdata_t _object_Kechild;
#endif
#if defined(ECHRNG)
  object_initdata_t _object_Kechrng;
#endif
#if defined(ECLONEME)
  object_initdata_t _object_Kecloneme;
#endif
#if defined(ECOMM)
  object_initdata_t _object_Kecomm;
#endif
#if defined(ECONNABORTED)
  object_initdata_t _object_Keconnaborted;
#endif
#if defined(ECONNREFUSED)
  object_initdata_t _object_Keconnrefused;
#endif
#if defined(ECONNRESET)
  object_initdata_t _object_Keconnreset;
#endif
#if defined(EDEADLK)
  object_initdata_t _object_Kedeadlk;
#endif
#if defined(EDEADLOCK)
  object_initdata_t _object_Kedeadlock;
#endif
#if defined(EDESTADDRREQ)
  object_initdata_t _object_Kedestaddrreq;
#endif
#if defined(EDIRIOCTL)
  object_initdata_t _object_Kedirioctl;
#endif
#if defined(EDIRTY)
  object_initdata_t _object_Kedirty;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kedittime;
#endif
#if defined(EDOM)
  object_initdata_t _object_Kedom;
#endif
#if defined(EDOOFUS)
  object_initdata_t _object_Kedoofus;
#endif
#if defined(EDOTDOT)
  object_initdata_t _object_Kedotdot;
#endif
#if defined(EDQUOT)
  object_initdata_t _object_Kedquot;
#endif
#if defined(EDUPPKG)
  object_initdata_t _object_Keduppkg;
#endif
#if defined(EEXIST)
  object_initdata_t _object_Keexist;
#endif
#if defined(EFAIL)
  object_initdata_t _object_Kefail;
#endif
#if defined(EFAULT)
  object_initdata_t _object_Kefault;
#endif
#if defined(EFBIG)
  object_initdata_t _object_Kefbig;
#endif
#if defined(EFTYPE)
  object_initdata_t _object_Keftype;
#endif
#if defined(EHOSTDOWN)
  object_initdata_t _object_Kehostdown;
#endif
#if defined(EHOSTUNREACH)
  object_initdata_t _object_Kehostunreach;
#endif
#if defined(EHWPOISON)
  object_initdata_t _object_Kehwpoison;
#endif
#if defined(EIDRM)
  object_initdata_t _object_Keidrm;
#endif
#if defined(EILSEQ)
  object_initdata_t _object_Keilseq;
#endif
#if defined(EINIT)
  object_initdata_t _object_Keinit;
#endif
#if defined(EINPROG)
  object_initdata_t _object_Keinprog;
#endif
#if defined(EINPROGRESS)
  object_initdata_t _object_Keinprogress;
#endif
#if defined(EINTR)
  object_initdata_t _object_Keintr;
#endif
#if defined(EINVAL)
  object_initdata_t _object_Keinval;
#endif
#if defined(EIO)
  object_initdata_t _object_Keio;
#endif
#if defined(EISCONN)
  object_initdata_t _object_Keisconn;
#endif
#if defined(EISDIR)
  object_initdata_t _object_Keisdir;
#endif
#if defined(EISNAM)
  object_initdata_t _object_Keisnam;
#endif
#if defined(EISNAME)
  object_initdata_t _object_Keisname;
#endif
#if defined(EJUSTRETURN)
  object_initdata_t _object_Kejustreturn;
#endif
#if defined(EKEYEXPIRED)
  object_initdata_t _object_Kekeyexpired;
#endif
#if defined(EKEYREJECTED)
  object_initdata_t _object_Kekeyrejected;
#endif
#if defined(EKEYREVOKED)
  object_initdata_t _object_Kekeyrevoked;
#endif
#if defined(EL2HLT)
  object_initdata_t _object_Kel2hlt;
#endif
#if defined(EL2NSYNC)
  object_initdata_t _object_Kel2nsync;
#endif
#if defined(EL3HLT)
  object_initdata_t _object_Kel3hlt;
#endif
#if defined(EL3RST)
  object_initdata_t _object_Kel3rst;
#endif
#if defined(ELAST)
  object_initdata_t _object_Kelast;
#endif
#if defined(ELBIN)
  object_initdata_t _object_Kelbin;
#endif
  object_initdata_t _object_Kelement_type;
#if defined(ELIBACC)
  object_initdata_t _object_Kelibacc;
#endif
#if defined(ELIBBAD)
  object_initdata_t _object_Kelibbad;
#endif
#if defined(ELIBEXEC)
  object_initdata_t _object_Kelibexec;
#endif
#if defined(ELIBMAX)
  object_initdata_t _object_Kelibmax;
#endif
#if defined(ELIBSCN)
  object_initdata_t _object_Kelibscn;
#endif
#if defined(ELNRNG)
  object_initdata_t _object_Kelnrng;
#endif
#if defined(ELOCKUNMAPPED)
  object_initdata_t _object_Kelockunmapped;
#endif
#if defined(ELOOP)
  object_initdata_t _object_Keloop;
#endif
#if defined(EMEDIUMTYPE)
  object_initdata_t _object_Kemediumtype;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_EMERG))
  object_initdata_t _object_Kemerg;
#endif
#if defined(EMFILE)
  object_initdata_t _object_Kemfile;
#endif
#if defined(EMLINK)
  object_initdata_t _object_Kemlink;
#endif
#if ((defined(HAVE_UTMPX_H)) && (defined(EMPTY))) || (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN))
  object_initdata_t _object_Kempty;
#endif
#if defined(EMSGSIZE)
  object_initdata_t _object_Kemsgsize;
#endif
#if defined(EMTIMERS)
  object_initdata_t _object_Kemtimers;
#endif
#if defined(EMULTIHOP)
  object_initdata_t _object_Kemultihop;
#endif
#if defined(ENAMETOOLONG)
  object_initdata_t _object_Kenametoolong;
#endif
#if defined(ENAVAIL)
  object_initdata_t _object_Kenavail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_ENCRYPTED))
  object_initdata_t _object_Kencrypted;
#endif
#if defined(ENEEDAUTH)
  object_initdata_t _object_Keneedauth;
#endif
#if defined(ENET)
  object_initdata_t _object_Kenet;
#endif
#if defined(ENETDOWN)
  object_initdata_t _object_Kenetdown;
#endif
#if defined(ENETRESET)
  object_initdata_t _object_Kenetreset;
#endif
#if defined(ENETUNREACH)
  object_initdata_t _object_Kenetunreach;
#endif
#if defined(ENFILE)
  object_initdata_t _object_Kenfile;
#endif
#if defined(ENMFILE)
  object_initdata_t _object_Kenmfile;
#endif
#if defined(ENOANO)
  object_initdata_t _object_Kenoano;
#endif
#if defined(ENOATTR)
  object_initdata_t _object_Kenoattr;
#endif
#if defined(ENOBUFS)
  object_initdata_t _object_Kenobufs;
#endif
#if defined(ENOCSI)
  object_initdata_t _object_Kenocsi;
#endif
#if defined(ENODATA)
  object_initdata_t _object_Kenodata;
#endif
#if defined(ENODEV)
  object_initdata_t _object_Kenodev;
#endif
#if defined(ENOENT)
  object_initdata_t _object_Kenoent;
#endif
#if defined(ENOEXEC)
  object_initdata_t _object_Kenoexec;
#endif
#if defined(ENOFILE)
  object_initdata_t _object_Kenofile;
#endif
#if defined(ENOIOCTL)
  object_initdata_t _object_Kenoioctl;
#endif
#if defined(ENOKEY)
  object_initdata_t _object_Kenokey;
#endif
#if defined(ENOLCK)
  object_initdata_t _object_Kenolck;
#endif
#if defined(ENOLINK)
  object_initdata_t _object_Kenolink;
#endif
#if defined(ENOMEDIUM)
  object_initdata_t _object_Kenomedium;
#endif
#if defined(ENOMEM)
  object_initdata_t _object_Kenomem;
#endif
#if defined(ENOMSG)
  object_initdata_t _object_Kenomsg;
#endif
#if defined(ENONET)
  object_initdata_t _object_Kenonet;
#endif
#if defined(ENOPKG)
  object_initdata_t _object_Kenopkg;
#endif
#if defined(ENOPROTOOPT)
  object_initdata_t _object_Kenoprotoopt;
#endif
#if defined(ENOSHARE)
  object_initdata_t _object_Kenoshare;
#endif
#if defined(ENOSPC)
  object_initdata_t _object_Kenospc;
#endif
#if defined(ENOSR)
  object_initdata_t _object_Kenosr;
#endif
#if defined(ENOSTR)
  object_initdata_t _object_Kenostr;
#endif
#if defined(ENOSYM)
  object_initdata_t _object_Kenosym;
#endif
#if defined(ENOSYS)
  object_initdata_t _object_Kenosys;
#endif
#if defined(ENOTACTIVE)
  object_initdata_t _object_Kenotactive;
#endif
#if defined(ENOTBLK)
  object_initdata_t _object_Kenotblk;
#endif
#if defined(ENOTCONN)
  object_initdata_t _object_Kenotconn;
#endif
#if defined(ENOTDIR)
  object_initdata_t _object_Kenotdir;
#endif
#if defined(ENOTEMPTY)
  object_initdata_t _object_Kenotempty;
#endif
#if defined(ENOTNAM)
  object_initdata_t _object_Kenotnam;
#endif
#if defined(ENOTRECOVERABLE)
  object_initdata_t _object_Kenotrecoverable;
#endif
#if defined(ENOTSOCK)
  object_initdata_t _object_Kenotsock;
#endif
#if defined(ENOTSUP)
  object_initdata_t _object_Kenotsup;
#endif
#if defined(ENOTTY)
  object_initdata_t _object_Kenotty;
#endif
#if defined(ENOTUNIQ)
  object_initdata_t _object_Kenotuniq;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kenterprise;
#endif
#if defined(ENXIO)
  object_initdata_t _object_Kenxio;
#endif
#if defined(EOPNOTSUPP)
  object_initdata_t _object_Keopnotsupp;
#endif
#if defined(EOVERFLOW)
  object_initdata_t _object_Keoverflow;
#endif
#if defined(EOWNERDEAD)
  object_initdata_t _object_Keownerdead;
#endif
#if defined(EPERM)
  object_initdata_t _object_Keperm;
#endif
#if defined(EPFNOSUPPORT)
  object_initdata_t _object_Kepfnosupport;
#endif
#if defined(EPIPE)
  object_initdata_t _object_Kepipe;
#endif
#if defined(EPROCLIM)
  object_initdata_t _object_Keproclim;
#endif
#if defined(EPROCUNAVAIL)
  object_initdata_t _object_Keprocunavail;
#endif
#if defined(EPROGMISMATCH)
  object_initdata_t _object_Keprogmismatch;
#endif
#if defined(EPROGUNAVAIL)
  object_initdata_t _object_Keprogunavail;
#endif
#if defined(EPROTO)
  object_initdata_t _object_Keproto;
#endif
#if defined(EPROTONOSUPPORT)
  object_initdata_t _object_Keprotonosupport;
#endif
#if defined(EPROTOTYPE)
  object_initdata_t _object_Keprototype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_CANT_CREATE))
  object_initdata_t _object_Kept_s_cant_create;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_CANT_PERFORM_OP))
  object_initdata_t _object_Kept_s_cant_perform_op;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_INVALID_ENTRY))
  object_initdata_t _object_Kept_s_invalid_entry;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_NOT_REGISTERED))
  object_initdata_t _object_Kept_s_not_registered;
#endif
#if defined(ERANGE)
  object_initdata_t _object_Kerange;
#endif
#if defined(EREMCHG)
  object_initdata_t _object_Keremchg;
#endif
#if defined(EREMDEV)
  object_initdata_t _object_Keremdev;
#endif
#if defined(EREMOTE)
  object_initdata_t _object_Keremote;
#endif
#if defined(EREMOTEIO)
  object_initdata_t _object_Keremoteio;
#endif
#if defined(EREMOTERELEASE)
  object_initdata_t _object_Keremoterelease;
#endif
#if defined(ERESTART)
  object_initdata_t _object_Kerestart;
#endif
#if defined(ERESTARTNOHAND)
  object_initdata_t _object_Kerestartnohand;
#endif
#if defined(ERESTARTNOINTR)
  object_initdata_t _object_Kerestartnointr;
#endif
#if defined(ERESTARTSYS)
  object_initdata_t _object_Kerestartsys;
#endif
#if defined(ERFKILL)
  object_initdata_t _object_Kerfkill;
#endif
#if defined(EROFS)
  object_initdata_t _object_Kerofs;
#endif
#if defined(ERPCMISMATCH)
  object_initdata_t _object_Kerpcmismatch;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_ERR))
  object_initdata_t _object_Kerr;
#endif
#if defined(ERREMOTE)
  object_initdata_t _object_Kerremote;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kerror;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DENIED))
  object_initdata_t _object_Kerror_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_BY_POLICY))
  object_initdata_t _object_Kerror_access_disabled_by_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_WEBBLADE))
  object_initdata_t _object_Kerror_access_disabled_webblade;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER))
  object_initdata_t _object_Kerror_access_disabled_webblade_tamper;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_DISABLED))
  object_initdata_t _object_Kerror_account_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_EXPIRED))
  object_initdata_t _object_Kerror_account_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_LOCKED_OUT))
  object_initdata_t _object_Kerror_account_locked_out;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_RESTRICTION))
  object_initdata_t _object_Kerror_account_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACTIVATION_COUNT_EXCEEDED))
  object_initdata_t _object_Kerror_activation_count_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACTIVE_CONNECTIONS))
  object_initdata_t _object_Kerror_active_connections;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADAP_HDW_ERR))
  object_initdata_t _object_Kerror_adap_hdw_err;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADDRESS_ALREADY_ASSOCIATED))
  object_initdata_t _object_Kerror_address_already_associated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADDRESS_NOT_ASSOCIATED))
  object_initdata_t _object_Kerror_address_not_associated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALIAS_EXISTS))
  object_initdata_t _object_Kerror_alias_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALLOTTED_SPACE_EXCEEDED))
  object_initdata_t _object_Kerror_allotted_space_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALL_NODES_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_all_nodes_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED))
  object_initdata_t _object_Kerror_all_user_trust_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_ASSIGNED))
  object_initdata_t _object_Kerror_already_assigned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_FIBER))
  object_initdata_t _object_Kerror_already_fiber;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_INITIALIZED))
  object_initdata_t _object_Kerror_already_initialized;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_REGISTERED))
  object_initdata_t _object_Kerror_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_RUNNING_LKG))
  object_initdata_t _object_Kerror_already_running_lkg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_THREAD))
  object_initdata_t _object_Kerror_already_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_WAITING))
  object_initdata_t _object_Kerror_already_waiting;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_APPHELP_BLOCK))
  object_initdata_t _object_Kerror_apphelp_block;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_APP_WRONG_OS))
  object_initdata_t _object_Kerror_app_wrong_os;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ARENA_TRASHED))
  object_initdata_t _object_Kerror_arena_trashed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ARITHMETIC_OVERFLOW))
  object_initdata_t _object_Kerror_arithmetic_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ATOMIC_LOCKS_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_atomic_locks_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_AUTODATASEG_EXCEEDS_64k))
  object_initdata_t _object_Kerror_autodataseg_exceeds_64k;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BADDB))
  object_initdata_t _object_Kerror_baddb;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BADKEY))
  object_initdata_t _object_Kerror_badkey;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_ARGUMENTS))
  object_initdata_t _object_Kerror_bad_arguments;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_COMMAND))
  object_initdata_t _object_Kerror_bad_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_CONFIGURATION))
  object_initdata_t _object_Kerror_bad_configuration;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DESCRIPTOR_FORMAT))
  object_initdata_t _object_Kerror_bad_descriptor_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DEVICE))
  object_initdata_t _object_Kerror_bad_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DEV_TYPE))
  object_initdata_t _object_Kerror_bad_dev_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DRIVER))
  object_initdata_t _object_Kerror_bad_driver;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DRIVER_LEVEL))
  object_initdata_t _object_Kerror_bad_driver_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_ENVIRONMENT))
  object_initdata_t _object_Kerror_bad_environment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_EXE_FORMAT))
  object_initdata_t _object_Kerror_bad_exe_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_FORMAT))
  object_initdata_t _object_Kerror_bad_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_IMPERSONATION_LEVEL))
  object_initdata_t _object_Kerror_bad_impersonation_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_INHERITANCE_ACL))
  object_initdata_t _object_Kerror_bad_inheritance_acl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_LENGTH))
  object_initdata_t _object_Kerror_bad_length;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_LOGON_SESSION_STATE))
  object_initdata_t _object_Kerror_bad_logon_session_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NETPATH))
  object_initdata_t _object_Kerror_bad_netpath;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NET_NAME))
  object_initdata_t _object_Kerror_bad_net_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NET_RESP))
  object_initdata_t _object_Kerror_bad_net_resp;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PATHNAME))
  object_initdata_t _object_Kerror_bad_pathname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PIPE))
  object_initdata_t _object_Kerror_bad_pipe;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PROFILE))
  object_initdata_t _object_Kerror_bad_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PROVIDER))
  object_initdata_t _object_Kerror_bad_provider;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_QUERY_SYNTAX))
  object_initdata_t _object_Kerror_bad_query_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_RECOVERY_POLICY))
  object_initdata_t _object_Kerror_bad_recovery_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_REM_ADAP))
  object_initdata_t _object_Kerror_bad_rem_adap;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_THREADID_ADDR))
  object_initdata_t _object_Kerror_bad_threadid_addr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_TOKEN_TYPE))
  object_initdata_t _object_Kerror_bad_token_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_UNIT))
  object_initdata_t _object_Kerror_bad_unit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_USERNAME))
  object_initdata_t _object_Kerror_bad_username;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_USER_PROFILE))
  object_initdata_t _object_Kerror_bad_user_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_VALIDATION_CLASS))
  object_initdata_t _object_Kerror_bad_validation_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BEGINNING_OF_MEDIA))
  object_initdata_t _object_Kerror_beginning_of_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BOOT_ALREADY_ACCEPTED))
  object_initdata_t _object_Kerror_boot_already_accepted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BROKEN_PIPE))
  object_initdata_t _object_Kerror_broken_pipe;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUFFER_OVERFLOW))
  object_initdata_t _object_Kerror_buffer_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUSY))
  object_initdata_t _object_Kerror_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUSY_DRIVE))
  object_initdata_t _object_Kerror_busy_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUS_RESET))
  object_initdata_t _object_Kerror_bus_reset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CALLBACK_SUPPLIED_INVALID_DATA))
  object_initdata_t _object_Kerror_callback_supplied_invalid_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CALL_NOT_IMPLEMENTED))
  object_initdata_t _object_Kerror_call_not_implemented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANCELLED))
  object_initdata_t _object_Kerror_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANCEL_VIOLATION))
  object_initdata_t _object_Kerror_cancel_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_COPY))
  object_initdata_t _object_Kerror_cannot_copy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_DETECT_DRIVER_FAILURE))
  object_initdata_t _object_Kerror_cannot_detect_driver_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_DETECT_PROCESS_ABORT))
  object_initdata_t _object_Kerror_cannot_detect_process_abort;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_FIND_WND_CLASS))
  object_initdata_t _object_Kerror_cannot_find_wnd_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_IMPERSONATE))
  object_initdata_t _object_Kerror_cannot_impersonate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_MAKE))
  object_initdata_t _object_Kerror_cannot_make;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_OPEN_PROFILE))
  object_initdata_t _object_Kerror_cannot_open_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTOPEN))
  object_initdata_t _object_Kerror_cantopen;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTREAD))
  object_initdata_t _object_Kerror_cantread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTWRITE))
  object_initdata_t _object_Kerror_cantwrite;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_ACCESS_DOMAIN_INFO))
  object_initdata_t _object_Kerror_cant_access_domain_info;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_ACCESS_FILE))
  object_initdata_t _object_Kerror_cant_access_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_DELETE_LAST_ITEM))
  object_initdata_t _object_Kerror_cant_delete_last_item;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_DISABLE_MANDATORY))
  object_initdata_t _object_Kerror_cant_disable_mandatory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_EVICT_ACTIVE_NODE))
  object_initdata_t _object_Kerror_cant_evict_active_node;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_OPEN_ANONYMOUS))
  object_initdata_t _object_Kerror_cant_open_anonymous;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_RESOLVE_FILENAME))
  object_initdata_t _object_Kerror_cant_resolve_filename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CAN_NOT_COMPLETE))
  object_initdata_t _object_Kerror_can_not_complete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CAN_NOT_DEL_LOCAL_WINS))
  object_initdata_t _object_Kerror_can_not_del_local_wins;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_MUST_BE_VOLATILE))
  object_initdata_t _object_Kerror_child_must_be_volatile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_NOT_COMPLETE))
  object_initdata_t _object_Kerror_child_not_complete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_WINDOW_MENU))
  object_initdata_t _object_Kerror_child_window_menu;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CIRCULAR_DEPENDENCY))
  object_initdata_t _object_Kerror_circular_dependency;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_class_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_DOES_NOT_EXIST))
  object_initdata_t _object_Kerror_class_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_HAS_WINDOWS))
  object_initdata_t _object_Kerror_class_has_windows;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_CARTRIDGE_INSTALLED))
  object_initdata_t _object_Kerror_cleaner_cartridge_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_CARTRIDGE_SPENT))
  object_initdata_t _object_Kerror_cleaner_cartridge_spent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_SLOT_NOT_SET))
  object_initdata_t _object_Kerror_cleaner_slot_not_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_SLOT_SET))
  object_initdata_t _object_Kerror_cleaner_slot_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLIPBOARD_NOT_OPEN))
  object_initdata_t _object_Kerror_clipboard_not_open;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLIPPING_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_clipping_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_ALREADY_COMMITTED))
  object_initdata_t _object_Kerror_cluscfg_already_committed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_ROLLBACK_FAILED))
  object_initdata_t _object_Kerror_cluscfg_rollback_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT))
  object_initdata_t _object_Kerror_cluscfg_system_disk_drive_letter_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND))
  object_initdata_t _object_Kerror_clusterlog_chkpoint_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_CORRUPT))
  object_initdata_t _object_Kerror_clusterlog_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE))
  object_initdata_t _object_Kerror_clusterlog_exceeds_maxsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE))
  object_initdata_t _object_Kerror_clusterlog_not_enough_space;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE))
  object_initdata_t _object_Kerror_clusterlog_record_exceeds_maxsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME))
  object_initdata_t _object_Kerror_cluster_cant_create_dup_cluster_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_DATABASE_SEQMISMATCH))
  object_initdata_t _object_Kerror_cluster_database_seqmismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP))
  object_initdata_t _object_Kerror_cluster_evict_without_cleanup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_GUM_NOT_LOCKER))
  object_initdata_t _object_Kerror_cluster_gum_not_locker;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INCOMPATIBLE_VERSIONS))
  object_initdata_t _object_Kerror_cluster_incompatible_versions;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INSTANCE_ID_MISMATCH))
  object_initdata_t _object_Kerror_cluster_instance_id_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NETWORK))
  object_initdata_t _object_Kerror_cluster_invalid_network;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NETWORK_PROVIDER))
  object_initdata_t _object_Kerror_cluster_invalid_network_provider;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NODE))
  object_initdata_t _object_Kerror_cluster_invalid_node;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_REQUEST))
  object_initdata_t _object_Kerror_cluster_invalid_request;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_IPADDR_IN_USE))
  object_initdata_t _object_Kerror_cluster_ipaddr_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_ABORTED))
  object_initdata_t _object_Kerror_cluster_join_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_IN_PROGRESS))
  object_initdata_t _object_Kerror_cluster_join_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS))
  object_initdata_t _object_Kerror_cluster_join_not_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_LAST_INTERNAL_NETWORK))
  object_initdata_t _object_Kerror_cluster_last_internal_network;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND))
  object_initdata_t _object_Kerror_cluster_local_node_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED))
  object_initdata_t _object_Kerror_cluster_maxnum_of_resources_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MEMBERSHIP_HALT))
  object_initdata_t _object_Kerror_cluster_membership_halt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE))
  object_initdata_t _object_Kerror_cluster_membership_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME))
  object_initdata_t _object_Kerror_cluster_mismatched_computer_acct_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETINTERFACE_EXISTS))
  object_initdata_t _object_Kerror_cluster_netinterface_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETINTERFACE_NOT_FOUND))
  object_initdata_t _object_Kerror_cluster_netinterface_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE))
  object_initdata_t _object_Kerror_cluster_network_already_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_ALREADY_ONLINE))
  object_initdata_t _object_Kerror_cluster_network_already_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_EXISTS))
  object_initdata_t _object_Kerror_cluster_network_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS))
  object_initdata_t _object_Kerror_cluster_network_has_dependents;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_FOUND))
  object_initdata_t _object_Kerror_cluster_network_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP))
  object_initdata_t _object_Kerror_cluster_network_not_found_for_ip;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_INTERNAL))
  object_initdata_t _object_Kerror_cluster_network_not_internal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_DOWN))
  object_initdata_t _object_Kerror_cluster_node_already_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT))
  object_initdata_t _object_Kerror_cluster_node_already_has_dfs_root;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_MEMBER))
  object_initdata_t _object_Kerror_cluster_node_already_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_UP))
  object_initdata_t _object_Kerror_cluster_node_already_up;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_DOWN))
  object_initdata_t _object_Kerror_cluster_node_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_EXISTS))
  object_initdata_t _object_Kerror_cluster_node_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_FOUND))
  object_initdata_t _object_Kerror_cluster_node_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_MEMBER))
  object_initdata_t _object_Kerror_cluster_node_not_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_PAUSED))
  object_initdata_t _object_Kerror_cluster_node_not_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_READY))
  object_initdata_t _object_Kerror_cluster_node_not_ready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_PAUSED))
  object_initdata_t _object_Kerror_cluster_node_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_SHUTTING_DOWN))
  object_initdata_t _object_Kerror_cluster_node_shutting_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_UNREACHABLE))
  object_initdata_t _object_Kerror_cluster_node_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_UP))
  object_initdata_t _object_Kerror_cluster_node_up;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED))
  object_initdata_t _object_Kerror_cluster_no_rpc_packages_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NO_SECURITY_CONTEXT))
  object_initdata_t _object_Kerror_cluster_no_security_context;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_OLD_VERSION))
  object_initdata_t _object_Kerror_cluster_old_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST))
  object_initdata_t _object_Kerror_cluster_owner_not_in_preflist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_PARAMETER_MISMATCH))
  object_initdata_t _object_Kerror_cluster_parameter_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH))
  object_initdata_t _object_Kerror_cluster_property_data_type_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_QUORUMLOG_NOT_FOUND))
  object_initdata_t _object_Kerror_cluster_quorumlog_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESNAME_NOT_FOUND))
  object_initdata_t _object_Kerror_cluster_resname_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND))
  object_initdata_t _object_Kerror_cluster_resource_type_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_cluster_restype_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_SHUTTING_DOWN))
  object_initdata_t _object_Kerror_cluster_shutting_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED))
  object_initdata_t _object_Kerror_cluster_system_config_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_WRONG_OS_VERSION))
  object_initdata_t _object_Kerror_cluster_wrong_os_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COLORSPACE_MISMATCH))
  object_initdata_t _object_Kerror_colorspace_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COMMITMENT_LIMIT))
  object_initdata_t _object_Kerror_commitment_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTED_OTHER_PASSWORD))
  object_initdata_t _object_Kerror_connected_other_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT))
  object_initdata_t _object_Kerror_connected_other_password_default;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_ABORTED))
  object_initdata_t _object_Kerror_connection_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_ACTIVE))
  object_initdata_t _object_Kerror_connection_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_COUNT_LIMIT))
  object_initdata_t _object_Kerror_connection_count_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_INVALID))
  object_initdata_t _object_Kerror_connection_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_REFUSED))
  object_initdata_t _object_Kerror_connection_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_UNAVAIL))
  object_initdata_t _object_Kerror_connection_unavail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTEXT_EXPIRED))
  object_initdata_t _object_Kerror_context_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTINUE))
  object_initdata_t _object_Kerror_continue;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTROLLING_IEPORT))
  object_initdata_t _object_Kerror_controlling_ieport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTROL_ID_NOT_FOUND))
  object_initdata_t _object_Kerror_control_id_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CORE_RESOURCE))
  object_initdata_t _object_Kerror_core_resource;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COUNTER_TIMEOUT))
  object_initdata_t _object_Kerror_counter_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CRC))
  object_initdata_t _object_Kerror_crc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CREATE_FAILED))
  object_initdata_t _object_Kerror_create_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CSCSHARE_OFFLINE))
  object_initdata_t _object_Kerror_cscshare_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_BAD_VIDEO_MODE))
  object_initdata_t _object_Kerror_ctx_bad_video_mode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY))
  object_initdata_t _object_Kerror_ctx_cannot_make_eventlog_entry;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_LICENSE_IN_USE))
  object_initdata_t _object_Kerror_ctx_client_license_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_LICENSE_NOT_SET))
  object_initdata_t _object_Kerror_ctx_client_license_not_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_QUERY_TIMEOUT))
  object_initdata_t _object_Kerror_ctx_client_query_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLOSE_PENDING))
  object_initdata_t _object_Kerror_ctx_close_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CONSOLE_CONNECT))
  object_initdata_t _object_Kerror_ctx_console_connect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CONSOLE_DISCONNECT))
  object_initdata_t _object_Kerror_ctx_console_disconnect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_GRAPHICS_INVALID))
  object_initdata_t _object_Kerror_ctx_graphics_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_MODEMNAME))
  object_initdata_t _object_Kerror_ctx_invalid_modemname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_PD))
  object_initdata_t _object_Kerror_ctx_invalid_pd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_WD))
  object_initdata_t _object_Kerror_ctx_invalid_wd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_CLIENT_INVALID))
  object_initdata_t _object_Kerror_ctx_license_client_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_EXPIRED))
  object_initdata_t _object_Kerror_ctx_license_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_ctx_license_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LOGON_DISABLED))
  object_initdata_t _object_Kerror_ctx_logon_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_INF_NOT_FOUND))
  object_initdata_t _object_Kerror_ctx_modem_inf_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_BUSY))
  object_initdata_t _object_Kerror_ctx_modem_response_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_ERROR))
  object_initdata_t _object_Kerror_ctx_modem_response_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_NO_CARRIER))
  object_initdata_t _object_Kerror_ctx_modem_response_no_carrier;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE))
  object_initdata_t _object_Kerror_ctx_modem_response_no_dialtone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_TIMEOUT))
  object_initdata_t _object_Kerror_ctx_modem_response_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_VOICE))
  object_initdata_t _object_Kerror_ctx_modem_response_voice;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_NOT_CONSOLE))
  object_initdata_t _object_Kerror_ctx_not_console;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_NO_OUTBUF))
  object_initdata_t _object_Kerror_ctx_no_outbuf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_PD_NOT_FOUND))
  object_initdata_t _object_Kerror_ctx_pd_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SERVICE_NAME_COLLISION))
  object_initdata_t _object_Kerror_ctx_service_name_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_DENIED))
  object_initdata_t _object_Kerror_ctx_shadow_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_DISABLED))
  object_initdata_t _object_Kerror_ctx_shadow_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE))
  object_initdata_t _object_Kerror_ctx_shadow_ended_by_mode_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_INVALID))
  object_initdata_t _object_Kerror_ctx_shadow_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_NOT_RUNNING))
  object_initdata_t _object_Kerror_ctx_shadow_not_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_TD_ERROR))
  object_initdata_t _object_Kerror_ctx_td_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WD_NOT_FOUND))
  object_initdata_t _object_Kerror_ctx_wd_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_ACCESS_DENIED))
  object_initdata_t _object_Kerror_ctx_winstation_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_ctx_winstation_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_BUSY))
  object_initdata_t _object_Kerror_ctx_winstation_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_NAME_INVALID))
  object_initdata_t _object_Kerror_ctx_winstation_name_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_NOT_FOUND))
  object_initdata_t _object_Kerror_ctx_winstation_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CURRENT_DIRECTORY))
  object_initdata_t _object_Kerror_current_directory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CURRENT_DOMAIN_NOT_ALLOWED))
  object_initdata_t _object_Kerror_current_domain_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_BACKUP_CORRUPT))
  object_initdata_t _object_Kerror_database_backup_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_DOES_NOT_EXIST))
  object_initdata_t _object_Kerror_database_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_FAILURE))
  object_initdata_t _object_Kerror_database_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_FULL))
  object_initdata_t _object_Kerror_database_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATATYPE_MISMATCH))
  object_initdata_t _object_Kerror_datatype_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DC_NOT_FOUND))
  object_initdata_t _object_Kerror_dc_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DDE_FAIL))
  object_initdata_t _object_Kerror_dde_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEBUGGER_INACTIVE))
  object_initdata_t _object_Kerror_debugger_inactive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DECRYPTION_FAILED))
  object_initdata_t _object_Kerror_decryption_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DELETE_PENDING))
  object_initdata_t _object_Kerror_delete_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DELETING_ICM_XFORM))
  object_initdata_t _object_Kerror_deleting_icm_xform;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_dependency_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_NOT_ALLOWED))
  object_initdata_t _object_Kerror_dependency_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_NOT_FOUND))
  object_initdata_t _object_Kerror_dependency_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENT_RESOURCE_EXISTS))
  object_initdata_t _object_Kerror_dependent_resource_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENT_SERVICES_RUNNING))
  object_initdata_t _object_Kerror_dependent_services_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DESTINATION_ELEMENT_FULL))
  object_initdata_t _object_Kerror_destination_element_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DESTROY_OBJECT_OF_OTHER_THREAD))
  object_initdata_t _object_Kerror_destroy_object_of_other_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_ALREADY_REMEMBERED))
  object_initdata_t _object_Kerror_device_already_remembered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_DOOR_OPEN))
  object_initdata_t _object_Kerror_device_door_open;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_IN_USE))
  object_initdata_t _object_Kerror_device_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_device_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_CONNECTED))
  object_initdata_t _object_Kerror_device_not_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_PARTITIONED))
  object_initdata_t _object_Kerror_device_not_partitioned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REINITIALIZATION_NEEDED))
  object_initdata_t _object_Kerror_device_reinitialization_needed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REMOVED))
  object_initdata_t _object_Kerror_device_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REQUIRES_CLEANING))
  object_initdata_t _object_Kerror_device_requires_cleaning;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEV_NOT_EXIST))
  object_initdata_t _object_Kerror_dev_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DHCP_ADDRESS_CONFLICT))
  object_initdata_t _object_Kerror_dhcp_address_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIFFERENT_SERVICE_ACCOUNT))
  object_initdata_t _object_Kerror_different_service_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIRECTORY))
  object_initdata_t _object_Kerror_directory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIRECT_ACCESS_HANDLE))
  object_initdata_t _object_Kerror_direct_access_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_EFS_DISALLOWED))
  object_initdata_t _object_Kerror_dir_efs_disallowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_NOT_EMPTY))
  object_initdata_t _object_Kerror_dir_not_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_NOT_ROOT))
  object_initdata_t _object_Kerror_dir_not_root;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISCARDED))
  object_initdata_t _object_Kerror_discarded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_CHANGE))
  object_initdata_t _object_Kerror_disk_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_CORRUPT))
  object_initdata_t _object_Kerror_disk_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_FULL))
  object_initdata_t _object_Kerror_disk_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_OPERATION_FAILED))
  object_initdata_t _object_Kerror_disk_operation_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_RECALIBRATE_FAILED))
  object_initdata_t _object_Kerror_disk_recalibrate_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_RESET_FAILED))
  object_initdata_t _object_Kerror_disk_reset_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_TOO_FRAGMENTED))
  object_initdata_t _object_Kerror_disk_too_fragmented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DLL_INIT_FAILED))
  object_initdata_t _object_Kerror_dll_init_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DLL_NOT_FOUND))
  object_initdata_t _object_Kerror_dll_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_CONTROLLER_EXISTS))
  object_initdata_t _object_Kerror_domain_controller_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_CONTROLLER_NOT_FOUND))
  object_initdata_t _object_Kerror_domain_controller_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_EXISTS))
  object_initdata_t _object_Kerror_domain_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_LIMIT_EXCEEDED))
  object_initdata_t _object_Kerror_domain_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_TRUST_INCONSISTENT))
  object_initdata_t _object_Kerror_domain_trust_inconsistent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOWNGRADE_DETECTED))
  object_initdata_t _object_Kerror_downgrade_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVER_BLOCKED))
  object_initdata_t _object_Kerror_driver_blocked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVE_LOCKED))
  object_initdata_t _object_Kerror_drive_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVE_MEDIA_MISMATCH))
  object_initdata_t _object_Kerror_drive_media_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ADD_REPLICA_INHIBITED))
  object_initdata_t _object_Kerror_ds_add_replica_inhibited;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ADMIN_LIMIT_EXCEEDED))
  object_initdata_t _object_Kerror_ds_admin_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AFFECTS_MULTIPLE_DSAS))
  object_initdata_t _object_Kerror_ds_affects_multiple_dsas;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER))
  object_initdata_t _object_Kerror_ds_ag_cant_have_universal_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIASED_OBJ_MISSING))
  object_initdata_t _object_Kerror_ds_aliased_obj_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_DEREF_PROBLEM))
  object_initdata_t _object_Kerror_ds_alias_deref_problem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_POINTS_TO_ALIAS))
  object_initdata_t _object_Kerror_ds_alias_points_to_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_PROBLEM))
  object_initdata_t _object_Kerror_ds_alias_problem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS))
  object_initdata_t _object_Kerror_ds_attribute_or_value_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_OWNED_BY_SAM))
  object_initdata_t _object_Kerror_ds_attribute_owned_by_sam;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED))
  object_initdata_t _object_Kerror_ds_attribute_type_undefined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_ds_att_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_IS_NOT_ON_OBJ))
  object_initdata_t _object_Kerror_ds_att_is_not_on_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_NOT_DEF_FOR_CLASS))
  object_initdata_t _object_Kerror_ds_att_not_def_for_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_NOT_DEF_IN_SCHEMA))
  object_initdata_t _object_Kerror_ds_att_not_def_in_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_SCHEMA_REQ_ID))
  object_initdata_t _object_Kerror_ds_att_schema_req_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_SCHEMA_REQ_SYNTAX))
  object_initdata_t _object_Kerror_ds_att_schema_req_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_VAL_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_ds_att_val_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTHORIZATION_FAILED))
  object_initdata_t _object_Kerror_ds_authorization_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTH_METHOD_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_ds_auth_method_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTH_UNKNOWN))
  object_initdata_t _object_Kerror_ds_auth_unknown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUX_CLS_TEST_FAIL))
  object_initdata_t _object_Kerror_ds_aux_cls_test_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BACKLINK_WITHOUT_LINK))
  object_initdata_t _object_Kerror_ds_backlink_without_link;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_ATT_SCHEMA_SYNTAX))
  object_initdata_t _object_Kerror_ds_bad_att_schema_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_HIERARCHY_FILE))
  object_initdata_t _object_Kerror_ds_bad_hierarchy_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_INSTANCE_TYPE))
  object_initdata_t _object_Kerror_ds_bad_instance_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_NAME_SYNTAX))
  object_initdata_t _object_Kerror_ds_bad_name_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_RDN_ATT_ID_SYNTAX))
  object_initdata_t _object_Kerror_ds_bad_rdn_att_id_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED))
  object_initdata_t _object_Kerror_ds_build_hierarchy_table_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BUSY))
  object_initdata_t _object_Kerror_ds_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD))
  object_initdata_t _object_Kerror_ds_cant_access_remote_part_of_ad;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_ATT_VALUES))
  object_initdata_t _object_Kerror_ds_cant_add_att_values;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_SYSTEM_ONLY))
  object_initdata_t _object_Kerror_ds_cant_add_system_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_TO_GC))
  object_initdata_t _object_Kerror_ds_cant_add_to_gc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CACHE_ATT))
  object_initdata_t _object_Kerror_ds_cant_cache_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CACHE_CLASS))
  object_initdata_t _object_Kerror_ds_cant_cache_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC))
  object_initdata_t _object_Kerror_ds_cant_create_in_nondomain_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CREATE_UNDER_SCHEMA))
  object_initdata_t _object_Kerror_ds_cant_create_under_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DELETE))
  object_initdata_t _object_Kerror_ds_cant_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DELETE_DSA_OBJ))
  object_initdata_t _object_Kerror_ds_cant_delete_dsa_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEL_MASTER_CROSSREF))
  object_initdata_t _object_Kerror_ds_cant_del_master_crossref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC))
  object_initdata_t _object_Kerror_ds_cant_demote_with_writeable_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEREF_ALIAS))
  object_initdata_t _object_Kerror_ds_cant_deref_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN))
  object_initdata_t _object_Kerror_ds_cant_derive_spn_for_deleted_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF))
  object_initdata_t _object_Kerror_ds_cant_derive_spn_without_server_ref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN))
  object_initdata_t _object_Kerror_ds_cant_find_dc_for_src_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_DSA_OBJ))
  object_initdata_t _object_Kerror_ds_cant_find_dsa_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_EXPECTED_NC))
  object_initdata_t _object_Kerror_ds_cant_find_expected_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_NC_IN_CACHE))
  object_initdata_t _object_Kerror_ds_cant_find_nc_in_cache;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MIX_MASTER_AND_REPS))
  object_initdata_t _object_Kerror_ds_cant_mix_master_and_reps;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_OBJ_CLASS))
  object_initdata_t _object_Kerror_ds_cant_mod_obj_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_PRIMARYGROUPID))
  object_initdata_t _object_Kerror_ds_cant_mod_primarygroupid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_SYSTEM_ONLY))
  object_initdata_t _object_Kerror_ds_cant_mod_system_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_ACCOUNT_GROUP))
  object_initdata_t _object_Kerror_ds_cant_move_account_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_APP_BASIC_GROUP))
  object_initdata_t _object_Kerror_ds_cant_move_app_basic_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_APP_QUERY_GROUP))
  object_initdata_t _object_Kerror_ds_cant_move_app_query_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_DELETED_OBJECT))
  object_initdata_t _object_Kerror_ds_cant_move_deleted_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_RESOURCE_GROUP))
  object_initdata_t _object_Kerror_ds_cant_move_resource_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ON_NON_LEAF))
  object_initdata_t _object_Kerror_ds_cant_on_non_leaf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ON_RDN))
  object_initdata_t _object_Kerror_ds_cant_on_rdn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REMOVE_ATT_CACHE))
  object_initdata_t _object_Kerror_ds_cant_remove_att_cache;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REMOVE_CLASS_CACHE))
  object_initdata_t _object_Kerror_ds_cant_remove_class_cache;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REM_MISSING_ATT))
  object_initdata_t _object_Kerror_ds_cant_rem_missing_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REM_MISSING_ATT_VAL))
  object_initdata_t _object_Kerror_ds_cant_rem_missing_att_val;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REPLACE_HIDDEN_REC))
  object_initdata_t _object_Kerror_ds_cant_replace_hidden_rec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_ATTS))
  object_initdata_t _object_Kerror_ds_cant_retrieve_atts;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_CHILD))
  object_initdata_t _object_Kerror_ds_cant_retrieve_child;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_DN))
  object_initdata_t _object_Kerror_ds_cant_retrieve_dn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_INSTANCE))
  object_initdata_t _object_Kerror_ds_cant_retrieve_instance;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_SD))
  object_initdata_t _object_Kerror_ds_cant_retrieve_sd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_START))
  object_initdata_t _object_Kerror_ds_cant_start;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ))
  object_initdata_t _object_Kerror_ds_cant_tree_delete_critical_obj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS))
  object_initdata_t _object_Kerror_ds_cant_with_acct_group_membershps;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CHILDREN_EXIST))
  object_initdata_t _object_Kerror_ds_children_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLASS_MUST_BE_CONCRETE))
  object_initdata_t _object_Kerror_ds_class_must_be_concrete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLASS_NOT_DSA))
  object_initdata_t _object_Kerror_ds_class_not_dsa;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLIENT_LOOP))
  object_initdata_t _object_Kerror_ds_client_loop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CODE_INCONSISTENCY))
  object_initdata_t _object_Kerror_ds_code_inconsistency;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COMPARE_FALSE))
  object_initdata_t _object_Kerror_ds_compare_false;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COMPARE_TRUE))
  object_initdata_t _object_Kerror_ds_compare_true;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONFIDENTIALITY_REQUIRED))
  object_initdata_t _object_Kerror_ds_confidentiality_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONFIG_PARAM_MISSING))
  object_initdata_t _object_Kerror_ds_config_param_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONSTRAINT_VIOLATION))
  object_initdata_t _object_Kerror_ds_constraint_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONSTRUCTED_ATT_MOD))
  object_initdata_t _object_Kerror_ds_constructed_att_mod;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONTROL_NOT_FOUND))
  object_initdata_t _object_Kerror_ds_control_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_CONTACT_FSMO))
  object_initdata_t _object_Kerror_ds_couldnt_contact_fsmo;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE))
  object_initdata_t _object_Kerror_ds_couldnt_identify_objects_for_tree_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE))
  object_initdata_t _object_Kerror_ds_couldnt_lock_tree_for_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_UPDATE_SPNS))
  object_initdata_t _object_Kerror_ds_couldnt_update_spns;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COUNTING_AB_INDICES_FAILED))
  object_initdata_t _object_Kerror_ds_counting_ab_indices_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD))
  object_initdata_t _object_Kerror_ds_cross_domain_cleanup_reqd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_DOM_MOVE_ERROR))
  object_initdata_t _object_Kerror_ds_cross_dom_move_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_NC_DN_RENAME))
  object_initdata_t _object_Kerror_ds_cross_nc_dn_rename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_REF_BUSY))
  object_initdata_t _object_Kerror_ds_cross_ref_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_REF_EXISTS))
  object_initdata_t _object_Kerror_ds_cross_ref_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE))
  object_initdata_t _object_Kerror_ds_cr_impossible_to_validate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2))
  object_initdata_t _object_Kerror_ds_cr_impossible_to_validate_v2;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DATABASE_ERROR))
  object_initdata_t _object_Kerror_ds_database_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DECODING_ERROR))
  object_initdata_t _object_Kerror_ds_decoding_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED))
  object_initdata_t _object_Kerror_ds_destination_auditing_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST))
  object_initdata_t _object_Kerror_ds_destination_domain_not_in_forest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DIFFERENT_REPL_EPOCHS))
  object_initdata_t _object_Kerror_ds_different_repl_epochs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_RENAME_IN_PROGRESS))
  object_initdata_t _object_Kerror_ds_domain_rename_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_VERSION_TOO_HIGH))
  object_initdata_t _object_Kerror_ds_domain_version_too_high;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_VERSION_TOO_LOW))
  object_initdata_t _object_Kerror_ds_domain_version_too_low;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_ABANDON_SYNC))
  object_initdata_t _object_Kerror_ds_dra_abandon_sync;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_ACCESS_DENIED))
  object_initdata_t _object_Kerror_ds_dra_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_DN))
  object_initdata_t _object_Kerror_ds_dra_bad_dn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_INSTANCE_TYPE))
  object_initdata_t _object_Kerror_ds_dra_bad_instance_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_NC))
  object_initdata_t _object_Kerror_ds_dra_bad_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BUSY))
  object_initdata_t _object_Kerror_ds_dra_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_CONNECTION_FAILED))
  object_initdata_t _object_Kerror_ds_dra_connection_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_DB_ERROR))
  object_initdata_t _object_Kerror_ds_dra_db_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_DN_EXISTS))
  object_initdata_t _object_Kerror_ds_dra_dn_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT))
  object_initdata_t _object_Kerror_ds_dra_earlier_schema_conlict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_EXTN_CONNECTION_FAILED))
  object_initdata_t _object_Kerror_ds_dra_extn_connection_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_GENERIC))
  object_initdata_t _object_Kerror_ds_dra_generic;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET))
  object_initdata_t _object_Kerror_ds_dra_incompatible_partial_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INCONSISTENT_DIT))
  object_initdata_t _object_Kerror_ds_dra_inconsistent_dit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INTERNAL_ERROR))
  object_initdata_t _object_Kerror_ds_dra_internal_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INVALID_PARAMETER))
  object_initdata_t _object_Kerror_ds_dra_invalid_parameter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_MAIL_PROBLEM))
  object_initdata_t _object_Kerror_ds_dra_mail_problem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_MISSING_PARENT))
  object_initdata_t _object_Kerror_ds_dra_missing_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NAME_COLLISION))
  object_initdata_t _object_Kerror_ds_dra_name_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_ds_dra_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NO_REPLICA))
  object_initdata_t _object_Kerror_ds_dra_no_replica;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OBJ_IS_REP_SOURCE))
  object_initdata_t _object_Kerror_ds_dra_obj_is_rep_source;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OBJ_NC_MISMATCH))
  object_initdata_t _object_Kerror_ds_dra_obj_nc_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OUT_OF_MEM))
  object_initdata_t _object_Kerror_ds_dra_out_of_mem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_PREEMPTED))
  object_initdata_t _object_Kerror_ds_dra_preempted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REF_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_ds_dra_ref_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REF_NOT_FOUND))
  object_initdata_t _object_Kerror_ds_dra_ref_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REPL_PENDING))
  object_initdata_t _object_Kerror_ds_dra_repl_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_RPC_CANCELLED))
  object_initdata_t _object_Kerror_ds_dra_rpc_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_CONFLICT))
  object_initdata_t _object_Kerror_ds_dra_schema_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_INFO_SHIP))
  object_initdata_t _object_Kerror_ds_dra_schema_info_ship;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_MISMATCH))
  object_initdata_t _object_Kerror_ds_dra_schema_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SHUTDOWN))
  object_initdata_t _object_Kerror_ds_dra_shutdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SINK_DISABLED))
  object_initdata_t _object_Kerror_ds_dra_sink_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_DISABLED))
  object_initdata_t _object_Kerror_ds_dra_source_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA))
  object_initdata_t _object_Kerror_ds_dra_source_is_partial_replica;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_REINSTALLED))
  object_initdata_t _object_Kerror_ds_dra_source_reinstalled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRS_EXTENSIONS_CHANGED))
  object_initdata_t _object_Kerror_ds_drs_extensions_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DSA_MUST_BE_INT_MASTER))
  object_initdata_t _object_Kerror_ds_dsa_must_be_int_master;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DST_DOMAIN_NOT_NATIVE))
  object_initdata_t _object_Kerror_ds_dst_domain_not_native;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DST_NC_MISMATCH))
  object_initdata_t _object_Kerror_ds_dst_nc_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DS_REQUIRED))
  object_initdata_t _object_Kerror_ds_ds_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUPLICATE_ID_FOUND))
  object_initdata_t _object_Kerror_ds_duplicate_id_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_LDAP_DISPLAY_NAME))
  object_initdata_t _object_Kerror_ds_dup_ldap_display_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_LINK_ID))
  object_initdata_t _object_Kerror_ds_dup_link_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_MAPI_ID))
  object_initdata_t _object_Kerror_ds_dup_mapi_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_MSDS_INTID))
  object_initdata_t _object_Kerror_ds_dup_msds_intid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_OID))
  object_initdata_t _object_Kerror_ds_dup_oid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_RDN))
  object_initdata_t _object_Kerror_ds_dup_rdn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_SCHEMA_ID_GUID))
  object_initdata_t _object_Kerror_ds_dup_schema_id_guid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ENCODING_ERROR))
  object_initdata_t _object_Kerror_ds_encoding_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EPOCH_MISMATCH))
  object_initdata_t _object_Kerror_ds_epoch_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTING_AD_CHILD_NC))
  object_initdata_t _object_Kerror_ds_existing_ad_child_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_AUX_CLS))
  object_initdata_t _object_Kerror_ds_exists_in_aux_cls;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_MAY_HAVE))
  object_initdata_t _object_Kerror_ds_exists_in_may_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_MUST_HAVE))
  object_initdata_t _object_Kerror_ds_exists_in_must_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_POSS_SUP))
  object_initdata_t _object_Kerror_ds_exists_in_poss_sup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_RDNATTID))
  object_initdata_t _object_Kerror_ds_exists_in_rdnattid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_SUB_CLS))
  object_initdata_t _object_Kerror_ds_exists_in_sub_cls;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FILTER_UNKNOWN))
  object_initdata_t _object_Kerror_ds_filter_unknown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS))
  object_initdata_t _object_Kerror_ds_filter_uses_contructed_attrs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FOREST_VERSION_TOO_HIGH))
  object_initdata_t _object_Kerror_ds_forest_version_too_high;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FOREST_VERSION_TOO_LOW))
  object_initdata_t _object_Kerror_ds_forest_version_too_low;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GCVERIFY_ERROR))
  object_initdata_t _object_Kerror_ds_gcverify_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GC_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_ds_gc_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GC_REQUIRED))
  object_initdata_t _object_Kerror_ds_gc_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GENERIC_ERROR))
  object_initdata_t _object_Kerror_ds_generic_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER))
  object_initdata_t _object_Kerror_ds_global_cant_have_crossdomain_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER))
  object_initdata_t _object_Kerror_ds_global_cant_have_local_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER))
  object_initdata_t _object_Kerror_ds_global_cant_have_universal_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GOVERNSID_MISSING))
  object_initdata_t _object_Kerror_ds_governsid_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GROUP_CONVERSION_ERROR))
  object_initdata_t _object_Kerror_ds_group_conversion_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_HAVE_PRIMARY_MEMBERS))
  object_initdata_t _object_Kerror_ds_have_primary_members;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED))
  object_initdata_t _object_Kerror_ds_hierarchy_table_malloc_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD))
  object_initdata_t _object_Kerror_ds_illegal_base_schema_mod;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_MOD_OPERATION))
  object_initdata_t _object_Kerror_ds_illegal_mod_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_SUPERIOR))
  object_initdata_t _object_Kerror_ds_illegal_superior;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION))
  object_initdata_t _object_Kerror_ds_illegal_xdom_move_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INAPPROPRIATE_AUTH))
  object_initdata_t _object_Kerror_ds_inappropriate_auth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INAPPROPRIATE_MATCHING))
  object_initdata_t _object_Kerror_ds_inappropriate_matching;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCOMPATIBLE_CONTROLS_USED))
  object_initdata_t _object_Kerror_ds_incompatible_controls_used;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCOMPATIBLE_VERSION))
  object_initdata_t _object_Kerror_ds_incompatible_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCORRECT_ROLE_OWNER))
  object_initdata_t _object_Kerror_ds_incorrect_role_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INIT_FAILURE))
  object_initdata_t _object_Kerror_ds_init_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INIT_FAILURE_CONSOLE))
  object_initdata_t _object_Kerror_ds_init_failure_console;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE))
  object_initdata_t _object_Kerror_ds_install_no_sch_version_in_inifile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_NO_SRC_SCH_VERSION))
  object_initdata_t _object_Kerror_ds_install_no_src_sch_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_SCHEMA_MISMATCH))
  object_initdata_t _object_Kerror_ds_install_schema_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT))
  object_initdata_t _object_Kerror_ds_insufficient_attr_to_create_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSUFF_ACCESS_RIGHTS))
  object_initdata_t _object_Kerror_ds_insuff_access_rights;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INTERNAL_FAILURE))
  object_initdata_t _object_Kerror_ds_internal_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_ATTRIBUTE_SYNTAX))
  object_initdata_t _object_Kerror_ds_invalid_attribute_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_DMD))
  object_initdata_t _object_Kerror_ds_invalid_dmd;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_DN_SYNTAX))
  object_initdata_t _object_Kerror_ds_invalid_dn_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_GROUP_TYPE))
  object_initdata_t _object_Kerror_ds_invalid_group_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_LDAP_DISPLAY_NAME))
  object_initdata_t _object_Kerror_ds_invalid_ldap_display_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_NAME_FOR_SPN))
  object_initdata_t _object_Kerror_ds_invalid_name_for_spn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_ROLE_OWNER))
  object_initdata_t _object_Kerror_ds_invalid_role_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_SCRIPT))
  object_initdata_t _object_Kerror_ds_invalid_script;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_SEARCH_FLAG))
  object_initdata_t _object_Kerror_ds_invalid_search_flag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_IS_LEAF))
  object_initdata_t _object_Kerror_ds_is_leaf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_KEY_NOT_UNIQUE))
  object_initdata_t _object_Kerror_ds_key_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LINK_ID_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_ds_link_id_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER))
  object_initdata_t _object_Kerror_ds_local_cant_have_crossdomain_local_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_ERROR))
  object_initdata_t _object_Kerror_ds_local_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY))
  object_initdata_t _object_Kerror_ds_local_member_of_local_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOOKUP_FAILURE))
  object_initdata_t _object_Kerror_ds_lookup_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOOP_DETECT))
  object_initdata_t _object_Kerror_ds_loop_detect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOW_DSA_VERSION))
  object_initdata_t _object_Kerror_ds_low_dsa_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4))
  object_initdata_t _object_Kerror_ds_machine_account_created_prent4;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED))
  object_initdata_t _object_Kerror_ds_machine_account_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MASTERDSA_REQUIRED))
  object_initdata_t _object_Kerror_ds_masterdsa_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MAX_OBJ_SIZE_EXCEEDED))
  object_initdata_t _object_Kerror_ds_max_obj_size_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY))
  object_initdata_t _object_Kerror_ds_membership_evaluated_locally;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_EXPECTED_ATT))
  object_initdata_t _object_Kerror_ds_missing_expected_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_FSMO_SETTINGS))
  object_initdata_t _object_Kerror_ds_missing_fsmo_settings;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER))
  object_initdata_t _object_Kerror_ds_missing_infrastructure_container;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_REQUIRED_ATT))
  object_initdata_t _object_Kerror_ds_missing_required_att;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_SUPREF))
  object_initdata_t _object_Kerror_ds_missing_supref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG))
  object_initdata_t _object_Kerror_ds_modifydn_disallowed_by_flag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE))
  object_initdata_t _object_Kerror_ds_modifydn_disallowed_by_instance_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_WRONG_GRANDPARENT))
  object_initdata_t _object_Kerror_ds_modifydn_wrong_grandparent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MUST_BE_RUN_ON_DST_DC))
  object_initdata_t _object_Kerror_ds_must_be_run_on_dst_dc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_DOMAIN_ONLY))
  object_initdata_t _object_Kerror_ds_name_error_domain_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NOT_FOUND))
  object_initdata_t _object_Kerror_ds_name_error_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NOT_UNIQUE))
  object_initdata_t _object_Kerror_ds_name_error_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NO_MAPPING))
  object_initdata_t _object_Kerror_ds_name_error_no_mapping;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING))
  object_initdata_t _object_Kerror_ds_name_error_no_syntactical_mapping;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_RESOLVING))
  object_initdata_t _object_Kerror_ds_name_error_resolving;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_TRUST_REFERRAL))
  object_initdata_t _object_Kerror_ds_name_error_trust_referral;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_NOT_UNIQUE))
  object_initdata_t _object_Kerror_ds_name_not_unique;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_REFERENCE_INVALID))
  object_initdata_t _object_Kerror_ds_name_reference_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TOO_LONG))
  object_initdata_t _object_Kerror_ds_name_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TOO_MANY_PARTS))
  object_initdata_t _object_Kerror_ds_name_too_many_parts;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TYPE_UNKNOWN))
  object_initdata_t _object_Kerror_ds_name_type_unknown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_UNPARSEABLE))
  object_initdata_t _object_Kerror_ds_name_unparseable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_VALUE_TOO_LONG))
  object_initdata_t _object_Kerror_ds_name_value_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAMING_MASTER_GC))
  object_initdata_t _object_Kerror_ds_naming_master_gc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAMING_VIOLATION))
  object_initdata_t _object_Kerror_ds_naming_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NCNAME_MISSING_CR_REF))
  object_initdata_t _object_Kerror_ds_ncname_missing_cr_ref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NCNAME_MUST_BE_NC))
  object_initdata_t _object_Kerror_ds_ncname_must_be_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NC_MUST_HAVE_NC_PARENT))
  object_initdata_t _object_Kerror_ds_nc_must_have_nc_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NC_STILL_HAS_DSAS))
  object_initdata_t _object_Kerror_ds_nc_still_has_dsas;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_MAY_HAVE))
  object_initdata_t _object_Kerror_ds_nonexistent_may_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_MUST_HAVE))
  object_initdata_t _object_Kerror_ds_nonexistent_must_have;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_POSS_SUP))
  object_initdata_t _object_Kerror_ds_nonexistent_poss_sup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONSAFE_SCHEMA_CHANGE))
  object_initdata_t _object_Kerror_ds_nonsafe_schema_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NON_BASE_SEARCH))
  object_initdata_t _object_Kerror_ds_non_base_search;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX))
  object_initdata_t _object_Kerror_ds_notify_filter_too_complex;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_AN_OBJECT))
  object_initdata_t _object_Kerror_ds_not_an_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC))
  object_initdata_t _object_Kerror_ds_not_authoritive_for_dst_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_CLOSEST))
  object_initdata_t _object_Kerror_ds_not_closest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_INSTALLED))
  object_initdata_t _object_Kerror_ds_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_ON_BACKLINK))
  object_initdata_t _object_Kerror_ds_not_on_backlink;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_ds_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_SUPPORTED_SORT_ORDER))
  object_initdata_t _object_Kerror_ds_not_supported_sort_order;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_ATTRIBUTE_OR_VALUE))
  object_initdata_t _object_Kerror_ds_no_attribute_or_value;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN))
  object_initdata_t _object_Kerror_ds_no_behavior_version_in_mixeddomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHAINED_EVAL))
  object_initdata_t _object_Kerror_ds_no_chained_eval;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHAINING))
  object_initdata_t _object_Kerror_ds_no_chaining;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHECKPOINT_WITH_PDC))
  object_initdata_t _object_Kerror_ds_no_checkpoint_with_pdc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CROSSREF_FOR_NC))
  object_initdata_t _object_Kerror_ds_no_crossref_for_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_DELETED_NAME))
  object_initdata_t _object_Kerror_ds_no_deleted_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS))
  object_initdata_t _object_Kerror_ds_no_fpo_in_universal_groups;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_MORE_RIDS))
  object_initdata_t _object_Kerror_ds_no_more_rids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_MSDS_INTID))
  object_initdata_t _object_Kerror_ds_no_msds_intid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN))
  object_initdata_t _object_Kerror_ds_no_nest_globalgroup_in_mixeddomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN))
  object_initdata_t _object_Kerror_ds_no_nest_localgroup_in_mixeddomain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC))
  object_initdata_t _object_Kerror_ds_no_object_move_in_schema_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_PARENT_OBJECT))
  object_initdata_t _object_Kerror_ds_no_parent_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION))
  object_initdata_t _object_Kerror_ds_no_pkt_privacy_on_connection;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA))
  object_initdata_t _object_Kerror_ds_no_rdn_defined_in_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_REF_DOMAIN))
  object_initdata_t _object_Kerror_ds_no_ref_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_REQUESTED_ATTS_FOUND))
  object_initdata_t _object_Kerror_ds_no_requested_atts_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RESULTS_RETURNED))
  object_initdata_t _object_Kerror_ds_no_results_returned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RIDS_ALLOCATED))
  object_initdata_t _object_Kerror_ds_no_rids_allocated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_SUCH_OBJECT))
  object_initdata_t _object_Kerror_ds_no_such_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_TREE_DELETE_ABOVE_NC))
  object_initdata_t _object_Kerror_ds_no_tree_delete_above_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NTDSCRIPT_PROCESS_ERROR))
  object_initdata_t _object_Kerror_ds_ntdscript_process_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NTDSCRIPT_SYNTAX_ERROR))
  object_initdata_t _object_Kerror_ds_ntdscript_syntax_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_BEING_REMOVED))
  object_initdata_t _object_Kerror_ds_object_being_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_CLASS_REQUIRED))
  object_initdata_t _object_Kerror_ds_object_class_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_RESULTS_TOO_LARGE))
  object_initdata_t _object_Kerror_ds_object_results_too_large;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_NOT_DEFINED))
  object_initdata_t _object_Kerror_ds_obj_class_not_defined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_NOT_SUBCLASS))
  object_initdata_t _object_Kerror_ds_obj_class_not_subclass;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_VIOLATION))
  object_initdata_t _object_Kerror_ds_obj_class_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_GUID_EXISTS))
  object_initdata_t _object_Kerror_ds_obj_guid_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_NOT_FOUND))
  object_initdata_t _object_Kerror_ds_obj_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_STRING_NAME_EXISTS))
  object_initdata_t _object_Kerror_ds_obj_string_name_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_TOO_LARGE))
  object_initdata_t _object_Kerror_ds_obj_too_large;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OFFSET_RANGE_ERROR))
  object_initdata_t _object_Kerror_ds_offset_range_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OPERATIONS_ERROR))
  object_initdata_t _object_Kerror_ds_operations_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OUT_OF_SCOPE))
  object_initdata_t _object_Kerror_ds_out_of_scope;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OUT_OF_VERSION_STORE))
  object_initdata_t _object_Kerror_ds_out_of_version_store;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PARAM_ERROR))
  object_initdata_t _object_Kerror_ds_param_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PARENT_IS_AN_ALIAS))
  object_initdata_t _object_Kerror_ds_parent_is_an_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PDC_OPERATION_IN_PROGRESS))
  object_initdata_t _object_Kerror_ds_pdc_operation_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PROTOCOL_ERROR))
  object_initdata_t _object_Kerror_ds_protocol_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RANGE_CONSTRAINT))
  object_initdata_t _object_Kerror_ds_range_constraint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RDN_DOESNT_MATCH_SCHEMA))
  object_initdata_t _object_Kerror_ds_rdn_doesnt_match_schema;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RECALCSCHEMA_FAILED))
  object_initdata_t _object_Kerror_ds_recalcschema_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFERRAL))
  object_initdata_t _object_Kerror_ds_referral;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFERRAL_LIMIT_EXCEEDED))
  object_initdata_t _object_Kerror_ds_referral_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFUSING_FSMO_ROLES))
  object_initdata_t _object_Kerror_ds_refusing_fsmo_roles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REMOTE_CROSSREF_OP_FAILED))
  object_initdata_t _object_Kerror_ds_remote_crossref_op_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REPLICATOR_ONLY))
  object_initdata_t _object_Kerror_ds_replicator_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR))
  object_initdata_t _object_Kerror_ds_replica_set_change_not_allowed_on_disabled_cr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RESERVED_LINK_ID))
  object_initdata_t _object_Kerror_ds_reserved_link_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RIDMGR_INIT_ERROR))
  object_initdata_t _object_Kerror_ds_ridmgr_init_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROLE_NOT_VERIFIED))
  object_initdata_t _object_Kerror_ds_role_not_verified;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_CANT_BE_SUBREF))
  object_initdata_t _object_Kerror_ds_root_cant_be_subref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_MUST_BE_NC))
  object_initdata_t _object_Kerror_ds_root_must_be_nc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_REQUIRES_CLASS_TOP))
  object_initdata_t _object_Kerror_ds_root_requires_class_top;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_INIT_FAILURE))
  object_initdata_t _object_Kerror_ds_sam_init_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_INIT_FAILURE_CONSOLE))
  object_initdata_t _object_Kerror_ds_sam_init_failure_console;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY))
  object_initdata_t _object_Kerror_ds_sam_need_bootkey_floppy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD))
  object_initdata_t _object_Kerror_ds_sam_need_bootkey_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_ALLOC_FAILED))
  object_initdata_t _object_Kerror_ds_schema_alloc_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_NOT_LOADED))
  object_initdata_t _object_Kerror_ds_schema_not_loaded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_UPDATE_DISALLOWED))
  object_initdata_t _object_Kerror_ds_schema_update_disallowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SECURITY_CHECKING_ERROR))
  object_initdata_t _object_Kerror_ds_security_checking_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SECURITY_ILLEGAL_MODIFY))
  object_initdata_t _object_Kerror_ds_security_illegal_modify;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEC_DESC_INVALID))
  object_initdata_t _object_Kerror_ds_sec_desc_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEC_DESC_TOO_SHORT))
  object_initdata_t _object_Kerror_ds_sec_desc_too_short;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEMANTIC_ATT_TEST))
  object_initdata_t _object_Kerror_ds_semantic_att_test;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SENSITIVE_GROUP_VIOLATION))
  object_initdata_t _object_Kerror_ds_sensitive_group_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SERVER_DOWN))
  object_initdata_t _object_Kerror_ds_server_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SHUTTING_DOWN))
  object_initdata_t _object_Kerror_ds_shutting_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SINGLE_USER_MODE_FAILED))
  object_initdata_t _object_Kerror_ds_single_user_mode_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SINGLE_VALUE_CONSTRAINT))
  object_initdata_t _object_Kerror_ds_single_value_constraint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SIZELIMIT_EXCEEDED))
  object_initdata_t _object_Kerror_ds_sizelimit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SORT_CONTROL_MISSING))
  object_initdata_t _object_Kerror_ds_sort_control_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SOURCE_AUDITING_NOT_ENABLED))
  object_initdata_t _object_Kerror_ds_source_auditing_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SOURCE_DOMAIN_IN_FOREST))
  object_initdata_t _object_Kerror_ds_source_domain_in_forest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_AND_DST_NC_IDENTICAL))
  object_initdata_t _object_Kerror_ds_src_and_dst_nc_identical;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH))
  object_initdata_t _object_Kerror_ds_src_and_dst_object_class_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER))
  object_initdata_t _object_Kerror_ds_src_dc_must_be_sp4_or_greater;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_GUID_MISMATCH))
  object_initdata_t _object_Kerror_ds_src_guid_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_NAME_MISMATCH))
  object_initdata_t _object_Kerror_ds_src_name_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER))
  object_initdata_t _object_Kerror_ds_src_obj_not_group_or_user;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_SID_EXISTS_IN_FOREST))
  object_initdata_t _object_Kerror_ds_src_sid_exists_in_forest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_STRING_SD_CONVERSION_FAILED))
  object_initdata_t _object_Kerror_ds_string_sd_conversion_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_STRONG_AUTH_REQUIRED))
  object_initdata_t _object_Kerror_ds_strong_auth_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUBREF_MUST_HAVE_PARENT))
  object_initdata_t _object_Kerror_ds_subref_must_have_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD))
  object_initdata_t _object_Kerror_ds_subtree_notify_not_nc_head;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUB_CLS_TEST_FAIL))
  object_initdata_t _object_Kerror_ds_sub_cls_test_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SYNTAX_MISMATCH))
  object_initdata_t _object_Kerror_ds_syntax_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_THREAD_LIMIT_EXCEEDED))
  object_initdata_t _object_Kerror_ds_thread_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_TIMELIMIT_EXCEEDED))
  object_initdata_t _object_Kerror_ds_timelimit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_TREE_DELETE_NOT_FINISHED))
  object_initdata_t _object_Kerror_ds_tree_delete_not_finished;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNABLE_TO_SURRENDER_ROLES))
  object_initdata_t _object_Kerror_ds_unable_to_surrender_roles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNAVAILABLE))
  object_initdata_t _object_Kerror_ds_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNAVAILABLE_CRIT_EXTENSION))
  object_initdata_t _object_Kerror_ds_unavailable_crit_extension;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNICODEPWD_NOT_IN_QUOTES))
  object_initdata_t _object_Kerror_ds_unicodepwd_not_in_quotes;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER))
  object_initdata_t _object_Kerror_ds_universal_cant_have_local_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNKNOWN_ERROR))
  object_initdata_t _object_Kerror_ds_unknown_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNKNOWN_OPERATION))
  object_initdata_t _object_Kerror_ds_unknown_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNWILLING_TO_PERFORM))
  object_initdata_t _object_Kerror_ds_unwilling_to_perform;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_USER_BUFFER_TO_SMALL))
  object_initdata_t _object_Kerror_ds_user_buffer_to_small;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL))
  object_initdata_t _object_Kerror_ds_wko_container_cannot_be_special;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WRONG_LINKED_ATT_SYNTAX))
  object_initdata_t _object_Kerror_ds_wrong_linked_att_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WRONG_OM_OBJ_CLASS))
  object_initdata_t _object_Kerror_ds_wrong_om_obj_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUPLICATE_SERVICE_NAME))
  object_initdata_t _object_Kerror_duplicate_service_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUPLICATE_TAG))
  object_initdata_t _object_Kerror_duplicate_tag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUP_DOMAINNAME))
  object_initdata_t _object_Kerror_dup_domainname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUP_NAME))
  object_initdata_t _object_Kerror_dup_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DYNLINK_FROM_INVALID_RING))
  object_initdata_t _object_Kerror_dynlink_from_invalid_ring;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EAS_DIDNT_FIT))
  object_initdata_t _object_Kerror_eas_didnt_fit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EAS_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_eas_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_ACCESS_DENIED))
  object_initdata_t _object_Kerror_ea_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_FILE_CORRUPT))
  object_initdata_t _object_Kerror_ea_file_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_LIST_INCONSISTENT))
  object_initdata_t _object_Kerror_ea_list_inconsistent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_TABLE_FULL))
  object_initdata_t _object_Kerror_ea_table_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_ALG_BLOB_TOO_BIG))
  object_initdata_t _object_Kerror_efs_alg_blob_too_big;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_DISABLED))
  object_initdata_t _object_Kerror_efs_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_SERVER_NOT_TRUSTED))
  object_initdata_t _object_Kerror_efs_server_not_trusted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_VERSION_NOT_SUPPORT))
  object_initdata_t _object_Kerror_efs_version_not_support;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EMPTY))
  object_initdata_t _object_Kerror_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ENCRYPTION_FAILED))
  object_initdata_t _object_Kerror_encryption_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_END_OF_MEDIA))
  object_initdata_t _object_Kerror_end_of_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ENVVAR_NOT_FOUND))
  object_initdata_t _object_Kerror_envvar_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EOM_OVERFLOW))
  object_initdata_t _object_Kerror_eom_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_CANT_START))
  object_initdata_t _object_Kerror_eventlog_cant_start;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_FILE_CHANGED))
  object_initdata_t _object_Kerror_eventlog_file_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_FILE_CORRUPT))
  object_initdata_t _object_Kerror_eventlog_file_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXCEPTION_IN_SERVICE))
  object_initdata_t _object_Kerror_exception_in_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXCL_SEM_ALREADY_OWNED))
  object_initdata_t _object_Kerror_excl_sem_already_owned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY))
  object_initdata_t _object_Kerror_exe_cannot_modify_signed_binary;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY))
  object_initdata_t _object_Kerror_exe_cannot_modify_strong_signed_binary;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_MACHINE_TYPE_MISMATCH))
  object_initdata_t _object_Kerror_exe_machine_type_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_MARKED_INVALID))
  object_initdata_t _object_Kerror_exe_marked_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXTENDED_ERROR))
  object_initdata_t _object_Kerror_extended_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FAILED_SERVICE_CONTROLLER_CONNECT))
  object_initdata_t _object_Kerror_failed_service_controller_connect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FAIL_I24))
  object_initdata_t _object_Kerror_fail_i24;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILEMARK_DETECTED))
  object_initdata_t _object_Kerror_filemark_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILENAME_EXCED_RANGE))
  object_initdata_t _object_Kerror_filename_exced_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_CORRUPT))
  object_initdata_t _object_Kerror_file_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_ENCRYPTED))
  object_initdata_t _object_Kerror_file_encrypted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_EXISTS))
  object_initdata_t _object_Kerror_file_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_INVALID))
  object_initdata_t _object_Kerror_file_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_NOT_ENCRYPTED))
  object_initdata_t _object_Kerror_file_not_encrypted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_NOT_FOUND))
  object_initdata_t _object_Kerror_file_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_OFFLINE))
  object_initdata_t _object_Kerror_file_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_READ_ONLY))
  object_initdata_t _object_Kerror_file_read_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_BAD_REGISTERS))
  object_initdata_t _object_Kerror_floppy_bad_registers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_ID_MARK_NOT_FOUND))
  object_initdata_t _object_Kerror_floppy_id_mark_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_UNKNOWN_ERROR))
  object_initdata_t _object_Kerror_floppy_unknown_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_WRONG_CYLINDER))
  object_initdata_t _object_Kerror_floppy_wrong_cylinder;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FULLSCREEN_MODE))
  object_initdata_t _object_Kerror_fullscreen_mode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FULL_BACKUP))
  object_initdata_t _object_Kerror_full_backup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FUNCTION_FAILED))
  object_initdata_t _object_Kerror_function_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FUNCTION_NOT_CALLED))
  object_initdata_t _object_Kerror_function_not_called;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GENERIC_NOT_MAPPED))
  object_initdata_t _object_Kerror_generic_not_mapped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GEN_FAILURE))
  object_initdata_t _object_Kerror_gen_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GLOBAL_ONLY_HOOK))
  object_initdata_t _object_Kerror_global_only_hook;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GRACEFUL_DISCONNECT))
  object_initdata_t _object_Kerror_graceful_disconnect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_EXISTS))
  object_initdata_t _object_Kerror_group_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_group_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_FOUND))
  object_initdata_t _object_Kerror_group_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_ONLINE))
  object_initdata_t _object_Kerror_group_not_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HANDLE_DISK_FULL))
  object_initdata_t _object_Kerror_handle_disk_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HANDLE_EOF))
  object_initdata_t _object_Kerror_handle_eof;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_NEEDS_HMOD))
  object_initdata_t _object_Kerror_hook_needs_hmod;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_NOT_INSTALLED))
  object_initdata_t _object_Kerror_hook_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_TYPE_NOT_ALLOWED))
  object_initdata_t _object_Kerror_hook_type_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_DOWN))
  object_initdata_t _object_Kerror_host_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_host_node_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_GROUP_OWNER))
  object_initdata_t _object_Kerror_host_node_not_group_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_RESOURCE_OWNER))
  object_initdata_t _object_Kerror_host_node_not_resource_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_UNREACHABLE))
  object_initdata_t _object_Kerror_host_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOTKEY_ALREADY_REGISTERED))
  object_initdata_t _object_Kerror_hotkey_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOTKEY_NOT_REGISTERED))
  object_initdata_t _object_Kerror_hotkey_not_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HWNDS_HAVE_DIFF_PARENT))
  object_initdata_t _object_Kerror_hwnds_have_diff_parent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ICM_NOT_ENABLED))
  object_initdata_t _object_Kerror_icm_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ILLEGAL_ELEMENT_ADDRESS))
  object_initdata_t _object_Kerror_illegal_element_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ILL_FORMED_PASSWORD))
  object_initdata_t _object_Kerror_ill_formed_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INCORRECT_ADDRESS))
  object_initdata_t _object_Kerror_incorrect_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INC_BACKUP))
  object_initdata_t _object_Kerror_inc_backup;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INDEX_ABSENT))
  object_initdata_t _object_Kerror_index_absent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INDIGENOUS_TYPE))
  object_initdata_t _object_Kerror_indigenous_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INFLOOP_IN_RELOC_CHAIN))
  object_initdata_t _object_Kerror_infloop_in_reloc_chain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_ALREADY_RUNNING))
  object_initdata_t _object_Kerror_install_already_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_FAILURE))
  object_initdata_t _object_Kerror_install_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_LANGUAGE_UNSUPPORTED))
  object_initdata_t _object_Kerror_install_language_unsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_LOG_FAILURE))
  object_initdata_t _object_Kerror_install_log_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_NOTUSED))
  object_initdata_t _object_Kerror_install_notused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_INVALID))
  object_initdata_t _object_Kerror_install_package_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_OPEN_FAILED))
  object_initdata_t _object_Kerror_install_package_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_REJECTED))
  object_initdata_t _object_Kerror_install_package_rejected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_VERSION))
  object_initdata_t _object_Kerror_install_package_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PLATFORM_UNSUPPORTED))
  object_initdata_t _object_Kerror_install_platform_unsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_REMOTE_DISALLOWED))
  object_initdata_t _object_Kerror_install_remote_disallowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_REMOTE_PROHIBITED))
  object_initdata_t _object_Kerror_install_remote_prohibited;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SERVICE_FAILURE))
  object_initdata_t _object_Kerror_install_service_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SOURCE_ABSENT))
  object_initdata_t _object_Kerror_install_source_absent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SUSPEND))
  object_initdata_t _object_Kerror_install_suspend;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TEMP_UNWRITABLE))
  object_initdata_t _object_Kerror_install_temp_unwritable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TRANSFORM_FAILURE))
  object_initdata_t _object_Kerror_install_transform_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TRANSFORM_REJECTED))
  object_initdata_t _object_Kerror_install_transform_rejected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_UI_FAILURE))
  object_initdata_t _object_Kerror_install_ui_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_USEREXIT))
  object_initdata_t _object_Kerror_install_userexit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSUFFICIENT_BUFFER))
  object_initdata_t _object_Kerror_insufficient_buffer;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_DB_CORRUPTION))
  object_initdata_t _object_Kerror_internal_db_corruption;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_DB_ERROR))
  object_initdata_t _object_Kerror_internal_db_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_ERROR))
  object_initdata_t _object_Kerror_internal_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCEL_HANDLE))
  object_initdata_t _object_Kerror_invalid_accel_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCESS))
  object_initdata_t _object_Kerror_invalid_access;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCOUNT_NAME))
  object_initdata_t _object_Kerror_invalid_account_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACL))
  object_initdata_t _object_Kerror_invalid_acl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ADDRESS))
  object_initdata_t _object_Kerror_invalid_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_AT_INTERRUPT_TIME))
  object_initdata_t _object_Kerror_invalid_at_interrupt_time;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_BLOCK))
  object_initdata_t _object_Kerror_invalid_block;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_BLOCK_LENGTH))
  object_initdata_t _object_Kerror_invalid_block_length;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CATEGORY))
  object_initdata_t _object_Kerror_invalid_category;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CLEANER))
  object_initdata_t _object_Kerror_invalid_cleaner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CMM))
  object_initdata_t _object_Kerror_invalid_cmm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COLORINDEX))
  object_initdata_t _object_Kerror_invalid_colorindex;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COLORSPACE))
  object_initdata_t _object_Kerror_invalid_colorspace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMBOBOX_MESSAGE))
  object_initdata_t _object_Kerror_invalid_combobox_message;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMMAND_LINE))
  object_initdata_t _object_Kerror_invalid_command_line;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMPUTERNAME))
  object_initdata_t _object_Kerror_invalid_computername;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CURSOR_HANDLE))
  object_initdata_t _object_Kerror_invalid_cursor_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DATA))
  object_initdata_t _object_Kerror_invalid_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DATATYPE))
  object_initdata_t _object_Kerror_invalid_datatype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DLL))
  object_initdata_t _object_Kerror_invalid_dll;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAINNAME))
  object_initdata_t _object_Kerror_invalid_domainname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAIN_ROLE))
  object_initdata_t _object_Kerror_invalid_domain_role;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAIN_STATE))
  object_initdata_t _object_Kerror_invalid_domain_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DRIVE))
  object_initdata_t _object_Kerror_invalid_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DRIVE_OBJECT))
  object_initdata_t _object_Kerror_invalid_drive_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DWP_HANDLE))
  object_initdata_t _object_Kerror_invalid_dwp_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EA_HANDLE))
  object_initdata_t _object_Kerror_invalid_ea_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EA_NAME))
  object_initdata_t _object_Kerror_invalid_ea_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EDIT_HEIGHT))
  object_initdata_t _object_Kerror_invalid_edit_height;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ENVIRONMENT))
  object_initdata_t _object_Kerror_invalid_environment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EVENTNAME))
  object_initdata_t _object_Kerror_invalid_eventname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EVENT_COUNT))
  object_initdata_t _object_Kerror_invalid_event_count;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EXE_SIGNATURE))
  object_initdata_t _object_Kerror_invalid_exe_signature;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FIELD))
  object_initdata_t _object_Kerror_invalid_field;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FILTER_PROC))
  object_initdata_t _object_Kerror_invalid_filter_proc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FLAGS))
  object_initdata_t _object_Kerror_invalid_flags;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FLAG_NUMBER))
  object_initdata_t _object_Kerror_invalid_flag_number;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FORM_NAME))
  object_initdata_t _object_Kerror_invalid_form_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FORM_SIZE))
  object_initdata_t _object_Kerror_invalid_form_size;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FUNCTION))
  object_initdata_t _object_Kerror_invalid_function;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GROUPNAME))
  object_initdata_t _object_Kerror_invalid_groupname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GROUP_ATTRIBUTES))
  object_initdata_t _object_Kerror_invalid_group_attributes;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GW_COMMAND))
  object_initdata_t _object_Kerror_invalid_gw_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HANDLE))
  object_initdata_t _object_Kerror_invalid_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HANDLE_STATE))
  object_initdata_t _object_Kerror_invalid_handle_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HOOK_FILTER))
  object_initdata_t _object_Kerror_invalid_hook_filter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HOOK_HANDLE))
  object_initdata_t _object_Kerror_invalid_hook_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ICON_HANDLE))
  object_initdata_t _object_Kerror_invalid_icon_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ID_AUTHORITY))
  object_initdata_t _object_Kerror_invalid_id_authority;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_IMPORT_OF_NON_DLL))
  object_initdata_t _object_Kerror_invalid_import_of_non_dll;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_INDEX))
  object_initdata_t _object_Kerror_invalid_index;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_KEYBOARD_HANDLE))
  object_initdata_t _object_Kerror_invalid_keyboard_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LB_MESSAGE))
  object_initdata_t _object_Kerror_invalid_lb_message;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LEVEL))
  object_initdata_t _object_Kerror_invalid_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LIBRARY))
  object_initdata_t _object_Kerror_invalid_library;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LIST_FORMAT))
  object_initdata_t _object_Kerror_invalid_list_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LOGON_HOURS))
  object_initdata_t _object_Kerror_invalid_logon_hours;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LOGON_TYPE))
  object_initdata_t _object_Kerror_invalid_logon_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEDIA))
  object_initdata_t _object_Kerror_invalid_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEDIA_POOL))
  object_initdata_t _object_Kerror_invalid_media_pool;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEMBER))
  object_initdata_t _object_Kerror_invalid_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MENU_HANDLE))
  object_initdata_t _object_Kerror_invalid_menu_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGE))
  object_initdata_t _object_Kerror_invalid_message;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGEDEST))
  object_initdata_t _object_Kerror_invalid_messagedest;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGENAME))
  object_initdata_t _object_Kerror_invalid_messagename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MINALLOCSIZE))
  object_initdata_t _object_Kerror_invalid_minallocsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MODULETYPE))
  object_initdata_t _object_Kerror_invalid_moduletype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MONITOR_HANDLE))
  object_initdata_t _object_Kerror_invalid_monitor_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MSGBOX_STYLE))
  object_initdata_t _object_Kerror_invalid_msgbox_style;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_NAME))
  object_initdata_t _object_Kerror_invalid_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_NETNAME))
  object_initdata_t _object_Kerror_invalid_netname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPERATION))
  object_initdata_t _object_Kerror_invalid_operation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPERATION_ON_QUORUM))
  object_initdata_t _object_Kerror_invalid_operation_on_quorum;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPLOCK_PROTOCOL))
  object_initdata_t _object_Kerror_invalid_oplock_protocol;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ORDINAL))
  object_initdata_t _object_Kerror_invalid_ordinal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OWNER))
  object_initdata_t _object_Kerror_invalid_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PARAMETER))
  object_initdata_t _object_Kerror_invalid_parameter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PASSWORD))
  object_initdata_t _object_Kerror_invalid_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PASSWORDNAME))
  object_initdata_t _object_Kerror_invalid_passwordname;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PIXEL_FORMAT))
  object_initdata_t _object_Kerror_invalid_pixel_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRIMARY_GROUP))
  object_initdata_t _object_Kerror_invalid_primary_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_COMMAND))
  object_initdata_t _object_Kerror_invalid_printer_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_NAME))
  object_initdata_t _object_Kerror_invalid_printer_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_STATE))
  object_initdata_t _object_Kerror_invalid_printer_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINT_MONITOR))
  object_initdata_t _object_Kerror_invalid_print_monitor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRIORITY))
  object_initdata_t _object_Kerror_invalid_priority;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PROFILE))
  object_initdata_t _object_Kerror_invalid_profile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_REPARSE_DATA))
  object_initdata_t _object_Kerror_invalid_reparse_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SCROLLBAR_RANGE))
  object_initdata_t _object_Kerror_invalid_scrollbar_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SECURITY_DESCR))
  object_initdata_t _object_Kerror_invalid_security_descr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEGDPL))
  object_initdata_t _object_Kerror_invalid_segdpl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEGMENT_NUMBER))
  object_initdata_t _object_Kerror_invalid_segment_number;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEPARATOR_FILE))
  object_initdata_t _object_Kerror_invalid_separator_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVER_STATE))
  object_initdata_t _object_Kerror_invalid_server_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICENAME))
  object_initdata_t _object_Kerror_invalid_servicename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_ACCOUNT))
  object_initdata_t _object_Kerror_invalid_service_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_CONTROL))
  object_initdata_t _object_Kerror_invalid_service_control;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_LOCK))
  object_initdata_t _object_Kerror_invalid_service_lock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SHARENAME))
  object_initdata_t _object_Kerror_invalid_sharename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SHOWWIN_COMMAND))
  object_initdata_t _object_Kerror_invalid_showwin_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SID))
  object_initdata_t _object_Kerror_invalid_sid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SIGNAL_NUMBER))
  object_initdata_t _object_Kerror_invalid_signal_number;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SPI_VALUE))
  object_initdata_t _object_Kerror_invalid_spi_value;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STACKSEG))
  object_initdata_t _object_Kerror_invalid_stackseg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STARTING_CODESEG))
  object_initdata_t _object_Kerror_invalid_starting_codeseg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STATE))
  object_initdata_t _object_Kerror_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SUB_AUTHORITY))
  object_initdata_t _object_Kerror_invalid_sub_authority;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TABLE))
  object_initdata_t _object_Kerror_invalid_table;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TARGET_HANDLE))
  object_initdata_t _object_Kerror_invalid_target_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_THREAD_ID))
  object_initdata_t _object_Kerror_invalid_thread_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TIME))
  object_initdata_t _object_Kerror_invalid_time;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TRANSFORM))
  object_initdata_t _object_Kerror_invalid_transform;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_USER_BUFFER))
  object_initdata_t _object_Kerror_invalid_user_buffer;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_VERIFY_SWITCH))
  object_initdata_t _object_Kerror_invalid_verify_switch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WINDOW_HANDLE))
  object_initdata_t _object_Kerror_invalid_window_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WINDOW_STYLE))
  object_initdata_t _object_Kerror_invalid_window_style;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WORKSTATION))
  object_initdata_t _object_Kerror_invalid_workstation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IOPL_NOT_ENABLED))
  object_initdata_t _object_Kerror_iopl_not_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_DEVICE))
  object_initdata_t _object_Kerror_io_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_INCOMPLETE))
  object_initdata_t _object_Kerror_io_incomplete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_PENDING))
  object_initdata_t _object_Kerror_io_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_default_mm_auth_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_default_mm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_default_qm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED))
  object_initdata_t _object_Kerror_ipsec_ike_add_update_key_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ATTRIB_FAIL))
  object_initdata_t _object_Kerror_ipsec_ike_attrib_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_AUTH_FAIL))
  object_initdata_t _object_Kerror_ipsec_ike_auth_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_BENIGN_REINIT))
  object_initdata_t _object_Kerror_ipsec_ike_benign_reinit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_CRL_FAILED))
  object_initdata_t _object_Kerror_ipsec_ike_crl_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DECRYPT))
  object_initdata_t _object_Kerror_ipsec_ike_decrypt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DH_FAIL))
  object_initdata_t _object_Kerror_ipsec_ike_dh_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DH_FAILURE))
  object_initdata_t _object_Kerror_ipsec_ike_dh_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DROP_NO_RESPONSE))
  object_initdata_t _object_Kerror_ipsec_ike_drop_no_response;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ENCRYPT))
  object_initdata_t _object_Kerror_ipsec_ike_encrypt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ERROR))
  object_initdata_t _object_Kerror_ipsec_ike_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_FAILQUERYSSP))
  object_initdata_t _object_Kerror_ipsec_ike_failqueryssp;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_FAILSSPINIT))
  object_initdata_t _object_Kerror_ipsec_ike_failsspinit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR))
  object_initdata_t _object_Kerror_ipsec_ike_general_processing_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_GETSPIFAIL))
  object_initdata_t _object_Kerror_ipsec_ike_getspifail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_AUTH_ALG))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_auth_alg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_cert_keylen;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_CERT_TYPE))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_cert_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_COOKIE))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_cookie;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_encrypt_alg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_FILTER))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_filter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_GROUP))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_hash;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH_ALG))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_hash_alg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH_SIZE))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_hash_size;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HEADER))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_header;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_KEY_USAGE))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_key_usage;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_PAYLOAD))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_payload;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_POLICY))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_responder_lifetime_notify;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SIG))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_sig;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SIGNATURE))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_signature;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SITUATION))
  object_initdata_t _object_Kerror_ipsec_ike_invalid_situation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_KERBEROS_ERROR))
  object_initdata_t _object_Kerror_ipsec_ike_kerberos_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_LOAD_FAILED))
  object_initdata_t _object_Kerror_ipsec_ike_load_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_LOAD_SOFT_SA))
  object_initdata_t _object_Kerror_ipsec_ike_load_soft_sa;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_ACQUIRE_DROP))
  object_initdata_t _object_Kerror_ipsec_ike_mm_acquire_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_DELAY_DROP))
  object_initdata_t _object_Kerror_ipsec_ike_mm_delay_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_LIMIT))
  object_initdata_t _object_Kerror_ipsec_ike_mm_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEGOTIATION_DISABLED))
  object_initdata_t _object_Kerror_ipsec_ike_negotiation_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEGOTIATION_PENDING))
  object_initdata_t _object_Kerror_ipsec_ike_negotiation_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEG_STATUS_END))
  object_initdata_t _object_Kerror_ipsec_ike_neg_status_end;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NOTCBPRIV))
  object_initdata_t _object_Kerror_ipsec_ike_notcbpriv;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_CERT))
  object_initdata_t _object_Kerror_ipsec_ike_no_cert;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_MM_POLICY))
  object_initdata_t _object_Kerror_ipsec_ike_no_mm_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PEER_CERT))
  object_initdata_t _object_Kerror_ipsec_ike_no_peer_cert;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_POLICY))
  object_initdata_t _object_Kerror_ipsec_ike_no_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PRIVATE_KEY))
  object_initdata_t _object_Kerror_ipsec_ike_no_private_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PUBLIC_KEY))
  object_initdata_t _object_Kerror_ipsec_ike_no_public_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_OUT_OF_MEMORY))
  object_initdata_t _object_Kerror_ipsec_ike_out_of_memory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PEER_CRL_FAILED))
  object_initdata_t _object_Kerror_ipsec_ike_peer_crl_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_POLICY_CHANGE))
  object_initdata_t _object_Kerror_ipsec_ike_policy_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_POLICY_MATCH))
  object_initdata_t _object_Kerror_ipsec_ike_policy_match;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR))
  object_initdata_t _object_Kerror_ipsec_ike_process_err;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_cert;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_cert_req;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_DELETE))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_HASH))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_hash;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_ID))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_KE))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_ke;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_NONCE))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_nonce;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_notify;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_PROP))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_prop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_SA))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_sa;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_SIG))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_sig;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_TRANS))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_trans;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR))
  object_initdata_t _object_Kerror_ipsec_ike_process_err_vendor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QM_ACQUIRE_DROP))
  object_initdata_t _object_Kerror_ipsec_ike_qm_acquire_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QM_DELAY_DROP))
  object_initdata_t _object_Kerror_ipsec_ike_qm_delay_drop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QUEUE_DROP_MM))
  object_initdata_t _object_Kerror_ipsec_ike_queue_drop_mm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM))
  object_initdata_t _object_Kerror_ipsec_ike_queue_drop_no_mm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_RPC_DELETE))
  object_initdata_t _object_Kerror_ipsec_ike_rpc_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SA_DELETED))
  object_initdata_t _object_Kerror_ipsec_ike_sa_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SA_REAPED))
  object_initdata_t _object_Kerror_ipsec_ike_sa_reaped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SECLOADFAIL))
  object_initdata_t _object_Kerror_ipsec_ike_secloadfail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN))
  object_initdata_t _object_Kerror_ipsec_ike_soft_sa_torn_down;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SRVACQFAIL))
  object_initdata_t _object_Kerror_ipsec_ike_srvacqfail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SRVQUERYCRED))
  object_initdata_t _object_Kerror_ipsec_ike_srvquerycred;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_TIMED_OUT))
  object_initdata_t _object_Kerror_ipsec_ike_timed_out;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_UNKNOWN_DOI))
  object_initdata_t _object_Kerror_ipsec_ike_unknown_doi;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_UNSUPPORTED_ID))
  object_initdata_t _object_Kerror_ipsec_ike_unsupported_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_EXISTS))
  object_initdata_t _object_Kerror_ipsec_mm_auth_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_IN_USE))
  object_initdata_t _object_Kerror_ipsec_mm_auth_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_mm_auth_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_PENDING_DELETION))
  object_initdata_t _object_Kerror_ipsec_mm_auth_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_EXISTS))
  object_initdata_t _object_Kerror_ipsec_mm_filter_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_mm_filter_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_PENDING_DELETION))
  object_initdata_t _object_Kerror_ipsec_mm_filter_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_EXISTS))
  object_initdata_t _object_Kerror_ipsec_mm_policy_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_IN_USE))
  object_initdata_t _object_Kerror_ipsec_mm_policy_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_mm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_PENDING_DELETION))
  object_initdata_t _object_Kerror_ipsec_mm_policy_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_EXISTS))
  object_initdata_t _object_Kerror_ipsec_qm_policy_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_IN_USE))
  object_initdata_t _object_Kerror_ipsec_qm_policy_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_qm_policy_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_PENDING_DELETION))
  object_initdata_t _object_Kerror_ipsec_qm_policy_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_EXISTS))
  object_initdata_t _object_Kerror_ipsec_transport_filter_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_transport_filter_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION))
  object_initdata_t _object_Kerror_ipsec_transport_filter_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_EXISTS))
  object_initdata_t _object_Kerror_ipsec_tunnel_filter_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND))
  object_initdata_t _object_Kerror_ipsec_tunnel_filter_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION))
  object_initdata_t _object_Kerror_ipsec_tunnel_filter_pending_deletion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IRQ_BUSY))
  object_initdata_t _object_Kerror_irq_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOINED))
  object_initdata_t _object_Kerror_is_joined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOIN_PATH))
  object_initdata_t _object_Kerror_is_join_path;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOIN_TARGET))
  object_initdata_t _object_Kerror_is_join_target;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBSTED))
  object_initdata_t _object_Kerror_is_substed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBST_PATH))
  object_initdata_t _object_Kerror_is_subst_path;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBST_TARGET))
  object_initdata_t _object_Kerror_is_subst_target;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ITERATED_DATA_EXCEEDS_64k))
  object_initdata_t _object_Kerror_iterated_data_exceeds_64k;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOIN_TO_JOIN))
  object_initdata_t _object_Kerror_join_to_join;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOIN_TO_SUBST))
  object_initdata_t _object_Kerror_join_to_subst;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_DELETE_IN_PROGRESS))
  object_initdata_t _object_Kerror_journal_delete_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_ENTRY_DELETED))
  object_initdata_t _object_Kerror_journal_entry_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_HOOK_SET))
  object_initdata_t _object_Kerror_journal_hook_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_NOT_ACTIVE))
  object_initdata_t _object_Kerror_journal_not_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KEY_DELETED))
  object_initdata_t _object_Kerror_key_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KEY_HAS_CHILDREN))
  object_initdata_t _object_Kerror_key_has_children;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KM_DRIVER_BLOCKED))
  object_initdata_t _object_Kerror_km_driver_blocked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LABEL_TOO_LONG))
  object_initdata_t _object_Kerror_label_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LAST_ADMIN))
  object_initdata_t _object_Kerror_last_admin;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LB_WITHOUT_TABSTOPS))
  object_initdata_t _object_Kerror_lb_without_tabstops;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LIBRARY_FULL))
  object_initdata_t _object_Kerror_library_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LIBRARY_OFFLINE))
  object_initdata_t _object_Kerror_library_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LICENSE_QUOTA_EXCEEDED))
  object_initdata_t _object_Kerror_license_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LISTBOX_ID_NOT_FOUND))
  object_initdata_t _object_Kerror_listbox_id_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LM_CROSS_ENCRYPTION_REQUIRED))
  object_initdata_t _object_Kerror_lm_cross_encryption_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCAL_USER_SESSION_KEY))
  object_initdata_t _object_Kerror_local_user_session_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCKED))
  object_initdata_t _object_Kerror_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCK_FAILED))
  object_initdata_t _object_Kerror_lock_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCK_VIOLATION))
  object_initdata_t _object_Kerror_lock_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGIN_TIME_RESTRICTION))
  object_initdata_t _object_Kerror_login_time_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGIN_WKSTA_RESTRICTION))
  object_initdata_t _object_Kerror_login_wksta_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_FAILURE))
  object_initdata_t _object_Kerror_logon_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_NOT_GRANTED))
  object_initdata_t _object_Kerror_logon_not_granted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_SESSION_COLLISION))
  object_initdata_t _object_Kerror_logon_session_collision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_SESSION_EXISTS))
  object_initdata_t _object_Kerror_logon_session_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_TYPE_NOT_GRANTED))
  object_initdata_t _object_Kerror_logon_type_not_granted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOG_FILE_FULL))
  object_initdata_t _object_Kerror_log_file_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LUIDS_EXHAUSTED))
  object_initdata_t _object_Kerror_luids_exhausted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MACHINE_LOCKED))
  object_initdata_t _object_Kerror_machine_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAGAZINE_NOT_PRESENT))
  object_initdata_t _object_Kerror_magazine_not_present;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAPPED_ALIGNMENT))
  object_initdata_t _object_Kerror_mapped_alignment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAX_THRDS_REACHED))
  object_initdata_t _object_Kerror_max_thrds_reached;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_CHANGED))
  object_initdata_t _object_Kerror_media_changed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_INCOMPATIBLE))
  object_initdata_t _object_Kerror_media_incompatible;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_media_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_OFFLINE))
  object_initdata_t _object_Kerror_media_offline;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_UNAVAILABLE))
  object_initdata_t _object_Kerror_media_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIUM_NOT_ACCESSIBLE))
  object_initdata_t _object_Kerror_medium_not_accessible;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBERS_PRIMARY_GROUP))
  object_initdata_t _object_Kerror_members_primary_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_IN_ALIAS))
  object_initdata_t _object_Kerror_member_in_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_IN_GROUP))
  object_initdata_t _object_Kerror_member_in_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_NOT_IN_ALIAS))
  object_initdata_t _object_Kerror_member_not_in_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_NOT_IN_GROUP))
  object_initdata_t _object_Kerror_member_not_in_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MENU_ITEM_NOT_FOUND))
  object_initdata_t _object_Kerror_menu_item_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MESSAGE_EXCEEDS_MAX_SIZE))
  object_initdata_t _object_Kerror_message_exceeds_max_size;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MESSAGE_SYNC_ONLY))
  object_initdata_t _object_Kerror_message_sync_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_METAFILE_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_metafile_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_META_EXPANSION_TOO_LONG))
  object_initdata_t _object_Kerror_meta_expansion_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MOD_NOT_FOUND))
  object_initdata_t _object_Kerror_mod_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MORE_DATA))
  object_initdata_t _object_Kerror_more_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MORE_WRITES))
  object_initdata_t _object_Kerror_more_writes;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MR_MID_NOT_FOUND))
  object_initdata_t _object_Kerror_mr_mid_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MUTUAL_AUTH_FAILED))
  object_initdata_t _object_Kerror_mutual_auth_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NEGATIVE_SEEK))
  object_initdata_t _object_Kerror_negative_seek;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NESTING_NOT_ALLOWED))
  object_initdata_t _object_Kerror_nesting_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETLOGON_NOT_STARTED))
  object_initdata_t _object_Kerror_netlogon_not_started;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETNAME_DELETED))
  object_initdata_t _object_Kerror_netname_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_ACCESS_DENIED))
  object_initdata_t _object_Kerror_network_access_denied;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_BUSY))
  object_initdata_t _object_Kerror_network_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_network_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_UNREACHABLE))
  object_initdata_t _object_Kerror_network_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NET_WRITE_FAULT))
  object_initdata_t _object_Kerror_net_write_fault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOACCESS))
  object_initdata_t _object_Kerror_noaccess;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_CANNOT_BE_CLUSTERED))
  object_initdata_t _object_Kerror_node_cannot_be_clustered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_CANT_HOST_RESOURCE))
  object_initdata_t _object_Kerror_node_cant_host_resource;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_node_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT))
  object_initdata_t _object_Kerror_nologon_interdomain_trust_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_SERVER_TRUST_ACCOUNT))
  object_initdata_t _object_Kerror_nologon_server_trust_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT))
  object_initdata_t _object_Kerror_nologon_workstation_trust_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NONE_MAPPED))
  object_initdata_t _object_Kerror_none_mapped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NONPAGED_SYSTEM_RESOURCES))
  object_initdata_t _object_Kerror_nonpaged_system_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_ACCOUNT_SID))
  object_initdata_t _object_Kerror_non_account_sid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_DOMAIN_SID))
  object_initdata_t _object_Kerror_non_domain_sid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_MDICHILD_WINDOW))
  object_initdata_t _object_Kerror_non_mdichild_window;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOTIFY_ENUM_DIR))
  object_initdata_t _object_Kerror_notify_enum_dir;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ALL_ASSIGNED))
  object_initdata_t _object_Kerror_not_all_assigned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_AUTHENTICATED))
  object_initdata_t _object_Kerror_not_authenticated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_A_REPARSE_POINT))
  object_initdata_t _object_Kerror_not_a_reparse_point;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CHILD_WINDOW))
  object_initdata_t _object_Kerror_not_child_window;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CONNECTED))
  object_initdata_t _object_Kerror_not_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CONTAINER))
  object_initdata_t _object_Kerror_not_container;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_DOS_DISK))
  object_initdata_t _object_Kerror_not_dos_disk;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_EMPTY))
  object_initdata_t _object_Kerror_not_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_MEMORY))
  object_initdata_t _object_Kerror_not_enough_memory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_QUOTA))
  object_initdata_t _object_Kerror_not_enough_quota;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_SERVER_MEMORY))
  object_initdata_t _object_Kerror_not_enough_server_memory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_EXPORT_FORMAT))
  object_initdata_t _object_Kerror_not_export_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_FOUND))
  object_initdata_t _object_Kerror_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_JOINED))
  object_initdata_t _object_Kerror_not_joined;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOCKED))
  object_initdata_t _object_Kerror_not_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOGGED_ON))
  object_initdata_t _object_Kerror_not_logged_on;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOGON_PROCESS))
  object_initdata_t _object_Kerror_not_logon_process;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_OWNER))
  object_initdata_t _object_Kerror_not_owner;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_QUORUM_CAPABLE))
  object_initdata_t _object_Kerror_not_quorum_capable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_QUORUM_CLASS))
  object_initdata_t _object_Kerror_not_quorum_class;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_READY))
  object_initdata_t _object_Kerror_not_ready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_REGISTRY_FILE))
  object_initdata_t _object_Kerror_not_registry_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SAFEBOOT_SERVICE))
  object_initdata_t _object_Kerror_not_safeboot_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SAME_DEVICE))
  object_initdata_t _object_Kerror_not_same_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUBSTED))
  object_initdata_t _object_Kerror_not_substed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED_ON_SBS))
  object_initdata_t _object_Kerror_not_supported_on_sbs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER))
  object_initdata_t _object_Kerror_not_supported_on_standard_server;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_ASSOCIATION))
  object_initdata_t _object_Kerror_no_association;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_BROWSER_SERVERS_FOUND))
  object_initdata_t _object_Kerror_no_browser_servers_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_DATA))
  object_initdata_t _object_Kerror_no_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_DATA_DETECTED))
  object_initdata_t _object_Kerror_no_data_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_EFS))
  object_initdata_t _object_Kerror_no_efs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_IMPERSONATION_TOKEN))
  object_initdata_t _object_Kerror_no_impersonation_token;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_INHERITANCE))
  object_initdata_t _object_Kerror_no_inheritance;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_LOGON_SERVERS))
  object_initdata_t _object_Kerror_no_logon_servers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_LOG_SPACE))
  object_initdata_t _object_Kerror_no_log_space;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MATCH))
  object_initdata_t _object_Kerror_no_match;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MEDIA_IN_DRIVE))
  object_initdata_t _object_Kerror_no_media_in_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_DEVICES))
  object_initdata_t _object_Kerror_no_more_devices;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_FILES))
  object_initdata_t _object_Kerror_no_more_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_ITEMS))
  object_initdata_t _object_Kerror_no_more_items;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_SEARCH_HANDLES))
  object_initdata_t _object_Kerror_no_more_search_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_USER_HANDLES))
  object_initdata_t _object_Kerror_no_more_user_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_NETWORK))
  object_initdata_t _object_Kerror_no_network;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_NET_OR_BAD_PATH))
  object_initdata_t _object_Kerror_no_net_or_bad_path;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_PROC_SLOTS))
  object_initdata_t _object_Kerror_no_proc_slots;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_PROMOTION_ACTIVE))
  object_initdata_t _object_Kerror_no_promotion_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_QUOTAS_FOR_ACCOUNT))
  object_initdata_t _object_Kerror_no_quotas_for_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_RECOVERY_POLICY))
  object_initdata_t _object_Kerror_no_recovery_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_RECOVERY_PROGRAM))
  object_initdata_t _object_Kerror_no_recovery_program;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SCROLLBARS))
  object_initdata_t _object_Kerror_no_scrollbars;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SECURITY_ON_OBJECT))
  object_initdata_t _object_Kerror_no_security_on_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SHUTDOWN_IN_PROGRESS))
  object_initdata_t _object_Kerror_no_shutdown_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SIGNAL_SENT))
  object_initdata_t _object_Kerror_no_signal_sent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SITENAME))
  object_initdata_t _object_Kerror_no_sitename;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SPOOL_SPACE))
  object_initdata_t _object_Kerror_no_spool_space;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_ALIAS))
  object_initdata_t _object_Kerror_no_such_alias;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_DOMAIN))
  object_initdata_t _object_Kerror_no_such_domain;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_GROUP))
  object_initdata_t _object_Kerror_no_such_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_LOGON_SESSION))
  object_initdata_t _object_Kerror_no_such_logon_session;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_MEMBER))
  object_initdata_t _object_Kerror_no_such_member;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_PACKAGE))
  object_initdata_t _object_Kerror_no_such_package;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_PRIVILEGE))
  object_initdata_t _object_Kerror_no_such_privilege;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_SITE))
  object_initdata_t _object_Kerror_no_such_site;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_USER))
  object_initdata_t _object_Kerror_no_such_user;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUPPORTING_DRIVES))
  object_initdata_t _object_Kerror_no_supporting_drives;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SYSTEM_MENU))
  object_initdata_t _object_Kerror_no_system_menu;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SYSTEM_RESOURCES))
  object_initdata_t _object_Kerror_no_system_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TOKEN))
  object_initdata_t _object_Kerror_no_token;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRACKING_SERVICE))
  object_initdata_t _object_Kerror_no_tracking_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRUST_LSA_SECRET))
  object_initdata_t _object_Kerror_no_trust_lsa_secret;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRUST_SAM_ACCOUNT))
  object_initdata_t _object_Kerror_no_trust_sam_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_UNICODE_TRANSLATION))
  object_initdata_t _object_Kerror_no_unicode_translation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_USER_KEYS))
  object_initdata_t _object_Kerror_no_user_keys;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_USER_SESSION_KEY))
  object_initdata_t _object_Kerror_no_user_session_key;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_VOLUME_ID))
  object_initdata_t _object_Kerror_no_volume_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_VOLUME_LABEL))
  object_initdata_t _object_Kerror_no_volume_label;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_WILDCARD_CHARACTERS))
  object_initdata_t _object_Kerror_no_wildcard_characters;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NT_CROSS_ENCRYPTION_REQUIRED))
  object_initdata_t _object_Kerror_nt_cross_encryption_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NULL_LM_PASSWORD))
  object_initdata_t _object_Kerror_null_lm_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_object_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_IN_LIST))
  object_initdata_t _object_Kerror_object_in_list;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_NOT_FOUND))
  object_initdata_t _object_Kerror_object_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OLD_WIN_VERSION))
  object_initdata_t _object_Kerror_old_win_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ONLY_IF_CONNECTED))
  object_initdata_t _object_Kerror_only_if_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPEN_FAILED))
  object_initdata_t _object_Kerror_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPEN_FILES))
  object_initdata_t _object_Kerror_open_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPERATION_ABORTED))
  object_initdata_t _object_Kerror_operation_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPLOCK_NOT_GRANTED))
  object_initdata_t _object_Kerror_oplock_not_granted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUTOFMEMORY))
  object_initdata_t _object_Kerror_outofmemory;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUT_OF_PAPER))
  object_initdata_t _object_Kerror_out_of_paper;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUT_OF_STRUCTURES))
  object_initdata_t _object_Kerror_out_of_structures;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OVERRIDE_NOCHANGES))
  object_initdata_t _object_Kerror_override_nochanges;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PAGED_SYSTEM_RESOURCES))
  object_initdata_t _object_Kerror_paged_system_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PAGEFILE_QUOTA))
  object_initdata_t _object_Kerror_pagefile_quota;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARAMETER_QUOTA_EXCEEDED))
  object_initdata_t _object_Kerror_parameter_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARTIAL_COPY))
  object_initdata_t _object_Kerror_partial_copy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARTITION_FAILURE))
  object_initdata_t _object_Kerror_partition_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_EXPIRED))
  object_initdata_t _object_Kerror_password_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_MUST_CHANGE))
  object_initdata_t _object_Kerror_password_must_change;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_RESTRICTION))
  object_initdata_t _object_Kerror_password_restriction;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_INVALID))
  object_initdata_t _object_Kerror_patch_package_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_OPEN_FAILED))
  object_initdata_t _object_Kerror_patch_package_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_REJECTED))
  object_initdata_t _object_Kerror_patch_package_rejected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_UNSUPPORTED))
  object_initdata_t _object_Kerror_patch_package_unsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_TARGET_NOT_FOUND))
  object_initdata_t _object_Kerror_patch_target_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATH_BUSY))
  object_initdata_t _object_Kerror_path_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATH_NOT_FOUND))
  object_initdata_t _object_Kerror_path_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PER_USER_TRUST_QUOTA_EXCEEDED))
  object_initdata_t _object_Kerror_per_user_trust_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_BUSY))
  object_initdata_t _object_Kerror_pipe_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_CONNECTED))
  object_initdata_t _object_Kerror_pipe_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_LISTENING))
  object_initdata_t _object_Kerror_pipe_listening;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_NOT_CONNECTED))
  object_initdata_t _object_Kerror_pipe_not_connected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PKINIT_FAILURE))
  object_initdata_t _object_Kerror_pkinit_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POINT_NOT_FOUND))
  object_initdata_t _object_Kerror_point_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POLICY_OBJECT_NOT_FOUND))
  object_initdata_t _object_Kerror_policy_object_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POLICY_ONLY_IN_DS))
  object_initdata_t _object_Kerror_policy_only_in_ds;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POPUP_ALREADY_ACTIVE))
  object_initdata_t _object_Kerror_popup_already_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PORT_UNREACHABLE))
  object_initdata_t _object_Kerror_port_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POSSIBLE_DEADLOCK))
  object_initdata_t _object_Kerror_possible_deadlock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POTENTIAL_FILE_FOUND))
  object_initdata_t _object_Kerror_potential_file_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_ALREADY_EXISTS))
  object_initdata_t _object_Kerror_printer_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DELETED))
  object_initdata_t _object_Kerror_printer_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_ALREADY_INSTALLED))
  object_initdata_t _object_Kerror_printer_driver_already_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_BLOCKED))
  object_initdata_t _object_Kerror_printer_driver_blocked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_IN_USE))
  object_initdata_t _object_Kerror_printer_driver_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_WARNED))
  object_initdata_t _object_Kerror_printer_driver_warned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_HAS_JOBS_QUEUED))
  object_initdata_t _object_Kerror_printer_has_jobs_queued;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_NOT_FOUND))
  object_initdata_t _object_Kerror_printer_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTQ_FULL))
  object_initdata_t _object_Kerror_printq_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_CANCELLED))
  object_initdata_t _object_Kerror_print_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_MONITOR_ALREADY_INSTALLED))
  object_initdata_t _object_Kerror_print_monitor_already_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_MONITOR_IN_USE))
  object_initdata_t _object_Kerror_print_monitor_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED))
  object_initdata_t _object_Kerror_print_processor_already_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRIVATE_DIALOG_INDEX))
  object_initdata_t _object_Kerror_private_dialog_index;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRIVILEGE_NOT_HELD))
  object_initdata_t _object_Kerror_privilege_not_held;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROCESS_ABORTED))
  object_initdata_t _object_Kerror_process_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROC_NOT_FOUND))
  object_initdata_t _object_Kerror_proc_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRODUCT_UNINSTALLED))
  object_initdata_t _object_Kerror_product_uninstalled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRODUCT_VERSION))
  object_initdata_t _object_Kerror_product_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE))
  object_initdata_t _object_Kerror_profile_not_associated_with_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROFILE_NOT_FOUND))
  object_initdata_t _object_Kerror_profile_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROMOTION_ACTIVE))
  object_initdata_t _object_Kerror_promotion_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROTOCOL_UNREACHABLE))
  object_initdata_t _object_Kerror_protocol_unreachable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUMLOG_OPEN_FAILED))
  object_initdata_t _object_Kerror_quorumlog_open_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_DISK_NOT_FOUND))
  object_initdata_t _object_Kerror_quorum_disk_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_OWNER_ALIVE))
  object_initdata_t _object_Kerror_quorum_owner_alive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_RESOURCE))
  object_initdata_t _object_Kerror_quorum_resource;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_RESOURCE_ONLINE_FAILED))
  object_initdata_t _object_Kerror_quorum_resource_online_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_READ_FAULT))
  object_initdata_t _object_Kerror_read_fault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RECOVERY_FAILURE))
  object_initdata_t _object_Kerror_recovery_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REC_NON_EXISTENT))
  object_initdata_t _object_Kerror_rec_non_existent;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REDIRECTOR_HAS_OPEN_HANDLES))
  object_initdata_t _object_Kerror_redirector_has_open_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REDIR_PAUSED))
  object_initdata_t _object_Kerror_redir_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_CORRUPT))
  object_initdata_t _object_Kerror_registry_corrupt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_IO_FAILED))
  object_initdata_t _object_Kerror_registry_io_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_RECOVERED))
  object_initdata_t _object_Kerror_registry_recovered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REG_NAT_CONSUMPTION))
  object_initdata_t _object_Kerror_reg_nat_consumption;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RELOC_CHAIN_XEEDS_SEGLIM))
  object_initdata_t _object_Kerror_reloc_chain_xeeds_seglim;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_SESSION_LIMIT_EXCEEDED))
  object_initdata_t _object_Kerror_remote_session_limit_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_STORAGE_MEDIA_ERROR))
  object_initdata_t _object_Kerror_remote_storage_media_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_STORAGE_NOT_ACTIVE))
  object_initdata_t _object_Kerror_remote_storage_not_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REM_NOT_LIST))
  object_initdata_t _object_Kerror_rem_not_list;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_ATTRIBUTE_CONFLICT))
  object_initdata_t _object_Kerror_reparse_attribute_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_TAG_INVALID))
  object_initdata_t _object_Kerror_reparse_tag_invalid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_TAG_MISMATCH))
  object_initdata_t _object_Kerror_reparse_tag_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUEST_ABORTED))
  object_initdata_t _object_Kerror_request_aborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUEST_REFUSED))
  object_initdata_t _object_Kerror_request_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION))
  object_initdata_t _object_Kerror_requires_interactive_windowstation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQ_NOT_ACCEP))
  object_initdata_t _object_Kerror_req_not_accep;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_CREATE_FAILED))
  object_initdata_t _object_Kerror_resmon_create_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_INVALID_STATE))
  object_initdata_t _object_Kerror_resmon_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_ONLINE_FAILED))
  object_initdata_t _object_Kerror_resmon_online_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_DATA_NOT_FOUND))
  object_initdata_t _object_Kerror_resource_data_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_DISABLED))
  object_initdata_t _object_Kerror_resource_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_FAILED))
  object_initdata_t _object_Kerror_resource_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_LANG_NOT_FOUND))
  object_initdata_t _object_Kerror_resource_lang_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NAME_NOT_FOUND))
  object_initdata_t _object_Kerror_resource_name_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_AVAILABLE))
  object_initdata_t _object_Kerror_resource_not_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_FOUND))
  object_initdata_t _object_Kerror_resource_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_ONLINE))
  object_initdata_t _object_Kerror_resource_not_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_PRESENT))
  object_initdata_t _object_Kerror_resource_not_present;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_ONLINE))
  object_initdata_t _object_Kerror_resource_online;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_PROPERTIES_STORED))
  object_initdata_t _object_Kerror_resource_properties_stored;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_PROPERTY_UNCHANGEABLE))
  object_initdata_t _object_Kerror_resource_property_unchangeable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_TYPE_NOT_FOUND))
  object_initdata_t _object_Kerror_resource_type_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RETRY))
  object_initdata_t _object_Kerror_retry;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REVISION_MISMATCH))
  object_initdata_t _object_Kerror_revision_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RING2SEG_MUST_BE_MOVABLE))
  object_initdata_t _object_Kerror_ring2seg_must_be_movable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RING2_STACK_IN_USE))
  object_initdata_t _object_Kerror_ring2_stack_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RMODE_APP))
  object_initdata_t _object_Kerror_rmode_app;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RPL_NOT_ALLOWED))
  object_initdata_t _object_Kerror_rpl_not_allowed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RXACT_COMMIT_FAILURE))
  object_initdata_t _object_Kerror_rxact_commit_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RXACT_INVALID_STATE))
  object_initdata_t _object_Kerror_rxact_invalid_state;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SAME_DRIVE))
  object_initdata_t _object_Kerror_same_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SAM_INIT_FAILURE))
  object_initdata_t _object_Kerror_sam_init_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SCOPE_NOT_FOUND))
  object_initdata_t _object_Kerror_scope_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SCREEN_ALREADY_LOCKED))
  object_initdata_t _object_Kerror_screen_already_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SECRET_TOO_LONG))
  object_initdata_t _object_Kerror_secret_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SECTOR_NOT_FOUND))
  object_initdata_t _object_Kerror_sector_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEEK))
  object_initdata_t _object_Kerror_seek;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEEK_ON_DEVICE))
  object_initdata_t _object_Kerror_seek_on_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_IS_SET))
  object_initdata_t _object_Kerror_sem_is_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_NOT_FOUND))
  object_initdata_t _object_Kerror_sem_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_OWNER_DIED))
  object_initdata_t _object_Kerror_sem_owner_died;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_TIMEOUT))
  object_initdata_t _object_Kerror_sem_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_USER_LIMIT))
  object_initdata_t _object_Kerror_sem_user_limit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERIAL_NO_DEVICE))
  object_initdata_t _object_Kerror_serial_no_device;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_DISABLED))
  object_initdata_t _object_Kerror_server_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_HAS_OPEN_HANDLES))
  object_initdata_t _object_Kerror_server_has_open_handles;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_NOT_DISABLED))
  object_initdata_t _object_Kerror_server_not_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_SHUTDOWN_IN_PROGRESS))
  object_initdata_t _object_Kerror_server_shutdown_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_ALREADY_RUNNING))
  object_initdata_t _object_Kerror_service_already_running;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_CANNOT_ACCEPT_CTRL))
  object_initdata_t _object_Kerror_service_cannot_accept_ctrl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DATABASE_LOCKED))
  object_initdata_t _object_Kerror_service_database_locked;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DEPENDENCY_DELETED))
  object_initdata_t _object_Kerror_service_dependency_deleted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DEPENDENCY_FAIL))
  object_initdata_t _object_Kerror_service_dependency_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DISABLED))
  object_initdata_t _object_Kerror_service_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DOES_NOT_EXIST))
  object_initdata_t _object_Kerror_service_does_not_exist;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_EXISTS))
  object_initdata_t _object_Kerror_service_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_LOGON_FAILED))
  object_initdata_t _object_Kerror_service_logon_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_MARKED_FOR_DELETE))
  object_initdata_t _object_Kerror_service_marked_for_delete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NEVER_STARTED))
  object_initdata_t _object_Kerror_service_never_started;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_ACTIVE))
  object_initdata_t _object_Kerror_service_not_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_FOUND))
  object_initdata_t _object_Kerror_service_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_IN_EXE))
  object_initdata_t _object_Kerror_service_not_in_exe;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NO_THREAD))
  object_initdata_t _object_Kerror_service_no_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_REQUEST_TIMEOUT))
  object_initdata_t _object_Kerror_service_request_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_SPECIFIC_ERROR))
  object_initdata_t _object_Kerror_service_specific_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_START_HANG))
  object_initdata_t _object_Kerror_service_start_hang;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SESSION_CREDENTIAL_CONFLICT))
  object_initdata_t _object_Kerror_session_credential_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SETCOUNT_ON_BAD_LB))
  object_initdata_t _object_Kerror_setcount_on_bad_lb;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SETMARK_DETECTED))
  object_initdata_t _object_Kerror_setmark_detected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_NOT_FOUND))
  object_initdata_t _object_Kerror_set_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_POWER_STATE_FAILED))
  object_initdata_t _object_Kerror_set_power_state_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_POWER_STATE_VETOED))
  object_initdata_t _object_Kerror_set_power_state_vetoed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARED_POLICY))
  object_initdata_t _object_Kerror_shared_policy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_BUFFER_EXCEEDED))
  object_initdata_t _object_Kerror_sharing_buffer_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_PAUSED))
  object_initdata_t _object_Kerror_sharing_paused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_VIOLATION))
  object_initdata_t _object_Kerror_sharing_violation;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHUTDOWN_CLUSTER))
  object_initdata_t _object_Kerror_shutdown_cluster;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHUTDOWN_IN_PROGRESS))
  object_initdata_t _object_Kerror_shutdown_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SIGNAL_PENDING))
  object_initdata_t _object_Kerror_signal_pending;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SIGNAL_REFUSED))
  object_initdata_t _object_Kerror_signal_refused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SINGLE_INSTANCE_APP))
  object_initdata_t _object_Kerror_single_instance_app;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SMARTCARD_SUBSYSTEM_FAILURE))
  object_initdata_t _object_Kerror_smartcard_subsystem_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SOME_NOT_MAPPED))
  object_initdata_t _object_Kerror_some_not_mapped;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SOURCE_ELEMENT_EMPTY))
  object_initdata_t _object_Kerror_source_element_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_ACCOUNT))
  object_initdata_t _object_Kerror_special_account;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_GROUP))
  object_initdata_t _object_Kerror_special_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_USER))
  object_initdata_t _object_Kerror_special_user;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPL_NO_ADDJOB))
  object_initdata_t _object_Kerror_spl_no_addjob;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPL_NO_STARTDOC))
  object_initdata_t _object_Kerror_spl_no_startdoc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPOOL_FILE_NOT_FOUND))
  object_initdata_t _object_Kerror_spool_file_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STACK_BUFFER_OVERRUN))
  object_initdata_t _object_Kerror_stack_buffer_overrun;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STACK_OVERFLOW))
  object_initdata_t _object_Kerror_stack_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STATIC_INIT))
  object_initdata_t _object_Kerror_static_init;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUBST_TO_JOIN))
  object_initdata_t _object_Kerror_subst_to_join;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUBST_TO_SUBST))
  object_initdata_t _object_Kerror_subst_to_subst;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_REBOOT_INITIATED))
  object_initdata_t _object_Kerror_success_reboot_initiated;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_REBOOT_REQUIRED))
  object_initdata_t _object_Kerror_success_reboot_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_RESTART_REQUIRED))
  object_initdata_t _object_Kerror_success_restart_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SWAPERROR))
  object_initdata_t _object_Kerror_swaperror;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ACTIVATION_CONTEXT_DISABLED))
  object_initdata_t _object_Kerror_sxs_activation_context_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ASSEMBLY_NOT_FOUND))
  object_initdata_t _object_Kerror_sxs_assembly_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_CANT_GEN_ACTCTX))
  object_initdata_t _object_Kerror_sxs_cant_gen_actctx;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_ASSEMBLY_NAME))
  object_initdata_t _object_Kerror_sxs_duplicate_assembly_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_CLSID))
  object_initdata_t _object_Kerror_sxs_duplicate_clsid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_DLL_NAME))
  object_initdata_t _object_Kerror_sxs_duplicate_dll_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_IID))
  object_initdata_t _object_Kerror_sxs_duplicate_iid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_PROGID))
  object_initdata_t _object_Kerror_sxs_duplicate_progid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_TLBID))
  object_initdata_t _object_Kerror_sxs_duplicate_tlbid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME))
  object_initdata_t _object_Kerror_sxs_duplicate_windowclass_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_FILE_HASH_MISMATCH))
  object_initdata_t _object_Kerror_sxs_file_hash_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ACTCTXDATA_FORMAT))
  object_initdata_t _object_Kerror_sxs_invalid_actctxdata_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE))
  object_initdata_t _object_Kerror_sxs_invalid_assembly_identity_attribute;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME))
  object_initdata_t _object_Kerror_sxs_invalid_assembly_identity_attribute_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_XML_NAMESPACE_URI))
  object_initdata_t _object_Kerror_sxs_invalid_xml_namespace_uri;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_KEY_NOT_FOUND))
  object_initdata_t _object_Kerror_sxs_key_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED))
  object_initdata_t _object_Kerror_sxs_leaf_manifest_dependency_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_FORMAT_ERROR))
  object_initdata_t _object_Kerror_sxs_manifest_format_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE))
  object_initdata_t _object_Kerror_sxs_manifest_invalid_required_default_namespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE))
  object_initdata_t _object_Kerror_sxs_manifest_missing_required_default_namespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_PARSE_ERROR))
  object_initdata_t _object_Kerror_sxs_manifest_parse_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE))
  object_initdata_t _object_Kerror_sxs_missing_assembly_identity_attribute;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_POLICY_PARSE_ERROR))
  object_initdata_t _object_Kerror_sxs_policy_parse_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT))
  object_initdata_t _object_Kerror_sxs_private_manifest_cross_path_with_reparse_point;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET))
  object_initdata_t _object_Kerror_sxs_process_default_already_set;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING))
  object_initdata_t _object_Kerror_sxs_protection_catalog_file_missing;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_CATALOG_NOT_VALID))
  object_initdata_t _object_Kerror_sxs_protection_catalog_not_valid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT))
  object_initdata_t _object_Kerror_sxs_protection_public_key_too_short;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_RECOVERY_FAILED))
  object_initdata_t _object_Kerror_sxs_protection_recovery_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED))
  object_initdata_t _object_Kerror_sxs_root_manifest_dependency_not_installed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_SECTION_NOT_FOUND))
  object_initdata_t _object_Kerror_sxs_section_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_THREAD_QUERIES_DISABLED))
  object_initdata_t _object_Kerror_sxs_thread_queries_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNKNOWN_ENCODING))
  object_initdata_t _object_Kerror_sxs_unknown_encoding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNKNOWN_ENCODING_GROUP))
  object_initdata_t _object_Kerror_sxs_unknown_encoding_group;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNTRANSLATABLE_HRESULT))
  object_initdata_t _object_Kerror_sxs_untranslatable_hresult;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_VERSION_CONFLICT))
  object_initdata_t _object_Kerror_sxs_version_conflict;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_WRONG_SECTION_TYPE))
  object_initdata_t _object_Kerror_sxs_wrong_section_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADCHARDATA))
  object_initdata_t _object_Kerror_sxs_xml_e_badchardata;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADCHARINSTRING))
  object_initdata_t _object_Kerror_sxs_xml_e_badcharinstring;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADNAMECHAR))
  object_initdata_t _object_Kerror_sxs_xml_e_badnamechar;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADPEREFINSUBSET))
  object_initdata_t _object_Kerror_sxs_xml_e_badperefinsubset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADSTARTNAMECHAR))
  object_initdata_t _object_Kerror_sxs_xml_e_badstartnamechar;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADXMLCASE))
  object_initdata_t _object_Kerror_sxs_xml_e_badxmlcase;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADXMLDECL))
  object_initdata_t _object_Kerror_sxs_xml_e_badxmldecl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_COMMENTSYNTAX))
  object_initdata_t _object_Kerror_sxs_xml_e_commentsyntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_DUPLICATEATTRIBUTE))
  object_initdata_t _object_Kerror_sxs_xml_e_duplicateattribute;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE))
  object_initdata_t _object_Kerror_sxs_xml_e_expectingclosequote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_EXPECTINGTAGEND))
  object_initdata_t _object_Kerror_sxs_xml_e_expectingtagend;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INCOMPLETE_ENCODING))
  object_initdata_t _object_Kerror_sxs_xml_e_incomplete_encoding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INTERNALERROR))
  object_initdata_t _object_Kerror_sxs_xml_e_internalerror;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDATROOTLEVEL))
  object_initdata_t _object_Kerror_sxs_xml_e_invalidatrootlevel;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDENCODING))
  object_initdata_t _object_Kerror_sxs_xml_e_invalidencoding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDSWITCH))
  object_initdata_t _object_Kerror_sxs_xml_e_invalidswitch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_DECIMAL))
  object_initdata_t _object_Kerror_sxs_xml_e_invalid_decimal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_HEXIDECIMAL))
  object_initdata_t _object_Kerror_sxs_xml_e_invalid_hexidecimal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_STANDALONE))
  object_initdata_t _object_Kerror_sxs_xml_e_invalid_standalone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_UNICODE))
  object_initdata_t _object_Kerror_sxs_xml_e_invalid_unicode;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_VERSION))
  object_initdata_t _object_Kerror_sxs_xml_e_invalid_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGEQUALS))
  object_initdata_t _object_Kerror_sxs_xml_e_missingequals;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGQUOTE))
  object_initdata_t _object_Kerror_sxs_xml_e_missingquote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGROOT))
  object_initdata_t _object_Kerror_sxs_xml_e_missingroot;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGSEMICOLON))
  object_initdata_t _object_Kerror_sxs_xml_e_missingsemicolon;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGWHITESPACE))
  object_initdata_t _object_Kerror_sxs_xml_e_missingwhitespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSING_PAREN))
  object_initdata_t _object_Kerror_sxs_xml_e_missing_paren;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MULTIPLEROOTS))
  object_initdata_t _object_Kerror_sxs_xml_e_multipleroots;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MULTIPLE_COLONS))
  object_initdata_t _object_Kerror_sxs_xml_e_multiple_colons;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_RESERVEDNAMESPACE))
  object_initdata_t _object_Kerror_sxs_xml_e_reservednamespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNBALANCEDPAREN))
  object_initdata_t _object_Kerror_sxs_xml_e_unbalancedparen;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDCDATA))
  object_initdata_t _object_Kerror_sxs_xml_e_unclosedcdata;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDCOMMENT))
  object_initdata_t _object_Kerror_sxs_xml_e_unclosedcomment;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDDECL))
  object_initdata_t _object_Kerror_sxs_xml_e_uncloseddecl;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDENDTAG))
  object_initdata_t _object_Kerror_sxs_xml_e_unclosedendtag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDSTARTTAG))
  object_initdata_t _object_Kerror_sxs_xml_e_unclosedstarttag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDSTRING))
  object_initdata_t _object_Kerror_sxs_xml_e_unclosedstring;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDTAG))
  object_initdata_t _object_Kerror_sxs_xml_e_unclosedtag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTEDENDTAG))
  object_initdata_t _object_Kerror_sxs_xml_e_unexpectedendtag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTEDEOF))
  object_initdata_t _object_Kerror_sxs_xml_e_unexpectedeof;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTED_STANDALONE))
  object_initdata_t _object_Kerror_sxs_xml_e_unexpected_standalone;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE))
  object_initdata_t _object_Kerror_sxs_xml_e_unexpected_whitespace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK))
  object_initdata_t _object_Kerror_sxs_xml_e_whitespaceorquestionmark;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_XMLDECLSYNTAX))
  object_initdata_t _object_Kerror_sxs_xml_e_xmldeclsyntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED))
  object_initdata_t _object_Kerror_sync_foreground_refresh_required;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SYSTEM_TRACE))
  object_initdata_t _object_Kerror_system_trace;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TAG_NOT_FOUND))
  object_initdata_t _object_Kerror_tag_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TAG_NOT_PRESENT))
  object_initdata_t _object_Kerror_tag_not_present;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_THREAD_1_INACTIVE))
  object_initdata_t _object_Kerror_thread_1_inactive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TIMEOUT))
  object_initdata_t _object_Kerror_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TIME_SKEW))
  object_initdata_t _object_Kerror_time_skew;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TLW_WITH_WSCHILD))
  object_initdata_t _object_Kerror_tlw_with_wschild;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOKEN_ALREADY_IN_USE))
  object_initdata_t _object_Kerror_token_already_in_use;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_CMDS))
  object_initdata_t _object_Kerror_too_many_cmds;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_CONTEXT_IDS))
  object_initdata_t _object_Kerror_too_many_context_ids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_LINKS))
  object_initdata_t _object_Kerror_too_many_links;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_LUIDS_REQUESTED))
  object_initdata_t _object_Kerror_too_many_luids_requested;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_MODULES))
  object_initdata_t _object_Kerror_too_many_modules;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_MUXWAITERS))
  object_initdata_t _object_Kerror_too_many_muxwaiters;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_NAMES))
  object_initdata_t _object_Kerror_too_many_names;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_OPEN_FILES))
  object_initdata_t _object_Kerror_too_many_open_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_POSTS))
  object_initdata_t _object_Kerror_too_many_posts;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SECRETS))
  object_initdata_t _object_Kerror_too_many_secrets;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SEMAPHORES))
  object_initdata_t _object_Kerror_too_many_semaphores;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SEM_REQUESTS))
  object_initdata_t _object_Kerror_too_many_sem_requests;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SESS))
  object_initdata_t _object_Kerror_too_many_sess;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SIDS))
  object_initdata_t _object_Kerror_too_many_sids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_TCBS))
  object_initdata_t _object_Kerror_too_many_tcbs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRANSFORM_NOT_SUPPORTED))
  object_initdata_t _object_Kerror_transform_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRANSPORT_FULL))
  object_initdata_t _object_Kerror_transport_full;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUSTED_DOMAIN_FAILURE))
  object_initdata_t _object_Kerror_trusted_domain_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUSTED_RELATIONSHIP_FAILURE))
  object_initdata_t _object_Kerror_trusted_relationship_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUST_FAILURE))
  object_initdata_t _object_Kerror_trust_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_CLEAN))
  object_initdata_t _object_Kerror_unable_to_clean;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA))
  object_initdata_t _object_Kerror_unable_to_eject_mounted_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_DRIVE))
  object_initdata_t _object_Kerror_unable_to_inventory_drive;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_SLOT))
  object_initdata_t _object_Kerror_unable_to_inventory_slot;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_TRANSPORT))
  object_initdata_t _object_Kerror_unable_to_inventory_transport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_LOAD_MEDIUM))
  object_initdata_t _object_Kerror_unable_to_load_medium;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_LOCK_MEDIA))
  object_initdata_t _object_Kerror_unable_to_lock_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT))
  object_initdata_t _object_Kerror_unable_to_move_replacement;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT_2))
  object_initdata_t _object_Kerror_unable_to_move_replacement_2;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_REMOVE_REPLACED))
  object_initdata_t _object_Kerror_unable_to_remove_replaced;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_UNLOAD_MEDIA))
  object_initdata_t _object_Kerror_unable_to_unload_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNEXPECTED_OMID))
  object_initdata_t _object_Kerror_unexpected_omid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNEXP_NET_ERR))
  object_initdata_t _object_Kerror_unexp_net_err;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_COMPONENT))
  object_initdata_t _object_Kerror_unknown_component;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_FEATURE))
  object_initdata_t _object_Kerror_unknown_feature;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PORT))
  object_initdata_t _object_Kerror_unknown_port;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINTER_DRIVER))
  object_initdata_t _object_Kerror_unknown_printer_driver;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINTPROCESSOR))
  object_initdata_t _object_Kerror_unknown_printprocessor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINT_MONITOR))
  object_initdata_t _object_Kerror_unknown_print_monitor;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRODUCT))
  object_initdata_t _object_Kerror_unknown_product;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PROPERTY))
  object_initdata_t _object_Kerror_unknown_property;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_REVISION))
  object_initdata_t _object_Kerror_unknown_revision;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNRECOGNIZED_MEDIA))
  object_initdata_t _object_Kerror_unrecognized_media;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNRECOGNIZED_VOLUME))
  object_initdata_t _object_Kerror_unrecognized_volume;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNSUPPORTED_TYPE))
  object_initdata_t _object_Kerror_unsupported_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED))
  object_initdata_t _object_Kerror_user_delete_trust_quota_exceeded;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_EXISTS))
  object_initdata_t _object_Kerror_user_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_MAPPED_FILE))
  object_initdata_t _object_Kerror_user_mapped_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VC_DISCONNECTED))
  object_initdata_t _object_Kerror_vc_disconnected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_CONTAINS_SYS_FILES))
  object_initdata_t _object_Kerror_volume_contains_sys_files;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_NOT_SIS_ENABLED))
  object_initdata_t _object_Kerror_volume_not_sis_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_NOT_SUPPORT_EFS))
  object_initdata_t _object_Kerror_volume_not_support_efs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WAIT_NO_CHILDREN))
  object_initdata_t _object_Kerror_wait_no_children;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_NOT_COMBOBOX))
  object_initdata_t _object_Kerror_window_not_combobox;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_NOT_DIALOG))
  object_initdata_t _object_Kerror_window_not_dialog;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_OF_OTHER_THREAD))
  object_initdata_t _object_Kerror_window_of_other_thread;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINS_INTERNAL))
  object_initdata_t _object_Kerror_wins_internal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ALREADY_DISABLED))
  object_initdata_t _object_Kerror_wmi_already_disabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ALREADY_ENABLED))
  object_initdata_t _object_Kerror_wmi_already_enabled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_DP_FAILED))
  object_initdata_t _object_Kerror_wmi_dp_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_DP_NOT_FOUND))
  object_initdata_t _object_Kerror_wmi_dp_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_GUID_DISCONNECTED))
  object_initdata_t _object_Kerror_wmi_guid_disconnected;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_GUID_NOT_FOUND))
  object_initdata_t _object_Kerror_wmi_guid_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INSTANCE_NOT_FOUND))
  object_initdata_t _object_Kerror_wmi_instance_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INVALID_MOF))
  object_initdata_t _object_Kerror_wmi_invalid_mof;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INVALID_REGINFO))
  object_initdata_t _object_Kerror_wmi_invalid_reginfo;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ITEMID_NOT_FOUND))
  object_initdata_t _object_Kerror_wmi_itemid_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_READ_ONLY))
  object_initdata_t _object_Kerror_wmi_read_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_SERVER_UNAVAILABLE))
  object_initdata_t _object_Kerror_wmi_server_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_SET_FAILURE))
  object_initdata_t _object_Kerror_wmi_set_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_TRY_AGAIN))
  object_initdata_t _object_Kerror_wmi_try_again;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_UNRESOLVED_INSTANCE_REF))
  object_initdata_t _object_Kerror_wmi_unresolved_instance_ref;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WORKING_SET_QUOTA))
  object_initdata_t _object_Kerror_working_set_quota;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRITE_FAULT))
  object_initdata_t _object_Kerror_write_fault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRITE_PROTECT))
  object_initdata_t _object_Kerror_write_protect;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_DISK))
  object_initdata_t _object_Kerror_wrong_disk;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_EFS))
  object_initdata_t _object_Kerror_wrong_efs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_PASSWORD))
  object_initdata_t _object_Kerror_wrong_password;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_TARGET_NAME))
  object_initdata_t _object_Kerror_wrong_target_name;
#endif
#if defined(ESHUTDOWN)
  object_initdata_t _object_Keshutdown;
#endif
#if defined(ESOCKTNOSUPPORT)
  object_initdata_t _object_Kesocktnosupport;
#endif
#if defined(ESPIPE)
  object_initdata_t _object_Kespipe;
#endif
#if defined(ESRCH)
  object_initdata_t _object_Kesrch;
#endif
#if defined(ESRMNT)
  object_initdata_t _object_Kesrmnt;
#endif
#if defined(ESTALE)
  object_initdata_t _object_Kestale;
#endif
#if defined(ESTRPIPE)
  object_initdata_t _object_Kestrpipe;
#endif
#if defined(ETIME)
  object_initdata_t _object_Ketime;
#endif
#if defined(ETIMEDOUT)
  object_initdata_t _object_Ketimedout;
#endif
#if defined(ETOOMANYREFS)
  object_initdata_t _object_Ketoomanyrefs;
#endif
#if defined(ETXTBSY)
  object_initdata_t _object_Ketxtbsy;
#endif
#if defined(EUCLEAN)
  object_initdata_t _object_Keuclean;
#endif
#if defined(EUNATCH)
  object_initdata_t _object_Keunatch;
#endif
#if defined(EUSERS)
  object_initdata_t _object_Keusers;
#endif
#if defined(EVERSION)
  object_initdata_t _object_Keversion;
#endif
#if defined(EWOULDBLOCK)
  object_initdata_t _object_Kewouldblock;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_EXCL))
  object_initdata_t _object_Kexcl;
#endif
#if defined(EXDEV)
  object_initdata_t _object_Kexdev;
#endif
#if defined(EXFULL)
  object_initdata_t _object_Kexfull;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_Kexited;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_EXPR_NEST_MAX))
  object_initdata_t _object_Kexpr_nest_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kext;
#endif
  object_initdata_t _object_Kexternal_format;
#if (defined(HAVE_NFTW)) && (defined(FTW_F))
  object_initdata_t _object_Kf;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  object_initdata_t _object_Kfacility;
#endif
#if defined(S_IFBLK)
  object_initdata_t _object_Kfblk;
#endif
#if defined(S_IFCHR)
  object_initdata_t _object_Kfchr;
#endif
#if (defined(HAVE_FCNTL)) && (defined(F_GETFD))
  object_initdata_t _object_Kfd;
#endif
#if defined(S_IFDIR)
  object_initdata_t _object_Kfdir;
#endif
#if defined(HAVE_NFTW)
  object_initdata_t _object_Kfd_limit;
#endif
#if defined(S_IFIFO)
  object_initdata_t _object_Kfifo;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_FILESIZEBITS))
  object_initdata_t _object_Kfilesizebits;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kfiletime;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_NAMED_STREAMS))
  object_initdata_t _object_Kfile_named_streams;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_READ_ONLY_VOLUME))
  object_initdata_t _object_Kfile_read_only_volume;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_ENCRYPTION))
  object_initdata_t _object_Kfile_supports_encryption;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_OBJECT_IDS))
  object_initdata_t _object_Kfile_supports_object_ids;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_REPARSE_POINTS))
  object_initdata_t _object_Kfile_supports_reparse_points;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_SPARSE_FILES))
  object_initdata_t _object_Kfile_supports_sparse_files;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_VOLUME_QUOTAS))
  object_initdata_t _object_Kfile_volume_quotas;
#endif
#if (defined(HAVE_FCNTL)) && (defined(F_GETFL))
  object_initdata_t _object_Kfl;
#endif
#if defined(S_IFREG)
  object_initdata_t _object_Kfreg;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_AUTHENTICATION))
  object_initdata_t _object_Kfrs_err_authentication;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_CHILD_TO_PARENT_COMM))
  object_initdata_t _object_Kfrs_err_child_to_parent_comm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INSUFFICIENT_PRIV))
  object_initdata_t _object_Kfrs_err_insufficient_priv;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INTERNAL))
  object_initdata_t _object_Kfrs_err_internal;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INTERNAL_API))
  object_initdata_t _object_Kfrs_err_internal_api;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INVALID_API_SEQUENCE))
  object_initdata_t _object_Kfrs_err_invalid_api_sequence;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INVALID_SERVICE_PARAMETER))
  object_initdata_t _object_Kfrs_err_invalid_service_parameter;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_AUTHENTICATION))
  object_initdata_t _object_Kfrs_err_parent_authentication;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_INSUFFICIENT_PRIV))
  object_initdata_t _object_Kfrs_err_parent_insufficient_priv;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_TO_CHILD_COMM))
  object_initdata_t _object_Kfrs_err_parent_to_child_comm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SERVICE_COMM))
  object_initdata_t _object_Kfrs_err_service_comm;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_STARTING_SERVICE))
  object_initdata_t _object_Kfrs_err_starting_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_STOPPING_SERVICE))
  object_initdata_t _object_Kfrs_err_stopping_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_DEMOTE))
  object_initdata_t _object_Kfrs_err_sysvol_demote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_IS_BUSY))
  object_initdata_t _object_Kfrs_err_sysvol_is_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_POPULATE))
  object_initdata_t _object_Kfrs_err_sysvol_populate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_POPULATE_TIMEOUT))
  object_initdata_t _object_Kfrs_err_sysvol_populate_timeout;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_FSIZE))
  object_initdata_t _object_Kfsize;
#endif
#if defined(S_IFSOCK)
  object_initdata_t _object_Kfsock;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_FSYNC))
  object_initdata_t _object_Kfsync;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_CASE_IS_PRESERVED))
  object_initdata_t _object_Kfs_case_is_preserved;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_CASE_SENSITIVE))
  object_initdata_t _object_Kfs_case_sensitive;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_FILE_COMPRESSION))
  object_initdata_t _object_Kfs_file_compression;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_FILE_ENCRYPTION))
  object_initdata_t _object_Kfs_file_encryption;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_PERSISTENT_ACLS))
  object_initdata_t _object_Kfs_persistent_acls;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_UNICODE_STORED_ON_DISK))
  object_initdata_t _object_Kfs_unicode_stored_on_disk;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_VOL_IS_COMPRESSED))
  object_initdata_t _object_Kfs_vol_is_compressed;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_FTP))
  object_initdata_t _object_Kftp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_GETGR_R_SIZE_MAX))
  object_initdata_t _object_Kgetgr_r_size_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_GETPW_R_SIZE_MAX))
  object_initdata_t _object_Kgetpw_r_size_max;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  object_initdata_t _object_Kgid;
#endif
  object_initdata_t _object_Khardlink;
  object_initdata_t _object_Khardlink_or_copy;
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_HIDDEN))
  object_initdata_t _object_Khidden;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(HIGH_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  object_initdata_t _object_Khigh;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_HOST_NAME_MAX))
  object_initdata_t _object_Khost_name_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Khot_key;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ki1;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ki2;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ki4;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ki8;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_IA32_ON_WIN64))
  object_initdata_t _object_Kia32_on_win64;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_IA64))
  object_initdata_t _object_Kia64;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kicon;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(IDLE_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  object_initdata_t _object_Kidle;
#endif
  object_initdata_t _object_Kif_does_not_exist;
  object_initdata_t _object_Kif_exists;
#if (defined(HAVE_SYSLOG)) && (defined(LOG_INFO))
  object_initdata_t _object_Kinfo;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kinitid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(INIT_PROCESS))
  object_initdata_t _object_Kinit_process;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_INTEL))
  object_initdata_t _object_Kintel;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_IOV_MAX))
  object_initdata_t _object_Kiov_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_IPV6))
  object_initdata_t _object_Kipv6;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_JOB_CONTROL))
  object_initdata_t _object_Kjob_control;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_KERN))
  object_initdata_t _object_Kkern;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kkeywords;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_LARGEFILE))
  object_initdata_t _object_Klargefile;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Klastauthor;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Klastprinted;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Klastsave_dtm;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  object_initdata_t _object_Klength;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_LINE_MAX))
  object_initdata_t _object_Kline_max;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_LINK_MAX))
  object_initdata_t _object_Klink_max;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL0))
  object_initdata_t _object_Klocal0;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL1))
  object_initdata_t _object_Klocal1;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL2))
  object_initdata_t _object_Klocal2;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL3))
  object_initdata_t _object_Klocal3;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL4))
  object_initdata_t _object_Klocal4;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL5))
  object_initdata_t _object_Klocal5;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL6))
  object_initdata_t _object_Klocal6;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL7))
  object_initdata_t _object_Klocal7;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Klocale;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_LOCKS))
  object_initdata_t _object_Klocks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_LOGIN_NAME_MAX))
  object_initdata_t _object_Klogin_name_max;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(LOGIN_PROCESS))
  object_initdata_t _object_Klogin_process;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(LOW_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  object_initdata_t _object_Klow;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LPR))
  object_initdata_t _object_Klpr;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Klpstr;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Klpwstr;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_LWP))
  object_initdata_t _object_Klwp;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_MAIL))
  object_initdata_t _object_Kmail;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MAPPED_FILES))
  object_initdata_t _object_Kmapped_files;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kmax;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_MAX_CANON))
  object_initdata_t _object_Kmax_canon;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_MAX_INPUT))
  object_initdata_t _object_Kmax_input;
#endif
#if ((defined(HAVE_SYSCONF)) && (defined(_SC_MEMLOCK))) || ((defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_MEMLOCK)))
  object_initdata_t _object_Kmemlock;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MEMLOCK_RANGE))
  object_initdata_t _object_Kmemlock_range;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MEMORY_PROTECTION))
  object_initdata_t _object_Kmemory_protection;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MESSAGE_PASSING))
  object_initdata_t _object_Kmessage_passing;
#endif
  object_initdata_t _object_Kmethod;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kmin;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_MIPS))
  object_initdata_t _object_Kmips;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  object_initdata_t _object_Kmode;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MONOTONIC_CLOCK))
  object_initdata_t _object_Kmonotonic_clock;
#endif
#if defined(HAVE_NFTW)
  object_initdata_t _object_Kmount;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MQ_OPEN_MAX))
  object_initdata_t _object_Kmq_open_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MQ_PRIO_MAX))
  object_initdata_t _object_Kmq_prio_max;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_MSIL))
  object_initdata_t _object_Kmsil;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  object_initdata_t _object_Kmtime;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_NAME_MAX))
  object_initdata_t _object_Kname_max;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  object_initdata_t _object_Kndelay;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_NEWS))
  object_initdata_t _object_Knews;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(NEW_TIME))
  object_initdata_t _object_Knew_time;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NGROUPS_MAX))
  object_initdata_t _object_Kngroups_max;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOCTTY))
  object_initdata_t _object_Knoctty;
#endif
  object_initdata_t _object_Knoescape;
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_NOFILE))
  object_initdata_t _object_Knofile;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOFOLLOW))
  object_initdata_t _object_Knofollow;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_Knohang;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOINHERIT))
  object_initdata_t _object_Knoinherit;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NONBLOCK))
  object_initdata_t _object_Knonblock;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE))) || ((defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_NORMAL)))
  object_initdata_t _object_Knormal;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_NOTICE))
  object_initdata_t _object_Knotice;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Knotimplemented;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_NOT_CONTENT_INDEXED))
  object_initdata_t _object_Knot_content_indexed;
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))) || (defined(HAVE_SYS_RESOURCE_H))
  object_initdata_t _object_Knowait;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_NO_TRUNC))
  object_initdata_t _object_Kno_trunc;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_NPROC))
  object_initdata_t _object_Knproc;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NPROCESSORS_CONF))
  object_initdata_t _object_Knprocessors_conf;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NPROCESSORS_ONLN))
  object_initdata_t _object_Knprocessors_onln;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_NS))
  object_initdata_t _object_Kns;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Knt;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Knull;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  object_initdata_t _object_Kodelay;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_OFFLINE))
  object_initdata_t _object_Koffline;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(OLD_TIME))
  object_initdata_t _object_Kold_time;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_OPEN_MAX))
  object_initdata_t _object_Kopen_max;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_OID))
  object_initdata_t _object_Kor_invalid_oid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_OXID))
  object_initdata_t _object_Kor_invalid_oxid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_SET))
  object_initdata_t _object_Kor_invalid_set;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kpagecount;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PAGESIZE))
  object_initdata_t _object_Kpagesize;
#endif
#if ((defined(HAVE_CONFSTR)) && (defined(_CS_PATH))) || (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN))
  object_initdata_t _object_Kpath;
#endif
  object_initdata_t _object_Kpathname;
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PATH_MAX))
  object_initdata_t _object_Kpath_max;
#endif
  object_initdata_t _object_Kperiod;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kpersonal;
#endif
#if defined(PRIO_PGRP)
  object_initdata_t _object_Kpgrp;
#endif
#if defined(HAVE_NFTW)
  object_initdata_t _object_Kphys;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PHYS_PAGES))
  object_initdata_t _object_Kphys_pages;
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))) || (defined(HAVE_SYS_RESOURCE_H))
  object_initdata_t _object_Kpid;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PIPE_BUF))
  object_initdata_t _object_Kpipe_buf;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_CFLAGS))
  object_initdata_t _object_Kposix_v6_ilp32_off32_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_LDFLAGS))
  object_initdata_t _object_Kposix_v6_ilp32_off32_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_LIBS))
  object_initdata_t _object_Kposix_v6_ilp32_off32_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_CFLAGS))
  object_initdata_t _object_Kposix_v6_ilp32_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS))
  object_initdata_t _object_Kposix_v6_ilp32_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_LIBS))
  object_initdata_t _object_Kposix_v6_ilp32_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_CFLAGS))
  object_initdata_t _object_Kposix_v6_lp64_off64_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_LDFLAGS))
  object_initdata_t _object_Kposix_v6_lp64_off64_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_LIBS))
  object_initdata_t _object_Kposix_v6_lp64_off64_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS))
  object_initdata_t _object_Kposix_v6_lpbig_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS))
  object_initdata_t _object_Kposix_v6_lpbig_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_LIBS))
  object_initdata_t _object_Kposix_v6_lpbig_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_WIDTH_RESTRICTED_ENVS))
  object_initdata_t _object_Kposix_v6_width_restricted_envs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_CFLAGS))
  object_initdata_t _object_Kposix_v7_ilp32_off32_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_LDFLAGS))
  object_initdata_t _object_Kposix_v7_ilp32_off32_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_LIBS))
  object_initdata_t _object_Kposix_v7_ilp32_off32_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_CFLAGS))
  object_initdata_t _object_Kposix_v7_ilp32_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS))
  object_initdata_t _object_Kposix_v7_ilp32_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_LIBS))
  object_initdata_t _object_Kposix_v7_ilp32_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_CFLAGS))
  object_initdata_t _object_Kposix_v7_lp64_off64_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_LDFLAGS))
  object_initdata_t _object_Kposix_v7_lp64_off64_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_LIBS))
  object_initdata_t _object_Kposix_v7_lp64_off64_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS))
  object_initdata_t _object_Kposix_v7_lpbig_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS))
  object_initdata_t _object_Kposix_v7_lpbig_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_LIBS))
  object_initdata_t _object_Kposix_v7_lpbig_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_THREADS_CFLAGS))
  object_initdata_t _object_Kposix_v7_threads_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_THREADS_LDFLAGS))
  object_initdata_t _object_Kposix_v7_threads_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_WIDTH_RESTRICTED_ENVS))
  object_initdata_t _object_Kposix_v7_width_restricted_envs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_PPC))
  object_initdata_t _object_Kppc;
#endif
  object_initdata_t _object_Kpreserve;
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PRIORITIZED_IO))
  object_initdata_t _object_Kprioritized_io;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PRIORITY_SCHEDULING))
  object_initdata_t _object_Kpriority_scheduling;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PRIO_IO))
  object_initdata_t _object_Kprio_io;
#endif
#if defined(PRIO_PROCESS)
  object_initdata_t _object_Kprocess;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kr4;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kr8;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RAW_SOCKETS))
  object_initdata_t _object_Kraw_sockets;
#endif
#if defined(HAVE_FCNTL)
  object_initdata_t _object_Krdonly;
#endif
#if defined(HAVE_FCNTL)
  object_initdata_t _object_Krdwr;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_READER_WRITER_LOCKS))
  object_initdata_t _object_Kreader_writer_locks;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_READONLY))
  object_initdata_t _object_Kreadonly;
#endif
#if ((defined(WIN32_NATIVE)) && (defined(REALTIME_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  object_initdata_t _object_Krealtime;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_REALTIME_SIGNALS))
  object_initdata_t _object_Krealtime_signals;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_INCR_XFER_SIZE))
  object_initdata_t _object_Krec_incr_xfer_size;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_MAX_XFER_SIZE))
  object_initdata_t _object_Krec_max_xfer_size;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_MIN_XFER_SIZE))
  object_initdata_t _object_Krec_min_xfer_size;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_XFER_ALIGN))
  object_initdata_t _object_Krec_xfer_align;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_REGEXP))
  object_initdata_t _object_Kregexp;
#endif
  object_initdata_t _object_Krename;
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_REPARSE_POINT))
  object_initdata_t _object_Kreparse_point;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Krevnumber;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RE_DUP_MAX))
  object_initdata_t _object_Kre_dup_max;
#endif
#if defined(S_IRGRP)
  object_initdata_t _object_Krgrp;
#endif
#if defined(S_IROTH)
  object_initdata_t _object_Kroth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ADDRESS_ERROR))
  object_initdata_t _object_Krpc_s_address_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ALREADY_LISTENING))
  object_initdata_t _object_Krpc_s_already_listening;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ALREADY_REGISTERED))
  object_initdata_t _object_Krpc_s_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_BINDING_HAS_NO_AUTH))
  object_initdata_t _object_Krpc_s_binding_has_no_auth;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_BINDING_INCOMPLETE))
  object_initdata_t _object_Krpc_s_binding_incomplete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_CANCELLED))
  object_initdata_t _object_Krpc_s_call_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_FAILED))
  object_initdata_t _object_Krpc_s_call_failed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_FAILED_DNE))
  object_initdata_t _object_Krpc_s_call_failed_dne;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_IN_PROGRESS))
  object_initdata_t _object_Krpc_s_call_in_progress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CANNOT_SUPPORT))
  object_initdata_t _object_Krpc_s_cannot_support;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CANT_CREATE_ENDPOINT))
  object_initdata_t _object_Krpc_s_cant_create_endpoint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_COMM_FAILURE))
  object_initdata_t _object_Krpc_s_comm_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_DUPLICATE_ENDPOINT))
  object_initdata_t _object_Krpc_s_duplicate_endpoint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_ALREADY_EXISTS))
  object_initdata_t _object_Krpc_s_entry_already_exists;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_NOT_FOUND))
  object_initdata_t _object_Krpc_s_entry_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_TYPE_MISMATCH))
  object_initdata_t _object_Krpc_s_entry_type_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_DIV_ZERO))
  object_initdata_t _object_Krpc_s_fp_div_zero;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_OVERFLOW))
  object_initdata_t _object_Krpc_s_fp_overflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_UNDERFLOW))
  object_initdata_t _object_Krpc_s_fp_underflow;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GROUP_MEMBER_NOT_FOUND))
  object_initdata_t _object_Krpc_s_group_member_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GRP_ELT_NOT_ADDED))
  object_initdata_t _object_Krpc_s_grp_elt_not_added;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GRP_ELT_NOT_REMOVED))
  object_initdata_t _object_Krpc_s_grp_elt_not_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INCOMPLETE_NAME))
  object_initdata_t _object_Krpc_s_incomplete_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERFACE_NOT_EXPORTED))
  object_initdata_t _object_Krpc_s_interface_not_exported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERFACE_NOT_FOUND))
  object_initdata_t _object_Krpc_s_interface_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERNAL_ERROR))
  object_initdata_t _object_Krpc_s_internal_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ASYNC_CALL))
  object_initdata_t _object_Krpc_s_invalid_async_call;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ASYNC_HANDLE))
  object_initdata_t _object_Krpc_s_invalid_async_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_AUTH_IDENTITY))
  object_initdata_t _object_Krpc_s_invalid_auth_identity;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_BINDING))
  object_initdata_t _object_Krpc_s_invalid_binding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_BOUND))
  object_initdata_t _object_Krpc_s_invalid_bound;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ENDPOINT_FORMAT))
  object_initdata_t _object_Krpc_s_invalid_endpoint_format;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NAF_ID))
  object_initdata_t _object_Krpc_s_invalid_naf_id;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NAME_SYNTAX))
  object_initdata_t _object_Krpc_s_invalid_name_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NETWORK_OPTIONS))
  object_initdata_t _object_Krpc_s_invalid_network_options;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NET_ADDR))
  object_initdata_t _object_Krpc_s_invalid_net_addr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_OBJECT))
  object_initdata_t _object_Krpc_s_invalid_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_RPC_PROTSEQ))
  object_initdata_t _object_Krpc_s_invalid_rpc_protseq;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_STRING_BINDING))
  object_initdata_t _object_Krpc_s_invalid_string_binding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_STRING_UUID))
  object_initdata_t _object_Krpc_s_invalid_string_uuid;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_TAG))
  object_initdata_t _object_Krpc_s_invalid_tag;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_TIMEOUT))
  object_initdata_t _object_Krpc_s_invalid_timeout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_VERS_OPTION))
  object_initdata_t _object_Krpc_s_invalid_vers_option;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_MAX_CALLS_TOO_SMALL))
  object_initdata_t _object_Krpc_s_max_calls_too_small;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NAME_SERVICE_UNAVAILABLE))
  object_initdata_t _object_Krpc_s_name_service_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOTHING_TO_EXPORT))
  object_initdata_t _object_Krpc_s_nothing_to_export;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_ALL_OBJS_EXPORTED))
  object_initdata_t _object_Krpc_s_not_all_objs_exported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_ALL_OBJS_UNEXPORTED))
  object_initdata_t _object_Krpc_s_not_all_objs_unexported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_CANCELLED))
  object_initdata_t _object_Krpc_s_not_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_LISTENING))
  object_initdata_t _object_Krpc_s_not_listening;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_RPC_ERROR))
  object_initdata_t _object_Krpc_s_not_rpc_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_BINDINGS))
  object_initdata_t _object_Krpc_s_no_bindings;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_CALL_ACTIVE))
  object_initdata_t _object_Krpc_s_no_call_active;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_CONTEXT_AVAILABLE))
  object_initdata_t _object_Krpc_s_no_context_available;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_ENDPOINT_FOUND))
  object_initdata_t _object_Krpc_s_no_endpoint_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_ENTRY_NAME))
  object_initdata_t _object_Krpc_s_no_entry_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_INTERFACES))
  object_initdata_t _object_Krpc_s_no_interfaces;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_MORE_BINDINGS))
  object_initdata_t _object_Krpc_s_no_more_bindings;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_MORE_MEMBERS))
  object_initdata_t _object_Krpc_s_no_more_members;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PRINC_NAME))
  object_initdata_t _object_Krpc_s_no_princ_name;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PROTSEQS))
  object_initdata_t _object_Krpc_s_no_protseqs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PROTSEQS_REGISTERED))
  object_initdata_t _object_Krpc_s_no_protseqs_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_OBJECT_NOT_FOUND))
  object_initdata_t _object_Krpc_s_object_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_OUT_OF_RESOURCES))
  object_initdata_t _object_Krpc_s_out_of_resources;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PRF_ELT_NOT_ADDED))
  object_initdata_t _object_Krpc_s_prf_elt_not_added;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PRF_ELT_NOT_REMOVED))
  object_initdata_t _object_Krpc_s_prf_elt_not_removed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROCNUM_OUT_OF_RANGE))
  object_initdata_t _object_Krpc_s_procnum_out_of_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROFILE_NOT_ADDED))
  object_initdata_t _object_Krpc_s_profile_not_added;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTOCOL_ERROR))
  object_initdata_t _object_Krpc_s_protocol_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTSEQ_NOT_FOUND))
  object_initdata_t _object_Krpc_s_protseq_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTSEQ_NOT_SUPPORTED))
  object_initdata_t _object_Krpc_s_protseq_not_supported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SEC_PKG_ERROR))
  object_initdata_t _object_Krpc_s_sec_pkg_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SEND_INCOMPLETE))
  object_initdata_t _object_Krpc_s_send_incomplete;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SERVER_TOO_BUSY))
  object_initdata_t _object_Krpc_s_server_too_busy;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SERVER_UNAVAILABLE))
  object_initdata_t _object_Krpc_s_server_unavailable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_STRING_TOO_LONG))
  object_initdata_t _object_Krpc_s_string_too_long;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_TYPE_ALREADY_REGISTERED))
  object_initdata_t _object_Krpc_s_type_already_registered;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_LEVEL))
  object_initdata_t _object_Krpc_s_unknown_authn_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_SERVICE))
  object_initdata_t _object_Krpc_s_unknown_authn_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_TYPE))
  object_initdata_t _object_Krpc_s_unknown_authn_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHZ_SERVICE))
  object_initdata_t _object_Krpc_s_unknown_authz_service;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_IF))
  object_initdata_t _object_Krpc_s_unknown_if;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_MGR_TYPE))
  object_initdata_t _object_Krpc_s_unknown_mgr_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_AUTHN_LEVEL))
  object_initdata_t _object_Krpc_s_unsupported_authn_level;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_NAME_SYNTAX))
  object_initdata_t _object_Krpc_s_unsupported_name_syntax;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_TRANS_SYN))
  object_initdata_t _object_Krpc_s_unsupported_trans_syn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_TYPE))
  object_initdata_t _object_Krpc_s_unsupported_type;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UUID_LOCAL_ONLY))
  object_initdata_t _object_Krpc_s_uuid_local_only;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UUID_NO_ADDRESS))
  object_initdata_t _object_Krpc_s_uuid_no_address;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_WRONG_KIND_OF_BINDING))
  object_initdata_t _object_Krpc_s_wrong_kind_of_binding;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ZERO_DIVIDE))
  object_initdata_t _object_Krpc_s_zero_divide;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_BAD_STUB_DATA))
  object_initdata_t _object_Krpc_x_bad_stub_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_BYTE_COUNT_TOO_SMALL))
  object_initdata_t _object_Krpc_x_byte_count_too_small;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_ENUM_VALUE_OUT_OF_RANGE))
  object_initdata_t _object_Krpc_x_enum_value_out_of_range;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_INVALID_ES_ACTION))
  object_initdata_t _object_Krpc_x_invalid_es_action;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_INVALID_PIPE_OBJECT))
  object_initdata_t _object_Krpc_x_invalid_pipe_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_NO_MORE_ENTRIES))
  object_initdata_t _object_Krpc_x_no_more_entries;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_NULL_REF_POINTER))
  object_initdata_t _object_Krpc_x_null_ref_pointer;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_CLOSED))
  object_initdata_t _object_Krpc_x_pipe_closed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_DISCIPLINE_ERROR))
  object_initdata_t _object_Krpc_x_pipe_discipline_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_EMPTY))
  object_initdata_t _object_Krpc_x_pipe_empty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CANNOT_GET_CALL_HANDLE))
  object_initdata_t _object_Krpc_x_ss_cannot_get_call_handle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CHAR_TRANS_OPEN_FAIL))
  object_initdata_t _object_Krpc_x_ss_char_trans_open_fail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CHAR_TRANS_SHORT_FILE))
  object_initdata_t _object_Krpc_x_ss_char_trans_short_file;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CONTEXT_DAMAGED))
  object_initdata_t _object_Krpc_x_ss_context_damaged;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_HANDLES_MISMATCH))
  object_initdata_t _object_Krpc_x_ss_handles_mismatch;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_IN_NULL_CONTEXT))
  object_initdata_t _object_Krpc_x_ss_in_null_context;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_ES_VERSION))
  object_initdata_t _object_Krpc_x_wrong_es_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_PIPE_ORDER))
  object_initdata_t _object_Krpc_x_wrong_pipe_order;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_PIPE_VERSION))
  object_initdata_t _object_Krpc_x_wrong_pipe_version;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_STUB_VERSION))
  object_initdata_t _object_Krpc_x_wrong_stub_version;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_RSS))
  object_initdata_t _object_Krss;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RTSIG_MAX))
  object_initdata_t _object_Krtsig_max;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(RUN_LVL))
  object_initdata_t _object_Krun_lvl;
#endif
#if defined(S_IRUSR)
  object_initdata_t _object_Krusr;
#endif
#if defined(S_IRWXG)
  object_initdata_t _object_Krwxg;
#endif
#if defined(S_IRWXO)
  object_initdata_t _object_Krwxo;
#endif
#if defined(S_IRWXU)
  object_initdata_t _object_Krwxu;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SAVED_IDS))
  object_initdata_t _object_Ksaved_ids;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SCHED_E_SERVICE_NOT_LOCALSYSTEM))
  object_initdata_t _object_Ksched_e_service_not_localsystem;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_ISSUING_CA_UNTRUSTED))
  object_initdata_t _object_Ksec_e_issuing_ca_untrusted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_PKINIT_CLIENT_FAILUR))
  object_initdata_t _object_Ksec_e_pkinit_client_failur;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_REVOCATION_OFFLINE_C))
  object_initdata_t _object_Ksec_e_revocation_offline_c;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_SMARTCARD_CERT_EXPIRED))
  object_initdata_t _object_Ksec_e_smartcard_cert_expired;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_SMARTCARD_CERT_REVOKED))
  object_initdata_t _object_Ksec_e_smartcard_cert_revoked;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_SELF))
  object_initdata_t _object_Kself;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEMAPHORES))
  object_initdata_t _object_Ksemaphores;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEM_NSEMS_MAX))
  object_initdata_t _object_Ksem_nsems_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEM_VALUE_MAX))
  object_initdata_t _object_Ksem_value_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kserver;
#endif
#if defined(S_ISGID)
  object_initdata_t _object_Ksgid;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  object_initdata_t _object_Kshared;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SHARED_MEMORY_OBJECTS))
  object_initdata_t _object_Kshared_memory_objects;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SHELL))
  object_initdata_t _object_Kshell;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kshift;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kshow_command;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_SHX))
  object_initdata_t _object_Kshx;
#endif
#if defined(SIGABRT)
  object_initdata_t _object_Ksigabrt;
#endif
#if defined(SIGALRM)
  object_initdata_t _object_Ksigalrm;
#endif
#if defined(SIGBUS)
  object_initdata_t _object_Ksigbus;
#endif
#if defined(SIGCHLD)
  object_initdata_t _object_Ksigchld;
#endif
#if defined(SIGCONT)
  object_initdata_t _object_Ksigcont;
#endif
#if defined(SIGFPE)
  object_initdata_t _object_Ksigfpe;
#endif
#if defined(SIGHUP)
  object_initdata_t _object_Ksighup;
#endif
#if defined(SIGILL)
  object_initdata_t _object_Ksigill;
#endif
#if defined(SIGINT)
  object_initdata_t _object_Ksigint;
#endif
#if defined(SIGKILL)
  object_initdata_t _object_Ksigkill;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_Ksignaled;
#endif
#if defined(SIGPIPE)
  object_initdata_t _object_Ksigpipe;
#endif
#if defined(SIGPOLL)
  object_initdata_t _object_Ksigpoll;
#endif
#if defined(SIGPROF)
  object_initdata_t _object_Ksigprof;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SIGQUEUE_MAX))
  object_initdata_t _object_Ksigqueue_max;
#endif
#if defined(SIGQUIT)
  object_initdata_t _object_Ksigquit;
#endif
#if defined(SIGSEGV)
  object_initdata_t _object_Ksigsegv;
#endif
#if defined(SIGSTOP)
  object_initdata_t _object_Ksigstop;
#endif
#if defined(SIGSYS)
  object_initdata_t _object_Ksigsys;
#endif
#if defined(SIGTERM)
  object_initdata_t _object_Ksigterm;
#endif
#if defined(SIGTRAP)
  object_initdata_t _object_Ksigtrap;
#endif
#if defined(SIGTSTP)
  object_initdata_t _object_Ksigtstp;
#endif
#if defined(SIGTTIN)
  object_initdata_t _object_Ksigttin;
#endif
#if defined(SIGTTOU)
  object_initdata_t _object_Ksigttou;
#endif
#if defined(SIGURG)
  object_initdata_t _object_Ksigurg;
#endif
#if defined(SIGUSR1)
  object_initdata_t _object_Ksigusr1;
#endif
#if defined(SIGUSR2)
  object_initdata_t _object_Ksigusr2;
#endif
#if defined(SIGVTALRM)
  object_initdata_t _object_Ksigvtalrm;
#endif
#if defined(SIGXCPU)
  object_initdata_t _object_Ksigxcpu;
#endif
#if defined(SIGXFSZ)
  object_initdata_t _object_Ksigxfsz;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_SL))
  object_initdata_t _object_Ksl;
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_SLN))
  object_initdata_t _object_Ksln;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ksmallbusiness;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ksmallbusiness_restricted;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SOCK_MAXBUF))
  object_initdata_t _object_Ksock_maxbuf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_SPARSE_FILE))
  object_initdata_t _object_Ksparse_file;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPAWN))
  object_initdata_t _object_Kspawn;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPIN_LOCKS))
  object_initdata_t _object_Kspin_locks;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPORADIC_SERVER))
  object_initdata_t _object_Ksporadic_server;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SS_REPL_MAX))
  object_initdata_t _object_Kss_repl_max;
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_STACK))
  object_initdata_t _object_Kstack;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  object_initdata_t _object_Kstart;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_Kstopped;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_STREAM_MAX))
  object_initdata_t _object_Kstream_max;
#endif
#if defined(STRUNCATE)
  object_initdata_t _object_Kstruncate;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_APPEND))
  object_initdata_t _object_Kst_append;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_IMMUTABLE))
  object_initdata_t _object_Kst_immutable;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_MANDLOCK))
  object_initdata_t _object_Kst_mandlock;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOATIME))
  object_initdata_t _object_Kst_noatime;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NODEV))
  object_initdata_t _object_Kst_nodev;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NODIRATIME))
  object_initdata_t _object_Kst_nodiratime;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOEXEC))
  object_initdata_t _object_Kst_noexec;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOSUID))
  object_initdata_t _object_Kst_nosuid;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOTRUNC))
  object_initdata_t _object_Kst_notrunc;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_RDONLY))
  object_initdata_t _object_Kst_rdonly;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_SYNCHRONOUS))
  object_initdata_t _object_Kst_synchronous;
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_WRITE))
  object_initdata_t _object_Kst_write;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ksubject;
#endif
#if defined(S_ISUID)
  object_initdata_t _object_Ksuid;
#endif
#if defined(S_ISVTX)
  object_initdata_t _object_Ksvtx;
#endif
  object_initdata_t _object_Ksymlink;
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SYMLINK_MAX))
  object_initdata_t _object_Ksymlink_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SYMLOOP_MAX))
  object_initdata_t _object_Ksymloop_max;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_SYNC))
  object_initdata_t _object_Ksync;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SYNCHRONIZED_IO))
  object_initdata_t _object_Ksynchronized_io;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SYNC_IO))
  object_initdata_t _object_Ksync_io;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_SYSLOG))
  object_initdata_t _object_Ksyslog;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_SYSTEM))
  object_initdata_t _object_Ksystem;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ktemplate;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_TEMPORARY))
  object_initdata_t _object_Ktemporary;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kterminal;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_TEXT))
  object_initdata_t _object_Ktext;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_THREAD))
  object_initdata_t _object_Kthread;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREADS))
  object_initdata_t _object_Kthreads;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ATTR_STACKADDR))
  object_initdata_t _object_Kthread_attr_stackaddr;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ATTR_STACKSIZE))
  object_initdata_t _object_Kthread_attr_stacksize;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_CPUTIME))
  object_initdata_t _object_Kthread_cputime;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_DESTRUCTOR_ITERATIONS))
  object_initdata_t _object_Kthread_destructor_iterations;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_KEYS_MAX))
  object_initdata_t _object_Kthread_keys_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIORITY_SCHEDULING))
  object_initdata_t _object_Kthread_priority_scheduling;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIO_INHERIT))
  object_initdata_t _object_Kthread_prio_inherit;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIO_PROTECT))
  object_initdata_t _object_Kthread_prio_protect;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PROCESS_SHARED))
  object_initdata_t _object_Kthread_process_shared;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ROBUST_PRIO_INHERIT))
  object_initdata_t _object_Kthread_robust_prio_inherit;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ROBUST_PRIO_PROTECT))
  object_initdata_t _object_Kthread_robust_prio_protect;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_SAFE_FUNCTIONS))
  object_initdata_t _object_Kthread_safe_functions;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_SPORADIC_SERVER))
  object_initdata_t _object_Kthread_sporadic_server;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_STACK_MIN))
  object_initdata_t _object_Kthread_stack_min;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_THREADS_MAX))
  object_initdata_t _object_Kthread_threads_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kthumbnail;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMEOUTS))
  object_initdata_t _object_Ktimeouts;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMERS))
  object_initdata_t _object_Ktimers;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMER_MAX))
  object_initdata_t _object_Ktimer_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Ktitle;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE))
  object_initdata_t _object_Ktrace;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_EVENT_FILTER))
  object_initdata_t _object_Ktrace_event_filter;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_EVENT_NAME_MAX))
  object_initdata_t _object_Ktrace_event_name_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_INHERIT))
  object_initdata_t _object_Ktrace_inherit;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_LOG))
  object_initdata_t _object_Ktrace_log;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_NAME_MAX))
  object_initdata_t _object_Ktrace_name_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_SYS_MAX))
  object_initdata_t _object_Ktrace_sys_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_USER_EVENT_MAX))
  object_initdata_t _object_Ktrace_user_event_max;
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_TRUNC))
  object_initdata_t _object_Ktrunc;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TTY_NAME_MAX))
  object_initdata_t _object_Ktty_name_max;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TYPED_MEMORY_OBJECTS))
  object_initdata_t _object_Ktyped_memory_objects;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TZNAME_MAX))
  object_initdata_t _object_Ktzname_max;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kui1;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kui2;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kui4;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kui8;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  object_initdata_t _object_Kuid;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kuint;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_UNKNOWN))
  object_initdata_t _object_Kunknown;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_Kuntraced;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_Kusage;
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(LOG_USER))) || (defined(PRIO_USER))
  object_initdata_t _object_Kuser;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kuser_defined;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(USER_PROCESS))
  object_initdata_t _object_Kuser_process;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_UUCP))
  object_initdata_t _object_Kuucp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_ILP32_OFF32))
  object_initdata_t _object_Kv6_ilp32_off32;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_ILP32_OFFBIG))
  object_initdata_t _object_Kv6_ilp32_offbig;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_LP64_OFF64))
  object_initdata_t _object_Kv6_lp64_off64;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_LPBIG_OFFBIG))
  object_initdata_t _object_Kv6_lpbig_offbig;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_V7_ENV))
  object_initdata_t _object_Kv7_env;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_ILP32_OFF32))
  object_initdata_t _object_Kv7_ilp32_off32;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_ILP32_OFFBIG))
  object_initdata_t _object_Kv7_ilp32_offbig;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_LP64_OFF64))
  object_initdata_t _object_Kv7_lp64_off64;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_LPBIG_OFFBIG))
  object_initdata_t _object_Kv7_lpbig_offbig;
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_VDISABLE))
  object_initdata_t _object_Kvdisable;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_VERSION))
  object_initdata_t _object_Kversion;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WAIT_TIMEOUT))
  object_initdata_t _object_Kwait_timeout;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_WARNING))
  object_initdata_t _object_Kwarning;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WARNING_IPSEC_MM_POLICY_PRUNED))
  object_initdata_t _object_Kwarning_ipsec_mm_policy_pruned;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WARNING_IPSEC_QM_POLICY_PRUNED))
  object_initdata_t _object_Kwarning_ipsec_qm_policy_pruned;
#endif
#if defined(S_IWGRP)
  object_initdata_t _object_Kwgrp;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kwindows;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kwordcount;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kworking_directory;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_Kworkstation;
#endif
#if defined(S_IWOTH)
  object_initdata_t _object_Kwoth;
#endif
#if defined(HAVE_FCNTL)
  object_initdata_t _object_Kwronly;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSABASEERR))
  object_initdata_t _object_Kwsabaseerr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEACCES))
  object_initdata_t _object_Kwsaeacces;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEADDRINUSE))
  object_initdata_t _object_Kwsaeaddrinuse;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEADDRNOTAVAIL))
  object_initdata_t _object_Kwsaeaddrnotavail;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEAFNOSUPPORT))
  object_initdata_t _object_Kwsaeafnosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEALREADY))
  object_initdata_t _object_Kwsaealready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEBADF))
  object_initdata_t _object_Kwsaebadf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECANCELLED))
  object_initdata_t _object_Kwsaecancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNABORTED))
  object_initdata_t _object_Kwsaeconnaborted;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNREFUSED))
  object_initdata_t _object_Kwsaeconnrefused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNRESET))
  object_initdata_t _object_Kwsaeconnreset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDESTADDRREQ))
  object_initdata_t _object_Kwsaedestaddrreq;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDISCON))
  object_initdata_t _object_Kwsaediscon;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDQUOT))
  object_initdata_t _object_Kwsaedquot;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEFAULT))
  object_initdata_t _object_Kwsaefault;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEHOSTDOWN))
  object_initdata_t _object_Kwsaehostdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEHOSTUNREACH))
  object_initdata_t _object_Kwsaehostunreach;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINPROGRESS))
  object_initdata_t _object_Kwsaeinprogress;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINTR))
  object_initdata_t _object_Kwsaeintr;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVAL))
  object_initdata_t _object_Kwsaeinval;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVALIDPROCTABLE))
  object_initdata_t _object_Kwsaeinvalidproctable;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVALIDPROVIDER))
  object_initdata_t _object_Kwsaeinvalidprovider;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEISCONN))
  object_initdata_t _object_Kwsaeisconn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAELOOP))
  object_initdata_t _object_Kwsaeloop;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEMFILE))
  object_initdata_t _object_Kwsaemfile;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEMSGSIZE))
  object_initdata_t _object_Kwsaemsgsize;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENAMETOOLONG))
  object_initdata_t _object_Kwsaenametoolong;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETDOWN))
  object_initdata_t _object_Kwsaenetdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETRESET))
  object_initdata_t _object_Kwsaenetreset;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETUNREACH))
  object_initdata_t _object_Kwsaenetunreach;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOBUFS))
  object_initdata_t _object_Kwsaenobufs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOMORE))
  object_initdata_t _object_Kwsaenomore;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOPROTOOPT))
  object_initdata_t _object_Kwsaenoprotoopt;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTCONN))
  object_initdata_t _object_Kwsaenotconn;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTEMPTY))
  object_initdata_t _object_Kwsaenotempty;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTSOCK))
  object_initdata_t _object_Kwsaenotsock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEOPNOTSUPP))
  object_initdata_t _object_Kwsaeopnotsupp;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPFNOSUPPORT))
  object_initdata_t _object_Kwsaepfnosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROCLIM))
  object_initdata_t _object_Kwsaeproclim;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROTONOSUPPORT))
  object_initdata_t _object_Kwsaeprotonosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROTOTYPE))
  object_initdata_t _object_Kwsaeprototype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROVIDERFAILEDINIT))
  object_initdata_t _object_Kwsaeproviderfailedinit;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEREFUSED))
  object_initdata_t _object_Kwsaerefused;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEREMOTE))
  object_initdata_t _object_Kwsaeremote;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESHUTDOWN))
  object_initdata_t _object_Kwsaeshutdown;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESOCKTNOSUPPORT))
  object_initdata_t _object_Kwsaesocktnosupport;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESTALE))
  object_initdata_t _object_Kwsaestale;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAETIMEDOUT))
  object_initdata_t _object_Kwsaetimedout;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAETOOMANYREFS))
  object_initdata_t _object_Kwsaetoomanyrefs;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEUSERS))
  object_initdata_t _object_Kwsaeusers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEWOULDBLOCK))
  object_initdata_t _object_Kwsaewouldblock;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAHOST_NOT_FOUND))
  object_initdata_t _object_Kwsahost_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANOTINITIALISED))
  object_initdata_t _object_Kwsanotinitialised;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANO_DATA))
  object_initdata_t _object_Kwsano_data;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANO_RECOVERY))
  object_initdata_t _object_Kwsano_recovery;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASERVICE_NOT_FOUND))
  object_initdata_t _object_Kwsaservice_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASYSCALLFAILURE))
  object_initdata_t _object_Kwsasyscallfailure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASYSNOTREADY))
  object_initdata_t _object_Kwsasysnotready;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSATRY_AGAIN))
  object_initdata_t _object_Kwsatry_again;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSATYPE_NOT_FOUND))
  object_initdata_t _object_Kwsatype_not_found;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAVERNOTSUPPORTED))
  object_initdata_t _object_Kwsavernotsupported;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_E_CANCELLED))
  object_initdata_t _object_Kwsa_e_cancelled;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_E_NO_MORE))
  object_initdata_t _object_Kwsa_e_no_more;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ADMISSION_FAILURE))
  object_initdata_t _object_Kwsa_qos_admission_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_BAD_OBJECT))
  object_initdata_t _object_Kwsa_qos_bad_object;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_BAD_STYLE))
  object_initdata_t _object_Kwsa_qos_bad_style;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERCOUNT))
  object_initdata_t _object_Kwsa_qos_efiltercount;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERSTYLE))
  object_initdata_t _object_Kwsa_qos_efilterstyle;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERTYPE))
  object_initdata_t _object_Kwsa_qos_efiltertype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWCOUNT))
  object_initdata_t _object_Kwsa_qos_eflowcount;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWDESC))
  object_initdata_t _object_Kwsa_qos_eflowdesc;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWSPEC))
  object_initdata_t _object_Kwsa_qos_eflowspec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EOBJLENGTH))
  object_initdata_t _object_Kwsa_qos_eobjlength;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPOLICYOBJ))
  object_initdata_t _object_Kwsa_qos_epolicyobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPROVSPECBUF))
  object_initdata_t _object_Kwsa_qos_eprovspecbuf;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPSFILTERSPEC))
  object_initdata_t _object_Kwsa_qos_epsfilterspec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPSFLOWSPEC))
  object_initdata_t _object_Kwsa_qos_epsflowspec;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESDMODEOBJ))
  object_initdata_t _object_Kwsa_qos_esdmodeobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESERVICETYPE))
  object_initdata_t _object_Kwsa_qos_eservicetype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESHAPERATEOBJ))
  object_initdata_t _object_Kwsa_qos_eshaperateobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EUNKNOWNPSOBJ))
  object_initdata_t _object_Kwsa_qos_eunknownpsobj;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_GENERIC_ERROR))
  object_initdata_t _object_Kwsa_qos_generic_error;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_NO_RECEIVERS))
  object_initdata_t _object_Kwsa_qos_no_receivers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_NO_SENDERS))
  object_initdata_t _object_Kwsa_qos_no_senders;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_POLICY_FAILURE))
  object_initdata_t _object_Kwsa_qos_policy_failure;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_RECEIVERS))
  object_initdata_t _object_Kwsa_qos_receivers;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_REQUEST_CONFIRMED))
  object_initdata_t _object_Kwsa_qos_request_confirmed;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_RESERVED_PETYPE))
  object_initdata_t _object_Kwsa_qos_reserved_petype;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_SENDERS))
  object_initdata_t _object_Kwsa_qos_senders;
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_TRAFFIC_CTRL_ERROR))
  object_initdata_t _object_Kwsa_qos_traffic_ctrl_error;
#endif
#if defined(S_IWUSR)
  object_initdata_t _object_Kwusr;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_ILP32_OFF32))
  object_initdata_t _object_Kxbs5_ilp32_off32;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_CFLAGS))
  object_initdata_t _object_Kxbs5_ilp32_off32_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LDFLAGS))
  object_initdata_t _object_Kxbs5_ilp32_off32_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LIBS))
  object_initdata_t _object_Kxbs5_ilp32_off32_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LINTFLAGS))
  object_initdata_t _object_Kxbs5_ilp32_off32_lintflags;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_ILP32_OFFBIG))
  object_initdata_t _object_Kxbs5_ilp32_offbig;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_CFLAGS))
  object_initdata_t _object_Kxbs5_ilp32_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LDFLAGS))
  object_initdata_t _object_Kxbs5_ilp32_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LIBS))
  object_initdata_t _object_Kxbs5_ilp32_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LINTFLAGS))
  object_initdata_t _object_Kxbs5_ilp32_offbig_lintflags;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_LP64_OFF64))
  object_initdata_t _object_Kxbs5_lp64_off64;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_CFLAGS))
  object_initdata_t _object_Kxbs5_lp64_off64_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LDFLAGS))
  object_initdata_t _object_Kxbs5_lp64_off64_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LIBS))
  object_initdata_t _object_Kxbs5_lp64_off64_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LINTFLAGS))
  object_initdata_t _object_Kxbs5_lp64_off64_lintflags;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_LPBIG_OFFBIG))
  object_initdata_t _object_Kxbs5_lpbig_offbig;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_CFLAGS))
  object_initdata_t _object_Kxbs5_lpbig_offbig_cflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LDFLAGS))
  object_initdata_t _object_Kxbs5_lpbig_offbig_ldflags;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LIBS))
  object_initdata_t _object_Kxbs5_lpbig_offbig_libs;
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LINTFLAGS))
  object_initdata_t _object_Kxbs5_lpbig_offbig_lintflags;
#endif
#if defined(S_IXGRP)
  object_initdata_t _object_Kxgrp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_CRYPT))
  object_initdata_t _object_Kxopen_crypt;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_ENH_I18N))
  object_initdata_t _object_Kxopen_enh_i18n;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_LEGACY))
  object_initdata_t _object_Kxopen_legacy;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_REALTIME))
  object_initdata_t _object_Kxopen_realtime;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_REALTIME_THREADS))
  object_initdata_t _object_Kxopen_realtime_threads;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_SHM))
  object_initdata_t _object_Kxopen_shm;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_STREAMS))
  object_initdata_t _object_Kxopen_streams;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_UNIX))
  object_initdata_t _object_Kxopen_unix;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_UUCP))
  object_initdata_t _object_Kxopen_uucp;
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_VERSION))
  object_initdata_t _object_Kxopen_version;
#endif
#if defined(S_IXOTH)
  object_initdata_t _object_Kxoth;
#endif
#if defined(S_IXUSR)
  object_initdata_t _object_Kxusr;
#endif
  object_initdata_t _object_posix__copy_file;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_posix__make_file_info;
#endif
#if (defined(HAVE_FSTAT) && defined(HAVE_STAT)) || (defined(HAVE_NFTW))
  object_initdata_t _object_posix__make_file_stat;
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  object_initdata_t _object_posix__make_group_info;
#endif
  object_initdata_t _object_posix__make_hostent;
#if defined(HAVE_GETRLIMIT)
  object_initdata_t _object_posix__make_rlimit;
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  object_initdata_t _object_posix__make_service;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_posix__make_shortcut_info;
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  object_initdata_t _object_posix__make_stat_vfs;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_posix__make_system_info;
#endif
  object_initdata_t _object_posix__make_uname;
#if defined(HAVE_SYS_RESOURCE_H)
  object_initdata_t _object_posix__make_usage;
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  object_initdata_t _object_posix__make_user_info;
#endif
#if defined(HAVE_UTMPX_H)
  object_initdata_t _object_posix__make_utmpx;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_posix__make_version;
#endif
#if defined(WIN32_NATIVE)
  object_initdata_t _object_posix__mkfilever;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object_posix__mkmemstat;
#endif
  object_initdata_t _object_posix__resolve_host_ipaddr;
#if defined(HAVE_SETRLIMIT)
  object_initdata_t _object_posix__rlimit;
#endif
#if defined(HAVE_UTMPX_H)
  object_initdata_t _object_posix__utmpx;
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!(defined(WIN32_NATIVE))) && (!(defined(HAVE_CHMOD)))
  object_initdata_t _object__22chmod_28_29_22;
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!(defined(HAVE_CHOWN)))
  object_initdata_t _object__22chown_28_29_22;
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!defined(WIN32_NATIVE)) && (!(defined(HAVE_UTIME)))
  object_initdata_t _object__22utime_28_29_22;
#endif
  object_initdata_t _object__28member_200_201_202_29;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object__28member_20_alt_20_control_20_ext_20_shift_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object__28member_20_normal_20_max_20_min_29;
#endif
  object_initdata_t _object__28or_20null_20integer_20_28member_20_copy_20_symlink_20_hardlink_20_hardlink_or_copy_20_rename_29_29;
  object_initdata_t _object___28or_20integer_20_28member_defined_28e2big_29_Ke2big_defined_28eacces_29_Keacces_defined_28eaddrinuse_29_Keaddrinuse_defined_28eaddrnotavail_29_Keaddrnotavail_defined_28eafnosupport_29_Keafnosupport_defined_28eagain_29_Keagain_defined_28ealready_29_Kealready_defined_28ebadf_29_Kebadf_defined_28ebadmsg_29_Kebadmsg_defined_28ebusy_29_Kebusy_defined_28ecanceled_29_Kecanceled_defined_28echild_29_Kechild_defined_28econnaborted_29_Keconnaborted_defined_28econnrefused_29_Keconnrefused_defined_28econnreset_29_Keconnreset_defined_28edeadlk_29_Kedeadlk_defined_28edestaddrreq_29_Kedestaddrreq_defined_28edom_29_Kedom_defined_28edquot_29_Kedquot_defined_28eexist_29_Keexist_defined_28efault_29_Kefault_defined_28efbig_29_Kefbig_defined_28ehostunreach_29_Kehostunreach_defined_28eidrm_29_Keidrm_defined_28eilseq_29_Keilseq_defined_28einprogress_29_Keinprogress_defined_28eintr_29_Keintr_defined_28einval_29_Keinval_defined_28eio_29_Keio_defined_28eisconn_29_Keisconn_defined_28eisdir_29_Keisdir_defined_28eloop_29_Keloop_defined_28emfile_29_Kemfile_defined_28emlink_29_Kemlink_defined_28emsgsize_29_Kemsgsize_defined_28emultihop_29_Kemultihop_defined_28enametoolong_29_Kenametoolong_defined_28enetdown_29_Kenetdown_defined_28enetreset_29_Kenetreset_defined_28enetunreach_29_Kenetunreach_defined_28enfile_29_Kenfile_defined_28enobufs_29_Kenobufs_defined_28enodata_29_Kenodata_defined_28enodev_29_Kenodev_defined_28enoent_29_Kenoent_defined_28enoexec_29_Kenoexec_defined_28enolck_29_Kenolck_defined_28enolink_29_Kenolink_defined_28enomem_29_Kenomem_defined_28enomsg_29_Kenomsg_defined_28enoprotoopt_29_Kenoprotoopt_defined_28enospc_29_Kenospc_defined_28enosr_29_Kenosr_defined_28enostr_29_Kenostr_defined_28enosys_29_Kenosys_defined_28enotconn_29_Kenotconn_defined_28enotdir_29_Kenotdir_defined_28enotempty_29_Kenotempty_defined_28enotsock_29_Kenotsock_defined_28enotsup_29_Kenotsup_defined_28enotty_29_Kenotty_defined_28enxio_29_Kenxio_defined_28eopnotsupp_29_Keopnotsupp_defined_28eoverflow_29_Keo;
#if defined(HAVE_UTMPX_H)
  object_initdata_t _object___28or_20integer_20_28member_defined_28empty_29_Kempty_defined_28run_lvl_29_Krun_lvl_defined_28boot_time_29_Kboot_time_defined_28old_time_29_Kold_time_defined_28new_time_29_Knew_time_defined_28user_process_29_Kuser_process_defined_28init_process_29_Kinit_process_defined_28login_process_29_Klogin_process_defined_28dead_process_29_Kdead_process_defined_28accounting_29_Kaccounting__29_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object___28or_20integer_20_28member_defined_28error_invalid_function_29_Kerror_invalid_function_defined_28error_file_not_found_29_Kerror_file_not_found_defined_28error_path_not_found_29_Kerror_path_not_found_defined_28error_too_many_open_files_29_Kerror_too_many_open_files_defined_28error_access_denied_29_Kerror_access_denied_defined_28error_invalid_handle_29_Kerror_invalid_handle_defined_28error_arena_trashed_29_Kerror_arena_trashed_defined_28error_not_enough_memory_29_Kerror_not_enough_memory_defined_28error_invalid_block_29_Kerror_invalid_block_defined_28error_bad_environment_29_Kerror_bad_environment_defined_28error_bad_format_29_Kerror_bad_format_defined_28error_invalid_access_29_Kerror_invalid_access_defined_28error_invalid_data_29_Kerror_invalid_data_defined_28error_outofmemory_29_Kerror_outofmemory_defined_28error_invalid_drive_29_Kerror_invalid_drive_defined_28error_current_directory_29_Kerror_current_directory_defined_28error_not_same_device_29_Kerror_not_same_device_defined_28error_no_more_files_29_Kerror_no_more_files_defined_28error_write_protect_29_Kerror_write_protect_defined_28error_bad_unit_29_Kerror_bad_unit_defined_28error_not_ready_29_Kerror_not_ready_defined_28error_bad_command_29_Kerror_bad_command_defined_28error_crc_29_Kerror_crc_defined_28error_bad_length_29_Kerror_bad_length_defined_28error_seek_29_Kerror_seek_defined_28error_not_dos_disk_29_Kerror_not_dos_disk_defined_28error_sector_not_found_29_Kerror_sector_not_found_defined_28error_out_of_paper_29_Kerror_out_of_paper_defined_28error_write_fault_29_Kerror_write_fault_defined_28error_read_fault_29_Kerror_read_fault_defined_28error_gen_failure_29_Kerror_gen_failure_defined_28error_sharing_violation_29_Kerror_sharing_violation_defined_28error_lock_violation_29_Kerror_lock_violation_defined_28error_wrong_disk_29_Kerror_wrong_disk_defined_28error_sharing_buffer_exceeded_29_Kerror_sharing_buffer_exceeded_defined_28error_handle_eof_29_Kerror_handle_eof_defined_28error_handle_disk_full_29_Kerror;
#endif
#if defined(HAVE_FCNTL)
  object_initdata_t _object___28or_20integer_20_28member_defined_28fd_cloexec_29_Kcloexec__29_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object___28or_20integer_20_28member_defined_28file_attribute_archive_29_Karchive_defined_28file_attribute_compressed_29_Kcompressed_defined_28file_attribute_device_29_Kdevice_defined_28file_attribute_directory_29_Kdirectory_defined_28file_attribute_encrypted_29_Kencrypted_defined_28file_attribute_hidden_29_Khidden_defined_28file_attribute_normal_29_Knormal_defined_28file_attribute_not_content_indexed_29_Knot_content_indexed_defined_28file_attribute_offline_29_Koffline_defined_28file_attribute_readonly_29_Kreadonly_defined_28file_attribute_reparse_point_29_Kreparse_point_defined_28file_attribute_sparse_file_29_Ksparse_file_defined_28file_attribute_system_29_Ksystem_defined_28file_attribute_temporary_29_Ktemporary__29_29;
#endif
#if defined(HAVE_NFTW)
  object_initdata_t _object___28or_20integer_20_28member_defined_28ftw_f_29_Kf_defined_28ftw_d_29_Kd_defined_28ftw_dp_29_Kdp_defined_28ftw_sl_29_Ksl_defined_28ftw_sln_29_Ksln_defined_28ftw_dnr_29_Kdnr_defined_28ftw_ns_29_Kns__29_29;
#endif
#if defined(HAVE_FCNTL)
  object_initdata_t _object___28or_20integer_20_28member_defined_28f_getfd_29_Kfd_defined_28f_getfl_29_Kfl__29_29;
#endif
#if defined(HAVE_SYSLOG)
  object_initdata_t _object___28or_20integer_20_28member_defined_28log_emerg_29_Kemerg_defined_28log_alert_29_Kalert_defined_28log_crit_29_Kcrit_defined_28log_err_29_Kerr_defined_28log_warning_29_Kwarning_defined_28log_notice_29_Knotice_defined_28log_info_29_Kinfo_defined_28log_debug_29_Kdebug__29_29;
#endif
#if defined(HAVE_FCNTL)
  object_initdata_t _object___28or_20integer_20_28member_defined_28o_rdonly_29_Krdonly_defined_28o_wronly_29_Kwronly_defined_28o_rdwr_29_Krdwr_defined_28o_append_29_Kappend_defined_28o_creat_29_Kcreat_defined_28o_trunc_29_Ktrunc_defined_28o_excl_29_Kexcl_defined_28o_noctty_29_Knoctty_defined_28o_sync_29_Ksync_defined_28o_nonblock_29_Knonblock_defined_28o_binary_29_Kbinary_defined_28o_text_29_Ktext_defined_28o_noinherit_29_Knoinherit_defined_28o_direct_29_Kdirect_defined_28o_largefile_29_Klargefile_defined_28o_directory_29_Kdirectory_defined_28o_nofollow_29_Knofollow__29_29;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  object_initdata_t _object___28or_20integer_20_28member_defined_28processor_architecture_intel_29_Kintel_defined_28processor_architecture_mips_29_Kmips_defined_28processor_architecture_alpha_29_Kalpha_defined_28processor_architecture_ppc_29_Kppc_defined_28processor_architecture_shx_29_Kshx_defined_28processor_architecture_arm_29_Karm_defined_28processor_architecture_ia64_29_Kia64_defined_28processor_architecture_alpha64_29_Kalpha64_defined_28processor_architecture_msil_29_Kmsil_defined_28processor_architecture_amd64_29_Kamd64_defined_28processor_architecture_ia32_on_win64_29_Kia32_on_win64_defined_28processor_architecture_unknown_29_Kunknown__29_29;
#endif
#if defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)
  object_initdata_t _object___28or_20integer_20_28member_defined_28rlimit_cpu_29_Kcpu_defined_28rlimit_fsize_29_Kfsize_defined_28rlimit_data_29_Kdata_defined_28rlimit_stack_29_Kstack_defined_28rlimit_core_29_Kcore_defined_28rlimit_rss_29_Krss_defined_28rlimit_nofile_29_Knofile_defined_28rlimit_as_29_Kas_defined_28rlimit_nproc_29_Knproc_defined_28rlimit_memlock_29_Kmemlock_defined_28rlimit_locks_29_Klocks__29_29;
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  object_initdata_t _object___28or_20integer_20_28member_defined_28rusage_self_29_Kself_defined_28rusage_children_29_Kchildren_defined_28rusage_thread_29_Kthread_defined_28rusage_lwp_29_Klwp__29_29;
#endif
  object_initdata_t _object___28or_20integer_20_28member_defined_28sigabrt_29_Ksigabrt_defined_28sigalrm_29_Ksigalrm_defined_28sigbus_29_Ksigbus_defined_28sigchld_29_Ksigchld_defined_28sigcont_29_Ksigcont_defined_28sigfpe_29_Ksigfpe_defined_28sighup_29_Ksighup_defined_28sigill_29_Ksigill_defined_28sigint_29_Ksigint_defined_28sigkill_29_Ksigkill_defined_28sigpipe_29_Ksigpipe_defined_28sigquit_29_Ksigquit_defined_28sigsegv_29_Ksigsegv_defined_28sigstop_29_Ksigstop_defined_28sigterm_29_Ksigterm_defined_28sigtstp_29_Ksigtstp_defined_28sigttin_29_Ksigttin_defined_28sigttou_29_Ksigttou_defined_28sigusr1_29_Ksigusr1_defined_28sigusr2_29_Ksigusr2_defined_28sigpoll_29_Ksigpoll_defined_28sigprof_29_Ksigprof_defined_28sigsys_29_Ksigsys_defined_28sigtrap_29_Ksigtrap_defined_28sigurg_29_Ksigurg_defined_28sigvtalrm_29_Ksigvtalrm_defined_28sigxcpu_29_Ksigxcpu_defined_28sigxfsz_29_Ksigxfsz__29_29;
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  object_initdata_t _object___28or_20integer_20_28member_defined_28st_rdonly_29_Kst_rdonly_defined_28st_nosuid_29_Kst_nosuid_defined_28st_notrunc_29_Kst_notrunc_defined_28st_nodev_29_Kst_nodev_defined_28st_noexec_29_Kst_noexec_defined_28st_synchronous_29_Kst_synchronous_defined_28st_mandlock_29_Kst_mandlock_defined_28st_write_29_Kst_write_defined_28st_append_29_Kst_append_defined_28st_immutable_29_Kst_immutable_defined_28st_noatime_29_Kst_noatime_defined_28st_nodiratime_29_Kst_nodiratime_defined_28file_named_streams_29_Kfile_named_streams_defined_28file_read_only_volume_29_Kfile_read_only_volume_defined_28file_supports_object_ids_29_Kfile_supports_object_ids_defined_28file_supports_reparse_points_29_Kfile_supports_reparse_points_defined_28file_supports_sparse_files_29_Kfile_supports_sparse_files_defined_28file_volume_quotas_29_Kfile_volume_quotas_defined_28file_supports_encryption_29_Kfile_supports_encryption_defined_28fs_case_is_preserved_29_Kfs_case_is_preserved_defined_28fs_case_sensitive_29_Kfs_case_sensitive_defined_28fs_file_compression_29_Kfs_file_compression_defined_28fs_file_encryption_29_Kfs_file_encryption_defined_28fs_persistent_acls_29_Kfs_persistent_acls_defined_28fs_unicode_stored_on_disk_29_Kfs_unicode_stored_on_disk_defined_28fs_vol_is_compressed_29_Kfs_vol_is_compressed__29_29;
#endif
  object_initdata_t _object___28or_20integer_20_28member_defined_28s_ififo_29_Kfifo_defined_28s_ifsock_29_Kfsock_defined_28s_ifchr_29_Kfchr_defined_28s_ifdir_29_Kfdir_defined_28s_ifblk_29_Kfblk_defined_28s_ifreg_29_Kfreg__29_29;
  object_initdata_t _object___28or_20integer_20_28member_defined_28s_isuid_29_Ksuid_defined_28s_isgid_29_Ksgid_defined_28s_isvtx_29_Ksvtx_defined_28s_irwxu_29_Krwxu_defined_28s_irusr_29_Krusr_defined_28s_iwusr_29_Kwusr_defined_28s_ixusr_29_Kxusr_defined_28s_irwxg_29_Krwxg_defined_28s_irgrp_29_Krgrp_defined_28s_iwgrp_29_Kwgrp_defined_28s_ixgrp_29_Kxgrp_defined_28s_irwxo_29_Krwxo_defined_28s_iroth_29_Kroth_defined_28s_iwoth_29_Kwoth_defined_28s_ixoth_29_Kxoth__29_29;
#if defined(HAVE_CONFSTR)
  object_initdata_t _object___28or_20integer_20_28member_defined_28_cs_path_29_Kpath_defined_28_cs_posix_v6_ilp32_off32_cflags_29_Kposix_v6_ilp32_off32_cflags_defined_28_cs_posix_v6_ilp32_off32_ldflags_29_Kposix_v6_ilp32_off32_ldflags_defined_28_cs_posix_v6_ilp32_off32_libs_29_Kposix_v6_ilp32_off32_libs_defined_28_cs_posix_v6_ilp32_offbig_cflags_29_Kposix_v6_ilp32_offbig_cflags_defined_28_cs_posix_v6_ilp32_offbig_ldflags_29_Kposix_v6_ilp32_offbig_ldflags_defined_28_cs_posix_v6_ilp32_offbig_libs_29_Kposix_v6_ilp32_offbig_libs_defined_28_cs_posix_v6_lp64_off64_cflags_29_Kposix_v6_lp64_off64_cflags_defined_28_cs_posix_v6_lp64_off64_ldflags_29_Kposix_v6_lp64_off64_ldflags_defined_28_cs_posix_v6_lp64_off64_libs_29_Kposix_v6_lp64_off64_libs_defined_28_cs_posix_v6_lpbig_offbig_cflags_29_Kposix_v6_lpbig_offbig_cflags_defined_28_cs_posix_v6_lpbig_offbig_ldflags_29_Kposix_v6_lpbig_offbig_ldflags_defined_28_cs_posix_v6_lpbig_offbig_libs_29_Kposix_v6_lpbig_offbig_libs_defined_28_cs_posix_v6_width_restricted_envs_29_Kposix_v6_width_restricted_envs_defined_28_cs_v7_env_29_Kv7_env_defined_28_cs_posix_v7_ilp32_off32_cflags_29_Kposix_v7_ilp32_off32_cflags_defined_28_cs_posix_v7_ilp32_off32_ldflags_29_Kposix_v7_ilp32_off32_ldflags_defined_28_cs_posix_v7_ilp32_off32_libs_29_Kposix_v7_ilp32_off32_libs_defined_28_cs_posix_v7_ilp32_offbig_cflags_29_Kposix_v7_ilp32_offbig_cflags_defined_28_cs_posix_v7_ilp32_offbig_ldflags_29_Kposix_v7_ilp32_offbig_ldflags_defined_28_cs_posix_v7_ilp32_offbig_libs_29_Kposix_v7_ilp32_offbig_libs_defined_28_cs_posix_v7_lp64_off64_cflags_29_Kposix_v7_lp64_off64_cflags_defined_28_cs_posix_v7_lp64_off64_ldflags_29_Kposix_v7_lp64_off64_ldflags_defined_28_cs_posix_v7_lp64_off64_libs_29_Kposix_v7_lp64_off64_libs_defined_28_cs_posix_v7_lpbig_offbig_cflags_29_Kposix_v7_lpbig_offbig_cflags_defined_28_cs_posix_v7_lpbig_offbig_ldflags_29_Kposix_v7_lpbig_offbig_ldflags_defined_28_cs_posix_v7_lpbig_offbig_libs_29_Kposix_v7_lpbig_offbig_libs_defined_28_cs_posix_v7_threads_cflags_29_Kposix_v7_t;
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  object_initdata_t _object___28or_20integer_20_28member_defined_28_pc_filesizebits_29_Kfilesizebits_defined_28_pc_link_max_29_Klink_max_defined_28_pc_max_canon_29_Kmax_canon_defined_28_pc_max_input_29_Kmax_input_defined_28_pc_name_max_29_Kname_max_defined_28_pc_path_max_29_Kpath_max_defined_28_pc_pipe_buf_29_Kpipe_buf_defined_28_pc_2_symlinks_29_K2_symlinks_defined_28_pc_alloc_size_min_29_Kalloc_size_min_defined_28_pc_rec_incr_xfer_size_29_Krec_incr_xfer_size_defined_28_pc_rec_max_xfer_size_29_Krec_max_xfer_size_defined_28_pc_rec_min_xfer_size_29_Krec_min_xfer_size_defined_28_pc_rec_xfer_align_29_Krec_xfer_align_defined_28_pc_symlink_max_29_Ksymlink_max_defined_28_pc_chown_restricted_29_Kchown_restricted_defined_28_pc_no_trunc_29_Kno_trunc_defined_28_pc_vdisable_29_Kvdisable_defined_28_pc_async_io_29_Kasync_io_defined_28_pc_prio_io_29_Kprio_io_defined_28_pc_sync_io_29_Ksync_io_defined_28_pc_sock_maxbuf_29_Ksock_maxbuf__29_29;
#endif
#if defined(HAVE_SYSCONF)
  object_initdata_t _object___28or_20integer_20_28member_defined_28_sc_aio_listio_max_29_Kaio_listio_max_defined_28_sc_aio_max_29_Kaio_max_defined_28_sc_aio_prio_delta_max_29_Kaio_prio_delta_max_defined_28_sc_arg_max_29_Karg_max_defined_28_sc_atexit_max_29_Katexit_max_defined_28_sc_bc_base_max_29_Kbc_base_max_defined_28_sc_bc_dim_max_29_Kbc_dim_max_defined_28_sc_bc_scale_max_29_Kbc_scale_max_defined_28_sc_bc_string_max_29_Kbc_string_max_defined_28_sc_child_max_29_Kchild_max_defined_28_sc_clk_tck_29_Kclk_tck_defined_28_sc_coll_weights_max_29_Kcoll_weights_max_defined_28_sc_delaytimer_max_29_Kdelaytimer_max_defined_28_sc_expr_nest_max_29_Kexpr_nest_max_defined_28_sc_host_name_max_29_Khost_name_max_defined_28_sc_iov_max_29_Kiov_max_defined_28_sc_line_max_29_Kline_max_defined_28_sc_login_name_max_29_Klogin_name_max_defined_28_sc_ngroups_max_29_Kngroups_max_defined_28_sc_getgr_r_size_max_29_Kgetgr_r_size_max_defined_28_sc_getpw_r_size_max_29_Kgetpw_r_size_max_defined_28_sc_mq_open_max_29_Kmq_open_max_defined_28_sc_mq_prio_max_29_Kmq_prio_max_defined_28_sc_open_max_29_Kopen_max_defined_28_sc_advisory_info_29_Kadvisory_info_defined_28_sc_barriers_29_Kbarriers_defined_28_sc_asynchronous_io_29_Kasynchronous_io_defined_28_sc_clock_selection_29_Kclock_selection_defined_28_sc_cputime_29_Kcputime_defined_28_sc_fsync_29_Kfsync_defined_28_sc_ipv6_29_Kipv6_defined_28_sc_job_control_29_Kjob_control_defined_28_sc_mapped_files_29_Kmapped_files_defined_28_sc_memlock_29_Kmemlock_defined_28_sc_memlock_range_29_Kmemlock_range_defined_28_sc_memory_protection_29_Kmemory_protection_defined_28_sc_message_passing_29_Kmessage_passing_defined_28_sc_monotonic_clock_29_Kmonotonic_clock_defined_28_sc_prioritized_io_29_Kprioritized_io_defined_28_sc_priority_scheduling_29_Kpriority_scheduling_defined_28_sc_raw_sockets_29_Kraw_sockets_defined_28_sc_reader_writer_locks_29_Kreader_writer_locks_defined_28_sc_realtime_signals_29_Krealtime_signals_defined_28_sc_regexp_29_Kregexp_defined_28_sc_saved_ids_29_Ksaved_ids_defined_28;
#endif
#if defined(HAVE_SYSLOG)
  object_initdata_t _object___28or_20null_20integer_20_28member_defined_28log_kern_29_Kkern_defined_28log_user_29_Kuser_defined_28log_mail_29_Kmail_defined_28log_news_29_Knews_defined_28log_uucp_29_Kuucp_defined_28log_daemon_29_Kdaemon_defined_28log_auth_29_Kauth_defined_28log_cron_29_Kcron_defined_28log_lpr_29_Klpr_defined_28log_syslog_29_Ksyslog_defined_28log_authpriv_29_Kauthpriv_defined_28log_ftp_29_Kftp_defined_28log_local0_29_Klocal0_defined_28log_local1_29_Klocal1_defined_28log_local2_29_Klocal2_defined_28log_local3_29_Klocal3_defined_28log_local4_29_Klocal4_defined_28log_local5_29_Klocal5_defined_28log_local6_29_Klocal6_defined_28log_local7_29_Klocal7__29_29;
#endif
  object_initdata_t _object___28or_20null_20integer_20_28member_defined_28prio_process_29_Kprocess_defined_28prio_pgrp_29_Kpgrp_defined_28prio_user_29_Kuser__29_29;
#if defined(WIN32_NATIVE)
  object_initdata_t _object___28or_20null_20integer_20_28member_defined_28realtime_priority_class_29_Krealtime_defined_28high_priority_class_29_Khigh_defined_28above_normal_priority_class_29_Kabove_normal_defined_28normal_priority_class_29_Knormal_defined_28below_normal_priority_class_29_Kbelow_normal_defined_28low_priority_class_29_Klow_defined_28idle_priority_class_29_Kidle__29_29;
#endif
#if !(defined(WIN32_NATIVE))
  object_initdata_t _object___28or_20null_20integer_20_28member_Krealtime_Khigh_Kabove_normal_Knormal_Kbelow_normal_Klow_Kidle__29_29;
#endif
  object_initdata_t _my_stdin;
  object_initdata_t _my_stdout;
  object_initdata_t _my_stderr;
  int _dummy_to_avoid_trailing_comma_in_initializer;
} module__syscalls__object_tab_initdata = {
  { "CL:DECODE-UNIVERSAL-TIME" },
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_CHAR_TERM))
  { ":2-CHAR-TERM" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_C_BIND))
  { ":2-C-BIND" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_C_DEV))
  { ":2-C-DEV" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_FORT_DEV))
  { ":2-FORT-DEV" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_FORT_RUN))
  { ":2-FORT-RUN" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_LOCALEDEF))
  { ":2-LOCALEDEF" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS))
  { ":2-PBS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_ACCOUNTING))
  { ":2-PBS-ACCOUNTING" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_CHECKPOINT))
  { ":2-PBS-CHECKPOINT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_LOCATE))
  { ":2-PBS-LOCATE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_MESSAGE))
  { ":2-PBS-MESSAGE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_PBS_TRACK))
  { ":2-PBS-TRACK" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_SW_DEV))
  { ":2-SW-DEV" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_2_SYMLINKS))
  { ":2-SYMLINKS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_UPE))
  { ":2-UPE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_2_VERSION))
  { ":2-VERSION" },
#endif
#if ((defined(WIN32_NATIVE)) && (defined(ABOVE_NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  { ":ABOVE-NORMAL" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(ACCOUNTING))
  { ":ACCOUNTING" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ADVISORY_INFO))
  { ":ADVISORY-INFO" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_LISTIO_MAX))
  { ":AIO-LISTIO-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_MAX))
  { ":AIO-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AIO_PRIO_DELTA_MAX))
  { ":AIO-PRIO-DELTA-MAX" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_ALERT))
  { ":ALERT" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_ALLOC_SIZE_MIN))
  { ":ALLOC-SIZE-MIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ALPHA))
  { ":ALPHA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ALPHA64))
  { ":ALPHA64" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":ALT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_AMD64))
  { ":AMD64" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_APPEND))
  { ":APPEND" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":APPNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_ARCHIVE))
  { ":ARCHIVE" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":ARGUMENTS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ARG_MAX))
  { ":ARG-MAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_ARM))
  { ":ARM" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_AS))
  { ":AS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ASYNCHRONOUS_IO))
  { ":ASYNCHRONOUS-IO" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_ASYNC_IO))
  { ":ASYNC-IO" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_ATEXIT_MAX))
  { ":ATEXIT-MAX" },
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  { ":ATIME" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_AUTH))
  { ":AUTH" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":AUTHOR" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_AUTHPRIV))
  { ":AUTHPRIV" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_AVPHYS_PAGES))
  { ":AVPHYS-PAGES" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":BACKOFFICE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BARRIERS))
  { ":BARRIERS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_BASE_MAX))
  { ":BC-BASE-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_DIM_MAX))
  { ":BC-DIM-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_SCALE_MAX))
  { ":BC-SCALE-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_BC_STRING_MAX))
  { ":BC-STRING-MAX" },
#endif
#if ((defined(WIN32_NATIVE)) && (defined(BELOW_NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  { ":BELOW-NORMAL" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_BINARY))
  { ":BINARY" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":BLOB" },
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  { ":BLOCK" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":BOOL" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(BOOT_TIME))
  { ":BOOT-TIME" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":BSTR" },
#endif
  { ":BUFFERED" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":BUILT-IN" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":BYREF" },
#endif
  { ":CASE-SENSITIVE" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":CHARCOUNT" },
#endif
#if defined(HAVE_NFTW)
  { ":CHDIR" },
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_CHILDREN))
  { ":CHILDREN" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CHILD_MAX))
  { ":CHILD-MAX" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_CHOWN_RESTRICTED))
  { ":CHOWN-RESTRICTED" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":CLIPBOARD-FORMAT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CLK_TCK))
  { ":CLK-TCK" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CLOCK_SELECTION))
  { ":CLOCK-SELECTION" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(FD_CLOEXEC))
  { ":CLOEXEC" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":CODEPAGE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_COLL_WEIGHTS_MAX))
  { ":COLL-WEIGHTS-MAX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":COMMENTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_COMPRESSED))
  { ":COMPRESSED" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  { ":CONS" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { ":CONTINUED" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":CONTROL" },
#endif
  { ":COPY" },
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_CORE))
  { ":CORE" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_CPU))
  { ":CPU" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_CPUTIME))
  { ":CPUTIME" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_CREAT))
  { ":CREAT" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":CREATE-DTM" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_CRIT))
  { ":CRIT" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_CRON))
  { ":CRON" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":CY" },
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_D))
  { ":D" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_DAEMON))
  { ":DAEMON" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_DATA))
  { ":DATA" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":DATACENTER" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":DATE" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(DEAD_PROCESS))
  { ":DEAD-PROCESS" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_DEBUG))
  { ":DEBUG" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_DELAYTIMER_MAX))
  { ":DELAYTIMER-MAX" },
#endif
#if defined(HAVE_NFTW)
  { ":DEPTH" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":DESCRIPTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_DEVICE))
  { ":DEVICE" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_DIRECT))
  { ":DIRECT" },
#endif
  { ":DIRECTION" },
#if ((defined(HAVE_FCNTL)) && (defined(O_DIRECTORY))) || ((defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_DIRECTORY)))
  { ":DIRECTORY" },
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_DNR))
  { ":DNR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_AUTOZONE_ALREADY_EXISTS))
  { ":DNS_ERROR_AUTOZONE_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_AXFR))
  { ":DNS_ERROR_AXFR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_BAD_PACKET))
  { ":DNS_ERROR_BAD_PACKET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CANNOT_FIND_ROOT_HINTS))
  { ":DNS_ERROR_CANNOT_FIND_ROOT_HINTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CNAME_COLLISION))
  { ":DNS_ERROR_CNAME_COLLISION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_CNAME_LOOP))
  { ":DNS_ERROR_CNAME_LOOP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DATAFILE_OPEN_FAILURE))
  { ":DNS_ERROR_DATAFILE_OPEN_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DATAFILE_PARSING))
  { ":DNS_ERROR_DATAFILE_PARSING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_ALREADY_ENLISTED))
  { ":DNS_ERROR_DP_ALREADY_ENLISTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_ALREADY_EXISTS))
  { ":DNS_ERROR_DP_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_DOES_NOT_EXIST))
  { ":DNS_ERROR_DP_DOES_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_NOT_AVAILABLE))
  { ":DNS_ERROR_DP_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DP_NOT_ENLISTED))
  { ":DNS_ERROR_DP_NOT_ENLISTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DS_UNAVAILABLE))
  { ":DNS_ERROR_DS_UNAVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_DS_ZONE_ALREADY_EXISTS))
  { ":DNS_ERROR_DS_ZONE_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_FILE_WRITEBACK_FAILED))
  { ":DNS_ERROR_FILE_WRITEBACK_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_FORWARDER_ALREADY_EXISTS))
  { ":DNS_ERROR_FORWARDER_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INCONSISTENT_ROOT_HINTS))
  { ":DNS_ERROR_INCONSISTENT_ROOT_HINTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_DATAFILE_NAME))
  { ":DNS_ERROR_INVALID_DATAFILE_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_IP_ADDRESS))
  { ":DNS_ERROR_INVALID_IP_ADDRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_NAME_CHAR))
  { ":DNS_ERROR_INVALID_NAME_CHAR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_PROPERTY))
  { ":DNS_ERROR_INVALID_PROPERTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_TYPE))
  { ":DNS_ERROR_INVALID_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_ZONE_OPERATION))
  { ":DNS_ERROR_INVALID_ZONE_OPERATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_INVALID_ZONE_TYPE))
  { ":DNS_ERROR_INVALID_ZONE_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NAME_DOES_NOT_EXIST))
  { ":DNS_ERROR_NAME_DOES_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NAME_NOT_IN_ZONE))
  { ":DNS_ERROR_NAME_NOT_IN_ZONE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NBSTAT_INIT_FAILED))
  { ":DNS_ERROR_NBSTAT_INIT_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NEED_SECONDARY_ADDRESSES))
  { ":DNS_ERROR_NEED_SECONDARY_ADDRESSES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NEED_WINS_SERVERS))
  { ":DNS_ERROR_NEED_WINS_SERVERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NODE_CREATION_FAILED))
  { ":DNS_ERROR_NODE_CREATION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NODE_IS_CNAME))
  { ":DNS_ERROR_NODE_IS_CNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NON_RFC_NAME))
  { ":DNS_ERROR_NON_RFC_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER))
  { ":DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION))
  { ":DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NOT_UNIQUE))
  { ":DNS_ERROR_NOT_UNIQUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE))
  { ":DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_CREATE_CACHE_DATA))
  { ":DNS_ERROR_NO_CREATE_CACHE_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_DNS_SERVERS))
  { ":DNS_ERROR_NO_DNS_SERVERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_PACKET))
  { ":DNS_ERROR_NO_PACKET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_TCPIP))
  { ":DNS_ERROR_NO_TCPIP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NO_ZONE_INFO))
  { ":DNS_ERROR_NO_ZONE_INFO" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_NUMERIC_NAME))
  { ":DNS_ERROR_NUMERIC_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_PRIMARY_REQUIRES_DATAFILE))
  { ":DNS_ERROR_PRIMARY_REQUIRES_DATAFILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE))
  { ":DNS_ERROR_RCODE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADKEY))
  { ":DNS_ERROR_RCODE_BADKEY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADSIG))
  { ":DNS_ERROR_RCODE_BADSIG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_BADTIME))
  { ":DNS_ERROR_RCODE_BADTIME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_FORMAT_ERROR))
  { ":DNS_ERROR_RCODE_FORMAT_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NAME_ERROR))
  { ":DNS_ERROR_RCODE_NAME_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOTAUTH))
  { ":DNS_ERROR_RCODE_NOTAUTH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOTZONE))
  { ":DNS_ERROR_RCODE_NOTZONE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NOT_IMPLEMENTED))
  { ":DNS_ERROR_RCODE_NOT_IMPLEMENTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_NXRRSET))
  { ":DNS_ERROR_RCODE_NXRRSET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_REFUSED))
  { ":DNS_ERROR_RCODE_REFUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_SERVER_FAILURE))
  { ":DNS_ERROR_RCODE_SERVER_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_YXDOMAIN))
  { ":DNS_ERROR_RCODE_YXDOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RCODE_YXRRSET))
  { ":DNS_ERROR_RCODE_YXRRSET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_ALREADY_EXISTS))
  { ":DNS_ERROR_RECORD_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_DOES_NOT_EXIST))
  { ":DNS_ERROR_RECORD_DOES_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_FORMAT))
  { ":DNS_ERROR_RECORD_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT))
  { ":DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_RECORD_TIMED_OUT))
  { ":DNS_ERROR_RECORD_TIMED_OUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SECONDARY_DATA))
  { ":DNS_ERROR_SECONDARY_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP))
  { ":DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_SOA_DELETE_INVALID))
  { ":DNS_ERROR_SOA_DELETE_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_TRY_AGAIN_LATER))
  { ":DNS_ERROR_TRY_AGAIN_LATER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_UNKNOWN_RECORD_TYPE))
  { ":DNS_ERROR_UNKNOWN_RECORD_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_UNSECURE_PACKET))
  { ":DNS_ERROR_UNSECURE_PACKET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_WINS_INIT_FAILED))
  { ":DNS_ERROR_WINS_INIT_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_ALREADY_EXISTS))
  { ":DNS_ERROR_ZONE_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_CONFIGURATION_ERROR))
  { ":DNS_ERROR_ZONE_CONFIGURATION_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_CREATION_FAILED))
  { ":DNS_ERROR_ZONE_CREATION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_DOES_NOT_EXIST))
  { ":DNS_ERROR_ZONE_DOES_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_HAS_NO_NS_RECORDS))
  { ":DNS_ERROR_ZONE_HAS_NO_NS_RECORDS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_HAS_NO_SOA_RECORD))
  { ":DNS_ERROR_ZONE_HAS_NO_SOA_RECORD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_IS_SHUTDOWN))
  { ":DNS_ERROR_ZONE_IS_SHUTDOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_LOCKED))
  { ":DNS_ERROR_ZONE_LOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_NOT_SECONDARY))
  { ":DNS_ERROR_ZONE_NOT_SECONDARY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_ERROR_ZONE_REQUIRES_MASTER_IP))
  { ":DNS_ERROR_ZONE_REQUIRES_MASTER_IP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_ADDED_LOCAL_WINS))
  { ":DNS_INFO_ADDED_LOCAL_WINS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_AXFR_COMPLETE))
  { ":DNS_INFO_AXFR_COMPLETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_INFO_NO_RECORDS))
  { ":DNS_INFO_NO_RECORDS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_CONTINUE_NEEDED))
  { ":DNS_STATUS_CONTINUE_NEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_DOTTED_NAME))
  { ":DNS_STATUS_DOTTED_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_FQDN))
  { ":DNS_STATUS_FQDN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_STATUS_SINGLE_PART_NAME))
  { ":DNS_STATUS_SINGLE_PART_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_WARNING_DOMAIN_UNDELETED))
  { ":DNS_WARNING_DOMAIN_UNDELETED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(DNS_WARNING_PTR_CREATE_FAILED))
  { ":DNS_WARNING_PTR_CREATE_FAILED" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":DOC-SECURITY" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":DOMAIN-CONTROLLER" },
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_DP))
  { ":DP" },
#endif
#if defined(E2BIG)
  { ":E2BIG" },
#endif
#if defined(EACCES)
  { ":EACCES" },
#endif
#if defined(EADDRINUSE)
  { ":EADDRINUSE" },
#endif
#if defined(EADDRNOTAVAIL)
  { ":EADDRNOTAVAIL" },
#endif
#if defined(EADV)
  { ":EADV" },
#endif
#if defined(EAFNOSUPPORT)
  { ":EAFNOSUPPORT" },
#endif
#if defined(EAGAIN)
  { ":EAGAIN" },
#endif
#if defined(EAIO)
  { ":EAIO" },
#endif
#if defined(EALIGN)
  { ":EALIGN" },
#endif
#if defined(EALREADY)
  { ":EALREADY" },
#endif
#if defined(EAUTH)
  { ":EAUTH" },
#endif
#if defined(EBADE)
  { ":EBADE" },
#endif
#if defined(EBADF)
  { ":EBADF" },
#endif
#if defined(EBADFD)
  { ":EBADFD" },
#endif
#if defined(EBADMSG)
  { ":EBADMSG" },
#endif
#if defined(EBADR)
  { ":EBADR" },
#endif
#if defined(EBADRPC)
  { ":EBADRPC" },
#endif
#if defined(EBADRQC)
  { ":EBADRQC" },
#endif
#if defined(EBADSLT)
  { ":EBADSLT" },
#endif
#if defined(EBFONT)
  { ":EBFONT" },
#endif
#if defined(EBUSY)
  { ":EBUSY" },
#endif
#if defined(ECANCELED)
  { ":ECANCELED" },
#endif
#if defined(ECASECLASH)
  { ":ECASECLASH" },
#endif
#if defined(ECHILD)
  { ":ECHILD" },
#endif
#if defined(ECHRNG)
  { ":ECHRNG" },
#endif
#if defined(ECLONEME)
  { ":ECLONEME" },
#endif
#if defined(ECOMM)
  { ":ECOMM" },
#endif
#if defined(ECONNABORTED)
  { ":ECONNABORTED" },
#endif
#if defined(ECONNREFUSED)
  { ":ECONNREFUSED" },
#endif
#if defined(ECONNRESET)
  { ":ECONNRESET" },
#endif
#if defined(EDEADLK)
  { ":EDEADLK" },
#endif
#if defined(EDEADLOCK)
  { ":EDEADLOCK" },
#endif
#if defined(EDESTADDRREQ)
  { ":EDESTADDRREQ" },
#endif
#if defined(EDIRIOCTL)
  { ":EDIRIOCTL" },
#endif
#if defined(EDIRTY)
  { ":EDIRTY" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":EDITTIME" },
#endif
#if defined(EDOM)
  { ":EDOM" },
#endif
#if defined(EDOOFUS)
  { ":EDOOFUS" },
#endif
#if defined(EDOTDOT)
  { ":EDOTDOT" },
#endif
#if defined(EDQUOT)
  { ":EDQUOT" },
#endif
#if defined(EDUPPKG)
  { ":EDUPPKG" },
#endif
#if defined(EEXIST)
  { ":EEXIST" },
#endif
#if defined(EFAIL)
  { ":EFAIL" },
#endif
#if defined(EFAULT)
  { ":EFAULT" },
#endif
#if defined(EFBIG)
  { ":EFBIG" },
#endif
#if defined(EFTYPE)
  { ":EFTYPE" },
#endif
#if defined(EHOSTDOWN)
  { ":EHOSTDOWN" },
#endif
#if defined(EHOSTUNREACH)
  { ":EHOSTUNREACH" },
#endif
#if defined(EHWPOISON)
  { ":EHWPOISON" },
#endif
#if defined(EIDRM)
  { ":EIDRM" },
#endif
#if defined(EILSEQ)
  { ":EILSEQ" },
#endif
#if defined(EINIT)
  { ":EINIT" },
#endif
#if defined(EINPROG)
  { ":EINPROG" },
#endif
#if defined(EINPROGRESS)
  { ":EINPROGRESS" },
#endif
#if defined(EINTR)
  { ":EINTR" },
#endif
#if defined(EINVAL)
  { ":EINVAL" },
#endif
#if defined(EIO)
  { ":EIO" },
#endif
#if defined(EISCONN)
  { ":EISCONN" },
#endif
#if defined(EISDIR)
  { ":EISDIR" },
#endif
#if defined(EISNAM)
  { ":EISNAM" },
#endif
#if defined(EISNAME)
  { ":EISNAME" },
#endif
#if defined(EJUSTRETURN)
  { ":EJUSTRETURN" },
#endif
#if defined(EKEYEXPIRED)
  { ":EKEYEXPIRED" },
#endif
#if defined(EKEYREJECTED)
  { ":EKEYREJECTED" },
#endif
#if defined(EKEYREVOKED)
  { ":EKEYREVOKED" },
#endif
#if defined(EL2HLT)
  { ":EL2HLT" },
#endif
#if defined(EL2NSYNC)
  { ":EL2NSYNC" },
#endif
#if defined(EL3HLT)
  { ":EL3HLT" },
#endif
#if defined(EL3RST)
  { ":EL3RST" },
#endif
#if defined(ELAST)
  { ":ELAST" },
#endif
#if defined(ELBIN)
  { ":ELBIN" },
#endif
  { ":ELEMENT-TYPE" },
#if defined(ELIBACC)
  { ":ELIBACC" },
#endif
#if defined(ELIBBAD)
  { ":ELIBBAD" },
#endif
#if defined(ELIBEXEC)
  { ":ELIBEXEC" },
#endif
#if defined(ELIBMAX)
  { ":ELIBMAX" },
#endif
#if defined(ELIBSCN)
  { ":ELIBSCN" },
#endif
#if defined(ELNRNG)
  { ":ELNRNG" },
#endif
#if defined(ELOCKUNMAPPED)
  { ":ELOCKUNMAPPED" },
#endif
#if defined(ELOOP)
  { ":ELOOP" },
#endif
#if defined(EMEDIUMTYPE)
  { ":EMEDIUMTYPE" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_EMERG))
  { ":EMERG" },
#endif
#if defined(EMFILE)
  { ":EMFILE" },
#endif
#if defined(EMLINK)
  { ":EMLINK" },
#endif
#if ((defined(HAVE_UTMPX_H)) && (defined(EMPTY))) || (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN))
  { ":EMPTY" },
#endif
#if defined(EMSGSIZE)
  { ":EMSGSIZE" },
#endif
#if defined(EMTIMERS)
  { ":EMTIMERS" },
#endif
#if defined(EMULTIHOP)
  { ":EMULTIHOP" },
#endif
#if defined(ENAMETOOLONG)
  { ":ENAMETOOLONG" },
#endif
#if defined(ENAVAIL)
  { ":ENAVAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_ENCRYPTED))
  { ":ENCRYPTED" },
#endif
#if defined(ENEEDAUTH)
  { ":ENEEDAUTH" },
#endif
#if defined(ENET)
  { ":ENET" },
#endif
#if defined(ENETDOWN)
  { ":ENETDOWN" },
#endif
#if defined(ENETRESET)
  { ":ENETRESET" },
#endif
#if defined(ENETUNREACH)
  { ":ENETUNREACH" },
#endif
#if defined(ENFILE)
  { ":ENFILE" },
#endif
#if defined(ENMFILE)
  { ":ENMFILE" },
#endif
#if defined(ENOANO)
  { ":ENOANO" },
#endif
#if defined(ENOATTR)
  { ":ENOATTR" },
#endif
#if defined(ENOBUFS)
  { ":ENOBUFS" },
#endif
#if defined(ENOCSI)
  { ":ENOCSI" },
#endif
#if defined(ENODATA)
  { ":ENODATA" },
#endif
#if defined(ENODEV)
  { ":ENODEV" },
#endif
#if defined(ENOENT)
  { ":ENOENT" },
#endif
#if defined(ENOEXEC)
  { ":ENOEXEC" },
#endif
#if defined(ENOFILE)
  { ":ENOFILE" },
#endif
#if defined(ENOIOCTL)
  { ":ENOIOCTL" },
#endif
#if defined(ENOKEY)
  { ":ENOKEY" },
#endif
#if defined(ENOLCK)
  { ":ENOLCK" },
#endif
#if defined(ENOLINK)
  { ":ENOLINK" },
#endif
#if defined(ENOMEDIUM)
  { ":ENOMEDIUM" },
#endif
#if defined(ENOMEM)
  { ":ENOMEM" },
#endif
#if defined(ENOMSG)
  { ":ENOMSG" },
#endif
#if defined(ENONET)
  { ":ENONET" },
#endif
#if defined(ENOPKG)
  { ":ENOPKG" },
#endif
#if defined(ENOPROTOOPT)
  { ":ENOPROTOOPT" },
#endif
#if defined(ENOSHARE)
  { ":ENOSHARE" },
#endif
#if defined(ENOSPC)
  { ":ENOSPC" },
#endif
#if defined(ENOSR)
  { ":ENOSR" },
#endif
#if defined(ENOSTR)
  { ":ENOSTR" },
#endif
#if defined(ENOSYM)
  { ":ENOSYM" },
#endif
#if defined(ENOSYS)
  { ":ENOSYS" },
#endif
#if defined(ENOTACTIVE)
  { ":ENOTACTIVE" },
#endif
#if defined(ENOTBLK)
  { ":ENOTBLK" },
#endif
#if defined(ENOTCONN)
  { ":ENOTCONN" },
#endif
#if defined(ENOTDIR)
  { ":ENOTDIR" },
#endif
#if defined(ENOTEMPTY)
  { ":ENOTEMPTY" },
#endif
#if defined(ENOTNAM)
  { ":ENOTNAM" },
#endif
#if defined(ENOTRECOVERABLE)
  { ":ENOTRECOVERABLE" },
#endif
#if defined(ENOTSOCK)
  { ":ENOTSOCK" },
#endif
#if defined(ENOTSUP)
  { ":ENOTSUP" },
#endif
#if defined(ENOTTY)
  { ":ENOTTY" },
#endif
#if defined(ENOTUNIQ)
  { ":ENOTUNIQ" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":ENTERPRISE" },
#endif
#if defined(ENXIO)
  { ":ENXIO" },
#endif
#if defined(EOPNOTSUPP)
  { ":EOPNOTSUPP" },
#endif
#if defined(EOVERFLOW)
  { ":EOVERFLOW" },
#endif
#if defined(EOWNERDEAD)
  { ":EOWNERDEAD" },
#endif
#if defined(EPERM)
  { ":EPERM" },
#endif
#if defined(EPFNOSUPPORT)
  { ":EPFNOSUPPORT" },
#endif
#if defined(EPIPE)
  { ":EPIPE" },
#endif
#if defined(EPROCLIM)
  { ":EPROCLIM" },
#endif
#if defined(EPROCUNAVAIL)
  { ":EPROCUNAVAIL" },
#endif
#if defined(EPROGMISMATCH)
  { ":EPROGMISMATCH" },
#endif
#if defined(EPROGUNAVAIL)
  { ":EPROGUNAVAIL" },
#endif
#if defined(EPROTO)
  { ":EPROTO" },
#endif
#if defined(EPROTONOSUPPORT)
  { ":EPROTONOSUPPORT" },
#endif
#if defined(EPROTOTYPE)
  { ":EPROTOTYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_CANT_CREATE))
  { ":EPT_S_CANT_CREATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_CANT_PERFORM_OP))
  { ":EPT_S_CANT_PERFORM_OP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_INVALID_ENTRY))
  { ":EPT_S_INVALID_ENTRY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(EPT_S_NOT_REGISTERED))
  { ":EPT_S_NOT_REGISTERED" },
#endif
#if defined(ERANGE)
  { ":ERANGE" },
#endif
#if defined(EREMCHG)
  { ":EREMCHG" },
#endif
#if defined(EREMDEV)
  { ":EREMDEV" },
#endif
#if defined(EREMOTE)
  { ":EREMOTE" },
#endif
#if defined(EREMOTEIO)
  { ":EREMOTEIO" },
#endif
#if defined(EREMOTERELEASE)
  { ":EREMOTERELEASE" },
#endif
#if defined(ERESTART)
  { ":ERESTART" },
#endif
#if defined(ERESTARTNOHAND)
  { ":ERESTARTNOHAND" },
#endif
#if defined(ERESTARTNOINTR)
  { ":ERESTARTNOINTR" },
#endif
#if defined(ERESTARTSYS)
  { ":ERESTARTSYS" },
#endif
#if defined(ERFKILL)
  { ":ERFKILL" },
#endif
#if defined(EROFS)
  { ":EROFS" },
#endif
#if defined(ERPCMISMATCH)
  { ":ERPCMISMATCH" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_ERR))
  { ":ERR" },
#endif
#if defined(ERREMOTE)
  { ":ERREMOTE" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DENIED))
  { ":ERROR_ACCESS_DENIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_BY_POLICY))
  { ":ERROR_ACCESS_DISABLED_BY_POLICY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_WEBBLADE))
  { ":ERROR_ACCESS_DISABLED_WEBBLADE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER))
  { ":ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_DISABLED))
  { ":ERROR_ACCOUNT_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_EXPIRED))
  { ":ERROR_ACCOUNT_EXPIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_LOCKED_OUT))
  { ":ERROR_ACCOUNT_LOCKED_OUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACCOUNT_RESTRICTION))
  { ":ERROR_ACCOUNT_RESTRICTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACTIVATION_COUNT_EXCEEDED))
  { ":ERROR_ACTIVATION_COUNT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ACTIVE_CONNECTIONS))
  { ":ERROR_ACTIVE_CONNECTIONS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADAP_HDW_ERR))
  { ":ERROR_ADAP_HDW_ERR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADDRESS_ALREADY_ASSOCIATED))
  { ":ERROR_ADDRESS_ALREADY_ASSOCIATED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ADDRESS_NOT_ASSOCIATED))
  { ":ERROR_ADDRESS_NOT_ASSOCIATED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALIAS_EXISTS))
  { ":ERROR_ALIAS_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALLOTTED_SPACE_EXCEEDED))
  { ":ERROR_ALLOTTED_SPACE_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALL_NODES_NOT_AVAILABLE))
  { ":ERROR_ALL_NODES_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED))
  { ":ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_ASSIGNED))
  { ":ERROR_ALREADY_ASSIGNED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_EXISTS))
  { ":ERROR_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_FIBER))
  { ":ERROR_ALREADY_FIBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_INITIALIZED))
  { ":ERROR_ALREADY_INITIALIZED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_REGISTERED))
  { ":ERROR_ALREADY_REGISTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_RUNNING_LKG))
  { ":ERROR_ALREADY_RUNNING_LKG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_THREAD))
  { ":ERROR_ALREADY_THREAD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ALREADY_WAITING))
  { ":ERROR_ALREADY_WAITING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_APPHELP_BLOCK))
  { ":ERROR_APPHELP_BLOCK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_APP_WRONG_OS))
  { ":ERROR_APP_WRONG_OS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ARENA_TRASHED))
  { ":ERROR_ARENA_TRASHED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ARITHMETIC_OVERFLOW))
  { ":ERROR_ARITHMETIC_OVERFLOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ATOMIC_LOCKS_NOT_SUPPORTED))
  { ":ERROR_ATOMIC_LOCKS_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_AUTODATASEG_EXCEEDS_64k))
  { ":ERROR_AUTODATASEG_EXCEEDS_64K" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BADDB))
  { ":ERROR_BADDB" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BADKEY))
  { ":ERROR_BADKEY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_ARGUMENTS))
  { ":ERROR_BAD_ARGUMENTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_COMMAND))
  { ":ERROR_BAD_COMMAND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_CONFIGURATION))
  { ":ERROR_BAD_CONFIGURATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DESCRIPTOR_FORMAT))
  { ":ERROR_BAD_DESCRIPTOR_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DEVICE))
  { ":ERROR_BAD_DEVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DEV_TYPE))
  { ":ERROR_BAD_DEV_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DRIVER))
  { ":ERROR_BAD_DRIVER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_DRIVER_LEVEL))
  { ":ERROR_BAD_DRIVER_LEVEL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_ENVIRONMENT))
  { ":ERROR_BAD_ENVIRONMENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_EXE_FORMAT))
  { ":ERROR_BAD_EXE_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_FORMAT))
  { ":ERROR_BAD_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_IMPERSONATION_LEVEL))
  { ":ERROR_BAD_IMPERSONATION_LEVEL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_INHERITANCE_ACL))
  { ":ERROR_BAD_INHERITANCE_ACL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_LENGTH))
  { ":ERROR_BAD_LENGTH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_LOGON_SESSION_STATE))
  { ":ERROR_BAD_LOGON_SESSION_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NETPATH))
  { ":ERROR_BAD_NETPATH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NET_NAME))
  { ":ERROR_BAD_NET_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_NET_RESP))
  { ":ERROR_BAD_NET_RESP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PATHNAME))
  { ":ERROR_BAD_PATHNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PIPE))
  { ":ERROR_BAD_PIPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PROFILE))
  { ":ERROR_BAD_PROFILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_PROVIDER))
  { ":ERROR_BAD_PROVIDER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_QUERY_SYNTAX))
  { ":ERROR_BAD_QUERY_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_RECOVERY_POLICY))
  { ":ERROR_BAD_RECOVERY_POLICY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_REM_ADAP))
  { ":ERROR_BAD_REM_ADAP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_THREADID_ADDR))
  { ":ERROR_BAD_THREADID_ADDR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_TOKEN_TYPE))
  { ":ERROR_BAD_TOKEN_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_UNIT))
  { ":ERROR_BAD_UNIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_USERNAME))
  { ":ERROR_BAD_USERNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_USER_PROFILE))
  { ":ERROR_BAD_USER_PROFILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BAD_VALIDATION_CLASS))
  { ":ERROR_BAD_VALIDATION_CLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BEGINNING_OF_MEDIA))
  { ":ERROR_BEGINNING_OF_MEDIA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BOOT_ALREADY_ACCEPTED))
  { ":ERROR_BOOT_ALREADY_ACCEPTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BROKEN_PIPE))
  { ":ERROR_BROKEN_PIPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUFFER_OVERFLOW))
  { ":ERROR_BUFFER_OVERFLOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUSY))
  { ":ERROR_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUSY_DRIVE))
  { ":ERROR_BUSY_DRIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_BUS_RESET))
  { ":ERROR_BUS_RESET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CALLBACK_SUPPLIED_INVALID_DATA))
  { ":ERROR_CALLBACK_SUPPLIED_INVALID_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CALL_NOT_IMPLEMENTED))
  { ":ERROR_CALL_NOT_IMPLEMENTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANCELLED))
  { ":ERROR_CANCELLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANCEL_VIOLATION))
  { ":ERROR_CANCEL_VIOLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_COPY))
  { ":ERROR_CANNOT_COPY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_DETECT_DRIVER_FAILURE))
  { ":ERROR_CANNOT_DETECT_DRIVER_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_DETECT_PROCESS_ABORT))
  { ":ERROR_CANNOT_DETECT_PROCESS_ABORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_FIND_WND_CLASS))
  { ":ERROR_CANNOT_FIND_WND_CLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_IMPERSONATE))
  { ":ERROR_CANNOT_IMPERSONATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_MAKE))
  { ":ERROR_CANNOT_MAKE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANNOT_OPEN_PROFILE))
  { ":ERROR_CANNOT_OPEN_PROFILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTOPEN))
  { ":ERROR_CANTOPEN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTREAD))
  { ":ERROR_CANTREAD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANTWRITE))
  { ":ERROR_CANTWRITE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_ACCESS_DOMAIN_INFO))
  { ":ERROR_CANT_ACCESS_DOMAIN_INFO" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_ACCESS_FILE))
  { ":ERROR_CANT_ACCESS_FILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_DELETE_LAST_ITEM))
  { ":ERROR_CANT_DELETE_LAST_ITEM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_DISABLE_MANDATORY))
  { ":ERROR_CANT_DISABLE_MANDATORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_EVICT_ACTIVE_NODE))
  { ":ERROR_CANT_EVICT_ACTIVE_NODE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_OPEN_ANONYMOUS))
  { ":ERROR_CANT_OPEN_ANONYMOUS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CANT_RESOLVE_FILENAME))
  { ":ERROR_CANT_RESOLVE_FILENAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CAN_NOT_COMPLETE))
  { ":ERROR_CAN_NOT_COMPLETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CAN_NOT_DEL_LOCAL_WINS))
  { ":ERROR_CAN_NOT_DEL_LOCAL_WINS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_MUST_BE_VOLATILE))
  { ":ERROR_CHILD_MUST_BE_VOLATILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_NOT_COMPLETE))
  { ":ERROR_CHILD_NOT_COMPLETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CHILD_WINDOW_MENU))
  { ":ERROR_CHILD_WINDOW_MENU" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CIRCULAR_DEPENDENCY))
  { ":ERROR_CIRCULAR_DEPENDENCY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_ALREADY_EXISTS))
  { ":ERROR_CLASS_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_DOES_NOT_EXIST))
  { ":ERROR_CLASS_DOES_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLASS_HAS_WINDOWS))
  { ":ERROR_CLASS_HAS_WINDOWS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_CARTRIDGE_INSTALLED))
  { ":ERROR_CLEANER_CARTRIDGE_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_CARTRIDGE_SPENT))
  { ":ERROR_CLEANER_CARTRIDGE_SPENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_SLOT_NOT_SET))
  { ":ERROR_CLEANER_SLOT_NOT_SET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLEANER_SLOT_SET))
  { ":ERROR_CLEANER_SLOT_SET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLIPBOARD_NOT_OPEN))
  { ":ERROR_CLIPBOARD_NOT_OPEN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLIPPING_NOT_SUPPORTED))
  { ":ERROR_CLIPPING_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_ALREADY_COMMITTED))
  { ":ERROR_CLUSCFG_ALREADY_COMMITTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_ROLLBACK_FAILED))
  { ":ERROR_CLUSCFG_ROLLBACK_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT))
  { ":ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND))
  { ":ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_CORRUPT))
  { ":ERROR_CLUSTERLOG_CORRUPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE))
  { ":ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE))
  { ":ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE))
  { ":ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME))
  { ":ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_DATABASE_SEQMISMATCH))
  { ":ERROR_CLUSTER_DATABASE_SEQMISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP))
  { ":ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_GUM_NOT_LOCKER))
  { ":ERROR_CLUSTER_GUM_NOT_LOCKER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INCOMPATIBLE_VERSIONS))
  { ":ERROR_CLUSTER_INCOMPATIBLE_VERSIONS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INSTANCE_ID_MISMATCH))
  { ":ERROR_CLUSTER_INSTANCE_ID_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NETWORK))
  { ":ERROR_CLUSTER_INVALID_NETWORK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NETWORK_PROVIDER))
  { ":ERROR_CLUSTER_INVALID_NETWORK_PROVIDER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_NODE))
  { ":ERROR_CLUSTER_INVALID_NODE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_INVALID_REQUEST))
  { ":ERROR_CLUSTER_INVALID_REQUEST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_IPADDR_IN_USE))
  { ":ERROR_CLUSTER_IPADDR_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_ABORTED))
  { ":ERROR_CLUSTER_JOIN_ABORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_IN_PROGRESS))
  { ":ERROR_CLUSTER_JOIN_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS))
  { ":ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_LAST_INTERNAL_NETWORK))
  { ":ERROR_CLUSTER_LAST_INTERNAL_NETWORK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND))
  { ":ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED))
  { ":ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MEMBERSHIP_HALT))
  { ":ERROR_CLUSTER_MEMBERSHIP_HALT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE))
  { ":ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME))
  { ":ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETINTERFACE_EXISTS))
  { ":ERROR_CLUSTER_NETINTERFACE_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETINTERFACE_NOT_FOUND))
  { ":ERROR_CLUSTER_NETINTERFACE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE))
  { ":ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_ALREADY_ONLINE))
  { ":ERROR_CLUSTER_NETWORK_ALREADY_ONLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_EXISTS))
  { ":ERROR_CLUSTER_NETWORK_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS))
  { ":ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_FOUND))
  { ":ERROR_CLUSTER_NETWORK_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP))
  { ":ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NETWORK_NOT_INTERNAL))
  { ":ERROR_CLUSTER_NETWORK_NOT_INTERNAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_DOWN))
  { ":ERROR_CLUSTER_NODE_ALREADY_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT))
  { ":ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_MEMBER))
  { ":ERROR_CLUSTER_NODE_ALREADY_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_ALREADY_UP))
  { ":ERROR_CLUSTER_NODE_ALREADY_UP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_DOWN))
  { ":ERROR_CLUSTER_NODE_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_EXISTS))
  { ":ERROR_CLUSTER_NODE_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_FOUND))
  { ":ERROR_CLUSTER_NODE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_MEMBER))
  { ":ERROR_CLUSTER_NODE_NOT_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_PAUSED))
  { ":ERROR_CLUSTER_NODE_NOT_PAUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_NOT_READY))
  { ":ERROR_CLUSTER_NODE_NOT_READY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_PAUSED))
  { ":ERROR_CLUSTER_NODE_PAUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_SHUTTING_DOWN))
  { ":ERROR_CLUSTER_NODE_SHUTTING_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_UNREACHABLE))
  { ":ERROR_CLUSTER_NODE_UNREACHABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NODE_UP))
  { ":ERROR_CLUSTER_NODE_UP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED))
  { ":ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_NO_SECURITY_CONTEXT))
  { ":ERROR_CLUSTER_NO_SECURITY_CONTEXT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_OLD_VERSION))
  { ":ERROR_CLUSTER_OLD_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST))
  { ":ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_PARAMETER_MISMATCH))
  { ":ERROR_CLUSTER_PARAMETER_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH))
  { ":ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_QUORUMLOG_NOT_FOUND))
  { ":ERROR_CLUSTER_QUORUMLOG_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESNAME_NOT_FOUND))
  { ":ERROR_CLUSTER_RESNAME_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND))
  { ":ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED))
  { ":ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_SHUTTING_DOWN))
  { ":ERROR_CLUSTER_SHUTTING_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED))
  { ":ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CLUSTER_WRONG_OS_VERSION))
  { ":ERROR_CLUSTER_WRONG_OS_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COLORSPACE_MISMATCH))
  { ":ERROR_COLORSPACE_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COMMITMENT_LIMIT))
  { ":ERROR_COMMITMENT_LIMIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTED_OTHER_PASSWORD))
  { ":ERROR_CONNECTED_OTHER_PASSWORD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT))
  { ":ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_ABORTED))
  { ":ERROR_CONNECTION_ABORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_ACTIVE))
  { ":ERROR_CONNECTION_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_COUNT_LIMIT))
  { ":ERROR_CONNECTION_COUNT_LIMIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_INVALID))
  { ":ERROR_CONNECTION_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_REFUSED))
  { ":ERROR_CONNECTION_REFUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONNECTION_UNAVAIL))
  { ":ERROR_CONNECTION_UNAVAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTEXT_EXPIRED))
  { ":ERROR_CONTEXT_EXPIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTINUE))
  { ":ERROR_CONTINUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTROLLING_IEPORT))
  { ":ERROR_CONTROLLING_IEPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CONTROL_ID_NOT_FOUND))
  { ":ERROR_CONTROL_ID_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CORE_RESOURCE))
  { ":ERROR_CORE_RESOURCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_COUNTER_TIMEOUT))
  { ":ERROR_COUNTER_TIMEOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CRC))
  { ":ERROR_CRC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CREATE_FAILED))
  { ":ERROR_CREATE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CSCSHARE_OFFLINE))
  { ":ERROR_CSCSHARE_OFFLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_BAD_VIDEO_MODE))
  { ":ERROR_CTX_BAD_VIDEO_MODE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY))
  { ":ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_LICENSE_IN_USE))
  { ":ERROR_CTX_CLIENT_LICENSE_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_LICENSE_NOT_SET))
  { ":ERROR_CTX_CLIENT_LICENSE_NOT_SET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLIENT_QUERY_TIMEOUT))
  { ":ERROR_CTX_CLIENT_QUERY_TIMEOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CLOSE_PENDING))
  { ":ERROR_CTX_CLOSE_PENDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CONSOLE_CONNECT))
  { ":ERROR_CTX_CONSOLE_CONNECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_CONSOLE_DISCONNECT))
  { ":ERROR_CTX_CONSOLE_DISCONNECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_GRAPHICS_INVALID))
  { ":ERROR_CTX_GRAPHICS_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_MODEMNAME))
  { ":ERROR_CTX_INVALID_MODEMNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_PD))
  { ":ERROR_CTX_INVALID_PD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_INVALID_WD))
  { ":ERROR_CTX_INVALID_WD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_CLIENT_INVALID))
  { ":ERROR_CTX_LICENSE_CLIENT_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_EXPIRED))
  { ":ERROR_CTX_LICENSE_EXPIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LICENSE_NOT_AVAILABLE))
  { ":ERROR_CTX_LICENSE_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_LOGON_DISABLED))
  { ":ERROR_CTX_LOGON_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_INF_NOT_FOUND))
  { ":ERROR_CTX_MODEM_INF_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_BUSY))
  { ":ERROR_CTX_MODEM_RESPONSE_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_ERROR))
  { ":ERROR_CTX_MODEM_RESPONSE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_NO_CARRIER))
  { ":ERROR_CTX_MODEM_RESPONSE_NO_CARRIER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE))
  { ":ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_TIMEOUT))
  { ":ERROR_CTX_MODEM_RESPONSE_TIMEOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_MODEM_RESPONSE_VOICE))
  { ":ERROR_CTX_MODEM_RESPONSE_VOICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_NOT_CONSOLE))
  { ":ERROR_CTX_NOT_CONSOLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_NO_OUTBUF))
  { ":ERROR_CTX_NO_OUTBUF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_PD_NOT_FOUND))
  { ":ERROR_CTX_PD_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SERVICE_NAME_COLLISION))
  { ":ERROR_CTX_SERVICE_NAME_COLLISION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_DENIED))
  { ":ERROR_CTX_SHADOW_DENIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_DISABLED))
  { ":ERROR_CTX_SHADOW_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE))
  { ":ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_INVALID))
  { ":ERROR_CTX_SHADOW_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_SHADOW_NOT_RUNNING))
  { ":ERROR_CTX_SHADOW_NOT_RUNNING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_TD_ERROR))
  { ":ERROR_CTX_TD_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WD_NOT_FOUND))
  { ":ERROR_CTX_WD_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_ACCESS_DENIED))
  { ":ERROR_CTX_WINSTATION_ACCESS_DENIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_ALREADY_EXISTS))
  { ":ERROR_CTX_WINSTATION_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_BUSY))
  { ":ERROR_CTX_WINSTATION_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_NAME_INVALID))
  { ":ERROR_CTX_WINSTATION_NAME_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CTX_WINSTATION_NOT_FOUND))
  { ":ERROR_CTX_WINSTATION_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CURRENT_DIRECTORY))
  { ":ERROR_CURRENT_DIRECTORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_CURRENT_DOMAIN_NOT_ALLOWED))
  { ":ERROR_CURRENT_DOMAIN_NOT_ALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_BACKUP_CORRUPT))
  { ":ERROR_DATABASE_BACKUP_CORRUPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_DOES_NOT_EXIST))
  { ":ERROR_DATABASE_DOES_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_FAILURE))
  { ":ERROR_DATABASE_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATABASE_FULL))
  { ":ERROR_DATABASE_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DATATYPE_MISMATCH))
  { ":ERROR_DATATYPE_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DC_NOT_FOUND))
  { ":ERROR_DC_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DDE_FAIL))
  { ":ERROR_DDE_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEBUGGER_INACTIVE))
  { ":ERROR_DEBUGGER_INACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DECRYPTION_FAILED))
  { ":ERROR_DECRYPTION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DELETE_PENDING))
  { ":ERROR_DELETE_PENDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DELETING_ICM_XFORM))
  { ":ERROR_DELETING_ICM_XFORM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_ALREADY_EXISTS))
  { ":ERROR_DEPENDENCY_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_NOT_ALLOWED))
  { ":ERROR_DEPENDENCY_NOT_ALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENCY_NOT_FOUND))
  { ":ERROR_DEPENDENCY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENT_RESOURCE_EXISTS))
  { ":ERROR_DEPENDENT_RESOURCE_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEPENDENT_SERVICES_RUNNING))
  { ":ERROR_DEPENDENT_SERVICES_RUNNING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DESTINATION_ELEMENT_FULL))
  { ":ERROR_DESTINATION_ELEMENT_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DESTROY_OBJECT_OF_OTHER_THREAD))
  { ":ERROR_DESTROY_OBJECT_OF_OTHER_THREAD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_ALREADY_REMEMBERED))
  { ":ERROR_DEVICE_ALREADY_REMEMBERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_DOOR_OPEN))
  { ":ERROR_DEVICE_DOOR_OPEN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_IN_USE))
  { ":ERROR_DEVICE_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_AVAILABLE))
  { ":ERROR_DEVICE_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_CONNECTED))
  { ":ERROR_DEVICE_NOT_CONNECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_NOT_PARTITIONED))
  { ":ERROR_DEVICE_NOT_PARTITIONED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REINITIALIZATION_NEEDED))
  { ":ERROR_DEVICE_REINITIALIZATION_NEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REMOVED))
  { ":ERROR_DEVICE_REMOVED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEVICE_REQUIRES_CLEANING))
  { ":ERROR_DEVICE_REQUIRES_CLEANING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DEV_NOT_EXIST))
  { ":ERROR_DEV_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DHCP_ADDRESS_CONFLICT))
  { ":ERROR_DHCP_ADDRESS_CONFLICT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIFFERENT_SERVICE_ACCOUNT))
  { ":ERROR_DIFFERENT_SERVICE_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIRECTORY))
  { ":ERROR_DIRECTORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIRECT_ACCESS_HANDLE))
  { ":ERROR_DIRECT_ACCESS_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_EFS_DISALLOWED))
  { ":ERROR_DIR_EFS_DISALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_NOT_EMPTY))
  { ":ERROR_DIR_NOT_EMPTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DIR_NOT_ROOT))
  { ":ERROR_DIR_NOT_ROOT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISCARDED))
  { ":ERROR_DISCARDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_CHANGE))
  { ":ERROR_DISK_CHANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_CORRUPT))
  { ":ERROR_DISK_CORRUPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_FULL))
  { ":ERROR_DISK_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_OPERATION_FAILED))
  { ":ERROR_DISK_OPERATION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_RECALIBRATE_FAILED))
  { ":ERROR_DISK_RECALIBRATE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_RESET_FAILED))
  { ":ERROR_DISK_RESET_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DISK_TOO_FRAGMENTED))
  { ":ERROR_DISK_TOO_FRAGMENTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DLL_INIT_FAILED))
  { ":ERROR_DLL_INIT_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DLL_NOT_FOUND))
  { ":ERROR_DLL_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_CONTROLLER_EXISTS))
  { ":ERROR_DOMAIN_CONTROLLER_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_CONTROLLER_NOT_FOUND))
  { ":ERROR_DOMAIN_CONTROLLER_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_EXISTS))
  { ":ERROR_DOMAIN_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_LIMIT_EXCEEDED))
  { ":ERROR_DOMAIN_LIMIT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOMAIN_TRUST_INCONSISTENT))
  { ":ERROR_DOMAIN_TRUST_INCONSISTENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DOWNGRADE_DETECTED))
  { ":ERROR_DOWNGRADE_DETECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVER_BLOCKED))
  { ":ERROR_DRIVER_BLOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVE_LOCKED))
  { ":ERROR_DRIVE_LOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DRIVE_MEDIA_MISMATCH))
  { ":ERROR_DRIVE_MEDIA_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ADD_REPLICA_INHIBITED))
  { ":ERROR_DS_ADD_REPLICA_INHIBITED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ADMIN_LIMIT_EXCEEDED))
  { ":ERROR_DS_ADMIN_LIMIT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AFFECTS_MULTIPLE_DSAS))
  { ":ERROR_DS_AFFECTS_MULTIPLE_DSAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER))
  { ":ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIASED_OBJ_MISSING))
  { ":ERROR_DS_ALIASED_OBJ_MISSING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_DEREF_PROBLEM))
  { ":ERROR_DS_ALIAS_DEREF_PROBLEM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_POINTS_TO_ALIAS))
  { ":ERROR_DS_ALIAS_POINTS_TO_ALIAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ALIAS_PROBLEM))
  { ":ERROR_DS_ALIAS_PROBLEM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS))
  { ":ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_OWNED_BY_SAM))
  { ":ERROR_DS_ATTRIBUTE_OWNED_BY_SAM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED))
  { ":ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_ALREADY_EXISTS))
  { ":ERROR_DS_ATT_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_IS_NOT_ON_OBJ))
  { ":ERROR_DS_ATT_IS_NOT_ON_OBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_NOT_DEF_FOR_CLASS))
  { ":ERROR_DS_ATT_NOT_DEF_FOR_CLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_NOT_DEF_IN_SCHEMA))
  { ":ERROR_DS_ATT_NOT_DEF_IN_SCHEMA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_SCHEMA_REQ_ID))
  { ":ERROR_DS_ATT_SCHEMA_REQ_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_SCHEMA_REQ_SYNTAX))
  { ":ERROR_DS_ATT_SCHEMA_REQ_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ATT_VAL_ALREADY_EXISTS))
  { ":ERROR_DS_ATT_VAL_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTHORIZATION_FAILED))
  { ":ERROR_DS_AUTHORIZATION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTH_METHOD_NOT_SUPPORTED))
  { ":ERROR_DS_AUTH_METHOD_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUTH_UNKNOWN))
  { ":ERROR_DS_AUTH_UNKNOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_AUX_CLS_TEST_FAIL))
  { ":ERROR_DS_AUX_CLS_TEST_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BACKLINK_WITHOUT_LINK))
  { ":ERROR_DS_BACKLINK_WITHOUT_LINK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_ATT_SCHEMA_SYNTAX))
  { ":ERROR_DS_BAD_ATT_SCHEMA_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_HIERARCHY_FILE))
  { ":ERROR_DS_BAD_HIERARCHY_FILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_INSTANCE_TYPE))
  { ":ERROR_DS_BAD_INSTANCE_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_NAME_SYNTAX))
  { ":ERROR_DS_BAD_NAME_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BAD_RDN_ATT_ID_SYNTAX))
  { ":ERROR_DS_BAD_RDN_ATT_ID_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED))
  { ":ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_BUSY))
  { ":ERROR_DS_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD))
  { ":ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_ATT_VALUES))
  { ":ERROR_DS_CANT_ADD_ATT_VALUES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_SYSTEM_ONLY))
  { ":ERROR_DS_CANT_ADD_SYSTEM_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ADD_TO_GC))
  { ":ERROR_DS_CANT_ADD_TO_GC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CACHE_ATT))
  { ":ERROR_DS_CANT_CACHE_ATT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CACHE_CLASS))
  { ":ERROR_DS_CANT_CACHE_CLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC))
  { ":ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_CREATE_UNDER_SCHEMA))
  { ":ERROR_DS_CANT_CREATE_UNDER_SCHEMA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DELETE))
  { ":ERROR_DS_CANT_DELETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DELETE_DSA_OBJ))
  { ":ERROR_DS_CANT_DELETE_DSA_OBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEL_MASTER_CROSSREF))
  { ":ERROR_DS_CANT_DEL_MASTER_CROSSREF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC))
  { ":ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DEREF_ALIAS))
  { ":ERROR_DS_CANT_DEREF_ALIAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN))
  { ":ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF))
  { ":ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN))
  { ":ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_DSA_OBJ))
  { ":ERROR_DS_CANT_FIND_DSA_OBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_EXPECTED_NC))
  { ":ERROR_DS_CANT_FIND_EXPECTED_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_FIND_NC_IN_CACHE))
  { ":ERROR_DS_CANT_FIND_NC_IN_CACHE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MIX_MASTER_AND_REPS))
  { ":ERROR_DS_CANT_MIX_MASTER_AND_REPS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_OBJ_CLASS))
  { ":ERROR_DS_CANT_MOD_OBJ_CLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_PRIMARYGROUPID))
  { ":ERROR_DS_CANT_MOD_PRIMARYGROUPID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOD_SYSTEM_ONLY))
  { ":ERROR_DS_CANT_MOD_SYSTEM_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_ACCOUNT_GROUP))
  { ":ERROR_DS_CANT_MOVE_ACCOUNT_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_APP_BASIC_GROUP))
  { ":ERROR_DS_CANT_MOVE_APP_BASIC_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_APP_QUERY_GROUP))
  { ":ERROR_DS_CANT_MOVE_APP_QUERY_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_DELETED_OBJECT))
  { ":ERROR_DS_CANT_MOVE_DELETED_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_MOVE_RESOURCE_GROUP))
  { ":ERROR_DS_CANT_MOVE_RESOURCE_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ON_NON_LEAF))
  { ":ERROR_DS_CANT_ON_NON_LEAF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_ON_RDN))
  { ":ERROR_DS_CANT_ON_RDN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REMOVE_ATT_CACHE))
  { ":ERROR_DS_CANT_REMOVE_ATT_CACHE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REMOVE_CLASS_CACHE))
  { ":ERROR_DS_CANT_REMOVE_CLASS_CACHE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REM_MISSING_ATT))
  { ":ERROR_DS_CANT_REM_MISSING_ATT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REM_MISSING_ATT_VAL))
  { ":ERROR_DS_CANT_REM_MISSING_ATT_VAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_REPLACE_HIDDEN_REC))
  { ":ERROR_DS_CANT_REPLACE_HIDDEN_REC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_ATTS))
  { ":ERROR_DS_CANT_RETRIEVE_ATTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_CHILD))
  { ":ERROR_DS_CANT_RETRIEVE_CHILD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_DN))
  { ":ERROR_DS_CANT_RETRIEVE_DN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_INSTANCE))
  { ":ERROR_DS_CANT_RETRIEVE_INSTANCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_RETRIEVE_SD))
  { ":ERROR_DS_CANT_RETRIEVE_SD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_START))
  { ":ERROR_DS_CANT_START" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ))
  { ":ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS))
  { ":ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CHILDREN_EXIST))
  { ":ERROR_DS_CHILDREN_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLASS_MUST_BE_CONCRETE))
  { ":ERROR_DS_CLASS_MUST_BE_CONCRETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLASS_NOT_DSA))
  { ":ERROR_DS_CLASS_NOT_DSA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CLIENT_LOOP))
  { ":ERROR_DS_CLIENT_LOOP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CODE_INCONSISTENCY))
  { ":ERROR_DS_CODE_INCONSISTENCY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COMPARE_FALSE))
  { ":ERROR_DS_COMPARE_FALSE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COMPARE_TRUE))
  { ":ERROR_DS_COMPARE_TRUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONFIDENTIALITY_REQUIRED))
  { ":ERROR_DS_CONFIDENTIALITY_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONFIG_PARAM_MISSING))
  { ":ERROR_DS_CONFIG_PARAM_MISSING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONSTRAINT_VIOLATION))
  { ":ERROR_DS_CONSTRAINT_VIOLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONSTRUCTED_ATT_MOD))
  { ":ERROR_DS_CONSTRUCTED_ATT_MOD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CONTROL_NOT_FOUND))
  { ":ERROR_DS_CONTROL_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_CONTACT_FSMO))
  { ":ERROR_DS_COULDNT_CONTACT_FSMO" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE))
  { ":ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE))
  { ":ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COULDNT_UPDATE_SPNS))
  { ":ERROR_DS_COULDNT_UPDATE_SPNS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_COUNTING_AB_INDICES_FAILED))
  { ":ERROR_DS_COUNTING_AB_INDICES_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD))
  { ":ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_DOM_MOVE_ERROR))
  { ":ERROR_DS_CROSS_DOM_MOVE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_NC_DN_RENAME))
  { ":ERROR_DS_CROSS_NC_DN_RENAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_REF_BUSY))
  { ":ERROR_DS_CROSS_REF_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CROSS_REF_EXISTS))
  { ":ERROR_DS_CROSS_REF_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE))
  { ":ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2))
  { ":ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DATABASE_ERROR))
  { ":ERROR_DS_DATABASE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DECODING_ERROR))
  { ":ERROR_DS_DECODING_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED))
  { ":ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST))
  { ":ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DIFFERENT_REPL_EPOCHS))
  { ":ERROR_DS_DIFFERENT_REPL_EPOCHS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_RENAME_IN_PROGRESS))
  { ":ERROR_DS_DOMAIN_RENAME_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_VERSION_TOO_HIGH))
  { ":ERROR_DS_DOMAIN_VERSION_TOO_HIGH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DOMAIN_VERSION_TOO_LOW))
  { ":ERROR_DS_DOMAIN_VERSION_TOO_LOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_ABANDON_SYNC))
  { ":ERROR_DS_DRA_ABANDON_SYNC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_ACCESS_DENIED))
  { ":ERROR_DS_DRA_ACCESS_DENIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_DN))
  { ":ERROR_DS_DRA_BAD_DN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_INSTANCE_TYPE))
  { ":ERROR_DS_DRA_BAD_INSTANCE_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BAD_NC))
  { ":ERROR_DS_DRA_BAD_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_BUSY))
  { ":ERROR_DS_DRA_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_CONNECTION_FAILED))
  { ":ERROR_DS_DRA_CONNECTION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_DB_ERROR))
  { ":ERROR_DS_DRA_DB_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_DN_EXISTS))
  { ":ERROR_DS_DRA_DN_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT))
  { ":ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_EXTN_CONNECTION_FAILED))
  { ":ERROR_DS_DRA_EXTN_CONNECTION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_GENERIC))
  { ":ERROR_DS_DRA_GENERIC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET))
  { ":ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INCONSISTENT_DIT))
  { ":ERROR_DS_DRA_INCONSISTENT_DIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INTERNAL_ERROR))
  { ":ERROR_DS_DRA_INTERNAL_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_INVALID_PARAMETER))
  { ":ERROR_DS_DRA_INVALID_PARAMETER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_MAIL_PROBLEM))
  { ":ERROR_DS_DRA_MAIL_PROBLEM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_MISSING_PARENT))
  { ":ERROR_DS_DRA_MISSING_PARENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NAME_COLLISION))
  { ":ERROR_DS_DRA_NAME_COLLISION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NOT_SUPPORTED))
  { ":ERROR_DS_DRA_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_NO_REPLICA))
  { ":ERROR_DS_DRA_NO_REPLICA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OBJ_IS_REP_SOURCE))
  { ":ERROR_DS_DRA_OBJ_IS_REP_SOURCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OBJ_NC_MISMATCH))
  { ":ERROR_DS_DRA_OBJ_NC_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_OUT_OF_MEM))
  { ":ERROR_DS_DRA_OUT_OF_MEM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_PREEMPTED))
  { ":ERROR_DS_DRA_PREEMPTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REF_ALREADY_EXISTS))
  { ":ERROR_DS_DRA_REF_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REF_NOT_FOUND))
  { ":ERROR_DS_DRA_REF_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_REPL_PENDING))
  { ":ERROR_DS_DRA_REPL_PENDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_RPC_CANCELLED))
  { ":ERROR_DS_DRA_RPC_CANCELLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_CONFLICT))
  { ":ERROR_DS_DRA_SCHEMA_CONFLICT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_INFO_SHIP))
  { ":ERROR_DS_DRA_SCHEMA_INFO_SHIP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SCHEMA_MISMATCH))
  { ":ERROR_DS_DRA_SCHEMA_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SHUTDOWN))
  { ":ERROR_DS_DRA_SHUTDOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SINK_DISABLED))
  { ":ERROR_DS_DRA_SINK_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_DISABLED))
  { ":ERROR_DS_DRA_SOURCE_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA))
  { ":ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRA_SOURCE_REINSTALLED))
  { ":ERROR_DS_DRA_SOURCE_REINSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DRS_EXTENSIONS_CHANGED))
  { ":ERROR_DS_DRS_EXTENSIONS_CHANGED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DSA_MUST_BE_INT_MASTER))
  { ":ERROR_DS_DSA_MUST_BE_INT_MASTER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DST_DOMAIN_NOT_NATIVE))
  { ":ERROR_DS_DST_DOMAIN_NOT_NATIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DST_NC_MISMATCH))
  { ":ERROR_DS_DST_NC_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DS_REQUIRED))
  { ":ERROR_DS_DS_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUPLICATE_ID_FOUND))
  { ":ERROR_DS_DUPLICATE_ID_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_LDAP_DISPLAY_NAME))
  { ":ERROR_DS_DUP_LDAP_DISPLAY_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_LINK_ID))
  { ":ERROR_DS_DUP_LINK_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_MAPI_ID))
  { ":ERROR_DS_DUP_MAPI_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_MSDS_INTID))
  { ":ERROR_DS_DUP_MSDS_INTID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_OID))
  { ":ERROR_DS_DUP_OID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_RDN))
  { ":ERROR_DS_DUP_RDN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_DUP_SCHEMA_ID_GUID))
  { ":ERROR_DS_DUP_SCHEMA_ID_GUID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ENCODING_ERROR))
  { ":ERROR_DS_ENCODING_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EPOCH_MISMATCH))
  { ":ERROR_DS_EPOCH_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTING_AD_CHILD_NC))
  { ":ERROR_DS_EXISTING_AD_CHILD_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_AUX_CLS))
  { ":ERROR_DS_EXISTS_IN_AUX_CLS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_MAY_HAVE))
  { ":ERROR_DS_EXISTS_IN_MAY_HAVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_MUST_HAVE))
  { ":ERROR_DS_EXISTS_IN_MUST_HAVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_POSS_SUP))
  { ":ERROR_DS_EXISTS_IN_POSS_SUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_RDNATTID))
  { ":ERROR_DS_EXISTS_IN_RDNATTID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_EXISTS_IN_SUB_CLS))
  { ":ERROR_DS_EXISTS_IN_SUB_CLS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FILTER_UNKNOWN))
  { ":ERROR_DS_FILTER_UNKNOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS))
  { ":ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FOREST_VERSION_TOO_HIGH))
  { ":ERROR_DS_FOREST_VERSION_TOO_HIGH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_FOREST_VERSION_TOO_LOW))
  { ":ERROR_DS_FOREST_VERSION_TOO_LOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GCVERIFY_ERROR))
  { ":ERROR_DS_GCVERIFY_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GC_NOT_AVAILABLE))
  { ":ERROR_DS_GC_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GC_REQUIRED))
  { ":ERROR_DS_GC_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GENERIC_ERROR))
  { ":ERROR_DS_GENERIC_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER))
  { ":ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER))
  { ":ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER))
  { ":ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GOVERNSID_MISSING))
  { ":ERROR_DS_GOVERNSID_MISSING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_GROUP_CONVERSION_ERROR))
  { ":ERROR_DS_GROUP_CONVERSION_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_HAVE_PRIMARY_MEMBERS))
  { ":ERROR_DS_HAVE_PRIMARY_MEMBERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED))
  { ":ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD))
  { ":ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_MOD_OPERATION))
  { ":ERROR_DS_ILLEGAL_MOD_OPERATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_SUPERIOR))
  { ":ERROR_DS_ILLEGAL_SUPERIOR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION))
  { ":ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INAPPROPRIATE_AUTH))
  { ":ERROR_DS_INAPPROPRIATE_AUTH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INAPPROPRIATE_MATCHING))
  { ":ERROR_DS_INAPPROPRIATE_MATCHING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCOMPATIBLE_CONTROLS_USED))
  { ":ERROR_DS_INCOMPATIBLE_CONTROLS_USED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCOMPATIBLE_VERSION))
  { ":ERROR_DS_INCOMPATIBLE_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INCORRECT_ROLE_OWNER))
  { ":ERROR_DS_INCORRECT_ROLE_OWNER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INIT_FAILURE))
  { ":ERROR_DS_INIT_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INIT_FAILURE_CONSOLE))
  { ":ERROR_DS_INIT_FAILURE_CONSOLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE))
  { ":ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_NO_SRC_SCH_VERSION))
  { ":ERROR_DS_INSTALL_NO_SRC_SCH_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSTALL_SCHEMA_MISMATCH))
  { ":ERROR_DS_INSTALL_SCHEMA_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT))
  { ":ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INSUFF_ACCESS_RIGHTS))
  { ":ERROR_DS_INSUFF_ACCESS_RIGHTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INTERNAL_FAILURE))
  { ":ERROR_DS_INTERNAL_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_ATTRIBUTE_SYNTAX))
  { ":ERROR_DS_INVALID_ATTRIBUTE_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_DMD))
  { ":ERROR_DS_INVALID_DMD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_DN_SYNTAX))
  { ":ERROR_DS_INVALID_DN_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_GROUP_TYPE))
  { ":ERROR_DS_INVALID_GROUP_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_LDAP_DISPLAY_NAME))
  { ":ERROR_DS_INVALID_LDAP_DISPLAY_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_NAME_FOR_SPN))
  { ":ERROR_DS_INVALID_NAME_FOR_SPN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_ROLE_OWNER))
  { ":ERROR_DS_INVALID_ROLE_OWNER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_SCRIPT))
  { ":ERROR_DS_INVALID_SCRIPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_INVALID_SEARCH_FLAG))
  { ":ERROR_DS_INVALID_SEARCH_FLAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_IS_LEAF))
  { ":ERROR_DS_IS_LEAF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_KEY_NOT_UNIQUE))
  { ":ERROR_DS_KEY_NOT_UNIQUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LINK_ID_NOT_AVAILABLE))
  { ":ERROR_DS_LINK_ID_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER))
  { ":ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_ERROR))
  { ":ERROR_DS_LOCAL_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY))
  { ":ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOOKUP_FAILURE))
  { ":ERROR_DS_LOOKUP_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOOP_DETECT))
  { ":ERROR_DS_LOOP_DETECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_LOW_DSA_VERSION))
  { ":ERROR_DS_LOW_DSA_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4))
  { ":ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED))
  { ":ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MASTERDSA_REQUIRED))
  { ":ERROR_DS_MASTERDSA_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MAX_OBJ_SIZE_EXCEEDED))
  { ":ERROR_DS_MAX_OBJ_SIZE_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY))
  { ":ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_EXPECTED_ATT))
  { ":ERROR_DS_MISSING_EXPECTED_ATT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_FSMO_SETTINGS))
  { ":ERROR_DS_MISSING_FSMO_SETTINGS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER))
  { ":ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_REQUIRED_ATT))
  { ":ERROR_DS_MISSING_REQUIRED_ATT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MISSING_SUPREF))
  { ":ERROR_DS_MISSING_SUPREF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG))
  { ":ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE))
  { ":ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MODIFYDN_WRONG_GRANDPARENT))
  { ":ERROR_DS_MODIFYDN_WRONG_GRANDPARENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_MUST_BE_RUN_ON_DST_DC))
  { ":ERROR_DS_MUST_BE_RUN_ON_DST_DC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_DOMAIN_ONLY))
  { ":ERROR_DS_NAME_ERROR_DOMAIN_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NOT_FOUND))
  { ":ERROR_DS_NAME_ERROR_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NOT_UNIQUE))
  { ":ERROR_DS_NAME_ERROR_NOT_UNIQUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NO_MAPPING))
  { ":ERROR_DS_NAME_ERROR_NO_MAPPING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING))
  { ":ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_RESOLVING))
  { ":ERROR_DS_NAME_ERROR_RESOLVING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_ERROR_TRUST_REFERRAL))
  { ":ERROR_DS_NAME_ERROR_TRUST_REFERRAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_NOT_UNIQUE))
  { ":ERROR_DS_NAME_NOT_UNIQUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_REFERENCE_INVALID))
  { ":ERROR_DS_NAME_REFERENCE_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TOO_LONG))
  { ":ERROR_DS_NAME_TOO_LONG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TOO_MANY_PARTS))
  { ":ERROR_DS_NAME_TOO_MANY_PARTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_TYPE_UNKNOWN))
  { ":ERROR_DS_NAME_TYPE_UNKNOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_UNPARSEABLE))
  { ":ERROR_DS_NAME_UNPARSEABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAME_VALUE_TOO_LONG))
  { ":ERROR_DS_NAME_VALUE_TOO_LONG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAMING_MASTER_GC))
  { ":ERROR_DS_NAMING_MASTER_GC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NAMING_VIOLATION))
  { ":ERROR_DS_NAMING_VIOLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NCNAME_MISSING_CR_REF))
  { ":ERROR_DS_NCNAME_MISSING_CR_REF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NCNAME_MUST_BE_NC))
  { ":ERROR_DS_NCNAME_MUST_BE_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NC_MUST_HAVE_NC_PARENT))
  { ":ERROR_DS_NC_MUST_HAVE_NC_PARENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NC_STILL_HAS_DSAS))
  { ":ERROR_DS_NC_STILL_HAS_DSAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_MAY_HAVE))
  { ":ERROR_DS_NONEXISTENT_MAY_HAVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_MUST_HAVE))
  { ":ERROR_DS_NONEXISTENT_MUST_HAVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONEXISTENT_POSS_SUP))
  { ":ERROR_DS_NONEXISTENT_POSS_SUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NONSAFE_SCHEMA_CHANGE))
  { ":ERROR_DS_NONSAFE_SCHEMA_CHANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NON_BASE_SEARCH))
  { ":ERROR_DS_NON_BASE_SEARCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX))
  { ":ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_AN_OBJECT))
  { ":ERROR_DS_NOT_AN_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC))
  { ":ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_CLOSEST))
  { ":ERROR_DS_NOT_CLOSEST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_INSTALLED))
  { ":ERROR_DS_NOT_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_ON_BACKLINK))
  { ":ERROR_DS_NOT_ON_BACKLINK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_SUPPORTED))
  { ":ERROR_DS_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NOT_SUPPORTED_SORT_ORDER))
  { ":ERROR_DS_NOT_SUPPORTED_SORT_ORDER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_ATTRIBUTE_OR_VALUE))
  { ":ERROR_DS_NO_ATTRIBUTE_OR_VALUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN))
  { ":ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHAINED_EVAL))
  { ":ERROR_DS_NO_CHAINED_EVAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHAINING))
  { ":ERROR_DS_NO_CHAINING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CHECKPOINT_WITH_PDC))
  { ":ERROR_DS_NO_CHECKPOINT_WITH_PDC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_CROSSREF_FOR_NC))
  { ":ERROR_DS_NO_CROSSREF_FOR_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_DELETED_NAME))
  { ":ERROR_DS_NO_DELETED_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS))
  { ":ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_MORE_RIDS))
  { ":ERROR_DS_NO_MORE_RIDS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_MSDS_INTID))
  { ":ERROR_DS_NO_MSDS_INTID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN))
  { ":ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN))
  { ":ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC))
  { ":ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_PARENT_OBJECT))
  { ":ERROR_DS_NO_PARENT_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION))
  { ":ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA))
  { ":ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_REF_DOMAIN))
  { ":ERROR_DS_NO_REF_DOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_REQUESTED_ATTS_FOUND))
  { ":ERROR_DS_NO_REQUESTED_ATTS_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RESULTS_RETURNED))
  { ":ERROR_DS_NO_RESULTS_RETURNED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_RIDS_ALLOCATED))
  { ":ERROR_DS_NO_RIDS_ALLOCATED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_SUCH_OBJECT))
  { ":ERROR_DS_NO_SUCH_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NO_TREE_DELETE_ABOVE_NC))
  { ":ERROR_DS_NO_TREE_DELETE_ABOVE_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NTDSCRIPT_PROCESS_ERROR))
  { ":ERROR_DS_NTDSCRIPT_PROCESS_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_NTDSCRIPT_SYNTAX_ERROR))
  { ":ERROR_DS_NTDSCRIPT_SYNTAX_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_BEING_REMOVED))
  { ":ERROR_DS_OBJECT_BEING_REMOVED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_CLASS_REQUIRED))
  { ":ERROR_DS_OBJECT_CLASS_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJECT_RESULTS_TOO_LARGE))
  { ":ERROR_DS_OBJECT_RESULTS_TOO_LARGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_NOT_DEFINED))
  { ":ERROR_DS_OBJ_CLASS_NOT_DEFINED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_NOT_SUBCLASS))
  { ":ERROR_DS_OBJ_CLASS_NOT_SUBCLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_CLASS_VIOLATION))
  { ":ERROR_DS_OBJ_CLASS_VIOLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_GUID_EXISTS))
  { ":ERROR_DS_OBJ_GUID_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_NOT_FOUND))
  { ":ERROR_DS_OBJ_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_STRING_NAME_EXISTS))
  { ":ERROR_DS_OBJ_STRING_NAME_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OBJ_TOO_LARGE))
  { ":ERROR_DS_OBJ_TOO_LARGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OFFSET_RANGE_ERROR))
  { ":ERROR_DS_OFFSET_RANGE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OPERATIONS_ERROR))
  { ":ERROR_DS_OPERATIONS_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OUT_OF_SCOPE))
  { ":ERROR_DS_OUT_OF_SCOPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_OUT_OF_VERSION_STORE))
  { ":ERROR_DS_OUT_OF_VERSION_STORE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PARAM_ERROR))
  { ":ERROR_DS_PARAM_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PARENT_IS_AN_ALIAS))
  { ":ERROR_DS_PARENT_IS_AN_ALIAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PDC_OPERATION_IN_PROGRESS))
  { ":ERROR_DS_PDC_OPERATION_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_PROTOCOL_ERROR))
  { ":ERROR_DS_PROTOCOL_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RANGE_CONSTRAINT))
  { ":ERROR_DS_RANGE_CONSTRAINT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RDN_DOESNT_MATCH_SCHEMA))
  { ":ERROR_DS_RDN_DOESNT_MATCH_SCHEMA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RECALCSCHEMA_FAILED))
  { ":ERROR_DS_RECALCSCHEMA_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFERRAL))
  { ":ERROR_DS_REFERRAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFERRAL_LIMIT_EXCEEDED))
  { ":ERROR_DS_REFERRAL_LIMIT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REFUSING_FSMO_ROLES))
  { ":ERROR_DS_REFUSING_FSMO_ROLES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REMOTE_CROSSREF_OP_FAILED))
  { ":ERROR_DS_REMOTE_CROSSREF_OP_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REPLICATOR_ONLY))
  { ":ERROR_DS_REPLICATOR_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR))
  { ":ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RESERVED_LINK_ID))
  { ":ERROR_DS_RESERVED_LINK_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_RIDMGR_INIT_ERROR))
  { ":ERROR_DS_RIDMGR_INIT_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROLE_NOT_VERIFIED))
  { ":ERROR_DS_ROLE_NOT_VERIFIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_CANT_BE_SUBREF))
  { ":ERROR_DS_ROOT_CANT_BE_SUBREF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_MUST_BE_NC))
  { ":ERROR_DS_ROOT_MUST_BE_NC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_ROOT_REQUIRES_CLASS_TOP))
  { ":ERROR_DS_ROOT_REQUIRES_CLASS_TOP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_INIT_FAILURE))
  { ":ERROR_DS_SAM_INIT_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_INIT_FAILURE_CONSOLE))
  { ":ERROR_DS_SAM_INIT_FAILURE_CONSOLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY))
  { ":ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD))
  { ":ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_ALLOC_FAILED))
  { ":ERROR_DS_SCHEMA_ALLOC_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_NOT_LOADED))
  { ":ERROR_DS_SCHEMA_NOT_LOADED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SCHEMA_UPDATE_DISALLOWED))
  { ":ERROR_DS_SCHEMA_UPDATE_DISALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SECURITY_CHECKING_ERROR))
  { ":ERROR_DS_SECURITY_CHECKING_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SECURITY_ILLEGAL_MODIFY))
  { ":ERROR_DS_SECURITY_ILLEGAL_MODIFY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEC_DESC_INVALID))
  { ":ERROR_DS_SEC_DESC_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEC_DESC_TOO_SHORT))
  { ":ERROR_DS_SEC_DESC_TOO_SHORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SEMANTIC_ATT_TEST))
  { ":ERROR_DS_SEMANTIC_ATT_TEST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SENSITIVE_GROUP_VIOLATION))
  { ":ERROR_DS_SENSITIVE_GROUP_VIOLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SERVER_DOWN))
  { ":ERROR_DS_SERVER_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SHUTTING_DOWN))
  { ":ERROR_DS_SHUTTING_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SINGLE_USER_MODE_FAILED))
  { ":ERROR_DS_SINGLE_USER_MODE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SINGLE_VALUE_CONSTRAINT))
  { ":ERROR_DS_SINGLE_VALUE_CONSTRAINT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SIZELIMIT_EXCEEDED))
  { ":ERROR_DS_SIZELIMIT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SORT_CONTROL_MISSING))
  { ":ERROR_DS_SORT_CONTROL_MISSING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SOURCE_AUDITING_NOT_ENABLED))
  { ":ERROR_DS_SOURCE_AUDITING_NOT_ENABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SOURCE_DOMAIN_IN_FOREST))
  { ":ERROR_DS_SOURCE_DOMAIN_IN_FOREST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_AND_DST_NC_IDENTICAL))
  { ":ERROR_DS_SRC_AND_DST_NC_IDENTICAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH))
  { ":ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER))
  { ":ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_GUID_MISMATCH))
  { ":ERROR_DS_SRC_GUID_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_NAME_MISMATCH))
  { ":ERROR_DS_SRC_NAME_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER))
  { ":ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SRC_SID_EXISTS_IN_FOREST))
  { ":ERROR_DS_SRC_SID_EXISTS_IN_FOREST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_STRING_SD_CONVERSION_FAILED))
  { ":ERROR_DS_STRING_SD_CONVERSION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_STRONG_AUTH_REQUIRED))
  { ":ERROR_DS_STRONG_AUTH_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUBREF_MUST_HAVE_PARENT))
  { ":ERROR_DS_SUBREF_MUST_HAVE_PARENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD))
  { ":ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SUB_CLS_TEST_FAIL))
  { ":ERROR_DS_SUB_CLS_TEST_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_SYNTAX_MISMATCH))
  { ":ERROR_DS_SYNTAX_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_THREAD_LIMIT_EXCEEDED))
  { ":ERROR_DS_THREAD_LIMIT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_TIMELIMIT_EXCEEDED))
  { ":ERROR_DS_TIMELIMIT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_TREE_DELETE_NOT_FINISHED))
  { ":ERROR_DS_TREE_DELETE_NOT_FINISHED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNABLE_TO_SURRENDER_ROLES))
  { ":ERROR_DS_UNABLE_TO_SURRENDER_ROLES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNAVAILABLE))
  { ":ERROR_DS_UNAVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNAVAILABLE_CRIT_EXTENSION))
  { ":ERROR_DS_UNAVAILABLE_CRIT_EXTENSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNICODEPWD_NOT_IN_QUOTES))
  { ":ERROR_DS_UNICODEPWD_NOT_IN_QUOTES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER))
  { ":ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNKNOWN_ERROR))
  { ":ERROR_DS_UNKNOWN_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNKNOWN_OPERATION))
  { ":ERROR_DS_UNKNOWN_OPERATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_UNWILLING_TO_PERFORM))
  { ":ERROR_DS_UNWILLING_TO_PERFORM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_USER_BUFFER_TO_SMALL))
  { ":ERROR_DS_USER_BUFFER_TO_SMALL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL))
  { ":ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WRONG_LINKED_ATT_SYNTAX))
  { ":ERROR_DS_WRONG_LINKED_ATT_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DS_WRONG_OM_OBJ_CLASS))
  { ":ERROR_DS_WRONG_OM_OBJ_CLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUPLICATE_SERVICE_NAME))
  { ":ERROR_DUPLICATE_SERVICE_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUPLICATE_TAG))
  { ":ERROR_DUPLICATE_TAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUP_DOMAINNAME))
  { ":ERROR_DUP_DOMAINNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DUP_NAME))
  { ":ERROR_DUP_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_DYNLINK_FROM_INVALID_RING))
  { ":ERROR_DYNLINK_FROM_INVALID_RING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EAS_DIDNT_FIT))
  { ":ERROR_EAS_DIDNT_FIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EAS_NOT_SUPPORTED))
  { ":ERROR_EAS_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_ACCESS_DENIED))
  { ":ERROR_EA_ACCESS_DENIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_FILE_CORRUPT))
  { ":ERROR_EA_FILE_CORRUPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_LIST_INCONSISTENT))
  { ":ERROR_EA_LIST_INCONSISTENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EA_TABLE_FULL))
  { ":ERROR_EA_TABLE_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_ALG_BLOB_TOO_BIG))
  { ":ERROR_EFS_ALG_BLOB_TOO_BIG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_DISABLED))
  { ":ERROR_EFS_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_SERVER_NOT_TRUSTED))
  { ":ERROR_EFS_SERVER_NOT_TRUSTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EFS_VERSION_NOT_SUPPORT))
  { ":ERROR_EFS_VERSION_NOT_SUPPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EMPTY))
  { ":ERROR_EMPTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ENCRYPTION_FAILED))
  { ":ERROR_ENCRYPTION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_END_OF_MEDIA))
  { ":ERROR_END_OF_MEDIA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ENVVAR_NOT_FOUND))
  { ":ERROR_ENVVAR_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EOM_OVERFLOW))
  { ":ERROR_EOM_OVERFLOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_CANT_START))
  { ":ERROR_EVENTLOG_CANT_START" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_FILE_CHANGED))
  { ":ERROR_EVENTLOG_FILE_CHANGED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EVENTLOG_FILE_CORRUPT))
  { ":ERROR_EVENTLOG_FILE_CORRUPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXCEPTION_IN_SERVICE))
  { ":ERROR_EXCEPTION_IN_SERVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXCL_SEM_ALREADY_OWNED))
  { ":ERROR_EXCL_SEM_ALREADY_OWNED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY))
  { ":ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY))
  { ":ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_MACHINE_TYPE_MISMATCH))
  { ":ERROR_EXE_MACHINE_TYPE_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXE_MARKED_INVALID))
  { ":ERROR_EXE_MARKED_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_EXTENDED_ERROR))
  { ":ERROR_EXTENDED_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FAILED_SERVICE_CONTROLLER_CONNECT))
  { ":ERROR_FAILED_SERVICE_CONTROLLER_CONNECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FAIL_I24))
  { ":ERROR_FAIL_I24" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILEMARK_DETECTED))
  { ":ERROR_FILEMARK_DETECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILENAME_EXCED_RANGE))
  { ":ERROR_FILENAME_EXCED_RANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_CORRUPT))
  { ":ERROR_FILE_CORRUPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_ENCRYPTED))
  { ":ERROR_FILE_ENCRYPTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_EXISTS))
  { ":ERROR_FILE_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_INVALID))
  { ":ERROR_FILE_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_NOT_ENCRYPTED))
  { ":ERROR_FILE_NOT_ENCRYPTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_NOT_FOUND))
  { ":ERROR_FILE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_OFFLINE))
  { ":ERROR_FILE_OFFLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FILE_READ_ONLY))
  { ":ERROR_FILE_READ_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_BAD_REGISTERS))
  { ":ERROR_FLOPPY_BAD_REGISTERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_ID_MARK_NOT_FOUND))
  { ":ERROR_FLOPPY_ID_MARK_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_UNKNOWN_ERROR))
  { ":ERROR_FLOPPY_UNKNOWN_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FLOPPY_WRONG_CYLINDER))
  { ":ERROR_FLOPPY_WRONG_CYLINDER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FULLSCREEN_MODE))
  { ":ERROR_FULLSCREEN_MODE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FULL_BACKUP))
  { ":ERROR_FULL_BACKUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FUNCTION_FAILED))
  { ":ERROR_FUNCTION_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_FUNCTION_NOT_CALLED))
  { ":ERROR_FUNCTION_NOT_CALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GENERIC_NOT_MAPPED))
  { ":ERROR_GENERIC_NOT_MAPPED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GEN_FAILURE))
  { ":ERROR_GEN_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GLOBAL_ONLY_HOOK))
  { ":ERROR_GLOBAL_ONLY_HOOK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GRACEFUL_DISCONNECT))
  { ":ERROR_GRACEFUL_DISCONNECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_EXISTS))
  { ":ERROR_GROUP_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_AVAILABLE))
  { ":ERROR_GROUP_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_FOUND))
  { ":ERROR_GROUP_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_GROUP_NOT_ONLINE))
  { ":ERROR_GROUP_NOT_ONLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HANDLE_DISK_FULL))
  { ":ERROR_HANDLE_DISK_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HANDLE_EOF))
  { ":ERROR_HANDLE_EOF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_NEEDS_HMOD))
  { ":ERROR_HOOK_NEEDS_HMOD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_NOT_INSTALLED))
  { ":ERROR_HOOK_NOT_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOOK_TYPE_NOT_ALLOWED))
  { ":ERROR_HOOK_TYPE_NOT_ALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_DOWN))
  { ":ERROR_HOST_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_AVAILABLE))
  { ":ERROR_HOST_NODE_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_GROUP_OWNER))
  { ":ERROR_HOST_NODE_NOT_GROUP_OWNER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_NODE_NOT_RESOURCE_OWNER))
  { ":ERROR_HOST_NODE_NOT_RESOURCE_OWNER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOST_UNREACHABLE))
  { ":ERROR_HOST_UNREACHABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOTKEY_ALREADY_REGISTERED))
  { ":ERROR_HOTKEY_ALREADY_REGISTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HOTKEY_NOT_REGISTERED))
  { ":ERROR_HOTKEY_NOT_REGISTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_HWNDS_HAVE_DIFF_PARENT))
  { ":ERROR_HWNDS_HAVE_DIFF_PARENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ICM_NOT_ENABLED))
  { ":ERROR_ICM_NOT_ENABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ILLEGAL_ELEMENT_ADDRESS))
  { ":ERROR_ILLEGAL_ELEMENT_ADDRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ILL_FORMED_PASSWORD))
  { ":ERROR_ILL_FORMED_PASSWORD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INCORRECT_ADDRESS))
  { ":ERROR_INCORRECT_ADDRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INC_BACKUP))
  { ":ERROR_INC_BACKUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INDEX_ABSENT))
  { ":ERROR_INDEX_ABSENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INDIGENOUS_TYPE))
  { ":ERROR_INDIGENOUS_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INFLOOP_IN_RELOC_CHAIN))
  { ":ERROR_INFLOOP_IN_RELOC_CHAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_ALREADY_RUNNING))
  { ":ERROR_INSTALL_ALREADY_RUNNING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_FAILURE))
  { ":ERROR_INSTALL_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_LANGUAGE_UNSUPPORTED))
  { ":ERROR_INSTALL_LANGUAGE_UNSUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_LOG_FAILURE))
  { ":ERROR_INSTALL_LOG_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_NOTUSED))
  { ":ERROR_INSTALL_NOTUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_INVALID))
  { ":ERROR_INSTALL_PACKAGE_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_OPEN_FAILED))
  { ":ERROR_INSTALL_PACKAGE_OPEN_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_REJECTED))
  { ":ERROR_INSTALL_PACKAGE_REJECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PACKAGE_VERSION))
  { ":ERROR_INSTALL_PACKAGE_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_PLATFORM_UNSUPPORTED))
  { ":ERROR_INSTALL_PLATFORM_UNSUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_REMOTE_DISALLOWED))
  { ":ERROR_INSTALL_REMOTE_DISALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_REMOTE_PROHIBITED))
  { ":ERROR_INSTALL_REMOTE_PROHIBITED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SERVICE_FAILURE))
  { ":ERROR_INSTALL_SERVICE_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SOURCE_ABSENT))
  { ":ERROR_INSTALL_SOURCE_ABSENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_SUSPEND))
  { ":ERROR_INSTALL_SUSPEND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TEMP_UNWRITABLE))
  { ":ERROR_INSTALL_TEMP_UNWRITABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TRANSFORM_FAILURE))
  { ":ERROR_INSTALL_TRANSFORM_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_TRANSFORM_REJECTED))
  { ":ERROR_INSTALL_TRANSFORM_REJECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_UI_FAILURE))
  { ":ERROR_INSTALL_UI_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSTALL_USEREXIT))
  { ":ERROR_INSTALL_USEREXIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INSUFFICIENT_BUFFER))
  { ":ERROR_INSUFFICIENT_BUFFER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_DB_CORRUPTION))
  { ":ERROR_INTERNAL_DB_CORRUPTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_DB_ERROR))
  { ":ERROR_INTERNAL_DB_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INTERNAL_ERROR))
  { ":ERROR_INTERNAL_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCEL_HANDLE))
  { ":ERROR_INVALID_ACCEL_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCESS))
  { ":ERROR_INVALID_ACCESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACCOUNT_NAME))
  { ":ERROR_INVALID_ACCOUNT_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ACL))
  { ":ERROR_INVALID_ACL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ADDRESS))
  { ":ERROR_INVALID_ADDRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_AT_INTERRUPT_TIME))
  { ":ERROR_INVALID_AT_INTERRUPT_TIME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_BLOCK))
  { ":ERROR_INVALID_BLOCK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_BLOCK_LENGTH))
  { ":ERROR_INVALID_BLOCK_LENGTH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CATEGORY))
  { ":ERROR_INVALID_CATEGORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CLEANER))
  { ":ERROR_INVALID_CLEANER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CMM))
  { ":ERROR_INVALID_CMM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COLORINDEX))
  { ":ERROR_INVALID_COLORINDEX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COLORSPACE))
  { ":ERROR_INVALID_COLORSPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMBOBOX_MESSAGE))
  { ":ERROR_INVALID_COMBOBOX_MESSAGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMMAND_LINE))
  { ":ERROR_INVALID_COMMAND_LINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_COMPUTERNAME))
  { ":ERROR_INVALID_COMPUTERNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_CURSOR_HANDLE))
  { ":ERROR_INVALID_CURSOR_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DATA))
  { ":ERROR_INVALID_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DATATYPE))
  { ":ERROR_INVALID_DATATYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DLL))
  { ":ERROR_INVALID_DLL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAINNAME))
  { ":ERROR_INVALID_DOMAINNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAIN_ROLE))
  { ":ERROR_INVALID_DOMAIN_ROLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DOMAIN_STATE))
  { ":ERROR_INVALID_DOMAIN_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DRIVE))
  { ":ERROR_INVALID_DRIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DRIVE_OBJECT))
  { ":ERROR_INVALID_DRIVE_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_DWP_HANDLE))
  { ":ERROR_INVALID_DWP_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EA_HANDLE))
  { ":ERROR_INVALID_EA_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EA_NAME))
  { ":ERROR_INVALID_EA_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EDIT_HEIGHT))
  { ":ERROR_INVALID_EDIT_HEIGHT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ENVIRONMENT))
  { ":ERROR_INVALID_ENVIRONMENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EVENTNAME))
  { ":ERROR_INVALID_EVENTNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EVENT_COUNT))
  { ":ERROR_INVALID_EVENT_COUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_EXE_SIGNATURE))
  { ":ERROR_INVALID_EXE_SIGNATURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FIELD))
  { ":ERROR_INVALID_FIELD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FILTER_PROC))
  { ":ERROR_INVALID_FILTER_PROC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FLAGS))
  { ":ERROR_INVALID_FLAGS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FLAG_NUMBER))
  { ":ERROR_INVALID_FLAG_NUMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FORM_NAME))
  { ":ERROR_INVALID_FORM_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FORM_SIZE))
  { ":ERROR_INVALID_FORM_SIZE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_FUNCTION))
  { ":ERROR_INVALID_FUNCTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GROUPNAME))
  { ":ERROR_INVALID_GROUPNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GROUP_ATTRIBUTES))
  { ":ERROR_INVALID_GROUP_ATTRIBUTES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_GW_COMMAND))
  { ":ERROR_INVALID_GW_COMMAND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HANDLE))
  { ":ERROR_INVALID_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HANDLE_STATE))
  { ":ERROR_INVALID_HANDLE_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HOOK_FILTER))
  { ":ERROR_INVALID_HOOK_FILTER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_HOOK_HANDLE))
  { ":ERROR_INVALID_HOOK_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ICON_HANDLE))
  { ":ERROR_INVALID_ICON_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ID_AUTHORITY))
  { ":ERROR_INVALID_ID_AUTHORITY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_IMPORT_OF_NON_DLL))
  { ":ERROR_INVALID_IMPORT_OF_NON_DLL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_INDEX))
  { ":ERROR_INVALID_INDEX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_KEYBOARD_HANDLE))
  { ":ERROR_INVALID_KEYBOARD_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LB_MESSAGE))
  { ":ERROR_INVALID_LB_MESSAGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LEVEL))
  { ":ERROR_INVALID_LEVEL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LIBRARY))
  { ":ERROR_INVALID_LIBRARY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LIST_FORMAT))
  { ":ERROR_INVALID_LIST_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LOGON_HOURS))
  { ":ERROR_INVALID_LOGON_HOURS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_LOGON_TYPE))
  { ":ERROR_INVALID_LOGON_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEDIA))
  { ":ERROR_INVALID_MEDIA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEDIA_POOL))
  { ":ERROR_INVALID_MEDIA_POOL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MEMBER))
  { ":ERROR_INVALID_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MENU_HANDLE))
  { ":ERROR_INVALID_MENU_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGE))
  { ":ERROR_INVALID_MESSAGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGEDEST))
  { ":ERROR_INVALID_MESSAGEDEST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MESSAGENAME))
  { ":ERROR_INVALID_MESSAGENAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MINALLOCSIZE))
  { ":ERROR_INVALID_MINALLOCSIZE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MODULETYPE))
  { ":ERROR_INVALID_MODULETYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MONITOR_HANDLE))
  { ":ERROR_INVALID_MONITOR_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_MSGBOX_STYLE))
  { ":ERROR_INVALID_MSGBOX_STYLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_NAME))
  { ":ERROR_INVALID_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_NETNAME))
  { ":ERROR_INVALID_NETNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPERATION))
  { ":ERROR_INVALID_OPERATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPERATION_ON_QUORUM))
  { ":ERROR_INVALID_OPERATION_ON_QUORUM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OPLOCK_PROTOCOL))
  { ":ERROR_INVALID_OPLOCK_PROTOCOL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_ORDINAL))
  { ":ERROR_INVALID_ORDINAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_OWNER))
  { ":ERROR_INVALID_OWNER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PARAMETER))
  { ":ERROR_INVALID_PARAMETER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PASSWORD))
  { ":ERROR_INVALID_PASSWORD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PASSWORDNAME))
  { ":ERROR_INVALID_PASSWORDNAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PIXEL_FORMAT))
  { ":ERROR_INVALID_PIXEL_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRIMARY_GROUP))
  { ":ERROR_INVALID_PRIMARY_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_COMMAND))
  { ":ERROR_INVALID_PRINTER_COMMAND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_NAME))
  { ":ERROR_INVALID_PRINTER_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINTER_STATE))
  { ":ERROR_INVALID_PRINTER_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRINT_MONITOR))
  { ":ERROR_INVALID_PRINT_MONITOR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PRIORITY))
  { ":ERROR_INVALID_PRIORITY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_PROFILE))
  { ":ERROR_INVALID_PROFILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_REPARSE_DATA))
  { ":ERROR_INVALID_REPARSE_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SCROLLBAR_RANGE))
  { ":ERROR_INVALID_SCROLLBAR_RANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SECURITY_DESCR))
  { ":ERROR_INVALID_SECURITY_DESCR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEGDPL))
  { ":ERROR_INVALID_SEGDPL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEGMENT_NUMBER))
  { ":ERROR_INVALID_SEGMENT_NUMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SEPARATOR_FILE))
  { ":ERROR_INVALID_SEPARATOR_FILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVER_STATE))
  { ":ERROR_INVALID_SERVER_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICENAME))
  { ":ERROR_INVALID_SERVICENAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_ACCOUNT))
  { ":ERROR_INVALID_SERVICE_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_CONTROL))
  { ":ERROR_INVALID_SERVICE_CONTROL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SERVICE_LOCK))
  { ":ERROR_INVALID_SERVICE_LOCK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SHARENAME))
  { ":ERROR_INVALID_SHARENAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SHOWWIN_COMMAND))
  { ":ERROR_INVALID_SHOWWIN_COMMAND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SID))
  { ":ERROR_INVALID_SID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SIGNAL_NUMBER))
  { ":ERROR_INVALID_SIGNAL_NUMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SPI_VALUE))
  { ":ERROR_INVALID_SPI_VALUE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STACKSEG))
  { ":ERROR_INVALID_STACKSEG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STARTING_CODESEG))
  { ":ERROR_INVALID_STARTING_CODESEG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_STATE))
  { ":ERROR_INVALID_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_SUB_AUTHORITY))
  { ":ERROR_INVALID_SUB_AUTHORITY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TABLE))
  { ":ERROR_INVALID_TABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TARGET_HANDLE))
  { ":ERROR_INVALID_TARGET_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_THREAD_ID))
  { ":ERROR_INVALID_THREAD_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TIME))
  { ":ERROR_INVALID_TIME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_TRANSFORM))
  { ":ERROR_INVALID_TRANSFORM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_USER_BUFFER))
  { ":ERROR_INVALID_USER_BUFFER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_VERIFY_SWITCH))
  { ":ERROR_INVALID_VERIFY_SWITCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WINDOW_HANDLE))
  { ":ERROR_INVALID_WINDOW_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WINDOW_STYLE))
  { ":ERROR_INVALID_WINDOW_STYLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_INVALID_WORKSTATION))
  { ":ERROR_INVALID_WORKSTATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IOPL_NOT_ENABLED))
  { ":ERROR_IOPL_NOT_ENABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_DEVICE))
  { ":ERROR_IO_DEVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_INCOMPLETE))
  { ":ERROR_IO_INCOMPLETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IO_PENDING))
  { ":ERROR_IO_PENDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND))
  { ":ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND))
  { ":ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND))
  { ":ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED))
  { ":ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ATTRIB_FAIL))
  { ":ERROR_IPSEC_IKE_ATTRIB_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_AUTH_FAIL))
  { ":ERROR_IPSEC_IKE_AUTH_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_BENIGN_REINIT))
  { ":ERROR_IPSEC_IKE_BENIGN_REINIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_CRL_FAILED))
  { ":ERROR_IPSEC_IKE_CRL_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DECRYPT))
  { ":ERROR_IPSEC_IKE_DECRYPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DH_FAIL))
  { ":ERROR_IPSEC_IKE_DH_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DH_FAILURE))
  { ":ERROR_IPSEC_IKE_DH_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_DROP_NO_RESPONSE))
  { ":ERROR_IPSEC_IKE_DROP_NO_RESPONSE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ENCRYPT))
  { ":ERROR_IPSEC_IKE_ENCRYPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_ERROR))
  { ":ERROR_IPSEC_IKE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_FAILQUERYSSP))
  { ":ERROR_IPSEC_IKE_FAILQUERYSSP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_FAILSSPINIT))
  { ":ERROR_IPSEC_IKE_FAILSSPINIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR))
  { ":ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_GETSPIFAIL))
  { ":ERROR_IPSEC_IKE_GETSPIFAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_AUTH_ALG))
  { ":ERROR_IPSEC_IKE_INVALID_AUTH_ALG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN))
  { ":ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_CERT_TYPE))
  { ":ERROR_IPSEC_IKE_INVALID_CERT_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_COOKIE))
  { ":ERROR_IPSEC_IKE_INVALID_COOKIE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG))
  { ":ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_FILTER))
  { ":ERROR_IPSEC_IKE_INVALID_FILTER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_GROUP))
  { ":ERROR_IPSEC_IKE_INVALID_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH))
  { ":ERROR_IPSEC_IKE_INVALID_HASH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH_ALG))
  { ":ERROR_IPSEC_IKE_INVALID_HASH_ALG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HASH_SIZE))
  { ":ERROR_IPSEC_IKE_INVALID_HASH_SIZE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_HEADER))
  { ":ERROR_IPSEC_IKE_INVALID_HEADER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_KEY_USAGE))
  { ":ERROR_IPSEC_IKE_INVALID_KEY_USAGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_PAYLOAD))
  { ":ERROR_IPSEC_IKE_INVALID_PAYLOAD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_POLICY))
  { ":ERROR_IPSEC_IKE_INVALID_POLICY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY))
  { ":ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SIG))
  { ":ERROR_IPSEC_IKE_INVALID_SIG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SIGNATURE))
  { ":ERROR_IPSEC_IKE_INVALID_SIGNATURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_INVALID_SITUATION))
  { ":ERROR_IPSEC_IKE_INVALID_SITUATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_KERBEROS_ERROR))
  { ":ERROR_IPSEC_IKE_KERBEROS_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_LOAD_FAILED))
  { ":ERROR_IPSEC_IKE_LOAD_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_LOAD_SOFT_SA))
  { ":ERROR_IPSEC_IKE_LOAD_SOFT_SA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_ACQUIRE_DROP))
  { ":ERROR_IPSEC_IKE_MM_ACQUIRE_DROP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_DELAY_DROP))
  { ":ERROR_IPSEC_IKE_MM_DELAY_DROP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_MM_LIMIT))
  { ":ERROR_IPSEC_IKE_MM_LIMIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEGOTIATION_DISABLED))
  { ":ERROR_IPSEC_IKE_NEGOTIATION_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEGOTIATION_PENDING))
  { ":ERROR_IPSEC_IKE_NEGOTIATION_PENDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NEG_STATUS_END))
  { ":ERROR_IPSEC_IKE_NEG_STATUS_END" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NOTCBPRIV))
  { ":ERROR_IPSEC_IKE_NOTCBPRIV" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_CERT))
  { ":ERROR_IPSEC_IKE_NO_CERT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_MM_POLICY))
  { ":ERROR_IPSEC_IKE_NO_MM_POLICY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PEER_CERT))
  { ":ERROR_IPSEC_IKE_NO_PEER_CERT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_POLICY))
  { ":ERROR_IPSEC_IKE_NO_POLICY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PRIVATE_KEY))
  { ":ERROR_IPSEC_IKE_NO_PRIVATE_KEY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_NO_PUBLIC_KEY))
  { ":ERROR_IPSEC_IKE_NO_PUBLIC_KEY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_OUT_OF_MEMORY))
  { ":ERROR_IPSEC_IKE_OUT_OF_MEMORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PEER_CRL_FAILED))
  { ":ERROR_IPSEC_IKE_PEER_CRL_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_POLICY_CHANGE))
  { ":ERROR_IPSEC_IKE_POLICY_CHANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_POLICY_MATCH))
  { ":ERROR_IPSEC_IKE_POLICY_MATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_CERT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_DELETE))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_DELETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_HASH))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_HASH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_ID))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_KE))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_KE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_NONCE))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_NONCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_PROP))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_PROP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_SA))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_SA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_SIG))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_SIG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_TRANS))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_TRANS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR))
  { ":ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QM_ACQUIRE_DROP))
  { ":ERROR_IPSEC_IKE_QM_ACQUIRE_DROP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QM_DELAY_DROP))
  { ":ERROR_IPSEC_IKE_QM_DELAY_DROP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QUEUE_DROP_MM))
  { ":ERROR_IPSEC_IKE_QUEUE_DROP_MM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM))
  { ":ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_RPC_DELETE))
  { ":ERROR_IPSEC_IKE_RPC_DELETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SA_DELETED))
  { ":ERROR_IPSEC_IKE_SA_DELETED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SA_REAPED))
  { ":ERROR_IPSEC_IKE_SA_REAPED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SECLOADFAIL))
  { ":ERROR_IPSEC_IKE_SECLOADFAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN))
  { ":ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SRVACQFAIL))
  { ":ERROR_IPSEC_IKE_SRVACQFAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_SRVQUERYCRED))
  { ":ERROR_IPSEC_IKE_SRVQUERYCRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_TIMED_OUT))
  { ":ERROR_IPSEC_IKE_TIMED_OUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_UNKNOWN_DOI))
  { ":ERROR_IPSEC_IKE_UNKNOWN_DOI" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_IKE_UNSUPPORTED_ID))
  { ":ERROR_IPSEC_IKE_UNSUPPORTED_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_EXISTS))
  { ":ERROR_IPSEC_MM_AUTH_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_IN_USE))
  { ":ERROR_IPSEC_MM_AUTH_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_NOT_FOUND))
  { ":ERROR_IPSEC_MM_AUTH_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_AUTH_PENDING_DELETION))
  { ":ERROR_IPSEC_MM_AUTH_PENDING_DELETION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_EXISTS))
  { ":ERROR_IPSEC_MM_FILTER_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_NOT_FOUND))
  { ":ERROR_IPSEC_MM_FILTER_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_FILTER_PENDING_DELETION))
  { ":ERROR_IPSEC_MM_FILTER_PENDING_DELETION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_EXISTS))
  { ":ERROR_IPSEC_MM_POLICY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_IN_USE))
  { ":ERROR_IPSEC_MM_POLICY_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_NOT_FOUND))
  { ":ERROR_IPSEC_MM_POLICY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_MM_POLICY_PENDING_DELETION))
  { ":ERROR_IPSEC_MM_POLICY_PENDING_DELETION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_EXISTS))
  { ":ERROR_IPSEC_QM_POLICY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_IN_USE))
  { ":ERROR_IPSEC_QM_POLICY_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_NOT_FOUND))
  { ":ERROR_IPSEC_QM_POLICY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_QM_POLICY_PENDING_DELETION))
  { ":ERROR_IPSEC_QM_POLICY_PENDING_DELETION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_EXISTS))
  { ":ERROR_IPSEC_TRANSPORT_FILTER_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND))
  { ":ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION))
  { ":ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_EXISTS))
  { ":ERROR_IPSEC_TUNNEL_FILTER_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND))
  { ":ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION))
  { ":ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IRQ_BUSY))
  { ":ERROR_IRQ_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOINED))
  { ":ERROR_IS_JOINED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOIN_PATH))
  { ":ERROR_IS_JOIN_PATH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_JOIN_TARGET))
  { ":ERROR_IS_JOIN_TARGET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBSTED))
  { ":ERROR_IS_SUBSTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBST_PATH))
  { ":ERROR_IS_SUBST_PATH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_IS_SUBST_TARGET))
  { ":ERROR_IS_SUBST_TARGET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ITERATED_DATA_EXCEEDS_64k))
  { ":ERROR_ITERATED_DATA_EXCEEDS_64K" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOIN_TO_JOIN))
  { ":ERROR_JOIN_TO_JOIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOIN_TO_SUBST))
  { ":ERROR_JOIN_TO_SUBST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_DELETE_IN_PROGRESS))
  { ":ERROR_JOURNAL_DELETE_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_ENTRY_DELETED))
  { ":ERROR_JOURNAL_ENTRY_DELETED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_HOOK_SET))
  { ":ERROR_JOURNAL_HOOK_SET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_JOURNAL_NOT_ACTIVE))
  { ":ERROR_JOURNAL_NOT_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KEY_DELETED))
  { ":ERROR_KEY_DELETED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KEY_HAS_CHILDREN))
  { ":ERROR_KEY_HAS_CHILDREN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_KM_DRIVER_BLOCKED))
  { ":ERROR_KM_DRIVER_BLOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LABEL_TOO_LONG))
  { ":ERROR_LABEL_TOO_LONG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LAST_ADMIN))
  { ":ERROR_LAST_ADMIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LB_WITHOUT_TABSTOPS))
  { ":ERROR_LB_WITHOUT_TABSTOPS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LIBRARY_FULL))
  { ":ERROR_LIBRARY_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LIBRARY_OFFLINE))
  { ":ERROR_LIBRARY_OFFLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LICENSE_QUOTA_EXCEEDED))
  { ":ERROR_LICENSE_QUOTA_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LISTBOX_ID_NOT_FOUND))
  { ":ERROR_LISTBOX_ID_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LM_CROSS_ENCRYPTION_REQUIRED))
  { ":ERROR_LM_CROSS_ENCRYPTION_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCAL_USER_SESSION_KEY))
  { ":ERROR_LOCAL_USER_SESSION_KEY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCKED))
  { ":ERROR_LOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCK_FAILED))
  { ":ERROR_LOCK_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOCK_VIOLATION))
  { ":ERROR_LOCK_VIOLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGIN_TIME_RESTRICTION))
  { ":ERROR_LOGIN_TIME_RESTRICTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGIN_WKSTA_RESTRICTION))
  { ":ERROR_LOGIN_WKSTA_RESTRICTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_FAILURE))
  { ":ERROR_LOGON_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_NOT_GRANTED))
  { ":ERROR_LOGON_NOT_GRANTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_SESSION_COLLISION))
  { ":ERROR_LOGON_SESSION_COLLISION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_SESSION_EXISTS))
  { ":ERROR_LOGON_SESSION_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOGON_TYPE_NOT_GRANTED))
  { ":ERROR_LOGON_TYPE_NOT_GRANTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LOG_FILE_FULL))
  { ":ERROR_LOG_FILE_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_LUIDS_EXHAUSTED))
  { ":ERROR_LUIDS_EXHAUSTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MACHINE_LOCKED))
  { ":ERROR_MACHINE_LOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAGAZINE_NOT_PRESENT))
  { ":ERROR_MAGAZINE_NOT_PRESENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAPPED_ALIGNMENT))
  { ":ERROR_MAPPED_ALIGNMENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MAX_THRDS_REACHED))
  { ":ERROR_MAX_THRDS_REACHED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_CHANGED))
  { ":ERROR_MEDIA_CHANGED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_INCOMPATIBLE))
  { ":ERROR_MEDIA_INCOMPATIBLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_NOT_AVAILABLE))
  { ":ERROR_MEDIA_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_OFFLINE))
  { ":ERROR_MEDIA_OFFLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIA_UNAVAILABLE))
  { ":ERROR_MEDIA_UNAVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEDIUM_NOT_ACCESSIBLE))
  { ":ERROR_MEDIUM_NOT_ACCESSIBLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBERS_PRIMARY_GROUP))
  { ":ERROR_MEMBERS_PRIMARY_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_IN_ALIAS))
  { ":ERROR_MEMBER_IN_ALIAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_IN_GROUP))
  { ":ERROR_MEMBER_IN_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_NOT_IN_ALIAS))
  { ":ERROR_MEMBER_NOT_IN_ALIAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MEMBER_NOT_IN_GROUP))
  { ":ERROR_MEMBER_NOT_IN_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MENU_ITEM_NOT_FOUND))
  { ":ERROR_MENU_ITEM_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MESSAGE_EXCEEDS_MAX_SIZE))
  { ":ERROR_MESSAGE_EXCEEDS_MAX_SIZE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MESSAGE_SYNC_ONLY))
  { ":ERROR_MESSAGE_SYNC_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_METAFILE_NOT_SUPPORTED))
  { ":ERROR_METAFILE_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_META_EXPANSION_TOO_LONG))
  { ":ERROR_META_EXPANSION_TOO_LONG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MOD_NOT_FOUND))
  { ":ERROR_MOD_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MORE_DATA))
  { ":ERROR_MORE_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MORE_WRITES))
  { ":ERROR_MORE_WRITES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MR_MID_NOT_FOUND))
  { ":ERROR_MR_MID_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_MUTUAL_AUTH_FAILED))
  { ":ERROR_MUTUAL_AUTH_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NEGATIVE_SEEK))
  { ":ERROR_NEGATIVE_SEEK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NESTING_NOT_ALLOWED))
  { ":ERROR_NESTING_NOT_ALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETLOGON_NOT_STARTED))
  { ":ERROR_NETLOGON_NOT_STARTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETNAME_DELETED))
  { ":ERROR_NETNAME_DELETED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_ACCESS_DENIED))
  { ":ERROR_NETWORK_ACCESS_DENIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_BUSY))
  { ":ERROR_NETWORK_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_NOT_AVAILABLE))
  { ":ERROR_NETWORK_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NETWORK_UNREACHABLE))
  { ":ERROR_NETWORK_UNREACHABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NET_WRITE_FAULT))
  { ":ERROR_NET_WRITE_FAULT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOACCESS))
  { ":ERROR_NOACCESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_CANNOT_BE_CLUSTERED))
  { ":ERROR_NODE_CANNOT_BE_CLUSTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_CANT_HOST_RESOURCE))
  { ":ERROR_NODE_CANT_HOST_RESOURCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NODE_NOT_AVAILABLE))
  { ":ERROR_NODE_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT))
  { ":ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_SERVER_TRUST_ACCOUNT))
  { ":ERROR_NOLOGON_SERVER_TRUST_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT))
  { ":ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NONE_MAPPED))
  { ":ERROR_NONE_MAPPED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NONPAGED_SYSTEM_RESOURCES))
  { ":ERROR_NONPAGED_SYSTEM_RESOURCES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_ACCOUNT_SID))
  { ":ERROR_NON_ACCOUNT_SID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_DOMAIN_SID))
  { ":ERROR_NON_DOMAIN_SID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NON_MDICHILD_WINDOW))
  { ":ERROR_NON_MDICHILD_WINDOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOTIFY_ENUM_DIR))
  { ":ERROR_NOTIFY_ENUM_DIR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ALL_ASSIGNED))
  { ":ERROR_NOT_ALL_ASSIGNED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_AUTHENTICATED))
  { ":ERROR_NOT_AUTHENTICATED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_A_REPARSE_POINT))
  { ":ERROR_NOT_A_REPARSE_POINT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CHILD_WINDOW))
  { ":ERROR_NOT_CHILD_WINDOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CONNECTED))
  { ":ERROR_NOT_CONNECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_CONTAINER))
  { ":ERROR_NOT_CONTAINER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_DOS_DISK))
  { ":ERROR_NOT_DOS_DISK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_EMPTY))
  { ":ERROR_NOT_EMPTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_MEMORY))
  { ":ERROR_NOT_ENOUGH_MEMORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_QUOTA))
  { ":ERROR_NOT_ENOUGH_QUOTA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_ENOUGH_SERVER_MEMORY))
  { ":ERROR_NOT_ENOUGH_SERVER_MEMORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_EXPORT_FORMAT))
  { ":ERROR_NOT_EXPORT_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_FOUND))
  { ":ERROR_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_JOINED))
  { ":ERROR_NOT_JOINED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOCKED))
  { ":ERROR_NOT_LOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOGGED_ON))
  { ":ERROR_NOT_LOGGED_ON" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_LOGON_PROCESS))
  { ":ERROR_NOT_LOGON_PROCESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_OWNER))
  { ":ERROR_NOT_OWNER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_QUORUM_CAPABLE))
  { ":ERROR_NOT_QUORUM_CAPABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_QUORUM_CLASS))
  { ":ERROR_NOT_QUORUM_CLASS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_READY))
  { ":ERROR_NOT_READY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_REGISTRY_FILE))
  { ":ERROR_NOT_REGISTRY_FILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SAFEBOOT_SERVICE))
  { ":ERROR_NOT_SAFEBOOT_SERVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SAME_DEVICE))
  { ":ERROR_NOT_SAME_DEVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUBSTED))
  { ":ERROR_NOT_SUBSTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED))
  { ":ERROR_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED_ON_SBS))
  { ":ERROR_NOT_SUPPORTED_ON_SBS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER))
  { ":ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_ASSOCIATION))
  { ":ERROR_NO_ASSOCIATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_BROWSER_SERVERS_FOUND))
  { ":ERROR_NO_BROWSER_SERVERS_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_DATA))
  { ":ERROR_NO_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_DATA_DETECTED))
  { ":ERROR_NO_DATA_DETECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_EFS))
  { ":ERROR_NO_EFS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_IMPERSONATION_TOKEN))
  { ":ERROR_NO_IMPERSONATION_TOKEN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_INHERITANCE))
  { ":ERROR_NO_INHERITANCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_LOGON_SERVERS))
  { ":ERROR_NO_LOGON_SERVERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_LOG_SPACE))
  { ":ERROR_NO_LOG_SPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MATCH))
  { ":ERROR_NO_MATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MEDIA_IN_DRIVE))
  { ":ERROR_NO_MEDIA_IN_DRIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_DEVICES))
  { ":ERROR_NO_MORE_DEVICES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_FILES))
  { ":ERROR_NO_MORE_FILES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_ITEMS))
  { ":ERROR_NO_MORE_ITEMS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_SEARCH_HANDLES))
  { ":ERROR_NO_MORE_SEARCH_HANDLES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_MORE_USER_HANDLES))
  { ":ERROR_NO_MORE_USER_HANDLES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_NETWORK))
  { ":ERROR_NO_NETWORK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_NET_OR_BAD_PATH))
  { ":ERROR_NO_NET_OR_BAD_PATH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_PROC_SLOTS))
  { ":ERROR_NO_PROC_SLOTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_PROMOTION_ACTIVE))
  { ":ERROR_NO_PROMOTION_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_QUOTAS_FOR_ACCOUNT))
  { ":ERROR_NO_QUOTAS_FOR_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_RECOVERY_POLICY))
  { ":ERROR_NO_RECOVERY_POLICY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_RECOVERY_PROGRAM))
  { ":ERROR_NO_RECOVERY_PROGRAM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SCROLLBARS))
  { ":ERROR_NO_SCROLLBARS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SECURITY_ON_OBJECT))
  { ":ERROR_NO_SECURITY_ON_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SHUTDOWN_IN_PROGRESS))
  { ":ERROR_NO_SHUTDOWN_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SIGNAL_SENT))
  { ":ERROR_NO_SIGNAL_SENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SITENAME))
  { ":ERROR_NO_SITENAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SPOOL_SPACE))
  { ":ERROR_NO_SPOOL_SPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_ALIAS))
  { ":ERROR_NO_SUCH_ALIAS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_DOMAIN))
  { ":ERROR_NO_SUCH_DOMAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_GROUP))
  { ":ERROR_NO_SUCH_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_LOGON_SESSION))
  { ":ERROR_NO_SUCH_LOGON_SESSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_MEMBER))
  { ":ERROR_NO_SUCH_MEMBER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_PACKAGE))
  { ":ERROR_NO_SUCH_PACKAGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_PRIVILEGE))
  { ":ERROR_NO_SUCH_PRIVILEGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_SITE))
  { ":ERROR_NO_SUCH_SITE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUCH_USER))
  { ":ERROR_NO_SUCH_USER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SUPPORTING_DRIVES))
  { ":ERROR_NO_SUPPORTING_DRIVES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SYSTEM_MENU))
  { ":ERROR_NO_SYSTEM_MENU" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_SYSTEM_RESOURCES))
  { ":ERROR_NO_SYSTEM_RESOURCES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TOKEN))
  { ":ERROR_NO_TOKEN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRACKING_SERVICE))
  { ":ERROR_NO_TRACKING_SERVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRUST_LSA_SECRET))
  { ":ERROR_NO_TRUST_LSA_SECRET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_TRUST_SAM_ACCOUNT))
  { ":ERROR_NO_TRUST_SAM_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_UNICODE_TRANSLATION))
  { ":ERROR_NO_UNICODE_TRANSLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_USER_KEYS))
  { ":ERROR_NO_USER_KEYS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_USER_SESSION_KEY))
  { ":ERROR_NO_USER_SESSION_KEY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_VOLUME_ID))
  { ":ERROR_NO_VOLUME_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_VOLUME_LABEL))
  { ":ERROR_NO_VOLUME_LABEL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NO_WILDCARD_CHARACTERS))
  { ":ERROR_NO_WILDCARD_CHARACTERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NT_CROSS_ENCRYPTION_REQUIRED))
  { ":ERROR_NT_CROSS_ENCRYPTION_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_NULL_LM_PASSWORD))
  { ":ERROR_NULL_LM_PASSWORD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_ALREADY_EXISTS))
  { ":ERROR_OBJECT_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_IN_LIST))
  { ":ERROR_OBJECT_IN_LIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OBJECT_NOT_FOUND))
  { ":ERROR_OBJECT_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OLD_WIN_VERSION))
  { ":ERROR_OLD_WIN_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_ONLY_IF_CONNECTED))
  { ":ERROR_ONLY_IF_CONNECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPEN_FAILED))
  { ":ERROR_OPEN_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPEN_FILES))
  { ":ERROR_OPEN_FILES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPERATION_ABORTED))
  { ":ERROR_OPERATION_ABORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OPLOCK_NOT_GRANTED))
  { ":ERROR_OPLOCK_NOT_GRANTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUTOFMEMORY))
  { ":ERROR_OUTOFMEMORY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUT_OF_PAPER))
  { ":ERROR_OUT_OF_PAPER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OUT_OF_STRUCTURES))
  { ":ERROR_OUT_OF_STRUCTURES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_OVERRIDE_NOCHANGES))
  { ":ERROR_OVERRIDE_NOCHANGES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PAGED_SYSTEM_RESOURCES))
  { ":ERROR_PAGED_SYSTEM_RESOURCES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PAGEFILE_QUOTA))
  { ":ERROR_PAGEFILE_QUOTA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARAMETER_QUOTA_EXCEEDED))
  { ":ERROR_PARAMETER_QUOTA_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARTIAL_COPY))
  { ":ERROR_PARTIAL_COPY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PARTITION_FAILURE))
  { ":ERROR_PARTITION_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_EXPIRED))
  { ":ERROR_PASSWORD_EXPIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_MUST_CHANGE))
  { ":ERROR_PASSWORD_MUST_CHANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PASSWORD_RESTRICTION))
  { ":ERROR_PASSWORD_RESTRICTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_INVALID))
  { ":ERROR_PATCH_PACKAGE_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_OPEN_FAILED))
  { ":ERROR_PATCH_PACKAGE_OPEN_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_REJECTED))
  { ":ERROR_PATCH_PACKAGE_REJECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_PACKAGE_UNSUPPORTED))
  { ":ERROR_PATCH_PACKAGE_UNSUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATCH_TARGET_NOT_FOUND))
  { ":ERROR_PATCH_TARGET_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATH_BUSY))
  { ":ERROR_PATH_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PATH_NOT_FOUND))
  { ":ERROR_PATH_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PER_USER_TRUST_QUOTA_EXCEEDED))
  { ":ERROR_PER_USER_TRUST_QUOTA_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_BUSY))
  { ":ERROR_PIPE_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_CONNECTED))
  { ":ERROR_PIPE_CONNECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_LISTENING))
  { ":ERROR_PIPE_LISTENING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PIPE_NOT_CONNECTED))
  { ":ERROR_PIPE_NOT_CONNECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PKINIT_FAILURE))
  { ":ERROR_PKINIT_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POINT_NOT_FOUND))
  { ":ERROR_POINT_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POLICY_OBJECT_NOT_FOUND))
  { ":ERROR_POLICY_OBJECT_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POLICY_ONLY_IN_DS))
  { ":ERROR_POLICY_ONLY_IN_DS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POPUP_ALREADY_ACTIVE))
  { ":ERROR_POPUP_ALREADY_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PORT_UNREACHABLE))
  { ":ERROR_PORT_UNREACHABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POSSIBLE_DEADLOCK))
  { ":ERROR_POSSIBLE_DEADLOCK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_POTENTIAL_FILE_FOUND))
  { ":ERROR_POTENTIAL_FILE_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_ALREADY_EXISTS))
  { ":ERROR_PRINTER_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DELETED))
  { ":ERROR_PRINTER_DELETED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_ALREADY_INSTALLED))
  { ":ERROR_PRINTER_DRIVER_ALREADY_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_BLOCKED))
  { ":ERROR_PRINTER_DRIVER_BLOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_IN_USE))
  { ":ERROR_PRINTER_DRIVER_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_DRIVER_WARNED))
  { ":ERROR_PRINTER_DRIVER_WARNED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_HAS_JOBS_QUEUED))
  { ":ERROR_PRINTER_HAS_JOBS_QUEUED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTER_NOT_FOUND))
  { ":ERROR_PRINTER_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINTQ_FULL))
  { ":ERROR_PRINTQ_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_CANCELLED))
  { ":ERROR_PRINT_CANCELLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_MONITOR_ALREADY_INSTALLED))
  { ":ERROR_PRINT_MONITOR_ALREADY_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_MONITOR_IN_USE))
  { ":ERROR_PRINT_MONITOR_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED))
  { ":ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRIVATE_DIALOG_INDEX))
  { ":ERROR_PRIVATE_DIALOG_INDEX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRIVILEGE_NOT_HELD))
  { ":ERROR_PRIVILEGE_NOT_HELD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROCESS_ABORTED))
  { ":ERROR_PROCESS_ABORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROC_NOT_FOUND))
  { ":ERROR_PROC_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRODUCT_UNINSTALLED))
  { ":ERROR_PRODUCT_UNINSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PRODUCT_VERSION))
  { ":ERROR_PRODUCT_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE))
  { ":ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROFILE_NOT_FOUND))
  { ":ERROR_PROFILE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROMOTION_ACTIVE))
  { ":ERROR_PROMOTION_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_PROTOCOL_UNREACHABLE))
  { ":ERROR_PROTOCOL_UNREACHABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUMLOG_OPEN_FAILED))
  { ":ERROR_QUORUMLOG_OPEN_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_DISK_NOT_FOUND))
  { ":ERROR_QUORUM_DISK_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_OWNER_ALIVE))
  { ":ERROR_QUORUM_OWNER_ALIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_RESOURCE))
  { ":ERROR_QUORUM_RESOURCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_QUORUM_RESOURCE_ONLINE_FAILED))
  { ":ERROR_QUORUM_RESOURCE_ONLINE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_READ_FAULT))
  { ":ERROR_READ_FAULT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RECOVERY_FAILURE))
  { ":ERROR_RECOVERY_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REC_NON_EXISTENT))
  { ":ERROR_REC_NON_EXISTENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REDIRECTOR_HAS_OPEN_HANDLES))
  { ":ERROR_REDIRECTOR_HAS_OPEN_HANDLES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REDIR_PAUSED))
  { ":ERROR_REDIR_PAUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_CORRUPT))
  { ":ERROR_REGISTRY_CORRUPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_IO_FAILED))
  { ":ERROR_REGISTRY_IO_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REGISTRY_RECOVERED))
  { ":ERROR_REGISTRY_RECOVERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REG_NAT_CONSUMPTION))
  { ":ERROR_REG_NAT_CONSUMPTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RELOC_CHAIN_XEEDS_SEGLIM))
  { ":ERROR_RELOC_CHAIN_XEEDS_SEGLIM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_SESSION_LIMIT_EXCEEDED))
  { ":ERROR_REMOTE_SESSION_LIMIT_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_STORAGE_MEDIA_ERROR))
  { ":ERROR_REMOTE_STORAGE_MEDIA_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REMOTE_STORAGE_NOT_ACTIVE))
  { ":ERROR_REMOTE_STORAGE_NOT_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REM_NOT_LIST))
  { ":ERROR_REM_NOT_LIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_ATTRIBUTE_CONFLICT))
  { ":ERROR_REPARSE_ATTRIBUTE_CONFLICT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_TAG_INVALID))
  { ":ERROR_REPARSE_TAG_INVALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REPARSE_TAG_MISMATCH))
  { ":ERROR_REPARSE_TAG_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUEST_ABORTED))
  { ":ERROR_REQUEST_ABORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUEST_REFUSED))
  { ":ERROR_REQUEST_REFUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION))
  { ":ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REQ_NOT_ACCEP))
  { ":ERROR_REQ_NOT_ACCEP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_CREATE_FAILED))
  { ":ERROR_RESMON_CREATE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_INVALID_STATE))
  { ":ERROR_RESMON_INVALID_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESMON_ONLINE_FAILED))
  { ":ERROR_RESMON_ONLINE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_DATA_NOT_FOUND))
  { ":ERROR_RESOURCE_DATA_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_DISABLED))
  { ":ERROR_RESOURCE_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_FAILED))
  { ":ERROR_RESOURCE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_LANG_NOT_FOUND))
  { ":ERROR_RESOURCE_LANG_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NAME_NOT_FOUND))
  { ":ERROR_RESOURCE_NAME_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_AVAILABLE))
  { ":ERROR_RESOURCE_NOT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_FOUND))
  { ":ERROR_RESOURCE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_ONLINE))
  { ":ERROR_RESOURCE_NOT_ONLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_NOT_PRESENT))
  { ":ERROR_RESOURCE_NOT_PRESENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_ONLINE))
  { ":ERROR_RESOURCE_ONLINE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_PROPERTIES_STORED))
  { ":ERROR_RESOURCE_PROPERTIES_STORED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_PROPERTY_UNCHANGEABLE))
  { ":ERROR_RESOURCE_PROPERTY_UNCHANGEABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RESOURCE_TYPE_NOT_FOUND))
  { ":ERROR_RESOURCE_TYPE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RETRY))
  { ":ERROR_RETRY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_REVISION_MISMATCH))
  { ":ERROR_REVISION_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RING2SEG_MUST_BE_MOVABLE))
  { ":ERROR_RING2SEG_MUST_BE_MOVABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RING2_STACK_IN_USE))
  { ":ERROR_RING2_STACK_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RMODE_APP))
  { ":ERROR_RMODE_APP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RPL_NOT_ALLOWED))
  { ":ERROR_RPL_NOT_ALLOWED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RXACT_COMMIT_FAILURE))
  { ":ERROR_RXACT_COMMIT_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_RXACT_INVALID_STATE))
  { ":ERROR_RXACT_INVALID_STATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SAME_DRIVE))
  { ":ERROR_SAME_DRIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SAM_INIT_FAILURE))
  { ":ERROR_SAM_INIT_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SCOPE_NOT_FOUND))
  { ":ERROR_SCOPE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SCREEN_ALREADY_LOCKED))
  { ":ERROR_SCREEN_ALREADY_LOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SECRET_TOO_LONG))
  { ":ERROR_SECRET_TOO_LONG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SECTOR_NOT_FOUND))
  { ":ERROR_SECTOR_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEEK))
  { ":ERROR_SEEK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEEK_ON_DEVICE))
  { ":ERROR_SEEK_ON_DEVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_IS_SET))
  { ":ERROR_SEM_IS_SET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_NOT_FOUND))
  { ":ERROR_SEM_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_OWNER_DIED))
  { ":ERROR_SEM_OWNER_DIED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_TIMEOUT))
  { ":ERROR_SEM_TIMEOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SEM_USER_LIMIT))
  { ":ERROR_SEM_USER_LIMIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERIAL_NO_DEVICE))
  { ":ERROR_SERIAL_NO_DEVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_DISABLED))
  { ":ERROR_SERVER_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_HAS_OPEN_HANDLES))
  { ":ERROR_SERVER_HAS_OPEN_HANDLES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_NOT_DISABLED))
  { ":ERROR_SERVER_NOT_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVER_SHUTDOWN_IN_PROGRESS))
  { ":ERROR_SERVER_SHUTDOWN_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_ALREADY_RUNNING))
  { ":ERROR_SERVICE_ALREADY_RUNNING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_CANNOT_ACCEPT_CTRL))
  { ":ERROR_SERVICE_CANNOT_ACCEPT_CTRL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DATABASE_LOCKED))
  { ":ERROR_SERVICE_DATABASE_LOCKED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DEPENDENCY_DELETED))
  { ":ERROR_SERVICE_DEPENDENCY_DELETED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DEPENDENCY_FAIL))
  { ":ERROR_SERVICE_DEPENDENCY_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DISABLED))
  { ":ERROR_SERVICE_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_DOES_NOT_EXIST))
  { ":ERROR_SERVICE_DOES_NOT_EXIST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_EXISTS))
  { ":ERROR_SERVICE_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_LOGON_FAILED))
  { ":ERROR_SERVICE_LOGON_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_MARKED_FOR_DELETE))
  { ":ERROR_SERVICE_MARKED_FOR_DELETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NEVER_STARTED))
  { ":ERROR_SERVICE_NEVER_STARTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_ACTIVE))
  { ":ERROR_SERVICE_NOT_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_FOUND))
  { ":ERROR_SERVICE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NOT_IN_EXE))
  { ":ERROR_SERVICE_NOT_IN_EXE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_NO_THREAD))
  { ":ERROR_SERVICE_NO_THREAD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_REQUEST_TIMEOUT))
  { ":ERROR_SERVICE_REQUEST_TIMEOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_SPECIFIC_ERROR))
  { ":ERROR_SERVICE_SPECIFIC_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SERVICE_START_HANG))
  { ":ERROR_SERVICE_START_HANG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SESSION_CREDENTIAL_CONFLICT))
  { ":ERROR_SESSION_CREDENTIAL_CONFLICT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SETCOUNT_ON_BAD_LB))
  { ":ERROR_SETCOUNT_ON_BAD_LB" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SETMARK_DETECTED))
  { ":ERROR_SETMARK_DETECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_NOT_FOUND))
  { ":ERROR_SET_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_POWER_STATE_FAILED))
  { ":ERROR_SET_POWER_STATE_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SET_POWER_STATE_VETOED))
  { ":ERROR_SET_POWER_STATE_VETOED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARED_POLICY))
  { ":ERROR_SHARED_POLICY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_BUFFER_EXCEEDED))
  { ":ERROR_SHARING_BUFFER_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_PAUSED))
  { ":ERROR_SHARING_PAUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHARING_VIOLATION))
  { ":ERROR_SHARING_VIOLATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHUTDOWN_CLUSTER))
  { ":ERROR_SHUTDOWN_CLUSTER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SHUTDOWN_IN_PROGRESS))
  { ":ERROR_SHUTDOWN_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SIGNAL_PENDING))
  { ":ERROR_SIGNAL_PENDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SIGNAL_REFUSED))
  { ":ERROR_SIGNAL_REFUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SINGLE_INSTANCE_APP))
  { ":ERROR_SINGLE_INSTANCE_APP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SMARTCARD_SUBSYSTEM_FAILURE))
  { ":ERROR_SMARTCARD_SUBSYSTEM_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SOME_NOT_MAPPED))
  { ":ERROR_SOME_NOT_MAPPED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SOURCE_ELEMENT_EMPTY))
  { ":ERROR_SOURCE_ELEMENT_EMPTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_ACCOUNT))
  { ":ERROR_SPECIAL_ACCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_GROUP))
  { ":ERROR_SPECIAL_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPECIAL_USER))
  { ":ERROR_SPECIAL_USER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPL_NO_ADDJOB))
  { ":ERROR_SPL_NO_ADDJOB" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPL_NO_STARTDOC))
  { ":ERROR_SPL_NO_STARTDOC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SPOOL_FILE_NOT_FOUND))
  { ":ERROR_SPOOL_FILE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STACK_BUFFER_OVERRUN))
  { ":ERROR_STACK_BUFFER_OVERRUN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STACK_OVERFLOW))
  { ":ERROR_STACK_OVERFLOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_STATIC_INIT))
  { ":ERROR_STATIC_INIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUBST_TO_JOIN))
  { ":ERROR_SUBST_TO_JOIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUBST_TO_SUBST))
  { ":ERROR_SUBST_TO_SUBST" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_REBOOT_INITIATED))
  { ":ERROR_SUCCESS_REBOOT_INITIATED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_REBOOT_REQUIRED))
  { ":ERROR_SUCCESS_REBOOT_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SUCCESS_RESTART_REQUIRED))
  { ":ERROR_SUCCESS_RESTART_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SWAPERROR))
  { ":ERROR_SWAPERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ACTIVATION_CONTEXT_DISABLED))
  { ":ERROR_SXS_ACTIVATION_CONTEXT_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ASSEMBLY_NOT_FOUND))
  { ":ERROR_SXS_ASSEMBLY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_CANT_GEN_ACTCTX))
  { ":ERROR_SXS_CANT_GEN_ACTCTX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_ASSEMBLY_NAME))
  { ":ERROR_SXS_DUPLICATE_ASSEMBLY_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_CLSID))
  { ":ERROR_SXS_DUPLICATE_CLSID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_DLL_NAME))
  { ":ERROR_SXS_DUPLICATE_DLL_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_IID))
  { ":ERROR_SXS_DUPLICATE_IID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_PROGID))
  { ":ERROR_SXS_DUPLICATE_PROGID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_TLBID))
  { ":ERROR_SXS_DUPLICATE_TLBID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME))
  { ":ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_FILE_HASH_MISMATCH))
  { ":ERROR_SXS_FILE_HASH_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ACTCTXDATA_FORMAT))
  { ":ERROR_SXS_INVALID_ACTCTXDATA_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE))
  { ":ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME))
  { ":ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_INVALID_XML_NAMESPACE_URI))
  { ":ERROR_SXS_INVALID_XML_NAMESPACE_URI" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_KEY_NOT_FOUND))
  { ":ERROR_SXS_KEY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED))
  { ":ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_FORMAT_ERROR))
  { ":ERROR_SXS_MANIFEST_FORMAT_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE))
  { ":ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE))
  { ":ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MANIFEST_PARSE_ERROR))
  { ":ERROR_SXS_MANIFEST_PARSE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE))
  { ":ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_POLICY_PARSE_ERROR))
  { ":ERROR_SXS_POLICY_PARSE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT))
  { ":ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET))
  { ":ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING))
  { ":ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_CATALOG_NOT_VALID))
  { ":ERROR_SXS_PROTECTION_CATALOG_NOT_VALID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT))
  { ":ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_PROTECTION_RECOVERY_FAILED))
  { ":ERROR_SXS_PROTECTION_RECOVERY_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED))
  { ":ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_SECTION_NOT_FOUND))
  { ":ERROR_SXS_SECTION_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_THREAD_QUERIES_DISABLED))
  { ":ERROR_SXS_THREAD_QUERIES_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNKNOWN_ENCODING))
  { ":ERROR_SXS_UNKNOWN_ENCODING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNKNOWN_ENCODING_GROUP))
  { ":ERROR_SXS_UNKNOWN_ENCODING_GROUP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_UNTRANSLATABLE_HRESULT))
  { ":ERROR_SXS_UNTRANSLATABLE_HRESULT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_VERSION_CONFLICT))
  { ":ERROR_SXS_VERSION_CONFLICT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_WRONG_SECTION_TYPE))
  { ":ERROR_SXS_WRONG_SECTION_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADCHARDATA))
  { ":ERROR_SXS_XML_E_BADCHARDATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADCHARINSTRING))
  { ":ERROR_SXS_XML_E_BADCHARINSTRING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADNAMECHAR))
  { ":ERROR_SXS_XML_E_BADNAMECHAR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADPEREFINSUBSET))
  { ":ERROR_SXS_XML_E_BADPEREFINSUBSET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADSTARTNAMECHAR))
  { ":ERROR_SXS_XML_E_BADSTARTNAMECHAR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADXMLCASE))
  { ":ERROR_SXS_XML_E_BADXMLCASE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_BADXMLDECL))
  { ":ERROR_SXS_XML_E_BADXMLDECL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_COMMENTSYNTAX))
  { ":ERROR_SXS_XML_E_COMMENTSYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_DUPLICATEATTRIBUTE))
  { ":ERROR_SXS_XML_E_DUPLICATEATTRIBUTE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE))
  { ":ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_EXPECTINGTAGEND))
  { ":ERROR_SXS_XML_E_EXPECTINGTAGEND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INCOMPLETE_ENCODING))
  { ":ERROR_SXS_XML_E_INCOMPLETE_ENCODING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INTERNALERROR))
  { ":ERROR_SXS_XML_E_INTERNALERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDATROOTLEVEL))
  { ":ERROR_SXS_XML_E_INVALIDATROOTLEVEL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDENCODING))
  { ":ERROR_SXS_XML_E_INVALIDENCODING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALIDSWITCH))
  { ":ERROR_SXS_XML_E_INVALIDSWITCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_DECIMAL))
  { ":ERROR_SXS_XML_E_INVALID_DECIMAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_HEXIDECIMAL))
  { ":ERROR_SXS_XML_E_INVALID_HEXIDECIMAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_STANDALONE))
  { ":ERROR_SXS_XML_E_INVALID_STANDALONE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_UNICODE))
  { ":ERROR_SXS_XML_E_INVALID_UNICODE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_INVALID_VERSION))
  { ":ERROR_SXS_XML_E_INVALID_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGEQUALS))
  { ":ERROR_SXS_XML_E_MISSINGEQUALS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGQUOTE))
  { ":ERROR_SXS_XML_E_MISSINGQUOTE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGROOT))
  { ":ERROR_SXS_XML_E_MISSINGROOT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGSEMICOLON))
  { ":ERROR_SXS_XML_E_MISSINGSEMICOLON" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSINGWHITESPACE))
  { ":ERROR_SXS_XML_E_MISSINGWHITESPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MISSING_PAREN))
  { ":ERROR_SXS_XML_E_MISSING_PAREN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MULTIPLEROOTS))
  { ":ERROR_SXS_XML_E_MULTIPLEROOTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_MULTIPLE_COLONS))
  { ":ERROR_SXS_XML_E_MULTIPLE_COLONS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_RESERVEDNAMESPACE))
  { ":ERROR_SXS_XML_E_RESERVEDNAMESPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNBALANCEDPAREN))
  { ":ERROR_SXS_XML_E_UNBALANCEDPAREN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDCDATA))
  { ":ERROR_SXS_XML_E_UNCLOSEDCDATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDCOMMENT))
  { ":ERROR_SXS_XML_E_UNCLOSEDCOMMENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDDECL))
  { ":ERROR_SXS_XML_E_UNCLOSEDDECL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDENDTAG))
  { ":ERROR_SXS_XML_E_UNCLOSEDENDTAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDSTARTTAG))
  { ":ERROR_SXS_XML_E_UNCLOSEDSTARTTAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDSTRING))
  { ":ERROR_SXS_XML_E_UNCLOSEDSTRING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNCLOSEDTAG))
  { ":ERROR_SXS_XML_E_UNCLOSEDTAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTEDENDTAG))
  { ":ERROR_SXS_XML_E_UNEXPECTEDENDTAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTEDEOF))
  { ":ERROR_SXS_XML_E_UNEXPECTEDEOF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTED_STANDALONE))
  { ":ERROR_SXS_XML_E_UNEXPECTED_STANDALONE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE))
  { ":ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK))
  { ":ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SXS_XML_E_XMLDECLSYNTAX))
  { ":ERROR_SXS_XML_E_XMLDECLSYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED))
  { ":ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_SYSTEM_TRACE))
  { ":ERROR_SYSTEM_TRACE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TAG_NOT_FOUND))
  { ":ERROR_TAG_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TAG_NOT_PRESENT))
  { ":ERROR_TAG_NOT_PRESENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_THREAD_1_INACTIVE))
  { ":ERROR_THREAD_1_INACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TIMEOUT))
  { ":ERROR_TIMEOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TIME_SKEW))
  { ":ERROR_TIME_SKEW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TLW_WITH_WSCHILD))
  { ":ERROR_TLW_WITH_WSCHILD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOKEN_ALREADY_IN_USE))
  { ":ERROR_TOKEN_ALREADY_IN_USE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_CMDS))
  { ":ERROR_TOO_MANY_CMDS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_CONTEXT_IDS))
  { ":ERROR_TOO_MANY_CONTEXT_IDS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_LINKS))
  { ":ERROR_TOO_MANY_LINKS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_LUIDS_REQUESTED))
  { ":ERROR_TOO_MANY_LUIDS_REQUESTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_MODULES))
  { ":ERROR_TOO_MANY_MODULES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_MUXWAITERS))
  { ":ERROR_TOO_MANY_MUXWAITERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_NAMES))
  { ":ERROR_TOO_MANY_NAMES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_OPEN_FILES))
  { ":ERROR_TOO_MANY_OPEN_FILES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_POSTS))
  { ":ERROR_TOO_MANY_POSTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SECRETS))
  { ":ERROR_TOO_MANY_SECRETS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SEMAPHORES))
  { ":ERROR_TOO_MANY_SEMAPHORES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SEM_REQUESTS))
  { ":ERROR_TOO_MANY_SEM_REQUESTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SESS))
  { ":ERROR_TOO_MANY_SESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_SIDS))
  { ":ERROR_TOO_MANY_SIDS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TOO_MANY_TCBS))
  { ":ERROR_TOO_MANY_TCBS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRANSFORM_NOT_SUPPORTED))
  { ":ERROR_TRANSFORM_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRANSPORT_FULL))
  { ":ERROR_TRANSPORT_FULL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUSTED_DOMAIN_FAILURE))
  { ":ERROR_TRUSTED_DOMAIN_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUSTED_RELATIONSHIP_FAILURE))
  { ":ERROR_TRUSTED_RELATIONSHIP_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_TRUST_FAILURE))
  { ":ERROR_TRUST_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_CLEAN))
  { ":ERROR_UNABLE_TO_CLEAN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA))
  { ":ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_DRIVE))
  { ":ERROR_UNABLE_TO_INVENTORY_DRIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_SLOT))
  { ":ERROR_UNABLE_TO_INVENTORY_SLOT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_INVENTORY_TRANSPORT))
  { ":ERROR_UNABLE_TO_INVENTORY_TRANSPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_LOAD_MEDIUM))
  { ":ERROR_UNABLE_TO_LOAD_MEDIUM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_LOCK_MEDIA))
  { ":ERROR_UNABLE_TO_LOCK_MEDIA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT))
  { ":ERROR_UNABLE_TO_MOVE_REPLACEMENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT_2))
  { ":ERROR_UNABLE_TO_MOVE_REPLACEMENT_2" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_REMOVE_REPLACED))
  { ":ERROR_UNABLE_TO_REMOVE_REPLACED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNABLE_TO_UNLOAD_MEDIA))
  { ":ERROR_UNABLE_TO_UNLOAD_MEDIA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNEXPECTED_OMID))
  { ":ERROR_UNEXPECTED_OMID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNEXP_NET_ERR))
  { ":ERROR_UNEXP_NET_ERR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_COMPONENT))
  { ":ERROR_UNKNOWN_COMPONENT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_FEATURE))
  { ":ERROR_UNKNOWN_FEATURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PORT))
  { ":ERROR_UNKNOWN_PORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINTER_DRIVER))
  { ":ERROR_UNKNOWN_PRINTER_DRIVER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINTPROCESSOR))
  { ":ERROR_UNKNOWN_PRINTPROCESSOR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRINT_MONITOR))
  { ":ERROR_UNKNOWN_PRINT_MONITOR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PRODUCT))
  { ":ERROR_UNKNOWN_PRODUCT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_PROPERTY))
  { ":ERROR_UNKNOWN_PROPERTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNKNOWN_REVISION))
  { ":ERROR_UNKNOWN_REVISION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNRECOGNIZED_MEDIA))
  { ":ERROR_UNRECOGNIZED_MEDIA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNRECOGNIZED_VOLUME))
  { ":ERROR_UNRECOGNIZED_VOLUME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_UNSUPPORTED_TYPE))
  { ":ERROR_UNSUPPORTED_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED))
  { ":ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_EXISTS))
  { ":ERROR_USER_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_USER_MAPPED_FILE))
  { ":ERROR_USER_MAPPED_FILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VC_DISCONNECTED))
  { ":ERROR_VC_DISCONNECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_CONTAINS_SYS_FILES))
  { ":ERROR_VOLUME_CONTAINS_SYS_FILES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_NOT_SIS_ENABLED))
  { ":ERROR_VOLUME_NOT_SIS_ENABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_VOLUME_NOT_SUPPORT_EFS))
  { ":ERROR_VOLUME_NOT_SUPPORT_EFS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WAIT_NO_CHILDREN))
  { ":ERROR_WAIT_NO_CHILDREN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_NOT_COMBOBOX))
  { ":ERROR_WINDOW_NOT_COMBOBOX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_NOT_DIALOG))
  { ":ERROR_WINDOW_NOT_DIALOG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINDOW_OF_OTHER_THREAD))
  { ":ERROR_WINDOW_OF_OTHER_THREAD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WINS_INTERNAL))
  { ":ERROR_WINS_INTERNAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ALREADY_DISABLED))
  { ":ERROR_WMI_ALREADY_DISABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ALREADY_ENABLED))
  { ":ERROR_WMI_ALREADY_ENABLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_DP_FAILED))
  { ":ERROR_WMI_DP_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_DP_NOT_FOUND))
  { ":ERROR_WMI_DP_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_GUID_DISCONNECTED))
  { ":ERROR_WMI_GUID_DISCONNECTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_GUID_NOT_FOUND))
  { ":ERROR_WMI_GUID_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INSTANCE_NOT_FOUND))
  { ":ERROR_WMI_INSTANCE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INVALID_MOF))
  { ":ERROR_WMI_INVALID_MOF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_INVALID_REGINFO))
  { ":ERROR_WMI_INVALID_REGINFO" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_ITEMID_NOT_FOUND))
  { ":ERROR_WMI_ITEMID_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_READ_ONLY))
  { ":ERROR_WMI_READ_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_SERVER_UNAVAILABLE))
  { ":ERROR_WMI_SERVER_UNAVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_SET_FAILURE))
  { ":ERROR_WMI_SET_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_TRY_AGAIN))
  { ":ERROR_WMI_TRY_AGAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WMI_UNRESOLVED_INSTANCE_REF))
  { ":ERROR_WMI_UNRESOLVED_INSTANCE_REF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WORKING_SET_QUOTA))
  { ":ERROR_WORKING_SET_QUOTA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRITE_FAULT))
  { ":ERROR_WRITE_FAULT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRITE_PROTECT))
  { ":ERROR_WRITE_PROTECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_DISK))
  { ":ERROR_WRONG_DISK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_EFS))
  { ":ERROR_WRONG_EFS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_PASSWORD))
  { ":ERROR_WRONG_PASSWORD" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(ERROR_WRONG_TARGET_NAME))
  { ":ERROR_WRONG_TARGET_NAME" },
#endif
#if defined(ESHUTDOWN)
  { ":ESHUTDOWN" },
#endif
#if defined(ESOCKTNOSUPPORT)
  { ":ESOCKTNOSUPPORT" },
#endif
#if defined(ESPIPE)
  { ":ESPIPE" },
#endif
#if defined(ESRCH)
  { ":ESRCH" },
#endif
#if defined(ESRMNT)
  { ":ESRMNT" },
#endif
#if defined(ESTALE)
  { ":ESTALE" },
#endif
#if defined(ESTRPIPE)
  { ":ESTRPIPE" },
#endif
#if defined(ETIME)
  { ":ETIME" },
#endif
#if defined(ETIMEDOUT)
  { ":ETIMEDOUT" },
#endif
#if defined(ETOOMANYREFS)
  { ":ETOOMANYREFS" },
#endif
#if defined(ETXTBSY)
  { ":ETXTBSY" },
#endif
#if defined(EUCLEAN)
  { ":EUCLEAN" },
#endif
#if defined(EUNATCH)
  { ":EUNATCH" },
#endif
#if defined(EUSERS)
  { ":EUSERS" },
#endif
#if defined(EVERSION)
  { ":EVERSION" },
#endif
#if defined(EWOULDBLOCK)
  { ":EWOULDBLOCK" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_EXCL))
  { ":EXCL" },
#endif
#if defined(EXDEV)
  { ":EXDEV" },
#endif
#if defined(EXFULL)
  { ":EXFULL" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { ":EXITED" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_EXPR_NEST_MAX))
  { ":EXPR-NEST-MAX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":EXT" },
#endif
  { ":EXTERNAL-FORMAT" },
#if (defined(HAVE_NFTW)) && (defined(FTW_F))
  { ":F" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  { ":FACILITY" },
#endif
#if defined(S_IFBLK)
  { ":FBLK" },
#endif
#if defined(S_IFCHR)
  { ":FCHR" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(F_GETFD))
  { ":FD" },
#endif
#if defined(S_IFDIR)
  { ":FDIR" },
#endif
#if defined(HAVE_NFTW)
  { ":FD-LIMIT" },
#endif
#if defined(S_IFIFO)
  { ":FIFO" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_FILESIZEBITS))
  { ":FILESIZEBITS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":FILETIME" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_NAMED_STREAMS))
  { ":FILE_NAMED_STREAMS" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_READ_ONLY_VOLUME))
  { ":FILE_READ_ONLY_VOLUME" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_ENCRYPTION))
  { ":FILE_SUPPORTS_ENCRYPTION" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_OBJECT_IDS))
  { ":FILE_SUPPORTS_OBJECT_IDS" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_REPARSE_POINTS))
  { ":FILE_SUPPORTS_REPARSE_POINTS" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_SUPPORTS_SPARSE_FILES))
  { ":FILE_SUPPORTS_SPARSE_FILES" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FILE_VOLUME_QUOTAS))
  { ":FILE_VOLUME_QUOTAS" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(F_GETFL))
  { ":FL" },
#endif
#if defined(S_IFREG)
  { ":FREG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_AUTHENTICATION))
  { ":FRS_ERR_AUTHENTICATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_CHILD_TO_PARENT_COMM))
  { ":FRS_ERR_CHILD_TO_PARENT_COMM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INSUFFICIENT_PRIV))
  { ":FRS_ERR_INSUFFICIENT_PRIV" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INTERNAL))
  { ":FRS_ERR_INTERNAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INTERNAL_API))
  { ":FRS_ERR_INTERNAL_API" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INVALID_API_SEQUENCE))
  { ":FRS_ERR_INVALID_API_SEQUENCE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_INVALID_SERVICE_PARAMETER))
  { ":FRS_ERR_INVALID_SERVICE_PARAMETER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_AUTHENTICATION))
  { ":FRS_ERR_PARENT_AUTHENTICATION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_INSUFFICIENT_PRIV))
  { ":FRS_ERR_PARENT_INSUFFICIENT_PRIV" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_PARENT_TO_CHILD_COMM))
  { ":FRS_ERR_PARENT_TO_CHILD_COMM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SERVICE_COMM))
  { ":FRS_ERR_SERVICE_COMM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_STARTING_SERVICE))
  { ":FRS_ERR_STARTING_SERVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_STOPPING_SERVICE))
  { ":FRS_ERR_STOPPING_SERVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_DEMOTE))
  { ":FRS_ERR_SYSVOL_DEMOTE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_IS_BUSY))
  { ":FRS_ERR_SYSVOL_IS_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_POPULATE))
  { ":FRS_ERR_SYSVOL_POPULATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FRS_ERR_SYSVOL_POPULATE_TIMEOUT))
  { ":FRS_ERR_SYSVOL_POPULATE_TIMEOUT" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_FSIZE))
  { ":FSIZE" },
#endif
#if defined(S_IFSOCK)
  { ":FSOCK" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_FSYNC))
  { ":FSYNC" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_CASE_IS_PRESERVED))
  { ":FS_CASE_IS_PRESERVED" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_CASE_SENSITIVE))
  { ":FS_CASE_SENSITIVE" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_FILE_COMPRESSION))
  { ":FS_FILE_COMPRESSION" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_FILE_ENCRYPTION))
  { ":FS_FILE_ENCRYPTION" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_PERSISTENT_ACLS))
  { ":FS_PERSISTENT_ACLS" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_UNICODE_STORED_ON_DISK))
  { ":FS_UNICODE_STORED_ON_DISK" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(FS_VOL_IS_COMPRESSED))
  { ":FS_VOL_IS_COMPRESSED" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_FTP))
  { ":FTP" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_GETGR_R_SIZE_MAX))
  { ":GETGR-R-SIZE-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_GETPW_R_SIZE_MAX))
  { ":GETPW-R-SIZE-MAX" },
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  { ":GID" },
#endif
  { ":HARDLINK" },
  { ":HARDLINK-OR-COPY" },
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_HIDDEN))
  { ":HIDDEN" },
#endif
#if ((defined(WIN32_NATIVE)) && (defined(HIGH_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  { ":HIGH" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_HOST_NAME_MAX))
  { ":HOST-NAME-MAX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":HOT-KEY" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":I1" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":I2" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":I4" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":I8" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_IA32_ON_WIN64))
  { ":IA32_ON_WIN64" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_IA64))
  { ":IA64" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":ICON" },
#endif
#if ((defined(WIN32_NATIVE)) && (defined(IDLE_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  { ":IDLE" },
#endif
  { ":IF-DOES-NOT-EXIST" },
  { ":IF-EXISTS" },
#if (defined(HAVE_SYSLOG)) && (defined(LOG_INFO))
  { ":INFO" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":INITID" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(INIT_PROCESS))
  { ":INIT-PROCESS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":INT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_INTEL))
  { ":INTEL" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_IOV_MAX))
  { ":IOV-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_IPV6))
  { ":IPV6" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_JOB_CONTROL))
  { ":JOB-CONTROL" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_KERN))
  { ":KERN" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":KEYWORDS" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_LARGEFILE))
  { ":LARGEFILE" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":LASTAUTHOR" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":LASTPRINTED" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":LASTSAVE-DTM" },
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  { ":LENGTH" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_LINE_MAX))
  { ":LINE-MAX" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_LINK_MAX))
  { ":LINK-MAX" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL0))
  { ":LOCAL0" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL1))
  { ":LOCAL1" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL2))
  { ":LOCAL2" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL3))
  { ":LOCAL3" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL4))
  { ":LOCAL4" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL5))
  { ":LOCAL5" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL6))
  { ":LOCAL6" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LOCAL7))
  { ":LOCAL7" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":LOCALE" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_LOCKS))
  { ":LOCKS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_LOGIN_NAME_MAX))
  { ":LOGIN-NAME-MAX" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(LOGIN_PROCESS))
  { ":LOGIN-PROCESS" },
#endif
#if ((defined(WIN32_NATIVE)) && (defined(LOW_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  { ":LOW" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_LPR))
  { ":LPR" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":LPSTR" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":LPWSTR" },
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_LWP))
  { ":LWP" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_MAIL))
  { ":MAIL" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MAPPED_FILES))
  { ":MAPPED-FILES" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":MAX" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_MAX_CANON))
  { ":MAX-CANON" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_MAX_INPUT))
  { ":MAX-INPUT" },
#endif
#if ((defined(HAVE_SYSCONF)) && (defined(_SC_MEMLOCK))) || ((defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_MEMLOCK)))
  { ":MEMLOCK" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MEMLOCK_RANGE))
  { ":MEMLOCK-RANGE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MEMORY_PROTECTION))
  { ":MEMORY-PROTECTION" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MESSAGE_PASSING))
  { ":MESSAGE-PASSING" },
#endif
  { ":METHOD" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":MIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_MIPS))
  { ":MIPS" },
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  { ":MODE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MONOTONIC_CLOCK))
  { ":MONOTONIC-CLOCK" },
#endif
#if defined(HAVE_NFTW)
  { ":MOUNT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MQ_OPEN_MAX))
  { ":MQ-OPEN-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_MQ_PRIO_MAX))
  { ":MQ-PRIO-MAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_MSIL))
  { ":MSIL" },
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  { ":MTIME" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_NAME_MAX))
  { ":NAME-MAX" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  { ":NDELAY" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_NEWS))
  { ":NEWS" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(NEW_TIME))
  { ":NEW-TIME" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NGROUPS_MAX))
  { ":NGROUPS-MAX" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOCTTY))
  { ":NOCTTY" },
#endif
  { ":NOESCAPE" },
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_NOFILE))
  { ":NOFILE" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOFOLLOW))
  { ":NOFOLLOW" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { ":NOHANG" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NOINHERIT))
  { ":NOINHERIT" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_NONBLOCK))
  { ":NONBLOCK" },
#endif
#if ((defined(WIN32_NATIVE)) && (defined(NORMAL_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE))) || ((defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_NORMAL)))
  { ":NORMAL" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_NOTICE))
  { ":NOTICE" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":NOTIMPLEMENTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_NOT_CONTENT_INDEXED))
  { ":NOT-CONTENT-INDEXED" },
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))) || (defined(HAVE_SYS_RESOURCE_H))
  { ":NOWAIT" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_NO_TRUNC))
  { ":NO-TRUNC" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_NPROC))
  { ":NPROC" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NPROCESSORS_CONF))
  { ":NPROCESSORS-CONF" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_NPROCESSORS_ONLN))
  { ":NPROCESSORS-ONLN" },
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_NS))
  { ":NS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":NT" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":NULL" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  { ":ODELAY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_OFFLINE))
  { ":OFFLINE" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(OLD_TIME))
  { ":OLD-TIME" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_OPEN_MAX))
  { ":OPEN-MAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_OID))
  { ":OR_INVALID_OID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_OXID))
  { ":OR_INVALID_OXID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(OR_INVALID_SET))
  { ":OR_INVALID_SET" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":PAGECOUNT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PAGESIZE))
  { ":PAGESIZE" },
#endif
#if ((defined(HAVE_CONFSTR)) && (defined(_CS_PATH))) || (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN))
  { ":PATH" },
#endif
  { ":PATHNAME" },
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PATH_MAX))
  { ":PATH-MAX" },
#endif
  { ":PERIOD" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":PERSONAL" },
#endif
#if defined(PRIO_PGRP)
  { ":PGRP" },
#endif
#if defined(HAVE_NFTW)
  { ":PHYS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PHYS_PAGES))
  { ":PHYS-PAGES" },
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))) || (defined(HAVE_SYS_RESOURCE_H))
  { ":PID" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PIPE_BUF))
  { ":PIPE-BUF" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_CFLAGS))
  { ":POSIX-V6-ILP32-OFF32-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_LDFLAGS))
  { ":POSIX-V6-ILP32-OFF32-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFF32_LIBS))
  { ":POSIX-V6-ILP32-OFF32-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_CFLAGS))
  { ":POSIX-V6-ILP32-OFFBIG-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS))
  { ":POSIX-V6-ILP32-OFFBIG-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_ILP32_OFFBIG_LIBS))
  { ":POSIX-V6-ILP32-OFFBIG-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_CFLAGS))
  { ":POSIX-V6-LP64-OFF64-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_LDFLAGS))
  { ":POSIX-V6-LP64-OFF64-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LP64_OFF64_LIBS))
  { ":POSIX-V6-LP64-OFF64-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS))
  { ":POSIX-V6-LPBIG-OFFBIG-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS))
  { ":POSIX-V6-LPBIG-OFFBIG-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_LPBIG_OFFBIG_LIBS))
  { ":POSIX-V6-LPBIG-OFFBIG-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V6_WIDTH_RESTRICTED_ENVS))
  { ":POSIX-V6-WIDTH-RESTRICTED-ENVS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_CFLAGS))
  { ":POSIX-V7-ILP32-OFF32-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_LDFLAGS))
  { ":POSIX-V7-ILP32-OFF32-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFF32_LIBS))
  { ":POSIX-V7-ILP32-OFF32-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_CFLAGS))
  { ":POSIX-V7-ILP32-OFFBIG-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS))
  { ":POSIX-V7-ILP32-OFFBIG-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_ILP32_OFFBIG_LIBS))
  { ":POSIX-V7-ILP32-OFFBIG-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_CFLAGS))
  { ":POSIX-V7-LP64-OFF64-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_LDFLAGS))
  { ":POSIX-V7-LP64-OFF64-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LP64_OFF64_LIBS))
  { ":POSIX-V7-LP64-OFF64-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS))
  { ":POSIX-V7-LPBIG-OFFBIG-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS))
  { ":POSIX-V7-LPBIG-OFFBIG-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_LPBIG_OFFBIG_LIBS))
  { ":POSIX-V7-LPBIG-OFFBIG-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_THREADS_CFLAGS))
  { ":POSIX-V7-THREADS-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_THREADS_LDFLAGS))
  { ":POSIX-V7-THREADS-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_POSIX_V7_WIDTH_RESTRICTED_ENVS))
  { ":POSIX-V7-WIDTH-RESTRICTED-ENVS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_PPC))
  { ":PPC" },
#endif
  { ":PRESERVE" },
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PRIORITIZED_IO))
  { ":PRIORITIZED-IO" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_PRIORITY_SCHEDULING))
  { ":PRIORITY-SCHEDULING" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_PRIO_IO))
  { ":PRIO-IO" },
#endif
#if defined(PRIO_PROCESS)
  { ":PROCESS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":R4" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":R8" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RAW_SOCKETS))
  { ":RAW-SOCKETS" },
#endif
#if defined(HAVE_FCNTL)
  { ":RDONLY" },
#endif
#if defined(HAVE_FCNTL)
  { ":RDWR" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_READER_WRITER_LOCKS))
  { ":READER-WRITER-LOCKS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_READONLY))
  { ":READONLY" },
#endif
#if ((defined(WIN32_NATIVE)) && (defined(REALTIME_PRIORITY_CLASS))) || (!(defined(WIN32_NATIVE)))
  { ":REALTIME" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_REALTIME_SIGNALS))
  { ":REALTIME-SIGNALS" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_INCR_XFER_SIZE))
  { ":REC-INCR-XFER-SIZE" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_MAX_XFER_SIZE))
  { ":REC-MAX-XFER-SIZE" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_MIN_XFER_SIZE))
  { ":REC-MIN-XFER-SIZE" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_REC_XFER_ALIGN))
  { ":REC-XFER-ALIGN" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_REGEXP))
  { ":REGEXP" },
#endif
  { ":RENAME" },
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_REPARSE_POINT))
  { ":REPARSE-POINT" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":REVNUMBER" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RE_DUP_MAX))
  { ":RE-DUP-MAX" },
#endif
#if defined(S_IRGRP)
  { ":RGRP" },
#endif
#if defined(S_IROTH)
  { ":ROTH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ADDRESS_ERROR))
  { ":RPC_S_ADDRESS_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ALREADY_LISTENING))
  { ":RPC_S_ALREADY_LISTENING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ALREADY_REGISTERED))
  { ":RPC_S_ALREADY_REGISTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_BINDING_HAS_NO_AUTH))
  { ":RPC_S_BINDING_HAS_NO_AUTH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_BINDING_INCOMPLETE))
  { ":RPC_S_BINDING_INCOMPLETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_CANCELLED))
  { ":RPC_S_CALL_CANCELLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_FAILED))
  { ":RPC_S_CALL_FAILED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_FAILED_DNE))
  { ":RPC_S_CALL_FAILED_DNE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CALL_IN_PROGRESS))
  { ":RPC_S_CALL_IN_PROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CANNOT_SUPPORT))
  { ":RPC_S_CANNOT_SUPPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_CANT_CREATE_ENDPOINT))
  { ":RPC_S_CANT_CREATE_ENDPOINT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_COMM_FAILURE))
  { ":RPC_S_COMM_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_DUPLICATE_ENDPOINT))
  { ":RPC_S_DUPLICATE_ENDPOINT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_ALREADY_EXISTS))
  { ":RPC_S_ENTRY_ALREADY_EXISTS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_NOT_FOUND))
  { ":RPC_S_ENTRY_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ENTRY_TYPE_MISMATCH))
  { ":RPC_S_ENTRY_TYPE_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_DIV_ZERO))
  { ":RPC_S_FP_DIV_ZERO" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_OVERFLOW))
  { ":RPC_S_FP_OVERFLOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_FP_UNDERFLOW))
  { ":RPC_S_FP_UNDERFLOW" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GROUP_MEMBER_NOT_FOUND))
  { ":RPC_S_GROUP_MEMBER_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GRP_ELT_NOT_ADDED))
  { ":RPC_S_GRP_ELT_NOT_ADDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_GRP_ELT_NOT_REMOVED))
  { ":RPC_S_GRP_ELT_NOT_REMOVED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INCOMPLETE_NAME))
  { ":RPC_S_INCOMPLETE_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERFACE_NOT_EXPORTED))
  { ":RPC_S_INTERFACE_NOT_EXPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERFACE_NOT_FOUND))
  { ":RPC_S_INTERFACE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INTERNAL_ERROR))
  { ":RPC_S_INTERNAL_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ASYNC_CALL))
  { ":RPC_S_INVALID_ASYNC_CALL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ASYNC_HANDLE))
  { ":RPC_S_INVALID_ASYNC_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_AUTH_IDENTITY))
  { ":RPC_S_INVALID_AUTH_IDENTITY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_BINDING))
  { ":RPC_S_INVALID_BINDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_BOUND))
  { ":RPC_S_INVALID_BOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_ENDPOINT_FORMAT))
  { ":RPC_S_INVALID_ENDPOINT_FORMAT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NAF_ID))
  { ":RPC_S_INVALID_NAF_ID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NAME_SYNTAX))
  { ":RPC_S_INVALID_NAME_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NETWORK_OPTIONS))
  { ":RPC_S_INVALID_NETWORK_OPTIONS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_NET_ADDR))
  { ":RPC_S_INVALID_NET_ADDR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_OBJECT))
  { ":RPC_S_INVALID_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_RPC_PROTSEQ))
  { ":RPC_S_INVALID_RPC_PROTSEQ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_STRING_BINDING))
  { ":RPC_S_INVALID_STRING_BINDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_STRING_UUID))
  { ":RPC_S_INVALID_STRING_UUID" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_TAG))
  { ":RPC_S_INVALID_TAG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_TIMEOUT))
  { ":RPC_S_INVALID_TIMEOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_INVALID_VERS_OPTION))
  { ":RPC_S_INVALID_VERS_OPTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_MAX_CALLS_TOO_SMALL))
  { ":RPC_S_MAX_CALLS_TOO_SMALL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NAME_SERVICE_UNAVAILABLE))
  { ":RPC_S_NAME_SERVICE_UNAVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOTHING_TO_EXPORT))
  { ":RPC_S_NOTHING_TO_EXPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_ALL_OBJS_EXPORTED))
  { ":RPC_S_NOT_ALL_OBJS_EXPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_ALL_OBJS_UNEXPORTED))
  { ":RPC_S_NOT_ALL_OBJS_UNEXPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_CANCELLED))
  { ":RPC_S_NOT_CANCELLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_LISTENING))
  { ":RPC_S_NOT_LISTENING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NOT_RPC_ERROR))
  { ":RPC_S_NOT_RPC_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_BINDINGS))
  { ":RPC_S_NO_BINDINGS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_CALL_ACTIVE))
  { ":RPC_S_NO_CALL_ACTIVE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_CONTEXT_AVAILABLE))
  { ":RPC_S_NO_CONTEXT_AVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_ENDPOINT_FOUND))
  { ":RPC_S_NO_ENDPOINT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_ENTRY_NAME))
  { ":RPC_S_NO_ENTRY_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_INTERFACES))
  { ":RPC_S_NO_INTERFACES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_MORE_BINDINGS))
  { ":RPC_S_NO_MORE_BINDINGS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_MORE_MEMBERS))
  { ":RPC_S_NO_MORE_MEMBERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PRINC_NAME))
  { ":RPC_S_NO_PRINC_NAME" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PROTSEQS))
  { ":RPC_S_NO_PROTSEQS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_NO_PROTSEQS_REGISTERED))
  { ":RPC_S_NO_PROTSEQS_REGISTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_OBJECT_NOT_FOUND))
  { ":RPC_S_OBJECT_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_OUT_OF_RESOURCES))
  { ":RPC_S_OUT_OF_RESOURCES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PRF_ELT_NOT_ADDED))
  { ":RPC_S_PRF_ELT_NOT_ADDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PRF_ELT_NOT_REMOVED))
  { ":RPC_S_PRF_ELT_NOT_REMOVED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROCNUM_OUT_OF_RANGE))
  { ":RPC_S_PROCNUM_OUT_OF_RANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROFILE_NOT_ADDED))
  { ":RPC_S_PROFILE_NOT_ADDED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTOCOL_ERROR))
  { ":RPC_S_PROTOCOL_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTSEQ_NOT_FOUND))
  { ":RPC_S_PROTSEQ_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_PROTSEQ_NOT_SUPPORTED))
  { ":RPC_S_PROTSEQ_NOT_SUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SEC_PKG_ERROR))
  { ":RPC_S_SEC_PKG_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SEND_INCOMPLETE))
  { ":RPC_S_SEND_INCOMPLETE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SERVER_TOO_BUSY))
  { ":RPC_S_SERVER_TOO_BUSY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_SERVER_UNAVAILABLE))
  { ":RPC_S_SERVER_UNAVAILABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_STRING_TOO_LONG))
  { ":RPC_S_STRING_TOO_LONG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_TYPE_ALREADY_REGISTERED))
  { ":RPC_S_TYPE_ALREADY_REGISTERED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_LEVEL))
  { ":RPC_S_UNKNOWN_AUTHN_LEVEL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_SERVICE))
  { ":RPC_S_UNKNOWN_AUTHN_SERVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHN_TYPE))
  { ":RPC_S_UNKNOWN_AUTHN_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_AUTHZ_SERVICE))
  { ":RPC_S_UNKNOWN_AUTHZ_SERVICE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_IF))
  { ":RPC_S_UNKNOWN_IF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNKNOWN_MGR_TYPE))
  { ":RPC_S_UNKNOWN_MGR_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_AUTHN_LEVEL))
  { ":RPC_S_UNSUPPORTED_AUTHN_LEVEL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_NAME_SYNTAX))
  { ":RPC_S_UNSUPPORTED_NAME_SYNTAX" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_TRANS_SYN))
  { ":RPC_S_UNSUPPORTED_TRANS_SYN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UNSUPPORTED_TYPE))
  { ":RPC_S_UNSUPPORTED_TYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UUID_LOCAL_ONLY))
  { ":RPC_S_UUID_LOCAL_ONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_UUID_NO_ADDRESS))
  { ":RPC_S_UUID_NO_ADDRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_WRONG_KIND_OF_BINDING))
  { ":RPC_S_WRONG_KIND_OF_BINDING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_S_ZERO_DIVIDE))
  { ":RPC_S_ZERO_DIVIDE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_BAD_STUB_DATA))
  { ":RPC_X_BAD_STUB_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_BYTE_COUNT_TOO_SMALL))
  { ":RPC_X_BYTE_COUNT_TOO_SMALL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_ENUM_VALUE_OUT_OF_RANGE))
  { ":RPC_X_ENUM_VALUE_OUT_OF_RANGE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_INVALID_ES_ACTION))
  { ":RPC_X_INVALID_ES_ACTION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_INVALID_PIPE_OBJECT))
  { ":RPC_X_INVALID_PIPE_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_NO_MORE_ENTRIES))
  { ":RPC_X_NO_MORE_ENTRIES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_NULL_REF_POINTER))
  { ":RPC_X_NULL_REF_POINTER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_CLOSED))
  { ":RPC_X_PIPE_CLOSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_DISCIPLINE_ERROR))
  { ":RPC_X_PIPE_DISCIPLINE_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_PIPE_EMPTY))
  { ":RPC_X_PIPE_EMPTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CANNOT_GET_CALL_HANDLE))
  { ":RPC_X_SS_CANNOT_GET_CALL_HANDLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CHAR_TRANS_OPEN_FAIL))
  { ":RPC_X_SS_CHAR_TRANS_OPEN_FAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CHAR_TRANS_SHORT_FILE))
  { ":RPC_X_SS_CHAR_TRANS_SHORT_FILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_CONTEXT_DAMAGED))
  { ":RPC_X_SS_CONTEXT_DAMAGED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_HANDLES_MISMATCH))
  { ":RPC_X_SS_HANDLES_MISMATCH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_SS_IN_NULL_CONTEXT))
  { ":RPC_X_SS_IN_NULL_CONTEXT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_ES_VERSION))
  { ":RPC_X_WRONG_ES_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_PIPE_ORDER))
  { ":RPC_X_WRONG_PIPE_ORDER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_PIPE_VERSION))
  { ":RPC_X_WRONG_PIPE_VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(RPC_X_WRONG_STUB_VERSION))
  { ":RPC_X_WRONG_STUB_VERSION" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_RSS))
  { ":RSS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_RTSIG_MAX))
  { ":RTSIG-MAX" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(RUN_LVL))
  { ":RUN-LVL" },
#endif
#if defined(S_IRUSR)
  { ":RUSR" },
#endif
#if defined(S_IRWXG)
  { ":RWXG" },
#endif
#if defined(S_IRWXO)
  { ":RWXO" },
#endif
#if defined(S_IRWXU)
  { ":RWXU" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":S" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SAVED_IDS))
  { ":SAVED-IDS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SCHED_E_SERVICE_NOT_LOCALSYSTEM))
  { ":SCHED_E_SERVICE_NOT_LOCALSYSTEM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_ISSUING_CA_UNTRUSTED))
  { ":SEC_E_ISSUING_CA_UNTRUSTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_PKINIT_CLIENT_FAILUR))
  { ":SEC_E_PKINIT_CLIENT_FAILUR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_REVOCATION_OFFLINE_C))
  { ":SEC_E_REVOCATION_OFFLINE_C" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_SMARTCARD_CERT_EXPIRED))
  { ":SEC_E_SMARTCARD_CERT_EXPIRED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(SEC_E_SMARTCARD_CERT_REVOKED))
  { ":SEC_E_SMARTCARD_CERT_REVOKED" },
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_SELF))
  { ":SELF" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEMAPHORES))
  { ":SEMAPHORES" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEM_NSEMS_MAX))
  { ":SEM-NSEMS-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SEM_VALUE_MAX))
  { ":SEM-VALUE-MAX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":SERVER" },
#endif
#if defined(S_ISGID)
  { ":SGID" },
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  { ":SHARED" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SHARED_MEMORY_OBJECTS))
  { ":SHARED-MEMORY-OBJECTS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SHELL))
  { ":SHELL" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":SHIFT" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":SHOW-COMMAND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_SHX))
  { ":SHX" },
#endif
#if defined(SIGABRT)
  { ":SIGABRT" },
#endif
#if defined(SIGALRM)
  { ":SIGALRM" },
#endif
#if defined(SIGBUS)
  { ":SIGBUS" },
#endif
#if defined(SIGCHLD)
  { ":SIGCHLD" },
#endif
#if defined(SIGCONT)
  { ":SIGCONT" },
#endif
#if defined(SIGFPE)
  { ":SIGFPE" },
#endif
#if defined(SIGHUP)
  { ":SIGHUP" },
#endif
#if defined(SIGILL)
  { ":SIGILL" },
#endif
#if defined(SIGINT)
  { ":SIGINT" },
#endif
#if defined(SIGKILL)
  { ":SIGKILL" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { ":SIGNALED" },
#endif
#if defined(SIGPIPE)
  { ":SIGPIPE" },
#endif
#if defined(SIGPOLL)
  { ":SIGPOLL" },
#endif
#if defined(SIGPROF)
  { ":SIGPROF" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SIGQUEUE_MAX))
  { ":SIGQUEUE-MAX" },
#endif
#if defined(SIGQUIT)
  { ":SIGQUIT" },
#endif
#if defined(SIGSEGV)
  { ":SIGSEGV" },
#endif
#if defined(SIGSTOP)
  { ":SIGSTOP" },
#endif
#if defined(SIGSYS)
  { ":SIGSYS" },
#endif
#if defined(SIGTERM)
  { ":SIGTERM" },
#endif
#if defined(SIGTRAP)
  { ":SIGTRAP" },
#endif
#if defined(SIGTSTP)
  { ":SIGTSTP" },
#endif
#if defined(SIGTTIN)
  { ":SIGTTIN" },
#endif
#if defined(SIGTTOU)
  { ":SIGTTOU" },
#endif
#if defined(SIGURG)
  { ":SIGURG" },
#endif
#if defined(SIGUSR1)
  { ":SIGUSR1" },
#endif
#if defined(SIGUSR2)
  { ":SIGUSR2" },
#endif
#if defined(SIGVTALRM)
  { ":SIGVTALRM" },
#endif
#if defined(SIGXCPU)
  { ":SIGXCPU" },
#endif
#if defined(SIGXFSZ)
  { ":SIGXFSZ" },
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_SL))
  { ":SL" },
#endif
#if (defined(HAVE_NFTW)) && (defined(FTW_SLN))
  { ":SLN" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":SMALLBUSINESS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":SMALLBUSINESS-RESTRICTED" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SOCK_MAXBUF))
  { ":SOCK-MAXBUF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_SPARSE_FILE))
  { ":SPARSE-FILE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPAWN))
  { ":SPAWN" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPIN_LOCKS))
  { ":SPIN-LOCKS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SPORADIC_SERVER))
  { ":SPORADIC-SERVER" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SS_REPL_MAX))
  { ":SS-REPL-MAX" },
#endif
#if (defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)) && (defined(RLIMIT_STACK))
  { ":STACK" },
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  { ":START" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { ":STOPPED" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_STREAM_MAX))
  { ":STREAM-MAX" },
#endif
#if defined(STRUNCATE)
  { ":STRUNCATE" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_APPEND))
  { ":ST_APPEND" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_IMMUTABLE))
  { ":ST_IMMUTABLE" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_MANDLOCK))
  { ":ST_MANDLOCK" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOATIME))
  { ":ST_NOATIME" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NODEV))
  { ":ST_NODEV" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NODIRATIME))
  { ":ST_NODIRATIME" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOEXEC))
  { ":ST_NOEXEC" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOSUID))
  { ":ST_NOSUID" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_NOTRUNC))
  { ":ST_NOTRUNC" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_RDONLY))
  { ":ST_RDONLY" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_SYNCHRONOUS))
  { ":ST_SYNCHRONOUS" },
#endif
#if (defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))) && (defined(ST_WRITE))
  { ":ST_WRITE" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":SUBJECT" },
#endif
#if defined(S_ISUID)
  { ":SUID" },
#endif
#if defined(S_ISVTX)
  { ":SVTX" },
#endif
  { ":SYMLINK" },
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SYMLINK_MAX))
  { ":SYMLINK-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SYMLOOP_MAX))
  { ":SYMLOOP-MAX" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_SYNC))
  { ":SYNC" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_SYNCHRONIZED_IO))
  { ":SYNCHRONIZED-IO" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_SYNC_IO))
  { ":SYNC-IO" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_SYSLOG))
  { ":SYSLOG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_SYSTEM))
  { ":SYSTEM" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":TEMPLATE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(FILE_ATTRIBUTE_TEMPORARY))
  { ":TEMPORARY" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":TERMINAL" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_TEXT))
  { ":TEXT" },
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE)) && (defined(RUSAGE_THREAD))
  { ":THREAD" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREADS))
  { ":THREADS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ATTR_STACKADDR))
  { ":THREAD-ATTR-STACKADDR" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ATTR_STACKSIZE))
  { ":THREAD-ATTR-STACKSIZE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_CPUTIME))
  { ":THREAD-CPUTIME" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_DESTRUCTOR_ITERATIONS))
  { ":THREAD-DESTRUCTOR-ITERATIONS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_KEYS_MAX))
  { ":THREAD-KEYS-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIORITY_SCHEDULING))
  { ":THREAD-PRIORITY-SCHEDULING" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIO_INHERIT))
  { ":THREAD-PRIO-INHERIT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PRIO_PROTECT))
  { ":THREAD-PRIO-PROTECT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_PROCESS_SHARED))
  { ":THREAD-PROCESS-SHARED" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ROBUST_PRIO_INHERIT))
  { ":THREAD-ROBUST-PRIO-INHERIT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_ROBUST_PRIO_PROTECT))
  { ":THREAD-ROBUST-PRIO-PROTECT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_SAFE_FUNCTIONS))
  { ":THREAD-SAFE-FUNCTIONS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_SPORADIC_SERVER))
  { ":THREAD-SPORADIC-SERVER" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_STACK_MIN))
  { ":THREAD-STACK-MIN" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_THREAD_THREADS_MAX))
  { ":THREAD-THREADS-MAX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":THUMBNAIL" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMEOUTS))
  { ":TIMEOUTS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMERS))
  { ":TIMERS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TIMER_MAX))
  { ":TIMER-MAX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":TITLE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE))
  { ":TRACE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_EVENT_FILTER))
  { ":TRACE-EVENT-FILTER" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_EVENT_NAME_MAX))
  { ":TRACE-EVENT-NAME-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_INHERIT))
  { ":TRACE-INHERIT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_LOG))
  { ":TRACE-LOG" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_NAME_MAX))
  { ":TRACE-NAME-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_SYS_MAX))
  { ":TRACE-SYS-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TRACE_USER_EVENT_MAX))
  { ":TRACE-USER-EVENT-MAX" },
#endif
#if (defined(HAVE_FCNTL)) && (defined(O_TRUNC))
  { ":TRUNC" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TTY_NAME_MAX))
  { ":TTY-NAME-MAX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TYPED_MEMORY_OBJECTS))
  { ":TYPED-MEMORY-OBJECTS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_TZNAME_MAX))
  { ":TZNAME-MAX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":UI1" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":UI2" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":UI4" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":UI8" },
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  { ":UID" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":UINT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(PROCESSOR_ARCHITECTURE_UNKNOWN))
  { ":UNKNOWN" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { ":UNTRACED" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { ":USAGE" },
#endif
#if ((defined(HAVE_SYSLOG)) && (defined(LOG_USER))) || (defined(PRIO_USER))
  { ":USER" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":USER-DEFINED" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(USER_PROCESS))
  { ":USER-PROCESS" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_UUCP))
  { ":UUCP" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_ILP32_OFF32))
  { ":V6-ILP32-OFF32" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_ILP32_OFFBIG))
  { ":V6-ILP32-OFFBIG" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_LP64_OFF64))
  { ":V6-LP64-OFF64" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V6_LPBIG_OFFBIG))
  { ":V6-LPBIG-OFFBIG" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_V7_ENV))
  { ":V7-ENV" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_ILP32_OFF32))
  { ":V7-ILP32-OFF32" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_ILP32_OFFBIG))
  { ":V7-ILP32-OFFBIG" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_LP64_OFF64))
  { ":V7-LP64-OFF64" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_V7_LPBIG_OFFBIG))
  { ":V7-LPBIG-OFFBIG" },
#endif
#if (defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)) && (defined(_PC_VDISABLE))
  { ":VDISABLE" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_VERSION))
  { ":VERSION" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WAIT_TIMEOUT))
  { ":WAIT_TIMEOUT" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(LOG_WARNING))
  { ":WARNING" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WARNING_IPSEC_MM_POLICY_PRUNED))
  { ":WARNING_IPSEC_MM_POLICY_PRUNED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WARNING_IPSEC_QM_POLICY_PRUNED))
  { ":WARNING_IPSEC_QM_POLICY_PRUNED" },
#endif
#if defined(S_IWGRP)
  { ":WGRP" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":WINDOWS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":WORDCOUNT" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":WORKING-DIRECTORY" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { ":WORKSTATION" },
#endif
#if defined(S_IWOTH)
  { ":WOTH" },
#endif
#if defined(HAVE_FCNTL)
  { ":WRONLY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSABASEERR))
  { ":WSABASEERR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEACCES))
  { ":WSAEACCES" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEADDRINUSE))
  { ":WSAEADDRINUSE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEADDRNOTAVAIL))
  { ":WSAEADDRNOTAVAIL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEAFNOSUPPORT))
  { ":WSAEAFNOSUPPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEALREADY))
  { ":WSAEALREADY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEBADF))
  { ":WSAEBADF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECANCELLED))
  { ":WSAECANCELLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNABORTED))
  { ":WSAECONNABORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNREFUSED))
  { ":WSAECONNREFUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAECONNRESET))
  { ":WSAECONNRESET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDESTADDRREQ))
  { ":WSAEDESTADDRREQ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDISCON))
  { ":WSAEDISCON" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEDQUOT))
  { ":WSAEDQUOT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEFAULT))
  { ":WSAEFAULT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEHOSTDOWN))
  { ":WSAEHOSTDOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEHOSTUNREACH))
  { ":WSAEHOSTUNREACH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINPROGRESS))
  { ":WSAEINPROGRESS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINTR))
  { ":WSAEINTR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVAL))
  { ":WSAEINVAL" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVALIDPROCTABLE))
  { ":WSAEINVALIDPROCTABLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEINVALIDPROVIDER))
  { ":WSAEINVALIDPROVIDER" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEISCONN))
  { ":WSAEISCONN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAELOOP))
  { ":WSAELOOP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEMFILE))
  { ":WSAEMFILE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEMSGSIZE))
  { ":WSAEMSGSIZE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENAMETOOLONG))
  { ":WSAENAMETOOLONG" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETDOWN))
  { ":WSAENETDOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETRESET))
  { ":WSAENETRESET" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENETUNREACH))
  { ":WSAENETUNREACH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOBUFS))
  { ":WSAENOBUFS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOMORE))
  { ":WSAENOMORE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOPROTOOPT))
  { ":WSAENOPROTOOPT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTCONN))
  { ":WSAENOTCONN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTEMPTY))
  { ":WSAENOTEMPTY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAENOTSOCK))
  { ":WSAENOTSOCK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEOPNOTSUPP))
  { ":WSAEOPNOTSUPP" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPFNOSUPPORT))
  { ":WSAEPFNOSUPPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROCLIM))
  { ":WSAEPROCLIM" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROTONOSUPPORT))
  { ":WSAEPROTONOSUPPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROTOTYPE))
  { ":WSAEPROTOTYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEPROVIDERFAILEDINIT))
  { ":WSAEPROVIDERFAILEDINIT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEREFUSED))
  { ":WSAEREFUSED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEREMOTE))
  { ":WSAEREMOTE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESHUTDOWN))
  { ":WSAESHUTDOWN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESOCKTNOSUPPORT))
  { ":WSAESOCKTNOSUPPORT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAESTALE))
  { ":WSAESTALE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAETIMEDOUT))
  { ":WSAETIMEDOUT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAETOOMANYREFS))
  { ":WSAETOOMANYREFS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEUSERS))
  { ":WSAEUSERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAEWOULDBLOCK))
  { ":WSAEWOULDBLOCK" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAHOST_NOT_FOUND))
  { ":WSAHOST_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANOTINITIALISED))
  { ":WSANOTINITIALISED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANO_DATA))
  { ":WSANO_DATA" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSANO_RECOVERY))
  { ":WSANO_RECOVERY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASERVICE_NOT_FOUND))
  { ":WSASERVICE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASYSCALLFAILURE))
  { ":WSASYSCALLFAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSASYSNOTREADY))
  { ":WSASYSNOTREADY" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSATRY_AGAIN))
  { ":WSATRY_AGAIN" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSATYPE_NOT_FOUND))
  { ":WSATYPE_NOT_FOUND" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSAVERNOTSUPPORTED))
  { ":WSAVERNOTSUPPORTED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_E_CANCELLED))
  { ":WSA_E_CANCELLED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_E_NO_MORE))
  { ":WSA_E_NO_MORE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ADMISSION_FAILURE))
  { ":WSA_QOS_ADMISSION_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_BAD_OBJECT))
  { ":WSA_QOS_BAD_OBJECT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_BAD_STYLE))
  { ":WSA_QOS_BAD_STYLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERCOUNT))
  { ":WSA_QOS_EFILTERCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERSTYLE))
  { ":WSA_QOS_EFILTERSTYLE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFILTERTYPE))
  { ":WSA_QOS_EFILTERTYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWCOUNT))
  { ":WSA_QOS_EFLOWCOUNT" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWDESC))
  { ":WSA_QOS_EFLOWDESC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EFLOWSPEC))
  { ":WSA_QOS_EFLOWSPEC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EOBJLENGTH))
  { ":WSA_QOS_EOBJLENGTH" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPOLICYOBJ))
  { ":WSA_QOS_EPOLICYOBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPROVSPECBUF))
  { ":WSA_QOS_EPROVSPECBUF" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPSFILTERSPEC))
  { ":WSA_QOS_EPSFILTERSPEC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EPSFLOWSPEC))
  { ":WSA_QOS_EPSFLOWSPEC" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESDMODEOBJ))
  { ":WSA_QOS_ESDMODEOBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESERVICETYPE))
  { ":WSA_QOS_ESERVICETYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_ESHAPERATEOBJ))
  { ":WSA_QOS_ESHAPERATEOBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_EUNKNOWNPSOBJ))
  { ":WSA_QOS_EUNKNOWNPSOBJ" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_GENERIC_ERROR))
  { ":WSA_QOS_GENERIC_ERROR" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_NO_RECEIVERS))
  { ":WSA_QOS_NO_RECEIVERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_NO_SENDERS))
  { ":WSA_QOS_NO_SENDERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_POLICY_FAILURE))
  { ":WSA_QOS_POLICY_FAILURE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_RECEIVERS))
  { ":WSA_QOS_RECEIVERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_REQUEST_CONFIRMED))
  { ":WSA_QOS_REQUEST_CONFIRMED" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_RESERVED_PETYPE))
  { ":WSA_QOS_RESERVED_PETYPE" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_SENDERS))
  { ":WSA_QOS_SENDERS" },
#endif
#if (defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)) && (defined(WSA_QOS_TRAFFIC_CTRL_ERROR))
  { ":WSA_QOS_TRAFFIC_CTRL_ERROR" },
#endif
#if defined(S_IWUSR)
  { ":WUSR" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_ILP32_OFF32))
  { ":XBS5-ILP32-OFF32" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_CFLAGS))
  { ":XBS5-ILP32-OFF32-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LDFLAGS))
  { ":XBS5-ILP32-OFF32-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LIBS))
  { ":XBS5-ILP32-OFF32-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFF32_LINTFLAGS))
  { ":XBS5-ILP32-OFF32-LINTFLAGS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_ILP32_OFFBIG))
  { ":XBS5-ILP32-OFFBIG" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_CFLAGS))
  { ":XBS5-ILP32-OFFBIG-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LDFLAGS))
  { ":XBS5-ILP32-OFFBIG-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LIBS))
  { ":XBS5-ILP32-OFFBIG-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_ILP32_OFFBIG_LINTFLAGS))
  { ":XBS5-ILP32-OFFBIG-LINTFLAGS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_LP64_OFF64))
  { ":XBS5-LP64-OFF64" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_CFLAGS))
  { ":XBS5-LP64-OFF64-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LDFLAGS))
  { ":XBS5-LP64-OFF64-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LIBS))
  { ":XBS5-LP64-OFF64-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LP64_OFF64_LINTFLAGS))
  { ":XBS5-LP64-OFF64-LINTFLAGS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XBS5_LPBIG_OFFBIG))
  { ":XBS5-LPBIG-OFFBIG" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_CFLAGS))
  { ":XBS5-LPBIG-OFFBIG-CFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LDFLAGS))
  { ":XBS5-LPBIG-OFFBIG-LDFLAGS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LIBS))
  { ":XBS5-LPBIG-OFFBIG-LIBS" },
#endif
#if (defined(HAVE_CONFSTR)) && (defined(_CS_XBS5_LPBIG_OFFBIG_LINTFLAGS))
  { ":XBS5-LPBIG-OFFBIG-LINTFLAGS" },
#endif
#if defined(S_IXGRP)
  { ":XGRP" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_CRYPT))
  { ":XOPEN-CRYPT" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_ENH_I18N))
  { ":XOPEN-ENH-I18N" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_LEGACY))
  { ":XOPEN-LEGACY" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_REALTIME))
  { ":XOPEN-REALTIME" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_REALTIME_THREADS))
  { ":XOPEN-REALTIME-THREADS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_SHM))
  { ":XOPEN-SHM" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_STREAMS))
  { ":XOPEN-STREAMS" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_UNIX))
  { ":XOPEN-UNIX" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_UUCP))
  { ":XOPEN-UUCP" },
#endif
#if (defined(HAVE_SYSCONF)) && (defined(_SC_XOPEN_VERSION))
  { ":XOPEN-VERSION" },
#endif
#if defined(S_IXOTH)
  { ":XOTH" },
#endif
#if defined(S_IXUSR)
  { ":XUSR" },
#endif
  { "POSIX::COPY-FILE" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX::MAKE-FILE-INFO" },
#endif
#if (defined(HAVE_FSTAT) && defined(HAVE_STAT)) || (defined(HAVE_NFTW))
  { "POSIX::MAKE-FILE-STAT" },
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  { "POSIX::MAKE-GROUP-INFO" },
#endif
  { "POSIX::MAKE-HOSTENT" },
#if defined(HAVE_GETRLIMIT)
  { "POSIX::MAKE-RLIMIT" },
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  { "POSIX::MAKE-SERVICE" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX::MAKE-SHORTCUT-INFO" },
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  { "POSIX::MAKE-STAT-VFS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX::MAKE-SYSTEM-INFO" },
#endif
  { "POSIX::MAKE-UNAME" },
#if defined(HAVE_SYS_RESOURCE_H)
  { "POSIX::MAKE-USAGE" },
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  { "POSIX::MAKE-USER-INFO" },
#endif
#if defined(HAVE_UTMPX_H)
  { "POSIX::MAKE-UTMPX" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX::MAKE-VERSION" },
#endif
#if defined(WIN32_NATIVE)
  { "POSIX::MKFILEVER" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX::MKMEMSTAT" },
#endif
  { "POSIX::RESOLVE-HOST-IPADDR" },
#if defined(HAVE_SETRLIMIT)
  { "POSIX::RLIMIT" },
#endif
#if defined(HAVE_UTMPX_H)
  { "POSIX::UTMPX" },
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!(defined(WIN32_NATIVE))) && (!(defined(HAVE_CHMOD)))
  { "\"chmod()\"" },
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!(defined(HAVE_CHOWN)))
  { "\"chown()\"" },
#endif
#if (defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))) && (!defined(WIN32_NATIVE)) && (!(defined(HAVE_UTIME)))
  { "\"utime()\"" },
#endif
  { "(MEMBER 0 1 2)" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "(MEMBER :ALT :CONTROL :EXT :SHIFT)" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "(MEMBER :NORMAL :MAX :MIN)" },
#endif
  { "(OR NULL INTEGER (MEMBER :COPY :SYMLINK :HARDLINK :HARDLINK-OR-COPY :RENAME))" },
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(E2BIG)
    " " ":E2BIG"
#  endif
#  if defined(EACCES)
    " " ":EACCES"
#  endif
#  if defined(EADDRINUSE)
    " " ":EADDRINUSE"
#  endif
#  if defined(EADDRNOTAVAIL)
    " " ":EADDRNOTAVAIL"
#  endif
#  if defined(EAFNOSUPPORT)
    " " ":EAFNOSUPPORT"
#  endif
#  if defined(EAGAIN)
    " " ":EAGAIN"
#  endif
#  if defined(EALREADY)
    " " ":EALREADY"
#  endif
#  if defined(EBADF)
    " " ":EBADF"
#  endif
#  if defined(EBADMSG)
    " " ":EBADMSG"
#  endif
#  if defined(EBUSY)
    " " ":EBUSY"
#  endif
#  if defined(ECANCELED)
    " " ":ECANCELED"
#  endif
#  if defined(ECHILD)
    " " ":ECHILD"
#  endif
#  if defined(ECONNABORTED)
    " " ":ECONNABORTED"
#  endif
#  if defined(ECONNREFUSED)
    " " ":ECONNREFUSED"
#  endif
#  if defined(ECONNRESET)
    " " ":ECONNRESET"
#  endif
#  if defined(EDEADLK)
    " " ":EDEADLK"
#  endif
#  if defined(EDESTADDRREQ)
    " " ":EDESTADDRREQ"
#  endif
#  if defined(EDOM)
    " " ":EDOM"
#  endif
#  if defined(EDQUOT)
    " " ":EDQUOT"
#  endif
#  if defined(EEXIST)
    " " ":EEXIST"
#  endif
#  if defined(EFAULT)
    " " ":EFAULT"
#  endif
#  if defined(EFBIG)
    " " ":EFBIG"
#  endif
#  if defined(EHOSTUNREACH)
    " " ":EHOSTUNREACH"
#  endif
#  if defined(EIDRM)
    " " ":EIDRM"
#  endif
#  if defined(EILSEQ)
    " " ":EILSEQ"
#  endif
#  if defined(EINPROGRESS)
    " " ":EINPROGRESS"
#  endif
#  if defined(EINTR)
    " " ":EINTR"
#  endif
#  if defined(EINVAL)
    " " ":EINVAL"
#  endif
#  if defined(EIO)
    " " ":EIO"
#  endif
#  if defined(EISCONN)
    " " ":EISCONN"
#  endif
#  if defined(EISDIR)
    " " ":EISDIR"
#  endif
#  if defined(ELOOP)
    " " ":ELOOP"
#  endif
#  if defined(EMFILE)
    " " ":EMFILE"
#  endif
#  if defined(EMLINK)
    " " ":EMLINK"
#  endif
#  if defined(EMSGSIZE)
    " " ":EMSGSIZE"
#  endif
#  if defined(EMULTIHOP)
    " " ":EMULTIHOP"
#  endif
#  if defined(ENAMETOOLONG)
    " " ":ENAMETOOLONG"
#  endif
#  if defined(ENETDOWN)
    " " ":ENETDOWN"
#  endif
#  if defined(ENETRESET)
    " " ":ENETRESET"
#  endif
#  if defined(ENETUNREACH)
    " " ":ENETUNREACH"
#  endif
#  if defined(ENFILE)
    " " ":ENFILE"
#  endif
#  if defined(ENOBUFS)
    " " ":ENOBUFS"
#  endif
#  if defined(ENODATA)
    " " ":ENODATA"
#  endif
#  if defined(ENODEV)
    " " ":ENODEV"
#  endif
#  if defined(ENOENT)
    " " ":ENOENT"
#  endif
#  if defined(ENOEXEC)
    " " ":ENOEXEC"
#  endif
#  if defined(ENOLCK)
    " " ":ENOLCK"
#  endif
#  if defined(ENOLINK)
    " " ":ENOLINK"
#  endif
#  if defined(ENOMEM)
    " " ":ENOMEM"
#  endif
#  if defined(ENOMSG)
    " " ":ENOMSG"
#  endif
#  if defined(ENOPROTOOPT)
    " " ":ENOPROTOOPT"
#  endif
#  if defined(ENOSPC)
    " " ":ENOSPC"
#  endif
#  if defined(ENOSR)
    " " ":ENOSR"
#  endif
#  if defined(ENOSTR)
    " " ":ENOSTR"
#  endif
#  if defined(ENOSYS)
    " " ":ENOSYS"
#  endif
#  if defined(ENOTCONN)
    " " ":ENOTCONN"
#  endif
#  if defined(ENOTDIR)
    " " ":ENOTDIR"
#  endif
#  if defined(ENOTEMPTY)
    " " ":ENOTEMPTY"
#  endif
#  if defined(ENOTSOCK)
    " " ":ENOTSOCK"
#  endif
#  if defined(ENOTSUP)
    " " ":ENOTSUP"
#  endif
#  if defined(ENOTTY)
    " " ":ENOTTY"
#  endif
#  if defined(ENXIO)
    " " ":ENXIO"
#  endif
#  if defined(EOPNOTSUPP)
    " " ":EOPNOTSUPP"
#  endif
#  if defined(EOVERFLOW)
    " " ":EOVERFLOW"
#  endif
#  if defined(EPERM)
    " " ":EPERM"
#  endif
#  if defined(EPIPE)
    " " ":EPIPE"
#  endif
#  if defined(EPROTO)
    " " ":EPROTO"
#  endif
#  if defined(EPROTONOSUPPORT)
    " " ":EPROTONOSUPPORT"
#  endif
#  if defined(EPROTOTYPE)
    " " ":EPROTOTYPE"
#  endif
#  if defined(ERANGE)
    " " ":ERANGE"
#  endif
#  if defined(EROFS)
    " " ":EROFS"
#  endif
#  if defined(ESPIPE)
    " " ":ESPIPE"
#  endif
#  if defined(ESRCH)
    " " ":ESRCH"
#  endif
#  if defined(ESTALE)
    " " ":ESTALE"
#  endif
#  if defined(ETIME)
    " " ":ETIME"
#  endif
#  if defined(ETIMEDOUT)
    " " ":ETIMEDOUT"
#  endif
#  if defined(ETXTBSY)
    " " ":ETXTBSY"
#  endif
#  if defined(EWOULDBLOCK)
    " " ":EWOULDBLOCK"
#  endif
#  if defined(EXDEV)
    " " ":EXDEV"
#  endif
#  if defined(ERREMOTE)
    " " ":ERREMOTE"
#  endif
#  if defined(EADV)
    " " ":EADV"
#  endif
#  if defined(EBADE)
    " " ":EBADE"
#  endif
#  if defined(EBADFD)
    " " ":EBADFD"
#  endif
#  if defined(EBADR)
    " " ":EBADR"
#  endif
#  if defined(EBADRQC)
    " " ":EBADRQC"
#  endif
#  if defined(EBADSLT)
    " " ":EBADSLT"
#  endif
#  if defined(EBFONT)
    " " ":EBFONT"
#  endif
#  if defined(ECHRNG)
    " " ":ECHRNG"
#  endif
#  if defined(ECOMM)
    " " ":ECOMM"
#  endif
#  if defined(EDEADLOCK)
    " " ":EDEADLOCK"
#  endif
#  if defined(EDOTDOT)
    " " ":EDOTDOT"
#  endif
#  if defined(EHOSTDOWN)
    " " ":EHOSTDOWN"
#  endif
#  if defined(EISNAM)
    " " ":EISNAM"
#  endif
#  if defined(EL2HLT)
    " " ":EL2HLT"
#  endif
#  if defined(EL2NSYNC)
    " " ":EL2NSYNC"
#  endif
#  if defined(EL3HLT)
    " " ":EL3HLT"
#  endif
#  if defined(EL3RST)
    " " ":EL3RST"
#  endif
#  if defined(ELIBACC)
    " " ":ELIBACC"
#  endif
#  if defined(ELIBBAD)
    " " ":ELIBBAD"
#  endif
#  if defined(ELIBEXEC)
    " " ":ELIBEXEC"
#  endif
#  if defined(ELIBMAX)
    " " ":ELIBMAX"
#  endif
#  if defined(ELIBSCN)
    " " ":ELIBSCN"
#  endif
#  if defined(ELNRNG)
    " " ":ELNRNG"
#  endif
#  if defined(EMEDIUMTYPE)
    " " ":EMEDIUMTYPE"
#  endif
#  if defined(ENAVAIL)
    " " ":ENAVAIL"
#  endif
#  if defined(ENOANO)
    " " ":ENOANO"
#  endif
#  if defined(ENOCSI)
    " " ":ENOCSI"
#  endif
#  if defined(ENOMEDIUM)
    " " ":ENOMEDIUM"
#  endif
#  if defined(ENONET)
    " " ":ENONET"
#  endif
#  if defined(ENOPKG)
    " " ":ENOPKG"
#  endif
#  if defined(ENOTBLK)
    " " ":ENOTBLK"
#  endif
#  if defined(ENOTNAM)
    " " ":ENOTNAM"
#  endif
#  if defined(ENOTUNIQ)
    " " ":ENOTUNIQ"
#  endif
#  if defined(EPFNOSUPPORT)
    " " ":EPFNOSUPPORT"
#  endif
#  if defined(EREMCHG)
    " " ":EREMCHG"
#  endif
#  if defined(EREMOTE)
    " " ":EREMOTE"
#  endif
#  if defined(EREMOTEIO)
    " " ":EREMOTEIO"
#  endif
#  if defined(ERESTART)
    " " ":ERESTART"
#  endif
#  if defined(ESHUTDOWN)
    " " ":ESHUTDOWN"
#  endif
#  if defined(ESOCKTNOSUPPORT)
    " " ":ESOCKTNOSUPPORT"
#  endif
#  if defined(ESRMNT)
    " " ":ESRMNT"
#  endif
#  if defined(ESTRPIPE)
    " " ":ESTRPIPE"
#  endif
#  if defined(ETOOMANYREFS)
    " " ":ETOOMANYREFS"
#  endif
#  if defined(EUCLEAN)
    " " ":EUCLEAN"
#  endif
#  if defined(EUNATCH)
    " " ":EUNATCH"
#  endif
#  if defined(EUSERS)
    " " ":EUSERS"
#  endif
#  if defined(EXFULL)
    " " ":EXFULL"
#  endif
#  if defined(ENOKEY)
    " " ":ENOKEY"
#  endif
#  if defined(EKEYEXPIRED)
    " " ":EKEYEXPIRED"
#  endif
#  if defined(EKEYREVOKED)
    " " ":EKEYREVOKED"
#  endif
#  if defined(EKEYREJECTED)
    " " ":EKEYREJECTED"
#  endif
#  if defined(ERFKILL)
    " " ":ERFKILL"
#  endif
#  if defined(EHWPOISON)
    " " ":EHWPOISON"
#  endif
#  if defined(STRUNCATE)
    " " ":STRUNCATE"
#  endif
#  if defined(ECASECLASH)
    " " ":ECASECLASH"
#  endif
#  if defined(EFTYPE)
    " " ":EFTYPE"
#  endif
#  if defined(ELBIN)
    " " ":ELBIN"
#  endif
#  if defined(ENMFILE)
    " " ":ENMFILE"
#  endif
#  if defined(ENOFILE)
    " " ":ENOFILE"
#  endif
#  if defined(ENOSHARE)
    " " ":ENOSHARE"
#  endif
#  if defined(EPROCLIM)
    " " ":EPROCLIM"
#  endif
#  if defined(EAUTH)
    " " ":EAUTH"
#  endif
#  if defined(EBADRPC)
    " " ":EBADRPC"
#  endif
#  if defined(EDIRIOCTL)
    " " ":EDIRIOCTL"
#  endif
#  if defined(EDOOFUS)
    " " ":EDOOFUS"
#  endif
#  if defined(EJUSTRETURN)
    " " ":EJUSTRETURN"
#  endif
#  if defined(ELAST)
    " " ":ELAST"
#  endif
#  if defined(ENEEDAUTH)
    " " ":ENEEDAUTH"
#  endif
#  if defined(ENOATTR)
    " " ":ENOATTR"
#  endif
#  if defined(ENOIOCTL)
    " " ":ENOIOCTL"
#  endif
#  if defined(EPROCUNAVAIL)
    " " ":EPROCUNAVAIL"
#  endif
#  if defined(EPROGMISMATCH)
    " " ":EPROGMISMATCH"
#  endif
#  if defined(EPROGUNAVAIL)
    " " ":EPROGUNAVAIL"
#  endif
#  if defined(ERPCMISMATCH)
    " " ":ERPCMISMATCH"
#  endif
#  if defined(ELOCKUNMAPPED)
    " " ":ELOCKUNMAPPED"
#  endif
#  if defined(ENOTACTIVE)
    " " ":ENOTACTIVE"
#  endif
#  if defined(ENOTRECOVERABLE)
    " " ":ENOTRECOVERABLE"
#  endif
#  if defined(EOWNERDEAD)
    " " ":EOWNERDEAD"
#  endif
#  if defined(EALIGN)
    " " ":EALIGN"
#  endif
#  if defined(EDIRTY)
    " " ":EDIRTY"
#  endif
#  if defined(EDUPPKG)
    " " ":EDUPPKG"
#  endif
#  if defined(EINIT)
    " " ":EINIT"
#  endif
#  if defined(EISNAME)
    " " ":EISNAME"
#  endif
#  if defined(ENET)
    " " ":ENET"
#  endif
#  if defined(ENOSYM)
    " " ":ENOSYM"
#  endif
#  if defined(EREMDEV)
    " " ":EREMDEV"
#  endif
#  if defined(EREMOTERELEASE)
    " " ":EREMOTERELEASE"
#  endif
#  if defined(EVERSION)
    " " ":EVERSION"
#  endif
#  if defined(EAIO)
    " " ":EAIO"
#  endif
#  if defined(ECLONEME)
    " " ":ECLONEME"
#  endif
#  if defined(EFAIL)
    " " ":EFAIL"
#  endif
#  if defined(EINPROG)
    " " ":EINPROG"
#  endif
#  if defined(EMTIMERS)
    " " ":EMTIMERS"
#  endif
#  if defined(ERESTARTNOHAND)
    " " ":ERESTARTNOHAND"
#  endif
#  if defined(ERESTARTNOINTR)
    " " ":ERESTARTNOINTR"
#  endif
#  if defined(ERESTARTSYS)
    " " ":ERESTARTSYS"
#  endif
    " " "))"
  },
#if defined(HAVE_UTMPX_H)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(EMPTY)
    " " ":EMPTY"
#  endif
#  if defined(RUN_LVL)
    " " ":RUN-LVL"
#  endif
#  if defined(BOOT_TIME)
    " " ":BOOT-TIME"
#  endif
#  if defined(OLD_TIME)
    " " ":OLD-TIME"
#  endif
#  if defined(NEW_TIME)
    " " ":NEW-TIME"
#  endif
#  if defined(USER_PROCESS)
    " " ":USER-PROCESS"
#  endif
#  if defined(INIT_PROCESS)
    " " ":INIT-PROCESS"
#  endif
#  if defined(LOGIN_PROCESS)
    " " ":LOGIN-PROCESS"
#  endif
#  if defined(DEAD_PROCESS)
    " " ":DEAD-PROCESS"
#  endif
#  if defined(ACCOUNTING)
    " " ":ACCOUNTING"
#  endif
    " " "))"
  },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(ERROR_INVALID_FUNCTION)
    " " ":ERROR_INVALID_FUNCTION"
#  endif
#  if defined(ERROR_FILE_NOT_FOUND)
    " " ":ERROR_FILE_NOT_FOUND"
#  endif
#  if defined(ERROR_PATH_NOT_FOUND)
    " " ":ERROR_PATH_NOT_FOUND"
#  endif
#  if defined(ERROR_TOO_MANY_OPEN_FILES)
    " " ":ERROR_TOO_MANY_OPEN_FILES"
#  endif
#  if defined(ERROR_ACCESS_DENIED)
    " " ":ERROR_ACCESS_DENIED"
#  endif
#  if defined(ERROR_INVALID_HANDLE)
    " " ":ERROR_INVALID_HANDLE"
#  endif
#  if defined(ERROR_ARENA_TRASHED)
    " " ":ERROR_ARENA_TRASHED"
#  endif
#  if defined(ERROR_NOT_ENOUGH_MEMORY)
    " " ":ERROR_NOT_ENOUGH_MEMORY"
#  endif
#  if defined(ERROR_INVALID_BLOCK)
    " " ":ERROR_INVALID_BLOCK"
#  endif
#  if defined(ERROR_BAD_ENVIRONMENT)
    " " ":ERROR_BAD_ENVIRONMENT"
#  endif
#  if defined(ERROR_BAD_FORMAT)
    " " ":ERROR_BAD_FORMAT"
#  endif
#  if defined(ERROR_INVALID_ACCESS)
    " " ":ERROR_INVALID_ACCESS"
#  endif
#  if defined(ERROR_INVALID_DATA)
    " " ":ERROR_INVALID_DATA"
#  endif
#  if defined(ERROR_OUTOFMEMORY)
    " " ":ERROR_OUTOFMEMORY"
#  endif
#  if defined(ERROR_INVALID_DRIVE)
    " " ":ERROR_INVALID_DRIVE"
#  endif
#  if defined(ERROR_CURRENT_DIRECTORY)
    " " ":ERROR_CURRENT_DIRECTORY"
#  endif
#  if defined(ERROR_NOT_SAME_DEVICE)
    " " ":ERROR_NOT_SAME_DEVICE"
#  endif
#  if defined(ERROR_NO_MORE_FILES)
    " " ":ERROR_NO_MORE_FILES"
#  endif
#  if defined(ERROR_WRITE_PROTECT)
    " " ":ERROR_WRITE_PROTECT"
#  endif
#  if defined(ERROR_BAD_UNIT)
    " " ":ERROR_BAD_UNIT"
#  endif
#  if defined(ERROR_NOT_READY)
    " " ":ERROR_NOT_READY"
#  endif
#  if defined(ERROR_BAD_COMMAND)
    " " ":ERROR_BAD_COMMAND"
#  endif
#  if defined(ERROR_CRC)
    " " ":ERROR_CRC"
#  endif
#  if defined(ERROR_BAD_LENGTH)
    " " ":ERROR_BAD_LENGTH"
#  endif
#  if defined(ERROR_SEEK)
    " " ":ERROR_SEEK"
#  endif
#  if defined(ERROR_NOT_DOS_DISK)
    " " ":ERROR_NOT_DOS_DISK"
#  endif
#  if defined(ERROR_SECTOR_NOT_FOUND)
    " " ":ERROR_SECTOR_NOT_FOUND"
#  endif
#  if defined(ERROR_OUT_OF_PAPER)
    " " ":ERROR_OUT_OF_PAPER"
#  endif
#  if defined(ERROR_WRITE_FAULT)
    " " ":ERROR_WRITE_FAULT"
#  endif
#  if defined(ERROR_READ_FAULT)
    " " ":ERROR_READ_FAULT"
#  endif
#  if defined(ERROR_GEN_FAILURE)
    " " ":ERROR_GEN_FAILURE"
#  endif
#  if defined(ERROR_SHARING_VIOLATION)
    " " ":ERROR_SHARING_VIOLATION"
#  endif
#  if defined(ERROR_LOCK_VIOLATION)
    " " ":ERROR_LOCK_VIOLATION"
#  endif
#  if defined(ERROR_WRONG_DISK)
    " " ":ERROR_WRONG_DISK"
#  endif
#  if defined(ERROR_SHARING_BUFFER_EXCEEDED)
    " " ":ERROR_SHARING_BUFFER_EXCEEDED"
#  endif
#  if defined(ERROR_HANDLE_EOF)
    " " ":ERROR_HANDLE_EOF"
#  endif
#  if defined(ERROR_HANDLE_DISK_FULL)
    " " ":ERROR_HANDLE_DISK_FULL"
#  endif
#  if defined(ERROR_NOT_SUPPORTED)
    " " ":ERROR_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_REM_NOT_LIST)
    " " ":ERROR_REM_NOT_LIST"
#  endif
#  if defined(ERROR_DUP_NAME)
    " " ":ERROR_DUP_NAME"
#  endif
#  if defined(ERROR_BAD_NETPATH)
    " " ":ERROR_BAD_NETPATH"
#  endif
#  if defined(ERROR_NETWORK_BUSY)
    " " ":ERROR_NETWORK_BUSY"
#  endif
#  if defined(ERROR_DEV_NOT_EXIST)
    " " ":ERROR_DEV_NOT_EXIST"
#  endif
#  if defined(ERROR_TOO_MANY_CMDS)
    " " ":ERROR_TOO_MANY_CMDS"
#  endif
#  if defined(ERROR_ADAP_HDW_ERR)
    " " ":ERROR_ADAP_HDW_ERR"
#  endif
#  if defined(ERROR_BAD_NET_RESP)
    " " ":ERROR_BAD_NET_RESP"
#  endif
#  if defined(ERROR_UNEXP_NET_ERR)
    " " ":ERROR_UNEXP_NET_ERR"
#  endif
#  if defined(ERROR_BAD_REM_ADAP)
    " " ":ERROR_BAD_REM_ADAP"
#  endif
#  if defined(ERROR_PRINTQ_FULL)
    " " ":ERROR_PRINTQ_FULL"
#  endif
#  if defined(ERROR_NO_SPOOL_SPACE)
    " " ":ERROR_NO_SPOOL_SPACE"
#  endif
#  if defined(ERROR_PRINT_CANCELLED)
    " " ":ERROR_PRINT_CANCELLED"
#  endif
#  if defined(ERROR_NETNAME_DELETED)
    " " ":ERROR_NETNAME_DELETED"
#  endif
#  if defined(ERROR_NETWORK_ACCESS_DENIED)
    " " ":ERROR_NETWORK_ACCESS_DENIED"
#  endif
#  if defined(ERROR_BAD_DEV_TYPE)
    " " ":ERROR_BAD_DEV_TYPE"
#  endif
#  if defined(ERROR_BAD_NET_NAME)
    " " ":ERROR_BAD_NET_NAME"
#  endif
#  if defined(ERROR_TOO_MANY_NAMES)
    " " ":ERROR_TOO_MANY_NAMES"
#  endif
#  if defined(ERROR_TOO_MANY_SESS)
    " " ":ERROR_TOO_MANY_SESS"
#  endif
#  if defined(ERROR_SHARING_PAUSED)
    " " ":ERROR_SHARING_PAUSED"
#  endif
#  if defined(ERROR_REQ_NOT_ACCEP)
    " " ":ERROR_REQ_NOT_ACCEP"
#  endif
#  if defined(ERROR_REDIR_PAUSED)
    " " ":ERROR_REDIR_PAUSED"
#  endif
#  if defined(ERROR_FILE_EXISTS)
    " " ":ERROR_FILE_EXISTS"
#  endif
#  if defined(ERROR_CANNOT_MAKE)
    " " ":ERROR_CANNOT_MAKE"
#  endif
#  if defined(ERROR_FAIL_I24)
    " " ":ERROR_FAIL_I24"
#  endif
#  if defined(ERROR_OUT_OF_STRUCTURES)
    " " ":ERROR_OUT_OF_STRUCTURES"
#  endif
#  if defined(ERROR_ALREADY_ASSIGNED)
    " " ":ERROR_ALREADY_ASSIGNED"
#  endif
#  if defined(ERROR_INVALID_PASSWORD)
    " " ":ERROR_INVALID_PASSWORD"
#  endif
#  if defined(ERROR_INVALID_PARAMETER)
    " " ":ERROR_INVALID_PARAMETER"
#  endif
#  if defined(ERROR_NET_WRITE_FAULT)
    " " ":ERROR_NET_WRITE_FAULT"
#  endif
#  if defined(ERROR_NO_PROC_SLOTS)
    " " ":ERROR_NO_PROC_SLOTS"
#  endif
#  if defined(ERROR_TOO_MANY_SEMAPHORES)
    " " ":ERROR_TOO_MANY_SEMAPHORES"
#  endif
#  if defined(ERROR_EXCL_SEM_ALREADY_OWNED)
    " " ":ERROR_EXCL_SEM_ALREADY_OWNED"
#  endif
#  if defined(ERROR_SEM_IS_SET)
    " " ":ERROR_SEM_IS_SET"
#  endif
#  if defined(ERROR_TOO_MANY_SEM_REQUESTS)
    " " ":ERROR_TOO_MANY_SEM_REQUESTS"
#  endif
#  if defined(ERROR_INVALID_AT_INTERRUPT_TIME)
    " " ":ERROR_INVALID_AT_INTERRUPT_TIME"
#  endif
#  if defined(ERROR_SEM_OWNER_DIED)
    " " ":ERROR_SEM_OWNER_DIED"
#  endif
#  if defined(ERROR_SEM_USER_LIMIT)
    " " ":ERROR_SEM_USER_LIMIT"
#  endif
#  if defined(ERROR_DISK_CHANGE)
    " " ":ERROR_DISK_CHANGE"
#  endif
#  if defined(ERROR_DRIVE_LOCKED)
    " " ":ERROR_DRIVE_LOCKED"
#  endif
#  if defined(ERROR_BROKEN_PIPE)
    " " ":ERROR_BROKEN_PIPE"
#  endif
#  if defined(ERROR_OPEN_FAILED)
    " " ":ERROR_OPEN_FAILED"
#  endif
#  if defined(ERROR_BUFFER_OVERFLOW)
    " " ":ERROR_BUFFER_OVERFLOW"
#  endif
#  if defined(ERROR_DISK_FULL)
    " " ":ERROR_DISK_FULL"
#  endif
#  if defined(ERROR_NO_MORE_SEARCH_HANDLES)
    " " ":ERROR_NO_MORE_SEARCH_HANDLES"
#  endif
#  if defined(ERROR_INVALID_TARGET_HANDLE)
    " " ":ERROR_INVALID_TARGET_HANDLE"
#  endif
#  if defined(ERROR_INVALID_CATEGORY)
    " " ":ERROR_INVALID_CATEGORY"
#  endif
#  if defined(ERROR_INVALID_VERIFY_SWITCH)
    " " ":ERROR_INVALID_VERIFY_SWITCH"
#  endif
#  if defined(ERROR_BAD_DRIVER_LEVEL)
    " " ":ERROR_BAD_DRIVER_LEVEL"
#  endif
#  if defined(ERROR_CALL_NOT_IMPLEMENTED)
    " " ":ERROR_CALL_NOT_IMPLEMENTED"
#  endif
#  if defined(ERROR_SEM_TIMEOUT)
    " " ":ERROR_SEM_TIMEOUT"
#  endif
#  if defined(ERROR_INSUFFICIENT_BUFFER)
    " " ":ERROR_INSUFFICIENT_BUFFER"
#  endif
#  if defined(ERROR_INVALID_NAME)
    " " ":ERROR_INVALID_NAME"
#  endif
#  if defined(ERROR_INVALID_LEVEL)
    " " ":ERROR_INVALID_LEVEL"
#  endif
#  if defined(ERROR_NO_VOLUME_LABEL)
    " " ":ERROR_NO_VOLUME_LABEL"
#  endif
#  if defined(ERROR_MOD_NOT_FOUND)
    " " ":ERROR_MOD_NOT_FOUND"
#  endif
#  if defined(ERROR_PROC_NOT_FOUND)
    " " ":ERROR_PROC_NOT_FOUND"
#  endif
#  if defined(ERROR_WAIT_NO_CHILDREN)
    " " ":ERROR_WAIT_NO_CHILDREN"
#  endif
#  if defined(ERROR_CHILD_NOT_COMPLETE)
    " " ":ERROR_CHILD_NOT_COMPLETE"
#  endif
#  if defined(ERROR_DIRECT_ACCESS_HANDLE)
    " " ":ERROR_DIRECT_ACCESS_HANDLE"
#  endif
#  if defined(ERROR_NEGATIVE_SEEK)
    " " ":ERROR_NEGATIVE_SEEK"
#  endif
#  if defined(ERROR_SEEK_ON_DEVICE)
    " " ":ERROR_SEEK_ON_DEVICE"
#  endif
#  if defined(ERROR_IS_JOIN_TARGET)
    " " ":ERROR_IS_JOIN_TARGET"
#  endif
#  if defined(ERROR_IS_JOINED)
    " " ":ERROR_IS_JOINED"
#  endif
#  if defined(ERROR_IS_SUBSTED)
    " " ":ERROR_IS_SUBSTED"
#  endif
#  if defined(ERROR_NOT_JOINED)
    " " ":ERROR_NOT_JOINED"
#  endif
#  if defined(ERROR_NOT_SUBSTED)
    " " ":ERROR_NOT_SUBSTED"
#  endif
#  if defined(ERROR_JOIN_TO_JOIN)
    " " ":ERROR_JOIN_TO_JOIN"
#  endif
#  if defined(ERROR_SUBST_TO_SUBST)
    " " ":ERROR_SUBST_TO_SUBST"
#  endif
#  if defined(ERROR_JOIN_TO_SUBST)
    " " ":ERROR_JOIN_TO_SUBST"
#  endif
#  if defined(ERROR_SUBST_TO_JOIN)
    " " ":ERROR_SUBST_TO_JOIN"
#  endif
#  if defined(ERROR_BUSY_DRIVE)
    " " ":ERROR_BUSY_DRIVE"
#  endif
#  if defined(ERROR_SAME_DRIVE)
    " " ":ERROR_SAME_DRIVE"
#  endif
#  if defined(ERROR_DIR_NOT_ROOT)
    " " ":ERROR_DIR_NOT_ROOT"
#  endif
#  if defined(ERROR_DIR_NOT_EMPTY)
    " " ":ERROR_DIR_NOT_EMPTY"
#  endif
#  if defined(ERROR_IS_SUBST_PATH)
    " " ":ERROR_IS_SUBST_PATH"
#  endif
#  if defined(ERROR_IS_JOIN_PATH)
    " " ":ERROR_IS_JOIN_PATH"
#  endif
#  if defined(ERROR_PATH_BUSY)
    " " ":ERROR_PATH_BUSY"
#  endif
#  if defined(ERROR_IS_SUBST_TARGET)
    " " ":ERROR_IS_SUBST_TARGET"
#  endif
#  if defined(ERROR_SYSTEM_TRACE)
    " " ":ERROR_SYSTEM_TRACE"
#  endif
#  if defined(ERROR_INVALID_EVENT_COUNT)
    " " ":ERROR_INVALID_EVENT_COUNT"
#  endif
#  if defined(ERROR_TOO_MANY_MUXWAITERS)
    " " ":ERROR_TOO_MANY_MUXWAITERS"
#  endif
#  if defined(ERROR_INVALID_LIST_FORMAT)
    " " ":ERROR_INVALID_LIST_FORMAT"
#  endif
#  if defined(ERROR_LABEL_TOO_LONG)
    " " ":ERROR_LABEL_TOO_LONG"
#  endif
#  if defined(ERROR_TOO_MANY_TCBS)
    " " ":ERROR_TOO_MANY_TCBS"
#  endif
#  if defined(ERROR_SIGNAL_REFUSED)
    " " ":ERROR_SIGNAL_REFUSED"
#  endif
#  if defined(ERROR_DISCARDED)
    " " ":ERROR_DISCARDED"
#  endif
#  if defined(ERROR_NOT_LOCKED)
    " " ":ERROR_NOT_LOCKED"
#  endif
#  if defined(ERROR_BAD_THREADID_ADDR)
    " " ":ERROR_BAD_THREADID_ADDR"
#  endif
#  if defined(ERROR_BAD_ARGUMENTS)
    " " ":ERROR_BAD_ARGUMENTS"
#  endif
#  if defined(ERROR_BAD_PATHNAME)
    " " ":ERROR_BAD_PATHNAME"
#  endif
#  if defined(ERROR_SIGNAL_PENDING)
    " " ":ERROR_SIGNAL_PENDING"
#  endif
#  if defined(ERROR_MAX_THRDS_REACHED)
    " " ":ERROR_MAX_THRDS_REACHED"
#  endif
#  if defined(ERROR_LOCK_FAILED)
    " " ":ERROR_LOCK_FAILED"
#  endif
#  if defined(ERROR_BUSY)
    " " ":ERROR_BUSY"
#  endif
#  if defined(ERROR_CANCEL_VIOLATION)
    " " ":ERROR_CANCEL_VIOLATION"
#  endif
#  if defined(ERROR_ATOMIC_LOCKS_NOT_SUPPORTED)
    " " ":ERROR_ATOMIC_LOCKS_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_INVALID_SEGMENT_NUMBER)
    " " ":ERROR_INVALID_SEGMENT_NUMBER"
#  endif
#  if defined(ERROR_INVALID_ORDINAL)
    " " ":ERROR_INVALID_ORDINAL"
#  endif
#  if defined(ERROR_ALREADY_EXISTS)
    " " ":ERROR_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_INVALID_FLAG_NUMBER)
    " " ":ERROR_INVALID_FLAG_NUMBER"
#  endif
#  if defined(ERROR_SEM_NOT_FOUND)
    " " ":ERROR_SEM_NOT_FOUND"
#  endif
#  if defined(ERROR_INVALID_STARTING_CODESEG)
    " " ":ERROR_INVALID_STARTING_CODESEG"
#  endif
#  if defined(ERROR_INVALID_STACKSEG)
    " " ":ERROR_INVALID_STACKSEG"
#  endif
#  if defined(ERROR_INVALID_MODULETYPE)
    " " ":ERROR_INVALID_MODULETYPE"
#  endif
#  if defined(ERROR_INVALID_EXE_SIGNATURE)
    " " ":ERROR_INVALID_EXE_SIGNATURE"
#  endif
#  if defined(ERROR_EXE_MARKED_INVALID)
    " " ":ERROR_EXE_MARKED_INVALID"
#  endif
#  if defined(ERROR_BAD_EXE_FORMAT)
    " " ":ERROR_BAD_EXE_FORMAT"
#  endif
#  if defined(ERROR_ITERATED_DATA_EXCEEDS_64k)
    " " ":ERROR_ITERATED_DATA_EXCEEDS_64K"
#  endif
#  if defined(ERROR_INVALID_MINALLOCSIZE)
    " " ":ERROR_INVALID_MINALLOCSIZE"
#  endif
#  if defined(ERROR_DYNLINK_FROM_INVALID_RING)
    " " ":ERROR_DYNLINK_FROM_INVALID_RING"
#  endif
#  if defined(ERROR_IOPL_NOT_ENABLED)
    " " ":ERROR_IOPL_NOT_ENABLED"
#  endif
#  if defined(ERROR_INVALID_SEGDPL)
    " " ":ERROR_INVALID_SEGDPL"
#  endif
#  if defined(ERROR_AUTODATASEG_EXCEEDS_64k)
    " " ":ERROR_AUTODATASEG_EXCEEDS_64K"
#  endif
#  if defined(ERROR_RING2SEG_MUST_BE_MOVABLE)
    " " ":ERROR_RING2SEG_MUST_BE_MOVABLE"
#  endif
#  if defined(ERROR_RELOC_CHAIN_XEEDS_SEGLIM)
    " " ":ERROR_RELOC_CHAIN_XEEDS_SEGLIM"
#  endif
#  if defined(ERROR_INFLOOP_IN_RELOC_CHAIN)
    " " ":ERROR_INFLOOP_IN_RELOC_CHAIN"
#  endif
#  if defined(ERROR_ENVVAR_NOT_FOUND)
    " " ":ERROR_ENVVAR_NOT_FOUND"
#  endif
#  if defined(ERROR_NO_SIGNAL_SENT)
    " " ":ERROR_NO_SIGNAL_SENT"
#  endif
#  if defined(ERROR_FILENAME_EXCED_RANGE)
    " " ":ERROR_FILENAME_EXCED_RANGE"
#  endif
#  if defined(ERROR_RING2_STACK_IN_USE)
    " " ":ERROR_RING2_STACK_IN_USE"
#  endif
#  if defined(ERROR_META_EXPANSION_TOO_LONG)
    " " ":ERROR_META_EXPANSION_TOO_LONG"
#  endif
#  if defined(ERROR_INVALID_SIGNAL_NUMBER)
    " " ":ERROR_INVALID_SIGNAL_NUMBER"
#  endif
#  if defined(ERROR_THREAD_1_INACTIVE)
    " " ":ERROR_THREAD_1_INACTIVE"
#  endif
#  if defined(ERROR_LOCKED)
    " " ":ERROR_LOCKED"
#  endif
#  if defined(ERROR_TOO_MANY_MODULES)
    " " ":ERROR_TOO_MANY_MODULES"
#  endif
#  if defined(ERROR_NESTING_NOT_ALLOWED)
    " " ":ERROR_NESTING_NOT_ALLOWED"
#  endif
#  if defined(ERROR_EXE_MACHINE_TYPE_MISMATCH)
    " " ":ERROR_EXE_MACHINE_TYPE_MISMATCH"
#  endif
#  if defined(ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY)
    " " ":ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY"
#  endif
#  if defined(ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY)
    " " ":ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY"
#  endif
#  if defined(ERROR_BAD_PIPE)
    " " ":ERROR_BAD_PIPE"
#  endif
#  if defined(ERROR_PIPE_BUSY)
    " " ":ERROR_PIPE_BUSY"
#  endif
#  if defined(ERROR_NO_DATA)
    " " ":ERROR_NO_DATA"
#  endif
#  if defined(ERROR_PIPE_NOT_CONNECTED)
    " " ":ERROR_PIPE_NOT_CONNECTED"
#  endif
#  if defined(ERROR_MORE_DATA)
    " " ":ERROR_MORE_DATA"
#  endif
#  if defined(ERROR_VC_DISCONNECTED)
    " " ":ERROR_VC_DISCONNECTED"
#  endif
#  if defined(ERROR_INVALID_EA_NAME)
    " " ":ERROR_INVALID_EA_NAME"
#  endif
#  if defined(ERROR_EA_LIST_INCONSISTENT)
    " " ":ERROR_EA_LIST_INCONSISTENT"
#  endif
#  if defined(WAIT_TIMEOUT)
    " " ":WAIT_TIMEOUT"
#  endif
#  if defined(ERROR_NO_MORE_ITEMS)
    " " ":ERROR_NO_MORE_ITEMS"
#  endif
#  if defined(ERROR_CANNOT_COPY)
    " " ":ERROR_CANNOT_COPY"
#  endif
#  if defined(ERROR_DIRECTORY)
    " " ":ERROR_DIRECTORY"
#  endif
#  if defined(ERROR_EAS_DIDNT_FIT)
    " " ":ERROR_EAS_DIDNT_FIT"
#  endif
#  if defined(ERROR_EA_FILE_CORRUPT)
    " " ":ERROR_EA_FILE_CORRUPT"
#  endif
#  if defined(ERROR_EA_TABLE_FULL)
    " " ":ERROR_EA_TABLE_FULL"
#  endif
#  if defined(ERROR_INVALID_EA_HANDLE)
    " " ":ERROR_INVALID_EA_HANDLE"
#  endif
#  if defined(ERROR_EAS_NOT_SUPPORTED)
    " " ":ERROR_EAS_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_NOT_OWNER)
    " " ":ERROR_NOT_OWNER"
#  endif
#  if defined(ERROR_TOO_MANY_POSTS)
    " " ":ERROR_TOO_MANY_POSTS"
#  endif
#  if defined(ERROR_PARTIAL_COPY)
    " " ":ERROR_PARTIAL_COPY"
#  endif
#  if defined(ERROR_OPLOCK_NOT_GRANTED)
    " " ":ERROR_OPLOCK_NOT_GRANTED"
#  endif
#  if defined(ERROR_INVALID_OPLOCK_PROTOCOL)
    " " ":ERROR_INVALID_OPLOCK_PROTOCOL"
#  endif
#  if defined(ERROR_DISK_TOO_FRAGMENTED)
    " " ":ERROR_DISK_TOO_FRAGMENTED"
#  endif
#  if defined(ERROR_DELETE_PENDING)
    " " ":ERROR_DELETE_PENDING"
#  endif
#  if defined(ERROR_MR_MID_NOT_FOUND)
    " " ":ERROR_MR_MID_NOT_FOUND"
#  endif
#  if defined(ERROR_SCOPE_NOT_FOUND)
    " " ":ERROR_SCOPE_NOT_FOUND"
#  endif
#  if defined(ERROR_INVALID_ADDRESS)
    " " ":ERROR_INVALID_ADDRESS"
#  endif
#  if defined(ERROR_ARITHMETIC_OVERFLOW)
    " " ":ERROR_ARITHMETIC_OVERFLOW"
#  endif
#  if defined(ERROR_PIPE_CONNECTED)
    " " ":ERROR_PIPE_CONNECTED"
#  endif
#  if defined(ERROR_PIPE_LISTENING)
    " " ":ERROR_PIPE_LISTENING"
#  endif
#  if defined(ERROR_EA_ACCESS_DENIED)
    " " ":ERROR_EA_ACCESS_DENIED"
#  endif
#  if defined(ERROR_OPERATION_ABORTED)
    " " ":ERROR_OPERATION_ABORTED"
#  endif
#  if defined(ERROR_IO_INCOMPLETE)
    " " ":ERROR_IO_INCOMPLETE"
#  endif
#  if defined(ERROR_IO_PENDING)
    " " ":ERROR_IO_PENDING"
#  endif
#  if defined(ERROR_NOACCESS)
    " " ":ERROR_NOACCESS"
#  endif
#  if defined(ERROR_SWAPERROR)
    " " ":ERROR_SWAPERROR"
#  endif
#  if defined(ERROR_STACK_OVERFLOW)
    " " ":ERROR_STACK_OVERFLOW"
#  endif
#  if defined(ERROR_INVALID_MESSAGE)
    " " ":ERROR_INVALID_MESSAGE"
#  endif
#  if defined(ERROR_CAN_NOT_COMPLETE)
    " " ":ERROR_CAN_NOT_COMPLETE"
#  endif
#  if defined(ERROR_INVALID_FLAGS)
    " " ":ERROR_INVALID_FLAGS"
#  endif
#  if defined(ERROR_UNRECOGNIZED_VOLUME)
    " " ":ERROR_UNRECOGNIZED_VOLUME"
#  endif
#  if defined(ERROR_FILE_INVALID)
    " " ":ERROR_FILE_INVALID"
#  endif
#  if defined(ERROR_FULLSCREEN_MODE)
    " " ":ERROR_FULLSCREEN_MODE"
#  endif
#  if defined(ERROR_NO_TOKEN)
    " " ":ERROR_NO_TOKEN"
#  endif
#  if defined(ERROR_BADDB)
    " " ":ERROR_BADDB"
#  endif
#  if defined(ERROR_BADKEY)
    " " ":ERROR_BADKEY"
#  endif
#  if defined(ERROR_CANTOPEN)
    " " ":ERROR_CANTOPEN"
#  endif
#  if defined(ERROR_CANTREAD)
    " " ":ERROR_CANTREAD"
#  endif
#  if defined(ERROR_CANTWRITE)
    " " ":ERROR_CANTWRITE"
#  endif
#  if defined(ERROR_REGISTRY_RECOVERED)
    " " ":ERROR_REGISTRY_RECOVERED"
#  endif
#  if defined(ERROR_REGISTRY_CORRUPT)
    " " ":ERROR_REGISTRY_CORRUPT"
#  endif
#  if defined(ERROR_REGISTRY_IO_FAILED)
    " " ":ERROR_REGISTRY_IO_FAILED"
#  endif
#  if defined(ERROR_NOT_REGISTRY_FILE)
    " " ":ERROR_NOT_REGISTRY_FILE"
#  endif
#  if defined(ERROR_KEY_DELETED)
    " " ":ERROR_KEY_DELETED"
#  endif
#  if defined(ERROR_NO_LOG_SPACE)
    " " ":ERROR_NO_LOG_SPACE"
#  endif
#  if defined(ERROR_KEY_HAS_CHILDREN)
    " " ":ERROR_KEY_HAS_CHILDREN"
#  endif
#  if defined(ERROR_CHILD_MUST_BE_VOLATILE)
    " " ":ERROR_CHILD_MUST_BE_VOLATILE"
#  endif
#  if defined(ERROR_NOTIFY_ENUM_DIR)
    " " ":ERROR_NOTIFY_ENUM_DIR"
#  endif
#  if defined(ERROR_DEPENDENT_SERVICES_RUNNING)
    " " ":ERROR_DEPENDENT_SERVICES_RUNNING"
#  endif
#  if defined(ERROR_INVALID_SERVICE_CONTROL)
    " " ":ERROR_INVALID_SERVICE_CONTROL"
#  endif
#  if defined(ERROR_SERVICE_REQUEST_TIMEOUT)
    " " ":ERROR_SERVICE_REQUEST_TIMEOUT"
#  endif
#  if defined(ERROR_SERVICE_NO_THREAD)
    " " ":ERROR_SERVICE_NO_THREAD"
#  endif
#  if defined(ERROR_SERVICE_DATABASE_LOCKED)
    " " ":ERROR_SERVICE_DATABASE_LOCKED"
#  endif
#  if defined(ERROR_SERVICE_ALREADY_RUNNING)
    " " ":ERROR_SERVICE_ALREADY_RUNNING"
#  endif
#  if defined(ERROR_INVALID_SERVICE_ACCOUNT)
    " " ":ERROR_INVALID_SERVICE_ACCOUNT"
#  endif
#  if defined(ERROR_SERVICE_DISABLED)
    " " ":ERROR_SERVICE_DISABLED"
#  endif
#  if defined(ERROR_CIRCULAR_DEPENDENCY)
    " " ":ERROR_CIRCULAR_DEPENDENCY"
#  endif
#  if defined(ERROR_SERVICE_DOES_NOT_EXIST)
    " " ":ERROR_SERVICE_DOES_NOT_EXIST"
#  endif
#  if defined(ERROR_SERVICE_CANNOT_ACCEPT_CTRL)
    " " ":ERROR_SERVICE_CANNOT_ACCEPT_CTRL"
#  endif
#  if defined(ERROR_SERVICE_NOT_ACTIVE)
    " " ":ERROR_SERVICE_NOT_ACTIVE"
#  endif
#  if defined(ERROR_FAILED_SERVICE_CONTROLLER_CONNECT)
    " " ":ERROR_FAILED_SERVICE_CONTROLLER_CONNECT"
#  endif
#  if defined(ERROR_EXCEPTION_IN_SERVICE)
    " " ":ERROR_EXCEPTION_IN_SERVICE"
#  endif
#  if defined(ERROR_DATABASE_DOES_NOT_EXIST)
    " " ":ERROR_DATABASE_DOES_NOT_EXIST"
#  endif
#  if defined(ERROR_SERVICE_SPECIFIC_ERROR)
    " " ":ERROR_SERVICE_SPECIFIC_ERROR"
#  endif
#  if defined(ERROR_PROCESS_ABORTED)
    " " ":ERROR_PROCESS_ABORTED"
#  endif
#  if defined(ERROR_SERVICE_DEPENDENCY_FAIL)
    " " ":ERROR_SERVICE_DEPENDENCY_FAIL"
#  endif
#  if defined(ERROR_SERVICE_LOGON_FAILED)
    " " ":ERROR_SERVICE_LOGON_FAILED"
#  endif
#  if defined(ERROR_SERVICE_START_HANG)
    " " ":ERROR_SERVICE_START_HANG"
#  endif
#  if defined(ERROR_INVALID_SERVICE_LOCK)
    " " ":ERROR_INVALID_SERVICE_LOCK"
#  endif
#  if defined(ERROR_SERVICE_MARKED_FOR_DELETE)
    " " ":ERROR_SERVICE_MARKED_FOR_DELETE"
#  endif
#  if defined(ERROR_SERVICE_EXISTS)
    " " ":ERROR_SERVICE_EXISTS"
#  endif
#  if defined(ERROR_ALREADY_RUNNING_LKG)
    " " ":ERROR_ALREADY_RUNNING_LKG"
#  endif
#  if defined(ERROR_SERVICE_DEPENDENCY_DELETED)
    " " ":ERROR_SERVICE_DEPENDENCY_DELETED"
#  endif
#  if defined(ERROR_BOOT_ALREADY_ACCEPTED)
    " " ":ERROR_BOOT_ALREADY_ACCEPTED"
#  endif
#  if defined(ERROR_SERVICE_NEVER_STARTED)
    " " ":ERROR_SERVICE_NEVER_STARTED"
#  endif
#  if defined(ERROR_DUPLICATE_SERVICE_NAME)
    " " ":ERROR_DUPLICATE_SERVICE_NAME"
#  endif
#  if defined(ERROR_DIFFERENT_SERVICE_ACCOUNT)
    " " ":ERROR_DIFFERENT_SERVICE_ACCOUNT"
#  endif
#  if defined(ERROR_CANNOT_DETECT_DRIVER_FAILURE)
    " " ":ERROR_CANNOT_DETECT_DRIVER_FAILURE"
#  endif
#  if defined(ERROR_CANNOT_DETECT_PROCESS_ABORT)
    " " ":ERROR_CANNOT_DETECT_PROCESS_ABORT"
#  endif
#  if defined(ERROR_NO_RECOVERY_PROGRAM)
    " " ":ERROR_NO_RECOVERY_PROGRAM"
#  endif
#  if defined(ERROR_SERVICE_NOT_IN_EXE)
    " " ":ERROR_SERVICE_NOT_IN_EXE"
#  endif
#  if defined(ERROR_NOT_SAFEBOOT_SERVICE)
    " " ":ERROR_NOT_SAFEBOOT_SERVICE"
#  endif
#  if defined(ERROR_END_OF_MEDIA)
    " " ":ERROR_END_OF_MEDIA"
#  endif
#  if defined(ERROR_FILEMARK_DETECTED)
    " " ":ERROR_FILEMARK_DETECTED"
#  endif
#  if defined(ERROR_BEGINNING_OF_MEDIA)
    " " ":ERROR_BEGINNING_OF_MEDIA"
#  endif
#  if defined(ERROR_SETMARK_DETECTED)
    " " ":ERROR_SETMARK_DETECTED"
#  endif
#  if defined(ERROR_NO_DATA_DETECTED)
    " " ":ERROR_NO_DATA_DETECTED"
#  endif
#  if defined(ERROR_PARTITION_FAILURE)
    " " ":ERROR_PARTITION_FAILURE"
#  endif
#  if defined(ERROR_INVALID_BLOCK_LENGTH)
    " " ":ERROR_INVALID_BLOCK_LENGTH"
#  endif
#  if defined(ERROR_DEVICE_NOT_PARTITIONED)
    " " ":ERROR_DEVICE_NOT_PARTITIONED"
#  endif
#  if defined(ERROR_UNABLE_TO_LOCK_MEDIA)
    " " ":ERROR_UNABLE_TO_LOCK_MEDIA"
#  endif
#  if defined(ERROR_UNABLE_TO_UNLOAD_MEDIA)
    " " ":ERROR_UNABLE_TO_UNLOAD_MEDIA"
#  endif
#  if defined(ERROR_MEDIA_CHANGED)
    " " ":ERROR_MEDIA_CHANGED"
#  endif
#  if defined(ERROR_BUS_RESET)
    " " ":ERROR_BUS_RESET"
#  endif
#  if defined(ERROR_NO_MEDIA_IN_DRIVE)
    " " ":ERROR_NO_MEDIA_IN_DRIVE"
#  endif
#  if defined(ERROR_NO_UNICODE_TRANSLATION)
    " " ":ERROR_NO_UNICODE_TRANSLATION"
#  endif
#  if defined(ERROR_DLL_INIT_FAILED)
    " " ":ERROR_DLL_INIT_FAILED"
#  endif
#  if defined(ERROR_SHUTDOWN_IN_PROGRESS)
    " " ":ERROR_SHUTDOWN_IN_PROGRESS"
#  endif
#  if defined(ERROR_NO_SHUTDOWN_IN_PROGRESS)
    " " ":ERROR_NO_SHUTDOWN_IN_PROGRESS"
#  endif
#  if defined(ERROR_IO_DEVICE)
    " " ":ERROR_IO_DEVICE"
#  endif
#  if defined(ERROR_SERIAL_NO_DEVICE)
    " " ":ERROR_SERIAL_NO_DEVICE"
#  endif
#  if defined(ERROR_IRQ_BUSY)
    " " ":ERROR_IRQ_BUSY"
#  endif
#  if defined(ERROR_MORE_WRITES)
    " " ":ERROR_MORE_WRITES"
#  endif
#  if defined(ERROR_COUNTER_TIMEOUT)
    " " ":ERROR_COUNTER_TIMEOUT"
#  endif
#  if defined(ERROR_FLOPPY_ID_MARK_NOT_FOUND)
    " " ":ERROR_FLOPPY_ID_MARK_NOT_FOUND"
#  endif
#  if defined(ERROR_FLOPPY_WRONG_CYLINDER)
    " " ":ERROR_FLOPPY_WRONG_CYLINDER"
#  endif
#  if defined(ERROR_FLOPPY_UNKNOWN_ERROR)
    " " ":ERROR_FLOPPY_UNKNOWN_ERROR"
#  endif
#  if defined(ERROR_FLOPPY_BAD_REGISTERS)
    " " ":ERROR_FLOPPY_BAD_REGISTERS"
#  endif
#  if defined(ERROR_DISK_RECALIBRATE_FAILED)
    " " ":ERROR_DISK_RECALIBRATE_FAILED"
#  endif
#  if defined(ERROR_DISK_OPERATION_FAILED)
    " " ":ERROR_DISK_OPERATION_FAILED"
#  endif
#  if defined(ERROR_DISK_RESET_FAILED)
    " " ":ERROR_DISK_RESET_FAILED"
#  endif
#  if defined(ERROR_EOM_OVERFLOW)
    " " ":ERROR_EOM_OVERFLOW"
#  endif
#  if defined(ERROR_NOT_ENOUGH_SERVER_MEMORY)
    " " ":ERROR_NOT_ENOUGH_SERVER_MEMORY"
#  endif
#  if defined(ERROR_POSSIBLE_DEADLOCK)
    " " ":ERROR_POSSIBLE_DEADLOCK"
#  endif
#  if defined(ERROR_MAPPED_ALIGNMENT)
    " " ":ERROR_MAPPED_ALIGNMENT"
#  endif
#  if defined(ERROR_SET_POWER_STATE_VETOED)
    " " ":ERROR_SET_POWER_STATE_VETOED"
#  endif
#  if defined(ERROR_SET_POWER_STATE_FAILED)
    " " ":ERROR_SET_POWER_STATE_FAILED"
#  endif
#  if defined(ERROR_TOO_MANY_LINKS)
    " " ":ERROR_TOO_MANY_LINKS"
#  endif
#  if defined(ERROR_OLD_WIN_VERSION)
    " " ":ERROR_OLD_WIN_VERSION"
#  endif
#  if defined(ERROR_APP_WRONG_OS)
    " " ":ERROR_APP_WRONG_OS"
#  endif
#  if defined(ERROR_SINGLE_INSTANCE_APP)
    " " ":ERROR_SINGLE_INSTANCE_APP"
#  endif
#  if defined(ERROR_RMODE_APP)
    " " ":ERROR_RMODE_APP"
#  endif
#  if defined(ERROR_INVALID_DLL)
    " " ":ERROR_INVALID_DLL"
#  endif
#  if defined(ERROR_NO_ASSOCIATION)
    " " ":ERROR_NO_ASSOCIATION"
#  endif
#  if defined(ERROR_DDE_FAIL)
    " " ":ERROR_DDE_FAIL"
#  endif
#  if defined(ERROR_DLL_NOT_FOUND)
    " " ":ERROR_DLL_NOT_FOUND"
#  endif
#  if defined(ERROR_NO_MORE_USER_HANDLES)
    " " ":ERROR_NO_MORE_USER_HANDLES"
#  endif
#  if defined(ERROR_MESSAGE_SYNC_ONLY)
    " " ":ERROR_MESSAGE_SYNC_ONLY"
#  endif
#  if defined(ERROR_SOURCE_ELEMENT_EMPTY)
    " " ":ERROR_SOURCE_ELEMENT_EMPTY"
#  endif
#  if defined(ERROR_DESTINATION_ELEMENT_FULL)
    " " ":ERROR_DESTINATION_ELEMENT_FULL"
#  endif
#  if defined(ERROR_ILLEGAL_ELEMENT_ADDRESS)
    " " ":ERROR_ILLEGAL_ELEMENT_ADDRESS"
#  endif
#  if defined(ERROR_MAGAZINE_NOT_PRESENT)
    " " ":ERROR_MAGAZINE_NOT_PRESENT"
#  endif
#  if defined(ERROR_DEVICE_REINITIALIZATION_NEEDED)
    " " ":ERROR_DEVICE_REINITIALIZATION_NEEDED"
#  endif
#  if defined(ERROR_DEVICE_REQUIRES_CLEANING)
    " " ":ERROR_DEVICE_REQUIRES_CLEANING"
#  endif
#  if defined(ERROR_DEVICE_DOOR_OPEN)
    " " ":ERROR_DEVICE_DOOR_OPEN"
#  endif
#  if defined(ERROR_DEVICE_NOT_CONNECTED)
    " " ":ERROR_DEVICE_NOT_CONNECTED"
#  endif
#  if defined(ERROR_NOT_FOUND)
    " " ":ERROR_NOT_FOUND"
#  endif
#  if defined(ERROR_NO_MATCH)
    " " ":ERROR_NO_MATCH"
#  endif
#  if defined(ERROR_SET_NOT_FOUND)
    " " ":ERROR_SET_NOT_FOUND"
#  endif
#  if defined(ERROR_POINT_NOT_FOUND)
    " " ":ERROR_POINT_NOT_FOUND"
#  endif
#  if defined(ERROR_NO_TRACKING_SERVICE)
    " " ":ERROR_NO_TRACKING_SERVICE"
#  endif
#  if defined(ERROR_NO_VOLUME_ID)
    " " ":ERROR_NO_VOLUME_ID"
#  endif
#  if defined(ERROR_UNABLE_TO_REMOVE_REPLACED)
    " " ":ERROR_UNABLE_TO_REMOVE_REPLACED"
#  endif
#  if defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT)
    " " ":ERROR_UNABLE_TO_MOVE_REPLACEMENT"
#  endif
#  if defined(ERROR_UNABLE_TO_MOVE_REPLACEMENT_2)
    " " ":ERROR_UNABLE_TO_MOVE_REPLACEMENT_2"
#  endif
#  if defined(ERROR_JOURNAL_DELETE_IN_PROGRESS)
    " " ":ERROR_JOURNAL_DELETE_IN_PROGRESS"
#  endif
#  if defined(ERROR_JOURNAL_NOT_ACTIVE)
    " " ":ERROR_JOURNAL_NOT_ACTIVE"
#  endif
#  if defined(ERROR_POTENTIAL_FILE_FOUND)
    " " ":ERROR_POTENTIAL_FILE_FOUND"
#  endif
#  if defined(ERROR_JOURNAL_ENTRY_DELETED)
    " " ":ERROR_JOURNAL_ENTRY_DELETED"
#  endif
#  if defined(ERROR_BAD_DEVICE)
    " " ":ERROR_BAD_DEVICE"
#  endif
#  if defined(ERROR_CONNECTION_UNAVAIL)
    " " ":ERROR_CONNECTION_UNAVAIL"
#  endif
#  if defined(ERROR_DEVICE_ALREADY_REMEMBERED)
    " " ":ERROR_DEVICE_ALREADY_REMEMBERED"
#  endif
#  if defined(ERROR_NO_NET_OR_BAD_PATH)
    " " ":ERROR_NO_NET_OR_BAD_PATH"
#  endif
#  if defined(ERROR_BAD_PROVIDER)
    " " ":ERROR_BAD_PROVIDER"
#  endif
#  if defined(ERROR_CANNOT_OPEN_PROFILE)
    " " ":ERROR_CANNOT_OPEN_PROFILE"
#  endif
#  if defined(ERROR_BAD_PROFILE)
    " " ":ERROR_BAD_PROFILE"
#  endif
#  if defined(ERROR_NOT_CONTAINER)
    " " ":ERROR_NOT_CONTAINER"
#  endif
#  if defined(ERROR_EXTENDED_ERROR)
    " " ":ERROR_EXTENDED_ERROR"
#  endif
#  if defined(ERROR_INVALID_GROUPNAME)
    " " ":ERROR_INVALID_GROUPNAME"
#  endif
#  if defined(ERROR_INVALID_COMPUTERNAME)
    " " ":ERROR_INVALID_COMPUTERNAME"
#  endif
#  if defined(ERROR_INVALID_EVENTNAME)
    " " ":ERROR_INVALID_EVENTNAME"
#  endif
#  if defined(ERROR_INVALID_DOMAINNAME)
    " " ":ERROR_INVALID_DOMAINNAME"
#  endif
#  if defined(ERROR_INVALID_SERVICENAME)
    " " ":ERROR_INVALID_SERVICENAME"
#  endif
#  if defined(ERROR_INVALID_NETNAME)
    " " ":ERROR_INVALID_NETNAME"
#  endif
#  if defined(ERROR_INVALID_SHARENAME)
    " " ":ERROR_INVALID_SHARENAME"
#  endif
#  if defined(ERROR_INVALID_PASSWORDNAME)
    " " ":ERROR_INVALID_PASSWORDNAME"
#  endif
#  if defined(ERROR_INVALID_MESSAGENAME)
    " " ":ERROR_INVALID_MESSAGENAME"
#  endif
#  if defined(ERROR_INVALID_MESSAGEDEST)
    " " ":ERROR_INVALID_MESSAGEDEST"
#  endif
#  if defined(ERROR_SESSION_CREDENTIAL_CONFLICT)
    " " ":ERROR_SESSION_CREDENTIAL_CONFLICT"
#  endif
#  if defined(ERROR_REMOTE_SESSION_LIMIT_EXCEEDED)
    " " ":ERROR_REMOTE_SESSION_LIMIT_EXCEEDED"
#  endif
#  if defined(ERROR_DUP_DOMAINNAME)
    " " ":ERROR_DUP_DOMAINNAME"
#  endif
#  if defined(ERROR_NO_NETWORK)
    " " ":ERROR_NO_NETWORK"
#  endif
#  if defined(ERROR_CANCELLED)
    " " ":ERROR_CANCELLED"
#  endif
#  if defined(ERROR_USER_MAPPED_FILE)
    " " ":ERROR_USER_MAPPED_FILE"
#  endif
#  if defined(ERROR_CONNECTION_REFUSED)
    " " ":ERROR_CONNECTION_REFUSED"
#  endif
#  if defined(ERROR_GRACEFUL_DISCONNECT)
    " " ":ERROR_GRACEFUL_DISCONNECT"
#  endif
#  if defined(ERROR_ADDRESS_ALREADY_ASSOCIATED)
    " " ":ERROR_ADDRESS_ALREADY_ASSOCIATED"
#  endif
#  if defined(ERROR_ADDRESS_NOT_ASSOCIATED)
    " " ":ERROR_ADDRESS_NOT_ASSOCIATED"
#  endif
#  if defined(ERROR_CONNECTION_INVALID)
    " " ":ERROR_CONNECTION_INVALID"
#  endif
#  if defined(ERROR_CONNECTION_ACTIVE)
    " " ":ERROR_CONNECTION_ACTIVE"
#  endif
#  if defined(ERROR_NETWORK_UNREACHABLE)
    " " ":ERROR_NETWORK_UNREACHABLE"
#  endif
#  if defined(ERROR_HOST_UNREACHABLE)
    " " ":ERROR_HOST_UNREACHABLE"
#  endif
#  if defined(ERROR_PROTOCOL_UNREACHABLE)
    " " ":ERROR_PROTOCOL_UNREACHABLE"
#  endif
#  if defined(ERROR_PORT_UNREACHABLE)
    " " ":ERROR_PORT_UNREACHABLE"
#  endif
#  if defined(ERROR_REQUEST_ABORTED)
    " " ":ERROR_REQUEST_ABORTED"
#  endif
#  if defined(ERROR_CONNECTION_ABORTED)
    " " ":ERROR_CONNECTION_ABORTED"
#  endif
#  if defined(ERROR_RETRY)
    " " ":ERROR_RETRY"
#  endif
#  if defined(ERROR_CONNECTION_COUNT_LIMIT)
    " " ":ERROR_CONNECTION_COUNT_LIMIT"
#  endif
#  if defined(ERROR_LOGIN_TIME_RESTRICTION)
    " " ":ERROR_LOGIN_TIME_RESTRICTION"
#  endif
#  if defined(ERROR_LOGIN_WKSTA_RESTRICTION)
    " " ":ERROR_LOGIN_WKSTA_RESTRICTION"
#  endif
#  if defined(ERROR_INCORRECT_ADDRESS)
    " " ":ERROR_INCORRECT_ADDRESS"
#  endif
#  if defined(ERROR_ALREADY_REGISTERED)
    " " ":ERROR_ALREADY_REGISTERED"
#  endif
#  if defined(ERROR_SERVICE_NOT_FOUND)
    " " ":ERROR_SERVICE_NOT_FOUND"
#  endif
#  if defined(ERROR_NOT_AUTHENTICATED)
    " " ":ERROR_NOT_AUTHENTICATED"
#  endif
#  if defined(ERROR_NOT_LOGGED_ON)
    " " ":ERROR_NOT_LOGGED_ON"
#  endif
#  if defined(ERROR_CONTINUE)
    " " ":ERROR_CONTINUE"
#  endif
#  if defined(ERROR_ALREADY_INITIALIZED)
    " " ":ERROR_ALREADY_INITIALIZED"
#  endif
#  if defined(ERROR_NO_MORE_DEVICES)
    " " ":ERROR_NO_MORE_DEVICES"
#  endif
#  if defined(ERROR_NO_SUCH_SITE)
    " " ":ERROR_NO_SUCH_SITE"
#  endif
#  if defined(ERROR_DOMAIN_CONTROLLER_EXISTS)
    " " ":ERROR_DOMAIN_CONTROLLER_EXISTS"
#  endif
#  if defined(ERROR_ONLY_IF_CONNECTED)
    " " ":ERROR_ONLY_IF_CONNECTED"
#  endif
#  if defined(ERROR_OVERRIDE_NOCHANGES)
    " " ":ERROR_OVERRIDE_NOCHANGES"
#  endif
#  if defined(ERROR_BAD_USER_PROFILE)
    " " ":ERROR_BAD_USER_PROFILE"
#  endif
#  if defined(ERROR_NOT_SUPPORTED_ON_SBS)
    " " ":ERROR_NOT_SUPPORTED_ON_SBS"
#  endif
#  if defined(ERROR_SERVER_SHUTDOWN_IN_PROGRESS)
    " " ":ERROR_SERVER_SHUTDOWN_IN_PROGRESS"
#  endif
#  if defined(ERROR_HOST_DOWN)
    " " ":ERROR_HOST_DOWN"
#  endif
#  if defined(ERROR_NON_ACCOUNT_SID)
    " " ":ERROR_NON_ACCOUNT_SID"
#  endif
#  if defined(ERROR_NON_DOMAIN_SID)
    " " ":ERROR_NON_DOMAIN_SID"
#  endif
#  if defined(ERROR_APPHELP_BLOCK)
    " " ":ERROR_APPHELP_BLOCK"
#  endif
#  if defined(ERROR_ACCESS_DISABLED_BY_POLICY)
    " " ":ERROR_ACCESS_DISABLED_BY_POLICY"
#  endif
#  if defined(ERROR_REG_NAT_CONSUMPTION)
    " " ":ERROR_REG_NAT_CONSUMPTION"
#  endif
#  if defined(ERROR_CSCSHARE_OFFLINE)
    " " ":ERROR_CSCSHARE_OFFLINE"
#  endif
#  if defined(ERROR_PKINIT_FAILURE)
    " " ":ERROR_PKINIT_FAILURE"
#  endif
#  if defined(ERROR_SMARTCARD_SUBSYSTEM_FAILURE)
    " " ":ERROR_SMARTCARD_SUBSYSTEM_FAILURE"
#  endif
#  if defined(ERROR_DOWNGRADE_DETECTED)
    " " ":ERROR_DOWNGRADE_DETECTED"
#  endif
#  if defined(SEC_E_SMARTCARD_CERT_REVOKED)
    " " ":SEC_E_SMARTCARD_CERT_REVOKED"
#  endif
#  if defined(SEC_E_ISSUING_CA_UNTRUSTED)
    " " ":SEC_E_ISSUING_CA_UNTRUSTED"
#  endif
#  if defined(SEC_E_REVOCATION_OFFLINE_C)
    " " ":SEC_E_REVOCATION_OFFLINE_C"
#  endif
#  if defined(SEC_E_PKINIT_CLIENT_FAILUR)
    " " ":SEC_E_PKINIT_CLIENT_FAILUR"
#  endif
#  if defined(SEC_E_SMARTCARD_CERT_EXPIRED)
    " " ":SEC_E_SMARTCARD_CERT_EXPIRED"
#  endif
#  if defined(ERROR_MACHINE_LOCKED)
    " " ":ERROR_MACHINE_LOCKED"
#  endif
#  if defined(ERROR_CALLBACK_SUPPLIED_INVALID_DATA)
    " " ":ERROR_CALLBACK_SUPPLIED_INVALID_DATA"
#  endif
#  if defined(ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED)
    " " ":ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED"
#  endif
#  if defined(ERROR_DRIVER_BLOCKED)
    " " ":ERROR_DRIVER_BLOCKED"
#  endif
#  if defined(ERROR_INVALID_IMPORT_OF_NON_DLL)
    " " ":ERROR_INVALID_IMPORT_OF_NON_DLL"
#  endif
#  if defined(ERROR_ACCESS_DISABLED_WEBBLADE)
    " " ":ERROR_ACCESS_DISABLED_WEBBLADE"
#  endif
#  if defined(ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER)
    " " ":ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER"
#  endif
#  if defined(ERROR_RECOVERY_FAILURE)
    " " ":ERROR_RECOVERY_FAILURE"
#  endif
#  if defined(ERROR_ALREADY_FIBER)
    " " ":ERROR_ALREADY_FIBER"
#  endif
#  if defined(ERROR_ALREADY_THREAD)
    " " ":ERROR_ALREADY_THREAD"
#  endif
#  if defined(ERROR_STACK_BUFFER_OVERRUN)
    " " ":ERROR_STACK_BUFFER_OVERRUN"
#  endif
#  if defined(ERROR_PARAMETER_QUOTA_EXCEEDED)
    " " ":ERROR_PARAMETER_QUOTA_EXCEEDED"
#  endif
#  if defined(ERROR_DEBUGGER_INACTIVE)
    " " ":ERROR_DEBUGGER_INACTIVE"
#  endif
#  if defined(ERROR_NOT_ALL_ASSIGNED)
    " " ":ERROR_NOT_ALL_ASSIGNED"
#  endif
#  if defined(ERROR_SOME_NOT_MAPPED)
    " " ":ERROR_SOME_NOT_MAPPED"
#  endif
#  if defined(ERROR_NO_QUOTAS_FOR_ACCOUNT)
    " " ":ERROR_NO_QUOTAS_FOR_ACCOUNT"
#  endif
#  if defined(ERROR_LOCAL_USER_SESSION_KEY)
    " " ":ERROR_LOCAL_USER_SESSION_KEY"
#  endif
#  if defined(ERROR_NULL_LM_PASSWORD)
    " " ":ERROR_NULL_LM_PASSWORD"
#  endif
#  if defined(ERROR_UNKNOWN_REVISION)
    " " ":ERROR_UNKNOWN_REVISION"
#  endif
#  if defined(ERROR_REVISION_MISMATCH)
    " " ":ERROR_REVISION_MISMATCH"
#  endif
#  if defined(ERROR_INVALID_OWNER)
    " " ":ERROR_INVALID_OWNER"
#  endif
#  if defined(ERROR_INVALID_PRIMARY_GROUP)
    " " ":ERROR_INVALID_PRIMARY_GROUP"
#  endif
#  if defined(ERROR_NO_IMPERSONATION_TOKEN)
    " " ":ERROR_NO_IMPERSONATION_TOKEN"
#  endif
#  if defined(ERROR_CANT_DISABLE_MANDATORY)
    " " ":ERROR_CANT_DISABLE_MANDATORY"
#  endif
#  if defined(ERROR_NO_LOGON_SERVERS)
    " " ":ERROR_NO_LOGON_SERVERS"
#  endif
#  if defined(ERROR_NO_SUCH_LOGON_SESSION)
    " " ":ERROR_NO_SUCH_LOGON_SESSION"
#  endif
#  if defined(ERROR_NO_SUCH_PRIVILEGE)
    " " ":ERROR_NO_SUCH_PRIVILEGE"
#  endif
#  if defined(ERROR_PRIVILEGE_NOT_HELD)
    " " ":ERROR_PRIVILEGE_NOT_HELD"
#  endif
#  if defined(ERROR_INVALID_ACCOUNT_NAME)
    " " ":ERROR_INVALID_ACCOUNT_NAME"
#  endif
#  if defined(ERROR_USER_EXISTS)
    " " ":ERROR_USER_EXISTS"
#  endif
#  if defined(ERROR_NO_SUCH_USER)
    " " ":ERROR_NO_SUCH_USER"
#  endif
#  if defined(ERROR_GROUP_EXISTS)
    " " ":ERROR_GROUP_EXISTS"
#  endif
#  if defined(ERROR_NO_SUCH_GROUP)
    " " ":ERROR_NO_SUCH_GROUP"
#  endif
#  if defined(ERROR_MEMBER_IN_GROUP)
    " " ":ERROR_MEMBER_IN_GROUP"
#  endif
#  if defined(ERROR_MEMBER_NOT_IN_GROUP)
    " " ":ERROR_MEMBER_NOT_IN_GROUP"
#  endif
#  if defined(ERROR_LAST_ADMIN)
    " " ":ERROR_LAST_ADMIN"
#  endif
#  if defined(ERROR_WRONG_PASSWORD)
    " " ":ERROR_WRONG_PASSWORD"
#  endif
#  if defined(ERROR_ILL_FORMED_PASSWORD)
    " " ":ERROR_ILL_FORMED_PASSWORD"
#  endif
#  if defined(ERROR_PASSWORD_RESTRICTION)
    " " ":ERROR_PASSWORD_RESTRICTION"
#  endif
#  if defined(ERROR_LOGON_FAILURE)
    " " ":ERROR_LOGON_FAILURE"
#  endif
#  if defined(ERROR_ACCOUNT_RESTRICTION)
    " " ":ERROR_ACCOUNT_RESTRICTION"
#  endif
#  if defined(ERROR_INVALID_LOGON_HOURS)
    " " ":ERROR_INVALID_LOGON_HOURS"
#  endif
#  if defined(ERROR_INVALID_WORKSTATION)
    " " ":ERROR_INVALID_WORKSTATION"
#  endif
#  if defined(ERROR_PASSWORD_EXPIRED)
    " " ":ERROR_PASSWORD_EXPIRED"
#  endif
#  if defined(ERROR_ACCOUNT_DISABLED)
    " " ":ERROR_ACCOUNT_DISABLED"
#  endif
#  if defined(ERROR_NONE_MAPPED)
    " " ":ERROR_NONE_MAPPED"
#  endif
#  if defined(ERROR_TOO_MANY_LUIDS_REQUESTED)
    " " ":ERROR_TOO_MANY_LUIDS_REQUESTED"
#  endif
#  if defined(ERROR_LUIDS_EXHAUSTED)
    " " ":ERROR_LUIDS_EXHAUSTED"
#  endif
#  if defined(ERROR_INVALID_SUB_AUTHORITY)
    " " ":ERROR_INVALID_SUB_AUTHORITY"
#  endif
#  if defined(ERROR_INVALID_ACL)
    " " ":ERROR_INVALID_ACL"
#  endif
#  if defined(ERROR_INVALID_SID)
    " " ":ERROR_INVALID_SID"
#  endif
#  if defined(ERROR_INVALID_SECURITY_DESCR)
    " " ":ERROR_INVALID_SECURITY_DESCR"
#  endif
#  if defined(ERROR_BAD_INHERITANCE_ACL)
    " " ":ERROR_BAD_INHERITANCE_ACL"
#  endif
#  if defined(ERROR_SERVER_DISABLED)
    " " ":ERROR_SERVER_DISABLED"
#  endif
#  if defined(ERROR_SERVER_NOT_DISABLED)
    " " ":ERROR_SERVER_NOT_DISABLED"
#  endif
#  if defined(ERROR_INVALID_ID_AUTHORITY)
    " " ":ERROR_INVALID_ID_AUTHORITY"
#  endif
#  if defined(ERROR_ALLOTTED_SPACE_EXCEEDED)
    " " ":ERROR_ALLOTTED_SPACE_EXCEEDED"
#  endif
#  if defined(ERROR_INVALID_GROUP_ATTRIBUTES)
    " " ":ERROR_INVALID_GROUP_ATTRIBUTES"
#  endif
#  if defined(ERROR_BAD_IMPERSONATION_LEVEL)
    " " ":ERROR_BAD_IMPERSONATION_LEVEL"
#  endif
#  if defined(ERROR_CANT_OPEN_ANONYMOUS)
    " " ":ERROR_CANT_OPEN_ANONYMOUS"
#  endif
#  if defined(ERROR_BAD_VALIDATION_CLASS)
    " " ":ERROR_BAD_VALIDATION_CLASS"
#  endif
#  if defined(ERROR_BAD_TOKEN_TYPE)
    " " ":ERROR_BAD_TOKEN_TYPE"
#  endif
#  if defined(ERROR_NO_SECURITY_ON_OBJECT)
    " " ":ERROR_NO_SECURITY_ON_OBJECT"
#  endif
#  if defined(ERROR_CANT_ACCESS_DOMAIN_INFO)
    " " ":ERROR_CANT_ACCESS_DOMAIN_INFO"
#  endif
#  if defined(ERROR_INVALID_SERVER_STATE)
    " " ":ERROR_INVALID_SERVER_STATE"
#  endif
#  if defined(ERROR_INVALID_DOMAIN_STATE)
    " " ":ERROR_INVALID_DOMAIN_STATE"
#  endif
#  if defined(ERROR_INVALID_DOMAIN_ROLE)
    " " ":ERROR_INVALID_DOMAIN_ROLE"
#  endif
#  if defined(ERROR_NO_SUCH_DOMAIN)
    " " ":ERROR_NO_SUCH_DOMAIN"
#  endif
#  if defined(ERROR_DOMAIN_EXISTS)
    " " ":ERROR_DOMAIN_EXISTS"
#  endif
#  if defined(ERROR_DOMAIN_LIMIT_EXCEEDED)
    " " ":ERROR_DOMAIN_LIMIT_EXCEEDED"
#  endif
#  if defined(ERROR_INTERNAL_DB_CORRUPTION)
    " " ":ERROR_INTERNAL_DB_CORRUPTION"
#  endif
#  if defined(ERROR_INTERNAL_ERROR)
    " " ":ERROR_INTERNAL_ERROR"
#  endif
#  if defined(ERROR_GENERIC_NOT_MAPPED)
    " " ":ERROR_GENERIC_NOT_MAPPED"
#  endif
#  if defined(ERROR_BAD_DESCRIPTOR_FORMAT)
    " " ":ERROR_BAD_DESCRIPTOR_FORMAT"
#  endif
#  if defined(ERROR_NOT_LOGON_PROCESS)
    " " ":ERROR_NOT_LOGON_PROCESS"
#  endif
#  if defined(ERROR_LOGON_SESSION_EXISTS)
    " " ":ERROR_LOGON_SESSION_EXISTS"
#  endif
#  if defined(ERROR_NO_SUCH_PACKAGE)
    " " ":ERROR_NO_SUCH_PACKAGE"
#  endif
#  if defined(ERROR_BAD_LOGON_SESSION_STATE)
    " " ":ERROR_BAD_LOGON_SESSION_STATE"
#  endif
#  if defined(ERROR_LOGON_SESSION_COLLISION)
    " " ":ERROR_LOGON_SESSION_COLLISION"
#  endif
#  if defined(ERROR_INVALID_LOGON_TYPE)
    " " ":ERROR_INVALID_LOGON_TYPE"
#  endif
#  if defined(ERROR_CANNOT_IMPERSONATE)
    " " ":ERROR_CANNOT_IMPERSONATE"
#  endif
#  if defined(ERROR_RXACT_INVALID_STATE)
    " " ":ERROR_RXACT_INVALID_STATE"
#  endif
#  if defined(ERROR_RXACT_COMMIT_FAILURE)
    " " ":ERROR_RXACT_COMMIT_FAILURE"
#  endif
#  if defined(ERROR_SPECIAL_ACCOUNT)
    " " ":ERROR_SPECIAL_ACCOUNT"
#  endif
#  if defined(ERROR_SPECIAL_GROUP)
    " " ":ERROR_SPECIAL_GROUP"
#  endif
#  if defined(ERROR_SPECIAL_USER)
    " " ":ERROR_SPECIAL_USER"
#  endif
#  if defined(ERROR_MEMBERS_PRIMARY_GROUP)
    " " ":ERROR_MEMBERS_PRIMARY_GROUP"
#  endif
#  if defined(ERROR_TOKEN_ALREADY_IN_USE)
    " " ":ERROR_TOKEN_ALREADY_IN_USE"
#  endif
#  if defined(ERROR_NO_SUCH_ALIAS)
    " " ":ERROR_NO_SUCH_ALIAS"
#  endif
#  if defined(ERROR_MEMBER_NOT_IN_ALIAS)
    " " ":ERROR_MEMBER_NOT_IN_ALIAS"
#  endif
#  if defined(ERROR_MEMBER_IN_ALIAS)
    " " ":ERROR_MEMBER_IN_ALIAS"
#  endif
#  if defined(ERROR_ALIAS_EXISTS)
    " " ":ERROR_ALIAS_EXISTS"
#  endif
#  if defined(ERROR_LOGON_NOT_GRANTED)
    " " ":ERROR_LOGON_NOT_GRANTED"
#  endif
#  if defined(ERROR_TOO_MANY_SECRETS)
    " " ":ERROR_TOO_MANY_SECRETS"
#  endif
#  if defined(ERROR_SECRET_TOO_LONG)
    " " ":ERROR_SECRET_TOO_LONG"
#  endif
#  if defined(ERROR_INTERNAL_DB_ERROR)
    " " ":ERROR_INTERNAL_DB_ERROR"
#  endif
#  if defined(ERROR_TOO_MANY_CONTEXT_IDS)
    " " ":ERROR_TOO_MANY_CONTEXT_IDS"
#  endif
#  if defined(ERROR_LOGON_TYPE_NOT_GRANTED)
    " " ":ERROR_LOGON_TYPE_NOT_GRANTED"
#  endif
#  if defined(ERROR_NT_CROSS_ENCRYPTION_REQUIRED)
    " " ":ERROR_NT_CROSS_ENCRYPTION_REQUIRED"
#  endif
#  if defined(ERROR_NO_SUCH_MEMBER)
    " " ":ERROR_NO_SUCH_MEMBER"
#  endif
#  if defined(ERROR_INVALID_MEMBER)
    " " ":ERROR_INVALID_MEMBER"
#  endif
#  if defined(ERROR_TOO_MANY_SIDS)
    " " ":ERROR_TOO_MANY_SIDS"
#  endif
#  if defined(ERROR_LM_CROSS_ENCRYPTION_REQUIRED)
    " " ":ERROR_LM_CROSS_ENCRYPTION_REQUIRED"
#  endif
#  if defined(ERROR_NO_INHERITANCE)
    " " ":ERROR_NO_INHERITANCE"
#  endif
#  if defined(ERROR_FILE_CORRUPT)
    " " ":ERROR_FILE_CORRUPT"
#  endif
#  if defined(ERROR_DISK_CORRUPT)
    " " ":ERROR_DISK_CORRUPT"
#  endif
#  if defined(ERROR_NO_USER_SESSION_KEY)
    " " ":ERROR_NO_USER_SESSION_KEY"
#  endif
#  if defined(ERROR_LICENSE_QUOTA_EXCEEDED)
    " " ":ERROR_LICENSE_QUOTA_EXCEEDED"
#  endif
#  if defined(ERROR_WRONG_TARGET_NAME)
    " " ":ERROR_WRONG_TARGET_NAME"
#  endif
#  if defined(ERROR_MUTUAL_AUTH_FAILED)
    " " ":ERROR_MUTUAL_AUTH_FAILED"
#  endif
#  if defined(ERROR_TIME_SKEW)
    " " ":ERROR_TIME_SKEW"
#  endif
#  if defined(ERROR_CURRENT_DOMAIN_NOT_ALLOWED)
    " " ":ERROR_CURRENT_DOMAIN_NOT_ALLOWED"
#  endif
#  if defined(ERROR_INVALID_WINDOW_HANDLE)
    " " ":ERROR_INVALID_WINDOW_HANDLE"
#  endif
#  if defined(ERROR_INVALID_MENU_HANDLE)
    " " ":ERROR_INVALID_MENU_HANDLE"
#  endif
#  if defined(ERROR_INVALID_CURSOR_HANDLE)
    " " ":ERROR_INVALID_CURSOR_HANDLE"
#  endif
#  if defined(ERROR_INVALID_ACCEL_HANDLE)
    " " ":ERROR_INVALID_ACCEL_HANDLE"
#  endif
#  if defined(ERROR_INVALID_HOOK_HANDLE)
    " " ":ERROR_INVALID_HOOK_HANDLE"
#  endif
#  if defined(ERROR_INVALID_DWP_HANDLE)
    " " ":ERROR_INVALID_DWP_HANDLE"
#  endif
#  if defined(ERROR_TLW_WITH_WSCHILD)
    " " ":ERROR_TLW_WITH_WSCHILD"
#  endif
#  if defined(ERROR_CANNOT_FIND_WND_CLASS)
    " " ":ERROR_CANNOT_FIND_WND_CLASS"
#  endif
#  if defined(ERROR_WINDOW_OF_OTHER_THREAD)
    " " ":ERROR_WINDOW_OF_OTHER_THREAD"
#  endif
#  if defined(ERROR_HOTKEY_ALREADY_REGISTERED)
    " " ":ERROR_HOTKEY_ALREADY_REGISTERED"
#  endif
#  if defined(ERROR_CLASS_ALREADY_EXISTS)
    " " ":ERROR_CLASS_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_CLASS_DOES_NOT_EXIST)
    " " ":ERROR_CLASS_DOES_NOT_EXIST"
#  endif
#  if defined(ERROR_CLASS_HAS_WINDOWS)
    " " ":ERROR_CLASS_HAS_WINDOWS"
#  endif
#  if defined(ERROR_INVALID_INDEX)
    " " ":ERROR_INVALID_INDEX"
#  endif
#  if defined(ERROR_INVALID_ICON_HANDLE)
    " " ":ERROR_INVALID_ICON_HANDLE"
#  endif
#  if defined(ERROR_PRIVATE_DIALOG_INDEX)
    " " ":ERROR_PRIVATE_DIALOG_INDEX"
#  endif
#  if defined(ERROR_LISTBOX_ID_NOT_FOUND)
    " " ":ERROR_LISTBOX_ID_NOT_FOUND"
#  endif
#  if defined(ERROR_NO_WILDCARD_CHARACTERS)
    " " ":ERROR_NO_WILDCARD_CHARACTERS"
#  endif
#  if defined(ERROR_CLIPBOARD_NOT_OPEN)
    " " ":ERROR_CLIPBOARD_NOT_OPEN"
#  endif
#  if defined(ERROR_HOTKEY_NOT_REGISTERED)
    " " ":ERROR_HOTKEY_NOT_REGISTERED"
#  endif
#  if defined(ERROR_WINDOW_NOT_DIALOG)
    " " ":ERROR_WINDOW_NOT_DIALOG"
#  endif
#  if defined(ERROR_CONTROL_ID_NOT_FOUND)
    " " ":ERROR_CONTROL_ID_NOT_FOUND"
#  endif
#  if defined(ERROR_INVALID_COMBOBOX_MESSAGE)
    " " ":ERROR_INVALID_COMBOBOX_MESSAGE"
#  endif
#  if defined(ERROR_WINDOW_NOT_COMBOBOX)
    " " ":ERROR_WINDOW_NOT_COMBOBOX"
#  endif
#  if defined(ERROR_INVALID_EDIT_HEIGHT)
    " " ":ERROR_INVALID_EDIT_HEIGHT"
#  endif
#  if defined(ERROR_DC_NOT_FOUND)
    " " ":ERROR_DC_NOT_FOUND"
#  endif
#  if defined(ERROR_INVALID_HOOK_FILTER)
    " " ":ERROR_INVALID_HOOK_FILTER"
#  endif
#  if defined(ERROR_INVALID_FILTER_PROC)
    " " ":ERROR_INVALID_FILTER_PROC"
#  endif
#  if defined(ERROR_HOOK_NEEDS_HMOD)
    " " ":ERROR_HOOK_NEEDS_HMOD"
#  endif
#  if defined(ERROR_GLOBAL_ONLY_HOOK)
    " " ":ERROR_GLOBAL_ONLY_HOOK"
#  endif
#  if defined(ERROR_JOURNAL_HOOK_SET)
    " " ":ERROR_JOURNAL_HOOK_SET"
#  endif
#  if defined(ERROR_HOOK_NOT_INSTALLED)
    " " ":ERROR_HOOK_NOT_INSTALLED"
#  endif
#  if defined(ERROR_INVALID_LB_MESSAGE)
    " " ":ERROR_INVALID_LB_MESSAGE"
#  endif
#  if defined(ERROR_SETCOUNT_ON_BAD_LB)
    " " ":ERROR_SETCOUNT_ON_BAD_LB"
#  endif
#  if defined(ERROR_LB_WITHOUT_TABSTOPS)
    " " ":ERROR_LB_WITHOUT_TABSTOPS"
#  endif
#  if defined(ERROR_DESTROY_OBJECT_OF_OTHER_THREAD)
    " " ":ERROR_DESTROY_OBJECT_OF_OTHER_THREAD"
#  endif
#  if defined(ERROR_CHILD_WINDOW_MENU)
    " " ":ERROR_CHILD_WINDOW_MENU"
#  endif
#  if defined(ERROR_NO_SYSTEM_MENU)
    " " ":ERROR_NO_SYSTEM_MENU"
#  endif
#  if defined(ERROR_INVALID_MSGBOX_STYLE)
    " " ":ERROR_INVALID_MSGBOX_STYLE"
#  endif
#  if defined(ERROR_INVALID_SPI_VALUE)
    " " ":ERROR_INVALID_SPI_VALUE"
#  endif
#  if defined(ERROR_SCREEN_ALREADY_LOCKED)
    " " ":ERROR_SCREEN_ALREADY_LOCKED"
#  endif
#  if defined(ERROR_HWNDS_HAVE_DIFF_PARENT)
    " " ":ERROR_HWNDS_HAVE_DIFF_PARENT"
#  endif
#  if defined(ERROR_NOT_CHILD_WINDOW)
    " " ":ERROR_NOT_CHILD_WINDOW"
#  endif
#  if defined(ERROR_INVALID_GW_COMMAND)
    " " ":ERROR_INVALID_GW_COMMAND"
#  endif
#  if defined(ERROR_INVALID_THREAD_ID)
    " " ":ERROR_INVALID_THREAD_ID"
#  endif
#  if defined(ERROR_NON_MDICHILD_WINDOW)
    " " ":ERROR_NON_MDICHILD_WINDOW"
#  endif
#  if defined(ERROR_POPUP_ALREADY_ACTIVE)
    " " ":ERROR_POPUP_ALREADY_ACTIVE"
#  endif
#  if defined(ERROR_NO_SCROLLBARS)
    " " ":ERROR_NO_SCROLLBARS"
#  endif
#  if defined(ERROR_INVALID_SCROLLBAR_RANGE)
    " " ":ERROR_INVALID_SCROLLBAR_RANGE"
#  endif
#  if defined(ERROR_INVALID_SHOWWIN_COMMAND)
    " " ":ERROR_INVALID_SHOWWIN_COMMAND"
#  endif
#  if defined(ERROR_NO_SYSTEM_RESOURCES)
    " " ":ERROR_NO_SYSTEM_RESOURCES"
#  endif
#  if defined(ERROR_NONPAGED_SYSTEM_RESOURCES)
    " " ":ERROR_NONPAGED_SYSTEM_RESOURCES"
#  endif
#  if defined(ERROR_PAGED_SYSTEM_RESOURCES)
    " " ":ERROR_PAGED_SYSTEM_RESOURCES"
#  endif
#  if defined(ERROR_WORKING_SET_QUOTA)
    " " ":ERROR_WORKING_SET_QUOTA"
#  endif
#  if defined(ERROR_PAGEFILE_QUOTA)
    " " ":ERROR_PAGEFILE_QUOTA"
#  endif
#  if defined(ERROR_COMMITMENT_LIMIT)
    " " ":ERROR_COMMITMENT_LIMIT"
#  endif
#  if defined(ERROR_MENU_ITEM_NOT_FOUND)
    " " ":ERROR_MENU_ITEM_NOT_FOUND"
#  endif
#  if defined(ERROR_INVALID_KEYBOARD_HANDLE)
    " " ":ERROR_INVALID_KEYBOARD_HANDLE"
#  endif
#  if defined(ERROR_HOOK_TYPE_NOT_ALLOWED)
    " " ":ERROR_HOOK_TYPE_NOT_ALLOWED"
#  endif
#  if defined(ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION)
    " " ":ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION"
#  endif
#  if defined(ERROR_TIMEOUT)
    " " ":ERROR_TIMEOUT"
#  endif
#  if defined(ERROR_INVALID_MONITOR_HANDLE)
    " " ":ERROR_INVALID_MONITOR_HANDLE"
#  endif
#  if defined(ERROR_EVENTLOG_FILE_CORRUPT)
    " " ":ERROR_EVENTLOG_FILE_CORRUPT"
#  endif
#  if defined(ERROR_EVENTLOG_CANT_START)
    " " ":ERROR_EVENTLOG_CANT_START"
#  endif
#  if defined(ERROR_LOG_FILE_FULL)
    " " ":ERROR_LOG_FILE_FULL"
#  endif
#  if defined(ERROR_EVENTLOG_FILE_CHANGED)
    " " ":ERROR_EVENTLOG_FILE_CHANGED"
#  endif
#  if defined(ERROR_INSTALL_SERVICE_FAILURE)
    " " ":ERROR_INSTALL_SERVICE_FAILURE"
#  endif
#  if defined(ERROR_INSTALL_USEREXIT)
    " " ":ERROR_INSTALL_USEREXIT"
#  endif
#  if defined(ERROR_INSTALL_FAILURE)
    " " ":ERROR_INSTALL_FAILURE"
#  endif
#  if defined(ERROR_INSTALL_SUSPEND)
    " " ":ERROR_INSTALL_SUSPEND"
#  endif
#  if defined(ERROR_UNKNOWN_PRODUCT)
    " " ":ERROR_UNKNOWN_PRODUCT"
#  endif
#  if defined(ERROR_UNKNOWN_FEATURE)
    " " ":ERROR_UNKNOWN_FEATURE"
#  endif
#  if defined(ERROR_UNKNOWN_COMPONENT)
    " " ":ERROR_UNKNOWN_COMPONENT"
#  endif
#  if defined(ERROR_UNKNOWN_PROPERTY)
    " " ":ERROR_UNKNOWN_PROPERTY"
#  endif
#  if defined(ERROR_INVALID_HANDLE_STATE)
    " " ":ERROR_INVALID_HANDLE_STATE"
#  endif
#  if defined(ERROR_BAD_CONFIGURATION)
    " " ":ERROR_BAD_CONFIGURATION"
#  endif
#  if defined(ERROR_INDEX_ABSENT)
    " " ":ERROR_INDEX_ABSENT"
#  endif
#  if defined(ERROR_INSTALL_SOURCE_ABSENT)
    " " ":ERROR_INSTALL_SOURCE_ABSENT"
#  endif
#  if defined(ERROR_INSTALL_PACKAGE_VERSION)
    " " ":ERROR_INSTALL_PACKAGE_VERSION"
#  endif
#  if defined(ERROR_PRODUCT_UNINSTALLED)
    " " ":ERROR_PRODUCT_UNINSTALLED"
#  endif
#  if defined(ERROR_BAD_QUERY_SYNTAX)
    " " ":ERROR_BAD_QUERY_SYNTAX"
#  endif
#  if defined(ERROR_INVALID_FIELD)
    " " ":ERROR_INVALID_FIELD"
#  endif
#  if defined(ERROR_DEVICE_REMOVED)
    " " ":ERROR_DEVICE_REMOVED"
#  endif
#  if defined(ERROR_INSTALL_ALREADY_RUNNING)
    " " ":ERROR_INSTALL_ALREADY_RUNNING"
#  endif
#  if defined(ERROR_INSTALL_PACKAGE_OPEN_FAILED)
    " " ":ERROR_INSTALL_PACKAGE_OPEN_FAILED"
#  endif
#  if defined(ERROR_INSTALL_PACKAGE_INVALID)
    " " ":ERROR_INSTALL_PACKAGE_INVALID"
#  endif
#  if defined(ERROR_INSTALL_UI_FAILURE)
    " " ":ERROR_INSTALL_UI_FAILURE"
#  endif
#  if defined(ERROR_INSTALL_LOG_FAILURE)
    " " ":ERROR_INSTALL_LOG_FAILURE"
#  endif
#  if defined(ERROR_INSTALL_LANGUAGE_UNSUPPORTED)
    " " ":ERROR_INSTALL_LANGUAGE_UNSUPPORTED"
#  endif
#  if defined(ERROR_INSTALL_TRANSFORM_FAILURE)
    " " ":ERROR_INSTALL_TRANSFORM_FAILURE"
#  endif
#  if defined(ERROR_INSTALL_PACKAGE_REJECTED)
    " " ":ERROR_INSTALL_PACKAGE_REJECTED"
#  endif
#  if defined(ERROR_FUNCTION_NOT_CALLED)
    " " ":ERROR_FUNCTION_NOT_CALLED"
#  endif
#  if defined(ERROR_FUNCTION_FAILED)
    " " ":ERROR_FUNCTION_FAILED"
#  endif
#  if defined(ERROR_INVALID_TABLE)
    " " ":ERROR_INVALID_TABLE"
#  endif
#  if defined(ERROR_DATATYPE_MISMATCH)
    " " ":ERROR_DATATYPE_MISMATCH"
#  endif
#  if defined(ERROR_UNSUPPORTED_TYPE)
    " " ":ERROR_UNSUPPORTED_TYPE"
#  endif
#  if defined(ERROR_CREATE_FAILED)
    " " ":ERROR_CREATE_FAILED"
#  endif
#  if defined(ERROR_INSTALL_TEMP_UNWRITABLE)
    " " ":ERROR_INSTALL_TEMP_UNWRITABLE"
#  endif
#  if defined(ERROR_INSTALL_PLATFORM_UNSUPPORTED)
    " " ":ERROR_INSTALL_PLATFORM_UNSUPPORTED"
#  endif
#  if defined(ERROR_INSTALL_NOTUSED)
    " " ":ERROR_INSTALL_NOTUSED"
#  endif
#  if defined(ERROR_PATCH_PACKAGE_OPEN_FAILED)
    " " ":ERROR_PATCH_PACKAGE_OPEN_FAILED"
#  endif
#  if defined(ERROR_PATCH_PACKAGE_INVALID)
    " " ":ERROR_PATCH_PACKAGE_INVALID"
#  endif
#  if defined(ERROR_PATCH_PACKAGE_UNSUPPORTED)
    " " ":ERROR_PATCH_PACKAGE_UNSUPPORTED"
#  endif
#  if defined(ERROR_PRODUCT_VERSION)
    " " ":ERROR_PRODUCT_VERSION"
#  endif
#  if defined(ERROR_INVALID_COMMAND_LINE)
    " " ":ERROR_INVALID_COMMAND_LINE"
#  endif
#  if defined(ERROR_INSTALL_REMOTE_DISALLOWED)
    " " ":ERROR_INSTALL_REMOTE_DISALLOWED"
#  endif
#  if defined(ERROR_SUCCESS_REBOOT_INITIATED)
    " " ":ERROR_SUCCESS_REBOOT_INITIATED"
#  endif
#  if defined(ERROR_PATCH_TARGET_NOT_FOUND)
    " " ":ERROR_PATCH_TARGET_NOT_FOUND"
#  endif
#  if defined(ERROR_PATCH_PACKAGE_REJECTED)
    " " ":ERROR_PATCH_PACKAGE_REJECTED"
#  endif
#  if defined(ERROR_INSTALL_TRANSFORM_REJECTED)
    " " ":ERROR_INSTALL_TRANSFORM_REJECTED"
#  endif
#  if defined(ERROR_INSTALL_REMOTE_PROHIBITED)
    " " ":ERROR_INSTALL_REMOTE_PROHIBITED"
#  endif
#  if defined(RPC_S_INVALID_STRING_BINDING)
    " " ":RPC_S_INVALID_STRING_BINDING"
#  endif
#  if defined(RPC_S_WRONG_KIND_OF_BINDING)
    " " ":RPC_S_WRONG_KIND_OF_BINDING"
#  endif
#  if defined(RPC_S_INVALID_BINDING)
    " " ":RPC_S_INVALID_BINDING"
#  endif
#  if defined(RPC_S_PROTSEQ_NOT_SUPPORTED)
    " " ":RPC_S_PROTSEQ_NOT_SUPPORTED"
#  endif
#  if defined(RPC_S_INVALID_RPC_PROTSEQ)
    " " ":RPC_S_INVALID_RPC_PROTSEQ"
#  endif
#  if defined(RPC_S_INVALID_STRING_UUID)
    " " ":RPC_S_INVALID_STRING_UUID"
#  endif
#  if defined(RPC_S_INVALID_ENDPOINT_FORMAT)
    " " ":RPC_S_INVALID_ENDPOINT_FORMAT"
#  endif
#  if defined(RPC_S_INVALID_NET_ADDR)
    " " ":RPC_S_INVALID_NET_ADDR"
#  endif
#  if defined(RPC_S_NO_ENDPOINT_FOUND)
    " " ":RPC_S_NO_ENDPOINT_FOUND"
#  endif
#  if defined(RPC_S_INVALID_TIMEOUT)
    " " ":RPC_S_INVALID_TIMEOUT"
#  endif
#  if defined(RPC_S_OBJECT_NOT_FOUND)
    " " ":RPC_S_OBJECT_NOT_FOUND"
#  endif
#  if defined(RPC_S_ALREADY_REGISTERED)
    " " ":RPC_S_ALREADY_REGISTERED"
#  endif
#  if defined(RPC_S_TYPE_ALREADY_REGISTERED)
    " " ":RPC_S_TYPE_ALREADY_REGISTERED"
#  endif
#  if defined(RPC_S_ALREADY_LISTENING)
    " " ":RPC_S_ALREADY_LISTENING"
#  endif
#  if defined(RPC_S_NO_PROTSEQS_REGISTERED)
    " " ":RPC_S_NO_PROTSEQS_REGISTERED"
#  endif
#  if defined(RPC_S_NOT_LISTENING)
    " " ":RPC_S_NOT_LISTENING"
#  endif
#  if defined(RPC_S_UNKNOWN_MGR_TYPE)
    " " ":RPC_S_UNKNOWN_MGR_TYPE"
#  endif
#  if defined(RPC_S_UNKNOWN_IF)
    " " ":RPC_S_UNKNOWN_IF"
#  endif
#  if defined(RPC_S_NO_BINDINGS)
    " " ":RPC_S_NO_BINDINGS"
#  endif
#  if defined(RPC_S_NO_PROTSEQS)
    " " ":RPC_S_NO_PROTSEQS"
#  endif
#  if defined(RPC_S_CANT_CREATE_ENDPOINT)
    " " ":RPC_S_CANT_CREATE_ENDPOINT"
#  endif
#  if defined(RPC_S_OUT_OF_RESOURCES)
    " " ":RPC_S_OUT_OF_RESOURCES"
#  endif
#  if defined(RPC_S_SERVER_UNAVAILABLE)
    " " ":RPC_S_SERVER_UNAVAILABLE"
#  endif
#  if defined(RPC_S_SERVER_TOO_BUSY)
    " " ":RPC_S_SERVER_TOO_BUSY"
#  endif
#  if defined(RPC_S_INVALID_NETWORK_OPTIONS)
    " " ":RPC_S_INVALID_NETWORK_OPTIONS"
#  endif
#  if defined(RPC_S_NO_CALL_ACTIVE)
    " " ":RPC_S_NO_CALL_ACTIVE"
#  endif
#  if defined(RPC_S_CALL_FAILED)
    " " ":RPC_S_CALL_FAILED"
#  endif
#  if defined(RPC_S_CALL_FAILED_DNE)
    " " ":RPC_S_CALL_FAILED_DNE"
#  endif
#  if defined(RPC_S_PROTOCOL_ERROR)
    " " ":RPC_S_PROTOCOL_ERROR"
#  endif
#  if defined(RPC_S_UNSUPPORTED_TRANS_SYN)
    " " ":RPC_S_UNSUPPORTED_TRANS_SYN"
#  endif
#  if defined(RPC_S_UNSUPPORTED_TYPE)
    " " ":RPC_S_UNSUPPORTED_TYPE"
#  endif
#  if defined(RPC_S_INVALID_TAG)
    " " ":RPC_S_INVALID_TAG"
#  endif
#  if defined(RPC_S_INVALID_BOUND)
    " " ":RPC_S_INVALID_BOUND"
#  endif
#  if defined(RPC_S_NO_ENTRY_NAME)
    " " ":RPC_S_NO_ENTRY_NAME"
#  endif
#  if defined(RPC_S_INVALID_NAME_SYNTAX)
    " " ":RPC_S_INVALID_NAME_SYNTAX"
#  endif
#  if defined(RPC_S_UNSUPPORTED_NAME_SYNTAX)
    " " ":RPC_S_UNSUPPORTED_NAME_SYNTAX"
#  endif
#  if defined(RPC_S_UUID_NO_ADDRESS)
    " " ":RPC_S_UUID_NO_ADDRESS"
#  endif
#  if defined(RPC_S_DUPLICATE_ENDPOINT)
    " " ":RPC_S_DUPLICATE_ENDPOINT"
#  endif
#  if defined(RPC_S_UNKNOWN_AUTHN_TYPE)
    " " ":RPC_S_UNKNOWN_AUTHN_TYPE"
#  endif
#  if defined(RPC_S_MAX_CALLS_TOO_SMALL)
    " " ":RPC_S_MAX_CALLS_TOO_SMALL"
#  endif
#  if defined(RPC_S_STRING_TOO_LONG)
    " " ":RPC_S_STRING_TOO_LONG"
#  endif
#  if defined(RPC_S_PROTSEQ_NOT_FOUND)
    " " ":RPC_S_PROTSEQ_NOT_FOUND"
#  endif
#  if defined(RPC_S_PROCNUM_OUT_OF_RANGE)
    " " ":RPC_S_PROCNUM_OUT_OF_RANGE"
#  endif
#  if defined(RPC_S_BINDING_HAS_NO_AUTH)
    " " ":RPC_S_BINDING_HAS_NO_AUTH"
#  endif
#  if defined(RPC_S_UNKNOWN_AUTHN_SERVICE)
    " " ":RPC_S_UNKNOWN_AUTHN_SERVICE"
#  endif
#  if defined(RPC_S_UNKNOWN_AUTHN_LEVEL)
    " " ":RPC_S_UNKNOWN_AUTHN_LEVEL"
#  endif
#  if defined(RPC_S_INVALID_AUTH_IDENTITY)
    " " ":RPC_S_INVALID_AUTH_IDENTITY"
#  endif
#  if defined(RPC_S_UNKNOWN_AUTHZ_SERVICE)
    " " ":RPC_S_UNKNOWN_AUTHZ_SERVICE"
#  endif
#  if defined(EPT_S_INVALID_ENTRY)
    " " ":EPT_S_INVALID_ENTRY"
#  endif
#  if defined(EPT_S_CANT_PERFORM_OP)
    " " ":EPT_S_CANT_PERFORM_OP"
#  endif
#  if defined(EPT_S_NOT_REGISTERED)
    " " ":EPT_S_NOT_REGISTERED"
#  endif
#  if defined(RPC_S_NOTHING_TO_EXPORT)
    " " ":RPC_S_NOTHING_TO_EXPORT"
#  endif
#  if defined(RPC_S_INCOMPLETE_NAME)
    " " ":RPC_S_INCOMPLETE_NAME"
#  endif
#  if defined(RPC_S_INVALID_VERS_OPTION)
    " " ":RPC_S_INVALID_VERS_OPTION"
#  endif
#  if defined(RPC_S_NO_MORE_MEMBERS)
    " " ":RPC_S_NO_MORE_MEMBERS"
#  endif
#  if defined(RPC_S_NOT_ALL_OBJS_UNEXPORTED)
    " " ":RPC_S_NOT_ALL_OBJS_UNEXPORTED"
#  endif
#  if defined(RPC_S_INTERFACE_NOT_FOUND)
    " " ":RPC_S_INTERFACE_NOT_FOUND"
#  endif
#  if defined(RPC_S_ENTRY_ALREADY_EXISTS)
    " " ":RPC_S_ENTRY_ALREADY_EXISTS"
#  endif
#  if defined(RPC_S_ENTRY_NOT_FOUND)
    " " ":RPC_S_ENTRY_NOT_FOUND"
#  endif
#  if defined(RPC_S_NAME_SERVICE_UNAVAILABLE)
    " " ":RPC_S_NAME_SERVICE_UNAVAILABLE"
#  endif
#  if defined(RPC_S_INVALID_NAF_ID)
    " " ":RPC_S_INVALID_NAF_ID"
#  endif
#  if defined(RPC_S_CANNOT_SUPPORT)
    " " ":RPC_S_CANNOT_SUPPORT"
#  endif
#  if defined(RPC_S_NO_CONTEXT_AVAILABLE)
    " " ":RPC_S_NO_CONTEXT_AVAILABLE"
#  endif
#  if defined(RPC_S_INTERNAL_ERROR)
    " " ":RPC_S_INTERNAL_ERROR"
#  endif
#  if defined(RPC_S_ZERO_DIVIDE)
    " " ":RPC_S_ZERO_DIVIDE"
#  endif
#  if defined(RPC_S_ADDRESS_ERROR)
    " " ":RPC_S_ADDRESS_ERROR"
#  endif
#  if defined(RPC_S_FP_DIV_ZERO)
    " " ":RPC_S_FP_DIV_ZERO"
#  endif
#  if defined(RPC_S_FP_UNDERFLOW)
    " " ":RPC_S_FP_UNDERFLOW"
#  endif
#  if defined(RPC_S_FP_OVERFLOW)
    " " ":RPC_S_FP_OVERFLOW"
#  endif
#  if defined(RPC_X_NO_MORE_ENTRIES)
    " " ":RPC_X_NO_MORE_ENTRIES"
#  endif
#  if defined(RPC_X_SS_CHAR_TRANS_OPEN_FAIL)
    " " ":RPC_X_SS_CHAR_TRANS_OPEN_FAIL"
#  endif
#  if defined(RPC_X_SS_CHAR_TRANS_SHORT_FILE)
    " " ":RPC_X_SS_CHAR_TRANS_SHORT_FILE"
#  endif
#  if defined(RPC_X_SS_IN_NULL_CONTEXT)
    " " ":RPC_X_SS_IN_NULL_CONTEXT"
#  endif
#  if defined(RPC_X_SS_CONTEXT_DAMAGED)
    " " ":RPC_X_SS_CONTEXT_DAMAGED"
#  endif
#  if defined(RPC_X_SS_HANDLES_MISMATCH)
    " " ":RPC_X_SS_HANDLES_MISMATCH"
#  endif
#  if defined(RPC_X_SS_CANNOT_GET_CALL_HANDLE)
    " " ":RPC_X_SS_CANNOT_GET_CALL_HANDLE"
#  endif
#  if defined(RPC_X_NULL_REF_POINTER)
    " " ":RPC_X_NULL_REF_POINTER"
#  endif
#  if defined(RPC_X_ENUM_VALUE_OUT_OF_RANGE)
    " " ":RPC_X_ENUM_VALUE_OUT_OF_RANGE"
#  endif
#  if defined(RPC_X_BYTE_COUNT_TOO_SMALL)
    " " ":RPC_X_BYTE_COUNT_TOO_SMALL"
#  endif
#  if defined(RPC_X_BAD_STUB_DATA)
    " " ":RPC_X_BAD_STUB_DATA"
#  endif
#  if defined(ERROR_INVALID_USER_BUFFER)
    " " ":ERROR_INVALID_USER_BUFFER"
#  endif
#  if defined(ERROR_UNRECOGNIZED_MEDIA)
    " " ":ERROR_UNRECOGNIZED_MEDIA"
#  endif
#  if defined(ERROR_NO_TRUST_LSA_SECRET)
    " " ":ERROR_NO_TRUST_LSA_SECRET"
#  endif
#  if defined(ERROR_NO_TRUST_SAM_ACCOUNT)
    " " ":ERROR_NO_TRUST_SAM_ACCOUNT"
#  endif
#  if defined(ERROR_TRUSTED_DOMAIN_FAILURE)
    " " ":ERROR_TRUSTED_DOMAIN_FAILURE"
#  endif
#  if defined(ERROR_TRUSTED_RELATIONSHIP_FAILURE)
    " " ":ERROR_TRUSTED_RELATIONSHIP_FAILURE"
#  endif
#  if defined(ERROR_TRUST_FAILURE)
    " " ":ERROR_TRUST_FAILURE"
#  endif
#  if defined(RPC_S_CALL_IN_PROGRESS)
    " " ":RPC_S_CALL_IN_PROGRESS"
#  endif
#  if defined(ERROR_NETLOGON_NOT_STARTED)
    " " ":ERROR_NETLOGON_NOT_STARTED"
#  endif
#  if defined(ERROR_ACCOUNT_EXPIRED)
    " " ":ERROR_ACCOUNT_EXPIRED"
#  endif
#  if defined(ERROR_REDIRECTOR_HAS_OPEN_HANDLES)
    " " ":ERROR_REDIRECTOR_HAS_OPEN_HANDLES"
#  endif
#  if defined(ERROR_PRINTER_DRIVER_ALREADY_INSTALLED)
    " " ":ERROR_PRINTER_DRIVER_ALREADY_INSTALLED"
#  endif
#  if defined(ERROR_UNKNOWN_PORT)
    " " ":ERROR_UNKNOWN_PORT"
#  endif
#  if defined(ERROR_UNKNOWN_PRINTER_DRIVER)
    " " ":ERROR_UNKNOWN_PRINTER_DRIVER"
#  endif
#  if defined(ERROR_UNKNOWN_PRINTPROCESSOR)
    " " ":ERROR_UNKNOWN_PRINTPROCESSOR"
#  endif
#  if defined(ERROR_INVALID_SEPARATOR_FILE)
    " " ":ERROR_INVALID_SEPARATOR_FILE"
#  endif
#  if defined(ERROR_INVALID_PRIORITY)
    " " ":ERROR_INVALID_PRIORITY"
#  endif
#  if defined(ERROR_INVALID_PRINTER_NAME)
    " " ":ERROR_INVALID_PRINTER_NAME"
#  endif
#  if defined(ERROR_PRINTER_ALREADY_EXISTS)
    " " ":ERROR_PRINTER_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_INVALID_PRINTER_COMMAND)
    " " ":ERROR_INVALID_PRINTER_COMMAND"
#  endif
#  if defined(ERROR_INVALID_DATATYPE)
    " " ":ERROR_INVALID_DATATYPE"
#  endif
#  if defined(ERROR_INVALID_ENVIRONMENT)
    " " ":ERROR_INVALID_ENVIRONMENT"
#  endif
#  if defined(RPC_S_NO_MORE_BINDINGS)
    " " ":RPC_S_NO_MORE_BINDINGS"
#  endif
#  if defined(ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT)
    " " ":ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT"
#  endif
#  if defined(ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT)
    " " ":ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT"
#  endif
#  if defined(ERROR_NOLOGON_SERVER_TRUST_ACCOUNT)
    " " ":ERROR_NOLOGON_SERVER_TRUST_ACCOUNT"
#  endif
#  if defined(ERROR_DOMAIN_TRUST_INCONSISTENT)
    " " ":ERROR_DOMAIN_TRUST_INCONSISTENT"
#  endif
#  if defined(ERROR_SERVER_HAS_OPEN_HANDLES)
    " " ":ERROR_SERVER_HAS_OPEN_HANDLES"
#  endif
#  if defined(ERROR_RESOURCE_DATA_NOT_FOUND)
    " " ":ERROR_RESOURCE_DATA_NOT_FOUND"
#  endif
#  if defined(ERROR_RESOURCE_TYPE_NOT_FOUND)
    " " ":ERROR_RESOURCE_TYPE_NOT_FOUND"
#  endif
#  if defined(ERROR_RESOURCE_NAME_NOT_FOUND)
    " " ":ERROR_RESOURCE_NAME_NOT_FOUND"
#  endif
#  if defined(ERROR_RESOURCE_LANG_NOT_FOUND)
    " " ":ERROR_RESOURCE_LANG_NOT_FOUND"
#  endif
#  if defined(ERROR_NOT_ENOUGH_QUOTA)
    " " ":ERROR_NOT_ENOUGH_QUOTA"
#  endif
#  if defined(RPC_S_NO_INTERFACES)
    " " ":RPC_S_NO_INTERFACES"
#  endif
#  if defined(RPC_S_CALL_CANCELLED)
    " " ":RPC_S_CALL_CANCELLED"
#  endif
#  if defined(RPC_S_BINDING_INCOMPLETE)
    " " ":RPC_S_BINDING_INCOMPLETE"
#  endif
#  if defined(RPC_S_COMM_FAILURE)
    " " ":RPC_S_COMM_FAILURE"
#  endif
#  if defined(RPC_S_UNSUPPORTED_AUTHN_LEVEL)
    " " ":RPC_S_UNSUPPORTED_AUTHN_LEVEL"
#  endif
#  if defined(RPC_S_NO_PRINC_NAME)
    " " ":RPC_S_NO_PRINC_NAME"
#  endif
#  if defined(RPC_S_NOT_RPC_ERROR)
    " " ":RPC_S_NOT_RPC_ERROR"
#  endif
#  if defined(RPC_S_UUID_LOCAL_ONLY)
    " " ":RPC_S_UUID_LOCAL_ONLY"
#  endif
#  if defined(RPC_S_SEC_PKG_ERROR)
    " " ":RPC_S_SEC_PKG_ERROR"
#  endif
#  if defined(RPC_S_NOT_CANCELLED)
    " " ":RPC_S_NOT_CANCELLED"
#  endif
#  if defined(RPC_X_INVALID_ES_ACTION)
    " " ":RPC_X_INVALID_ES_ACTION"
#  endif
#  if defined(RPC_X_WRONG_ES_VERSION)
    " " ":RPC_X_WRONG_ES_VERSION"
#  endif
#  if defined(RPC_X_WRONG_STUB_VERSION)
    " " ":RPC_X_WRONG_STUB_VERSION"
#  endif
#  if defined(RPC_X_INVALID_PIPE_OBJECT)
    " " ":RPC_X_INVALID_PIPE_OBJECT"
#  endif
#  if defined(RPC_X_WRONG_PIPE_ORDER)
    " " ":RPC_X_WRONG_PIPE_ORDER"
#  endif
#  if defined(RPC_X_WRONG_PIPE_VERSION)
    " " ":RPC_X_WRONG_PIPE_VERSION"
#  endif
#  if defined(RPC_S_GROUP_MEMBER_NOT_FOUND)
    " " ":RPC_S_GROUP_MEMBER_NOT_FOUND"
#  endif
#  if defined(EPT_S_CANT_CREATE)
    " " ":EPT_S_CANT_CREATE"
#  endif
#  if defined(RPC_S_INVALID_OBJECT)
    " " ":RPC_S_INVALID_OBJECT"
#  endif
#  if defined(ERROR_INVALID_TIME)
    " " ":ERROR_INVALID_TIME"
#  endif
#  if defined(ERROR_INVALID_FORM_NAME)
    " " ":ERROR_INVALID_FORM_NAME"
#  endif
#  if defined(ERROR_INVALID_FORM_SIZE)
    " " ":ERROR_INVALID_FORM_SIZE"
#  endif
#  if defined(ERROR_ALREADY_WAITING)
    " " ":ERROR_ALREADY_WAITING"
#  endif
#  if defined(ERROR_PRINTER_DELETED)
    " " ":ERROR_PRINTER_DELETED"
#  endif
#  if defined(ERROR_INVALID_PRINTER_STATE)
    " " ":ERROR_INVALID_PRINTER_STATE"
#  endif
#  if defined(ERROR_PASSWORD_MUST_CHANGE)
    " " ":ERROR_PASSWORD_MUST_CHANGE"
#  endif
#  if defined(ERROR_DOMAIN_CONTROLLER_NOT_FOUND)
    " " ":ERROR_DOMAIN_CONTROLLER_NOT_FOUND"
#  endif
#  if defined(ERROR_ACCOUNT_LOCKED_OUT)
    " " ":ERROR_ACCOUNT_LOCKED_OUT"
#  endif
#  if defined(OR_INVALID_OXID)
    " " ":OR_INVALID_OXID"
#  endif
#  if defined(OR_INVALID_OID)
    " " ":OR_INVALID_OID"
#  endif
#  if defined(OR_INVALID_SET)
    " " ":OR_INVALID_SET"
#  endif
#  if defined(RPC_S_SEND_INCOMPLETE)
    " " ":RPC_S_SEND_INCOMPLETE"
#  endif
#  if defined(RPC_S_INVALID_ASYNC_HANDLE)
    " " ":RPC_S_INVALID_ASYNC_HANDLE"
#  endif
#  if defined(RPC_S_INVALID_ASYNC_CALL)
    " " ":RPC_S_INVALID_ASYNC_CALL"
#  endif
#  if defined(RPC_X_PIPE_CLOSED)
    " " ":RPC_X_PIPE_CLOSED"
#  endif
#  if defined(RPC_X_PIPE_DISCIPLINE_ERROR)
    " " ":RPC_X_PIPE_DISCIPLINE_ERROR"
#  endif
#  if defined(RPC_X_PIPE_EMPTY)
    " " ":RPC_X_PIPE_EMPTY"
#  endif
#  if defined(ERROR_NO_SITENAME)
    " " ":ERROR_NO_SITENAME"
#  endif
#  if defined(ERROR_CANT_ACCESS_FILE)
    " " ":ERROR_CANT_ACCESS_FILE"
#  endif
#  if defined(ERROR_CANT_RESOLVE_FILENAME)
    " " ":ERROR_CANT_RESOLVE_FILENAME"
#  endif
#  if defined(RPC_S_ENTRY_TYPE_MISMATCH)
    " " ":RPC_S_ENTRY_TYPE_MISMATCH"
#  endif
#  if defined(RPC_S_NOT_ALL_OBJS_EXPORTED)
    " " ":RPC_S_NOT_ALL_OBJS_EXPORTED"
#  endif
#  if defined(RPC_S_INTERFACE_NOT_EXPORTED)
    " " ":RPC_S_INTERFACE_NOT_EXPORTED"
#  endif
#  if defined(RPC_S_PROFILE_NOT_ADDED)
    " " ":RPC_S_PROFILE_NOT_ADDED"
#  endif
#  if defined(RPC_S_PRF_ELT_NOT_ADDED)
    " " ":RPC_S_PRF_ELT_NOT_ADDED"
#  endif
#  if defined(RPC_S_PRF_ELT_NOT_REMOVED)
    " " ":RPC_S_PRF_ELT_NOT_REMOVED"
#  endif
#  if defined(RPC_S_GRP_ELT_NOT_ADDED)
    " " ":RPC_S_GRP_ELT_NOT_ADDED"
#  endif
#  if defined(RPC_S_GRP_ELT_NOT_REMOVED)
    " " ":RPC_S_GRP_ELT_NOT_REMOVED"
#  endif
#  if defined(ERROR_KM_DRIVER_BLOCKED)
    " " ":ERROR_KM_DRIVER_BLOCKED"
#  endif
#  if defined(ERROR_CONTEXT_EXPIRED)
    " " ":ERROR_CONTEXT_EXPIRED"
#  endif
#  if defined(ERROR_PER_USER_TRUST_QUOTA_EXCEEDED)
    " " ":ERROR_PER_USER_TRUST_QUOTA_EXCEEDED"
#  endif
#  if defined(ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED)
    " " ":ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED"
#  endif
#  if defined(ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED)
    " " ":ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED"
#  endif
#  if defined(ERROR_INVALID_PIXEL_FORMAT)
    " " ":ERROR_INVALID_PIXEL_FORMAT"
#  endif
#  if defined(ERROR_BAD_DRIVER)
    " " ":ERROR_BAD_DRIVER"
#  endif
#  if defined(ERROR_INVALID_WINDOW_STYLE)
    " " ":ERROR_INVALID_WINDOW_STYLE"
#  endif
#  if defined(ERROR_METAFILE_NOT_SUPPORTED)
    " " ":ERROR_METAFILE_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_TRANSFORM_NOT_SUPPORTED)
    " " ":ERROR_TRANSFORM_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_CLIPPING_NOT_SUPPORTED)
    " " ":ERROR_CLIPPING_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_INVALID_CMM)
    " " ":ERROR_INVALID_CMM"
#  endif
#  if defined(ERROR_INVALID_PROFILE)
    " " ":ERROR_INVALID_PROFILE"
#  endif
#  if defined(ERROR_TAG_NOT_FOUND)
    " " ":ERROR_TAG_NOT_FOUND"
#  endif
#  if defined(ERROR_TAG_NOT_PRESENT)
    " " ":ERROR_TAG_NOT_PRESENT"
#  endif
#  if defined(ERROR_DUPLICATE_TAG)
    " " ":ERROR_DUPLICATE_TAG"
#  endif
#  if defined(ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE)
    " " ":ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE"
#  endif
#  if defined(ERROR_PROFILE_NOT_FOUND)
    " " ":ERROR_PROFILE_NOT_FOUND"
#  endif
#  if defined(ERROR_INVALID_COLORSPACE)
    " " ":ERROR_INVALID_COLORSPACE"
#  endif
#  if defined(ERROR_ICM_NOT_ENABLED)
    " " ":ERROR_ICM_NOT_ENABLED"
#  endif
#  if defined(ERROR_DELETING_ICM_XFORM)
    " " ":ERROR_DELETING_ICM_XFORM"
#  endif
#  if defined(ERROR_INVALID_TRANSFORM)
    " " ":ERROR_INVALID_TRANSFORM"
#  endif
#  if defined(ERROR_COLORSPACE_MISMATCH)
    " " ":ERROR_COLORSPACE_MISMATCH"
#  endif
#  if defined(ERROR_INVALID_COLORINDEX)
    " " ":ERROR_INVALID_COLORINDEX"
#  endif
#  if defined(ERROR_CONNECTED_OTHER_PASSWORD)
    " " ":ERROR_CONNECTED_OTHER_PASSWORD"
#  endif
#  if defined(ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT)
    " " ":ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT"
#  endif
#  if defined(ERROR_BAD_USERNAME)
    " " ":ERROR_BAD_USERNAME"
#  endif
#  if defined(ERROR_NOT_CONNECTED)
    " " ":ERROR_NOT_CONNECTED"
#  endif
#  if defined(ERROR_OPEN_FILES)
    " " ":ERROR_OPEN_FILES"
#  endif
#  if defined(ERROR_ACTIVE_CONNECTIONS)
    " " ":ERROR_ACTIVE_CONNECTIONS"
#  endif
#  if defined(ERROR_DEVICE_IN_USE)
    " " ":ERROR_DEVICE_IN_USE"
#  endif
#  if defined(ERROR_UNKNOWN_PRINT_MONITOR)
    " " ":ERROR_UNKNOWN_PRINT_MONITOR"
#  endif
#  if defined(ERROR_PRINTER_DRIVER_IN_USE)
    " " ":ERROR_PRINTER_DRIVER_IN_USE"
#  endif
#  if defined(ERROR_SPOOL_FILE_NOT_FOUND)
    " " ":ERROR_SPOOL_FILE_NOT_FOUND"
#  endif
#  if defined(ERROR_SPL_NO_STARTDOC)
    " " ":ERROR_SPL_NO_STARTDOC"
#  endif
#  if defined(ERROR_SPL_NO_ADDJOB)
    " " ":ERROR_SPL_NO_ADDJOB"
#  endif
#  if defined(ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED)
    " " ":ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED"
#  endif
#  if defined(ERROR_PRINT_MONITOR_ALREADY_INSTALLED)
    " " ":ERROR_PRINT_MONITOR_ALREADY_INSTALLED"
#  endif
#  if defined(ERROR_INVALID_PRINT_MONITOR)
    " " ":ERROR_INVALID_PRINT_MONITOR"
#  endif
#  if defined(ERROR_PRINT_MONITOR_IN_USE)
    " " ":ERROR_PRINT_MONITOR_IN_USE"
#  endif
#  if defined(ERROR_PRINTER_HAS_JOBS_QUEUED)
    " " ":ERROR_PRINTER_HAS_JOBS_QUEUED"
#  endif
#  if defined(ERROR_SUCCESS_REBOOT_REQUIRED)
    " " ":ERROR_SUCCESS_REBOOT_REQUIRED"
#  endif
#  if defined(ERROR_SUCCESS_RESTART_REQUIRED)
    " " ":ERROR_SUCCESS_RESTART_REQUIRED"
#  endif
#  if defined(ERROR_PRINTER_NOT_FOUND)
    " " ":ERROR_PRINTER_NOT_FOUND"
#  endif
#  if defined(ERROR_PRINTER_DRIVER_WARNED)
    " " ":ERROR_PRINTER_DRIVER_WARNED"
#  endif
#  if defined(ERROR_PRINTER_DRIVER_BLOCKED)
    " " ":ERROR_PRINTER_DRIVER_BLOCKED"
#  endif
#  if defined(ERROR_WINS_INTERNAL)
    " " ":ERROR_WINS_INTERNAL"
#  endif
#  if defined(ERROR_CAN_NOT_DEL_LOCAL_WINS)
    " " ":ERROR_CAN_NOT_DEL_LOCAL_WINS"
#  endif
#  if defined(ERROR_STATIC_INIT)
    " " ":ERROR_STATIC_INIT"
#  endif
#  if defined(ERROR_INC_BACKUP)
    " " ":ERROR_INC_BACKUP"
#  endif
#  if defined(ERROR_FULL_BACKUP)
    " " ":ERROR_FULL_BACKUP"
#  endif
#  if defined(ERROR_REC_NON_EXISTENT)
    " " ":ERROR_REC_NON_EXISTENT"
#  endif
#  if defined(ERROR_RPL_NOT_ALLOWED)
    " " ":ERROR_RPL_NOT_ALLOWED"
#  endif
#  if defined(ERROR_DHCP_ADDRESS_CONFLICT)
    " " ":ERROR_DHCP_ADDRESS_CONFLICT"
#  endif
#  if defined(ERROR_WMI_GUID_NOT_FOUND)
    " " ":ERROR_WMI_GUID_NOT_FOUND"
#  endif
#  if defined(ERROR_WMI_INSTANCE_NOT_FOUND)
    " " ":ERROR_WMI_INSTANCE_NOT_FOUND"
#  endif
#  if defined(ERROR_WMI_ITEMID_NOT_FOUND)
    " " ":ERROR_WMI_ITEMID_NOT_FOUND"
#  endif
#  if defined(ERROR_WMI_TRY_AGAIN)
    " " ":ERROR_WMI_TRY_AGAIN"
#  endif
#  if defined(ERROR_WMI_DP_NOT_FOUND)
    " " ":ERROR_WMI_DP_NOT_FOUND"
#  endif
#  if defined(ERROR_WMI_UNRESOLVED_INSTANCE_REF)
    " " ":ERROR_WMI_UNRESOLVED_INSTANCE_REF"
#  endif
#  if defined(ERROR_WMI_ALREADY_ENABLED)
    " " ":ERROR_WMI_ALREADY_ENABLED"
#  endif
#  if defined(ERROR_WMI_GUID_DISCONNECTED)
    " " ":ERROR_WMI_GUID_DISCONNECTED"
#  endif
#  if defined(ERROR_WMI_SERVER_UNAVAILABLE)
    " " ":ERROR_WMI_SERVER_UNAVAILABLE"
#  endif
#  if defined(ERROR_WMI_DP_FAILED)
    " " ":ERROR_WMI_DP_FAILED"
#  endif
#  if defined(ERROR_WMI_INVALID_MOF)
    " " ":ERROR_WMI_INVALID_MOF"
#  endif
#  if defined(ERROR_WMI_INVALID_REGINFO)
    " " ":ERROR_WMI_INVALID_REGINFO"
#  endif
#  if defined(ERROR_WMI_ALREADY_DISABLED)
    " " ":ERROR_WMI_ALREADY_DISABLED"
#  endif
#  if defined(ERROR_WMI_READ_ONLY)
    " " ":ERROR_WMI_READ_ONLY"
#  endif
#  if defined(ERROR_WMI_SET_FAILURE)
    " " ":ERROR_WMI_SET_FAILURE"
#  endif
#  if defined(ERROR_INVALID_MEDIA)
    " " ":ERROR_INVALID_MEDIA"
#  endif
#  if defined(ERROR_INVALID_LIBRARY)
    " " ":ERROR_INVALID_LIBRARY"
#  endif
#  if defined(ERROR_INVALID_MEDIA_POOL)
    " " ":ERROR_INVALID_MEDIA_POOL"
#  endif
#  if defined(ERROR_DRIVE_MEDIA_MISMATCH)
    " " ":ERROR_DRIVE_MEDIA_MISMATCH"
#  endif
#  if defined(ERROR_MEDIA_OFFLINE)
    " " ":ERROR_MEDIA_OFFLINE"
#  endif
#  if defined(ERROR_LIBRARY_OFFLINE)
    " " ":ERROR_LIBRARY_OFFLINE"
#  endif
#  if defined(ERROR_EMPTY)
    " " ":ERROR_EMPTY"
#  endif
#  if defined(ERROR_NOT_EMPTY)
    " " ":ERROR_NOT_EMPTY"
#  endif
#  if defined(ERROR_MEDIA_UNAVAILABLE)
    " " ":ERROR_MEDIA_UNAVAILABLE"
#  endif
#  if defined(ERROR_RESOURCE_DISABLED)
    " " ":ERROR_RESOURCE_DISABLED"
#  endif
#  if defined(ERROR_INVALID_CLEANER)
    " " ":ERROR_INVALID_CLEANER"
#  endif
#  if defined(ERROR_UNABLE_TO_CLEAN)
    " " ":ERROR_UNABLE_TO_CLEAN"
#  endif
#  if defined(ERROR_OBJECT_NOT_FOUND)
    " " ":ERROR_OBJECT_NOT_FOUND"
#  endif
#  if defined(ERROR_DATABASE_FAILURE)
    " " ":ERROR_DATABASE_FAILURE"
#  endif
#  if defined(ERROR_DATABASE_FULL)
    " " ":ERROR_DATABASE_FULL"
#  endif
#  if defined(ERROR_MEDIA_INCOMPATIBLE)
    " " ":ERROR_MEDIA_INCOMPATIBLE"
#  endif
#  if defined(ERROR_RESOURCE_NOT_PRESENT)
    " " ":ERROR_RESOURCE_NOT_PRESENT"
#  endif
#  if defined(ERROR_INVALID_OPERATION)
    " " ":ERROR_INVALID_OPERATION"
#  endif
#  if defined(ERROR_MEDIA_NOT_AVAILABLE)
    " " ":ERROR_MEDIA_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_DEVICE_NOT_AVAILABLE)
    " " ":ERROR_DEVICE_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_REQUEST_REFUSED)
    " " ":ERROR_REQUEST_REFUSED"
#  endif
#  if defined(ERROR_INVALID_DRIVE_OBJECT)
    " " ":ERROR_INVALID_DRIVE_OBJECT"
#  endif
#  if defined(ERROR_LIBRARY_FULL)
    " " ":ERROR_LIBRARY_FULL"
#  endif
#  if defined(ERROR_MEDIUM_NOT_ACCESSIBLE)
    " " ":ERROR_MEDIUM_NOT_ACCESSIBLE"
#  endif
#  if defined(ERROR_UNABLE_TO_LOAD_MEDIUM)
    " " ":ERROR_UNABLE_TO_LOAD_MEDIUM"
#  endif
#  if defined(ERROR_UNABLE_TO_INVENTORY_DRIVE)
    " " ":ERROR_UNABLE_TO_INVENTORY_DRIVE"
#  endif
#  if defined(ERROR_UNABLE_TO_INVENTORY_SLOT)
    " " ":ERROR_UNABLE_TO_INVENTORY_SLOT"
#  endif
#  if defined(ERROR_UNABLE_TO_INVENTORY_TRANSPORT)
    " " ":ERROR_UNABLE_TO_INVENTORY_TRANSPORT"
#  endif
#  if defined(ERROR_TRANSPORT_FULL)
    " " ":ERROR_TRANSPORT_FULL"
#  endif
#  if defined(ERROR_CONTROLLING_IEPORT)
    " " ":ERROR_CONTROLLING_IEPORT"
#  endif
#  if defined(ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA)
    " " ":ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA"
#  endif
#  if defined(ERROR_CLEANER_SLOT_SET)
    " " ":ERROR_CLEANER_SLOT_SET"
#  endif
#  if defined(ERROR_CLEANER_SLOT_NOT_SET)
    " " ":ERROR_CLEANER_SLOT_NOT_SET"
#  endif
#  if defined(ERROR_CLEANER_CARTRIDGE_SPENT)
    " " ":ERROR_CLEANER_CARTRIDGE_SPENT"
#  endif
#  if defined(ERROR_UNEXPECTED_OMID)
    " " ":ERROR_UNEXPECTED_OMID"
#  endif
#  if defined(ERROR_CANT_DELETE_LAST_ITEM)
    " " ":ERROR_CANT_DELETE_LAST_ITEM"
#  endif
#  if defined(ERROR_MESSAGE_EXCEEDS_MAX_SIZE)
    " " ":ERROR_MESSAGE_EXCEEDS_MAX_SIZE"
#  endif
#  if defined(ERROR_VOLUME_CONTAINS_SYS_FILES)
    " " ":ERROR_VOLUME_CONTAINS_SYS_FILES"
#  endif
#  if defined(ERROR_INDIGENOUS_TYPE)
    " " ":ERROR_INDIGENOUS_TYPE"
#  endif
#  if defined(ERROR_NO_SUPPORTING_DRIVES)
    " " ":ERROR_NO_SUPPORTING_DRIVES"
#  endif
#  if defined(ERROR_CLEANER_CARTRIDGE_INSTALLED)
    " " ":ERROR_CLEANER_CARTRIDGE_INSTALLED"
#  endif
#  if defined(ERROR_FILE_OFFLINE)
    " " ":ERROR_FILE_OFFLINE"
#  endif
#  if defined(ERROR_REMOTE_STORAGE_NOT_ACTIVE)
    " " ":ERROR_REMOTE_STORAGE_NOT_ACTIVE"
#  endif
#  if defined(ERROR_REMOTE_STORAGE_MEDIA_ERROR)
    " " ":ERROR_REMOTE_STORAGE_MEDIA_ERROR"
#  endif
#  if defined(ERROR_NOT_A_REPARSE_POINT)
    " " ":ERROR_NOT_A_REPARSE_POINT"
#  endif
#  if defined(ERROR_REPARSE_ATTRIBUTE_CONFLICT)
    " " ":ERROR_REPARSE_ATTRIBUTE_CONFLICT"
#  endif
#  if defined(ERROR_INVALID_REPARSE_DATA)
    " " ":ERROR_INVALID_REPARSE_DATA"
#  endif
#  if defined(ERROR_REPARSE_TAG_INVALID)
    " " ":ERROR_REPARSE_TAG_INVALID"
#  endif
#  if defined(ERROR_REPARSE_TAG_MISMATCH)
    " " ":ERROR_REPARSE_TAG_MISMATCH"
#  endif
#  if defined(ERROR_VOLUME_NOT_SIS_ENABLED)
    " " ":ERROR_VOLUME_NOT_SIS_ENABLED"
#  endif
#  if defined(ERROR_DEPENDENT_RESOURCE_EXISTS)
    " " ":ERROR_DEPENDENT_RESOURCE_EXISTS"
#  endif
#  if defined(ERROR_DEPENDENCY_NOT_FOUND)
    " " ":ERROR_DEPENDENCY_NOT_FOUND"
#  endif
#  if defined(ERROR_DEPENDENCY_ALREADY_EXISTS)
    " " ":ERROR_DEPENDENCY_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_RESOURCE_NOT_ONLINE)
    " " ":ERROR_RESOURCE_NOT_ONLINE"
#  endif
#  if defined(ERROR_HOST_NODE_NOT_AVAILABLE)
    " " ":ERROR_HOST_NODE_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_RESOURCE_NOT_AVAILABLE)
    " " ":ERROR_RESOURCE_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_RESOURCE_NOT_FOUND)
    " " ":ERROR_RESOURCE_NOT_FOUND"
#  endif
#  if defined(ERROR_SHUTDOWN_CLUSTER)
    " " ":ERROR_SHUTDOWN_CLUSTER"
#  endif
#  if defined(ERROR_CANT_EVICT_ACTIVE_NODE)
    " " ":ERROR_CANT_EVICT_ACTIVE_NODE"
#  endif
#  if defined(ERROR_OBJECT_ALREADY_EXISTS)
    " " ":ERROR_OBJECT_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_OBJECT_IN_LIST)
    " " ":ERROR_OBJECT_IN_LIST"
#  endif
#  if defined(ERROR_GROUP_NOT_AVAILABLE)
    " " ":ERROR_GROUP_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_GROUP_NOT_FOUND)
    " " ":ERROR_GROUP_NOT_FOUND"
#  endif
#  if defined(ERROR_GROUP_NOT_ONLINE)
    " " ":ERROR_GROUP_NOT_ONLINE"
#  endif
#  if defined(ERROR_HOST_NODE_NOT_RESOURCE_OWNER)
    " " ":ERROR_HOST_NODE_NOT_RESOURCE_OWNER"
#  endif
#  if defined(ERROR_HOST_NODE_NOT_GROUP_OWNER)
    " " ":ERROR_HOST_NODE_NOT_GROUP_OWNER"
#  endif
#  if defined(ERROR_RESMON_CREATE_FAILED)
    " " ":ERROR_RESMON_CREATE_FAILED"
#  endif
#  if defined(ERROR_RESMON_ONLINE_FAILED)
    " " ":ERROR_RESMON_ONLINE_FAILED"
#  endif
#  if defined(ERROR_RESOURCE_ONLINE)
    " " ":ERROR_RESOURCE_ONLINE"
#  endif
#  if defined(ERROR_QUORUM_RESOURCE)
    " " ":ERROR_QUORUM_RESOURCE"
#  endif
#  if defined(ERROR_NOT_QUORUM_CAPABLE)
    " " ":ERROR_NOT_QUORUM_CAPABLE"
#  endif
#  if defined(ERROR_CLUSTER_SHUTTING_DOWN)
    " " ":ERROR_CLUSTER_SHUTTING_DOWN"
#  endif
#  if defined(ERROR_INVALID_STATE)
    " " ":ERROR_INVALID_STATE"
#  endif
#  if defined(ERROR_RESOURCE_PROPERTIES_STORED)
    " " ":ERROR_RESOURCE_PROPERTIES_STORED"
#  endif
#  if defined(ERROR_NOT_QUORUM_CLASS)
    " " ":ERROR_NOT_QUORUM_CLASS"
#  endif
#  if defined(ERROR_CORE_RESOURCE)
    " " ":ERROR_CORE_RESOURCE"
#  endif
#  if defined(ERROR_QUORUM_RESOURCE_ONLINE_FAILED)
    " " ":ERROR_QUORUM_RESOURCE_ONLINE_FAILED"
#  endif
#  if defined(ERROR_QUORUMLOG_OPEN_FAILED)
    " " ":ERROR_QUORUMLOG_OPEN_FAILED"
#  endif
#  if defined(ERROR_CLUSTERLOG_CORRUPT)
    " " ":ERROR_CLUSTERLOG_CORRUPT"
#  endif
#  if defined(ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE)
    " " ":ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE"
#  endif
#  if defined(ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE)
    " " ":ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE"
#  endif
#  if defined(ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND)
    " " ":ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE)
    " " ":ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE"
#  endif
#  if defined(ERROR_QUORUM_OWNER_ALIVE)
    " " ":ERROR_QUORUM_OWNER_ALIVE"
#  endif
#  if defined(ERROR_NETWORK_NOT_AVAILABLE)
    " " ":ERROR_NETWORK_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_NODE_NOT_AVAILABLE)
    " " ":ERROR_NODE_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_ALL_NODES_NOT_AVAILABLE)
    " " ":ERROR_ALL_NODES_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_RESOURCE_FAILED)
    " " ":ERROR_RESOURCE_FAILED"
#  endif
#  if defined(ERROR_CLUSTER_INVALID_NODE)
    " " ":ERROR_CLUSTER_INVALID_NODE"
#  endif
#  if defined(ERROR_CLUSTER_NODE_EXISTS)
    " " ":ERROR_CLUSTER_NODE_EXISTS"
#  endif
#  if defined(ERROR_CLUSTER_JOIN_IN_PROGRESS)
    " " ":ERROR_CLUSTER_JOIN_IN_PROGRESS"
#  endif
#  if defined(ERROR_CLUSTER_NODE_NOT_FOUND)
    " " ":ERROR_CLUSTER_NODE_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND)
    " " ":ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTER_NETWORK_EXISTS)
    " " ":ERROR_CLUSTER_NETWORK_EXISTS"
#  endif
#  if defined(ERROR_CLUSTER_NETWORK_NOT_FOUND)
    " " ":ERROR_CLUSTER_NETWORK_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTER_NETINTERFACE_EXISTS)
    " " ":ERROR_CLUSTER_NETINTERFACE_EXISTS"
#  endif
#  if defined(ERROR_CLUSTER_NETINTERFACE_NOT_FOUND)
    " " ":ERROR_CLUSTER_NETINTERFACE_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTER_INVALID_REQUEST)
    " " ":ERROR_CLUSTER_INVALID_REQUEST"
#  endif
#  if defined(ERROR_CLUSTER_INVALID_NETWORK_PROVIDER)
    " " ":ERROR_CLUSTER_INVALID_NETWORK_PROVIDER"
#  endif
#  if defined(ERROR_CLUSTER_NODE_DOWN)
    " " ":ERROR_CLUSTER_NODE_DOWN"
#  endif
#  if defined(ERROR_CLUSTER_NODE_UNREACHABLE)
    " " ":ERROR_CLUSTER_NODE_UNREACHABLE"
#  endif
#  if defined(ERROR_CLUSTER_NODE_NOT_MEMBER)
    " " ":ERROR_CLUSTER_NODE_NOT_MEMBER"
#  endif
#  if defined(ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS)
    " " ":ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS"
#  endif
#  if defined(ERROR_CLUSTER_INVALID_NETWORK)
    " " ":ERROR_CLUSTER_INVALID_NETWORK"
#  endif
#  if defined(ERROR_CLUSTER_NODE_UP)
    " " ":ERROR_CLUSTER_NODE_UP"
#  endif
#  if defined(ERROR_CLUSTER_IPADDR_IN_USE)
    " " ":ERROR_CLUSTER_IPADDR_IN_USE"
#  endif
#  if defined(ERROR_CLUSTER_NODE_NOT_PAUSED)
    " " ":ERROR_CLUSTER_NODE_NOT_PAUSED"
#  endif
#  if defined(ERROR_CLUSTER_NO_SECURITY_CONTEXT)
    " " ":ERROR_CLUSTER_NO_SECURITY_CONTEXT"
#  endif
#  if defined(ERROR_CLUSTER_NETWORK_NOT_INTERNAL)
    " " ":ERROR_CLUSTER_NETWORK_NOT_INTERNAL"
#  endif
#  if defined(ERROR_CLUSTER_NODE_ALREADY_UP)
    " " ":ERROR_CLUSTER_NODE_ALREADY_UP"
#  endif
#  if defined(ERROR_CLUSTER_NODE_ALREADY_DOWN)
    " " ":ERROR_CLUSTER_NODE_ALREADY_DOWN"
#  endif
#  if defined(ERROR_CLUSTER_NETWORK_ALREADY_ONLINE)
    " " ":ERROR_CLUSTER_NETWORK_ALREADY_ONLINE"
#  endif
#  if defined(ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE)
    " " ":ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE"
#  endif
#  if defined(ERROR_CLUSTER_NODE_ALREADY_MEMBER)
    " " ":ERROR_CLUSTER_NODE_ALREADY_MEMBER"
#  endif
#  if defined(ERROR_CLUSTER_LAST_INTERNAL_NETWORK)
    " " ":ERROR_CLUSTER_LAST_INTERNAL_NETWORK"
#  endif
#  if defined(ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS)
    " " ":ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS"
#  endif
#  if defined(ERROR_INVALID_OPERATION_ON_QUORUM)
    " " ":ERROR_INVALID_OPERATION_ON_QUORUM"
#  endif
#  if defined(ERROR_DEPENDENCY_NOT_ALLOWED)
    " " ":ERROR_DEPENDENCY_NOT_ALLOWED"
#  endif
#  if defined(ERROR_CLUSTER_NODE_PAUSED)
    " " ":ERROR_CLUSTER_NODE_PAUSED"
#  endif
#  if defined(ERROR_NODE_CANT_HOST_RESOURCE)
    " " ":ERROR_NODE_CANT_HOST_RESOURCE"
#  endif
#  if defined(ERROR_CLUSTER_NODE_NOT_READY)
    " " ":ERROR_CLUSTER_NODE_NOT_READY"
#  endif
#  if defined(ERROR_CLUSTER_NODE_SHUTTING_DOWN)
    " " ":ERROR_CLUSTER_NODE_SHUTTING_DOWN"
#  endif
#  if defined(ERROR_CLUSTER_JOIN_ABORTED)
    " " ":ERROR_CLUSTER_JOIN_ABORTED"
#  endif
#  if defined(ERROR_CLUSTER_INCOMPATIBLE_VERSIONS)
    " " ":ERROR_CLUSTER_INCOMPATIBLE_VERSIONS"
#  endif
#  if defined(ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED)
    " " ":ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED"
#  endif
#  if defined(ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED)
    " " ":ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED"
#  endif
#  if defined(ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND)
    " " ":ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED)
    " " ":ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_CLUSTER_RESNAME_NOT_FOUND)
    " " ":ERROR_CLUSTER_RESNAME_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED)
    " " ":ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED"
#  endif
#  if defined(ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST)
    " " ":ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST"
#  endif
#  if defined(ERROR_CLUSTER_DATABASE_SEQMISMATCH)
    " " ":ERROR_CLUSTER_DATABASE_SEQMISMATCH"
#  endif
#  if defined(ERROR_RESMON_INVALID_STATE)
    " " ":ERROR_RESMON_INVALID_STATE"
#  endif
#  if defined(ERROR_CLUSTER_GUM_NOT_LOCKER)
    " " ":ERROR_CLUSTER_GUM_NOT_LOCKER"
#  endif
#  if defined(ERROR_QUORUM_DISK_NOT_FOUND)
    " " ":ERROR_QUORUM_DISK_NOT_FOUND"
#  endif
#  if defined(ERROR_DATABASE_BACKUP_CORRUPT)
    " " ":ERROR_DATABASE_BACKUP_CORRUPT"
#  endif
#  if defined(ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT)
    " " ":ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT"
#  endif
#  if defined(ERROR_RESOURCE_PROPERTY_UNCHANGEABLE)
    " " ":ERROR_RESOURCE_PROPERTY_UNCHANGEABLE"
#  endif
#  if defined(ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE)
    " " ":ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE"
#  endif
#  if defined(ERROR_CLUSTER_QUORUMLOG_NOT_FOUND)
    " " ":ERROR_CLUSTER_QUORUMLOG_NOT_FOUND"
#  endif
#  if defined(ERROR_CLUSTER_MEMBERSHIP_HALT)
    " " ":ERROR_CLUSTER_MEMBERSHIP_HALT"
#  endif
#  if defined(ERROR_CLUSTER_INSTANCE_ID_MISMATCH)
    " " ":ERROR_CLUSTER_INSTANCE_ID_MISMATCH"
#  endif
#  if defined(ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP)
    " " ":ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP"
#  endif
#  if defined(ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH)
    " " ":ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH"
#  endif
#  if defined(ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP)
    " " ":ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP"
#  endif
#  if defined(ERROR_CLUSTER_PARAMETER_MISMATCH)
    " " ":ERROR_CLUSTER_PARAMETER_MISMATCH"
#  endif
#  if defined(ERROR_NODE_CANNOT_BE_CLUSTERED)
    " " ":ERROR_NODE_CANNOT_BE_CLUSTERED"
#  endif
#  if defined(ERROR_CLUSTER_WRONG_OS_VERSION)
    " " ":ERROR_CLUSTER_WRONG_OS_VERSION"
#  endif
#  if defined(ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME)
    " " ":ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME"
#  endif
#  if defined(ERROR_CLUSCFG_ALREADY_COMMITTED)
    " " ":ERROR_CLUSCFG_ALREADY_COMMITTED"
#  endif
#  if defined(ERROR_CLUSCFG_ROLLBACK_FAILED)
    " " ":ERROR_CLUSCFG_ROLLBACK_FAILED"
#  endif
#  if defined(ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT)
    " " ":ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT"
#  endif
#  if defined(ERROR_CLUSTER_OLD_VERSION)
    " " ":ERROR_CLUSTER_OLD_VERSION"
#  endif
#  if defined(ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME)
    " " ":ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME"
#  endif
#  if defined(ERROR_ENCRYPTION_FAILED)
    " " ":ERROR_ENCRYPTION_FAILED"
#  endif
#  if defined(ERROR_DECRYPTION_FAILED)
    " " ":ERROR_DECRYPTION_FAILED"
#  endif
#  if defined(ERROR_FILE_ENCRYPTED)
    " " ":ERROR_FILE_ENCRYPTED"
#  endif
#  if defined(ERROR_NO_RECOVERY_POLICY)
    " " ":ERROR_NO_RECOVERY_POLICY"
#  endif
#  if defined(ERROR_NO_EFS)
    " " ":ERROR_NO_EFS"
#  endif
#  if defined(ERROR_WRONG_EFS)
    " " ":ERROR_WRONG_EFS"
#  endif
#  if defined(ERROR_NO_USER_KEYS)
    " " ":ERROR_NO_USER_KEYS"
#  endif
#  if defined(ERROR_FILE_NOT_ENCRYPTED)
    " " ":ERROR_FILE_NOT_ENCRYPTED"
#  endif
#  if defined(ERROR_NOT_EXPORT_FORMAT)
    " " ":ERROR_NOT_EXPORT_FORMAT"
#  endif
#  if defined(ERROR_FILE_READ_ONLY)
    " " ":ERROR_FILE_READ_ONLY"
#  endif
#  if defined(ERROR_DIR_EFS_DISALLOWED)
    " " ":ERROR_DIR_EFS_DISALLOWED"
#  endif
#  if defined(ERROR_EFS_SERVER_NOT_TRUSTED)
    " " ":ERROR_EFS_SERVER_NOT_TRUSTED"
#  endif
#  if defined(ERROR_BAD_RECOVERY_POLICY)
    " " ":ERROR_BAD_RECOVERY_POLICY"
#  endif
#  if defined(ERROR_EFS_ALG_BLOB_TOO_BIG)
    " " ":ERROR_EFS_ALG_BLOB_TOO_BIG"
#  endif
#  if defined(ERROR_VOLUME_NOT_SUPPORT_EFS)
    " " ":ERROR_VOLUME_NOT_SUPPORT_EFS"
#  endif
#  if defined(ERROR_EFS_DISABLED)
    " " ":ERROR_EFS_DISABLED"
#  endif
#  if defined(ERROR_EFS_VERSION_NOT_SUPPORT)
    " " ":ERROR_EFS_VERSION_NOT_SUPPORT"
#  endif
#  if defined(ERROR_NO_BROWSER_SERVERS_FOUND)
    " " ":ERROR_NO_BROWSER_SERVERS_FOUND"
#  endif
#  if defined(SCHED_E_SERVICE_NOT_LOCALSYSTEM)
    " " ":SCHED_E_SERVICE_NOT_LOCALSYSTEM"
#  endif
#  if defined(ERROR_CTX_WINSTATION_NAME_INVALID)
    " " ":ERROR_CTX_WINSTATION_NAME_INVALID"
#  endif
#  if defined(ERROR_CTX_INVALID_PD)
    " " ":ERROR_CTX_INVALID_PD"
#  endif
#  if defined(ERROR_CTX_PD_NOT_FOUND)
    " " ":ERROR_CTX_PD_NOT_FOUND"
#  endif
#  if defined(ERROR_CTX_WD_NOT_FOUND)
    " " ":ERROR_CTX_WD_NOT_FOUND"
#  endif
#  if defined(ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY)
    " " ":ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY"
#  endif
#  if defined(ERROR_CTX_SERVICE_NAME_COLLISION)
    " " ":ERROR_CTX_SERVICE_NAME_COLLISION"
#  endif
#  if defined(ERROR_CTX_CLOSE_PENDING)
    " " ":ERROR_CTX_CLOSE_PENDING"
#  endif
#  if defined(ERROR_CTX_NO_OUTBUF)
    " " ":ERROR_CTX_NO_OUTBUF"
#  endif
#  if defined(ERROR_CTX_MODEM_INF_NOT_FOUND)
    " " ":ERROR_CTX_MODEM_INF_NOT_FOUND"
#  endif
#  if defined(ERROR_CTX_INVALID_MODEMNAME)
    " " ":ERROR_CTX_INVALID_MODEMNAME"
#  endif
#  if defined(ERROR_CTX_MODEM_RESPONSE_ERROR)
    " " ":ERROR_CTX_MODEM_RESPONSE_ERROR"
#  endif
#  if defined(ERROR_CTX_MODEM_RESPONSE_TIMEOUT)
    " " ":ERROR_CTX_MODEM_RESPONSE_TIMEOUT"
#  endif
#  if defined(ERROR_CTX_MODEM_RESPONSE_NO_CARRIER)
    " " ":ERROR_CTX_MODEM_RESPONSE_NO_CARRIER"
#  endif
#  if defined(ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE)
    " " ":ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE"
#  endif
#  if defined(ERROR_CTX_MODEM_RESPONSE_BUSY)
    " " ":ERROR_CTX_MODEM_RESPONSE_BUSY"
#  endif
#  if defined(ERROR_CTX_MODEM_RESPONSE_VOICE)
    " " ":ERROR_CTX_MODEM_RESPONSE_VOICE"
#  endif
#  if defined(ERROR_CTX_TD_ERROR)
    " " ":ERROR_CTX_TD_ERROR"
#  endif
#  if defined(ERROR_CTX_WINSTATION_NOT_FOUND)
    " " ":ERROR_CTX_WINSTATION_NOT_FOUND"
#  endif
#  if defined(ERROR_CTX_WINSTATION_ALREADY_EXISTS)
    " " ":ERROR_CTX_WINSTATION_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_CTX_WINSTATION_BUSY)
    " " ":ERROR_CTX_WINSTATION_BUSY"
#  endif
#  if defined(ERROR_CTX_BAD_VIDEO_MODE)
    " " ":ERROR_CTX_BAD_VIDEO_MODE"
#  endif
#  if defined(ERROR_CTX_GRAPHICS_INVALID)
    " " ":ERROR_CTX_GRAPHICS_INVALID"
#  endif
#  if defined(ERROR_CTX_LOGON_DISABLED)
    " " ":ERROR_CTX_LOGON_DISABLED"
#  endif
#  if defined(ERROR_CTX_NOT_CONSOLE)
    " " ":ERROR_CTX_NOT_CONSOLE"
#  endif
#  if defined(ERROR_CTX_CLIENT_QUERY_TIMEOUT)
    " " ":ERROR_CTX_CLIENT_QUERY_TIMEOUT"
#  endif
#  if defined(ERROR_CTX_CONSOLE_DISCONNECT)
    " " ":ERROR_CTX_CONSOLE_DISCONNECT"
#  endif
#  if defined(ERROR_CTX_CONSOLE_CONNECT)
    " " ":ERROR_CTX_CONSOLE_CONNECT"
#  endif
#  if defined(ERROR_CTX_SHADOW_DENIED)
    " " ":ERROR_CTX_SHADOW_DENIED"
#  endif
#  if defined(ERROR_CTX_WINSTATION_ACCESS_DENIED)
    " " ":ERROR_CTX_WINSTATION_ACCESS_DENIED"
#  endif
#  if defined(ERROR_CTX_INVALID_WD)
    " " ":ERROR_CTX_INVALID_WD"
#  endif
#  if defined(ERROR_CTX_SHADOW_INVALID)
    " " ":ERROR_CTX_SHADOW_INVALID"
#  endif
#  if defined(ERROR_CTX_SHADOW_DISABLED)
    " " ":ERROR_CTX_SHADOW_DISABLED"
#  endif
#  if defined(ERROR_CTX_CLIENT_LICENSE_IN_USE)
    " " ":ERROR_CTX_CLIENT_LICENSE_IN_USE"
#  endif
#  if defined(ERROR_CTX_CLIENT_LICENSE_NOT_SET)
    " " ":ERROR_CTX_CLIENT_LICENSE_NOT_SET"
#  endif
#  if defined(ERROR_CTX_LICENSE_NOT_AVAILABLE)
    " " ":ERROR_CTX_LICENSE_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_CTX_LICENSE_CLIENT_INVALID)
    " " ":ERROR_CTX_LICENSE_CLIENT_INVALID"
#  endif
#  if defined(ERROR_CTX_LICENSE_EXPIRED)
    " " ":ERROR_CTX_LICENSE_EXPIRED"
#  endif
#  if defined(ERROR_CTX_SHADOW_NOT_RUNNING)
    " " ":ERROR_CTX_SHADOW_NOT_RUNNING"
#  endif
#  if defined(ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE)
    " " ":ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE"
#  endif
#  if defined(ERROR_ACTIVATION_COUNT_EXCEEDED)
    " " ":ERROR_ACTIVATION_COUNT_EXCEEDED"
#  endif
#  if defined(FRS_ERR_INVALID_API_SEQUENCE)
    " " ":FRS_ERR_INVALID_API_SEQUENCE"
#  endif
#  if defined(FRS_ERR_STARTING_SERVICE)
    " " ":FRS_ERR_STARTING_SERVICE"
#  endif
#  if defined(FRS_ERR_STOPPING_SERVICE)
    " " ":FRS_ERR_STOPPING_SERVICE"
#  endif
#  if defined(FRS_ERR_INTERNAL_API)
    " " ":FRS_ERR_INTERNAL_API"
#  endif
#  if defined(FRS_ERR_INTERNAL)
    " " ":FRS_ERR_INTERNAL"
#  endif
#  if defined(FRS_ERR_SERVICE_COMM)
    " " ":FRS_ERR_SERVICE_COMM"
#  endif
#  if defined(FRS_ERR_INSUFFICIENT_PRIV)
    " " ":FRS_ERR_INSUFFICIENT_PRIV"
#  endif
#  if defined(FRS_ERR_AUTHENTICATION)
    " " ":FRS_ERR_AUTHENTICATION"
#  endif
#  if defined(FRS_ERR_PARENT_INSUFFICIENT_PRIV)
    " " ":FRS_ERR_PARENT_INSUFFICIENT_PRIV"
#  endif
#  if defined(FRS_ERR_PARENT_AUTHENTICATION)
    " " ":FRS_ERR_PARENT_AUTHENTICATION"
#  endif
#  if defined(FRS_ERR_CHILD_TO_PARENT_COMM)
    " " ":FRS_ERR_CHILD_TO_PARENT_COMM"
#  endif
#  if defined(FRS_ERR_PARENT_TO_CHILD_COMM)
    " " ":FRS_ERR_PARENT_TO_CHILD_COMM"
#  endif
#  if defined(FRS_ERR_SYSVOL_POPULATE)
    " " ":FRS_ERR_SYSVOL_POPULATE"
#  endif
#  if defined(FRS_ERR_SYSVOL_POPULATE_TIMEOUT)
    " " ":FRS_ERR_SYSVOL_POPULATE_TIMEOUT"
#  endif
#  if defined(FRS_ERR_SYSVOL_IS_BUSY)
    " " ":FRS_ERR_SYSVOL_IS_BUSY"
#  endif
#  if defined(FRS_ERR_SYSVOL_DEMOTE)
    " " ":FRS_ERR_SYSVOL_DEMOTE"
#  endif
#  if defined(FRS_ERR_INVALID_SERVICE_PARAMETER)
    " " ":FRS_ERR_INVALID_SERVICE_PARAMETER"
#  endif
#  if defined(ERROR_DS_NOT_INSTALLED)
    " " ":ERROR_DS_NOT_INSTALLED"
#  endif
#  if defined(ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY)
    " " ":ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY"
#  endif
#  if defined(ERROR_DS_NO_ATTRIBUTE_OR_VALUE)
    " " ":ERROR_DS_NO_ATTRIBUTE_OR_VALUE"
#  endif
#  if defined(ERROR_DS_INVALID_ATTRIBUTE_SYNTAX)
    " " ":ERROR_DS_INVALID_ATTRIBUTE_SYNTAX"
#  endif
#  if defined(ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED)
    " " ":ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED"
#  endif
#  if defined(ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS)
    " " ":ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS"
#  endif
#  if defined(ERROR_DS_BUSY)
    " " ":ERROR_DS_BUSY"
#  endif
#  if defined(ERROR_DS_UNAVAILABLE)
    " " ":ERROR_DS_UNAVAILABLE"
#  endif
#  if defined(ERROR_DS_NO_RIDS_ALLOCATED)
    " " ":ERROR_DS_NO_RIDS_ALLOCATED"
#  endif
#  if defined(ERROR_DS_NO_MORE_RIDS)
    " " ":ERROR_DS_NO_MORE_RIDS"
#  endif
#  if defined(ERROR_DS_INCORRECT_ROLE_OWNER)
    " " ":ERROR_DS_INCORRECT_ROLE_OWNER"
#  endif
#  if defined(ERROR_DS_RIDMGR_INIT_ERROR)
    " " ":ERROR_DS_RIDMGR_INIT_ERROR"
#  endif
#  if defined(ERROR_DS_OBJ_CLASS_VIOLATION)
    " " ":ERROR_DS_OBJ_CLASS_VIOLATION"
#  endif
#  if defined(ERROR_DS_CANT_ON_NON_LEAF)
    " " ":ERROR_DS_CANT_ON_NON_LEAF"
#  endif
#  if defined(ERROR_DS_CANT_ON_RDN)
    " " ":ERROR_DS_CANT_ON_RDN"
#  endif
#  if defined(ERROR_DS_CANT_MOD_OBJ_CLASS)
    " " ":ERROR_DS_CANT_MOD_OBJ_CLASS"
#  endif
#  if defined(ERROR_DS_CROSS_DOM_MOVE_ERROR)
    " " ":ERROR_DS_CROSS_DOM_MOVE_ERROR"
#  endif
#  if defined(ERROR_DS_GC_NOT_AVAILABLE)
    " " ":ERROR_DS_GC_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_SHARED_POLICY)
    " " ":ERROR_SHARED_POLICY"
#  endif
#  if defined(ERROR_POLICY_OBJECT_NOT_FOUND)
    " " ":ERROR_POLICY_OBJECT_NOT_FOUND"
#  endif
#  if defined(ERROR_POLICY_ONLY_IN_DS)
    " " ":ERROR_POLICY_ONLY_IN_DS"
#  endif
#  if defined(ERROR_PROMOTION_ACTIVE)
    " " ":ERROR_PROMOTION_ACTIVE"
#  endif
#  if defined(ERROR_NO_PROMOTION_ACTIVE)
    " " ":ERROR_NO_PROMOTION_ACTIVE"
#  endif
#  if defined(ERROR_DS_OPERATIONS_ERROR)
    " " ":ERROR_DS_OPERATIONS_ERROR"
#  endif
#  if defined(ERROR_DS_PROTOCOL_ERROR)
    " " ":ERROR_DS_PROTOCOL_ERROR"
#  endif
#  if defined(ERROR_DS_TIMELIMIT_EXCEEDED)
    " " ":ERROR_DS_TIMELIMIT_EXCEEDED"
#  endif
#  if defined(ERROR_DS_SIZELIMIT_EXCEEDED)
    " " ":ERROR_DS_SIZELIMIT_EXCEEDED"
#  endif
#  if defined(ERROR_DS_ADMIN_LIMIT_EXCEEDED)
    " " ":ERROR_DS_ADMIN_LIMIT_EXCEEDED"
#  endif
#  if defined(ERROR_DS_COMPARE_FALSE)
    " " ":ERROR_DS_COMPARE_FALSE"
#  endif
#  if defined(ERROR_DS_COMPARE_TRUE)
    " " ":ERROR_DS_COMPARE_TRUE"
#  endif
#  if defined(ERROR_DS_AUTH_METHOD_NOT_SUPPORTED)
    " " ":ERROR_DS_AUTH_METHOD_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_DS_STRONG_AUTH_REQUIRED)
    " " ":ERROR_DS_STRONG_AUTH_REQUIRED"
#  endif
#  if defined(ERROR_DS_INAPPROPRIATE_AUTH)
    " " ":ERROR_DS_INAPPROPRIATE_AUTH"
#  endif
#  if defined(ERROR_DS_AUTH_UNKNOWN)
    " " ":ERROR_DS_AUTH_UNKNOWN"
#  endif
#  if defined(ERROR_DS_REFERRAL)
    " " ":ERROR_DS_REFERRAL"
#  endif
#  if defined(ERROR_DS_UNAVAILABLE_CRIT_EXTENSION)
    " " ":ERROR_DS_UNAVAILABLE_CRIT_EXTENSION"
#  endif
#  if defined(ERROR_DS_CONFIDENTIALITY_REQUIRED)
    " " ":ERROR_DS_CONFIDENTIALITY_REQUIRED"
#  endif
#  if defined(ERROR_DS_INAPPROPRIATE_MATCHING)
    " " ":ERROR_DS_INAPPROPRIATE_MATCHING"
#  endif
#  if defined(ERROR_DS_CONSTRAINT_VIOLATION)
    " " ":ERROR_DS_CONSTRAINT_VIOLATION"
#  endif
#  if defined(ERROR_DS_NO_SUCH_OBJECT)
    " " ":ERROR_DS_NO_SUCH_OBJECT"
#  endif
#  if defined(ERROR_DS_ALIAS_PROBLEM)
    " " ":ERROR_DS_ALIAS_PROBLEM"
#  endif
#  if defined(ERROR_DS_INVALID_DN_SYNTAX)
    " " ":ERROR_DS_INVALID_DN_SYNTAX"
#  endif
#  if defined(ERROR_DS_IS_LEAF)
    " " ":ERROR_DS_IS_LEAF"
#  endif
#  if defined(ERROR_DS_ALIAS_DEREF_PROBLEM)
    " " ":ERROR_DS_ALIAS_DEREF_PROBLEM"
#  endif
#  if defined(ERROR_DS_UNWILLING_TO_PERFORM)
    " " ":ERROR_DS_UNWILLING_TO_PERFORM"
#  endif
#  if defined(ERROR_DS_LOOP_DETECT)
    " " ":ERROR_DS_LOOP_DETECT"
#  endif
#  if defined(ERROR_DS_NAMING_VIOLATION)
    " " ":ERROR_DS_NAMING_VIOLATION"
#  endif
#  if defined(ERROR_DS_OBJECT_RESULTS_TOO_LARGE)
    " " ":ERROR_DS_OBJECT_RESULTS_TOO_LARGE"
#  endif
#  if defined(ERROR_DS_AFFECTS_MULTIPLE_DSAS)
    " " ":ERROR_DS_AFFECTS_MULTIPLE_DSAS"
#  endif
#  if defined(ERROR_DS_SERVER_DOWN)
    " " ":ERROR_DS_SERVER_DOWN"
#  endif
#  if defined(ERROR_DS_LOCAL_ERROR)
    " " ":ERROR_DS_LOCAL_ERROR"
#  endif
#  if defined(ERROR_DS_ENCODING_ERROR)
    " " ":ERROR_DS_ENCODING_ERROR"
#  endif
#  if defined(ERROR_DS_DECODING_ERROR)
    " " ":ERROR_DS_DECODING_ERROR"
#  endif
#  if defined(ERROR_DS_FILTER_UNKNOWN)
    " " ":ERROR_DS_FILTER_UNKNOWN"
#  endif
#  if defined(ERROR_DS_PARAM_ERROR)
    " " ":ERROR_DS_PARAM_ERROR"
#  endif
#  if defined(ERROR_DS_NOT_SUPPORTED)
    " " ":ERROR_DS_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_DS_NO_RESULTS_RETURNED)
    " " ":ERROR_DS_NO_RESULTS_RETURNED"
#  endif
#  if defined(ERROR_DS_CONTROL_NOT_FOUND)
    " " ":ERROR_DS_CONTROL_NOT_FOUND"
#  endif
#  if defined(ERROR_DS_CLIENT_LOOP)
    " " ":ERROR_DS_CLIENT_LOOP"
#  endif
#  if defined(ERROR_DS_REFERRAL_LIMIT_EXCEEDED)
    " " ":ERROR_DS_REFERRAL_LIMIT_EXCEEDED"
#  endif
#  if defined(ERROR_DS_SORT_CONTROL_MISSING)
    " " ":ERROR_DS_SORT_CONTROL_MISSING"
#  endif
#  if defined(ERROR_DS_OFFSET_RANGE_ERROR)
    " " ":ERROR_DS_OFFSET_RANGE_ERROR"
#  endif
#  if defined(ERROR_DS_ROOT_MUST_BE_NC)
    " " ":ERROR_DS_ROOT_MUST_BE_NC"
#  endif
#  if defined(ERROR_DS_ADD_REPLICA_INHIBITED)
    " " ":ERROR_DS_ADD_REPLICA_INHIBITED"
#  endif
#  if defined(ERROR_DS_ATT_NOT_DEF_IN_SCHEMA)
    " " ":ERROR_DS_ATT_NOT_DEF_IN_SCHEMA"
#  endif
#  if defined(ERROR_DS_MAX_OBJ_SIZE_EXCEEDED)
    " " ":ERROR_DS_MAX_OBJ_SIZE_EXCEEDED"
#  endif
#  if defined(ERROR_DS_OBJ_STRING_NAME_EXISTS)
    " " ":ERROR_DS_OBJ_STRING_NAME_EXISTS"
#  endif
#  if defined(ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA)
    " " ":ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA"
#  endif
#  if defined(ERROR_DS_RDN_DOESNT_MATCH_SCHEMA)
    " " ":ERROR_DS_RDN_DOESNT_MATCH_SCHEMA"
#  endif
#  if defined(ERROR_DS_NO_REQUESTED_ATTS_FOUND)
    " " ":ERROR_DS_NO_REQUESTED_ATTS_FOUND"
#  endif
#  if defined(ERROR_DS_USER_BUFFER_TO_SMALL)
    " " ":ERROR_DS_USER_BUFFER_TO_SMALL"
#  endif
#  if defined(ERROR_DS_ATT_IS_NOT_ON_OBJ)
    " " ":ERROR_DS_ATT_IS_NOT_ON_OBJ"
#  endif
#  if defined(ERROR_DS_ILLEGAL_MOD_OPERATION)
    " " ":ERROR_DS_ILLEGAL_MOD_OPERATION"
#  endif
#  if defined(ERROR_DS_OBJ_TOO_LARGE)
    " " ":ERROR_DS_OBJ_TOO_LARGE"
#  endif
#  if defined(ERROR_DS_BAD_INSTANCE_TYPE)
    " " ":ERROR_DS_BAD_INSTANCE_TYPE"
#  endif
#  if defined(ERROR_DS_MASTERDSA_REQUIRED)
    " " ":ERROR_DS_MASTERDSA_REQUIRED"
#  endif
#  if defined(ERROR_DS_OBJECT_CLASS_REQUIRED)
    " " ":ERROR_DS_OBJECT_CLASS_REQUIRED"
#  endif
#  if defined(ERROR_DS_MISSING_REQUIRED_ATT)
    " " ":ERROR_DS_MISSING_REQUIRED_ATT"
#  endif
#  if defined(ERROR_DS_ATT_NOT_DEF_FOR_CLASS)
    " " ":ERROR_DS_ATT_NOT_DEF_FOR_CLASS"
#  endif
#  if defined(ERROR_DS_ATT_ALREADY_EXISTS)
    " " ":ERROR_DS_ATT_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_DS_CANT_ADD_ATT_VALUES)
    " " ":ERROR_DS_CANT_ADD_ATT_VALUES"
#  endif
#  if defined(ERROR_DS_SINGLE_VALUE_CONSTRAINT)
    " " ":ERROR_DS_SINGLE_VALUE_CONSTRAINT"
#  endif
#  if defined(ERROR_DS_RANGE_CONSTRAINT)
    " " ":ERROR_DS_RANGE_CONSTRAINT"
#  endif
#  if defined(ERROR_DS_ATT_VAL_ALREADY_EXISTS)
    " " ":ERROR_DS_ATT_VAL_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_DS_CANT_REM_MISSING_ATT)
    " " ":ERROR_DS_CANT_REM_MISSING_ATT"
#  endif
#  if defined(ERROR_DS_CANT_REM_MISSING_ATT_VAL)
    " " ":ERROR_DS_CANT_REM_MISSING_ATT_VAL"
#  endif
#  if defined(ERROR_DS_ROOT_CANT_BE_SUBREF)
    " " ":ERROR_DS_ROOT_CANT_BE_SUBREF"
#  endif
#  if defined(ERROR_DS_NO_CHAINING)
    " " ":ERROR_DS_NO_CHAINING"
#  endif
#  if defined(ERROR_DS_NO_CHAINED_EVAL)
    " " ":ERROR_DS_NO_CHAINED_EVAL"
#  endif
#  if defined(ERROR_DS_NO_PARENT_OBJECT)
    " " ":ERROR_DS_NO_PARENT_OBJECT"
#  endif
#  if defined(ERROR_DS_PARENT_IS_AN_ALIAS)
    " " ":ERROR_DS_PARENT_IS_AN_ALIAS"
#  endif
#  if defined(ERROR_DS_CANT_MIX_MASTER_AND_REPS)
    " " ":ERROR_DS_CANT_MIX_MASTER_AND_REPS"
#  endif
#  if defined(ERROR_DS_CHILDREN_EXIST)
    " " ":ERROR_DS_CHILDREN_EXIST"
#  endif
#  if defined(ERROR_DS_OBJ_NOT_FOUND)
    " " ":ERROR_DS_OBJ_NOT_FOUND"
#  endif
#  if defined(ERROR_DS_ALIASED_OBJ_MISSING)
    " " ":ERROR_DS_ALIASED_OBJ_MISSING"
#  endif
#  if defined(ERROR_DS_BAD_NAME_SYNTAX)
    " " ":ERROR_DS_BAD_NAME_SYNTAX"
#  endif
#  if defined(ERROR_DS_ALIAS_POINTS_TO_ALIAS)
    " " ":ERROR_DS_ALIAS_POINTS_TO_ALIAS"
#  endif
#  if defined(ERROR_DS_CANT_DEREF_ALIAS)
    " " ":ERROR_DS_CANT_DEREF_ALIAS"
#  endif
#  if defined(ERROR_DS_OUT_OF_SCOPE)
    " " ":ERROR_DS_OUT_OF_SCOPE"
#  endif
#  if defined(ERROR_DS_OBJECT_BEING_REMOVED)
    " " ":ERROR_DS_OBJECT_BEING_REMOVED"
#  endif
#  if defined(ERROR_DS_CANT_DELETE_DSA_OBJ)
    " " ":ERROR_DS_CANT_DELETE_DSA_OBJ"
#  endif
#  if defined(ERROR_DS_GENERIC_ERROR)
    " " ":ERROR_DS_GENERIC_ERROR"
#  endif
#  if defined(ERROR_DS_DSA_MUST_BE_INT_MASTER)
    " " ":ERROR_DS_DSA_MUST_BE_INT_MASTER"
#  endif
#  if defined(ERROR_DS_CLASS_NOT_DSA)
    " " ":ERROR_DS_CLASS_NOT_DSA"
#  endif
#  if defined(ERROR_DS_INSUFF_ACCESS_RIGHTS)
    " " ":ERROR_DS_INSUFF_ACCESS_RIGHTS"
#  endif
#  if defined(ERROR_DS_ILLEGAL_SUPERIOR)
    " " ":ERROR_DS_ILLEGAL_SUPERIOR"
#  endif
#  if defined(ERROR_DS_ATTRIBUTE_OWNED_BY_SAM)
    " " ":ERROR_DS_ATTRIBUTE_OWNED_BY_SAM"
#  endif
#  if defined(ERROR_DS_NAME_TOO_MANY_PARTS)
    " " ":ERROR_DS_NAME_TOO_MANY_PARTS"
#  endif
#  if defined(ERROR_DS_NAME_TOO_LONG)
    " " ":ERROR_DS_NAME_TOO_LONG"
#  endif
#  if defined(ERROR_DS_NAME_VALUE_TOO_LONG)
    " " ":ERROR_DS_NAME_VALUE_TOO_LONG"
#  endif
#  if defined(ERROR_DS_NAME_UNPARSEABLE)
    " " ":ERROR_DS_NAME_UNPARSEABLE"
#  endif
#  if defined(ERROR_DS_NAME_TYPE_UNKNOWN)
    " " ":ERROR_DS_NAME_TYPE_UNKNOWN"
#  endif
#  if defined(ERROR_DS_NOT_AN_OBJECT)
    " " ":ERROR_DS_NOT_AN_OBJECT"
#  endif
#  if defined(ERROR_DS_SEC_DESC_TOO_SHORT)
    " " ":ERROR_DS_SEC_DESC_TOO_SHORT"
#  endif
#  if defined(ERROR_DS_SEC_DESC_INVALID)
    " " ":ERROR_DS_SEC_DESC_INVALID"
#  endif
#  if defined(ERROR_DS_NO_DELETED_NAME)
    " " ":ERROR_DS_NO_DELETED_NAME"
#  endif
#  if defined(ERROR_DS_SUBREF_MUST_HAVE_PARENT)
    " " ":ERROR_DS_SUBREF_MUST_HAVE_PARENT"
#  endif
#  if defined(ERROR_DS_NCNAME_MUST_BE_NC)
    " " ":ERROR_DS_NCNAME_MUST_BE_NC"
#  endif
#  if defined(ERROR_DS_CANT_ADD_SYSTEM_ONLY)
    " " ":ERROR_DS_CANT_ADD_SYSTEM_ONLY"
#  endif
#  if defined(ERROR_DS_CLASS_MUST_BE_CONCRETE)
    " " ":ERROR_DS_CLASS_MUST_BE_CONCRETE"
#  endif
#  if defined(ERROR_DS_INVALID_DMD)
    " " ":ERROR_DS_INVALID_DMD"
#  endif
#  if defined(ERROR_DS_OBJ_GUID_EXISTS)
    " " ":ERROR_DS_OBJ_GUID_EXISTS"
#  endif
#  if defined(ERROR_DS_NOT_ON_BACKLINK)
    " " ":ERROR_DS_NOT_ON_BACKLINK"
#  endif
#  if defined(ERROR_DS_NO_CROSSREF_FOR_NC)
    " " ":ERROR_DS_NO_CROSSREF_FOR_NC"
#  endif
#  if defined(ERROR_DS_SHUTTING_DOWN)
    " " ":ERROR_DS_SHUTTING_DOWN"
#  endif
#  if defined(ERROR_DS_UNKNOWN_OPERATION)
    " " ":ERROR_DS_UNKNOWN_OPERATION"
#  endif
#  if defined(ERROR_DS_INVALID_ROLE_OWNER)
    " " ":ERROR_DS_INVALID_ROLE_OWNER"
#  endif
#  if defined(ERROR_DS_COULDNT_CONTACT_FSMO)
    " " ":ERROR_DS_COULDNT_CONTACT_FSMO"
#  endif
#  if defined(ERROR_DS_CROSS_NC_DN_RENAME)
    " " ":ERROR_DS_CROSS_NC_DN_RENAME"
#  endif
#  if defined(ERROR_DS_CANT_MOD_SYSTEM_ONLY)
    " " ":ERROR_DS_CANT_MOD_SYSTEM_ONLY"
#  endif
#  if defined(ERROR_DS_REPLICATOR_ONLY)
    " " ":ERROR_DS_REPLICATOR_ONLY"
#  endif
#  if defined(ERROR_DS_OBJ_CLASS_NOT_DEFINED)
    " " ":ERROR_DS_OBJ_CLASS_NOT_DEFINED"
#  endif
#  if defined(ERROR_DS_OBJ_CLASS_NOT_SUBCLASS)
    " " ":ERROR_DS_OBJ_CLASS_NOT_SUBCLASS"
#  endif
#  if defined(ERROR_DS_NAME_REFERENCE_INVALID)
    " " ":ERROR_DS_NAME_REFERENCE_INVALID"
#  endif
#  if defined(ERROR_DS_CROSS_REF_EXISTS)
    " " ":ERROR_DS_CROSS_REF_EXISTS"
#  endif
#  if defined(ERROR_DS_CANT_DEL_MASTER_CROSSREF)
    " " ":ERROR_DS_CANT_DEL_MASTER_CROSSREF"
#  endif
#  if defined(ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD)
    " " ":ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD"
#  endif
#  if defined(ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX)
    " " ":ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX"
#  endif
#  if defined(ERROR_DS_DUP_RDN)
    " " ":ERROR_DS_DUP_RDN"
#  endif
#  if defined(ERROR_DS_DUP_OID)
    " " ":ERROR_DS_DUP_OID"
#  endif
#  if defined(ERROR_DS_DUP_MAPI_ID)
    " " ":ERROR_DS_DUP_MAPI_ID"
#  endif
#  if defined(ERROR_DS_DUP_SCHEMA_ID_GUID)
    " " ":ERROR_DS_DUP_SCHEMA_ID_GUID"
#  endif
#  if defined(ERROR_DS_DUP_LDAP_DISPLAY_NAME)
    " " ":ERROR_DS_DUP_LDAP_DISPLAY_NAME"
#  endif
#  if defined(ERROR_DS_SEMANTIC_ATT_TEST)
    " " ":ERROR_DS_SEMANTIC_ATT_TEST"
#  endif
#  if defined(ERROR_DS_SYNTAX_MISMATCH)
    " " ":ERROR_DS_SYNTAX_MISMATCH"
#  endif
#  if defined(ERROR_DS_EXISTS_IN_MUST_HAVE)
    " " ":ERROR_DS_EXISTS_IN_MUST_HAVE"
#  endif
#  if defined(ERROR_DS_EXISTS_IN_MAY_HAVE)
    " " ":ERROR_DS_EXISTS_IN_MAY_HAVE"
#  endif
#  if defined(ERROR_DS_NONEXISTENT_MAY_HAVE)
    " " ":ERROR_DS_NONEXISTENT_MAY_HAVE"
#  endif
#  if defined(ERROR_DS_NONEXISTENT_MUST_HAVE)
    " " ":ERROR_DS_NONEXISTENT_MUST_HAVE"
#  endif
#  if defined(ERROR_DS_AUX_CLS_TEST_FAIL)
    " " ":ERROR_DS_AUX_CLS_TEST_FAIL"
#  endif
#  if defined(ERROR_DS_NONEXISTENT_POSS_SUP)
    " " ":ERROR_DS_NONEXISTENT_POSS_SUP"
#  endif
#  if defined(ERROR_DS_SUB_CLS_TEST_FAIL)
    " " ":ERROR_DS_SUB_CLS_TEST_FAIL"
#  endif
#  if defined(ERROR_DS_BAD_RDN_ATT_ID_SYNTAX)
    " " ":ERROR_DS_BAD_RDN_ATT_ID_SYNTAX"
#  endif
#  if defined(ERROR_DS_EXISTS_IN_AUX_CLS)
    " " ":ERROR_DS_EXISTS_IN_AUX_CLS"
#  endif
#  if defined(ERROR_DS_EXISTS_IN_SUB_CLS)
    " " ":ERROR_DS_EXISTS_IN_SUB_CLS"
#  endif
#  if defined(ERROR_DS_EXISTS_IN_POSS_SUP)
    " " ":ERROR_DS_EXISTS_IN_POSS_SUP"
#  endif
#  if defined(ERROR_DS_RECALCSCHEMA_FAILED)
    " " ":ERROR_DS_RECALCSCHEMA_FAILED"
#  endif
#  if defined(ERROR_DS_TREE_DELETE_NOT_FINISHED)
    " " ":ERROR_DS_TREE_DELETE_NOT_FINISHED"
#  endif
#  if defined(ERROR_DS_CANT_DELETE)
    " " ":ERROR_DS_CANT_DELETE"
#  endif
#  if defined(ERROR_DS_ATT_SCHEMA_REQ_ID)
    " " ":ERROR_DS_ATT_SCHEMA_REQ_ID"
#  endif
#  if defined(ERROR_DS_BAD_ATT_SCHEMA_SYNTAX)
    " " ":ERROR_DS_BAD_ATT_SCHEMA_SYNTAX"
#  endif
#  if defined(ERROR_DS_CANT_CACHE_ATT)
    " " ":ERROR_DS_CANT_CACHE_ATT"
#  endif
#  if defined(ERROR_DS_CANT_CACHE_CLASS)
    " " ":ERROR_DS_CANT_CACHE_CLASS"
#  endif
#  if defined(ERROR_DS_CANT_REMOVE_ATT_CACHE)
    " " ":ERROR_DS_CANT_REMOVE_ATT_CACHE"
#  endif
#  if defined(ERROR_DS_CANT_REMOVE_CLASS_CACHE)
    " " ":ERROR_DS_CANT_REMOVE_CLASS_CACHE"
#  endif
#  if defined(ERROR_DS_CANT_RETRIEVE_DN)
    " " ":ERROR_DS_CANT_RETRIEVE_DN"
#  endif
#  if defined(ERROR_DS_MISSING_SUPREF)
    " " ":ERROR_DS_MISSING_SUPREF"
#  endif
#  if defined(ERROR_DS_CANT_RETRIEVE_INSTANCE)
    " " ":ERROR_DS_CANT_RETRIEVE_INSTANCE"
#  endif
#  if defined(ERROR_DS_CODE_INCONSISTENCY)
    " " ":ERROR_DS_CODE_INCONSISTENCY"
#  endif
#  if defined(ERROR_DS_DATABASE_ERROR)
    " " ":ERROR_DS_DATABASE_ERROR"
#  endif
#  if defined(ERROR_DS_GOVERNSID_MISSING)
    " " ":ERROR_DS_GOVERNSID_MISSING"
#  endif
#  if defined(ERROR_DS_MISSING_EXPECTED_ATT)
    " " ":ERROR_DS_MISSING_EXPECTED_ATT"
#  endif
#  if defined(ERROR_DS_NCNAME_MISSING_CR_REF)
    " " ":ERROR_DS_NCNAME_MISSING_CR_REF"
#  endif
#  if defined(ERROR_DS_SECURITY_CHECKING_ERROR)
    " " ":ERROR_DS_SECURITY_CHECKING_ERROR"
#  endif
#  if defined(ERROR_DS_SCHEMA_NOT_LOADED)
    " " ":ERROR_DS_SCHEMA_NOT_LOADED"
#  endif
#  if defined(ERROR_DS_SCHEMA_ALLOC_FAILED)
    " " ":ERROR_DS_SCHEMA_ALLOC_FAILED"
#  endif
#  if defined(ERROR_DS_ATT_SCHEMA_REQ_SYNTAX)
    " " ":ERROR_DS_ATT_SCHEMA_REQ_SYNTAX"
#  endif
#  if defined(ERROR_DS_GCVERIFY_ERROR)
    " " ":ERROR_DS_GCVERIFY_ERROR"
#  endif
#  if defined(ERROR_DS_DRA_SCHEMA_MISMATCH)
    " " ":ERROR_DS_DRA_SCHEMA_MISMATCH"
#  endif
#  if defined(ERROR_DS_CANT_FIND_DSA_OBJ)
    " " ":ERROR_DS_CANT_FIND_DSA_OBJ"
#  endif
#  if defined(ERROR_DS_CANT_FIND_EXPECTED_NC)
    " " ":ERROR_DS_CANT_FIND_EXPECTED_NC"
#  endif
#  if defined(ERROR_DS_CANT_FIND_NC_IN_CACHE)
    " " ":ERROR_DS_CANT_FIND_NC_IN_CACHE"
#  endif
#  if defined(ERROR_DS_CANT_RETRIEVE_CHILD)
    " " ":ERROR_DS_CANT_RETRIEVE_CHILD"
#  endif
#  if defined(ERROR_DS_SECURITY_ILLEGAL_MODIFY)
    " " ":ERROR_DS_SECURITY_ILLEGAL_MODIFY"
#  endif
#  if defined(ERROR_DS_CANT_REPLACE_HIDDEN_REC)
    " " ":ERROR_DS_CANT_REPLACE_HIDDEN_REC"
#  endif
#  if defined(ERROR_DS_BAD_HIERARCHY_FILE)
    " " ":ERROR_DS_BAD_HIERARCHY_FILE"
#  endif
#  if defined(ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED)
    " " ":ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED"
#  endif
#  if defined(ERROR_DS_CONFIG_PARAM_MISSING)
    " " ":ERROR_DS_CONFIG_PARAM_MISSING"
#  endif
#  if defined(ERROR_DS_COUNTING_AB_INDICES_FAILED)
    " " ":ERROR_DS_COUNTING_AB_INDICES_FAILED"
#  endif
#  if defined(ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED)
    " " ":ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED"
#  endif
#  if defined(ERROR_DS_INTERNAL_FAILURE)
    " " ":ERROR_DS_INTERNAL_FAILURE"
#  endif
#  if defined(ERROR_DS_UNKNOWN_ERROR)
    " " ":ERROR_DS_UNKNOWN_ERROR"
#  endif
#  if defined(ERROR_DS_ROOT_REQUIRES_CLASS_TOP)
    " " ":ERROR_DS_ROOT_REQUIRES_CLASS_TOP"
#  endif
#  if defined(ERROR_DS_REFUSING_FSMO_ROLES)
    " " ":ERROR_DS_REFUSING_FSMO_ROLES"
#  endif
#  if defined(ERROR_DS_MISSING_FSMO_SETTINGS)
    " " ":ERROR_DS_MISSING_FSMO_SETTINGS"
#  endif
#  if defined(ERROR_DS_UNABLE_TO_SURRENDER_ROLES)
    " " ":ERROR_DS_UNABLE_TO_SURRENDER_ROLES"
#  endif
#  if defined(ERROR_DS_DRA_GENERIC)
    " " ":ERROR_DS_DRA_GENERIC"
#  endif
#  if defined(ERROR_DS_DRA_INVALID_PARAMETER)
    " " ":ERROR_DS_DRA_INVALID_PARAMETER"
#  endif
#  if defined(ERROR_DS_DRA_BUSY)
    " " ":ERROR_DS_DRA_BUSY"
#  endif
#  if defined(ERROR_DS_DRA_BAD_DN)
    " " ":ERROR_DS_DRA_BAD_DN"
#  endif
#  if defined(ERROR_DS_DRA_BAD_NC)
    " " ":ERROR_DS_DRA_BAD_NC"
#  endif
#  if defined(ERROR_DS_DRA_DN_EXISTS)
    " " ":ERROR_DS_DRA_DN_EXISTS"
#  endif
#  if defined(ERROR_DS_DRA_INTERNAL_ERROR)
    " " ":ERROR_DS_DRA_INTERNAL_ERROR"
#  endif
#  if defined(ERROR_DS_DRA_INCONSISTENT_DIT)
    " " ":ERROR_DS_DRA_INCONSISTENT_DIT"
#  endif
#  if defined(ERROR_DS_DRA_CONNECTION_FAILED)
    " " ":ERROR_DS_DRA_CONNECTION_FAILED"
#  endif
#  if defined(ERROR_DS_DRA_BAD_INSTANCE_TYPE)
    " " ":ERROR_DS_DRA_BAD_INSTANCE_TYPE"
#  endif
#  if defined(ERROR_DS_DRA_OUT_OF_MEM)
    " " ":ERROR_DS_DRA_OUT_OF_MEM"
#  endif
#  if defined(ERROR_DS_DRA_MAIL_PROBLEM)
    " " ":ERROR_DS_DRA_MAIL_PROBLEM"
#  endif
#  if defined(ERROR_DS_DRA_REF_ALREADY_EXISTS)
    " " ":ERROR_DS_DRA_REF_ALREADY_EXISTS"
#  endif
#  if defined(ERROR_DS_DRA_REF_NOT_FOUND)
    " " ":ERROR_DS_DRA_REF_NOT_FOUND"
#  endif
#  if defined(ERROR_DS_DRA_OBJ_IS_REP_SOURCE)
    " " ":ERROR_DS_DRA_OBJ_IS_REP_SOURCE"
#  endif
#  if defined(ERROR_DS_DRA_DB_ERROR)
    " " ":ERROR_DS_DRA_DB_ERROR"
#  endif
#  if defined(ERROR_DS_DRA_NO_REPLICA)
    " " ":ERROR_DS_DRA_NO_REPLICA"
#  endif
#  if defined(ERROR_DS_DRA_ACCESS_DENIED)
    " " ":ERROR_DS_DRA_ACCESS_DENIED"
#  endif
#  if defined(ERROR_DS_DRA_NOT_SUPPORTED)
    " " ":ERROR_DS_DRA_NOT_SUPPORTED"
#  endif
#  if defined(ERROR_DS_DRA_RPC_CANCELLED)
    " " ":ERROR_DS_DRA_RPC_CANCELLED"
#  endif
#  if defined(ERROR_DS_DRA_SOURCE_DISABLED)
    " " ":ERROR_DS_DRA_SOURCE_DISABLED"
#  endif
#  if defined(ERROR_DS_DRA_SINK_DISABLED)
    " " ":ERROR_DS_DRA_SINK_DISABLED"
#  endif
#  if defined(ERROR_DS_DRA_NAME_COLLISION)
    " " ":ERROR_DS_DRA_NAME_COLLISION"
#  endif
#  if defined(ERROR_DS_DRA_SOURCE_REINSTALLED)
    " " ":ERROR_DS_DRA_SOURCE_REINSTALLED"
#  endif
#  if defined(ERROR_DS_DRA_MISSING_PARENT)
    " " ":ERROR_DS_DRA_MISSING_PARENT"
#  endif
#  if defined(ERROR_DS_DRA_PREEMPTED)
    " " ":ERROR_DS_DRA_PREEMPTED"
#  endif
#  if defined(ERROR_DS_DRA_ABANDON_SYNC)
    " " ":ERROR_DS_DRA_ABANDON_SYNC"
#  endif
#  if defined(ERROR_DS_DRA_SHUTDOWN)
    " " ":ERROR_DS_DRA_SHUTDOWN"
#  endif
#  if defined(ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET)
    " " ":ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET"
#  endif
#  if defined(ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA)
    " " ":ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA"
#  endif
#  if defined(ERROR_DS_DRA_EXTN_CONNECTION_FAILED)
    " " ":ERROR_DS_DRA_EXTN_CONNECTION_FAILED"
#  endif
#  if defined(ERROR_DS_INSTALL_SCHEMA_MISMATCH)
    " " ":ERROR_DS_INSTALL_SCHEMA_MISMATCH"
#  endif
#  if defined(ERROR_DS_DUP_LINK_ID)
    " " ":ERROR_DS_DUP_LINK_ID"
#  endif
#  if defined(ERROR_DS_NAME_ERROR_RESOLVING)
    " " ":ERROR_DS_NAME_ERROR_RESOLVING"
#  endif
#  if defined(ERROR_DS_NAME_ERROR_NOT_FOUND)
    " " ":ERROR_DS_NAME_ERROR_NOT_FOUND"
#  endif
#  if defined(ERROR_DS_NAME_ERROR_NOT_UNIQUE)
    " " ":ERROR_DS_NAME_ERROR_NOT_UNIQUE"
#  endif
#  if defined(ERROR_DS_NAME_ERROR_NO_MAPPING)
    " " ":ERROR_DS_NAME_ERROR_NO_MAPPING"
#  endif
#  if defined(ERROR_DS_NAME_ERROR_DOMAIN_ONLY)
    " " ":ERROR_DS_NAME_ERROR_DOMAIN_ONLY"
#  endif
#  if defined(ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING)
    " " ":ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING"
#  endif
#  if defined(ERROR_DS_CONSTRUCTED_ATT_MOD)
    " " ":ERROR_DS_CONSTRUCTED_ATT_MOD"
#  endif
#  if defined(ERROR_DS_WRONG_OM_OBJ_CLASS)
    " " ":ERROR_DS_WRONG_OM_OBJ_CLASS"
#  endif
#  if defined(ERROR_DS_DRA_REPL_PENDING)
    " " ":ERROR_DS_DRA_REPL_PENDING"
#  endif
#  if defined(ERROR_DS_DS_REQUIRED)
    " " ":ERROR_DS_DS_REQUIRED"
#  endif
#  if defined(ERROR_DS_INVALID_LDAP_DISPLAY_NAME)
    " " ":ERROR_DS_INVALID_LDAP_DISPLAY_NAME"
#  endif
#  if defined(ERROR_DS_NON_BASE_SEARCH)
    " " ":ERROR_DS_NON_BASE_SEARCH"
#  endif
#  if defined(ERROR_DS_CANT_RETRIEVE_ATTS)
    " " ":ERROR_DS_CANT_RETRIEVE_ATTS"
#  endif
#  if defined(ERROR_DS_BACKLINK_WITHOUT_LINK)
    " " ":ERROR_DS_BACKLINK_WITHOUT_LINK"
#  endif
#  if defined(ERROR_DS_EPOCH_MISMATCH)
    " " ":ERROR_DS_EPOCH_MISMATCH"
#  endif
#  if defined(ERROR_DS_SRC_NAME_MISMATCH)
    " " ":ERROR_DS_SRC_NAME_MISMATCH"
#  endif
#  if defined(ERROR_DS_SRC_AND_DST_NC_IDENTICAL)
    " " ":ERROR_DS_SRC_AND_DST_NC_IDENTICAL"
#  endif
#  if defined(ERROR_DS_DST_NC_MISMATCH)
    " " ":ERROR_DS_DST_NC_MISMATCH"
#  endif
#  if defined(ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC)
    " " ":ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC"
#  endif
#  if defined(ERROR_DS_SRC_GUID_MISMATCH)
    " " ":ERROR_DS_SRC_GUID_MISMATCH"
#  endif
#  if defined(ERROR_DS_CANT_MOVE_DELETED_OBJECT)
    " " ":ERROR_DS_CANT_MOVE_DELETED_OBJECT"
#  endif
#  if defined(ERROR_DS_PDC_OPERATION_IN_PROGRESS)
    " " ":ERROR_DS_PDC_OPERATION_IN_PROGRESS"
#  endif
#  if defined(ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD)
    " " ":ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD"
#  endif
#  if defined(ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION)
    " " ":ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION"
#  endif
#  if defined(ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS)
    " " ":ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS"
#  endif
#  if defined(ERROR_DS_NC_MUST_HAVE_NC_PARENT)
    " " ":ERROR_DS_NC_MUST_HAVE_NC_PARENT"
#  endif
#  if defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE)
    " " ":ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE"
#  endif
#  if defined(ERROR_DS_DST_DOMAIN_NOT_NATIVE)
    " " ":ERROR_DS_DST_DOMAIN_NOT_NATIVE"
#  endif
#  if defined(ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER)
    " " ":ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER"
#  endif
#  if defined(ERROR_DS_CANT_MOVE_ACCOUNT_GROUP)
    " " ":ERROR_DS_CANT_MOVE_ACCOUNT_GROUP"
#  endif
#  if defined(ERROR_DS_CANT_MOVE_RESOURCE_GROUP)
    " " ":ERROR_DS_CANT_MOVE_RESOURCE_GROUP"
#  endif
#  if defined(ERROR_DS_INVALID_SEARCH_FLAG)
    " " ":ERROR_DS_INVALID_SEARCH_FLAG"
#  endif
#  if defined(ERROR_DS_NO_TREE_DELETE_ABOVE_NC)
    " " ":ERROR_DS_NO_TREE_DELETE_ABOVE_NC"
#  endif
#  if defined(ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE)
    " " ":ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE"
#  endif
#  if defined(ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE)
    " " ":ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE"
#  endif
#  if defined(ERROR_DS_SAM_INIT_FAILURE)
    " " ":ERROR_DS_SAM_INIT_FAILURE"
#  endif
#  if defined(ERROR_DS_SENSITIVE_GROUP_VIOLATION)
    " " ":ERROR_DS_SENSITIVE_GROUP_VIOLATION"
#  endif
#  if defined(ERROR_DS_CANT_MOD_PRIMARYGROUPID)
    " " ":ERROR_DS_CANT_MOD_PRIMARYGROUPID"
#  endif
#  if defined(ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD)
    " " ":ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD"
#  endif
#  if defined(ERROR_DS_NONSAFE_SCHEMA_CHANGE)
    " " ":ERROR_DS_NONSAFE_SCHEMA_CHANGE"
#  endif
#  if defined(ERROR_DS_SCHEMA_UPDATE_DISALLOWED)
    " " ":ERROR_DS_SCHEMA_UPDATE_DISALLOWED"
#  endif
#  if defined(ERROR_DS_CANT_CREATE_UNDER_SCHEMA)
    " " ":ERROR_DS_CANT_CREATE_UNDER_SCHEMA"
#  endif
#  if defined(ERROR_DS_INSTALL_NO_SRC_SCH_VERSION)
    " " ":ERROR_DS_INSTALL_NO_SRC_SCH_VERSION"
#  endif
#  if defined(ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE)
    " " ":ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE"
#  endif
#  if defined(ERROR_DS_INVALID_GROUP_TYPE)
    " " ":ERROR_DS_INVALID_GROUP_TYPE"
#  endif
#  if defined(ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN)
    " " ":ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN"
#  endif
#  if defined(ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN)
    " " ":ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN"
#  endif
#  if defined(ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER)
    " " ":ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER"
#  endif
#  if defined(ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER)
    " " ":ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER"
#  endif
#  if defined(ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER)
    " " ":ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER"
#  endif
#  if defined(ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER)
    " " ":ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER"
#  endif
#  if defined(ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER)
    " " ":ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER"
#  endif
#  if defined(ERROR_DS_HAVE_PRIMARY_MEMBERS)
    " " ":ERROR_DS_HAVE_PRIMARY_MEMBERS"
#  endif
#  if defined(ERROR_DS_STRING_SD_CONVERSION_FAILED)
    " " ":ERROR_DS_STRING_SD_CONVERSION_FAILED"
#  endif
#  if defined(ERROR_DS_NAMING_MASTER_GC)
    " " ":ERROR_DS_NAMING_MASTER_GC"
#  endif
#  if defined(ERROR_DS_LOOKUP_FAILURE)
    " " ":ERROR_DS_LOOKUP_FAILURE"
#  endif
#  if defined(ERROR_DS_COULDNT_UPDATE_SPNS)
    " " ":ERROR_DS_COULDNT_UPDATE_SPNS"
#  endif
#  if defined(ERROR_DS_CANT_RETRIEVE_SD)
    " " ":ERROR_DS_CANT_RETRIEVE_SD"
#  endif
#  if defined(ERROR_DS_KEY_NOT_UNIQUE)
    " " ":ERROR_DS_KEY_NOT_UNIQUE"
#  endif
#  if defined(ERROR_DS_WRONG_LINKED_ATT_SYNTAX)
    " " ":ERROR_DS_WRONG_LINKED_ATT_SYNTAX"
#  endif
#  if defined(ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD)
    " " ":ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD"
#  endif
#  if defined(ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY)
    " " ":ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY"
#  endif
#  if defined(ERROR_DS_CANT_START)
    " " ":ERROR_DS_CANT_START"
#  endif
#  if defined(ERROR_DS_INIT_FAILURE)
    " " ":ERROR_DS_INIT_FAILURE"
#  endif
#  if defined(ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION)
    " " ":ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION"
#  endif
#  if defined(ERROR_DS_SOURCE_DOMAIN_IN_FOREST)
    " " ":ERROR_DS_SOURCE_DOMAIN_IN_FOREST"
#  endif
#  if defined(ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST)
    " " ":ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST"
#  endif
#  if defined(ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED)
    " " ":ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED"
#  endif
#  if defined(ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN)
    " " ":ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN"
#  endif
#  if defined(ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER)
    " " ":ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER"
#  endif
#  if defined(ERROR_DS_SRC_SID_EXISTS_IN_FOREST)
    " " ":ERROR_DS_SRC_SID_EXISTS_IN_FOREST"
#  endif
#  if defined(ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH)
    " " ":ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH"
#  endif
#  if defined(ERROR_SAM_INIT_FAILURE)
    " " ":ERROR_SAM_INIT_FAILURE"
#  endif
#  if defined(ERROR_DS_DRA_SCHEMA_INFO_SHIP)
    " " ":ERROR_DS_DRA_SCHEMA_INFO_SHIP"
#  endif
#  if defined(ERROR_DS_DRA_SCHEMA_CONFLICT)
    " " ":ERROR_DS_DRA_SCHEMA_CONFLICT"
#  endif
#  if defined(ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT)
    " " ":ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT"
#  endif
#  if defined(ERROR_DS_DRA_OBJ_NC_MISMATCH)
    " " ":ERROR_DS_DRA_OBJ_NC_MISMATCH"
#  endif
#  if defined(ERROR_DS_NC_STILL_HAS_DSAS)
    " " ":ERROR_DS_NC_STILL_HAS_DSAS"
#  endif
#  if defined(ERROR_DS_GC_REQUIRED)
    " " ":ERROR_DS_GC_REQUIRED"
#  endif
#  if defined(ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY)
    " " ":ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY"
#  endif
#  if defined(ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS)
    " " ":ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS"
#  endif
#  if defined(ERROR_DS_CANT_ADD_TO_GC)
    " " ":ERROR_DS_CANT_ADD_TO_GC"
#  endif
#  if defined(ERROR_DS_NO_CHECKPOINT_WITH_PDC)
    " " ":ERROR_DS_NO_CHECKPOINT_WITH_PDC"
#  endif
#  if defined(ERROR_DS_SOURCE_AUDITING_NOT_ENABLED)
    " " ":ERROR_DS_SOURCE_AUDITING_NOT_ENABLED"
#  endif
#  if defined(ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC)
    " " ":ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC"
#  endif
#  if defined(ERROR_DS_INVALID_NAME_FOR_SPN)
    " " ":ERROR_DS_INVALID_NAME_FOR_SPN"
#  endif
#  if defined(ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS)
    " " ":ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS"
#  endif
#  if defined(ERROR_DS_UNICODEPWD_NOT_IN_QUOTES)
    " " ":ERROR_DS_UNICODEPWD_NOT_IN_QUOTES"
#  endif
#  if defined(ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED)
    " " ":ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED"
#  endif
#  if defined(ERROR_DS_MUST_BE_RUN_ON_DST_DC)
    " " ":ERROR_DS_MUST_BE_RUN_ON_DST_DC"
#  endif
#  if defined(ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER)
    " " ":ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER"
#  endif
#  if defined(ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ)
    " " ":ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ"
#  endif
#  if defined(ERROR_DS_INIT_FAILURE_CONSOLE)
    " " ":ERROR_DS_INIT_FAILURE_CONSOLE"
#  endif
#  if defined(ERROR_DS_SAM_INIT_FAILURE_CONSOLE)
    " " ":ERROR_DS_SAM_INIT_FAILURE_CONSOLE"
#  endif
#  if defined(ERROR_DS_FOREST_VERSION_TOO_HIGH)
    " " ":ERROR_DS_FOREST_VERSION_TOO_HIGH"
#  endif
#  if defined(ERROR_DS_DOMAIN_VERSION_TOO_HIGH)
    " " ":ERROR_DS_DOMAIN_VERSION_TOO_HIGH"
#  endif
#  if defined(ERROR_DS_FOREST_VERSION_TOO_LOW)
    " " ":ERROR_DS_FOREST_VERSION_TOO_LOW"
#  endif
#  if defined(ERROR_DS_DOMAIN_VERSION_TOO_LOW)
    " " ":ERROR_DS_DOMAIN_VERSION_TOO_LOW"
#  endif
#  if defined(ERROR_DS_INCOMPATIBLE_VERSION)
    " " ":ERROR_DS_INCOMPATIBLE_VERSION"
#  endif
#  if defined(ERROR_DS_LOW_DSA_VERSION)
    " " ":ERROR_DS_LOW_DSA_VERSION"
#  endif
#  if defined(ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN)
    " " ":ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN"
#  endif
#  if defined(ERROR_DS_NOT_SUPPORTED_SORT_ORDER)
    " " ":ERROR_DS_NOT_SUPPORTED_SORT_ORDER"
#  endif
#  if defined(ERROR_DS_NAME_NOT_UNIQUE)
    " " ":ERROR_DS_NAME_NOT_UNIQUE"
#  endif
#  if defined(ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4)
    " " ":ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4"
#  endif
#  if defined(ERROR_DS_OUT_OF_VERSION_STORE)
    " " ":ERROR_DS_OUT_OF_VERSION_STORE"
#  endif
#  if defined(ERROR_DS_INCOMPATIBLE_CONTROLS_USED)
    " " ":ERROR_DS_INCOMPATIBLE_CONTROLS_USED"
#  endif
#  if defined(ERROR_DS_NO_REF_DOMAIN)
    " " ":ERROR_DS_NO_REF_DOMAIN"
#  endif
#  if defined(ERROR_DS_RESERVED_LINK_ID)
    " " ":ERROR_DS_RESERVED_LINK_ID"
#  endif
#  if defined(ERROR_DS_LINK_ID_NOT_AVAILABLE)
    " " ":ERROR_DS_LINK_ID_NOT_AVAILABLE"
#  endif
#  if defined(ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER)
    " " ":ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER"
#  endif
#  if defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE)
    " " ":ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE"
#  endif
#  if defined(ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC)
    " " ":ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC"
#  endif
#  if defined(ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG)
    " " ":ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG"
#  endif
#  if defined(ERROR_DS_MODIFYDN_WRONG_GRANDPARENT)
    " " ":ERROR_DS_MODIFYDN_WRONG_GRANDPARENT"
#  endif
#  if defined(ERROR_DS_NAME_ERROR_TRUST_REFERRAL)
    " " ":ERROR_DS_NAME_ERROR_TRUST_REFERRAL"
#  endif
#  if defined(ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER)
    " " ":ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER"
#  endif
#  if defined(ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD)
    " " ":ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD"
#  endif
#  if defined(ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2)
    " " ":ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2"
#  endif
#  if defined(ERROR_DS_THREAD_LIMIT_EXCEEDED)
    " " ":ERROR_DS_THREAD_LIMIT_EXCEEDED"
#  endif
#  if defined(ERROR_DS_NOT_CLOSEST)
    " " ":ERROR_DS_NOT_CLOSEST"
#  endif
#  if defined(ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF)
    " " ":ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF"
#  endif
#  if defined(ERROR_DS_SINGLE_USER_MODE_FAILED)
    " " ":ERROR_DS_SINGLE_USER_MODE_FAILED"
#  endif
#  if defined(ERROR_DS_NTDSCRIPT_SYNTAX_ERROR)
    " " ":ERROR_DS_NTDSCRIPT_SYNTAX_ERROR"
#  endif
#  if defined(ERROR_DS_NTDSCRIPT_PROCESS_ERROR)
    " " ":ERROR_DS_NTDSCRIPT_PROCESS_ERROR"
#  endif
#  if defined(ERROR_DS_DIFFERENT_REPL_EPOCHS)
    " " ":ERROR_DS_DIFFERENT_REPL_EPOCHS"
#  endif
#  if defined(ERROR_DS_DRS_EXTENSIONS_CHANGED)
    " " ":ERROR_DS_DRS_EXTENSIONS_CHANGED"
#  endif
#  if defined(ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR)
    " " ":ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR"
#  endif
#  if defined(ERROR_DS_NO_MSDS_INTID)
    " " ":ERROR_DS_NO_MSDS_INTID"
#  endif
#  if defined(ERROR_DS_DUP_MSDS_INTID)
    " " ":ERROR_DS_DUP_MSDS_INTID"
#  endif
#  if defined(ERROR_DS_EXISTS_IN_RDNATTID)
    " " ":ERROR_DS_EXISTS_IN_RDNATTID"
#  endif
#  if defined(ERROR_DS_AUTHORIZATION_FAILED)
    " " ":ERROR_DS_AUTHORIZATION_FAILED"
#  endif
#  if defined(ERROR_DS_INVALID_SCRIPT)
    " " ":ERROR_DS_INVALID_SCRIPT"
#  endif
#  if defined(ERROR_DS_REMOTE_CROSSREF_OP_FAILED)
    " " ":ERROR_DS_REMOTE_CROSSREF_OP_FAILED"
#  endif
#  if defined(ERROR_DS_CROSS_REF_BUSY)
    " " ":ERROR_DS_CROSS_REF_BUSY"
#  endif
#  if defined(ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN)
    " " ":ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN"
#  endif
#  if defined(ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC)
    " " ":ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC"
#  endif
#  if defined(ERROR_DS_DUPLICATE_ID_FOUND)
    " " ":ERROR_DS_DUPLICATE_ID_FOUND"
#  endif
#  if defined(ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT)
    " " ":ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT"
#  endif
#  if defined(ERROR_DS_GROUP_CONVERSION_ERROR)
    " " ":ERROR_DS_GROUP_CONVERSION_ERROR"
#  endif
#  if defined(ERROR_DS_CANT_MOVE_APP_BASIC_GROUP)
    " " ":ERROR_DS_CANT_MOVE_APP_BASIC_GROUP"
#  endif
#  if defined(ERROR_DS_CANT_MOVE_APP_QUERY_GROUP)
    " " ":ERROR_DS_CANT_MOVE_APP_QUERY_GROUP"
#  endif
#  if defined(ERROR_DS_ROLE_NOT_VERIFIED)
    " " ":ERROR_DS_ROLE_NOT_VERIFIED"
#  endif
#  if defined(ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL)
    " " ":ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL"
#  endif
#  if defined(ERROR_DS_DOMAIN_RENAME_IN_PROGRESS)
    " " ":ERROR_DS_DOMAIN_RENAME_IN_PROGRESS"
#  endif
#  if defined(ERROR_DS_EXISTING_AD_CHILD_NC)
    " " ":ERROR_DS_EXISTING_AD_CHILD_NC"
#  endif
#  if defined(DNS_ERROR_RCODE_FORMAT_ERROR)
    " " ":DNS_ERROR_RCODE_FORMAT_ERROR"
#  endif
#  if defined(DNS_ERROR_RCODE_SERVER_FAILURE)
    " " ":DNS_ERROR_RCODE_SERVER_FAILURE"
#  endif
#  if defined(DNS_ERROR_RCODE_NAME_ERROR)
    " " ":DNS_ERROR_RCODE_NAME_ERROR"
#  endif
#  if defined(DNS_ERROR_RCODE_NOT_IMPLEMENTED)
    " " ":DNS_ERROR_RCODE_NOT_IMPLEMENTED"
#  endif
#  if defined(DNS_ERROR_RCODE_REFUSED)
    " " ":DNS_ERROR_RCODE_REFUSED"
#  endif
#  if defined(DNS_ERROR_RCODE_YXDOMAIN)
    " " ":DNS_ERROR_RCODE_YXDOMAIN"
#  endif
#  if defined(DNS_ERROR_RCODE_YXRRSET)
    " " ":DNS_ERROR_RCODE_YXRRSET"
#  endif
#  if defined(DNS_ERROR_RCODE_NXRRSET)
    " " ":DNS_ERROR_RCODE_NXRRSET"
#  endif
#  if defined(DNS_ERROR_RCODE_NOTAUTH)
    " " ":DNS_ERROR_RCODE_NOTAUTH"
#  endif
#  if defined(DNS_ERROR_RCODE_NOTZONE)
    " " ":DNS_ERROR_RCODE_NOTZONE"
#  endif
#  if defined(DNS_ERROR_RCODE_BADSIG)
    " " ":DNS_ERROR_RCODE_BADSIG"
#  endif
#  if defined(DNS_ERROR_RCODE_BADKEY)
    " " ":DNS_ERROR_RCODE_BADKEY"
#  endif
#  if defined(DNS_ERROR_RCODE_BADTIME)
    " " ":DNS_ERROR_RCODE_BADTIME"
#  endif
#  if defined(DNS_INFO_NO_RECORDS)
    " " ":DNS_INFO_NO_RECORDS"
#  endif
#  if defined(DNS_ERROR_BAD_PACKET)
    " " ":DNS_ERROR_BAD_PACKET"
#  endif
#  if defined(DNS_ERROR_NO_PACKET)
    " " ":DNS_ERROR_NO_PACKET"
#  endif
#  if defined(DNS_ERROR_RCODE)
    " " ":DNS_ERROR_RCODE"
#  endif
#  if defined(DNS_ERROR_UNSECURE_PACKET)
    " " ":DNS_ERROR_UNSECURE_PACKET"
#  endif
#  if defined(DNS_ERROR_INVALID_TYPE)
    " " ":DNS_ERROR_INVALID_TYPE"
#  endif
#  if defined(DNS_ERROR_INVALID_IP_ADDRESS)
    " " ":DNS_ERROR_INVALID_IP_ADDRESS"
#  endif
#  if defined(DNS_ERROR_INVALID_PROPERTY)
    " " ":DNS_ERROR_INVALID_PROPERTY"
#  endif
#  if defined(DNS_ERROR_TRY_AGAIN_LATER)
    " " ":DNS_ERROR_TRY_AGAIN_LATER"
#  endif
#  if defined(DNS_ERROR_NOT_UNIQUE)
    " " ":DNS_ERROR_NOT_UNIQUE"
#  endif
#  if defined(DNS_ERROR_NON_RFC_NAME)
    " " ":DNS_ERROR_NON_RFC_NAME"
#  endif
#  if defined(DNS_STATUS_FQDN)
    " " ":DNS_STATUS_FQDN"
#  endif
#  if defined(DNS_STATUS_DOTTED_NAME)
    " " ":DNS_STATUS_DOTTED_NAME"
#  endif
#  if defined(DNS_STATUS_SINGLE_PART_NAME)
    " " ":DNS_STATUS_SINGLE_PART_NAME"
#  endif
#  if defined(DNS_ERROR_INVALID_NAME_CHAR)
    " " ":DNS_ERROR_INVALID_NAME_CHAR"
#  endif
#  if defined(DNS_ERROR_NUMERIC_NAME)
    " " ":DNS_ERROR_NUMERIC_NAME"
#  endif
#  if defined(DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER)
    " " ":DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER"
#  endif
#  if defined(DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION)
    " " ":DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION"
#  endif
#  if defined(DNS_ERROR_CANNOT_FIND_ROOT_HINTS)
    " " ":DNS_ERROR_CANNOT_FIND_ROOT_HINTS"
#  endif
#  if defined(DNS_ERROR_INCONSISTENT_ROOT_HINTS)
    " " ":DNS_ERROR_INCONSISTENT_ROOT_HINTS"
#  endif
#  if defined(DNS_ERROR_ZONE_DOES_NOT_EXIST)
    " " ":DNS_ERROR_ZONE_DOES_NOT_EXIST"
#  endif
#  if defined(DNS_ERROR_NO_ZONE_INFO)
    " " ":DNS_ERROR_NO_ZONE_INFO"
#  endif
#  if defined(DNS_ERROR_INVALID_ZONE_OPERATION)
    " " ":DNS_ERROR_INVALID_ZONE_OPERATION"
#  endif
#  if defined(DNS_ERROR_ZONE_CONFIGURATION_ERROR)
    " " ":DNS_ERROR_ZONE_CONFIGURATION_ERROR"
#  endif
#  if defined(DNS_ERROR_ZONE_HAS_NO_SOA_RECORD)
    " " ":DNS_ERROR_ZONE_HAS_NO_SOA_RECORD"
#  endif
#  if defined(DNS_ERROR_ZONE_HAS_NO_NS_RECORDS)
    " " ":DNS_ERROR_ZONE_HAS_NO_NS_RECORDS"
#  endif
#  if defined(DNS_ERROR_ZONE_LOCKED)
    " " ":DNS_ERROR_ZONE_LOCKED"
#  endif
#  if defined(DNS_ERROR_ZONE_CREATION_FAILED)
    " " ":DNS_ERROR_ZONE_CREATION_FAILED"
#  endif
#  if defined(DNS_ERROR_ZONE_ALREADY_EXISTS)
    " " ":DNS_ERROR_ZONE_ALREADY_EXISTS"
#  endif
#  if defined(DNS_ERROR_AUTOZONE_ALREADY_EXISTS)
    " " ":DNS_ERROR_AUTOZONE_ALREADY_EXISTS"
#  endif
#  if defined(DNS_ERROR_INVALID_ZONE_TYPE)
    " " ":DNS_ERROR_INVALID_ZONE_TYPE"
#  endif
#  if defined(DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP)
    " " ":DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP"
#  endif
#  if defined(DNS_ERROR_ZONE_NOT_SECONDARY)
    " " ":DNS_ERROR_ZONE_NOT_SECONDARY"
#  endif
#  if defined(DNS_ERROR_NEED_SECONDARY_ADDRESSES)
    " " ":DNS_ERROR_NEED_SECONDARY_ADDRESSES"
#  endif
#  if defined(DNS_ERROR_WINS_INIT_FAILED)
    " " ":DNS_ERROR_WINS_INIT_FAILED"
#  endif
#  if defined(DNS_ERROR_NEED_WINS_SERVERS)
    " " ":DNS_ERROR_NEED_WINS_SERVERS"
#  endif
#  if defined(DNS_ERROR_NBSTAT_INIT_FAILED)
    " " ":DNS_ERROR_NBSTAT_INIT_FAILED"
#  endif
#  if defined(DNS_ERROR_SOA_DELETE_INVALID)
    " " ":DNS_ERROR_SOA_DELETE_INVALID"
#  endif
#  if defined(DNS_ERROR_FORWARDER_ALREADY_EXISTS)
    " " ":DNS_ERROR_FORWARDER_ALREADY_EXISTS"
#  endif
#  if defined(DNS_ERROR_ZONE_REQUIRES_MASTER_IP)
    " " ":DNS_ERROR_ZONE_REQUIRES_MASTER_IP"
#  endif
#  if defined(DNS_ERROR_ZONE_IS_SHUTDOWN)
    " " ":DNS_ERROR_ZONE_IS_SHUTDOWN"
#  endif
#  if defined(DNS_ERROR_PRIMARY_REQUIRES_DATAFILE)
    " " ":DNS_ERROR_PRIMARY_REQUIRES_DATAFILE"
#  endif
#  if defined(DNS_ERROR_INVALID_DATAFILE_NAME)
    " " ":DNS_ERROR_INVALID_DATAFILE_NAME"
#  endif
#  if defined(DNS_ERROR_DATAFILE_OPEN_FAILURE)
    " " ":DNS_ERROR_DATAFILE_OPEN_FAILURE"
#  endif
#  if defined(DNS_ERROR_FILE_WRITEBACK_FAILED)
    " " ":DNS_ERROR_FILE_WRITEBACK_FAILED"
#  endif
#  if defined(DNS_ERROR_DATAFILE_PARSING)
    " " ":DNS_ERROR_DATAFILE_PARSING"
#  endif
#  if defined(DNS_ERROR_RECORD_DOES_NOT_EXIST)
    " " ":DNS_ERROR_RECORD_DOES_NOT_EXIST"
#  endif
#  if defined(DNS_ERROR_RECORD_FORMAT)
    " " ":DNS_ERROR_RECORD_FORMAT"
#  endif
#  if defined(DNS_ERROR_NODE_CREATION_FAILED)
    " " ":DNS_ERROR_NODE_CREATION_FAILED"
#  endif
#  if defined(DNS_ERROR_UNKNOWN_RECORD_TYPE)
    " " ":DNS_ERROR_UNKNOWN_RECORD_TYPE"
#  endif
#  if defined(DNS_ERROR_RECORD_TIMED_OUT)
    " " ":DNS_ERROR_RECORD_TIMED_OUT"
#  endif
#  if defined(DNS_ERROR_NAME_NOT_IN_ZONE)
    " " ":DNS_ERROR_NAME_NOT_IN_ZONE"
#  endif
#  if defined(DNS_ERROR_CNAME_LOOP)
    " " ":DNS_ERROR_CNAME_LOOP"
#  endif
#  if defined(DNS_ERROR_NODE_IS_CNAME)
    " " ":DNS_ERROR_NODE_IS_CNAME"
#  endif
#  if defined(DNS_ERROR_CNAME_COLLISION)
    " " ":DNS_ERROR_CNAME_COLLISION"
#  endif
#  if defined(DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT)
    " " ":DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT"
#  endif
#  if defined(DNS_ERROR_RECORD_ALREADY_EXISTS)
    " " ":DNS_ERROR_RECORD_ALREADY_EXISTS"
#  endif
#  if defined(DNS_ERROR_SECONDARY_DATA)
    " " ":DNS_ERROR_SECONDARY_DATA"
#  endif
#  if defined(DNS_ERROR_NO_CREATE_CACHE_DATA)
    " " ":DNS_ERROR_NO_CREATE_CACHE_DATA"
#  endif
#  if defined(DNS_ERROR_NAME_DOES_NOT_EXIST)
    " " ":DNS_ERROR_NAME_DOES_NOT_EXIST"
#  endif
#  if defined(DNS_WARNING_PTR_CREATE_FAILED)
    " " ":DNS_WARNING_PTR_CREATE_FAILED"
#  endif
#  if defined(DNS_WARNING_DOMAIN_UNDELETED)
    " " ":DNS_WARNING_DOMAIN_UNDELETED"
#  endif
#  if defined(DNS_ERROR_DS_UNAVAILABLE)
    " " ":DNS_ERROR_DS_UNAVAILABLE"
#  endif
#  if defined(DNS_ERROR_DS_ZONE_ALREADY_EXISTS)
    " " ":DNS_ERROR_DS_ZONE_ALREADY_EXISTS"
#  endif
#  if defined(DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE)
    " " ":DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE"
#  endif
#  if defined(DNS_INFO_AXFR_COMPLETE)
    " " ":DNS_INFO_AXFR_COMPLETE"
#  endif
#  if defined(DNS_ERROR_AXFR)
    " " ":DNS_ERROR_AXFR"
#  endif
#  if defined(DNS_INFO_ADDED_LOCAL_WINS)
    " " ":DNS_INFO_ADDED_LOCAL_WINS"
#  endif
#  if defined(DNS_STATUS_CONTINUE_NEEDED)
    " " ":DNS_STATUS_CONTINUE_NEEDED"
#  endif
#  if defined(DNS_ERROR_NO_TCPIP)
    " " ":DNS_ERROR_NO_TCPIP"
#  endif
#  if defined(DNS_ERROR_NO_DNS_SERVERS)
    " " ":DNS_ERROR_NO_DNS_SERVERS"
#  endif
#  if defined(DNS_ERROR_DP_DOES_NOT_EXIST)
    " " ":DNS_ERROR_DP_DOES_NOT_EXIST"
#  endif
#  if defined(DNS_ERROR_DP_ALREADY_EXISTS)
    " " ":DNS_ERROR_DP_ALREADY_EXISTS"
#  endif
#  if defined(DNS_ERROR_DP_NOT_ENLISTED)
    " " ":DNS_ERROR_DP_NOT_ENLISTED"
#  endif
#  if defined(DNS_ERROR_DP_ALREADY_ENLISTED)
    " " ":DNS_ERROR_DP_ALREADY_ENLISTED"
#  endif
#  if defined(DNS_ERROR_DP_NOT_AVAILABLE)
    " " ":DNS_ERROR_DP_NOT_AVAILABLE"
#  endif
#  if defined(WSABASEERR)
    " " ":WSABASEERR"
#  endif
#  if defined(WSAEINTR)
    " " ":WSAEINTR"
#  endif
#  if defined(WSAEBADF)
    " " ":WSAEBADF"
#  endif
#  if defined(WSAEACCES)
    " " ":WSAEACCES"
#  endif
#  if defined(WSAEFAULT)
    " " ":WSAEFAULT"
#  endif
#  if defined(WSAEINVAL)
    " " ":WSAEINVAL"
#  endif
#  if defined(WSAEMFILE)
    " " ":WSAEMFILE"
#  endif
#  if defined(WSAEWOULDBLOCK)
    " " ":WSAEWOULDBLOCK"
#  endif
#  if defined(WSAEINPROGRESS)
    " " ":WSAEINPROGRESS"
#  endif
#  if defined(WSAEALREADY)
    " " ":WSAEALREADY"
#  endif
#  if defined(WSAENOTSOCK)
    " " ":WSAENOTSOCK"
#  endif
#  if defined(WSAEDESTADDRREQ)
    " " ":WSAEDESTADDRREQ"
#  endif
#  if defined(WSAEMSGSIZE)
    " " ":WSAEMSGSIZE"
#  endif
#  if defined(WSAEPROTOTYPE)
    " " ":WSAEPROTOTYPE"
#  endif
#  if defined(WSAENOPROTOOPT)
    " " ":WSAENOPROTOOPT"
#  endif
#  if defined(WSAEPROTONOSUPPORT)
    " " ":WSAEPROTONOSUPPORT"
#  endif
#  if defined(WSAESOCKTNOSUPPORT)
    " " ":WSAESOCKTNOSUPPORT"
#  endif
#  if defined(WSAEOPNOTSUPP)
    " " ":WSAEOPNOTSUPP"
#  endif
#  if defined(WSAEPFNOSUPPORT)
    " " ":WSAEPFNOSUPPORT"
#  endif
#  if defined(WSAEAFNOSUPPORT)
    " " ":WSAEAFNOSUPPORT"
#  endif
#  if defined(WSAEADDRINUSE)
    " " ":WSAEADDRINUSE"
#  endif
#  if defined(WSAEADDRNOTAVAIL)
    " " ":WSAEADDRNOTAVAIL"
#  endif
#  if defined(WSAENETDOWN)
    " " ":WSAENETDOWN"
#  endif
#  if defined(WSAENETUNREACH)
    " " ":WSAENETUNREACH"
#  endif
#  if defined(WSAENETRESET)
    " " ":WSAENETRESET"
#  endif
#  if defined(WSAECONNABORTED)
    " " ":WSAECONNABORTED"
#  endif
#  if defined(WSAECONNRESET)
    " " ":WSAECONNRESET"
#  endif
#  if defined(WSAENOBUFS)
    " " ":WSAENOBUFS"
#  endif
#  if defined(WSAEISCONN)
    " " ":WSAEISCONN"
#  endif
#  if defined(WSAENOTCONN)
    " " ":WSAENOTCONN"
#  endif
#  if defined(WSAESHUTDOWN)
    " " ":WSAESHUTDOWN"
#  endif
#  if defined(WSAETOOMANYREFS)
    " " ":WSAETOOMANYREFS"
#  endif
#  if defined(WSAETIMEDOUT)
    " " ":WSAETIMEDOUT"
#  endif
#  if defined(WSAECONNREFUSED)
    " " ":WSAECONNREFUSED"
#  endif
#  if defined(WSAELOOP)
    " " ":WSAELOOP"
#  endif
#  if defined(WSAENAMETOOLONG)
    " " ":WSAENAMETOOLONG"
#  endif
#  if defined(WSAEHOSTDOWN)
    " " ":WSAEHOSTDOWN"
#  endif
#  if defined(WSAEHOSTUNREACH)
    " " ":WSAEHOSTUNREACH"
#  endif
#  if defined(WSAENOTEMPTY)
    " " ":WSAENOTEMPTY"
#  endif
#  if defined(WSAEPROCLIM)
    " " ":WSAEPROCLIM"
#  endif
#  if defined(WSAEUSERS)
    " " ":WSAEUSERS"
#  endif
#  if defined(WSAEDQUOT)
    " " ":WSAEDQUOT"
#  endif
#  if defined(WSAESTALE)
    " " ":WSAESTALE"
#  endif
#  if defined(WSAEREMOTE)
    " " ":WSAEREMOTE"
#  endif
#  if defined(WSASYSNOTREADY)
    " " ":WSASYSNOTREADY"
#  endif
#  if defined(WSAVERNOTSUPPORTED)
    " " ":WSAVERNOTSUPPORTED"
#  endif
#  if defined(WSANOTINITIALISED)
    " " ":WSANOTINITIALISED"
#  endif
#  if defined(WSAEDISCON)
    " " ":WSAEDISCON"
#  endif
#  if defined(WSAENOMORE)
    " " ":WSAENOMORE"
#  endif
#  if defined(WSAECANCELLED)
    " " ":WSAECANCELLED"
#  endif
#  if defined(WSAEINVALIDPROCTABLE)
    " " ":WSAEINVALIDPROCTABLE"
#  endif
#  if defined(WSAEINVALIDPROVIDER)
    " " ":WSAEINVALIDPROVIDER"
#  endif
#  if defined(WSAEPROVIDERFAILEDINIT)
    " " ":WSAEPROVIDERFAILEDINIT"
#  endif
#  if defined(WSASYSCALLFAILURE)
    " " ":WSASYSCALLFAILURE"
#  endif
#  if defined(WSASERVICE_NOT_FOUND)
    " " ":WSASERVICE_NOT_FOUND"
#  endif
#  if defined(WSATYPE_NOT_FOUND)
    " " ":WSATYPE_NOT_FOUND"
#  endif
#  if defined(WSA_E_NO_MORE)
    " " ":WSA_E_NO_MORE"
#  endif
#  if defined(WSA_E_CANCELLED)
    " " ":WSA_E_CANCELLED"
#  endif
#  if defined(WSAEREFUSED)
    " " ":WSAEREFUSED"
#  endif
#  if defined(WSAHOST_NOT_FOUND)
    " " ":WSAHOST_NOT_FOUND"
#  endif
#  if defined(WSATRY_AGAIN)
    " " ":WSATRY_AGAIN"
#  endif
#  if defined(WSANO_RECOVERY)
    " " ":WSANO_RECOVERY"
#  endif
#  if defined(WSANO_DATA)
    " " ":WSANO_DATA"
#  endif
#  if defined(WSA_QOS_RECEIVERS)
    " " ":WSA_QOS_RECEIVERS"
#  endif
#  if defined(WSA_QOS_SENDERS)
    " " ":WSA_QOS_SENDERS"
#  endif
#  if defined(WSA_QOS_NO_SENDERS)
    " " ":WSA_QOS_NO_SENDERS"
#  endif
#  if defined(WSA_QOS_NO_RECEIVERS)
    " " ":WSA_QOS_NO_RECEIVERS"
#  endif
#  if defined(WSA_QOS_REQUEST_CONFIRMED)
    " " ":WSA_QOS_REQUEST_CONFIRMED"
#  endif
#  if defined(WSA_QOS_ADMISSION_FAILURE)
    " " ":WSA_QOS_ADMISSION_FAILURE"
#  endif
#  if defined(WSA_QOS_POLICY_FAILURE)
    " " ":WSA_QOS_POLICY_FAILURE"
#  endif
#  if defined(WSA_QOS_BAD_STYLE)
    " " ":WSA_QOS_BAD_STYLE"
#  endif
#  if defined(WSA_QOS_BAD_OBJECT)
    " " ":WSA_QOS_BAD_OBJECT"
#  endif
#  if defined(WSA_QOS_TRAFFIC_CTRL_ERROR)
    " " ":WSA_QOS_TRAFFIC_CTRL_ERROR"
#  endif
#  if defined(WSA_QOS_GENERIC_ERROR)
    " " ":WSA_QOS_GENERIC_ERROR"
#  endif
#  if defined(WSA_QOS_ESERVICETYPE)
    " " ":WSA_QOS_ESERVICETYPE"
#  endif
#  if defined(WSA_QOS_EFLOWSPEC)
    " " ":WSA_QOS_EFLOWSPEC"
#  endif
#  if defined(WSA_QOS_EPROVSPECBUF)
    " " ":WSA_QOS_EPROVSPECBUF"
#  endif
#  if defined(WSA_QOS_EFILTERSTYLE)
    " " ":WSA_QOS_EFILTERSTYLE"
#  endif
#  if defined(WSA_QOS_EFILTERTYPE)
    " " ":WSA_QOS_EFILTERTYPE"
#  endif
#  if defined(WSA_QOS_EFILTERCOUNT)
    " " ":WSA_QOS_EFILTERCOUNT"
#  endif
#  if defined(WSA_QOS_EOBJLENGTH)
    " " ":WSA_QOS_EOBJLENGTH"
#  endif
#  if defined(WSA_QOS_EFLOWCOUNT)
    " " ":WSA_QOS_EFLOWCOUNT"
#  endif
#  if defined(WSA_QOS_EUNKNOWNPSOBJ)
    " " ":WSA_QOS_EUNKNOWNPSOBJ"
#  endif
#  if defined(WSA_QOS_EPOLICYOBJ)
    " " ":WSA_QOS_EPOLICYOBJ"
#  endif
#  if defined(WSA_QOS_EFLOWDESC)
    " " ":WSA_QOS_EFLOWDESC"
#  endif
#  if defined(WSA_QOS_EPSFLOWSPEC)
    " " ":WSA_QOS_EPSFLOWSPEC"
#  endif
#  if defined(WSA_QOS_EPSFILTERSPEC)
    " " ":WSA_QOS_EPSFILTERSPEC"
#  endif
#  if defined(WSA_QOS_ESDMODEOBJ)
    " " ":WSA_QOS_ESDMODEOBJ"
#  endif
#  if defined(WSA_QOS_ESHAPERATEOBJ)
    " " ":WSA_QOS_ESHAPERATEOBJ"
#  endif
#  if defined(WSA_QOS_RESERVED_PETYPE)
    " " ":WSA_QOS_RESERVED_PETYPE"
#  endif
#  if defined(ERROR_IPSEC_QM_POLICY_EXISTS)
    " " ":ERROR_IPSEC_QM_POLICY_EXISTS"
#  endif
#  if defined(ERROR_IPSEC_QM_POLICY_NOT_FOUND)
    " " ":ERROR_IPSEC_QM_POLICY_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_QM_POLICY_IN_USE)
    " " ":ERROR_IPSEC_QM_POLICY_IN_USE"
#  endif
#  if defined(ERROR_IPSEC_MM_POLICY_EXISTS)
    " " ":ERROR_IPSEC_MM_POLICY_EXISTS"
#  endif
#  if defined(ERROR_IPSEC_MM_POLICY_NOT_FOUND)
    " " ":ERROR_IPSEC_MM_POLICY_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_MM_POLICY_IN_USE)
    " " ":ERROR_IPSEC_MM_POLICY_IN_USE"
#  endif
#  if defined(ERROR_IPSEC_MM_FILTER_EXISTS)
    " " ":ERROR_IPSEC_MM_FILTER_EXISTS"
#  endif
#  if defined(ERROR_IPSEC_MM_FILTER_NOT_FOUND)
    " " ":ERROR_IPSEC_MM_FILTER_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_TRANSPORT_FILTER_EXISTS)
    " " ":ERROR_IPSEC_TRANSPORT_FILTER_EXISTS"
#  endif
#  if defined(ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND)
    " " ":ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_MM_AUTH_EXISTS)
    " " ":ERROR_IPSEC_MM_AUTH_EXISTS"
#  endif
#  if defined(ERROR_IPSEC_MM_AUTH_NOT_FOUND)
    " " ":ERROR_IPSEC_MM_AUTH_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_MM_AUTH_IN_USE)
    " " ":ERROR_IPSEC_MM_AUTH_IN_USE"
#  endif
#  if defined(ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND)
    " " ":ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND)
    " " ":ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND)
    " " ":ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_TUNNEL_FILTER_EXISTS)
    " " ":ERROR_IPSEC_TUNNEL_FILTER_EXISTS"
#  endif
#  if defined(ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND)
    " " ":ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND"
#  endif
#  if defined(ERROR_IPSEC_MM_FILTER_PENDING_DELETION)
    " " ":ERROR_IPSEC_MM_FILTER_PENDING_DELETION"
#  endif
#  if defined(ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION)
    " " ":ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION"
#  endif
#  if defined(ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION)
    " " ":ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION"
#  endif
#  if defined(ERROR_IPSEC_MM_POLICY_PENDING_DELETION)
    " " ":ERROR_IPSEC_MM_POLICY_PENDING_DELETION"
#  endif
#  if defined(ERROR_IPSEC_MM_AUTH_PENDING_DELETION)
    " " ":ERROR_IPSEC_MM_AUTH_PENDING_DELETION"
#  endif
#  if defined(ERROR_IPSEC_QM_POLICY_PENDING_DELETION)
    " " ":ERROR_IPSEC_QM_POLICY_PENDING_DELETION"
#  endif
#  if defined(WARNING_IPSEC_MM_POLICY_PRUNED)
    " " ":WARNING_IPSEC_MM_POLICY_PRUNED"
#  endif
#  if defined(WARNING_IPSEC_QM_POLICY_PRUNED)
    " " ":WARNING_IPSEC_QM_POLICY_PRUNED"
#  endif
#  if defined(ERROR_IPSEC_IKE_AUTH_FAIL)
    " " ":ERROR_IPSEC_IKE_AUTH_FAIL"
#  endif
#  if defined(ERROR_IPSEC_IKE_ATTRIB_FAIL)
    " " ":ERROR_IPSEC_IKE_ATTRIB_FAIL"
#  endif
#  if defined(ERROR_IPSEC_IKE_NEGOTIATION_PENDING)
    " " ":ERROR_IPSEC_IKE_NEGOTIATION_PENDING"
#  endif
#  if defined(ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR)
    " " ":ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR"
#  endif
#  if defined(ERROR_IPSEC_IKE_TIMED_OUT)
    " " ":ERROR_IPSEC_IKE_TIMED_OUT"
#  endif
#  if defined(ERROR_IPSEC_IKE_NO_CERT)
    " " ":ERROR_IPSEC_IKE_NO_CERT"
#  endif
#  if defined(ERROR_IPSEC_IKE_SA_DELETED)
    " " ":ERROR_IPSEC_IKE_SA_DELETED"
#  endif
#  if defined(ERROR_IPSEC_IKE_SA_REAPED)
    " " ":ERROR_IPSEC_IKE_SA_REAPED"
#  endif
#  if defined(ERROR_IPSEC_IKE_MM_ACQUIRE_DROP)
    " " ":ERROR_IPSEC_IKE_MM_ACQUIRE_DROP"
#  endif
#  if defined(ERROR_IPSEC_IKE_QM_ACQUIRE_DROP)
    " " ":ERROR_IPSEC_IKE_QM_ACQUIRE_DROP"
#  endif
#  if defined(ERROR_IPSEC_IKE_QUEUE_DROP_MM)
    " " ":ERROR_IPSEC_IKE_QUEUE_DROP_MM"
#  endif
#  if defined(ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM)
    " " ":ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM"
#  endif
#  if defined(ERROR_IPSEC_IKE_DROP_NO_RESPONSE)
    " " ":ERROR_IPSEC_IKE_DROP_NO_RESPONSE"
#  endif
#  if defined(ERROR_IPSEC_IKE_MM_DELAY_DROP)
    " " ":ERROR_IPSEC_IKE_MM_DELAY_DROP"
#  endif
#  if defined(ERROR_IPSEC_IKE_QM_DELAY_DROP)
    " " ":ERROR_IPSEC_IKE_QM_DELAY_DROP"
#  endif
#  if defined(ERROR_IPSEC_IKE_ERROR)
    " " ":ERROR_IPSEC_IKE_ERROR"
#  endif
#  if defined(ERROR_IPSEC_IKE_CRL_FAILED)
    " " ":ERROR_IPSEC_IKE_CRL_FAILED"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_KEY_USAGE)
    " " ":ERROR_IPSEC_IKE_INVALID_KEY_USAGE"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_CERT_TYPE)
    " " ":ERROR_IPSEC_IKE_INVALID_CERT_TYPE"
#  endif
#  if defined(ERROR_IPSEC_IKE_NO_PRIVATE_KEY)
    " " ":ERROR_IPSEC_IKE_NO_PRIVATE_KEY"
#  endif
#  if defined(ERROR_IPSEC_IKE_DH_FAIL)
    " " ":ERROR_IPSEC_IKE_DH_FAIL"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_HEADER)
    " " ":ERROR_IPSEC_IKE_INVALID_HEADER"
#  endif
#  if defined(ERROR_IPSEC_IKE_NO_POLICY)
    " " ":ERROR_IPSEC_IKE_NO_POLICY"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_SIGNATURE)
    " " ":ERROR_IPSEC_IKE_INVALID_SIGNATURE"
#  endif
#  if defined(ERROR_IPSEC_IKE_KERBEROS_ERROR)
    " " ":ERROR_IPSEC_IKE_KERBEROS_ERROR"
#  endif
#  if defined(ERROR_IPSEC_IKE_NO_PUBLIC_KEY)
    " " ":ERROR_IPSEC_IKE_NO_PUBLIC_KEY"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_SA)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_SA"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_PROP)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_PROP"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_TRANS)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_TRANS"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_KE)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_KE"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_ID)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_ID"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_CERT"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_HASH)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_HASH"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_SIG)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_SIG"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_NONCE)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_NONCE"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_DELETE)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_DELETE"
#  endif
#  if defined(ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR)
    " " ":ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_PAYLOAD)
    " " ":ERROR_IPSEC_IKE_INVALID_PAYLOAD"
#  endif
#  if defined(ERROR_IPSEC_IKE_LOAD_SOFT_SA)
    " " ":ERROR_IPSEC_IKE_LOAD_SOFT_SA"
#  endif
#  if defined(ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN)
    " " ":ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_COOKIE)
    " " ":ERROR_IPSEC_IKE_INVALID_COOKIE"
#  endif
#  if defined(ERROR_IPSEC_IKE_NO_PEER_CERT)
    " " ":ERROR_IPSEC_IKE_NO_PEER_CERT"
#  endif
#  if defined(ERROR_IPSEC_IKE_PEER_CRL_FAILED)
    " " ":ERROR_IPSEC_IKE_PEER_CRL_FAILED"
#  endif
#  if defined(ERROR_IPSEC_IKE_POLICY_CHANGE)
    " " ":ERROR_IPSEC_IKE_POLICY_CHANGE"
#  endif
#  if defined(ERROR_IPSEC_IKE_NO_MM_POLICY)
    " " ":ERROR_IPSEC_IKE_NO_MM_POLICY"
#  endif
#  if defined(ERROR_IPSEC_IKE_NOTCBPRIV)
    " " ":ERROR_IPSEC_IKE_NOTCBPRIV"
#  endif
#  if defined(ERROR_IPSEC_IKE_SECLOADFAIL)
    " " ":ERROR_IPSEC_IKE_SECLOADFAIL"
#  endif
#  if defined(ERROR_IPSEC_IKE_FAILSSPINIT)
    " " ":ERROR_IPSEC_IKE_FAILSSPINIT"
#  endif
#  if defined(ERROR_IPSEC_IKE_FAILQUERYSSP)
    " " ":ERROR_IPSEC_IKE_FAILQUERYSSP"
#  endif
#  if defined(ERROR_IPSEC_IKE_SRVACQFAIL)
    " " ":ERROR_IPSEC_IKE_SRVACQFAIL"
#  endif
#  if defined(ERROR_IPSEC_IKE_SRVQUERYCRED)
    " " ":ERROR_IPSEC_IKE_SRVQUERYCRED"
#  endif
#  if defined(ERROR_IPSEC_IKE_GETSPIFAIL)
    " " ":ERROR_IPSEC_IKE_GETSPIFAIL"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_FILTER)
    " " ":ERROR_IPSEC_IKE_INVALID_FILTER"
#  endif
#  if defined(ERROR_IPSEC_IKE_OUT_OF_MEMORY)
    " " ":ERROR_IPSEC_IKE_OUT_OF_MEMORY"
#  endif
#  if defined(ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED)
    " " ":ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_POLICY)
    " " ":ERROR_IPSEC_IKE_INVALID_POLICY"
#  endif
#  if defined(ERROR_IPSEC_IKE_UNKNOWN_DOI)
    " " ":ERROR_IPSEC_IKE_UNKNOWN_DOI"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_SITUATION)
    " " ":ERROR_IPSEC_IKE_INVALID_SITUATION"
#  endif
#  if defined(ERROR_IPSEC_IKE_DH_FAILURE)
    " " ":ERROR_IPSEC_IKE_DH_FAILURE"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_GROUP)
    " " ":ERROR_IPSEC_IKE_INVALID_GROUP"
#  endif
#  if defined(ERROR_IPSEC_IKE_ENCRYPT)
    " " ":ERROR_IPSEC_IKE_ENCRYPT"
#  endif
#  if defined(ERROR_IPSEC_IKE_DECRYPT)
    " " ":ERROR_IPSEC_IKE_DECRYPT"
#  endif
#  if defined(ERROR_IPSEC_IKE_POLICY_MATCH)
    " " ":ERROR_IPSEC_IKE_POLICY_MATCH"
#  endif
#  if defined(ERROR_IPSEC_IKE_UNSUPPORTED_ID)
    " " ":ERROR_IPSEC_IKE_UNSUPPORTED_ID"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_HASH)
    " " ":ERROR_IPSEC_IKE_INVALID_HASH"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_HASH_ALG)
    " " ":ERROR_IPSEC_IKE_INVALID_HASH_ALG"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_HASH_SIZE)
    " " ":ERROR_IPSEC_IKE_INVALID_HASH_SIZE"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG)
    " " ":ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_AUTH_ALG)
    " " ":ERROR_IPSEC_IKE_INVALID_AUTH_ALG"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_SIG)
    " " ":ERROR_IPSEC_IKE_INVALID_SIG"
#  endif
#  if defined(ERROR_IPSEC_IKE_LOAD_FAILED)
    " " ":ERROR_IPSEC_IKE_LOAD_FAILED"
#  endif
#  if defined(ERROR_IPSEC_IKE_RPC_DELETE)
    " " ":ERROR_IPSEC_IKE_RPC_DELETE"
#  endif
#  if defined(ERROR_IPSEC_IKE_BENIGN_REINIT)
    " " ":ERROR_IPSEC_IKE_BENIGN_REINIT"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY)
    " " ":ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY"
#  endif
#  if defined(ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN)
    " " ":ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN"
#  endif
#  if defined(ERROR_IPSEC_IKE_MM_LIMIT)
    " " ":ERROR_IPSEC_IKE_MM_LIMIT"
#  endif
#  if defined(ERROR_IPSEC_IKE_NEGOTIATION_DISABLED)
    " " ":ERROR_IPSEC_IKE_NEGOTIATION_DISABLED"
#  endif
#  if defined(ERROR_IPSEC_IKE_NEG_STATUS_END)
    " " ":ERROR_IPSEC_IKE_NEG_STATUS_END"
#  endif
#  if defined(ERROR_SXS_SECTION_NOT_FOUND)
    " " ":ERROR_SXS_SECTION_NOT_FOUND"
#  endif
#  if defined(ERROR_SXS_CANT_GEN_ACTCTX)
    " " ":ERROR_SXS_CANT_GEN_ACTCTX"
#  endif
#  if defined(ERROR_SXS_INVALID_ACTCTXDATA_FORMAT)
    " " ":ERROR_SXS_INVALID_ACTCTXDATA_FORMAT"
#  endif
#  if defined(ERROR_SXS_ASSEMBLY_NOT_FOUND)
    " " ":ERROR_SXS_ASSEMBLY_NOT_FOUND"
#  endif
#  if defined(ERROR_SXS_MANIFEST_FORMAT_ERROR)
    " " ":ERROR_SXS_MANIFEST_FORMAT_ERROR"
#  endif
#  if defined(ERROR_SXS_MANIFEST_PARSE_ERROR)
    " " ":ERROR_SXS_MANIFEST_PARSE_ERROR"
#  endif
#  if defined(ERROR_SXS_ACTIVATION_CONTEXT_DISABLED)
    " " ":ERROR_SXS_ACTIVATION_CONTEXT_DISABLED"
#  endif
#  if defined(ERROR_SXS_KEY_NOT_FOUND)
    " " ":ERROR_SXS_KEY_NOT_FOUND"
#  endif
#  if defined(ERROR_SXS_VERSION_CONFLICT)
    " " ":ERROR_SXS_VERSION_CONFLICT"
#  endif
#  if defined(ERROR_SXS_WRONG_SECTION_TYPE)
    " " ":ERROR_SXS_WRONG_SECTION_TYPE"
#  endif
#  if defined(ERROR_SXS_THREAD_QUERIES_DISABLED)
    " " ":ERROR_SXS_THREAD_QUERIES_DISABLED"
#  endif
#  if defined(ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET)
    " " ":ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET"
#  endif
#  if defined(ERROR_SXS_UNKNOWN_ENCODING_GROUP)
    " " ":ERROR_SXS_UNKNOWN_ENCODING_GROUP"
#  endif
#  if defined(ERROR_SXS_UNKNOWN_ENCODING)
    " " ":ERROR_SXS_UNKNOWN_ENCODING"
#  endif
#  if defined(ERROR_SXS_INVALID_XML_NAMESPACE_URI)
    " " ":ERROR_SXS_INVALID_XML_NAMESPACE_URI"
#  endif
#  if defined(ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED)
    " " ":ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED"
#  endif
#  if defined(ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED)
    " " ":ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED"
#  endif
#  if defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE)
    " " ":ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE"
#  endif
#  if defined(ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE)
    " " ":ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE"
#  endif
#  if defined(ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE)
    " " ":ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE"
#  endif
#  if defined(ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT)
    " " ":ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT"
#  endif
#  if defined(ERROR_SXS_DUPLICATE_DLL_NAME)
    " " ":ERROR_SXS_DUPLICATE_DLL_NAME"
#  endif
#  if defined(ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME)
    " " ":ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME"
#  endif
#  if defined(ERROR_SXS_DUPLICATE_CLSID)
    " " ":ERROR_SXS_DUPLICATE_CLSID"
#  endif
#  if defined(ERROR_SXS_DUPLICATE_IID)
    " " ":ERROR_SXS_DUPLICATE_IID"
#  endif
#  if defined(ERROR_SXS_DUPLICATE_TLBID)
    " " ":ERROR_SXS_DUPLICATE_TLBID"
#  endif
#  if defined(ERROR_SXS_DUPLICATE_PROGID)
    " " ":ERROR_SXS_DUPLICATE_PROGID"
#  endif
#  if defined(ERROR_SXS_DUPLICATE_ASSEMBLY_NAME)
    " " ":ERROR_SXS_DUPLICATE_ASSEMBLY_NAME"
#  endif
#  if defined(ERROR_SXS_FILE_HASH_MISMATCH)
    " " ":ERROR_SXS_FILE_HASH_MISMATCH"
#  endif
#  if defined(ERROR_SXS_POLICY_PARSE_ERROR)
    " " ":ERROR_SXS_POLICY_PARSE_ERROR"
#  endif
#  if defined(ERROR_SXS_XML_E_MISSINGQUOTE)
    " " ":ERROR_SXS_XML_E_MISSINGQUOTE"
#  endif
#  if defined(ERROR_SXS_XML_E_COMMENTSYNTAX)
    " " ":ERROR_SXS_XML_E_COMMENTSYNTAX"
#  endif
#  if defined(ERROR_SXS_XML_E_BADSTARTNAMECHAR)
    " " ":ERROR_SXS_XML_E_BADSTARTNAMECHAR"
#  endif
#  if defined(ERROR_SXS_XML_E_BADNAMECHAR)
    " " ":ERROR_SXS_XML_E_BADNAMECHAR"
#  endif
#  if defined(ERROR_SXS_XML_E_BADCHARINSTRING)
    " " ":ERROR_SXS_XML_E_BADCHARINSTRING"
#  endif
#  if defined(ERROR_SXS_XML_E_XMLDECLSYNTAX)
    " " ":ERROR_SXS_XML_E_XMLDECLSYNTAX"
#  endif
#  if defined(ERROR_SXS_XML_E_BADCHARDATA)
    " " ":ERROR_SXS_XML_E_BADCHARDATA"
#  endif
#  if defined(ERROR_SXS_XML_E_MISSINGWHITESPACE)
    " " ":ERROR_SXS_XML_E_MISSINGWHITESPACE"
#  endif
#  if defined(ERROR_SXS_XML_E_EXPECTINGTAGEND)
    " " ":ERROR_SXS_XML_E_EXPECTINGTAGEND"
#  endif
#  if defined(ERROR_SXS_XML_E_MISSINGSEMICOLON)
    " " ":ERROR_SXS_XML_E_MISSINGSEMICOLON"
#  endif
#  if defined(ERROR_SXS_XML_E_UNBALANCEDPAREN)
    " " ":ERROR_SXS_XML_E_UNBALANCEDPAREN"
#  endif
#  if defined(ERROR_SXS_XML_E_INTERNALERROR)
    " " ":ERROR_SXS_XML_E_INTERNALERROR"
#  endif
#  if defined(ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE)
    " " ":ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE"
#  endif
#  if defined(ERROR_SXS_XML_E_INCOMPLETE_ENCODING)
    " " ":ERROR_SXS_XML_E_INCOMPLETE_ENCODING"
#  endif
#  if defined(ERROR_SXS_XML_E_MISSING_PAREN)
    " " ":ERROR_SXS_XML_E_MISSING_PAREN"
#  endif
#  if defined(ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE)
    " " ":ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE"
#  endif
#  if defined(ERROR_SXS_XML_E_MULTIPLE_COLONS)
    " " ":ERROR_SXS_XML_E_MULTIPLE_COLONS"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALID_DECIMAL)
    " " ":ERROR_SXS_XML_E_INVALID_DECIMAL"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALID_HEXIDECIMAL)
    " " ":ERROR_SXS_XML_E_INVALID_HEXIDECIMAL"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALID_UNICODE)
    " " ":ERROR_SXS_XML_E_INVALID_UNICODE"
#  endif
#  if defined(ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK)
    " " ":ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK"
#  endif
#  if defined(ERROR_SXS_XML_E_UNEXPECTEDENDTAG)
    " " ":ERROR_SXS_XML_E_UNEXPECTEDENDTAG"
#  endif
#  if defined(ERROR_SXS_XML_E_UNCLOSEDTAG)
    " " ":ERROR_SXS_XML_E_UNCLOSEDTAG"
#  endif
#  if defined(ERROR_SXS_XML_E_DUPLICATEATTRIBUTE)
    " " ":ERROR_SXS_XML_E_DUPLICATEATTRIBUTE"
#  endif
#  if defined(ERROR_SXS_XML_E_MULTIPLEROOTS)
    " " ":ERROR_SXS_XML_E_MULTIPLEROOTS"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALIDATROOTLEVEL)
    " " ":ERROR_SXS_XML_E_INVALIDATROOTLEVEL"
#  endif
#  if defined(ERROR_SXS_XML_E_BADXMLDECL)
    " " ":ERROR_SXS_XML_E_BADXMLDECL"
#  endif
#  if defined(ERROR_SXS_XML_E_MISSINGROOT)
    " " ":ERROR_SXS_XML_E_MISSINGROOT"
#  endif
#  if defined(ERROR_SXS_XML_E_UNEXPECTEDEOF)
    " " ":ERROR_SXS_XML_E_UNEXPECTEDEOF"
#  endif
#  if defined(ERROR_SXS_XML_E_BADPEREFINSUBSET)
    " " ":ERROR_SXS_XML_E_BADPEREFINSUBSET"
#  endif
#  if defined(ERROR_SXS_XML_E_UNCLOSEDSTARTTAG)
    " " ":ERROR_SXS_XML_E_UNCLOSEDSTARTTAG"
#  endif
#  if defined(ERROR_SXS_XML_E_UNCLOSEDENDTAG)
    " " ":ERROR_SXS_XML_E_UNCLOSEDENDTAG"
#  endif
#  if defined(ERROR_SXS_XML_E_UNCLOSEDSTRING)
    " " ":ERROR_SXS_XML_E_UNCLOSEDSTRING"
#  endif
#  if defined(ERROR_SXS_XML_E_UNCLOSEDCOMMENT)
    " " ":ERROR_SXS_XML_E_UNCLOSEDCOMMENT"
#  endif
#  if defined(ERROR_SXS_XML_E_UNCLOSEDDECL)
    " " ":ERROR_SXS_XML_E_UNCLOSEDDECL"
#  endif
#  if defined(ERROR_SXS_XML_E_UNCLOSEDCDATA)
    " " ":ERROR_SXS_XML_E_UNCLOSEDCDATA"
#  endif
#  if defined(ERROR_SXS_XML_E_RESERVEDNAMESPACE)
    " " ":ERROR_SXS_XML_E_RESERVEDNAMESPACE"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALIDENCODING)
    " " ":ERROR_SXS_XML_E_INVALIDENCODING"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALIDSWITCH)
    " " ":ERROR_SXS_XML_E_INVALIDSWITCH"
#  endif
#  if defined(ERROR_SXS_XML_E_BADXMLCASE)
    " " ":ERROR_SXS_XML_E_BADXMLCASE"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALID_STANDALONE)
    " " ":ERROR_SXS_XML_E_INVALID_STANDALONE"
#  endif
#  if defined(ERROR_SXS_XML_E_UNEXPECTED_STANDALONE)
    " " ":ERROR_SXS_XML_E_UNEXPECTED_STANDALONE"
#  endif
#  if defined(ERROR_SXS_XML_E_INVALID_VERSION)
    " " ":ERROR_SXS_XML_E_INVALID_VERSION"
#  endif
#  if defined(ERROR_SXS_XML_E_MISSINGEQUALS)
    " " ":ERROR_SXS_XML_E_MISSINGEQUALS"
#  endif
#  if defined(ERROR_SXS_PROTECTION_RECOVERY_FAILED)
    " " ":ERROR_SXS_PROTECTION_RECOVERY_FAILED"
#  endif
#  if defined(ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT)
    " " ":ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT"
#  endif
#  if defined(ERROR_SXS_PROTECTION_CATALOG_NOT_VALID)
    " " ":ERROR_SXS_PROTECTION_CATALOG_NOT_VALID"
#  endif
#  if defined(ERROR_SXS_UNTRANSLATABLE_HRESULT)
    " " ":ERROR_SXS_UNTRANSLATABLE_HRESULT"
#  endif
#  if defined(ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING)
    " " ":ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING"
#  endif
#  if defined(ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE)
    " " ":ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE"
#  endif
#  if defined(ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME)
    " " ":ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_FCNTL)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(FD_CLOEXEC)
    " " ":CLOEXEC"
#  endif
    " " "))"
  },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(FILE_ATTRIBUTE_ARCHIVE)
    " " ":ARCHIVE"
#  endif
#  if defined(FILE_ATTRIBUTE_COMPRESSED)
    " " ":COMPRESSED"
#  endif
#  if defined(FILE_ATTRIBUTE_DEVICE)
    " " ":DEVICE"
#  endif
#  if defined(FILE_ATTRIBUTE_DIRECTORY)
    " " ":DIRECTORY"
#  endif
#  if defined(FILE_ATTRIBUTE_ENCRYPTED)
    " " ":ENCRYPTED"
#  endif
#  if defined(FILE_ATTRIBUTE_HIDDEN)
    " " ":HIDDEN"
#  endif
#  if defined(FILE_ATTRIBUTE_NORMAL)
    " " ":NORMAL"
#  endif
#  if defined(FILE_ATTRIBUTE_NOT_CONTENT_INDEXED)
    " " ":NOT-CONTENT-INDEXED"
#  endif
#  if defined(FILE_ATTRIBUTE_OFFLINE)
    " " ":OFFLINE"
#  endif
#  if defined(FILE_ATTRIBUTE_READONLY)
    " " ":READONLY"
#  endif
#  if defined(FILE_ATTRIBUTE_REPARSE_POINT)
    " " ":REPARSE-POINT"
#  endif
#  if defined(FILE_ATTRIBUTE_SPARSE_FILE)
    " " ":SPARSE-FILE"
#  endif
#  if defined(FILE_ATTRIBUTE_SYSTEM)
    " " ":SYSTEM"
#  endif
#  if defined(FILE_ATTRIBUTE_TEMPORARY)
    " " ":TEMPORARY"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_NFTW)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(FTW_F)
    " " ":F"
#  endif
#  if defined(FTW_D)
    " " ":D"
#  endif
#  if defined(FTW_DP)
    " " ":DP"
#  endif
#  if defined(FTW_SL)
    " " ":SL"
#  endif
#  if defined(FTW_SLN)
    " " ":SLN"
#  endif
#  if defined(FTW_DNR)
    " " ":DNR"
#  endif
#  if defined(FTW_NS)
    " " ":NS"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_FCNTL)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(F_GETFD)
    " " ":FD"
#  endif
#  if defined(F_GETFL)
    " " ":FL"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_SYSLOG)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(LOG_EMERG)
    " " ":EMERG"
#  endif
#  if defined(LOG_ALERT)
    " " ":ALERT"
#  endif
#  if defined(LOG_CRIT)
    " " ":CRIT"
#  endif
#  if defined(LOG_ERR)
    " " ":ERR"
#  endif
#  if defined(LOG_WARNING)
    " " ":WARNING"
#  endif
#  if defined(LOG_NOTICE)
    " " ":NOTICE"
#  endif
#  if defined(LOG_INFO)
    " " ":INFO"
#  endif
#  if defined(LOG_DEBUG)
    " " ":DEBUG"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_FCNTL)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(O_RDONLY)
    " " ":RDONLY"
#  endif
#  if defined(O_WRONLY)
    " " ":WRONLY"
#  endif
#  if defined(O_RDWR)
    " " ":RDWR"
#  endif
#  if defined(O_APPEND)
    " " ":APPEND"
#  endif
#  if defined(O_CREAT)
    " " ":CREAT"
#  endif
#  if defined(O_TRUNC)
    " " ":TRUNC"
#  endif
#  if defined(O_EXCL)
    " " ":EXCL"
#  endif
#  if defined(O_NOCTTY)
    " " ":NOCTTY"
#  endif
#  if defined(O_SYNC)
    " " ":SYNC"
#  endif
#  if defined(O_NONBLOCK)
    " " ":NONBLOCK"
#  endif
#  if defined(O_BINARY)
    " " ":BINARY"
#  endif
#  if defined(O_TEXT)
    " " ":TEXT"
#  endif
#  if defined(O_NOINHERIT)
    " " ":NOINHERIT"
#  endif
#  if defined(O_DIRECT)
    " " ":DIRECT"
#  endif
#  if defined(O_LARGEFILE)
    " " ":LARGEFILE"
#  endif
#  if defined(O_DIRECTORY)
    " " ":DIRECTORY"
#  endif
#  if defined(O_NOFOLLOW)
    " " ":NOFOLLOW"
#  endif
    " " "))"
  },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(PROCESSOR_ARCHITECTURE_INTEL)
    " " ":INTEL"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_MIPS)
    " " ":MIPS"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_ALPHA)
    " " ":ALPHA"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_PPC)
    " " ":PPC"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_SHX)
    " " ":SHX"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_ARM)
    " " ":ARM"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_IA64)
    " " ":IA64"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_ALPHA64)
    " " ":ALPHA64"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_MSIL)
    " " ":MSIL"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_AMD64)
    " " ":AMD64"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_IA32_ON_WIN64)
    " " ":IA32_ON_WIN64"
#  endif
#  if defined(PROCESSOR_ARCHITECTURE_UNKNOWN)
    " " ":UNKNOWN"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(RLIMIT_CPU)
    " " ":CPU"
#  endif
#  if defined(RLIMIT_FSIZE)
    " " ":FSIZE"
#  endif
#  if defined(RLIMIT_DATA)
    " " ":DATA"
#  endif
#  if defined(RLIMIT_STACK)
    " " ":STACK"
#  endif
#  if defined(RLIMIT_CORE)
    " " ":CORE"
#  endif
#  if defined(RLIMIT_RSS)
    " " ":RSS"
#  endif
#  if defined(RLIMIT_NOFILE)
    " " ":NOFILE"
#  endif
#  if defined(RLIMIT_AS)
    " " ":AS"
#  endif
#  if defined(RLIMIT_NPROC)
    " " ":NPROC"
#  endif
#  if defined(RLIMIT_MEMLOCK)
    " " ":MEMLOCK"
#  endif
#  if defined(RLIMIT_LOCKS)
    " " ":LOCKS"
#  endif
    " " "))"
  },
#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(RUSAGE_SELF)
    " " ":SELF"
#  endif
#  if defined(RUSAGE_CHILDREN)
    " " ":CHILDREN"
#  endif
#  if defined(RUSAGE_THREAD)
    " " ":THREAD"
#  endif
#  if defined(RUSAGE_LWP)
    " " ":LWP"
#  endif
    " " "))"
  },
#endif
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(SIGABRT)
    " " ":SIGABRT"
#  endif
#  if defined(SIGALRM)
    " " ":SIGALRM"
#  endif
#  if defined(SIGBUS)
    " " ":SIGBUS"
#  endif
#  if defined(SIGCHLD)
    " " ":SIGCHLD"
#  endif
#  if defined(SIGCONT)
    " " ":SIGCONT"
#  endif
#  if defined(SIGFPE)
    " " ":SIGFPE"
#  endif
#  if defined(SIGHUP)
    " " ":SIGHUP"
#  endif
#  if defined(SIGILL)
    " " ":SIGILL"
#  endif
#  if defined(SIGINT)
    " " ":SIGINT"
#  endif
#  if defined(SIGKILL)
    " " ":SIGKILL"
#  endif
#  if defined(SIGPIPE)
    " " ":SIGPIPE"
#  endif
#  if defined(SIGQUIT)
    " " ":SIGQUIT"
#  endif
#  if defined(SIGSEGV)
    " " ":SIGSEGV"
#  endif
#  if defined(SIGSTOP)
    " " ":SIGSTOP"
#  endif
#  if defined(SIGTERM)
    " " ":SIGTERM"
#  endif
#  if defined(SIGTSTP)
    " " ":SIGTSTP"
#  endif
#  if defined(SIGTTIN)
    " " ":SIGTTIN"
#  endif
#  if defined(SIGTTOU)
    " " ":SIGTTOU"
#  endif
#  if defined(SIGUSR1)
    " " ":SIGUSR1"
#  endif
#  if defined(SIGUSR2)
    " " ":SIGUSR2"
#  endif
#  if defined(SIGPOLL)
    " " ":SIGPOLL"
#  endif
#  if defined(SIGPROF)
    " " ":SIGPROF"
#  endif
#  if defined(SIGSYS)
    " " ":SIGSYS"
#  endif
#  if defined(SIGTRAP)
    " " ":SIGTRAP"
#  endif
#  if defined(SIGURG)
    " " ":SIGURG"
#  endif
#  if defined(SIGVTALRM)
    " " ":SIGVTALRM"
#  endif
#  if defined(SIGXCPU)
    " " ":SIGXCPU"
#  endif
#  if defined(SIGXFSZ)
    " " ":SIGXFSZ"
#  endif
    " " "))"
  },
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(ST_RDONLY)
    " " ":ST_RDONLY"
#  endif
#  if defined(ST_NOSUID)
    " " ":ST_NOSUID"
#  endif
#  if defined(ST_NOTRUNC)
    " " ":ST_NOTRUNC"
#  endif
#  if defined(ST_NODEV)
    " " ":ST_NODEV"
#  endif
#  if defined(ST_NOEXEC)
    " " ":ST_NOEXEC"
#  endif
#  if defined(ST_SYNCHRONOUS)
    " " ":ST_SYNCHRONOUS"
#  endif
#  if defined(ST_MANDLOCK)
    " " ":ST_MANDLOCK"
#  endif
#  if defined(ST_WRITE)
    " " ":ST_WRITE"
#  endif
#  if defined(ST_APPEND)
    " " ":ST_APPEND"
#  endif
#  if defined(ST_IMMUTABLE)
    " " ":ST_IMMUTABLE"
#  endif
#  if defined(ST_NOATIME)
    " " ":ST_NOATIME"
#  endif
#  if defined(ST_NODIRATIME)
    " " ":ST_NODIRATIME"
#  endif
#  if defined(FILE_NAMED_STREAMS)
    " " ":FILE_NAMED_STREAMS"
#  endif
#  if defined(FILE_READ_ONLY_VOLUME)
    " " ":FILE_READ_ONLY_VOLUME"
#  endif
#  if defined(FILE_SUPPORTS_OBJECT_IDS)
    " " ":FILE_SUPPORTS_OBJECT_IDS"
#  endif
#  if defined(FILE_SUPPORTS_REPARSE_POINTS)
    " " ":FILE_SUPPORTS_REPARSE_POINTS"
#  endif
#  if defined(FILE_SUPPORTS_SPARSE_FILES)
    " " ":FILE_SUPPORTS_SPARSE_FILES"
#  endif
#  if defined(FILE_VOLUME_QUOTAS)
    " " ":FILE_VOLUME_QUOTAS"
#  endif
#  if defined(FILE_SUPPORTS_ENCRYPTION)
    " " ":FILE_SUPPORTS_ENCRYPTION"
#  endif
#  if defined(FS_CASE_IS_PRESERVED)
    " " ":FS_CASE_IS_PRESERVED"
#  endif
#  if defined(FS_CASE_SENSITIVE)
    " " ":FS_CASE_SENSITIVE"
#  endif
#  if defined(FS_FILE_COMPRESSION)
    " " ":FS_FILE_COMPRESSION"
#  endif
#  if defined(FS_FILE_ENCRYPTION)
    " " ":FS_FILE_ENCRYPTION"
#  endif
#  if defined(FS_PERSISTENT_ACLS)
    " " ":FS_PERSISTENT_ACLS"
#  endif
#  if defined(FS_UNICODE_STORED_ON_DISK)
    " " ":FS_UNICODE_STORED_ON_DISK"
#  endif
#  if defined(FS_VOL_IS_COMPRESSED)
    " " ":FS_VOL_IS_COMPRESSED"
#  endif
    " " "))"
  },
#endif
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(S_IFIFO)
    " " ":FIFO"
#  endif
#  if defined(S_IFSOCK)
    " " ":FSOCK"
#  endif
#  if defined(S_IFCHR)
    " " ":FCHR"
#  endif
#  if defined(S_IFDIR)
    " " ":FDIR"
#  endif
#  if defined(S_IFBLK)
    " " ":FBLK"
#  endif
#  if defined(S_IFREG)
    " " ":FREG"
#  endif
    " " "))"
  },
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(S_ISUID)
    " " ":SUID"
#  endif
#  if defined(S_ISGID)
    " " ":SGID"
#  endif
#  if defined(S_ISVTX)
    " " ":SVTX"
#  endif
#  if defined(S_IRWXU)
    " " ":RWXU"
#  endif
#  if defined(S_IRUSR)
    " " ":RUSR"
#  endif
#  if defined(S_IWUSR)
    " " ":WUSR"
#  endif
#  if defined(S_IXUSR)
    " " ":XUSR"
#  endif
#  if defined(S_IRWXG)
    " " ":RWXG"
#  endif
#  if defined(S_IRGRP)
    " " ":RGRP"
#  endif
#  if defined(S_IWGRP)
    " " ":WGRP"
#  endif
#  if defined(S_IXGRP)
    " " ":XGRP"
#  endif
#  if defined(S_IRWXO)
    " " ":RWXO"
#  endif
#  if defined(S_IROTH)
    " " ":ROTH"
#  endif
#  if defined(S_IWOTH)
    " " ":WOTH"
#  endif
#  if defined(S_IXOTH)
    " " ":XOTH"
#  endif
    " " "))"
  },
#if defined(HAVE_CONFSTR)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(_CS_PATH)
    " " ":PATH"
#  endif
#  if defined(_CS_POSIX_V6_ILP32_OFF32_CFLAGS)
    " " ":POSIX-V6-ILP32-OFF32-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_ILP32_OFF32_LDFLAGS)
    " " ":POSIX-V6-ILP32-OFF32-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_ILP32_OFF32_LIBS)
    " " ":POSIX-V6-ILP32-OFF32-LIBS"
#  endif
#  if defined(_CS_POSIX_V6_ILP32_OFFBIG_CFLAGS)
    " " ":POSIX-V6-ILP32-OFFBIG-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS)
    " " ":POSIX-V6-ILP32-OFFBIG-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_ILP32_OFFBIG_LIBS)
    " " ":POSIX-V6-ILP32-OFFBIG-LIBS"
#  endif
#  if defined(_CS_POSIX_V6_LP64_OFF64_CFLAGS)
    " " ":POSIX-V6-LP64-OFF64-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_LP64_OFF64_LDFLAGS)
    " " ":POSIX-V6-LP64-OFF64-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_LP64_OFF64_LIBS)
    " " ":POSIX-V6-LP64-OFF64-LIBS"
#  endif
#  if defined(_CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS)
    " " ":POSIX-V6-LPBIG-OFFBIG-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS)
    " " ":POSIX-V6-LPBIG-OFFBIG-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V6_LPBIG_OFFBIG_LIBS)
    " " ":POSIX-V6-LPBIG-OFFBIG-LIBS"
#  endif
#  if defined(_CS_POSIX_V6_WIDTH_RESTRICTED_ENVS)
    " " ":POSIX-V6-WIDTH-RESTRICTED-ENVS"
#  endif
#  if defined(_CS_V7_ENV)
    " " ":V7-ENV"
#  endif
#  if defined(_CS_POSIX_V7_ILP32_OFF32_CFLAGS)
    " " ":POSIX-V7-ILP32-OFF32-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_ILP32_OFF32_LDFLAGS)
    " " ":POSIX-V7-ILP32-OFF32-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_ILP32_OFF32_LIBS)
    " " ":POSIX-V7-ILP32-OFF32-LIBS"
#  endif
#  if defined(_CS_POSIX_V7_ILP32_OFFBIG_CFLAGS)
    " " ":POSIX-V7-ILP32-OFFBIG-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS)
    " " ":POSIX-V7-ILP32-OFFBIG-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_ILP32_OFFBIG_LIBS)
    " " ":POSIX-V7-ILP32-OFFBIG-LIBS"
#  endif
#  if defined(_CS_POSIX_V7_LP64_OFF64_CFLAGS)
    " " ":POSIX-V7-LP64-OFF64-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_LP64_OFF64_LDFLAGS)
    " " ":POSIX-V7-LP64-OFF64-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_LP64_OFF64_LIBS)
    " " ":POSIX-V7-LP64-OFF64-LIBS"
#  endif
#  if defined(_CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS)
    " " ":POSIX-V7-LPBIG-OFFBIG-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS)
    " " ":POSIX-V7-LPBIG-OFFBIG-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_LPBIG_OFFBIG_LIBS)
    " " ":POSIX-V7-LPBIG-OFFBIG-LIBS"
#  endif
#  if defined(_CS_POSIX_V7_THREADS_CFLAGS)
    " " ":POSIX-V7-THREADS-CFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_THREADS_LDFLAGS)
    " " ":POSIX-V7-THREADS-LDFLAGS"
#  endif
#  if defined(_CS_POSIX_V7_WIDTH_RESTRICTED_ENVS)
    " " ":POSIX-V7-WIDTH-RESTRICTED-ENVS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFF32_CFLAGS)
    " " ":XBS5-ILP32-OFF32-CFLAGS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFF32_LDFLAGS)
    " " ":XBS5-ILP32-OFF32-LDFLAGS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFF32_LIBS)
    " " ":XBS5-ILP32-OFF32-LIBS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFF32_LINTFLAGS)
    " " ":XBS5-ILP32-OFF32-LINTFLAGS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFFBIG_CFLAGS)
    " " ":XBS5-ILP32-OFFBIG-CFLAGS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFFBIG_LDFLAGS)
    " " ":XBS5-ILP32-OFFBIG-LDFLAGS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFFBIG_LIBS)
    " " ":XBS5-ILP32-OFFBIG-LIBS"
#  endif
#  if defined(_CS_XBS5_ILP32_OFFBIG_LINTFLAGS)
    " " ":XBS5-ILP32-OFFBIG-LINTFLAGS"
#  endif
#  if defined(_CS_XBS5_LP64_OFF64_CFLAGS)
    " " ":XBS5-LP64-OFF64-CFLAGS"
#  endif
#  if defined(_CS_XBS5_LP64_OFF64_LDFLAGS)
    " " ":XBS5-LP64-OFF64-LDFLAGS"
#  endif
#  if defined(_CS_XBS5_LP64_OFF64_LIBS)
    " " ":XBS5-LP64-OFF64-LIBS"
#  endif
#  if defined(_CS_XBS5_LP64_OFF64_LINTFLAGS)
    " " ":XBS5-LP64-OFF64-LINTFLAGS"
#  endif
#  if defined(_CS_XBS5_LPBIG_OFFBIG_CFLAGS)
    " " ":XBS5-LPBIG-OFFBIG-CFLAGS"
#  endif
#  if defined(_CS_XBS5_LPBIG_OFFBIG_LDFLAGS)
    " " ":XBS5-LPBIG-OFFBIG-LDFLAGS"
#  endif
#  if defined(_CS_XBS5_LPBIG_OFFBIG_LIBS)
    " " ":XBS5-LPBIG-OFFBIG-LIBS"
#  endif
#  if defined(_CS_XBS5_LPBIG_OFFBIG_LINTFLAGS)
    " " ":XBS5-LPBIG-OFFBIG-LINTFLAGS"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(_PC_FILESIZEBITS)
    " " ":FILESIZEBITS"
#  endif
#  if defined(_PC_LINK_MAX)
    " " ":LINK-MAX"
#  endif
#  if defined(_PC_MAX_CANON)
    " " ":MAX-CANON"
#  endif
#  if defined(_PC_MAX_INPUT)
    " " ":MAX-INPUT"
#  endif
#  if defined(_PC_NAME_MAX)
    " " ":NAME-MAX"
#  endif
#  if defined(_PC_PATH_MAX)
    " " ":PATH-MAX"
#  endif
#  if defined(_PC_PIPE_BUF)
    " " ":PIPE-BUF"
#  endif
#  if defined(_PC_2_SYMLINKS)
    " " ":2-SYMLINKS"
#  endif
#  if defined(_PC_ALLOC_SIZE_MIN)
    " " ":ALLOC-SIZE-MIN"
#  endif
#  if defined(_PC_REC_INCR_XFER_SIZE)
    " " ":REC-INCR-XFER-SIZE"
#  endif
#  if defined(_PC_REC_MAX_XFER_SIZE)
    " " ":REC-MAX-XFER-SIZE"
#  endif
#  if defined(_PC_REC_MIN_XFER_SIZE)
    " " ":REC-MIN-XFER-SIZE"
#  endif
#  if defined(_PC_REC_XFER_ALIGN)
    " " ":REC-XFER-ALIGN"
#  endif
#  if defined(_PC_SYMLINK_MAX)
    " " ":SYMLINK-MAX"
#  endif
#  if defined(_PC_CHOWN_RESTRICTED)
    " " ":CHOWN-RESTRICTED"
#  endif
#  if defined(_PC_NO_TRUNC)
    " " ":NO-TRUNC"
#  endif
#  if defined(_PC_VDISABLE)
    " " ":VDISABLE"
#  endif
#  if defined(_PC_ASYNC_IO)
    " " ":ASYNC-IO"
#  endif
#  if defined(_PC_PRIO_IO)
    " " ":PRIO-IO"
#  endif
#  if defined(_PC_SYNC_IO)
    " " ":SYNC-IO"
#  endif
#  if defined(_PC_SOCK_MAXBUF)
    " " ":SOCK-MAXBUF"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_SYSCONF)
  {
    " " "(OR INTEGER (MEMBER"
#  if defined(_SC_AIO_LISTIO_MAX)
    " " ":AIO-LISTIO-MAX"
#  endif
#  if defined(_SC_AIO_MAX)
    " " ":AIO-MAX"
#  endif
#  if defined(_SC_AIO_PRIO_DELTA_MAX)
    " " ":AIO-PRIO-DELTA-MAX"
#  endif
#  if defined(_SC_ARG_MAX)
    " " ":ARG-MAX"
#  endif
#  if defined(_SC_ATEXIT_MAX)
    " " ":ATEXIT-MAX"
#  endif
#  if defined(_SC_BC_BASE_MAX)
    " " ":BC-BASE-MAX"
#  endif
#  if defined(_SC_BC_DIM_MAX)
    " " ":BC-DIM-MAX"
#  endif
#  if defined(_SC_BC_SCALE_MAX)
    " " ":BC-SCALE-MAX"
#  endif
#  if defined(_SC_BC_STRING_MAX)
    " " ":BC-STRING-MAX"
#  endif
#  if defined(_SC_CHILD_MAX)
    " " ":CHILD-MAX"
#  endif
#  if defined(_SC_CLK_TCK)
    " " ":CLK-TCK"
#  endif
#  if defined(_SC_COLL_WEIGHTS_MAX)
    " " ":COLL-WEIGHTS-MAX"
#  endif
#  if defined(_SC_DELAYTIMER_MAX)
    " " ":DELAYTIMER-MAX"
#  endif
#  if defined(_SC_EXPR_NEST_MAX)
    " " ":EXPR-NEST-MAX"
#  endif
#  if defined(_SC_HOST_NAME_MAX)
    " " ":HOST-NAME-MAX"
#  endif
#  if defined(_SC_IOV_MAX)
    " " ":IOV-MAX"
#  endif
#  if defined(_SC_LINE_MAX)
    " " ":LINE-MAX"
#  endif
#  if defined(_SC_LOGIN_NAME_MAX)
    " " ":LOGIN-NAME-MAX"
#  endif
#  if defined(_SC_NGROUPS_MAX)
    " " ":NGROUPS-MAX"
#  endif
#  if defined(_SC_GETGR_R_SIZE_MAX)
    " " ":GETGR-R-SIZE-MAX"
#  endif
#  if defined(_SC_GETPW_R_SIZE_MAX)
    " " ":GETPW-R-SIZE-MAX"
#  endif
#  if defined(_SC_MQ_OPEN_MAX)
    " " ":MQ-OPEN-MAX"
#  endif
#  if defined(_SC_MQ_PRIO_MAX)
    " " ":MQ-PRIO-MAX"
#  endif
#  if defined(_SC_OPEN_MAX)
    " " ":OPEN-MAX"
#  endif
#  if defined(_SC_ADVISORY_INFO)
    " " ":ADVISORY-INFO"
#  endif
#  if defined(_SC_BARRIERS)
    " " ":BARRIERS"
#  endif
#  if defined(_SC_ASYNCHRONOUS_IO)
    " " ":ASYNCHRONOUS-IO"
#  endif
#  if defined(_SC_CLOCK_SELECTION)
    " " ":CLOCK-SELECTION"
#  endif
#  if defined(_SC_CPUTIME)
    " " ":CPUTIME"
#  endif
#  if defined(_SC_FSYNC)
    " " ":FSYNC"
#  endif
#  if defined(_SC_IPV6)
    " " ":IPV6"
#  endif
#  if defined(_SC_JOB_CONTROL)
    " " ":JOB-CONTROL"
#  endif
#  if defined(_SC_MAPPED_FILES)
    " " ":MAPPED-FILES"
#  endif
#  if defined(_SC_MEMLOCK)
    " " ":MEMLOCK"
#  endif
#  if defined(_SC_MEMLOCK_RANGE)
    " " ":MEMLOCK-RANGE"
#  endif
#  if defined(_SC_MEMORY_PROTECTION)
    " " ":MEMORY-PROTECTION"
#  endif
#  if defined(_SC_MESSAGE_PASSING)
    " " ":MESSAGE-PASSING"
#  endif
#  if defined(_SC_MONOTONIC_CLOCK)
    " " ":MONOTONIC-CLOCK"
#  endif
#  if defined(_SC_PRIORITIZED_IO)
    " " ":PRIORITIZED-IO"
#  endif
#  if defined(_SC_PRIORITY_SCHEDULING)
    " " ":PRIORITY-SCHEDULING"
#  endif
#  if defined(_SC_RAW_SOCKETS)
    " " ":RAW-SOCKETS"
#  endif
#  if defined(_SC_READER_WRITER_LOCKS)
    " " ":READER-WRITER-LOCKS"
#  endif
#  if defined(_SC_REALTIME_SIGNALS)
    " " ":REALTIME-SIGNALS"
#  endif
#  if defined(_SC_REGEXP)
    " " ":REGEXP"
#  endif
#  if defined(_SC_SAVED_IDS)
    " " ":SAVED-IDS"
#  endif
#  if defined(_SC_SEMAPHORES)
    " " ":SEMAPHORES"
#  endif
#  if defined(_SC_SHARED_MEMORY_OBJECTS)
    " " ":SHARED-MEMORY-OBJECTS"
#  endif
#  if defined(_SC_SHELL)
    " " ":SHELL"
#  endif
#  if defined(_SC_SPAWN)
    " " ":SPAWN"
#  endif
#  if defined(_SC_SPIN_LOCKS)
    " " ":SPIN-LOCKS"
#  endif
#  if defined(_SC_SPORADIC_SERVER)
    " " ":SPORADIC-SERVER"
#  endif
#  if defined(_SC_SS_REPL_MAX)
    " " ":SS-REPL-MAX"
#  endif
#  if defined(_SC_SYNCHRONIZED_IO)
    " " ":SYNCHRONIZED-IO"
#  endif
#  if defined(_SC_THREAD_ATTR_STACKADDR)
    " " ":THREAD-ATTR-STACKADDR"
#  endif
#  if defined(_SC_THREAD_ATTR_STACKSIZE)
    " " ":THREAD-ATTR-STACKSIZE"
#  endif
#  if defined(_SC_THREAD_CPUTIME)
    " " ":THREAD-CPUTIME"
#  endif
#  if defined(_SC_THREAD_PRIO_INHERIT)
    " " ":THREAD-PRIO-INHERIT"
#  endif
#  if defined(_SC_THREAD_PRIO_PROTECT)
    " " ":THREAD-PRIO-PROTECT"
#  endif
#  if defined(_SC_THREAD_PRIORITY_SCHEDULING)
    " " ":THREAD-PRIORITY-SCHEDULING"
#  endif
#  if defined(_SC_THREAD_PROCESS_SHARED)
    " " ":THREAD-PROCESS-SHARED"
#  endif
#  if defined(_SC_THREAD_SAFE_FUNCTIONS)
    " " ":THREAD-SAFE-FUNCTIONS"
#  endif
#  if defined(_SC_THREAD_SPORADIC_SERVER)
    " " ":THREAD-SPORADIC-SERVER"
#  endif
#  if defined(_SC_THREAD_ROBUST_PRIO_INHERIT)
    " " ":THREAD-ROBUST-PRIO-INHERIT"
#  endif
#  if defined(_SC_THREAD_ROBUST_PRIO_PROTECT)
    " " ":THREAD-ROBUST-PRIO-PROTECT"
#  endif
#  if defined(_SC_THREADS)
    " " ":THREADS"
#  endif
#  if defined(_SC_TIMEOUTS)
    " " ":TIMEOUTS"
#  endif
#  if defined(_SC_TIMERS)
    " " ":TIMERS"
#  endif
#  if defined(_SC_TRACE)
    " " ":TRACE"
#  endif
#  if defined(_SC_TRACE_EVENT_FILTER)
    " " ":TRACE-EVENT-FILTER"
#  endif
#  if defined(_SC_TRACE_EVENT_NAME_MAX)
    " " ":TRACE-EVENT-NAME-MAX"
#  endif
#  if defined(_SC_TRACE_INHERIT)
    " " ":TRACE-INHERIT"
#  endif
#  if defined(_SC_TRACE_LOG)
    " " ":TRACE-LOG"
#  endif
#  if defined(_SC_TRACE_NAME_MAX)
    " " ":TRACE-NAME-MAX"
#  endif
#  if defined(_SC_TRACE_SYS_MAX)
    " " ":TRACE-SYS-MAX"
#  endif
#  if defined(_SC_TRACE_USER_EVENT_MAX)
    " " ":TRACE-USER-EVENT-MAX"
#  endif
#  if defined(_SC_TYPED_MEMORY_OBJECTS)
    " " ":TYPED-MEMORY-OBJECTS"
#  endif
#  if defined(_SC_VERSION)
    " " ":VERSION"
#  endif
#  if defined(_SC_XBS5_ILP32_OFF32)
    " " ":XBS5-ILP32-OFF32"
#  endif
#  if defined(_SC_XBS5_ILP32_OFFBIG)
    " " ":XBS5-ILP32-OFFBIG"
#  endif
#  if defined(_SC_XBS5_LP64_OFF64)
    " " ":XBS5-LP64-OFF64"
#  endif
#  if defined(_SC_XBS5_LPBIG_OFFBIG)
    " " ":XBS5-LPBIG-OFFBIG"
#  endif
#  if defined(_SC_V6_ILP32_OFF32)
    " " ":V6-ILP32-OFF32"
#  endif
#  if defined(_SC_V6_ILP32_OFFBIG)
    " " ":V6-ILP32-OFFBIG"
#  endif
#  if defined(_SC_V6_LP64_OFF64)
    " " ":V6-LP64-OFF64"
#  endif
#  if defined(_SC_V6_LPBIG_OFFBIG)
    " " ":V6-LPBIG-OFFBIG"
#  endif
#  if defined(_SC_V7_ILP32_OFF32)
    " " ":V7-ILP32-OFF32"
#  endif
#  if defined(_SC_V7_ILP32_OFFBIG)
    " " ":V7-ILP32-OFFBIG"
#  endif
#  if defined(_SC_V7_LP64_OFF64)
    " " ":V7-LP64-OFF64"
#  endif
#  if defined(_SC_V7_LPBIG_OFFBIG)
    " " ":V7-LPBIG-OFFBIG"
#  endif
#  if defined(_SC_2_C_BIND)
    " " ":2-C-BIND"
#  endif
#  if defined(_SC_2_C_DEV)
    " " ":2-C-DEV"
#  endif
#  if defined(_SC_2_CHAR_TERM)
    " " ":2-CHAR-TERM"
#  endif
#  if defined(_SC_2_FORT_DEV)
    " " ":2-FORT-DEV"
#  endif
#  if defined(_SC_2_FORT_RUN)
    " " ":2-FORT-RUN"
#  endif
#  if defined(_SC_2_LOCALEDEF)
    " " ":2-LOCALEDEF"
#  endif
#  if defined(_SC_2_PBS)
    " " ":2-PBS"
#  endif
#  if defined(_SC_2_PBS_ACCOUNTING)
    " " ":2-PBS-ACCOUNTING"
#  endif
#  if defined(_SC_2_PBS_CHECKPOINT)
    " " ":2-PBS-CHECKPOINT"
#  endif
#  if defined(_SC_2_PBS_LOCATE)
    " " ":2-PBS-LOCATE"
#  endif
#  if defined(_SC_2_PBS_MESSAGE)
    " " ":2-PBS-MESSAGE"
#  endif
#  if defined(_SC_2_PBS_TRACK)
    " " ":2-PBS-TRACK"
#  endif
#  if defined(_SC_2_SW_DEV)
    " " ":2-SW-DEV"
#  endif
#  if defined(_SC_2_UPE)
    " " ":2-UPE"
#  endif
#  if defined(_SC_2_VERSION)
    " " ":2-VERSION"
#  endif
#  if defined(_SC_PAGESIZE)
    " " ":PAGESIZE"
#  endif
#  if defined(_SC_PHYS_PAGES)
    " " ":PHYS-PAGES"
#  endif
#  if defined(_SC_AVPHYS_PAGES)
    " " ":AVPHYS-PAGES"
#  endif
#  if defined(_SC_THREAD_DESTRUCTOR_ITERATIONS)
    " " ":THREAD-DESTRUCTOR-ITERATIONS"
#  endif
#  if defined(_SC_THREAD_KEYS_MAX)
    " " ":THREAD-KEYS-MAX"
#  endif
#  if defined(_SC_THREAD_STACK_MIN)
    " " ":THREAD-STACK-MIN"
#  endif
#  if defined(_SC_THREAD_THREADS_MAX)
    " " ":THREAD-THREADS-MAX"
#  endif
#  if defined(_SC_RE_DUP_MAX)
    " " ":RE-DUP-MAX"
#  endif
#  if defined(_SC_RTSIG_MAX)
    " " ":RTSIG-MAX"
#  endif
#  if defined(_SC_SEM_NSEMS_MAX)
    " " ":SEM-NSEMS-MAX"
#  endif
#  if defined(_SC_SEM_VALUE_MAX)
    " " ":SEM-VALUE-MAX"
#  endif
#  if defined(_SC_SIGQUEUE_MAX)
    " " ":SIGQUEUE-MAX"
#  endif
#  if defined(_SC_STREAM_MAX)
    " " ":STREAM-MAX"
#  endif
#  if defined(_SC_SYMLOOP_MAX)
    " " ":SYMLOOP-MAX"
#  endif
#  if defined(_SC_TIMER_MAX)
    " " ":TIMER-MAX"
#  endif
#  if defined(_SC_TTY_NAME_MAX)
    " " ":TTY-NAME-MAX"
#  endif
#  if defined(_SC_TZNAME_MAX)
    " " ":TZNAME-MAX"
#  endif
#  if defined(_SC_XOPEN_CRYPT)
    " " ":XOPEN-CRYPT"
#  endif
#  if defined(_SC_XOPEN_ENH_I18N)
    " " ":XOPEN-ENH-I18N"
#  endif
#  if defined(_SC_XOPEN_LEGACY)
    " " ":XOPEN-LEGACY"
#  endif
#  if defined(_SC_XOPEN_REALTIME)
    " " ":XOPEN-REALTIME"
#  endif
#  if defined(_SC_XOPEN_REALTIME_THREADS)
    " " ":XOPEN-REALTIME-THREADS"
#  endif
#  if defined(_SC_XOPEN_SHM)
    " " ":XOPEN-SHM"
#  endif
#  if defined(_SC_XOPEN_STREAMS)
    " " ":XOPEN-STREAMS"
#  endif
#  if defined(_SC_XOPEN_UNIX)
    " " ":XOPEN-UNIX"
#  endif
#  if defined(_SC_XOPEN_UUCP)
    " " ":XOPEN-UUCP"
#  endif
#  if defined(_SC_XOPEN_VERSION)
    " " ":XOPEN-VERSION"
#  endif
#  if defined(_SC_NPROCESSORS_CONF)
    " " ":NPROCESSORS-CONF"
#  endif
#  if defined(_SC_NPROCESSORS_ONLN)
    " " ":NPROCESSORS-ONLN"
#  endif
    " " "))"
  },
#endif
#if defined(HAVE_SYSLOG)
  {
    " " "(OR NULL INTEGER (MEMBER"
#  if defined(LOG_KERN)
    " " ":KERN"
#  endif
#  if defined(LOG_USER)
    " " ":USER"
#  endif
#  if defined(LOG_MAIL)
    " " ":MAIL"
#  endif
#  if defined(LOG_NEWS)
    " " ":NEWS"
#  endif
#  if defined(LOG_UUCP)
    " " ":UUCP"
#  endif
#  if defined(LOG_DAEMON)
    " " ":DAEMON"
#  endif
#  if defined(LOG_AUTH)
    " " ":AUTH"
#  endif
#  if defined(LOG_CRON)
    " " ":CRON"
#  endif
#  if defined(LOG_LPR)
    " " ":LPR"
#  endif
#  if defined(LOG_SYSLOG)
    " " ":SYSLOG"
#  endif
#  if defined(LOG_AUTHPRIV)
    " " ":AUTHPRIV"
#  endif
#  if defined(LOG_FTP)
    " " ":FTP"
#  endif
#  if defined(LOG_LOCAL0)
    " " ":LOCAL0"
#  endif
#  if defined(LOG_LOCAL1)
    " " ":LOCAL1"
#  endif
#  if defined(LOG_LOCAL2)
    " " ":LOCAL2"
#  endif
#  if defined(LOG_LOCAL3)
    " " ":LOCAL3"
#  endif
#  if defined(LOG_LOCAL4)
    " " ":LOCAL4"
#  endif
#  if defined(LOG_LOCAL5)
    " " ":LOCAL5"
#  endif
#  if defined(LOG_LOCAL6)
    " " ":LOCAL6"
#  endif
#  if defined(LOG_LOCAL7)
    " " ":LOCAL7"
#  endif
    " " "))"
  },
#endif
  {
    " " "(OR NULL INTEGER (MEMBER"
#  if defined(PRIO_PROCESS)
    " " ":PROCESS"
#  endif
#  if defined(PRIO_PGRP)
    " " ":PGRP"
#  endif
#  if defined(PRIO_USER)
    " " ":USER"
#  endif
    " " "))"
  },
#if defined(WIN32_NATIVE)
  {
    " " "(OR NULL INTEGER (MEMBER"
#  if defined(REALTIME_PRIORITY_CLASS)
    " " ":REALTIME"
#  endif
#  if defined(HIGH_PRIORITY_CLASS)
    " " ":HIGH"
#  endif
#  if defined(ABOVE_NORMAL_PRIORITY_CLASS)
    " " ":ABOVE-NORMAL"
#  endif
#  if defined(NORMAL_PRIORITY_CLASS)
    " " ":NORMAL"
#  endif
#  if defined(BELOW_NORMAL_PRIORITY_CLASS)
    " " ":BELOW-NORMAL"
#  endif
#  if defined(LOW_PRIORITY_CLASS)
    " " ":LOW"
#  endif
#  if defined(IDLE_PRIORITY_CLASS)
    " " ":IDLE"
#  endif
    " " "))"
  },
#endif
#if !(defined(WIN32_NATIVE))
  {
    " " "(OR NULL INTEGER (MEMBER"
    " " ":REALTIME"
    " " ":HIGH"
    " " ":ABOVE-NORMAL"
    " " ":NORMAL"
    " " ":BELOW-NORMAL"
    " " ":LOW"
    " " ":IDLE"
    " " "))"
  },
#endif
  { "NIL" },
  { "NIL" },
  { "NIL" },
  0
};

struct module__syscalls__subr_tab_t {
  VAROBJECTS_ALIGNMENT_DUMMY_DECL
  subr_t _subr_os_bogomips;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_os_clipboard;
#endif
  subr_t _subr_os_file_owner;
#if defined(WIN32_NATIVE)
  subr_t _subr_os_file_version;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_os_last_error;
#endif
  subr_t _subr_os_priority;
  subr_t _subr_os_version_3C;
  subr_t _subr_os_version_3C_3D;
  subr_t _subr_os_version_3E;
  subr_t _subr_os_version_3E_3D;
  subr_t _subr_os_version_compare;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_os__25set_clipboard;
#endif
  subr_t _subr_os__25set_priority;
#if defined(HAVE_SYS_RESOURCE_H)
  subr_t _subr_posix_begin_subprocesses;
#endif
#if defined(HAVE_CHROOT)
  subr_t _subr_posix_chroot;
#endif
  subr_t _subr_posix_clearerr;
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_CLOSELOG))
  subr_t _subr_posix_closelog;
#endif
#if defined(HAVE_CONFSTR)
  subr_t _subr_posix_confstr;
#endif
#if defined(DEBUG_SPVW)
  subr_t _subr_posix_constsym;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_convert_attributes;
#endif
  subr_t _subr_posix_convert_mode;
  subr_t _subr_posix_copy_file;
#if defined(HAVE_GETDOMAINNAME)
  subr_t _subr_posix_domainname;
#endif
  subr_t _subr_posix_duplicate_handle;
#if defined(HAVE_GETEGID)
  subr_t _subr_posix_egid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_ENDUTXENT))
  subr_t _subr_posix_endutxent;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  subr_t _subr_posix_end_subprocesses;
#endif
#if defined(HAVE_ERFC)
  subr_t _subr_posix_erf;
#endif
#if defined(HAVE_ERFC)
  subr_t _subr_posix_erfc;
#endif
  subr_t _subr_posix_errno;
#if defined(HAVE_GETEUID)
  subr_t _subr_posix_euid;
#endif
  subr_t _subr_posix_fclose;
  subr_t _subr_posix_fdopen;
  subr_t _subr_posix_feof;
  subr_t _subr_posix_ferror;
  subr_t _subr_posix_fflush;
  subr_t _subr_posix_fileno;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_file_info;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_file_properties;
#endif
  subr_t _subr_posix_file_size;
#if defined(HAVE_FSTAT) && defined(HAVE_STAT)
  subr_t _subr_posix_file_stat;
#endif
#if defined(HAVE_NFTW)
  subr_t _subr_posix_file_tree_walk;
#endif
  subr_t _subr_posix_fnmatch;
  subr_t _subr_posix_fopen;
  subr_t _subr_posix_freopen;
#if defined(HAVE_GETDATE) && defined(HAVE_DECL_GETDATE_ERR)
  subr_t _subr_posix_getdate;
#endif
#if defined(HAVE_GETPGRP)
  subr_t _subr_posix_getpgrp;
#endif
#if defined(HAVE_GETPPID)
  subr_t _subr_posix_getppid;
#endif
#if defined(HAVE_GETSID)
  subr_t _subr_posix_getsid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXENT))
  subr_t _subr_posix_getutxent;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXID))
  subr_t _subr_posix_getutxid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXLINE))
  subr_t _subr_posix_getutxline;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_get_user_sid;
#endif
#if defined(HAVE_GETGID)
  subr_t _subr_posix_gid;
#endif
#if defined(HAVE_GETGROUPS)
  subr_t _subr_posix_groups;
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  subr_t _subr_posix_group_info;
#endif
#if defined(HAVE_GETHOSTID)
  subr_t _subr_posix_hostid;
#endif
  subr_t _subr_posix_j0;
  subr_t _subr_posix_j1;
  subr_t _subr_posix_jn;
#if defined(HAVE_KILL)
  subr_t _subr_posix_kill;
#endif
#if defined(HAVE_LGAMMA) || HAVE_DECL_LGAMMA_R
  subr_t _subr_posix_lgamma;
#endif
  subr_t _subr_posix_loadavg;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_make_shortcut;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_memory_status;
#endif
  subr_t _subr_posix_mkdtemp;
  subr_t _subr_posix_mknod;
  subr_t _subr_posix_mkstemp;
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  subr_t _subr_posix_openlog;
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  subr_t _subr_posix_pathconf;
#endif
#if defined(HAVE_GETPGID)
  subr_t _subr_posix_pgid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_PUTUTXLINE))
  subr_t _subr_posix_pututxline;
#endif
  subr_t _subr_posix_resolve_host_ipaddr;
#if defined(HAVE_GETRLIMIT)
  subr_t _subr_posix_rlimit;
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  subr_t _subr_posix_service;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_SETLOGMASK))
  subr_t _subr_posix_setlogmask;
#endif
#if defined(HAVE_SETPGRP)
  subr_t _subr_posix_setpgrp;
#endif
#if defined(HAVE_SETREGID)
  subr_t _subr_posix_setregid;
#endif
#if defined(HAVE_SETREUID)
  subr_t _subr_posix_setreuid;
#endif
#if defined(HAVE_SETSID)
  subr_t _subr_posix_setsid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_SETUTXENT))
  subr_t _subr_posix_setutxent;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  subr_t _subr_posix_set_file_stat;
#endif
#if defined(HAVE_SETRLIMIT)
  subr_t _subr_posix_set_rlimit;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_shortcut_info;
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  subr_t _subr_posix_stat_vfs;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  subr_t _subr_posix_stream_lock;
#endif
#if defined(HAVE_FCNTL)
  subr_t _subr_posix_stream_options;
#endif
  subr_t _subr_posix_string_time;
#if defined(WIN32_NATIVE) || defined(HAVE_SYNC) || defined(HAVE_FSYNC)
  subr_t _subr_posix_sync;
#endif
#if defined(HAVE_SYSCONF)
  subr_t _subr_posix_sysconf;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_system_info;
#endif
#if defined(HAVE_TGAMMA)
  subr_t _subr_posix_tgamma;
#endif
#if defined(HAVE_GETUID)
  subr_t _subr_posix_uid;
#endif
#if defined(HAVE_UMASK)
  subr_t _subr_posix_umask;
#endif
  subr_t _subr_posix_uname;
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  subr_t _subr_posix_usage;
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  subr_t _subr_posix_user_info;
#endif
#if defined(HAVE_GETUSERSHELL) && defined(HAVE_ENDUSERSHELL)
  subr_t _subr_posix_user_shells;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_version;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  subr_t _subr_posix_wait;
#endif
  subr_t _subr_posix_y0;
  subr_t _subr_posix_y1;
  subr_t _subr_posix_yn;
  subr_t _subr_posix__25fgetc;
  subr_t _subr_posix__25fputc;
#if defined(HAVE_SETDOMAINNAME)
  subr_t _subr_posix__25setdomainname;
#endif
#if defined(HAVE_SETEGID)
  subr_t _subr_posix__25setegid;
#endif
#if defined(HAVE_SETEUID)
  subr_t _subr_posix__25seteuid;
#endif
#if defined(HAVE_SETGID)
  subr_t _subr_posix__25setgid;
#endif
#if defined(HAVE_SETGROUPS)
  subr_t _subr_posix__25setgroups;
#endif
#if defined(HAVE_SETHOSTID) && !defined(_AIX)
  subr_t _subr_posix__25sethostid;
#endif
#if defined(HAVE_SETPGID)
  subr_t _subr_posix__25setpgid;
#endif
#if defined(HAVE_SETUID)
  subr_t _subr_posix__25setuid;
#endif
  subr_t _subr_posix__25set_file_size;
  subr_t _subr_posix__25stdio;
#if defined(HAVE_SYSLOG)
  subr_t _subr_posix__25syslog;
#endif
  subr_t _subr_posix__25ungetc;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_sys_format_message;
#endif
  subr_t _subr_sys_strerror;
  int _dummy_to_avoid_trailing_comma_in_initializer;
};
extern struct module__syscalls__subr_tab_t module__syscalls__subr_tab;

struct module__syscalls__stab_t {
  subr_t _subr_os_bogomips;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_os_clipboard;
#endif
  subr_t _subr_os_file_owner;
#if defined(WIN32_NATIVE)
  subr_t _subr_os_file_version;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_os_last_error;
#endif
  subr_t _subr_os_priority;
  subr_t _subr_os_version_3C;
  subr_t _subr_os_version_3C_3D;
  subr_t _subr_os_version_3E;
  subr_t _subr_os_version_3E_3D;
  subr_t _subr_os_version_compare;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_os__25set_clipboard;
#endif
  subr_t _subr_os__25set_priority;
#if defined(HAVE_SYS_RESOURCE_H)
  subr_t _subr_posix_begin_subprocesses;
#endif
#if defined(HAVE_CHROOT)
  subr_t _subr_posix_chroot;
#endif
  subr_t _subr_posix_clearerr;
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_CLOSELOG))
  subr_t _subr_posix_closelog;
#endif
#if defined(HAVE_CONFSTR)
  subr_t _subr_posix_confstr;
#endif
#if defined(DEBUG_SPVW)
  subr_t _subr_posix_constsym;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_convert_attributes;
#endif
  subr_t _subr_posix_convert_mode;
  subr_t _subr_posix_copy_file;
#if defined(HAVE_GETDOMAINNAME)
  subr_t _subr_posix_domainname;
#endif
  subr_t _subr_posix_duplicate_handle;
#if defined(HAVE_GETEGID)
  subr_t _subr_posix_egid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_ENDUTXENT))
  subr_t _subr_posix_endutxent;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  subr_t _subr_posix_end_subprocesses;
#endif
#if defined(HAVE_ERFC)
  subr_t _subr_posix_erf;
#endif
#if defined(HAVE_ERFC)
  subr_t _subr_posix_erfc;
#endif
  subr_t _subr_posix_errno;
#if defined(HAVE_GETEUID)
  subr_t _subr_posix_euid;
#endif
  subr_t _subr_posix_fclose;
  subr_t _subr_posix_fdopen;
  subr_t _subr_posix_feof;
  subr_t _subr_posix_ferror;
  subr_t _subr_posix_fflush;
  subr_t _subr_posix_fileno;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_file_info;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_file_properties;
#endif
  subr_t _subr_posix_file_size;
#if defined(HAVE_FSTAT) && defined(HAVE_STAT)
  subr_t _subr_posix_file_stat;
#endif
#if defined(HAVE_NFTW)
  subr_t _subr_posix_file_tree_walk;
#endif
  subr_t _subr_posix_fnmatch;
  subr_t _subr_posix_fopen;
  subr_t _subr_posix_freopen;
#if defined(HAVE_GETDATE) && defined(HAVE_DECL_GETDATE_ERR)
  subr_t _subr_posix_getdate;
#endif
#if defined(HAVE_GETPGRP)
  subr_t _subr_posix_getpgrp;
#endif
#if defined(HAVE_GETPPID)
  subr_t _subr_posix_getppid;
#endif
#if defined(HAVE_GETSID)
  subr_t _subr_posix_getsid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXENT))
  subr_t _subr_posix_getutxent;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXID))
  subr_t _subr_posix_getutxid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXLINE))
  subr_t _subr_posix_getutxline;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_get_user_sid;
#endif
#if defined(HAVE_GETGID)
  subr_t _subr_posix_gid;
#endif
#if defined(HAVE_GETGROUPS)
  subr_t _subr_posix_groups;
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  subr_t _subr_posix_group_info;
#endif
#if defined(HAVE_GETHOSTID)
  subr_t _subr_posix_hostid;
#endif
  subr_t _subr_posix_j0;
  subr_t _subr_posix_j1;
  subr_t _subr_posix_jn;
#if defined(HAVE_KILL)
  subr_t _subr_posix_kill;
#endif
#if defined(HAVE_LGAMMA) || HAVE_DECL_LGAMMA_R
  subr_t _subr_posix_lgamma;
#endif
  subr_t _subr_posix_loadavg;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_make_shortcut;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_memory_status;
#endif
  subr_t _subr_posix_mkdtemp;
  subr_t _subr_posix_mknod;
  subr_t _subr_posix_mkstemp;
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  subr_t _subr_posix_openlog;
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  subr_t _subr_posix_pathconf;
#endif
#if defined(HAVE_GETPGID)
  subr_t _subr_posix_pgid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_PUTUTXLINE))
  subr_t _subr_posix_pututxline;
#endif
  subr_t _subr_posix_resolve_host_ipaddr;
#if defined(HAVE_GETRLIMIT)
  subr_t _subr_posix_rlimit;
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  subr_t _subr_posix_service;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_SETLOGMASK))
  subr_t _subr_posix_setlogmask;
#endif
#if defined(HAVE_SETPGRP)
  subr_t _subr_posix_setpgrp;
#endif
#if defined(HAVE_SETREGID)
  subr_t _subr_posix_setregid;
#endif
#if defined(HAVE_SETREUID)
  subr_t _subr_posix_setreuid;
#endif
#if defined(HAVE_SETSID)
  subr_t _subr_posix_setsid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_SETUTXENT))
  subr_t _subr_posix_setutxent;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  subr_t _subr_posix_set_file_stat;
#endif
#if defined(HAVE_SETRLIMIT)
  subr_t _subr_posix_set_rlimit;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_shortcut_info;
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  subr_t _subr_posix_stat_vfs;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  subr_t _subr_posix_stream_lock;
#endif
#if defined(HAVE_FCNTL)
  subr_t _subr_posix_stream_options;
#endif
  subr_t _subr_posix_string_time;
#if defined(WIN32_NATIVE) || defined(HAVE_SYNC) || defined(HAVE_FSYNC)
  subr_t _subr_posix_sync;
#endif
#if defined(HAVE_SYSCONF)
  subr_t _subr_posix_sysconf;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_system_info;
#endif
#if defined(HAVE_TGAMMA)
  subr_t _subr_posix_tgamma;
#endif
#if defined(HAVE_GETUID)
  subr_t _subr_posix_uid;
#endif
#if defined(HAVE_UMASK)
  subr_t _subr_posix_umask;
#endif
  subr_t _subr_posix_uname;
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  subr_t _subr_posix_usage;
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  subr_t _subr_posix_user_info;
#endif
#if defined(HAVE_GETUSERSHELL) && defined(HAVE_ENDUSERSHELL)
  subr_t _subr_posix_user_shells;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_posix_version;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  subr_t _subr_posix_wait;
#endif
  subr_t _subr_posix_y0;
  subr_t _subr_posix_y1;
  subr_t _subr_posix_yn;
  subr_t _subr_posix__25fgetc;
  subr_t _subr_posix__25fputc;
#if defined(HAVE_SETDOMAINNAME)
  subr_t _subr_posix__25setdomainname;
#endif
#if defined(HAVE_SETEGID)
  subr_t _subr_posix__25setegid;
#endif
#if defined(HAVE_SETEUID)
  subr_t _subr_posix__25seteuid;
#endif
#if defined(HAVE_SETGID)
  subr_t _subr_posix__25setgid;
#endif
#if defined(HAVE_SETGROUPS)
  subr_t _subr_posix__25setgroups;
#endif
#if defined(HAVE_SETHOSTID) && !defined(_AIX)
  subr_t _subr_posix__25sethostid;
#endif
#if defined(HAVE_SETPGID)
  subr_t _subr_posix__25setpgid;
#endif
#if defined(HAVE_SETUID)
  subr_t _subr_posix__25setuid;
#endif
  subr_t _subr_posix__25set_file_size;
  subr_t _subr_posix__25stdio;
#if defined(HAVE_SYSLOG)
  subr_t _subr_posix__25syslog;
#endif
  subr_t _subr_posix__25ungetc;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_t _subr_sys_format_message;
#endif
  subr_t _subr_sys_strerror;
  int _dummy_to_avoid_trailing_comma_in_initializer;
};
static struct module__syscalls__stab_t* this_module_stab;

#if defined(HAVE_SYSLOG)
static uintL syslog_opt_flags (void) {
  uintL flags = 0
#  ifdef LOG_NOWAIT
    | (missingp(STACK_(0)) ? 0 : LOG_NOWAIT)
#  endif
#  ifdef LOG_ODELAY
    | (missingp(STACK_(1)) ? 0 : LOG_ODELAY)
#  endif
#  ifdef LOG_NDELAY
    | (missingp(STACK_(2)) ? 0 : LOG_NDELAY)
#  endif
#  ifdef LOG_CONS
    | (missingp(STACK_(3)) ? 0 : LOG_CONS)
#  endif
#  ifdef LOG_PID
    | (missingp(STACK_(4)) ? 0 : LOG_PID)
#  endif
   ;
  skipSTACK(5);
  return flags;
}

#endif
#if defined(HAVE_SYS_RESOURCE_H)
static uintL wait_flags (void) {
  uintL flags = 0
#  ifdef WNOWAIT
    | (missingp(STACK_(0)) ? 0 : WNOWAIT)
#  endif
#  ifdef WCONTINUED
    | (missingp(STACK_(1)) ? 0 : WCONTINUED)
#  endif
#  ifdef WEXITED
    | (missingp(STACK_(2)) ? 0 : WEXITED)
#  endif
#  ifdef WSTOPPED
    | (missingp(STACK_(3)) ? 0 : WSTOPPED)
#  endif
#  ifdef WUNTRACED
    | (missingp(STACK_(4)) ? 0 : WUNTRACED)
#  endif
#  ifdef WNOHANG
    | (missingp(STACK_(5)) ? 0 : WNOHANG)
#  endif
   ;
  skipSTACK(6);
  return flags;
}

#endif
#if defined(HAVE_NFTW)
static uintL ftw_flags (void) {
  uintL flags = 0
#  ifdef FTW_PHYS
    | (missingp(STACK_(0)) ? 0 : FTW_PHYS)
#  endif
#  ifdef FTW_MOUNT
    | (missingp(STACK_(1)) ? 0 : FTW_MOUNT)
#  endif
#  ifdef FTW_DEPTH
    | (missingp(STACK_(2)) ? 0 : FTW_DEPTH)
#  endif
#  ifdef FTW_CHDIR
    | (missingp(STACK_(3)) ? 0 : FTW_CHDIR)
#  endif
   ;
  skipSTACK(4);
  return flags;
}

#endif
static uintL fnm_flags (void) {
  uintL flags = 0
#  ifdef FNM_NOESCAPE
    | (missingp(STACK_(0)) ? 0 : FNM_NOESCAPE)
#  endif
#  ifdef FNM_PERIOD
    | (missingp(STACK_(1)) ? 0 : FNM_PERIOD)
#  endif
#  ifdef FNM_PATHNAME
    | (missingp(STACK_(2)) ? 0 : FNM_PATHNAME)
#  endif
#  ifdef FNM_CASEFOLD
    | (!nullp(STACK_(3)) ? 0 : FNM_CASEFOLD)
#  endif
   ;
  skipSTACK(4);
  return flags;
}


#if defined(HAVE_FCNTL)
static const c_lisp_pair_t check_fcntl_cmd_table[] = {
 #ifdef F_GETFD
  { F_GETFD, &(O(object_Kfd)) },
 #endif
 #ifdef F_GETFL
  { F_GETFL, &(O(object_Kfl)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_fcntl_cmd_map = {
  check_fcntl_cmd_table,
  (sizeof(check_fcntl_cmd_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_fcntl_cmd"
};
#define check_fcntl_cmd(a) (int)map_lisp_to_c(a,&check_fcntl_cmd_map)
#define check_fcntl_cmd_reverse(a) map_c_to_lisp(a,&check_fcntl_cmd_map)

#endif
#if defined(HAVE_FCNTL)
static const c_lisp_pair_t check_fl_flags_table[] = {
 #ifdef O_RDONLY
  { O_RDONLY, &(O(object_Krdonly)) },
 #endif
 #ifdef O_WRONLY
  { O_WRONLY, &(O(object_Kwronly)) },
 #endif
 #ifdef O_RDWR
  { O_RDWR, &(O(object_Krdwr)) },
 #endif
 #ifdef O_APPEND
  { O_APPEND, &(O(object_Kappend)) },
 #endif
 #ifdef O_CREAT
  { O_CREAT, &(O(object_Kcreat)) },
 #endif
 #ifdef O_TRUNC
  { O_TRUNC, &(O(object_Ktrunc)) },
 #endif
 #ifdef O_EXCL
  { O_EXCL, &(O(object_Kexcl)) },
 #endif
 #ifdef O_NOCTTY
  { O_NOCTTY, &(O(object_Knoctty)) },
 #endif
 #ifdef O_SYNC
  { O_SYNC, &(O(object_Ksync)) },
 #endif
 #ifdef O_NONBLOCK
  { O_NONBLOCK, &(O(object_Knonblock)) },
 #endif
 #ifdef O_BINARY
  { O_BINARY, &(O(object_Kbinary)) },
 #endif
 #ifdef O_TEXT
  { O_TEXT, &(O(object_Ktext)) },
 #endif
 #ifdef O_NOINHERIT
  { O_NOINHERIT, &(O(object_Knoinherit)) },
 #endif
 #ifdef O_DIRECT
  { O_DIRECT, &(O(object_Kdirect)) },
 #endif
 #ifdef O_LARGEFILE
  { O_LARGEFILE, &(O(object_Klargefile)) },
 #endif
 #ifdef O_DIRECTORY
  { O_DIRECTORY, &(O(object_Kdirectory)) },
 #endif
 #ifdef O_NOFOLLOW
  { O_NOFOLLOW, &(O(object_Knofollow)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_fl_flags_map = {
  check_fl_flags_table,
  (sizeof(check_fl_flags_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_fl_flags"
};
#define check_fl_flags(a) (int)map_lisp_to_c(a,&check_fl_flags_map)
#define check_fl_flags_reverse(a) map_c_to_lisp(a,&check_fl_flags_map)
#define check_fl_flags_to_list(a) map_c_to_list(a,&check_fl_flags_map)
#define check_fl_flags_of_list(a) map_list_to_c(a,&check_fl_flags_map)

#endif
#if defined(HAVE_FCNTL)
static const c_lisp_pair_t check_fd_flags_table[] = {
 #ifdef FD_CLOEXEC
  { FD_CLOEXEC, &(O(object_Kcloexec)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_fd_flags_map = {
  check_fd_flags_table,
  (sizeof(check_fd_flags_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_fd_flags"
};
#define check_fd_flags(a) (int)map_lisp_to_c(a,&check_fd_flags_map)
#define check_fd_flags_reverse(a) map_c_to_lisp(a,&check_fd_flags_map)
#define check_fd_flags_to_list(a) map_c_to_list(a,&check_fd_flags_map)
#define check_fd_flags_of_list(a) map_list_to_c(a,&check_fd_flags_map)

#endif
#if defined(HAVE_SYSLOG)
static const c_lisp_pair_t check_syslog_severity_table[] = {
 #ifdef LOG_EMERG
  { LOG_EMERG, &(O(object_Kemerg)) },
 #endif
 #ifdef LOG_ALERT
  { LOG_ALERT, &(O(object_Kalert)) },
 #endif
 #ifdef LOG_CRIT
  { LOG_CRIT, &(O(object_Kcrit)) },
 #endif
 #ifdef LOG_ERR
  { LOG_ERR, &(O(object_Kerr)) },
 #endif
 #ifdef LOG_WARNING
  { LOG_WARNING, &(O(object_Kwarning)) },
 #endif
 #ifdef LOG_NOTICE
  { LOG_NOTICE, &(O(object_Knotice)) },
 #endif
 #ifdef LOG_INFO
  { LOG_INFO, &(O(object_Kinfo)) },
 #endif
 #ifdef LOG_DEBUG
  { LOG_DEBUG, &(O(object_Kdebug)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_syslog_severity_map = {
  check_syslog_severity_table,
  (sizeof(check_syslog_severity_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_syslog_severity"
};
#define check_syslog_severity(a) (int)map_lisp_to_c(a,&check_syslog_severity_map)
#define check_syslog_severity_reverse(a) map_c_to_lisp(a,&check_syslog_severity_map)

#endif
#if defined(HAVE_SYSLOG)
static const c_lisp_pair_t check_syslog_facility_table[] = {
 #ifdef LOG_KERN
  { LOG_KERN, &(O(object_Kkern)) },
 #endif
 #ifdef LOG_USER
  { LOG_USER, &(O(object_Kuser)) },
 #endif
 #ifdef LOG_MAIL
  { LOG_MAIL, &(O(object_Kmail)) },
 #endif
 #ifdef LOG_NEWS
  { LOG_NEWS, &(O(object_Knews)) },
 #endif
 #ifdef LOG_UUCP
  { LOG_UUCP, &(O(object_Kuucp)) },
 #endif
 #ifdef LOG_DAEMON
  { LOG_DAEMON, &(O(object_Kdaemon)) },
 #endif
 #ifdef LOG_AUTH
  { LOG_AUTH, &(O(object_Kauth)) },
 #endif
 #ifdef LOG_CRON
  { LOG_CRON, &(O(object_Kcron)) },
 #endif
 #ifdef LOG_LPR
  { LOG_LPR, &(O(object_Klpr)) },
 #endif
 #ifdef LOG_SYSLOG
  { LOG_SYSLOG, &(O(object_Ksyslog)) },
 #endif
 #ifdef LOG_AUTHPRIV
  { LOG_AUTHPRIV, &(O(object_Kauthpriv)) },
 #endif
 #ifdef LOG_FTP
  { LOG_FTP, &(O(object_Kftp)) },
 #endif
 #ifdef LOG_LOCAL0
  { LOG_LOCAL0, &(O(object_Klocal0)) },
 #endif
 #ifdef LOG_LOCAL1
  { LOG_LOCAL1, &(O(object_Klocal1)) },
 #endif
 #ifdef LOG_LOCAL2
  { LOG_LOCAL2, &(O(object_Klocal2)) },
 #endif
 #ifdef LOG_LOCAL3
  { LOG_LOCAL3, &(O(object_Klocal3)) },
 #endif
 #ifdef LOG_LOCAL4
  { LOG_LOCAL4, &(O(object_Klocal4)) },
 #endif
 #ifdef LOG_LOCAL5
  { LOG_LOCAL5, &(O(object_Klocal5)) },
 #endif
 #ifdef LOG_LOCAL6
  { LOG_LOCAL6, &(O(object_Klocal6)) },
 #endif
 #ifdef LOG_LOCAL7
  { LOG_LOCAL7, &(O(object_Klocal7)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_syslog_facility_map = {
  check_syslog_facility_table,
  (sizeof(check_syslog_facility_table)/sizeof(c_lisp_pair_t))-1,
# if defined(LOG_USER)
  LOG_USER,true,
# else
  0,false,
# endif
  "check_syslog_facility"
};
#define check_syslog_facility(a) (int)map_lisp_to_c(a,&check_syslog_facility_map)
#define check_syslog_facility_reverse(a) map_c_to_lisp(a,&check_syslog_facility_map)

#endif
#if defined(HAVE_UTMPX_H)
static const c_lisp_pair_t check_ut_type_table[] = {
 #ifdef EMPTY
  { EMPTY, &(O(object_Kempty)) },
 #endif
 #ifdef RUN_LVL
  { RUN_LVL, &(O(object_Krun_lvl)) },
 #endif
 #ifdef BOOT_TIME
  { BOOT_TIME, &(O(object_Kboot_time)) },
 #endif
 #ifdef OLD_TIME
  { OLD_TIME, &(O(object_Kold_time)) },
 #endif
 #ifdef NEW_TIME
  { NEW_TIME, &(O(object_Knew_time)) },
 #endif
 #ifdef USER_PROCESS
  { USER_PROCESS, &(O(object_Kuser_process)) },
 #endif
 #ifdef INIT_PROCESS
  { INIT_PROCESS, &(O(object_Kinit_process)) },
 #endif
 #ifdef LOGIN_PROCESS
  { LOGIN_PROCESS, &(O(object_Klogin_process)) },
 #endif
 #ifdef DEAD_PROCESS
  { DEAD_PROCESS, &(O(object_Kdead_process)) },
 #endif
 #ifdef ACCOUNTING
  { ACCOUNTING, &(O(object_Kaccounting)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_ut_type_map = {
  check_ut_type_table,
  (sizeof(check_ut_type_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_ut_type"
};
#define check_ut_type(a) (int)map_lisp_to_c(a,&check_ut_type_map)
#define check_ut_type_reverse(a) map_c_to_lisp(a,&check_ut_type_map)

#endif
static const c_lisp_pair_t check_signal_table[] = {
 #ifdef SIGABRT
  { SIGABRT, &(O(object_Ksigabrt)) },
 #endif
 #ifdef SIGALRM
  { SIGALRM, &(O(object_Ksigalrm)) },
 #endif
 #ifdef SIGBUS
  { SIGBUS, &(O(object_Ksigbus)) },
 #endif
 #ifdef SIGCHLD
  { SIGCHLD, &(O(object_Ksigchld)) },
 #endif
 #ifdef SIGCONT
  { SIGCONT, &(O(object_Ksigcont)) },
 #endif
 #ifdef SIGFPE
  { SIGFPE, &(O(object_Ksigfpe)) },
 #endif
 #ifdef SIGHUP
  { SIGHUP, &(O(object_Ksighup)) },
 #endif
 #ifdef SIGILL
  { SIGILL, &(O(object_Ksigill)) },
 #endif
 #ifdef SIGINT
  { SIGINT, &(O(object_Ksigint)) },
 #endif
 #ifdef SIGKILL
  { SIGKILL, &(O(object_Ksigkill)) },
 #endif
 #ifdef SIGPIPE
  { SIGPIPE, &(O(object_Ksigpipe)) },
 #endif
 #ifdef SIGQUIT
  { SIGQUIT, &(O(object_Ksigquit)) },
 #endif
 #ifdef SIGSEGV
  { SIGSEGV, &(O(object_Ksigsegv)) },
 #endif
 #ifdef SIGSTOP
  { SIGSTOP, &(O(object_Ksigstop)) },
 #endif
 #ifdef SIGTERM
  { SIGTERM, &(O(object_Ksigterm)) },
 #endif
 #ifdef SIGTSTP
  { SIGTSTP, &(O(object_Ksigtstp)) },
 #endif
 #ifdef SIGTTIN
  { SIGTTIN, &(O(object_Ksigttin)) },
 #endif
 #ifdef SIGTTOU
  { SIGTTOU, &(O(object_Ksigttou)) },
 #endif
 #ifdef SIGUSR1
  { SIGUSR1, &(O(object_Ksigusr1)) },
 #endif
 #ifdef SIGUSR2
  { SIGUSR2, &(O(object_Ksigusr2)) },
 #endif
 #ifdef SIGPOLL
  { SIGPOLL, &(O(object_Ksigpoll)) },
 #endif
 #ifdef SIGPROF
  { SIGPROF, &(O(object_Ksigprof)) },
 #endif
 #ifdef SIGSYS
  { SIGSYS, &(O(object_Ksigsys)) },
 #endif
 #ifdef SIGTRAP
  { SIGTRAP, &(O(object_Ksigtrap)) },
 #endif
 #ifdef SIGURG
  { SIGURG, &(O(object_Ksigurg)) },
 #endif
 #ifdef SIGVTALRM
  { SIGVTALRM, &(O(object_Ksigvtalrm)) },
 #endif
 #ifdef SIGXCPU
  { SIGXCPU, &(O(object_Ksigxcpu)) },
 #endif
 #ifdef SIGXFSZ
  { SIGXFSZ, &(O(object_Ksigxfsz)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_signal_map = {
  check_signal_table,
  (sizeof(check_signal_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_signal"
};
#define check_signal(a) (int)map_lisp_to_c(a,&check_signal_map)
#define check_signal_reverse(a) map_c_to_lisp(a,&check_signal_map)

#if defined(WIN32_NATIVE)
static const c_lisp_pair_t check_priority_value_table[] = {
 #ifdef REALTIME_PRIORITY_CLASS
  { REALTIME_PRIORITY_CLASS, &(O(object_Krealtime)) },
 #endif
 #ifdef HIGH_PRIORITY_CLASS
  { HIGH_PRIORITY_CLASS, &(O(object_Khigh)) },
 #endif
 #ifdef ABOVE_NORMAL_PRIORITY_CLASS
  { ABOVE_NORMAL_PRIORITY_CLASS, &(O(object_Kabove_normal)) },
 #endif
 #ifdef NORMAL_PRIORITY_CLASS
  { NORMAL_PRIORITY_CLASS, &(O(object_Knormal)) },
 #endif
 #ifdef BELOW_NORMAL_PRIORITY_CLASS
  { BELOW_NORMAL_PRIORITY_CLASS, &(O(object_Kbelow_normal)) },
 #endif
 #ifdef LOW_PRIORITY_CLASS
  { LOW_PRIORITY_CLASS, &(O(object_Klow)) },
 #endif
 #ifdef IDLE_PRIORITY_CLASS
  { IDLE_PRIORITY_CLASS, &(O(object_Kidle)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_priority_value_map = {
  check_priority_value_table,
  (sizeof(check_priority_value_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_priority_value"
};
#define check_priority_value(a) (int)map_lisp_to_c(a,&check_priority_value_map)
#define check_priority_value_reverse(a) map_c_to_lisp(a,&check_priority_value_map)

#endif
#if !(defined(WIN32_NATIVE))
static const c_lisp_pair_t check_priority_value_table[] = {
  { -NZERO, &(O(object_Krealtime)) },
  { (-NZERO/2), &(O(object_Khigh)) },
  { (-NZERO/4), &(O(object_Kabove_normal)) },
  { 0, &(O(object_Knormal)) },
  { (NZERO/4), &(O(object_Kbelow_normal)) },
  { (NZERO/2), &(O(object_Klow)) },
  { NZERO, &(O(object_Kidle)) },
  { 0, NULL }
};
static const c_lisp_map_t check_priority_value_map = {
  check_priority_value_table,
  (sizeof(check_priority_value_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_priority_value"
};
#define check_priority_value(a) (int)map_lisp_to_c(a,&check_priority_value_map)
#define check_priority_value_reverse(a) map_c_to_lisp(a,&check_priority_value_map)

#endif
static const c_lisp_pair_t check_priority_which_table[] = {
 #ifdef PRIO_PROCESS
  { PRIO_PROCESS, &(O(object_Kprocess)) },
 #endif
 #ifdef PRIO_PGRP
  { PRIO_PGRP, &(O(object_Kpgrp)) },
 #endif
 #ifdef PRIO_USER
  { PRIO_USER, &(O(object_Kuser)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_priority_which_map = {
  check_priority_which_table,
  (sizeof(check_priority_which_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_priority_which"
};
#define check_priority_which(a) (int)map_lisp_to_c(a,&check_priority_which_map)
#define check_priority_which_reverse(a) map_c_to_lisp(a,&check_priority_which_map)

#if defined(HAVE_SYSCONF)
static const c_lisp_pair_t sysconf_arg_table[] = {
 #ifdef _SC_AIO_LISTIO_MAX
  { _SC_AIO_LISTIO_MAX, &(O(object_Kaio_listio_max)) },
 #endif
 #ifdef _SC_AIO_MAX
  { _SC_AIO_MAX, &(O(object_Kaio_max)) },
 #endif
 #ifdef _SC_AIO_PRIO_DELTA_MAX
  { _SC_AIO_PRIO_DELTA_MAX, &(O(object_Kaio_prio_delta_max)) },
 #endif
 #ifdef _SC_ARG_MAX
  { _SC_ARG_MAX, &(O(object_Karg_max)) },
 #endif
 #ifdef _SC_ATEXIT_MAX
  { _SC_ATEXIT_MAX, &(O(object_Katexit_max)) },
 #endif
 #ifdef _SC_BC_BASE_MAX
  { _SC_BC_BASE_MAX, &(O(object_Kbc_base_max)) },
 #endif
 #ifdef _SC_BC_DIM_MAX
  { _SC_BC_DIM_MAX, &(O(object_Kbc_dim_max)) },
 #endif
 #ifdef _SC_BC_SCALE_MAX
  { _SC_BC_SCALE_MAX, &(O(object_Kbc_scale_max)) },
 #endif
 #ifdef _SC_BC_STRING_MAX
  { _SC_BC_STRING_MAX, &(O(object_Kbc_string_max)) },
 #endif
 #ifdef _SC_CHILD_MAX
  { _SC_CHILD_MAX, &(O(object_Kchild_max)) },
 #endif
 #ifdef _SC_CLK_TCK
  { _SC_CLK_TCK, &(O(object_Kclk_tck)) },
 #endif
 #ifdef _SC_COLL_WEIGHTS_MAX
  { _SC_COLL_WEIGHTS_MAX, &(O(object_Kcoll_weights_max)) },
 #endif
 #ifdef _SC_DELAYTIMER_MAX
  { _SC_DELAYTIMER_MAX, &(O(object_Kdelaytimer_max)) },
 #endif
 #ifdef _SC_EXPR_NEST_MAX
  { _SC_EXPR_NEST_MAX, &(O(object_Kexpr_nest_max)) },
 #endif
 #ifdef _SC_HOST_NAME_MAX
  { _SC_HOST_NAME_MAX, &(O(object_Khost_name_max)) },
 #endif
 #ifdef _SC_IOV_MAX
  { _SC_IOV_MAX, &(O(object_Kiov_max)) },
 #endif
 #ifdef _SC_LINE_MAX
  { _SC_LINE_MAX, &(O(object_Kline_max)) },
 #endif
 #ifdef _SC_LOGIN_NAME_MAX
  { _SC_LOGIN_NAME_MAX, &(O(object_Klogin_name_max)) },
 #endif
 #ifdef _SC_NGROUPS_MAX
  { _SC_NGROUPS_MAX, &(O(object_Kngroups_max)) },
 #endif
 #ifdef _SC_GETGR_R_SIZE_MAX
  { _SC_GETGR_R_SIZE_MAX, &(O(object_Kgetgr_r_size_max)) },
 #endif
 #ifdef _SC_GETPW_R_SIZE_MAX
  { _SC_GETPW_R_SIZE_MAX, &(O(object_Kgetpw_r_size_max)) },
 #endif
 #ifdef _SC_MQ_OPEN_MAX
  { _SC_MQ_OPEN_MAX, &(O(object_Kmq_open_max)) },
 #endif
 #ifdef _SC_MQ_PRIO_MAX
  { _SC_MQ_PRIO_MAX, &(O(object_Kmq_prio_max)) },
 #endif
 #ifdef _SC_OPEN_MAX
  { _SC_OPEN_MAX, &(O(object_Kopen_max)) },
 #endif
 #ifdef _SC_ADVISORY_INFO
  { _SC_ADVISORY_INFO, &(O(object_Kadvisory_info)) },
 #endif
 #ifdef _SC_BARRIERS
  { _SC_BARRIERS, &(O(object_Kbarriers)) },
 #endif
 #ifdef _SC_ASYNCHRONOUS_IO
  { _SC_ASYNCHRONOUS_IO, &(O(object_Kasynchronous_io)) },
 #endif
 #ifdef _SC_CLOCK_SELECTION
  { _SC_CLOCK_SELECTION, &(O(object_Kclock_selection)) },
 #endif
 #ifdef _SC_CPUTIME
  { _SC_CPUTIME, &(O(object_Kcputime)) },
 #endif
 #ifdef _SC_FSYNC
  { _SC_FSYNC, &(O(object_Kfsync)) },
 #endif
 #ifdef _SC_IPV6
  { _SC_IPV6, &(O(object_Kipv6)) },
 #endif
 #ifdef _SC_JOB_CONTROL
  { _SC_JOB_CONTROL, &(O(object_Kjob_control)) },
 #endif
 #ifdef _SC_MAPPED_FILES
  { _SC_MAPPED_FILES, &(O(object_Kmapped_files)) },
 #endif
 #ifdef _SC_MEMLOCK
  { _SC_MEMLOCK, &(O(object_Kmemlock)) },
 #endif
 #ifdef _SC_MEMLOCK_RANGE
  { _SC_MEMLOCK_RANGE, &(O(object_Kmemlock_range)) },
 #endif
 #ifdef _SC_MEMORY_PROTECTION
  { _SC_MEMORY_PROTECTION, &(O(object_Kmemory_protection)) },
 #endif
 #ifdef _SC_MESSAGE_PASSING
  { _SC_MESSAGE_PASSING, &(O(object_Kmessage_passing)) },
 #endif
 #ifdef _SC_MONOTONIC_CLOCK
  { _SC_MONOTONIC_CLOCK, &(O(object_Kmonotonic_clock)) },
 #endif
 #ifdef _SC_PRIORITIZED_IO
  { _SC_PRIORITIZED_IO, &(O(object_Kprioritized_io)) },
 #endif
 #ifdef _SC_PRIORITY_SCHEDULING
  { _SC_PRIORITY_SCHEDULING, &(O(object_Kpriority_scheduling)) },
 #endif
 #ifdef _SC_RAW_SOCKETS
  { _SC_RAW_SOCKETS, &(O(object_Kraw_sockets)) },
 #endif
 #ifdef _SC_READER_WRITER_LOCKS
  { _SC_READER_WRITER_LOCKS, &(O(object_Kreader_writer_locks)) },
 #endif
 #ifdef _SC_REALTIME_SIGNALS
  { _SC_REALTIME_SIGNALS, &(O(object_Krealtime_signals)) },
 #endif
 #ifdef _SC_REGEXP
  { _SC_REGEXP, &(O(object_Kregexp)) },
 #endif
 #ifdef _SC_SAVED_IDS
  { _SC_SAVED_IDS, &(O(object_Ksaved_ids)) },
 #endif
 #ifdef _SC_SEMAPHORES
  { _SC_SEMAPHORES, &(O(object_Ksemaphores)) },
 #endif
 #ifdef _SC_SHARED_MEMORY_OBJECTS
  { _SC_SHARED_MEMORY_OBJECTS, &(O(object_Kshared_memory_objects)) },
 #endif
 #ifdef _SC_SHELL
  { _SC_SHELL, &(O(object_Kshell)) },
 #endif
 #ifdef _SC_SPAWN
  { _SC_SPAWN, &(O(object_Kspawn)) },
 #endif
 #ifdef _SC_SPIN_LOCKS
  { _SC_SPIN_LOCKS, &(O(object_Kspin_locks)) },
 #endif
 #ifdef _SC_SPORADIC_SERVER
  { _SC_SPORADIC_SERVER, &(O(object_Ksporadic_server)) },
 #endif
 #ifdef _SC_SS_REPL_MAX
  { _SC_SS_REPL_MAX, &(O(object_Kss_repl_max)) },
 #endif
 #ifdef _SC_SYNCHRONIZED_IO
  { _SC_SYNCHRONIZED_IO, &(O(object_Ksynchronized_io)) },
 #endif
 #ifdef _SC_THREAD_ATTR_STACKADDR
  { _SC_THREAD_ATTR_STACKADDR, &(O(object_Kthread_attr_stackaddr)) },
 #endif
 #ifdef _SC_THREAD_ATTR_STACKSIZE
  { _SC_THREAD_ATTR_STACKSIZE, &(O(object_Kthread_attr_stacksize)) },
 #endif
 #ifdef _SC_THREAD_CPUTIME
  { _SC_THREAD_CPUTIME, &(O(object_Kthread_cputime)) },
 #endif
 #ifdef _SC_THREAD_PRIO_INHERIT
  { _SC_THREAD_PRIO_INHERIT, &(O(object_Kthread_prio_inherit)) },
 #endif
 #ifdef _SC_THREAD_PRIO_PROTECT
  { _SC_THREAD_PRIO_PROTECT, &(O(object_Kthread_prio_protect)) },
 #endif
 #ifdef _SC_THREAD_PRIORITY_SCHEDULING
  { _SC_THREAD_PRIORITY_SCHEDULING, &(O(object_Kthread_priority_scheduling)) },
 #endif
 #ifdef _SC_THREAD_PROCESS_SHARED
  { _SC_THREAD_PROCESS_SHARED, &(O(object_Kthread_process_shared)) },
 #endif
 #ifdef _SC_THREAD_SAFE_FUNCTIONS
  { _SC_THREAD_SAFE_FUNCTIONS, &(O(object_Kthread_safe_functions)) },
 #endif
 #ifdef _SC_THREAD_SPORADIC_SERVER
  { _SC_THREAD_SPORADIC_SERVER, &(O(object_Kthread_sporadic_server)) },
 #endif
 #ifdef _SC_THREAD_ROBUST_PRIO_INHERIT
  { _SC_THREAD_ROBUST_PRIO_INHERIT, &(O(object_Kthread_robust_prio_inherit)) },
 #endif
 #ifdef _SC_THREAD_ROBUST_PRIO_PROTECT
  { _SC_THREAD_ROBUST_PRIO_PROTECT, &(O(object_Kthread_robust_prio_protect)) },
 #endif
 #ifdef _SC_THREADS
  { _SC_THREADS, &(O(object_Kthreads)) },
 #endif
 #ifdef _SC_TIMEOUTS
  { _SC_TIMEOUTS, &(O(object_Ktimeouts)) },
 #endif
 #ifdef _SC_TIMERS
  { _SC_TIMERS, &(O(object_Ktimers)) },
 #endif
 #ifdef _SC_TRACE
  { _SC_TRACE, &(O(object_Ktrace)) },
 #endif
 #ifdef _SC_TRACE_EVENT_FILTER
  { _SC_TRACE_EVENT_FILTER, &(O(object_Ktrace_event_filter)) },
 #endif
 #ifdef _SC_TRACE_EVENT_NAME_MAX
  { _SC_TRACE_EVENT_NAME_MAX, &(O(object_Ktrace_event_name_max)) },
 #endif
 #ifdef _SC_TRACE_INHERIT
  { _SC_TRACE_INHERIT, &(O(object_Ktrace_inherit)) },
 #endif
 #ifdef _SC_TRACE_LOG
  { _SC_TRACE_LOG, &(O(object_Ktrace_log)) },
 #endif
 #ifdef _SC_TRACE_NAME_MAX
  { _SC_TRACE_NAME_MAX, &(O(object_Ktrace_name_max)) },
 #endif
 #ifdef _SC_TRACE_SYS_MAX
  { _SC_TRACE_SYS_MAX, &(O(object_Ktrace_sys_max)) },
 #endif
 #ifdef _SC_TRACE_USER_EVENT_MAX
  { _SC_TRACE_USER_EVENT_MAX, &(O(object_Ktrace_user_event_max)) },
 #endif
 #ifdef _SC_TYPED_MEMORY_OBJECTS
  { _SC_TYPED_MEMORY_OBJECTS, &(O(object_Ktyped_memory_objects)) },
 #endif
 #ifdef _SC_VERSION
  { _SC_VERSION, &(O(object_Kversion)) },
 #endif
 #ifdef _SC_XBS5_ILP32_OFF32
  { _SC_XBS5_ILP32_OFF32, &(O(object_Kxbs5_ilp32_off32)) },
 #endif
 #ifdef _SC_XBS5_ILP32_OFFBIG
  { _SC_XBS5_ILP32_OFFBIG, &(O(object_Kxbs5_ilp32_offbig)) },
 #endif
 #ifdef _SC_XBS5_LP64_OFF64
  { _SC_XBS5_LP64_OFF64, &(O(object_Kxbs5_lp64_off64)) },
 #endif
 #ifdef _SC_XBS5_LPBIG_OFFBIG
  { _SC_XBS5_LPBIG_OFFBIG, &(O(object_Kxbs5_lpbig_offbig)) },
 #endif
 #ifdef _SC_V6_ILP32_OFF32
  { _SC_V6_ILP32_OFF32, &(O(object_Kv6_ilp32_off32)) },
 #endif
 #ifdef _SC_V6_ILP32_OFFBIG
  { _SC_V6_ILP32_OFFBIG, &(O(object_Kv6_ilp32_offbig)) },
 #endif
 #ifdef _SC_V6_LP64_OFF64
  { _SC_V6_LP64_OFF64, &(O(object_Kv6_lp64_off64)) },
 #endif
 #ifdef _SC_V6_LPBIG_OFFBIG
  { _SC_V6_LPBIG_OFFBIG, &(O(object_Kv6_lpbig_offbig)) },
 #endif
 #ifdef _SC_V7_ILP32_OFF32
  { _SC_V7_ILP32_OFF32, &(O(object_Kv7_ilp32_off32)) },
 #endif
 #ifdef _SC_V7_ILP32_OFFBIG
  { _SC_V7_ILP32_OFFBIG, &(O(object_Kv7_ilp32_offbig)) },
 #endif
 #ifdef _SC_V7_LP64_OFF64
  { _SC_V7_LP64_OFF64, &(O(object_Kv7_lp64_off64)) },
 #endif
 #ifdef _SC_V7_LPBIG_OFFBIG
  { _SC_V7_LPBIG_OFFBIG, &(O(object_Kv7_lpbig_offbig)) },
 #endif
 #ifdef _SC_2_C_BIND
  { _SC_2_C_BIND, &(O(object_K2_c_bind)) },
 #endif
 #ifdef _SC_2_C_DEV
  { _SC_2_C_DEV, &(O(object_K2_c_dev)) },
 #endif
 #ifdef _SC_2_CHAR_TERM
  { _SC_2_CHAR_TERM, &(O(object_K2_char_term)) },
 #endif
 #ifdef _SC_2_FORT_DEV
  { _SC_2_FORT_DEV, &(O(object_K2_fort_dev)) },
 #endif
 #ifdef _SC_2_FORT_RUN
  { _SC_2_FORT_RUN, &(O(object_K2_fort_run)) },
 #endif
 #ifdef _SC_2_LOCALEDEF
  { _SC_2_LOCALEDEF, &(O(object_K2_localedef)) },
 #endif
 #ifdef _SC_2_PBS
  { _SC_2_PBS, &(O(object_K2_pbs)) },
 #endif
 #ifdef _SC_2_PBS_ACCOUNTING
  { _SC_2_PBS_ACCOUNTING, &(O(object_K2_pbs_accounting)) },
 #endif
 #ifdef _SC_2_PBS_CHECKPOINT
  { _SC_2_PBS_CHECKPOINT, &(O(object_K2_pbs_checkpoint)) },
 #endif
 #ifdef _SC_2_PBS_LOCATE
  { _SC_2_PBS_LOCATE, &(O(object_K2_pbs_locate)) },
 #endif
 #ifdef _SC_2_PBS_MESSAGE
  { _SC_2_PBS_MESSAGE, &(O(object_K2_pbs_message)) },
 #endif
 #ifdef _SC_2_PBS_TRACK
  { _SC_2_PBS_TRACK, &(O(object_K2_pbs_track)) },
 #endif
 #ifdef _SC_2_SW_DEV
  { _SC_2_SW_DEV, &(O(object_K2_sw_dev)) },
 #endif
 #ifdef _SC_2_UPE
  { _SC_2_UPE, &(O(object_K2_upe)) },
 #endif
 #ifdef _SC_2_VERSION
  { _SC_2_VERSION, &(O(object_K2_version)) },
 #endif
 #ifdef _SC_PAGESIZE
  { _SC_PAGESIZE, &(O(object_Kpagesize)) },
 #endif
 #ifdef _SC_PHYS_PAGES
  { _SC_PHYS_PAGES, &(O(object_Kphys_pages)) },
 #endif
 #ifdef _SC_AVPHYS_PAGES
  { _SC_AVPHYS_PAGES, &(O(object_Kavphys_pages)) },
 #endif
 #ifdef _SC_THREAD_DESTRUCTOR_ITERATIONS
  { _SC_THREAD_DESTRUCTOR_ITERATIONS, &(O(object_Kthread_destructor_iterations)) },
 #endif
 #ifdef _SC_THREAD_KEYS_MAX
  { _SC_THREAD_KEYS_MAX, &(O(object_Kthread_keys_max)) },
 #endif
 #ifdef _SC_THREAD_STACK_MIN
  { _SC_THREAD_STACK_MIN, &(O(object_Kthread_stack_min)) },
 #endif
 #ifdef _SC_THREAD_THREADS_MAX
  { _SC_THREAD_THREADS_MAX, &(O(object_Kthread_threads_max)) },
 #endif
 #ifdef _SC_RE_DUP_MAX
  { _SC_RE_DUP_MAX, &(O(object_Kre_dup_max)) },
 #endif
 #ifdef _SC_RTSIG_MAX
  { _SC_RTSIG_MAX, &(O(object_Krtsig_max)) },
 #endif
 #ifdef _SC_SEM_NSEMS_MAX
  { _SC_SEM_NSEMS_MAX, &(O(object_Ksem_nsems_max)) },
 #endif
 #ifdef _SC_SEM_VALUE_MAX
  { _SC_SEM_VALUE_MAX, &(O(object_Ksem_value_max)) },
 #endif
 #ifdef _SC_SIGQUEUE_MAX
  { _SC_SIGQUEUE_MAX, &(O(object_Ksigqueue_max)) },
 #endif
 #ifdef _SC_STREAM_MAX
  { _SC_STREAM_MAX, &(O(object_Kstream_max)) },
 #endif
 #ifdef _SC_SYMLOOP_MAX
  { _SC_SYMLOOP_MAX, &(O(object_Ksymloop_max)) },
 #endif
 #ifdef _SC_TIMER_MAX
  { _SC_TIMER_MAX, &(O(object_Ktimer_max)) },
 #endif
 #ifdef _SC_TTY_NAME_MAX
  { _SC_TTY_NAME_MAX, &(O(object_Ktty_name_max)) },
 #endif
 #ifdef _SC_TZNAME_MAX
  { _SC_TZNAME_MAX, &(O(object_Ktzname_max)) },
 #endif
 #ifdef _SC_XOPEN_CRYPT
  { _SC_XOPEN_CRYPT, &(O(object_Kxopen_crypt)) },
 #endif
 #ifdef _SC_XOPEN_ENH_I18N
  { _SC_XOPEN_ENH_I18N, &(O(object_Kxopen_enh_i18n)) },
 #endif
 #ifdef _SC_XOPEN_LEGACY
  { _SC_XOPEN_LEGACY, &(O(object_Kxopen_legacy)) },
 #endif
 #ifdef _SC_XOPEN_REALTIME
  { _SC_XOPEN_REALTIME, &(O(object_Kxopen_realtime)) },
 #endif
 #ifdef _SC_XOPEN_REALTIME_THREADS
  { _SC_XOPEN_REALTIME_THREADS, &(O(object_Kxopen_realtime_threads)) },
 #endif
 #ifdef _SC_XOPEN_SHM
  { _SC_XOPEN_SHM, &(O(object_Kxopen_shm)) },
 #endif
 #ifdef _SC_XOPEN_STREAMS
  { _SC_XOPEN_STREAMS, &(O(object_Kxopen_streams)) },
 #endif
 #ifdef _SC_XOPEN_UNIX
  { _SC_XOPEN_UNIX, &(O(object_Kxopen_unix)) },
 #endif
 #ifdef _SC_XOPEN_UUCP
  { _SC_XOPEN_UUCP, &(O(object_Kxopen_uucp)) },
 #endif
 #ifdef _SC_XOPEN_VERSION
  { _SC_XOPEN_VERSION, &(O(object_Kxopen_version)) },
 #endif
 #ifdef _SC_NPROCESSORS_CONF
  { _SC_NPROCESSORS_CONF, &(O(object_Knprocessors_conf)) },
 #endif
 #ifdef _SC_NPROCESSORS_ONLN
  { _SC_NPROCESSORS_ONLN, &(O(object_Knprocessors_onln)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t sysconf_arg_map = {
  sysconf_arg_table,
  (sizeof(sysconf_arg_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "sysconf_arg"
};
#define sysconf_arg(a) (int)map_lisp_to_c(a,&sysconf_arg_map)
#define sysconf_arg_reverse(a) map_c_to_lisp(a,&sysconf_arg_map)

#endif
#if defined(HAVE_CONFSTR)
static const c_lisp_pair_t confstr_arg_table[] = {
 #ifdef _CS_PATH
  { _CS_PATH, &(O(object_Kpath)) },
 #endif
 #ifdef _CS_POSIX_V6_ILP32_OFF32_CFLAGS
  { _CS_POSIX_V6_ILP32_OFF32_CFLAGS, &(O(object_Kposix_v6_ilp32_off32_cflags)) },
 #endif
 #ifdef _CS_POSIX_V6_ILP32_OFF32_LDFLAGS
  { _CS_POSIX_V6_ILP32_OFF32_LDFLAGS, &(O(object_Kposix_v6_ilp32_off32_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V6_ILP32_OFF32_LIBS
  { _CS_POSIX_V6_ILP32_OFF32_LIBS, &(O(object_Kposix_v6_ilp32_off32_libs)) },
 #endif
 #ifdef _CS_POSIX_V6_ILP32_OFFBIG_CFLAGS
  { _CS_POSIX_V6_ILP32_OFFBIG_CFLAGS, &(O(object_Kposix_v6_ilp32_offbig_cflags)) },
 #endif
 #ifdef _CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS
  { _CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS, &(O(object_Kposix_v6_ilp32_offbig_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V6_ILP32_OFFBIG_LIBS
  { _CS_POSIX_V6_ILP32_OFFBIG_LIBS, &(O(object_Kposix_v6_ilp32_offbig_libs)) },
 #endif
 #ifdef _CS_POSIX_V6_LP64_OFF64_CFLAGS
  { _CS_POSIX_V6_LP64_OFF64_CFLAGS, &(O(object_Kposix_v6_lp64_off64_cflags)) },
 #endif
 #ifdef _CS_POSIX_V6_LP64_OFF64_LDFLAGS
  { _CS_POSIX_V6_LP64_OFF64_LDFLAGS, &(O(object_Kposix_v6_lp64_off64_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V6_LP64_OFF64_LIBS
  { _CS_POSIX_V6_LP64_OFF64_LIBS, &(O(object_Kposix_v6_lp64_off64_libs)) },
 #endif
 #ifdef _CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS
  { _CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS, &(O(object_Kposix_v6_lpbig_offbig_cflags)) },
 #endif
 #ifdef _CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS
  { _CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS, &(O(object_Kposix_v6_lpbig_offbig_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V6_LPBIG_OFFBIG_LIBS
  { _CS_POSIX_V6_LPBIG_OFFBIG_LIBS, &(O(object_Kposix_v6_lpbig_offbig_libs)) },
 #endif
 #ifdef _CS_POSIX_V6_WIDTH_RESTRICTED_ENVS
  { _CS_POSIX_V6_WIDTH_RESTRICTED_ENVS, &(O(object_Kposix_v6_width_restricted_envs)) },
 #endif
 #ifdef _CS_V7_ENV
  { _CS_V7_ENV, &(O(object_Kv7_env)) },
 #endif
 #ifdef _CS_POSIX_V7_ILP32_OFF32_CFLAGS
  { _CS_POSIX_V7_ILP32_OFF32_CFLAGS, &(O(object_Kposix_v7_ilp32_off32_cflags)) },
 #endif
 #ifdef _CS_POSIX_V7_ILP32_OFF32_LDFLAGS
  { _CS_POSIX_V7_ILP32_OFF32_LDFLAGS, &(O(object_Kposix_v7_ilp32_off32_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V7_ILP32_OFF32_LIBS
  { _CS_POSIX_V7_ILP32_OFF32_LIBS, &(O(object_Kposix_v7_ilp32_off32_libs)) },
 #endif
 #ifdef _CS_POSIX_V7_ILP32_OFFBIG_CFLAGS
  { _CS_POSIX_V7_ILP32_OFFBIG_CFLAGS, &(O(object_Kposix_v7_ilp32_offbig_cflags)) },
 #endif
 #ifdef _CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS
  { _CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS, &(O(object_Kposix_v7_ilp32_offbig_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V7_ILP32_OFFBIG_LIBS
  { _CS_POSIX_V7_ILP32_OFFBIG_LIBS, &(O(object_Kposix_v7_ilp32_offbig_libs)) },
 #endif
 #ifdef _CS_POSIX_V7_LP64_OFF64_CFLAGS
  { _CS_POSIX_V7_LP64_OFF64_CFLAGS, &(O(object_Kposix_v7_lp64_off64_cflags)) },
 #endif
 #ifdef _CS_POSIX_V7_LP64_OFF64_LDFLAGS
  { _CS_POSIX_V7_LP64_OFF64_LDFLAGS, &(O(object_Kposix_v7_lp64_off64_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V7_LP64_OFF64_LIBS
  { _CS_POSIX_V7_LP64_OFF64_LIBS, &(O(object_Kposix_v7_lp64_off64_libs)) },
 #endif
 #ifdef _CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS
  { _CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS, &(O(object_Kposix_v7_lpbig_offbig_cflags)) },
 #endif
 #ifdef _CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS
  { _CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS, &(O(object_Kposix_v7_lpbig_offbig_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V7_LPBIG_OFFBIG_LIBS
  { _CS_POSIX_V7_LPBIG_OFFBIG_LIBS, &(O(object_Kposix_v7_lpbig_offbig_libs)) },
 #endif
 #ifdef _CS_POSIX_V7_THREADS_CFLAGS
  { _CS_POSIX_V7_THREADS_CFLAGS, &(O(object_Kposix_v7_threads_cflags)) },
 #endif
 #ifdef _CS_POSIX_V7_THREADS_LDFLAGS
  { _CS_POSIX_V7_THREADS_LDFLAGS, &(O(object_Kposix_v7_threads_ldflags)) },
 #endif
 #ifdef _CS_POSIX_V7_WIDTH_RESTRICTED_ENVS
  { _CS_POSIX_V7_WIDTH_RESTRICTED_ENVS, &(O(object_Kposix_v7_width_restricted_envs)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFF32_CFLAGS
  { _CS_XBS5_ILP32_OFF32_CFLAGS, &(O(object_Kxbs5_ilp32_off32_cflags)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFF32_LDFLAGS
  { _CS_XBS5_ILP32_OFF32_LDFLAGS, &(O(object_Kxbs5_ilp32_off32_ldflags)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFF32_LIBS
  { _CS_XBS5_ILP32_OFF32_LIBS, &(O(object_Kxbs5_ilp32_off32_libs)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFF32_LINTFLAGS
  { _CS_XBS5_ILP32_OFF32_LINTFLAGS, &(O(object_Kxbs5_ilp32_off32_lintflags)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFFBIG_CFLAGS
  { _CS_XBS5_ILP32_OFFBIG_CFLAGS, &(O(object_Kxbs5_ilp32_offbig_cflags)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFFBIG_LDFLAGS
  { _CS_XBS5_ILP32_OFFBIG_LDFLAGS, &(O(object_Kxbs5_ilp32_offbig_ldflags)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFFBIG_LIBS
  { _CS_XBS5_ILP32_OFFBIG_LIBS, &(O(object_Kxbs5_ilp32_offbig_libs)) },
 #endif
 #ifdef _CS_XBS5_ILP32_OFFBIG_LINTFLAGS
  { _CS_XBS5_ILP32_OFFBIG_LINTFLAGS, &(O(object_Kxbs5_ilp32_offbig_lintflags)) },
 #endif
 #ifdef _CS_XBS5_LP64_OFF64_CFLAGS
  { _CS_XBS5_LP64_OFF64_CFLAGS, &(O(object_Kxbs5_lp64_off64_cflags)) },
 #endif
 #ifdef _CS_XBS5_LP64_OFF64_LDFLAGS
  { _CS_XBS5_LP64_OFF64_LDFLAGS, &(O(object_Kxbs5_lp64_off64_ldflags)) },
 #endif
 #ifdef _CS_XBS5_LP64_OFF64_LIBS
  { _CS_XBS5_LP64_OFF64_LIBS, &(O(object_Kxbs5_lp64_off64_libs)) },
 #endif
 #ifdef _CS_XBS5_LP64_OFF64_LINTFLAGS
  { _CS_XBS5_LP64_OFF64_LINTFLAGS, &(O(object_Kxbs5_lp64_off64_lintflags)) },
 #endif
 #ifdef _CS_XBS5_LPBIG_OFFBIG_CFLAGS
  { _CS_XBS5_LPBIG_OFFBIG_CFLAGS, &(O(object_Kxbs5_lpbig_offbig_cflags)) },
 #endif
 #ifdef _CS_XBS5_LPBIG_OFFBIG_LDFLAGS
  { _CS_XBS5_LPBIG_OFFBIG_LDFLAGS, &(O(object_Kxbs5_lpbig_offbig_ldflags)) },
 #endif
 #ifdef _CS_XBS5_LPBIG_OFFBIG_LIBS
  { _CS_XBS5_LPBIG_OFFBIG_LIBS, &(O(object_Kxbs5_lpbig_offbig_libs)) },
 #endif
 #ifdef _CS_XBS5_LPBIG_OFFBIG_LINTFLAGS
  { _CS_XBS5_LPBIG_OFFBIG_LINTFLAGS, &(O(object_Kxbs5_lpbig_offbig_lintflags)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t confstr_arg_map = {
  confstr_arg_table,
  (sizeof(confstr_arg_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "confstr_arg"
};
#define confstr_arg(a) (int)map_lisp_to_c(a,&confstr_arg_map)
#define confstr_arg_reverse(a) map_c_to_lisp(a,&confstr_arg_map)

#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
static const c_lisp_pair_t pathconf_arg_table[] = {
 #ifdef _PC_FILESIZEBITS
  { _PC_FILESIZEBITS, &(O(object_Kfilesizebits)) },
 #endif
 #ifdef _PC_LINK_MAX
  { _PC_LINK_MAX, &(O(object_Klink_max)) },
 #endif
 #ifdef _PC_MAX_CANON
  { _PC_MAX_CANON, &(O(object_Kmax_canon)) },
 #endif
 #ifdef _PC_MAX_INPUT
  { _PC_MAX_INPUT, &(O(object_Kmax_input)) },
 #endif
 #ifdef _PC_NAME_MAX
  { _PC_NAME_MAX, &(O(object_Kname_max)) },
 #endif
 #ifdef _PC_PATH_MAX
  { _PC_PATH_MAX, &(O(object_Kpath_max)) },
 #endif
 #ifdef _PC_PIPE_BUF
  { _PC_PIPE_BUF, &(O(object_Kpipe_buf)) },
 #endif
 #ifdef _PC_2_SYMLINKS
  { _PC_2_SYMLINKS, &(O(object_K2_symlinks)) },
 #endif
 #ifdef _PC_ALLOC_SIZE_MIN
  { _PC_ALLOC_SIZE_MIN, &(O(object_Kalloc_size_min)) },
 #endif
 #ifdef _PC_REC_INCR_XFER_SIZE
  { _PC_REC_INCR_XFER_SIZE, &(O(object_Krec_incr_xfer_size)) },
 #endif
 #ifdef _PC_REC_MAX_XFER_SIZE
  { _PC_REC_MAX_XFER_SIZE, &(O(object_Krec_max_xfer_size)) },
 #endif
 #ifdef _PC_REC_MIN_XFER_SIZE
  { _PC_REC_MIN_XFER_SIZE, &(O(object_Krec_min_xfer_size)) },
 #endif
 #ifdef _PC_REC_XFER_ALIGN
  { _PC_REC_XFER_ALIGN, &(O(object_Krec_xfer_align)) },
 #endif
 #ifdef _PC_SYMLINK_MAX
  { _PC_SYMLINK_MAX, &(O(object_Ksymlink_max)) },
 #endif
 #ifdef _PC_CHOWN_RESTRICTED
  { _PC_CHOWN_RESTRICTED, &(O(object_Kchown_restricted)) },
 #endif
 #ifdef _PC_NO_TRUNC
  { _PC_NO_TRUNC, &(O(object_Kno_trunc)) },
 #endif
 #ifdef _PC_VDISABLE
  { _PC_VDISABLE, &(O(object_Kvdisable)) },
 #endif
 #ifdef _PC_ASYNC_IO
  { _PC_ASYNC_IO, &(O(object_Kasync_io)) },
 #endif
 #ifdef _PC_PRIO_IO
  { _PC_PRIO_IO, &(O(object_Kprio_io)) },
 #endif
 #ifdef _PC_SYNC_IO
  { _PC_SYNC_IO, &(O(object_Ksync_io)) },
 #endif
 #ifdef _PC_SOCK_MAXBUF
  { _PC_SOCK_MAXBUF, &(O(object_Ksock_maxbuf)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t pathconf_arg_map = {
  pathconf_arg_table,
  (sizeof(pathconf_arg_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "pathconf_arg"
};
#define pathconf_arg(a) (int)map_lisp_to_c(a,&pathconf_arg_map)
#define pathconf_arg_reverse(a) map_c_to_lisp(a,&pathconf_arg_map)

#endif
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
static const c_lisp_pair_t check_rusage_table[] = {
 #ifdef RUSAGE_SELF
  { RUSAGE_SELF, &(O(object_Kself)) },
 #endif
 #ifdef RUSAGE_CHILDREN
  { RUSAGE_CHILDREN, &(O(object_Kchildren)) },
 #endif
 #ifdef RUSAGE_THREAD
  { RUSAGE_THREAD, &(O(object_Kthread)) },
 #endif
 #ifdef RUSAGE_LWP
  { RUSAGE_LWP, &(O(object_Klwp)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_rusage_map = {
  check_rusage_table,
  (sizeof(check_rusage_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_rusage"
};
#define check_rusage(a) (int)map_lisp_to_c(a,&check_rusage_map)
#define check_rusage_reverse(a) map_c_to_lisp(a,&check_rusage_map)

#endif
#if defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)
static const c_lisp_pair_t getrlimit_arg_table[] = {
 #ifdef RLIMIT_CPU
  { RLIMIT_CPU, &(O(object_Kcpu)) },
 #endif
 #ifdef RLIMIT_FSIZE
  { RLIMIT_FSIZE, &(O(object_Kfsize)) },
 #endif
 #ifdef RLIMIT_DATA
  { RLIMIT_DATA, &(O(object_Kdata)) },
 #endif
 #ifdef RLIMIT_STACK
  { RLIMIT_STACK, &(O(object_Kstack)) },
 #endif
 #ifdef RLIMIT_CORE
  { RLIMIT_CORE, &(O(object_Kcore)) },
 #endif
 #ifdef RLIMIT_RSS
  { RLIMIT_RSS, &(O(object_Krss)) },
 #endif
 #ifdef RLIMIT_NOFILE
  { RLIMIT_NOFILE, &(O(object_Knofile)) },
 #endif
 #ifdef RLIMIT_AS
  { RLIMIT_AS, &(O(object_Kas)) },
 #endif
 #ifdef RLIMIT_NPROC
  { RLIMIT_NPROC, &(O(object_Knproc)) },
 #endif
 #ifdef RLIMIT_MEMLOCK
  { RLIMIT_MEMLOCK, &(O(object_Kmemlock)) },
 #endif
 #ifdef RLIMIT_LOCKS
  { RLIMIT_LOCKS, &(O(object_Klocks)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t getrlimit_arg_map = {
  getrlimit_arg_table,
  (sizeof(getrlimit_arg_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "getrlimit_arg"
};
#define getrlimit_arg(a) (int)map_lisp_to_c(a,&getrlimit_arg_map)
#define getrlimit_arg_reverse(a) map_c_to_lisp(a,&getrlimit_arg_map)

#endif
#if defined(HAVE_NFTW)
static const c_lisp_pair_t check_ftw_kind_table[] = {
 #ifdef FTW_F
  { FTW_F, &(O(object_Kf)) },
 #endif
 #ifdef FTW_D
  { FTW_D, &(O(object_Kd)) },
 #endif
 #ifdef FTW_DP
  { FTW_DP, &(O(object_Kdp)) },
 #endif
 #ifdef FTW_SL
  { FTW_SL, &(O(object_Ksl)) },
 #endif
 #ifdef FTW_SLN
  { FTW_SLN, &(O(object_Ksln)) },
 #endif
 #ifdef FTW_DNR
  { FTW_DNR, &(O(object_Kdnr)) },
 #endif
 #ifdef FTW_NS
  { FTW_NS, &(O(object_Kns)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_ftw_kind_map = {
  check_ftw_kind_table,
  (sizeof(check_ftw_kind_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_ftw_kind"
};
#define check_ftw_kind(a) (int)map_lisp_to_c(a,&check_ftw_kind_map)
#define check_ftw_kind_reverse(a) map_c_to_lisp(a,&check_ftw_kind_map)

#endif
static const c_lisp_pair_t check_chmod_mode_table[] = {
 #ifdef S_ISUID
  { S_ISUID, &(O(object_Ksuid)) },
 #endif
 #ifdef S_ISGID
  { S_ISGID, &(O(object_Ksgid)) },
 #endif
 #ifdef S_ISVTX
  { S_ISVTX, &(O(object_Ksvtx)) },
 #endif
 #ifdef S_IRWXU
  { S_IRWXU, &(O(object_Krwxu)) },
 #endif
 #ifdef S_IRUSR
  { S_IRUSR, &(O(object_Krusr)) },
 #endif
 #ifdef S_IWUSR
  { S_IWUSR, &(O(object_Kwusr)) },
 #endif
 #ifdef S_IXUSR
  { S_IXUSR, &(O(object_Kxusr)) },
 #endif
 #ifdef S_IRWXG
  { S_IRWXG, &(O(object_Krwxg)) },
 #endif
 #ifdef S_IRGRP
  { S_IRGRP, &(O(object_Krgrp)) },
 #endif
 #ifdef S_IWGRP
  { S_IWGRP, &(O(object_Kwgrp)) },
 #endif
 #ifdef S_IXGRP
  { S_IXGRP, &(O(object_Kxgrp)) },
 #endif
 #ifdef S_IRWXO
  { S_IRWXO, &(O(object_Krwxo)) },
 #endif
 #ifdef S_IROTH
  { S_IROTH, &(O(object_Kroth)) },
 #endif
 #ifdef S_IWOTH
  { S_IWOTH, &(O(object_Kwoth)) },
 #endif
 #ifdef S_IXOTH
  { S_IXOTH, &(O(object_Kxoth)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_chmod_mode_map = {
  check_chmod_mode_table,
  (sizeof(check_chmod_mode_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_chmod_mode"
};
#define check_chmod_mode(a) (mode_t)map_lisp_to_c(a,&check_chmod_mode_map)
#define check_chmod_mode_reverse(a) map_c_to_lisp(a,&check_chmod_mode_map)
#define check_chmod_mode_to_list(a) map_c_to_list(a,&check_chmod_mode_map)
#define check_chmod_mode_of_list(a) map_list_to_c(a,&check_chmod_mode_map)

static const c_lisp_pair_t mknod_type_check_table[] = {
 #ifdef S_IFIFO
  { S_IFIFO, &(O(object_Kfifo)) },
 #endif
 #ifdef S_IFSOCK
  { S_IFSOCK, &(O(object_Kfsock)) },
 #endif
 #ifdef S_IFCHR
  { S_IFCHR, &(O(object_Kfchr)) },
 #endif
 #ifdef S_IFDIR
  { S_IFDIR, &(O(object_Kfdir)) },
 #endif
 #ifdef S_IFBLK
  { S_IFBLK, &(O(object_Kfblk)) },
 #endif
 #ifdef S_IFREG
  { S_IFREG, &(O(object_Kfreg)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t mknod_type_check_map = {
  mknod_type_check_table,
  (sizeof(mknod_type_check_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "mknod_type_check"
};
#define mknod_type_check(a) (int)map_lisp_to_c(a,&mknod_type_check_map)
#define mknod_type_check_reverse(a) map_c_to_lisp(a,&mknod_type_check_map)

#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
static const c_lisp_pair_t vfs_flags_table[] = {
 #ifdef ST_RDONLY
  { ST_RDONLY, &(O(object_Kst_rdonly)) },
 #endif
 #ifdef ST_NOSUID
  { ST_NOSUID, &(O(object_Kst_nosuid)) },
 #endif
 #ifdef ST_NOTRUNC
  { ST_NOTRUNC, &(O(object_Kst_notrunc)) },
 #endif
 #ifdef ST_NODEV
  { ST_NODEV, &(O(object_Kst_nodev)) },
 #endif
 #ifdef ST_NOEXEC
  { ST_NOEXEC, &(O(object_Kst_noexec)) },
 #endif
 #ifdef ST_SYNCHRONOUS
  { ST_SYNCHRONOUS, &(O(object_Kst_synchronous)) },
 #endif
 #ifdef ST_MANDLOCK
  { ST_MANDLOCK, &(O(object_Kst_mandlock)) },
 #endif
 #ifdef ST_WRITE
  { ST_WRITE, &(O(object_Kst_write)) },
 #endif
 #ifdef ST_APPEND
  { ST_APPEND, &(O(object_Kst_append)) },
 #endif
 #ifdef ST_IMMUTABLE
  { ST_IMMUTABLE, &(O(object_Kst_immutable)) },
 #endif
 #ifdef ST_NOATIME
  { ST_NOATIME, &(O(object_Kst_noatime)) },
 #endif
 #ifdef ST_NODIRATIME
  { ST_NODIRATIME, &(O(object_Kst_nodiratime)) },
 #endif
 #ifdef FILE_NAMED_STREAMS
  { FILE_NAMED_STREAMS, &(O(object_Kfile_named_streams)) },
 #endif
 #ifdef FILE_READ_ONLY_VOLUME
  { FILE_READ_ONLY_VOLUME, &(O(object_Kfile_read_only_volume)) },
 #endif
 #ifdef FILE_SUPPORTS_OBJECT_IDS
  { FILE_SUPPORTS_OBJECT_IDS, &(O(object_Kfile_supports_object_ids)) },
 #endif
 #ifdef FILE_SUPPORTS_REPARSE_POINTS
  { FILE_SUPPORTS_REPARSE_POINTS, &(O(object_Kfile_supports_reparse_points)) },
 #endif
 #ifdef FILE_SUPPORTS_SPARSE_FILES
  { FILE_SUPPORTS_SPARSE_FILES, &(O(object_Kfile_supports_sparse_files)) },
 #endif
 #ifdef FILE_VOLUME_QUOTAS
  { FILE_VOLUME_QUOTAS, &(O(object_Kfile_volume_quotas)) },
 #endif
 #ifdef FILE_SUPPORTS_ENCRYPTION
  { FILE_SUPPORTS_ENCRYPTION, &(O(object_Kfile_supports_encryption)) },
 #endif
 #ifdef FS_CASE_IS_PRESERVED
  { FS_CASE_IS_PRESERVED, &(O(object_Kfs_case_is_preserved)) },
 #endif
 #ifdef FS_CASE_SENSITIVE
  { FS_CASE_SENSITIVE, &(O(object_Kfs_case_sensitive)) },
 #endif
 #ifdef FS_FILE_COMPRESSION
  { FS_FILE_COMPRESSION, &(O(object_Kfs_file_compression)) },
 #endif
 #ifdef FS_FILE_ENCRYPTION
  { FS_FILE_ENCRYPTION, &(O(object_Kfs_file_encryption)) },
 #endif
 #ifdef FS_PERSISTENT_ACLS
  { FS_PERSISTENT_ACLS, &(O(object_Kfs_persistent_acls)) },
 #endif
 #ifdef FS_UNICODE_STORED_ON_DISK
  { FS_UNICODE_STORED_ON_DISK, &(O(object_Kfs_unicode_stored_on_disk)) },
 #endif
 #ifdef FS_VOL_IS_COMPRESSED
  { FS_VOL_IS_COMPRESSED, &(O(object_Kfs_vol_is_compressed)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t vfs_flags_map = {
  vfs_flags_table,
  (sizeof(vfs_flags_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "vfs_flags"
};
#define vfs_flags(a) (int)map_lisp_to_c(a,&vfs_flags_map)
#define vfs_flags_reverse(a) map_c_to_lisp(a,&vfs_flags_map)
#define vfs_flags_to_list(a) map_c_to_list(a,&vfs_flags_map)
#define vfs_flags_of_list(a) map_list_to_c(a,&vfs_flags_map)

#endif
static const c_lisp_pair_t check_copy_method_table[] = {
  { COPY_METHOD_COPY, &(O(object_Kcopy)) },
  { COPY_METHOD_SYMLINK, &(O(object_Ksymlink)) },
  { COPY_METHOD_HARDLINK, &(O(object_Khardlink)) },
  { COPY_METHOD_HARDLINK_OR_COPY, &(O(object_Khardlink_or_copy)) },
  { COPY_METHOD_RENAME, &(O(object_Krename)) },
  { 0, NULL }
};
static const c_lisp_map_t check_copy_method_map = {
  check_copy_method_table,
  (sizeof(check_copy_method_table)/sizeof(c_lisp_pair_t))-1,
  COPY_METHOD_COPY,true,
  "check_copy_method"
};
#define check_copy_method(a) (copy_method_t)map_lisp_to_c(a,&check_copy_method_map)
#define check_copy_method_reverse(a) map_c_to_lisp(a,&check_copy_method_map)

#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
static const c_lisp_pair_t check_file_attributes_table[] = {
 #ifdef FILE_ATTRIBUTE_ARCHIVE
  { FILE_ATTRIBUTE_ARCHIVE, &(O(object_Karchive)) },
 #endif
 #ifdef FILE_ATTRIBUTE_COMPRESSED
  { FILE_ATTRIBUTE_COMPRESSED, &(O(object_Kcompressed)) },
 #endif
 #ifdef FILE_ATTRIBUTE_DEVICE
  { FILE_ATTRIBUTE_DEVICE, &(O(object_Kdevice)) },
 #endif
 #ifdef FILE_ATTRIBUTE_DIRECTORY
  { FILE_ATTRIBUTE_DIRECTORY, &(O(object_Kdirectory)) },
 #endif
 #ifdef FILE_ATTRIBUTE_ENCRYPTED
  { FILE_ATTRIBUTE_ENCRYPTED, &(O(object_Kencrypted)) },
 #endif
 #ifdef FILE_ATTRIBUTE_HIDDEN
  { FILE_ATTRIBUTE_HIDDEN, &(O(object_Khidden)) },
 #endif
 #ifdef FILE_ATTRIBUTE_NORMAL
  { FILE_ATTRIBUTE_NORMAL, &(O(object_Knormal)) },
 #endif
 #ifdef FILE_ATTRIBUTE_NOT_CONTENT_INDEXED
  { FILE_ATTRIBUTE_NOT_CONTENT_INDEXED, &(O(object_Knot_content_indexed)) },
 #endif
 #ifdef FILE_ATTRIBUTE_OFFLINE
  { FILE_ATTRIBUTE_OFFLINE, &(O(object_Koffline)) },
 #endif
 #ifdef FILE_ATTRIBUTE_READONLY
  { FILE_ATTRIBUTE_READONLY, &(O(object_Kreadonly)) },
 #endif
 #ifdef FILE_ATTRIBUTE_REPARSE_POINT
  { FILE_ATTRIBUTE_REPARSE_POINT, &(O(object_Kreparse_point)) },
 #endif
 #ifdef FILE_ATTRIBUTE_SPARSE_FILE
  { FILE_ATTRIBUTE_SPARSE_FILE, &(O(object_Ksparse_file)) },
 #endif
 #ifdef FILE_ATTRIBUTE_SYSTEM
  { FILE_ATTRIBUTE_SYSTEM, &(O(object_Ksystem)) },
 #endif
 #ifdef FILE_ATTRIBUTE_TEMPORARY
  { FILE_ATTRIBUTE_TEMPORARY, &(O(object_Ktemporary)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_file_attributes_map = {
  check_file_attributes_table,
  (sizeof(check_file_attributes_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_file_attributes"
};
#define check_file_attributes(a) (DWORD)map_lisp_to_c(a,&check_file_attributes_map)
#define check_file_attributes_reverse(a) map_c_to_lisp(a,&check_file_attributes_map)
#define check_file_attributes_to_list(a) map_c_to_list(a,&check_file_attributes_map)
#define check_file_attributes_of_list(a) map_list_to_c(a,&check_file_attributes_map)

#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
static const c_lisp_pair_t processor_architecture_table[] = {
 #ifdef PROCESSOR_ARCHITECTURE_INTEL
  { PROCESSOR_ARCHITECTURE_INTEL, &(O(object_Kintel)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_MIPS
  { PROCESSOR_ARCHITECTURE_MIPS, &(O(object_Kmips)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_ALPHA
  { PROCESSOR_ARCHITECTURE_ALPHA, &(O(object_Kalpha)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_PPC
  { PROCESSOR_ARCHITECTURE_PPC, &(O(object_Kppc)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_SHX
  { PROCESSOR_ARCHITECTURE_SHX, &(O(object_Kshx)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_ARM
  { PROCESSOR_ARCHITECTURE_ARM, &(O(object_Karm)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_IA64
  { PROCESSOR_ARCHITECTURE_IA64, &(O(object_Kia64)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_ALPHA64
  { PROCESSOR_ARCHITECTURE_ALPHA64, &(O(object_Kalpha64)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_MSIL
  { PROCESSOR_ARCHITECTURE_MSIL, &(O(object_Kmsil)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_AMD64
  { PROCESSOR_ARCHITECTURE_AMD64, &(O(object_Kamd64)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_IA32_ON_WIN64
  { PROCESSOR_ARCHITECTURE_IA32_ON_WIN64, &(O(object_Kia32_on_win64)) },
 #endif
 #ifdef PROCESSOR_ARCHITECTURE_UNKNOWN
  { PROCESSOR_ARCHITECTURE_UNKNOWN, &(O(object_Kunknown)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t processor_architecture_map = {
  processor_architecture_table,
  (sizeof(processor_architecture_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "processor_architecture"
};
#define processor_architecture(a) (WORD)map_lisp_to_c(a,&processor_architecture_map)
#define processor_architecture_reverse(a) map_c_to_lisp(a,&processor_architecture_map)

#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
static const c_lisp_pair_t check_last_error_table[] = {
 #ifdef ERROR_INVALID_FUNCTION
  { ERROR_INVALID_FUNCTION, &(O(object_Kerror_invalid_function)) },
 #endif
 #ifdef ERROR_FILE_NOT_FOUND
  { ERROR_FILE_NOT_FOUND, &(O(object_Kerror_file_not_found)) },
 #endif
 #ifdef ERROR_PATH_NOT_FOUND
  { ERROR_PATH_NOT_FOUND, &(O(object_Kerror_path_not_found)) },
 #endif
 #ifdef ERROR_TOO_MANY_OPEN_FILES
  { ERROR_TOO_MANY_OPEN_FILES, &(O(object_Kerror_too_many_open_files)) },
 #endif
 #ifdef ERROR_ACCESS_DENIED
  { ERROR_ACCESS_DENIED, &(O(object_Kerror_access_denied)) },
 #endif
 #ifdef ERROR_INVALID_HANDLE
  { ERROR_INVALID_HANDLE, &(O(object_Kerror_invalid_handle)) },
 #endif
 #ifdef ERROR_ARENA_TRASHED
  { ERROR_ARENA_TRASHED, &(O(object_Kerror_arena_trashed)) },
 #endif
 #ifdef ERROR_NOT_ENOUGH_MEMORY
  { ERROR_NOT_ENOUGH_MEMORY, &(O(object_Kerror_not_enough_memory)) },
 #endif
 #ifdef ERROR_INVALID_BLOCK
  { ERROR_INVALID_BLOCK, &(O(object_Kerror_invalid_block)) },
 #endif
 #ifdef ERROR_BAD_ENVIRONMENT
  { ERROR_BAD_ENVIRONMENT, &(O(object_Kerror_bad_environment)) },
 #endif
 #ifdef ERROR_BAD_FORMAT
  { ERROR_BAD_FORMAT, &(O(object_Kerror_bad_format)) },
 #endif
 #ifdef ERROR_INVALID_ACCESS
  { ERROR_INVALID_ACCESS, &(O(object_Kerror_invalid_access)) },
 #endif
 #ifdef ERROR_INVALID_DATA
  { ERROR_INVALID_DATA, &(O(object_Kerror_invalid_data)) },
 #endif
 #ifdef ERROR_OUTOFMEMORY
  { ERROR_OUTOFMEMORY, &(O(object_Kerror_outofmemory)) },
 #endif
 #ifdef ERROR_INVALID_DRIVE
  { ERROR_INVALID_DRIVE, &(O(object_Kerror_invalid_drive)) },
 #endif
 #ifdef ERROR_CURRENT_DIRECTORY
  { ERROR_CURRENT_DIRECTORY, &(O(object_Kerror_current_directory)) },
 #endif
 #ifdef ERROR_NOT_SAME_DEVICE
  { ERROR_NOT_SAME_DEVICE, &(O(object_Kerror_not_same_device)) },
 #endif
 #ifdef ERROR_NO_MORE_FILES
  { ERROR_NO_MORE_FILES, &(O(object_Kerror_no_more_files)) },
 #endif
 #ifdef ERROR_WRITE_PROTECT
  { ERROR_WRITE_PROTECT, &(O(object_Kerror_write_protect)) },
 #endif
 #ifdef ERROR_BAD_UNIT
  { ERROR_BAD_UNIT, &(O(object_Kerror_bad_unit)) },
 #endif
 #ifdef ERROR_NOT_READY
  { ERROR_NOT_READY, &(O(object_Kerror_not_ready)) },
 #endif
 #ifdef ERROR_BAD_COMMAND
  { ERROR_BAD_COMMAND, &(O(object_Kerror_bad_command)) },
 #endif
 #ifdef ERROR_CRC
  { ERROR_CRC, &(O(object_Kerror_crc)) },
 #endif
 #ifdef ERROR_BAD_LENGTH
  { ERROR_BAD_LENGTH, &(O(object_Kerror_bad_length)) },
 #endif
 #ifdef ERROR_SEEK
  { ERROR_SEEK, &(O(object_Kerror_seek)) },
 #endif
 #ifdef ERROR_NOT_DOS_DISK
  { ERROR_NOT_DOS_DISK, &(O(object_Kerror_not_dos_disk)) },
 #endif
 #ifdef ERROR_SECTOR_NOT_FOUND
  { ERROR_SECTOR_NOT_FOUND, &(O(object_Kerror_sector_not_found)) },
 #endif
 #ifdef ERROR_OUT_OF_PAPER
  { ERROR_OUT_OF_PAPER, &(O(object_Kerror_out_of_paper)) },
 #endif
 #ifdef ERROR_WRITE_FAULT
  { ERROR_WRITE_FAULT, &(O(object_Kerror_write_fault)) },
 #endif
 #ifdef ERROR_READ_FAULT
  { ERROR_READ_FAULT, &(O(object_Kerror_read_fault)) },
 #endif
 #ifdef ERROR_GEN_FAILURE
  { ERROR_GEN_FAILURE, &(O(object_Kerror_gen_failure)) },
 #endif
 #ifdef ERROR_SHARING_VIOLATION
  { ERROR_SHARING_VIOLATION, &(O(object_Kerror_sharing_violation)) },
 #endif
 #ifdef ERROR_LOCK_VIOLATION
  { ERROR_LOCK_VIOLATION, &(O(object_Kerror_lock_violation)) },
 #endif
 #ifdef ERROR_WRONG_DISK
  { ERROR_WRONG_DISK, &(O(object_Kerror_wrong_disk)) },
 #endif
 #ifdef ERROR_SHARING_BUFFER_EXCEEDED
  { ERROR_SHARING_BUFFER_EXCEEDED, &(O(object_Kerror_sharing_buffer_exceeded)) },
 #endif
 #ifdef ERROR_HANDLE_EOF
  { ERROR_HANDLE_EOF, &(O(object_Kerror_handle_eof)) },
 #endif
 #ifdef ERROR_HANDLE_DISK_FULL
  { ERROR_HANDLE_DISK_FULL, &(O(object_Kerror_handle_disk_full)) },
 #endif
 #ifdef ERROR_NOT_SUPPORTED
  { ERROR_NOT_SUPPORTED, &(O(object_Kerror_not_supported)) },
 #endif
 #ifdef ERROR_REM_NOT_LIST
  { ERROR_REM_NOT_LIST, &(O(object_Kerror_rem_not_list)) },
 #endif
 #ifdef ERROR_DUP_NAME
  { ERROR_DUP_NAME, &(O(object_Kerror_dup_name)) },
 #endif
 #ifdef ERROR_BAD_NETPATH
  { ERROR_BAD_NETPATH, &(O(object_Kerror_bad_netpath)) },
 #endif
 #ifdef ERROR_NETWORK_BUSY
  { ERROR_NETWORK_BUSY, &(O(object_Kerror_network_busy)) },
 #endif
 #ifdef ERROR_DEV_NOT_EXIST
  { ERROR_DEV_NOT_EXIST, &(O(object_Kerror_dev_not_exist)) },
 #endif
 #ifdef ERROR_TOO_MANY_CMDS
  { ERROR_TOO_MANY_CMDS, &(O(object_Kerror_too_many_cmds)) },
 #endif
 #ifdef ERROR_ADAP_HDW_ERR
  { ERROR_ADAP_HDW_ERR, &(O(object_Kerror_adap_hdw_err)) },
 #endif
 #ifdef ERROR_BAD_NET_RESP
  { ERROR_BAD_NET_RESP, &(O(object_Kerror_bad_net_resp)) },
 #endif
 #ifdef ERROR_UNEXP_NET_ERR
  { ERROR_UNEXP_NET_ERR, &(O(object_Kerror_unexp_net_err)) },
 #endif
 #ifdef ERROR_BAD_REM_ADAP
  { ERROR_BAD_REM_ADAP, &(O(object_Kerror_bad_rem_adap)) },
 #endif
 #ifdef ERROR_PRINTQ_FULL
  { ERROR_PRINTQ_FULL, &(O(object_Kerror_printq_full)) },
 #endif
 #ifdef ERROR_NO_SPOOL_SPACE
  { ERROR_NO_SPOOL_SPACE, &(O(object_Kerror_no_spool_space)) },
 #endif
 #ifdef ERROR_PRINT_CANCELLED
  { ERROR_PRINT_CANCELLED, &(O(object_Kerror_print_cancelled)) },
 #endif
 #ifdef ERROR_NETNAME_DELETED
  { ERROR_NETNAME_DELETED, &(O(object_Kerror_netname_deleted)) },
 #endif
 #ifdef ERROR_NETWORK_ACCESS_DENIED
  { ERROR_NETWORK_ACCESS_DENIED, &(O(object_Kerror_network_access_denied)) },
 #endif
 #ifdef ERROR_BAD_DEV_TYPE
  { ERROR_BAD_DEV_TYPE, &(O(object_Kerror_bad_dev_type)) },
 #endif
 #ifdef ERROR_BAD_NET_NAME
  { ERROR_BAD_NET_NAME, &(O(object_Kerror_bad_net_name)) },
 #endif
 #ifdef ERROR_TOO_MANY_NAMES
  { ERROR_TOO_MANY_NAMES, &(O(object_Kerror_too_many_names)) },
 #endif
 #ifdef ERROR_TOO_MANY_SESS
  { ERROR_TOO_MANY_SESS, &(O(object_Kerror_too_many_sess)) },
 #endif
 #ifdef ERROR_SHARING_PAUSED
  { ERROR_SHARING_PAUSED, &(O(object_Kerror_sharing_paused)) },
 #endif
 #ifdef ERROR_REQ_NOT_ACCEP
  { ERROR_REQ_NOT_ACCEP, &(O(object_Kerror_req_not_accep)) },
 #endif
 #ifdef ERROR_REDIR_PAUSED
  { ERROR_REDIR_PAUSED, &(O(object_Kerror_redir_paused)) },
 #endif
 #ifdef ERROR_FILE_EXISTS
  { ERROR_FILE_EXISTS, &(O(object_Kerror_file_exists)) },
 #endif
 #ifdef ERROR_CANNOT_MAKE
  { ERROR_CANNOT_MAKE, &(O(object_Kerror_cannot_make)) },
 #endif
 #ifdef ERROR_FAIL_I24
  { ERROR_FAIL_I24, &(O(object_Kerror_fail_i24)) },
 #endif
 #ifdef ERROR_OUT_OF_STRUCTURES
  { ERROR_OUT_OF_STRUCTURES, &(O(object_Kerror_out_of_structures)) },
 #endif
 #ifdef ERROR_ALREADY_ASSIGNED
  { ERROR_ALREADY_ASSIGNED, &(O(object_Kerror_already_assigned)) },
 #endif
 #ifdef ERROR_INVALID_PASSWORD
  { ERROR_INVALID_PASSWORD, &(O(object_Kerror_invalid_password)) },
 #endif
 #ifdef ERROR_INVALID_PARAMETER
  { ERROR_INVALID_PARAMETER, &(O(object_Kerror_invalid_parameter)) },
 #endif
 #ifdef ERROR_NET_WRITE_FAULT
  { ERROR_NET_WRITE_FAULT, &(O(object_Kerror_net_write_fault)) },
 #endif
 #ifdef ERROR_NO_PROC_SLOTS
  { ERROR_NO_PROC_SLOTS, &(O(object_Kerror_no_proc_slots)) },
 #endif
 #ifdef ERROR_TOO_MANY_SEMAPHORES
  { ERROR_TOO_MANY_SEMAPHORES, &(O(object_Kerror_too_many_semaphores)) },
 #endif
 #ifdef ERROR_EXCL_SEM_ALREADY_OWNED
  { ERROR_EXCL_SEM_ALREADY_OWNED, &(O(object_Kerror_excl_sem_already_owned)) },
 #endif
 #ifdef ERROR_SEM_IS_SET
  { ERROR_SEM_IS_SET, &(O(object_Kerror_sem_is_set)) },
 #endif
 #ifdef ERROR_TOO_MANY_SEM_REQUESTS
  { ERROR_TOO_MANY_SEM_REQUESTS, &(O(object_Kerror_too_many_sem_requests)) },
 #endif
 #ifdef ERROR_INVALID_AT_INTERRUPT_TIME
  { ERROR_INVALID_AT_INTERRUPT_TIME, &(O(object_Kerror_invalid_at_interrupt_time)) },
 #endif
 #ifdef ERROR_SEM_OWNER_DIED
  { ERROR_SEM_OWNER_DIED, &(O(object_Kerror_sem_owner_died)) },
 #endif
 #ifdef ERROR_SEM_USER_LIMIT
  { ERROR_SEM_USER_LIMIT, &(O(object_Kerror_sem_user_limit)) },
 #endif
 #ifdef ERROR_DISK_CHANGE
  { ERROR_DISK_CHANGE, &(O(object_Kerror_disk_change)) },
 #endif
 #ifdef ERROR_DRIVE_LOCKED
  { ERROR_DRIVE_LOCKED, &(O(object_Kerror_drive_locked)) },
 #endif
 #ifdef ERROR_BROKEN_PIPE
  { ERROR_BROKEN_PIPE, &(O(object_Kerror_broken_pipe)) },
 #endif
 #ifdef ERROR_OPEN_FAILED
  { ERROR_OPEN_FAILED, &(O(object_Kerror_open_failed)) },
 #endif
 #ifdef ERROR_BUFFER_OVERFLOW
  { ERROR_BUFFER_OVERFLOW, &(O(object_Kerror_buffer_overflow)) },
 #endif
 #ifdef ERROR_DISK_FULL
  { ERROR_DISK_FULL, &(O(object_Kerror_disk_full)) },
 #endif
 #ifdef ERROR_NO_MORE_SEARCH_HANDLES
  { ERROR_NO_MORE_SEARCH_HANDLES, &(O(object_Kerror_no_more_search_handles)) },
 #endif
 #ifdef ERROR_INVALID_TARGET_HANDLE
  { ERROR_INVALID_TARGET_HANDLE, &(O(object_Kerror_invalid_target_handle)) },
 #endif
 #ifdef ERROR_INVALID_CATEGORY
  { ERROR_INVALID_CATEGORY, &(O(object_Kerror_invalid_category)) },
 #endif
 #ifdef ERROR_INVALID_VERIFY_SWITCH
  { ERROR_INVALID_VERIFY_SWITCH, &(O(object_Kerror_invalid_verify_switch)) },
 #endif
 #ifdef ERROR_BAD_DRIVER_LEVEL
  { ERROR_BAD_DRIVER_LEVEL, &(O(object_Kerror_bad_driver_level)) },
 #endif
 #ifdef ERROR_CALL_NOT_IMPLEMENTED
  { ERROR_CALL_NOT_IMPLEMENTED, &(O(object_Kerror_call_not_implemented)) },
 #endif
 #ifdef ERROR_SEM_TIMEOUT
  { ERROR_SEM_TIMEOUT, &(O(object_Kerror_sem_timeout)) },
 #endif
 #ifdef ERROR_INSUFFICIENT_BUFFER
  { ERROR_INSUFFICIENT_BUFFER, &(O(object_Kerror_insufficient_buffer)) },
 #endif
 #ifdef ERROR_INVALID_NAME
  { ERROR_INVALID_NAME, &(O(object_Kerror_invalid_name)) },
 #endif
 #ifdef ERROR_INVALID_LEVEL
  { ERROR_INVALID_LEVEL, &(O(object_Kerror_invalid_level)) },
 #endif
 #ifdef ERROR_NO_VOLUME_LABEL
  { ERROR_NO_VOLUME_LABEL, &(O(object_Kerror_no_volume_label)) },
 #endif
 #ifdef ERROR_MOD_NOT_FOUND
  { ERROR_MOD_NOT_FOUND, &(O(object_Kerror_mod_not_found)) },
 #endif
 #ifdef ERROR_PROC_NOT_FOUND
  { ERROR_PROC_NOT_FOUND, &(O(object_Kerror_proc_not_found)) },
 #endif
 #ifdef ERROR_WAIT_NO_CHILDREN
  { ERROR_WAIT_NO_CHILDREN, &(O(object_Kerror_wait_no_children)) },
 #endif
 #ifdef ERROR_CHILD_NOT_COMPLETE
  { ERROR_CHILD_NOT_COMPLETE, &(O(object_Kerror_child_not_complete)) },
 #endif
 #ifdef ERROR_DIRECT_ACCESS_HANDLE
  { ERROR_DIRECT_ACCESS_HANDLE, &(O(object_Kerror_direct_access_handle)) },
 #endif
 #ifdef ERROR_NEGATIVE_SEEK
  { ERROR_NEGATIVE_SEEK, &(O(object_Kerror_negative_seek)) },
 #endif
 #ifdef ERROR_SEEK_ON_DEVICE
  { ERROR_SEEK_ON_DEVICE, &(O(object_Kerror_seek_on_device)) },
 #endif
 #ifdef ERROR_IS_JOIN_TARGET
  { ERROR_IS_JOIN_TARGET, &(O(object_Kerror_is_join_target)) },
 #endif
 #ifdef ERROR_IS_JOINED
  { ERROR_IS_JOINED, &(O(object_Kerror_is_joined)) },
 #endif
 #ifdef ERROR_IS_SUBSTED
  { ERROR_IS_SUBSTED, &(O(object_Kerror_is_substed)) },
 #endif
 #ifdef ERROR_NOT_JOINED
  { ERROR_NOT_JOINED, &(O(object_Kerror_not_joined)) },
 #endif
 #ifdef ERROR_NOT_SUBSTED
  { ERROR_NOT_SUBSTED, &(O(object_Kerror_not_substed)) },
 #endif
 #ifdef ERROR_JOIN_TO_JOIN
  { ERROR_JOIN_TO_JOIN, &(O(object_Kerror_join_to_join)) },
 #endif
 #ifdef ERROR_SUBST_TO_SUBST
  { ERROR_SUBST_TO_SUBST, &(O(object_Kerror_subst_to_subst)) },
 #endif
 #ifdef ERROR_JOIN_TO_SUBST
  { ERROR_JOIN_TO_SUBST, &(O(object_Kerror_join_to_subst)) },
 #endif
 #ifdef ERROR_SUBST_TO_JOIN
  { ERROR_SUBST_TO_JOIN, &(O(object_Kerror_subst_to_join)) },
 #endif
 #ifdef ERROR_BUSY_DRIVE
  { ERROR_BUSY_DRIVE, &(O(object_Kerror_busy_drive)) },
 #endif
 #ifdef ERROR_SAME_DRIVE
  { ERROR_SAME_DRIVE, &(O(object_Kerror_same_drive)) },
 #endif
 #ifdef ERROR_DIR_NOT_ROOT
  { ERROR_DIR_NOT_ROOT, &(O(object_Kerror_dir_not_root)) },
 #endif
 #ifdef ERROR_DIR_NOT_EMPTY
  { ERROR_DIR_NOT_EMPTY, &(O(object_Kerror_dir_not_empty)) },
 #endif
 #ifdef ERROR_IS_SUBST_PATH
  { ERROR_IS_SUBST_PATH, &(O(object_Kerror_is_subst_path)) },
 #endif
 #ifdef ERROR_IS_JOIN_PATH
  { ERROR_IS_JOIN_PATH, &(O(object_Kerror_is_join_path)) },
 #endif
 #ifdef ERROR_PATH_BUSY
  { ERROR_PATH_BUSY, &(O(object_Kerror_path_busy)) },
 #endif
 #ifdef ERROR_IS_SUBST_TARGET
  { ERROR_IS_SUBST_TARGET, &(O(object_Kerror_is_subst_target)) },
 #endif
 #ifdef ERROR_SYSTEM_TRACE
  { ERROR_SYSTEM_TRACE, &(O(object_Kerror_system_trace)) },
 #endif
 #ifdef ERROR_INVALID_EVENT_COUNT
  { ERROR_INVALID_EVENT_COUNT, &(O(object_Kerror_invalid_event_count)) },
 #endif
 #ifdef ERROR_TOO_MANY_MUXWAITERS
  { ERROR_TOO_MANY_MUXWAITERS, &(O(object_Kerror_too_many_muxwaiters)) },
 #endif
 #ifdef ERROR_INVALID_LIST_FORMAT
  { ERROR_INVALID_LIST_FORMAT, &(O(object_Kerror_invalid_list_format)) },
 #endif
 #ifdef ERROR_LABEL_TOO_LONG
  { ERROR_LABEL_TOO_LONG, &(O(object_Kerror_label_too_long)) },
 #endif
 #ifdef ERROR_TOO_MANY_TCBS
  { ERROR_TOO_MANY_TCBS, &(O(object_Kerror_too_many_tcbs)) },
 #endif
 #ifdef ERROR_SIGNAL_REFUSED
  { ERROR_SIGNAL_REFUSED, &(O(object_Kerror_signal_refused)) },
 #endif
 #ifdef ERROR_DISCARDED
  { ERROR_DISCARDED, &(O(object_Kerror_discarded)) },
 #endif
 #ifdef ERROR_NOT_LOCKED
  { ERROR_NOT_LOCKED, &(O(object_Kerror_not_locked)) },
 #endif
 #ifdef ERROR_BAD_THREADID_ADDR
  { ERROR_BAD_THREADID_ADDR, &(O(object_Kerror_bad_threadid_addr)) },
 #endif
 #ifdef ERROR_BAD_ARGUMENTS
  { ERROR_BAD_ARGUMENTS, &(O(object_Kerror_bad_arguments)) },
 #endif
 #ifdef ERROR_BAD_PATHNAME
  { ERROR_BAD_PATHNAME, &(O(object_Kerror_bad_pathname)) },
 #endif
 #ifdef ERROR_SIGNAL_PENDING
  { ERROR_SIGNAL_PENDING, &(O(object_Kerror_signal_pending)) },
 #endif
 #ifdef ERROR_MAX_THRDS_REACHED
  { ERROR_MAX_THRDS_REACHED, &(O(object_Kerror_max_thrds_reached)) },
 #endif
 #ifdef ERROR_LOCK_FAILED
  { ERROR_LOCK_FAILED, &(O(object_Kerror_lock_failed)) },
 #endif
 #ifdef ERROR_BUSY
  { ERROR_BUSY, &(O(object_Kerror_busy)) },
 #endif
 #ifdef ERROR_CANCEL_VIOLATION
  { ERROR_CANCEL_VIOLATION, &(O(object_Kerror_cancel_violation)) },
 #endif
 #ifdef ERROR_ATOMIC_LOCKS_NOT_SUPPORTED
  { ERROR_ATOMIC_LOCKS_NOT_SUPPORTED, &(O(object_Kerror_atomic_locks_not_supported)) },
 #endif
 #ifdef ERROR_INVALID_SEGMENT_NUMBER
  { ERROR_INVALID_SEGMENT_NUMBER, &(O(object_Kerror_invalid_segment_number)) },
 #endif
 #ifdef ERROR_INVALID_ORDINAL
  { ERROR_INVALID_ORDINAL, &(O(object_Kerror_invalid_ordinal)) },
 #endif
 #ifdef ERROR_ALREADY_EXISTS
  { ERROR_ALREADY_EXISTS, &(O(object_Kerror_already_exists)) },
 #endif
 #ifdef ERROR_INVALID_FLAG_NUMBER
  { ERROR_INVALID_FLAG_NUMBER, &(O(object_Kerror_invalid_flag_number)) },
 #endif
 #ifdef ERROR_SEM_NOT_FOUND
  { ERROR_SEM_NOT_FOUND, &(O(object_Kerror_sem_not_found)) },
 #endif
 #ifdef ERROR_INVALID_STARTING_CODESEG
  { ERROR_INVALID_STARTING_CODESEG, &(O(object_Kerror_invalid_starting_codeseg)) },
 #endif
 #ifdef ERROR_INVALID_STACKSEG
  { ERROR_INVALID_STACKSEG, &(O(object_Kerror_invalid_stackseg)) },
 #endif
 #ifdef ERROR_INVALID_MODULETYPE
  { ERROR_INVALID_MODULETYPE, &(O(object_Kerror_invalid_moduletype)) },
 #endif
 #ifdef ERROR_INVALID_EXE_SIGNATURE
  { ERROR_INVALID_EXE_SIGNATURE, &(O(object_Kerror_invalid_exe_signature)) },
 #endif
 #ifdef ERROR_EXE_MARKED_INVALID
  { ERROR_EXE_MARKED_INVALID, &(O(object_Kerror_exe_marked_invalid)) },
 #endif
 #ifdef ERROR_BAD_EXE_FORMAT
  { ERROR_BAD_EXE_FORMAT, &(O(object_Kerror_bad_exe_format)) },
 #endif
 #ifdef ERROR_ITERATED_DATA_EXCEEDS_64k
  { ERROR_ITERATED_DATA_EXCEEDS_64k, &(O(object_Kerror_iterated_data_exceeds_64k)) },
 #endif
 #ifdef ERROR_INVALID_MINALLOCSIZE
  { ERROR_INVALID_MINALLOCSIZE, &(O(object_Kerror_invalid_minallocsize)) },
 #endif
 #ifdef ERROR_DYNLINK_FROM_INVALID_RING
  { ERROR_DYNLINK_FROM_INVALID_RING, &(O(object_Kerror_dynlink_from_invalid_ring)) },
 #endif
 #ifdef ERROR_IOPL_NOT_ENABLED
  { ERROR_IOPL_NOT_ENABLED, &(O(object_Kerror_iopl_not_enabled)) },
 #endif
 #ifdef ERROR_INVALID_SEGDPL
  { ERROR_INVALID_SEGDPL, &(O(object_Kerror_invalid_segdpl)) },
 #endif
 #ifdef ERROR_AUTODATASEG_EXCEEDS_64k
  { ERROR_AUTODATASEG_EXCEEDS_64k, &(O(object_Kerror_autodataseg_exceeds_64k)) },
 #endif
 #ifdef ERROR_RING2SEG_MUST_BE_MOVABLE
  { ERROR_RING2SEG_MUST_BE_MOVABLE, &(O(object_Kerror_ring2seg_must_be_movable)) },
 #endif
 #ifdef ERROR_RELOC_CHAIN_XEEDS_SEGLIM
  { ERROR_RELOC_CHAIN_XEEDS_SEGLIM, &(O(object_Kerror_reloc_chain_xeeds_seglim)) },
 #endif
 #ifdef ERROR_INFLOOP_IN_RELOC_CHAIN
  { ERROR_INFLOOP_IN_RELOC_CHAIN, &(O(object_Kerror_infloop_in_reloc_chain)) },
 #endif
 #ifdef ERROR_ENVVAR_NOT_FOUND
  { ERROR_ENVVAR_NOT_FOUND, &(O(object_Kerror_envvar_not_found)) },
 #endif
 #ifdef ERROR_NO_SIGNAL_SENT
  { ERROR_NO_SIGNAL_SENT, &(O(object_Kerror_no_signal_sent)) },
 #endif
 #ifdef ERROR_FILENAME_EXCED_RANGE
  { ERROR_FILENAME_EXCED_RANGE, &(O(object_Kerror_filename_exced_range)) },
 #endif
 #ifdef ERROR_RING2_STACK_IN_USE
  { ERROR_RING2_STACK_IN_USE, &(O(object_Kerror_ring2_stack_in_use)) },
 #endif
 #ifdef ERROR_META_EXPANSION_TOO_LONG
  { ERROR_META_EXPANSION_TOO_LONG, &(O(object_Kerror_meta_expansion_too_long)) },
 #endif
 #ifdef ERROR_INVALID_SIGNAL_NUMBER
  { ERROR_INVALID_SIGNAL_NUMBER, &(O(object_Kerror_invalid_signal_number)) },
 #endif
 #ifdef ERROR_THREAD_1_INACTIVE
  { ERROR_THREAD_1_INACTIVE, &(O(object_Kerror_thread_1_inactive)) },
 #endif
 #ifdef ERROR_LOCKED
  { ERROR_LOCKED, &(O(object_Kerror_locked)) },
 #endif
 #ifdef ERROR_TOO_MANY_MODULES
  { ERROR_TOO_MANY_MODULES, &(O(object_Kerror_too_many_modules)) },
 #endif
 #ifdef ERROR_NESTING_NOT_ALLOWED
  { ERROR_NESTING_NOT_ALLOWED, &(O(object_Kerror_nesting_not_allowed)) },
 #endif
 #ifdef ERROR_EXE_MACHINE_TYPE_MISMATCH
  { ERROR_EXE_MACHINE_TYPE_MISMATCH, &(O(object_Kerror_exe_machine_type_mismatch)) },
 #endif
 #ifdef ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY
  { ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY, &(O(object_Kerror_exe_cannot_modify_signed_binary)) },
 #endif
 #ifdef ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY
  { ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY, &(O(object_Kerror_exe_cannot_modify_strong_signed_binary)) },
 #endif
 #ifdef ERROR_BAD_PIPE
  { ERROR_BAD_PIPE, &(O(object_Kerror_bad_pipe)) },
 #endif
 #ifdef ERROR_PIPE_BUSY
  { ERROR_PIPE_BUSY, &(O(object_Kerror_pipe_busy)) },
 #endif
 #ifdef ERROR_NO_DATA
  { ERROR_NO_DATA, &(O(object_Kerror_no_data)) },
 #endif
 #ifdef ERROR_PIPE_NOT_CONNECTED
  { ERROR_PIPE_NOT_CONNECTED, &(O(object_Kerror_pipe_not_connected)) },
 #endif
 #ifdef ERROR_MORE_DATA
  { ERROR_MORE_DATA, &(O(object_Kerror_more_data)) },
 #endif
 #ifdef ERROR_VC_DISCONNECTED
  { ERROR_VC_DISCONNECTED, &(O(object_Kerror_vc_disconnected)) },
 #endif
 #ifdef ERROR_INVALID_EA_NAME
  { ERROR_INVALID_EA_NAME, &(O(object_Kerror_invalid_ea_name)) },
 #endif
 #ifdef ERROR_EA_LIST_INCONSISTENT
  { ERROR_EA_LIST_INCONSISTENT, &(O(object_Kerror_ea_list_inconsistent)) },
 #endif
 #ifdef WAIT_TIMEOUT
  { WAIT_TIMEOUT, &(O(object_Kwait_timeout)) },
 #endif
 #ifdef ERROR_NO_MORE_ITEMS
  { ERROR_NO_MORE_ITEMS, &(O(object_Kerror_no_more_items)) },
 #endif
 #ifdef ERROR_CANNOT_COPY
  { ERROR_CANNOT_COPY, &(O(object_Kerror_cannot_copy)) },
 #endif
 #ifdef ERROR_DIRECTORY
  { ERROR_DIRECTORY, &(O(object_Kerror_directory)) },
 #endif
 #ifdef ERROR_EAS_DIDNT_FIT
  { ERROR_EAS_DIDNT_FIT, &(O(object_Kerror_eas_didnt_fit)) },
 #endif
 #ifdef ERROR_EA_FILE_CORRUPT
  { ERROR_EA_FILE_CORRUPT, &(O(object_Kerror_ea_file_corrupt)) },
 #endif
 #ifdef ERROR_EA_TABLE_FULL
  { ERROR_EA_TABLE_FULL, &(O(object_Kerror_ea_table_full)) },
 #endif
 #ifdef ERROR_INVALID_EA_HANDLE
  { ERROR_INVALID_EA_HANDLE, &(O(object_Kerror_invalid_ea_handle)) },
 #endif
 #ifdef ERROR_EAS_NOT_SUPPORTED
  { ERROR_EAS_NOT_SUPPORTED, &(O(object_Kerror_eas_not_supported)) },
 #endif
 #ifdef ERROR_NOT_OWNER
  { ERROR_NOT_OWNER, &(O(object_Kerror_not_owner)) },
 #endif
 #ifdef ERROR_TOO_MANY_POSTS
  { ERROR_TOO_MANY_POSTS, &(O(object_Kerror_too_many_posts)) },
 #endif
 #ifdef ERROR_PARTIAL_COPY
  { ERROR_PARTIAL_COPY, &(O(object_Kerror_partial_copy)) },
 #endif
 #ifdef ERROR_OPLOCK_NOT_GRANTED
  { ERROR_OPLOCK_NOT_GRANTED, &(O(object_Kerror_oplock_not_granted)) },
 #endif
 #ifdef ERROR_INVALID_OPLOCK_PROTOCOL
  { ERROR_INVALID_OPLOCK_PROTOCOL, &(O(object_Kerror_invalid_oplock_protocol)) },
 #endif
 #ifdef ERROR_DISK_TOO_FRAGMENTED
  { ERROR_DISK_TOO_FRAGMENTED, &(O(object_Kerror_disk_too_fragmented)) },
 #endif
 #ifdef ERROR_DELETE_PENDING
  { ERROR_DELETE_PENDING, &(O(object_Kerror_delete_pending)) },
 #endif
 #ifdef ERROR_MR_MID_NOT_FOUND
  { ERROR_MR_MID_NOT_FOUND, &(O(object_Kerror_mr_mid_not_found)) },
 #endif
 #ifdef ERROR_SCOPE_NOT_FOUND
  { ERROR_SCOPE_NOT_FOUND, &(O(object_Kerror_scope_not_found)) },
 #endif
 #ifdef ERROR_INVALID_ADDRESS
  { ERROR_INVALID_ADDRESS, &(O(object_Kerror_invalid_address)) },
 #endif
 #ifdef ERROR_ARITHMETIC_OVERFLOW
  { ERROR_ARITHMETIC_OVERFLOW, &(O(object_Kerror_arithmetic_overflow)) },
 #endif
 #ifdef ERROR_PIPE_CONNECTED
  { ERROR_PIPE_CONNECTED, &(O(object_Kerror_pipe_connected)) },
 #endif
 #ifdef ERROR_PIPE_LISTENING
  { ERROR_PIPE_LISTENING, &(O(object_Kerror_pipe_listening)) },
 #endif
 #ifdef ERROR_EA_ACCESS_DENIED
  { ERROR_EA_ACCESS_DENIED, &(O(object_Kerror_ea_access_denied)) },
 #endif
 #ifdef ERROR_OPERATION_ABORTED
  { ERROR_OPERATION_ABORTED, &(O(object_Kerror_operation_aborted)) },
 #endif
 #ifdef ERROR_IO_INCOMPLETE
  { ERROR_IO_INCOMPLETE, &(O(object_Kerror_io_incomplete)) },
 #endif
 #ifdef ERROR_IO_PENDING
  { ERROR_IO_PENDING, &(O(object_Kerror_io_pending)) },
 #endif
 #ifdef ERROR_NOACCESS
  { ERROR_NOACCESS, &(O(object_Kerror_noaccess)) },
 #endif
 #ifdef ERROR_SWAPERROR
  { ERROR_SWAPERROR, &(O(object_Kerror_swaperror)) },
 #endif
 #ifdef ERROR_STACK_OVERFLOW
  { ERROR_STACK_OVERFLOW, &(O(object_Kerror_stack_overflow)) },
 #endif
 #ifdef ERROR_INVALID_MESSAGE
  { ERROR_INVALID_MESSAGE, &(O(object_Kerror_invalid_message)) },
 #endif
 #ifdef ERROR_CAN_NOT_COMPLETE
  { ERROR_CAN_NOT_COMPLETE, &(O(object_Kerror_can_not_complete)) },
 #endif
 #ifdef ERROR_INVALID_FLAGS
  { ERROR_INVALID_FLAGS, &(O(object_Kerror_invalid_flags)) },
 #endif
 #ifdef ERROR_UNRECOGNIZED_VOLUME
  { ERROR_UNRECOGNIZED_VOLUME, &(O(object_Kerror_unrecognized_volume)) },
 #endif
 #ifdef ERROR_FILE_INVALID
  { ERROR_FILE_INVALID, &(O(object_Kerror_file_invalid)) },
 #endif
 #ifdef ERROR_FULLSCREEN_MODE
  { ERROR_FULLSCREEN_MODE, &(O(object_Kerror_fullscreen_mode)) },
 #endif
 #ifdef ERROR_NO_TOKEN
  { ERROR_NO_TOKEN, &(O(object_Kerror_no_token)) },
 #endif
 #ifdef ERROR_BADDB
  { ERROR_BADDB, &(O(object_Kerror_baddb)) },
 #endif
 #ifdef ERROR_BADKEY
  { ERROR_BADKEY, &(O(object_Kerror_badkey)) },
 #endif
 #ifdef ERROR_CANTOPEN
  { ERROR_CANTOPEN, &(O(object_Kerror_cantopen)) },
 #endif
 #ifdef ERROR_CANTREAD
  { ERROR_CANTREAD, &(O(object_Kerror_cantread)) },
 #endif
 #ifdef ERROR_CANTWRITE
  { ERROR_CANTWRITE, &(O(object_Kerror_cantwrite)) },
 #endif
 #ifdef ERROR_REGISTRY_RECOVERED
  { ERROR_REGISTRY_RECOVERED, &(O(object_Kerror_registry_recovered)) },
 #endif
 #ifdef ERROR_REGISTRY_CORRUPT
  { ERROR_REGISTRY_CORRUPT, &(O(object_Kerror_registry_corrupt)) },
 #endif
 #ifdef ERROR_REGISTRY_IO_FAILED
  { ERROR_REGISTRY_IO_FAILED, &(O(object_Kerror_registry_io_failed)) },
 #endif
 #ifdef ERROR_NOT_REGISTRY_FILE
  { ERROR_NOT_REGISTRY_FILE, &(O(object_Kerror_not_registry_file)) },
 #endif
 #ifdef ERROR_KEY_DELETED
  { ERROR_KEY_DELETED, &(O(object_Kerror_key_deleted)) },
 #endif
 #ifdef ERROR_NO_LOG_SPACE
  { ERROR_NO_LOG_SPACE, &(O(object_Kerror_no_log_space)) },
 #endif
 #ifdef ERROR_KEY_HAS_CHILDREN
  { ERROR_KEY_HAS_CHILDREN, &(O(object_Kerror_key_has_children)) },
 #endif
 #ifdef ERROR_CHILD_MUST_BE_VOLATILE
  { ERROR_CHILD_MUST_BE_VOLATILE, &(O(object_Kerror_child_must_be_volatile)) },
 #endif
 #ifdef ERROR_NOTIFY_ENUM_DIR
  { ERROR_NOTIFY_ENUM_DIR, &(O(object_Kerror_notify_enum_dir)) },
 #endif
 #ifdef ERROR_DEPENDENT_SERVICES_RUNNING
  { ERROR_DEPENDENT_SERVICES_RUNNING, &(O(object_Kerror_dependent_services_running)) },
 #endif
 #ifdef ERROR_INVALID_SERVICE_CONTROL
  { ERROR_INVALID_SERVICE_CONTROL, &(O(object_Kerror_invalid_service_control)) },
 #endif
 #ifdef ERROR_SERVICE_REQUEST_TIMEOUT
  { ERROR_SERVICE_REQUEST_TIMEOUT, &(O(object_Kerror_service_request_timeout)) },
 #endif
 #ifdef ERROR_SERVICE_NO_THREAD
  { ERROR_SERVICE_NO_THREAD, &(O(object_Kerror_service_no_thread)) },
 #endif
 #ifdef ERROR_SERVICE_DATABASE_LOCKED
  { ERROR_SERVICE_DATABASE_LOCKED, &(O(object_Kerror_service_database_locked)) },
 #endif
 #ifdef ERROR_SERVICE_ALREADY_RUNNING
  { ERROR_SERVICE_ALREADY_RUNNING, &(O(object_Kerror_service_already_running)) },
 #endif
 #ifdef ERROR_INVALID_SERVICE_ACCOUNT
  { ERROR_INVALID_SERVICE_ACCOUNT, &(O(object_Kerror_invalid_service_account)) },
 #endif
 #ifdef ERROR_SERVICE_DISABLED
  { ERROR_SERVICE_DISABLED, &(O(object_Kerror_service_disabled)) },
 #endif
 #ifdef ERROR_CIRCULAR_DEPENDENCY
  { ERROR_CIRCULAR_DEPENDENCY, &(O(object_Kerror_circular_dependency)) },
 #endif
 #ifdef ERROR_SERVICE_DOES_NOT_EXIST
  { ERROR_SERVICE_DOES_NOT_EXIST, &(O(object_Kerror_service_does_not_exist)) },
 #endif
 #ifdef ERROR_SERVICE_CANNOT_ACCEPT_CTRL
  { ERROR_SERVICE_CANNOT_ACCEPT_CTRL, &(O(object_Kerror_service_cannot_accept_ctrl)) },
 #endif
 #ifdef ERROR_SERVICE_NOT_ACTIVE
  { ERROR_SERVICE_NOT_ACTIVE, &(O(object_Kerror_service_not_active)) },
 #endif
 #ifdef ERROR_FAILED_SERVICE_CONTROLLER_CONNECT
  { ERROR_FAILED_SERVICE_CONTROLLER_CONNECT, &(O(object_Kerror_failed_service_controller_connect)) },
 #endif
 #ifdef ERROR_EXCEPTION_IN_SERVICE
  { ERROR_EXCEPTION_IN_SERVICE, &(O(object_Kerror_exception_in_service)) },
 #endif
 #ifdef ERROR_DATABASE_DOES_NOT_EXIST
  { ERROR_DATABASE_DOES_NOT_EXIST, &(O(object_Kerror_database_does_not_exist)) },
 #endif
 #ifdef ERROR_SERVICE_SPECIFIC_ERROR
  { ERROR_SERVICE_SPECIFIC_ERROR, &(O(object_Kerror_service_specific_error)) },
 #endif
 #ifdef ERROR_PROCESS_ABORTED
  { ERROR_PROCESS_ABORTED, &(O(object_Kerror_process_aborted)) },
 #endif
 #ifdef ERROR_SERVICE_DEPENDENCY_FAIL
  { ERROR_SERVICE_DEPENDENCY_FAIL, &(O(object_Kerror_service_dependency_fail)) },
 #endif
 #ifdef ERROR_SERVICE_LOGON_FAILED
  { ERROR_SERVICE_LOGON_FAILED, &(O(object_Kerror_service_logon_failed)) },
 #endif
 #ifdef ERROR_SERVICE_START_HANG
  { ERROR_SERVICE_START_HANG, &(O(object_Kerror_service_start_hang)) },
 #endif
 #ifdef ERROR_INVALID_SERVICE_LOCK
  { ERROR_INVALID_SERVICE_LOCK, &(O(object_Kerror_invalid_service_lock)) },
 #endif
 #ifdef ERROR_SERVICE_MARKED_FOR_DELETE
  { ERROR_SERVICE_MARKED_FOR_DELETE, &(O(object_Kerror_service_marked_for_delete)) },
 #endif
 #ifdef ERROR_SERVICE_EXISTS
  { ERROR_SERVICE_EXISTS, &(O(object_Kerror_service_exists)) },
 #endif
 #ifdef ERROR_ALREADY_RUNNING_LKG
  { ERROR_ALREADY_RUNNING_LKG, &(O(object_Kerror_already_running_lkg)) },
 #endif
 #ifdef ERROR_SERVICE_DEPENDENCY_DELETED
  { ERROR_SERVICE_DEPENDENCY_DELETED, &(O(object_Kerror_service_dependency_deleted)) },
 #endif
 #ifdef ERROR_BOOT_ALREADY_ACCEPTED
  { ERROR_BOOT_ALREADY_ACCEPTED, &(O(object_Kerror_boot_already_accepted)) },
 #endif
 #ifdef ERROR_SERVICE_NEVER_STARTED
  { ERROR_SERVICE_NEVER_STARTED, &(O(object_Kerror_service_never_started)) },
 #endif
 #ifdef ERROR_DUPLICATE_SERVICE_NAME
  { ERROR_DUPLICATE_SERVICE_NAME, &(O(object_Kerror_duplicate_service_name)) },
 #endif
 #ifdef ERROR_DIFFERENT_SERVICE_ACCOUNT
  { ERROR_DIFFERENT_SERVICE_ACCOUNT, &(O(object_Kerror_different_service_account)) },
 #endif
 #ifdef ERROR_CANNOT_DETECT_DRIVER_FAILURE
  { ERROR_CANNOT_DETECT_DRIVER_FAILURE, &(O(object_Kerror_cannot_detect_driver_failure)) },
 #endif
 #ifdef ERROR_CANNOT_DETECT_PROCESS_ABORT
  { ERROR_CANNOT_DETECT_PROCESS_ABORT, &(O(object_Kerror_cannot_detect_process_abort)) },
 #endif
 #ifdef ERROR_NO_RECOVERY_PROGRAM
  { ERROR_NO_RECOVERY_PROGRAM, &(O(object_Kerror_no_recovery_program)) },
 #endif
 #ifdef ERROR_SERVICE_NOT_IN_EXE
  { ERROR_SERVICE_NOT_IN_EXE, &(O(object_Kerror_service_not_in_exe)) },
 #endif
 #ifdef ERROR_NOT_SAFEBOOT_SERVICE
  { ERROR_NOT_SAFEBOOT_SERVICE, &(O(object_Kerror_not_safeboot_service)) },
 #endif
 #ifdef ERROR_END_OF_MEDIA
  { ERROR_END_OF_MEDIA, &(O(object_Kerror_end_of_media)) },
 #endif
 #ifdef ERROR_FILEMARK_DETECTED
  { ERROR_FILEMARK_DETECTED, &(O(object_Kerror_filemark_detected)) },
 #endif
 #ifdef ERROR_BEGINNING_OF_MEDIA
  { ERROR_BEGINNING_OF_MEDIA, &(O(object_Kerror_beginning_of_media)) },
 #endif
 #ifdef ERROR_SETMARK_DETECTED
  { ERROR_SETMARK_DETECTED, &(O(object_Kerror_setmark_detected)) },
 #endif
 #ifdef ERROR_NO_DATA_DETECTED
  { ERROR_NO_DATA_DETECTED, &(O(object_Kerror_no_data_detected)) },
 #endif
 #ifdef ERROR_PARTITION_FAILURE
  { ERROR_PARTITION_FAILURE, &(O(object_Kerror_partition_failure)) },
 #endif
 #ifdef ERROR_INVALID_BLOCK_LENGTH
  { ERROR_INVALID_BLOCK_LENGTH, &(O(object_Kerror_invalid_block_length)) },
 #endif
 #ifdef ERROR_DEVICE_NOT_PARTITIONED
  { ERROR_DEVICE_NOT_PARTITIONED, &(O(object_Kerror_device_not_partitioned)) },
 #endif
 #ifdef ERROR_UNABLE_TO_LOCK_MEDIA
  { ERROR_UNABLE_TO_LOCK_MEDIA, &(O(object_Kerror_unable_to_lock_media)) },
 #endif
 #ifdef ERROR_UNABLE_TO_UNLOAD_MEDIA
  { ERROR_UNABLE_TO_UNLOAD_MEDIA, &(O(object_Kerror_unable_to_unload_media)) },
 #endif
 #ifdef ERROR_MEDIA_CHANGED
  { ERROR_MEDIA_CHANGED, &(O(object_Kerror_media_changed)) },
 #endif
 #ifdef ERROR_BUS_RESET
  { ERROR_BUS_RESET, &(O(object_Kerror_bus_reset)) },
 #endif
 #ifdef ERROR_NO_MEDIA_IN_DRIVE
  { ERROR_NO_MEDIA_IN_DRIVE, &(O(object_Kerror_no_media_in_drive)) },
 #endif
 #ifdef ERROR_NO_UNICODE_TRANSLATION
  { ERROR_NO_UNICODE_TRANSLATION, &(O(object_Kerror_no_unicode_translation)) },
 #endif
 #ifdef ERROR_DLL_INIT_FAILED
  { ERROR_DLL_INIT_FAILED, &(O(object_Kerror_dll_init_failed)) },
 #endif
 #ifdef ERROR_SHUTDOWN_IN_PROGRESS
  { ERROR_SHUTDOWN_IN_PROGRESS, &(O(object_Kerror_shutdown_in_progress)) },
 #endif
 #ifdef ERROR_NO_SHUTDOWN_IN_PROGRESS
  { ERROR_NO_SHUTDOWN_IN_PROGRESS, &(O(object_Kerror_no_shutdown_in_progress)) },
 #endif
 #ifdef ERROR_IO_DEVICE
  { ERROR_IO_DEVICE, &(O(object_Kerror_io_device)) },
 #endif
 #ifdef ERROR_SERIAL_NO_DEVICE
  { ERROR_SERIAL_NO_DEVICE, &(O(object_Kerror_serial_no_device)) },
 #endif
 #ifdef ERROR_IRQ_BUSY
  { ERROR_IRQ_BUSY, &(O(object_Kerror_irq_busy)) },
 #endif
 #ifdef ERROR_MORE_WRITES
  { ERROR_MORE_WRITES, &(O(object_Kerror_more_writes)) },
 #endif
 #ifdef ERROR_COUNTER_TIMEOUT
  { ERROR_COUNTER_TIMEOUT, &(O(object_Kerror_counter_timeout)) },
 #endif
 #ifdef ERROR_FLOPPY_ID_MARK_NOT_FOUND
  { ERROR_FLOPPY_ID_MARK_NOT_FOUND, &(O(object_Kerror_floppy_id_mark_not_found)) },
 #endif
 #ifdef ERROR_FLOPPY_WRONG_CYLINDER
  { ERROR_FLOPPY_WRONG_CYLINDER, &(O(object_Kerror_floppy_wrong_cylinder)) },
 #endif
 #ifdef ERROR_FLOPPY_UNKNOWN_ERROR
  { ERROR_FLOPPY_UNKNOWN_ERROR, &(O(object_Kerror_floppy_unknown_error)) },
 #endif
 #ifdef ERROR_FLOPPY_BAD_REGISTERS
  { ERROR_FLOPPY_BAD_REGISTERS, &(O(object_Kerror_floppy_bad_registers)) },
 #endif
 #ifdef ERROR_DISK_RECALIBRATE_FAILED
  { ERROR_DISK_RECALIBRATE_FAILED, &(O(object_Kerror_disk_recalibrate_failed)) },
 #endif
 #ifdef ERROR_DISK_OPERATION_FAILED
  { ERROR_DISK_OPERATION_FAILED, &(O(object_Kerror_disk_operation_failed)) },
 #endif
 #ifdef ERROR_DISK_RESET_FAILED
  { ERROR_DISK_RESET_FAILED, &(O(object_Kerror_disk_reset_failed)) },
 #endif
 #ifdef ERROR_EOM_OVERFLOW
  { ERROR_EOM_OVERFLOW, &(O(object_Kerror_eom_overflow)) },
 #endif
 #ifdef ERROR_NOT_ENOUGH_SERVER_MEMORY
  { ERROR_NOT_ENOUGH_SERVER_MEMORY, &(O(object_Kerror_not_enough_server_memory)) },
 #endif
 #ifdef ERROR_POSSIBLE_DEADLOCK
  { ERROR_POSSIBLE_DEADLOCK, &(O(object_Kerror_possible_deadlock)) },
 #endif
 #ifdef ERROR_MAPPED_ALIGNMENT
  { ERROR_MAPPED_ALIGNMENT, &(O(object_Kerror_mapped_alignment)) },
 #endif
 #ifdef ERROR_SET_POWER_STATE_VETOED
  { ERROR_SET_POWER_STATE_VETOED, &(O(object_Kerror_set_power_state_vetoed)) },
 #endif
 #ifdef ERROR_SET_POWER_STATE_FAILED
  { ERROR_SET_POWER_STATE_FAILED, &(O(object_Kerror_set_power_state_failed)) },
 #endif
 #ifdef ERROR_TOO_MANY_LINKS
  { ERROR_TOO_MANY_LINKS, &(O(object_Kerror_too_many_links)) },
 #endif
 #ifdef ERROR_OLD_WIN_VERSION
  { ERROR_OLD_WIN_VERSION, &(O(object_Kerror_old_win_version)) },
 #endif
 #ifdef ERROR_APP_WRONG_OS
  { ERROR_APP_WRONG_OS, &(O(object_Kerror_app_wrong_os)) },
 #endif
 #ifdef ERROR_SINGLE_INSTANCE_APP
  { ERROR_SINGLE_INSTANCE_APP, &(O(object_Kerror_single_instance_app)) },
 #endif
 #ifdef ERROR_RMODE_APP
  { ERROR_RMODE_APP, &(O(object_Kerror_rmode_app)) },
 #endif
 #ifdef ERROR_INVALID_DLL
  { ERROR_INVALID_DLL, &(O(object_Kerror_invalid_dll)) },
 #endif
 #ifdef ERROR_NO_ASSOCIATION
  { ERROR_NO_ASSOCIATION, &(O(object_Kerror_no_association)) },
 #endif
 #ifdef ERROR_DDE_FAIL
  { ERROR_DDE_FAIL, &(O(object_Kerror_dde_fail)) },
 #endif
 #ifdef ERROR_DLL_NOT_FOUND
  { ERROR_DLL_NOT_FOUND, &(O(object_Kerror_dll_not_found)) },
 #endif
 #ifdef ERROR_NO_MORE_USER_HANDLES
  { ERROR_NO_MORE_USER_HANDLES, &(O(object_Kerror_no_more_user_handles)) },
 #endif
 #ifdef ERROR_MESSAGE_SYNC_ONLY
  { ERROR_MESSAGE_SYNC_ONLY, &(O(object_Kerror_message_sync_only)) },
 #endif
 #ifdef ERROR_SOURCE_ELEMENT_EMPTY
  { ERROR_SOURCE_ELEMENT_EMPTY, &(O(object_Kerror_source_element_empty)) },
 #endif
 #ifdef ERROR_DESTINATION_ELEMENT_FULL
  { ERROR_DESTINATION_ELEMENT_FULL, &(O(object_Kerror_destination_element_full)) },
 #endif
 #ifdef ERROR_ILLEGAL_ELEMENT_ADDRESS
  { ERROR_ILLEGAL_ELEMENT_ADDRESS, &(O(object_Kerror_illegal_element_address)) },
 #endif
 #ifdef ERROR_MAGAZINE_NOT_PRESENT
  { ERROR_MAGAZINE_NOT_PRESENT, &(O(object_Kerror_magazine_not_present)) },
 #endif
 #ifdef ERROR_DEVICE_REINITIALIZATION_NEEDED
  { ERROR_DEVICE_REINITIALIZATION_NEEDED, &(O(object_Kerror_device_reinitialization_needed)) },
 #endif
 #ifdef ERROR_DEVICE_REQUIRES_CLEANING
  { ERROR_DEVICE_REQUIRES_CLEANING, &(O(object_Kerror_device_requires_cleaning)) },
 #endif
 #ifdef ERROR_DEVICE_DOOR_OPEN
  { ERROR_DEVICE_DOOR_OPEN, &(O(object_Kerror_device_door_open)) },
 #endif
 #ifdef ERROR_DEVICE_NOT_CONNECTED
  { ERROR_DEVICE_NOT_CONNECTED, &(O(object_Kerror_device_not_connected)) },
 #endif
 #ifdef ERROR_NOT_FOUND
  { ERROR_NOT_FOUND, &(O(object_Kerror_not_found)) },
 #endif
 #ifdef ERROR_NO_MATCH
  { ERROR_NO_MATCH, &(O(object_Kerror_no_match)) },
 #endif
 #ifdef ERROR_SET_NOT_FOUND
  { ERROR_SET_NOT_FOUND, &(O(object_Kerror_set_not_found)) },
 #endif
 #ifdef ERROR_POINT_NOT_FOUND
  { ERROR_POINT_NOT_FOUND, &(O(object_Kerror_point_not_found)) },
 #endif
 #ifdef ERROR_NO_TRACKING_SERVICE
  { ERROR_NO_TRACKING_SERVICE, &(O(object_Kerror_no_tracking_service)) },
 #endif
 #ifdef ERROR_NO_VOLUME_ID
  { ERROR_NO_VOLUME_ID, &(O(object_Kerror_no_volume_id)) },
 #endif
 #ifdef ERROR_UNABLE_TO_REMOVE_REPLACED
  { ERROR_UNABLE_TO_REMOVE_REPLACED, &(O(object_Kerror_unable_to_remove_replaced)) },
 #endif
 #ifdef ERROR_UNABLE_TO_MOVE_REPLACEMENT
  { ERROR_UNABLE_TO_MOVE_REPLACEMENT, &(O(object_Kerror_unable_to_move_replacement)) },
 #endif
 #ifdef ERROR_UNABLE_TO_MOVE_REPLACEMENT_2
  { ERROR_UNABLE_TO_MOVE_REPLACEMENT_2, &(O(object_Kerror_unable_to_move_replacement_2)) },
 #endif
 #ifdef ERROR_JOURNAL_DELETE_IN_PROGRESS
  { ERROR_JOURNAL_DELETE_IN_PROGRESS, &(O(object_Kerror_journal_delete_in_progress)) },
 #endif
 #ifdef ERROR_JOURNAL_NOT_ACTIVE
  { ERROR_JOURNAL_NOT_ACTIVE, &(O(object_Kerror_journal_not_active)) },
 #endif
 #ifdef ERROR_POTENTIAL_FILE_FOUND
  { ERROR_POTENTIAL_FILE_FOUND, &(O(object_Kerror_potential_file_found)) },
 #endif
 #ifdef ERROR_JOURNAL_ENTRY_DELETED
  { ERROR_JOURNAL_ENTRY_DELETED, &(O(object_Kerror_journal_entry_deleted)) },
 #endif
 #ifdef ERROR_BAD_DEVICE
  { ERROR_BAD_DEVICE, &(O(object_Kerror_bad_device)) },
 #endif
 #ifdef ERROR_CONNECTION_UNAVAIL
  { ERROR_CONNECTION_UNAVAIL, &(O(object_Kerror_connection_unavail)) },
 #endif
 #ifdef ERROR_DEVICE_ALREADY_REMEMBERED
  { ERROR_DEVICE_ALREADY_REMEMBERED, &(O(object_Kerror_device_already_remembered)) },
 #endif
 #ifdef ERROR_NO_NET_OR_BAD_PATH
  { ERROR_NO_NET_OR_BAD_PATH, &(O(object_Kerror_no_net_or_bad_path)) },
 #endif
 #ifdef ERROR_BAD_PROVIDER
  { ERROR_BAD_PROVIDER, &(O(object_Kerror_bad_provider)) },
 #endif
 #ifdef ERROR_CANNOT_OPEN_PROFILE
  { ERROR_CANNOT_OPEN_PROFILE, &(O(object_Kerror_cannot_open_profile)) },
 #endif
 #ifdef ERROR_BAD_PROFILE
  { ERROR_BAD_PROFILE, &(O(object_Kerror_bad_profile)) },
 #endif
 #ifdef ERROR_NOT_CONTAINER
  { ERROR_NOT_CONTAINER, &(O(object_Kerror_not_container)) },
 #endif
 #ifdef ERROR_EXTENDED_ERROR
  { ERROR_EXTENDED_ERROR, &(O(object_Kerror_extended_error)) },
 #endif
 #ifdef ERROR_INVALID_GROUPNAME
  { ERROR_INVALID_GROUPNAME, &(O(object_Kerror_invalid_groupname)) },
 #endif
 #ifdef ERROR_INVALID_COMPUTERNAME
  { ERROR_INVALID_COMPUTERNAME, &(O(object_Kerror_invalid_computername)) },
 #endif
 #ifdef ERROR_INVALID_EVENTNAME
  { ERROR_INVALID_EVENTNAME, &(O(object_Kerror_invalid_eventname)) },
 #endif
 #ifdef ERROR_INVALID_DOMAINNAME
  { ERROR_INVALID_DOMAINNAME, &(O(object_Kerror_invalid_domainname)) },
 #endif
 #ifdef ERROR_INVALID_SERVICENAME
  { ERROR_INVALID_SERVICENAME, &(O(object_Kerror_invalid_servicename)) },
 #endif
 #ifdef ERROR_INVALID_NETNAME
  { ERROR_INVALID_NETNAME, &(O(object_Kerror_invalid_netname)) },
 #endif
 #ifdef ERROR_INVALID_SHARENAME
  { ERROR_INVALID_SHARENAME, &(O(object_Kerror_invalid_sharename)) },
 #endif
 #ifdef ERROR_INVALID_PASSWORDNAME
  { ERROR_INVALID_PASSWORDNAME, &(O(object_Kerror_invalid_passwordname)) },
 #endif
 #ifdef ERROR_INVALID_MESSAGENAME
  { ERROR_INVALID_MESSAGENAME, &(O(object_Kerror_invalid_messagename)) },
 #endif
 #ifdef ERROR_INVALID_MESSAGEDEST
  { ERROR_INVALID_MESSAGEDEST, &(O(object_Kerror_invalid_messagedest)) },
 #endif
 #ifdef ERROR_SESSION_CREDENTIAL_CONFLICT
  { ERROR_SESSION_CREDENTIAL_CONFLICT, &(O(object_Kerror_session_credential_conflict)) },
 #endif
 #ifdef ERROR_REMOTE_SESSION_LIMIT_EXCEEDED
  { ERROR_REMOTE_SESSION_LIMIT_EXCEEDED, &(O(object_Kerror_remote_session_limit_exceeded)) },
 #endif
 #ifdef ERROR_DUP_DOMAINNAME
  { ERROR_DUP_DOMAINNAME, &(O(object_Kerror_dup_domainname)) },
 #endif
 #ifdef ERROR_NO_NETWORK
  { ERROR_NO_NETWORK, &(O(object_Kerror_no_network)) },
 #endif
 #ifdef ERROR_CANCELLED
  { ERROR_CANCELLED, &(O(object_Kerror_cancelled)) },
 #endif
 #ifdef ERROR_USER_MAPPED_FILE
  { ERROR_USER_MAPPED_FILE, &(O(object_Kerror_user_mapped_file)) },
 #endif
 #ifdef ERROR_CONNECTION_REFUSED
  { ERROR_CONNECTION_REFUSED, &(O(object_Kerror_connection_refused)) },
 #endif
 #ifdef ERROR_GRACEFUL_DISCONNECT
  { ERROR_GRACEFUL_DISCONNECT, &(O(object_Kerror_graceful_disconnect)) },
 #endif
 #ifdef ERROR_ADDRESS_ALREADY_ASSOCIATED
  { ERROR_ADDRESS_ALREADY_ASSOCIATED, &(O(object_Kerror_address_already_associated)) },
 #endif
 #ifdef ERROR_ADDRESS_NOT_ASSOCIATED
  { ERROR_ADDRESS_NOT_ASSOCIATED, &(O(object_Kerror_address_not_associated)) },
 #endif
 #ifdef ERROR_CONNECTION_INVALID
  { ERROR_CONNECTION_INVALID, &(O(object_Kerror_connection_invalid)) },
 #endif
 #ifdef ERROR_CONNECTION_ACTIVE
  { ERROR_CONNECTION_ACTIVE, &(O(object_Kerror_connection_active)) },
 #endif
 #ifdef ERROR_NETWORK_UNREACHABLE
  { ERROR_NETWORK_UNREACHABLE, &(O(object_Kerror_network_unreachable)) },
 #endif
 #ifdef ERROR_HOST_UNREACHABLE
  { ERROR_HOST_UNREACHABLE, &(O(object_Kerror_host_unreachable)) },
 #endif
 #ifdef ERROR_PROTOCOL_UNREACHABLE
  { ERROR_PROTOCOL_UNREACHABLE, &(O(object_Kerror_protocol_unreachable)) },
 #endif
 #ifdef ERROR_PORT_UNREACHABLE
  { ERROR_PORT_UNREACHABLE, &(O(object_Kerror_port_unreachable)) },
 #endif
 #ifdef ERROR_REQUEST_ABORTED
  { ERROR_REQUEST_ABORTED, &(O(object_Kerror_request_aborted)) },
 #endif
 #ifdef ERROR_CONNECTION_ABORTED
  { ERROR_CONNECTION_ABORTED, &(O(object_Kerror_connection_aborted)) },
 #endif
 #ifdef ERROR_RETRY
  { ERROR_RETRY, &(O(object_Kerror_retry)) },
 #endif
 #ifdef ERROR_CONNECTION_COUNT_LIMIT
  { ERROR_CONNECTION_COUNT_LIMIT, &(O(object_Kerror_connection_count_limit)) },
 #endif
 #ifdef ERROR_LOGIN_TIME_RESTRICTION
  { ERROR_LOGIN_TIME_RESTRICTION, &(O(object_Kerror_login_time_restriction)) },
 #endif
 #ifdef ERROR_LOGIN_WKSTA_RESTRICTION
  { ERROR_LOGIN_WKSTA_RESTRICTION, &(O(object_Kerror_login_wksta_restriction)) },
 #endif
 #ifdef ERROR_INCORRECT_ADDRESS
  { ERROR_INCORRECT_ADDRESS, &(O(object_Kerror_incorrect_address)) },
 #endif
 #ifdef ERROR_ALREADY_REGISTERED
  { ERROR_ALREADY_REGISTERED, &(O(object_Kerror_already_registered)) },
 #endif
 #ifdef ERROR_SERVICE_NOT_FOUND
  { ERROR_SERVICE_NOT_FOUND, &(O(object_Kerror_service_not_found)) },
 #endif
 #ifdef ERROR_NOT_AUTHENTICATED
  { ERROR_NOT_AUTHENTICATED, &(O(object_Kerror_not_authenticated)) },
 #endif
 #ifdef ERROR_NOT_LOGGED_ON
  { ERROR_NOT_LOGGED_ON, &(O(object_Kerror_not_logged_on)) },
 #endif
 #ifdef ERROR_CONTINUE
  { ERROR_CONTINUE, &(O(object_Kerror_continue)) },
 #endif
 #ifdef ERROR_ALREADY_INITIALIZED
  { ERROR_ALREADY_INITIALIZED, &(O(object_Kerror_already_initialized)) },
 #endif
 #ifdef ERROR_NO_MORE_DEVICES
  { ERROR_NO_MORE_DEVICES, &(O(object_Kerror_no_more_devices)) },
 #endif
 #ifdef ERROR_NO_SUCH_SITE
  { ERROR_NO_SUCH_SITE, &(O(object_Kerror_no_such_site)) },
 #endif
 #ifdef ERROR_DOMAIN_CONTROLLER_EXISTS
  { ERROR_DOMAIN_CONTROLLER_EXISTS, &(O(object_Kerror_domain_controller_exists)) },
 #endif
 #ifdef ERROR_ONLY_IF_CONNECTED
  { ERROR_ONLY_IF_CONNECTED, &(O(object_Kerror_only_if_connected)) },
 #endif
 #ifdef ERROR_OVERRIDE_NOCHANGES
  { ERROR_OVERRIDE_NOCHANGES, &(O(object_Kerror_override_nochanges)) },
 #endif
 #ifdef ERROR_BAD_USER_PROFILE
  { ERROR_BAD_USER_PROFILE, &(O(object_Kerror_bad_user_profile)) },
 #endif
 #ifdef ERROR_NOT_SUPPORTED_ON_SBS
  { ERROR_NOT_SUPPORTED_ON_SBS, &(O(object_Kerror_not_supported_on_sbs)) },
 #endif
 #ifdef ERROR_SERVER_SHUTDOWN_IN_PROGRESS
  { ERROR_SERVER_SHUTDOWN_IN_PROGRESS, &(O(object_Kerror_server_shutdown_in_progress)) },
 #endif
 #ifdef ERROR_HOST_DOWN
  { ERROR_HOST_DOWN, &(O(object_Kerror_host_down)) },
 #endif
 #ifdef ERROR_NON_ACCOUNT_SID
  { ERROR_NON_ACCOUNT_SID, &(O(object_Kerror_non_account_sid)) },
 #endif
 #ifdef ERROR_NON_DOMAIN_SID
  { ERROR_NON_DOMAIN_SID, &(O(object_Kerror_non_domain_sid)) },
 #endif
 #ifdef ERROR_APPHELP_BLOCK
  { ERROR_APPHELP_BLOCK, &(O(object_Kerror_apphelp_block)) },
 #endif
 #ifdef ERROR_ACCESS_DISABLED_BY_POLICY
  { ERROR_ACCESS_DISABLED_BY_POLICY, &(O(object_Kerror_access_disabled_by_policy)) },
 #endif
 #ifdef ERROR_REG_NAT_CONSUMPTION
  { ERROR_REG_NAT_CONSUMPTION, &(O(object_Kerror_reg_nat_consumption)) },
 #endif
 #ifdef ERROR_CSCSHARE_OFFLINE
  { ERROR_CSCSHARE_OFFLINE, &(O(object_Kerror_cscshare_offline)) },
 #endif
 #ifdef ERROR_PKINIT_FAILURE
  { ERROR_PKINIT_FAILURE, &(O(object_Kerror_pkinit_failure)) },
 #endif
 #ifdef ERROR_SMARTCARD_SUBSYSTEM_FAILURE
  { ERROR_SMARTCARD_SUBSYSTEM_FAILURE, &(O(object_Kerror_smartcard_subsystem_failure)) },
 #endif
 #ifdef ERROR_DOWNGRADE_DETECTED
  { ERROR_DOWNGRADE_DETECTED, &(O(object_Kerror_downgrade_detected)) },
 #endif
 #ifdef SEC_E_SMARTCARD_CERT_REVOKED
  { SEC_E_SMARTCARD_CERT_REVOKED, &(O(object_Ksec_e_smartcard_cert_revoked)) },
 #endif
 #ifdef SEC_E_ISSUING_CA_UNTRUSTED
  { SEC_E_ISSUING_CA_UNTRUSTED, &(O(object_Ksec_e_issuing_ca_untrusted)) },
 #endif
 #ifdef SEC_E_REVOCATION_OFFLINE_C
  { SEC_E_REVOCATION_OFFLINE_C, &(O(object_Ksec_e_revocation_offline_c)) },
 #endif
 #ifdef SEC_E_PKINIT_CLIENT_FAILUR
  { SEC_E_PKINIT_CLIENT_FAILUR, &(O(object_Ksec_e_pkinit_client_failur)) },
 #endif
 #ifdef SEC_E_SMARTCARD_CERT_EXPIRED
  { SEC_E_SMARTCARD_CERT_EXPIRED, &(O(object_Ksec_e_smartcard_cert_expired)) },
 #endif
 #ifdef ERROR_MACHINE_LOCKED
  { ERROR_MACHINE_LOCKED, &(O(object_Kerror_machine_locked)) },
 #endif
 #ifdef ERROR_CALLBACK_SUPPLIED_INVALID_DATA
  { ERROR_CALLBACK_SUPPLIED_INVALID_DATA, &(O(object_Kerror_callback_supplied_invalid_data)) },
 #endif
 #ifdef ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED
  { ERROR_SYNC_FOREGROUND_REFRESH_REQUIRED, &(O(object_Kerror_sync_foreground_refresh_required)) },
 #endif
 #ifdef ERROR_DRIVER_BLOCKED
  { ERROR_DRIVER_BLOCKED, &(O(object_Kerror_driver_blocked)) },
 #endif
 #ifdef ERROR_INVALID_IMPORT_OF_NON_DLL
  { ERROR_INVALID_IMPORT_OF_NON_DLL, &(O(object_Kerror_invalid_import_of_non_dll)) },
 #endif
 #ifdef ERROR_ACCESS_DISABLED_WEBBLADE
  { ERROR_ACCESS_DISABLED_WEBBLADE, &(O(object_Kerror_access_disabled_webblade)) },
 #endif
 #ifdef ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER
  { ERROR_ACCESS_DISABLED_WEBBLADE_TAMPER, &(O(object_Kerror_access_disabled_webblade_tamper)) },
 #endif
 #ifdef ERROR_RECOVERY_FAILURE
  { ERROR_RECOVERY_FAILURE, &(O(object_Kerror_recovery_failure)) },
 #endif
 #ifdef ERROR_ALREADY_FIBER
  { ERROR_ALREADY_FIBER, &(O(object_Kerror_already_fiber)) },
 #endif
 #ifdef ERROR_ALREADY_THREAD
  { ERROR_ALREADY_THREAD, &(O(object_Kerror_already_thread)) },
 #endif
 #ifdef ERROR_STACK_BUFFER_OVERRUN
  { ERROR_STACK_BUFFER_OVERRUN, &(O(object_Kerror_stack_buffer_overrun)) },
 #endif
 #ifdef ERROR_PARAMETER_QUOTA_EXCEEDED
  { ERROR_PARAMETER_QUOTA_EXCEEDED, &(O(object_Kerror_parameter_quota_exceeded)) },
 #endif
 #ifdef ERROR_DEBUGGER_INACTIVE
  { ERROR_DEBUGGER_INACTIVE, &(O(object_Kerror_debugger_inactive)) },
 #endif
 #ifdef ERROR_NOT_ALL_ASSIGNED
  { ERROR_NOT_ALL_ASSIGNED, &(O(object_Kerror_not_all_assigned)) },
 #endif
 #ifdef ERROR_SOME_NOT_MAPPED
  { ERROR_SOME_NOT_MAPPED, &(O(object_Kerror_some_not_mapped)) },
 #endif
 #ifdef ERROR_NO_QUOTAS_FOR_ACCOUNT
  { ERROR_NO_QUOTAS_FOR_ACCOUNT, &(O(object_Kerror_no_quotas_for_account)) },
 #endif
 #ifdef ERROR_LOCAL_USER_SESSION_KEY
  { ERROR_LOCAL_USER_SESSION_KEY, &(O(object_Kerror_local_user_session_key)) },
 #endif
 #ifdef ERROR_NULL_LM_PASSWORD
  { ERROR_NULL_LM_PASSWORD, &(O(object_Kerror_null_lm_password)) },
 #endif
 #ifdef ERROR_UNKNOWN_REVISION
  { ERROR_UNKNOWN_REVISION, &(O(object_Kerror_unknown_revision)) },
 #endif
 #ifdef ERROR_REVISION_MISMATCH
  { ERROR_REVISION_MISMATCH, &(O(object_Kerror_revision_mismatch)) },
 #endif
 #ifdef ERROR_INVALID_OWNER
  { ERROR_INVALID_OWNER, &(O(object_Kerror_invalid_owner)) },
 #endif
 #ifdef ERROR_INVALID_PRIMARY_GROUP
  { ERROR_INVALID_PRIMARY_GROUP, &(O(object_Kerror_invalid_primary_group)) },
 #endif
 #ifdef ERROR_NO_IMPERSONATION_TOKEN
  { ERROR_NO_IMPERSONATION_TOKEN, &(O(object_Kerror_no_impersonation_token)) },
 #endif
 #ifdef ERROR_CANT_DISABLE_MANDATORY
  { ERROR_CANT_DISABLE_MANDATORY, &(O(object_Kerror_cant_disable_mandatory)) },
 #endif
 #ifdef ERROR_NO_LOGON_SERVERS
  { ERROR_NO_LOGON_SERVERS, &(O(object_Kerror_no_logon_servers)) },
 #endif
 #ifdef ERROR_NO_SUCH_LOGON_SESSION
  { ERROR_NO_SUCH_LOGON_SESSION, &(O(object_Kerror_no_such_logon_session)) },
 #endif
 #ifdef ERROR_NO_SUCH_PRIVILEGE
  { ERROR_NO_SUCH_PRIVILEGE, &(O(object_Kerror_no_such_privilege)) },
 #endif
 #ifdef ERROR_PRIVILEGE_NOT_HELD
  { ERROR_PRIVILEGE_NOT_HELD, &(O(object_Kerror_privilege_not_held)) },
 #endif
 #ifdef ERROR_INVALID_ACCOUNT_NAME
  { ERROR_INVALID_ACCOUNT_NAME, &(O(object_Kerror_invalid_account_name)) },
 #endif
 #ifdef ERROR_USER_EXISTS
  { ERROR_USER_EXISTS, &(O(object_Kerror_user_exists)) },
 #endif
 #ifdef ERROR_NO_SUCH_USER
  { ERROR_NO_SUCH_USER, &(O(object_Kerror_no_such_user)) },
 #endif
 #ifdef ERROR_GROUP_EXISTS
  { ERROR_GROUP_EXISTS, &(O(object_Kerror_group_exists)) },
 #endif
 #ifdef ERROR_NO_SUCH_GROUP
  { ERROR_NO_SUCH_GROUP, &(O(object_Kerror_no_such_group)) },
 #endif
 #ifdef ERROR_MEMBER_IN_GROUP
  { ERROR_MEMBER_IN_GROUP, &(O(object_Kerror_member_in_group)) },
 #endif
 #ifdef ERROR_MEMBER_NOT_IN_GROUP
  { ERROR_MEMBER_NOT_IN_GROUP, &(O(object_Kerror_member_not_in_group)) },
 #endif
 #ifdef ERROR_LAST_ADMIN
  { ERROR_LAST_ADMIN, &(O(object_Kerror_last_admin)) },
 #endif
 #ifdef ERROR_WRONG_PASSWORD
  { ERROR_WRONG_PASSWORD, &(O(object_Kerror_wrong_password)) },
 #endif
 #ifdef ERROR_ILL_FORMED_PASSWORD
  { ERROR_ILL_FORMED_PASSWORD, &(O(object_Kerror_ill_formed_password)) },
 #endif
 #ifdef ERROR_PASSWORD_RESTRICTION
  { ERROR_PASSWORD_RESTRICTION, &(O(object_Kerror_password_restriction)) },
 #endif
 #ifdef ERROR_LOGON_FAILURE
  { ERROR_LOGON_FAILURE, &(O(object_Kerror_logon_failure)) },
 #endif
 #ifdef ERROR_ACCOUNT_RESTRICTION
  { ERROR_ACCOUNT_RESTRICTION, &(O(object_Kerror_account_restriction)) },
 #endif
 #ifdef ERROR_INVALID_LOGON_HOURS
  { ERROR_INVALID_LOGON_HOURS, &(O(object_Kerror_invalid_logon_hours)) },
 #endif
 #ifdef ERROR_INVALID_WORKSTATION
  { ERROR_INVALID_WORKSTATION, &(O(object_Kerror_invalid_workstation)) },
 #endif
 #ifdef ERROR_PASSWORD_EXPIRED
  { ERROR_PASSWORD_EXPIRED, &(O(object_Kerror_password_expired)) },
 #endif
 #ifdef ERROR_ACCOUNT_DISABLED
  { ERROR_ACCOUNT_DISABLED, &(O(object_Kerror_account_disabled)) },
 #endif
 #ifdef ERROR_NONE_MAPPED
  { ERROR_NONE_MAPPED, &(O(object_Kerror_none_mapped)) },
 #endif
 #ifdef ERROR_TOO_MANY_LUIDS_REQUESTED
  { ERROR_TOO_MANY_LUIDS_REQUESTED, &(O(object_Kerror_too_many_luids_requested)) },
 #endif
 #ifdef ERROR_LUIDS_EXHAUSTED
  { ERROR_LUIDS_EXHAUSTED, &(O(object_Kerror_luids_exhausted)) },
 #endif
 #ifdef ERROR_INVALID_SUB_AUTHORITY
  { ERROR_INVALID_SUB_AUTHORITY, &(O(object_Kerror_invalid_sub_authority)) },
 #endif
 #ifdef ERROR_INVALID_ACL
  { ERROR_INVALID_ACL, &(O(object_Kerror_invalid_acl)) },
 #endif
 #ifdef ERROR_INVALID_SID
  { ERROR_INVALID_SID, &(O(object_Kerror_invalid_sid)) },
 #endif
 #ifdef ERROR_INVALID_SECURITY_DESCR
  { ERROR_INVALID_SECURITY_DESCR, &(O(object_Kerror_invalid_security_descr)) },
 #endif
 #ifdef ERROR_BAD_INHERITANCE_ACL
  { ERROR_BAD_INHERITANCE_ACL, &(O(object_Kerror_bad_inheritance_acl)) },
 #endif
 #ifdef ERROR_SERVER_DISABLED
  { ERROR_SERVER_DISABLED, &(O(object_Kerror_server_disabled)) },
 #endif
 #ifdef ERROR_SERVER_NOT_DISABLED
  { ERROR_SERVER_NOT_DISABLED, &(O(object_Kerror_server_not_disabled)) },
 #endif
 #ifdef ERROR_INVALID_ID_AUTHORITY
  { ERROR_INVALID_ID_AUTHORITY, &(O(object_Kerror_invalid_id_authority)) },
 #endif
 #ifdef ERROR_ALLOTTED_SPACE_EXCEEDED
  { ERROR_ALLOTTED_SPACE_EXCEEDED, &(O(object_Kerror_allotted_space_exceeded)) },
 #endif
 #ifdef ERROR_INVALID_GROUP_ATTRIBUTES
  { ERROR_INVALID_GROUP_ATTRIBUTES, &(O(object_Kerror_invalid_group_attributes)) },
 #endif
 #ifdef ERROR_BAD_IMPERSONATION_LEVEL
  { ERROR_BAD_IMPERSONATION_LEVEL, &(O(object_Kerror_bad_impersonation_level)) },
 #endif
 #ifdef ERROR_CANT_OPEN_ANONYMOUS
  { ERROR_CANT_OPEN_ANONYMOUS, &(O(object_Kerror_cant_open_anonymous)) },
 #endif
 #ifdef ERROR_BAD_VALIDATION_CLASS
  { ERROR_BAD_VALIDATION_CLASS, &(O(object_Kerror_bad_validation_class)) },
 #endif
 #ifdef ERROR_BAD_TOKEN_TYPE
  { ERROR_BAD_TOKEN_TYPE, &(O(object_Kerror_bad_token_type)) },
 #endif
 #ifdef ERROR_NO_SECURITY_ON_OBJECT
  { ERROR_NO_SECURITY_ON_OBJECT, &(O(object_Kerror_no_security_on_object)) },
 #endif
 #ifdef ERROR_CANT_ACCESS_DOMAIN_INFO
  { ERROR_CANT_ACCESS_DOMAIN_INFO, &(O(object_Kerror_cant_access_domain_info)) },
 #endif
 #ifdef ERROR_INVALID_SERVER_STATE
  { ERROR_INVALID_SERVER_STATE, &(O(object_Kerror_invalid_server_state)) },
 #endif
 #ifdef ERROR_INVALID_DOMAIN_STATE
  { ERROR_INVALID_DOMAIN_STATE, &(O(object_Kerror_invalid_domain_state)) },
 #endif
 #ifdef ERROR_INVALID_DOMAIN_ROLE
  { ERROR_INVALID_DOMAIN_ROLE, &(O(object_Kerror_invalid_domain_role)) },
 #endif
 #ifdef ERROR_NO_SUCH_DOMAIN
  { ERROR_NO_SUCH_DOMAIN, &(O(object_Kerror_no_such_domain)) },
 #endif
 #ifdef ERROR_DOMAIN_EXISTS
  { ERROR_DOMAIN_EXISTS, &(O(object_Kerror_domain_exists)) },
 #endif
 #ifdef ERROR_DOMAIN_LIMIT_EXCEEDED
  { ERROR_DOMAIN_LIMIT_EXCEEDED, &(O(object_Kerror_domain_limit_exceeded)) },
 #endif
 #ifdef ERROR_INTERNAL_DB_CORRUPTION
  { ERROR_INTERNAL_DB_CORRUPTION, &(O(object_Kerror_internal_db_corruption)) },
 #endif
 #ifdef ERROR_INTERNAL_ERROR
  { ERROR_INTERNAL_ERROR, &(O(object_Kerror_internal_error)) },
 #endif
 #ifdef ERROR_GENERIC_NOT_MAPPED
  { ERROR_GENERIC_NOT_MAPPED, &(O(object_Kerror_generic_not_mapped)) },
 #endif
 #ifdef ERROR_BAD_DESCRIPTOR_FORMAT
  { ERROR_BAD_DESCRIPTOR_FORMAT, &(O(object_Kerror_bad_descriptor_format)) },
 #endif
 #ifdef ERROR_NOT_LOGON_PROCESS
  { ERROR_NOT_LOGON_PROCESS, &(O(object_Kerror_not_logon_process)) },
 #endif
 #ifdef ERROR_LOGON_SESSION_EXISTS
  { ERROR_LOGON_SESSION_EXISTS, &(O(object_Kerror_logon_session_exists)) },
 #endif
 #ifdef ERROR_NO_SUCH_PACKAGE
  { ERROR_NO_SUCH_PACKAGE, &(O(object_Kerror_no_such_package)) },
 #endif
 #ifdef ERROR_BAD_LOGON_SESSION_STATE
  { ERROR_BAD_LOGON_SESSION_STATE, &(O(object_Kerror_bad_logon_session_state)) },
 #endif
 #ifdef ERROR_LOGON_SESSION_COLLISION
  { ERROR_LOGON_SESSION_COLLISION, &(O(object_Kerror_logon_session_collision)) },
 #endif
 #ifdef ERROR_INVALID_LOGON_TYPE
  { ERROR_INVALID_LOGON_TYPE, &(O(object_Kerror_invalid_logon_type)) },
 #endif
 #ifdef ERROR_CANNOT_IMPERSONATE
  { ERROR_CANNOT_IMPERSONATE, &(O(object_Kerror_cannot_impersonate)) },
 #endif
 #ifdef ERROR_RXACT_INVALID_STATE
  { ERROR_RXACT_INVALID_STATE, &(O(object_Kerror_rxact_invalid_state)) },
 #endif
 #ifdef ERROR_RXACT_COMMIT_FAILURE
  { ERROR_RXACT_COMMIT_FAILURE, &(O(object_Kerror_rxact_commit_failure)) },
 #endif
 #ifdef ERROR_SPECIAL_ACCOUNT
  { ERROR_SPECIAL_ACCOUNT, &(O(object_Kerror_special_account)) },
 #endif
 #ifdef ERROR_SPECIAL_GROUP
  { ERROR_SPECIAL_GROUP, &(O(object_Kerror_special_group)) },
 #endif
 #ifdef ERROR_SPECIAL_USER
  { ERROR_SPECIAL_USER, &(O(object_Kerror_special_user)) },
 #endif
 #ifdef ERROR_MEMBERS_PRIMARY_GROUP
  { ERROR_MEMBERS_PRIMARY_GROUP, &(O(object_Kerror_members_primary_group)) },
 #endif
 #ifdef ERROR_TOKEN_ALREADY_IN_USE
  { ERROR_TOKEN_ALREADY_IN_USE, &(O(object_Kerror_token_already_in_use)) },
 #endif
 #ifdef ERROR_NO_SUCH_ALIAS
  { ERROR_NO_SUCH_ALIAS, &(O(object_Kerror_no_such_alias)) },
 #endif
 #ifdef ERROR_MEMBER_NOT_IN_ALIAS
  { ERROR_MEMBER_NOT_IN_ALIAS, &(O(object_Kerror_member_not_in_alias)) },
 #endif
 #ifdef ERROR_MEMBER_IN_ALIAS
  { ERROR_MEMBER_IN_ALIAS, &(O(object_Kerror_member_in_alias)) },
 #endif
 #ifdef ERROR_ALIAS_EXISTS
  { ERROR_ALIAS_EXISTS, &(O(object_Kerror_alias_exists)) },
 #endif
 #ifdef ERROR_LOGON_NOT_GRANTED
  { ERROR_LOGON_NOT_GRANTED, &(O(object_Kerror_logon_not_granted)) },
 #endif
 #ifdef ERROR_TOO_MANY_SECRETS
  { ERROR_TOO_MANY_SECRETS, &(O(object_Kerror_too_many_secrets)) },
 #endif
 #ifdef ERROR_SECRET_TOO_LONG
  { ERROR_SECRET_TOO_LONG, &(O(object_Kerror_secret_too_long)) },
 #endif
 #ifdef ERROR_INTERNAL_DB_ERROR
  { ERROR_INTERNAL_DB_ERROR, &(O(object_Kerror_internal_db_error)) },
 #endif
 #ifdef ERROR_TOO_MANY_CONTEXT_IDS
  { ERROR_TOO_MANY_CONTEXT_IDS, &(O(object_Kerror_too_many_context_ids)) },
 #endif
 #ifdef ERROR_LOGON_TYPE_NOT_GRANTED
  { ERROR_LOGON_TYPE_NOT_GRANTED, &(O(object_Kerror_logon_type_not_granted)) },
 #endif
 #ifdef ERROR_NT_CROSS_ENCRYPTION_REQUIRED
  { ERROR_NT_CROSS_ENCRYPTION_REQUIRED, &(O(object_Kerror_nt_cross_encryption_required)) },
 #endif
 #ifdef ERROR_NO_SUCH_MEMBER
  { ERROR_NO_SUCH_MEMBER, &(O(object_Kerror_no_such_member)) },
 #endif
 #ifdef ERROR_INVALID_MEMBER
  { ERROR_INVALID_MEMBER, &(O(object_Kerror_invalid_member)) },
 #endif
 #ifdef ERROR_TOO_MANY_SIDS
  { ERROR_TOO_MANY_SIDS, &(O(object_Kerror_too_many_sids)) },
 #endif
 #ifdef ERROR_LM_CROSS_ENCRYPTION_REQUIRED
  { ERROR_LM_CROSS_ENCRYPTION_REQUIRED, &(O(object_Kerror_lm_cross_encryption_required)) },
 #endif
 #ifdef ERROR_NO_INHERITANCE
  { ERROR_NO_INHERITANCE, &(O(object_Kerror_no_inheritance)) },
 #endif
 #ifdef ERROR_FILE_CORRUPT
  { ERROR_FILE_CORRUPT, &(O(object_Kerror_file_corrupt)) },
 #endif
 #ifdef ERROR_DISK_CORRUPT
  { ERROR_DISK_CORRUPT, &(O(object_Kerror_disk_corrupt)) },
 #endif
 #ifdef ERROR_NO_USER_SESSION_KEY
  { ERROR_NO_USER_SESSION_KEY, &(O(object_Kerror_no_user_session_key)) },
 #endif
 #ifdef ERROR_LICENSE_QUOTA_EXCEEDED
  { ERROR_LICENSE_QUOTA_EXCEEDED, &(O(object_Kerror_license_quota_exceeded)) },
 #endif
 #ifdef ERROR_WRONG_TARGET_NAME
  { ERROR_WRONG_TARGET_NAME, &(O(object_Kerror_wrong_target_name)) },
 #endif
 #ifdef ERROR_MUTUAL_AUTH_FAILED
  { ERROR_MUTUAL_AUTH_FAILED, &(O(object_Kerror_mutual_auth_failed)) },
 #endif
 #ifdef ERROR_TIME_SKEW
  { ERROR_TIME_SKEW, &(O(object_Kerror_time_skew)) },
 #endif
 #ifdef ERROR_CURRENT_DOMAIN_NOT_ALLOWED
  { ERROR_CURRENT_DOMAIN_NOT_ALLOWED, &(O(object_Kerror_current_domain_not_allowed)) },
 #endif
 #ifdef ERROR_INVALID_WINDOW_HANDLE
  { ERROR_INVALID_WINDOW_HANDLE, &(O(object_Kerror_invalid_window_handle)) },
 #endif
 #ifdef ERROR_INVALID_MENU_HANDLE
  { ERROR_INVALID_MENU_HANDLE, &(O(object_Kerror_invalid_menu_handle)) },
 #endif
 #ifdef ERROR_INVALID_CURSOR_HANDLE
  { ERROR_INVALID_CURSOR_HANDLE, &(O(object_Kerror_invalid_cursor_handle)) },
 #endif
 #ifdef ERROR_INVALID_ACCEL_HANDLE
  { ERROR_INVALID_ACCEL_HANDLE, &(O(object_Kerror_invalid_accel_handle)) },
 #endif
 #ifdef ERROR_INVALID_HOOK_HANDLE
  { ERROR_INVALID_HOOK_HANDLE, &(O(object_Kerror_invalid_hook_handle)) },
 #endif
 #ifdef ERROR_INVALID_DWP_HANDLE
  { ERROR_INVALID_DWP_HANDLE, &(O(object_Kerror_invalid_dwp_handle)) },
 #endif
 #ifdef ERROR_TLW_WITH_WSCHILD
  { ERROR_TLW_WITH_WSCHILD, &(O(object_Kerror_tlw_with_wschild)) },
 #endif
 #ifdef ERROR_CANNOT_FIND_WND_CLASS
  { ERROR_CANNOT_FIND_WND_CLASS, &(O(object_Kerror_cannot_find_wnd_class)) },
 #endif
 #ifdef ERROR_WINDOW_OF_OTHER_THREAD
  { ERROR_WINDOW_OF_OTHER_THREAD, &(O(object_Kerror_window_of_other_thread)) },
 #endif
 #ifdef ERROR_HOTKEY_ALREADY_REGISTERED
  { ERROR_HOTKEY_ALREADY_REGISTERED, &(O(object_Kerror_hotkey_already_registered)) },
 #endif
 #ifdef ERROR_CLASS_ALREADY_EXISTS
  { ERROR_CLASS_ALREADY_EXISTS, &(O(object_Kerror_class_already_exists)) },
 #endif
 #ifdef ERROR_CLASS_DOES_NOT_EXIST
  { ERROR_CLASS_DOES_NOT_EXIST, &(O(object_Kerror_class_does_not_exist)) },
 #endif
 #ifdef ERROR_CLASS_HAS_WINDOWS
  { ERROR_CLASS_HAS_WINDOWS, &(O(object_Kerror_class_has_windows)) },
 #endif
 #ifdef ERROR_INVALID_INDEX
  { ERROR_INVALID_INDEX, &(O(object_Kerror_invalid_index)) },
 #endif
 #ifdef ERROR_INVALID_ICON_HANDLE
  { ERROR_INVALID_ICON_HANDLE, &(O(object_Kerror_invalid_icon_handle)) },
 #endif
 #ifdef ERROR_PRIVATE_DIALOG_INDEX
  { ERROR_PRIVATE_DIALOG_INDEX, &(O(object_Kerror_private_dialog_index)) },
 #endif
 #ifdef ERROR_LISTBOX_ID_NOT_FOUND
  { ERROR_LISTBOX_ID_NOT_FOUND, &(O(object_Kerror_listbox_id_not_found)) },
 #endif
 #ifdef ERROR_NO_WILDCARD_CHARACTERS
  { ERROR_NO_WILDCARD_CHARACTERS, &(O(object_Kerror_no_wildcard_characters)) },
 #endif
 #ifdef ERROR_CLIPBOARD_NOT_OPEN
  { ERROR_CLIPBOARD_NOT_OPEN, &(O(object_Kerror_clipboard_not_open)) },
 #endif
 #ifdef ERROR_HOTKEY_NOT_REGISTERED
  { ERROR_HOTKEY_NOT_REGISTERED, &(O(object_Kerror_hotkey_not_registered)) },
 #endif
 #ifdef ERROR_WINDOW_NOT_DIALOG
  { ERROR_WINDOW_NOT_DIALOG, &(O(object_Kerror_window_not_dialog)) },
 #endif
 #ifdef ERROR_CONTROL_ID_NOT_FOUND
  { ERROR_CONTROL_ID_NOT_FOUND, &(O(object_Kerror_control_id_not_found)) },
 #endif
 #ifdef ERROR_INVALID_COMBOBOX_MESSAGE
  { ERROR_INVALID_COMBOBOX_MESSAGE, &(O(object_Kerror_invalid_combobox_message)) },
 #endif
 #ifdef ERROR_WINDOW_NOT_COMBOBOX
  { ERROR_WINDOW_NOT_COMBOBOX, &(O(object_Kerror_window_not_combobox)) },
 #endif
 #ifdef ERROR_INVALID_EDIT_HEIGHT
  { ERROR_INVALID_EDIT_HEIGHT, &(O(object_Kerror_invalid_edit_height)) },
 #endif
 #ifdef ERROR_DC_NOT_FOUND
  { ERROR_DC_NOT_FOUND, &(O(object_Kerror_dc_not_found)) },
 #endif
 #ifdef ERROR_INVALID_HOOK_FILTER
  { ERROR_INVALID_HOOK_FILTER, &(O(object_Kerror_invalid_hook_filter)) },
 #endif
 #ifdef ERROR_INVALID_FILTER_PROC
  { ERROR_INVALID_FILTER_PROC, &(O(object_Kerror_invalid_filter_proc)) },
 #endif
 #ifdef ERROR_HOOK_NEEDS_HMOD
  { ERROR_HOOK_NEEDS_HMOD, &(O(object_Kerror_hook_needs_hmod)) },
 #endif
 #ifdef ERROR_GLOBAL_ONLY_HOOK
  { ERROR_GLOBAL_ONLY_HOOK, &(O(object_Kerror_global_only_hook)) },
 #endif
 #ifdef ERROR_JOURNAL_HOOK_SET
  { ERROR_JOURNAL_HOOK_SET, &(O(object_Kerror_journal_hook_set)) },
 #endif
 #ifdef ERROR_HOOK_NOT_INSTALLED
  { ERROR_HOOK_NOT_INSTALLED, &(O(object_Kerror_hook_not_installed)) },
 #endif
 #ifdef ERROR_INVALID_LB_MESSAGE
  { ERROR_INVALID_LB_MESSAGE, &(O(object_Kerror_invalid_lb_message)) },
 #endif
 #ifdef ERROR_SETCOUNT_ON_BAD_LB
  { ERROR_SETCOUNT_ON_BAD_LB, &(O(object_Kerror_setcount_on_bad_lb)) },
 #endif
 #ifdef ERROR_LB_WITHOUT_TABSTOPS
  { ERROR_LB_WITHOUT_TABSTOPS, &(O(object_Kerror_lb_without_tabstops)) },
 #endif
 #ifdef ERROR_DESTROY_OBJECT_OF_OTHER_THREAD
  { ERROR_DESTROY_OBJECT_OF_OTHER_THREAD, &(O(object_Kerror_destroy_object_of_other_thread)) },
 #endif
 #ifdef ERROR_CHILD_WINDOW_MENU
  { ERROR_CHILD_WINDOW_MENU, &(O(object_Kerror_child_window_menu)) },
 #endif
 #ifdef ERROR_NO_SYSTEM_MENU
  { ERROR_NO_SYSTEM_MENU, &(O(object_Kerror_no_system_menu)) },
 #endif
 #ifdef ERROR_INVALID_MSGBOX_STYLE
  { ERROR_INVALID_MSGBOX_STYLE, &(O(object_Kerror_invalid_msgbox_style)) },
 #endif
 #ifdef ERROR_INVALID_SPI_VALUE
  { ERROR_INVALID_SPI_VALUE, &(O(object_Kerror_invalid_spi_value)) },
 #endif
 #ifdef ERROR_SCREEN_ALREADY_LOCKED
  { ERROR_SCREEN_ALREADY_LOCKED, &(O(object_Kerror_screen_already_locked)) },
 #endif
 #ifdef ERROR_HWNDS_HAVE_DIFF_PARENT
  { ERROR_HWNDS_HAVE_DIFF_PARENT, &(O(object_Kerror_hwnds_have_diff_parent)) },
 #endif
 #ifdef ERROR_NOT_CHILD_WINDOW
  { ERROR_NOT_CHILD_WINDOW, &(O(object_Kerror_not_child_window)) },
 #endif
 #ifdef ERROR_INVALID_GW_COMMAND
  { ERROR_INVALID_GW_COMMAND, &(O(object_Kerror_invalid_gw_command)) },
 #endif
 #ifdef ERROR_INVALID_THREAD_ID
  { ERROR_INVALID_THREAD_ID, &(O(object_Kerror_invalid_thread_id)) },
 #endif
 #ifdef ERROR_NON_MDICHILD_WINDOW
  { ERROR_NON_MDICHILD_WINDOW, &(O(object_Kerror_non_mdichild_window)) },
 #endif
 #ifdef ERROR_POPUP_ALREADY_ACTIVE
  { ERROR_POPUP_ALREADY_ACTIVE, &(O(object_Kerror_popup_already_active)) },
 #endif
 #ifdef ERROR_NO_SCROLLBARS
  { ERROR_NO_SCROLLBARS, &(O(object_Kerror_no_scrollbars)) },
 #endif
 #ifdef ERROR_INVALID_SCROLLBAR_RANGE
  { ERROR_INVALID_SCROLLBAR_RANGE, &(O(object_Kerror_invalid_scrollbar_range)) },
 #endif
 #ifdef ERROR_INVALID_SHOWWIN_COMMAND
  { ERROR_INVALID_SHOWWIN_COMMAND, &(O(object_Kerror_invalid_showwin_command)) },
 #endif
 #ifdef ERROR_NO_SYSTEM_RESOURCES
  { ERROR_NO_SYSTEM_RESOURCES, &(O(object_Kerror_no_system_resources)) },
 #endif
 #ifdef ERROR_NONPAGED_SYSTEM_RESOURCES
  { ERROR_NONPAGED_SYSTEM_RESOURCES, &(O(object_Kerror_nonpaged_system_resources)) },
 #endif
 #ifdef ERROR_PAGED_SYSTEM_RESOURCES
  { ERROR_PAGED_SYSTEM_RESOURCES, &(O(object_Kerror_paged_system_resources)) },
 #endif
 #ifdef ERROR_WORKING_SET_QUOTA
  { ERROR_WORKING_SET_QUOTA, &(O(object_Kerror_working_set_quota)) },
 #endif
 #ifdef ERROR_PAGEFILE_QUOTA
  { ERROR_PAGEFILE_QUOTA, &(O(object_Kerror_pagefile_quota)) },
 #endif
 #ifdef ERROR_COMMITMENT_LIMIT
  { ERROR_COMMITMENT_LIMIT, &(O(object_Kerror_commitment_limit)) },
 #endif
 #ifdef ERROR_MENU_ITEM_NOT_FOUND
  { ERROR_MENU_ITEM_NOT_FOUND, &(O(object_Kerror_menu_item_not_found)) },
 #endif
 #ifdef ERROR_INVALID_KEYBOARD_HANDLE
  { ERROR_INVALID_KEYBOARD_HANDLE, &(O(object_Kerror_invalid_keyboard_handle)) },
 #endif
 #ifdef ERROR_HOOK_TYPE_NOT_ALLOWED
  { ERROR_HOOK_TYPE_NOT_ALLOWED, &(O(object_Kerror_hook_type_not_allowed)) },
 #endif
 #ifdef ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION
  { ERROR_REQUIRES_INTERACTIVE_WINDOWSTATION, &(O(object_Kerror_requires_interactive_windowstation)) },
 #endif
 #ifdef ERROR_TIMEOUT
  { ERROR_TIMEOUT, &(O(object_Kerror_timeout)) },
 #endif
 #ifdef ERROR_INVALID_MONITOR_HANDLE
  { ERROR_INVALID_MONITOR_HANDLE, &(O(object_Kerror_invalid_monitor_handle)) },
 #endif
 #ifdef ERROR_EVENTLOG_FILE_CORRUPT
  { ERROR_EVENTLOG_FILE_CORRUPT, &(O(object_Kerror_eventlog_file_corrupt)) },
 #endif
 #ifdef ERROR_EVENTLOG_CANT_START
  { ERROR_EVENTLOG_CANT_START, &(O(object_Kerror_eventlog_cant_start)) },
 #endif
 #ifdef ERROR_LOG_FILE_FULL
  { ERROR_LOG_FILE_FULL, &(O(object_Kerror_log_file_full)) },
 #endif
 #ifdef ERROR_EVENTLOG_FILE_CHANGED
  { ERROR_EVENTLOG_FILE_CHANGED, &(O(object_Kerror_eventlog_file_changed)) },
 #endif
 #ifdef ERROR_INSTALL_SERVICE_FAILURE
  { ERROR_INSTALL_SERVICE_FAILURE, &(O(object_Kerror_install_service_failure)) },
 #endif
 #ifdef ERROR_INSTALL_USEREXIT
  { ERROR_INSTALL_USEREXIT, &(O(object_Kerror_install_userexit)) },
 #endif
 #ifdef ERROR_INSTALL_FAILURE
  { ERROR_INSTALL_FAILURE, &(O(object_Kerror_install_failure)) },
 #endif
 #ifdef ERROR_INSTALL_SUSPEND
  { ERROR_INSTALL_SUSPEND, &(O(object_Kerror_install_suspend)) },
 #endif
 #ifdef ERROR_UNKNOWN_PRODUCT
  { ERROR_UNKNOWN_PRODUCT, &(O(object_Kerror_unknown_product)) },
 #endif
 #ifdef ERROR_UNKNOWN_FEATURE
  { ERROR_UNKNOWN_FEATURE, &(O(object_Kerror_unknown_feature)) },
 #endif
 #ifdef ERROR_UNKNOWN_COMPONENT
  { ERROR_UNKNOWN_COMPONENT, &(O(object_Kerror_unknown_component)) },
 #endif
 #ifdef ERROR_UNKNOWN_PROPERTY
  { ERROR_UNKNOWN_PROPERTY, &(O(object_Kerror_unknown_property)) },
 #endif
 #ifdef ERROR_INVALID_HANDLE_STATE
  { ERROR_INVALID_HANDLE_STATE, &(O(object_Kerror_invalid_handle_state)) },
 #endif
 #ifdef ERROR_BAD_CONFIGURATION
  { ERROR_BAD_CONFIGURATION, &(O(object_Kerror_bad_configuration)) },
 #endif
 #ifdef ERROR_INDEX_ABSENT
  { ERROR_INDEX_ABSENT, &(O(object_Kerror_index_absent)) },
 #endif
 #ifdef ERROR_INSTALL_SOURCE_ABSENT
  { ERROR_INSTALL_SOURCE_ABSENT, &(O(object_Kerror_install_source_absent)) },
 #endif
 #ifdef ERROR_INSTALL_PACKAGE_VERSION
  { ERROR_INSTALL_PACKAGE_VERSION, &(O(object_Kerror_install_package_version)) },
 #endif
 #ifdef ERROR_PRODUCT_UNINSTALLED
  { ERROR_PRODUCT_UNINSTALLED, &(O(object_Kerror_product_uninstalled)) },
 #endif
 #ifdef ERROR_BAD_QUERY_SYNTAX
  { ERROR_BAD_QUERY_SYNTAX, &(O(object_Kerror_bad_query_syntax)) },
 #endif
 #ifdef ERROR_INVALID_FIELD
  { ERROR_INVALID_FIELD, &(O(object_Kerror_invalid_field)) },
 #endif
 #ifdef ERROR_DEVICE_REMOVED
  { ERROR_DEVICE_REMOVED, &(O(object_Kerror_device_removed)) },
 #endif
 #ifdef ERROR_INSTALL_ALREADY_RUNNING
  { ERROR_INSTALL_ALREADY_RUNNING, &(O(object_Kerror_install_already_running)) },
 #endif
 #ifdef ERROR_INSTALL_PACKAGE_OPEN_FAILED
  { ERROR_INSTALL_PACKAGE_OPEN_FAILED, &(O(object_Kerror_install_package_open_failed)) },
 #endif
 #ifdef ERROR_INSTALL_PACKAGE_INVALID
  { ERROR_INSTALL_PACKAGE_INVALID, &(O(object_Kerror_install_package_invalid)) },
 #endif
 #ifdef ERROR_INSTALL_UI_FAILURE
  { ERROR_INSTALL_UI_FAILURE, &(O(object_Kerror_install_ui_failure)) },
 #endif
 #ifdef ERROR_INSTALL_LOG_FAILURE
  { ERROR_INSTALL_LOG_FAILURE, &(O(object_Kerror_install_log_failure)) },
 #endif
 #ifdef ERROR_INSTALL_LANGUAGE_UNSUPPORTED
  { ERROR_INSTALL_LANGUAGE_UNSUPPORTED, &(O(object_Kerror_install_language_unsupported)) },
 #endif
 #ifdef ERROR_INSTALL_TRANSFORM_FAILURE
  { ERROR_INSTALL_TRANSFORM_FAILURE, &(O(object_Kerror_install_transform_failure)) },
 #endif
 #ifdef ERROR_INSTALL_PACKAGE_REJECTED
  { ERROR_INSTALL_PACKAGE_REJECTED, &(O(object_Kerror_install_package_rejected)) },
 #endif
 #ifdef ERROR_FUNCTION_NOT_CALLED
  { ERROR_FUNCTION_NOT_CALLED, &(O(object_Kerror_function_not_called)) },
 #endif
 #ifdef ERROR_FUNCTION_FAILED
  { ERROR_FUNCTION_FAILED, &(O(object_Kerror_function_failed)) },
 #endif
 #ifdef ERROR_INVALID_TABLE
  { ERROR_INVALID_TABLE, &(O(object_Kerror_invalid_table)) },
 #endif
 #ifdef ERROR_DATATYPE_MISMATCH
  { ERROR_DATATYPE_MISMATCH, &(O(object_Kerror_datatype_mismatch)) },
 #endif
 #ifdef ERROR_UNSUPPORTED_TYPE
  { ERROR_UNSUPPORTED_TYPE, &(O(object_Kerror_unsupported_type)) },
 #endif
 #ifdef ERROR_CREATE_FAILED
  { ERROR_CREATE_FAILED, &(O(object_Kerror_create_failed)) },
 #endif
 #ifdef ERROR_INSTALL_TEMP_UNWRITABLE
  { ERROR_INSTALL_TEMP_UNWRITABLE, &(O(object_Kerror_install_temp_unwritable)) },
 #endif
 #ifdef ERROR_INSTALL_PLATFORM_UNSUPPORTED
  { ERROR_INSTALL_PLATFORM_UNSUPPORTED, &(O(object_Kerror_install_platform_unsupported)) },
 #endif
 #ifdef ERROR_INSTALL_NOTUSED
  { ERROR_INSTALL_NOTUSED, &(O(object_Kerror_install_notused)) },
 #endif
 #ifdef ERROR_PATCH_PACKAGE_OPEN_FAILED
  { ERROR_PATCH_PACKAGE_OPEN_FAILED, &(O(object_Kerror_patch_package_open_failed)) },
 #endif
 #ifdef ERROR_PATCH_PACKAGE_INVALID
  { ERROR_PATCH_PACKAGE_INVALID, &(O(object_Kerror_patch_package_invalid)) },
 #endif
 #ifdef ERROR_PATCH_PACKAGE_UNSUPPORTED
  { ERROR_PATCH_PACKAGE_UNSUPPORTED, &(O(object_Kerror_patch_package_unsupported)) },
 #endif
 #ifdef ERROR_PRODUCT_VERSION
  { ERROR_PRODUCT_VERSION, &(O(object_Kerror_product_version)) },
 #endif
 #ifdef ERROR_INVALID_COMMAND_LINE
  { ERROR_INVALID_COMMAND_LINE, &(O(object_Kerror_invalid_command_line)) },
 #endif
 #ifdef ERROR_INSTALL_REMOTE_DISALLOWED
  { ERROR_INSTALL_REMOTE_DISALLOWED, &(O(object_Kerror_install_remote_disallowed)) },
 #endif
 #ifdef ERROR_SUCCESS_REBOOT_INITIATED
  { ERROR_SUCCESS_REBOOT_INITIATED, &(O(object_Kerror_success_reboot_initiated)) },
 #endif
 #ifdef ERROR_PATCH_TARGET_NOT_FOUND
  { ERROR_PATCH_TARGET_NOT_FOUND, &(O(object_Kerror_patch_target_not_found)) },
 #endif
 #ifdef ERROR_PATCH_PACKAGE_REJECTED
  { ERROR_PATCH_PACKAGE_REJECTED, &(O(object_Kerror_patch_package_rejected)) },
 #endif
 #ifdef ERROR_INSTALL_TRANSFORM_REJECTED
  { ERROR_INSTALL_TRANSFORM_REJECTED, &(O(object_Kerror_install_transform_rejected)) },
 #endif
 #ifdef ERROR_INSTALL_REMOTE_PROHIBITED
  { ERROR_INSTALL_REMOTE_PROHIBITED, &(O(object_Kerror_install_remote_prohibited)) },
 #endif
 #ifdef RPC_S_INVALID_STRING_BINDING
  { RPC_S_INVALID_STRING_BINDING, &(O(object_Krpc_s_invalid_string_binding)) },
 #endif
 #ifdef RPC_S_WRONG_KIND_OF_BINDING
  { RPC_S_WRONG_KIND_OF_BINDING, &(O(object_Krpc_s_wrong_kind_of_binding)) },
 #endif
 #ifdef RPC_S_INVALID_BINDING
  { RPC_S_INVALID_BINDING, &(O(object_Krpc_s_invalid_binding)) },
 #endif
 #ifdef RPC_S_PROTSEQ_NOT_SUPPORTED
  { RPC_S_PROTSEQ_NOT_SUPPORTED, &(O(object_Krpc_s_protseq_not_supported)) },
 #endif
 #ifdef RPC_S_INVALID_RPC_PROTSEQ
  { RPC_S_INVALID_RPC_PROTSEQ, &(O(object_Krpc_s_invalid_rpc_protseq)) },
 #endif
 #ifdef RPC_S_INVALID_STRING_UUID
  { RPC_S_INVALID_STRING_UUID, &(O(object_Krpc_s_invalid_string_uuid)) },
 #endif
 #ifdef RPC_S_INVALID_ENDPOINT_FORMAT
  { RPC_S_INVALID_ENDPOINT_FORMAT, &(O(object_Krpc_s_invalid_endpoint_format)) },
 #endif
 #ifdef RPC_S_INVALID_NET_ADDR
  { RPC_S_INVALID_NET_ADDR, &(O(object_Krpc_s_invalid_net_addr)) },
 #endif
 #ifdef RPC_S_NO_ENDPOINT_FOUND
  { RPC_S_NO_ENDPOINT_FOUND, &(O(object_Krpc_s_no_endpoint_found)) },
 #endif
 #ifdef RPC_S_INVALID_TIMEOUT
  { RPC_S_INVALID_TIMEOUT, &(O(object_Krpc_s_invalid_timeout)) },
 #endif
 #ifdef RPC_S_OBJECT_NOT_FOUND
  { RPC_S_OBJECT_NOT_FOUND, &(O(object_Krpc_s_object_not_found)) },
 #endif
 #ifdef RPC_S_ALREADY_REGISTERED
  { RPC_S_ALREADY_REGISTERED, &(O(object_Krpc_s_already_registered)) },
 #endif
 #ifdef RPC_S_TYPE_ALREADY_REGISTERED
  { RPC_S_TYPE_ALREADY_REGISTERED, &(O(object_Krpc_s_type_already_registered)) },
 #endif
 #ifdef RPC_S_ALREADY_LISTENING
  { RPC_S_ALREADY_LISTENING, &(O(object_Krpc_s_already_listening)) },
 #endif
 #ifdef RPC_S_NO_PROTSEQS_REGISTERED
  { RPC_S_NO_PROTSEQS_REGISTERED, &(O(object_Krpc_s_no_protseqs_registered)) },
 #endif
 #ifdef RPC_S_NOT_LISTENING
  { RPC_S_NOT_LISTENING, &(O(object_Krpc_s_not_listening)) },
 #endif
 #ifdef RPC_S_UNKNOWN_MGR_TYPE
  { RPC_S_UNKNOWN_MGR_TYPE, &(O(object_Krpc_s_unknown_mgr_type)) },
 #endif
 #ifdef RPC_S_UNKNOWN_IF
  { RPC_S_UNKNOWN_IF, &(O(object_Krpc_s_unknown_if)) },
 #endif
 #ifdef RPC_S_NO_BINDINGS
  { RPC_S_NO_BINDINGS, &(O(object_Krpc_s_no_bindings)) },
 #endif
 #ifdef RPC_S_NO_PROTSEQS
  { RPC_S_NO_PROTSEQS, &(O(object_Krpc_s_no_protseqs)) },
 #endif
 #ifdef RPC_S_CANT_CREATE_ENDPOINT
  { RPC_S_CANT_CREATE_ENDPOINT, &(O(object_Krpc_s_cant_create_endpoint)) },
 #endif
 #ifdef RPC_S_OUT_OF_RESOURCES
  { RPC_S_OUT_OF_RESOURCES, &(O(object_Krpc_s_out_of_resources)) },
 #endif
 #ifdef RPC_S_SERVER_UNAVAILABLE
  { RPC_S_SERVER_UNAVAILABLE, &(O(object_Krpc_s_server_unavailable)) },
 #endif
 #ifdef RPC_S_SERVER_TOO_BUSY
  { RPC_S_SERVER_TOO_BUSY, &(O(object_Krpc_s_server_too_busy)) },
 #endif
 #ifdef RPC_S_INVALID_NETWORK_OPTIONS
  { RPC_S_INVALID_NETWORK_OPTIONS, &(O(object_Krpc_s_invalid_network_options)) },
 #endif
 #ifdef RPC_S_NO_CALL_ACTIVE
  { RPC_S_NO_CALL_ACTIVE, &(O(object_Krpc_s_no_call_active)) },
 #endif
 #ifdef RPC_S_CALL_FAILED
  { RPC_S_CALL_FAILED, &(O(object_Krpc_s_call_failed)) },
 #endif
 #ifdef RPC_S_CALL_FAILED_DNE
  { RPC_S_CALL_FAILED_DNE, &(O(object_Krpc_s_call_failed_dne)) },
 #endif
 #ifdef RPC_S_PROTOCOL_ERROR
  { RPC_S_PROTOCOL_ERROR, &(O(object_Krpc_s_protocol_error)) },
 #endif
 #ifdef RPC_S_UNSUPPORTED_TRANS_SYN
  { RPC_S_UNSUPPORTED_TRANS_SYN, &(O(object_Krpc_s_unsupported_trans_syn)) },
 #endif
 #ifdef RPC_S_UNSUPPORTED_TYPE
  { RPC_S_UNSUPPORTED_TYPE, &(O(object_Krpc_s_unsupported_type)) },
 #endif
 #ifdef RPC_S_INVALID_TAG
  { RPC_S_INVALID_TAG, &(O(object_Krpc_s_invalid_tag)) },
 #endif
 #ifdef RPC_S_INVALID_BOUND
  { RPC_S_INVALID_BOUND, &(O(object_Krpc_s_invalid_bound)) },
 #endif
 #ifdef RPC_S_NO_ENTRY_NAME
  { RPC_S_NO_ENTRY_NAME, &(O(object_Krpc_s_no_entry_name)) },
 #endif
 #ifdef RPC_S_INVALID_NAME_SYNTAX
  { RPC_S_INVALID_NAME_SYNTAX, &(O(object_Krpc_s_invalid_name_syntax)) },
 #endif
 #ifdef RPC_S_UNSUPPORTED_NAME_SYNTAX
  { RPC_S_UNSUPPORTED_NAME_SYNTAX, &(O(object_Krpc_s_unsupported_name_syntax)) },
 #endif
 #ifdef RPC_S_UUID_NO_ADDRESS
  { RPC_S_UUID_NO_ADDRESS, &(O(object_Krpc_s_uuid_no_address)) },
 #endif
 #ifdef RPC_S_DUPLICATE_ENDPOINT
  { RPC_S_DUPLICATE_ENDPOINT, &(O(object_Krpc_s_duplicate_endpoint)) },
 #endif
 #ifdef RPC_S_UNKNOWN_AUTHN_TYPE
  { RPC_S_UNKNOWN_AUTHN_TYPE, &(O(object_Krpc_s_unknown_authn_type)) },
 #endif
 #ifdef RPC_S_MAX_CALLS_TOO_SMALL
  { RPC_S_MAX_CALLS_TOO_SMALL, &(O(object_Krpc_s_max_calls_too_small)) },
 #endif
 #ifdef RPC_S_STRING_TOO_LONG
  { RPC_S_STRING_TOO_LONG, &(O(object_Krpc_s_string_too_long)) },
 #endif
 #ifdef RPC_S_PROTSEQ_NOT_FOUND
  { RPC_S_PROTSEQ_NOT_FOUND, &(O(object_Krpc_s_protseq_not_found)) },
 #endif
 #ifdef RPC_S_PROCNUM_OUT_OF_RANGE
  { RPC_S_PROCNUM_OUT_OF_RANGE, &(O(object_Krpc_s_procnum_out_of_range)) },
 #endif
 #ifdef RPC_S_BINDING_HAS_NO_AUTH
  { RPC_S_BINDING_HAS_NO_AUTH, &(O(object_Krpc_s_binding_has_no_auth)) },
 #endif
 #ifdef RPC_S_UNKNOWN_AUTHN_SERVICE
  { RPC_S_UNKNOWN_AUTHN_SERVICE, &(O(object_Krpc_s_unknown_authn_service)) },
 #endif
 #ifdef RPC_S_UNKNOWN_AUTHN_LEVEL
  { RPC_S_UNKNOWN_AUTHN_LEVEL, &(O(object_Krpc_s_unknown_authn_level)) },
 #endif
 #ifdef RPC_S_INVALID_AUTH_IDENTITY
  { RPC_S_INVALID_AUTH_IDENTITY, &(O(object_Krpc_s_invalid_auth_identity)) },
 #endif
 #ifdef RPC_S_UNKNOWN_AUTHZ_SERVICE
  { RPC_S_UNKNOWN_AUTHZ_SERVICE, &(O(object_Krpc_s_unknown_authz_service)) },
 #endif
 #ifdef EPT_S_INVALID_ENTRY
  { EPT_S_INVALID_ENTRY, &(O(object_Kept_s_invalid_entry)) },
 #endif
 #ifdef EPT_S_CANT_PERFORM_OP
  { EPT_S_CANT_PERFORM_OP, &(O(object_Kept_s_cant_perform_op)) },
 #endif
 #ifdef EPT_S_NOT_REGISTERED
  { EPT_S_NOT_REGISTERED, &(O(object_Kept_s_not_registered)) },
 #endif
 #ifdef RPC_S_NOTHING_TO_EXPORT
  { RPC_S_NOTHING_TO_EXPORT, &(O(object_Krpc_s_nothing_to_export)) },
 #endif
 #ifdef RPC_S_INCOMPLETE_NAME
  { RPC_S_INCOMPLETE_NAME, &(O(object_Krpc_s_incomplete_name)) },
 #endif
 #ifdef RPC_S_INVALID_VERS_OPTION
  { RPC_S_INVALID_VERS_OPTION, &(O(object_Krpc_s_invalid_vers_option)) },
 #endif
 #ifdef RPC_S_NO_MORE_MEMBERS
  { RPC_S_NO_MORE_MEMBERS, &(O(object_Krpc_s_no_more_members)) },
 #endif
 #ifdef RPC_S_NOT_ALL_OBJS_UNEXPORTED
  { RPC_S_NOT_ALL_OBJS_UNEXPORTED, &(O(object_Krpc_s_not_all_objs_unexported)) },
 #endif
 #ifdef RPC_S_INTERFACE_NOT_FOUND
  { RPC_S_INTERFACE_NOT_FOUND, &(O(object_Krpc_s_interface_not_found)) },
 #endif
 #ifdef RPC_S_ENTRY_ALREADY_EXISTS
  { RPC_S_ENTRY_ALREADY_EXISTS, &(O(object_Krpc_s_entry_already_exists)) },
 #endif
 #ifdef RPC_S_ENTRY_NOT_FOUND
  { RPC_S_ENTRY_NOT_FOUND, &(O(object_Krpc_s_entry_not_found)) },
 #endif
 #ifdef RPC_S_NAME_SERVICE_UNAVAILABLE
  { RPC_S_NAME_SERVICE_UNAVAILABLE, &(O(object_Krpc_s_name_service_unavailable)) },
 #endif
 #ifdef RPC_S_INVALID_NAF_ID
  { RPC_S_INVALID_NAF_ID, &(O(object_Krpc_s_invalid_naf_id)) },
 #endif
 #ifdef RPC_S_CANNOT_SUPPORT
  { RPC_S_CANNOT_SUPPORT, &(O(object_Krpc_s_cannot_support)) },
 #endif
 #ifdef RPC_S_NO_CONTEXT_AVAILABLE
  { RPC_S_NO_CONTEXT_AVAILABLE, &(O(object_Krpc_s_no_context_available)) },
 #endif
 #ifdef RPC_S_INTERNAL_ERROR
  { RPC_S_INTERNAL_ERROR, &(O(object_Krpc_s_internal_error)) },
 #endif
 #ifdef RPC_S_ZERO_DIVIDE
  { RPC_S_ZERO_DIVIDE, &(O(object_Krpc_s_zero_divide)) },
 #endif
 #ifdef RPC_S_ADDRESS_ERROR
  { RPC_S_ADDRESS_ERROR, &(O(object_Krpc_s_address_error)) },
 #endif
 #ifdef RPC_S_FP_DIV_ZERO
  { RPC_S_FP_DIV_ZERO, &(O(object_Krpc_s_fp_div_zero)) },
 #endif
 #ifdef RPC_S_FP_UNDERFLOW
  { RPC_S_FP_UNDERFLOW, &(O(object_Krpc_s_fp_underflow)) },
 #endif
 #ifdef RPC_S_FP_OVERFLOW
  { RPC_S_FP_OVERFLOW, &(O(object_Krpc_s_fp_overflow)) },
 #endif
 #ifdef RPC_X_NO_MORE_ENTRIES
  { RPC_X_NO_MORE_ENTRIES, &(O(object_Krpc_x_no_more_entries)) },
 #endif
 #ifdef RPC_X_SS_CHAR_TRANS_OPEN_FAIL
  { RPC_X_SS_CHAR_TRANS_OPEN_FAIL, &(O(object_Krpc_x_ss_char_trans_open_fail)) },
 #endif
 #ifdef RPC_X_SS_CHAR_TRANS_SHORT_FILE
  { RPC_X_SS_CHAR_TRANS_SHORT_FILE, &(O(object_Krpc_x_ss_char_trans_short_file)) },
 #endif
 #ifdef RPC_X_SS_IN_NULL_CONTEXT
  { RPC_X_SS_IN_NULL_CONTEXT, &(O(object_Krpc_x_ss_in_null_context)) },
 #endif
 #ifdef RPC_X_SS_CONTEXT_DAMAGED
  { RPC_X_SS_CONTEXT_DAMAGED, &(O(object_Krpc_x_ss_context_damaged)) },
 #endif
 #ifdef RPC_X_SS_HANDLES_MISMATCH
  { RPC_X_SS_HANDLES_MISMATCH, &(O(object_Krpc_x_ss_handles_mismatch)) },
 #endif
 #ifdef RPC_X_SS_CANNOT_GET_CALL_HANDLE
  { RPC_X_SS_CANNOT_GET_CALL_HANDLE, &(O(object_Krpc_x_ss_cannot_get_call_handle)) },
 #endif
 #ifdef RPC_X_NULL_REF_POINTER
  { RPC_X_NULL_REF_POINTER, &(O(object_Krpc_x_null_ref_pointer)) },
 #endif
 #ifdef RPC_X_ENUM_VALUE_OUT_OF_RANGE
  { RPC_X_ENUM_VALUE_OUT_OF_RANGE, &(O(object_Krpc_x_enum_value_out_of_range)) },
 #endif
 #ifdef RPC_X_BYTE_COUNT_TOO_SMALL
  { RPC_X_BYTE_COUNT_TOO_SMALL, &(O(object_Krpc_x_byte_count_too_small)) },
 #endif
 #ifdef RPC_X_BAD_STUB_DATA
  { RPC_X_BAD_STUB_DATA, &(O(object_Krpc_x_bad_stub_data)) },
 #endif
 #ifdef ERROR_INVALID_USER_BUFFER
  { ERROR_INVALID_USER_BUFFER, &(O(object_Kerror_invalid_user_buffer)) },
 #endif
 #ifdef ERROR_UNRECOGNIZED_MEDIA
  { ERROR_UNRECOGNIZED_MEDIA, &(O(object_Kerror_unrecognized_media)) },
 #endif
 #ifdef ERROR_NO_TRUST_LSA_SECRET
  { ERROR_NO_TRUST_LSA_SECRET, &(O(object_Kerror_no_trust_lsa_secret)) },
 #endif
 #ifdef ERROR_NO_TRUST_SAM_ACCOUNT
  { ERROR_NO_TRUST_SAM_ACCOUNT, &(O(object_Kerror_no_trust_sam_account)) },
 #endif
 #ifdef ERROR_TRUSTED_DOMAIN_FAILURE
  { ERROR_TRUSTED_DOMAIN_FAILURE, &(O(object_Kerror_trusted_domain_failure)) },
 #endif
 #ifdef ERROR_TRUSTED_RELATIONSHIP_FAILURE
  { ERROR_TRUSTED_RELATIONSHIP_FAILURE, &(O(object_Kerror_trusted_relationship_failure)) },
 #endif
 #ifdef ERROR_TRUST_FAILURE
  { ERROR_TRUST_FAILURE, &(O(object_Kerror_trust_failure)) },
 #endif
 #ifdef RPC_S_CALL_IN_PROGRESS
  { RPC_S_CALL_IN_PROGRESS, &(O(object_Krpc_s_call_in_progress)) },
 #endif
 #ifdef ERROR_NETLOGON_NOT_STARTED
  { ERROR_NETLOGON_NOT_STARTED, &(O(object_Kerror_netlogon_not_started)) },
 #endif
 #ifdef ERROR_ACCOUNT_EXPIRED
  { ERROR_ACCOUNT_EXPIRED, &(O(object_Kerror_account_expired)) },
 #endif
 #ifdef ERROR_REDIRECTOR_HAS_OPEN_HANDLES
  { ERROR_REDIRECTOR_HAS_OPEN_HANDLES, &(O(object_Kerror_redirector_has_open_handles)) },
 #endif
 #ifdef ERROR_PRINTER_DRIVER_ALREADY_INSTALLED
  { ERROR_PRINTER_DRIVER_ALREADY_INSTALLED, &(O(object_Kerror_printer_driver_already_installed)) },
 #endif
 #ifdef ERROR_UNKNOWN_PORT
  { ERROR_UNKNOWN_PORT, &(O(object_Kerror_unknown_port)) },
 #endif
 #ifdef ERROR_UNKNOWN_PRINTER_DRIVER
  { ERROR_UNKNOWN_PRINTER_DRIVER, &(O(object_Kerror_unknown_printer_driver)) },
 #endif
 #ifdef ERROR_UNKNOWN_PRINTPROCESSOR
  { ERROR_UNKNOWN_PRINTPROCESSOR, &(O(object_Kerror_unknown_printprocessor)) },
 #endif
 #ifdef ERROR_INVALID_SEPARATOR_FILE
  { ERROR_INVALID_SEPARATOR_FILE, &(O(object_Kerror_invalid_separator_file)) },
 #endif
 #ifdef ERROR_INVALID_PRIORITY
  { ERROR_INVALID_PRIORITY, &(O(object_Kerror_invalid_priority)) },
 #endif
 #ifdef ERROR_INVALID_PRINTER_NAME
  { ERROR_INVALID_PRINTER_NAME, &(O(object_Kerror_invalid_printer_name)) },
 #endif
 #ifdef ERROR_PRINTER_ALREADY_EXISTS
  { ERROR_PRINTER_ALREADY_EXISTS, &(O(object_Kerror_printer_already_exists)) },
 #endif
 #ifdef ERROR_INVALID_PRINTER_COMMAND
  { ERROR_INVALID_PRINTER_COMMAND, &(O(object_Kerror_invalid_printer_command)) },
 #endif
 #ifdef ERROR_INVALID_DATATYPE
  { ERROR_INVALID_DATATYPE, &(O(object_Kerror_invalid_datatype)) },
 #endif
 #ifdef ERROR_INVALID_ENVIRONMENT
  { ERROR_INVALID_ENVIRONMENT, &(O(object_Kerror_invalid_environment)) },
 #endif
 #ifdef RPC_S_NO_MORE_BINDINGS
  { RPC_S_NO_MORE_BINDINGS, &(O(object_Krpc_s_no_more_bindings)) },
 #endif
 #ifdef ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT
  { ERROR_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT, &(O(object_Kerror_nologon_interdomain_trust_account)) },
 #endif
 #ifdef ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT
  { ERROR_NOLOGON_WORKSTATION_TRUST_ACCOUNT, &(O(object_Kerror_nologon_workstation_trust_account)) },
 #endif
 #ifdef ERROR_NOLOGON_SERVER_TRUST_ACCOUNT
  { ERROR_NOLOGON_SERVER_TRUST_ACCOUNT, &(O(object_Kerror_nologon_server_trust_account)) },
 #endif
 #ifdef ERROR_DOMAIN_TRUST_INCONSISTENT
  { ERROR_DOMAIN_TRUST_INCONSISTENT, &(O(object_Kerror_domain_trust_inconsistent)) },
 #endif
 #ifdef ERROR_SERVER_HAS_OPEN_HANDLES
  { ERROR_SERVER_HAS_OPEN_HANDLES, &(O(object_Kerror_server_has_open_handles)) },
 #endif
 #ifdef ERROR_RESOURCE_DATA_NOT_FOUND
  { ERROR_RESOURCE_DATA_NOT_FOUND, &(O(object_Kerror_resource_data_not_found)) },
 #endif
 #ifdef ERROR_RESOURCE_TYPE_NOT_FOUND
  { ERROR_RESOURCE_TYPE_NOT_FOUND, &(O(object_Kerror_resource_type_not_found)) },
 #endif
 #ifdef ERROR_RESOURCE_NAME_NOT_FOUND
  { ERROR_RESOURCE_NAME_NOT_FOUND, &(O(object_Kerror_resource_name_not_found)) },
 #endif
 #ifdef ERROR_RESOURCE_LANG_NOT_FOUND
  { ERROR_RESOURCE_LANG_NOT_FOUND, &(O(object_Kerror_resource_lang_not_found)) },
 #endif
 #ifdef ERROR_NOT_ENOUGH_QUOTA
  { ERROR_NOT_ENOUGH_QUOTA, &(O(object_Kerror_not_enough_quota)) },
 #endif
 #ifdef RPC_S_NO_INTERFACES
  { RPC_S_NO_INTERFACES, &(O(object_Krpc_s_no_interfaces)) },
 #endif
 #ifdef RPC_S_CALL_CANCELLED
  { RPC_S_CALL_CANCELLED, &(O(object_Krpc_s_call_cancelled)) },
 #endif
 #ifdef RPC_S_BINDING_INCOMPLETE
  { RPC_S_BINDING_INCOMPLETE, &(O(object_Krpc_s_binding_incomplete)) },
 #endif
 #ifdef RPC_S_COMM_FAILURE
  { RPC_S_COMM_FAILURE, &(O(object_Krpc_s_comm_failure)) },
 #endif
 #ifdef RPC_S_UNSUPPORTED_AUTHN_LEVEL
  { RPC_S_UNSUPPORTED_AUTHN_LEVEL, &(O(object_Krpc_s_unsupported_authn_level)) },
 #endif
 #ifdef RPC_S_NO_PRINC_NAME
  { RPC_S_NO_PRINC_NAME, &(O(object_Krpc_s_no_princ_name)) },
 #endif
 #ifdef RPC_S_NOT_RPC_ERROR
  { RPC_S_NOT_RPC_ERROR, &(O(object_Krpc_s_not_rpc_error)) },
 #endif
 #ifdef RPC_S_UUID_LOCAL_ONLY
  { RPC_S_UUID_LOCAL_ONLY, &(O(object_Krpc_s_uuid_local_only)) },
 #endif
 #ifdef RPC_S_SEC_PKG_ERROR
  { RPC_S_SEC_PKG_ERROR, &(O(object_Krpc_s_sec_pkg_error)) },
 #endif
 #ifdef RPC_S_NOT_CANCELLED
  { RPC_S_NOT_CANCELLED, &(O(object_Krpc_s_not_cancelled)) },
 #endif
 #ifdef RPC_X_INVALID_ES_ACTION
  { RPC_X_INVALID_ES_ACTION, &(O(object_Krpc_x_invalid_es_action)) },
 #endif
 #ifdef RPC_X_WRONG_ES_VERSION
  { RPC_X_WRONG_ES_VERSION, &(O(object_Krpc_x_wrong_es_version)) },
 #endif
 #ifdef RPC_X_WRONG_STUB_VERSION
  { RPC_X_WRONG_STUB_VERSION, &(O(object_Krpc_x_wrong_stub_version)) },
 #endif
 #ifdef RPC_X_INVALID_PIPE_OBJECT
  { RPC_X_INVALID_PIPE_OBJECT, &(O(object_Krpc_x_invalid_pipe_object)) },
 #endif
 #ifdef RPC_X_WRONG_PIPE_ORDER
  { RPC_X_WRONG_PIPE_ORDER, &(O(object_Krpc_x_wrong_pipe_order)) },
 #endif
 #ifdef RPC_X_WRONG_PIPE_VERSION
  { RPC_X_WRONG_PIPE_VERSION, &(O(object_Krpc_x_wrong_pipe_version)) },
 #endif
 #ifdef RPC_S_GROUP_MEMBER_NOT_FOUND
  { RPC_S_GROUP_MEMBER_NOT_FOUND, &(O(object_Krpc_s_group_member_not_found)) },
 #endif
 #ifdef EPT_S_CANT_CREATE
  { EPT_S_CANT_CREATE, &(O(object_Kept_s_cant_create)) },
 #endif
 #ifdef RPC_S_INVALID_OBJECT
  { RPC_S_INVALID_OBJECT, &(O(object_Krpc_s_invalid_object)) },
 #endif
 #ifdef ERROR_INVALID_TIME
  { ERROR_INVALID_TIME, &(O(object_Kerror_invalid_time)) },
 #endif
 #ifdef ERROR_INVALID_FORM_NAME
  { ERROR_INVALID_FORM_NAME, &(O(object_Kerror_invalid_form_name)) },
 #endif
 #ifdef ERROR_INVALID_FORM_SIZE
  { ERROR_INVALID_FORM_SIZE, &(O(object_Kerror_invalid_form_size)) },
 #endif
 #ifdef ERROR_ALREADY_WAITING
  { ERROR_ALREADY_WAITING, &(O(object_Kerror_already_waiting)) },
 #endif
 #ifdef ERROR_PRINTER_DELETED
  { ERROR_PRINTER_DELETED, &(O(object_Kerror_printer_deleted)) },
 #endif
 #ifdef ERROR_INVALID_PRINTER_STATE
  { ERROR_INVALID_PRINTER_STATE, &(O(object_Kerror_invalid_printer_state)) },
 #endif
 #ifdef ERROR_PASSWORD_MUST_CHANGE
  { ERROR_PASSWORD_MUST_CHANGE, &(O(object_Kerror_password_must_change)) },
 #endif
 #ifdef ERROR_DOMAIN_CONTROLLER_NOT_FOUND
  { ERROR_DOMAIN_CONTROLLER_NOT_FOUND, &(O(object_Kerror_domain_controller_not_found)) },
 #endif
 #ifdef ERROR_ACCOUNT_LOCKED_OUT
  { ERROR_ACCOUNT_LOCKED_OUT, &(O(object_Kerror_account_locked_out)) },
 #endif
 #ifdef OR_INVALID_OXID
  { OR_INVALID_OXID, &(O(object_Kor_invalid_oxid)) },
 #endif
 #ifdef OR_INVALID_OID
  { OR_INVALID_OID, &(O(object_Kor_invalid_oid)) },
 #endif
 #ifdef OR_INVALID_SET
  { OR_INVALID_SET, &(O(object_Kor_invalid_set)) },
 #endif
 #ifdef RPC_S_SEND_INCOMPLETE
  { RPC_S_SEND_INCOMPLETE, &(O(object_Krpc_s_send_incomplete)) },
 #endif
 #ifdef RPC_S_INVALID_ASYNC_HANDLE
  { RPC_S_INVALID_ASYNC_HANDLE, &(O(object_Krpc_s_invalid_async_handle)) },
 #endif
 #ifdef RPC_S_INVALID_ASYNC_CALL
  { RPC_S_INVALID_ASYNC_CALL, &(O(object_Krpc_s_invalid_async_call)) },
 #endif
 #ifdef RPC_X_PIPE_CLOSED
  { RPC_X_PIPE_CLOSED, &(O(object_Krpc_x_pipe_closed)) },
 #endif
 #ifdef RPC_X_PIPE_DISCIPLINE_ERROR
  { RPC_X_PIPE_DISCIPLINE_ERROR, &(O(object_Krpc_x_pipe_discipline_error)) },
 #endif
 #ifdef RPC_X_PIPE_EMPTY
  { RPC_X_PIPE_EMPTY, &(O(object_Krpc_x_pipe_empty)) },
 #endif
 #ifdef ERROR_NO_SITENAME
  { ERROR_NO_SITENAME, &(O(object_Kerror_no_sitename)) },
 #endif
 #ifdef ERROR_CANT_ACCESS_FILE
  { ERROR_CANT_ACCESS_FILE, &(O(object_Kerror_cant_access_file)) },
 #endif
 #ifdef ERROR_CANT_RESOLVE_FILENAME
  { ERROR_CANT_RESOLVE_FILENAME, &(O(object_Kerror_cant_resolve_filename)) },
 #endif
 #ifdef RPC_S_ENTRY_TYPE_MISMATCH
  { RPC_S_ENTRY_TYPE_MISMATCH, &(O(object_Krpc_s_entry_type_mismatch)) },
 #endif
 #ifdef RPC_S_NOT_ALL_OBJS_EXPORTED
  { RPC_S_NOT_ALL_OBJS_EXPORTED, &(O(object_Krpc_s_not_all_objs_exported)) },
 #endif
 #ifdef RPC_S_INTERFACE_NOT_EXPORTED
  { RPC_S_INTERFACE_NOT_EXPORTED, &(O(object_Krpc_s_interface_not_exported)) },
 #endif
 #ifdef RPC_S_PROFILE_NOT_ADDED
  { RPC_S_PROFILE_NOT_ADDED, &(O(object_Krpc_s_profile_not_added)) },
 #endif
 #ifdef RPC_S_PRF_ELT_NOT_ADDED
  { RPC_S_PRF_ELT_NOT_ADDED, &(O(object_Krpc_s_prf_elt_not_added)) },
 #endif
 #ifdef RPC_S_PRF_ELT_NOT_REMOVED
  { RPC_S_PRF_ELT_NOT_REMOVED, &(O(object_Krpc_s_prf_elt_not_removed)) },
 #endif
 #ifdef RPC_S_GRP_ELT_NOT_ADDED
  { RPC_S_GRP_ELT_NOT_ADDED, &(O(object_Krpc_s_grp_elt_not_added)) },
 #endif
 #ifdef RPC_S_GRP_ELT_NOT_REMOVED
  { RPC_S_GRP_ELT_NOT_REMOVED, &(O(object_Krpc_s_grp_elt_not_removed)) },
 #endif
 #ifdef ERROR_KM_DRIVER_BLOCKED
  { ERROR_KM_DRIVER_BLOCKED, &(O(object_Kerror_km_driver_blocked)) },
 #endif
 #ifdef ERROR_CONTEXT_EXPIRED
  { ERROR_CONTEXT_EXPIRED, &(O(object_Kerror_context_expired)) },
 #endif
 #ifdef ERROR_PER_USER_TRUST_QUOTA_EXCEEDED
  { ERROR_PER_USER_TRUST_QUOTA_EXCEEDED, &(O(object_Kerror_per_user_trust_quota_exceeded)) },
 #endif
 #ifdef ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED
  { ERROR_ALL_USER_TRUST_QUOTA_EXCEEDED, &(O(object_Kerror_all_user_trust_quota_exceeded)) },
 #endif
 #ifdef ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED
  { ERROR_USER_DELETE_TRUST_QUOTA_EXCEEDED, &(O(object_Kerror_user_delete_trust_quota_exceeded)) },
 #endif
 #ifdef ERROR_INVALID_PIXEL_FORMAT
  { ERROR_INVALID_PIXEL_FORMAT, &(O(object_Kerror_invalid_pixel_format)) },
 #endif
 #ifdef ERROR_BAD_DRIVER
  { ERROR_BAD_DRIVER, &(O(object_Kerror_bad_driver)) },
 #endif
 #ifdef ERROR_INVALID_WINDOW_STYLE
  { ERROR_INVALID_WINDOW_STYLE, &(O(object_Kerror_invalid_window_style)) },
 #endif
 #ifdef ERROR_METAFILE_NOT_SUPPORTED
  { ERROR_METAFILE_NOT_SUPPORTED, &(O(object_Kerror_metafile_not_supported)) },
 #endif
 #ifdef ERROR_TRANSFORM_NOT_SUPPORTED
  { ERROR_TRANSFORM_NOT_SUPPORTED, &(O(object_Kerror_transform_not_supported)) },
 #endif
 #ifdef ERROR_CLIPPING_NOT_SUPPORTED
  { ERROR_CLIPPING_NOT_SUPPORTED, &(O(object_Kerror_clipping_not_supported)) },
 #endif
 #ifdef ERROR_INVALID_CMM
  { ERROR_INVALID_CMM, &(O(object_Kerror_invalid_cmm)) },
 #endif
 #ifdef ERROR_INVALID_PROFILE
  { ERROR_INVALID_PROFILE, &(O(object_Kerror_invalid_profile)) },
 #endif
 #ifdef ERROR_TAG_NOT_FOUND
  { ERROR_TAG_NOT_FOUND, &(O(object_Kerror_tag_not_found)) },
 #endif
 #ifdef ERROR_TAG_NOT_PRESENT
  { ERROR_TAG_NOT_PRESENT, &(O(object_Kerror_tag_not_present)) },
 #endif
 #ifdef ERROR_DUPLICATE_TAG
  { ERROR_DUPLICATE_TAG, &(O(object_Kerror_duplicate_tag)) },
 #endif
 #ifdef ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE
  { ERROR_PROFILE_NOT_ASSOCIATED_WITH_DEVICE, &(O(object_Kerror_profile_not_associated_with_device)) },
 #endif
 #ifdef ERROR_PROFILE_NOT_FOUND
  { ERROR_PROFILE_NOT_FOUND, &(O(object_Kerror_profile_not_found)) },
 #endif
 #ifdef ERROR_INVALID_COLORSPACE
  { ERROR_INVALID_COLORSPACE, &(O(object_Kerror_invalid_colorspace)) },
 #endif
 #ifdef ERROR_ICM_NOT_ENABLED
  { ERROR_ICM_NOT_ENABLED, &(O(object_Kerror_icm_not_enabled)) },
 #endif
 #ifdef ERROR_DELETING_ICM_XFORM
  { ERROR_DELETING_ICM_XFORM, &(O(object_Kerror_deleting_icm_xform)) },
 #endif
 #ifdef ERROR_INVALID_TRANSFORM
  { ERROR_INVALID_TRANSFORM, &(O(object_Kerror_invalid_transform)) },
 #endif
 #ifdef ERROR_COLORSPACE_MISMATCH
  { ERROR_COLORSPACE_MISMATCH, &(O(object_Kerror_colorspace_mismatch)) },
 #endif
 #ifdef ERROR_INVALID_COLORINDEX
  { ERROR_INVALID_COLORINDEX, &(O(object_Kerror_invalid_colorindex)) },
 #endif
 #ifdef ERROR_CONNECTED_OTHER_PASSWORD
  { ERROR_CONNECTED_OTHER_PASSWORD, &(O(object_Kerror_connected_other_password)) },
 #endif
 #ifdef ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT
  { ERROR_CONNECTED_OTHER_PASSWORD_DEFAULT, &(O(object_Kerror_connected_other_password_default)) },
 #endif
 #ifdef ERROR_BAD_USERNAME
  { ERROR_BAD_USERNAME, &(O(object_Kerror_bad_username)) },
 #endif
 #ifdef ERROR_NOT_CONNECTED
  { ERROR_NOT_CONNECTED, &(O(object_Kerror_not_connected)) },
 #endif
 #ifdef ERROR_OPEN_FILES
  { ERROR_OPEN_FILES, &(O(object_Kerror_open_files)) },
 #endif
 #ifdef ERROR_ACTIVE_CONNECTIONS
  { ERROR_ACTIVE_CONNECTIONS, &(O(object_Kerror_active_connections)) },
 #endif
 #ifdef ERROR_DEVICE_IN_USE
  { ERROR_DEVICE_IN_USE, &(O(object_Kerror_device_in_use)) },
 #endif
 #ifdef ERROR_UNKNOWN_PRINT_MONITOR
  { ERROR_UNKNOWN_PRINT_MONITOR, &(O(object_Kerror_unknown_print_monitor)) },
 #endif
 #ifdef ERROR_PRINTER_DRIVER_IN_USE
  { ERROR_PRINTER_DRIVER_IN_USE, &(O(object_Kerror_printer_driver_in_use)) },
 #endif
 #ifdef ERROR_SPOOL_FILE_NOT_FOUND
  { ERROR_SPOOL_FILE_NOT_FOUND, &(O(object_Kerror_spool_file_not_found)) },
 #endif
 #ifdef ERROR_SPL_NO_STARTDOC
  { ERROR_SPL_NO_STARTDOC, &(O(object_Kerror_spl_no_startdoc)) },
 #endif
 #ifdef ERROR_SPL_NO_ADDJOB
  { ERROR_SPL_NO_ADDJOB, &(O(object_Kerror_spl_no_addjob)) },
 #endif
 #ifdef ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED
  { ERROR_PRINT_PROCESSOR_ALREADY_INSTALLED, &(O(object_Kerror_print_processor_already_installed)) },
 #endif
 #ifdef ERROR_PRINT_MONITOR_ALREADY_INSTALLED
  { ERROR_PRINT_MONITOR_ALREADY_INSTALLED, &(O(object_Kerror_print_monitor_already_installed)) },
 #endif
 #ifdef ERROR_INVALID_PRINT_MONITOR
  { ERROR_INVALID_PRINT_MONITOR, &(O(object_Kerror_invalid_print_monitor)) },
 #endif
 #ifdef ERROR_PRINT_MONITOR_IN_USE
  { ERROR_PRINT_MONITOR_IN_USE, &(O(object_Kerror_print_monitor_in_use)) },
 #endif
 #ifdef ERROR_PRINTER_HAS_JOBS_QUEUED
  { ERROR_PRINTER_HAS_JOBS_QUEUED, &(O(object_Kerror_printer_has_jobs_queued)) },
 #endif
 #ifdef ERROR_SUCCESS_REBOOT_REQUIRED
  { ERROR_SUCCESS_REBOOT_REQUIRED, &(O(object_Kerror_success_reboot_required)) },
 #endif
 #ifdef ERROR_SUCCESS_RESTART_REQUIRED
  { ERROR_SUCCESS_RESTART_REQUIRED, &(O(object_Kerror_success_restart_required)) },
 #endif
 #ifdef ERROR_PRINTER_NOT_FOUND
  { ERROR_PRINTER_NOT_FOUND, &(O(object_Kerror_printer_not_found)) },
 #endif
 #ifdef ERROR_PRINTER_DRIVER_WARNED
  { ERROR_PRINTER_DRIVER_WARNED, &(O(object_Kerror_printer_driver_warned)) },
 #endif
 #ifdef ERROR_PRINTER_DRIVER_BLOCKED
  { ERROR_PRINTER_DRIVER_BLOCKED, &(O(object_Kerror_printer_driver_blocked)) },
 #endif
 #ifdef ERROR_WINS_INTERNAL
  { ERROR_WINS_INTERNAL, &(O(object_Kerror_wins_internal)) },
 #endif
 #ifdef ERROR_CAN_NOT_DEL_LOCAL_WINS
  { ERROR_CAN_NOT_DEL_LOCAL_WINS, &(O(object_Kerror_can_not_del_local_wins)) },
 #endif
 #ifdef ERROR_STATIC_INIT
  { ERROR_STATIC_INIT, &(O(object_Kerror_static_init)) },
 #endif
 #ifdef ERROR_INC_BACKUP
  { ERROR_INC_BACKUP, &(O(object_Kerror_inc_backup)) },
 #endif
 #ifdef ERROR_FULL_BACKUP
  { ERROR_FULL_BACKUP, &(O(object_Kerror_full_backup)) },
 #endif
 #ifdef ERROR_REC_NON_EXISTENT
  { ERROR_REC_NON_EXISTENT, &(O(object_Kerror_rec_non_existent)) },
 #endif
 #ifdef ERROR_RPL_NOT_ALLOWED
  { ERROR_RPL_NOT_ALLOWED, &(O(object_Kerror_rpl_not_allowed)) },
 #endif
 #ifdef ERROR_DHCP_ADDRESS_CONFLICT
  { ERROR_DHCP_ADDRESS_CONFLICT, &(O(object_Kerror_dhcp_address_conflict)) },
 #endif
 #ifdef ERROR_WMI_GUID_NOT_FOUND
  { ERROR_WMI_GUID_NOT_FOUND, &(O(object_Kerror_wmi_guid_not_found)) },
 #endif
 #ifdef ERROR_WMI_INSTANCE_NOT_FOUND
  { ERROR_WMI_INSTANCE_NOT_FOUND, &(O(object_Kerror_wmi_instance_not_found)) },
 #endif
 #ifdef ERROR_WMI_ITEMID_NOT_FOUND
  { ERROR_WMI_ITEMID_NOT_FOUND, &(O(object_Kerror_wmi_itemid_not_found)) },
 #endif
 #ifdef ERROR_WMI_TRY_AGAIN
  { ERROR_WMI_TRY_AGAIN, &(O(object_Kerror_wmi_try_again)) },
 #endif
 #ifdef ERROR_WMI_DP_NOT_FOUND
  { ERROR_WMI_DP_NOT_FOUND, &(O(object_Kerror_wmi_dp_not_found)) },
 #endif
 #ifdef ERROR_WMI_UNRESOLVED_INSTANCE_REF
  { ERROR_WMI_UNRESOLVED_INSTANCE_REF, &(O(object_Kerror_wmi_unresolved_instance_ref)) },
 #endif
 #ifdef ERROR_WMI_ALREADY_ENABLED
  { ERROR_WMI_ALREADY_ENABLED, &(O(object_Kerror_wmi_already_enabled)) },
 #endif
 #ifdef ERROR_WMI_GUID_DISCONNECTED
  { ERROR_WMI_GUID_DISCONNECTED, &(O(object_Kerror_wmi_guid_disconnected)) },
 #endif
 #ifdef ERROR_WMI_SERVER_UNAVAILABLE
  { ERROR_WMI_SERVER_UNAVAILABLE, &(O(object_Kerror_wmi_server_unavailable)) },
 #endif
 #ifdef ERROR_WMI_DP_FAILED
  { ERROR_WMI_DP_FAILED, &(O(object_Kerror_wmi_dp_failed)) },
 #endif
 #ifdef ERROR_WMI_INVALID_MOF
  { ERROR_WMI_INVALID_MOF, &(O(object_Kerror_wmi_invalid_mof)) },
 #endif
 #ifdef ERROR_WMI_INVALID_REGINFO
  { ERROR_WMI_INVALID_REGINFO, &(O(object_Kerror_wmi_invalid_reginfo)) },
 #endif
 #ifdef ERROR_WMI_ALREADY_DISABLED
  { ERROR_WMI_ALREADY_DISABLED, &(O(object_Kerror_wmi_already_disabled)) },
 #endif
 #ifdef ERROR_WMI_READ_ONLY
  { ERROR_WMI_READ_ONLY, &(O(object_Kerror_wmi_read_only)) },
 #endif
 #ifdef ERROR_WMI_SET_FAILURE
  { ERROR_WMI_SET_FAILURE, &(O(object_Kerror_wmi_set_failure)) },
 #endif
 #ifdef ERROR_INVALID_MEDIA
  { ERROR_INVALID_MEDIA, &(O(object_Kerror_invalid_media)) },
 #endif
 #ifdef ERROR_INVALID_LIBRARY
  { ERROR_INVALID_LIBRARY, &(O(object_Kerror_invalid_library)) },
 #endif
 #ifdef ERROR_INVALID_MEDIA_POOL
  { ERROR_INVALID_MEDIA_POOL, &(O(object_Kerror_invalid_media_pool)) },
 #endif
 #ifdef ERROR_DRIVE_MEDIA_MISMATCH
  { ERROR_DRIVE_MEDIA_MISMATCH, &(O(object_Kerror_drive_media_mismatch)) },
 #endif
 #ifdef ERROR_MEDIA_OFFLINE
  { ERROR_MEDIA_OFFLINE, &(O(object_Kerror_media_offline)) },
 #endif
 #ifdef ERROR_LIBRARY_OFFLINE
  { ERROR_LIBRARY_OFFLINE, &(O(object_Kerror_library_offline)) },
 #endif
 #ifdef ERROR_EMPTY
  { ERROR_EMPTY, &(O(object_Kerror_empty)) },
 #endif
 #ifdef ERROR_NOT_EMPTY
  { ERROR_NOT_EMPTY, &(O(object_Kerror_not_empty)) },
 #endif
 #ifdef ERROR_MEDIA_UNAVAILABLE
  { ERROR_MEDIA_UNAVAILABLE, &(O(object_Kerror_media_unavailable)) },
 #endif
 #ifdef ERROR_RESOURCE_DISABLED
  { ERROR_RESOURCE_DISABLED, &(O(object_Kerror_resource_disabled)) },
 #endif
 #ifdef ERROR_INVALID_CLEANER
  { ERROR_INVALID_CLEANER, &(O(object_Kerror_invalid_cleaner)) },
 #endif
 #ifdef ERROR_UNABLE_TO_CLEAN
  { ERROR_UNABLE_TO_CLEAN, &(O(object_Kerror_unable_to_clean)) },
 #endif
 #ifdef ERROR_OBJECT_NOT_FOUND
  { ERROR_OBJECT_NOT_FOUND, &(O(object_Kerror_object_not_found)) },
 #endif
 #ifdef ERROR_DATABASE_FAILURE
  { ERROR_DATABASE_FAILURE, &(O(object_Kerror_database_failure)) },
 #endif
 #ifdef ERROR_DATABASE_FULL
  { ERROR_DATABASE_FULL, &(O(object_Kerror_database_full)) },
 #endif
 #ifdef ERROR_MEDIA_INCOMPATIBLE
  { ERROR_MEDIA_INCOMPATIBLE, &(O(object_Kerror_media_incompatible)) },
 #endif
 #ifdef ERROR_RESOURCE_NOT_PRESENT
  { ERROR_RESOURCE_NOT_PRESENT, &(O(object_Kerror_resource_not_present)) },
 #endif
 #ifdef ERROR_INVALID_OPERATION
  { ERROR_INVALID_OPERATION, &(O(object_Kerror_invalid_operation)) },
 #endif
 #ifdef ERROR_MEDIA_NOT_AVAILABLE
  { ERROR_MEDIA_NOT_AVAILABLE, &(O(object_Kerror_media_not_available)) },
 #endif
 #ifdef ERROR_DEVICE_NOT_AVAILABLE
  { ERROR_DEVICE_NOT_AVAILABLE, &(O(object_Kerror_device_not_available)) },
 #endif
 #ifdef ERROR_REQUEST_REFUSED
  { ERROR_REQUEST_REFUSED, &(O(object_Kerror_request_refused)) },
 #endif
 #ifdef ERROR_INVALID_DRIVE_OBJECT
  { ERROR_INVALID_DRIVE_OBJECT, &(O(object_Kerror_invalid_drive_object)) },
 #endif
 #ifdef ERROR_LIBRARY_FULL
  { ERROR_LIBRARY_FULL, &(O(object_Kerror_library_full)) },
 #endif
 #ifdef ERROR_MEDIUM_NOT_ACCESSIBLE
  { ERROR_MEDIUM_NOT_ACCESSIBLE, &(O(object_Kerror_medium_not_accessible)) },
 #endif
 #ifdef ERROR_UNABLE_TO_LOAD_MEDIUM
  { ERROR_UNABLE_TO_LOAD_MEDIUM, &(O(object_Kerror_unable_to_load_medium)) },
 #endif
 #ifdef ERROR_UNABLE_TO_INVENTORY_DRIVE
  { ERROR_UNABLE_TO_INVENTORY_DRIVE, &(O(object_Kerror_unable_to_inventory_drive)) },
 #endif
 #ifdef ERROR_UNABLE_TO_INVENTORY_SLOT
  { ERROR_UNABLE_TO_INVENTORY_SLOT, &(O(object_Kerror_unable_to_inventory_slot)) },
 #endif
 #ifdef ERROR_UNABLE_TO_INVENTORY_TRANSPORT
  { ERROR_UNABLE_TO_INVENTORY_TRANSPORT, &(O(object_Kerror_unable_to_inventory_transport)) },
 #endif
 #ifdef ERROR_TRANSPORT_FULL
  { ERROR_TRANSPORT_FULL, &(O(object_Kerror_transport_full)) },
 #endif
 #ifdef ERROR_CONTROLLING_IEPORT
  { ERROR_CONTROLLING_IEPORT, &(O(object_Kerror_controlling_ieport)) },
 #endif
 #ifdef ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA
  { ERROR_UNABLE_TO_EJECT_MOUNTED_MEDIA, &(O(object_Kerror_unable_to_eject_mounted_media)) },
 #endif
 #ifdef ERROR_CLEANER_SLOT_SET
  { ERROR_CLEANER_SLOT_SET, &(O(object_Kerror_cleaner_slot_set)) },
 #endif
 #ifdef ERROR_CLEANER_SLOT_NOT_SET
  { ERROR_CLEANER_SLOT_NOT_SET, &(O(object_Kerror_cleaner_slot_not_set)) },
 #endif
 #ifdef ERROR_CLEANER_CARTRIDGE_SPENT
  { ERROR_CLEANER_CARTRIDGE_SPENT, &(O(object_Kerror_cleaner_cartridge_spent)) },
 #endif
 #ifdef ERROR_UNEXPECTED_OMID
  { ERROR_UNEXPECTED_OMID, &(O(object_Kerror_unexpected_omid)) },
 #endif
 #ifdef ERROR_CANT_DELETE_LAST_ITEM
  { ERROR_CANT_DELETE_LAST_ITEM, &(O(object_Kerror_cant_delete_last_item)) },
 #endif
 #ifdef ERROR_MESSAGE_EXCEEDS_MAX_SIZE
  { ERROR_MESSAGE_EXCEEDS_MAX_SIZE, &(O(object_Kerror_message_exceeds_max_size)) },
 #endif
 #ifdef ERROR_VOLUME_CONTAINS_SYS_FILES
  { ERROR_VOLUME_CONTAINS_SYS_FILES, &(O(object_Kerror_volume_contains_sys_files)) },
 #endif
 #ifdef ERROR_INDIGENOUS_TYPE
  { ERROR_INDIGENOUS_TYPE, &(O(object_Kerror_indigenous_type)) },
 #endif
 #ifdef ERROR_NO_SUPPORTING_DRIVES
  { ERROR_NO_SUPPORTING_DRIVES, &(O(object_Kerror_no_supporting_drives)) },
 #endif
 #ifdef ERROR_CLEANER_CARTRIDGE_INSTALLED
  { ERROR_CLEANER_CARTRIDGE_INSTALLED, &(O(object_Kerror_cleaner_cartridge_installed)) },
 #endif
 #ifdef ERROR_FILE_OFFLINE
  { ERROR_FILE_OFFLINE, &(O(object_Kerror_file_offline)) },
 #endif
 #ifdef ERROR_REMOTE_STORAGE_NOT_ACTIVE
  { ERROR_REMOTE_STORAGE_NOT_ACTIVE, &(O(object_Kerror_remote_storage_not_active)) },
 #endif
 #ifdef ERROR_REMOTE_STORAGE_MEDIA_ERROR
  { ERROR_REMOTE_STORAGE_MEDIA_ERROR, &(O(object_Kerror_remote_storage_media_error)) },
 #endif
 #ifdef ERROR_NOT_A_REPARSE_POINT
  { ERROR_NOT_A_REPARSE_POINT, &(O(object_Kerror_not_a_reparse_point)) },
 #endif
 #ifdef ERROR_REPARSE_ATTRIBUTE_CONFLICT
  { ERROR_REPARSE_ATTRIBUTE_CONFLICT, &(O(object_Kerror_reparse_attribute_conflict)) },
 #endif
 #ifdef ERROR_INVALID_REPARSE_DATA
  { ERROR_INVALID_REPARSE_DATA, &(O(object_Kerror_invalid_reparse_data)) },
 #endif
 #ifdef ERROR_REPARSE_TAG_INVALID
  { ERROR_REPARSE_TAG_INVALID, &(O(object_Kerror_reparse_tag_invalid)) },
 #endif
 #ifdef ERROR_REPARSE_TAG_MISMATCH
  { ERROR_REPARSE_TAG_MISMATCH, &(O(object_Kerror_reparse_tag_mismatch)) },
 #endif
 #ifdef ERROR_VOLUME_NOT_SIS_ENABLED
  { ERROR_VOLUME_NOT_SIS_ENABLED, &(O(object_Kerror_volume_not_sis_enabled)) },
 #endif
 #ifdef ERROR_DEPENDENT_RESOURCE_EXISTS
  { ERROR_DEPENDENT_RESOURCE_EXISTS, &(O(object_Kerror_dependent_resource_exists)) },
 #endif
 #ifdef ERROR_DEPENDENCY_NOT_FOUND
  { ERROR_DEPENDENCY_NOT_FOUND, &(O(object_Kerror_dependency_not_found)) },
 #endif
 #ifdef ERROR_DEPENDENCY_ALREADY_EXISTS
  { ERROR_DEPENDENCY_ALREADY_EXISTS, &(O(object_Kerror_dependency_already_exists)) },
 #endif
 #ifdef ERROR_RESOURCE_NOT_ONLINE
  { ERROR_RESOURCE_NOT_ONLINE, &(O(object_Kerror_resource_not_online)) },
 #endif
 #ifdef ERROR_HOST_NODE_NOT_AVAILABLE
  { ERROR_HOST_NODE_NOT_AVAILABLE, &(O(object_Kerror_host_node_not_available)) },
 #endif
 #ifdef ERROR_RESOURCE_NOT_AVAILABLE
  { ERROR_RESOURCE_NOT_AVAILABLE, &(O(object_Kerror_resource_not_available)) },
 #endif
 #ifdef ERROR_RESOURCE_NOT_FOUND
  { ERROR_RESOURCE_NOT_FOUND, &(O(object_Kerror_resource_not_found)) },
 #endif
 #ifdef ERROR_SHUTDOWN_CLUSTER
  { ERROR_SHUTDOWN_CLUSTER, &(O(object_Kerror_shutdown_cluster)) },
 #endif
 #ifdef ERROR_CANT_EVICT_ACTIVE_NODE
  { ERROR_CANT_EVICT_ACTIVE_NODE, &(O(object_Kerror_cant_evict_active_node)) },
 #endif
 #ifdef ERROR_OBJECT_ALREADY_EXISTS
  { ERROR_OBJECT_ALREADY_EXISTS, &(O(object_Kerror_object_already_exists)) },
 #endif
 #ifdef ERROR_OBJECT_IN_LIST
  { ERROR_OBJECT_IN_LIST, &(O(object_Kerror_object_in_list)) },
 #endif
 #ifdef ERROR_GROUP_NOT_AVAILABLE
  { ERROR_GROUP_NOT_AVAILABLE, &(O(object_Kerror_group_not_available)) },
 #endif
 #ifdef ERROR_GROUP_NOT_FOUND
  { ERROR_GROUP_NOT_FOUND, &(O(object_Kerror_group_not_found)) },
 #endif
 #ifdef ERROR_GROUP_NOT_ONLINE
  { ERROR_GROUP_NOT_ONLINE, &(O(object_Kerror_group_not_online)) },
 #endif
 #ifdef ERROR_HOST_NODE_NOT_RESOURCE_OWNER
  { ERROR_HOST_NODE_NOT_RESOURCE_OWNER, &(O(object_Kerror_host_node_not_resource_owner)) },
 #endif
 #ifdef ERROR_HOST_NODE_NOT_GROUP_OWNER
  { ERROR_HOST_NODE_NOT_GROUP_OWNER, &(O(object_Kerror_host_node_not_group_owner)) },
 #endif
 #ifdef ERROR_RESMON_CREATE_FAILED
  { ERROR_RESMON_CREATE_FAILED, &(O(object_Kerror_resmon_create_failed)) },
 #endif
 #ifdef ERROR_RESMON_ONLINE_FAILED
  { ERROR_RESMON_ONLINE_FAILED, &(O(object_Kerror_resmon_online_failed)) },
 #endif
 #ifdef ERROR_RESOURCE_ONLINE
  { ERROR_RESOURCE_ONLINE, &(O(object_Kerror_resource_online)) },
 #endif
 #ifdef ERROR_QUORUM_RESOURCE
  { ERROR_QUORUM_RESOURCE, &(O(object_Kerror_quorum_resource)) },
 #endif
 #ifdef ERROR_NOT_QUORUM_CAPABLE
  { ERROR_NOT_QUORUM_CAPABLE, &(O(object_Kerror_not_quorum_capable)) },
 #endif
 #ifdef ERROR_CLUSTER_SHUTTING_DOWN
  { ERROR_CLUSTER_SHUTTING_DOWN, &(O(object_Kerror_cluster_shutting_down)) },
 #endif
 #ifdef ERROR_INVALID_STATE
  { ERROR_INVALID_STATE, &(O(object_Kerror_invalid_state)) },
 #endif
 #ifdef ERROR_RESOURCE_PROPERTIES_STORED
  { ERROR_RESOURCE_PROPERTIES_STORED, &(O(object_Kerror_resource_properties_stored)) },
 #endif
 #ifdef ERROR_NOT_QUORUM_CLASS
  { ERROR_NOT_QUORUM_CLASS, &(O(object_Kerror_not_quorum_class)) },
 #endif
 #ifdef ERROR_CORE_RESOURCE
  { ERROR_CORE_RESOURCE, &(O(object_Kerror_core_resource)) },
 #endif
 #ifdef ERROR_QUORUM_RESOURCE_ONLINE_FAILED
  { ERROR_QUORUM_RESOURCE_ONLINE_FAILED, &(O(object_Kerror_quorum_resource_online_failed)) },
 #endif
 #ifdef ERROR_QUORUMLOG_OPEN_FAILED
  { ERROR_QUORUMLOG_OPEN_FAILED, &(O(object_Kerror_quorumlog_open_failed)) },
 #endif
 #ifdef ERROR_CLUSTERLOG_CORRUPT
  { ERROR_CLUSTERLOG_CORRUPT, &(O(object_Kerror_clusterlog_corrupt)) },
 #endif
 #ifdef ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE
  { ERROR_CLUSTERLOG_RECORD_EXCEEDS_MAXSIZE, &(O(object_Kerror_clusterlog_record_exceeds_maxsize)) },
 #endif
 #ifdef ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE
  { ERROR_CLUSTERLOG_EXCEEDS_MAXSIZE, &(O(object_Kerror_clusterlog_exceeds_maxsize)) },
 #endif
 #ifdef ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND
  { ERROR_CLUSTERLOG_CHKPOINT_NOT_FOUND, &(O(object_Kerror_clusterlog_chkpoint_not_found)) },
 #endif
 #ifdef ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE
  { ERROR_CLUSTERLOG_NOT_ENOUGH_SPACE, &(O(object_Kerror_clusterlog_not_enough_space)) },
 #endif
 #ifdef ERROR_QUORUM_OWNER_ALIVE
  { ERROR_QUORUM_OWNER_ALIVE, &(O(object_Kerror_quorum_owner_alive)) },
 #endif
 #ifdef ERROR_NETWORK_NOT_AVAILABLE
  { ERROR_NETWORK_NOT_AVAILABLE, &(O(object_Kerror_network_not_available)) },
 #endif
 #ifdef ERROR_NODE_NOT_AVAILABLE
  { ERROR_NODE_NOT_AVAILABLE, &(O(object_Kerror_node_not_available)) },
 #endif
 #ifdef ERROR_ALL_NODES_NOT_AVAILABLE
  { ERROR_ALL_NODES_NOT_AVAILABLE, &(O(object_Kerror_all_nodes_not_available)) },
 #endif
 #ifdef ERROR_RESOURCE_FAILED
  { ERROR_RESOURCE_FAILED, &(O(object_Kerror_resource_failed)) },
 #endif
 #ifdef ERROR_CLUSTER_INVALID_NODE
  { ERROR_CLUSTER_INVALID_NODE, &(O(object_Kerror_cluster_invalid_node)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_EXISTS
  { ERROR_CLUSTER_NODE_EXISTS, &(O(object_Kerror_cluster_node_exists)) },
 #endif
 #ifdef ERROR_CLUSTER_JOIN_IN_PROGRESS
  { ERROR_CLUSTER_JOIN_IN_PROGRESS, &(O(object_Kerror_cluster_join_in_progress)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_NOT_FOUND
  { ERROR_CLUSTER_NODE_NOT_FOUND, &(O(object_Kerror_cluster_node_not_found)) },
 #endif
 #ifdef ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND
  { ERROR_CLUSTER_LOCAL_NODE_NOT_FOUND, &(O(object_Kerror_cluster_local_node_not_found)) },
 #endif
 #ifdef ERROR_CLUSTER_NETWORK_EXISTS
  { ERROR_CLUSTER_NETWORK_EXISTS, &(O(object_Kerror_cluster_network_exists)) },
 #endif
 #ifdef ERROR_CLUSTER_NETWORK_NOT_FOUND
  { ERROR_CLUSTER_NETWORK_NOT_FOUND, &(O(object_Kerror_cluster_network_not_found)) },
 #endif
 #ifdef ERROR_CLUSTER_NETINTERFACE_EXISTS
  { ERROR_CLUSTER_NETINTERFACE_EXISTS, &(O(object_Kerror_cluster_netinterface_exists)) },
 #endif
 #ifdef ERROR_CLUSTER_NETINTERFACE_NOT_FOUND
  { ERROR_CLUSTER_NETINTERFACE_NOT_FOUND, &(O(object_Kerror_cluster_netinterface_not_found)) },
 #endif
 #ifdef ERROR_CLUSTER_INVALID_REQUEST
  { ERROR_CLUSTER_INVALID_REQUEST, &(O(object_Kerror_cluster_invalid_request)) },
 #endif
 #ifdef ERROR_CLUSTER_INVALID_NETWORK_PROVIDER
  { ERROR_CLUSTER_INVALID_NETWORK_PROVIDER, &(O(object_Kerror_cluster_invalid_network_provider)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_DOWN
  { ERROR_CLUSTER_NODE_DOWN, &(O(object_Kerror_cluster_node_down)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_UNREACHABLE
  { ERROR_CLUSTER_NODE_UNREACHABLE, &(O(object_Kerror_cluster_node_unreachable)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_NOT_MEMBER
  { ERROR_CLUSTER_NODE_NOT_MEMBER, &(O(object_Kerror_cluster_node_not_member)) },
 #endif
 #ifdef ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS
  { ERROR_CLUSTER_JOIN_NOT_IN_PROGRESS, &(O(object_Kerror_cluster_join_not_in_progress)) },
 #endif
 #ifdef ERROR_CLUSTER_INVALID_NETWORK
  { ERROR_CLUSTER_INVALID_NETWORK, &(O(object_Kerror_cluster_invalid_network)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_UP
  { ERROR_CLUSTER_NODE_UP, &(O(object_Kerror_cluster_node_up)) },
 #endif
 #ifdef ERROR_CLUSTER_IPADDR_IN_USE
  { ERROR_CLUSTER_IPADDR_IN_USE, &(O(object_Kerror_cluster_ipaddr_in_use)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_NOT_PAUSED
  { ERROR_CLUSTER_NODE_NOT_PAUSED, &(O(object_Kerror_cluster_node_not_paused)) },
 #endif
 #ifdef ERROR_CLUSTER_NO_SECURITY_CONTEXT
  { ERROR_CLUSTER_NO_SECURITY_CONTEXT, &(O(object_Kerror_cluster_no_security_context)) },
 #endif
 #ifdef ERROR_CLUSTER_NETWORK_NOT_INTERNAL
  { ERROR_CLUSTER_NETWORK_NOT_INTERNAL, &(O(object_Kerror_cluster_network_not_internal)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_ALREADY_UP
  { ERROR_CLUSTER_NODE_ALREADY_UP, &(O(object_Kerror_cluster_node_already_up)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_ALREADY_DOWN
  { ERROR_CLUSTER_NODE_ALREADY_DOWN, &(O(object_Kerror_cluster_node_already_down)) },
 #endif
 #ifdef ERROR_CLUSTER_NETWORK_ALREADY_ONLINE
  { ERROR_CLUSTER_NETWORK_ALREADY_ONLINE, &(O(object_Kerror_cluster_network_already_online)) },
 #endif
 #ifdef ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE
  { ERROR_CLUSTER_NETWORK_ALREADY_OFFLINE, &(O(object_Kerror_cluster_network_already_offline)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_ALREADY_MEMBER
  { ERROR_CLUSTER_NODE_ALREADY_MEMBER, &(O(object_Kerror_cluster_node_already_member)) },
 #endif
 #ifdef ERROR_CLUSTER_LAST_INTERNAL_NETWORK
  { ERROR_CLUSTER_LAST_INTERNAL_NETWORK, &(O(object_Kerror_cluster_last_internal_network)) },
 #endif
 #ifdef ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS
  { ERROR_CLUSTER_NETWORK_HAS_DEPENDENTS, &(O(object_Kerror_cluster_network_has_dependents)) },
 #endif
 #ifdef ERROR_INVALID_OPERATION_ON_QUORUM
  { ERROR_INVALID_OPERATION_ON_QUORUM, &(O(object_Kerror_invalid_operation_on_quorum)) },
 #endif
 #ifdef ERROR_DEPENDENCY_NOT_ALLOWED
  { ERROR_DEPENDENCY_NOT_ALLOWED, &(O(object_Kerror_dependency_not_allowed)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_PAUSED
  { ERROR_CLUSTER_NODE_PAUSED, &(O(object_Kerror_cluster_node_paused)) },
 #endif
 #ifdef ERROR_NODE_CANT_HOST_RESOURCE
  { ERROR_NODE_CANT_HOST_RESOURCE, &(O(object_Kerror_node_cant_host_resource)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_NOT_READY
  { ERROR_CLUSTER_NODE_NOT_READY, &(O(object_Kerror_cluster_node_not_ready)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_SHUTTING_DOWN
  { ERROR_CLUSTER_NODE_SHUTTING_DOWN, &(O(object_Kerror_cluster_node_shutting_down)) },
 #endif
 #ifdef ERROR_CLUSTER_JOIN_ABORTED
  { ERROR_CLUSTER_JOIN_ABORTED, &(O(object_Kerror_cluster_join_aborted)) },
 #endif
 #ifdef ERROR_CLUSTER_INCOMPATIBLE_VERSIONS
  { ERROR_CLUSTER_INCOMPATIBLE_VERSIONS, &(O(object_Kerror_cluster_incompatible_versions)) },
 #endif
 #ifdef ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED
  { ERROR_CLUSTER_MAXNUM_OF_RESOURCES_EXCEEDED, &(O(object_Kerror_cluster_maxnum_of_resources_exceeded)) },
 #endif
 #ifdef ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED
  { ERROR_CLUSTER_SYSTEM_CONFIG_CHANGED, &(O(object_Kerror_cluster_system_config_changed)) },
 #endif
 #ifdef ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND
  { ERROR_CLUSTER_RESOURCE_TYPE_NOT_FOUND, &(O(object_Kerror_cluster_resource_type_not_found)) },
 #endif
 #ifdef ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED
  { ERROR_CLUSTER_RESTYPE_NOT_SUPPORTED, &(O(object_Kerror_cluster_restype_not_supported)) },
 #endif
 #ifdef ERROR_CLUSTER_RESNAME_NOT_FOUND
  { ERROR_CLUSTER_RESNAME_NOT_FOUND, &(O(object_Kerror_cluster_resname_not_found)) },
 #endif
 #ifdef ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED
  { ERROR_CLUSTER_NO_RPC_PACKAGES_REGISTERED, &(O(object_Kerror_cluster_no_rpc_packages_registered)) },
 #endif
 #ifdef ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST
  { ERROR_CLUSTER_OWNER_NOT_IN_PREFLIST, &(O(object_Kerror_cluster_owner_not_in_preflist)) },
 #endif
 #ifdef ERROR_CLUSTER_DATABASE_SEQMISMATCH
  { ERROR_CLUSTER_DATABASE_SEQMISMATCH, &(O(object_Kerror_cluster_database_seqmismatch)) },
 #endif
 #ifdef ERROR_RESMON_INVALID_STATE
  { ERROR_RESMON_INVALID_STATE, &(O(object_Kerror_resmon_invalid_state)) },
 #endif
 #ifdef ERROR_CLUSTER_GUM_NOT_LOCKER
  { ERROR_CLUSTER_GUM_NOT_LOCKER, &(O(object_Kerror_cluster_gum_not_locker)) },
 #endif
 #ifdef ERROR_QUORUM_DISK_NOT_FOUND
  { ERROR_QUORUM_DISK_NOT_FOUND, &(O(object_Kerror_quorum_disk_not_found)) },
 #endif
 #ifdef ERROR_DATABASE_BACKUP_CORRUPT
  { ERROR_DATABASE_BACKUP_CORRUPT, &(O(object_Kerror_database_backup_corrupt)) },
 #endif
 #ifdef ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT
  { ERROR_CLUSTER_NODE_ALREADY_HAS_DFS_ROOT, &(O(object_Kerror_cluster_node_already_has_dfs_root)) },
 #endif
 #ifdef ERROR_RESOURCE_PROPERTY_UNCHANGEABLE
  { ERROR_RESOURCE_PROPERTY_UNCHANGEABLE, &(O(object_Kerror_resource_property_unchangeable)) },
 #endif
 #ifdef ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE
  { ERROR_CLUSTER_MEMBERSHIP_INVALID_STATE, &(O(object_Kerror_cluster_membership_invalid_state)) },
 #endif
 #ifdef ERROR_CLUSTER_QUORUMLOG_NOT_FOUND
  { ERROR_CLUSTER_QUORUMLOG_NOT_FOUND, &(O(object_Kerror_cluster_quorumlog_not_found)) },
 #endif
 #ifdef ERROR_CLUSTER_MEMBERSHIP_HALT
  { ERROR_CLUSTER_MEMBERSHIP_HALT, &(O(object_Kerror_cluster_membership_halt)) },
 #endif
 #ifdef ERROR_CLUSTER_INSTANCE_ID_MISMATCH
  { ERROR_CLUSTER_INSTANCE_ID_MISMATCH, &(O(object_Kerror_cluster_instance_id_mismatch)) },
 #endif
 #ifdef ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP
  { ERROR_CLUSTER_NETWORK_NOT_FOUND_FOR_IP, &(O(object_Kerror_cluster_network_not_found_for_ip)) },
 #endif
 #ifdef ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH
  { ERROR_CLUSTER_PROPERTY_DATA_TYPE_MISMATCH, &(O(object_Kerror_cluster_property_data_type_mismatch)) },
 #endif
 #ifdef ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP
  { ERROR_CLUSTER_EVICT_WITHOUT_CLEANUP, &(O(object_Kerror_cluster_evict_without_cleanup)) },
 #endif
 #ifdef ERROR_CLUSTER_PARAMETER_MISMATCH
  { ERROR_CLUSTER_PARAMETER_MISMATCH, &(O(object_Kerror_cluster_parameter_mismatch)) },
 #endif
 #ifdef ERROR_NODE_CANNOT_BE_CLUSTERED
  { ERROR_NODE_CANNOT_BE_CLUSTERED, &(O(object_Kerror_node_cannot_be_clustered)) },
 #endif
 #ifdef ERROR_CLUSTER_WRONG_OS_VERSION
  { ERROR_CLUSTER_WRONG_OS_VERSION, &(O(object_Kerror_cluster_wrong_os_version)) },
 #endif
 #ifdef ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME
  { ERROR_CLUSTER_CANT_CREATE_DUP_CLUSTER_NAME, &(O(object_Kerror_cluster_cant_create_dup_cluster_name)) },
 #endif
 #ifdef ERROR_CLUSCFG_ALREADY_COMMITTED
  { ERROR_CLUSCFG_ALREADY_COMMITTED, &(O(object_Kerror_cluscfg_already_committed)) },
 #endif
 #ifdef ERROR_CLUSCFG_ROLLBACK_FAILED
  { ERROR_CLUSCFG_ROLLBACK_FAILED, &(O(object_Kerror_cluscfg_rollback_failed)) },
 #endif
 #ifdef ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT
  { ERROR_CLUSCFG_SYSTEM_DISK_DRIVE_LETTER_CONFLICT, &(O(object_Kerror_cluscfg_system_disk_drive_letter_conflict)) },
 #endif
 #ifdef ERROR_CLUSTER_OLD_VERSION
  { ERROR_CLUSTER_OLD_VERSION, &(O(object_Kerror_cluster_old_version)) },
 #endif
 #ifdef ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME
  { ERROR_CLUSTER_MISMATCHED_COMPUTER_ACCT_NAME, &(O(object_Kerror_cluster_mismatched_computer_acct_name)) },
 #endif
 #ifdef ERROR_ENCRYPTION_FAILED
  { ERROR_ENCRYPTION_FAILED, &(O(object_Kerror_encryption_failed)) },
 #endif
 #ifdef ERROR_DECRYPTION_FAILED
  { ERROR_DECRYPTION_FAILED, &(O(object_Kerror_decryption_failed)) },
 #endif
 #ifdef ERROR_FILE_ENCRYPTED
  { ERROR_FILE_ENCRYPTED, &(O(object_Kerror_file_encrypted)) },
 #endif
 #ifdef ERROR_NO_RECOVERY_POLICY
  { ERROR_NO_RECOVERY_POLICY, &(O(object_Kerror_no_recovery_policy)) },
 #endif
 #ifdef ERROR_NO_EFS
  { ERROR_NO_EFS, &(O(object_Kerror_no_efs)) },
 #endif
 #ifdef ERROR_WRONG_EFS
  { ERROR_WRONG_EFS, &(O(object_Kerror_wrong_efs)) },
 #endif
 #ifdef ERROR_NO_USER_KEYS
  { ERROR_NO_USER_KEYS, &(O(object_Kerror_no_user_keys)) },
 #endif
 #ifdef ERROR_FILE_NOT_ENCRYPTED
  { ERROR_FILE_NOT_ENCRYPTED, &(O(object_Kerror_file_not_encrypted)) },
 #endif
 #ifdef ERROR_NOT_EXPORT_FORMAT
  { ERROR_NOT_EXPORT_FORMAT, &(O(object_Kerror_not_export_format)) },
 #endif
 #ifdef ERROR_FILE_READ_ONLY
  { ERROR_FILE_READ_ONLY, &(O(object_Kerror_file_read_only)) },
 #endif
 #ifdef ERROR_DIR_EFS_DISALLOWED
  { ERROR_DIR_EFS_DISALLOWED, &(O(object_Kerror_dir_efs_disallowed)) },
 #endif
 #ifdef ERROR_EFS_SERVER_NOT_TRUSTED
  { ERROR_EFS_SERVER_NOT_TRUSTED, &(O(object_Kerror_efs_server_not_trusted)) },
 #endif
 #ifdef ERROR_BAD_RECOVERY_POLICY
  { ERROR_BAD_RECOVERY_POLICY, &(O(object_Kerror_bad_recovery_policy)) },
 #endif
 #ifdef ERROR_EFS_ALG_BLOB_TOO_BIG
  { ERROR_EFS_ALG_BLOB_TOO_BIG, &(O(object_Kerror_efs_alg_blob_too_big)) },
 #endif
 #ifdef ERROR_VOLUME_NOT_SUPPORT_EFS
  { ERROR_VOLUME_NOT_SUPPORT_EFS, &(O(object_Kerror_volume_not_support_efs)) },
 #endif
 #ifdef ERROR_EFS_DISABLED
  { ERROR_EFS_DISABLED, &(O(object_Kerror_efs_disabled)) },
 #endif
 #ifdef ERROR_EFS_VERSION_NOT_SUPPORT
  { ERROR_EFS_VERSION_NOT_SUPPORT, &(O(object_Kerror_efs_version_not_support)) },
 #endif
 #ifdef ERROR_NO_BROWSER_SERVERS_FOUND
  { ERROR_NO_BROWSER_SERVERS_FOUND, &(O(object_Kerror_no_browser_servers_found)) },
 #endif
 #ifdef SCHED_E_SERVICE_NOT_LOCALSYSTEM
  { SCHED_E_SERVICE_NOT_LOCALSYSTEM, &(O(object_Ksched_e_service_not_localsystem)) },
 #endif
 #ifdef ERROR_CTX_WINSTATION_NAME_INVALID
  { ERROR_CTX_WINSTATION_NAME_INVALID, &(O(object_Kerror_ctx_winstation_name_invalid)) },
 #endif
 #ifdef ERROR_CTX_INVALID_PD
  { ERROR_CTX_INVALID_PD, &(O(object_Kerror_ctx_invalid_pd)) },
 #endif
 #ifdef ERROR_CTX_PD_NOT_FOUND
  { ERROR_CTX_PD_NOT_FOUND, &(O(object_Kerror_ctx_pd_not_found)) },
 #endif
 #ifdef ERROR_CTX_WD_NOT_FOUND
  { ERROR_CTX_WD_NOT_FOUND, &(O(object_Kerror_ctx_wd_not_found)) },
 #endif
 #ifdef ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY
  { ERROR_CTX_CANNOT_MAKE_EVENTLOG_ENTRY, &(O(object_Kerror_ctx_cannot_make_eventlog_entry)) },
 #endif
 #ifdef ERROR_CTX_SERVICE_NAME_COLLISION
  { ERROR_CTX_SERVICE_NAME_COLLISION, &(O(object_Kerror_ctx_service_name_collision)) },
 #endif
 #ifdef ERROR_CTX_CLOSE_PENDING
  { ERROR_CTX_CLOSE_PENDING, &(O(object_Kerror_ctx_close_pending)) },
 #endif
 #ifdef ERROR_CTX_NO_OUTBUF
  { ERROR_CTX_NO_OUTBUF, &(O(object_Kerror_ctx_no_outbuf)) },
 #endif
 #ifdef ERROR_CTX_MODEM_INF_NOT_FOUND
  { ERROR_CTX_MODEM_INF_NOT_FOUND, &(O(object_Kerror_ctx_modem_inf_not_found)) },
 #endif
 #ifdef ERROR_CTX_INVALID_MODEMNAME
  { ERROR_CTX_INVALID_MODEMNAME, &(O(object_Kerror_ctx_invalid_modemname)) },
 #endif
 #ifdef ERROR_CTX_MODEM_RESPONSE_ERROR
  { ERROR_CTX_MODEM_RESPONSE_ERROR, &(O(object_Kerror_ctx_modem_response_error)) },
 #endif
 #ifdef ERROR_CTX_MODEM_RESPONSE_TIMEOUT
  { ERROR_CTX_MODEM_RESPONSE_TIMEOUT, &(O(object_Kerror_ctx_modem_response_timeout)) },
 #endif
 #ifdef ERROR_CTX_MODEM_RESPONSE_NO_CARRIER
  { ERROR_CTX_MODEM_RESPONSE_NO_CARRIER, &(O(object_Kerror_ctx_modem_response_no_carrier)) },
 #endif
 #ifdef ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE
  { ERROR_CTX_MODEM_RESPONSE_NO_DIALTONE, &(O(object_Kerror_ctx_modem_response_no_dialtone)) },
 #endif
 #ifdef ERROR_CTX_MODEM_RESPONSE_BUSY
  { ERROR_CTX_MODEM_RESPONSE_BUSY, &(O(object_Kerror_ctx_modem_response_busy)) },
 #endif
 #ifdef ERROR_CTX_MODEM_RESPONSE_VOICE
  { ERROR_CTX_MODEM_RESPONSE_VOICE, &(O(object_Kerror_ctx_modem_response_voice)) },
 #endif
 #ifdef ERROR_CTX_TD_ERROR
  { ERROR_CTX_TD_ERROR, &(O(object_Kerror_ctx_td_error)) },
 #endif
 #ifdef ERROR_CTX_WINSTATION_NOT_FOUND
  { ERROR_CTX_WINSTATION_NOT_FOUND, &(O(object_Kerror_ctx_winstation_not_found)) },
 #endif
 #ifdef ERROR_CTX_WINSTATION_ALREADY_EXISTS
  { ERROR_CTX_WINSTATION_ALREADY_EXISTS, &(O(object_Kerror_ctx_winstation_already_exists)) },
 #endif
 #ifdef ERROR_CTX_WINSTATION_BUSY
  { ERROR_CTX_WINSTATION_BUSY, &(O(object_Kerror_ctx_winstation_busy)) },
 #endif
 #ifdef ERROR_CTX_BAD_VIDEO_MODE
  { ERROR_CTX_BAD_VIDEO_MODE, &(O(object_Kerror_ctx_bad_video_mode)) },
 #endif
 #ifdef ERROR_CTX_GRAPHICS_INVALID
  { ERROR_CTX_GRAPHICS_INVALID, &(O(object_Kerror_ctx_graphics_invalid)) },
 #endif
 #ifdef ERROR_CTX_LOGON_DISABLED
  { ERROR_CTX_LOGON_DISABLED, &(O(object_Kerror_ctx_logon_disabled)) },
 #endif
 #ifdef ERROR_CTX_NOT_CONSOLE
  { ERROR_CTX_NOT_CONSOLE, &(O(object_Kerror_ctx_not_console)) },
 #endif
 #ifdef ERROR_CTX_CLIENT_QUERY_TIMEOUT
  { ERROR_CTX_CLIENT_QUERY_TIMEOUT, &(O(object_Kerror_ctx_client_query_timeout)) },
 #endif
 #ifdef ERROR_CTX_CONSOLE_DISCONNECT
  { ERROR_CTX_CONSOLE_DISCONNECT, &(O(object_Kerror_ctx_console_disconnect)) },
 #endif
 #ifdef ERROR_CTX_CONSOLE_CONNECT
  { ERROR_CTX_CONSOLE_CONNECT, &(O(object_Kerror_ctx_console_connect)) },
 #endif
 #ifdef ERROR_CTX_SHADOW_DENIED
  { ERROR_CTX_SHADOW_DENIED, &(O(object_Kerror_ctx_shadow_denied)) },
 #endif
 #ifdef ERROR_CTX_WINSTATION_ACCESS_DENIED
  { ERROR_CTX_WINSTATION_ACCESS_DENIED, &(O(object_Kerror_ctx_winstation_access_denied)) },
 #endif
 #ifdef ERROR_CTX_INVALID_WD
  { ERROR_CTX_INVALID_WD, &(O(object_Kerror_ctx_invalid_wd)) },
 #endif
 #ifdef ERROR_CTX_SHADOW_INVALID
  { ERROR_CTX_SHADOW_INVALID, &(O(object_Kerror_ctx_shadow_invalid)) },
 #endif
 #ifdef ERROR_CTX_SHADOW_DISABLED
  { ERROR_CTX_SHADOW_DISABLED, &(O(object_Kerror_ctx_shadow_disabled)) },
 #endif
 #ifdef ERROR_CTX_CLIENT_LICENSE_IN_USE
  { ERROR_CTX_CLIENT_LICENSE_IN_USE, &(O(object_Kerror_ctx_client_license_in_use)) },
 #endif
 #ifdef ERROR_CTX_CLIENT_LICENSE_NOT_SET
  { ERROR_CTX_CLIENT_LICENSE_NOT_SET, &(O(object_Kerror_ctx_client_license_not_set)) },
 #endif
 #ifdef ERROR_CTX_LICENSE_NOT_AVAILABLE
  { ERROR_CTX_LICENSE_NOT_AVAILABLE, &(O(object_Kerror_ctx_license_not_available)) },
 #endif
 #ifdef ERROR_CTX_LICENSE_CLIENT_INVALID
  { ERROR_CTX_LICENSE_CLIENT_INVALID, &(O(object_Kerror_ctx_license_client_invalid)) },
 #endif
 #ifdef ERROR_CTX_LICENSE_EXPIRED
  { ERROR_CTX_LICENSE_EXPIRED, &(O(object_Kerror_ctx_license_expired)) },
 #endif
 #ifdef ERROR_CTX_SHADOW_NOT_RUNNING
  { ERROR_CTX_SHADOW_NOT_RUNNING, &(O(object_Kerror_ctx_shadow_not_running)) },
 #endif
 #ifdef ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE
  { ERROR_CTX_SHADOW_ENDED_BY_MODE_CHANGE, &(O(object_Kerror_ctx_shadow_ended_by_mode_change)) },
 #endif
 #ifdef ERROR_ACTIVATION_COUNT_EXCEEDED
  { ERROR_ACTIVATION_COUNT_EXCEEDED, &(O(object_Kerror_activation_count_exceeded)) },
 #endif
 #ifdef FRS_ERR_INVALID_API_SEQUENCE
  { FRS_ERR_INVALID_API_SEQUENCE, &(O(object_Kfrs_err_invalid_api_sequence)) },
 #endif
 #ifdef FRS_ERR_STARTING_SERVICE
  { FRS_ERR_STARTING_SERVICE, &(O(object_Kfrs_err_starting_service)) },
 #endif
 #ifdef FRS_ERR_STOPPING_SERVICE
  { FRS_ERR_STOPPING_SERVICE, &(O(object_Kfrs_err_stopping_service)) },
 #endif
 #ifdef FRS_ERR_INTERNAL_API
  { FRS_ERR_INTERNAL_API, &(O(object_Kfrs_err_internal_api)) },
 #endif
 #ifdef FRS_ERR_INTERNAL
  { FRS_ERR_INTERNAL, &(O(object_Kfrs_err_internal)) },
 #endif
 #ifdef FRS_ERR_SERVICE_COMM
  { FRS_ERR_SERVICE_COMM, &(O(object_Kfrs_err_service_comm)) },
 #endif
 #ifdef FRS_ERR_INSUFFICIENT_PRIV
  { FRS_ERR_INSUFFICIENT_PRIV, &(O(object_Kfrs_err_insufficient_priv)) },
 #endif
 #ifdef FRS_ERR_AUTHENTICATION
  { FRS_ERR_AUTHENTICATION, &(O(object_Kfrs_err_authentication)) },
 #endif
 #ifdef FRS_ERR_PARENT_INSUFFICIENT_PRIV
  { FRS_ERR_PARENT_INSUFFICIENT_PRIV, &(O(object_Kfrs_err_parent_insufficient_priv)) },
 #endif
 #ifdef FRS_ERR_PARENT_AUTHENTICATION
  { FRS_ERR_PARENT_AUTHENTICATION, &(O(object_Kfrs_err_parent_authentication)) },
 #endif
 #ifdef FRS_ERR_CHILD_TO_PARENT_COMM
  { FRS_ERR_CHILD_TO_PARENT_COMM, &(O(object_Kfrs_err_child_to_parent_comm)) },
 #endif
 #ifdef FRS_ERR_PARENT_TO_CHILD_COMM
  { FRS_ERR_PARENT_TO_CHILD_COMM, &(O(object_Kfrs_err_parent_to_child_comm)) },
 #endif
 #ifdef FRS_ERR_SYSVOL_POPULATE
  { FRS_ERR_SYSVOL_POPULATE, &(O(object_Kfrs_err_sysvol_populate)) },
 #endif
 #ifdef FRS_ERR_SYSVOL_POPULATE_TIMEOUT
  { FRS_ERR_SYSVOL_POPULATE_TIMEOUT, &(O(object_Kfrs_err_sysvol_populate_timeout)) },
 #endif
 #ifdef FRS_ERR_SYSVOL_IS_BUSY
  { FRS_ERR_SYSVOL_IS_BUSY, &(O(object_Kfrs_err_sysvol_is_busy)) },
 #endif
 #ifdef FRS_ERR_SYSVOL_DEMOTE
  { FRS_ERR_SYSVOL_DEMOTE, &(O(object_Kfrs_err_sysvol_demote)) },
 #endif
 #ifdef FRS_ERR_INVALID_SERVICE_PARAMETER
  { FRS_ERR_INVALID_SERVICE_PARAMETER, &(O(object_Kfrs_err_invalid_service_parameter)) },
 #endif
 #ifdef ERROR_DS_NOT_INSTALLED
  { ERROR_DS_NOT_INSTALLED, &(O(object_Kerror_ds_not_installed)) },
 #endif
 #ifdef ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY
  { ERROR_DS_MEMBERSHIP_EVALUATED_LOCALLY, &(O(object_Kerror_ds_membership_evaluated_locally)) },
 #endif
 #ifdef ERROR_DS_NO_ATTRIBUTE_OR_VALUE
  { ERROR_DS_NO_ATTRIBUTE_OR_VALUE, &(O(object_Kerror_ds_no_attribute_or_value)) },
 #endif
 #ifdef ERROR_DS_INVALID_ATTRIBUTE_SYNTAX
  { ERROR_DS_INVALID_ATTRIBUTE_SYNTAX, &(O(object_Kerror_ds_invalid_attribute_syntax)) },
 #endif
 #ifdef ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED
  { ERROR_DS_ATTRIBUTE_TYPE_UNDEFINED, &(O(object_Kerror_ds_attribute_type_undefined)) },
 #endif
 #ifdef ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS
  { ERROR_DS_ATTRIBUTE_OR_VALUE_EXISTS, &(O(object_Kerror_ds_attribute_or_value_exists)) },
 #endif
 #ifdef ERROR_DS_BUSY
  { ERROR_DS_BUSY, &(O(object_Kerror_ds_busy)) },
 #endif
 #ifdef ERROR_DS_UNAVAILABLE
  { ERROR_DS_UNAVAILABLE, &(O(object_Kerror_ds_unavailable)) },
 #endif
 #ifdef ERROR_DS_NO_RIDS_ALLOCATED
  { ERROR_DS_NO_RIDS_ALLOCATED, &(O(object_Kerror_ds_no_rids_allocated)) },
 #endif
 #ifdef ERROR_DS_NO_MORE_RIDS
  { ERROR_DS_NO_MORE_RIDS, &(O(object_Kerror_ds_no_more_rids)) },
 #endif
 #ifdef ERROR_DS_INCORRECT_ROLE_OWNER
  { ERROR_DS_INCORRECT_ROLE_OWNER, &(O(object_Kerror_ds_incorrect_role_owner)) },
 #endif
 #ifdef ERROR_DS_RIDMGR_INIT_ERROR
  { ERROR_DS_RIDMGR_INIT_ERROR, &(O(object_Kerror_ds_ridmgr_init_error)) },
 #endif
 #ifdef ERROR_DS_OBJ_CLASS_VIOLATION
  { ERROR_DS_OBJ_CLASS_VIOLATION, &(O(object_Kerror_ds_obj_class_violation)) },
 #endif
 #ifdef ERROR_DS_CANT_ON_NON_LEAF
  { ERROR_DS_CANT_ON_NON_LEAF, &(O(object_Kerror_ds_cant_on_non_leaf)) },
 #endif
 #ifdef ERROR_DS_CANT_ON_RDN
  { ERROR_DS_CANT_ON_RDN, &(O(object_Kerror_ds_cant_on_rdn)) },
 #endif
 #ifdef ERROR_DS_CANT_MOD_OBJ_CLASS
  { ERROR_DS_CANT_MOD_OBJ_CLASS, &(O(object_Kerror_ds_cant_mod_obj_class)) },
 #endif
 #ifdef ERROR_DS_CROSS_DOM_MOVE_ERROR
  { ERROR_DS_CROSS_DOM_MOVE_ERROR, &(O(object_Kerror_ds_cross_dom_move_error)) },
 #endif
 #ifdef ERROR_DS_GC_NOT_AVAILABLE
  { ERROR_DS_GC_NOT_AVAILABLE, &(O(object_Kerror_ds_gc_not_available)) },
 #endif
 #ifdef ERROR_SHARED_POLICY
  { ERROR_SHARED_POLICY, &(O(object_Kerror_shared_policy)) },
 #endif
 #ifdef ERROR_POLICY_OBJECT_NOT_FOUND
  { ERROR_POLICY_OBJECT_NOT_FOUND, &(O(object_Kerror_policy_object_not_found)) },
 #endif
 #ifdef ERROR_POLICY_ONLY_IN_DS
  { ERROR_POLICY_ONLY_IN_DS, &(O(object_Kerror_policy_only_in_ds)) },
 #endif
 #ifdef ERROR_PROMOTION_ACTIVE
  { ERROR_PROMOTION_ACTIVE, &(O(object_Kerror_promotion_active)) },
 #endif
 #ifdef ERROR_NO_PROMOTION_ACTIVE
  { ERROR_NO_PROMOTION_ACTIVE, &(O(object_Kerror_no_promotion_active)) },
 #endif
 #ifdef ERROR_DS_OPERATIONS_ERROR
  { ERROR_DS_OPERATIONS_ERROR, &(O(object_Kerror_ds_operations_error)) },
 #endif
 #ifdef ERROR_DS_PROTOCOL_ERROR
  { ERROR_DS_PROTOCOL_ERROR, &(O(object_Kerror_ds_protocol_error)) },
 #endif
 #ifdef ERROR_DS_TIMELIMIT_EXCEEDED
  { ERROR_DS_TIMELIMIT_EXCEEDED, &(O(object_Kerror_ds_timelimit_exceeded)) },
 #endif
 #ifdef ERROR_DS_SIZELIMIT_EXCEEDED
  { ERROR_DS_SIZELIMIT_EXCEEDED, &(O(object_Kerror_ds_sizelimit_exceeded)) },
 #endif
 #ifdef ERROR_DS_ADMIN_LIMIT_EXCEEDED
  { ERROR_DS_ADMIN_LIMIT_EXCEEDED, &(O(object_Kerror_ds_admin_limit_exceeded)) },
 #endif
 #ifdef ERROR_DS_COMPARE_FALSE
  { ERROR_DS_COMPARE_FALSE, &(O(object_Kerror_ds_compare_false)) },
 #endif
 #ifdef ERROR_DS_COMPARE_TRUE
  { ERROR_DS_COMPARE_TRUE, &(O(object_Kerror_ds_compare_true)) },
 #endif
 #ifdef ERROR_DS_AUTH_METHOD_NOT_SUPPORTED
  { ERROR_DS_AUTH_METHOD_NOT_SUPPORTED, &(O(object_Kerror_ds_auth_method_not_supported)) },
 #endif
 #ifdef ERROR_DS_STRONG_AUTH_REQUIRED
  { ERROR_DS_STRONG_AUTH_REQUIRED, &(O(object_Kerror_ds_strong_auth_required)) },
 #endif
 #ifdef ERROR_DS_INAPPROPRIATE_AUTH
  { ERROR_DS_INAPPROPRIATE_AUTH, &(O(object_Kerror_ds_inappropriate_auth)) },
 #endif
 #ifdef ERROR_DS_AUTH_UNKNOWN
  { ERROR_DS_AUTH_UNKNOWN, &(O(object_Kerror_ds_auth_unknown)) },
 #endif
 #ifdef ERROR_DS_REFERRAL
  { ERROR_DS_REFERRAL, &(O(object_Kerror_ds_referral)) },
 #endif
 #ifdef ERROR_DS_UNAVAILABLE_CRIT_EXTENSION
  { ERROR_DS_UNAVAILABLE_CRIT_EXTENSION, &(O(object_Kerror_ds_unavailable_crit_extension)) },
 #endif
 #ifdef ERROR_DS_CONFIDENTIALITY_REQUIRED
  { ERROR_DS_CONFIDENTIALITY_REQUIRED, &(O(object_Kerror_ds_confidentiality_required)) },
 #endif
 #ifdef ERROR_DS_INAPPROPRIATE_MATCHING
  { ERROR_DS_INAPPROPRIATE_MATCHING, &(O(object_Kerror_ds_inappropriate_matching)) },
 #endif
 #ifdef ERROR_DS_CONSTRAINT_VIOLATION
  { ERROR_DS_CONSTRAINT_VIOLATION, &(O(object_Kerror_ds_constraint_violation)) },
 #endif
 #ifdef ERROR_DS_NO_SUCH_OBJECT
  { ERROR_DS_NO_SUCH_OBJECT, &(O(object_Kerror_ds_no_such_object)) },
 #endif
 #ifdef ERROR_DS_ALIAS_PROBLEM
  { ERROR_DS_ALIAS_PROBLEM, &(O(object_Kerror_ds_alias_problem)) },
 #endif
 #ifdef ERROR_DS_INVALID_DN_SYNTAX
  { ERROR_DS_INVALID_DN_SYNTAX, &(O(object_Kerror_ds_invalid_dn_syntax)) },
 #endif
 #ifdef ERROR_DS_IS_LEAF
  { ERROR_DS_IS_LEAF, &(O(object_Kerror_ds_is_leaf)) },
 #endif
 #ifdef ERROR_DS_ALIAS_DEREF_PROBLEM
  { ERROR_DS_ALIAS_DEREF_PROBLEM, &(O(object_Kerror_ds_alias_deref_problem)) },
 #endif
 #ifdef ERROR_DS_UNWILLING_TO_PERFORM
  { ERROR_DS_UNWILLING_TO_PERFORM, &(O(object_Kerror_ds_unwilling_to_perform)) },
 #endif
 #ifdef ERROR_DS_LOOP_DETECT
  { ERROR_DS_LOOP_DETECT, &(O(object_Kerror_ds_loop_detect)) },
 #endif
 #ifdef ERROR_DS_NAMING_VIOLATION
  { ERROR_DS_NAMING_VIOLATION, &(O(object_Kerror_ds_naming_violation)) },
 #endif
 #ifdef ERROR_DS_OBJECT_RESULTS_TOO_LARGE
  { ERROR_DS_OBJECT_RESULTS_TOO_LARGE, &(O(object_Kerror_ds_object_results_too_large)) },
 #endif
 #ifdef ERROR_DS_AFFECTS_MULTIPLE_DSAS
  { ERROR_DS_AFFECTS_MULTIPLE_DSAS, &(O(object_Kerror_ds_affects_multiple_dsas)) },
 #endif
 #ifdef ERROR_DS_SERVER_DOWN
  { ERROR_DS_SERVER_DOWN, &(O(object_Kerror_ds_server_down)) },
 #endif
 #ifdef ERROR_DS_LOCAL_ERROR
  { ERROR_DS_LOCAL_ERROR, &(O(object_Kerror_ds_local_error)) },
 #endif
 #ifdef ERROR_DS_ENCODING_ERROR
  { ERROR_DS_ENCODING_ERROR, &(O(object_Kerror_ds_encoding_error)) },
 #endif
 #ifdef ERROR_DS_DECODING_ERROR
  { ERROR_DS_DECODING_ERROR, &(O(object_Kerror_ds_decoding_error)) },
 #endif
 #ifdef ERROR_DS_FILTER_UNKNOWN
  { ERROR_DS_FILTER_UNKNOWN, &(O(object_Kerror_ds_filter_unknown)) },
 #endif
 #ifdef ERROR_DS_PARAM_ERROR
  { ERROR_DS_PARAM_ERROR, &(O(object_Kerror_ds_param_error)) },
 #endif
 #ifdef ERROR_DS_NOT_SUPPORTED
  { ERROR_DS_NOT_SUPPORTED, &(O(object_Kerror_ds_not_supported)) },
 #endif
 #ifdef ERROR_DS_NO_RESULTS_RETURNED
  { ERROR_DS_NO_RESULTS_RETURNED, &(O(object_Kerror_ds_no_results_returned)) },
 #endif
 #ifdef ERROR_DS_CONTROL_NOT_FOUND
  { ERROR_DS_CONTROL_NOT_FOUND, &(O(object_Kerror_ds_control_not_found)) },
 #endif
 #ifdef ERROR_DS_CLIENT_LOOP
  { ERROR_DS_CLIENT_LOOP, &(O(object_Kerror_ds_client_loop)) },
 #endif
 #ifdef ERROR_DS_REFERRAL_LIMIT_EXCEEDED
  { ERROR_DS_REFERRAL_LIMIT_EXCEEDED, &(O(object_Kerror_ds_referral_limit_exceeded)) },
 #endif
 #ifdef ERROR_DS_SORT_CONTROL_MISSING
  { ERROR_DS_SORT_CONTROL_MISSING, &(O(object_Kerror_ds_sort_control_missing)) },
 #endif
 #ifdef ERROR_DS_OFFSET_RANGE_ERROR
  { ERROR_DS_OFFSET_RANGE_ERROR, &(O(object_Kerror_ds_offset_range_error)) },
 #endif
 #ifdef ERROR_DS_ROOT_MUST_BE_NC
  { ERROR_DS_ROOT_MUST_BE_NC, &(O(object_Kerror_ds_root_must_be_nc)) },
 #endif
 #ifdef ERROR_DS_ADD_REPLICA_INHIBITED
  { ERROR_DS_ADD_REPLICA_INHIBITED, &(O(object_Kerror_ds_add_replica_inhibited)) },
 #endif
 #ifdef ERROR_DS_ATT_NOT_DEF_IN_SCHEMA
  { ERROR_DS_ATT_NOT_DEF_IN_SCHEMA, &(O(object_Kerror_ds_att_not_def_in_schema)) },
 #endif
 #ifdef ERROR_DS_MAX_OBJ_SIZE_EXCEEDED
  { ERROR_DS_MAX_OBJ_SIZE_EXCEEDED, &(O(object_Kerror_ds_max_obj_size_exceeded)) },
 #endif
 #ifdef ERROR_DS_OBJ_STRING_NAME_EXISTS
  { ERROR_DS_OBJ_STRING_NAME_EXISTS, &(O(object_Kerror_ds_obj_string_name_exists)) },
 #endif
 #ifdef ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA
  { ERROR_DS_NO_RDN_DEFINED_IN_SCHEMA, &(O(object_Kerror_ds_no_rdn_defined_in_schema)) },
 #endif
 #ifdef ERROR_DS_RDN_DOESNT_MATCH_SCHEMA
  { ERROR_DS_RDN_DOESNT_MATCH_SCHEMA, &(O(object_Kerror_ds_rdn_doesnt_match_schema)) },
 #endif
 #ifdef ERROR_DS_NO_REQUESTED_ATTS_FOUND
  { ERROR_DS_NO_REQUESTED_ATTS_FOUND, &(O(object_Kerror_ds_no_requested_atts_found)) },
 #endif
 #ifdef ERROR_DS_USER_BUFFER_TO_SMALL
  { ERROR_DS_USER_BUFFER_TO_SMALL, &(O(object_Kerror_ds_user_buffer_to_small)) },
 #endif
 #ifdef ERROR_DS_ATT_IS_NOT_ON_OBJ
  { ERROR_DS_ATT_IS_NOT_ON_OBJ, &(O(object_Kerror_ds_att_is_not_on_obj)) },
 #endif
 #ifdef ERROR_DS_ILLEGAL_MOD_OPERATION
  { ERROR_DS_ILLEGAL_MOD_OPERATION, &(O(object_Kerror_ds_illegal_mod_operation)) },
 #endif
 #ifdef ERROR_DS_OBJ_TOO_LARGE
  { ERROR_DS_OBJ_TOO_LARGE, &(O(object_Kerror_ds_obj_too_large)) },
 #endif
 #ifdef ERROR_DS_BAD_INSTANCE_TYPE
  { ERROR_DS_BAD_INSTANCE_TYPE, &(O(object_Kerror_ds_bad_instance_type)) },
 #endif
 #ifdef ERROR_DS_MASTERDSA_REQUIRED
  { ERROR_DS_MASTERDSA_REQUIRED, &(O(object_Kerror_ds_masterdsa_required)) },
 #endif
 #ifdef ERROR_DS_OBJECT_CLASS_REQUIRED
  { ERROR_DS_OBJECT_CLASS_REQUIRED, &(O(object_Kerror_ds_object_class_required)) },
 #endif
 #ifdef ERROR_DS_MISSING_REQUIRED_ATT
  { ERROR_DS_MISSING_REQUIRED_ATT, &(O(object_Kerror_ds_missing_required_att)) },
 #endif
 #ifdef ERROR_DS_ATT_NOT_DEF_FOR_CLASS
  { ERROR_DS_ATT_NOT_DEF_FOR_CLASS, &(O(object_Kerror_ds_att_not_def_for_class)) },
 #endif
 #ifdef ERROR_DS_ATT_ALREADY_EXISTS
  { ERROR_DS_ATT_ALREADY_EXISTS, &(O(object_Kerror_ds_att_already_exists)) },
 #endif
 #ifdef ERROR_DS_CANT_ADD_ATT_VALUES
  { ERROR_DS_CANT_ADD_ATT_VALUES, &(O(object_Kerror_ds_cant_add_att_values)) },
 #endif
 #ifdef ERROR_DS_SINGLE_VALUE_CONSTRAINT
  { ERROR_DS_SINGLE_VALUE_CONSTRAINT, &(O(object_Kerror_ds_single_value_constraint)) },
 #endif
 #ifdef ERROR_DS_RANGE_CONSTRAINT
  { ERROR_DS_RANGE_CONSTRAINT, &(O(object_Kerror_ds_range_constraint)) },
 #endif
 #ifdef ERROR_DS_ATT_VAL_ALREADY_EXISTS
  { ERROR_DS_ATT_VAL_ALREADY_EXISTS, &(O(object_Kerror_ds_att_val_already_exists)) },
 #endif
 #ifdef ERROR_DS_CANT_REM_MISSING_ATT
  { ERROR_DS_CANT_REM_MISSING_ATT, &(O(object_Kerror_ds_cant_rem_missing_att)) },
 #endif
 #ifdef ERROR_DS_CANT_REM_MISSING_ATT_VAL
  { ERROR_DS_CANT_REM_MISSING_ATT_VAL, &(O(object_Kerror_ds_cant_rem_missing_att_val)) },
 #endif
 #ifdef ERROR_DS_ROOT_CANT_BE_SUBREF
  { ERROR_DS_ROOT_CANT_BE_SUBREF, &(O(object_Kerror_ds_root_cant_be_subref)) },
 #endif
 #ifdef ERROR_DS_NO_CHAINING
  { ERROR_DS_NO_CHAINING, &(O(object_Kerror_ds_no_chaining)) },
 #endif
 #ifdef ERROR_DS_NO_CHAINED_EVAL
  { ERROR_DS_NO_CHAINED_EVAL, &(O(object_Kerror_ds_no_chained_eval)) },
 #endif
 #ifdef ERROR_DS_NO_PARENT_OBJECT
  { ERROR_DS_NO_PARENT_OBJECT, &(O(object_Kerror_ds_no_parent_object)) },
 #endif
 #ifdef ERROR_DS_PARENT_IS_AN_ALIAS
  { ERROR_DS_PARENT_IS_AN_ALIAS, &(O(object_Kerror_ds_parent_is_an_alias)) },
 #endif
 #ifdef ERROR_DS_CANT_MIX_MASTER_AND_REPS
  { ERROR_DS_CANT_MIX_MASTER_AND_REPS, &(O(object_Kerror_ds_cant_mix_master_and_reps)) },
 #endif
 #ifdef ERROR_DS_CHILDREN_EXIST
  { ERROR_DS_CHILDREN_EXIST, &(O(object_Kerror_ds_children_exist)) },
 #endif
 #ifdef ERROR_DS_OBJ_NOT_FOUND
  { ERROR_DS_OBJ_NOT_FOUND, &(O(object_Kerror_ds_obj_not_found)) },
 #endif
 #ifdef ERROR_DS_ALIASED_OBJ_MISSING
  { ERROR_DS_ALIASED_OBJ_MISSING, &(O(object_Kerror_ds_aliased_obj_missing)) },
 #endif
 #ifdef ERROR_DS_BAD_NAME_SYNTAX
  { ERROR_DS_BAD_NAME_SYNTAX, &(O(object_Kerror_ds_bad_name_syntax)) },
 #endif
 #ifdef ERROR_DS_ALIAS_POINTS_TO_ALIAS
  { ERROR_DS_ALIAS_POINTS_TO_ALIAS, &(O(object_Kerror_ds_alias_points_to_alias)) },
 #endif
 #ifdef ERROR_DS_CANT_DEREF_ALIAS
  { ERROR_DS_CANT_DEREF_ALIAS, &(O(object_Kerror_ds_cant_deref_alias)) },
 #endif
 #ifdef ERROR_DS_OUT_OF_SCOPE
  { ERROR_DS_OUT_OF_SCOPE, &(O(object_Kerror_ds_out_of_scope)) },
 #endif
 #ifdef ERROR_DS_OBJECT_BEING_REMOVED
  { ERROR_DS_OBJECT_BEING_REMOVED, &(O(object_Kerror_ds_object_being_removed)) },
 #endif
 #ifdef ERROR_DS_CANT_DELETE_DSA_OBJ
  { ERROR_DS_CANT_DELETE_DSA_OBJ, &(O(object_Kerror_ds_cant_delete_dsa_obj)) },
 #endif
 #ifdef ERROR_DS_GENERIC_ERROR
  { ERROR_DS_GENERIC_ERROR, &(O(object_Kerror_ds_generic_error)) },
 #endif
 #ifdef ERROR_DS_DSA_MUST_BE_INT_MASTER
  { ERROR_DS_DSA_MUST_BE_INT_MASTER, &(O(object_Kerror_ds_dsa_must_be_int_master)) },
 #endif
 #ifdef ERROR_DS_CLASS_NOT_DSA
  { ERROR_DS_CLASS_NOT_DSA, &(O(object_Kerror_ds_class_not_dsa)) },
 #endif
 #ifdef ERROR_DS_INSUFF_ACCESS_RIGHTS
  { ERROR_DS_INSUFF_ACCESS_RIGHTS, &(O(object_Kerror_ds_insuff_access_rights)) },
 #endif
 #ifdef ERROR_DS_ILLEGAL_SUPERIOR
  { ERROR_DS_ILLEGAL_SUPERIOR, &(O(object_Kerror_ds_illegal_superior)) },
 #endif
 #ifdef ERROR_DS_ATTRIBUTE_OWNED_BY_SAM
  { ERROR_DS_ATTRIBUTE_OWNED_BY_SAM, &(O(object_Kerror_ds_attribute_owned_by_sam)) },
 #endif
 #ifdef ERROR_DS_NAME_TOO_MANY_PARTS
  { ERROR_DS_NAME_TOO_MANY_PARTS, &(O(object_Kerror_ds_name_too_many_parts)) },
 #endif
 #ifdef ERROR_DS_NAME_TOO_LONG
  { ERROR_DS_NAME_TOO_LONG, &(O(object_Kerror_ds_name_too_long)) },
 #endif
 #ifdef ERROR_DS_NAME_VALUE_TOO_LONG
  { ERROR_DS_NAME_VALUE_TOO_LONG, &(O(object_Kerror_ds_name_value_too_long)) },
 #endif
 #ifdef ERROR_DS_NAME_UNPARSEABLE
  { ERROR_DS_NAME_UNPARSEABLE, &(O(object_Kerror_ds_name_unparseable)) },
 #endif
 #ifdef ERROR_DS_NAME_TYPE_UNKNOWN
  { ERROR_DS_NAME_TYPE_UNKNOWN, &(O(object_Kerror_ds_name_type_unknown)) },
 #endif
 #ifdef ERROR_DS_NOT_AN_OBJECT
  { ERROR_DS_NOT_AN_OBJECT, &(O(object_Kerror_ds_not_an_object)) },
 #endif
 #ifdef ERROR_DS_SEC_DESC_TOO_SHORT
  { ERROR_DS_SEC_DESC_TOO_SHORT, &(O(object_Kerror_ds_sec_desc_too_short)) },
 #endif
 #ifdef ERROR_DS_SEC_DESC_INVALID
  { ERROR_DS_SEC_DESC_INVALID, &(O(object_Kerror_ds_sec_desc_invalid)) },
 #endif
 #ifdef ERROR_DS_NO_DELETED_NAME
  { ERROR_DS_NO_DELETED_NAME, &(O(object_Kerror_ds_no_deleted_name)) },
 #endif
 #ifdef ERROR_DS_SUBREF_MUST_HAVE_PARENT
  { ERROR_DS_SUBREF_MUST_HAVE_PARENT, &(O(object_Kerror_ds_subref_must_have_parent)) },
 #endif
 #ifdef ERROR_DS_NCNAME_MUST_BE_NC
  { ERROR_DS_NCNAME_MUST_BE_NC, &(O(object_Kerror_ds_ncname_must_be_nc)) },
 #endif
 #ifdef ERROR_DS_CANT_ADD_SYSTEM_ONLY
  { ERROR_DS_CANT_ADD_SYSTEM_ONLY, &(O(object_Kerror_ds_cant_add_system_only)) },
 #endif
 #ifdef ERROR_DS_CLASS_MUST_BE_CONCRETE
  { ERROR_DS_CLASS_MUST_BE_CONCRETE, &(O(object_Kerror_ds_class_must_be_concrete)) },
 #endif
 #ifdef ERROR_DS_INVALID_DMD
  { ERROR_DS_INVALID_DMD, &(O(object_Kerror_ds_invalid_dmd)) },
 #endif
 #ifdef ERROR_DS_OBJ_GUID_EXISTS
  { ERROR_DS_OBJ_GUID_EXISTS, &(O(object_Kerror_ds_obj_guid_exists)) },
 #endif
 #ifdef ERROR_DS_NOT_ON_BACKLINK
  { ERROR_DS_NOT_ON_BACKLINK, &(O(object_Kerror_ds_not_on_backlink)) },
 #endif
 #ifdef ERROR_DS_NO_CROSSREF_FOR_NC
  { ERROR_DS_NO_CROSSREF_FOR_NC, &(O(object_Kerror_ds_no_crossref_for_nc)) },
 #endif
 #ifdef ERROR_DS_SHUTTING_DOWN
  { ERROR_DS_SHUTTING_DOWN, &(O(object_Kerror_ds_shutting_down)) },
 #endif
 #ifdef ERROR_DS_UNKNOWN_OPERATION
  { ERROR_DS_UNKNOWN_OPERATION, &(O(object_Kerror_ds_unknown_operation)) },
 #endif
 #ifdef ERROR_DS_INVALID_ROLE_OWNER
  { ERROR_DS_INVALID_ROLE_OWNER, &(O(object_Kerror_ds_invalid_role_owner)) },
 #endif
 #ifdef ERROR_DS_COULDNT_CONTACT_FSMO
  { ERROR_DS_COULDNT_CONTACT_FSMO, &(O(object_Kerror_ds_couldnt_contact_fsmo)) },
 #endif
 #ifdef ERROR_DS_CROSS_NC_DN_RENAME
  { ERROR_DS_CROSS_NC_DN_RENAME, &(O(object_Kerror_ds_cross_nc_dn_rename)) },
 #endif
 #ifdef ERROR_DS_CANT_MOD_SYSTEM_ONLY
  { ERROR_DS_CANT_MOD_SYSTEM_ONLY, &(O(object_Kerror_ds_cant_mod_system_only)) },
 #endif
 #ifdef ERROR_DS_REPLICATOR_ONLY
  { ERROR_DS_REPLICATOR_ONLY, &(O(object_Kerror_ds_replicator_only)) },
 #endif
 #ifdef ERROR_DS_OBJ_CLASS_NOT_DEFINED
  { ERROR_DS_OBJ_CLASS_NOT_DEFINED, &(O(object_Kerror_ds_obj_class_not_defined)) },
 #endif
 #ifdef ERROR_DS_OBJ_CLASS_NOT_SUBCLASS
  { ERROR_DS_OBJ_CLASS_NOT_SUBCLASS, &(O(object_Kerror_ds_obj_class_not_subclass)) },
 #endif
 #ifdef ERROR_DS_NAME_REFERENCE_INVALID
  { ERROR_DS_NAME_REFERENCE_INVALID, &(O(object_Kerror_ds_name_reference_invalid)) },
 #endif
 #ifdef ERROR_DS_CROSS_REF_EXISTS
  { ERROR_DS_CROSS_REF_EXISTS, &(O(object_Kerror_ds_cross_ref_exists)) },
 #endif
 #ifdef ERROR_DS_CANT_DEL_MASTER_CROSSREF
  { ERROR_DS_CANT_DEL_MASTER_CROSSREF, &(O(object_Kerror_ds_cant_del_master_crossref)) },
 #endif
 #ifdef ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD
  { ERROR_DS_SUBTREE_NOTIFY_NOT_NC_HEAD, &(O(object_Kerror_ds_subtree_notify_not_nc_head)) },
 #endif
 #ifdef ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX
  { ERROR_DS_NOTIFY_FILTER_TOO_COMPLEX, &(O(object_Kerror_ds_notify_filter_too_complex)) },
 #endif
 #ifdef ERROR_DS_DUP_RDN
  { ERROR_DS_DUP_RDN, &(O(object_Kerror_ds_dup_rdn)) },
 #endif
 #ifdef ERROR_DS_DUP_OID
  { ERROR_DS_DUP_OID, &(O(object_Kerror_ds_dup_oid)) },
 #endif
 #ifdef ERROR_DS_DUP_MAPI_ID
  { ERROR_DS_DUP_MAPI_ID, &(O(object_Kerror_ds_dup_mapi_id)) },
 #endif
 #ifdef ERROR_DS_DUP_SCHEMA_ID_GUID
  { ERROR_DS_DUP_SCHEMA_ID_GUID, &(O(object_Kerror_ds_dup_schema_id_guid)) },
 #endif
 #ifdef ERROR_DS_DUP_LDAP_DISPLAY_NAME
  { ERROR_DS_DUP_LDAP_DISPLAY_NAME, &(O(object_Kerror_ds_dup_ldap_display_name)) },
 #endif
 #ifdef ERROR_DS_SEMANTIC_ATT_TEST
  { ERROR_DS_SEMANTIC_ATT_TEST, &(O(object_Kerror_ds_semantic_att_test)) },
 #endif
 #ifdef ERROR_DS_SYNTAX_MISMATCH
  { ERROR_DS_SYNTAX_MISMATCH, &(O(object_Kerror_ds_syntax_mismatch)) },
 #endif
 #ifdef ERROR_DS_EXISTS_IN_MUST_HAVE
  { ERROR_DS_EXISTS_IN_MUST_HAVE, &(O(object_Kerror_ds_exists_in_must_have)) },
 #endif
 #ifdef ERROR_DS_EXISTS_IN_MAY_HAVE
  { ERROR_DS_EXISTS_IN_MAY_HAVE, &(O(object_Kerror_ds_exists_in_may_have)) },
 #endif
 #ifdef ERROR_DS_NONEXISTENT_MAY_HAVE
  { ERROR_DS_NONEXISTENT_MAY_HAVE, &(O(object_Kerror_ds_nonexistent_may_have)) },
 #endif
 #ifdef ERROR_DS_NONEXISTENT_MUST_HAVE
  { ERROR_DS_NONEXISTENT_MUST_HAVE, &(O(object_Kerror_ds_nonexistent_must_have)) },
 #endif
 #ifdef ERROR_DS_AUX_CLS_TEST_FAIL
  { ERROR_DS_AUX_CLS_TEST_FAIL, &(O(object_Kerror_ds_aux_cls_test_fail)) },
 #endif
 #ifdef ERROR_DS_NONEXISTENT_POSS_SUP
  { ERROR_DS_NONEXISTENT_POSS_SUP, &(O(object_Kerror_ds_nonexistent_poss_sup)) },
 #endif
 #ifdef ERROR_DS_SUB_CLS_TEST_FAIL
  { ERROR_DS_SUB_CLS_TEST_FAIL, &(O(object_Kerror_ds_sub_cls_test_fail)) },
 #endif
 #ifdef ERROR_DS_BAD_RDN_ATT_ID_SYNTAX
  { ERROR_DS_BAD_RDN_ATT_ID_SYNTAX, &(O(object_Kerror_ds_bad_rdn_att_id_syntax)) },
 #endif
 #ifdef ERROR_DS_EXISTS_IN_AUX_CLS
  { ERROR_DS_EXISTS_IN_AUX_CLS, &(O(object_Kerror_ds_exists_in_aux_cls)) },
 #endif
 #ifdef ERROR_DS_EXISTS_IN_SUB_CLS
  { ERROR_DS_EXISTS_IN_SUB_CLS, &(O(object_Kerror_ds_exists_in_sub_cls)) },
 #endif
 #ifdef ERROR_DS_EXISTS_IN_POSS_SUP
  { ERROR_DS_EXISTS_IN_POSS_SUP, &(O(object_Kerror_ds_exists_in_poss_sup)) },
 #endif
 #ifdef ERROR_DS_RECALCSCHEMA_FAILED
  { ERROR_DS_RECALCSCHEMA_FAILED, &(O(object_Kerror_ds_recalcschema_failed)) },
 #endif
 #ifdef ERROR_DS_TREE_DELETE_NOT_FINISHED
  { ERROR_DS_TREE_DELETE_NOT_FINISHED, &(O(object_Kerror_ds_tree_delete_not_finished)) },
 #endif
 #ifdef ERROR_DS_CANT_DELETE
  { ERROR_DS_CANT_DELETE, &(O(object_Kerror_ds_cant_delete)) },
 #endif
 #ifdef ERROR_DS_ATT_SCHEMA_REQ_ID
  { ERROR_DS_ATT_SCHEMA_REQ_ID, &(O(object_Kerror_ds_att_schema_req_id)) },
 #endif
 #ifdef ERROR_DS_BAD_ATT_SCHEMA_SYNTAX
  { ERROR_DS_BAD_ATT_SCHEMA_SYNTAX, &(O(object_Kerror_ds_bad_att_schema_syntax)) },
 #endif
 #ifdef ERROR_DS_CANT_CACHE_ATT
  { ERROR_DS_CANT_CACHE_ATT, &(O(object_Kerror_ds_cant_cache_att)) },
 #endif
 #ifdef ERROR_DS_CANT_CACHE_CLASS
  { ERROR_DS_CANT_CACHE_CLASS, &(O(object_Kerror_ds_cant_cache_class)) },
 #endif
 #ifdef ERROR_DS_CANT_REMOVE_ATT_CACHE
  { ERROR_DS_CANT_REMOVE_ATT_CACHE, &(O(object_Kerror_ds_cant_remove_att_cache)) },
 #endif
 #ifdef ERROR_DS_CANT_REMOVE_CLASS_CACHE
  { ERROR_DS_CANT_REMOVE_CLASS_CACHE, &(O(object_Kerror_ds_cant_remove_class_cache)) },
 #endif
 #ifdef ERROR_DS_CANT_RETRIEVE_DN
  { ERROR_DS_CANT_RETRIEVE_DN, &(O(object_Kerror_ds_cant_retrieve_dn)) },
 #endif
 #ifdef ERROR_DS_MISSING_SUPREF
  { ERROR_DS_MISSING_SUPREF, &(O(object_Kerror_ds_missing_supref)) },
 #endif
 #ifdef ERROR_DS_CANT_RETRIEVE_INSTANCE
  { ERROR_DS_CANT_RETRIEVE_INSTANCE, &(O(object_Kerror_ds_cant_retrieve_instance)) },
 #endif
 #ifdef ERROR_DS_CODE_INCONSISTENCY
  { ERROR_DS_CODE_INCONSISTENCY, &(O(object_Kerror_ds_code_inconsistency)) },
 #endif
 #ifdef ERROR_DS_DATABASE_ERROR
  { ERROR_DS_DATABASE_ERROR, &(O(object_Kerror_ds_database_error)) },
 #endif
 #ifdef ERROR_DS_GOVERNSID_MISSING
  { ERROR_DS_GOVERNSID_MISSING, &(O(object_Kerror_ds_governsid_missing)) },
 #endif
 #ifdef ERROR_DS_MISSING_EXPECTED_ATT
  { ERROR_DS_MISSING_EXPECTED_ATT, &(O(object_Kerror_ds_missing_expected_att)) },
 #endif
 #ifdef ERROR_DS_NCNAME_MISSING_CR_REF
  { ERROR_DS_NCNAME_MISSING_CR_REF, &(O(object_Kerror_ds_ncname_missing_cr_ref)) },
 #endif
 #ifdef ERROR_DS_SECURITY_CHECKING_ERROR
  { ERROR_DS_SECURITY_CHECKING_ERROR, &(O(object_Kerror_ds_security_checking_error)) },
 #endif
 #ifdef ERROR_DS_SCHEMA_NOT_LOADED
  { ERROR_DS_SCHEMA_NOT_LOADED, &(O(object_Kerror_ds_schema_not_loaded)) },
 #endif
 #ifdef ERROR_DS_SCHEMA_ALLOC_FAILED
  { ERROR_DS_SCHEMA_ALLOC_FAILED, &(O(object_Kerror_ds_schema_alloc_failed)) },
 #endif
 #ifdef ERROR_DS_ATT_SCHEMA_REQ_SYNTAX
  { ERROR_DS_ATT_SCHEMA_REQ_SYNTAX, &(O(object_Kerror_ds_att_schema_req_syntax)) },
 #endif
 #ifdef ERROR_DS_GCVERIFY_ERROR
  { ERROR_DS_GCVERIFY_ERROR, &(O(object_Kerror_ds_gcverify_error)) },
 #endif
 #ifdef ERROR_DS_DRA_SCHEMA_MISMATCH
  { ERROR_DS_DRA_SCHEMA_MISMATCH, &(O(object_Kerror_ds_dra_schema_mismatch)) },
 #endif
 #ifdef ERROR_DS_CANT_FIND_DSA_OBJ
  { ERROR_DS_CANT_FIND_DSA_OBJ, &(O(object_Kerror_ds_cant_find_dsa_obj)) },
 #endif
 #ifdef ERROR_DS_CANT_FIND_EXPECTED_NC
  { ERROR_DS_CANT_FIND_EXPECTED_NC, &(O(object_Kerror_ds_cant_find_expected_nc)) },
 #endif
 #ifdef ERROR_DS_CANT_FIND_NC_IN_CACHE
  { ERROR_DS_CANT_FIND_NC_IN_CACHE, &(O(object_Kerror_ds_cant_find_nc_in_cache)) },
 #endif
 #ifdef ERROR_DS_CANT_RETRIEVE_CHILD
  { ERROR_DS_CANT_RETRIEVE_CHILD, &(O(object_Kerror_ds_cant_retrieve_child)) },
 #endif
 #ifdef ERROR_DS_SECURITY_ILLEGAL_MODIFY
  { ERROR_DS_SECURITY_ILLEGAL_MODIFY, &(O(object_Kerror_ds_security_illegal_modify)) },
 #endif
 #ifdef ERROR_DS_CANT_REPLACE_HIDDEN_REC
  { ERROR_DS_CANT_REPLACE_HIDDEN_REC, &(O(object_Kerror_ds_cant_replace_hidden_rec)) },
 #endif
 #ifdef ERROR_DS_BAD_HIERARCHY_FILE
  { ERROR_DS_BAD_HIERARCHY_FILE, &(O(object_Kerror_ds_bad_hierarchy_file)) },
 #endif
 #ifdef ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED
  { ERROR_DS_BUILD_HIERARCHY_TABLE_FAILED, &(O(object_Kerror_ds_build_hierarchy_table_failed)) },
 #endif
 #ifdef ERROR_DS_CONFIG_PARAM_MISSING
  { ERROR_DS_CONFIG_PARAM_MISSING, &(O(object_Kerror_ds_config_param_missing)) },
 #endif
 #ifdef ERROR_DS_COUNTING_AB_INDICES_FAILED
  { ERROR_DS_COUNTING_AB_INDICES_FAILED, &(O(object_Kerror_ds_counting_ab_indices_failed)) },
 #endif
 #ifdef ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED
  { ERROR_DS_HIERARCHY_TABLE_MALLOC_FAILED, &(O(object_Kerror_ds_hierarchy_table_malloc_failed)) },
 #endif
 #ifdef ERROR_DS_INTERNAL_FAILURE
  { ERROR_DS_INTERNAL_FAILURE, &(O(object_Kerror_ds_internal_failure)) },
 #endif
 #ifdef ERROR_DS_UNKNOWN_ERROR
  { ERROR_DS_UNKNOWN_ERROR, &(O(object_Kerror_ds_unknown_error)) },
 #endif
 #ifdef ERROR_DS_ROOT_REQUIRES_CLASS_TOP
  { ERROR_DS_ROOT_REQUIRES_CLASS_TOP, &(O(object_Kerror_ds_root_requires_class_top)) },
 #endif
 #ifdef ERROR_DS_REFUSING_FSMO_ROLES
  { ERROR_DS_REFUSING_FSMO_ROLES, &(O(object_Kerror_ds_refusing_fsmo_roles)) },
 #endif
 #ifdef ERROR_DS_MISSING_FSMO_SETTINGS
  { ERROR_DS_MISSING_FSMO_SETTINGS, &(O(object_Kerror_ds_missing_fsmo_settings)) },
 #endif
 #ifdef ERROR_DS_UNABLE_TO_SURRENDER_ROLES
  { ERROR_DS_UNABLE_TO_SURRENDER_ROLES, &(O(object_Kerror_ds_unable_to_surrender_roles)) },
 #endif
 #ifdef ERROR_DS_DRA_GENERIC
  { ERROR_DS_DRA_GENERIC, &(O(object_Kerror_ds_dra_generic)) },
 #endif
 #ifdef ERROR_DS_DRA_INVALID_PARAMETER
  { ERROR_DS_DRA_INVALID_PARAMETER, &(O(object_Kerror_ds_dra_invalid_parameter)) },
 #endif
 #ifdef ERROR_DS_DRA_BUSY
  { ERROR_DS_DRA_BUSY, &(O(object_Kerror_ds_dra_busy)) },
 #endif
 #ifdef ERROR_DS_DRA_BAD_DN
  { ERROR_DS_DRA_BAD_DN, &(O(object_Kerror_ds_dra_bad_dn)) },
 #endif
 #ifdef ERROR_DS_DRA_BAD_NC
  { ERROR_DS_DRA_BAD_NC, &(O(object_Kerror_ds_dra_bad_nc)) },
 #endif
 #ifdef ERROR_DS_DRA_DN_EXISTS
  { ERROR_DS_DRA_DN_EXISTS, &(O(object_Kerror_ds_dra_dn_exists)) },
 #endif
 #ifdef ERROR_DS_DRA_INTERNAL_ERROR
  { ERROR_DS_DRA_INTERNAL_ERROR, &(O(object_Kerror_ds_dra_internal_error)) },
 #endif
 #ifdef ERROR_DS_DRA_INCONSISTENT_DIT
  { ERROR_DS_DRA_INCONSISTENT_DIT, &(O(object_Kerror_ds_dra_inconsistent_dit)) },
 #endif
 #ifdef ERROR_DS_DRA_CONNECTION_FAILED
  { ERROR_DS_DRA_CONNECTION_FAILED, &(O(object_Kerror_ds_dra_connection_failed)) },
 #endif
 #ifdef ERROR_DS_DRA_BAD_INSTANCE_TYPE
  { ERROR_DS_DRA_BAD_INSTANCE_TYPE, &(O(object_Kerror_ds_dra_bad_instance_type)) },
 #endif
 #ifdef ERROR_DS_DRA_OUT_OF_MEM
  { ERROR_DS_DRA_OUT_OF_MEM, &(O(object_Kerror_ds_dra_out_of_mem)) },
 #endif
 #ifdef ERROR_DS_DRA_MAIL_PROBLEM
  { ERROR_DS_DRA_MAIL_PROBLEM, &(O(object_Kerror_ds_dra_mail_problem)) },
 #endif
 #ifdef ERROR_DS_DRA_REF_ALREADY_EXISTS
  { ERROR_DS_DRA_REF_ALREADY_EXISTS, &(O(object_Kerror_ds_dra_ref_already_exists)) },
 #endif
 #ifdef ERROR_DS_DRA_REF_NOT_FOUND
  { ERROR_DS_DRA_REF_NOT_FOUND, &(O(object_Kerror_ds_dra_ref_not_found)) },
 #endif
 #ifdef ERROR_DS_DRA_OBJ_IS_REP_SOURCE
  { ERROR_DS_DRA_OBJ_IS_REP_SOURCE, &(O(object_Kerror_ds_dra_obj_is_rep_source)) },
 #endif
 #ifdef ERROR_DS_DRA_DB_ERROR
  { ERROR_DS_DRA_DB_ERROR, &(O(object_Kerror_ds_dra_db_error)) },
 #endif
 #ifdef ERROR_DS_DRA_NO_REPLICA
  { ERROR_DS_DRA_NO_REPLICA, &(O(object_Kerror_ds_dra_no_replica)) },
 #endif
 #ifdef ERROR_DS_DRA_ACCESS_DENIED
  { ERROR_DS_DRA_ACCESS_DENIED, &(O(object_Kerror_ds_dra_access_denied)) },
 #endif
 #ifdef ERROR_DS_DRA_NOT_SUPPORTED
  { ERROR_DS_DRA_NOT_SUPPORTED, &(O(object_Kerror_ds_dra_not_supported)) },
 #endif
 #ifdef ERROR_DS_DRA_RPC_CANCELLED
  { ERROR_DS_DRA_RPC_CANCELLED, &(O(object_Kerror_ds_dra_rpc_cancelled)) },
 #endif
 #ifdef ERROR_DS_DRA_SOURCE_DISABLED
  { ERROR_DS_DRA_SOURCE_DISABLED, &(O(object_Kerror_ds_dra_source_disabled)) },
 #endif
 #ifdef ERROR_DS_DRA_SINK_DISABLED
  { ERROR_DS_DRA_SINK_DISABLED, &(O(object_Kerror_ds_dra_sink_disabled)) },
 #endif
 #ifdef ERROR_DS_DRA_NAME_COLLISION
  { ERROR_DS_DRA_NAME_COLLISION, &(O(object_Kerror_ds_dra_name_collision)) },
 #endif
 #ifdef ERROR_DS_DRA_SOURCE_REINSTALLED
  { ERROR_DS_DRA_SOURCE_REINSTALLED, &(O(object_Kerror_ds_dra_source_reinstalled)) },
 #endif
 #ifdef ERROR_DS_DRA_MISSING_PARENT
  { ERROR_DS_DRA_MISSING_PARENT, &(O(object_Kerror_ds_dra_missing_parent)) },
 #endif
 #ifdef ERROR_DS_DRA_PREEMPTED
  { ERROR_DS_DRA_PREEMPTED, &(O(object_Kerror_ds_dra_preempted)) },
 #endif
 #ifdef ERROR_DS_DRA_ABANDON_SYNC
  { ERROR_DS_DRA_ABANDON_SYNC, &(O(object_Kerror_ds_dra_abandon_sync)) },
 #endif
 #ifdef ERROR_DS_DRA_SHUTDOWN
  { ERROR_DS_DRA_SHUTDOWN, &(O(object_Kerror_ds_dra_shutdown)) },
 #endif
 #ifdef ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET
  { ERROR_DS_DRA_INCOMPATIBLE_PARTIAL_SET, &(O(object_Kerror_ds_dra_incompatible_partial_set)) },
 #endif
 #ifdef ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA
  { ERROR_DS_DRA_SOURCE_IS_PARTIAL_REPLICA, &(O(object_Kerror_ds_dra_source_is_partial_replica)) },
 #endif
 #ifdef ERROR_DS_DRA_EXTN_CONNECTION_FAILED
  { ERROR_DS_DRA_EXTN_CONNECTION_FAILED, &(O(object_Kerror_ds_dra_extn_connection_failed)) },
 #endif
 #ifdef ERROR_DS_INSTALL_SCHEMA_MISMATCH
  { ERROR_DS_INSTALL_SCHEMA_MISMATCH, &(O(object_Kerror_ds_install_schema_mismatch)) },
 #endif
 #ifdef ERROR_DS_DUP_LINK_ID
  { ERROR_DS_DUP_LINK_ID, &(O(object_Kerror_ds_dup_link_id)) },
 #endif
 #ifdef ERROR_DS_NAME_ERROR_RESOLVING
  { ERROR_DS_NAME_ERROR_RESOLVING, &(O(object_Kerror_ds_name_error_resolving)) },
 #endif
 #ifdef ERROR_DS_NAME_ERROR_NOT_FOUND
  { ERROR_DS_NAME_ERROR_NOT_FOUND, &(O(object_Kerror_ds_name_error_not_found)) },
 #endif
 #ifdef ERROR_DS_NAME_ERROR_NOT_UNIQUE
  { ERROR_DS_NAME_ERROR_NOT_UNIQUE, &(O(object_Kerror_ds_name_error_not_unique)) },
 #endif
 #ifdef ERROR_DS_NAME_ERROR_NO_MAPPING
  { ERROR_DS_NAME_ERROR_NO_MAPPING, &(O(object_Kerror_ds_name_error_no_mapping)) },
 #endif
 #ifdef ERROR_DS_NAME_ERROR_DOMAIN_ONLY
  { ERROR_DS_NAME_ERROR_DOMAIN_ONLY, &(O(object_Kerror_ds_name_error_domain_only)) },
 #endif
 #ifdef ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING
  { ERROR_DS_NAME_ERROR_NO_SYNTACTICAL_MAPPING, &(O(object_Kerror_ds_name_error_no_syntactical_mapping)) },
 #endif
 #ifdef ERROR_DS_CONSTRUCTED_ATT_MOD
  { ERROR_DS_CONSTRUCTED_ATT_MOD, &(O(object_Kerror_ds_constructed_att_mod)) },
 #endif
 #ifdef ERROR_DS_WRONG_OM_OBJ_CLASS
  { ERROR_DS_WRONG_OM_OBJ_CLASS, &(O(object_Kerror_ds_wrong_om_obj_class)) },
 #endif
 #ifdef ERROR_DS_DRA_REPL_PENDING
  { ERROR_DS_DRA_REPL_PENDING, &(O(object_Kerror_ds_dra_repl_pending)) },
 #endif
 #ifdef ERROR_DS_DS_REQUIRED
  { ERROR_DS_DS_REQUIRED, &(O(object_Kerror_ds_ds_required)) },
 #endif
 #ifdef ERROR_DS_INVALID_LDAP_DISPLAY_NAME
  { ERROR_DS_INVALID_LDAP_DISPLAY_NAME, &(O(object_Kerror_ds_invalid_ldap_display_name)) },
 #endif
 #ifdef ERROR_DS_NON_BASE_SEARCH
  { ERROR_DS_NON_BASE_SEARCH, &(O(object_Kerror_ds_non_base_search)) },
 #endif
 #ifdef ERROR_DS_CANT_RETRIEVE_ATTS
  { ERROR_DS_CANT_RETRIEVE_ATTS, &(O(object_Kerror_ds_cant_retrieve_atts)) },
 #endif
 #ifdef ERROR_DS_BACKLINK_WITHOUT_LINK
  { ERROR_DS_BACKLINK_WITHOUT_LINK, &(O(object_Kerror_ds_backlink_without_link)) },
 #endif
 #ifdef ERROR_DS_EPOCH_MISMATCH
  { ERROR_DS_EPOCH_MISMATCH, &(O(object_Kerror_ds_epoch_mismatch)) },
 #endif
 #ifdef ERROR_DS_SRC_NAME_MISMATCH
  { ERROR_DS_SRC_NAME_MISMATCH, &(O(object_Kerror_ds_src_name_mismatch)) },
 #endif
 #ifdef ERROR_DS_SRC_AND_DST_NC_IDENTICAL
  { ERROR_DS_SRC_AND_DST_NC_IDENTICAL, &(O(object_Kerror_ds_src_and_dst_nc_identical)) },
 #endif
 #ifdef ERROR_DS_DST_NC_MISMATCH
  { ERROR_DS_DST_NC_MISMATCH, &(O(object_Kerror_ds_dst_nc_mismatch)) },
 #endif
 #ifdef ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC
  { ERROR_DS_NOT_AUTHORITIVE_FOR_DST_NC, &(O(object_Kerror_ds_not_authoritive_for_dst_nc)) },
 #endif
 #ifdef ERROR_DS_SRC_GUID_MISMATCH
  { ERROR_DS_SRC_GUID_MISMATCH, &(O(object_Kerror_ds_src_guid_mismatch)) },
 #endif
 #ifdef ERROR_DS_CANT_MOVE_DELETED_OBJECT
  { ERROR_DS_CANT_MOVE_DELETED_OBJECT, &(O(object_Kerror_ds_cant_move_deleted_object)) },
 #endif
 #ifdef ERROR_DS_PDC_OPERATION_IN_PROGRESS
  { ERROR_DS_PDC_OPERATION_IN_PROGRESS, &(O(object_Kerror_ds_pdc_operation_in_progress)) },
 #endif
 #ifdef ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD
  { ERROR_DS_CROSS_DOMAIN_CLEANUP_REQD, &(O(object_Kerror_ds_cross_domain_cleanup_reqd)) },
 #endif
 #ifdef ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION
  { ERROR_DS_ILLEGAL_XDOM_MOVE_OPERATION, &(O(object_Kerror_ds_illegal_xdom_move_operation)) },
 #endif
 #ifdef ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS
  { ERROR_DS_CANT_WITH_ACCT_GROUP_MEMBERSHPS, &(O(object_Kerror_ds_cant_with_acct_group_membershps)) },
 #endif
 #ifdef ERROR_DS_NC_MUST_HAVE_NC_PARENT
  { ERROR_DS_NC_MUST_HAVE_NC_PARENT, &(O(object_Kerror_ds_nc_must_have_nc_parent)) },
 #endif
 #ifdef ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE
  { ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE, &(O(object_Kerror_ds_cr_impossible_to_validate)) },
 #endif
 #ifdef ERROR_DS_DST_DOMAIN_NOT_NATIVE
  { ERROR_DS_DST_DOMAIN_NOT_NATIVE, &(O(object_Kerror_ds_dst_domain_not_native)) },
 #endif
 #ifdef ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER
  { ERROR_DS_MISSING_INFRASTRUCTURE_CONTAINER, &(O(object_Kerror_ds_missing_infrastructure_container)) },
 #endif
 #ifdef ERROR_DS_CANT_MOVE_ACCOUNT_GROUP
  { ERROR_DS_CANT_MOVE_ACCOUNT_GROUP, &(O(object_Kerror_ds_cant_move_account_group)) },
 #endif
 #ifdef ERROR_DS_CANT_MOVE_RESOURCE_GROUP
  { ERROR_DS_CANT_MOVE_RESOURCE_GROUP, &(O(object_Kerror_ds_cant_move_resource_group)) },
 #endif
 #ifdef ERROR_DS_INVALID_SEARCH_FLAG
  { ERROR_DS_INVALID_SEARCH_FLAG, &(O(object_Kerror_ds_invalid_search_flag)) },
 #endif
 #ifdef ERROR_DS_NO_TREE_DELETE_ABOVE_NC
  { ERROR_DS_NO_TREE_DELETE_ABOVE_NC, &(O(object_Kerror_ds_no_tree_delete_above_nc)) },
 #endif
 #ifdef ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE
  { ERROR_DS_COULDNT_LOCK_TREE_FOR_DELETE, &(O(object_Kerror_ds_couldnt_lock_tree_for_delete)) },
 #endif
 #ifdef ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE
  { ERROR_DS_COULDNT_IDENTIFY_OBJECTS_FOR_TREE_DELETE, &(O(object_Kerror_ds_couldnt_identify_objects_for_tree_delete)) },
 #endif
 #ifdef ERROR_DS_SAM_INIT_FAILURE
  { ERROR_DS_SAM_INIT_FAILURE, &(O(object_Kerror_ds_sam_init_failure)) },
 #endif
 #ifdef ERROR_DS_SENSITIVE_GROUP_VIOLATION
  { ERROR_DS_SENSITIVE_GROUP_VIOLATION, &(O(object_Kerror_ds_sensitive_group_violation)) },
 #endif
 #ifdef ERROR_DS_CANT_MOD_PRIMARYGROUPID
  { ERROR_DS_CANT_MOD_PRIMARYGROUPID, &(O(object_Kerror_ds_cant_mod_primarygroupid)) },
 #endif
 #ifdef ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD
  { ERROR_DS_ILLEGAL_BASE_SCHEMA_MOD, &(O(object_Kerror_ds_illegal_base_schema_mod)) },
 #endif
 #ifdef ERROR_DS_NONSAFE_SCHEMA_CHANGE
  { ERROR_DS_NONSAFE_SCHEMA_CHANGE, &(O(object_Kerror_ds_nonsafe_schema_change)) },
 #endif
 #ifdef ERROR_DS_SCHEMA_UPDATE_DISALLOWED
  { ERROR_DS_SCHEMA_UPDATE_DISALLOWED, &(O(object_Kerror_ds_schema_update_disallowed)) },
 #endif
 #ifdef ERROR_DS_CANT_CREATE_UNDER_SCHEMA
  { ERROR_DS_CANT_CREATE_UNDER_SCHEMA, &(O(object_Kerror_ds_cant_create_under_schema)) },
 #endif
 #ifdef ERROR_DS_INSTALL_NO_SRC_SCH_VERSION
  { ERROR_DS_INSTALL_NO_SRC_SCH_VERSION, &(O(object_Kerror_ds_install_no_src_sch_version)) },
 #endif
 #ifdef ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE
  { ERROR_DS_INSTALL_NO_SCH_VERSION_IN_INIFILE, &(O(object_Kerror_ds_install_no_sch_version_in_inifile)) },
 #endif
 #ifdef ERROR_DS_INVALID_GROUP_TYPE
  { ERROR_DS_INVALID_GROUP_TYPE, &(O(object_Kerror_ds_invalid_group_type)) },
 #endif
 #ifdef ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN
  { ERROR_DS_NO_NEST_GLOBALGROUP_IN_MIXEDDOMAIN, &(O(object_Kerror_ds_no_nest_globalgroup_in_mixeddomain)) },
 #endif
 #ifdef ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN
  { ERROR_DS_NO_NEST_LOCALGROUP_IN_MIXEDDOMAIN, &(O(object_Kerror_ds_no_nest_localgroup_in_mixeddomain)) },
 #endif
 #ifdef ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER
  { ERROR_DS_GLOBAL_CANT_HAVE_LOCAL_MEMBER, &(O(object_Kerror_ds_global_cant_have_local_member)) },
 #endif
 #ifdef ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER
  { ERROR_DS_GLOBAL_CANT_HAVE_UNIVERSAL_MEMBER, &(O(object_Kerror_ds_global_cant_have_universal_member)) },
 #endif
 #ifdef ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER
  { ERROR_DS_UNIVERSAL_CANT_HAVE_LOCAL_MEMBER, &(O(object_Kerror_ds_universal_cant_have_local_member)) },
 #endif
 #ifdef ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER
  { ERROR_DS_GLOBAL_CANT_HAVE_CROSSDOMAIN_MEMBER, &(O(object_Kerror_ds_global_cant_have_crossdomain_member)) },
 #endif
 #ifdef ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER
  { ERROR_DS_LOCAL_CANT_HAVE_CROSSDOMAIN_LOCAL_MEMBER, &(O(object_Kerror_ds_local_cant_have_crossdomain_local_member)) },
 #endif
 #ifdef ERROR_DS_HAVE_PRIMARY_MEMBERS
  { ERROR_DS_HAVE_PRIMARY_MEMBERS, &(O(object_Kerror_ds_have_primary_members)) },
 #endif
 #ifdef ERROR_DS_STRING_SD_CONVERSION_FAILED
  { ERROR_DS_STRING_SD_CONVERSION_FAILED, &(O(object_Kerror_ds_string_sd_conversion_failed)) },
 #endif
 #ifdef ERROR_DS_NAMING_MASTER_GC
  { ERROR_DS_NAMING_MASTER_GC, &(O(object_Kerror_ds_naming_master_gc)) },
 #endif
 #ifdef ERROR_DS_LOOKUP_FAILURE
  { ERROR_DS_LOOKUP_FAILURE, &(O(object_Kerror_ds_lookup_failure)) },
 #endif
 #ifdef ERROR_DS_COULDNT_UPDATE_SPNS
  { ERROR_DS_COULDNT_UPDATE_SPNS, &(O(object_Kerror_ds_couldnt_update_spns)) },
 #endif
 #ifdef ERROR_DS_CANT_RETRIEVE_SD
  { ERROR_DS_CANT_RETRIEVE_SD, &(O(object_Kerror_ds_cant_retrieve_sd)) },
 #endif
 #ifdef ERROR_DS_KEY_NOT_UNIQUE
  { ERROR_DS_KEY_NOT_UNIQUE, &(O(object_Kerror_ds_key_not_unique)) },
 #endif
 #ifdef ERROR_DS_WRONG_LINKED_ATT_SYNTAX
  { ERROR_DS_WRONG_LINKED_ATT_SYNTAX, &(O(object_Kerror_ds_wrong_linked_att_syntax)) },
 #endif
 #ifdef ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD
  { ERROR_DS_SAM_NEED_BOOTKEY_PASSWORD, &(O(object_Kerror_ds_sam_need_bootkey_password)) },
 #endif
 #ifdef ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY
  { ERROR_DS_SAM_NEED_BOOTKEY_FLOPPY, &(O(object_Kerror_ds_sam_need_bootkey_floppy)) },
 #endif
 #ifdef ERROR_DS_CANT_START
  { ERROR_DS_CANT_START, &(O(object_Kerror_ds_cant_start)) },
 #endif
 #ifdef ERROR_DS_INIT_FAILURE
  { ERROR_DS_INIT_FAILURE, &(O(object_Kerror_ds_init_failure)) },
 #endif
 #ifdef ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION
  { ERROR_DS_NO_PKT_PRIVACY_ON_CONNECTION, &(O(object_Kerror_ds_no_pkt_privacy_on_connection)) },
 #endif
 #ifdef ERROR_DS_SOURCE_DOMAIN_IN_FOREST
  { ERROR_DS_SOURCE_DOMAIN_IN_FOREST, &(O(object_Kerror_ds_source_domain_in_forest)) },
 #endif
 #ifdef ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST
  { ERROR_DS_DESTINATION_DOMAIN_NOT_IN_FOREST, &(O(object_Kerror_ds_destination_domain_not_in_forest)) },
 #endif
 #ifdef ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED
  { ERROR_DS_DESTINATION_AUDITING_NOT_ENABLED, &(O(object_Kerror_ds_destination_auditing_not_enabled)) },
 #endif
 #ifdef ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN
  { ERROR_DS_CANT_FIND_DC_FOR_SRC_DOMAIN, &(O(object_Kerror_ds_cant_find_dc_for_src_domain)) },
 #endif
 #ifdef ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER
  { ERROR_DS_SRC_OBJ_NOT_GROUP_OR_USER, &(O(object_Kerror_ds_src_obj_not_group_or_user)) },
 #endif
 #ifdef ERROR_DS_SRC_SID_EXISTS_IN_FOREST
  { ERROR_DS_SRC_SID_EXISTS_IN_FOREST, &(O(object_Kerror_ds_src_sid_exists_in_forest)) },
 #endif
 #ifdef ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH
  { ERROR_DS_SRC_AND_DST_OBJECT_CLASS_MISMATCH, &(O(object_Kerror_ds_src_and_dst_object_class_mismatch)) },
 #endif
 #ifdef ERROR_SAM_INIT_FAILURE
  { ERROR_SAM_INIT_FAILURE, &(O(object_Kerror_sam_init_failure)) },
 #endif
 #ifdef ERROR_DS_DRA_SCHEMA_INFO_SHIP
  { ERROR_DS_DRA_SCHEMA_INFO_SHIP, &(O(object_Kerror_ds_dra_schema_info_ship)) },
 #endif
 #ifdef ERROR_DS_DRA_SCHEMA_CONFLICT
  { ERROR_DS_DRA_SCHEMA_CONFLICT, &(O(object_Kerror_ds_dra_schema_conflict)) },
 #endif
 #ifdef ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT
  { ERROR_DS_DRA_EARLIER_SCHEMA_CONLICT, &(O(object_Kerror_ds_dra_earlier_schema_conlict)) },
 #endif
 #ifdef ERROR_DS_DRA_OBJ_NC_MISMATCH
  { ERROR_DS_DRA_OBJ_NC_MISMATCH, &(O(object_Kerror_ds_dra_obj_nc_mismatch)) },
 #endif
 #ifdef ERROR_DS_NC_STILL_HAS_DSAS
  { ERROR_DS_NC_STILL_HAS_DSAS, &(O(object_Kerror_ds_nc_still_has_dsas)) },
 #endif
 #ifdef ERROR_DS_GC_REQUIRED
  { ERROR_DS_GC_REQUIRED, &(O(object_Kerror_ds_gc_required)) },
 #endif
 #ifdef ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY
  { ERROR_DS_LOCAL_MEMBER_OF_LOCAL_ONLY, &(O(object_Kerror_ds_local_member_of_local_only)) },
 #endif
 #ifdef ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS
  { ERROR_DS_NO_FPO_IN_UNIVERSAL_GROUPS, &(O(object_Kerror_ds_no_fpo_in_universal_groups)) },
 #endif
 #ifdef ERROR_DS_CANT_ADD_TO_GC
  { ERROR_DS_CANT_ADD_TO_GC, &(O(object_Kerror_ds_cant_add_to_gc)) },
 #endif
 #ifdef ERROR_DS_NO_CHECKPOINT_WITH_PDC
  { ERROR_DS_NO_CHECKPOINT_WITH_PDC, &(O(object_Kerror_ds_no_checkpoint_with_pdc)) },
 #endif
 #ifdef ERROR_DS_SOURCE_AUDITING_NOT_ENABLED
  { ERROR_DS_SOURCE_AUDITING_NOT_ENABLED, &(O(object_Kerror_ds_source_auditing_not_enabled)) },
 #endif
 #ifdef ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC
  { ERROR_DS_CANT_CREATE_IN_NONDOMAIN_NC, &(O(object_Kerror_ds_cant_create_in_nondomain_nc)) },
 #endif
 #ifdef ERROR_DS_INVALID_NAME_FOR_SPN
  { ERROR_DS_INVALID_NAME_FOR_SPN, &(O(object_Kerror_ds_invalid_name_for_spn)) },
 #endif
 #ifdef ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS
  { ERROR_DS_FILTER_USES_CONTRUCTED_ATTRS, &(O(object_Kerror_ds_filter_uses_contructed_attrs)) },
 #endif
 #ifdef ERROR_DS_UNICODEPWD_NOT_IN_QUOTES
  { ERROR_DS_UNICODEPWD_NOT_IN_QUOTES, &(O(object_Kerror_ds_unicodepwd_not_in_quotes)) },
 #endif
 #ifdef ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED
  { ERROR_DS_MACHINE_ACCOUNT_QUOTA_EXCEEDED, &(O(object_Kerror_ds_machine_account_quota_exceeded)) },
 #endif
 #ifdef ERROR_DS_MUST_BE_RUN_ON_DST_DC
  { ERROR_DS_MUST_BE_RUN_ON_DST_DC, &(O(object_Kerror_ds_must_be_run_on_dst_dc)) },
 #endif
 #ifdef ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER
  { ERROR_DS_SRC_DC_MUST_BE_SP4_OR_GREATER, &(O(object_Kerror_ds_src_dc_must_be_sp4_or_greater)) },
 #endif
 #ifdef ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ
  { ERROR_DS_CANT_TREE_DELETE_CRITICAL_OBJ, &(O(object_Kerror_ds_cant_tree_delete_critical_obj)) },
 #endif
 #ifdef ERROR_DS_INIT_FAILURE_CONSOLE
  { ERROR_DS_INIT_FAILURE_CONSOLE, &(O(object_Kerror_ds_init_failure_console)) },
 #endif
 #ifdef ERROR_DS_SAM_INIT_FAILURE_CONSOLE
  { ERROR_DS_SAM_INIT_FAILURE_CONSOLE, &(O(object_Kerror_ds_sam_init_failure_console)) },
 #endif
 #ifdef ERROR_DS_FOREST_VERSION_TOO_HIGH
  { ERROR_DS_FOREST_VERSION_TOO_HIGH, &(O(object_Kerror_ds_forest_version_too_high)) },
 #endif
 #ifdef ERROR_DS_DOMAIN_VERSION_TOO_HIGH
  { ERROR_DS_DOMAIN_VERSION_TOO_HIGH, &(O(object_Kerror_ds_domain_version_too_high)) },
 #endif
 #ifdef ERROR_DS_FOREST_VERSION_TOO_LOW
  { ERROR_DS_FOREST_VERSION_TOO_LOW, &(O(object_Kerror_ds_forest_version_too_low)) },
 #endif
 #ifdef ERROR_DS_DOMAIN_VERSION_TOO_LOW
  { ERROR_DS_DOMAIN_VERSION_TOO_LOW, &(O(object_Kerror_ds_domain_version_too_low)) },
 #endif
 #ifdef ERROR_DS_INCOMPATIBLE_VERSION
  { ERROR_DS_INCOMPATIBLE_VERSION, &(O(object_Kerror_ds_incompatible_version)) },
 #endif
 #ifdef ERROR_DS_LOW_DSA_VERSION
  { ERROR_DS_LOW_DSA_VERSION, &(O(object_Kerror_ds_low_dsa_version)) },
 #endif
 #ifdef ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN
  { ERROR_DS_NO_BEHAVIOR_VERSION_IN_MIXEDDOMAIN, &(O(object_Kerror_ds_no_behavior_version_in_mixeddomain)) },
 #endif
 #ifdef ERROR_DS_NOT_SUPPORTED_SORT_ORDER
  { ERROR_DS_NOT_SUPPORTED_SORT_ORDER, &(O(object_Kerror_ds_not_supported_sort_order)) },
 #endif
 #ifdef ERROR_DS_NAME_NOT_UNIQUE
  { ERROR_DS_NAME_NOT_UNIQUE, &(O(object_Kerror_ds_name_not_unique)) },
 #endif
 #ifdef ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4
  { ERROR_DS_MACHINE_ACCOUNT_CREATED_PRENT4, &(O(object_Kerror_ds_machine_account_created_prent4)) },
 #endif
 #ifdef ERROR_DS_OUT_OF_VERSION_STORE
  { ERROR_DS_OUT_OF_VERSION_STORE, &(O(object_Kerror_ds_out_of_version_store)) },
 #endif
 #ifdef ERROR_DS_INCOMPATIBLE_CONTROLS_USED
  { ERROR_DS_INCOMPATIBLE_CONTROLS_USED, &(O(object_Kerror_ds_incompatible_controls_used)) },
 #endif
 #ifdef ERROR_DS_NO_REF_DOMAIN
  { ERROR_DS_NO_REF_DOMAIN, &(O(object_Kerror_ds_no_ref_domain)) },
 #endif
 #ifdef ERROR_DS_RESERVED_LINK_ID
  { ERROR_DS_RESERVED_LINK_ID, &(O(object_Kerror_ds_reserved_link_id)) },
 #endif
 #ifdef ERROR_DS_LINK_ID_NOT_AVAILABLE
  { ERROR_DS_LINK_ID_NOT_AVAILABLE, &(O(object_Kerror_ds_link_id_not_available)) },
 #endif
 #ifdef ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER
  { ERROR_DS_AG_CANT_HAVE_UNIVERSAL_MEMBER, &(O(object_Kerror_ds_ag_cant_have_universal_member)) },
 #endif
 #ifdef ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE
  { ERROR_DS_MODIFYDN_DISALLOWED_BY_INSTANCE_TYPE, &(O(object_Kerror_ds_modifydn_disallowed_by_instance_type)) },
 #endif
 #ifdef ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC
  { ERROR_DS_NO_OBJECT_MOVE_IN_SCHEMA_NC, &(O(object_Kerror_ds_no_object_move_in_schema_nc)) },
 #endif
 #ifdef ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG
  { ERROR_DS_MODIFYDN_DISALLOWED_BY_FLAG, &(O(object_Kerror_ds_modifydn_disallowed_by_flag)) },
 #endif
 #ifdef ERROR_DS_MODIFYDN_WRONG_GRANDPARENT
  { ERROR_DS_MODIFYDN_WRONG_GRANDPARENT, &(O(object_Kerror_ds_modifydn_wrong_grandparent)) },
 #endif
 #ifdef ERROR_DS_NAME_ERROR_TRUST_REFERRAL
  { ERROR_DS_NAME_ERROR_TRUST_REFERRAL, &(O(object_Kerror_ds_name_error_trust_referral)) },
 #endif
 #ifdef ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER
  { ERROR_NOT_SUPPORTED_ON_STANDARD_SERVER, &(O(object_Kerror_not_supported_on_standard_server)) },
 #endif
 #ifdef ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD
  { ERROR_DS_CANT_ACCESS_REMOTE_PART_OF_AD, &(O(object_Kerror_ds_cant_access_remote_part_of_ad)) },
 #endif
 #ifdef ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2
  { ERROR_DS_CR_IMPOSSIBLE_TO_VALIDATE_V2, &(O(object_Kerror_ds_cr_impossible_to_validate_v2)) },
 #endif
 #ifdef ERROR_DS_THREAD_LIMIT_EXCEEDED
  { ERROR_DS_THREAD_LIMIT_EXCEEDED, &(O(object_Kerror_ds_thread_limit_exceeded)) },
 #endif
 #ifdef ERROR_DS_NOT_CLOSEST
  { ERROR_DS_NOT_CLOSEST, &(O(object_Kerror_ds_not_closest)) },
 #endif
 #ifdef ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF
  { ERROR_DS_CANT_DERIVE_SPN_WITHOUT_SERVER_REF, &(O(object_Kerror_ds_cant_derive_spn_without_server_ref)) },
 #endif
 #ifdef ERROR_DS_SINGLE_USER_MODE_FAILED
  { ERROR_DS_SINGLE_USER_MODE_FAILED, &(O(object_Kerror_ds_single_user_mode_failed)) },
 #endif
 #ifdef ERROR_DS_NTDSCRIPT_SYNTAX_ERROR
  { ERROR_DS_NTDSCRIPT_SYNTAX_ERROR, &(O(object_Kerror_ds_ntdscript_syntax_error)) },
 #endif
 #ifdef ERROR_DS_NTDSCRIPT_PROCESS_ERROR
  { ERROR_DS_NTDSCRIPT_PROCESS_ERROR, &(O(object_Kerror_ds_ntdscript_process_error)) },
 #endif
 #ifdef ERROR_DS_DIFFERENT_REPL_EPOCHS
  { ERROR_DS_DIFFERENT_REPL_EPOCHS, &(O(object_Kerror_ds_different_repl_epochs)) },
 #endif
 #ifdef ERROR_DS_DRS_EXTENSIONS_CHANGED
  { ERROR_DS_DRS_EXTENSIONS_CHANGED, &(O(object_Kerror_ds_drs_extensions_changed)) },
 #endif
 #ifdef ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR
  { ERROR_DS_REPLICA_SET_CHANGE_NOT_ALLOWED_ON_DISABLED_CR, &(O(object_Kerror_ds_replica_set_change_not_allowed_on_disabled_cr)) },
 #endif
 #ifdef ERROR_DS_NO_MSDS_INTID
  { ERROR_DS_NO_MSDS_INTID, &(O(object_Kerror_ds_no_msds_intid)) },
 #endif
 #ifdef ERROR_DS_DUP_MSDS_INTID
  { ERROR_DS_DUP_MSDS_INTID, &(O(object_Kerror_ds_dup_msds_intid)) },
 #endif
 #ifdef ERROR_DS_EXISTS_IN_RDNATTID
  { ERROR_DS_EXISTS_IN_RDNATTID, &(O(object_Kerror_ds_exists_in_rdnattid)) },
 #endif
 #ifdef ERROR_DS_AUTHORIZATION_FAILED
  { ERROR_DS_AUTHORIZATION_FAILED, &(O(object_Kerror_ds_authorization_failed)) },
 #endif
 #ifdef ERROR_DS_INVALID_SCRIPT
  { ERROR_DS_INVALID_SCRIPT, &(O(object_Kerror_ds_invalid_script)) },
 #endif
 #ifdef ERROR_DS_REMOTE_CROSSREF_OP_FAILED
  { ERROR_DS_REMOTE_CROSSREF_OP_FAILED, &(O(object_Kerror_ds_remote_crossref_op_failed)) },
 #endif
 #ifdef ERROR_DS_CROSS_REF_BUSY
  { ERROR_DS_CROSS_REF_BUSY, &(O(object_Kerror_ds_cross_ref_busy)) },
 #endif
 #ifdef ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN
  { ERROR_DS_CANT_DERIVE_SPN_FOR_DELETED_DOMAIN, &(O(object_Kerror_ds_cant_derive_spn_for_deleted_domain)) },
 #endif
 #ifdef ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC
  { ERROR_DS_CANT_DEMOTE_WITH_WRITEABLE_NC, &(O(object_Kerror_ds_cant_demote_with_writeable_nc)) },
 #endif
 #ifdef ERROR_DS_DUPLICATE_ID_FOUND
  { ERROR_DS_DUPLICATE_ID_FOUND, &(O(object_Kerror_ds_duplicate_id_found)) },
 #endif
 #ifdef ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT
  { ERROR_DS_INSUFFICIENT_ATTR_TO_CREATE_OBJECT, &(O(object_Kerror_ds_insufficient_attr_to_create_object)) },
 #endif
 #ifdef ERROR_DS_GROUP_CONVERSION_ERROR
  { ERROR_DS_GROUP_CONVERSION_ERROR, &(O(object_Kerror_ds_group_conversion_error)) },
 #endif
 #ifdef ERROR_DS_CANT_MOVE_APP_BASIC_GROUP
  { ERROR_DS_CANT_MOVE_APP_BASIC_GROUP, &(O(object_Kerror_ds_cant_move_app_basic_group)) },
 #endif
 #ifdef ERROR_DS_CANT_MOVE_APP_QUERY_GROUP
  { ERROR_DS_CANT_MOVE_APP_QUERY_GROUP, &(O(object_Kerror_ds_cant_move_app_query_group)) },
 #endif
 #ifdef ERROR_DS_ROLE_NOT_VERIFIED
  { ERROR_DS_ROLE_NOT_VERIFIED, &(O(object_Kerror_ds_role_not_verified)) },
 #endif
 #ifdef ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL
  { ERROR_DS_WKO_CONTAINER_CANNOT_BE_SPECIAL, &(O(object_Kerror_ds_wko_container_cannot_be_special)) },
 #endif
 #ifdef ERROR_DS_DOMAIN_RENAME_IN_PROGRESS
  { ERROR_DS_DOMAIN_RENAME_IN_PROGRESS, &(O(object_Kerror_ds_domain_rename_in_progress)) },
 #endif
 #ifdef ERROR_DS_EXISTING_AD_CHILD_NC
  { ERROR_DS_EXISTING_AD_CHILD_NC, &(O(object_Kerror_ds_existing_ad_child_nc)) },
 #endif
 #ifdef DNS_ERROR_RCODE_FORMAT_ERROR
  { DNS_ERROR_RCODE_FORMAT_ERROR, &(O(object_Kdns_error_rcode_format_error)) },
 #endif
 #ifdef DNS_ERROR_RCODE_SERVER_FAILURE
  { DNS_ERROR_RCODE_SERVER_FAILURE, &(O(object_Kdns_error_rcode_server_failure)) },
 #endif
 #ifdef DNS_ERROR_RCODE_NAME_ERROR
  { DNS_ERROR_RCODE_NAME_ERROR, &(O(object_Kdns_error_rcode_name_error)) },
 #endif
 #ifdef DNS_ERROR_RCODE_NOT_IMPLEMENTED
  { DNS_ERROR_RCODE_NOT_IMPLEMENTED, &(O(object_Kdns_error_rcode_not_implemented)) },
 #endif
 #ifdef DNS_ERROR_RCODE_REFUSED
  { DNS_ERROR_RCODE_REFUSED, &(O(object_Kdns_error_rcode_refused)) },
 #endif
 #ifdef DNS_ERROR_RCODE_YXDOMAIN
  { DNS_ERROR_RCODE_YXDOMAIN, &(O(object_Kdns_error_rcode_yxdomain)) },
 #endif
 #ifdef DNS_ERROR_RCODE_YXRRSET
  { DNS_ERROR_RCODE_YXRRSET, &(O(object_Kdns_error_rcode_yxrrset)) },
 #endif
 #ifdef DNS_ERROR_RCODE_NXRRSET
  { DNS_ERROR_RCODE_NXRRSET, &(O(object_Kdns_error_rcode_nxrrset)) },
 #endif
 #ifdef DNS_ERROR_RCODE_NOTAUTH
  { DNS_ERROR_RCODE_NOTAUTH, &(O(object_Kdns_error_rcode_notauth)) },
 #endif
 #ifdef DNS_ERROR_RCODE_NOTZONE
  { DNS_ERROR_RCODE_NOTZONE, &(O(object_Kdns_error_rcode_notzone)) },
 #endif
 #ifdef DNS_ERROR_RCODE_BADSIG
  { DNS_ERROR_RCODE_BADSIG, &(O(object_Kdns_error_rcode_badsig)) },
 #endif
 #ifdef DNS_ERROR_RCODE_BADKEY
  { DNS_ERROR_RCODE_BADKEY, &(O(object_Kdns_error_rcode_badkey)) },
 #endif
 #ifdef DNS_ERROR_RCODE_BADTIME
  { DNS_ERROR_RCODE_BADTIME, &(O(object_Kdns_error_rcode_badtime)) },
 #endif
 #ifdef DNS_INFO_NO_RECORDS
  { DNS_INFO_NO_RECORDS, &(O(object_Kdns_info_no_records)) },
 #endif
 #ifdef DNS_ERROR_BAD_PACKET
  { DNS_ERROR_BAD_PACKET, &(O(object_Kdns_error_bad_packet)) },
 #endif
 #ifdef DNS_ERROR_NO_PACKET
  { DNS_ERROR_NO_PACKET, &(O(object_Kdns_error_no_packet)) },
 #endif
 #ifdef DNS_ERROR_RCODE
  { DNS_ERROR_RCODE, &(O(object_Kdns_error_rcode)) },
 #endif
 #ifdef DNS_ERROR_UNSECURE_PACKET
  { DNS_ERROR_UNSECURE_PACKET, &(O(object_Kdns_error_unsecure_packet)) },
 #endif
 #ifdef DNS_ERROR_INVALID_TYPE
  { DNS_ERROR_INVALID_TYPE, &(O(object_Kdns_error_invalid_type)) },
 #endif
 #ifdef DNS_ERROR_INVALID_IP_ADDRESS
  { DNS_ERROR_INVALID_IP_ADDRESS, &(O(object_Kdns_error_invalid_ip_address)) },
 #endif
 #ifdef DNS_ERROR_INVALID_PROPERTY
  { DNS_ERROR_INVALID_PROPERTY, &(O(object_Kdns_error_invalid_property)) },
 #endif
 #ifdef DNS_ERROR_TRY_AGAIN_LATER
  { DNS_ERROR_TRY_AGAIN_LATER, &(O(object_Kdns_error_try_again_later)) },
 #endif
 #ifdef DNS_ERROR_NOT_UNIQUE
  { DNS_ERROR_NOT_UNIQUE, &(O(object_Kdns_error_not_unique)) },
 #endif
 #ifdef DNS_ERROR_NON_RFC_NAME
  { DNS_ERROR_NON_RFC_NAME, &(O(object_Kdns_error_non_rfc_name)) },
 #endif
 #ifdef DNS_STATUS_FQDN
  { DNS_STATUS_FQDN, &(O(object_Kdns_status_fqdn)) },
 #endif
 #ifdef DNS_STATUS_DOTTED_NAME
  { DNS_STATUS_DOTTED_NAME, &(O(object_Kdns_status_dotted_name)) },
 #endif
 #ifdef DNS_STATUS_SINGLE_PART_NAME
  { DNS_STATUS_SINGLE_PART_NAME, &(O(object_Kdns_status_single_part_name)) },
 #endif
 #ifdef DNS_ERROR_INVALID_NAME_CHAR
  { DNS_ERROR_INVALID_NAME_CHAR, &(O(object_Kdns_error_invalid_name_char)) },
 #endif
 #ifdef DNS_ERROR_NUMERIC_NAME
  { DNS_ERROR_NUMERIC_NAME, &(O(object_Kdns_error_numeric_name)) },
 #endif
 #ifdef DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER
  { DNS_ERROR_NOT_ALLOWED_ON_ROOT_SERVER, &(O(object_Kdns_error_not_allowed_on_root_server)) },
 #endif
 #ifdef DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION
  { DNS_ERROR_NOT_ALLOWED_UNDER_DELEGATION, &(O(object_Kdns_error_not_allowed_under_delegation)) },
 #endif
 #ifdef DNS_ERROR_CANNOT_FIND_ROOT_HINTS
  { DNS_ERROR_CANNOT_FIND_ROOT_HINTS, &(O(object_Kdns_error_cannot_find_root_hints)) },
 #endif
 #ifdef DNS_ERROR_INCONSISTENT_ROOT_HINTS
  { DNS_ERROR_INCONSISTENT_ROOT_HINTS, &(O(object_Kdns_error_inconsistent_root_hints)) },
 #endif
 #ifdef DNS_ERROR_ZONE_DOES_NOT_EXIST
  { DNS_ERROR_ZONE_DOES_NOT_EXIST, &(O(object_Kdns_error_zone_does_not_exist)) },
 #endif
 #ifdef DNS_ERROR_NO_ZONE_INFO
  { DNS_ERROR_NO_ZONE_INFO, &(O(object_Kdns_error_no_zone_info)) },
 #endif
 #ifdef DNS_ERROR_INVALID_ZONE_OPERATION
  { DNS_ERROR_INVALID_ZONE_OPERATION, &(O(object_Kdns_error_invalid_zone_operation)) },
 #endif
 #ifdef DNS_ERROR_ZONE_CONFIGURATION_ERROR
  { DNS_ERROR_ZONE_CONFIGURATION_ERROR, &(O(object_Kdns_error_zone_configuration_error)) },
 #endif
 #ifdef DNS_ERROR_ZONE_HAS_NO_SOA_RECORD
  { DNS_ERROR_ZONE_HAS_NO_SOA_RECORD, &(O(object_Kdns_error_zone_has_no_soa_record)) },
 #endif
 #ifdef DNS_ERROR_ZONE_HAS_NO_NS_RECORDS
  { DNS_ERROR_ZONE_HAS_NO_NS_RECORDS, &(O(object_Kdns_error_zone_has_no_ns_records)) },
 #endif
 #ifdef DNS_ERROR_ZONE_LOCKED
  { DNS_ERROR_ZONE_LOCKED, &(O(object_Kdns_error_zone_locked)) },
 #endif
 #ifdef DNS_ERROR_ZONE_CREATION_FAILED
  { DNS_ERROR_ZONE_CREATION_FAILED, &(O(object_Kdns_error_zone_creation_failed)) },
 #endif
 #ifdef DNS_ERROR_ZONE_ALREADY_EXISTS
  { DNS_ERROR_ZONE_ALREADY_EXISTS, &(O(object_Kdns_error_zone_already_exists)) },
 #endif
 #ifdef DNS_ERROR_AUTOZONE_ALREADY_EXISTS
  { DNS_ERROR_AUTOZONE_ALREADY_EXISTS, &(O(object_Kdns_error_autozone_already_exists)) },
 #endif
 #ifdef DNS_ERROR_INVALID_ZONE_TYPE
  { DNS_ERROR_INVALID_ZONE_TYPE, &(O(object_Kdns_error_invalid_zone_type)) },
 #endif
 #ifdef DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP
  { DNS_ERROR_SECONDARY_REQUIRES_MASTER_IP, &(O(object_Kdns_error_secondary_requires_master_ip)) },
 #endif
 #ifdef DNS_ERROR_ZONE_NOT_SECONDARY
  { DNS_ERROR_ZONE_NOT_SECONDARY, &(O(object_Kdns_error_zone_not_secondary)) },
 #endif
 #ifdef DNS_ERROR_NEED_SECONDARY_ADDRESSES
  { DNS_ERROR_NEED_SECONDARY_ADDRESSES, &(O(object_Kdns_error_need_secondary_addresses)) },
 #endif
 #ifdef DNS_ERROR_WINS_INIT_FAILED
  { DNS_ERROR_WINS_INIT_FAILED, &(O(object_Kdns_error_wins_init_failed)) },
 #endif
 #ifdef DNS_ERROR_NEED_WINS_SERVERS
  { DNS_ERROR_NEED_WINS_SERVERS, &(O(object_Kdns_error_need_wins_servers)) },
 #endif
 #ifdef DNS_ERROR_NBSTAT_INIT_FAILED
  { DNS_ERROR_NBSTAT_INIT_FAILED, &(O(object_Kdns_error_nbstat_init_failed)) },
 #endif
 #ifdef DNS_ERROR_SOA_DELETE_INVALID
  { DNS_ERROR_SOA_DELETE_INVALID, &(O(object_Kdns_error_soa_delete_invalid)) },
 #endif
 #ifdef DNS_ERROR_FORWARDER_ALREADY_EXISTS
  { DNS_ERROR_FORWARDER_ALREADY_EXISTS, &(O(object_Kdns_error_forwarder_already_exists)) },
 #endif
 #ifdef DNS_ERROR_ZONE_REQUIRES_MASTER_IP
  { DNS_ERROR_ZONE_REQUIRES_MASTER_IP, &(O(object_Kdns_error_zone_requires_master_ip)) },
 #endif
 #ifdef DNS_ERROR_ZONE_IS_SHUTDOWN
  { DNS_ERROR_ZONE_IS_SHUTDOWN, &(O(object_Kdns_error_zone_is_shutdown)) },
 #endif
 #ifdef DNS_ERROR_PRIMARY_REQUIRES_DATAFILE
  { DNS_ERROR_PRIMARY_REQUIRES_DATAFILE, &(O(object_Kdns_error_primary_requires_datafile)) },
 #endif
 #ifdef DNS_ERROR_INVALID_DATAFILE_NAME
  { DNS_ERROR_INVALID_DATAFILE_NAME, &(O(object_Kdns_error_invalid_datafile_name)) },
 #endif
 #ifdef DNS_ERROR_DATAFILE_OPEN_FAILURE
  { DNS_ERROR_DATAFILE_OPEN_FAILURE, &(O(object_Kdns_error_datafile_open_failure)) },
 #endif
 #ifdef DNS_ERROR_FILE_WRITEBACK_FAILED
  { DNS_ERROR_FILE_WRITEBACK_FAILED, &(O(object_Kdns_error_file_writeback_failed)) },
 #endif
 #ifdef DNS_ERROR_DATAFILE_PARSING
  { DNS_ERROR_DATAFILE_PARSING, &(O(object_Kdns_error_datafile_parsing)) },
 #endif
 #ifdef DNS_ERROR_RECORD_DOES_NOT_EXIST
  { DNS_ERROR_RECORD_DOES_NOT_EXIST, &(O(object_Kdns_error_record_does_not_exist)) },
 #endif
 #ifdef DNS_ERROR_RECORD_FORMAT
  { DNS_ERROR_RECORD_FORMAT, &(O(object_Kdns_error_record_format)) },
 #endif
 #ifdef DNS_ERROR_NODE_CREATION_FAILED
  { DNS_ERROR_NODE_CREATION_FAILED, &(O(object_Kdns_error_node_creation_failed)) },
 #endif
 #ifdef DNS_ERROR_UNKNOWN_RECORD_TYPE
  { DNS_ERROR_UNKNOWN_RECORD_TYPE, &(O(object_Kdns_error_unknown_record_type)) },
 #endif
 #ifdef DNS_ERROR_RECORD_TIMED_OUT
  { DNS_ERROR_RECORD_TIMED_OUT, &(O(object_Kdns_error_record_timed_out)) },
 #endif
 #ifdef DNS_ERROR_NAME_NOT_IN_ZONE
  { DNS_ERROR_NAME_NOT_IN_ZONE, &(O(object_Kdns_error_name_not_in_zone)) },
 #endif
 #ifdef DNS_ERROR_CNAME_LOOP
  { DNS_ERROR_CNAME_LOOP, &(O(object_Kdns_error_cname_loop)) },
 #endif
 #ifdef DNS_ERROR_NODE_IS_CNAME
  { DNS_ERROR_NODE_IS_CNAME, &(O(object_Kdns_error_node_is_cname)) },
 #endif
 #ifdef DNS_ERROR_CNAME_COLLISION
  { DNS_ERROR_CNAME_COLLISION, &(O(object_Kdns_error_cname_collision)) },
 #endif
 #ifdef DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT
  { DNS_ERROR_RECORD_ONLY_AT_ZONE_ROOT, &(O(object_Kdns_error_record_only_at_zone_root)) },
 #endif
 #ifdef DNS_ERROR_RECORD_ALREADY_EXISTS
  { DNS_ERROR_RECORD_ALREADY_EXISTS, &(O(object_Kdns_error_record_already_exists)) },
 #endif
 #ifdef DNS_ERROR_SECONDARY_DATA
  { DNS_ERROR_SECONDARY_DATA, &(O(object_Kdns_error_secondary_data)) },
 #endif
 #ifdef DNS_ERROR_NO_CREATE_CACHE_DATA
  { DNS_ERROR_NO_CREATE_CACHE_DATA, &(O(object_Kdns_error_no_create_cache_data)) },
 #endif
 #ifdef DNS_ERROR_NAME_DOES_NOT_EXIST
  { DNS_ERROR_NAME_DOES_NOT_EXIST, &(O(object_Kdns_error_name_does_not_exist)) },
 #endif
 #ifdef DNS_WARNING_PTR_CREATE_FAILED
  { DNS_WARNING_PTR_CREATE_FAILED, &(O(object_Kdns_warning_ptr_create_failed)) },
 #endif
 #ifdef DNS_WARNING_DOMAIN_UNDELETED
  { DNS_WARNING_DOMAIN_UNDELETED, &(O(object_Kdns_warning_domain_undeleted)) },
 #endif
 #ifdef DNS_ERROR_DS_UNAVAILABLE
  { DNS_ERROR_DS_UNAVAILABLE, &(O(object_Kdns_error_ds_unavailable)) },
 #endif
 #ifdef DNS_ERROR_DS_ZONE_ALREADY_EXISTS
  { DNS_ERROR_DS_ZONE_ALREADY_EXISTS, &(O(object_Kdns_error_ds_zone_already_exists)) },
 #endif
 #ifdef DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE
  { DNS_ERROR_NO_BOOTFILE_IF_DS_ZONE, &(O(object_Kdns_error_no_bootfile_if_ds_zone)) },
 #endif
 #ifdef DNS_INFO_AXFR_COMPLETE
  { DNS_INFO_AXFR_COMPLETE, &(O(object_Kdns_info_axfr_complete)) },
 #endif
 #ifdef DNS_ERROR_AXFR
  { DNS_ERROR_AXFR, &(O(object_Kdns_error_axfr)) },
 #endif
 #ifdef DNS_INFO_ADDED_LOCAL_WINS
  { DNS_INFO_ADDED_LOCAL_WINS, &(O(object_Kdns_info_added_local_wins)) },
 #endif
 #ifdef DNS_STATUS_CONTINUE_NEEDED
  { DNS_STATUS_CONTINUE_NEEDED, &(O(object_Kdns_status_continue_needed)) },
 #endif
 #ifdef DNS_ERROR_NO_TCPIP
  { DNS_ERROR_NO_TCPIP, &(O(object_Kdns_error_no_tcpip)) },
 #endif
 #ifdef DNS_ERROR_NO_DNS_SERVERS
  { DNS_ERROR_NO_DNS_SERVERS, &(O(object_Kdns_error_no_dns_servers)) },
 #endif
 #ifdef DNS_ERROR_DP_DOES_NOT_EXIST
  { DNS_ERROR_DP_DOES_NOT_EXIST, &(O(object_Kdns_error_dp_does_not_exist)) },
 #endif
 #ifdef DNS_ERROR_DP_ALREADY_EXISTS
  { DNS_ERROR_DP_ALREADY_EXISTS, &(O(object_Kdns_error_dp_already_exists)) },
 #endif
 #ifdef DNS_ERROR_DP_NOT_ENLISTED
  { DNS_ERROR_DP_NOT_ENLISTED, &(O(object_Kdns_error_dp_not_enlisted)) },
 #endif
 #ifdef DNS_ERROR_DP_ALREADY_ENLISTED
  { DNS_ERROR_DP_ALREADY_ENLISTED, &(O(object_Kdns_error_dp_already_enlisted)) },
 #endif
 #ifdef DNS_ERROR_DP_NOT_AVAILABLE
  { DNS_ERROR_DP_NOT_AVAILABLE, &(O(object_Kdns_error_dp_not_available)) },
 #endif
 #ifdef WSABASEERR
  { WSABASEERR, &(O(object_Kwsabaseerr)) },
 #endif
 #ifdef WSAEINTR
  { WSAEINTR, &(O(object_Kwsaeintr)) },
 #endif
 #ifdef WSAEBADF
  { WSAEBADF, &(O(object_Kwsaebadf)) },
 #endif
 #ifdef WSAEACCES
  { WSAEACCES, &(O(object_Kwsaeacces)) },
 #endif
 #ifdef WSAEFAULT
  { WSAEFAULT, &(O(object_Kwsaefault)) },
 #endif
 #ifdef WSAEINVAL
  { WSAEINVAL, &(O(object_Kwsaeinval)) },
 #endif
 #ifdef WSAEMFILE
  { WSAEMFILE, &(O(object_Kwsaemfile)) },
 #endif
 #ifdef WSAEWOULDBLOCK
  { WSAEWOULDBLOCK, &(O(object_Kwsaewouldblock)) },
 #endif
 #ifdef WSAEINPROGRESS
  { WSAEINPROGRESS, &(O(object_Kwsaeinprogress)) },
 #endif
 #ifdef WSAEALREADY
  { WSAEALREADY, &(O(object_Kwsaealready)) },
 #endif
 #ifdef WSAENOTSOCK
  { WSAENOTSOCK, &(O(object_Kwsaenotsock)) },
 #endif
 #ifdef WSAEDESTADDRREQ
  { WSAEDESTADDRREQ, &(O(object_Kwsaedestaddrreq)) },
 #endif
 #ifdef WSAEMSGSIZE
  { WSAEMSGSIZE, &(O(object_Kwsaemsgsize)) },
 #endif
 #ifdef WSAEPROTOTYPE
  { WSAEPROTOTYPE, &(O(object_Kwsaeprototype)) },
 #endif
 #ifdef WSAENOPROTOOPT
  { WSAENOPROTOOPT, &(O(object_Kwsaenoprotoopt)) },
 #endif
 #ifdef WSAEPROTONOSUPPORT
  { WSAEPROTONOSUPPORT, &(O(object_Kwsaeprotonosupport)) },
 #endif
 #ifdef WSAESOCKTNOSUPPORT
  { WSAESOCKTNOSUPPORT, &(O(object_Kwsaesocktnosupport)) },
 #endif
 #ifdef WSAEOPNOTSUPP
  { WSAEOPNOTSUPP, &(O(object_Kwsaeopnotsupp)) },
 #endif
 #ifdef WSAEPFNOSUPPORT
  { WSAEPFNOSUPPORT, &(O(object_Kwsaepfnosupport)) },
 #endif
 #ifdef WSAEAFNOSUPPORT
  { WSAEAFNOSUPPORT, &(O(object_Kwsaeafnosupport)) },
 #endif
 #ifdef WSAEADDRINUSE
  { WSAEADDRINUSE, &(O(object_Kwsaeaddrinuse)) },
 #endif
 #ifdef WSAEADDRNOTAVAIL
  { WSAEADDRNOTAVAIL, &(O(object_Kwsaeaddrnotavail)) },
 #endif
 #ifdef WSAENETDOWN
  { WSAENETDOWN, &(O(object_Kwsaenetdown)) },
 #endif
 #ifdef WSAENETUNREACH
  { WSAENETUNREACH, &(O(object_Kwsaenetunreach)) },
 #endif
 #ifdef WSAENETRESET
  { WSAENETRESET, &(O(object_Kwsaenetreset)) },
 #endif
 #ifdef WSAECONNABORTED
  { WSAECONNABORTED, &(O(object_Kwsaeconnaborted)) },
 #endif
 #ifdef WSAECONNRESET
  { WSAECONNRESET, &(O(object_Kwsaeconnreset)) },
 #endif
 #ifdef WSAENOBUFS
  { WSAENOBUFS, &(O(object_Kwsaenobufs)) },
 #endif
 #ifdef WSAEISCONN
  { WSAEISCONN, &(O(object_Kwsaeisconn)) },
 #endif
 #ifdef WSAENOTCONN
  { WSAENOTCONN, &(O(object_Kwsaenotconn)) },
 #endif
 #ifdef WSAESHUTDOWN
  { WSAESHUTDOWN, &(O(object_Kwsaeshutdown)) },
 #endif
 #ifdef WSAETOOMANYREFS
  { WSAETOOMANYREFS, &(O(object_Kwsaetoomanyrefs)) },
 #endif
 #ifdef WSAETIMEDOUT
  { WSAETIMEDOUT, &(O(object_Kwsaetimedout)) },
 #endif
 #ifdef WSAECONNREFUSED
  { WSAECONNREFUSED, &(O(object_Kwsaeconnrefused)) },
 #endif
 #ifdef WSAELOOP
  { WSAELOOP, &(O(object_Kwsaeloop)) },
 #endif
 #ifdef WSAENAMETOOLONG
  { WSAENAMETOOLONG, &(O(object_Kwsaenametoolong)) },
 #endif
 #ifdef WSAEHOSTDOWN
  { WSAEHOSTDOWN, &(O(object_Kwsaehostdown)) },
 #endif
 #ifdef WSAEHOSTUNREACH
  { WSAEHOSTUNREACH, &(O(object_Kwsaehostunreach)) },
 #endif
 #ifdef WSAENOTEMPTY
  { WSAENOTEMPTY, &(O(object_Kwsaenotempty)) },
 #endif
 #ifdef WSAEPROCLIM
  { WSAEPROCLIM, &(O(object_Kwsaeproclim)) },
 #endif
 #ifdef WSAEUSERS
  { WSAEUSERS, &(O(object_Kwsaeusers)) },
 #endif
 #ifdef WSAEDQUOT
  { WSAEDQUOT, &(O(object_Kwsaedquot)) },
 #endif
 #ifdef WSAESTALE
  { WSAESTALE, &(O(object_Kwsaestale)) },
 #endif
 #ifdef WSAEREMOTE
  { WSAEREMOTE, &(O(object_Kwsaeremote)) },
 #endif
 #ifdef WSASYSNOTREADY
  { WSASYSNOTREADY, &(O(object_Kwsasysnotready)) },
 #endif
 #ifdef WSAVERNOTSUPPORTED
  { WSAVERNOTSUPPORTED, &(O(object_Kwsavernotsupported)) },
 #endif
 #ifdef WSANOTINITIALISED
  { WSANOTINITIALISED, &(O(object_Kwsanotinitialised)) },
 #endif
 #ifdef WSAEDISCON
  { WSAEDISCON, &(O(object_Kwsaediscon)) },
 #endif
 #ifdef WSAENOMORE
  { WSAENOMORE, &(O(object_Kwsaenomore)) },
 #endif
 #ifdef WSAECANCELLED
  { WSAECANCELLED, &(O(object_Kwsaecancelled)) },
 #endif
 #ifdef WSAEINVALIDPROCTABLE
  { WSAEINVALIDPROCTABLE, &(O(object_Kwsaeinvalidproctable)) },
 #endif
 #ifdef WSAEINVALIDPROVIDER
  { WSAEINVALIDPROVIDER, &(O(object_Kwsaeinvalidprovider)) },
 #endif
 #ifdef WSAEPROVIDERFAILEDINIT
  { WSAEPROVIDERFAILEDINIT, &(O(object_Kwsaeproviderfailedinit)) },
 #endif
 #ifdef WSASYSCALLFAILURE
  { WSASYSCALLFAILURE, &(O(object_Kwsasyscallfailure)) },
 #endif
 #ifdef WSASERVICE_NOT_FOUND
  { WSASERVICE_NOT_FOUND, &(O(object_Kwsaservice_not_found)) },
 #endif
 #ifdef WSATYPE_NOT_FOUND
  { WSATYPE_NOT_FOUND, &(O(object_Kwsatype_not_found)) },
 #endif
 #ifdef WSA_E_NO_MORE
  { WSA_E_NO_MORE, &(O(object_Kwsa_e_no_more)) },
 #endif
 #ifdef WSA_E_CANCELLED
  { WSA_E_CANCELLED, &(O(object_Kwsa_e_cancelled)) },
 #endif
 #ifdef WSAEREFUSED
  { WSAEREFUSED, &(O(object_Kwsaerefused)) },
 #endif
 #ifdef WSAHOST_NOT_FOUND
  { WSAHOST_NOT_FOUND, &(O(object_Kwsahost_not_found)) },
 #endif
 #ifdef WSATRY_AGAIN
  { WSATRY_AGAIN, &(O(object_Kwsatry_again)) },
 #endif
 #ifdef WSANO_RECOVERY
  { WSANO_RECOVERY, &(O(object_Kwsano_recovery)) },
 #endif
 #ifdef WSANO_DATA
  { WSANO_DATA, &(O(object_Kwsano_data)) },
 #endif
 #ifdef WSA_QOS_RECEIVERS
  { WSA_QOS_RECEIVERS, &(O(object_Kwsa_qos_receivers)) },
 #endif
 #ifdef WSA_QOS_SENDERS
  { WSA_QOS_SENDERS, &(O(object_Kwsa_qos_senders)) },
 #endif
 #ifdef WSA_QOS_NO_SENDERS
  { WSA_QOS_NO_SENDERS, &(O(object_Kwsa_qos_no_senders)) },
 #endif
 #ifdef WSA_QOS_NO_RECEIVERS
  { WSA_QOS_NO_RECEIVERS, &(O(object_Kwsa_qos_no_receivers)) },
 #endif
 #ifdef WSA_QOS_REQUEST_CONFIRMED
  { WSA_QOS_REQUEST_CONFIRMED, &(O(object_Kwsa_qos_request_confirmed)) },
 #endif
 #ifdef WSA_QOS_ADMISSION_FAILURE
  { WSA_QOS_ADMISSION_FAILURE, &(O(object_Kwsa_qos_admission_failure)) },
 #endif
 #ifdef WSA_QOS_POLICY_FAILURE
  { WSA_QOS_POLICY_FAILURE, &(O(object_Kwsa_qos_policy_failure)) },
 #endif
 #ifdef WSA_QOS_BAD_STYLE
  { WSA_QOS_BAD_STYLE, &(O(object_Kwsa_qos_bad_style)) },
 #endif
 #ifdef WSA_QOS_BAD_OBJECT
  { WSA_QOS_BAD_OBJECT, &(O(object_Kwsa_qos_bad_object)) },
 #endif
 #ifdef WSA_QOS_TRAFFIC_CTRL_ERROR
  { WSA_QOS_TRAFFIC_CTRL_ERROR, &(O(object_Kwsa_qos_traffic_ctrl_error)) },
 #endif
 #ifdef WSA_QOS_GENERIC_ERROR
  { WSA_QOS_GENERIC_ERROR, &(O(object_Kwsa_qos_generic_error)) },
 #endif
 #ifdef WSA_QOS_ESERVICETYPE
  { WSA_QOS_ESERVICETYPE, &(O(object_Kwsa_qos_eservicetype)) },
 #endif
 #ifdef WSA_QOS_EFLOWSPEC
  { WSA_QOS_EFLOWSPEC, &(O(object_Kwsa_qos_eflowspec)) },
 #endif
 #ifdef WSA_QOS_EPROVSPECBUF
  { WSA_QOS_EPROVSPECBUF, &(O(object_Kwsa_qos_eprovspecbuf)) },
 #endif
 #ifdef WSA_QOS_EFILTERSTYLE
  { WSA_QOS_EFILTERSTYLE, &(O(object_Kwsa_qos_efilterstyle)) },
 #endif
 #ifdef WSA_QOS_EFILTERTYPE
  { WSA_QOS_EFILTERTYPE, &(O(object_Kwsa_qos_efiltertype)) },
 #endif
 #ifdef WSA_QOS_EFILTERCOUNT
  { WSA_QOS_EFILTERCOUNT, &(O(object_Kwsa_qos_efiltercount)) },
 #endif
 #ifdef WSA_QOS_EOBJLENGTH
  { WSA_QOS_EOBJLENGTH, &(O(object_Kwsa_qos_eobjlength)) },
 #endif
 #ifdef WSA_QOS_EFLOWCOUNT
  { WSA_QOS_EFLOWCOUNT, &(O(object_Kwsa_qos_eflowcount)) },
 #endif
 #ifdef WSA_QOS_EUNKNOWNPSOBJ
  { WSA_QOS_EUNKNOWNPSOBJ, &(O(object_Kwsa_qos_eunknownpsobj)) },
 #endif
 #ifdef WSA_QOS_EPOLICYOBJ
  { WSA_QOS_EPOLICYOBJ, &(O(object_Kwsa_qos_epolicyobj)) },
 #endif
 #ifdef WSA_QOS_EFLOWDESC
  { WSA_QOS_EFLOWDESC, &(O(object_Kwsa_qos_eflowdesc)) },
 #endif
 #ifdef WSA_QOS_EPSFLOWSPEC
  { WSA_QOS_EPSFLOWSPEC, &(O(object_Kwsa_qos_epsflowspec)) },
 #endif
 #ifdef WSA_QOS_EPSFILTERSPEC
  { WSA_QOS_EPSFILTERSPEC, &(O(object_Kwsa_qos_epsfilterspec)) },
 #endif
 #ifdef WSA_QOS_ESDMODEOBJ
  { WSA_QOS_ESDMODEOBJ, &(O(object_Kwsa_qos_esdmodeobj)) },
 #endif
 #ifdef WSA_QOS_ESHAPERATEOBJ
  { WSA_QOS_ESHAPERATEOBJ, &(O(object_Kwsa_qos_eshaperateobj)) },
 #endif
 #ifdef WSA_QOS_RESERVED_PETYPE
  { WSA_QOS_RESERVED_PETYPE, &(O(object_Kwsa_qos_reserved_petype)) },
 #endif
 #ifdef ERROR_IPSEC_QM_POLICY_EXISTS
  { ERROR_IPSEC_QM_POLICY_EXISTS, &(O(object_Kerror_ipsec_qm_policy_exists)) },
 #endif
 #ifdef ERROR_IPSEC_QM_POLICY_NOT_FOUND
  { ERROR_IPSEC_QM_POLICY_NOT_FOUND, &(O(object_Kerror_ipsec_qm_policy_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_QM_POLICY_IN_USE
  { ERROR_IPSEC_QM_POLICY_IN_USE, &(O(object_Kerror_ipsec_qm_policy_in_use)) },
 #endif
 #ifdef ERROR_IPSEC_MM_POLICY_EXISTS
  { ERROR_IPSEC_MM_POLICY_EXISTS, &(O(object_Kerror_ipsec_mm_policy_exists)) },
 #endif
 #ifdef ERROR_IPSEC_MM_POLICY_NOT_FOUND
  { ERROR_IPSEC_MM_POLICY_NOT_FOUND, &(O(object_Kerror_ipsec_mm_policy_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_MM_POLICY_IN_USE
  { ERROR_IPSEC_MM_POLICY_IN_USE, &(O(object_Kerror_ipsec_mm_policy_in_use)) },
 #endif
 #ifdef ERROR_IPSEC_MM_FILTER_EXISTS
  { ERROR_IPSEC_MM_FILTER_EXISTS, &(O(object_Kerror_ipsec_mm_filter_exists)) },
 #endif
 #ifdef ERROR_IPSEC_MM_FILTER_NOT_FOUND
  { ERROR_IPSEC_MM_FILTER_NOT_FOUND, &(O(object_Kerror_ipsec_mm_filter_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_TRANSPORT_FILTER_EXISTS
  { ERROR_IPSEC_TRANSPORT_FILTER_EXISTS, &(O(object_Kerror_ipsec_transport_filter_exists)) },
 #endif
 #ifdef ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND
  { ERROR_IPSEC_TRANSPORT_FILTER_NOT_FOUND, &(O(object_Kerror_ipsec_transport_filter_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_MM_AUTH_EXISTS
  { ERROR_IPSEC_MM_AUTH_EXISTS, &(O(object_Kerror_ipsec_mm_auth_exists)) },
 #endif
 #ifdef ERROR_IPSEC_MM_AUTH_NOT_FOUND
  { ERROR_IPSEC_MM_AUTH_NOT_FOUND, &(O(object_Kerror_ipsec_mm_auth_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_MM_AUTH_IN_USE
  { ERROR_IPSEC_MM_AUTH_IN_USE, &(O(object_Kerror_ipsec_mm_auth_in_use)) },
 #endif
 #ifdef ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND
  { ERROR_IPSEC_DEFAULT_MM_POLICY_NOT_FOUND, &(O(object_Kerror_ipsec_default_mm_policy_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND
  { ERROR_IPSEC_DEFAULT_MM_AUTH_NOT_FOUND, &(O(object_Kerror_ipsec_default_mm_auth_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND
  { ERROR_IPSEC_DEFAULT_QM_POLICY_NOT_FOUND, &(O(object_Kerror_ipsec_default_qm_policy_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_TUNNEL_FILTER_EXISTS
  { ERROR_IPSEC_TUNNEL_FILTER_EXISTS, &(O(object_Kerror_ipsec_tunnel_filter_exists)) },
 #endif
 #ifdef ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND
  { ERROR_IPSEC_TUNNEL_FILTER_NOT_FOUND, &(O(object_Kerror_ipsec_tunnel_filter_not_found)) },
 #endif
 #ifdef ERROR_IPSEC_MM_FILTER_PENDING_DELETION
  { ERROR_IPSEC_MM_FILTER_PENDING_DELETION, &(O(object_Kerror_ipsec_mm_filter_pending_deletion)) },
 #endif
 #ifdef ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION
  { ERROR_IPSEC_TRANSPORT_FILTER_PENDING_DELETION, &(O(object_Kerror_ipsec_transport_filter_pending_deletion)) },
 #endif
 #ifdef ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION
  { ERROR_IPSEC_TUNNEL_FILTER_PENDING_DELETION, &(O(object_Kerror_ipsec_tunnel_filter_pending_deletion)) },
 #endif
 #ifdef ERROR_IPSEC_MM_POLICY_PENDING_DELETION
  { ERROR_IPSEC_MM_POLICY_PENDING_DELETION, &(O(object_Kerror_ipsec_mm_policy_pending_deletion)) },
 #endif
 #ifdef ERROR_IPSEC_MM_AUTH_PENDING_DELETION
  { ERROR_IPSEC_MM_AUTH_PENDING_DELETION, &(O(object_Kerror_ipsec_mm_auth_pending_deletion)) },
 #endif
 #ifdef ERROR_IPSEC_QM_POLICY_PENDING_DELETION
  { ERROR_IPSEC_QM_POLICY_PENDING_DELETION, &(O(object_Kerror_ipsec_qm_policy_pending_deletion)) },
 #endif
 #ifdef WARNING_IPSEC_MM_POLICY_PRUNED
  { WARNING_IPSEC_MM_POLICY_PRUNED, &(O(object_Kwarning_ipsec_mm_policy_pruned)) },
 #endif
 #ifdef WARNING_IPSEC_QM_POLICY_PRUNED
  { WARNING_IPSEC_QM_POLICY_PRUNED, &(O(object_Kwarning_ipsec_qm_policy_pruned)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_AUTH_FAIL
  { ERROR_IPSEC_IKE_AUTH_FAIL, &(O(object_Kerror_ipsec_ike_auth_fail)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_ATTRIB_FAIL
  { ERROR_IPSEC_IKE_ATTRIB_FAIL, &(O(object_Kerror_ipsec_ike_attrib_fail)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NEGOTIATION_PENDING
  { ERROR_IPSEC_IKE_NEGOTIATION_PENDING, &(O(object_Kerror_ipsec_ike_negotiation_pending)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR
  { ERROR_IPSEC_IKE_GENERAL_PROCESSING_ERROR, &(O(object_Kerror_ipsec_ike_general_processing_error)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_TIMED_OUT
  { ERROR_IPSEC_IKE_TIMED_OUT, &(O(object_Kerror_ipsec_ike_timed_out)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NO_CERT
  { ERROR_IPSEC_IKE_NO_CERT, &(O(object_Kerror_ipsec_ike_no_cert)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_SA_DELETED
  { ERROR_IPSEC_IKE_SA_DELETED, &(O(object_Kerror_ipsec_ike_sa_deleted)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_SA_REAPED
  { ERROR_IPSEC_IKE_SA_REAPED, &(O(object_Kerror_ipsec_ike_sa_reaped)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_MM_ACQUIRE_DROP
  { ERROR_IPSEC_IKE_MM_ACQUIRE_DROP, &(O(object_Kerror_ipsec_ike_mm_acquire_drop)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_QM_ACQUIRE_DROP
  { ERROR_IPSEC_IKE_QM_ACQUIRE_DROP, &(O(object_Kerror_ipsec_ike_qm_acquire_drop)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_QUEUE_DROP_MM
  { ERROR_IPSEC_IKE_QUEUE_DROP_MM, &(O(object_Kerror_ipsec_ike_queue_drop_mm)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM
  { ERROR_IPSEC_IKE_QUEUE_DROP_NO_MM, &(O(object_Kerror_ipsec_ike_queue_drop_no_mm)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_DROP_NO_RESPONSE
  { ERROR_IPSEC_IKE_DROP_NO_RESPONSE, &(O(object_Kerror_ipsec_ike_drop_no_response)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_MM_DELAY_DROP
  { ERROR_IPSEC_IKE_MM_DELAY_DROP, &(O(object_Kerror_ipsec_ike_mm_delay_drop)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_QM_DELAY_DROP
  { ERROR_IPSEC_IKE_QM_DELAY_DROP, &(O(object_Kerror_ipsec_ike_qm_delay_drop)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_ERROR
  { ERROR_IPSEC_IKE_ERROR, &(O(object_Kerror_ipsec_ike_error)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_CRL_FAILED
  { ERROR_IPSEC_IKE_CRL_FAILED, &(O(object_Kerror_ipsec_ike_crl_failed)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_KEY_USAGE
  { ERROR_IPSEC_IKE_INVALID_KEY_USAGE, &(O(object_Kerror_ipsec_ike_invalid_key_usage)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_CERT_TYPE
  { ERROR_IPSEC_IKE_INVALID_CERT_TYPE, &(O(object_Kerror_ipsec_ike_invalid_cert_type)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NO_PRIVATE_KEY
  { ERROR_IPSEC_IKE_NO_PRIVATE_KEY, &(O(object_Kerror_ipsec_ike_no_private_key)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_DH_FAIL
  { ERROR_IPSEC_IKE_DH_FAIL, &(O(object_Kerror_ipsec_ike_dh_fail)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_HEADER
  { ERROR_IPSEC_IKE_INVALID_HEADER, &(O(object_Kerror_ipsec_ike_invalid_header)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NO_POLICY
  { ERROR_IPSEC_IKE_NO_POLICY, &(O(object_Kerror_ipsec_ike_no_policy)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_SIGNATURE
  { ERROR_IPSEC_IKE_INVALID_SIGNATURE, &(O(object_Kerror_ipsec_ike_invalid_signature)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_KERBEROS_ERROR
  { ERROR_IPSEC_IKE_KERBEROS_ERROR, &(O(object_Kerror_ipsec_ike_kerberos_error)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NO_PUBLIC_KEY
  { ERROR_IPSEC_IKE_NO_PUBLIC_KEY, &(O(object_Kerror_ipsec_ike_no_public_key)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR
  { ERROR_IPSEC_IKE_PROCESS_ERR, &(O(object_Kerror_ipsec_ike_process_err)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_SA
  { ERROR_IPSEC_IKE_PROCESS_ERR_SA, &(O(object_Kerror_ipsec_ike_process_err_sa)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_PROP
  { ERROR_IPSEC_IKE_PROCESS_ERR_PROP, &(O(object_Kerror_ipsec_ike_process_err_prop)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_TRANS
  { ERROR_IPSEC_IKE_PROCESS_ERR_TRANS, &(O(object_Kerror_ipsec_ike_process_err_trans)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_KE
  { ERROR_IPSEC_IKE_PROCESS_ERR_KE, &(O(object_Kerror_ipsec_ike_process_err_ke)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_ID
  { ERROR_IPSEC_IKE_PROCESS_ERR_ID, &(O(object_Kerror_ipsec_ike_process_err_id)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_CERT
  { ERROR_IPSEC_IKE_PROCESS_ERR_CERT, &(O(object_Kerror_ipsec_ike_process_err_cert)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ
  { ERROR_IPSEC_IKE_PROCESS_ERR_CERT_REQ, &(O(object_Kerror_ipsec_ike_process_err_cert_req)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_HASH
  { ERROR_IPSEC_IKE_PROCESS_ERR_HASH, &(O(object_Kerror_ipsec_ike_process_err_hash)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_SIG
  { ERROR_IPSEC_IKE_PROCESS_ERR_SIG, &(O(object_Kerror_ipsec_ike_process_err_sig)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_NONCE
  { ERROR_IPSEC_IKE_PROCESS_ERR_NONCE, &(O(object_Kerror_ipsec_ike_process_err_nonce)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY
  { ERROR_IPSEC_IKE_PROCESS_ERR_NOTIFY, &(O(object_Kerror_ipsec_ike_process_err_notify)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_DELETE
  { ERROR_IPSEC_IKE_PROCESS_ERR_DELETE, &(O(object_Kerror_ipsec_ike_process_err_delete)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR
  { ERROR_IPSEC_IKE_PROCESS_ERR_VENDOR, &(O(object_Kerror_ipsec_ike_process_err_vendor)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_PAYLOAD
  { ERROR_IPSEC_IKE_INVALID_PAYLOAD, &(O(object_Kerror_ipsec_ike_invalid_payload)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_LOAD_SOFT_SA
  { ERROR_IPSEC_IKE_LOAD_SOFT_SA, &(O(object_Kerror_ipsec_ike_load_soft_sa)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN
  { ERROR_IPSEC_IKE_SOFT_SA_TORN_DOWN, &(O(object_Kerror_ipsec_ike_soft_sa_torn_down)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_COOKIE
  { ERROR_IPSEC_IKE_INVALID_COOKIE, &(O(object_Kerror_ipsec_ike_invalid_cookie)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NO_PEER_CERT
  { ERROR_IPSEC_IKE_NO_PEER_CERT, &(O(object_Kerror_ipsec_ike_no_peer_cert)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_PEER_CRL_FAILED
  { ERROR_IPSEC_IKE_PEER_CRL_FAILED, &(O(object_Kerror_ipsec_ike_peer_crl_failed)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_POLICY_CHANGE
  { ERROR_IPSEC_IKE_POLICY_CHANGE, &(O(object_Kerror_ipsec_ike_policy_change)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NO_MM_POLICY
  { ERROR_IPSEC_IKE_NO_MM_POLICY, &(O(object_Kerror_ipsec_ike_no_mm_policy)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NOTCBPRIV
  { ERROR_IPSEC_IKE_NOTCBPRIV, &(O(object_Kerror_ipsec_ike_notcbpriv)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_SECLOADFAIL
  { ERROR_IPSEC_IKE_SECLOADFAIL, &(O(object_Kerror_ipsec_ike_secloadfail)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_FAILSSPINIT
  { ERROR_IPSEC_IKE_FAILSSPINIT, &(O(object_Kerror_ipsec_ike_failsspinit)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_FAILQUERYSSP
  { ERROR_IPSEC_IKE_FAILQUERYSSP, &(O(object_Kerror_ipsec_ike_failqueryssp)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_SRVACQFAIL
  { ERROR_IPSEC_IKE_SRVACQFAIL, &(O(object_Kerror_ipsec_ike_srvacqfail)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_SRVQUERYCRED
  { ERROR_IPSEC_IKE_SRVQUERYCRED, &(O(object_Kerror_ipsec_ike_srvquerycred)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_GETSPIFAIL
  { ERROR_IPSEC_IKE_GETSPIFAIL, &(O(object_Kerror_ipsec_ike_getspifail)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_FILTER
  { ERROR_IPSEC_IKE_INVALID_FILTER, &(O(object_Kerror_ipsec_ike_invalid_filter)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_OUT_OF_MEMORY
  { ERROR_IPSEC_IKE_OUT_OF_MEMORY, &(O(object_Kerror_ipsec_ike_out_of_memory)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED
  { ERROR_IPSEC_IKE_ADD_UPDATE_KEY_FAILED, &(O(object_Kerror_ipsec_ike_add_update_key_failed)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_POLICY
  { ERROR_IPSEC_IKE_INVALID_POLICY, &(O(object_Kerror_ipsec_ike_invalid_policy)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_UNKNOWN_DOI
  { ERROR_IPSEC_IKE_UNKNOWN_DOI, &(O(object_Kerror_ipsec_ike_unknown_doi)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_SITUATION
  { ERROR_IPSEC_IKE_INVALID_SITUATION, &(O(object_Kerror_ipsec_ike_invalid_situation)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_DH_FAILURE
  { ERROR_IPSEC_IKE_DH_FAILURE, &(O(object_Kerror_ipsec_ike_dh_failure)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_GROUP
  { ERROR_IPSEC_IKE_INVALID_GROUP, &(O(object_Kerror_ipsec_ike_invalid_group)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_ENCRYPT
  { ERROR_IPSEC_IKE_ENCRYPT, &(O(object_Kerror_ipsec_ike_encrypt)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_DECRYPT
  { ERROR_IPSEC_IKE_DECRYPT, &(O(object_Kerror_ipsec_ike_decrypt)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_POLICY_MATCH
  { ERROR_IPSEC_IKE_POLICY_MATCH, &(O(object_Kerror_ipsec_ike_policy_match)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_UNSUPPORTED_ID
  { ERROR_IPSEC_IKE_UNSUPPORTED_ID, &(O(object_Kerror_ipsec_ike_unsupported_id)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_HASH
  { ERROR_IPSEC_IKE_INVALID_HASH, &(O(object_Kerror_ipsec_ike_invalid_hash)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_HASH_ALG
  { ERROR_IPSEC_IKE_INVALID_HASH_ALG, &(O(object_Kerror_ipsec_ike_invalid_hash_alg)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_HASH_SIZE
  { ERROR_IPSEC_IKE_INVALID_HASH_SIZE, &(O(object_Kerror_ipsec_ike_invalid_hash_size)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG
  { ERROR_IPSEC_IKE_INVALID_ENCRYPT_ALG, &(O(object_Kerror_ipsec_ike_invalid_encrypt_alg)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_AUTH_ALG
  { ERROR_IPSEC_IKE_INVALID_AUTH_ALG, &(O(object_Kerror_ipsec_ike_invalid_auth_alg)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_SIG
  { ERROR_IPSEC_IKE_INVALID_SIG, &(O(object_Kerror_ipsec_ike_invalid_sig)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_LOAD_FAILED
  { ERROR_IPSEC_IKE_LOAD_FAILED, &(O(object_Kerror_ipsec_ike_load_failed)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_RPC_DELETE
  { ERROR_IPSEC_IKE_RPC_DELETE, &(O(object_Kerror_ipsec_ike_rpc_delete)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_BENIGN_REINIT
  { ERROR_IPSEC_IKE_BENIGN_REINIT, &(O(object_Kerror_ipsec_ike_benign_reinit)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY
  { ERROR_IPSEC_IKE_INVALID_RESPONDER_LIFETIME_NOTIFY, &(O(object_Kerror_ipsec_ike_invalid_responder_lifetime_notify)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN
  { ERROR_IPSEC_IKE_INVALID_CERT_KEYLEN, &(O(object_Kerror_ipsec_ike_invalid_cert_keylen)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_MM_LIMIT
  { ERROR_IPSEC_IKE_MM_LIMIT, &(O(object_Kerror_ipsec_ike_mm_limit)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NEGOTIATION_DISABLED
  { ERROR_IPSEC_IKE_NEGOTIATION_DISABLED, &(O(object_Kerror_ipsec_ike_negotiation_disabled)) },
 #endif
 #ifdef ERROR_IPSEC_IKE_NEG_STATUS_END
  { ERROR_IPSEC_IKE_NEG_STATUS_END, &(O(object_Kerror_ipsec_ike_neg_status_end)) },
 #endif
 #ifdef ERROR_SXS_SECTION_NOT_FOUND
  { ERROR_SXS_SECTION_NOT_FOUND, &(O(object_Kerror_sxs_section_not_found)) },
 #endif
 #ifdef ERROR_SXS_CANT_GEN_ACTCTX
  { ERROR_SXS_CANT_GEN_ACTCTX, &(O(object_Kerror_sxs_cant_gen_actctx)) },
 #endif
 #ifdef ERROR_SXS_INVALID_ACTCTXDATA_FORMAT
  { ERROR_SXS_INVALID_ACTCTXDATA_FORMAT, &(O(object_Kerror_sxs_invalid_actctxdata_format)) },
 #endif
 #ifdef ERROR_SXS_ASSEMBLY_NOT_FOUND
  { ERROR_SXS_ASSEMBLY_NOT_FOUND, &(O(object_Kerror_sxs_assembly_not_found)) },
 #endif
 #ifdef ERROR_SXS_MANIFEST_FORMAT_ERROR
  { ERROR_SXS_MANIFEST_FORMAT_ERROR, &(O(object_Kerror_sxs_manifest_format_error)) },
 #endif
 #ifdef ERROR_SXS_MANIFEST_PARSE_ERROR
  { ERROR_SXS_MANIFEST_PARSE_ERROR, &(O(object_Kerror_sxs_manifest_parse_error)) },
 #endif
 #ifdef ERROR_SXS_ACTIVATION_CONTEXT_DISABLED
  { ERROR_SXS_ACTIVATION_CONTEXT_DISABLED, &(O(object_Kerror_sxs_activation_context_disabled)) },
 #endif
 #ifdef ERROR_SXS_KEY_NOT_FOUND
  { ERROR_SXS_KEY_NOT_FOUND, &(O(object_Kerror_sxs_key_not_found)) },
 #endif
 #ifdef ERROR_SXS_VERSION_CONFLICT
  { ERROR_SXS_VERSION_CONFLICT, &(O(object_Kerror_sxs_version_conflict)) },
 #endif
 #ifdef ERROR_SXS_WRONG_SECTION_TYPE
  { ERROR_SXS_WRONG_SECTION_TYPE, &(O(object_Kerror_sxs_wrong_section_type)) },
 #endif
 #ifdef ERROR_SXS_THREAD_QUERIES_DISABLED
  { ERROR_SXS_THREAD_QUERIES_DISABLED, &(O(object_Kerror_sxs_thread_queries_disabled)) },
 #endif
 #ifdef ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET
  { ERROR_SXS_PROCESS_DEFAULT_ALREADY_SET, &(O(object_Kerror_sxs_process_default_already_set)) },
 #endif
 #ifdef ERROR_SXS_UNKNOWN_ENCODING_GROUP
  { ERROR_SXS_UNKNOWN_ENCODING_GROUP, &(O(object_Kerror_sxs_unknown_encoding_group)) },
 #endif
 #ifdef ERROR_SXS_UNKNOWN_ENCODING
  { ERROR_SXS_UNKNOWN_ENCODING, &(O(object_Kerror_sxs_unknown_encoding)) },
 #endif
 #ifdef ERROR_SXS_INVALID_XML_NAMESPACE_URI
  { ERROR_SXS_INVALID_XML_NAMESPACE_URI, &(O(object_Kerror_sxs_invalid_xml_namespace_uri)) },
 #endif
 #ifdef ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED
  { ERROR_SXS_ROOT_MANIFEST_DEPENDENCY_NOT_INSTALLED, &(O(object_Kerror_sxs_root_manifest_dependency_not_installed)) },
 #endif
 #ifdef ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED
  { ERROR_SXS_LEAF_MANIFEST_DEPENDENCY_NOT_INSTALLED, &(O(object_Kerror_sxs_leaf_manifest_dependency_not_installed)) },
 #endif
 #ifdef ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE
  { ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE, &(O(object_Kerror_sxs_invalid_assembly_identity_attribute)) },
 #endif
 #ifdef ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE
  { ERROR_SXS_MANIFEST_MISSING_REQUIRED_DEFAULT_NAMESPACE, &(O(object_Kerror_sxs_manifest_missing_required_default_namespace)) },
 #endif
 #ifdef ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE
  { ERROR_SXS_MANIFEST_INVALID_REQUIRED_DEFAULT_NAMESPACE, &(O(object_Kerror_sxs_manifest_invalid_required_default_namespace)) },
 #endif
 #ifdef ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT
  { ERROR_SXS_PRIVATE_MANIFEST_CROSS_PATH_WITH_REPARSE_POINT, &(O(object_Kerror_sxs_private_manifest_cross_path_with_reparse_point)) },
 #endif
 #ifdef ERROR_SXS_DUPLICATE_DLL_NAME
  { ERROR_SXS_DUPLICATE_DLL_NAME, &(O(object_Kerror_sxs_duplicate_dll_name)) },
 #endif
 #ifdef ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME
  { ERROR_SXS_DUPLICATE_WINDOWCLASS_NAME, &(O(object_Kerror_sxs_duplicate_windowclass_name)) },
 #endif
 #ifdef ERROR_SXS_DUPLICATE_CLSID
  { ERROR_SXS_DUPLICATE_CLSID, &(O(object_Kerror_sxs_duplicate_clsid)) },
 #endif
 #ifdef ERROR_SXS_DUPLICATE_IID
  { ERROR_SXS_DUPLICATE_IID, &(O(object_Kerror_sxs_duplicate_iid)) },
 #endif
 #ifdef ERROR_SXS_DUPLICATE_TLBID
  { ERROR_SXS_DUPLICATE_TLBID, &(O(object_Kerror_sxs_duplicate_tlbid)) },
 #endif
 #ifdef ERROR_SXS_DUPLICATE_PROGID
  { ERROR_SXS_DUPLICATE_PROGID, &(O(object_Kerror_sxs_duplicate_progid)) },
 #endif
 #ifdef ERROR_SXS_DUPLICATE_ASSEMBLY_NAME
  { ERROR_SXS_DUPLICATE_ASSEMBLY_NAME, &(O(object_Kerror_sxs_duplicate_assembly_name)) },
 #endif
 #ifdef ERROR_SXS_FILE_HASH_MISMATCH
  { ERROR_SXS_FILE_HASH_MISMATCH, &(O(object_Kerror_sxs_file_hash_mismatch)) },
 #endif
 #ifdef ERROR_SXS_POLICY_PARSE_ERROR
  { ERROR_SXS_POLICY_PARSE_ERROR, &(O(object_Kerror_sxs_policy_parse_error)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MISSINGQUOTE
  { ERROR_SXS_XML_E_MISSINGQUOTE, &(O(object_Kerror_sxs_xml_e_missingquote)) },
 #endif
 #ifdef ERROR_SXS_XML_E_COMMENTSYNTAX
  { ERROR_SXS_XML_E_COMMENTSYNTAX, &(O(object_Kerror_sxs_xml_e_commentsyntax)) },
 #endif
 #ifdef ERROR_SXS_XML_E_BADSTARTNAMECHAR
  { ERROR_SXS_XML_E_BADSTARTNAMECHAR, &(O(object_Kerror_sxs_xml_e_badstartnamechar)) },
 #endif
 #ifdef ERROR_SXS_XML_E_BADNAMECHAR
  { ERROR_SXS_XML_E_BADNAMECHAR, &(O(object_Kerror_sxs_xml_e_badnamechar)) },
 #endif
 #ifdef ERROR_SXS_XML_E_BADCHARINSTRING
  { ERROR_SXS_XML_E_BADCHARINSTRING, &(O(object_Kerror_sxs_xml_e_badcharinstring)) },
 #endif
 #ifdef ERROR_SXS_XML_E_XMLDECLSYNTAX
  { ERROR_SXS_XML_E_XMLDECLSYNTAX, &(O(object_Kerror_sxs_xml_e_xmldeclsyntax)) },
 #endif
 #ifdef ERROR_SXS_XML_E_BADCHARDATA
  { ERROR_SXS_XML_E_BADCHARDATA, &(O(object_Kerror_sxs_xml_e_badchardata)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MISSINGWHITESPACE
  { ERROR_SXS_XML_E_MISSINGWHITESPACE, &(O(object_Kerror_sxs_xml_e_missingwhitespace)) },
 #endif
 #ifdef ERROR_SXS_XML_E_EXPECTINGTAGEND
  { ERROR_SXS_XML_E_EXPECTINGTAGEND, &(O(object_Kerror_sxs_xml_e_expectingtagend)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MISSINGSEMICOLON
  { ERROR_SXS_XML_E_MISSINGSEMICOLON, &(O(object_Kerror_sxs_xml_e_missingsemicolon)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNBALANCEDPAREN
  { ERROR_SXS_XML_E_UNBALANCEDPAREN, &(O(object_Kerror_sxs_xml_e_unbalancedparen)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INTERNALERROR
  { ERROR_SXS_XML_E_INTERNALERROR, &(O(object_Kerror_sxs_xml_e_internalerror)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE
  { ERROR_SXS_XML_E_UNEXPECTED_WHITESPACE, &(O(object_Kerror_sxs_xml_e_unexpected_whitespace)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INCOMPLETE_ENCODING
  { ERROR_SXS_XML_E_INCOMPLETE_ENCODING, &(O(object_Kerror_sxs_xml_e_incomplete_encoding)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MISSING_PAREN
  { ERROR_SXS_XML_E_MISSING_PAREN, &(O(object_Kerror_sxs_xml_e_missing_paren)) },
 #endif
 #ifdef ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE
  { ERROR_SXS_XML_E_EXPECTINGCLOSEQUOTE, &(O(object_Kerror_sxs_xml_e_expectingclosequote)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MULTIPLE_COLONS
  { ERROR_SXS_XML_E_MULTIPLE_COLONS, &(O(object_Kerror_sxs_xml_e_multiple_colons)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALID_DECIMAL
  { ERROR_SXS_XML_E_INVALID_DECIMAL, &(O(object_Kerror_sxs_xml_e_invalid_decimal)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALID_HEXIDECIMAL
  { ERROR_SXS_XML_E_INVALID_HEXIDECIMAL, &(O(object_Kerror_sxs_xml_e_invalid_hexidecimal)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALID_UNICODE
  { ERROR_SXS_XML_E_INVALID_UNICODE, &(O(object_Kerror_sxs_xml_e_invalid_unicode)) },
 #endif
 #ifdef ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK
  { ERROR_SXS_XML_E_WHITESPACEORQUESTIONMARK, &(O(object_Kerror_sxs_xml_e_whitespaceorquestionmark)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNEXPECTEDENDTAG
  { ERROR_SXS_XML_E_UNEXPECTEDENDTAG, &(O(object_Kerror_sxs_xml_e_unexpectedendtag)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNCLOSEDTAG
  { ERROR_SXS_XML_E_UNCLOSEDTAG, &(O(object_Kerror_sxs_xml_e_unclosedtag)) },
 #endif
 #ifdef ERROR_SXS_XML_E_DUPLICATEATTRIBUTE
  { ERROR_SXS_XML_E_DUPLICATEATTRIBUTE, &(O(object_Kerror_sxs_xml_e_duplicateattribute)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MULTIPLEROOTS
  { ERROR_SXS_XML_E_MULTIPLEROOTS, &(O(object_Kerror_sxs_xml_e_multipleroots)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALIDATROOTLEVEL
  { ERROR_SXS_XML_E_INVALIDATROOTLEVEL, &(O(object_Kerror_sxs_xml_e_invalidatrootlevel)) },
 #endif
 #ifdef ERROR_SXS_XML_E_BADXMLDECL
  { ERROR_SXS_XML_E_BADXMLDECL, &(O(object_Kerror_sxs_xml_e_badxmldecl)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MISSINGROOT
  { ERROR_SXS_XML_E_MISSINGROOT, &(O(object_Kerror_sxs_xml_e_missingroot)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNEXPECTEDEOF
  { ERROR_SXS_XML_E_UNEXPECTEDEOF, &(O(object_Kerror_sxs_xml_e_unexpectedeof)) },
 #endif
 #ifdef ERROR_SXS_XML_E_BADPEREFINSUBSET
  { ERROR_SXS_XML_E_BADPEREFINSUBSET, &(O(object_Kerror_sxs_xml_e_badperefinsubset)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNCLOSEDSTARTTAG
  { ERROR_SXS_XML_E_UNCLOSEDSTARTTAG, &(O(object_Kerror_sxs_xml_e_unclosedstarttag)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNCLOSEDENDTAG
  { ERROR_SXS_XML_E_UNCLOSEDENDTAG, &(O(object_Kerror_sxs_xml_e_unclosedendtag)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNCLOSEDSTRING
  { ERROR_SXS_XML_E_UNCLOSEDSTRING, &(O(object_Kerror_sxs_xml_e_unclosedstring)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNCLOSEDCOMMENT
  { ERROR_SXS_XML_E_UNCLOSEDCOMMENT, &(O(object_Kerror_sxs_xml_e_unclosedcomment)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNCLOSEDDECL
  { ERROR_SXS_XML_E_UNCLOSEDDECL, &(O(object_Kerror_sxs_xml_e_uncloseddecl)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNCLOSEDCDATA
  { ERROR_SXS_XML_E_UNCLOSEDCDATA, &(O(object_Kerror_sxs_xml_e_unclosedcdata)) },
 #endif
 #ifdef ERROR_SXS_XML_E_RESERVEDNAMESPACE
  { ERROR_SXS_XML_E_RESERVEDNAMESPACE, &(O(object_Kerror_sxs_xml_e_reservednamespace)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALIDENCODING
  { ERROR_SXS_XML_E_INVALIDENCODING, &(O(object_Kerror_sxs_xml_e_invalidencoding)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALIDSWITCH
  { ERROR_SXS_XML_E_INVALIDSWITCH, &(O(object_Kerror_sxs_xml_e_invalidswitch)) },
 #endif
 #ifdef ERROR_SXS_XML_E_BADXMLCASE
  { ERROR_SXS_XML_E_BADXMLCASE, &(O(object_Kerror_sxs_xml_e_badxmlcase)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALID_STANDALONE
  { ERROR_SXS_XML_E_INVALID_STANDALONE, &(O(object_Kerror_sxs_xml_e_invalid_standalone)) },
 #endif
 #ifdef ERROR_SXS_XML_E_UNEXPECTED_STANDALONE
  { ERROR_SXS_XML_E_UNEXPECTED_STANDALONE, &(O(object_Kerror_sxs_xml_e_unexpected_standalone)) },
 #endif
 #ifdef ERROR_SXS_XML_E_INVALID_VERSION
  { ERROR_SXS_XML_E_INVALID_VERSION, &(O(object_Kerror_sxs_xml_e_invalid_version)) },
 #endif
 #ifdef ERROR_SXS_XML_E_MISSINGEQUALS
  { ERROR_SXS_XML_E_MISSINGEQUALS, &(O(object_Kerror_sxs_xml_e_missingequals)) },
 #endif
 #ifdef ERROR_SXS_PROTECTION_RECOVERY_FAILED
  { ERROR_SXS_PROTECTION_RECOVERY_FAILED, &(O(object_Kerror_sxs_protection_recovery_failed)) },
 #endif
 #ifdef ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT
  { ERROR_SXS_PROTECTION_PUBLIC_KEY_TOO_SHORT, &(O(object_Kerror_sxs_protection_public_key_too_short)) },
 #endif
 #ifdef ERROR_SXS_PROTECTION_CATALOG_NOT_VALID
  { ERROR_SXS_PROTECTION_CATALOG_NOT_VALID, &(O(object_Kerror_sxs_protection_catalog_not_valid)) },
 #endif
 #ifdef ERROR_SXS_UNTRANSLATABLE_HRESULT
  { ERROR_SXS_UNTRANSLATABLE_HRESULT, &(O(object_Kerror_sxs_untranslatable_hresult)) },
 #endif
 #ifdef ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING
  { ERROR_SXS_PROTECTION_CATALOG_FILE_MISSING, &(O(object_Kerror_sxs_protection_catalog_file_missing)) },
 #endif
 #ifdef ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE
  { ERROR_SXS_MISSING_ASSEMBLY_IDENTITY_ATTRIBUTE, &(O(object_Kerror_sxs_missing_assembly_identity_attribute)) },
 #endif
 #ifdef ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME
  { ERROR_SXS_INVALID_ASSEMBLY_IDENTITY_ATTRIBUTE_NAME, &(O(object_Kerror_sxs_invalid_assembly_identity_attribute_name)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_last_error_map = {
  check_last_error_table,
  (sizeof(check_last_error_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_last_error"
};
#define check_last_error(a) (DWORD)map_lisp_to_c(a,&check_last_error_map)
#define check_last_error_reverse(a) map_c_to_lisp(a,&check_last_error_map)

#endif
static const c_lisp_pair_t check_errno_table[] = {
 #ifdef E2BIG
  { E2BIG, &(O(object_Ke2big)) },
 #endif
 #ifdef EACCES
  { EACCES, &(O(object_Keacces)) },
 #endif
 #ifdef EADDRINUSE
  { EADDRINUSE, &(O(object_Keaddrinuse)) },
 #endif
 #ifdef EADDRNOTAVAIL
  { EADDRNOTAVAIL, &(O(object_Keaddrnotavail)) },
 #endif
 #ifdef EAFNOSUPPORT
  { EAFNOSUPPORT, &(O(object_Keafnosupport)) },
 #endif
 #ifdef EAGAIN
  { EAGAIN, &(O(object_Keagain)) },
 #endif
 #ifdef EALREADY
  { EALREADY, &(O(object_Kealready)) },
 #endif
 #ifdef EBADF
  { EBADF, &(O(object_Kebadf)) },
 #endif
 #ifdef EBADMSG
  { EBADMSG, &(O(object_Kebadmsg)) },
 #endif
 #ifdef EBUSY
  { EBUSY, &(O(object_Kebusy)) },
 #endif
 #ifdef ECANCELED
  { ECANCELED, &(O(object_Kecanceled)) },
 #endif
 #ifdef ECHILD
  { ECHILD, &(O(object_Kechild)) },
 #endif
 #ifdef ECONNABORTED
  { ECONNABORTED, &(O(object_Keconnaborted)) },
 #endif
 #ifdef ECONNREFUSED
  { ECONNREFUSED, &(O(object_Keconnrefused)) },
 #endif
 #ifdef ECONNRESET
  { ECONNRESET, &(O(object_Keconnreset)) },
 #endif
 #ifdef EDEADLK
  { EDEADLK, &(O(object_Kedeadlk)) },
 #endif
 #ifdef EDESTADDRREQ
  { EDESTADDRREQ, &(O(object_Kedestaddrreq)) },
 #endif
 #ifdef EDOM
  { EDOM, &(O(object_Kedom)) },
 #endif
 #ifdef EDQUOT
  { EDQUOT, &(O(object_Kedquot)) },
 #endif
 #ifdef EEXIST
  { EEXIST, &(O(object_Keexist)) },
 #endif
 #ifdef EFAULT
  { EFAULT, &(O(object_Kefault)) },
 #endif
 #ifdef EFBIG
  { EFBIG, &(O(object_Kefbig)) },
 #endif
 #ifdef EHOSTUNREACH
  { EHOSTUNREACH, &(O(object_Kehostunreach)) },
 #endif
 #ifdef EIDRM
  { EIDRM, &(O(object_Keidrm)) },
 #endif
 #ifdef EILSEQ
  { EILSEQ, &(O(object_Keilseq)) },
 #endif
 #ifdef EINPROGRESS
  { EINPROGRESS, &(O(object_Keinprogress)) },
 #endif
 #ifdef EINTR
  { EINTR, &(O(object_Keintr)) },
 #endif
 #ifdef EINVAL
  { EINVAL, &(O(object_Keinval)) },
 #endif
 #ifdef EIO
  { EIO, &(O(object_Keio)) },
 #endif
 #ifdef EISCONN
  { EISCONN, &(O(object_Keisconn)) },
 #endif
 #ifdef EISDIR
  { EISDIR, &(O(object_Keisdir)) },
 #endif
 #ifdef ELOOP
  { ELOOP, &(O(object_Keloop)) },
 #endif
 #ifdef EMFILE
  { EMFILE, &(O(object_Kemfile)) },
 #endif
 #ifdef EMLINK
  { EMLINK, &(O(object_Kemlink)) },
 #endif
 #ifdef EMSGSIZE
  { EMSGSIZE, &(O(object_Kemsgsize)) },
 #endif
 #ifdef EMULTIHOP
  { EMULTIHOP, &(O(object_Kemultihop)) },
 #endif
 #ifdef ENAMETOOLONG
  { ENAMETOOLONG, &(O(object_Kenametoolong)) },
 #endif
 #ifdef ENETDOWN
  { ENETDOWN, &(O(object_Kenetdown)) },
 #endif
 #ifdef ENETRESET
  { ENETRESET, &(O(object_Kenetreset)) },
 #endif
 #ifdef ENETUNREACH
  { ENETUNREACH, &(O(object_Kenetunreach)) },
 #endif
 #ifdef ENFILE
  { ENFILE, &(O(object_Kenfile)) },
 #endif
 #ifdef ENOBUFS
  { ENOBUFS, &(O(object_Kenobufs)) },
 #endif
 #ifdef ENODATA
  { ENODATA, &(O(object_Kenodata)) },
 #endif
 #ifdef ENODEV
  { ENODEV, &(O(object_Kenodev)) },
 #endif
 #ifdef ENOENT
  { ENOENT, &(O(object_Kenoent)) },
 #endif
 #ifdef ENOEXEC
  { ENOEXEC, &(O(object_Kenoexec)) },
 #endif
 #ifdef ENOLCK
  { ENOLCK, &(O(object_Kenolck)) },
 #endif
 #ifdef ENOLINK
  { ENOLINK, &(O(object_Kenolink)) },
 #endif
 #ifdef ENOMEM
  { ENOMEM, &(O(object_Kenomem)) },
 #endif
 #ifdef ENOMSG
  { ENOMSG, &(O(object_Kenomsg)) },
 #endif
 #ifdef ENOPROTOOPT
  { ENOPROTOOPT, &(O(object_Kenoprotoopt)) },
 #endif
 #ifdef ENOSPC
  { ENOSPC, &(O(object_Kenospc)) },
 #endif
 #ifdef ENOSR
  { ENOSR, &(O(object_Kenosr)) },
 #endif
 #ifdef ENOSTR
  { ENOSTR, &(O(object_Kenostr)) },
 #endif
 #ifdef ENOSYS
  { ENOSYS, &(O(object_Kenosys)) },
 #endif
 #ifdef ENOTCONN
  { ENOTCONN, &(O(object_Kenotconn)) },
 #endif
 #ifdef ENOTDIR
  { ENOTDIR, &(O(object_Kenotdir)) },
 #endif
 #ifdef ENOTEMPTY
  { ENOTEMPTY, &(O(object_Kenotempty)) },
 #endif
 #ifdef ENOTSOCK
  { ENOTSOCK, &(O(object_Kenotsock)) },
 #endif
 #ifdef ENOTSUP
  { ENOTSUP, &(O(object_Kenotsup)) },
 #endif
 #ifdef ENOTTY
  { ENOTTY, &(O(object_Kenotty)) },
 #endif
 #ifdef ENXIO
  { ENXIO, &(O(object_Kenxio)) },
 #endif
 #ifdef EOPNOTSUPP
  { EOPNOTSUPP, &(O(object_Keopnotsupp)) },
 #endif
 #ifdef EOVERFLOW
  { EOVERFLOW, &(O(object_Keoverflow)) },
 #endif
 #ifdef EPERM
  { EPERM, &(O(object_Keperm)) },
 #endif
 #ifdef EPIPE
  { EPIPE, &(O(object_Kepipe)) },
 #endif
 #ifdef EPROTO
  { EPROTO, &(O(object_Keproto)) },
 #endif
 #ifdef EPROTONOSUPPORT
  { EPROTONOSUPPORT, &(O(object_Keprotonosupport)) },
 #endif
 #ifdef EPROTOTYPE
  { EPROTOTYPE, &(O(object_Keprototype)) },
 #endif
 #ifdef ERANGE
  { ERANGE, &(O(object_Kerange)) },
 #endif
 #ifdef EROFS
  { EROFS, &(O(object_Kerofs)) },
 #endif
 #ifdef ESPIPE
  { ESPIPE, &(O(object_Kespipe)) },
 #endif
 #ifdef ESRCH
  { ESRCH, &(O(object_Kesrch)) },
 #endif
 #ifdef ESTALE
  { ESTALE, &(O(object_Kestale)) },
 #endif
 #ifdef ETIME
  { ETIME, &(O(object_Ketime)) },
 #endif
 #ifdef ETIMEDOUT
  { ETIMEDOUT, &(O(object_Ketimedout)) },
 #endif
 #ifdef ETXTBSY
  { ETXTBSY, &(O(object_Ketxtbsy)) },
 #endif
 #ifdef EWOULDBLOCK
  { EWOULDBLOCK, &(O(object_Kewouldblock)) },
 #endif
 #ifdef EXDEV
  { EXDEV, &(O(object_Kexdev)) },
 #endif
 #ifdef ERREMOTE
  { ERREMOTE, &(O(object_Kerremote)) },
 #endif
 #ifdef EADV
  { EADV, &(O(object_Keadv)) },
 #endif
 #ifdef EBADE
  { EBADE, &(O(object_Kebade)) },
 #endif
 #ifdef EBADFD
  { EBADFD, &(O(object_Kebadfd)) },
 #endif
 #ifdef EBADR
  { EBADR, &(O(object_Kebadr)) },
 #endif
 #ifdef EBADRQC
  { EBADRQC, &(O(object_Kebadrqc)) },
 #endif
 #ifdef EBADSLT
  { EBADSLT, &(O(object_Kebadslt)) },
 #endif
 #ifdef EBFONT
  { EBFONT, &(O(object_Kebfont)) },
 #endif
 #ifdef ECHRNG
  { ECHRNG, &(O(object_Kechrng)) },
 #endif
 #ifdef ECOMM
  { ECOMM, &(O(object_Kecomm)) },
 #endif
 #ifdef EDEADLOCK
  { EDEADLOCK, &(O(object_Kedeadlock)) },
 #endif
 #ifdef EDOTDOT
  { EDOTDOT, &(O(object_Kedotdot)) },
 #endif
 #ifdef EHOSTDOWN
  { EHOSTDOWN, &(O(object_Kehostdown)) },
 #endif
 #ifdef EISNAM
  { EISNAM, &(O(object_Keisnam)) },
 #endif
 #ifdef EL2HLT
  { EL2HLT, &(O(object_Kel2hlt)) },
 #endif
 #ifdef EL2NSYNC
  { EL2NSYNC, &(O(object_Kel2nsync)) },
 #endif
 #ifdef EL3HLT
  { EL3HLT, &(O(object_Kel3hlt)) },
 #endif
 #ifdef EL3RST
  { EL3RST, &(O(object_Kel3rst)) },
 #endif
 #ifdef ELIBACC
  { ELIBACC, &(O(object_Kelibacc)) },
 #endif
 #ifdef ELIBBAD
  { ELIBBAD, &(O(object_Kelibbad)) },
 #endif
 #ifdef ELIBEXEC
  { ELIBEXEC, &(O(object_Kelibexec)) },
 #endif
 #ifdef ELIBMAX
  { ELIBMAX, &(O(object_Kelibmax)) },
 #endif
 #ifdef ELIBSCN
  { ELIBSCN, &(O(object_Kelibscn)) },
 #endif
 #ifdef ELNRNG
  { ELNRNG, &(O(object_Kelnrng)) },
 #endif
 #ifdef EMEDIUMTYPE
  { EMEDIUMTYPE, &(O(object_Kemediumtype)) },
 #endif
 #ifdef ENAVAIL
  { ENAVAIL, &(O(object_Kenavail)) },
 #endif
 #ifdef ENOANO
  { ENOANO, &(O(object_Kenoano)) },
 #endif
 #ifdef ENOCSI
  { ENOCSI, &(O(object_Kenocsi)) },
 #endif
 #ifdef ENOMEDIUM
  { ENOMEDIUM, &(O(object_Kenomedium)) },
 #endif
 #ifdef ENONET
  { ENONET, &(O(object_Kenonet)) },
 #endif
 #ifdef ENOPKG
  { ENOPKG, &(O(object_Kenopkg)) },
 #endif
 #ifdef ENOTBLK
  { ENOTBLK, &(O(object_Kenotblk)) },
 #endif
 #ifdef ENOTNAM
  { ENOTNAM, &(O(object_Kenotnam)) },
 #endif
 #ifdef ENOTUNIQ
  { ENOTUNIQ, &(O(object_Kenotuniq)) },
 #endif
 #ifdef EPFNOSUPPORT
  { EPFNOSUPPORT, &(O(object_Kepfnosupport)) },
 #endif
 #ifdef EREMCHG
  { EREMCHG, &(O(object_Keremchg)) },
 #endif
 #ifdef EREMOTE
  { EREMOTE, &(O(object_Keremote)) },
 #endif
 #ifdef EREMOTEIO
  { EREMOTEIO, &(O(object_Keremoteio)) },
 #endif
 #ifdef ERESTART
  { ERESTART, &(O(object_Kerestart)) },
 #endif
 #ifdef ESHUTDOWN
  { ESHUTDOWN, &(O(object_Keshutdown)) },
 #endif
 #ifdef ESOCKTNOSUPPORT
  { ESOCKTNOSUPPORT, &(O(object_Kesocktnosupport)) },
 #endif
 #ifdef ESRMNT
  { ESRMNT, &(O(object_Kesrmnt)) },
 #endif
 #ifdef ESTRPIPE
  { ESTRPIPE, &(O(object_Kestrpipe)) },
 #endif
 #ifdef ETOOMANYREFS
  { ETOOMANYREFS, &(O(object_Ketoomanyrefs)) },
 #endif
 #ifdef EUCLEAN
  { EUCLEAN, &(O(object_Keuclean)) },
 #endif
 #ifdef EUNATCH
  { EUNATCH, &(O(object_Keunatch)) },
 #endif
 #ifdef EUSERS
  { EUSERS, &(O(object_Keusers)) },
 #endif
 #ifdef EXFULL
  { EXFULL, &(O(object_Kexfull)) },
 #endif
 #ifdef ENOKEY
  { ENOKEY, &(O(object_Kenokey)) },
 #endif
 #ifdef EKEYEXPIRED
  { EKEYEXPIRED, &(O(object_Kekeyexpired)) },
 #endif
 #ifdef EKEYREVOKED
  { EKEYREVOKED, &(O(object_Kekeyrevoked)) },
 #endif
 #ifdef EKEYREJECTED
  { EKEYREJECTED, &(O(object_Kekeyrejected)) },
 #endif
 #ifdef ERFKILL
  { ERFKILL, &(O(object_Kerfkill)) },
 #endif
 #ifdef EHWPOISON
  { EHWPOISON, &(O(object_Kehwpoison)) },
 #endif
 #ifdef STRUNCATE
  { STRUNCATE, &(O(object_Kstruncate)) },
 #endif
 #ifdef ECASECLASH
  { ECASECLASH, &(O(object_Kecaseclash)) },
 #endif
 #ifdef EFTYPE
  { EFTYPE, &(O(object_Keftype)) },
 #endif
 #ifdef ELBIN
  { ELBIN, &(O(object_Kelbin)) },
 #endif
 #ifdef ENMFILE
  { ENMFILE, &(O(object_Kenmfile)) },
 #endif
 #ifdef ENOFILE
  { ENOFILE, &(O(object_Kenofile)) },
 #endif
 #ifdef ENOSHARE
  { ENOSHARE, &(O(object_Kenoshare)) },
 #endif
 #ifdef EPROCLIM
  { EPROCLIM, &(O(object_Keproclim)) },
 #endif
 #ifdef EAUTH
  { EAUTH, &(O(object_Keauth)) },
 #endif
 #ifdef EBADRPC
  { EBADRPC, &(O(object_Kebadrpc)) },
 #endif
 #ifdef EDIRIOCTL
  { EDIRIOCTL, &(O(object_Kedirioctl)) },
 #endif
 #ifdef EDOOFUS
  { EDOOFUS, &(O(object_Kedoofus)) },
 #endif
 #ifdef EJUSTRETURN
  { EJUSTRETURN, &(O(object_Kejustreturn)) },
 #endif
 #ifdef ELAST
  { ELAST, &(O(object_Kelast)) },
 #endif
 #ifdef ENEEDAUTH
  { ENEEDAUTH, &(O(object_Keneedauth)) },
 #endif
 #ifdef ENOATTR
  { ENOATTR, &(O(object_Kenoattr)) },
 #endif
 #ifdef ENOIOCTL
  { ENOIOCTL, &(O(object_Kenoioctl)) },
 #endif
 #ifdef EPROCUNAVAIL
  { EPROCUNAVAIL, &(O(object_Keprocunavail)) },
 #endif
 #ifdef EPROGMISMATCH
  { EPROGMISMATCH, &(O(object_Keprogmismatch)) },
 #endif
 #ifdef EPROGUNAVAIL
  { EPROGUNAVAIL, &(O(object_Keprogunavail)) },
 #endif
 #ifdef ERPCMISMATCH
  { ERPCMISMATCH, &(O(object_Kerpcmismatch)) },
 #endif
 #ifdef ELOCKUNMAPPED
  { ELOCKUNMAPPED, &(O(object_Kelockunmapped)) },
 #endif
 #ifdef ENOTACTIVE
  { ENOTACTIVE, &(O(object_Kenotactive)) },
 #endif
 #ifdef ENOTRECOVERABLE
  { ENOTRECOVERABLE, &(O(object_Kenotrecoverable)) },
 #endif
 #ifdef EOWNERDEAD
  { EOWNERDEAD, &(O(object_Keownerdead)) },
 #endif
 #ifdef EALIGN
  { EALIGN, &(O(object_Kealign)) },
 #endif
 #ifdef EDIRTY
  { EDIRTY, &(O(object_Kedirty)) },
 #endif
 #ifdef EDUPPKG
  { EDUPPKG, &(O(object_Keduppkg)) },
 #endif
 #ifdef EINIT
  { EINIT, &(O(object_Keinit)) },
 #endif
 #ifdef EISNAME
  { EISNAME, &(O(object_Keisname)) },
 #endif
 #ifdef ENET
  { ENET, &(O(object_Kenet)) },
 #endif
 #ifdef ENOSYM
  { ENOSYM, &(O(object_Kenosym)) },
 #endif
 #ifdef EREMDEV
  { EREMDEV, &(O(object_Keremdev)) },
 #endif
 #ifdef EREMOTERELEASE
  { EREMOTERELEASE, &(O(object_Keremoterelease)) },
 #endif
 #ifdef EVERSION
  { EVERSION, &(O(object_Keversion)) },
 #endif
 #ifdef EAIO
  { EAIO, &(O(object_Keaio)) },
 #endif
 #ifdef ECLONEME
  { ECLONEME, &(O(object_Kecloneme)) },
 #endif
 #ifdef EFAIL
  { EFAIL, &(O(object_Kefail)) },
 #endif
 #ifdef EINPROG
  { EINPROG, &(O(object_Keinprog)) },
 #endif
 #ifdef EMTIMERS
  { EMTIMERS, &(O(object_Kemtimers)) },
 #endif
 #ifdef ERESTARTNOHAND
  { ERESTARTNOHAND, &(O(object_Kerestartnohand)) },
 #endif
 #ifdef ERESTARTNOINTR
  { ERESTARTNOINTR, &(O(object_Kerestartnointr)) },
 #endif
 #ifdef ERESTARTSYS
  { ERESTARTSYS, &(O(object_Kerestartsys)) },
 #endif
  { 0, NULL }
};
static const c_lisp_map_t check_errno_map = {
  check_errno_table,
  (sizeof(check_errno_table)/sizeof(c_lisp_pair_t))-1,
  0,true,
  "check_errno"
};
#define check_errno(a) (int)map_lisp_to_c(a,&check_errno_map)
#define check_errno_reverse(a) map_c_to_lisp(a,&check_errno_map)


#line 156

#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
/* we use posix fcntl() on unix and win32 LockFileEx() on win32.
   since cygwin supports fcntl(), we use it there, but another option
   would be to use cygwin get_osfhandle() + win32 LockFileEx(), see
   https://cygwin.com/ml/cygwin/2003-08/msg00588.html */

/* ============================== aux ============================== */

/* the input handle from input stream and output handle from output stream
 can trigger GC */
static Handle stream_get_handle (gcv_object_t *stream_) {
  if (uint_p(*stream_)) {
    Handle fd = (Handle)I_to_uint(*stream_);
    *stream_ = nullobj;
    return fd;
  } else {
    pushSTACK(*stream_); funcall(L(input_stream_p),1);
    return stream_lend_handle(stream_,!nullp(value1),NULL);
  }
}

/* signal the appropriate error error */
static _Noreturn void error_OS_stream (object stream) {
  if (eq(nullobj,stream)) OS_error();
  else OS_filestream_error(stream);
}

/* ============================== locking ============================== */
#if defined(WIN32_NATIVE)
/* LockFileEx does not exist on Windows95/98/ME. */
typedef BOOL (WINAPI * LockFileExFuncType)
  (HANDLE hFile, DWORD dwFlags, DWORD dwReserved,
   DWORD nNumberOfBytesToLockLow, DWORD nNumberOfBytesToLockHigh,
   LPOVERLAPPED lpOverlapped);
static LockFileExFuncType LockFileExFunc = NULL;
static BOOL my_LockFileEx
(HANDLE hFile, DWORD dwFlags, DWORD dwReserved,
 DWORD nNumberOfBytesToLockLow, DWORD nNumberOfBytesToLockHigh,
 LPOVERLAPPED lpOverlapped) {
  (void)dwFlags; (void)dwReserved;
  return LockFile(hFile,lpOverlapped->Offset,lpOverlapped->OffsetHigh,
                  nNumberOfBytesToLockLow,nNumberOfBytesToLockHigh);
}
typedef BOOL (WINAPI * UnlockFileExFuncType)
  (HANDLE hFile, DWORD dwReserved,
   DWORD nNumberOfBytesToUnlockLow, DWORD nNumberOfBytesToUnlockHigh,
   LPOVERLAPPED lpOverlapped);
static UnlockFileExFuncType UnlockFileExFunc = NULL;
static BOOL my_UnlockFileEx
(HANDLE hFile, DWORD dwReserved,
 DWORD nNumberOfBytesToUnlockLow, DWORD nNumberOfBytesToUnlockHigh,
 LPOVERLAPPED lpOverlapped) {
  (void)dwReserved;
  return UnlockFile(hFile,lpOverlapped->Offset,lpOverlapped->OffsetHigh,
                    nNumberOfBytesToUnlockLow,nNumberOfBytesToUnlockHigh);
}
#endif

#if defined(SIZEOF_OFF_T) && SIZEOF_OFF_T == 8
# define I_to_offset(x)  I_to_uint64(check_uint64(x))
#else
# define I_to_offset(x)  I_to_uint32(check_uint32(x))
#endif
DEFUN(POSIX::STREAM-LOCK, stream lockp &key :BLOCK SHARED :START :LENGTH,(subr_posix_stream_lock,seclass_default,2,0,norest,key,4,NIL))
{ /* the interface to fcntl(2) */
  Handle fd = (Handle)-1;
  bool lock_p = !nullp(STACK_4), failed_p;
  object stream;
  uintL start = missingp(STACK_1) ? 0 : I_to_UL(check_ulong(STACK_1));
#if defined(WIN32_NATIVE)
  uint64 length;
  DWORD flags = !lock_p ? 0 :
    (missingp(STACK_2) ? LOCKFILE_EXCLUSIVE_LOCK : 0) | /* (SHARED NIL) */
    (nullp(STACK_3) ? LOCKFILE_FAIL_IMMEDIATELY : 0);   /* (BLOCK T) */
  OVERLAPPED ol = {0,0,start,0,NULL};
#else
  off_t length;
  int cmd = nullp(STACK_3) ? F_SETLK : F_SETLKW; /* (BLOCK T) */
  struct flock fl;
  fl.l_type = !lock_p ? F_UNLCK :          /* unlock */
    missingp(STACK_2) ? F_WRLCK : F_RDLCK; /* (SHARED NIL) */
  fl.l_whence = SEEK_SET;
  fl.l_start = start;
#endif
  if (uint_p(STACK_5)) {        /* STREAM */
    fd = (Handle)I_to_uint(STACK_5);
    stream = nullobj;
  } else
    stream = open_file_stream_handle(STACK_5,&fd,false);
  if (missingp(STACK_0)) {     /* no :LENGTH => use file size */
    /* we use OS to get file size instead of calling FILE-LENGTH because
       on win32 FILE-LENGTH will fail with ERROR_LOCK_VIOLATION when the
       underlying file is locked */
#  if defined(WIN32_NATIVE)
    uint32 size_hi;
    uint32 size_lo;
    begin_blocking_system_call();
    size_lo = GetFileSize(fd,(DWORD*)&size_hi);
    /* Value returned can be (LONG) -1 even on success,
       check the last error code */
    failed_p = (size_lo == INVALID_FILE_SIZE) && (GetLastError() != 0);
    end_blocking_system_call();
    if (failed_p) goto stream_lock_error;
    length = ((uint64)size_hi << 32) | (uint64)size_lo;
#  elif defined(HAVE_FSTAT)
    struct stat st;
    begin_blocking_system_call();
    failed_p = (-1 == fstat(fd,&st));
    end_blocking_system_call();
    if (failed_p) goto stream_lock_error;
    length = st.st_size;
#  else
    length = 0;
#  endif
  } else
    length = I_to_offset(STACK_0);
  begin_blocking_system_call();
#if defined(WIN32_NATIVE)
  if (lock_p) {
    failed_p = !(*LockFileExFunc)(fd,flags,0,length,0,&ol);
    if (failed_p && nullp(STACK_3) && GetLastError() == ERROR_LOCK_VIOLATION)
      failed_p = lock_p = false; /* failed to lock, :BLOCK NIL */
  } else
    failed_p = !(*UnlockFileExFunc)(fd,0,length,0,&ol);
#else
  fl.l_len = length;
  if ((failed_p = (-1 == fcntl(fd,cmd,&fl)))
      && lock_p && (cmd == F_SETLK) && (errno == EACCES || errno == EAGAIN))
    failed_p = lock_p = false; /* failed to lock, :BLOCK NIL */
#endif
  end_blocking_system_call();
  if (failed_p) stream_lock_error:
    error_OS_stream(stream);
  skipSTACK(6);
  VALUES_IF(lock_p);
}
#endif  /* fcntl | WIN32_NATIVE */

/* ============================== fcntl ============================== */
#if defined(HAVE_FCNTL)

/* note that O_ACCMODE is treated specially */
#line 301


DEFUN(POSIX::STREAM-OPTIONS, stream cmd &optional value,(subr_posix_stream_options,seclass_default,2,1,norest,nokey,0,NIL))
{ /* http://opengroup.org/onlinepubs/9699919799/functions/fcntl.html */
  int cmd = check_fcntl_cmd(STACK_1);
  Handle fd = stream_get_handle(&STACK_2);
  int value;
  if (boundp(STACK_0)) {        /* SET */
    switch (cmd) {
      case F_GETFD: value = check_fd_flags_of_list(STACK_0);
        cmd = F_SETFD; break;
      case F_GETFL: value = check_fl_flags_of_list(STACK_0);
        cmd = F_SETFL; break;
      default: NOTREACHED;
    }
    begin_blocking_system_call();
    value = fcntl(fd,cmd,value);
    end_blocking_system_call();
    if (-1 == value) error_OS_stream(STACK_2);
    VALUES0;
  } else {                      /* GET */
    begin_blocking_system_call();
    value = fcntl(fd,cmd);
    end_blocking_system_call();
    if (-1 == value) error_OS_stream(STACK_2);
    switch (cmd) {
      case F_GETFD: value1 = check_fd_flags_to_list(value); break;
      case F_GETFL:
        switch (value & O_ACCMODE) {
          case O_RDONLY: STACK_0 = O(object_Krdonly); break;
          case O_WRONLY: STACK_0 = O(object_Kwronly); break;
          case O_RDWR: STACK_0 = O(object_Krdwr); break;
          default: NOTREACHED;
        }
        STACK_1 = check_fl_flags_to_list(value & ~O_ACCMODE);
        value1 = allocate_cons();
        Car(value1) = STACK_0;
        Cdr(value1) = STACK_1;
        break;
      default: NOTREACHED;
    }
    mv_count = 1;
  }
  skipSTACK(3);
}
#endif

/* call f on physical namestring of path and data
 > path: a pathname designator
 > f: system call
 < data: anything which f accepts
 < value1: the physical namestring of path (for error reporting)
 < returns whatever f returns
 NB: on success, unix functions return 0, while woe32 functions return 1 !
 can trigger GC */
static /*maygc*/ void* on_pnamestring
(object path, void* (*f) (const char*,void*), void* data) {
  void* ret;
  pushSTACK(physical_namestring(path)); /* save for blocking */
  with_string_0(STACK_0,GLO(pathname_encoding),pathz, {
      begin_blocking_system_call();
      ret = (*f)(pathz,data);
      end_blocking_system_call();
    });
  value1 = popSTACK();
  return ret;
}
#define ON_PNAMESTRING(p,f,d)  on_pnamestring(p,(void*(*)(const char*,void*))&(f),(void*)(d))

/* =========================== file truncate =========================== */
/* NB: woe32 has ftruncate, but, just like fstat, it does not accept a Handle,
   just an integer of an unknown nature */

#if defined(WIN32_NATIVE)
typedef LARGE_INTEGER file_offset_t;
static inline void I_to_file_offset (object obj, file_offset_t *length)
{ length->QuadPart = I_to_sint64(check_sint64(obj)); }
#elif defined(UNIX)
typedef off_t file_offset_t;
static inline void I_to_file_offset (object obj, file_offset_t *length)
{ *length = I_to_offset(obj); }
#else
# error file_offset_t is not defined
#endif

/* truncate a file, STACK_0 = path */
static void* path_truncate (const char *path, file_offset_t *length) {
#if defined(WIN32_NATIVE)
  HANDLE fd = CreateFile(path,GENERIC_WRITE,0,NULL,OPEN_EXISTING,
                         FILE_ATTRIBUTE_NORMAL,NULL);
  return (void*)(!(fd != INVALID_HANDLE_VALUE
                   && SetFilePointerEx(fd,*length,NULL,FILE_BEGIN)
                   && SetEndOfFile(fd)
                   && CloseHandle(fd)));
#elif defined(HAVE_TRUNCATE)
  return (void*)(uintP)truncate(path,*length);
#else
#error FILE-SIZE: no truncate and not woe32
#endif
}

/* truncate a stream, STACK_0 = stream */
static void stream_truncate (Handle fd, file_offset_t *length) {
  begin_blocking_system_call();
#if defined(WIN32_NATIVE)
  { LARGE_INTEGER cur_pos;
    if (!(SetFilePointerEx(fd,(LARGE_INTEGER){QuadPart:0},
                           &cur_pos,FILE_CURRENT)
          && SetFilePointerEx(fd,*length,NULL,FILE_BEGIN)
          && SetEndOfFile(fd)
          && SetFilePointerEx(fd,cur_pos,NULL,FILE_BEGIN)))
      { end_blocking_system_call(); OS_filestream_error(STACK_0); }
  }
#elif defined(HAVE_FTRUNCATE)
  if (ftruncate(fd,*length))
    { end_blocking_system_call(); OS_file_error(STACK_0); }
#else
#error FILE-SIZE: no ftruncate and not woe32
#endif
  end_blocking_system_call();
}

/* separate from SET-FILE-STAT because it works only on paths
   while (setf file-size) supports streams as well */
DEFUN(POSIX::%SET-FILE-SIZE, file new-size,(subr_posix__25set_file_size,seclass_default,2,0,norest,nokey,0,NIL)) {
  /* http://opengroup.org/onlinepubs/9699919799/functions/truncate.html
     http://opengroup.org/onlinepubs/9699919799/functions/ftruncate.html
     http://msdn.microsoft.com/en-us/library/aa365542(VS.85).aspx
     http://msdn.microsoft.com/en-us/library/aa365531(VS.85).aspx */
  file_offset_t length;
  Handle fd;
  I_to_file_offset(STACK_0,&length);
  /* stream_truncate uses STACK_0 for error reporting */
  pushSTACK(open_file_stream_handle(STACK_1,&fd,true));
  if (eq(nullobj,STACK_0)) {    /* not a stream - use path */
    if (ON_PNAMESTRING(STACK_2,path_truncate,&length))
      OS_file_error(value1);
  } else stream_truncate(fd,&length); /* stream - use fd */
  VALUES1(STACK_1); skipSTACK(3);
}

#if defined(WIN32_NATIVE)
static void* get_file_size (const char *path, file_offset_t *length) {
  Handle fd = CreateFile(path,GENERIC_WRITE,0,NULL,OPEN_EXISTING,
                         FILE_ATTRIBUTE_NORMAL,NULL);
  return (void*)(!(fd != INVALID_HANDLE_VALUE
                   && GetFileSizeEx(fd,length)
                   && CloseHandle(fd)));
}
#endif
DEFUN(POSIX:FILE-SIZE, file,(subr_posix_file_size,seclass_default,1,0,norest,nokey,0,NIL)) {
  /* we could implement this in Lisp like this:
     (defun file-size (file)
       (handler-case (file-length file)
         (file-error (c) (with-open-file (s file) (file-length s))))) */
  Handle fd;
  object stream = open_file_stream_handle(STACK_0,&fd,true);
  if (eq(nullobj,stream)) {    /* not a stream - use path */
   #if defined(WIN32_NATIVE)
    LARGE_INTEGER length;
    if (ON_PNAMESTRING(STACK_0,get_file_size,&length))
      OS_file_error(value1);
    VALUES1(sint64_to_I(length.QuadPart));
   #elif defined(HAVE_STAT)
    struct stat buf;
    if (ON_PNAMESTRING(STACK_0,stat,&buf))
      OS_file_error(value1);
    VALUES1(off_to_I(buf.st_size));
   #else
    #error FILE-SIZE: no stat and not woe32
   #endif
    skipSTACK(1);
  } else {                      /* stream - use FILE-LENGTH */
    STACK_0 = stream;
    funcall(L(file_length),1);
  }
}

/* ============================== syslog ============================== */
#if defined(HAVE_SYSLOG)
#line 482

#line 485


#if defined(HAVE_OPENLOG)
static char* log_ident=NULL;
DEFUN(POSIX:OPENLOG,ident &key PID CONS NDELAY ODELAY NOWAIT FACILITY,(subr_posix_openlog,seclass_default,1,0,norest,key,6,NIL)) {
  int facility = check_syslog_facility(popSTACK());
  int logopt = syslog_opt_flags();
  with_string_0(check_string(popSTACK()),GLO(misc_encoding),ident, {
      log_ident = (char*)clisp_realloc(log_ident,strlen(ident)+1);
      begin_blocking_system_call();
      strcpy(log_ident,ident);
      openlog(log_ident,logopt,facility);
      end_blocking_system_call();
    });
  VALUES0;
}
#endif
#if defined(HAVE_SETLOGMASK)
DEFUN(POSIX:SETLOGMASK, maskpri,(subr_posix_setlogmask,seclass_default,1,0,norest,nokey,0,NIL)) {
  int priority = (missingp(STACK_0) ? (skipSTACK(1),0) /*query*/ :
                  check_syslog_severity(popSTACK()));
  int logmask;
  begin_system_call();
  logmask = setlogmask(LOG_MASK(priority));
  end_system_call();
  VALUES1(check_syslog_severity_reverse(logmask));
}
#endif
DEFUN(POSIX::%SYSLOG, severity facility message,(subr_posix__25syslog,seclass_default,3,0,norest,nokey,0,NIL)) {
  int priority =
    check_syslog_severity(STACK_2) | check_syslog_facility(STACK_1);
  with_string_0(STACK_0 = check_string(STACK_0),GLO(misc_encoding),mesg, {
      begin_blocking_system_call();
      /* disable %m but avoid surprises with % special handling
         http://opengroup.org/onlinepubs/9699919799/functions/syslog.html */
      syslog(priority,"%s",mesg);
      end_blocking_system_call();
    });
  VALUES0; skipSTACK(3);
}
#if defined(HAVE_CLOSELOG)
DEFUN(POSIX:CLOSELOG,,(subr_posix_closelog,seclass_default,0,0,norest,nokey,0,NIL)) {
  begin_blocking_system_call();
  closelog();
#if defined(HAVE_OPENLOG)
  if(log_ident) { free(log_ident); log_ident=NULL; }
#endif
  end_blocking_system_call();
  VALUES0;
}
#endif
#endif  /* HAVE_SYSLOG */

/* ========================== time conversion ========================== */
/* call ENCODE-UNIVERSAL-TIME on struct tm and timezone  */
static Values tm_to_lisp (struct tm *tm, object timezone) {
  pushSTACK(fixnum(tm->tm_sec));
  pushSTACK(fixnum(tm->tm_min));
  pushSTACK(fixnum(tm->tm_hour));
  pushSTACK(fixnum(tm->tm_mday));
  pushSTACK(fixnum(1+tm->tm_mon));
  pushSTACK(fixnum(1900+tm->tm_year));
  pushSTACK(timezone);
  funcall(S(encode_universal_time),7);
}
DEFUN(POSIX:STRING-TIME, format &optional datum timezone,(subr_posix_string_time,seclass_default,1,2,norest,nokey,0,NIL))
{ /* http://opengroup.org/onlinepubs/9699919799/functions/strptime.html
     http://opengroup.org/onlinepubs/9699919799/functions/strftime.html */
  STACK_2 = check_string(STACK_2); /* format */
  if (missingp(STACK_1)) { /* datum defaults to the current time */
    funcall(L(get_universal_time),0);
    STACK_1 = value1;
  }
  if (stringp(STACK_1)) {          /* parse: strptime */
    struct tm tm;
    unsigned int offset;
    tm.tm_sec = 0; /* Seconds [0,60]. */
    tm.tm_min = 0; /* Minutes [0,59]. */
    tm.tm_hour = 0; /* Hour [0,23]. */
    tm.tm_mday = 1; /* Day of month [1,31]. */
    tm.tm_mon = 0; /* Month of year [0,11]. */
    tm.tm_year = 0; /* Years since 1900. */
    tm.tm_wday = 0; /* Day of week [0,6] (C: Sunday=0 <== CL: Monday=0 */
    tm.tm_isdst = false; /* Daylight Savings flag. */
    with_string_0(STACK_1,GLO(misc_encoding),buf, {
        with_string_0(STACK_2,GLO(misc_encoding),format, {
            char *ret;
            begin_system_call();
            if ((ret = strptime(buf,format,&tm))) offset = ret - buf;
            else offset = 0;
            end_system_call();
          });
      });
    if (offset == 0) {
      pushSTACK(STACK_1);/*datum*/ pushSTACK(STACK_(2+1));/*format*/
      pushSTACK(TheSubr(subr_self)->name);
      error(error_condition,GETTEXT("~S: invalid format ~S or datum ~S"));
    }
    tm_to_lisp(&tm,STACK_0);    /* set value1 */
    value2 = tm.tm_isdst > 0 ? T : NIL;
    value3 = fixnum(offset);
    mv_count = 3;
    skipSTACK(3);
  } else if (integerp(STACK_1)) { /* format: strftime */
    struct tm tm;
    funcall(O(object_cl_decode_universal_time),2);
    tm.tm_sec = posfixnum_to_V(value1); /* Seconds [0,60]. */
    tm.tm_min = posfixnum_to_V(value2); /* Minutes [0,59]. */
    tm.tm_hour = posfixnum_to_V(value3); /* Hour [0,23]. */
    tm.tm_mday = posfixnum_to_V(value4); /* Day of month [1,31]. */
    tm.tm_mon = posfixnum_to_V(value5) - 1; /* Month of year [0,11]. */
    tm.tm_year = posfixnum_to_V(value6) - 1900; /* Years since 1900. */
    /* Day of week [0,6] (C: Sunday=0 <== CL: Monday=0 */
    tm.tm_wday = (posfixnum_to_V(value7) + 1) % 7;
    tm.tm_isdst = !nullp(value8); /* Daylight Savings flag. */
    /* tm.tm_yday == Day of year [0,365]. -- use mkime() */
    { time_t ret;
      begin_system_call(); ret = mktime(&tm); end_system_call();
      if (ret == (time_t)-1) ANSIC_error();
    }
    with_string_0(STACK_0,GLO(misc_encoding),format, {
        /* at least 4 characters per each format char + safety */
        size_t bufsize = 4 * format_bytelen + 64;
        char* buf = (char*)alloca(bufsize);
        size_t retval;
        begin_system_call();
        retval = strftime(buf,bufsize,format,&tm);
        end_system_call();
        VALUES1(n_char_to_string(buf,retval,GLO(misc_encoding)));
      });
    skipSTACK(1);
  } else error_string_integer(STACK_1);
}

#if defined(HAVE_GETDATE) && defined(HAVE_DECL_GETDATE_ERR)
DEFUN(POSIX:GETDATE, timespec &optional timezone,(subr_posix_getdate,seclass_default,1,1,norest,nokey,0,NIL))
{ /* http://opengroup.org/onlinepubs/9699919799/functions/getdate.html */
  struct tm *tm;
 getdate_restart:
  STACK_1 = check_string(STACK_1);
  with_string_0(STACK_1,GLO(misc_encoding),timespec, {
      begin_system_call();
      tm = getdate(timespec);
      end_system_call();
    });
  if (tm == NULL) {
    pushSTACK(NIL);             /* no PLACE */
    pushSTACK(fixnum(getdate_err));
    pushSTACK(STACK_(1+2));
    pushSTACK(TheSubr(subr_self)->name);
    check_value(error_condition,GETTEXT("~S(~S): getdate error ~S"));
    STACK_1 = value1;
    goto getdate_restart;
  }
  tm_to_lisp(tm,STACK_0);
  skipSTACK(2);
}
#endif  /* HAVE_GETDATE & HAVE_DECL_GETDATE_ERR */

/* ========================== string comparison ========================== */
/* call strverscmp() on STACK_0 & STACK_1 and remove them from STACK */
static /*maygc*/ int string_version_compare (void) {
  int ret;
  STACK_0 = check_string(STACK_0);
  STACK_1 = check_string(STACK_1);
  with_string_0(STACK_0,GLO(misc_encoding),s1, {
      with_string_0(STACK_1,GLO(misc_encoding),s2, {
          begin_system_call(); ret = strverscmp(s2,s1); end_system_call();
        });
    });
  skipSTACK(2);
  return ret;
}

DEFUN(OS::VERSION-COMPARE, string1 string2,(subr_os_version_compare,seclass_default,2,0,norest,nokey,0,NIL)) {
  int ret=string_version_compare();
  VALUES1(ret<0 ? S(smaller) : ret>0 ? S(greater) : S(numequal));
}
DEFUN(OS:VERSION<,  string1 string2,(subr_os_version_3C,seclass_default,2,0,norest,nokey,0,NIL)){VALUES_IF(string_version_compare() <  0);}
DEFUN(OS:VERSION<=, string1 string2,(subr_os_version_3C_3D,seclass_default,2,0,norest,nokey,0,NIL)){VALUES_IF(string_version_compare() <= 0);}
DEFUN(OS:VERSION>,  string1 string2,(subr_os_version_3E,seclass_default,2,0,norest,nokey,0,NIL)){VALUES_IF(string_version_compare() >  0);}
DEFUN(OS:VERSION>=, string1 string2,(subr_os_version_3E_3D,seclass_default,2,0,norest,nokey,0,NIL)){VALUES_IF(string_version_compare() >= 0);}

/* ========================== temporary files ========================== */
#line 682
#define ENSURE_6X(name,template)                           if (name##_bytelen > 6                                       && name[name##_bytelen-1]=='X'                           && name[name##_bytelen-2]=='X'                           && name[name##_bytelen-3]=='X'                           && name[name##_bytelen-4]=='X'                           && name[name##_bytelen-5]=='X'                           && name[name##_bytelen-6]=='X') {                      c_template = name;                                     } else {                                                   c_template = (char*)alloca(name##_bytelen+6+1);          strcpy(c_template,name);                                 strcat(c_template,"XXXXXX");                           }
#if defined(WIN32_NATIVE)
# define allocate_lisp_handle(fd)  allocate_handle((HANDLE)_get_osfhandle(fd))
#else
# define allocate_lisp_handle allocate_handle
#endif
#line 689
DEFUN(POSIX:MKSTEMP, template &key :DIRECTION :BUFFERED :EXTERNAL-FORMAT        :ELEMENT-TYPE,(subr_posix_mkstemp,seclass_default,1,0,norest,key,4,NIL)) {
  /* http://opengroup.org/onlinepubs/9699919799/functions/mkstemp.html */
  object fname = physical_namestring(STACK_4);
  direction_t dir = (boundp(STACK_3) ? check_direction(STACK_3)
                     : DIRECTION_OUTPUT);
  int fd;
  with_string_0(fname,GLO(pathname_encoding),namez,{
      char *c_template;
      begin_blocking_system_call();
      ENSURE_6X(namez,c_template);
      fd = mkstemp(c_template);
      end_blocking_system_call();
      fname = asciz_to_string(c_template,GLO(pathname_encoding));
    });
  if (fd == -1) ANSIC_error();
  pushSTACK(fname);  funcall(L(pathname),1); STACK_4=value1; /* filename */
  pushSTACK(value1); funcall(L(truename),1); STACK_3=value1; /* truename */
  pushSTACK(allocate_lisp_handle(fd));
  /* stack layout: FD, eltype, extfmt, buff, truename, filename */
  VALUES1(make_file_stream(dir,false,true));
}

/* ================= user accounting database functions ================= */
#if defined(HAVE_UTMPX_H)
#line 714

static int check_utmpx (gcv_object_t *arg) {
  *arg = check_classname(*arg,O(object_posix__utmpx));
  return check_ut_type(TheStructure(*arg)->recdata[4]);
}
/* convert C struct utmpx to Lisp
 can trigger GC */
static Values utmpx_to_lisp (struct utmpx *utmpx, gcv_object_t *utmpx_o) {
  pushSTACK(check_ut_type_reverse(utmpx->ut_type));
  pushSTACK(safe_to_string(utmpx->ut_user));
  pushSTACK(safe_to_string(utmpx->ut_id));
  pushSTACK(safe_to_string(utmpx->ut_line));
  pushSTACK(L_to_I(utmpx->ut_pid));
#if defined(HAVE_UTMPX_UT_HOST)
  pushSTACK(safe_to_string(utmpx->ut_host));
#else
  pushSTACK(NIL);
#endif
  pushSTACK(sec_usec_number(utmpx->ut_tv.tv_sec,utmpx->ut_tv.tv_usec,1));
  if (utmpx_o) {
    TheStructure(*utmpx_o)->recdata[7] = popSTACK(); /* tv */
    TheStructure(*utmpx_o)->recdata[6] = popSTACK(); /* host */
    TheStructure(*utmpx_o)->recdata[5] = popSTACK(); /* pid */
    TheStructure(*utmpx_o)->recdata[4] = popSTACK(); /* line */
    TheStructure(*utmpx_o)->recdata[3] = popSTACK(); /* id */
    TheStructure(*utmpx_o)->recdata[2] = popSTACK(); /* user */
    TheStructure(*utmpx_o)->recdata[1] = popSTACK(); /* type */
    VALUES1(*utmpx_o);
  } else funcall(O(object_posix__make_utmpx),7);
}
#if defined(HAVE_ENDUTXENT)
DEFUN(POSIX::ENDUTXENT,,(subr_posix_endutxent,seclass_default,0,0,norest,nokey,0,NIL)) {
  begin_blocking_system_call(); endutxent(); end_blocking_system_call();
  VALUES0;
}
#endif
#if defined(HAVE_GETUTXENT)
DEFUN(POSIX::GETUTXENT, &optional utmpx,(subr_posix_getutxent,seclass_default,0,1,norest,nokey,0,NIL)) {
  struct utmpx *utmpx;
  if (!missingp(STACK_0)) STACK_0 = check_classname(STACK_0,O(object_posix__utmpx));
  begin_blocking_system_call(); utmpx=getutxent(); end_blocking_system_call();
  if (utmpx) utmpx_to_lisp(utmpx,missingp(STACK_0) ? NULL : &STACK_0);
  else VALUES1(NIL);
  skipSTACK(1);
}
#endif
#if defined(HAVE_GETUTXID)
DEFUN(POSIX::GETUTXID, id,(subr_posix_getutxid,seclass_default,1,0,norest,nokey,0,NIL)) {
  struct utmpx utmpx, *utmpx_p;
  utmpx.ut_type = check_utmpx(&STACK_0);
  begin_blocking_system_call();
  utmpx_p = getutxid(&utmpx);
  end_blocking_system_call();
  if (utmpx_p) utmpx_to_lisp(utmpx_p,&STACK_0);
  else VALUES1(NIL);
  skipSTACK(1);
}
#endif
#if defined(HAVE_GETUTXLINE)
DEFUN(POSIX::GETUTXLINE, line,(subr_posix_getutxline,seclass_default,1,0,norest,nokey,0,NIL)) {
  struct utmpx utmpx, *utmpx_p;
  utmpx.ut_type = check_utmpx(&STACK_0);
  begin_blocking_system_call();
  utmpx_p = getutxline(&utmpx);
  end_blocking_system_call();
  if (utmpx_p) utmpx_to_lisp(utmpx_p,&STACK_0);
  else VALUES1(NIL);
  skipSTACK(1);
}
#endif
#if defined(HAVE_PUTUTXLINE)
DEFUN(POSIX::PUTUTXLINE, utmpx,(subr_posix_pututxline,seclass_default,1,0,norest,nokey,0,NIL)) {
  struct utmpx utmpx, *utmpx_p;
  utmpx.ut_type = check_utmpx(&STACK_0);
  begin_blocking_system_call();
  utmpx_p = pututxline(&utmpx);
  end_blocking_system_call();
  if (utmpx_p) utmpx_to_lisp(utmpx_p,&STACK_0);
  else ANSIC_error();
  skipSTACK(1);
}
#endif
#if defined(HAVE_SETUTXENT)
DEFUN(POSIX::SETUTXENT,,(subr_posix_setutxent,seclass_default,0,0,norest,nokey,0,NIL)) {
  begin_blocking_system_call(); setutxent(); end_blocking_system_call();
  VALUES0;
}
#endif
#endif  /* HAVE_UTMPX_H */

/* ========================= processes & signals ========================= */
#if defined(HAVE_GETPPID)
DEFUN(POSIX:GETPPID,,(subr_posix_getppid,seclass_default,0,0,norest,nokey,0,NIL)) { GETTER0(pid,getppid); }
#endif
#if defined(HAVE_GETSID)
DEFUN(POSIX:GETSID, pid,(subr_posix_getsid,seclass_default,1,0,norest,nokey,0,NIL)) { GETTER1(pid,getsid); }
#endif
#if defined(HAVE_SETSID)
DEFUN(POSIX:SETSID,,(subr_posix_setsid,seclass_default,0,0,norest,nokey,0,NIL)) { GETTER0(pid,setsid); } /* sic! */
#endif
#if defined(HAVE_GETPGRP)
DEFUN(POSIX:GETPGRP,,(subr_posix_getpgrp,seclass_default,0,0,norest,nokey,0,NIL)) { GETTER0(pid,getpgrp); }
#endif
#if defined(HAVE_SETPGRP)
DEFUN(POSIX:SETPGRP,,(subr_posix_setpgrp,seclass_default,0,0,norest,nokey,0,NIL)) {
  pid_t ret;
# if defined(HAVE_SETPGRP_POSIX)
  begin_system_call(); ret=setpgrp(); end_system_call();
# else  /* BSD version, identical to setpgid() */
  begin_system_call(); ret=setpgrp(0,0); end_system_call();
# endif
  if (ret==(pid_t)-1) ANSIC_error();
  VALUES1(pid_to_I(ret));
}
#endif
#if defined(HAVE_GETPGID)
DEFUN(POSIX:PGID, pid,(subr_posix_pgid,seclass_default,1,0,norest,nokey,0,NIL)) { GETTER1(pid,getpgid); }
#endif
#if defined(HAVE_SETPGID)
DEFUN(POSIX::%SETPGID, pid pgid,(subr_posix__25setpgid,seclass_default,2,0,norest,nokey,0,NIL)) {
  pid_t pgid = I_to_pid(STACK_0);
  pid_t pid = I_to_pid(STACK_1);
  int ret;
  begin_system_call(); ret=setpgid(pid,pgid); end_system_call();
  if (ret==-1) ANSIC_error();
  VALUES1(STACK_0); skipSTACK(2);
}
#endif
#if defined(HAVE_SETREUID)
DEFUN(POSIX:SETREUID, ruid euid,(subr_posix_setreuid,seclass_default,2,0,norest,nokey,0,NIL)) { SETTER2(uid,setreuid); }
#endif
#if defined(HAVE_SETREGID)
DEFUN(POSIX:SETREGID, rgid egid,(subr_posix_setregid,seclass_default,2,0,norest,nokey,0,NIL)) { SETTER2(gid,setregid); }
#endif
/* http://opengroup.org/onlinepubs/9699919799/basedefs/signal.h.html */
#line 852

#if defined(HAVE_KILL)
DEFUN(POSIX:KILL, pid sig,(subr_posix_kill,seclass_default,2,0,norest,nokey,0,NIL)) {
  int sig = check_signal(STACK_0);
  pid_t pid = I_to_pid(STACK_1);
  int ret;
  begin_system_call(); ret=kill(pid,sig); end_system_call();
  if (ret==-1) ANSIC_error();
  VALUES0; skipSTACK(2);
}
#endif  /* HAVE_KILL */

/* ============================= file sync ============================= */
#if defined(WIN32_NATIVE) || defined(HAVE_SYNC) || defined(HAVE_FSYNC)
DEFUN(POSIX:SYNC, &optional file,(subr_posix_sync,seclass_default,0,1,norest,nokey,0,NIL)) {
  if (missingp(STACK_0)) {      /* sync() */
#  if defined(HAVE_SYNC)
    begin_blocking_system_call(); sync(); end_blocking_system_call();
#  endif
  } else {                      /* fsync() */
    Handle fd = stream_get_handle(&STACK_0);
    bool failed_p;
    begin_blocking_system_call();
#  if defined(HAVE_FSYNC)
    failed_p = (-1 == fsync(fd));
#  elif defined(WIN32_NATIVE)
    failed_p = (!FlushFileBuffers(fd));
#  endif
    end_blocking_system_call();
    if (failed_p) error_OS_stream(STACK_0);
  }
  VALUES0; skipSTACK(1);
}
#endif
/* ========================== process priority ========================== */
#if defined(WIN32_NATIVE)
#line 889

#else
#line 893

#endif

DEFUN(OS:PRIORITY, pid &optional which,(subr_os_priority,seclass_default,1,1,norest,nokey,0,NIL)) {
  int which = check_priority_which(popSTACK());
  int pid = I_to_uint32(check_uint32(popSTACK()));
  int res;
  bool failed_p;
#if defined(HAVE_GETPRIORITY)
  begin_system_call();
  errno = 0;
  res = getpriority(which,pid);
  failed_p = (errno != 0);
  end_system_call();
#elif defined(WIN32_NATIVE)
  {
    HANDLE handle;
    begin_system_call();
    handle = OpenProcess(PROCESS_QUERY_INFORMATION,FALSE,pid);
    if (handle != INVALID_HANDLE_VALUE) {
      res = (int)GetPriorityClass(handle);
      failed_p = (res == 0);
      CloseHandle(handle);
    } else failed_p = true;
    end_system_call();
  }
#else
#  error OS:PRIORITY is not defined
#endif
  if (failed_p) OS_error();
  VALUES1(check_priority_value_reverse(res));
}
DEFUN(OS::%SET-PRIORITY, value pid which,(subr_os__25set_priority,seclass_default,3,0,norest,nokey,0,NIL)) {
  int which = check_priority_which(popSTACK());
  int pid = I_to_uint32(check_uint32(popSTACK()));
  int value = check_priority_value(STACK_0);
  bool failed_p = true;
#if defined(HAVE_SETPRIORITY)
  begin_system_call();
  failed_p = (0 != setpriority(which,pid,value));
  end_system_call();
#elif defined(WIN32_NATIVE)
  {
    HANDLE handle;
    begin_system_call();
    handle = OpenProcess(PROCESS_QUERY_INFORMATION,FALSE,pid);
    if (handle != INVALID_HANDLE_VALUE) {
      failed_p = !SetPriorityClass(handle,value);
      CloseHandle(handle);
    }
    end_system_call();
  }
#else
#  error OS::%SET-PRIORITY is not defined
#endif
  if (failed_p) OS_error();
  VALUES1(popSTACK());
}

/* posix math functions in <math.h> */
/* Must include <math.h> */
#define decimal_string  solaris_decimal_string  /* needed on Solaris */
#undef floor  /* needed on Linux */
#include <math.h>
#define floor(a,b)  ((a) / (b))
#undef decimal_string

#define D_S           to_double(popSTACK())
#define I_S           to_int(popSTACK())
#line 963
#define N_D(n,v)     { double x=n; v=c_double_to_DF((dfloatjanus*)&x); }
#define VAL_D(func)   double res=func(D_S); N_D(res,value1)
#line 966
#define VAL_ID(func)    double xx=D_S; int nn=I_S; double res=func(nn,xx); N_D(res,value1)

#if defined(HAVE_ERFC)
DEFUNF(POSIX::ERF,x,(subr_posix_erf,seclass_foldable,1,0,norest,nokey,0,NIL)) { VAL_D(erf); mv_count=1; }
#endif
#if defined(HAVE_ERFC)
DEFUNF(POSIX::ERFC,x,(subr_posix_erfc,seclass_foldable,1,0,norest,nokey,0,NIL)) { VAL_D(erfc); mv_count=1; }
#endif

DEFUNF(POSIX::J0,x,(subr_posix_j0,seclass_foldable,1,0,norest,nokey,0,NIL)) { VAL_D(j0); mv_count=1; }
DEFUNF(POSIX::J1,x,(subr_posix_j1,seclass_foldable,1,0,norest,nokey,0,NIL)) { VAL_D(j1); mv_count=1; }
DEFUNF(POSIX::JN,i x,(subr_posix_jn,seclass_foldable,2,0,norest,nokey,0,NIL)) { VAL_ID(jn); mv_count=1; }
DEFUNF(POSIX::Y0,x,(subr_posix_y0,seclass_foldable,1,0,norest,nokey,0,NIL)) { VAL_D(y0); mv_count=1; }
DEFUNF(POSIX::Y1,x,(subr_posix_y1,seclass_foldable,1,0,norest,nokey,0,NIL)) { VAL_D(y1); mv_count=1; }
DEFUNF(POSIX::YN,i y,(subr_posix_yn,seclass_foldable,2,0,norest,nokey,0,NIL)) { VAL_ID(yn); mv_count=1; }
#if defined(HAVE_TGAMMA)
/* http://opengroup.org/onlinepubs/9699919799/functions/tgamma.html */
DEFUNF(POSIX::TGAMMA,x,(subr_posix_tgamma,seclass_foldable,1,0,norest,nokey,0,NIL)) { VAL_D(tgamma); mv_count=1; }
#endif

#if defined(HAVE_LGAMMA) || HAVE_DECL_LGAMMA_R
/* http://opengroup.org/onlinepubs/9699919799/functions/lgamma.html */
DEFUNF(POSIX::LGAMMA,x,(subr_posix_lgamma,seclass_foldable,1,0,norest,nokey,0,NIL)) {
# if HAVE_DECL_LGAMMA_R
  int sign;
  double res = lgamma_r(D_S,&sign);
  value2 = (sign > 0 ? Fixnum_1 : Fixnum_minus1);
# else
  double res = lgamma(D_S);
# if HAVE_DECL_SIGNGAM
  value2 = (signgam > 0 ? Fixnum_1 : Fixnum_minus1);
# else
  value2 = NIL;
# endif
# endif
  N_D(res,value1); mv_count=2;
}
#endif

static double bogomips (void)
{
  if (clock() != (clock_t)-1) {
    unsigned long loops = 1;
    while ((loops <<= 1)) {
      unsigned long ticks;
      volatile unsigned long ii;
      ticks = clock();
      for (ii = loops; ii > 0; ii--);
      ticks = clock() - ticks;
      if (ticks >= CLOCKS_PER_SEC)
        return (1.0 * loops / ticks) * (CLOCKS_PER_SEC / 500000.0);
    }
  }
  return -1.0;
}
DEFUN(OS:BOGOMIPS,,(subr_os_bogomips,seclass_default,0,0,norest,nokey,0,NIL)) { N_D(bogomips(),value1); mv_count=1; }

DEFUN(POSIX:LOADAVG, &optional percentp,(subr_posix_loadavg,seclass_default,0,1,norest,nokey,0,NIL)) {
  double loadavg[3];
  int ret;
  begin_system_call();
  ret = getloadavg(loadavg,3);
  end_system_call();
  if (ret != 3) ANSIC_error();
  mv_count=3;
  if (missingp(STACK_0)) {
    N_D(loadavg[0],value1); pushSTACK(value1);
    N_D(loadavg[1],value1); pushSTACK(value1);
    N_D(loadavg[2],value3);
    value2 = popSTACK(); value1 = popSTACK();
  } else { /* return % as ints, to avoid consing */
    value1 = fixnum((int)(loadavg[0]*100+0.49999999999));
    value2 = fixnum((int)(loadavg[1]*100+0.49999999999));
    value3 = fixnum((int)(loadavg[2]*100+0.49999999999));
  }
  skipSTACK(1);
}

#undef D_S
#undef I_S
#undef N_D
#undef VAL_D
#undef VAL_ID

/* ========= SYSTEM INFORMATION ========== */
#include <sys/utsname.h>
DEFUN(POSIX::UNAME,,(subr_posix_uname,seclass_default,0,0,norest,nokey,0,NIL))
{ /* Lisp interface to uname(2) */
  struct utsname utsname;
  begin_system_call(); uname(&utsname); end_system_call();
  pushSTACK(safe_to_string(utsname.sysname));
  pushSTACK(safe_to_string(utsname.nodename));
  pushSTACK(safe_to_string(utsname.release));
  pushSTACK(safe_to_string(utsname.version));
  pushSTACK(safe_to_string(utsname.machine));
  funcall(O(object_posix__make_uname),5);
}

#if defined(HAVE_SYSCONF)
#line 1097

DEFUN(POSIX::SYSCONF, &optional what,(subr_posix_sysconf,seclass_default,0,1,norest,nokey,0,NIL))
{ /* Lisp interface to sysconf(3c) */
  object what = popSTACK();
  if (!missingp(what)) {
    int cmd = sysconf_arg(what), res;
    begin_system_call(); res = sysconf(cmd); end_system_call();
    VALUES1(L_to_I(res));
  } else { /* all possible values */
    unsigned int pos = 0;
    for (; pos < sysconf_arg_map.size; pos++) {
      int res;
      begin_system_call();
      res = sysconf(sysconf_arg_map.table[pos].c_const);
      end_system_call();
      pushSTACK(*sysconf_arg_map.table[pos].l_const);
      pushSTACK(L_to_I(res));
    }
    VALUES1(listof(2*sysconf_arg_map.size));
  }
}
#endif /* HAVE_SYSCONF */

#if defined(HAVE_CONFSTR)
#line 1143

DEFUN(POSIX::CONFSTR, &optional what,(subr_posix_confstr,seclass_default,0,1,norest,nokey,0,NIL))
{ /* Lisp interface to confstr(3c) */
#line 1162
#define CS_S(cmd)    begin_system_call(); res = confstr(cmd,buf,BUFSIZ); end_system_call();    if (res == 0) value1 = T;                                                else if (res <= BUFSIZ) value1 = asciz_to_string(buf,GLO(misc_encoding));    else {                                                                     /* Here we cannot use alloca(), because alloca() is generally unsafe         for sizes > BUFSIZ. */                                                char *tmp = (char*)clisp_malloc(res);                                    begin_system_call();                                                     confstr(cmd,tmp,res);                                                    end_system_call();                                                       /* FIXME: asciz_to_string may signal an error in which case tmp leaks */      value1 = asciz_to_string(tmp,GLO(misc_encoding));                        begin_system_call();                                                     free(tmp);                                                               end_system_call();                                                     }

  size_t res;
  char buf[BUFSIZ];
  object what = popSTACK();
  if (!missingp(what)) {
    int cmd = confstr_arg(what);
    CS_S(cmd); mv_count = 1;
  } else { /* all possible values */
    unsigned int pos = 0;
    for (; pos < confstr_arg_map.size; pos++) {
      CS_S(confstr_arg_map.table[pos].c_const);
      pushSTACK(*confstr_arg_map.table[pos].l_const);
      pushSTACK(value1);
    }
    VALUES1(listof(2*confstr_arg_map.size));
  }
}
#endif /* HAVE_CONFSTR */

#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
#line 1187

#line 1206
#define DO_PATHCONF(f,spec,what)                                   if (missingp(what)) { /* all possible values */                    unsigned int pos = 0;                                            for (; pos < pathconf_arg_map.size; pos++) {                       long res;                                                        begin_system_call();                                             res = f(spec,pathconf_arg_map.table[pos].c_const);               end_system_call();                                               pushSTACK(*pathconf_arg_map.table[pos].l_const);                 pushSTACK(res == -1 ? S(Kerror) : L_to_I(res));                }                                                                VALUES1(listof(2*pathconf_arg_map.size));                      } else {                                                           long res;                                                        begin_system_call();                                             if ((res = f(spec,pathconf_arg(what))) == -1) ANSIC_error();     end_system_call();                                               VALUES1(L_to_I(res));                                          }
DEFUN(POSIX::PATHCONF, pathspec &optional what,(subr_posix_pathconf,seclass_default,1,1,norest,nokey,0,NIL))
{ /* http://opengroup.org/onlinepubs/9699919799/functions/pathconf.html */
  Handle fd;
  if (integerp(STACK_1)) {
    fd = I_to_UL(STACK_1);
   pathconf_fd: DO_PATHCONF(fpathconf,fd,STACK_0);
  } else {
    object file = open_file_stream_handle(STACK_1,&fd,true);
    if (eq(nullobj,file)) {    /* not an open stream ==> use truename */
      with_string_0(STACK_1 = physical_namestring(STACK_1),
                    GLO(pathname_encoding), namez,
          { DO_PATHCONF(pathconf,namez,STACK_0); });
    } else goto pathconf_fd;       /* open stream ==> use fd */
  }
  skipSTACK(2);
}
#endif  /* HAVE_PATHCONF && HAVE_FPATHCONF */

#if defined(HAVE_CHROOT)
DEFUN(POSIX::CHROOT, path,(subr_posix_chroot,seclass_default,1,0,norest,nokey,0,NIL))
{ /* http://opengroup.org/onlinepubs/007908799/xsh/chroot.html (LEGACY) */
  int status;
  STACK_0 = physical_namestring(STACK_0);
  with_string_0(STACK_0, GLO(pathname_encoding), namez, {
      begin_blocking_system_call();
      status = chroot(namez);
      end_blocking_system_call();
    });
  if (status) OS_file_error(STACK_0);
  skipSTACK(1); VALUES0;
}
#endif  /* HAVE_CHROOT */

#if defined(HAVE_SYS_RESOURCE_H)
static /*maygc*/ Values rusage_to_lisp (struct rusage *ru) {
  int count = 2;
  pushSTACK(sec_usec_number(ru->ru_utime.tv_sec,ru->ru_utime.tv_usec,0));
  pushSTACK(sec_usec_number(ru->ru_stime.tv_sec,ru->ru_stime.tv_usec,0));
  pushSTACK(L_to_I(ru->ru_maxrss)); count++;
  pushSTACK(L_to_I(ru->ru_ixrss)); count++;
  pushSTACK(L_to_I(ru->ru_idrss)); count++;
  pushSTACK(L_to_I(ru->ru_isrss)); count++;
  pushSTACK(L_to_I(ru->ru_minflt)); count++;
  pushSTACK(L_to_I(ru->ru_majflt)); count++;
  pushSTACK(L_to_I(ru->ru_nswap)); count++;
  pushSTACK(L_to_I(ru->ru_inblock)); count++;
  pushSTACK(L_to_I(ru->ru_oublock)); count++;
  pushSTACK(L_to_I(ru->ru_msgsnd)); count++;
  pushSTACK(L_to_I(ru->ru_msgrcv)); count++;
  pushSTACK(L_to_I(ru->ru_nsignals)); count++;
  pushSTACK(L_to_I(ru->ru_nvcsw)); count++;
  pushSTACK(L_to_I(ru->ru_nivcsw)); count++;
  funcall(O(object_posix__make_usage),count);
}

#if !defined(HAVE_WAIT4)
#  define wait4(p,s,o,r)  (errno=ENOSYS,ANSIC_error(),(pid_t)-1)
#endif

#line 1267
DEFUN(POSIX::WAIT, &key :PID :USAGE :NOHANG :UNTRACED :STOPPED :EXITED        :CONTINUED :NOWAIT,(subr_posix_wait,seclass_default,0,0,norest,key,8,NIL)) {
  int status, options = wait_flags();
  bool usage = !missingp(STACK_0);
  pid_t ret, pid = missingp(STACK_1) ? (pid_t)-1 : I_to_pid(STACK_1);
  struct rusage ru;
  begin_blocking_system_call();
  begin_want_sigcld();
  ret = usage ? wait4(pid,&status,options,&ru) : waitpid(pid,&status,options);
  end_want_sigcld();
  end_blocking_system_call();
  if (ret == (pid_t)-1) ANSIC_error();
  if (ret == (pid_t)0 && (options & WNOHANG))
    VALUES1(Fixnum_0);          /* no process changed status */
  else {                        /* some process changed status */
    if (usage) {
      rusage_to_lisp(&ru);
      STACK_0 = value1;
      mv_count = 4;
    } else mv_count = 3;
    STACK_1 = pid_to_I(ret);
    if (WIFEXITED(status)) {
      value2 = O(object_Kexited);
      value3 = fixnum(WEXITSTATUS(status));
    } else if (WIFSIGNALED(status)) {
      value3 = check_signal_reverse(WTERMSIG(status));
      value2 = O(object_Ksignaled);
    } else if (WIFSTOPPED(status)) {
      value3 = check_signal_reverse(WSTOPSIG(status));
      value2 = O(object_Kstopped);
#  if defined(WIFCONTINUED)       /* cygwin does not have this */
    } else if (WIFCONTINUED(status)) {
      value2 = O(object_Kcontinued);
      value3 = NIL;
#  endif
    } else {
      value2 = NIL;
      value3 = fixnum(status);
    }
    value1 = STACK_1;
    if (usage) value4 = STACK_0;
  }
  skipSTACK(2);
}

/* https://sourceforge.net/p/clisp/mailman/message/23010926/
   https://sourceforge.net/p/clisp/bugs/593/ */
DEFUN(POSIX::BEGIN-SUBPROCESSES,,(subr_posix_begin_subprocesses,seclass_default,0,0,norest,nokey,0,NIL)) {
    begin_system_call();
    begin_want_sigcld();
    end_system_call();
    VALUES0;
}
DEFUN(POSIX::END-SUBPROCESSES,,(subr_posix_end_subprocesses,seclass_default,0,0,norest,nokey,0,NIL)) {
    begin_system_call();
    end_want_sigcld();
    end_system_call();
    VALUES0;
}

#if defined(HAVE_GETRUSAGE)

DEFUN(POSIX::USAGE, &optional what,(subr_posix_usage,seclass_default,0,1,norest,nokey,0,NIL)) { /* getrusage(3) */
  struct rusage ru;
  object what = popSTACK();
  if (missingp(what)) {
    unsigned int pos;
    for (pos = 0; pos < check_rusage_map.size; pos++) {
      int status;
      pushSTACK(*check_rusage_map.table[pos].l_const);
      begin_system_call();
      status = getrusage(check_rusage_map.table[pos].c_const,&ru);
      end_system_call();
      if (status) pushSTACK(S(Kerror));
      else { rusage_to_lisp(&ru); pushSTACK(value1); }
    }
    VALUES1(listof(2*check_rusage_map.size));
  } else {
    int who = check_rusage(what);
    begin_system_call();
    if (getrusage(who,&ru)) ANSIC_error();
    end_system_call();
    rusage_to_lisp(&ru);
  }
}
#endif /* HAVE_GETRUSAGE */
#endif /* HAVE_SYS_RESOURCE_H */

#if defined(HAVE_GETRLIMIT) || defined(HAVE_SETRLIMIT)
#line 1356

#if SIZEOF_RLIM_T == 8
# define rlim_to_I_0(lim) uint64_to_I(lim)
# define I_to_rlim_0(lim) I_to_uint64(check_uint64(lim))
#else
# define rlim_to_I_0(lim) uint32_to_I(lim)
# define I_to_rlim_0(lim) I_to_uint32(check_uint32(lim))
#endif
static /* maygc */ inline object rlim_to_I (rlim_t lim)
{ return lim == RLIM_INFINITY ? NIL : rlim_to_I_0(lim); }
static /* maygc */ inline rlim_t I_to_rlim (object lim)
{ return missingp(lim) ? RLIM_INFINITY : I_to_rlim_0(lim); }
#endif /* HAVE_GETRLIMIT || HAVE_SETRLIMIT */
#if defined(HAVE_GETRLIMIT)
DEFUN(POSIX::RLIMIT, &optional what,(subr_posix_rlimit,seclass_default,0,1,norest,nokey,0,NIL))
{ /* getrlimit(3) */
  struct rlimit rl;
  object what = popSTACK();
  if (!missingp(what)) {
    int cmd = getrlimit_arg(what);
    begin_system_call();
    if (getrlimit(cmd,&rl)) ANSIC_error();
    end_system_call();
    pushSTACK(rlim_to_I(rl.rlim_cur)); pushSTACK(rlim_to_I(rl.rlim_max));
    VALUES2(STACK_1,STACK_0); skipSTACK(2);
  } else {
    unsigned int pos;
    for (pos = 0; pos < getrlimit_arg_map.size; pos++) {
      int status;
      pushSTACK(*getrlimit_arg_map.table[pos].l_const);
      begin_system_call();
      status = getrlimit(getrlimit_arg_map.table[pos].c_const,&rl);
      end_system_call();
      if (status) pushSTACK(S(Kerror));
      else {
        pushSTACK(rlim_to_I(rl.rlim_cur)); pushSTACK(rlim_to_I(rl.rlim_max));
        funcall(O(object_posix__make_rlimit),2); pushSTACK(value1);
      }
    }
    VALUES1(listof(2*getrlimit_arg_map.size));
  }
}
#endif /* HAVE_GETRLIMIT */
#if defined(HAVE_SETRLIMIT)
/* parse the RLIMIT structure
   NOTE: arg is intentionally not reset by check_classname
   to avoid argument modification
 can trigger GC */
static void check_rlimit (object arg, struct rlimit *rl) {
  pushSTACK(check_classname(arg,O(object_posix__rlimit)));
  rl->rlim_cur = I_to_rlim(TheStructure(STACK_0)->recdata[1]);
  rl->rlim_max = I_to_rlim(TheStructure(STACK_0)->recdata[2]);
  skipSTACK(1);
}
DEFUN(POSIX::SET-RLIMIT, what cur max,(subr_posix_set_rlimit,seclass_default,3,0,norest,nokey,0,NIL))
{ /* setrlimit(3): 3 ways to call:
   (setf (rlimit what) (values cur max))
   (setf (rlimit what) #S(rlimit :cur cur :max max))
   (setf (rlimit) rlimit-plist-as-returned-by-rlimit-without-arguments) */
  if (nullp(STACK_2)) {         /* 3rd way */
    if (!nullp(STACK_0)) goto rlimit_bad;
    STACK_0 = STACK_1;
    while (!endp(STACK_0)) {
      int what = getrlimit_arg(Car(STACK_0));
      struct rlimit rl;
      STACK_0 = Cdr(STACK_0);
      if (!consp(STACK_0)) { STACK_0 = NIL; goto rlimit_bad; }
      check_rlimit(Car(STACK_0),&rl);
      STACK_0 = Cdr(STACK_0);
      begin_system_call();
      if (setrlimit(what,&rl)) ANSIC_error();
      end_system_call();
    }
  } else {
    int what = getrlimit_arg(STACK_2);
    struct rlimit rl;
    if (nullp(STACK_1) || posfixnump(STACK_1)) { /* 1st way */
      rl.rlim_cur = I_to_rlim(STACK_1);
      rl.rlim_max = I_to_rlim(STACK_0);
    } else {                    /* 2nd way */
      if (!nullp(STACK_0)) goto rlimit_bad;
      check_rlimit(STACK_1,&rl);
    }
    begin_system_call();
    if (setrlimit(what,&rl)) ANSIC_error();
    end_system_call();
  }
  VALUES2(STACK_1,STACK_0); skipSTACK(3); return;
 rlimit_bad:
  pushSTACK(TheSubr(subr_self)->name);
  error(error_condition,GETTEXT("~S: bad arguments: ~S ~S ~S"));
}
#endif /* HAVE_SETRLIMIT */

/* ==== SOCKETS ===== */
#if defined(HAVE_NETDB_H)
# include <netdb.h>
#endif
#include <netinet/in.h>
#include <arpa/inet.h>

#line 1463
#define H_ERRMSG                                                                    (h_errno == HOST_NOT_FOUND ? "host not found" :                              (h_errno == TRY_AGAIN ? "try again later" :                                  (h_errno == NO_RECOVERY ? "a non-recoverable error occurred" :               (h_errno == NO_DATA ? "valid name, but no data for this host" :              (h_errno == NO_ADDRESS ? "no IP address for this host" :                     "unknown error")))))

#if 0
void print_he (struct hostent *he) {
  int ii;
  char **pp;
  struct in_addr in;
  printf("h_name: %s; h_length: %d; h_addrtype: %d\n [size in.s_addr: %d]\n",
         he->h_name,he->h_length,he->h_addrtype,sizeof(in.s_addr));
  for (pp = he->h_aliases; *pp != 0; pp++) printf("\t%s", *pp);
  printf("\n IP:");
  for (pp = he->h_addr_list; *pp != 0; pp++) {
    (void) memcpy(&in.s_addr, *pp, sizeof (in.s_addr));
    (void) printf("\t%s", inet_ntoa(in));
  }
  printf("\n");
}
#endif

/* C struct hostent --> Lisp HOSTENT structure
 can trigger GC */
Values hostent_to_lisp (struct hostent *he); /* used by NEW-CLX => not static */
Values hostent_to_lisp (struct hostent *he) {
  pushSTACK(ascii_to_string(he->h_name));
  push_string_array(he->h_aliases);
  { int ii = 0;
    for (; he->h_addr_list[ii]; ii++)
      pushSTACK(addr_to_string(he->h_addrtype,he->h_addr_list[ii]));
    { object tmp = listof(ii); pushSTACK(tmp); }}
  pushSTACK(fixnum(he->h_addrtype));
  funcall(O(object_posix__make_hostent),4);
}

DEFUN(POSIX::RESOLVE-HOST-IPADDR,&optional host,(subr_posix_resolve_host_ipaddr,seclass_default,0,1,norest,nokey,0,NIL))
{ /* Lisp interface to gethostbyname(3) and gethostbyaddr(3) */
  object arg = popSTACK();
  struct hostent *he = NULL;

  if (missingp(arg)) {
#  if !defined(HAVE_GETHOSTENT)
    VALUES1(NIL);
#  else
    int count = 0;
    begin_system_call();
    sethostent(1);
    for (; (he = gethostent()); count++) {
      end_system_call();
      hostent_to_lisp(he); pushSTACK(value1);
      begin_system_call();
    }
    endhostent();
    end_system_call();
    VALUES1(listof(count));
#  endif
    return;
  }

  he = resolve_host(arg);

  if (he == NULL) {
    pushSTACK(arg); pushSTACK(arg);
    STACK_1 = ascii_to_string(H_ERRMSG);
    pushSTACK(O(object_posix__resolve_host_ipaddr));
    error(error_condition,"~S (~S): ~S");
  }

  hostent_to_lisp(he);
}

#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
/* Lisp interface to getservbyport(3) and getservbyname(3) */

/* C struct servent --> Lisp SERVICE structure
 can trigger GC */
static Values servent_to_lisp (struct servent * se) {
  pushSTACK(safe_to_string(se->s_name));
  push_string_array(se->s_aliases);
  pushSTACK(L_to_I(ntohs(se->s_port)));
  pushSTACK(safe_to_string(se->s_proto));
  funcall(O(object_posix__make_service),4);
}

DEFUN(POSIX:SERVICE, &optional service-name protocol,(subr_posix_service,seclass_default,0,2,norest,nokey,0,NIL)) {
  object protocol = popSTACK();
  char *proto = NULL;
  char proto_buf[16];
  object serv;
  struct servent * se;
  if (!missingp(protocol)) {    /* check protocol */
    protocol = check_string(protocol);
    with_string_0(protocol,GLO(misc_encoding), protocolz, {
        begin_system_call();
        strncpy(proto_buf,protocolz,15);
        end_system_call();
      });
    proto = proto_buf;
    proto_buf[15] = 0;
  }
  serv = popSTACK();
  if (missingp(serv)) {
    uintL count = 0;
#  if defined(HAVE_SETSERVENT) && defined(HAVE_GETSERVENT) && defined(HAVE_ENDSERVENT)
    begin_system_call();
    setservent(1);
    while ((se = getservent()))
      if (proto==NULL || (se->s_proto && !strcmp(proto,se->s_proto))) {
        end_system_call();
        servent_to_lisp(se); pushSTACK(value1); count++;
        begin_system_call();
      }
    endservent();
    end_system_call();
#  else /* no getservent - emulate */
    uintL port;
    begin_system_call();
    for (port = 0; port < 0x10000; port++) {
      se = getservbyport(port,proto);
      if (se != NULL) {
        end_system_call();
        servent_to_lisp(se); pushSTACK(value1); count++;
        begin_system_call();
      }
    }
    end_system_call();
#  endif
    VALUES1(listof(count));
    return;
  } else if (symbolp(serv)) {
    serv = Symbol_name(serv);
    goto servent_string;
  } else if (stringp(serv)) { servent_string:
    with_string_0(serv,GLO(misc_encoding),servz, {
        begin_system_call();
        se = getservbyname(servz,proto);
        end_system_call();
      });
  } else if (integerp(serv)) {
    uintL port = I_to_UL(serv);
    begin_system_call();
    se = getservbyport(htons(port),proto);
    end_system_call();
  } else
    error_string_integer(serv);
  if (se == NULL) ANSIC_error();
  servent_to_lisp(se);
}

#endif /* getservbyname getservbyport */

#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)

#if defined(HAVE_GRP_H)
# include <grp.h>
#endif

/* C struct group --> Lisp GROUP-INFO structure
 can trigger GC */
static Values grp_to_lisp (struct group *group) {
  pushSTACK(safe_to_string(group->gr_name));
  pushSTACK(gid_to_I(group->gr_gid));
  push_string_array(group->gr_mem);
  funcall(O(object_posix__make_group_info),3);
}

DEFUN(POSIX::GROUP-INFO, &optional group,(subr_posix_group_info,seclass_default,0,1,norest,nokey,0,NIL))
{ /* return the GROUP-INFO for the group or a list thereof if it is NIL. */
  object group = popSTACK();
  struct group *gr = NULL;
  bool failed_p;
 group_info_restart:

# if defined(HAVE_GETGRENT) && defined(HAVE_SETGRENT) && defined(HAVE_ENDGRENT)
  if (missingp(group)) { /* all groups as a list */
    int count = 0;
    begin_system_call();
    setgrent();
    for (; (gr = getgrent()); count++) {
      end_system_call();
      grp_to_lisp(gr); pushSTACK(value1);
      begin_system_call();
    }
    endgrent();
    end_system_call();
    VALUES1(listof(count));
    return;
  }
# endif  /* setgrent getgrent endgrent */

  begin_system_call();
  errno = 0;
  if (integerp(group))
    gr = getgrgid(I_to_gid(group));
  else if (symbolp(group)) {
    group = Symbol_name(group);
    goto group_info_string;
  } else if (stringp(group)) { group_info_string:
    with_string_0(group,GLO(misc_encoding),groupz, { gr = getgrnam(groupz); });
  } else {
    end_system_call(); error_string_integer(group);
  }
  failed_p = (errno != 0);
  end_system_call();

  if (NULL == gr) {
    if (!failed_p) {
      pushSTACK(NIL);           /* no PLACE */
      pushSTACK(group); pushSTACK(TheSubr(subr_self)->name);
      check_value(error_condition,GETTEXT("~S(~S): No such group"));
      group = value1;
      goto group_info_restart;
    } else ANSIC_error();
  }
  grp_to_lisp(gr);
}
#endif  /* getgrgid getgrnam */

#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)

#if defined(HAVE_PWD_H)
# include <pwd.h>
#endif

/* C struct passwd --> Lisp USER-INFO structure
 can trigger GC */
static Values passwd_to_lisp (struct passwd *pwd) {
  pushSTACK(safe_to_string(pwd->pw_name));
  pushSTACK(safe_to_string(pwd->pw_passwd));
  pushSTACK(UL_to_I(pwd->pw_uid));
  pushSTACK(UL_to_I(pwd->pw_gid));
  pushSTACK(safe_to_string(pwd->pw_gecos));
  pushSTACK(safe_to_string(pwd->pw_dir));
  pushSTACK(safe_to_string(pwd->pw_shell));
  funcall(O(object_posix__make_user_info),7);
}

DEFUN(POSIX::USER-INFO, &optional user,(subr_posix_user_info,seclass_default,0,1,norest,nokey,0,NIL))
{ /* return the USER-INFO for the user or a list thereof if user is NIL. */
  object user = popSTACK();
  struct passwd *pwd = NULL;
  bool failed_p;
 user_info_restart:

# if defined(HAVE_GETPWENT) && defined(HAVE_SETPWENT) && defined(HAVE_ENDPWENT)
  if (missingp(user)) { /* all users as a list */
    int count = 0;
    begin_system_call();
    setpwent();
    for (; (pwd = getpwent()); ) {
      /* on cygwin uid of -1 is returned */
      /* when user has no entry in /etc/passwd */
      if (pwd->pw_uid == (uid_t) -1) continue;
      end_system_call();
      passwd_to_lisp(pwd); pushSTACK(value1);
      begin_system_call();
      count++;
    }
    endpwent();
    end_system_call();
    VALUES1(listof(count));
    return;
  }
# endif  /* setpwent getpwent endpwent */

  begin_system_call();
  errno = 0;
  if (integerp(user))
    pwd = getpwuid(I_to_uid(user));
  else if (eq(user,S(Kdefault))) {
    char *username = getlogin();
    if (username != NULL)
      pwd = getpwnam(username);
    else pwd = getpwuid(getuid());
  } else if (symbolp(user)) {
    user = Symbol_name(user);
    goto user_info_string;
  } else if (stringp(user)) { user_info_string:
    with_string_0(user,GLO(misc_encoding),userz, { pwd = getpwnam(userz); });
  } else {
    end_system_call(); error_string_integer(user);
  }
  failed_p = (errno != 0);
  end_system_call();

  if (NULL == pwd) {
    if (!failed_p) {
      pushSTACK(NIL);           /* no PLACE */
      pushSTACK(user); pushSTACK(TheSubr(subr_self)->name);
      check_value(error_condition,GETTEXT("~S(~S): No such user"));
      user = value1;
      goto user_info_restart;
    } else ANSIC_error();
  }
  passwd_to_lisp(pwd);
}
#elif defined(WIN32_NATIVE)
/* FIXME: use
 http://msdn.microsoft.com/library/en-us/netmgmt/netmgmt/user_info_1_str.asp
 http://msdn.microsoft.com/library/en-us/netmgmt/netmgmt/netusergetinfo.asp
 http://msdn.microsoft.com/library/en-us/netmgmt/netmgmt/netuserenum.asp */
#endif  /* user-info */

#if defined(HAVE_GETUSERSHELL) && defined(HAVE_ENDUSERSHELL)
DEFUN(POSIX:USER-SHELLS,,(subr_posix_user_shells,seclass_default,0,0,norest,nokey,0,NIL)) {
  int count = 0;
  char *shell;
  begin_system_call();
  for (;(shell = getusershell()); count++) {
    end_system_call();
    pushSTACK(asciz_to_string(shell,GLO(misc_encoding)));
    begin_system_call();
  }
  endusershell();
  end_system_call();
  VALUES1(listof(count));
}
#endif  /* HAVE_GETUSERSHELL & HAVE_ENDUSERSHELL */

#if defined(HAVE_GETUID)
DEFUN(POSIX:UID,,(subr_posix_uid,seclass_default,0,0,norest,nokey,0,NIL)){ GETTER0(uid,getuid); }
#endif
#if defined(HAVE_SETUID)
DEFUN(POSIX::%SETUID, uid,(subr_posix__25setuid,seclass_default,1,0,norest,nokey,0,NIL)) { SETTER1(uid,setuid); }
#endif
#if defined(HAVE_GETGID)
DEFUN(POSIX:GID,,(subr_posix_gid,seclass_default,0,0,norest,nokey,0,NIL)){ GETTER0(gid,getgid); }
#endif
#if defined(HAVE_SETGID)
DEFUN(POSIX::%SETGID, gid,(subr_posix__25setgid,seclass_default,1,0,norest,nokey,0,NIL)) { SETTER1(gid,setgid); }
#endif
#if defined(HAVE_GETEUID)
DEFUN(POSIX:EUID,,(subr_posix_euid,seclass_default,0,0,norest,nokey,0,NIL)){ GETTER0(uid,geteuid); }
#endif
#if defined(HAVE_SETEUID)
DEFUN(POSIX::%SETEUID, euid,(subr_posix__25seteuid,seclass_default,1,0,norest,nokey,0,NIL)) { SETTER1(uid,seteuid); }
#endif
#if defined(HAVE_GETEGID)
DEFUN(POSIX:EGID,,(subr_posix_egid,seclass_default,0,0,norest,nokey,0,NIL)){ GETTER0(gid,getegid); }
#endif
#if defined(HAVE_SETEGID)
DEFUN(POSIX::%SETEGID, egid,(subr_posix__25setegid,seclass_default,1,0,norest,nokey,0,NIL)) { SETTER1(gid,setegid); }
#endif
#if defined(HAVE_GETGROUPS)
DEFUN(POSIX:GROUPS,,(subr_posix_groups,seclass_default,0,0,norest,nokey,0,NIL)) {
  int group_count, ret;
  gid_t *groups;
  begin_system_call(); group_count = getgroups(0,NULL); end_system_call();
  groups = (gid_t*)alloca(sizeof(gid_t) * group_count);
  begin_system_call(); ret = getgroups(group_count,groups); end_system_call();
  if (ret == -1) ANSIC_error();
  while (ret--) pushSTACK(gid_to_I(*groups++));
  VALUES1(listof(group_count));
}
#endif
#if defined(HAVE_SETGROUPS)
DEFUN(POSIX::%SETGROUPS, groups,(subr_posix__25setgroups,seclass_default,1,0,norest,nokey,0,NIL)) {
  int group_count = llength1(STACK_0,NULL), i = group_count;
  gid_t *groups = (gid_t*)alloca(sizeof(gid_t) * group_count), *pgrp = groups;
  pushSTACK(STACK_0);
  while (i--) {
    *pgrp++ = I_to_gid(Car(STACK_0));
    STACK_0 = Cdr(STACK_0);
  }
  if (!nullp(popSTACK())) NOTREACHED;
  begin_system_call(); i = setgroups(group_count,groups); end_system_call();
  if (i == -1) ANSIC_error();
  VALUES1(popSTACK());
}
#endif
#if defined(HAVE_GETHOSTID)
/* http://opengroup.org/onlinepubs/9699919799/functions/gethostid.html */
/* this is returned as an integer, not as a string,
   because this is NOT the IP address:
   (posix:gethostid) ==> 430729603
   (rawsock:convert-address :inet 430729603) ==>  "131.105.172.25"
   (rawsock:htonl 430729603) ==> 2204740633
   (rawsock:convert-address :inet 2204740633) ==> "25.172.105.131"
   but (rawsock:resolve-host-ipaddr :default) ==> "172.25.131.105" */
DEFUN(POSIX:HOSTID,,(subr_posix_hostid,seclass_default,0,0,norest,nokey,0,NIL)) { GETTER(unsigned long,ulong,gethostid); }
#endif
#if defined(HAVE_SETHOSTID) && !defined(_AIX)
/* sethostid is not appropriately declared in the system header files on AIX 7. */
#define I_to_hid(x)   I_to_ulong(check_ulong(x))
DEFUN(POSIX::%SETHOSTID, hostid,(subr_posix__25sethostid,seclass_default,1,0,norest,nokey,0,NIL)) {
  unsigned long hid = I_to_ulong(check_ulong(STACK_0 = STACK_0));
  int e;
  begin_system_call(); errno = 0; sethostid(hid); e = errno; end_system_call();
  if (e) ANSIC_error();
  VALUES1(popSTACK());
}
#endif
#ifndef MAXHOSTNAMELEN          /* see unix.d */
# define MAXHOSTNAMELEN 256     /* see <sys/param.h> */
#endif
#if defined(HAVE_GETDOMAINNAME)
DEFUN(POSIX:DOMAINNAME,,(subr_posix_domainname,seclass_default,0,0,norest,nokey,0,NIL)) {
  char domain[MAXHOSTNAMELEN];
  int e;
  begin_system_call();
  e = getdomainname(domain,MAXHOSTNAMELEN);
  end_system_call();
  if (e) ANSIC_error();
  VALUES1(asciz_to_string(domain,GLO(misc_encoding)));
}
#endif
#if defined(HAVE_SETDOMAINNAME)
DEFUN(POSIX::%SETDOMAINNAME, domain,(subr_posix__25setdomainname,seclass_default,1,0,norest,nokey,0,NIL)) {
  int e;
  with_string_0(STACK_0 = check_string(STACK_0),GLO(misc_encoding),domain, {
      begin_system_call();
      e = setdomainname(domain,domain_len);
      end_system_call();
    });
  if (e) ANSIC_error();
  VALUES1(popSTACK());    /* return the argument for the sake of SETF */
}
#endif

#if defined(HAVE_FSTAT) && defined(HAVE_STAT)
static void file_stat_to_STACK (object file, const struct stat *ps) {
  pushSTACK(file);                    /* the object stat'ed */
  pushSTACK(L_to_I(ps->st_dev));      /* device */
#if defined(SIZEOF_INO_T) && SIZEOF_INO_T == 8
  pushSTACK(uint64_to_I(ps->st_ino)); /* inode */
#else
  pushSTACK(uint32_to_I(ps->st_ino)); /* inode */
#endif
#ifdef S_IFMT
  { /* protection & format */
    unsigned int fmt = ps->st_mode & S_IFMT;
    if (fmt) {
      pushSTACK(allocate_cons());
      Car(STACK_0) = mknod_type_check_reverse(fmt);
      Cdr(STACK_0) = check_chmod_mode_to_list(ps->st_mode & ~S_IFMT);
    } else pushSTACK(check_chmod_mode_to_list(ps->st_mode));
  }
#else
  pushSTACK(check_chmod_mode_to_list(ps->st_mode)); /* protection */
#endif
  pushSTACK(UL_to_I(ps->st_nlink));   /* number of hard links */
  pushSTACK(UL_to_I(ps->st_uid));     /* user ID of owner */
  pushSTACK(UL_to_I(ps->st_gid));     /* group ID of owner */
#if defined(HAVE_STAT_ST_RDEV)
  pushSTACK(L_to_I(ps->st_rdev));     /* device type (if inode device) */
#else
  pushSTACK(NIL);
#endif
  pushSTACK(off_to_I(ps->st_size)); /* total size, in bytes */
#if defined(HAVE_STAT_ST_BLKSIZE)
  pushSTACK(UL_to_I(ps->st_blksize)); /* blocksize for filesystem I/O */
#else
  pushSTACK(NIL);
#endif
#if defined(HAVE_STAT_ST_BLOCKS)
  pushSTACK(UL_to_I(ps->st_blocks));  /* number of blocks allocated */
#else
  pushSTACK(NIL);
#endif
  /* cannot use convert_time_to_universal() because this is used on win32 */
  pushSTACK(UL_to_I(ps->st_atime+UNIX_LISP_TIME_DIFF));/*time of last access*/
  pushSTACK(UL_to_I(ps->st_mtime+UNIX_LISP_TIME_DIFF));/*last modification*/
  pushSTACK(UL_to_I(ps->st_ctime+UNIX_LISP_TIME_DIFF));/*time of last change*/
}

DEFUN(POSIX::FILE-STAT, file &optional linkp,(subr_posix_file_stat,seclass_default,1,1,norest,nokey,0,NIL))
{ /* Lisp interface to stat(2), lstat(2) and fstat(2)
 the first arg can be a pathname designator or a file descriptor designator
 the return value is the FILE-STAT structure */
  bool link_p = missingp(STACK_0);
  struct stat buf;
  object file = STACK_1;
  bool error_p;

  if (integerp(file)) {
    begin_blocking_system_call();
    error_p = (fstat(I_to_UL(file),&buf) < 0);
    end_blocking_system_call();
    if (error_p) ANSIC_error();
  } else {
    Handle fd;
    file = open_file_stream_handle(STACK_1,&fd,true);
    if (eq(nullobj,file)) {     /* not a stream - treat as a pathname */
      if (ON_PNAMESTRING(STACK_1,*(link_p ? &stat : &lstat),&buf))
        OS_file_error(value1);
      file = value1;
    } else {                    /* file is a stream, fd is valid */
#    if defined(WIN32_NATIVE)
      /* woe32 does have fstat(), but it does not accept a file handle,
         only an integer of an unknown nature.
         FIXME: actually, the integer is an "OS File Hangle"
         accessibe via int _open_osfhandle (intptr_t osfhandle, int flags);
         http://msdn.microsoft.com/en-us/library/bdts1c9x.aspx
         however, it is not clear whether this itroduces a leak:
         the osfhandle is supposed to be closed by _close, but it also closes
         the original handle which is no good */
      BY_HANDLE_FILE_INFORMATION fi;
      begin_blocking_system_call();
      error_p = !GetFileInformationByHandle(fd,&fi);
      end_blocking_system_call();
      if (error_p) error_OS_stream(STACK_1);
      pushSTACK(STACK_1);       /* file */
      pushSTACK(uint32_to_I(fi.dwVolumeSerialNumber)); /* device */
      pushSTACK(UL2_to_I(fi.nFileIndexHigh,fi.nFileIndexLow)); /* "inode" */
      pushSTACK(check_file_attributes_to_list(fi.dwFileAttributes));
      pushSTACK(uint32_to_I(fi.nNumberOfLinks)); /* number of hard links */
      pushSTACK(NIL); pushSTACK(NIL);            /* no GID or UID */
      pushSTACK(NIL);                            /* no rdev */
      pushSTACK(UL2_to_I(fi.nFileSizeHigh,fi.nFileSizeLow)); /* size */
      pushSTACK(NIL); pushSTACK(NIL); /* no blocksize od blocks */
      pushSTACK(convert_time_to_universal(&(fi.ftLastAccessTime)));
      pushSTACK(convert_time_to_universal(&(fi.ftLastWriteTime)));
      pushSTACK(convert_time_to_universal(&(fi.ftCreationTime)));
      goto call_make_file_stat;
#    else
      begin_blocking_system_call();
      error_p = (fstat(fd,&buf) < 0);
      end_blocking_system_call();
      if (error_p) error_OS_stream(STACK_1);
      file = eq(nullobj,STACK_1) ? fixnum(fd) : (object)STACK_1; /* restore */
#    endif
    }
  }

  file_stat_to_STACK(file,&buf);
 call_make_file_stat:
  funcall(O(object_posix__make_file_stat),14);
  skipSTACK(2);                 /* drop linkp & file */
}
#endif  /* fstat lstat fstat */

#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
/* error-signalling replacement for chmod()
   STACK_O is the path - for error reporting
 return -1 on error and 0 on success
 can trigger GC */
static int my_chmod (char *path, mode_t mode) {
#if defined(WIN32_NATIVE)
  if (!SetFileAttributes(path,mode)) return -1;
#elif defined(HAVE_CHMOD)
  if (chmod(path,mode)) return -1;
#else
  end_blocking_system_call();
  pushSTACK(CLSTEXT("~S(~S ~S ~S): this platform lacks ~S"));
  pushSTACK(TheSubr(subr_self)->name); pushSTACK(STACK_2);
  pushSTACK(O(object_Kmode)); pushSTACK(fixnum(mode));
  pushSTACK(O(object__22chmod_28_29_22));
  funcall(S(warn),5);
  begin_blocking_system_call();
#endif
  return 0;
}
/* error-signalling replacement for chown()
   STACK_O is the path - for error reporting
 return -1 on error and 0 on success
 can trigger GC */
static int my_chown (char *path, uid_t uid, gid_t gid) {
#if defined(HAVE_CHOWN)
  if (chown(path,uid,gid)) return -1;
#else
  end_blocking_system_call();
  pushSTACK(CLSTEXT("~S(~S ~S ~S ~S ~S): this platform lacks ~S"));
  pushSTACK(TheSubr(subr_self)->name); pushSTACK(STACK_2);
  pushSTACK(O(object_Kuid)); pushSTACK((uid != (uid_t)-1) ? fixnum(uid) : NIL);
  pushSTACK(O(object_Kgid)); pushSTACK((gid != (gid_t)-1) ? fixnum(gid) : NIL);
  pushSTACK(O(object__22chown_28_29_22));
  funcall(S(warn),7);
  begin_blocking_system_call();
#endif
  return 0;
}
/* error-signalling replacement for utime()
   STACK_O is the path - for error reporting
 return -1 on error and 0 on success
 can trigger GC */
#if !defined(WIN32_NATIVE)
static int my_utime (char *path, bool utb_a, bool utb_m, struct utimbuf *utb) {
  if (utb_a && !utb_m) {
    struct stat st;
    if (stat(path,&st) < 0) return -1;
    utb->modtime = st.st_mtime;
  }
  if (utb_m && !utb_a) {
    struct stat st;
    if (stat(path,&st) < 0) return -1;
    utb->actime = st.st_atime;
  }
#if defined(HAVE_UTIME)
  if (utime(path,utb)) return -1;
#else
  end_blocking_system_call();
  pushSTACK(CLSTEXT("~S(~S ~S ~S ~S ~S): this platform lacks ~S"));
  pushSTACK(TheSubr(subr_self)->name); pushSTACK(STACK_2);
  pushSTACK(O(object_Katime));
  pushSTACK(utb_a ? convert_time_to_universal(&(utb->actime)) : NIL);
  pushSTACK(O(object_Kmtime));
  pushSTACK(utb_m ? convert_time_to_universal(&(utb->modtime)) : NIL);
  pushSTACK(O(object__22utime_28_29_22));
  funcall(S(warn),7);
  begin_blocking_system_call();
#endif
  return 0;
}
#else  /* WIN32_NATIVE */
/* win32 implementation of utime() is severely broken:
   http://www.codeproject.com/datetime/dstbugs.asp */
struct a_m_time { FILETIME actime; FILETIME modtime; };
static int my_utime (char *path, bool utb_a, bool utb_m, struct a_m_time *tm) {
  HANDLE hfile = CreateFile(path, GENERIC_WRITE, 0 , NULL, OPEN_EXISTING,
                            FILE_ATTRIBUTE_NORMAL, NULL);
  BOOL success_p;
  if (hfile == INVALID_HANDLE_VALUE) return -1;
  success_p = SetFileTime(hfile,NULL,utb_a ? &(tm->actime) : NULL,
                          utb_m ? &(tm->modtime) : NULL);
  CloseHandle(hfile);
  if (!success_p) return -1;
  return 0;
}
#endif  /* WIN32_NATIVE */
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
/* get WIN32_FIND_DATA from the PATH
 < sh - search handle (optional)
 < wfd - file information
 < value1 - the actual path used
 can trigger GC */
static void find_first_file (object path, WIN32_FIND_DATA *wfd, HANDLE *sh) {
  HANDLE s_h = ON_PNAMESTRING(path,FindFirstFile,wfd);
  if (s_h == INVALID_HANDLE_VALUE) OS_file_error(value1);
  if (sh) *sh = s_h;
  else {
    begin_blocking_system_call();
    FindClose(s_h);
    begin_blocking_system_call();
  }
}
/* get file times from an object
 can trigger GC */
static void get_file_time (object path, FILETIME *atime, FILETIME *mtime) {
  WIN32_FIND_DATA wfd;
  find_first_file(path,&wfd,NULL);
  if (atime) *atime = wfd.ftLastAccessTime;
  if (mtime) *mtime = wfd.ftLastWriteTime;
}
#endif  /* WIN32_NATIVE | UNIX_CYGWIN */
DEFUN(POSIX::SET-FILE-STAT, file &key ATIME MTIME MODE UID GID,(subr_posix_set_file_stat,seclass_default,1,0,norest,key,5,NIL))
{ /* interface to chmod(2), chown(2), utime(2)
     http://opengroup.org/onlinepubs/9699919799/functions/utime.html
     http://opengroup.org/onlinepubs/9699919799/functions/chown.html
     http://opengroup.org/onlinepubs/9699919799/functions/chmod.html */
  gid_t gid = (missingp(STACK_0) ? skipSTACK(1), (gid_t)-1
               : I_to_uint32(check_uint32(popSTACK())));
  uid_t uid = (missingp(STACK_0) ? skipSTACK(1), (uid_t)-1
               : I_to_uint32(check_uint32(popSTACK())));
  mode_t mode = (missingp(STACK_0) ? skipSTACK(1), (mode_t)-1
#               if defined(WIN32_NATIVE)
                 : (mode_t)check_file_attributes_of_list(popSTACK())
#               else
                 : check_chmod_mode_of_list(popSTACK())
#               endif
                 );
# if defined(WIN32_NATIVE)
  struct a_m_time utb;
# else
  struct utimbuf utb;
# endif
  bool utb_a = false, utb_m = false;
  int status = 0;
  if (!missingp(STACK_0)) {     /* mtime */
    if (integerp(STACK_0))
      convert_time_from_universal(STACK_0,&(utb.modtime));
    else if (eq(STACK_0,T)) {
      funcall(L(get_universal_time),0);
      convert_time_from_universal(value1,&(utb.modtime));
    } else {                    /* set from another file */
#    if defined(WIN32_NATIVE)
      get_file_time(STACK_0,NULL,&(utb.modtime));
#    else
      struct stat st;
      if (ON_PNAMESTRING(STACK_0,stat,&st)) OS_file_error(value1);
      utb.modtime = st.st_mtime;
#    endif
    }
    utb_m = true;
  }
  if (!missingp(STACK_1)) {     /* atime */
    if (integerp(STACK_1))
      convert_time_from_universal(STACK_1,&(utb.actime));
    else if (eq(STACK_1,T)) {
      funcall(L(get_universal_time),0);
      convert_time_from_universal(value1,&(utb.actime));
    } else {                    /* set from another file */
#    if defined(WIN32_NATIVE)
      get_file_time(STACK_0,&(utb.actime),NULL);
#    else
      struct stat st;
      if (ON_PNAMESTRING(STACK_1,stat,&st)) OS_file_error(value1);
      utb.actime = st.st_atime;
#    endif
   }
    utb_a = true;
  }
  skipSTACK(2);                 /* drop atime & mtime */
  STACK_0 = physical_namestring(STACK_0);
  with_string_0(STACK_0,GLO(pathname_encoding),path, {
      begin_blocking_system_call();
      if (status == 0 && (mode != (mode_t)-1))
        status = my_chmod(path,mode);
      if (status == 0 && ((uid != (uid_t)-1) || (gid != (gid_t)-1)))
        status = my_chown(path,uid,gid);
      if (status == 0 && (utb_a || utb_m))
        status = my_utime(path,utb_a,utb_m,&utb);
      end_blocking_system_call();
    });
  if (status < 0) OS_file_error(STACK_0);
  VALUES0; skipSTACK(1);
}
#endif  /* chmod chown utime */

#if defined(HAVE_NFTW)


/* STACK_0 = function to be called */
static int nftw_fn (const char *path, const struct stat *ps, int kind,
                    struct FTW *ftw) {
  end_blocking_system_call(); /* back to lisp land */
  pushSTACK(asciz_to_string(path,GLO(pathname_encoding)));
  if (kind != FTW_NS) {
    file_stat_to_STACK(STACK_0,ps);
    funcall(O(object_posix__make_file_stat),14);
    pushSTACK(value1);
  } else pushSTACK(NIL);
  pushSTACK(check_ftw_kind_reverse(kind));
  pushSTACK(fixnum(ftw->base));
  pushSTACK(fixnum(ftw->level));
  funcall(STACK_5,5);
  begin_blocking_system_call(); /* leave to blocking system call */
  if (nullp(value1)) return 0;
  else {                        /* terminate the walk, return the value */
    STACK_1 = value1;
    return 1;
  }
}
DEFUN(POSIX:FILE-TREE-WALK, path func &key FD-LIMIT CHDIR DEPTH MOUNT PHYS,(subr_posix_file_tree_walk,seclass_default,2,0,norest,key,5,NIL))
{ /* http://opengroup.org/onlinepubs/9699919799/functions/nftw.html */
  int flags = ftw_flags(), ret;
  int fd_limit = check_uint_defaulted(popSTACK(),5);
  STACK_1 = physical_namestring(STACK_1);
  with_string_0(STACK_1,GLO(pathname_encoding),path, {
      begin_blocking_system_call();
      ret = nftw(path,nftw_fn,fd_limit,flags);
      end_blocking_system_call();
    });
  VALUES1(ret ? (object)STACK_1 : NIL); skipSTACK(2);
}
#endif  /* HAVE_NFTW */

/* <http://opengroup.org/onlinepubs/9699919799/basedefs/sys/stat.h.html> */
#line 2221

DEFUN(POSIX::CONVERT-MODE, mode,(subr_posix_convert_mode,seclass_default,1,0,norest,nokey,0,NIL))
{ /* convert between symbolic and numeric permissions */
  VALUES1(integerp(STACK_0)
          ? check_chmod_mode_to_list(I_to_uint32(check_uint32(popSTACK())))
          : uint32_to_I(check_chmod_mode_of_list(popSTACK())));
}

#if defined(HAVE_UMASK)
DEFUN(POSIX::UMASK, cmask,(subr_posix_umask,seclass_default,1,0,norest,nokey,0,NIL))
{ /* lisp interface to umask(2)
     http://opengroup.org/onlinepubs/9699919799/functions/umask.html */
  mode_t cmask = check_chmod_mode_of_list(popSTACK());
  begin_system_call();
  cmask = umask(cmask);
  end_system_call();
  VALUES1(fixnum(cmask));
}
#endif  /* umask */

#line 2242

DEFUN(POSIX::MKNOD, path type mode,(subr_posix_mknod,seclass_default,3,0,norest,nokey,0,NIL))
{ /* lisp interface to mknod(2)
     http://opengroup.org/onlinepubs/9699919799/functions/mknod.html */
  object md = popSTACK();
  object tp = popSTACK();
  mode_t mode = check_chmod_mode_of_list(md) | mknod_type_check(tp);
  int ret;
  STACK_0 = physical_namestring(STACK_0);
  with_string_0(STACK_0,GLO(pathname_encoding),path, {
      begin_blocking_system_call();
      ret = mknod(path,mode,0);
      end_blocking_system_call();
    });
  if (ret) OS_file_error(STACK_0);
  skipSTACK(1);
  VALUES0;
}

DEFUN(POSIX:MKDTEMP, template,(subr_posix_mkdtemp,seclass_default,1,0,norest,nokey,0,NIL)) {
  object fname = physical_namestring(popSTACK());
  with_string_0(fname,GLO(pathname_encoding),namez,{
      char *c_template;
      if (namez[namez_bytelen-1] == '/') /* mkdtemp(".../") --> ENOENT */
        namez[--namez_bytelen] = 0;
      begin_blocking_system_call();
      ENSURE_6X(namez,c_template);
      c_template = mkdtemp(c_template);
      end_blocking_system_call();
      if (NULL == c_template) ANSIC_error();
      fname = asciz_to_string(c_template,GLO(pathname_encoding));
    });
  pushSTACK(fname);
  /* stack layout: the name of the new directory - without the trailing slash */
#if defined(WIN32_NATIVE)
  pushSTACK(GLO(backslash_string));
#else
  pushSTACK(GLO(slash_string));
#endif
  VALUES1(string_concat(2));
}

#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
#if defined(WIN32_NATIVE)
/* winsup/src/winsup/cygwin/syscalls.cc */
typedef unsigned long fsblkcnt_t;
typedef unsigned long fsfilcnt_t;
struct statvfs {
  unsigned long f_bsize;        /* file system block size */
  unsigned long f_frsize;       /* fragment size */
  fsblkcnt_t f_blocks;          /* size of fs in f_frsize units */
  fsblkcnt_t f_bfree;           /* free blocks in fs */
  fsblkcnt_t f_bavail;          /* free blocks avail to non-superuser */
  fsfilcnt_t f_files;           /* total file nodes in file system */
  fsfilcnt_t f_ffree;           /* free file nodes in fs */
  fsfilcnt_t f_favail;          /* avail file nodes in fs */
  unsigned long f_fsid;         /* file system id */
  unsigned long f_flag;         /* mount flags */
  unsigned long f_namemax;      /* maximum length of filenames */
  char f_volname[MAX_PATH];     /* volume name */
  char f_fstype[MAX_PATH];      /* file system type */
};
#define HAVE_STATVFS_F_VOLNAME
#define HAVE_STATVFS_F_FSTYPE
static int statvfs (const char *fname, struct statvfs *sfs) {
  /* GetDiskFreeSpaceEx must be called before GetDiskFreeSpace on
     WinME, to avoid the MS KB 314417 bug */
  ULARGE_INTEGER availb, freeb, totalb;
  DWORD spc, bps, availc, freec, totalc, vsn, maxlen, flags, bpc;
  char root[MAX_PATH], *rootp = root;
  if (fname[1] == ':') {        /* c:\ */
    *rootp++ = *fname++;
    *rootp++ = *fname++;
  } else if (fname[0] == '\\' && fname[1] == '\\') { /* \\host\dir\ */
    const char *cp = strchr(fname + 2,'\\');
    unsigned int len;
    if (cp) cp = strchr(cp+1,'\\'); /* just host, no dir => error later */
    memcpy(root,fname,(len = cp - fname));
    rootp = root + len;
  } else {
    SetLastError(ERROR_DIRECTORY);
    return -1;
  }
  *rootp++ = '\\';
  *rootp = 0;

  if (!GetDiskFreeSpace(root,&spc,&bps,&freec,&totalc))
    return -1;               /* bytes per sector, sectors per cluster */
  bpc = spc*bps;             /* bytes per cluster */
  if (GetDiskFreeSpaceEx(root,&availb,&totalb,&freeb)) {
    availc = availb.QuadPart / bpc;
    totalc = totalb.QuadPart / bpc;
    freec = freeb.QuadPart / bpc;
  } else
    availc = freec;
  if (!GetVolumeInformation(root,sfs->f_volname,MAX_PATH,&vsn,&maxlen,&flags,
                            sfs->f_fstype,MAX_PATH))
    return -1;
  sfs->f_bsize = bpc;
  sfs->f_frsize = bpc;
  sfs->f_blocks = totalc;
  sfs->f_bfree = freec;
  sfs->f_bavail = availc;
  sfs->f_files = (fsfilcnt_t)-1;
  sfs->f_ffree = (fsfilcnt_t)-1;
  sfs->f_favail = (fsfilcnt_t)-1;
  sfs->f_fsid = vsn;
  sfs->f_flag = flags;
  sfs->f_namemax = maxlen;
  return 0;
}
#endif
#line 2362

/* there is also a legacy interface (f)statfs()
   which is not POSIX and is not supported */
DEFUN(POSIX::STAT-VFS, file,(subr_posix_stat_vfs,seclass_default,1,0,norest,nokey,0,NIL))
{ /* Lisp interface to statvfs(2), fstatvfs(2)
 the first arg can be a pathname designator or a file descriptor designator
 the return value is the STAT-VFS structure */
  object file = STACK_0;
  struct statvfs buf;
  bool error_p;

#if defined(HAVE_FSTATVFS)
  if (integerp(file)) {
    begin_blocking_system_call();
    error_p = (fstatvfs(I_to_L(file),&buf) < 0);
    end_blocking_system_call();
    if (error_p) ANSIC_error();
  } else {
    Handle fd;
    file = open_file_stream_handle(file,&fd,true);
    if (!eq(nullobj,file)) { /* an open stream */
      pushSTACK(file);
      begin_blocking_system_call();
      error_p = (fstatvfs(fd,&buf) < 0);
      end_blocking_system_call();
      file = popSTACK();
      if (error_p) error_OS_stream(STACK_0);
      STACK_0 = file;
    } else
#endif
      if (ON_PNAMESTRING(STACK_0,statvfs,&buf))
        OS_file_error(value1);
#if defined(HAVE_FSTATVFS)
  }
#endif

  /* STACK_0 is already the object statvfs'ed */
#define pushSLOT(s) pushSTACK(s==(unsigned long)-1 ? NIL : ulong_to_I(s))
  pushSLOT(buf.f_bsize);  /* file system block size */
  pushSLOT(buf.f_frsize); /* fundamental file system block size */
#if defined(SIZEOF_FSBLKCNT_T) && SIZEOF_FSBLKCNT_T == 8
# define pushBSLOT(s) pushSTACK(s==(fsblkcnt_t)-1 ? NIL : uint64_to_I(s))
#else
# define pushBSLOT(s) pushSTACK(s==(fsblkcnt_t)-1 ? NIL : uint32_to_I(s))
#endif
  pushBSLOT(buf.f_blocks); /* total # of blocks on file system */
  pushBSLOT(buf.f_bfree);  /* total number of free blocks */
  pushBSLOT(buf.f_bavail); /* # of free blocks available to
                              non-privileged processes */
#undef pushBSLOT
#if defined(SIZEOF_FSFILCNT_T) && SIZEOF_FSFILCNT_T == 8
# define pushFSLOT(s) pushSTACK(s==(fsfilcnt_t)-1 ? NIL : uint64_to_I(s))
#else
# define pushFSLOT(s) pushSTACK(s==(fsfilcnt_t)-1 ? NIL : uint32_to_I(s))
#endif
  pushFSLOT(buf.f_files);  /* total # of file serial numbers */
  pushFSLOT(buf.f_ffree);  /* total # of free file serial numbers */
  pushFSLOT(buf.f_favail); /* # of file serial numbers available to
                              non-privileged processes */
#undef pushFSLOT
#if HAVE_SCALAR_FSID
  pushSLOT(buf.f_fsid);   /* file system ID */
#else
  /* On Linux, f_fsid of 'struct statfs' is a struct consisting of two ints.
     With glibc <= 2.1, f_fsid of 'struct statvfs' is the same. We are
     prepared to return one number only, so we just return the first int.
     This matches the behaviour of glibc >= 2.2 on 32-bit platforms. */
  pushSLOT((*(uintL*)&buf.f_fsid));   /* file system ID */
#endif
  pushSTACK(vfs_flags_to_list(buf.f_flag)); /* Bit mask of f_flag values. */
  pushSLOT(buf.f_namemax);      /* maximum filename length */
#if defined(HAVE_STATVFS_F_VOLNAME)
  pushSTACK(asciz_to_string(buf.f_volname,GLO(pathname_encoding)));
#else
  pushSTACK(NIL);
#endif
#if defined(HAVE_STATVFS_F_FSTYPE)
  pushSTACK(asciz_to_string(buf.f_fstype,GLO(pathname_encoding)));
#else
  pushSTACK(NIL);
#endif
  funcall(O(object_posix__make_stat_vfs),14);
#undef pushSLOT
}

#endif  /* fstatvfs statvfs */


#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)

/* Dynamically load some functions missing in Windows95/98/ME
   to work with Security IDentifiers (SIDs). */

#include <aclapi.h>

/* Added in Windows NT 4.0 */
typedef DWORD (WINAPI * GetSecurityInfoFunc_t)
  (HANDLE handle, SE_OBJECT_TYPE ObjectType, SECURITY_INFORMATION SecurityInfo,
   PSID* ppsidOwner, PSID* ppsidGroup, PACL* ppDacl, PACL* ppSacl,
   PSECURITY_DESCRIPTOR* ppSecurityDescriptor);
static GetSecurityInfoFunc_t GetSecurityInfoFunc;
#undef GetSecurityInfo
#define GetSecurityInfo (*GetSecurityInfoFunc)

/* Added in Windows NT Workstation */
typedef BOOL (WINAPI * LookupAccountSidFunc_t)
  (LPCTSTR lpSystemName, PSID lpSid, LPTSTR lpName, LPDWORD cchName,
   LPTSTR lpReferencedDomainName, LPDWORD cchReferencedDomainName,
   PSID_NAME_USE peUse);
static LookupAccountSidFunc_t LookupAccountSidFunc;
#undef LookupAccountSid
#define LookupAccountSid (*LookupAccountSidFunc)

/* Added in Windows NT Workstation */
typedef DWORD (WINAPI * GetLengthSidFunc_t) (PSID pSid);
static GetLengthSidFunc_t GetLengthSidFunc;
#undef GetLengthSid
#define GetLengthSid (*GetLengthSidFunc)

/* Added in Windows NT Workstation */
typedef BOOL (WINAPI * CopySidFunc_t)
  (DWORD nDestinationSidLength, PSID pDestinationSid, PSID pSourceSid);
static CopySidFunc_t CopySidFunc;
#undef CopySid
#define CopySid (*CopySidFunc)

/* Added in Windows NT Workstation */
typedef BOOL (WINAPI * EqualSidFunc_t) (PSID pSid1, PSID pSid2);
static EqualSidFunc_t EqualSidFunc;
#undef EqualSid
#define EqualSid (*EqualSidFunc)

/* Added in Windows 2000 Professional */
typedef BOOL (WINAPI * ConvertSidToStringSidFunc_t)
  (IN PSID Sid, OUT LPTSTR *StringSid);
static ConvertSidToStringSidFunc_t ConvertSidToStringSidFunc;
#undef ConvertSidToStringSid
#define ConvertSidToStringSid (*ConvertSidToStringSidFunc)

static BOOL initialized_sid_apis = FALSE;

static void initialize_sid_apis () {
  HMODULE advapi32 = LoadLibrary("advapi32.dll");
  if (advapi32 != NULL) {
    GetSecurityInfoFunc = (GetSecurityInfoFunc_t)
      GetProcAddress(advapi32, "GetSecurityInfo");
    LookupAccountSidFunc = (LookupAccountSidFunc_t)
      GetProcAddress(advapi32, "LookupAccountSidA");
    GetLengthSidFunc = (GetLengthSidFunc_t)
      GetProcAddress(advapi32, "GetLengthSid");
    CopySidFunc = (CopySidFunc_t) GetProcAddress(advapi32, "CopySid");
    EqualSidFunc = (EqualSidFunc_t) GetProcAddress(advapi32, "EqualSid");
    ConvertSidToStringSidFunc = (ConvertSidToStringSidFunc_t)
      GetProcAddress(advapi32, "ConvertSidToStringSidA");
  }
  initialized_sid_apis = TRUE;
}

#endif /* (WIN32_NATIVE || UNIX_CYGWIN) */

/* FILE-OWNER */

#if defined(HAVE_GETPWUID)
static const char * get_owner (const char *filename) {
  struct stat statbuf;
  if (lstat(filename, &statbuf) >= 0) {
    struct passwd *pwd = getpwuid(statbuf.st_uid);
    if (pwd)
      return pwd->pw_name;
  }
  return "";
}
#elif defined(WIN32_NATIVE)

/* A cache mapping SID -> owner. */
struct sid_cache_entry {
  PSID psid;
  char *name;
};
static struct sid_cache_entry *sid_cache = NULL;
static size_t sid_cache_count = 0;
static size_t sid_cache_allocated = 0;

static const char * sid_cache_get (PSID psid) {
  size_t i;
  for (i = 0; i < sid_cache_count; i++)
    if (EqualSid(psid, sid_cache[i].psid))
      return sid_cache[i].name;
  return NULL;
}

static void sid_cache_put (PSID psid, const char *name) {
  if (sid_cache_count == sid_cache_allocated) {
    size_t new_allocated = 2 * sid_cache_allocated + 5;
    sid_cache = (struct sid_cache_entry*)
      (sid_cache != NULL
       ? realloc(sid_cache, new_allocated * sizeof(struct sid_cache_entry))
       : malloc(new_allocated * sizeof(struct sid_cache_entry)));
    sid_cache_allocated = (sid_cache == NULL)?0:new_allocated;
  }
  if (sid_cache != NULL) {
    DWORD psid_len = GetLengthSid(psid);
    size_t name_len = strlen(name) + 1;
    char *memory = (char *)malloc(psid_len+name_len);
    if (memory == NULL)
      return;
    if (!CopySid(psid_len, memory, psid)) return;
    memcpy(memory+psid_len, name, name_len);
    sid_cache[sid_cache_count].psid = memory;
    sid_cache[sid_cache_count].name = memory + psid_len;
    sid_cache_count++;
  }
}

static const char * get_owner (const char *filename) {
  char *owner;

  if (!initialized_sid_apis)
    initialize_sid_apis();
  owner = "";
  if (GetSecurityInfoFunc != NULL
      && LookupAccountSidFunc != NULL
      && GetLengthSidFunc != NULL
      && CopySidFunc != NULL
      && EqualSidFunc != NULL) {
    /* On Windows, directories don't have an owner. */
    WIN32_FIND_DATA entry;
    HANDLE searchhandle = FindFirstFile(filename, &entry);
    if (searchhandle != INVALID_HANDLE_VALUE) {
      if (!(entry.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
        /* It's a file. */
        HANDLE filehandle =
         CreateFile(filename, GENERIC_READ,
                    FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
                    OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
        if (filehandle != INVALID_HANDLE_VALUE) {
          /* Get the owner. */
          PSID psid;
          PSECURITY_DESCRIPTOR psd;
          DWORD err =
            GetSecurityInfo(filehandle, SE_FILE_OBJECT,
                            OWNER_SECURITY_INFORMATION,
                            &psid, NULL, NULL, NULL, &psd);
          if (err == 0) {
            owner = (char*)sid_cache_get(psid);
            if (owner == NULL) {
              char buf1[256];
              DWORD buf1size = sizeof(buf1);
              char buf2[256];
              DWORD buf2size = sizeof(buf2);
              SID_NAME_USE role;
              if (!LookupAccountSid(NULL, psid, buf1, &buf1size, buf2,
                                    &buf2size, &role)) {
                char *s;
                if (ConvertSidToStringSidFunc != NULL
                    && !ConvertSidToStringSidFunc(psid, &s)) {
                  /* Fallback: Use S-R-I-S-S... notation.  */
                  strncpy(buf1, s, buf1size);
                  buf1[buf1size - 1] = '\0';
                  LocalFree(s);
                  owner = buf1;
                } else (owner = buf1)[0] = '\0';
              } else { /* DOMAIN\Account */
                int len = strlen(buf2);
                buf2[len] = '\\';
                strcpy(buf2+len+1,buf1);
                owner = buf2;
              }
              sid_cache_put(psid, owner);
            }
            LocalFree(psd);
          }
          CloseHandle(filehandle);
        }
      }
      FindClose(searchhandle);
    }
  }
  return owner;
}
#else /* neither HAVE_GETPWUID nor WIN32_NATIVE - should never happen! */
static const char * get_owner (const char *filename) { return ""; }
#endif

DEFUN(OS::FILE-OWNER, file,(subr_os_file_owner,seclass_default,1,0,norest,nokey,0,NIL)) {
  VALUES1(safe_to_string((char*)ON_PNAMESTRING(popSTACK(),get_owner,NULL)));
}

/* end of FILE-OWNER */

#if defined(WIN32_NATIVE)

/* Pointers to functions unavailable on windows 95, 98, ME */

typedef BOOL (WINAPI * CreateHardLinkFuncType)
  (LPCTSTR lpFileName, LPCTSTR lpExistingFileName,
   LPSECURITY_ATTRIBUTES lpSecurityAttributes);
static CreateHardLinkFuncType CreateHardLinkFunc = NULL;

typedef BOOL (WINAPI * BackupWriteFuncType)
  (HANDLE hFile, LPBYTE lpBuffer, DWORD nNumberOfBytesToWrite,
   LPDWORD lpNumberOfBytesWritten, BOOL bAbort, BOOL bProcessSecurity,
   LPVOID *lpContext);
static BackupWriteFuncType BackupWriteFunc = NULL;

static void init_win32_link (void) {
  HMODULE kernel32 = LoadLibrary ("kernel32.dll");
  if (kernel32 != NULL) {
    CreateHardLinkFunc = (CreateHardLinkFuncType)
      GetProcAddress (kernel32, "CreateHardLinkA");
    BackupWriteFunc = (BackupWriteFuncType)
      GetProcAddress (kernel32, "BackupWrite");
    LockFileExFunc = (LockFileExFuncType)
      GetProcAddress (kernel32, "LockFileEx");
    if (LockFileExFunc == NULL)
      LockFileExFunc = (LockFileExFuncType) &my_LockFileEx;
    UnlockFileExFunc = (UnlockFileExFuncType)
      GetProcAddress (kernel32, "UnlockFileEx");
    if (UnlockFileExFunc == NULL)
      UnlockFileExFunc = (UnlockFileExFuncType) &my_UnlockFileEx;
  }
}
#endif

#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
typedef HRESULT (WINAPI * StgOpenStorageExFuncType) (const WCHAR* pwcsName,
            DWORD grfMode, DWORD stgfmt, DWORD grfAttrs, void * reserved1,
            void * reserved2, REFIID riid, void ** ppObjectOpen);
static StgOpenStorageExFuncType StgOpenStorageExFunc = NULL;

static void init_win32_cygwin_open_storage (void) {
  HMODULE ole32 = LoadLibrary ("ole32.dll");
  if (ole32 != NULL)
    StgOpenStorageExFunc = (StgOpenStorageExFuncType)
      GetProcAddress (ole32, "StgOpenStorageEx");
}
#endif

/* COPY-FILE related functions. */

#if defined(WIN32_NATIVE)
/* Checks if it's safe to call OldHardLink */
static BOOL OldHardLinkGuard () {
  OSVERSIONINFO vi;
  if (BackupWriteFunc == NULL) return FALSE;
  vi.dwOSVersionInfoSize = sizeof(vi);
  if (!GetVersionEx(&vi)) return FALSE;
  return vi.dwPlatformId == VER_PLATFORM_WIN32_NT;
}

/* From knowledge base article Q234727
   This approach works on NT >= 3.51. */
static BOOL OldHardLink( LPCTSTR source, LPCTSTR dest ) {

   WCHAR  wsource[ MAX_PATH + 1 ];
   WCHAR  wdest[ MAX_PATH + 1 ];
   WCHAR  wdestfull[ MAX_PATH + 1 ];
   LPWSTR wdestfullfile;

   HANDLE hFileSource;

   WIN32_STREAM_ID StreamId;
   DWORD dwBytesWritten;
   LPVOID lpContext;
   DWORD cbPathLen;
   DWORD StreamHeaderSize;

   BOOL bSuccess;

   /* convert from ANSI to UNICODE */
   if (MultiByteToWideChar(CP_ACP, MB_ERR_INVALID_CHARS /*error on invalid chars*/,
     dest, -1/*null terminated*/, wdest, MAX_PATH + 1) == 0) return FALSE;

   /* open existing file that we link to */
   hFileSource = CreateFile(source, FILE_WRITE_ATTRIBUTES,
     FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
     NULL, /* sa */ OPEN_EXISTING, 0, NULL );

   if (hFileSource == INVALID_HANDLE_VALUE) return FALSE;

   /* validate and sanitize supplied link path and use the result
      the full path MUST be Unicode for BackupWrite */
   cbPathLen = GetFullPathNameW( wdest , MAX_PATH, wdestfull, &wdestfullfile);

   if (cbPathLen == 0) return FALSE;

   cbPathLen = (cbPathLen + 1) * sizeof(WCHAR); // adjust for byte count

   /* prepare and write the WIN32_STREAM_ID out */
   lpContext = NULL;

   StreamId.dwStreamId = BACKUP_LINK;
   StreamId.dwStreamAttributes = 0;
   StreamId.dwStreamNameSize = 0;
   StreamId.Size.HighPart = 0;
   StreamId.Size.LowPart = cbPathLen;

   /* compute length of variable size WIN32_STREAM_ID */
   StreamHeaderSize = (LPBYTE)&StreamId.cStreamName - (LPBYTE)&StreamId
                      + StreamId.dwStreamNameSize ;

   bSuccess = BackupWriteFunc(hFileSource,
                         (LPBYTE)&StreamId,  /* buffer to write */
                         StreamHeaderSize,   /* number of bytes to write */
                         &dwBytesWritten,
                         FALSE,              /* don't abort yet */
                         FALSE,              /* don't process security */
                         &lpContext);
   bSuccess &= BackupWriteFunc(hFileSource,(LPBYTE)wdestfull, cbPathLen,
        &dwBytesWritten, FALSE, FALSE, &lpContext);
   /* free context */
   bSuccess &= BackupWriteFunc(hFileSource,NULL,0,&dwBytesWritten,TRUE, FALSE,
        &lpContext);
   CloseHandle( hFileSource );
   return bSuccess;
}

static inline int MkHardLink (char* old_pathstring, char* new_pathstring) {
  if (CreateHardLinkFunc != NULL)
    return CreateHardLinkFunc(new_pathstring,old_pathstring,NULL);
  if (OldHardLinkGuard())
    return OldHardLink(old_pathstring,new_pathstring);
  SetLastError(ERROR_INVALID_FUNCTION); /* or what ? */
  return 0;
}
#endif

/* Hard/Soft Link a file
 > old_pathstring: old file name, ASCIZ-String
 > new_pathstring: new file name, ASCIZ-String
 > STACK_3: old pathname
 > STACK_1: new pathname */
#if defined(WIN32_NATIVE)
# define HAVE_LINK
#elif !defined(LINK_FOLLOWS_SYMLINKS) && defined(HAVE_REALPATH)
static inline int my_link (const char* source, const char* destination) {
# ifndef MAXPATHLEN             /* see unix.d */
#  define MAXPATHLEN  4096      /* see <sys/param.h> */
# endif
  char path_buffer[MAXPATHLEN];
  if (NULL == realpath(source,path_buffer)) OS_file_error(STACK_3);
  return link(path_buffer,destination);
}
#else
# define my_link link
#endif
#if defined(HAVE_LINK)
static bool hardlink_file (char* old_pathstring, char* new_pathstring,
                           bool error_p) {
  gcv_object_t *failed = NULL;
  begin_blocking_system_call();
# if defined(WIN32_NATIVE)
  if (MkHardLink(old_pathstring,new_pathstring) == FALSE)
    failed = (GetLastError() == ERROR_FILE_NOT_FOUND ? &STACK_3 : &STACK_1);
# else
  if (my_link(old_pathstring,new_pathstring) < 0)
    failed = (errno==ENOENT ? &STACK_3 : &STACK_1);
# endif
  end_blocking_system_call();
  if (failed && error_p) OS_file_error(*failed);
  return failed != NULL;
}
static /*maygc*/ bool hardlink_file_o (gcv_object_t *old_pathstring,
                                       gcv_object_t *new_pathstring,
                                       bool error_p) {
  bool status;
  *old_pathstring = physical_namestring(*old_pathstring);
  *new_pathstring = physical_namestring(*new_pathstring);
  with_string_0(*old_pathstring, GLO(pathname_encoding), source_asciz, {
    with_string_0(*new_pathstring, GLO(pathname_encoding), dest_asciz, {
      status = hardlink_file(source_asciz,dest_asciz,error_p);
    });
  });
  return status;
}
#endif
#if defined(HAVE_SYMLINK)
static inline void symlink_file (char* old_pathstring, char* new_pathstring) {
  gcv_object_t *failed = NULL;
  int len = strlen(new_pathstring) - 1;
  if ('/' == new_pathstring[len]) /* symlink to "foo/" => ENOENT */
    new_pathstring[len] = 0;
  begin_blocking_system_call();
  if (symlink(old_pathstring,new_pathstring) < 0) /* symlink file */
    failed = (errno==ENOENT ? &STACK_3 : &STACK_1);
  end_blocking_system_call();
  if (failed) OS_file_error(*failed);
}
#endif

/* Copy attributes from stream STACK_1 to stream STACK_0 and close them
   can trigger GC */
static void copy_attributes_and_close () {
  Handle source_fd = stream_lend_handle(&STACK_1,true,NULL);
  Handle dest_fd = stream_lend_handle(&STACK_0,false,NULL);
  struct stat source_sb;
  struct stat dest_sb;

# if defined(HAVE_FSTAT) && !defined(WIN32_NATIVE)
  begin_blocking_system_call();
  if (fstat(source_fd, &source_sb) == -1) {
    end_blocking_system_call();
    pushSTACK(file_stream_truename(STACK_1));
    goto close_and_err;
  }
  if (fstat(dest_fd, &dest_sb) == -1) {
    end_blocking_system_call();
    pushSTACK(file_stream_truename(STACK_0));
    goto close_and_err;
  }
  end_blocking_system_call();
# elif defined(HAVE_STAT)
  if (ON_PNAMESTRING(STACK_1,stat,&source_sb)) {
    pushSTACK(file_stream_truename(STACK_1));
    goto close_and_err;
  }
  if (ON_PNAMESTRING(STACK_0,stat,&dest_sb) == (void*)-1) {
    pushSTACK(file_stream_truename(STACK_0));
    goto close_and_err;
  }
# else
  goto close_success;
# endif

# if defined(WIN32_NATIVE) /*** file mode ***/
  { BOOL ret;
    BY_HANDLE_FILE_INFORMATION fi;
    begin_blocking_system_call();
    ret = GetFileInformationByHandle(source_fd,&fi);
    end_blocking_system_call();
    if (!ret) {
      pushSTACK(file_stream_truename(STACK_1));
      goto close_and_err;
    }
    if (!ON_PNAMESTRING(STACK_0,SetFileAttributes,fi.dwFileAttributes)) {
      pushSTACK(file_stream_truename(STACK_0));
      goto close_and_err;
    }
  }
# elif defined(HAVE_FCHMOD)
  begin_blocking_system_call();
  if (((source_sb.st_mode & 0777) != (dest_sb.st_mode & 0777))
      && (fchmod(dest_fd, source_sb.st_mode & 0777) == -1)) {
    end_blocking_system_call();
    pushSTACK(file_stream_truename(STACK_0));
    goto close_and_err;
  }
  end_blocking_system_call();
# elif defined(HAVE_CHMOD)
  if ((source_sb.st_mode & 0777) != (dest_sb.st_mode & 0777)
      && ON_PNAMESTRING(STACK_0,chmod,source_sb.st_mode & 0777)) {
    pushSTACK(file_stream_truename(STACK_0));
    goto close_and_err;
  }
# endif

# if defined(HAVE_FCHOWN) /*** owner/group ***/
  begin_blocking_system_call();
  if (fchown(dest_fd, source_sb.st_uid, source_sb.st_gid) == -1) {
    end_blocking_system_call();
    pushSTACK(file_stream_truename(STACK_0));
    goto close_and_err;
  }
  end_blocking_system_call();
# elif defined(HAVE_CHOWN)
  { int ret;
    with_string_0(physical_namestring(STACK_0),GLO(pathname_encoding),destz,{
        begin_blocking_system_call();
        ret = chown(destz, source_sb.st_uid, source_sb.st_gid);
        end_blocking_system_call();
      });
    if (ret == -1) {
      pushSTACK(file_stream_truename(STACK_0));
      goto close_and_err;
    }
  }
# endif

# if defined(HAVE_UTIME)
  /* we must close the streams now - before utime() -
     because close() modifies write and access times */
  builtin_stream_close(&STACK_0,0);
  builtin_stream_close(&STACK_1,0);
  { /*** access/mod times ***/
    struct utimbuf utb;
    /* first element of the array is access time, second is mod time. set
       both tv_usec to zero since the file system can't gurantee that
       kind of precision anyway. */
    utb.actime  = source_sb.st_atime;
    utb.modtime = source_sb.st_mtime;
    if (ON_PNAMESTRING(STACK_0,utime,&utb)) {
      pushSTACK(file_stream_truename(STACK_0));
      goto close_and_err;
    }
  }
  return;
# endif
 close_success:
  builtin_stream_close(&STACK_0,0);
  builtin_stream_close(&STACK_1,0);
  return;
 close_and_err:
  builtin_stream_close(&STACK_1,0);
  builtin_stream_close(&STACK_2,0);
  OS_file_error(STACK_0);
}

/* on success, push (source dest byte-count) on retval (an address in STACK)
 can trigger GC */
static void copy_file_low (object source, object dest,
                           bool preserve_p, if_exists_t if_exists,
                           if_does_not_exist_t if_not_exists,
                           gcv_object_t* retval) {
/* (let ((buffer (make-array buffer-size :element-type 'unsigned-byte)))
    (with-open-file (source-stream source :direction :input
                                   :element-type 'unsigned-byte)
     (with-open-file (dest-stream dest :direction (if append-p :append :output)
                                  :element-type 'unsigned-byte)
       (loop for bytes-read = (read-byte-sequence buffer source-stream)
             until (= 0 bytes-read)
             do (write-byte-sequence buffer dest-stream :end bytes-read)))))
*/
  uintL total_count = 0; /* return value: total byte count */
  /* create the two streams */
  pushSTACK(dest);
  /* input: */
  pushSTACK(source);            /* filename */
  pushSTACK(S(Kdirection)); pushSTACK(S(Kinput));
  pushSTACK(S(Kelement_type)); pushSTACK(S(unsigned_byte));
  pushSTACK(S(Kif_does_not_exist));
  pushSTACK(if_does_not_exist_symbol(if_not_exists));
  funcall(L(open),7); source = value1;
  if (nullp(source)) {
    skipSTACK(1); /* drop dest */
    return;
  }
  pushSTACK(STACK_0); STACK_1 = source;
  /* stack layout: 1: source stream; 0: dest path */
  /* output: */
  pushSTACK(S(Kdirection)); pushSTACK(S(Koutput));
  pushSTACK(S(Kelement_type)); pushSTACK(S(unsigned_byte));
  pushSTACK(S(Kif_exists)); pushSTACK(if_exists_symbol(if_exists));
  funcall(L(open),7); dest = value1;
  if (nullp(dest)) {
    builtin_stream_close(&STACK_0,0);
    skipSTACK(1); /* drop source */
    return;
  }
  pushSTACK(dest);
  /* stack layout: 0=output stream; 1=input stream */
  { /* make the bit buffer and copy data */
    uintL bytes_read;
    char buffer[strm_buffered_bufflen];
    /* stack layout: 0 - dest-stream; 1 - source-stream */
    Handle fd_in = stream_lend_handle(&STACK_1,true,NULL);
    Handle fd_ou = stream_lend_handle(&STACK_0,false,NULL);
    while ((bytes_read = fd_read(fd_in,buffer,strm_buffered_bufflen,
                                 persev_full))) {
      total_count += bytes_read;
      fd_write(fd_ou,buffer,bytes_read,persev_full);
    }
  }
  if (!preserve_p) {
    builtin_stream_close(&STACK_0,0);
    builtin_stream_close(&STACK_1,0);
  } else
    copy_attributes_and_close();
  /* clean up the stack */
  pushSTACK(allocate_cons());
  Cdr(STACK_0) = *retval;
  *retval = STACK_0;
  STACK_2 = file_stream_truename(STACK_2); /* source */
  STACK_1 = file_stream_truename(STACK_1); /* dest */
  STACK_0 = UL_to_I(total_count);
  Car(*retval) = listof(3);
}

#line 3040

/* copy just one file: source --> dest (both STRINGs, NIL or PATHNAME)
   can trigger GC */
static void copy_one_file (object source, object src_path,
                           object dest, object dest_path,
                           copy_method_t method, bool preserve_p,
                           if_exists_t if_exists,
                           if_does_not_exist_t if_not_exists,
                           gcv_object_t* retval) {
  pushSTACK(source); pushSTACK(src_path);
  pushSTACK(dest); pushSTACK(dest_path);
  XOUT(source,"copy_one_file");
  XOUT(src_path,"copy_one_file");
  XOUT(dest,"copy_one_file");
  XOUT(dest_path,"copy_one_file");
  /* merge source into dest: "cp foo bar/" --> "cp foo bar/foo" */
  pushSTACK(STACK_2); /* src_path */
  funcall(L(merge_pathnames),2); pushSTACK(value1); /* dest_path */

  if (method == COPY_METHOD_COPY) {
    copy_file_low(STACK_2,STACK_0,preserve_p,if_exists,if_not_exists,retval);
    skipSTACK(4);
    return;
  }

  pushSTACK(STACK_0); funcall(L(probe_pathname),1);
  if (!nullp(value1)) { /* destination exists; value1 == truename */
    pushSTACK(value1); STACK_2 = dest = value1;
    /* 5 STACK: 0=dest_true; 1=dest_path; 2=dest; 3=src_path; 4=src */
    switch (if_exists) {
      case IF_EXISTS_NIL: skipSTACK(5); return;
      case IF_EXISTS_APPEND:
        /* we know that method != COPY_METHOD_COPY - handled above! */
        pushSTACK(S(Kappend));
        pushSTACK(check_copy_method_reverse(method));
        pushSTACK(O(object_posix__copy_file));
        error(error_condition,GETTEXT("~S: ~S forbids ~S"));
      case IF_EXISTS_OVERWRITE:
      case IF_EXISTS_SUPERSEDE:
      case IF_EXISTS_RENAME_AND_DELETE:
        /* these are the same since (sym)link/rename are atomic */
        break;
      case IF_EXISTS_UNBOUND: case IF_EXISTS_ERROR:
      case IF_EXISTS_RENAME:    /* delegate to OPEN */
        pushSTACK(value1);      /* destination */
        pushSTACK(S(Kif_exists)); pushSTACK(if_exists_symbol(if_exists));
        pushSTACK(S(Kdirection)); pushSTACK(S(Koutput));
        funcall(L(open),5);
        pushSTACK(value1); builtin_stream_close(&STACK_0,0);
        funcall(L(delete_file),1);
        break;
      default: NOTREACHED;
    }
  } else pushSTACK(STACK_0); /* destination does not exist, use dest_path */

  pushSTACK(STACK_3); funcall(L(probe_pathname),1);
  if (nullp(value1)) { /* source does not exist */
    if (method == COPY_METHOD_RENAME || method == COPY_METHOD_HARDLINK
        || method == COPY_METHOD_HARDLINK_OR_COPY) {
      if (if_not_exists == IF_DOES_NOT_EXIST_NIL) {
        skipSTACK(5); return;
      } else { /* delegate error to OPEN */
        pushSTACK(STACK_3);     /* source */
        pushSTACK(S(Kif_does_not_exist));
        pushSTACK(if_does_not_exist_symbol(if_not_exists));
        pushSTACK(S(Kdirection)); pushSTACK(S(Kinput));
        funcall(L(open),5);
        NOTREACHED;
      }
    }
  }
  pushSTACK(value1);

  /* 6 STACK: 0=src_true; 1=dest_true ... */
  switch (method) {
    case COPY_METHOD_RENAME:
      pushSTACK(STACK_0); pushSTACK(STACK_2);
      pushSTACK(S(Kif_exists)); pushSTACK(if_exists_symbol(if_exists));
      funcall(L(rename_file),4);
      source = STACK_4; dest = STACK_1;
      break;
    case COPY_METHOD_HARDLINK_OR_COPY: {
#    if defined(HAVE_LINK)
      if (hardlink_file_o(&STACK_0/*source*/,&STACK_1/*dest*/,false))
        goto copy_one_file_copy;
      source = STACK_0; dest = STACK_1;
      break;
#    endif
    } /* FALLTHROUGH if no hardlinks */
    case COPY_METHOD_SYMLINK:
#    if defined(HAVE_SYMLINK)
      STACK_1 = physical_namestring(STACK_1); /* dest */
      /* use the original argument, not the truename here,
         so that the user can create relative symlinks */
      STACK_0 = (stringp(STACK_5) ? (object)STACK_5 /* source */
                 : physical_namestring(STACK_4));
      with_string_0(STACK_0, GLO(pathname_encoding), source_asciz, {
        with_string_0(STACK_1, GLO(pathname_encoding), dest_asciz,
                      { symlink_file(source_asciz,dest_asciz); });
      });
      source = STACK_0; dest = STACK_1;
      break;
#    endif
      /* FALLTHROUGH if no symlinks */
    case COPY_METHOD_HARDLINK:
#    if defined(HAVE_LINK)
      hardlink_file_o(&STACK_0/*source*/,&STACK_1/*dest*/,true);
      source = STACK_0; dest = STACK_1;
      break;
#    endif
      /* FALLTHROUGH if no hardlinks */
    default: copy_one_file_copy:
      copy_file_low(STACK_0,STACK_1,preserve_p,if_exists,if_not_exists,retval);
      skipSTACK(6);
      return;
  }
  /* update retval */
  STACK_0 = dest;
  STACK_1 = source;
  STACK_2 = allocate_cons();
  Cdr(STACK_2) = *retval;
  *retval = STACK_2;
  Car(*retval) = listof(2);
  skipSTACK(4);
}

/* (COPY-FILE source target &key method preserve (if-exists :supersede)
              (if-does-not-exist :error))
 source and target are pathname designators (whether or not they
 can be streams is up for debate). if target is missing a name or
 type designator it is taken from source.
 keywords:
 method := :hardlink      ; make a hard link
         | :symlink       ; make a symbolic link
         | :rename        ; move
         | :copy (or nil) ; make a copy
  if the underlying file system does not support a given operation
  a copy is made

 preserve := t ;; preserve as much of source-file's attributes as
               ;; possible
           | nil ;; don't try to preserve source-file's attributes
                 ;; when creating target-file
 for target:
 if-exists := :supersede ;; the existing file is superseded. that is
                         ;; a new file with the same name (and
                         ;; attributes if possible) is created.
            | :error ;; an error of type file-error is signaled.
            | :new-version ;; a new file is created with a larger
                           ;; version number
            | :rename ;; the existing file is renamed to "orig.bak"
            | :append ;; the contents of source-file are appended to
                      ;; the end of target-file
 for source:
 if-does-not-exist := nil ;; do nothing and return nil
                    | :error ;; (default) signal an error
 */
#line 3198
DEFUN(POSIX::COPY-FILE, source target &key METHOD :PRESERVE            :IF-EXISTS :IF-DOES-NOT-EXIST,(subr_posix_copy_file,seclass_default,2,0,norest,key,4,NIL)) {
  if_does_not_exist_t if_not_exists = check_if_does_not_exist(STACK_0);
  if_exists_t if_exists = check_if_exists(STACK_1);
  bool preserve_p = (!nullp(STACK_2) && boundp(STACK_2));
  bool wild_source_p, wild_dest_p;
  copy_method_t method = check_copy_method(STACK_3);
  STACK_1 = NIL; /* return value */
  /* stack: 5 - source; 4 - dest */
  pushSTACK(STACK_5); funcall(L(pathname),1); STACK_3 = value1;
  pushSTACK(STACK_3); funcall(L(wild_pathname_p),1);
  wild_source_p = !nullp(value1);
  pushSTACK(STACK_4); funcall(L(pathname),1); STACK_2 = value1;
  pushSTACK(STACK_2); funcall(L(wild_pathname_p),1);
  wild_dest_p = !nullp(value1);
  XOUT(STACK_3,"POSIX::COPY-FILE -- source");
  XOUT(STACK_2,"POSIX::COPY-FILE -- dest");
  if (wild_source_p) {
    pushSTACK(STACK_3);         /* source pathname */
    pushSTACK(S(Kif_does_not_exist)); pushSTACK(S(Kdiscard));
    funcall(L(directory),3);
    STACK_0 = value1;
    XOUT(STACK_0,"POSIX::COPY-FILE: source list");
    if (wild_dest_p) {
      while (!nullp(STACK_0)) {
        pushSTACK(Car(STACK_0)); /* truename */
        pushSTACK(STACK_(3+1)); /* source */
        pushSTACK(STACK_(2+2)); /* dest */
        funcall(L(translate_pathname),3);
        copy_one_file(NIL,Car(STACK_0),NIL,value1,method,
                      preserve_p,if_exists,if_not_exists,&STACK_1);
        STACK_0 = Cdr(STACK_0);
      }
    } else { /* non-wild dest, must be a directory */
      pushSTACK(STACK_2); funcall(L(probe_directory),1);
      if (nullp(value1)) {      /* dest is a non-exitent dir */
        pushSTACK(STACK_2); funcall(L(make_directory),1);
      }
      while (!nullp(STACK_0)) {
        copy_one_file(NIL,Car(STACK_0),STACK_4,STACK_2,method,
                      preserve_p,if_exists,if_not_exists,&STACK_1);
        STACK_0 = Cdr(STACK_0);
      }
    }
  } else /* non-wild source */
    copy_one_file(STACK_5,STACK_3,STACK_4,STACK_2,method,preserve_p,
                  if_exists,if_not_exists,&STACK_1);
  VALUES1(STACK_1);
  skipSTACK(6);
}

DEFUN(POSIX::DUPLICATE-HANDLE, old &optional new,(subr_posix_duplicate_handle,seclass_default,1,1,norest,nokey,0,NIL))
{ /* Lisp interface to dup(2)/dup2(2). */
  Handle new_handle = (Handle)check_uint_defaulted(popSTACK(),(uintL)-1);
  Handle old_handle = (Handle)I_to_uint(check_uint(popSTACK()));
  begin_blocking_system_call();
  if (new_handle == (Handle)(uintL)-1)
    new_handle = handle_dup(old_handle);
  else
    new_handle = handle_dup2(old_handle,new_handle);
  end_blocking_system_call();
  VALUES1(fixnum(new_handle));
}

#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
#include <shlobj.h>
/* for CLSID_ShellLink & IID_IShellLink */
#include <shldisp.h>
#include <shlguid.h>

/* also exists in w32shell.c
   redefining here for compilation with cygwin
   where no use of COM is made in base set */

static HRESULT BTCoCreateInstance(REFCLSID rclsid,  LPUNKNOWN pUnkOuter,
                                  DWORD dwClsContext, REFIID riid,
                                  LPVOID * ppv )
{
  HRESULT result;
  result = CoCreateInstance(rclsid, pUnkOuter, dwClsContext, riid, ppv);
  if (result != CO_E_NOTINITIALIZED
      || CoInitialize(NULL) != S_OK) return result;
  return CoCreateInstance(rclsid, pUnkOuter, dwClsContext, riid, ppv);
}

#line 3286

DEFUN(POSIX::CONVERT-ATTRIBUTES, attributes,(subr_posix_convert_attributes,seclass_default,1,0,norest,nokey,0,NIL))
{ /* convert between symbolic and numeric file attributes */
  if (posfixnump(STACK_0))
    VALUES1(check_file_attributes_to_list
            (I_to_uint32(check_uint32(popSTACK()))));
  else if (listp(STACK_0))
    VALUES1(fixnum(check_file_attributes_of_list(popSTACK())));
  else VALUES1(fixnum(check_file_attributes(popSTACK())));
}
/* convert the 8 members of WIN32_FIND_DATA to the FILE-INFO struct
 can trigger GC */
static Values wfd_to_file_info (WIN32_FIND_DATA *wfd) {
  pushSTACK(check_file_attributes_to_list(wfd->dwFileAttributes));
  pushSTACK(convert_time_to_universal_w32(&(wfd->ftCreationTime)));
  pushSTACK(convert_time_to_universal_w32(&(wfd->ftLastAccessTime)));
  pushSTACK(convert_time_to_universal_w32(&(wfd->ftLastWriteTime)));
  pushSTACK(UL2_to_I(wfd->nFileSizeHigh,wfd->nFileSizeLow));
  pushSTACK(asciz_to_string(wfd->cFileName,GLO(pathname_encoding)));
  pushSTACK(asciz_to_string(wfd->cAlternateFileName,GLO(pathname_encoding)));
  funcall(O(object_posix__make_file_info),7);
}

DEFUN(POSIX::FILE-INFO, file &optional all,(subr_posix_file_info,seclass_default,1,1,norest,nokey,0,NIL)) {
  WIN32_FIND_DATA wfd;
  if (missingp(STACK_0)) {
    find_first_file(STACK_1,&wfd,NULL);
    wfd_to_file_info(&wfd);
  } else {
    HANDLE sh;
    gcv_object_t *phys = &STACK_0;
    unsigned int count = 1;
    find_first_file(STACK_1,&wfd,&sh); *phys = value1; /* physical name */
    wfd_to_file_info(&wfd); pushSTACK(value1);
    while (1) {
      begin_blocking_system_call();
      if (!FindNextFile(sh,&wfd)) {
        end_blocking_system_call();
        if (GetLastError() == ERROR_NO_MORE_FILES) break;
        OS_file_error(*phys);
      }
      end_blocking_system_call();
      wfd_to_file_info(&wfd); pushSTACK(value1); count++;
    }
    begin_blocking_system_call(); FindClose(sh); end_blocking_system_call();
    VALUES1(listof(count));
  }
  skipSTACK(2);                 /* drop arguments */
}

#line 3337
DEFUN(POSIX::MAKE-SHORTCUT, file &key WORKING-DIRECTORY ARGUMENTS        SHOW-COMMAND ICON DESCRIPTION HOT-KEY PATH,(subr_posix_make_shortcut,seclass_default,1,0,norest,key,7,NIL)) {
  HRESULT hres;
  IShellLink* psl;
  IPersistFile* ppf;
  gcv_object_t *file = &STACK_7;

  /* Get a pointer to the IShellLink interface. */
  begin_blocking_system_call();
  hres = BTCoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
                            &IID_IShellLink, (LPVOID*)&psl);
  if (!SUCCEEDED(hres)) goto fail_none;
  end_blocking_system_call();
  if (!missingp(STACK_0)) {     /* PATH */
    object path = check_string(STACK_0);
    with_string_0(path,GLO(pathname_encoding),pathz, {
      begin_blocking_system_call();
      hres = psl->lpVtbl->SetPath(psl,pathz);
      if (!SUCCEEDED(hres)) goto fail_psl;
      end_blocking_system_call();
    });
  }
  skipSTACK(1);                 /* drop PATH */
  if (!missingp(STACK_0)) {     /* HOT-KEY */
    WORD hot_key = 0;
    object hk = STACK_0;
    BYTE *pb = (BYTE*)&hot_key;
   restart_hot_key:
    if (charp(hk)) hot_key = char_int(hk);
    else while (consp(hk)) {
      if (eq(Car(hk),O(object_Kcontrol))) pb[1] |= HOTKEYF_CONTROL;
      else if (eq(Car(hk),O(object_Kalt))) pb[1] |= HOTKEYF_ALT;
      else if (eq(Car(hk),O(object_Kext))) pb[1] |= HOTKEYF_EXT;
      else if (eq(Car(hk),O(object_Kshift))) pb[1] |= HOTKEYF_SHIFT;
      else if (charp(Car(hk))) {
        pb[0] = char_int(hk);
        break;
      } else {
        pushSTACK(NIL);         /* no PLACE */
        pushSTACK(hk);          /* TYPE-ERROR slot DATUM */
        pushSTACK(O(object__28member_20_alt_20_control_20_ext_20_shift_29)); /* EXPECTED-TYPE */
        pushSTACK(STACK_0); pushSTACK(hk); pushSTACK(TheSubr(subr_self)->name);
        check_value(type_error,GETTEXT("~S: ~S is not a ~S"));
        hk = value1;
        goto restart_hot_key;
      }
      hk = Cdr(hk);
    }
    if (pb[0] == 0) {           /* STACK_0 is the HOT-KEY arg */
      pushSTACK(TheSubr(subr_self)->name);
      error(error_condition,GETTEXT("~S: invalid hotkey spec ~S"));
    }
    begin_blocking_system_call();
    hres = psl->lpVtbl->SetHotkey(psl,hot_key);
    if (!SUCCEEDED(hres)) goto fail_psl;
    end_blocking_system_call();
  }
  skipSTACK(1);                 /* drop HOT-KEY */
  if (!missingp(STACK_0)) {     /* DESCRIPTION */
    object desc = check_string(STACK_0);
    with_string_0(desc,GLO(pathname_encoding),descz, {
      begin_blocking_system_call();
      hres = psl->lpVtbl->SetDescription(psl,descz);
      if (!SUCCEEDED(hres)) goto fail_psl;
      end_blocking_system_call();
    });
  }
  skipSTACK(1);                 /* drop DESCRIPTION */
  if (!missingp(STACK_0)) {     /* ICON */
    object icon_name;
    int icon_idx = 0;
    if (consp(STACK_0)) {       /* (file . index) or (file index) */
      icon_name = check_string(Car(STACK_0));
      icon_idx = I_to_uint32(check_uint32(consp(Cdr(STACK_0))
                                          ? Car(Cdr(STACK_0))
                                          : Cdr(STACK_0)));
    } else icon_name = check_string(STACK_0);
    with_string_0(icon_name,GLO(pathname_encoding),iconz, {
      begin_blocking_system_call();
      hres = psl->lpVtbl->SetIconLocation(psl,iconz,icon_idx);
      if (!SUCCEEDED(hres)) goto fail_psl;
      end_blocking_system_call();
    });
  }
  skipSTACK(1);                 /* drop ICON */
  if (!missingp(STACK_0)) {     /* SHOW-COMMAND */
    object sc = STACK_0;
    int sci;
   restart_show_command:
    if (eq(sc,S(Knormal))) sci = SW_SHOWNORMAL;
    else if (eq(sc,O(object_Kmax))) sci = SW_SHOWMAXIMIZED;
    else if (eq(sc,O(object_Kmin))) sci = SW_SHOWMINIMIZED;
    else {
      pushSTACK(NIL);           /* no PLACE */
      pushSTACK(sc);            /* TYPE-ERROR slot DATUM */
      pushSTACK(O(object__28member_20_normal_20_max_20_min_29)); /* EXPECTED-TYPE */
      pushSTACK(STACK_0); pushSTACK(sc); pushSTACK(TheSubr(subr_self)->name);
      check_value(type_error,GETTEXT("~S: ~S is not a ~S"));
      sc = value1;
      goto restart_show_command;
    }
    begin_blocking_system_call();
    hres = psl->lpVtbl->SetShowCmd(psl,sci);
    if (!SUCCEEDED(hres)) goto fail_psl;
    end_blocking_system_call();
  }
  skipSTACK(1);                 /* drop SHOW-COMMAND */
  if (!missingp(STACK_0)) {     /* ARGUMENTS */
    object args = check_string(STACK_0);
    with_string_0(args,GLO(pathname_encoding),argz, {
      begin_blocking_system_call();
      hres = psl->lpVtbl->SetArguments(psl,argz);
      if (!SUCCEEDED(hres)) goto fail_psl;
      end_blocking_system_call();
    });
  }
  skipSTACK(1);                 /* drop ARGUMENTS */
  if (!missingp(STACK_0)) {     /* WORKING-DIRECTORY */
    object wd = check_string(STACK_0);
    with_string_0(wd,GLO(pathname_encoding),wdz, {
      begin_blocking_system_call();
      hres = psl->lpVtbl->SetWorkingDirectory(psl,wdz);
      if (!SUCCEEDED(hres)) goto fail_psl;
      end_blocking_system_call();
    });
  }
  skipSTACK(1);                 /* drop WORKING-DIRECTORY */
  STACK_0 = physical_namestring(STACK_0); /* pathname */

  begin_blocking_system_call();
  hres = psl->lpVtbl->QueryInterface(psl,&IID_IPersistFile,(LPVOID*)&ppf);
  if (!SUCCEEDED(hres)) goto fail_psl;
  { /* Ensure that the string is Unicode & Save the shortcut. */
    WCHAR wsz[MAX_PATH];
    with_string_0(*file, GLO(pathname_encoding), pathz, {
      MultiByteToWideChar(CP_ACP, 0, pathz, -1, wsz, MAX_PATH);
      hres = ppf->lpVtbl->Save(ppf, wsz, TRUE);
      if (!SUCCEEDED(hres)) goto fail_ppf;
    });
  }
  ppf->lpVtbl->Release(ppf);
  psl->lpVtbl->Release(psl);
  end_blocking_system_call();
  VALUES1(popSTACK()); return;
 fail_ppf: ppf->lpVtbl->Release(ppf);
 fail_psl: psl->lpVtbl->Release(psl);
 fail_none: end_blocking_system_call(); OS_file_error(*file);
}

DEFUN(POSIX::SHORTCUT-INFO, file,(subr_posix_shortcut_info,seclass_default,1,0,norest,nokey,0,NIL)) {
  HRESULT hres;
  IShellLink* psl;
  char path[MAX_PATH], wd[MAX_PATH], args[MAX_PATH],
    icon[MAX_PATH], desc[MAX_PATH];
  WIN32_FIND_DATA wfd;
  IPersistFile* ppf;
  gcv_object_t *file = &STACK_0;
  int icon_idx, show_cmd;
  WORD hot_key;

  STACK_0 = physical_namestring(STACK_0);

  /* Get a pointer to the IShellLink interface. */
  begin_blocking_system_call();
  hres = BTCoCreateInstance(&CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER,
                            &IID_IShellLink, (LPVOID*)&psl);
  if (!SUCCEEDED(hres)) goto fail_none;
  /* Get a pointer to the IPersistFile interface. */
  hres = psl->lpVtbl->QueryInterface(psl,&IID_IPersistFile,(LPVOID*)&ppf);
  if (!SUCCEEDED(hres)) goto fail_psl;
  { /* Ensure that the string is Unicode & Load the shortcut. */
    WCHAR wsz[MAX_PATH];
    with_string_0(STACK_0, GLO(pathname_encoding), pathz, {
      MultiByteToWideChar(CP_ACP, 0, pathz, -1, wsz, MAX_PATH);
      hres = ppf->lpVtbl->Load(ppf, wsz, STGM_READ);
      if (!SUCCEEDED(hres)) goto fail_ppf;
    });
  }
  /* Resolve the link. */
  hres = psl->lpVtbl->Resolve(psl,NULL,0);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  /* 1 path, 2 file info */
  hres = psl->lpVtbl->GetPath(psl,path, MAX_PATH, &wfd, 4/*SLGP_RAWPATH*/);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  /* 3 working directory */
  hres = psl->lpVtbl->GetWorkingDirectory(psl,wd, MAX_PATH);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  /* 4 arguments */
  hres = psl->lpVtbl->GetArguments(psl,args, MAX_PATH);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  /* 5 show command */
  hres = psl->lpVtbl->GetShowCmd(psl,&show_cmd);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  /* 6 icon */
  hres = psl->lpVtbl->GetIconLocation(psl,icon, MAX_PATH, &icon_idx);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  /* 7 description */
  hres = psl->lpVtbl->GetDescription(psl,desc, MAX_PATH);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  /* 8 hot key */
  hres = psl->lpVtbl->GetHotkey(psl,&hot_key);
  if (!SUCCEEDED(hres)) goto fail_ppf;
  ppf->lpVtbl->Release(ppf);
  psl->lpVtbl->Release(psl);
  end_blocking_system_call();
  pushSTACK(asciz_to_string(path,GLO(pathname_encoding))); /* 1 */
  wfd_to_file_info(&wfd); pushSTACK(value1);               /* 2 */
  pushSTACK(asciz_to_string(wd,GLO(pathname_encoding)));   /* 3 */
  pushSTACK(asciz_to_string(args,GLO(pathname_encoding))); /* 4 */
  switch (show_cmd) {                                   /* 5 */
    case SW_SHOWNORMAL: pushSTACK(S(Knormal)); break;
    case SW_SHOWMAXIMIZED: pushSTACK(O(object_Kmax)); break;
    case SW_SHOWMINIMIZED: pushSTACK(O(object_Kmin)); break;
    default: NOTREACHED;
  }
  pushSTACK(asciz_to_string(icon,GLO(pathname_encoding)));
  pushSTACK(fixnum(icon_idx));
  { object tmp = listof(2); pushSTACK(tmp); }           /* 6 */
  pushSTACK(asciz_to_string(desc,GLO(pathname_encoding))); /* 7 */
  { int count=0;                                        /* 8 */
    BYTE *pb = (BYTE*)&hot_key;
    if (pb[1] & HOTKEYF_ALT) { pushSTACK(O(object_Kalt)); count++; }
    if (pb[1] & HOTKEYF_CONTROL) { pushSTACK(O(object_Kcontrol)); count++; }
    if (pb[1] & HOTKEYF_EXT) { pushSTACK(O(object_Kext)); count++; }
    if (pb[1] & HOTKEYF_SHIFT) { pushSTACK(O(object_Kshift)); count++; }
    pushSTACK(int_char(pb[0]));
    if (count) { object tmp = listof(count+1); pushSTACK(tmp); }
  }
  funcall(O(object_posix__make_shortcut_info),9);
  return;
 fail_ppf: ppf->lpVtbl->Release(ppf);
 fail_psl: psl->lpVtbl->Release(psl);
 fail_none: end_blocking_system_call(); OS_file_error(*file);
}

#line 3573

DEFUN(POSIX::SYSTEM-INFO,,(subr_posix_system_info,seclass_default,0,0,norest,nokey,0,NIL))
{ /* interface to GetSystemInfo() */
  SYSTEM_INFO si;
  begin_system_call();
  GetSystemInfo(&si);
  end_system_call();
  pushSTACK(processor_architecture_reverse(si.wProcessorArchitecture));
  pushSTACK(UL_to_I(si.dwPageSize));
  pushSTACK(UL_to_I((DWORD)si.lpMinimumApplicationAddress));
  pushSTACK(UL_to_I((DWORD)si.lpMaximumApplicationAddress));
  pushSTACK(UL_to_I(si.dwActiveProcessorMask));
  pushSTACK(UL_to_I(si.dwNumberOfProcessors));
  pushSTACK(UL_to_I(si.dwProcessorType));
  pushSTACK(UL_to_I(si.dwAllocationGranularity));
  pushSTACK(fixnum(si.wProcessorLevel));
  pushSTACK(fixnum(si.wProcessorRevision));
  funcall(O(object_posix__make_system_info),10);
}

DEFUN(POSIX::VERSION,,(subr_posix_version,seclass_default,0,0,norest,nokey,0,NIL))
{ /* interface to GetVersionEx() */
  OSVERSIONINFOEX vi;
  bool status;
  vi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
  begin_system_call();
  status = GetVersionEx((OSVERSIONINFO*)&vi);
  end_system_call();
  if (!status) OS_error();

  pushSTACK(UL_to_I(vi.dwMajorVersion));
  pushSTACK(UL_to_I(vi.dwMinorVersion));
  pushSTACK(UL_to_I(vi.dwBuildNumber));
  switch (vi.dwPlatformId) {
    case VER_PLATFORM_WIN32s:        pushSTACK(O(object_Ks)); break;
    case VER_PLATFORM_WIN32_WINDOWS: pushSTACK(O(object_Kwindows)); break;
    case VER_PLATFORM_WIN32_NT:      pushSTACK(O(object_Knt)); break;
    default: pushSTACK(UL_to_I(vi.dwPlatformId));
  }
  pushSTACK(safe_to_string(vi.szCSDVersion));
  pushSTACK(UL_to_I(vi.wServicePackMajor));
  pushSTACK(UL_to_I(vi.wServicePackMinor));
  { /* wSuiteMask */
    object suites = NIL;
    unsigned int count = 0;
    if (vi.wSuiteMask & VER_SUITE_BACKOFFICE)
      { pushSTACK(O(object_Kbackoffice)); count++; }
    if (vi.wSuiteMask & VER_SUITE_DATACENTER)
      { pushSTACK(O(object_Kdatacenter)); count++; }
    if (vi.wSuiteMask & VER_SUITE_ENTERPRISE)
      { pushSTACK(O(object_Kenterprise)); count++; }
    if (vi.wSuiteMask & VER_SUITE_SMALLBUSINESS)
      { pushSTACK(O(object_Ksmallbusiness)); count++; }
    if (vi.wSuiteMask & VER_SUITE_SMALLBUSINESS_RESTRICTED)
      { pushSTACK(O(object_Ksmallbusiness_restricted)); count++; }
    if (vi.wSuiteMask & VER_SUITE_TERMINAL)
      { pushSTACK(O(object_Kterminal)); count++; }
    if (vi.wSuiteMask & VER_SUITE_PERSONAL)
      { pushSTACK(O(object_Kpersonal)); count++; }
    if (count) suites = listof(count);
    pushSTACK(suites);
  }
  switch (vi.wProductType) {
    case VER_NT_WORKSTATION:       pushSTACK(O(object_Kworkstation)); break;
    case VER_NT_DOMAIN_CONTROLLER: pushSTACK(O(object_Kdomain_controller)); break;
    case VER_NT_SERVER:            pushSTACK(O(object_Kserver)); break;
    default: pushSTACK(UL_to_I(vi.wProductType));
  }
  funcall(O(object_posix__make_version),9);
}

DEFUN(POSIX::MEMORY-STATUS,,(subr_posix_memory_status,seclass_default,0,0,norest,nokey,0,NIL))
{ /* interface to GlobalMemoryStatus() */
#ifdef HAVE_GLOBALMEMORYSTATUSEX
  MEMORYSTATUSEX ms;
  bool status;
  ms.dwLength = sizeof(MEMORYSTATUSEX);
  begin_system_call();
  status = GlobalMemoryStatusEx(&ms);
  end_system_call();
  if (!status) OS_error();
  pushSTACK(UQ_to_I(ms.ullTotalPhys));
  pushSTACK(UQ_to_I(ms.ullAvailPhys));
  pushSTACK(UQ_to_I(ms.ullTotalPageFile));
  pushSTACK(UQ_to_I(ms.ullAvailPageFile));
  pushSTACK(UQ_to_I(ms.ullTotalVirtual));
  pushSTACK(UQ_to_I(ms.ullAvailVirtual));
#else
  MEMORYSTATUS ms;
  ms.dwLength = sizeof(MEMORYSTATUS);
  begin_system_call(); GlobalMemoryStatus(&ms); end_system_call();
  pushSTACK(UL_to_I(ms.dwTotalPhys));
  pushSTACK(UL_to_I(ms.dwAvailPhys));
  pushSTACK(UL_to_I(ms.dwTotalPageFile));
  pushSTACK(UL_to_I(ms.dwAvailPageFile));
  pushSTACK(UL_to_I(ms.dwTotalVirtual));
  pushSTACK(UL_to_I(ms.dwAvailVirtual));
#endif
  funcall(O(object_posix__mkmemstat),6);
}

/* FILE-PROPERTIES */

#ifndef PIDSI_TITLE
#define PIDSI_TITLE               0x00000002L
#define PIDSI_SUBJECT             0x00000003L
#define PIDSI_AUTHOR              0x00000004L
#define PIDSI_KEYWORDS            0x00000005L
#define PIDSI_COMMENTS            0x00000006L
#define PIDSI_TEMPLATE            0x00000007L
#define PIDSI_LASTAUTHOR          0x00000008L
#define PIDSI_REVNUMBER           0x00000009L
#define PIDSI_EDITTIME            0x0000000aL
#define PIDSI_LASTPRINTED         0x0000000bL
#define PIDSI_CREATE_DTM          0x0000000cL
#define PIDSI_LASTSAVE_DTM        0x0000000dL
#define PIDSI_PAGECOUNT           0x0000000eL
#define PIDSI_WORDCOUNT           0x0000000fL
#define PIDSI_CHARCOUNT           0x00000010L
#define PIDSI_THUMBNAIL           0x00000011L
#define PIDSI_APPNAME             0x00000012L
#define PIDSI_DOC_SECURITY        0x00000013L
#define PRSPEC_LPWSTR	( 0 )
#define PRSPEC_PROPID	( 1 )
#define STG_E_PROPSETMISMATCHED   0x800300F0L
#endif

/* Pushes corresponding value to STACK */
static int PropVariantToLisp (PROPVARIANT *pvar) {
  if(pvar->vt & VT_ARRAY) {
    pushSTACK(S(Karray));
    return 1;
  }
  if(pvar->vt & VT_BYREF) {
    pushSTACK(O(object_Kbyref));
    return 1;
  }
  switch(pvar->vt) {
    case VT_EMPTY: pushSTACK(O(object_Kempty)); break;
    case VT_NULL:  pushSTACK(O(object_Knull));  break;
    case VT_BLOB:  pushSTACK(O(object_Kblob));  break;
    case VT_BOOL:  pushSTACK(pvar->boolVal ? T : NIL); break;
    case VT_I1:    pushSTACK(sfixnum(pvar->cVal)); break;
    case VT_UI1:   pushSTACK(fixnum(pvar->bVal)); break;
    case VT_I2:    pushSTACK(sfixnum(pvar->iVal)); break;
    case VT_UI2:   pushSTACK(fixnum(pvar->uiVal)); break;
    case VT_I4:
    case VT_INT:   pushSTACK(L_to_I(pvar->lVal)); break;
    case VT_UI4:
    case VT_UINT:  pushSTACK(UL_to_I(pvar->ulVal)); break;
    case VT_ERROR: pushSTACK(UL_to_I(pvar->scode)); break;
    case VT_I8:    pushSTACK(sint64_to_I(*((sint64 *)&pvar->hVal))); break;
    case VT_CY: {
      double dbl = (*((sint64 *)&pvar->cyVal))/10000.0;
      pushSTACK(c_double_to_DF((dfloatjanus *)&dbl));
    } break;
    case VT_UI8: pushSTACK(uint64_to_I(*((uint64 *)&pvar->uhVal)));  break;
    case VT_R4:  pushSTACK(c_float_to_FF((ffloatjanus *)&pvar->fltVal)); break;
    case VT_R8:  pushSTACK(c_double_to_DF((dfloatjanus *)&pvar->dblVal));break;
    case VT_DATE:pushSTACK(c_double_to_DF((dfloatjanus *)&pvar->date)); break;
    case VT_BSTR:
      pushSTACK(n_char_to_string((const char *)pvar->bstrVal,
                                 *((DWORD *)(((const char *)pvar->bstrVal)-4)),
                                 Symbol_value(S(unicode_16_little_endian))));
      break;
    case VT_LPSTR:
      pushSTACK(safe_to_string(pvar->pszVal));
      break;
    case VT_LPWSTR:
      pushSTACK(n_char_to_string((const char *)pvar->pwszVal,
                                 wcslen(pvar->pwszVal)*2,
                                 Symbol_value(S(unicode_16_little_endian))));
      break;
    case VT_FILETIME:
      pushSTACK(convert_time_to_universal_w32(&(pvar->filetime))); break;
    case VT_CF: pushSTACK(O(object_Kclipboard_format)); break;
    default:    pushSTACK(O(object_Knotimplemented)); break;
  }
  return 1;
}
/* popSTACK -> pvar  */
static int LispToPropVariant (PROPVARIANT * pvar) {
  int rv = 0;int sfp = 0;
  VARTYPE typehint = VT_EMPTY;
  if (consp(STACK_0)) {
    /* (KW VALUE) OR (KW NIL) ? */
    if (!nullp(Cdr(STACK_0)) && !nullp(Car(STACK_0))
        && consp(Cdr(STACK_0)) && nullp(Cdr(Cdr(STACK_0)))
        && symbolp(Car(STACK_0))) {
           if (eq(Car(STACK_0),O(object_Ki1))) typehint = VT_I1;
      else if (eq(Car(STACK_0),O(object_Kui1))) typehint = VT_UI1;
      else if (eq(Car(STACK_0),O(object_Ki2))) typehint = VT_I2;
      else if (eq(Car(STACK_0),O(object_Kui2))) typehint = VT_UI2;
      else if (eq(Car(STACK_0),O(object_Ki4))) typehint = VT_I4;
      else if (eq(Car(STACK_0),O(object_Kint))) typehint = VT_INT;
      else if (eq(Car(STACK_0),O(object_Kui4))) typehint = VT_UI4;
      else if (eq(Car(STACK_0),O(object_Kuint))) typehint = VT_UINT;
      else if (eq(Car(STACK_0),O(object_Ki8))) typehint = VT_I8;
      else if (eq(Car(STACK_0),O(object_Kui8))) typehint = VT_UI8;
      else if (eq(Car(STACK_0),O(object_Kr4))) typehint = VT_R4;
      else if (eq(Car(STACK_0),O(object_Kr8))) typehint = VT_R8;
      else if (eq(Car(STACK_0),O(object_Kcy))) typehint = VT_CY;
      else if (eq(Car(STACK_0),O(object_Kdate))) typehint = VT_DATE;
      else if (eq(Car(STACK_0),O(object_Kbstr))) typehint = VT_BSTR;
      else if (eq(Car(STACK_0),O(object_Kbool))) typehint = VT_BOOL;
      else if (eq(Car(STACK_0),O(object_Kerror))) typehint = VT_ERROR;
      else if (eq(Car(STACK_0),O(object_Kfiletime))) typehint = VT_FILETIME;
      else if (eq(Car(STACK_0),O(object_Klpstr))) typehint = VT_LPSTR;
      else if (eq(Car(STACK_0),O(object_Klpwstr))) typehint = VT_LPWSTR;
      else { skipSTACK(1); return 0; }
      STACK_0 = Car(Cdr(STACK_0)); /* VALUE */
    } else { skipSTACK(1); return 0; }
  }
  if (stringp(STACK_0)
      && (typehint == VT_EMPTY || typehint == VT_BSTR
          || typehint == VT_LPSTR || typehint == VT_LPWSTR)) {
    if (typehint == VT_EMPTY) {
#    define STG_STRINGS_NONUNICODE
#    ifdef STG_STRINGS_UNICODE
      typehint = VT_LPWSTR;
#    else
      typehint = VT_LPSTR;
#    endif
    }
    do {
      uintL str_len;
      uintL str_offset;
      object str_string = unpack_string_ro(STACK_0,&str_len,&str_offset);
      const chart* ptr1;
      unpack_sstring_alloca(str_string,str_len,str_offset, ptr1=);
      if (typehint == VT_LPWSTR || typehint == VT_BSTR) {
        uintL str_bytelen =
          cslen(Symbol_value(S(unicode_16_little_endian)),ptr1,str_len);
        LPWSTR str = SysAllocStringByteLen(NULL,str_bytelen+4);
        if (typehint == VT_BSTR) {
          /* it's ok, SysAllocStringByteLen returns pointer after DWORD */
          *(((DWORD *)str)-1) = (DWORD)str_bytelen;
        }
        cstombs(Symbol_value(S(unicode_16_little_endian)),ptr1,str_len,
                (uintB *)str,str_bytelen);
        ((uintB *)str)[str_bytelen] = '\0';
        ((uintB *)str)[str_bytelen+1] = '\0';
        pvar->pwszVal = str;
        pvar->vt = typehint;
      } else { /* Win XP explorer seems to create ANSI strings. So do we. */
        uintL str_bytelen = cslen(GLO(misc_encoding),ptr1,str_len);
        char * str = (char *) SysAllocStringByteLen(NULL, str_bytelen+2);
        cstombs(GLO(misc_encoding),ptr1,str_len,(uintB *)str,str_bytelen);
        str[str_bytelen] = '\0';
        pvar->pszVal = str;
        pvar->vt = VT_LPSTR;
      }
      rv = 1;
    } while(0);
  } else if (integerp(STACK_0)) {
    if (typehint == VT_EMPTY) typehint = VT_FILETIME; /* assume FILETIME */
    if (typehint == VT_FILETIME) {
      pvar->vt = VT_FILETIME; rv = 1;
      convert_time_from_universal_w32(STACK_0,&(pvar->filetime));
    } else if (typehint == VT_I1) {
      pvar->vt = typehint; pvar->cVal = I_to_sint8(STACK_0); rv = 1;
    } else if (typehint == VT_UI1) {
      pvar->vt = typehint; pvar->bVal = I_to_uint8(STACK_0); rv = 1;
    } else if (typehint == VT_I2) {
      pvar->vt = typehint; pvar->iVal = I_to_sint16(STACK_0); rv = 1;
    } else if (typehint == VT_UI2) {
      pvar->vt = typehint; pvar->uiVal = I_to_uint16(STACK_0); rv = 1;
    } else if (typehint == VT_I4 || typehint == VT_INT) { /* VT_I4 != VT_INT */
      pvar->vt = typehint; pvar->lVal = I_to_sint32(STACK_0); rv = 1;
    } else if (typehint == VT_UI4 || typehint == VT_UINT) {
      pvar->vt = typehint; pvar->ulVal = I_to_uint32(STACK_0); rv = 1;
    } else if (typehint == VT_ERROR) {
      pvar->vt = typehint; pvar->scode = I_to_uint32(STACK_0); rv = 1;
    } else if (typehint == VT_I8) {
      pvar->vt = typehint;
      *((sint64 *)&pvar->hVal) = I_to_sint64(STACK_0);rv = 1;
    } else if (typehint == VT_UI8) {
      pvar->vt = typehint;
      *((uint64 *)&pvar->uhVal) = I_to_uint64(STACK_0);rv = 1;
    } else if (typehint == VT_CY) {
      sint64 i64 = I_to_uint64(STACK_0);
      pvar->vt = typehint;
      *((uint64 *)&pvar->cyVal) = i64*10000;rv = 1;
    }
  } else if ((sfp = single_float_p(STACK_0)) || double_float_p(STACK_0)) {
    if (typehint == VT_EMPTY) typehint = (sfp?VT_R4:VT_R8);
    if (typehint == VT_R4) {
      if (sfp) {
        pvar->vt = VT_R4;
        pvar->fltVal = 0;
        FF_to_c_float(STACK_0,(ffloatjanus *)&pvar->fltVal);
        rv = 1;
      }
    } else if (typehint == VT_R8) {
      pvar->vt = VT_R8;
      if (sfp) {
        float v = 0;
        FF_to_c_float(STACK_0,(ffloatjanus *)&v);
        pvar->dblVal = v;
      } else {
        pvar->dblVal = 0; /* DF_to_c_double takes only clean doubles */
        DF_to_c_double(STACK_0,(dfloatjanus *)&pvar->dblVal);
      }
      rv = 1;
    } else if (typehint == VT_DATE && double_float_p(STACK_0)) {
      /* A 64-bit floating point number representing the number of days
         (not seconds) since December 31, 1899. For example, January 1, 1900,
         is 2.0, January 2, 1900, is 3.0, and so on). This is stored in the
         same representation as VT_R8. */
      pvar->vt = VT_DATE;
      pvar->date = 0;
      DF_to_c_double(STACK_0,(dfloatjanus *)&pvar->date);
      rv = 1;
    } else if (typehint == VT_CY) {
      double dbl = 0; float v = 0;
      pvar->vt = typehint;
      if (sfp) {
        FF_to_c_float(STACK_0,(ffloatjanus *)&v);
        dbl = v;
      } else {
        DF_to_c_double(STACK_0,(dfloatjanus *)&dbl);
      }
      *((uint64 *)&pvar->cyVal) = (uint64) (dbl*10000 + 0.5);rv = 1;
    }
  } else if (symbolp(STACK_0)) {
    if (typehint == VT_EMPTY && eq(STACK_0,O(object_Kempty))) {
      pvar->vt = VT_EMPTY; rv = 1; } else
    if (typehint == VT_EMPTY && eq(STACK_0,O(object_Knull))) {
      pvar->vt = VT_NULL;  rv = 1; } else
    if (typehint == VT_BOOL && eq(STACK_0,NIL)) {
      pvar->vt = VT_BOOL; pvar->boolVal = FALSE;  rv = 1; } else
    if (typehint == VT_BOOL && eq(STACK_0,T)) {
      pvar->vt = VT_BOOL; pvar->boolVal = TRUE;  rv = 1; }
  }
  skipSTACK(1);
  return rv;
}

WINOLEAPI PropVariantClear(PROPVARIANT* pvar);

static PROPID kwtopropid (object kw) {
  if (eq(kw,O(object_Kcodepage))) return 1 /* PID_CODEPAGE */;
  if (eq(kw,O(object_Klocale))) return 0x80000000 /* PID_LOCALE */;
  if (eq(kw,O(object_Ktitle))) return PIDSI_TITLE;
  if (eq(kw,O(object_Ksubject))) return PIDSI_SUBJECT;
  if (eq(kw,O(object_Kauthor))) return PIDSI_AUTHOR;
  if (eq(kw,O(object_Kkeywords))) return PIDSI_KEYWORDS;
  if (eq(kw,O(object_Kcomments))) return PIDSI_COMMENTS;
  if (eq(kw,O(object_Ktemplate))) return PIDSI_TEMPLATE;
  if (eq(kw,O(object_Klastauthor))) return PIDSI_LASTAUTHOR;
  if (eq(kw,O(object_Krevnumber))) return PIDSI_REVNUMBER;
  if (eq(kw,O(object_Kedittime))) return PIDSI_EDITTIME;
  if (eq(kw,O(object_Klastprinted))) return PIDSI_LASTPRINTED;
  if (eq(kw,O(object_Kcreate_dtm))) return PIDSI_CREATE_DTM;
  if (eq(kw,O(object_Klastsave_dtm))) return PIDSI_LASTSAVE_DTM;
  if (eq(kw,O(object_Kpagecount))) return PIDSI_PAGECOUNT;
  if (eq(kw,O(object_Kwordcount))) return PIDSI_WORDCOUNT;
  if (eq(kw,O(object_Kcharcount))) return PIDSI_CHARCOUNT;
  if (eq(kw,O(object_Kthumbnail))) return PIDSI_THUMBNAIL;
  if (eq(kw,O(object_Kappname))) return PIDSI_APPNAME;
  if (eq(kw,O(object_Kdoc_security))) return PIDSI_DOC_SECURITY;
  return (PROPID)-1;
}

/* string -> PROPSPEC */
static void PropSpecSetStr (object str, PROPSPEC * pspec) {
  pspec->ulKind = PRSPEC_LPWSTR;
  { uintL str_len;
    uintL str_offset;
    object str_string = unpack_string_ro(str,&str_len,&str_offset);
    const chart* ptr1;
    unpack_sstring_alloca(str_string,str_len,str_offset, ptr1=);
    { uintL str_bytelen =
        cslen(Symbol_value(S(unicode_16_little_endian)),ptr1,str_len);
      pspec->lpwstr = (LPOLESTR)clisp_malloc(str_bytelen+2);
      begin_system_call();
      cstombs(Symbol_value(S(unicode_16_little_endian)),ptr1,str_len,
              (uintB *)pspec->lpwstr,str_bytelen);
      end_system_call();
      ((uintB *)pspec->lpwstr)[str_bytelen] = '\0';
      ((uintB *)pspec->lpwstr)[str_bytelen+1] = '\0';
    }
  }
}

/* list (ID STRING) -> PROPSPEC(ID), PROPSPEC(STR)
   STACK may don't match the pattern (then function returns false)
   any of pspec1, pspec2 can be NULL */
static int propspeclistp (object arg, PROPSPEC * pspec1,PROPSPEC * pspec2) {
  /* check if it is (INT STRING) */
  if (consp(arg) && !nullp(Cdr(arg)) && !nullp(Car(arg))
      && consp(Cdr(arg)) && nullp(Cdr(Cdr(arg)))
      && !nullp(Car(Cdr(arg)))
      && (integerp(Car(arg)) || symbolp(Car(arg)))
      && stringp(Car(Cdr(arg)))) {
    /* set pspec1 to ID and pspec2 to STRING */
    if (pspec1) {
      pspec1->ulKind = PRSPEC_PROPID;
      if (integerp(Car(arg)))
        pspec1->propid = I_to_UL(Car(arg));
      else {
        pspec1->propid = kwtopropid(Car(arg));
        if (pspec1->propid == (PROPID) -1)
          return 0;
      }
    }
    if (pspec2)
      PropSpecSetStr(Car(Cdr(arg)),pspec2);
    return 1;
  }
  return 0;
}

/* (keyword, int, list (ID STRING) or string) -> PROPSPEC
   uses malloc to allocate memory for string specifiers
     (when ulKind == PRSPEC_LPWSTR)
   pspec2 can be NULL */
static int PropSpecSet (object arg, PROPSPEC * pspec1, PROPSPEC * pspec2) {
  ZeroMemory(pspec1, sizeof(PROPSPEC));
  if (pspec2) ZeroMemory(pspec2, sizeof(PROPSPEC));
  if (symbolp(arg)) {
    pspec1->ulKind = PRSPEC_PROPID;
    pspec1->propid = kwtopropid(arg);
    if (pspec1->propid == (PROPID) -1) return 0;
    return 1;
  } else if (stringp(arg)) {
    PropSpecSetStr(arg,pspec1);
    return 1;
  } else if (integerp(arg)) {
    pspec1->ulKind = PRSPEC_PROPID;
    pspec1->propid = I_to_UL(arg);
    return 1;
  } else if (propspeclistp(arg,pspec1,pspec2)) return 2;
  return 0;
}

static const char * DecodeHRESULT (HRESULT hres) {
  static char buf[128];
#define msgcase(x) case x: return #x; break;
  switch (hres) {
  msgcase(E_UNEXPECTED)
  msgcase(STG_E_FILENOTFOUND)
  msgcase(STG_E_ACCESSDENIED)
  msgcase(STG_E_INSUFFICIENTMEMORY)
  msgcase(STG_E_INVALIDFUNCTION)
  msgcase(STG_E_REVERTED)
  msgcase(STG_E_INVALIDPARAMETER)
  msgcase(STG_E_INVALIDNAME)
  msgcase(S_FALSE)
  msgcase(STG_E_INVALIDPOINTER)
  msgcase(HRESULT_FROM_WIN32(ERROR_NO_UNICODE_TRANSLATION))
  msgcase(HRESULT_FROM_WIN32(ERROR_NOT_SUPPORTED))
  msgcase(STG_E_WRITEFAULT)
  msgcase(STG_E_MEDIUMFULL)
  msgcase(STG_E_PROPSETMISMATCHED)
  }
#undef msgcase
  sprintf(buf,"0x%x",hres);
  return buf;
}

#line 4051
#define with_string_0w(string,wcvar,statement)    do { uintL wcvar##_len;                       uintL wcvar##_offset;                       object wcvar##_string = unpack_string_ro(string,&wcvar##_len,&wcvar##_offset);      const chart* ptr1;                          unpack_sstring_alloca(wcvar##_string,wcvar##_len,wcvar##_offset, ptr1=);     {uintL wcvar##_bytelen =                      cslen(Symbol_value(S(unicode_16_little_endian)),ptr1,wcvar##_len);      DYNAMIC_ARRAY(wcvar##_data,uintB,wcvar##_bytelen+2);      cstombs(Symbol_value(S(unicode_16_little_endian)),ptr1,wcvar##_len,             &wcvar##_data[0],wcvar##_bytelen);      wcvar##_data[wcvar##_bytelen] = '\0';                 wcvar##_data[wcvar##_bytelen+1] = '\0';              {WCHAR* wcvar = (WCHAR*) &wcvar##_data[0];        statement                                            }                                                     FREE_DYNAMIC_ARRAY(wcvar##_data);                   }} while(0)

/* there's no PropVariantInit in my cygwin headers */
#line 4055
#define MyPropVariantInit(ppv)     begin_system_call();    ZeroMemory(ppv,sizeof(PROPVARIANT));end_system_call()

/* (OS::FILE-PROPERTIES filename set [specifier value|:INITID init-id]*)
     Wrapper for Win32 IPropertyStorage functionality
     filename - a compound file name or (on NTFS) name of any file
     set      - :BUILT-IN or :USER-DEFINED property set
     specifier - property specifier: integer, keyword, string or
       list of integer or keyword and string.
       Integer specifier - a property identifier
       Keyword:  :CODEPAGE, :LOCALE,   :TITLE, :SUBJECT, :AUTHOR,
                 :KEYWORDS, :COMMENTS, :TEMPLATE, :LASTAUTHOR,
                 :REVNUMBER, :EDITTIME, :LASTPRINTED,:CREATE-DTM,
                 :LASTSAVE-DTM, :PAGECOUNT, :WORDCOUNT, :CHARCOUNT,
                 :THUMBNAIL, :APPNAME, :DOC-SECURITY - predefined IDs.
       String: string property specifier. If no match is found, first
         ID >= init-id (which defaults to 2) is associated with the
         string and it's value is replaced with new value.
       (int|keyword string) - first element is used as specifier,
         string is associated with this ID.
     value - new value of the property, suitable lisp object, nil or list of
       keyword and value itself. If value is NIL, no assignment is done.
       :EMPTY and :NULL correspond VT_EMPTY and VT_NULL datatypes.
       Keyword in the list specifies the desired type of property being set.
       Supported types are :I1, :UI1, :I2, :UI2, :I4, :UI4, :UINT, :I8,
         :UI8, :R4, :R8, :DATE, :BSTR, :BOOL, :ERROR, :FILETIME,
         :LPSTR, :LPWSTR. FILETIMEs are converted to/from universal time format,
         while DATEs are not.

     returns multiple values - property contents before assignment. */
DEFUN(POSIX::FILE-PROPERTIES, file set &rest pairs,(subr_posix_file_properties,seclass_default,2,0,rest,nokey,0,NIL))
{ /* TODO: close interfaces even on errors;
           support more datatypes
           use IPropertySetStorage::Create when it doesn't exist */
  IPropertyStorage * ppropstg = NULL;
  IPropertySetStorage * ppropsetstg = NULL;
  HRESULT hres;
  FMTID const * fmtid = NULL;
  PROPSPEC * pspecrd = NULL;
  PROPSPEC * pspecwr = NULL;
  PROPVARIANT * pvarrd = NULL;
  PROPVARIANT * pvarwr = NULL;
  PROPID * propidwpnvec = NULL; /* args for WritePropertyNames */
  LPWSTR * lpwstrwpnvec = NULL;
  int ifile = argcount + 1;
  int iset = argcount;
  int i;
  unsigned int initid = 2;
  int use_wpn = 0; /* should WritePropertyNames be used ? */
  int nproprd = 0, npropwr = 0; /* npropd >= npropwr */
  int cproprd = 0, cpropwr = 0;
  int cwpnpar = 0;
  /* argcount is (length pairs), not the total arg count */
  /* no &rest ? no sense. */
  if (argcount == 0) {
    skipSTACK(2);
    VALUES0;
    return;
  }
  /* count the number of r/rw props, checking arglist sanity */
  if (argcount % 2)
    error_key_odd(argcount,TheSubr(subr_self)->name);
  for(i=argcount-1;i>=0;i--) {
    if (i % 2) { /* specifier */
      if (!symbolp(STACK_(i)) && !stringp(STACK_(i))
          && !posfixnump(STACK_(i))) {
        if (!propspeclistp(STACK_(i),NULL,NULL)) {
          pushSTACK(TheSubr(subr_self)->name);
          error(program_error,
            GETTEXT("~S: bad property specifier - it must be string, "
                    "positive number, list or keyword"));
        } else { use_wpn++; nproprd++; }
      } else if (symbolp(STACK_(i)) && eq(STACK_(i),O(object_Kinitid))) initid = 0;
      else nproprd++;
    } else { /* value */
      if (!initid) {
        if (integerp(STACK_(i))) initid = I_to_UL(STACK_(i));
        else {
          pushSTACK(STACK_(i));
          pushSTACK(TheSubr(subr_self)->name);
          error(program_error,GETTEXT("~S: bad INITID specifier: ~S"));
        }
      } else if (!nullp(STACK_(i))) npropwr++;
    }
  }
  if (!StgOpenStorageExFunc) {
    begin_system_call();
    SetLastError(ERROR_INVALID_FUNCTION);
    end_system_call();
    OS_error();
  }
  STACK_(ifile) = physical_namestring(STACK_(ifile));
  with_string_0w(STACK_(ifile), filename, {
      begin_blocking_system_call();
      hres = StgOpenStorageExFunc(filename,
                                  ((npropwr||use_wpn)?STGM_READWRITE:STGM_READ)
                                  | STGM_SHARE_EXCLUSIVE,
                                  4 /* STGFMT_ANY */, 0, NULL /*&stgOp*/, 0,
                                  &IID_IPropertySetStorage,
                                  (void **)&ppropsetstg);
      end_blocking_system_call();
  });
  if (FAILED(hres)) {
    pushSTACK(STACK_(ifile));
    pushSTACK(TheSubr(subr_self)->name);
    switch(hres) {
      case STG_E_FILENOTFOUND:
        error(file_error,GETTEXT("~S: file ~S does not exist"));
      case STG_E_FILEALREADYEXISTS:
        error(file_error,GETTEXT("~S: file ~S is not a compound file nor it is on the NTFS file system"));
      default:
        error(file_error,GETTEXT("~S: StgOpenStorageEx() failed on file ~S"));
    }
  }
  if (eq(STACK_(iset),O(object_Kuser_defined)))
    fmtid = &FMTID_UserDefinedProperties;
  else if (eq(STACK_(iset),O(object_Kbuilt_in)))
    fmtid = &FMTID_SummaryInformation;
  else {
    pushSTACK(STACK_(iset));
    pushSTACK(TheSubr(subr_self)->name);
    error(file_error,GETTEXT("~S: invalid property set specifier ~S"));
  }
  begin_blocking_system_call();
  hres = ppropsetstg->lpVtbl->Open(ppropsetstg, fmtid,
                                   ((npropwr||use_wpn)?STGM_READWRITE:STGM_READ)
                                   | STGM_SHARE_EXCLUSIVE, &ppropstg);
  end_blocking_system_call();
  if (FAILED(hres)) {
    pushSTACK(safe_to_string(DecodeHRESULT(hres)));
    pushSTACK(STACK_(ifile+1));
    pushSTACK(STACK_(iset+2));
    pushSTACK(TheSubr(subr_self)->name);
    error(file_error,GETTEXT("~S: unable to open ~S IPropertySetStorage of ~S: error ~S"));
  }
  /* fill the specifiers, init the variables */
  pspecrd =   (PROPSPEC *)clisp_malloc(sizeof(PROPSPEC)    * nproprd);
  pvarrd = (PROPVARIANT *)clisp_malloc(sizeof(PROPVARIANT) * nproprd);
  pspecwr =   (PROPSPEC *)clisp_malloc(sizeof(PROPSPEC)    * npropwr);
  pvarwr = (PROPVARIANT *)clisp_malloc(sizeof(PROPVARIANT) * npropwr);
  if (use_wpn) {
    propidwpnvec = (PROPID *)clisp_malloc(sizeof(PROPID)*use_wpn);
    lpwstrwpnvec = (LPWSTR *)clisp_malloc(sizeof(LPWSTR)*use_wpn);
  }
  for(i=0;i<argcount;i+=2) {
    /* i+1 - specifier, i - value */
    PROPSPEC second;
    int pssresult;
    if (symbolp(STACK_(i+1)) && eq(STACK_(i+1),O(object_Kinitid))) continue;
    pssresult = PropSpecSet(STACK_(i+1),pspecrd+nproprd-cproprd-1,&second);
    MyPropVariantInit(pvarrd+nproprd-cproprd-1);
    if (!nullp(STACK_(i))) {
      PropSpecSet(STACK_(i+1),pspecwr+npropwr-cpropwr-1,NULL);
      MyPropVariantInit(pvarwr+npropwr-cpropwr-1);
      pushSTACK(STACK_(i));
      if (!LispToPropVariant(pvarwr+npropwr-cpropwr-1)) {
        pushSTACK(STACK_(i));
        pushSTACK(TheSubr(subr_self)->name);
        error(error_condition,GETTEXT("~S: cannot convert ~S to PROPVARIANT"));
      }
      cpropwr++;
    }
    if (use_wpn && pssresult == 2) {
      propidwpnvec[cwpnpar] = pspecrd[nproprd-cproprd-1].propid;
      lpwstrwpnvec[cwpnpar] = second.lpwstr;
      cwpnpar++;
    }
    cproprd++;
  }
  hres = ppropstg->lpVtbl->ReadMultiple(ppropstg,nproprd, pspecrd, pvarrd);
  if(FAILED(hres)) {
    pushSTACK(safe_to_string(DecodeHRESULT(hres)));
    pushSTACK(TheSubr(subr_self)->name);
    error(error_condition,GETTEXT("~S: ReadMultiple error: ~S"));
  }
  if (npropwr > 0) {
    begin_blocking_system_call();
    hres = ppropstg->lpVtbl->WriteMultiple(ppropstg,npropwr,pspecwr,pvarwr,
                                           initid);
    end_blocking_system_call();
    if(FAILED(hres)) {
      pushSTACK(safe_to_string(DecodeHRESULT(hres)));
      pushSTACK(TheSubr(subr_self)->name);
      error(error_condition,GETTEXT("~S: WriteMultiple error: ~S"));
    }
  }
  for (i=0;i<nproprd;i++)
    if (!PropVariantToLisp(pvarrd+i)) {
      pushSTACK(fixnum(i));
      pushSTACK(TheSubr(subr_self)->name);
      error(error_condition,GETTEXT("~S: cannot convert value ~S to Lisp datatype"));
    }
  if (use_wpn) {
    hres = ppropstg->lpVtbl->WritePropertyNames(ppropstg,use_wpn,propidwpnvec,lpwstrwpnvec);
    if (FAILED(hres)) {
      pushSTACK(safe_to_string(DecodeHRESULT(hres)));
      pushSTACK(TheSubr(subr_self)->name);
      error(error_condition,GETTEXT("~S: WritePropertyNames: ~S"));
    }
  }
  if (sizeof(mv_space)/sizeof(mv_space[0]) < nproprd) {
    pushSTACK(TheSubr(subr_self)->name);
    error(program_error,GETTEXT("~S: multiple value count limit reached"));
  }
  mv_count = nproprd;
  for (i=0;i<nproprd;i++) mv_space[nproprd-i-1] = popSTACK();
  skipSTACK(argcount+2); /* two first args */
  begin_system_call();
  for (i=0;i<nproprd;i++) {
    PropVariantClear(pvarrd+i);
    if (pspecrd[i].ulKind == PRSPEC_LPWSTR) free(pspecrd[i].lpwstr);
  }
  for (i=0;i<npropwr;i++) {
    if (pvarwr[i].vt == VT_LPWSTR || pvarwr[i].vt == VT_BSTR)
      SysFreeString(pvarwr[i].pwszVal);
    if (pvarwr[i].vt == VT_LPSTR)
      SysFreeString((BSTR)pvarwr[i].pszVal);
    if (pspecwr[i].ulKind == PRSPEC_LPWSTR) free(pspecwr[i].lpwstr);
  }
  for (i=0;i<use_wpn;i++) free(lpwstrwpnvec[i]);
  free(pspecrd); free(pvarrd); free(pspecwr); free(pvarwr);
  free(propidwpnvec); free(lpwstrwpnvec);
  ppropstg->lpVtbl->Release(ppropstg);
  ppropsetstg->lpVtbl->Release(ppropsetstg);
  end_system_call();
}

#define SIDBUFSZ 256

/* (POSIX::GET-USER-SID &optional USERNAME)
   USERNAME: string representing user's name, possibly
   containing a domain name. Current process user's SID
   is returned if no USERNAME is specified.
   Returns string representation of user's security
   identifier (SID) in S-R-I-S-S notation.
   Function could be used in conjunction with file-owner */

DEFUN(POSIX::GET-USER-SID, &optional username,(subr_posix_get_user_sid,seclass_default,0,1,norest,nokey,0,NIL)) {
  char buf[SIDBUFSZ];
  PSID psid;
  LPSTR sidstr;

  if (!missingp(STACK_0)) {
    WCHAR domain[SIDBUFSZ];
    DWORD sz = SIDBUFSZ, domsz = SIDBUFSZ;
    SID_NAME_USE use;

    with_string_0w(check_string(STACK_0),wstr, {
      if (!LookupAccountNameW(NULL, wstr, (PSID) buf, &sz, domain, &domsz, &use))
        OS_error();
      psid = (PSID) buf;
    });
  } else {
    HANDLE token_handle = NULL;
    TOKEN_USER * tu = ((TOKEN_USER *) buf);
    DWORD required = 0;

    if (!OpenProcessToken(GetCurrentProcess(),
                          TOKEN_ADJUST_PRIVILEGES|TOKEN_QUERY,
                          &token_handle))
      OS_error();
    if (!GetTokenInformation(token_handle, TokenUser, tu, SIDBUFSZ, &required))
      OS_error();
    psid = tu->User.Sid;
  }
  if (!initialized_sid_apis)
    initialize_sid_apis();
  if (!ConvertSidToStringSidFunc) {
      pushSTACK(TheSubr(subr_self)->name);
      error(error_condition,GETTEXT("~S: SID management library is not initialized"));
  }
  if (!ConvertSidToStringSidFunc(psid, &sidstr)) OS_error();
  VALUES1(asciz_to_string(sidstr,GLO(misc_encoding)));
  LocalFree(sidstr);
  skipSTACK(1);
}

/* helpers for CLIPBOARD and %SET-CLIPBOARD */

static int nlines_a (const char * s) {
  int result = 1;
  while (*s) if (*s++ == '\n') result++;
  return result;
}

static int nlines_w (PWSTR s) {
  int result = 1;
  while (*s) if (*s++ == (WCHAR)'\n') result++;
  return result;
}

/* copy string and convert "\n" to "\r\n" */

static void strzcpy12_a (char * dest, const char * src) {
  do {
    if (*src == '\n') *dest++ = '\r';
    *dest = *src;
    if (!*src) break;
    dest++; src++;
  } while(true);
}

static void strzcpy12_w (PWSTR dest, PCWSTR src) {
  do {
    if (*src == (WCHAR)'\n') *dest++ = (WCHAR)'\r';
    *dest = *src;
    if (!*src) break;
    dest++; src++;
  } while(true);
}

/* copy string and convert "\r\n" to "\n" */

static void strzcpy21_a (char * dest, const char * src) {
  do {
    *dest = *src;
    if (!*src) break;
    if (*src != '\r') dest++;
    src++;
  } while(true);
}

static void strzcpy21_w (PWSTR dest, PCWSTR src) {
  do {
    *dest = *src;
    if (!*src) break;
    if (*src != (WCHAR)'\r') dest++;
    src++;
  } while(true);
}

/* %SET-CLIPBOARD: set the contents of Windows clipboard to the printed
   representation of argument (PRINC-TO-STRING is used). Returns T on
   success, NIL on failure. */
DEFUN(OS::%SET-CLIPBOARD, str,(subr_os__25set_clipboard,seclass_default,1,0,norest,nokey,0,NIL)) {
  int textset = 0;
  pushSTACK(STACK_0); /* to return from %SET-CLIPBOARD */
  funcall(L(princ_to_string), 1);
  begin_system_call();
  if (OpenClipboard(NULL)) {
    if( EmptyClipboard() ) {
#ifdef ENABLE_UNICODE
      OSVERSIONINFO v;
      v.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
      if (GetVersionEx(&v)) {
        if (v.dwPlatformId == VER_PLATFORM_WIN32_NT) { /* Windows NT */
            with_string_0w(value1, wstr, {
              HGLOBAL sglobal =
                GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE,
                            (wstr_bytelen + nlines_w(wstr) + 2)
                              * sizeof(WCHAR));
              if (sglobal != NULL) {
                void * slocal = GlobalLock(sglobal);
                if (slocal != NULL) {
                  end_system_call();
                  strzcpy12_w(slocal, wstr);
                  begin_system_call();
                  if ( SetClipboardData( CF_UNICODETEXT,  sglobal ) != NULL ) {
                    GlobalUnlock(sglobal);
                    textset = 1;
                  } else {
                    DWORD last = GetLastError();
                    GlobalFree(sglobal);
                    SetLastError(last);
                  }
                }
              }
            });
        } else {  /* Win95/98/Me - try ASCII */
#else
      { {
#endif
          with_string_0( value1, GLO(misc_encoding), cstr, {
            HGLOBAL sglobal =
              GlobalAlloc(GMEM_MOVEABLE|GMEM_DDESHARE, cstr_bytelen + nlines_a(cstr) + 1);
            if (sglobal != NULL) {
              void * slocal = GlobalLock(sglobal);
              if (slocal != NULL) {
                end_system_call();
                strzcpy12_a(slocal, cstr);
                begin_system_call();
                if (SetClipboardData( CF_TEXT,  sglobal ) != NULL ) {
                  GlobalUnlock(sglobal);
                  textset = 1;
                } else { /* GlobalFree only if SetClipboardData failed */
                  DWORD last = GetLastError();
                  GlobalFree(sglobal);
                  SetLastError(last);
                }
              }
            }
          });
#ifdef ENABLE_UNICODE
        } /* v.dwPlatformId */
      } /* GetVersionEx */
#else
      } } /* for MODPREP all brackets should be
             balanced like there are no ifdefs */
#endif
      CloseClipboard();
    } /* EmptyClipboard */
  } /* OpenClipboard */
  end_system_call();
  if (!textset) OS_error(); /* !textset => some system call failed
                               && LastError contain this error code */
  VALUES1(popSTACK());
}

/* CLIPBOARD: Returns the textual contents of Windows clipboard
   as a string. First try to get it as CF_UNICODETEXT, then CF_TEXT.
   On failure or when no text is available NIL is returned. */
DEFUN(OS:CLIPBOARD,,(subr_os_clipboard,seclass_default,0,0,norest,nokey,0,NIL)) {
  VALUES1(NIL);
  begin_blocking_system_call();
  if (OpenClipboard(NULL)) {
#ifdef ENABLE_UNICODE
    HGLOBAL gltext  = GetClipboardData(CF_UNICODETEXT);
    if (gltext != NULL) { /* UNICODE TEXT */
      PWSTR wstr = (PWSTR)GlobalLock(gltext);
      if (wstr != NULL) {
        DYNAMIC_ARRAY(buf, WCHAR, wcslen(wstr) + 1);
        end_blocking_system_call();
        strzcpy21_w(buf, wstr);
        VALUES1(n_char_to_string((const char *)buf, wcslen(buf) * sizeof(WCHAR),
                                 Symbol_value(S(unicode_16_little_endian))));
        FREE_DYNAMIC_ARRAY(buf);
        begin_blocking_system_call();
        GlobalUnlock(gltext);
      }
    } else { /* Probably system just do not support UNICODE */
#endif
      gltext = GetClipboardData(CF_TEXT); /* ANSI TEXT */
      if (gltext != NULL) {
        const char * str = (const char *)GlobalLock(gltext);
        if (str != NULL) {
          DYNAMIC_ARRAY(buf, char, strlen(str) + 1);
          end_blocking_system_call();
          strzcpy21_a(buf, str);
          VALUES1(asciz_to_string(buf, GLO(misc_encoding)));
          FREE_DYNAMIC_ARRAY(buf);
          begin_blocking_system_call();
          GlobalUnlock(gltext);
        }
      }
#ifdef ENABLE_UNICODE
    }
#endif
    CloseClipboard();
  }
  end_blocking_system_call();
}

#endif  /* WIN32_NATIVE || UNIX_CYGWIN */

#if defined(WIN32_NATIVE)

/* http://gnuwin32.sourceforge.net/version.c.txt */
static Values /*maygc*/ file_version (char *pathz) {
  DWORD dwHandle, dwLen;
  UINT BufLen;
  LPTSTR lpData, lpBuffer;
  VS_FIXEDFILEINFO *pFileInfo;
  BOOL status;
  begin_system_call();
  dwLen = GetFileVersionInfoSize(pathz,&dwHandle);
  if (dwLen == 0)
    OS_error();
  lpData = (LPTSTR)malloc(dwLen);
  if (lpData == NULL)
    OS_error();
  if (!GetFileVersionInfo(pathz,dwHandle,dwLen,lpData)) {
    free(lpData);
    OS_error();
  }
  if (!VerQueryValue(lpData,"\\",(LPVOID*)&pFileInfo,(PUINT)&BufLen)) {
    end_system_call();
    pushSTACK(data_to_sb8vector(lpData,dwLen));
    begin_system_call();
    free(lpData);
    end_system_call();
    pushSTACK(asciz_to_string(pathz,GLO(pathname_encoding)));
    pushSTACK(TheSubr(subr_self)->name);
    error(error_condition,GETTEXT("~S(~S): No root block in ~S"));
  }
  pushSTACK(UL_to_I(HIWORD(pFileInfo->dwFileVersionMS)));
  pushSTACK(UL_to_I(LOWORD(pFileInfo->dwFileVersionMS)));
  pushSTACK(UL_to_I(HIWORD(pFileInfo->dwFileVersionLS)));
  pushSTACK(UL_to_I(LOWORD(pFileInfo->dwFileVersionLS)));
#if defined(ENABLE_UNICODE)
# define SUBBLOCK  "\\StringFileInfo\\040904B0\\"
#else
# define SUBBLOCK  "\\StringFileInfo\\04090000\\"
#endif
#line 4553
#define TO_STACK(info)                                                     begin_system_call();                                                     status = VerQueryValue(lpData,SUBBLOCK info,(LPVOID*)&lpBuffer,                                 (PUINT)&BufLen);                                  end_system_call();                                                       while(lpBuffer[BufLen-1]==0) BufLen--;                                   pushSTACK(status?n_char_to_string(lpBuffer,BufLen,GLO(misc_encoding)):NIL)
  TO_STACK("Comments");
  TO_STACK("CompanyName");
  TO_STACK("FileDescription");
  TO_STACK("FileVersion");
  TO_STACK("InternalName");
  TO_STACK("LegalCopyright");
  TO_STACK("LegalTrademarks");
  TO_STACK("OriginalFilename");
  TO_STACK("ProductName");
  TO_STACK("ProductVersion");
  TO_STACK("PrivateBuild");
  TO_STACK("SpecialBuild");
  begin_system_call(); free(lpData); end_system_call();
  funcall(O(object_posix__mkfilever),16);
#undef TO_STACK
#undef SUBBLOCK
}
DEFUN(OS:FILE-VERSION,filename,(subr_os_file_version,seclass_default,1,0,norest,nokey,0,NIL)) {
  with_string_0(physical_namestring(popSTACK()),GLO(pathname_encoding),pathz,{
    file_version(pathz);
  });
}

#endif  /* WIN32_NATIVE */

/* STDIO inteface for postgresql et al and to access wild files like 'foo*' */
DEFUN(POSIX::FOPEN, path mode,(subr_posix_fopen,seclass_default,2,0,norest,nokey,0,NIL)) {
  STACK_0 = check_string(STACK_0);
  STACK_1 = check_string(STACK_1);
  with_string_0(STACK_1, GLO(pathname_encoding), pathz, {
      with_string_0(STACK_0, GLO(misc_encoding), modez, {
          FILE *fp;
          begin_blocking_system_call();
          fp = fopen(pathz,modez);
          end_blocking_system_call();
          if (fp) STACK_0 = allocate_fpointer((FOREIGN)fp);
          else ANSIC_error();
        });
    });
  VALUES1(STACK_0); skipSTACK(2);
}
DEFUN(POSIX::FDOPEN, fd mode,(subr_posix_fdopen,seclass_default,2,0,norest,nokey,0,NIL)) {
  STACK_0 = check_string(STACK_0);
  STACK_1 = check_sint(STACK_1);
  with_string_0(STACK_0, GLO(misc_encoding), modez, {
      FILE *fp;
      begin_blocking_system_call();
      fp = fdopen(I_to_sint(STACK_1),modez);
      end_blocking_system_call();
      if (fp) STACK_0 = allocate_fpointer((FOREIGN)fp);
      else ANSIC_error();
    });
  VALUES1(STACK_0); skipSTACK(2);
}
DEFUN(POSIX::FREOPEN, path mode file,(subr_posix_freopen,seclass_default,3,0,norest,nokey,0,NIL)) {
  STACK_2 = check_string(STACK_2); /* path */
  STACK_1 = check_string(STACK_1); /* mode */
  STACK_0 = check_fpointer(STACK_0,1); /* file */
  with_string_0(STACK_2, GLO(pathname_encoding), pathz, {
      with_string_0(STACK_1, GLO(misc_encoding), modez, {
          FILE *fp;
          begin_blocking_system_call();
          fp = freopen(pathz,modez,(FILE*)TheFpointer(STACK_0)->fp_pointer);
          end_blocking_system_call();
          if (fp) TheFpointer(STACK_0)->fp_pointer = fp;
          else ANSIC_error();
        });
    });
  VALUES0; skipSTACK(3);
}
#line 4630
#define FILE_FUNCTION(fun,finish)                                   int ret;                                                          STACK_0 = check_fpointer(STACK_0,1);                              begin_blocking_system_call();                                     ret = fun((FILE*)TheFpointer(STACK_0)->fp_pointer);               end_blocking_system_call();                                       finish; skipSTACK(1)
DEFUN(POSIX::FILENO, fp,(subr_posix_fileno,seclass_default,1,0,norest,nokey,0,NIL))
{ FILE_FUNCTION(fileno,{ if(ret==-1)ANSIC_error(); VALUES1(sint_to_I(ret));});}
DEFUN(POSIX::FEOF, fp,(subr_posix_feof,seclass_default,1,0,norest,nokey,0,NIL)) { FILE_FUNCTION(feof,VALUES_IF(ret)); }
DEFUN(POSIX::FERROR, fp,(subr_posix_ferror,seclass_default,1,0,norest,nokey,0,NIL)) { FILE_FUNCTION(ferror,VALUES_IF(ret)); }
DEFUN(POSIX::FCLOSE, fp,(subr_posix_fclose,seclass_default,1,0,norest,nokey,0,NIL))
{ FILE_FUNCTION(fclose,{ if (ret == EOF) ANSIC_error(); VALUES0; }); }
DEFUN(POSIX::FFLUSH, fp,(subr_posix_fflush,seclass_default,1,0,norest,nokey,0,NIL))
{ FILE_FUNCTION(fflush,{ if (ret == EOF) ANSIC_error(); VALUES0; }); }
/* no fputs & fgets because they will mess with encodings &c */
DEFUN(POSIX::CLEARERR, fp,(subr_posix_clearerr,seclass_default,1,0,norest,nokey,0,NIL)) {
  STACK_0 = check_fpointer(STACK_0,1);
  begin_blocking_system_call();
  clearerr((FILE*)TheFpointer(STACK_0)->fp_pointer);
  end_blocking_system_call();
  VALUES0; skipSTACK(1);
}

/* --- testing only! not exported! --- */
/* fgetc returns -1 on EOF instead of signaling an error. or signal?! */
DEFUN(POSIX::%FGETC, fp,(subr_posix__25fgetc,seclass_default,1,0,norest,nokey,0,NIL)) { FILE_FUNCTION(fgetc,VALUES1(sint_to_I(ret))); }
#line 4658
#define FILE_FUNCTION2(fun)                                                int ret;                                                                 STACK_0 = check_fpointer(STACK_0,1);                                     STACK_1 = check_sint(STACK_1);                                           begin_blocking_system_call();                                            ret = fun(I_to_sint(STACK_1),(FILE*)TheFpointer(STACK_0)->fp_pointer);    end_blocking_system_call();                                              VALUES1(sint_to_I(ret)); skipSTACK(2)
DEFUN(POSIX::%FPUTC, c fp,(subr_posix__25fputc,seclass_default,2,0,norest,nokey,0,NIL)) { FILE_FUNCTION2(fputc); }
DEFUN(POSIX::%UNGETC, c fp,(subr_posix__25ungetc,seclass_default,2,0,norest,nokey,0,NIL)) { FILE_FUNCTION2(ungetc); }

/* standard objects */
DEFVAR(my_stdin)
DEFVAR(my_stdout)
DEFVAR(my_stderr)
static void init_stdio (void) {
  TheFpointer(O(my_stdin))->fp_pointer = stdin;
  mark_fp_valid(TheFpointer(O(my_stdin)));
  TheFpointer(O(my_stdout))->fp_pointer = stdout;
  mark_fp_valid(TheFpointer(O(my_stdout)));
  TheFpointer(O(my_stderr))->fp_pointer = stderr;
  mark_fp_valid(TheFpointer(O(my_stderr)));
}
DEFUN(POSIX::%STDIO, &optional which,(subr_posix__25stdio,seclass_default,0,1,norest,nokey,0,NIL)) {
 stdio_restart:
  if (missingp(STACK_0)) {
    init_stdio();
    VALUES0;
  } else {
    int which = I_to_sint(STACK_0 = check_sint(STACK_0));
    switch (which) {
      case 0: VALUES1(O(my_stdin)); break;
      case 1: VALUES1(O(my_stdout)); break;
      case 2: VALUES1(O(my_stderr)); break;
      default:
        pushSTACK(NIL);         /* no PLACE */
        pushSTACK(STACK_1);     /* TYPE-ERROR slot DATUM */
        pushSTACK(O(object__28member_200_201_202_29)); /* EXPECTED-TYPE */
        pushSTACK(STACK_0); pushSTACK(STACK_2);
        pushSTACK(TheSubr(subr_self)->name);
        check_value(type_error,GETTEXT("~S: ~S is not a ~S"));
        STACK_0 = value1;
        goto stdio_restart;
    }
  }
  skipSTACK(1);
}

/* ========================= OS error printing ========================= */
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
# include <winerror.h>
/* http://cygwin.com/cgi-bin/cvsweb.cgi/src/winsup/w32api/include/winerror.h */
/* http://msdn.microsoft.com/en-us/library/aa914935.aspx */
#line 5638

object errno_to_symbol_w (long code);
object errno_to_symbol_w (long code) {return check_last_error_reverse(code);}
DEFUN(OS::LAST-ERROR, &optional newval,(subr_os_last_error,seclass_default,0,1,norest,nokey,0,NIL)) {
  if (eq(T,STACK_0)) {          /* all known error codes */
    int pos = 0;
    for (; pos < check_last_error_map.size; pos++) {
      pushSTACK(allocate_cons());
      Car(STACK_0) = uint32_to_I(check_last_error_map.table[pos].c_const);
      Cdr(STACK_0) = *check_last_error_map.table[pos].l_const;
    }
    VALUES1(listof(check_last_error_map.size));
  } else {
    DWORD error_code;
    if (missingp(STACK_0)) {
      begin_system_call();
      error_code = GetLastError();
      end_system_call();
      VALUES1(check_last_error_reverse(error_code));
    } else {
      if (uint32_p(STACK_0)) {
        error_code = I_to_uint32(STACK_0);
        VALUES1(check_last_error_reverse(error_code));
      } else {
        error_code = check_last_error(STACK_0);
        VALUES1(uint32_to_I(error_code));
      }
      begin_system_call();
      SetLastError(error_code);
      end_system_call();
    }
    VALUES1(check_last_error_reverse(error_code));
  }
  skipSTACK(1);
}
DEFUN(SYS::FORMAT-MESSAGE, &optional error_code,(subr_sys_format_message,seclass_default,0,1,norest,nokey,0,NIL)) {
  DWORD error_code;
  if (missingp(STACK_0)) {
    begin_system_call();
    error_code = GetLastError();
    end_system_call();
  } else error_code = check_last_error(STACK_0);
  STACK_0 = UL_to_I(error_code);
  funcall(L(format_message),1);
}
#endif  /* WIN32_NATIVE || UNIX_CYGWIN */

/* http://opengroup.org/onlinepubs/9699919799/basedefs/errno.h.html */
#line 5718

object errno_to_symbol_a (long code);
object errno_to_symbol_a (long code) { return check_errno_reverse(code); }
DEFUN(POSIX::ERRNO, &optional newval,(subr_posix_errno,seclass_default,0,1,norest,nokey,0,NIL)) {
  if (eq(T,STACK_0)) {          /* all known error codes */
    unsigned int pos = 0;
    for (; pos < check_errno_map.size; pos++) {
      pushSTACK(allocate_cons());
      Car(STACK_0) = sint_to_I(check_errno_map.table[pos].c_const);
      Cdr(STACK_0) = *check_errno_map.table[pos].l_const;
    }
    VALUES1(listof(check_errno_map.size));
  } else {
    int error_code;
    if (missingp(STACK_0)) {
      begin_system_call();
      error_code = errno;
      end_system_call();
      VALUES1(check_errno_reverse(error_code));
    } else {
      if (sint_p(STACK_0)) {
        error_code = I_to_sint(STACK_0);
        VALUES1(check_errno_reverse(error_code));
      } else {
        error_code = check_errno(STACK_0);
        VALUES1(sint_to_I(error_code));
      }
      begin_system_call();
      errno = error_code;
      end_system_call();
    }
  }
  skipSTACK(1);
}
DEFUN(SYS::STRERROR, &optional error_code,(subr_sys_strerror,seclass_default,0,1,norest,nokey,0,NIL)) {
  int error_code;
  if (missingp(STACK_0)) {
    /* errno access must be guarded with begin/end_system_call */
    begin_system_call();
    error_code = errno;
    end_system_call();
  } else error_code = check_errno(STACK_0);
  STACK_0 = L_to_I(error_code);
  funcall(L(strerror),1);
}

/* ========================= wildcard matching ========================= */

#line 5767
DEFUN(POSIX:FNMATCH, pattern string &key        :CASE-SENSITIVE PATHNAME PERIOD NOESCAPE,(subr_posix_fnmatch,seclass_default,2,0,norest,key,4,NIL)) {
  int flags = fnm_flags();
  STACK_0 = check_string(STACK_0);
  STACK_1 = check_string(STACK_1);
  with_string_0(STACK_0,GLO(misc_encoding),stringz, {
      with_string_0(STACK_1,GLO(misc_encoding),patternz, {
          begin_system_call();
          flags = fnmatch(patternz,stringz,flags);
          end_system_call();
        });
    });
  switch (flags) {
    case 0: VALUES1(T); break;
    case FNM_NOMATCH: VALUES1(NIL); break;
    default: pushSTACK(fixnum(flags));
      error(error_condition,GETTEXT("fnmatch: error ~S"));
  }
  skipSTACK(2);
}

#if defined(DEBUG_SPVW)
/* internal playground - see spvd.d & spvw_debug.d */
extern unsigned int get_constsym_count (void);
extern object get_constsym (unsigned int);
DEFUN(CONSTSYM, &optional pos,(subr_posix_constsym,seclass_default,0,1,norest,nokey,0,NIL)) {
  VALUES1(missingp(STACK_0) ? fixnum(get_constsym_count())
          : get_constsym(I_to_uint(check_uint(STACK_0))));
  skipSTACK(1);
}
#endif

void module__syscalls__init_function_2 (module_t* module);
void module__syscalls__init_function_2 (module_t* module) {
  module__syscalls__init_function_2__modprep(module);
#if defined(WIN32_NATIVE)
  init_win32_link();
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  init_win32_cygwin_open_storage();
#endif
#if defined(HAVE_FFI)
  init_stdio();
#endif
  /* The getdate() functions needs some configuration file, and the DATEMSK
     environment variable is supposed to point to it. See
     <http://pubs.opengroup.org/onlinepubs/9699919799/functions/getdate.html>.
     We ship such a configuration file. Here we set the DATEMSK environment
     variable if the user has not already set it. */
  { char* envval = getenv("DATEMSK");
    if (envval == NULL || envval[0] == '\0') {
      with_string_0(physical_namestring(GLO(lib_dir)),GLO(pathname_encoding),libdir_asciz, {
        const char* part1 = libdir_asciz;
        const char* part2 = "syscalls/datemsk";
        /* Concatenate these two parts. */
        size_t part1_len = asciz_length(part1);
        size_t part2_len = asciz_length(part2);
        DYNAMIC_ARRAY(datemsk_filename,char,part1_len+1+part2_len+1);
        { char* p = datemsk_filename;
          memcpy(p,part1,part1_len); p += part1_len;
          if (!(part1_len > 0 && part1[part1_len-1] == '/')) { *p++ = '/'; }
          memcpy(p,part2,part2_len); p += part2_len;
          *p = '\0';
        }
        setenv("DATEMSK",datemsk_filename,1);
        FREE_DYNAMIC_ARRAY(datemsk_filename);
      });
    }
  }
}


struct module__syscalls__subr_tab_t module__syscalls__subr_tab
  #if defined(HEAPCODES) && (alignment_long < varobject_alignment) && defined(__GNUC__)
    __attribute__ ((aligned (varobject_alignment)))
  #endif
  = {
  #if varobjects_misaligned
  { 0 },
  #endif
  LISPFUN_F(subr_os_bogomips,seclass_default,0,0,norest,nokey,0,NIL)
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_os_clipboard,seclass_default,0,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_os_file_owner,seclass_default,1,0,norest,nokey,0,NIL)
#if defined(WIN32_NATIVE)
  LISPFUN_F(subr_os_file_version,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_os_last_error,seclass_default,0,1,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_os_priority,seclass_default,1,1,norest,nokey,0,NIL)
  LISPFUN_F(subr_os_version_3C,seclass_default,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_os_version_3C_3D,seclass_default,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_os_version_3E,seclass_default,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_os_version_3E_3D,seclass_default,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_os_version_compare,seclass_default,2,0,norest,nokey,0,NIL)
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_os__25set_clipboard,seclass_default,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_os__25set_priority,seclass_default,3,0,norest,nokey,0,NIL)
#if defined(HAVE_SYS_RESOURCE_H)
  LISPFUN_F(subr_posix_begin_subprocesses,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_CHROOT)
  LISPFUN_F(subr_posix_chroot,seclass_default,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_clearerr,seclass_default,1,0,norest,nokey,0,NIL)
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_CLOSELOG))
  LISPFUN_F(subr_posix_closelog,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_CONFSTR)
  LISPFUN_F(subr_posix_confstr,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(DEBUG_SPVW)
  LISPFUN_F(subr_posix_constsym,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_convert_attributes,seclass_default,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_convert_mode,seclass_default,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_copy_file,seclass_default,2,0,norest,key,4,NIL)
#if defined(HAVE_GETDOMAINNAME)
  LISPFUN_F(subr_posix_domainname,seclass_default,0,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_duplicate_handle,seclass_default,1,1,norest,nokey,0,NIL)
#if defined(HAVE_GETEGID)
  LISPFUN_F(subr_posix_egid,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_ENDUTXENT))
  LISPFUN_F(subr_posix_endutxent,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  LISPFUN_F(subr_posix_end_subprocesses,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_ERFC)
  LISPFUN_F(subr_posix_erf,seclass_foldable,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_ERFC)
  LISPFUN_F(subr_posix_erfc,seclass_foldable,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_errno,seclass_default,0,1,norest,nokey,0,NIL)
#if defined(HAVE_GETEUID)
  LISPFUN_F(subr_posix_euid,seclass_default,0,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_fclose,seclass_default,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_fdopen,seclass_default,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_feof,seclass_default,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_ferror,seclass_default,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_fflush,seclass_default,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_fileno,seclass_default,1,0,norest,nokey,0,NIL)
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_file_info,seclass_default,1,1,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_file_properties,seclass_default,2,0,rest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_file_size,seclass_default,1,0,norest,nokey,0,NIL)
#if defined(HAVE_FSTAT) && defined(HAVE_STAT)
  LISPFUN_F(subr_posix_file_stat,seclass_default,1,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_NFTW)
  LISPFUN_F(subr_posix_file_tree_walk,seclass_default,2,0,norest,key,5,NIL)
#endif
  LISPFUN_F(subr_posix_fnmatch,seclass_default,2,0,norest,key,4,NIL)
  LISPFUN_F(subr_posix_fopen,seclass_default,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_freopen,seclass_default,3,0,norest,nokey,0,NIL)
#if defined(HAVE_GETDATE) && defined(HAVE_DECL_GETDATE_ERR)
  LISPFUN_F(subr_posix_getdate,seclass_default,1,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETPGRP)
  LISPFUN_F(subr_posix_getpgrp,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETPPID)
  LISPFUN_F(subr_posix_getppid,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETSID)
  LISPFUN_F(subr_posix_getsid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXENT))
  LISPFUN_F(subr_posix_getutxent,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXID))
  LISPFUN_F(subr_posix_getutxid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXLINE))
  LISPFUN_F(subr_posix_getutxline,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_get_user_sid,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETGID)
  LISPFUN_F(subr_posix_gid,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETGROUPS)
  LISPFUN_F(subr_posix_groups,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  LISPFUN_F(subr_posix_group_info,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETHOSTID)
  LISPFUN_F(subr_posix_hostid,seclass_default,0,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_j0,seclass_foldable,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_j1,seclass_foldable,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_jn,seclass_foldable,2,0,norest,nokey,0,NIL)
#if defined(HAVE_KILL)
  LISPFUN_F(subr_posix_kill,seclass_default,2,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_LGAMMA) || HAVE_DECL_LGAMMA_R
  LISPFUN_F(subr_posix_lgamma,seclass_foldable,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_loadavg,seclass_default,0,1,norest,nokey,0,NIL)
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_make_shortcut,seclass_default,1,0,norest,key,7,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_memory_status,seclass_default,0,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_mkdtemp,seclass_default,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_mknod,seclass_default,3,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_mkstemp,seclass_default,1,0,norest,key,4,NIL)
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  LISPFUN_F(subr_posix_openlog,seclass_default,1,0,norest,key,6,NIL)
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  LISPFUN_F(subr_posix_pathconf,seclass_default,1,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETPGID)
  LISPFUN_F(subr_posix_pgid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_PUTUTXLINE))
  LISPFUN_F(subr_posix_pututxline,seclass_default,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_resolve_host_ipaddr,seclass_default,0,1,norest,nokey,0,NIL)
#if defined(HAVE_GETRLIMIT)
  LISPFUN_F(subr_posix_rlimit,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  LISPFUN_F(subr_posix_service,seclass_default,0,2,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_SETLOGMASK))
  LISPFUN_F(subr_posix_setlogmask,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETPGRP)
  LISPFUN_F(subr_posix_setpgrp,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETREGID)
  LISPFUN_F(subr_posix_setregid,seclass_default,2,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETREUID)
  LISPFUN_F(subr_posix_setreuid,seclass_default,2,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETSID)
  LISPFUN_F(subr_posix_setsid,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_SETUTXENT))
  LISPFUN_F(subr_posix_setutxent,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  LISPFUN_F(subr_posix_set_file_stat,seclass_default,1,0,norest,key,5,NIL)
#endif
#if defined(HAVE_SETRLIMIT)
  LISPFUN_F(subr_posix_set_rlimit,seclass_default,3,0,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_shortcut_info,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  LISPFUN_F(subr_posix_stat_vfs,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  LISPFUN_F(subr_posix_stream_lock,seclass_default,2,0,norest,key,4,NIL)
#endif
#if defined(HAVE_FCNTL)
  LISPFUN_F(subr_posix_stream_options,seclass_default,2,1,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_string_time,seclass_default,1,2,norest,nokey,0,NIL)
#if defined(WIN32_NATIVE) || defined(HAVE_SYNC) || defined(HAVE_FSYNC)
  LISPFUN_F(subr_posix_sync,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SYSCONF)
  LISPFUN_F(subr_posix_sysconf,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_system_info,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_TGAMMA)
  LISPFUN_F(subr_posix_tgamma,seclass_foldable,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETUID)
  LISPFUN_F(subr_posix_uid,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_UMASK)
  LISPFUN_F(subr_posix_umask,seclass_default,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix_uname,seclass_default,0,0,norest,nokey,0,NIL)
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  LISPFUN_F(subr_posix_usage,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  LISPFUN_F(subr_posix_user_info,seclass_default,0,1,norest,nokey,0,NIL)
#endif
#if defined(HAVE_GETUSERSHELL) && defined(HAVE_ENDUSERSHELL)
  LISPFUN_F(subr_posix_user_shells,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_posix_version,seclass_default,0,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  LISPFUN_F(subr_posix_wait,seclass_default,0,0,norest,key,8,NIL)
#endif
  LISPFUN_F(subr_posix_y0,seclass_foldable,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_y1,seclass_foldable,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix_yn,seclass_foldable,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix__25fgetc,seclass_default,1,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix__25fputc,seclass_default,2,0,norest,nokey,0,NIL)
#if defined(HAVE_SETDOMAINNAME)
  LISPFUN_F(subr_posix__25setdomainname,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETEGID)
  LISPFUN_F(subr_posix__25setegid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETEUID)
  LISPFUN_F(subr_posix__25seteuid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETGID)
  LISPFUN_F(subr_posix__25setgid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETGROUPS)
  LISPFUN_F(subr_posix__25setgroups,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETHOSTID) && !defined(_AIX)
  LISPFUN_F(subr_posix__25sethostid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETPGID)
  LISPFUN_F(subr_posix__25setpgid,seclass_default,2,0,norest,nokey,0,NIL)
#endif
#if defined(HAVE_SETUID)
  LISPFUN_F(subr_posix__25setuid,seclass_default,1,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix__25set_file_size,seclass_default,2,0,norest,nokey,0,NIL)
  LISPFUN_F(subr_posix__25stdio,seclass_default,0,1,norest,nokey,0,NIL)
#if defined(HAVE_SYSLOG)
  LISPFUN_F(subr_posix__25syslog,seclass_default,3,0,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_posix__25ungetc,seclass_default,2,0,norest,nokey,0,NIL)
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  LISPFUN_F(subr_sys_format_message,seclass_default,0,1,norest,nokey,0,NIL)
#endif
  LISPFUN_F(subr_sys_strerror,seclass_default,0,1,norest,nokey,0,NIL)
  0
};
uintC module__syscalls__subr_tab_size = (sizeof(struct module__syscalls__subr_tab_t)-varobjects_misaligned-sizeof(int))/sizeof(subr_t);

struct module__syscalls__subr_tab_initdata_t {
  subr_initdata_t _subr_os_bogomips;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_os_clipboard;
#endif
  subr_initdata_t _subr_os_file_owner;
#if defined(WIN32_NATIVE)
  subr_initdata_t _subr_os_file_version;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_os_last_error;
#endif
  subr_initdata_t _subr_os_priority;
  subr_initdata_t _subr_os_version_3C;
  subr_initdata_t _subr_os_version_3C_3D;
  subr_initdata_t _subr_os_version_3E;
  subr_initdata_t _subr_os_version_3E_3D;
  subr_initdata_t _subr_os_version_compare;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_os__25set_clipboard;
#endif
  subr_initdata_t _subr_os__25set_priority;
#if defined(HAVE_SYS_RESOURCE_H)
  subr_initdata_t _subr_posix_begin_subprocesses;
#endif
#if defined(HAVE_CHROOT)
  subr_initdata_t _subr_posix_chroot;
#endif
  subr_initdata_t _subr_posix_clearerr;
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_CLOSELOG))
  subr_initdata_t _subr_posix_closelog;
#endif
#if defined(HAVE_CONFSTR)
  subr_initdata_t _subr_posix_confstr;
#endif
#if defined(DEBUG_SPVW)
  subr_initdata_t _subr_posix_constsym;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_convert_attributes;
#endif
  subr_initdata_t _subr_posix_convert_mode;
  subr_initdata_t _subr_posix_copy_file;
#if defined(HAVE_GETDOMAINNAME)
  subr_initdata_t _subr_posix_domainname;
#endif
  subr_initdata_t _subr_posix_duplicate_handle;
#if defined(HAVE_GETEGID)
  subr_initdata_t _subr_posix_egid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_ENDUTXENT))
  subr_initdata_t _subr_posix_endutxent;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  subr_initdata_t _subr_posix_end_subprocesses;
#endif
#if defined(HAVE_ERFC)
  subr_initdata_t _subr_posix_erf;
#endif
#if defined(HAVE_ERFC)
  subr_initdata_t _subr_posix_erfc;
#endif
  subr_initdata_t _subr_posix_errno;
#if defined(HAVE_GETEUID)
  subr_initdata_t _subr_posix_euid;
#endif
  subr_initdata_t _subr_posix_fclose;
  subr_initdata_t _subr_posix_fdopen;
  subr_initdata_t _subr_posix_feof;
  subr_initdata_t _subr_posix_ferror;
  subr_initdata_t _subr_posix_fflush;
  subr_initdata_t _subr_posix_fileno;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_file_info;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_file_properties;
#endif
  subr_initdata_t _subr_posix_file_size;
#if defined(HAVE_FSTAT) && defined(HAVE_STAT)
  subr_initdata_t _subr_posix_file_stat;
#endif
#if defined(HAVE_NFTW)
  subr_initdata_t _subr_posix_file_tree_walk;
#endif
  subr_initdata_t _subr_posix_fnmatch;
  subr_initdata_t _subr_posix_fopen;
  subr_initdata_t _subr_posix_freopen;
#if defined(HAVE_GETDATE) && defined(HAVE_DECL_GETDATE_ERR)
  subr_initdata_t _subr_posix_getdate;
#endif
#if defined(HAVE_GETPGRP)
  subr_initdata_t _subr_posix_getpgrp;
#endif
#if defined(HAVE_GETPPID)
  subr_initdata_t _subr_posix_getppid;
#endif
#if defined(HAVE_GETSID)
  subr_initdata_t _subr_posix_getsid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXENT))
  subr_initdata_t _subr_posix_getutxent;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXID))
  subr_initdata_t _subr_posix_getutxid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXLINE))
  subr_initdata_t _subr_posix_getutxline;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_get_user_sid;
#endif
#if defined(HAVE_GETGID)
  subr_initdata_t _subr_posix_gid;
#endif
#if defined(HAVE_GETGROUPS)
  subr_initdata_t _subr_posix_groups;
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  subr_initdata_t _subr_posix_group_info;
#endif
#if defined(HAVE_GETHOSTID)
  subr_initdata_t _subr_posix_hostid;
#endif
  subr_initdata_t _subr_posix_j0;
  subr_initdata_t _subr_posix_j1;
  subr_initdata_t _subr_posix_jn;
#if defined(HAVE_KILL)
  subr_initdata_t _subr_posix_kill;
#endif
#if defined(HAVE_LGAMMA) || HAVE_DECL_LGAMMA_R
  subr_initdata_t _subr_posix_lgamma;
#endif
  subr_initdata_t _subr_posix_loadavg;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_make_shortcut;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_memory_status;
#endif
  subr_initdata_t _subr_posix_mkdtemp;
  subr_initdata_t _subr_posix_mknod;
  subr_initdata_t _subr_posix_mkstemp;
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  subr_initdata_t _subr_posix_openlog;
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  subr_initdata_t _subr_posix_pathconf;
#endif
#if defined(HAVE_GETPGID)
  subr_initdata_t _subr_posix_pgid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_PUTUTXLINE))
  subr_initdata_t _subr_posix_pututxline;
#endif
  subr_initdata_t _subr_posix_resolve_host_ipaddr;
#if defined(HAVE_GETRLIMIT)
  subr_initdata_t _subr_posix_rlimit;
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  subr_initdata_t _subr_posix_service;
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_SETLOGMASK))
  subr_initdata_t _subr_posix_setlogmask;
#endif
#if defined(HAVE_SETPGRP)
  subr_initdata_t _subr_posix_setpgrp;
#endif
#if defined(HAVE_SETREGID)
  subr_initdata_t _subr_posix_setregid;
#endif
#if defined(HAVE_SETREUID)
  subr_initdata_t _subr_posix_setreuid;
#endif
#if defined(HAVE_SETSID)
  subr_initdata_t _subr_posix_setsid;
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_SETUTXENT))
  subr_initdata_t _subr_posix_setutxent;
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  subr_initdata_t _subr_posix_set_file_stat;
#endif
#if defined(HAVE_SETRLIMIT)
  subr_initdata_t _subr_posix_set_rlimit;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_shortcut_info;
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  subr_initdata_t _subr_posix_stat_vfs;
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  subr_initdata_t _subr_posix_stream_lock;
#endif
#if defined(HAVE_FCNTL)
  subr_initdata_t _subr_posix_stream_options;
#endif
  subr_initdata_t _subr_posix_string_time;
#if defined(WIN32_NATIVE) || defined(HAVE_SYNC) || defined(HAVE_FSYNC)
  subr_initdata_t _subr_posix_sync;
#endif
#if defined(HAVE_SYSCONF)
  subr_initdata_t _subr_posix_sysconf;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_system_info;
#endif
#if defined(HAVE_TGAMMA)
  subr_initdata_t _subr_posix_tgamma;
#endif
#if defined(HAVE_GETUID)
  subr_initdata_t _subr_posix_uid;
#endif
#if defined(HAVE_UMASK)
  subr_initdata_t _subr_posix_umask;
#endif
  subr_initdata_t _subr_posix_uname;
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  subr_initdata_t _subr_posix_usage;
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  subr_initdata_t _subr_posix_user_info;
#endif
#if defined(HAVE_GETUSERSHELL) && defined(HAVE_ENDUSERSHELL)
  subr_initdata_t _subr_posix_user_shells;
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_posix_version;
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  subr_initdata_t _subr_posix_wait;
#endif
  subr_initdata_t _subr_posix_y0;
  subr_initdata_t _subr_posix_y1;
  subr_initdata_t _subr_posix_yn;
  subr_initdata_t _subr_posix__25fgetc;
  subr_initdata_t _subr_posix__25fputc;
#if defined(HAVE_SETDOMAINNAME)
  subr_initdata_t _subr_posix__25setdomainname;
#endif
#if defined(HAVE_SETEGID)
  subr_initdata_t _subr_posix__25setegid;
#endif
#if defined(HAVE_SETEUID)
  subr_initdata_t _subr_posix__25seteuid;
#endif
#if defined(HAVE_SETGID)
  subr_initdata_t _subr_posix__25setgid;
#endif
#if defined(HAVE_SETGROUPS)
  subr_initdata_t _subr_posix__25setgroups;
#endif
#if defined(HAVE_SETHOSTID) && !defined(_AIX)
  subr_initdata_t _subr_posix__25sethostid;
#endif
#if defined(HAVE_SETPGID)
  subr_initdata_t _subr_posix__25setpgid;
#endif
#if defined(HAVE_SETUID)
  subr_initdata_t _subr_posix__25setuid;
#endif
  subr_initdata_t _subr_posix__25set_file_size;
  subr_initdata_t _subr_posix__25stdio;
#if defined(HAVE_SYSLOG)
  subr_initdata_t _subr_posix__25syslog;
#endif
  subr_initdata_t _subr_posix__25ungetc;
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  subr_initdata_t _subr_sys_format_message;
#endif
  subr_initdata_t _subr_sys_strerror;
  int _dummy_to_avoid_trailing_comma_in_initializer;
} module__syscalls__subr_tab_initdata = {
  { "OS", "BOGOMIPS" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "OS", "CLIPBOARD" },
#endif
  { "OS", "FILE-OWNER" },
#if defined(WIN32_NATIVE)
  { "OS", "FILE-VERSION" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "OS", "LAST-ERROR" },
#endif
  { "OS", "PRIORITY" },
  { "OS", "VERSION<" },
  { "OS", "VERSION<=" },
  { "OS", "VERSION>" },
  { "OS", "VERSION>=" },
  { "OS", "VERSION-COMPARE" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "OS", "%SET-CLIPBOARD" },
#endif
  { "OS", "%SET-PRIORITY" },
#if defined(HAVE_SYS_RESOURCE_H)
  { "POSIX", "BEGIN-SUBPROCESSES" },
#endif
#if defined(HAVE_CHROOT)
  { "POSIX", "CHROOT" },
#endif
  { "POSIX", "CLEARERR" },
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_CLOSELOG))
  { "POSIX", "CLOSELOG" },
#endif
#if defined(HAVE_CONFSTR)
  { "POSIX", "CONFSTR" },
#endif
#if defined(DEBUG_SPVW)
  { "POSIX", "CONSTSYM" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "CONVERT-ATTRIBUTES" },
#endif
  { "POSIX", "CONVERT-MODE" },
  { "POSIX", "COPY-FILE" },
#if defined(HAVE_GETDOMAINNAME)
  { "POSIX", "DOMAINNAME" },
#endif
  { "POSIX", "DUPLICATE-HANDLE" },
#if defined(HAVE_GETEGID)
  { "POSIX", "EGID" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_ENDUTXENT))
  { "POSIX", "ENDUTXENT" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { "POSIX", "END-SUBPROCESSES" },
#endif
#if defined(HAVE_ERFC)
  { "POSIX", "ERF" },
#endif
#if defined(HAVE_ERFC)
  { "POSIX", "ERFC" },
#endif
  { "POSIX", "ERRNO" },
#if defined(HAVE_GETEUID)
  { "POSIX", "EUID" },
#endif
  { "POSIX", "FCLOSE" },
  { "POSIX", "FDOPEN" },
  { "POSIX", "FEOF" },
  { "POSIX", "FERROR" },
  { "POSIX", "FFLUSH" },
  { "POSIX", "FILENO" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "FILE-INFO" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "FILE-PROPERTIES" },
#endif
  { "POSIX", "FILE-SIZE" },
#if defined(HAVE_FSTAT) && defined(HAVE_STAT)
  { "POSIX", "FILE-STAT" },
#endif
#if defined(HAVE_NFTW)
  { "POSIX", "FILE-TREE-WALK" },
#endif
  { "POSIX", "FNMATCH" },
  { "POSIX", "FOPEN" },
  { "POSIX", "FREOPEN" },
#if defined(HAVE_GETDATE) && defined(HAVE_DECL_GETDATE_ERR)
  { "POSIX", "GETDATE" },
#endif
#if defined(HAVE_GETPGRP)
  { "POSIX", "GETPGRP" },
#endif
#if defined(HAVE_GETPPID)
  { "POSIX", "GETPPID" },
#endif
#if defined(HAVE_GETSID)
  { "POSIX", "GETSID" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXENT))
  { "POSIX", "GETUTXENT" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXID))
  { "POSIX", "GETUTXID" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_GETUTXLINE))
  { "POSIX", "GETUTXLINE" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "GET-USER-SID" },
#endif
#if defined(HAVE_GETGID)
  { "POSIX", "GID" },
#endif
#if defined(HAVE_GETGROUPS)
  { "POSIX", "GROUPS" },
#endif
#if defined(HAVE_GETGRGID) && defined(HAVE_GETGRNAM)
  { "POSIX", "GROUP-INFO" },
#endif
#if defined(HAVE_GETHOSTID)
  { "POSIX", "HOSTID" },
#endif
  { "POSIX", "J0" },
  { "POSIX", "J1" },
  { "POSIX", "JN" },
#if defined(HAVE_KILL)
  { "POSIX", "KILL" },
#endif
#if defined(HAVE_LGAMMA) || HAVE_DECL_LGAMMA_R
  { "POSIX", "LGAMMA" },
#endif
  { "POSIX", "LOADAVG" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "MAKE-SHORTCUT" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "MEMORY-STATUS" },
#endif
  { "POSIX", "MKDTEMP" },
  { "POSIX", "MKNOD" },
  { "POSIX", "MKSTEMP" },
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  { "POSIX", "OPENLOG" },
#endif
#if defined(HAVE_PATHCONF) && defined(HAVE_FPATHCONF)
  { "POSIX", "PATHCONF" },
#endif
#if defined(HAVE_GETPGID)
  { "POSIX", "PGID" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_PUTUTXLINE))
  { "POSIX", "PUTUTXLINE" },
#endif
  { "POSIX", "RESOLVE-HOST-IPADDR" },
#if defined(HAVE_GETRLIMIT)
  { "POSIX", "RLIMIT" },
#endif
#if (defined(HAVE_GETSERVBYPORT) && defined(HAVE_GETSERVBYNAME)) || defined(WIN32_NATIVE)
  { "POSIX", "SERVICE" },
#endif
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_SETLOGMASK))
  { "POSIX", "SETLOGMASK" },
#endif
#if defined(HAVE_SETPGRP)
  { "POSIX", "SETPGRP" },
#endif
#if defined(HAVE_SETREGID)
  { "POSIX", "SETREGID" },
#endif
#if defined(HAVE_SETREUID)
  { "POSIX", "SETREUID" },
#endif
#if defined(HAVE_SETSID)
  { "POSIX", "SETSID" },
#endif
#if (defined(HAVE_UTMPX_H)) && (defined(HAVE_SETUTXENT))
  { "POSIX", "SETUTXENT" },
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  { "POSIX", "SET-FILE-STAT" },
#endif
#if defined(HAVE_SETRLIMIT)
  { "POSIX", "SET-RLIMIT" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "SHORTCUT-INFO" },
#endif
#if defined(WIN32_NATIVE) || (defined(HAVE_STATVFS) && defined(HAVE_SYS_STATVFS_H))
  { "POSIX", "STAT-VFS" },
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  { "POSIX", "STREAM-LOCK" },
#endif
#if defined(HAVE_FCNTL)
  { "POSIX", "STREAM-OPTIONS" },
#endif
  { "POSIX", "STRING-TIME" },
#if defined(WIN32_NATIVE) || defined(HAVE_SYNC) || defined(HAVE_FSYNC)
  { "POSIX", "SYNC" },
#endif
#if defined(HAVE_SYSCONF)
  { "POSIX", "SYSCONF" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "SYSTEM-INFO" },
#endif
#if defined(HAVE_TGAMMA)
  { "POSIX", "TGAMMA" },
#endif
#if defined(HAVE_GETUID)
  { "POSIX", "UID" },
#endif
#if defined(HAVE_UMASK)
  { "POSIX", "UMASK" },
#endif
  { "POSIX", "UNAME" },
#if (defined(HAVE_SYS_RESOURCE_H)) && (defined(HAVE_GETRUSAGE))
  { "POSIX", "USAGE" },
#endif
#if defined(HAVE_GETLOGIN) && defined(HAVE_GETPWNAM) && defined(HAVE_GETPWUID) && defined(HAVE_GETUID)
  { "POSIX", "USER-INFO" },
#endif
#if defined(HAVE_GETUSERSHELL) && defined(HAVE_ENDUSERSHELL)
  { "POSIX", "USER-SHELLS" },
#endif
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "POSIX", "VERSION" },
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  { "POSIX", "WAIT" },
#endif
  { "POSIX", "Y0" },
  { "POSIX", "Y1" },
  { "POSIX", "YN" },
  { "POSIX", "%FGETC" },
  { "POSIX", "%FPUTC" },
#if defined(HAVE_SETDOMAINNAME)
  { "POSIX", "%SETDOMAINNAME" },
#endif
#if defined(HAVE_SETEGID)
  { "POSIX", "%SETEGID" },
#endif
#if defined(HAVE_SETEUID)
  { "POSIX", "%SETEUID" },
#endif
#if defined(HAVE_SETGID)
  { "POSIX", "%SETGID" },
#endif
#if defined(HAVE_SETGROUPS)
  { "POSIX", "%SETGROUPS" },
#endif
#if defined(HAVE_SETHOSTID) && !defined(_AIX)
  { "POSIX", "%SETHOSTID" },
#endif
#if defined(HAVE_SETPGID)
  { "POSIX", "%SETPGID" },
#endif
#if defined(HAVE_SETUID)
  { "POSIX", "%SETUID" },
#endif
  { "POSIX", "%SET-FILE-SIZE" },
  { "POSIX", "%STDIO" },
#if defined(HAVE_SYSLOG)
  { "POSIX", "%SYSLOG" },
#endif
  { "POSIX", "%UNGETC" },
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  { "SYS", "FORMAT-MESSAGE" },
#endif
  { "SYS", "STRERROR" },
  0
};

void module__syscalls__init_function_1 (module_t* module)
{
  (void)module; /* avoid -Wunused-parameter */
  this_module_stab = (struct module__syscalls__stab_t*)module->stab;
  pushSTACK(O(object_Kmethod));
  pushSTACK(O(object_Kpreserve));
  pushSTACK(O(object_Kif_exists));
  pushSTACK(O(object_Kif_does_not_exist));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_copy_file.keywords = vectorof(4);
#if defined(HAVE_NFTW)
  pushSTACK(O(object_Kfd_limit));
  pushSTACK(O(object_Kchdir));
  pushSTACK(O(object_Kdepth));
  pushSTACK(O(object_Kmount));
  pushSTACK(O(object_Kphys));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_file_tree_walk.keywords = vectorof(5);
#endif
  pushSTACK(O(object_Kcase_sensitive));
  pushSTACK(O(object_Kpathname));
  pushSTACK(O(object_Kperiod));
  pushSTACK(O(object_Knoescape));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_fnmatch.keywords = vectorof(4);
#if defined(WIN32_NATIVE) || defined(UNIX_CYGWIN)
  pushSTACK(O(object_Kworking_directory));
  pushSTACK(O(object_Karguments));
  pushSTACK(O(object_Kshow_command));
  pushSTACK(O(object_Kicon));
  pushSTACK(O(object_Kdescription));
  pushSTACK(O(object_Khot_key));
  pushSTACK(O(object_Kpath));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_make_shortcut.keywords = vectorof(7);
#endif
  pushSTACK(O(object_Kdirection));
  pushSTACK(O(object_Kbuffered));
  pushSTACK(O(object_Kexternal_format));
  pushSTACK(O(object_Kelement_type));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_mkstemp.keywords = vectorof(4);
#if (defined(HAVE_SYSLOG)) && (defined(HAVE_OPENLOG))
  pushSTACK(O(object_Kpid));
  pushSTACK(O(object_Kcons));
  pushSTACK(O(object_Kndelay));
  pushSTACK(O(object_Kodelay));
  pushSTACK(O(object_Knowait));
  pushSTACK(O(object_Kfacility));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_openlog.keywords = vectorof(6);
#endif
#if defined(HAVE_STAT) && (defined(HAVE_CHMOD) || defined(HAVE_CHOWN) || defined(HAVE_UTIME))
  pushSTACK(O(object_Katime));
  pushSTACK(O(object_Kmtime));
  pushSTACK(O(object_Kmode));
  pushSTACK(O(object_Kuid));
  pushSTACK(O(object_Kgid));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_set_file_stat.keywords = vectorof(5);
#endif
#if defined(HAVE_FCNTL) || defined(WIN32_NATIVE)
  pushSTACK(O(object_Kblock));
  pushSTACK(O(object_Kshared));
  pushSTACK(O(object_Kstart));
  pushSTACK(O(object_Klength));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_stream_lock.keywords = vectorof(4);
#endif
#if defined(HAVE_SYS_RESOURCE_H)
  pushSTACK(O(object_Kpid));
  pushSTACK(O(object_Kusage));
  pushSTACK(O(object_Knohang));
  pushSTACK(O(object_Kuntraced));
  pushSTACK(O(object_Kstopped));
  pushSTACK(O(object_Kexited));
  pushSTACK(O(object_Kcontinued));
  pushSTACK(O(object_Knowait));
  ((struct module__syscalls__stab_t*)module->stab)->_subr_posix_wait.keywords = vectorof(8);
#endif
  O(my_stdin) = (allocate_fpointer(NULL));
  O(my_stdout) = (allocate_fpointer(NULL));
  O(my_stderr) = (allocate_fpointer(NULL));
}

void module__syscalls__init_function_2__modprep (module_t* module)
{
  (void)module; /* avoid -Wunused-parameter */
  this_module_stab = (struct module__syscalls__stab_t*)module->stab;
}

void module__syscalls__fini_function (module_t* module);
void module__syscalls__fini_function (module_t* module)
{
  (void)module; /* avoid -Wunused-parameter */
}
